(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 8.0' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       157,          7]
NotebookDataLength[   3604316,     134625]
NotebookOptionsPosition[   2623960,     104341]
NotebookOutlinePosition[   3095820,     117386]
CellTagsIndexPosition[   3079488,     116909]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["FeynCalc 8.0", "Title",
 CellChangeTimes->{3.50097011067782*^9},
 TextAlignment->Center],

Cell["Tools for quantum field theory calculations", "Subtitle",
 TextAlignment->Center],

Cell[TextData[StyleBox["Reference Guide",
 FontWeight->"Bold"]], "Subtitle",
 PageBreakBelow->True,
 TextAlignment->Center],

Cell["Rolf Mertig and Frederik Orellana", "Subsubtitle",
 TextAlignment->Center],

Cell[TextData[{
 "HTML version: ",
 ButtonBox["http://www.feyncalc.org/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Subsubtitle",
 TextAlignment->Center],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"$Version", ",", 
   RowBox[{"Date", "[", "]"}], ",", "$UserName"}], "}"}]], "Input",
 CellLabel->"In[1]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\<\"8.0 for Linux x86 (64-bit) (October 10, 2011)\"\>", ",", 
    RowBox[{"{", 
     RowBox[{
     "2012", ",", "1", ",", "13", ",", "14", ",", "18", ",", 
      "46.827432`8.423075318490865"}], "}"}], ",", "\<\"rolfm\"\>"}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[1]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions", "Section",
 CellTags->"Functions"],

Cell["Evaluation time and memory usage", "Commentary"],

Cell["\<\
This is only for timing purposes and can be omitted in general.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"absolutestarttime", "=", 
  RowBox[{"AbsoluteTime", "[", "]"}]}]], "Input",
 CellLabel->"In[2]:="],

Cell[BoxData[
 FormBox["3.535453127150952`16.30102007292504*^9", TraditionalForm]], "Output",\

 CellLabel->"Out[2]="]
}, Open  ]],

Cell["<<HighEnergyPhysics`FeynCalc`", "Input",
 CellLabel->"In[3]:="],

Cell["Loading FeynCalc from /home/rolfm/HighEnergyPhysics", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell[TextData[{
 StyleBox["FeynCalc",
  FontWeight->"Bold"],
 " ",
 "8.1.0",
 " For help, type ?FeynCalc, open ",
 ButtonBox["FeynCalcRef8.nb",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookOpen[
   "/home/rolfm/HighEnergyPhysics/Documentation/English/FeynCalcRef8.nb"],
  ButtonData:>{"Short Overview", "intro"},
  ButtonNote->"Open FeynCalcRef8.nb"],
 " or visit ",
 ButtonBox["www.feyncalc.org",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feyncalc.org/"], None},
  ButtonNote->"http://www.feyncalc.org/"]
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell[TextData[{
 "Loading FeynArts, see ",
 ButtonBox["www.feynarts.de",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.feynarts.de/"], None},
  ButtonNote->"http://www.feynarts.de/"],
 " for documentation"
}], "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell["FeynArts 3.4 patched for use with FeynCalc", "Text",
 CellMargins->{{Inherited, Inherited}, {1, 0}},
 GeneratedCell->True,
 CellAutoOverwrite->True],

Cell["Abbreviation", "ObjectName",
 PageBreakAbove->True,
 CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Abbreviation"],

Cell["\<\
Abbreviation is a function used by OneLoop and PaVeReduce for generating \
smaller files when saving results to the hard disk. The convention is that a \
definition like GP = GluonPropagator should be accompanied by the definition \
Abbreviation[GluonPropagator] = HoldForm[GP].\
\>", "Text",
 CellTags->"Abbreviation"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["$Abbreviations",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$Abbreviations", All, CellTags],
  ButtonData->"$Abbreviations",
  ButtonNote->"$Abbreviations"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["WriteOut",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "WriteOut", All, CellTags],
  ButtonData->"WriteOut",
  ButtonNote->"WriteOut"],
 ", ",
 ButtonBox["WriteOutPaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "WriteOutPaVe", All, CellTags],
  ButtonData->"WriteOutPaVe",
  ButtonNote->"WriteOutPaVe"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkPropagator", All, CellTags],
  ButtonData->"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"Abbreviation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Abbreviation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GP", "[", 
  RowBox[{"p", ",", " ", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", " ", "a"}], "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", " ", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Abbreviation",
 CellLabel->"In[4]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}], 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellTags->"Abbreviation",
 CellLabel->"Out[4]="]
}, Open  ]]
}, Open  ]],

Cell["Amplitude", "ObjectName",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Amplitude"],

Cell["\<\
Amplitude is a database of Feynman amplitudes. Amplitude[\"name\"] returns \
the amplitude corresponding to the string \"name\". A list of all defined \
names is obtained with Amplitude[]. New amplitudes can be added to the file \
\"Amplitude.m\". It is strongly recommended to use names that reflect the \
process. The option Gauge \[Rule] 1 means `t Hooft Feynman gauge; \
Polarization \[Rule] 0 gives unpolarized OPE-type amplitudes, Polarization \
\[Rule] 1 the polarized ones.\
\>", "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Amplitude", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[5]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"QuarkMass", "\[Rule]", "0"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[5]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmp",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmp", All, CellTags],
  ButtonData->"FeynAmp",
  ButtonNote->"FeynAmp"],
 "."
}], "Text",
 CellTags->"Amplitude"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Amplitude", "[", "]"}], "//", "Length"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[6]:="],

Cell[BoxData[
 FormBox["98", TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[6]="]
}, Open  ]],

Cell["This is the amplitude of a gluon self-energy diagram.", "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amplitude", "[", "\"\<se1g1\>\"", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[7]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["f",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{"e", "f"}], 
     RowBox[{"\[Beta]", "\[Sigma]"}]], "(", "q", ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"-", "q"}],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox[
      RowBox[{"-", "p"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm]}], ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{"c", "d"}], 
     RowBox[{"\[Alpha]", "\[Rho]"}]], "(", 
    FormBox[
     RowBox[{"p", "-", "q"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[7]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[8]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      SuperscriptBox["q", "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "q"}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Alpha]", "\[Rho]"}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Beta]", "\[Sigma]"}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Beta]", "\[Mu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Alpha]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Alpha]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Alpha]", "\[Mu]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Beta]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Beta]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Alpha]", "\[Beta]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]]}], "-", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["q",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], "+", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], ")"}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]], "-", 
          RowBox[{"2", " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["\[Sigma]",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}]}], ")"}]}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[8]="]
}, Open  ]],

Cell[TextData[{
 "This is the amplitude for graph 6.2 from the paper Z.Phys C ",
 StyleBox["70:",
  FontWeight->"Bold"],
 "637-654, 1996",
 StyleBox[".",
  FontWeight->"Bold"]
}], "Text",
 CellTags->"Amplitude"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{"q1", ",", "q2", ",", 
   RowBox[{"EpsEvaluate", "[", 
    RowBox[{"Trick", "[", 
     RowBox[{"Explicit", "[", 
      RowBox[{"Amplitude", "[", "\"\<gg2\>\"", "]"}], "]"}], "]"}], "]"}]}], 
  "]"}]], "Input",
 CellTags->"Amplitude",
 CellLabel->"In[9]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   RowBox[{
    StyleBox[
     RowBox[{
      SuperscriptBox["\[DifferentialD]", "D"], "q1"}],
     ZeroWidthTimes->True], "\[Integral]", 
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], "q2"}]}], "(", 
   FormBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", " ", "\[ImaginaryI]", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", 
         FormBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", "1"}], ")"}], "m"],
          TraditionalForm]}], ")"}], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "7"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "12"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "10"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "11"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "12"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubsuperscriptBox["g", "s", "4"], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "11"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "7"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\<\"c\"\>", "10"],
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "7"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "5"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "5"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "5"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "7"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "7"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "5"], 
            SubscriptBox["\[Lambda]", "7"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]], "+", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       FormBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q2",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]],
        TraditionalForm], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "1"],
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "5"],
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q2",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "7"], 
            SubscriptBox["\[Lambda]", "10"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox["\[Nu]",
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", 
            SubscriptBox["\[Lambda]", "10"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "7"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "7"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", 
            SubscriptBox["\[Lambda]", "7"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "10"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "11"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "10"],
               TraditionalForm],
              TraditionalForm]], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "10"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "10"], 
            SubscriptBox["\[Lambda]", "12"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "11"],
                TraditionalForm],
               TraditionalForm]]}], "+", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q1",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "11"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "10"], 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "12"],
                TraditionalForm],
               TraditionalForm]]}], "-", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]], "-", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "12"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{
            SubscriptBox["\[Lambda]", "1"], 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["q2",
               TraditionalForm], 
              FormBox[
               FormBox["\[Mu]",
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", 
            SubscriptBox["\[Lambda]", "11"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "1"],
               TraditionalForm],
              TraditionalForm]], "-", 
            RowBox[{"2", " ", 
             SuperscriptBox[
              FormBox["p",
               TraditionalForm], 
              FormBox[
               FormBox[
                SubscriptBox["\[Lambda]", "1"],
                TraditionalForm],
               TraditionalForm]]}]}], ")"}]}], "+", 
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", 
            SubscriptBox["\[Lambda]", "1"]}]], " ", 
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]], "+", 
            SuperscriptBox[
             FormBox["q2",
              TraditionalForm], 
             FormBox[
              FormBox[
               SubscriptBox["\[Lambda]", "11"],
               TraditionalForm],
              TraditionalForm]]}], ")"}]}]}], ")"}]}], ")"}], "/", 
     RowBox[{
      SuperscriptBox["q1", "2"], ".", 
      SuperscriptBox["q2", "2"], ".", 
      SuperscriptBox["q2", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q1", "-", "p"}], ")"}], "2"]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Amplitude",
 CellLabel->"Out[9]="]
}, Open  ]]
}, Open  ]],

Cell["Amputate", "ObjectName",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Amputate"],

Cell["\<\
Amputate[exp,q1,q2,...] amputates Eps and DiracGamma. Amputate[exp,q1,q2,Pair\
\[Rule]{p}] amputates also p.q1 and p.q2; Pair\[Rule]All amputates all except \
OPEDelta.\
\>", "Text",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Amputate", "]"}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[10]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Pair", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Amputate",
 CellLabel->"Out[10]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSlash", All, CellTags],
  ButtonData->"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Amputate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Amputate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
  RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[11]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Amputate",
 CellLabel->"Out[11]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amputate", "[", 
  RowBox[{"%", ",", "q"}], "]"}]], "Input",
 CellTags->"Amputate",
 CellLabel->"In[12]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$698", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       RowBox[{"$AL$698", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Amputate",
 CellLabel->"Out[12]="]
}, Open  ]]
}, Open  ]],

Cell["AnomalousDimension", "ObjectName",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AnomalousDimension"],

Cell["\<\
AnomalousDimension[name] is a database of anomalous dimensions of twist 2 \
operators.\
\>", "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "AnomalousDimension", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[13]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[13]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["SplittingFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SplittingFunction", All, CellTags],
  ButtonData->"SplittingFunction",
  ButtonNote->"SplittingFunction"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumS", All, CellTags],
  ButtonData->"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumT", All, CellTags],
  ButtonData->"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"AnomalousDimension"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AnomalousDimension"],

Cell["Polarized case", "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", " ", 
  RowBox[{"AnomalousDimension", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[14]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "FullSimplify"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[14]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"NS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gnsqq0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[15]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"8", " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     FractionBox["4", "m"], "+", 
     FractionBox["4", 
      RowBox[{"m", "+", "1"}]], "-", "6"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[15]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "qg", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsqg0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[16]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["8", "m"], "-", 
     FractionBox["16", 
      RowBox[{"m", "+", "1"}]]}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[16]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgq0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[17]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FractionBox["4", 
      RowBox[{"m", "+", "1"}]], "-", 
     FractionBox["8", "m"]}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[17]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgg0", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[18]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", "m"], "+", 
      FractionBox["16", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["22", "3"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"8", " ", 
     SubscriptBox["T", "f"]}], "3"]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[18]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"PS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "0", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gpsqq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[19]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"16", " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["2", 
      SuperscriptBox["m", "3"]], "-", 
     FractionBox["1", 
      SuperscriptBox["m", "2"]], "+", 
     FractionBox["1", 
      RowBox[{"m", "+", "1"}]], "+", 
     FractionBox["3", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
     FractionBox["2", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
     FractionBox["1", "m"]}], ")"}], " ", 
   SubscriptBox["C", "F"], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[19]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"NS", ",", "qq", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gnsqq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[20]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["C", "A"], " ", 
     SubscriptBox["C", "F"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["S", "~"], "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], "m"]}], "-", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], 
        RowBox[{"m", "+", "1"}]], "+", 
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "3"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "-", 
       RowBox[{"32", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "12"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox["44", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]], "-", 
       RowBox[{
        FractionBox["536", "9"], " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       RowBox[{
        FractionBox["88", "3"], " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "-", 
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "3"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox["212", 
        RowBox[{"9", " ", "m"}]], "-", 
       FractionBox["748", 
        RowBox[{"9", " ", 
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
       FractionBox["4", 
        RowBox[{"3", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
       FractionBox["16", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
       FractionBox["17", "3"]}], ")"}]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"32", " ", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["S", "~"], "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], "m"], "+", 
        FractionBox[
         RowBox[{"32", " ", 
          RowBox[{
           SubscriptBox[
            OverscriptBox["S", "~"], "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         RowBox[{"m", "+", "1"}]], "-", 
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "3"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        RowBox[{"64", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "12"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        FractionBox["8", 
         SuperscriptBox["m", "3"]], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "1"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         SuperscriptBox["m", "2"]], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "1"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], "m"], "+", 
        FractionBox[
         RowBox[{"16", " ", 
          RowBox[{
           SubscriptBox["S", "2"], "(", 
           FormBox[
            RowBox[{"m", "-", "1"}],
            TraditionalForm], ")"}]}], 
         RowBox[{"m", "+", "1"}]], "-", 
        RowBox[{"24", " ", 
         RowBox[{
          SubscriptBox["S", "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox["S", "12"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "+", 
        RowBox[{"32", " ", 
         RowBox[{
          SubscriptBox["S", "21"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "-", 
        FractionBox["40", "m"], "+", 
        FractionBox["40", 
         RowBox[{"m", "+", "1"}]], "+", 
        FractionBox["16", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
        FractionBox["40", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", "3"}], ")"}]}], 
     ")"}]}], "-", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["8", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]]}], "+", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       FractionBox["16", "3"], " ", 
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", 
       RowBox[{"9", " ", "m"}]], "+", 
      FractionBox["88", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["2", "3"]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[20]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "qg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsqg1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[21]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"16", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{
          SubscriptBox[
           OverscriptBox["S", "~"], "2"], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "m"]}], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["2", 
       SuperscriptBox["m", "3"]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox["3", 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox[
       RowBox[{
        SubsuperscriptBox["S", "1", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["4", "m"], "+", 
      FractionBox["3", 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["8", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["12", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}], "+", 
   RowBox[{"8", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["2", 
        SuperscriptBox["m", "3"]]}], "-", 
      FractionBox["1", 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["14", "m"], "-", 
      FractionBox["19", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["8", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["4", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[21]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gq", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgq1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[22]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox["28", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{
        SubsuperscriptBox["S", "1", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", "m"}]], "-", 
      FractionBox[
       RowBox[{"5", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "2"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["56", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["20", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["38", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["6", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["2", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]]}], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"3", " ", "m"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}], 
       RowBox[{"3", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["7", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["2", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "+", 
      FractionBox["1", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]]}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox["12", 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubsuperscriptBox["S", "1", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["15", "m"], "-", 
      FractionBox["6", 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["3", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "-", 
      FractionBox["2", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[22]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "gsgg1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[23]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SubsuperscriptBox["C", "A", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox[
          OverscriptBox["S", "~"], "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox[
         OverscriptBox["S", "~"], "12"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["8", 
       SuperscriptBox["m", "3"]], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox["m", "2"]], "+", 
      FractionBox["58", 
       RowBox[{"3", " ", 
        SuperscriptBox["m", "2"]}]], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      RowBox[{
       FractionBox["134", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], "m"], "-", 
      FractionBox[
       RowBox[{"16", " ", 
        RowBox[{
         SubscriptBox["S", "2"], "(", 
         FormBox[
          RowBox[{"m", "-", "1"}],
          TraditionalForm], ")"}]}], 
       RowBox[{"m", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox["S", "3"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "12"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{
        SubscriptBox["S", "21"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox["107", 
       RowBox[{"9", " ", "m"}]], "+", 
      FractionBox["241", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["86", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox["48", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]], "-", 
      FractionBox["16", "3"]}], ")"}]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox["1", 
        RowBox[{"3", " ", 
         SuperscriptBox["m", "2"]}]]}], "-", 
      RowBox[{
       FractionBox["5", "9"], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox[
         RowBox[{"m", "-", "1"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox["14", 
       RowBox[{"9", " ", "m"}]], "-", 
      FractionBox["19", 
       RowBox[{"9", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      FractionBox["1", 
       RowBox[{"3", " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "+", 
      FractionBox["1", "3"]}], ")"}]}], "+", 
   RowBox[{"8", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["4", 
       SuperscriptBox["m", "3"]], "-", 
      FractionBox["10", 
       SuperscriptBox["m", "2"]], "-", 
      FractionBox["10", 
       RowBox[{"m", "+", "1"}]], "+", 
      FractionBox["2", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "2"]], "+", 
      FractionBox["4", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
      FractionBox["10", "m"], "+", "1"}], ")"}], " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"]}]}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[23]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["\[Gamma]", 
    RowBox[{"S", ",", "gg", " "}], 
    RowBox[{
     RowBox[{"(", "1", ")"}], " "}]], TraditionalForm]]],
 "polarized (different representation)"
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AnomalousDimension", "[", "GSGG1", "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[24]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SubsuperscriptBox["C", "A", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       RowBox[{
        OverscriptBox["S", "~"], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{
         SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
         FormBox[
          FractionBox["m", "2"],
          TraditionalForm], "\[NoBreak]", ")"}]}], 
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{"m", "+", "1"}], ")"}]}]], "-", 
      RowBox[{
       SubsuperscriptBox["S", "3", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
       FormBox[
        FractionBox["m", "2"],
        TraditionalForm], "\[NoBreak]", ")"}], "-", 
      RowBox[{"4", " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox["m",
         TraditionalForm], ")"}], " ", 
       RowBox[{
        SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
        FormBox[
         FractionBox["m", "2"],
         TraditionalForm], "\[NoBreak]", ")"}]}], "-", 
      FractionBox[
       RowBox[{
        RowBox[{"m", " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"m", " ", 
            RowBox[{"(", 
             RowBox[{
              RowBox[{"m", " ", 
               RowBox[{"(", 
                RowBox[{
                 RowBox[{"m", " ", 
                  RowBox[{"(", 
                   RowBox[{
                    RowBox[{"48", " ", "m", " ", 
                    RowBox[{"(", 
                    RowBox[{"m", "+", "3"}], ")"}]}], "+", "469"}], ")"}]}], 
                 "+", "698"}], ")"}]}], "+", "7"}], ")"}]}], "+", "258"}], 
          ")"}]}], "+", "144"}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "3"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "3"]}]], "+", 
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"m", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"67", " ", "m", " ", 
              SuperscriptBox[
               RowBox[{"(", 
                RowBox[{"m", "+", "1"}], ")"}], "2"]}], "+", "144"}], ")"}]}],
           "+", "72"}], ")"}], " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox["m",
          TraditionalForm], ")"}]}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]]}], ")"}]}], "+", 
   RowBox[{"32", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"m", " ", 
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{
           RowBox[{"3", " ", "m", " ", 
            RowBox[{"(", 
             RowBox[{"m", "+", "1"}], ")"}]}], "+", "13"}], ")"}]}], "-", 
        "3"}], 
       RowBox[{"9", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"m", "+", "1"}], ")"}], "2"]}]], "-", 
      FractionBox[
       RowBox[{"5", " ", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox["m",
          TraditionalForm], ")"}]}], "9"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"8", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"m", " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"m", " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{"m", " ", 
              RowBox[{"(", 
               RowBox[{
                RowBox[{"m", " ", 
                 RowBox[{"(", 
                  RowBox[{
                   RowBox[{"m", " ", 
                    RowBox[{"(", 
                    RowBox[{"m", "+", "3"}], ")"}]}], "+", "5"}], ")"}]}], 
                "+", "1"}], ")"}]}], "-", "8"}], ")"}]}], "+", "2"}], ")"}]}],
        "+", "4"}], ")"}], " ", 
     SubscriptBox["C", "F"], " ", 
     SubscriptBox["T", "f"]}], 
    RowBox[{
     SuperscriptBox["m", "3"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"m", "+", "1"}], ")"}], "3"]}]]}], TraditionalForm]], "Output",\

 CellTags->"AnomalousDimension",
 CellLabel->"Out[24]="]
}, Open  ]],

Cell[TextData[{
 "Check that all odd moments give the same for the two representations of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Gamma]", 
     RowBox[{"S", ",", "gg", " "}], 
     RowBox[{
      RowBox[{"(", "1", ")"}], " "}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"AnomalousDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"%", "-", "%%"}], "/.", 
    RowBox[{"OPEm", "\[Rule]", "ij"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"ij", ",", "1", ",", "17", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"AnomalousDimension",
 CellLabel->"In[25]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", ",", "0", 
    ",", "0"}], "}"}], TraditionalForm]], "Output",
 CellTags->"AnomalousDimension",
 CellLabel->"Out[25]="]
}, Open  ]]
}, Open  ]],

Cell["AntiCommutator", "ObjectName",
 CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiCommutator"],

Cell["\<\
AntiCommutator[x, y] = c defines the anti-commutator of the non commuting \
objects x and y.\
\>", "Text",
 CellTags->"AntiCommutator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Commutator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Commutator", All, CellTags],
  ButtonData->"Commutator",
  ButtonNote->"Commutator"],
 ", ",
 ButtonBox["CommutatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CommutatorExplicit", All, CellTags],
  ButtonData->"CommutatorExplicit",
  ButtonNote->"CommutatorExplicit"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeclareNonCommutative", All, CellTags],
  ButtonData->"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"AntiCommutator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiCommutator"],

Cell[TextData[{
 "This declares ",
 StyleBox["a",
  FontFamily->"Courier"],
 " and ",
 StyleBox["b",
  FontFamily->"Courier"],
 " as noncommutative variables."
}], "Text",
 CellTags->"AntiCommutator"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[26]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[27]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "a", ",", "\[MediumSpace]", "b", "}"}], 
  TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[27]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[28]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "+", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[28]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"a", "+", "b"}], ",", 
    RowBox[{"a", "-", 
     RowBox[{"2", "b"}]}]}], " ", "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[29]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", 
      RowBox[{"2", " ", "b"}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "-", 
      RowBox[{"2", " ", "b"}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[29]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    RowBox[{"a", "+", "b"}], ",", 
    RowBox[{"a", "-", 
     RowBox[{"2", "b"}]}]}], " ", "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[30]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"a", ".", "b"}]}], "-", 
   RowBox[{"b", ".", "a"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"a", ".", "a"}]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{"b", ".", "b"}]}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[30]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"c", ",", "d", ",", 
   OverscriptBox["c", "~"], ",", 
   OverscriptBox["d", "~"]}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[31]:="],

Cell["Defining {c,d} = z results in replacements of c.d by z-d.c.", "Text",
 CellTags->"AntiCommutator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"c", ",", "d"}], "]"}], " ", "=", " ", "z"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[32]:="],

Cell[BoxData[
 FormBox["z", TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[32]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", " ", 
  RowBox[{"d", " ", ".", " ", "c", " ", ".", " ", "d"}], " ", "]"}]], "Input",\

 CellTags->"AntiCommutator",
 CellLabel->"In[33]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"d", " ", "z"}], "-", 
   RowBox[{"d", ".", "d", ".", "c"}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[33]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    OverscriptBox["d", "~"], ",", 
    OverscriptBox["c", "~"]}], "]"}], " ", "=", " ", 
  OverscriptBox["z", "~"]}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[34]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["z", "~"], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[34]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", " ", 
  RowBox[{
   OverscriptBox["d", "~"], " ", ".", " ", 
   OverscriptBox["c", "~"], " ", ".", " ", 
   OverscriptBox["d", "~"]}], " ", "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[35]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    OverscriptBox["d", "~"], " ", 
    OverscriptBox["z", "~"]}], "-", 
   RowBox[{
    OverscriptBox["c", "~"], ".", 
    OverscriptBox["d", "~"], ".", 
    OverscriptBox["d", "~"]}]}], TraditionalForm]], "Output",
 CellTags->"AntiCommutator",
 CellLabel->"Out[35]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", 
   OverscriptBox["c", "~"], ",", 
   OverscriptBox["d", "~"]}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[36]:="],

Cell[BoxData[
 RowBox[{"Unset", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"c", ",", "d"}], "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[37]:="],

Cell[BoxData[
 RowBox[{"Unset", "[", 
  RowBox[{"AntiCommutator", "[", 
   RowBox[{
    OverscriptBox["d", "~"], ",", 
    OverscriptBox["c", "~"]}], "]"}], "]"}]], "Input",
 CellTags->"AntiCommutator",
 CellLabel->"In[38]:="]
}, Open  ]],

Cell["AntiQuarkField", "ObjectName",
 CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiQuarkField"],

Cell["\<\
AntiQuarkField is the name of a fermionic field. AntiQuarkField is just a \
name with no functional properties. Only typeset rules are attached.\
\>", "Text",
 CellTags->"AntiQuarkField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuantumField", All, CellTags],
  ButtonData->"QuantumField",
  ButtonNote->"QuantumField"],
 ", ",
 ButtonBox["QuarkField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkField", All, CellTags],
  ButtonData->"QuarkField",
  ButtonNote->"QuarkField"],
 "."
}], "Text",
 CellTags->"AntiQuarkField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiQuarkField"],

Cell[CellGroupData[{

Cell[BoxData["AntiQuarkField"], "Input",
 CellTags->"AntiQuarkField",
 CellLabel->"In[39]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
 CellTags->"AntiQuarkField",
 CellLabel->"Out[39]="]
}, Open  ]]
}, Open  ]],

Cell["AntiSymmetrize", "ObjectName",
 CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"AntiSymmetrize"],

Cell["\<\
AntiSymmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect to the \
variables a1,a2, ... \
\>", "Text",
 CellTags->"AntiSymmetrize"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Symmetrize",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Symmetrize", All, CellTags],
  ButtonData->"Symmetrize",
  ButtonNote->"Symmetrize"],
 "."
}], "Text",
 CellTags->"AntiSymmetrize"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"AntiSymmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiSymmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"AntiSymmetrize",
 CellLabel->"In[40]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"a", ",", "b"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"AntiSymmetrize",
 CellLabel->"Out[40]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiSymmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellTags->"AntiSymmetrize",
 CellLabel->"In[41]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "y", ",", "z"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "z", ",", "y"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "x", ",", "y"}], ")"}], "-", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"AntiSymmetrize",
 CellLabel->"Out[41]="]
}, Open  ]]
}, Open  ]],

Cell["Anti5", "ObjectName",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Anti5"],

Cell[TextData[{
 "Anti5[exp] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "in exp to the right. Anti5[exp, n] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", 
    RowBox[{"5", " "}]], TraditionalForm]]],
 "n times to the right. Anti5[exp, -n] anticommutes all ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " n times to the left."
}], "Text",
 CellTags->"Anti5"],

Cell[TextData[{
 "The naive ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "scheme is used."
}], "Text",
 CellTags->"Anti5"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracOrder",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracOrder", All, CellTags],
  ButtonData->"DiracOrder",
  ButtonNote->"DiracOrder"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Anti5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"5", ",", "\[Mu]"}], "]"}], " "}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[42]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[42]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", "%", "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[43]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[43]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", 
   RowBox[{"-", "1"}]}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[44]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[44]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracMatrix", "[", 
  RowBox[{
  "5", ",", "\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[45]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[45]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", "2"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[46]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[46]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%%", ",", "Infinity"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[47]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[47]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", 
  RowBox[{"%", ",", 
   RowBox[{"-", "Infinity"}]}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[48]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], 
   SuperscriptBox["\[Gamma]", "\[Beta]"], 
   SuperscriptBox["\[Gamma]", "\[Gamma]"], 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[48]="]
}, Open  ]],

Cell[TextData[{
 "In the naive ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "- scheme D-dimensional ",
 Cell[BoxData[
  FormBox["\[Gamma]", TraditionalForm]]],
 "-matrices anticommute with ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Anti5"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{"5", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[49]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[49]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Anti5", "[", "%", "]"}]], "Input",
 CellTags->"Anti5",
 CellLabel->"In[50]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"]}]}], TraditionalForm]], "Output",
 CellTags->"Anti5",
 CellLabel->"Out[50]="]
}, Open  ]]
}, Open  ]],

Cell["Apart2", "ObjectName",
 CellTags->"Apart2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Apart2"],

Cell["\<\
Apart2[expr] partial fractions FeynAmpDenominators (and FAD's).\
\>", "Text",
 CellTags->"Apart2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FAD", All, CellTags],
  ButtonData->"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominator", All, CellTags],
  ButtonData->"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 "."
}], "Text",
 CellTags->"Apart2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Apart2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "m"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "M"}], "}"}], ",", 
   RowBox[{"q", "-", "p"}]}], "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[51]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "-", $CellContext`m^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "-", $CellContext`M^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"Apart2",
 CellLabel->"Out[51]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart2", "[", "%", "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[52]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q", "2"], "-", 
        SuperscriptBox["M", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], 
   RowBox[{
    SuperscriptBox["m", "2"], "-", 
    SuperscriptBox["M", "2"]}]], TraditionalForm]], "Output",
 CellTags->"Apart2",
 CellLabel->"Out[52]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", "%", "]"}], "]"}]], "Input",
 CellTags->"Apart2",
 CellLabel->"In[53]:="],

Cell[BoxData[
 FractionBox[
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "m"}], "}"}], ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q"}]}], "]"}], "-", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "M"}], "}"}], ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q"}]}], "]"}]}], 
  RowBox[{
   SuperscriptBox["m", "2"], "-", 
   SuperscriptBox["M", "2"]}]]], "Output",
 CellTags->"Apart2",
 CellLabel->"Out[53]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["A0", "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"A0"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{"A0", "[", 
    SuperscriptBox["m", "2"], "]"}], TraditionalForm]]],
 "is the Passarino-Veltman one-point integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "0"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[54]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"A0ToB0", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"A0",
 CellLabel->"Out[54]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "C0", All, CellTags],
  ButtonData->"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "D0", All, CellTags],
  ButtonData->"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"A0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"A0"],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "0"], TraditionalForm]]],
 " is not expressed in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  SuperscriptBox["m", "2"], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[55]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[55]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"A0", ",", 
    RowBox[{"A0ToB0", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[56]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  SuperscriptBox["m", "2"], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[57]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[57]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"A0", ",", 
    RowBox[{"A0ToB0", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[58]:="],

Cell[TextData[{
 "According to the rules of dimensional regularization ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["A", "0"], "(", "0", ")"}], TraditionalForm]]],
 " is set to 0."
}], "Text",
 CellTags->"A0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", "0", "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[59]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[59]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"A0", "[", 
  RowBox[{"SmallVariable", "[", 
   RowBox[{"M", "^", "2"}], "]"}], "]"}]], "Input",
 CellTags->"A0",
 CellLabel->"In[60]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"A0",
 CellLabel->"Out[60]="]
}, Open  ]]
}, Open  ]],

Cell["A0ToB0", "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"A0ToB0"],

Cell[TextData[{
 "A0ToB0 is an option for A0. If set to True, A0[",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "] is expressed by (1 + B0[0, ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "]) ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"A0ToB0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", " ", "A0", "]"}]], "Input",
 CellTags->"A0ToB0",
 CellLabel->"In[61]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"A0ToB0", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"A0ToB0",
 CellLabel->"Out[61]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["A0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "A0", All, CellTags],
  ButtonData->"A0",
  ButtonNote->"A0"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "C0", All, CellTags],
  ButtonData->"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "D0", All, CellTags],
  ButtonData->"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"A0ToB0"]
}, Open  ]],

Cell["BackgroundGluonVertex", "ObjectName",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"BackgroundGluonVertex"],

Cell[TextData[{
 "BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or \
BackgroundGluonVertex[ p,mu,a , q,nu,b , k,la,c ] yields the 3-gluon vertex \
in the background field gauge, where the first set of arguments corresponds \
to the external background field. BackgroundGluonVertex[{p,mu,a}, {q,nu,b}, \
{k,la,c}, {s,si,d}] or BackgroundGluonVertex[{mu,a}, {nu,b}, {la,c}, {",
 "si",
 ",d}] or BackgroundGluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
BackgroundGluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex, \
with {p,mu,a} and {k,la,c} denoting the external background fields. The \
gauge, dimension and the name of the coupling constant are determined by the \
options Gauge, Dimension and CouplingConstant. The Feynman rules are taken \
from L.Abbot ",
 StyleBox["NPB ",
  FontWeight->"Bold"],
 "185 (1981), 189-203; except that all momenta are incoming. Note that Abbots \
coupling constant convention is consistent with the default setting of \
GluonVertex."
}], "Text",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "BackgroundGluonVertex", "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[62]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[62]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"BackgroundGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[63]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          RowBox[{"-", "k"}], "+", "p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Lambda]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "p", "+", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "k"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[63]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[64]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u6",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[64]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[65]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          RowBox[{"-", 
           FractionBox["k", "\[Alpha]"]}], "+", "p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Lambda]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "p", "+", 
          FractionBox["q", "\[Alpha]"]}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "k"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[65]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"BackgroundGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Lambda]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}]], "Input",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"In[66]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         FractionBox[
          RowBox[{
           SuperscriptBox["g", 
            RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Sigma]"}]]}], "\[Alpha]"]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          SuperscriptBox["g", 
           RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]]}], "\[Alpha]"], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Mu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u7",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Lambda]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"BackgroundGluonVertex",
 CellLabel->"Out[66]="]
}, Open  ]]
}, Open  ]],

Cell["Bracket", "ObjectName",
 CellTags->"Bracket"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Bracket"],

Cell["Bracket is an option of Convolute.", "Text",
 CellTags->"Bracket"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Convolute", All, CellTags],
  ButtonData->"Convolute",
  ButtonNote->"Convolute"],
 "."
}], "Text",
 CellTags->"Bracket"]
}, Open  ]],

Cell["BReduce", "ObjectName",
 PageBreakAbove->True,
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"BReduce"],

Cell["\<\
BReduce is an option for B0, B00, B1, B11 determining whether reductions to \
A0 and B0 will be done. \
\>", "Text",
 CellTags->"BReduce"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["A0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "A0", All, CellTags],
  ButtonData->"A0",
  ButtonNote->"A0"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B00", All, CellTags],
  ButtonData->"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B1", All, CellTags],
  ButtonData->"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B11", All, CellTags],
  ButtonData->"B11",
  ButtonNote->"B11"],
 "."
}], "Text",
 CellTags->"BReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"BReduce"],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 " is not expressed in terms of ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["A", "0"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "s", ",", "s"}], "]"}]], "Input",
 CellTags->"BReduce",
 CellLabel->"In[67]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"BReduce",
 CellLabel->"Out[67]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellLabel->"In[68]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[68]="]
}, Open  ]],

Cell["\<\
With BReduce\[Rule]True, transformation is done.\
\>", "Text",
 CellTags->"BReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "s", ",", "s", ",", 
   RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"BReduce",
 CellLabel->"In[69]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", "s", ")"}], "s"], "-", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"BReduce",
 CellLabel->"Out[69]="]
}, Open  ]]
}, Open  ]],

Cell["B0", "ObjectName",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0"],

Cell[TextData[{
 "B0[pp, ma^2, mb^2] is the Passarino-Veltman two-point integral ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "0"], TraditionalForm]]],
 ". All arguments are scalars and have dimension mass^2. If the option \
BReduce is set to True certain B0's are reduced to A0's. Setting the option \
B0Unique to True simplifies B0[a,0,a] and B0[0,0,a]."
}], "Text",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B0", "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[70]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[70]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B1", All, CellTags],
  ButtonData->"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B00", All, CellTags],
  ButtonData->"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B11", All, CellTags],
  ButtonData->"B11",
  ButtonNote->"B11"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "p"}], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[71]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", 
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[71]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"B0", ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[72]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[73]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[73]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{
   SuperscriptBox["m", "2"], ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[74]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}], "+", "2"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[74]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", 
   RowBox[{"m", "^", "2"}], ",", 
   RowBox[{"m", "^", "2"}]}], "]"}]], "Input",
 CellTags->"B0",
 CellLabel->"In[75]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ")"}], TraditionalForm]], "Output",
 CellTags->"B0",
 CellLabel->"Out[75]="]
}, Open  ]]
}, Open  ]],

Cell["B0Real", "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0Real"],

Cell["\<\
B0Real is an option of B0 (default False). If set to True, B0 is assumed to \
be real and the relation B0[a,0,a] = 2 + B0[0,a,a]  is applied.\
\>", "Text",
 CellTags->"B0Real"],

Cell[TextData[{
 "See also: ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"B0Real"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0Real"],

Cell["By default the arguments are not assumed real.", "Text",
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"s", ",", "0", ",", "s"}], "]"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[76]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "2"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Real",
 CellLabel->"Out[76]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[77]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Real",
 CellLabel->"Out[77]="]
}, Open  ]],

Cell["\<\
With B0Real\[Rule]True, transformation is done.\
\>", "Text",
 CellTags->"B0Real"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"s", ",", "0", ",", "s", ",", 
   RowBox[{"B0Real", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"B0Real",
 CellLabel->"In[78]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "2"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Real",
 CellLabel->"Out[78]="]
}, Open  ]]
}, Open  ]],

Cell["B0Unique", "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B0Unique"],

Cell["\<\
B0Unique is an option of B0. If set to True, B0[0,0,m2] is replaced with \
(B0[0,m2,m2]+1) and B0[m2,0,m2] simplifies to (B0[0,m2,m2]+2).\
\>", "Text",
 CellTags->"B0Unique"],

Cell[TextData[{
 "See also: ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"B0Unique"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B0Unique"],

Cell["By default transformation is done.", "Text",
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", "s"}], "]"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[79]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", "s", ",", "s", ")"}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Unique",
 CellLabel->"Out[79]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "//", "Options"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[80]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"BReduce", "\[Rule]", "False"}], ",", 
    RowBox[{"B0Unique", "\[Rule]", "True"}], ",", 
    RowBox[{"B0Real", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Unique",
 CellLabel->"Out[80]="]
}, Open  ]],

Cell["\<\
With B0Real\[Rule]False, nothing happens.\
\>", "Text",
 CellTags->"B0Unique"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B0", "[", 
  RowBox[{"0", ",", "0", ",", "s", ",", 
   RowBox[{"B0Unique", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B0Unique",
 CellLabel->"In[81]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "0"], "(", "0", ",", "0", ",", "s", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"B0Unique",
 CellLabel->"Out[81]="]
}, Open  ]]
}, Open  ]],

Cell["B00", "ObjectName",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B00"],

Cell[TextData[{
 "B00[pp, ma^2,mb^2] is the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "00"], TraditionalForm]]],
 "-function, i.e., the coefficient function of the metric tensor. All \
arguments are scalars and have dimension mass^2."
}], "Text",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B00", "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[82]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B00",
 CellLabel->"Out[82]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B1", All, CellTags],
  ButtonData->"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B00"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B00"],

Cell[TextData[{
 "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e., ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"B00"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", "p", "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[83]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[83]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[84]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox["M", "2"], "+", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             SuperscriptBox["M", "2"], "-", 
             SuperscriptBox["m", "2"]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              SubscriptBox["B", "0"], "(", 
              FormBox[
               SuperscriptBox[
                FormBox["p",
                 TraditionalForm], "2"],
               TraditionalForm], ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}], "-", 
             RowBox[{
              SubscriptBox["B", "0"], "(", "0", ",", 
              SuperscriptBox["m", "2"], ",", 
              SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
          RowBox[{"2", " ", 
           FormBox[
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"],
            TraditionalForm]}]], "-", 
         RowBox[{
          FractionBox["1", "2"], " ", 
          RowBox[{
           SubscriptBox["B", "0"], "(", 
           FormBox[
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"],
            TraditionalForm], ",", 
           SuperscriptBox["m", "2"], ",", 
           SuperscriptBox["M", "2"], ")"}]}]}], ")"}]}], "+", 
      RowBox[{
       FormBox[
        SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", 
       FormBox[
        SuperscriptBox["M", "2"],
        TraditionalForm], ")"}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["M", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "18"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", 
       SuperscriptBox["m", "2"]}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}], "-", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[84]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[85]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", 
       FormBox[
        SuperscriptBox["m", "2"],
        TraditionalForm], ")"}], "-", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "18"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"6", " ", 
       SuperscriptBox["m", "2"]}], "-", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[85]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[86]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"B\"\>", "\<\"00\"\>"],
    TraditionalForm], "(", 
   FormBox[
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["M", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[86]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{"0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[87]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}]}], "+", 
   FractionBox[
    SuperscriptBox["m", "2"], "3"]}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[87]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B00", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SuperscriptBox["M", "2"], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B00",
 CellLabel->"In[88]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["m", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     SuperscriptBox["M", "2"], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["m", "2"], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}]}], "+", 
   FractionBox[
    SuperscriptBox["m", "2"], "3"]}], TraditionalForm]], "Output",
 CellTags->"B00",
 CellLabel->"Out[88]="]
}, Open  ]]
}, Open  ]],

Cell["B1", "ObjectName",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B1"],

Cell[TextData[{
 "B1[pp, ma^2, mb^2] the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "1"], TraditionalForm]]],
 "-function. All arguments are scalars and have dimension mass^2."
}], "Text",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B1", "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[89]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B1",
 CellLabel->"Out[89]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B00", All, CellTags],
  ButtonData->"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B11",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B11", All, CellTags],
  ButtonData->"B11",
  ButtonNote->"B11"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"B1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[90]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["M", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}], "-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
    RowBox[{"2", " ", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm]}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], ",", 
     SuperscriptBox["m", "2"], ",", 
     SuperscriptBox["M", "2"], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[90]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[91]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "1"], "(", 
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"], ")"}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[91]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"B1", ",", 
    RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[92]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[93]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", 
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm], ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[93]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"], ",", "0"}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[94]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], "-", "2"}], ")"}]}], "-", 
   FractionBox["1", "2"]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[94]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{"0", ",", "0", ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[95]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], "-", "1"}], ")"}]}], "+", 
   FractionBox["1", "4"]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[95]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{"pp", ",", 
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
   SubsuperscriptBox["m", "2", "2"]}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[96]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["m", "2", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubscriptBox["B", "0"], "(", "pp", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}]}], 
    RowBox[{"2", " ", "pp"}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["B", "0"], "(", "pp", ",", 
     SubsuperscriptBox["m", "e", "2"], ",", 
     SubsuperscriptBox["m", "2", "2"], ")"}]}]}], TraditionalForm]], "Output",\

 CellTags->"B1",
 CellLabel->"Out[96]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B1", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", 
   RowBox[{"SmallVariable", "[", 
    SubsuperscriptBox["m", "e", "2"], "]"}], ",", "0"}], "]"}]], "Input",
 CellTags->"B1",
 CellLabel->"In[97]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SubsuperscriptBox["m", "e", "2"], ",", 
        SubsuperscriptBox["m", "e", "2"], ")"}]}], "-", "2"}], ")"}]}], "-", 
   FractionBox["1", "2"]}], TraditionalForm]], "Output",
 CellTags->"B1",
 CellLabel->"Out[97]="]
}, Open  ]]
}, Open  ]],

Cell["B11", "ObjectName",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"B11"],

Cell[TextData[{
 "B11[pp, ma^2, mb^2] is the Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["B", "11"], TraditionalForm]]],
 "-function, i.e., the coefficient function of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], " ", 
    SuperscriptBox["p", "\[Nu]"]}], TraditionalForm]]],
 ". All arguments are scalars and have dimension mass",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Null]", "2"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "B11", "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[98]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B11",
 CellLabel->"Out[98]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["B00",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B00", All, CellTags],
  ButtonData->"B00",
  ButtonNote->"B00"],
 ", ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B1", All, CellTags],
  ButtonData->"B1",
  ButtonNote->"B1"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 "."
}], "Text",
 CellTags->"B11"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"B11"],

Cell[TextData[{
 "Remember that SP[p] is a short hand input for ScalarProduct[p,p], i.e. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "2"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"B11"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", "p", "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[99]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[99]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[100]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{"3", " ", 
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm]}]], 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         SuperscriptBox["m", "2"], ",", 
         SuperscriptBox["M", "2"], ")"}]}], ")"}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["m", "2"], "-", 
        SuperscriptBox["M", "2"], "+", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{
          RowBox[{"(", 
           RowBox[{
            SuperscriptBox["M", "2"], "-", 
            SuperscriptBox["m", "2"]}], ")"}], " ", 
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             SubscriptBox["B", "0"], "(", 
             FormBox[
              SuperscriptBox[
               FormBox["p",
                TraditionalForm], "2"],
              TraditionalForm], ",", 
             SuperscriptBox["m", "2"], ",", 
             SuperscriptBox["M", "2"], ")"}], "-", 
            RowBox[{
             SubscriptBox["B", "0"], "(", "0", ",", 
             SuperscriptBox["m", "2"], ",", 
             SuperscriptBox["M", "2"], ")"}]}], ")"}]}], 
         RowBox[{"2", " ", 
          FormBox[
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"],
           TraditionalForm]}]], "-", 
        RowBox[{
         FractionBox["1", "2"], " ", 
         RowBox[{
          SubscriptBox["B", "0"], "(", 
          FormBox[
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"],
           TraditionalForm], ",", 
          SuperscriptBox["m", "2"], ",", 
          SuperscriptBox["M", "2"], ")"}]}]}], ")"}]}], "+", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", 
      FormBox[
       SuperscriptBox["M", "2"],
       TraditionalForm], ")"}], "+", 
     RowBox[{
      FractionBox["1", "6"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"3", " ", 
          SuperscriptBox["m", "2"]}]}], "-", 
        RowBox[{"3", " ", 
         SuperscriptBox["M", "2"]}], "+", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[100]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[101]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "-", 
       SuperscriptBox["m", "2"]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", 
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ")"}]}], "+", 
    RowBox[{
     FormBox[
      SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
      TraditionalForm], "(", 
     FormBox[
      SuperscriptBox["m", "2"],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{"(", 
      RowBox[{
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["m", "2"]}]}], ")"}]}]}], 
   RowBox[{"3", " ", 
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[101]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"B11", ",", 
   RowBox[{"BReduce", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[102]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B11",
 CellLabel->"Out[102]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SP", "[", "p", "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["M", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[103]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"B\"\>", "\<\"11\"\>"],
    TraditionalForm], "(", 
   FormBox[
    FormBox[
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"],
     TraditionalForm],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   RowBox[{",", " "}], 
   FormBox[
    SuperscriptBox["M", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[103]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"B11", ",", 
   RowBox[{"BReduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[104]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"BReduce", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"B11",
 CellLabel->"Out[104]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{"0", ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[105]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", "0", ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[105]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"B11", "[", 
  RowBox[{
   RowBox[{"SmallVariable", "[", 
    SuperscriptBox["M", "2"], "]"}], ",", 
   SuperscriptBox["m", "2"], ",", 
   SuperscriptBox["m", "2"]}], "]"}]], "Input",
 CellTags->"B11",
 CellLabel->"In[106]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{
    SubscriptBox["B", "0"], "(", 
    SuperscriptBox["M", "2"], ",", 
    SuperscriptBox["m", "2"], ",", 
    SuperscriptBox["m", "2"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"B11",
 CellLabel->"Out[106]="]
}, Open  ]]
}, Open  ]],

Cell["CA", "ObjectName",
 CellTags->"CA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CA"],

Cell[TextData[{
 "CA is one of the Casimir operator eigenvalues of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") (CA = ",
 StyleBox["N",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"CA"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CF", All, CellTags],
  ButtonData->"CF",
  ButtonNote->"CF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CA"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CA"],

Cell[CellGroupData[{

Cell[BoxData["CA"], "Input",
 CellTags->"CA",
 CellLabel->"In[107]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "A"], TraditionalForm]], "Output",
 CellTags->"CA",
 CellLabel->"Out[107]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"CA", ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CA",
 CellLabel->"In[108]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellTags->"CA",
 CellLabel->"Out[108]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
 CellTags->"CA",
 CellLabel->"In[109]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellTags->"CA",
 CellLabel->"Out[109]="]
}, Open  ]]
}, Open  ]],

Cell["Calc", "ObjectName",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Calc"],

Cell["\<\
Calc[exp] performs several basic simplifications. Calc[exp] is the same as \
DotSimplify[DiracSimplify[Contract[DiracSimplify[Explicit[ \
SUNSimplify[Trick[exp], Explicit\[Rule]False] ]]]]].\
\>", "Text",
 CellTags->"Calc"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Trick", All, CellTags],
  ButtonData->"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"Calc"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Calc"],

Cell[TextData[{
 "This calculates ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
 " in 4 dimensions and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
 " in D dimensions."
}], "Text",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}], ",", " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[110]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4", ",", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[110]="]
}, Open  ]],

Cell[TextData[{
 "This simplifies ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["f", "abc"], " ", 
    RowBox[{
     SubscriptBox["f", "abe"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Calc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "e"}], "]"}]}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[111]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[111]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[112]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], "\[Nu]"], ")"}], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"p", "+", "r"}],
      TraditionalForm], "\[Mu]"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[112]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[113]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[113]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "-", "q"}], ",", "3"}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[114]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li3",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox[
      RowBox[{
       RowBox[{"-", "p"}], "-", "q"}],
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[114]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{"%", " ", 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "li1"}], "]"}]}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"q", ",", "li2"}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"-", "p"}], "-", "q"}], ",", "li3"}], "]"}]}], "]"}]], "Input",
 CellTags->"Calc",
 CellLabel->"In[115]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Calc",
 CellLabel->"Out[115]="]
}, Open  ]]
}, Open  ]],

Cell["CalcColorFactor", "ObjectName",
 CellTags->"CalcColorFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CalcColorFactor"],

Cell["\<\
CalcColorFactor[expr] calculates the color factor of expr.  CalcColorFactor \
is useful for application on FeynArts produced amplitudes. CalcColorFactor is \
just a macro function for CalcColorFactor[x_] := 
If[FreeQ2[FeynCalcInternal[x], SUNIndex],  x, 
                    SUNSimplify[SUNSimplify[  (If[ !FreeQ[#1, DiracGamma], \
DiracTrick[#1], #1] & )[  SUNSimplify[x]], Explicit -> False], Explicit -> \
True]].\
\>", "Text",
 CellTags->"CalcColorFactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CalcColorFactor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalcColorFactor", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[116]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[116]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Amplitude", "[", "\"\<q2se2\>\"", "]"}]], "Input",
 CellLabel->"In[117]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{
      SubscriptBox["c", "1"], 
      SubscriptBox["c", "3"]}], 
     RowBox[{
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Lambda]", "3"]}]], "(", "q2", ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["\[CapitalPi]", 
     RowBox[{
      SubscriptBox["c", "9"], 
      SubscriptBox["c", "11"]}], 
     RowBox[{
      SubscriptBox["\[Lambda]", "9"], 
      SubscriptBox["\[Lambda]", "11"]}]], "(", "q2", ")"}], " ", 
   RowBox[{
    SubsuperscriptBox["Q", 
     SubscriptBox["c", "11"], 
     SubscriptBox["\[Lambda]", "11"]], ".", 
    RowBox[{
     SubscriptBox["\[CapitalPi]", "q"], "(", 
     FormBox[
      RowBox[{"p", "-", "q2"}],
      TraditionalForm], ")"}], ".", 
    SubsuperscriptBox["Q", 
     SubscriptBox["c", "1"], 
     SubscriptBox["\[Lambda]", "1"]]}], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{"2", " ", 
        SubscriptBox["T", "f"], " ", 
        RowBox[{
         RowBox[{
          SubscriptBox["\[CapitalPi]", "q"], "(", 
          FormBox[
           RowBox[{"-", "q1"}],
           TraditionalForm], ")"}], ".", 
         SubsuperscriptBox["Q", 
          SubscriptBox["c", "9"], 
          SubscriptBox["\[Lambda]", "9"]], ".", 
         RowBox[{
          SubscriptBox["\[CapitalPi]", "q"], "(", 
          FormBox[
           RowBox[{"q2", "-", "q1"}],
           TraditionalForm], ")"}], ".", 
         SubsuperscriptBox["Q", 
          SubscriptBox["c", "3"], 
          SubscriptBox["\[Lambda]", "3"]]}]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellLabel->"Out[117]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CalcColorFactor", "@", 
  RowBox[{"Explicit", "@", "%"}]}]], "Input",
 CellLabel->"In[118]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      FractionBox["1", 
       SuperscriptBox["q2", "2"]], ")"}], "2"], " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Lambda]", "1"], 
      SubscriptBox["\[Lambda]", "3"]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Lambda]", "9"], 
      SubscriptBox["\[Lambda]", "11"]}]], " ", 
    SubsuperscriptBox["g", "s", "4"], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Lambda]", "11"],
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", "(", 
       FormBox[
        RowBox[{"p", "-", "q2"}],
        TraditionalForm], ")"}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Lambda]", "1"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{"-", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q1",
           TraditionalForm]}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "9"],
           TraditionalForm],
          TraditionalForm]], ".", 
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", "(", 
          FormBox[
           RowBox[{"q1", "-", "q2"}],
           TraditionalForm], ")"}], ")"}], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Lambda]", "3"],
           TraditionalForm],
          TraditionalForm]]}]}],
      TraditionalForm], ")"}]}], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"p", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[118]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProductCancel", All, CellTags],
  ButtonData->"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Series2", All, CellTags],
  ButtonData->"Series2",
  ButtonNote->"Series2"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TID", All, CellTags],
  ButtonData->"TID",
  ButtonNote->"TID"],
 "."
}], "Text",
 CellTags->"Collecting"]
}, Open  ]],

Cell["CalculateCounterTerm ***unfinished***", "ObjectName",
 CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CalculateCounterTerm"],

Cell["CalculateCounterTerm[exp, k] calculates the residue of exp.", "Text",
 CellTags->"CalculateCounterTerm"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"CalculateCounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "CalculateCounterTerm"}]], "Input",
 CellTags->"CalculateCounterTerm",
 CellLabel->"In[119]:="],

Cell[BoxData[
 StyleBox["\<\"CalculateCounterTerm[exp, k] calculates the residue of \
exp.\"\>", "MSG"]], "Print", "PrintUsage",
 CellTags->{"CalculateCounterTerm", "Info3535453139-5096475"}]
}, Open  ]]
}, Open  ]],

Cell["CancelQP", "ObjectName",
 CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CancelQP"],

Cell[TextData[{
 "CancelQP is an option for OneLoop. If set to True, cancelation of all ",
 StyleBox["q",
  FontSlant->"Italic"],
 ".",
 StyleBox["p",
  FontSlant->"Italic"],
 "'s and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["q", "2"], TraditionalForm]]],
 " is performed."
}], "Text",
 CellTags->"CancelQP"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"CancelQP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{
      RowBox[{"p", "+", 
       RowBox[{"2", "q2"}]}], ",", 
      RowBox[{"p", "+", "q2"}]}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "M"}], "}"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[120]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "+", "q2"}],
     TraditionalForm], ")", "\[CenterDot]", "(", 
    FormBox[
     RowBox[{"p", "+", 
      RowBox[{"2", " ", "q2"}]}],
     TraditionalForm], ")"}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox["q2", "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[120]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[121]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{"3", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"3", " ", 
          SuperscriptBox["m", "2"]}], "-", 
         RowBox[{"7", " ", 
          SuperscriptBox["M", "2"]}]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"3", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["M", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox["M", "2"]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[121]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[122]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{"8", " ", 
        RowBox[{"PaVe", "(", 
         RowBox[{"0", ",", "0", ",", 
          RowBox[{"{", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], "}"}], ",", 
          RowBox[{"{", 
           RowBox[{"0", ",", 
            SuperscriptBox["m", "2"]}], "}"}]}], ")"}]}], 
       SuperscriptBox["M", "2"]]}], "+", 
     FractionBox[
      RowBox[{"8", " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"0", ",", "0", ",", 
         RowBox[{"{", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox["m", "2"], ",", 
           SuperscriptBox["M", "2"]}], "}"}]}], ")"}]}], 
      SuperscriptBox["M", "2"]], "+", 
     FractionBox[
      RowBox[{"17", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"17", " ", 
          SuperscriptBox["m", "2"]}], "-", 
         RowBox[{"13", " ", 
          SuperscriptBox["M", "2"]}]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"19", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"19", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}], ")"}], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox["M", "2"]}], ")"}], "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"17", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["M", "2"], ")"}]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     RowBox[{
      FractionBox["17", "6"], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", "0", ",", 
       SuperscriptBox["m", "2"], ",", 
       SuperscriptBox["M", "2"], ")"}]}], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", "0", ",", 
        SuperscriptBox["m", "2"], ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["m", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox["M", "2"],
         TraditionalForm], ")"}]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"]}]], "+", 
     FractionBox[
      RowBox[{"2", " ", 
       SuperscriptBox["m", "4"]}], 
      RowBox[{"3", " ", 
       SuperscriptBox["M", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "-", 
     FractionBox[
      RowBox[{"13", " ", 
       SuperscriptBox["m", "2"]}], 
      RowBox[{"6", " ", 
       SuperscriptBox["M", "2"]}]], "-", 
     FractionBox["4", "3"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[122]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q2", ",", "p"}], "]"}], 
    RowBox[{"SPD", "[", 
     RowBox[{"q1", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q1", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "m"}], "}"}], ",", 
      RowBox[{"q1", "-", "p"}], ",", 
      RowBox[{"q2", "-", "p"}], ",", 
      RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[123]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[123]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[124]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q1",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
          TraditionalForm], "(", 
         FormBox[
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         FormBox[
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"],
          TraditionalForm], ",", 
         FormBox[
          RowBox[{
           RowBox[{"-", 
            RowBox[{"2", " ", 
             RowBox[{
              FormBox["p",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q1",
               TraditionalForm]}]}]}], "+", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], "+", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"]}],
          TraditionalForm], ",", 
         FormBox["0",
          TraditionalForm], ",", 
         FormBox[
          SuperscriptBox["m", "2"],
          TraditionalForm], ",", 
         FormBox["0",
          TraditionalForm], ")"}]}], 
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["m", "2"], "-", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{
             FormBox["p",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q1",
              TraditionalForm]}]}]}], "+", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "+", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}], ")"}]}]]}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"],
         TraditionalForm], ",", 
        FormBox[
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{
             FormBox["p",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q1",
              TraditionalForm]}]}]}], "+", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], "2"], "+", 
          SuperscriptBox[
           FormBox["q1",
            TraditionalForm], "2"]}],
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ",", 
        FormBox[
         SuperscriptBox["m", "2"],
         TraditionalForm], ",", 
        FormBox["0",
         TraditionalForm], ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["q1",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "-", 
     FractionBox[
      RowBox[{
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ",", "0", ",", "0", ")"}]}], 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[124]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q2", ",", "t", ",", 
   RowBox[{"CancelQP", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[125]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q1",
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"2", ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], ",", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"2", " ", 
              RowBox[{
               FormBox["p",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q1",
                TraditionalForm]}]}]}], "+", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"], "+", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], "2"]}], ",", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox["m", "2"], ",", "0", ",", "0"}], "}"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], " ", 
       RowBox[{"PaVe", "(", 
        RowBox[{"1", ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], ",", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"2", " ", 
              RowBox[{
               FormBox["p",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q1",
                TraditionalForm]}]}]}], "+", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], "2"], "+", 
            SuperscriptBox[
             FormBox["q1",
              TraditionalForm], "2"]}], ",", 
           SuperscriptBox[
            FormBox["q1",
             TraditionalForm], "2"]}], "}"}], ",", 
         RowBox[{"{", 
          RowBox[{
           SuperscriptBox["m", "2"], ",", "0", ",", "0"}], "}"}]}], ")"}]}], 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["m", "2"], "-", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"2", " ", 
           RowBox[{
            FormBox["p",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q1",
             TraditionalForm]}]}]}], "+", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], "+", 
         SuperscriptBox[
          FormBox["q1",
           TraditionalForm], "2"]}], ")"}]}]]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"CancelQP",
 CellLabel->"Out[125]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"CancelQP",
 CellLabel->"In[126]:="]
}, Open  ]],

Cell["Cases2", "ObjectName",
 CellTags->"Cases2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Cases2"],

Cell["\<\
Cases2[expr, f] returns a list of all objects in expr with head f. \
Cases2[expr,f] is equivalent to Cases2[{expr},f[___],Infinity]//Union. \
Cases2[expr, f, g, ...] or Cases2[expr, {f,g, ...}] is equivalent to \
Cases[{expr},f[___] | g[___] ...] .\
\>", "Text",
 CellTags->"Cases2"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Cases2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "a", "]"}], "+", 
    SuperscriptBox[
     RowBox[{"f", "[", "b", "]"}], "2"], "+", 
    RowBox[{"f", "[", 
     RowBox[{"c", ",", "d"}], "]"}]}], ",", "f"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[127]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"f", "(", "a", ")"}], ",", 
    RowBox[{"f", "(", "b", ")"}], ",", 
    RowBox[{"f", "(", 
     RowBox[{"c", ",", "d"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[127]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"y", "-", "z"}], "]"}]}], "+", 
    RowBox[{"g", "[", "y", "]"}]}], ",", "Sin", ",", "g"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[128]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"g", "(", "y", ")"}], ",", 
    RowBox[{"sin", "(", "x", ")"}], ",", 
    RowBox[{"sin", "(", 
     RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[128]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Sin", "[", "x", "]"}], " ", 
     RowBox[{"Sin", "[", 
      RowBox[{"y", "-", "z"}], "]"}]}], "+", 
    RowBox[{"g", "[", "x", "]"}], "+", 
    RowBox[{"g", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"Sin", ",", "g"}], "}"}]}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[129]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"g", "(", "x", ")"}], ",", 
    RowBox[{"g", "(", 
     RowBox[{"a", ",", "b", ",", "c"}], ")"}], ",", 
    RowBox[{"sin", "(", "x", ")"}], ",", 
    RowBox[{"sin", "(", 
     RowBox[{"y", "-", "z"}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[129]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
     RowBox[{"DiracSlash", "[", "q", "]"}]}], "+", 
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p", ",", "p"}], "]"}]}], ",", "Dot"}], "]"}]], "Input",
 CellTags->"Cases2",
 CellLabel->"In[130]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Cases2",
 CellLabel->"Out[130]="]
}, Open  ]]
}, Open  ]],

Cell["CF", "ObjectName",
 CellTags->"CF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CF"],

Cell[TextData[{
 "CF is one of the Casimir operator eigenvalues of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") (CF = (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["N", "2"], TraditionalForm]]],
 "-1)/(2 ",
 StyleBox["N",
  FontSlant->"Italic"],
 "))."
}], "Text",
 CellTags->"CF"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CA", All, CellTags],
  ButtonData->"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"CF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CF"],

Cell[CellGroupData[{

Cell[BoxData["CF"], "Input",
 CellTags->"CF",
 CellLabel->"In[131]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "F"], TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[131]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"CF", ",", " ", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"CF",
 CellLabel->"In[132]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["N", "2"], "-", "1"}], 
   RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[132]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[133]:="],

Cell["(-1 + SUNN^2)/(2*SUNN)", "Output",
 CellLabel->"Out[133]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["SUNN"], "Input",
 CellTags->"CF",
 CellLabel->"In[134]:="],

Cell[BoxData[
 FormBox["N", TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[134]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNN", "^", "2"}], "-", "1"}], ",", " ", 
   RowBox[{"SUNNToCACF", " ", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CF",
 CellLabel->"In[135]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"CF",
 CellLabel->"Out[135]="]
}, Open  ]]
}, Open  ]],

Cell["ChangeDimension", "ObjectName",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChangeDimension"],

Cell["\<\
ChangeDimension[exp, dim] changes all LorentzIndex and Momenta in exp to \
dimension dim (and also Levi-Civita-tensors, Dirac slashes and Dirac \
matrices).\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LorentzIndex", All, CellTags],
  ButtonData->"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Momentum", All, CellTags],
  ButtonData->"Momentum",
  ButtonNote->"Momentum"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Eps", All, CellTags],
  ButtonData->"Eps",
  ButtonNote->"Eps"],
 "."
}], "Text",
 CellTags->"ChangeDimension"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChangeDimension"],

Cell["\<\
Remember that LorentzIndex[mu, 4] is simplified to LorentzIndex[mu] and \
Momentum[p, 4] to Momentum[p]. Thus the fullowing objects are defined in four \
dimensions.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", " ", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[136]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\[Mu]",
     TraditionalForm], ",", "p"}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[136]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChangeDimension", "[", 
  RowBox[{"%", ",", " ", "D"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[137]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox["\[Mu]",
     TraditionalForm], ",", "p"}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[137]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[138]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "}"}]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[138]//StandardForm="]
}, Open  ]],

Cell["\<\
This changes all non-4-dimensional objects to 4-dimensional ones.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"%%", ",", " ", "4"}], "]"}], " ", "//", " ", 
  "StandardForm"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[139]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "}"}]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[139]//StandardForm="]
}, Open  ]],

Cell["Consider the following list of 4- and D-dimensional object.", "Text",
 CellTags->"ChangeDimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"MT", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
   RowBox[{
    RowBox[{"GAD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"f", "[", "D", "]"}]}]}], "}"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[140]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], ",", 
    RowBox[{
     RowBox[{"f", "(", "D", ")"}], " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[140]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", 
  RowBox[{"Contract", "[", "%", "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[141]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4", ",", 
    RowBox[{"D", " ", 
     RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[141]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", 
  RowBox[{"Contract", "[", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%%", ",", "n"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[142]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"n", ",", 
    RowBox[{"n", " ", 
     RowBox[{"f", "(", "D", ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[142]="]
}, Open  ]],

Cell["\<\
Any explicit occurence of D (like in f(D)) is not replaced by ChangeDimension.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[143]:="],

Cell["\<\
The option Dimension of Eps must be changed too, since with the default \
setting Dimension\[Rule]4 the arguments of Eps are automatically changed to 4 \
dimensions.\
\>", "Text",
 CellTags->"ChangeDimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[144]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", " ", 
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
   "]"}]}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[145]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[145]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"a1", ",", "4"}], "]"}]}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[146]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[146]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Contract", "[", 
   SuperscriptBox["a1", "2"], "]"}], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[147]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[147]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  SuperscriptBox["a2", "2"], "]"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[148]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "24"}], TraditionalForm]], "Output",
 CellTags->"ChangeDimension",
 CellLabel->"Out[148]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[149]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"a1", ",", "a2"}], "]"}], ";"}]], "Input",
 CellTags->"ChangeDimension",
 CellLabel->"In[150]:="]
}, Open  ]],

Cell["ChargeConjugationMatrix", "ObjectName",
 CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
 "ChargeConjugationMatrix denotes the charge conjugation matrix ",
 StyleBox["C",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrix"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChargeConjugationMatrixInv",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ChargeConjugationMatrixInv", All, CellTags],
  ButtonData->"ChargeConjugationMatrixInv",
  ButtonNote->"ChargeConjugationMatrixInv"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[151]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[151]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[152]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "T"]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[152]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[153]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[153]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[154]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox["5",
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[154]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", "^", "2"}]], "Input",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"In[155]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrix",
 CellLabel->"Out[155]="]
}, Open  ]]
}, Open  ]],

Cell["ChargeConjugationMatrixInv", "ObjectName",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],

Cell["\<\
ChargeConjugationMatrixInv is the inverse of ChargeConjugationMatrix. It is \
substituted immediately by -ChargeConjugationMatrix.\
\>", "Text",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChargeConjugationMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ChargeConjugationMatrix", All, CellTags],
  ButtonData->"ChargeConjugationMatrix",
  ButtonNote->"ChargeConjugationMatrix"],
 "."
}], "Text",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[156]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[156]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[157]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "T"]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[157]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  RowBox[{"DiracGamma", "[", "5", "]"}], ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[158]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[158]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[159]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox["5",
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[159]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrix", ".", 
  "ChargeConjugationMatrixInv"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[160]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"C", ".", 
   RowBox[{"(", 
    RowBox[{"-", "C"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[160]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Calc"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[161]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"C", ".", "C"}]}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[161]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ChargeConjugationMatrixInv", "^", "2"}]], "Input",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"In[162]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"ChargeConjugationMatrixInv",
 CellLabel->"Out[162]="]
}, Open  ]]
}, Open  ]],

Cell["CheckContext", "ObjectName",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CheckContext"],

Cell["\<\
CheckContext[string] yields True if the packaged associated with string is \
already loaded, and False otherwise.\
\>", "Text",
 CellTags->"CheckContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MakeContext",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MakeContext", All, CellTags],
  ButtonData->"MakeContext",
  ButtonNote->"MakeContext"],
 ", ",
 ButtonBox["$FeynCalcStuff",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$FeynCalcStuff", All, CellTags],
  ButtonData->"$FeynCalcStuff",
  ButtonNote->"$FeynCalcStuff"],
 "."
}], "Text",
 CellTags->"CheckContext"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CheckContext"],

Cell["\<\
If a FeynCalc symbol has not been used, the corresponding subpackage has not \
been loaded:\
\>", "Text",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[163]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"CheckContext",
 CellLabel->"Out[163]="]
}, Open  ]],

Cell["Using it makes FeynCalc load the subpackage:", "Text",
 CellTags->"CheckContext"],

Cell[CellGroupData[{

Cell[BoxData["CompleteSquare"], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[164]:="],

Cell[BoxData[
 FormBox["CompleteSquare", TraditionalForm]], "Output",
 CellTags->"CheckContext",
 CellLabel->"Out[164]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckContext", "[", "\"\<CompleteSquare\>\"", "]"}]], "Input",
 CellTags->"CheckContext",
 CellLabel->"In[165]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"CheckContext",
 CellLabel->"Out[165]="]
}, Open  ]]
}, Open  ]],

Cell["CheckDB", "ObjectName",
 CellTags->"CheckDB"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 GeneratedCell->True,
 CellAutoOverwrite->True,
 CellTags->"CheckDB"],

Cell["\<\
CheckDB[exp, fil] saves [with Put] or retrieves [with Get] exp from a file \
fil. It checks if the setting of the option Directory is a valid directory \
name and if fil is a valid file name and does exist. If it does, Get[fil] is \
executed. If fil does not exist, exp gets evaluated and saved to  fil. Saving \
and evaluating can be further controlled with the options ForceSave and \
NoSave. If the option Check is set to False  the return value is what is \
evaluated [see above]. If Check is set to True the return value is True or \
False depending on whether the evaluation of exp agrees with what is loaded \
from fil or fil does not exist.  Default value of Check : False. NOTICE : If \
fil ends with \".Gen\" or \".Mod\", the   setting of Directory is ignored and \
fil is  saved in the \"CouplingVectors\"  subdirectory of \"Phi\". If fil \
ends with  \".Fac\", the setting of Directory is  ignored and fil is saved in \
the \"Factors\" subdirectory of \"Phi\". If fil is a file   name with full \
path, the setting of  Directory is also ignored.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "CheckDB", "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[166]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Directory", "\[RuleDelayed]", 
     RowBox[{"ToFileName", "[", 
      RowBox[{
       RowBox[{"{", "$FeynCalcDirectory", "}"}], ",", "\<\"fcdb\"\>"}], 
      "]"}]}], ",", 
    RowBox[{"ForceSave", "\[Rule]", "False"}], ",", 
    RowBox[{"NoSave", "\[Rule]", "False"}], ",", 
    RowBox[{"Check", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[166]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "CheckDB", "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[167]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "HoldFirst", "}"}], TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[167]="]
}, Open  ]],

Cell["\<\
The first time the Table function is evaluated and the result saved into the \
test.s file.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckDB", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Print", "[", 
       RowBox[{"\"\<test \>\"", ",", "i"}], "]"}], ";", "i"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], ",", "\"\<test.s\>\""}], 
  "]"}]], "Input",
 CellLabel->"In[168]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"test \"\>", "\[InvisibleSpace]", "1"}],
   SequenceForm["test ", 1],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"test \"\>", "\[InvisibleSpace]", "2"}],
   SequenceForm["test ", 2],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]], "Output",
 CellLabel->"Out[168]="]
}, Open  ]],

Cell["\<\
Executing the same a second time will just load the result from test.s and \
not evaluate the Table function.\
\>", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CheckDB", "[", 
  RowBox[{
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"WriteString", "[", 
       RowBox[{"\"\<stdout\>\"", ",", "\"\<test\>\""}], "]"}], ";", "i"}], 
     ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "2"}], "}"}]}], "]"}], ",", "\"\<test.s\>\""}], 
  "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[169]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "2"}], "}"}], TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[169]="]
}, Open  ]],

Cell["This shows the actual saved value of test.s.", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Import", "[", 
  RowBox[{
   RowBox[{"ToFileName", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"$FeynCalcDirectory", ",", "\"\<fcdb\>\""}], "}"}], ",", 
     "\"\<test.s\>\""}], "]"}], ",", "\"\<Text\>\""}], "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[170]:="],

Cell[BoxData[
 FormBox["\<\"{1, 2}\"\>", TraditionalForm]], "Output",
 CellTags->"CheckDB",
 CellLabel->"Out[170]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{"ToFileName", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"$FeynCalcDirectory", ",", "\"\<fcdb\>\""}], "}"}], ",", 
    "\"\<test.s\>\""}], "]"}], "]"}]], "Input",
 CellTags->"CheckDB",
 CellLabel->"In[171]:="]
}, Open  ]],

Cell["ChiralityProjector", "ObjectName",
 CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChiralityProjector"],

Cell[TextData[{
 "ChiralityProjector[+1] denotes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
 ". ChiralityProjector[-1] denotes ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"ChiralityProjector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 "."
}], "Text",
 CellTags->"ChiralityProjector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChiralityProjector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ChiralityProjector", "[", 
    RowBox[{"+", "1"}], "]"}], ",", 
   RowBox[{"ChiralityProjector", "[", 
    RowBox[{"-", "1"}], "]"}]}], "}"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[172]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "6"], ",", 
    SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChiralityProjector",
 CellLabel->"Out[172]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCI", "[", "%", "]"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[173]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "6"], ",", 
    SuperscriptBox["\[Gamma]", "7"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ChiralityProjector",
 CellLabel->"Out[173]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{"%", ",", 
   RowBox[{"DiracSubstitute67", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ChiralityProjector",
 CellLabel->"In[174]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FractionBox[
      SuperscriptBox["\[Gamma]", "5"], "2"], "+", 
     FractionBox["1", "2"]}], ",", 
    RowBox[{
     FractionBox["1", "2"], "-", 
     FractionBox[
      SuperscriptBox["\[Gamma]", "5"], "2"]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ChiralityProjector",
 CellLabel->"Out[174]="]
}, Open  ]]
}, Open  ]],

Cell["Chisholm", "ObjectName",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Chisholm"],

Cell["\<\
Chisholm[x] substitutes products of three Dirac matrices or slashes by the \
Chisholm identity.\
\>", "Text",
 CellTags->"Chisholm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "EpsChisholm", All, CellTags],
  ButtonData->"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 ", ",
 ButtonBox["ChisholmSpinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ChisholmSpinor", All, CellTags],
  ButtonData->"ChisholmSpinor",
  ButtonNote->"ChisholmSpinor"],
 "."
}], "Text",
 CellTags->"Chisholm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[175]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["$MU$2349",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["$MU$2349",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Mu]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Beta]", "\[Mu]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[175]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[176]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[176]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", 
  RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[177]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2384",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2384",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[177]="]
}, Open  ]],

Cell["The $MU$ variables are unique indices.", "Text",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t1", ".", "t1"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[178]:="],

Cell[BoxData[
 FormBox["16", TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[178]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", " ", "=", 
  RowBox[{"Chisholm", "[", "t1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[179]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2416",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2416",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[179]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t2", ".", "t3"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[180]:="],

Cell[BoxData[
 FormBox["16", TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[180]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"DiracSlash", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[181]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["a",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[181]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", "t4", "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[182]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     FormBox["a",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}]}], "-", 
   RowBox[{
    RowBox[{
     FormBox["a",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["b",
      TraditionalForm]}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["a",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["b",
      TraditionalForm], "\[CenterDot]", 
     FormBox["c",
      TraditionalForm]}]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2456",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["$MU$2456",
        TraditionalForm],
       TraditionalForm], 
      FormBox["a",
       TraditionalForm], 
      FormBox["b",
       TraditionalForm], 
      FormBox["c",
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[182]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{
   "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", ",", 
    "\[Kappa]"}], "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[183]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Tau]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Kappa]"]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[183]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", 
  RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[184]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Tau]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Kappa]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2477",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2477",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2499",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2499",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2510",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2510",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2521",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2521",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[184]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", " ", "=", 
  RowBox[{"Chisholm", "[", "a1", "]"}]}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[185]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Tau]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Kappa]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2605",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2605",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2627",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2627",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2638",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2638",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["$MU$2649",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Kappa]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Tau]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["$MU$2649",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[185]="]
}, Open  ]],

Cell["Check that both a1.a1 and a2.a3 give the same.", "Text",
 CellTags->"Chisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"a1", ".", "a1"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[186]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "2048"}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[186]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"a2", ".", "a3"}], "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[187]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "2048"}], TraditionalForm]], "Output",
 CellTags->"Chisholm",
 CellLabel->"Out[187]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "t1", ",", "t2", ",", "t3", ",", "t4", ",", "a1", ",", "a2", ",", "a3"}], 
  "]"}]], "Input",
 CellTags->"Chisholm",
 CellLabel->"In[188]:="]
}, Open  ]],

Cell["ChisholmSpinor", "ObjectName",
 CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ChisholmSpinor"],

Cell["\<\
ChisholmSpinor[x] uses the Chisholm identity on a DiraGamma between spinors. \
As an optional second argument 1 or 2 may be given, indicating that \
ChisholmSpinor should only act on the first resp. second part of a product of \
spinor chains.\
\>", "Text",
 CellTags->"ChisholmSpinor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "EpsChisholm", All, CellTags],
  ButtonData->"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 ", ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Chisholm", All, CellTags],
  ButtonData->"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"ChisholmSpinor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ChisholmSpinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p1", ",", "m1"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ".", 
  RowBox[{"Spinor", "[", 
   RowBox[{"p2", ",", "m2"}], "]"}]}]], "Input",
 CellTags->"ChisholmSpinor",
 CellLabel->"In[189]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m1", ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p2",
     TraditionalForm], ",", "m2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ChisholmSpinor",
 CellLabel->"Out[189]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "ChisholmSpinor"}]], "Input",
 CellTags->"ChisholmSpinor",
 CellLabel->"In[190]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["alpha21",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox["p1",
        TraditionalForm], 
       FormBox["p2",
        TraditionalForm]}]], " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["alpha21",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "-", 
   FractionBox[
    RowBox[{"m1", " ", "m2", " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"m2", " ", 
     SuperscriptBox[
      FormBox["p1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"m1", " ", 
     SuperscriptBox[
      FormBox["p2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p1",
        TraditionalForm], ",", "m1", ")"}], ".", 
      RowBox[{"\[CurlyPhi]", "(", 
       FormBox["p2",
        TraditionalForm], ",", "m2", ")"}]}]}], 
    RowBox[{
     FormBox["p1",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p2",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"ChisholmSpinor",
 CellLabel->"Out[190]="]
}, Open  ]]
}, Open  ]],

Cell["ClearScalarProducts", "ObjectName",
 CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ClearScalarProducts"],

Cell["\<\
ClearScalarProducts removes all user-performed specific settings for \
ScalarProduct's.\
\>", "Text",
 CellTags->"ClearScalarProducts"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProduct", All, CellTags],
  ButtonData->"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SP", All, CellTags],
  ButtonData->"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SPD", All, CellTags],
  ButtonData->"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ClearScalarProducts"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ClearScalarProducts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "p"}], "]"}], "=", 
  RowBox[{"m", "^", "2"}]}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[191]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["m", "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[191]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[192]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["m", "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[192]="]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[193]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[194]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[194]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "p"}], "]"}]], "Input",
 CellTags->"ClearScalarProducts",
 CellLabel->"In[195]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ClearScalarProducts",
 CellLabel->"Out[195]="]
}, Open  ]]
}, Open  ]],

Cell["Collecting", "ObjectName",
 CellTags->"Collecting"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collecting"],

Cell["\<\
Collecting is an option of ScalarProductCancel, Series2, TID and related \
functions. Setting it to True will trigger some kind of collecting of the \
result.\
\>", "Text",
 CellTags->"Collecting"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProductCancel", All, CellTags],
  ButtonData->"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Series2", All, CellTags],
  ButtonData->"Series2",
  ButtonNote->"Series2"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TID", All, CellTags],
  ButtonData->"TID",
  ButtonNote->"TID"],
 "."
}], "Text",
 CellTags->"Collecting"]
}, Open  ]],

Cell["Collect2", "ObjectName",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collect2"],

Cell["\<\
Collect2[expr, x] collects together terms which are not free of any \
occurrence of x. Collect2[expr, {x1, x2, ...}] (or also Collect2[expr, x1, \
x2, ...]) collects together terms which are not free of any occurrence of x1, \
x2, .... The coefficients are put over a common denominator. If expr is \
expanded before collecting depends on the option Factoring, which may be set \
to Factor, Factor2, or any other function, which is applied to the \
coefficients. If expr is already expanded with respect to x (x1,x2, ...), the \
option Expanding can be set to False.\
\>", "Text",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Collect2", "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[196]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Denominator", "\[Rule]", "False"}], ",", 
    RowBox[{"Dot", "\[Rule]", "False"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "Factor"}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[196]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"Collect2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Collect2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"t1", "=", 
    RowBox[{"a", "+", 
     RowBox[{"r", " ", "a"}], "+", 
     RowBox[{
      SuperscriptBox["k", "2"], " ", 
      RowBox[{"f", "[", "a", "]"}]}], "-", 
     RowBox[{"k", " ", 
      RowBox[{"f", "[", "a", "]"}]}], "+", 
     FractionBox["x", "2"], "-", 
     FractionBox["y", "w"]}]}], ",", "a"}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[197]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "k", " ", 
    RowBox[{"f", "(", "a", ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"w", " ", "x"}], "-", 
     RowBox[{"2", " ", "y"}]}], 
    RowBox[{"2", " ", "w"}]]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[197]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{"t1", ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[198]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", "k"}], ")"}], " ", 
    RowBox[{"f", "(", "a", ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "-", 
   FractionBox["y", "w"], "+", 
   FractionBox["x", "2"]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[198]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{"t1", ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "Factor"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[199]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"k", "-", "1"}], ")"}], " ", "k", " ", 
    RowBox[{"f", "(", "a", ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{"r", "+", "1"}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"w", " ", "x"}], "-", 
     RowBox[{"2", " ", "y"}]}], 
    RowBox[{"2", " ", "w"}]]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[199]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", "a", " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "a"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}], "-", 
    RowBox[{
     SuperscriptBox["b", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
    SuperscriptBox["b", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[200]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"2", " ", 
     SuperscriptBox["a", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}]}], "-", 
   RowBox[{"a", " ", 
    SuperscriptBox["b", "2"], " ", "e"}], "+", 
   RowBox[{"2", " ", "a", " ", "b", " ", 
    RowBox[{"(", 
     RowBox[{"h", "-", "1"}], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["b", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"c", "+", "1"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[200]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}], ",", 
   "a", ",", 
   RowBox[{"IsolateNames", "\[Rule]", "L"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[201]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["a", "4"], " ", 
    TagBox[
     RowBox[{"L", "(", "22", ")"}],
     HoldForm]}], "+", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "3"], " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "22", ")"}],
      HoldForm], "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "22", ")"}],
      HoldForm], "3"]}], "+", 
   RowBox[{"5", " ", "a", " ", 
    SuperscriptBox[
     TagBox[
      RowBox[{"L", "(", "22", ")"}],
      HoldForm], "4"]}], "-", 
   SuperscriptBox[
    TagBox[
     RowBox[{"L", "(", "22", ")"}],
     HoldForm], "5"]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[201]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[202]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["a", "4"], " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}]}], "+", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "3"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "3"]}], "+", 
   RowBox[{"5", " ", "a", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "4"]}], "-", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}], "5"]}], 
  TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[202]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "L"}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[203]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect2", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c"}], ")"}], "3"], "]"}], ",", "a", ",", 
   RowBox[{"Factoring", "\[Rule]", "fun"}]}], "]"}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[204]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["a", "3"], " ", 
    RowBox[{"fun", "(", "1", ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"fun", "(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"3", " ", "b"}]}], "-", 
      RowBox[{"3", " ", "c"}]}], ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"fun", "(", 
     RowBox[{
      RowBox[{"3", " ", 
       SuperscriptBox["b", "2"]}], "+", 
      RowBox[{"6", " ", "b", " ", "c"}], "+", 
      RowBox[{"3", " ", 
       SuperscriptBox["c", "2"]}]}], ")"}]}], "+", 
   RowBox[{"fun", "(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["b", "3"]}], "-", 
     RowBox[{"3", " ", 
      SuperscriptBox["b", "2"], " ", "c"}], "-", 
     RowBox[{"3", " ", "b", " ", 
      SuperscriptBox["c", "2"]}], "-", 
     SuperscriptBox["c", "3"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[204]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"fun", "\[Rule]", "FactorTerms"}]}]], "Input",
 CellTags->"Collect2",
 CellLabel->"In[205]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "3"], "-", 
   RowBox[{"3", " ", 
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}]}], "+", 
   RowBox[{"3", " ", "a", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      RowBox[{"2", " ", "b", " ", "c"}], "+", 
      SuperscriptBox["c", "2"]}], ")"}]}], "-", 
   SuperscriptBox["b", "3"], "-", 
   RowBox[{"3", " ", 
    SuperscriptBox["b", "2"], " ", "c"}], "-", 
   RowBox[{"3", " ", "b", " ", 
    SuperscriptBox["c", "2"]}], "-", 
   SuperscriptBox["c", "3"]}], TraditionalForm]], "Output",
 CellTags->"Collect2",
 CellLabel->"Out[205]="]
}, Open  ]]
}, Open  ]],

Cell["Collect3", "ObjectName",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Collect3"],

Cell[TextData[{
 "Collect3[expr, {x, y, ...}] collects terms involving the same powers of \
monomials ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["x",
     FontSlant->"Plain"], 
    SubscriptBox[
     StyleBox["n",
      FontSlant->"Plain"], "1"]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["y",
     FontSlant->"Plain"], 
    SubscriptBox[
     StyleBox["n",
      FontSlant->"Plain"], "2"]], TraditionalForm]]],
 " ... An option Factor \[Rule] True/False can be  given, which factors the \
coefficients. The option Head (default Plus) determines the applied function \
to the list of monomials  mulitplied by their coefficients."
}], "Text",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Collect3", "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[206]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"Head", "\[Rule]", "Plus"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Collect3",
 CellLabel->"Out[206]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Collect2", All, CellTags],
  ButtonData->"Collect2",
  ButtonNote->"Collect2"],
 ", ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"Collect3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Collect3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"2", " ", "a", " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "a"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"h", "-", "1"}], ")"}]}], "-", 
    RowBox[{
     SuperscriptBox["b", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"e", " ", "a"}], "-", "c"}], ")"}]}], "+", 
    SuperscriptBox["b", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[207]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"2", "-", 
      RowBox[{"2", " ", "h"}]}], ")"}]}], "-", 
   RowBox[{"a", " ", 
    SuperscriptBox["b", "2"], " ", "e"}], "+", 
   RowBox[{"a", " ", "b", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "h"}], "-", "2"}], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["b", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"c", "+", "1"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Collect3",
 CellLabel->"Out[207]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect3", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"Collect3",
 CellLabel->"In[208]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "+", 
   RowBox[{
    SuperscriptBox["a", "4"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"5", " ", "b"}]}], "-", 
      RowBox[{"5", " ", "c"}], "-", 
      RowBox[{"5", " ", "d"}]}], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["a", "3"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"10", " ", 
       SuperscriptBox["b", "2"]}], "+", 
      RowBox[{"20", " ", "b", " ", "c"}], "+", 
      RowBox[{"20", " ", "b", " ", "d"}], "+", 
      RowBox[{"10", " ", 
       SuperscriptBox["c", "2"]}], "+", 
      RowBox[{"20", " ", "c", " ", "d"}], "+", 
      RowBox[{"10", " ", 
       SuperscriptBox["d", "2"]}]}], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"10", " ", 
        SuperscriptBox["b", "3"]}]}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", "c"}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", "d"}], "-", 
      RowBox[{"30", " ", "b", " ", 
       SuperscriptBox["c", "2"]}], "-", 
      RowBox[{"60", " ", "b", " ", "c", " ", "d"}], "-", 
      RowBox[{"30", " ", "b", " ", 
       SuperscriptBox["d", "2"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["c", "3"]}], "-", 
      RowBox[{"30", " ", 
       SuperscriptBox["c", "2"], " ", "d"}], "-", 
      RowBox[{"30", " ", "c", " ", 
       SuperscriptBox["d", "2"]}], "-", 
      RowBox[{"10", " ", 
       SuperscriptBox["d", "3"]}]}], ")"}]}], "+", 
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"5", " ", 
       SuperscriptBox["b", "4"]}], "+", 
      RowBox[{"20", " ", 
       SuperscriptBox["b", "3"], " ", "c"}], "+", 
      RowBox[{"20", " ", 
       SuperscriptBox["b", "3"], " ", "d"}], "+", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["c", "2"]}], "+", 
      RowBox[{"60", " ", 
       SuperscriptBox["b", "2"], " ", "c", " ", "d"}], "+", 
      RowBox[{"30", " ", 
       SuperscriptBox["b", "2"], " ", 
       SuperscriptBox["d", "2"]}], "+", 
      RowBox[{"20", " ", "b", " ", 
       SuperscriptBox["c", "3"]}], "+", 
      RowBox[{"60", " ", "b", " ", 
       SuperscriptBox["c", "2"], " ", "d"}], "+", 
      RowBox[{"60", " ", "b", " ", "c", " ", 
       SuperscriptBox["d", "2"]}], "+", 
      RowBox[{"20", " ", "b", " ", 
       SuperscriptBox["d", "3"]}], "+", 
      RowBox[{"5", " ", 
       SuperscriptBox["c", "4"]}], "+", 
      RowBox[{"20", " ", 
       SuperscriptBox["c", "3"], " ", "d"}], "+", 
      RowBox[{"30", " ", 
       SuperscriptBox["c", "2"], " ", 
       SuperscriptBox["d", "2"]}], "+", 
      RowBox[{"20", " ", "c", " ", 
       SuperscriptBox["d", "3"]}], "+", 
      RowBox[{"5", " ", 
       SuperscriptBox["d", "4"]}]}], ")"}]}], "-", 
   SuperscriptBox["b", "5"], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["b", "4"], " ", "c"}], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["b", "4"], " ", "d"}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["b", "3"], " ", 
    SuperscriptBox["c", "2"]}], "-", 
   RowBox[{"20", " ", 
    SuperscriptBox["b", "3"], " ", "c", " ", "d"}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["b", "3"], " ", 
    SuperscriptBox["d", "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["b", "2"], " ", 
    SuperscriptBox["c", "3"]}], "-", 
   RowBox[{"30", " ", 
    SuperscriptBox["b", "2"], " ", 
    SuperscriptBox["c", "2"], " ", "d"}], "-", 
   RowBox[{"30", " ", 
    SuperscriptBox["b", "2"], " ", "c", " ", 
    SuperscriptBox["d", "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["b", "2"], " ", 
    SuperscriptBox["d", "3"]}], "-", 
   RowBox[{"5", " ", "b", " ", 
    SuperscriptBox["c", "4"]}], "-", 
   RowBox[{"20", " ", "b", " ", 
    SuperscriptBox["c", "3"], " ", "d"}], "-", 
   RowBox[{"30", " ", "b", " ", 
    SuperscriptBox["c", "2"], " ", 
    SuperscriptBox["d", "2"]}], "-", 
   RowBox[{"20", " ", "b", " ", "c", " ", 
    SuperscriptBox["d", "3"]}], "-", 
   RowBox[{"5", " ", "b", " ", 
    SuperscriptBox["d", "4"]}], "-", 
   SuperscriptBox["c", "5"], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["c", "4"], " ", "d"}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["c", "3"], " ", 
    SuperscriptBox["d", "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["c", "2"], " ", 
    SuperscriptBox["d", "3"]}], "-", 
   RowBox[{"5", " ", "c", " ", 
    SuperscriptBox["d", "4"]}], "-", 
   SuperscriptBox["d", "5"]}], TraditionalForm]], "Output",
 CellTags->"Collect3",
 CellLabel->"Out[208]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Collect3", "[", 
  RowBox[{
   RowBox[{"Expand", "[", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"a", "-", "b", "-", "c", "-", "d"}], ")"}], "5"], "]"}], ",", 
   RowBox[{"{", "a", "}"}], ",", 
   RowBox[{"Factoring", "->", "Factor"}]}], "]"}]], "Input",
 CellLabel->"In[209]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "5"], "-", 
   RowBox[{"5", " ", 
    SuperscriptBox["a", "4"], " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}]}], "+", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "3"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "2"]}], "-", 
   RowBox[{"10", " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "3"]}], "+", 
   RowBox[{"5", " ", "a", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"b", "+", "c", "+", "d"}], ")"}], "4"]}], "-", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"b", "+", "c", "+", "d"}], ")"}], "5"]}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[209]="]
}, Open  ]]
}, Open  ]],

Cell["Combine", "ObjectName",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Combine"],

Cell["\<\
Combine[expr]puts terms in a sum over a common denominator,and cancels \
factors in the result. Combine is similar to Together, but accepts the option \
Expanding and works usually better than Together for polynomials involving \
rationals with sums in the denominator.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Combine", "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[210]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[210]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Factor2", All, CellTags],
  ButtonData->"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"Combine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Combine", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[211]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[211]="]
}, Open  ]],

Cell["\<\
Here the result from Together where the numerator is automatically expanded.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Together", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[212]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"a", " ", "c"}], "-", 
    RowBox[{"a", " ", "d"}], "-", 
    RowBox[{"b", " ", "c"}], "+", 
    RowBox[{"b", " ", "d"}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[212]="]
}, Open  ]],

Cell["\<\
If the option Expanding is set to True, the result of Combine is the same as \
Together, but uses a slightly different algorithm.\
\>", "Text",
 CellTags->"Combine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Combine", "[", 
  RowBox[{
   RowBox[{
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "-", "b"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"c", "-", "d"}], ")"}]}], "e"], "+", "g"}], ",", 
   RowBox[{"Expanding", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Combine",
 CellLabel->"In[213]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"a", " ", "c"}], "-", 
    RowBox[{"a", " ", "d"}], "-", 
    RowBox[{"b", " ", "c"}], "+", 
    RowBox[{"b", " ", "d"}], "+", 
    RowBox[{"e", " ", "g"}]}], "e"], TraditionalForm]], "Output",
 CellTags->"Combine",
 CellLabel->"Out[213]="]
}, Open  ]]
}, Open  ]],

Cell["CombineGraphs", "ObjectName",
 CellTags->"CombineGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CombineGraphs"],

Cell["CombineGraphs is an option for OneLoopSum.", "Text",
 CellTags->"CombineGraphs"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"CombineGraphs"]
}, Open  ]],

Cell["Commutator", "ObjectName",
 CellTags->"Commutator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Commutator"],

Cell["\<\
Commutator[x, y] = c defines the commutator between the non-commuting objects \
x and y. \
\>", "Text",
 CellTags->"Commutator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiCommutator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "AntiCommutator", All, CellTags],
  ButtonData->"AntiCommutator",
  ButtonNote->"AntiCommutator"],
 ", ",
 ButtonBox["CommutatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CommutatorExplicit", All, CellTags],
  ButtonData->"CommutatorExplicit",
  ButtonNote->"CommutatorExplicit"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeclareNonCommutative", All, CellTags],
  ButtonData->"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"Commutator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Commutator"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[214]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[215]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}],
   TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[215]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[216]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[216]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DotSimplify", "[", 
   RowBox[{"Commutator", "[", 
    RowBox[{
     RowBox[{"a", "+", "b"}], ",", 
     RowBox[{"c", "+", "d"}]}], "]"}], "]"}], " "}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[217]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "c"}], "-", 
   RowBox[{"c", ".", "a"}], "+", 
   RowBox[{"a", ".", "d"}], "-", 
   RowBox[{"d", ".", "a"}], "+", 
   RowBox[{"b", ".", "c"}], "-", 
   RowBox[{"c", ".", "b"}], "+", 
   RowBox[{"b", ".", "d"}], "-", 
   RowBox[{"d", ".", "b"}]}], TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[217]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[218]:="],

Cell["Verify the Jacobi identity.", "Text",
 CellTags->"Commutator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Chi]", "=", "Commutator"}], ";", " ", 
  RowBox[{"DeclareNonCommutative", "[", 
   RowBox[{"x", ",", "y", ",", "z"}], "]"}], ";"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[219]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"\[Chi]", "[", 
   RowBox[{"x", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"y", ",", "z"}], "]"}]}], "]"}], "+", 
  RowBox[{"\[Chi]", "[", 
   RowBox[{"y", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"z", ",", "x"}], "]"}]}], "]"}], "+", 
  RowBox[{"\[Chi]", "[", 
   RowBox[{"z", ",", 
    RowBox[{"\[Chi]", "[", 
     RowBox[{"x", ",", "y"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[220]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"[", "\[NoBreak]", "x", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "y", "\[NoBreak]", ",", "z", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}], "+", 
   RowBox[{"[", "\[NoBreak]", "y", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "z", "\[NoBreak]", ",", "x", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}], "+", 
   RowBox[{"[", "\[NoBreak]", "z", "\[NoBreak]", ",", 
    FormBox[
     RowBox[{
     "[", "\[NoBreak]", "x", "\[NoBreak]", ",", "y", "\[NoBreak]", "]"}],
     TraditionalForm], "\[NoBreak]", "]"}]}], TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[220]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "%", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[221]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Commutator",
 CellLabel->"Out[221]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "\[Chi]", "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[222]:="],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellTags->"Commutator",
 CellLabel->"In[223]:="]
}, Open  ]],

Cell["CommutatorExplicit", "ObjectName",
 CellTags->"CommutatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CommutatorExplicit"],

Cell["\<\
CommutatorExplicit[exp] substitutes any Commutator and AntiCommutator in exp \
by their definitions.\
\>", "Text",
 CellTags->"CommutatorExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"CommutatorExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CommutatorExplicit"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[224]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Commutator", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[225]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"[", "\[NoBreak]", "a", "\[NoBreak]", ",", "b", "\[NoBreak]", "]"}],
   TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[225]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[226]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "-", 
   RowBox[{"b", ".", "a"}]}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[226]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"AntiCommutator", "[", 
  RowBox[{
   RowBox[{"a", "-", "c"}], ",", 
   RowBox[{"b", "-", "d"}]}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[227]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   FormBox[
    RowBox[{"a", "-", "c"}],
    TraditionalForm], ",", "\[MediumSpace]", 
   FormBox[
    RowBox[{"b", "-", "d"}],
    TraditionalForm], "}"}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[227]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CommutatorExplicit", "[", "%", "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[228]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "c"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"b", "-", "d"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"b", "-", "d"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{"a", "-", "c"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[228]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CommutatorExplicit", "[", "%%", "]"}], "//", 
  "DotSimplify"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[229]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b"}], "+", 
   RowBox[{"b", ".", "a"}], "-", 
   RowBox[{"a", ".", "d"}], "-", 
   RowBox[{"d", ".", "a"}], "-", 
   RowBox[{"b", ".", "c"}], "-", 
   RowBox[{"c", ".", "b"}], "+", 
   RowBox[{"c", ".", "d"}], "+", 
   RowBox[{"d", ".", "c"}]}], TraditionalForm]], "Output",
 CellTags->"CommutatorExplicit",
 CellLabel->"Out[229]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"CommutatorExplicit",
 CellLabel->"In[230]:="]
}, Open  ]],

Cell["CompleteSquare", "ObjectName",
 CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CompleteSquare"],

Cell[TextData[{
 "Completes the square of a second order polynomial in the momentum x. \
CompleteSquare[",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["p",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "+",
 "b",
 " ",
 "p",
 "+",
 "c",
 ", ",
 "p",
 "] \[Rule] -",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["b",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "/(4 ",
 "a",
 ")+",
 "c",
 "+",
 "a",
 " (",
 "b",
 "/(2 ",
 "a",
 ")+",
 "x",
 ")^2. CompleteSquare[",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["p",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "+",
 "b",
 " ",
 "p",
 "+",
 "c",
 ", ",
 "p",
 ", ",
 "q",
 "] \[Rule] {-",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["b",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 "/(4 ",
 "a",
 ")+",
 "c",
 "+",
 "a",
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    StyleBox["q",
     FontSlant->"Plain"], "2"], TraditionalForm]]],
 ", ",
 "q",
 "\[Rule]",
 "b",
 "/(2 ",
 "a",
 ")+",
 "p",
 "}."
}], "Text",
 CellTags->"CompleteSquare"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CompleteSquare"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"5", 
    RowBox[{"SP", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"2", "p"}], "+", 
       RowBox[{"3", "r"}]}], ",", 
      RowBox[{"p", "+", "r"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellLabel->"In[231]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"5", " ", 
   RowBox[{"(", 
    FormBox[
     RowBox[{"p", "+", "r"}],
     TraditionalForm], ")", "\[CenterDot]", "(", 
    FormBox[
     RowBox[{
      RowBox[{"2", " ", "p"}], "+", 
      RowBox[{"3", " ", "r"}]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellLabel->"Out[231]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"CompleteSquare", "[", 
   RowBox[{"t1", ",", "p"}], "]"}]}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[232]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"10", " ", 
    RowBox[{"(", 
     TagBox[
      FormBox[
       RowBox[{"p", "+", 
        FractionBox[
         RowBox[{"5", " ", "r"}], "4"]}],
       TraditionalForm],
      TraditionalForm,
      Editable->True], ")", ".", "(", 
     TagBox[
      FormBox[
       RowBox[{"p", "+", 
        FractionBox[
         RowBox[{"5", " ", "r"}], "4"]}],
       TraditionalForm],
      TraditionalForm,
      Editable->True], ")"}]}], "-", 
   FractionBox[
    RowBox[{"5", " ", 
     SuperscriptBox[
      FormBox["r",
       TraditionalForm], "2"]}], "8"]}], TraditionalForm]], "Output",
 CellTags->"CompleteSquare",
 CellLabel->"Out[232]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"t1", "-", "t2"}], "//", "ScalarProductExpand"}], "//", 
  "Expand"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[233]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"CompleteSquare",
 CellLabel->"Out[233]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CompleteSquare", "[", 
  RowBox[{"t1", ",", "p", ",", "q"}], "]"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[234]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"10", " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], "2"]}], "-", 
     FractionBox[
      RowBox[{"5", " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], "2"]}], "8"]}], ",", 
    RowBox[{"q", "\[Rule]", 
     RowBox[{"p", "+", 
      FractionBox[
       RowBox[{"5", " ", "r"}], "4"]}]}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"CompleteSquare",
 CellLabel->"Out[234]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
 CellTags->"CompleteSquare",
 CellLabel->"In[235]:="]
}, Open  ]],

Cell["ComplexConjugate", "ObjectName",
 CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ComplexConjugate"],

Cell["\<\
ComplexConjugate[expr] complex conjugates expr. It operates on fermion lines, \
i.e., products of Spinor[..] .DiracMatrix[..] . Spinor[..], and changes all \
occuring LorentzIndex[mu] into LorentzIndex[ComplexIndex[mu]]. For taking the \
spin sum (i.e. constructing the traces) use FermionSpinSum. WARNING: In expr \
should be NO explicit I's in denominators!\
\>", "Text",
 CellTags->"ComplexConjugate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ComplexIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ComplexIndex", All, CellTags],
  ButtonData->"ComplexIndex",
  ButtonNote->"ComplexIndex"],
 ", ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FermionSpinSum", All, CellTags],
  ButtonData->"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LorentzIndex", All, CellTags],
  ButtonData->"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 "."
}], "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ComplexConjugate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[236]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Mu]", "\[Nu]"}]], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[236]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[237]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[237]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "5"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[238]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[238]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[239]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[239]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[240]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[240]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[241]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[241]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[242]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[242]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"ComplexConjugate", "[", 
    RowBox[{"MetricTensor", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[243]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[243]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{"k1", ",", "m"}], "]"}], ".", 
  RowBox[{"GA", "[", "\[Lambda]", "]"}], ".", 
  RowBox[{"SpinorU", "[", 
   RowBox[{"p1", ",", "m"}], "]"}]}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[244]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", "k1", ",", "m", ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Lambda]"], ".", 
   RowBox[{"u", "(", "p1", ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[244]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", "%", "]"}]], "Input",
 CellTags->"ComplexConjugate",
 CellLabel->"In[245]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m", ")"}], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Lambda]",
      TraditionalForm],
     TraditionalForm]], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["k1",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ComplexConjugate",
 CellLabel->"Out[245]="]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. Internally they \
are converted to Spinor objects.\
\>", "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],

Cell["Contract", "ObjectName",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Contract"],

Cell["\<\
Contract[expr] contracts pairs of Lorentz indices of metric tensors, \
four-vectors and (depending on the optionEpsContract) of Levi-Civita tensors \
in expr. For the contraction of Dirac matrices with each other use \
DiracSimplify. Contract[exp1, exp2] contracts (exp1*exp2), where exp1 and \
exp2 may be larger products of sums of metric tensors and 4-vectors. \
\>", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Contract", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[246]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"Contract3", "\[Rule]", "True"}], ",", 
    RowBox[{"EpsContract", "\[Rule]", "True"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}], ",", 
    RowBox[{"MomentumCombine", "\[Rule]", "True"}], ",", 
    RowBox[{"Rename", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "0"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Contract",
 CellLabel->"Out[246]="]
}, Open  ]],

Cell["\<\
The option setting Contract3 can be set to True resulting in a faster \
algorithm to contract products of tensors.\
\>", "Text"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumCombine", All, CellTags],
  ButtonData->"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->"Contract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[247]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Mu]"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[247]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[248]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Nu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[248]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[249]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["p", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[249]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[250]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[250]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Mu]", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[251]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Mu]", "\[Mu]"}]], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[251]="]
}, Open  ]],

Cell["The default dimension for MetricTensor is 4.", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[252]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[252]="]
}, Open  ]],

Cell["\<\
A short way to enter D-dimensional metric tensors is given by MTD. The \".\" \
as multiplication operator is not necessary but just convenient for \
typesetting.\
\>", "Text",
 CellTags->"Contract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[253]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[253]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[254]:="],

Cell[BoxData[
 FormBox["D", TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[254]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", ".", " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[255]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[255]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"q", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[256]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Mu]"], " ", 
   SubscriptBox["q", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[256]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", " ", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[257]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[257]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"a", "-", "b"}], ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[258]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{"a", "-", "b"}],
       HoldForm],
      TraditionalForm], ")"}], "\[Mu]"], " ", 
   SubscriptBox[
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{"p", "-", "q"}],
       HoldForm],
      TraditionalForm], ")"}], "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[258]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[259]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["a",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["a",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["b",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["b",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[259]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], "]"}], 
  " ", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Sigma]"}], "]"}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[260]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Sigma]"], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[260]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[261]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["p",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[261]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], "]"}], 
  " ", 
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], "]"}], 
  " "}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[262]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[262]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "%", "]"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[263]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "6"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Beta]", "\[Sigma]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[263]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
  RowBox[{
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Beta]"}], "]"}], 
   " ", 
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Alpha]", ",", "\[Sigma]"}], 
    "]"}]}]}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[264]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Alpha]", "\[Beta]"}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Alpha]", "\[Sigma]"}]]}], 
  TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[264]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", "%", "]"}], "//", "Factor2"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[265]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Beta]", "\[Sigma]"}]]}], TraditionalForm]], "Output",
 CellTags->"Contract",
 CellLabel->"Out[265]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"Contract",
 CellLabel->"In[266]:="]
}, Open  ]],

Cell["Contract1", "ObjectName",
 CellTags->"Contract1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Contract1"],

Cell["\<\
For covariant calculations it is not necessary to distinguish between co- and \
contravariant indices (except for the Levi-Civita tensor).
Therefore normally no distinction is made between upper and lower indices.
Howere, if needed, one can use Upper and Lower and then a distuction is made:
Contract1[exp] contracts Upper and Lower indices. Neither Upper and Upper nor \
Lower and Lower indices are contracted.\
\>", "Text",
 CellTags->"Contract1"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LorentzIndex", All, CellTags],
  ButtonData->"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Lower",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Lower", All, CellTags],
  ButtonData->"Lower",
  ButtonNote->"Lower"],
 ", ",
 ButtonBox["Upper",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Upper", All, CellTags],
  ButtonData->"Upper",
  ButtonNote->"Upper"],
 "."
}], "Text",
 CellTags->"Contract1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Contract1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FV", "[", 
     RowBox[{"p", ",", 
      RowBox[{"Lower", "[", "\[Mu]", "]"}]}], "]"}], 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[267]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[267]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FV", "[", 
     RowBox[{"p", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", 
      RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}]}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[268]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"Upper", "(", "\[Mu]", ")"}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"Upper", "(", "\[Mu]", ")"}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[268]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{
     RowBox[{"Lower", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[269]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[269]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{
     RowBox[{"Upper", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Upper", "[", "\[Mu]", "]"}]}], "]"}], "//", "FCI"}], "//", 
  "Contract1"}]], "Input",
 CellTags->"Contract1",
 CellLabel->"In[270]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{
    FormBox[
     RowBox[{"Upper", "(", "\[Mu]", ")"}],
     TraditionalForm], 
    FormBox[
     RowBox[{"Upper", "(", "\[Mu]", ")"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Contract1",
 CellLabel->"Out[270]="]
}, Open  ]]
}, Open  ]],

Cell["Convolute", "ObjectName",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Convolute"],

Cell[TextData[{
 "Convolute[f, g, x] convolutes ",
 StyleBox["f",
  FontSlant->"Italic"],
 "(",
 StyleBox["x",
  FontSlant->"Italic"],
 ") and ",
 StyleBox["g",
  FontSlant->"Italic"],
 "(",
 StyleBox["x",
  FontSlant->"Italic"],
 "), i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     SubscriptBox[
      StyleBox[
       RowBox[{
        StyleBox["d",
         FontSlant->"Plain"], 
        StyleBox["x",
         FontSlant->"Italic"]}]], "1"], " ", 
     RowBox[{
      SubsuperscriptBox["\[Integral]", "0", "1"], 
      RowBox[{
       SubscriptBox[
        StyleBox[
         RowBox[{
          StyleBox["d",
           FontSlant->"Plain"], "x"}]], "2"], " ", 
       RowBox[{"\[Delta]", "(", 
        RowBox[{"x", " ", "-", " ", 
         RowBox[{
          SubscriptBox["x", "1"], " ", 
          SubscriptBox["x", "2"]}]}], ")"}], " ", 
       RowBox[{"f", "(", 
        SubscriptBox["x", "1"], ")"}], " ", 
       RowBox[{
        RowBox[{"g", "(", 
         SubscriptBox["x", "2"], ")"}], " ", "."}]}]}]}]}], TraditionalForm]]],
 " Convolute[f, g] is equivalent to Convolute[f, g, x]. Convolute[exp, {x1, \
x2}] assumes that exp is polynomial in x1 and x2. Convolute uses \
table-look-up and does not do any integral calculations, only linear \
algebra."
}], "Text",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[271]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Bracket", "\[Rule]", 
     RowBox[{"{", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "}"}]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[271]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PlusDistribution", All, CellTags],
  ButtonData->"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 ", ",
 ButtonBox["ConvoluteTable",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ConvoluteTable", All, CellTags],
  ButtonData->"ConvoluteTable",
  ButtonNote->"ConvoluteTable"],
 "."
}], "Text",
 CellTags->"Convolute"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Convolute"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[272]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[272]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"x", ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[273]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[273]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[274]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[274]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[275]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[275]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"PlusDistribution", "[", 
    FractionBox["1", 
     RowBox[{"1", "-", "x"}]], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[276]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[276]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[277]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", "1"}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[277]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[278]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"\[Delta]", "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}]}]}], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"1", "-", "x"}]], "-", 
   FractionBox[
    RowBox[{"log", "(", "x", ")"}], 
    RowBox[{"1", "-", "x"}]]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[278]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[279]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[279]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[280]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[280]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[281]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", 
    RowBox[{"1", "-", "x"}], ")"}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[281]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[282]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}]}], "+", "x", "+", 
   RowBox[{"x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[282]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[283]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "x"}], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[283]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[284]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{"x", " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "x", ")"}]}], ")"}]}], "-", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[284]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[285]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[285]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[286]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], 
    RowBox[{"1", "-", "x"}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[286]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[287]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[287]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"x", ",", " ", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[288]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "x", " ", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[288]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", "x", "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[289]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[289]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[290]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[290]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[291]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", "x", "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", "1"}], TraditionalForm]], "Output",\

 CellTags->"Convolute",
 CellLabel->"Out[291]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", "x", "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[292]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "x"}], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", "x", "-", 
   RowBox[{
    FractionBox["1", "2"], " ", "x", " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{"log", "(", "x", ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[292]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{"1", ",", 
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", "x", "]"}]}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[293]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", "+", 
   RowBox[{"x", " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "x", ")"}]}], ")"}]}], "-", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[293]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"Log", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[294]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], "+", 
   RowBox[{"log", "(", "x", ")"}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[294]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox["1", 
    RowBox[{"1", "-", "x"}]], ",", 
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[295]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], 
     RowBox[{"1", "-", "x"}]]}], "+", 
   RowBox[{
    TemplateBox[{"3"},
     "Zeta"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"1", "-", "x"}]]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[295]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}]}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[296]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "+", 
   RowBox[{"2", " ", 
    TemplateBox[{"3","x"},
     "PolyLog"]}], "+", 
   RowBox[{
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", "x", ")"}], "-", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}], "+", 
      RowBox[{"log", "(", "x", ")"}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[296]="]
}, Open  ]]
}, Open  ]],

Cell["ConvoluteTable ", "ObjectName",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ConvoluteTable"],

Cell["\<\
ConvoluteTable[f, g, x] yields the convolution of f and g. ConvoluteTable is \
called by Convolute.\
\>", "Text",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Convolute", "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[297]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Bracket", "\[Rule]", 
     RowBox[{"{", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "}"}]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[297]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PlusDistribution", All, CellTags],
  ButtonData->"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 ", ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Convolute", All, CellTags],
  ButtonData->"Convolute",
  ButtonNote->"Convolute"],
 "."
}], "Text",
 CellTags->"ConvoluteTable"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ConvoluteTable"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"1", ",", "1"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[298]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[298]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"x", ",", "x"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[299]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "x"}], " ", 
   RowBox[{"log", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[299]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{"1", ",", "x"}], "]"}]], "Input",
 CellTags->"ConvoluteTable",
 CellLabel->"In[300]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"1", "-", "x"}], TraditionalForm]], "Output",
 CellTags->"ConvoluteTable",
 CellLabel->"Out[300]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ConvoluteTable", "[", 
  RowBox[{
   FractionBox[
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], 
    RowBox[{"1", "-", "x"}]], ",", "1"}], "]"}]], "Input",
 CellTags->"Convolute",
 CellLabel->"In[301]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Convolute",
 CellLabel->"Out[301]="]
}, Open  ]]
}, Open  ]],

Cell["CovariantFieldDerivative", "ObjectName",
 CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CovariantFieldDerivative"],

Cell["\<\
CovariantFieldDerivative[f[x],x,{li1,li2,...},opts] is a covariant derivative \
of f[x] with respect to space-time variables x and with Lorentz indices li1, \
li2,... CovariantFieldDerivative has only typesetting definitions by default. \
The user is must supply his/her own definition of the actual function.\
\>", "Text",
 CellTags->"CovariantFieldDerivative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CovariantD", All, CellTags],
  ButtonData->"CovariantD",
  ButtonNote->"CovariantD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["FieldDerivative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FieldDerivative", All, CellTags],
  ButtonData->"FieldDerivative",
  ButtonNote->"FieldDerivative"],
 "."
}], "Text",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CovariantFieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantFieldDerivative", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"QuantumField", "[", 
     RowBox[{"A", ",", 
      RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ",", "x", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"CovariantFieldDerivative",
 CellLabel->"In[302]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    TagBox[
     StyleBox["\<\"\[ScriptCapitalD]\"\>",
      FontSlant->"Italic"],
     StyleForm[#, FontSlant -> "Italic"]& ], 
    TagBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm,
     Editable->True]], "(", 
   TagBox[
    FormBox[
     RowBox[{
      SubscriptBox["A", "\[Mu]"], "(", "x", ")"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellTags->"CovariantFieldDerivative",
 CellLabel->"Out[302]="]
}, Open  ]]
}, Open  ]],

Cell["CounterT", "ObjectName",
 CellTags->"CounterT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CounterT"],

Cell["\<\
CounterT is a factor used by GluonPropagator and QuarkPropagator when \
CounterTerms is set to All.\
\>", "Text",
 CellTags->"CounterT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CounterTerm", All, CellTags],
  ButtonData->"CounterTerm",
  ButtonNote->"CounterTerm"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkPropagator", All, CellTags],
  ButtonData->"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"CounterT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CounterT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
   RowBox[{"CounterTerm", "\[Rule]", " ", "All"}]}], "]"}]], "Input",
 CellTags->"CounterT",
 CellLabel->"In[303]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"CounterT", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["C", "A"], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        SubscriptBox["S", "n"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox[
           RowBox[{"10", " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], "3"], "-", 
          RowBox[{
           FractionBox["10", "3"], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], "+", 
      FractionBox[
       RowBox[{"\[ImaginaryI]", " ", 
        SubscriptBox["T", "f"], " ", 
        SubsuperscriptBox["g", "s", "2"], " ", 
        SubscriptBox["S", "n"], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FractionBox["4", "3"], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], "2"], " ", 
           SuperscriptBox["g", 
            RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
          FractionBox[
           RowBox[{"4", " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm]], " ", 
            SuperscriptBox[
             FormBox["p",
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm]]}], "3"]}], ")"}]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]]}], ")"}]}], "-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellTags->"CounterT",
 CellLabel->"Out[303]="]
}, Open  ]]
}, Open  ]],

Cell["CounterTerm", "ObjectName",
 CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CounterTerm"],

Cell["\<\
CounterTerm[name] is a database of counter terms. CounterTerm is also an \
option for the Feynman rule functions QuarkGluonVertex, GluonPropagator, \
QuarkPropagator.\
\>", "Text",
 CellTags->"CounterTerm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CounterT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CounterT", All, CellTags],
  ButtonData->"CounterT",
  ButtonNote->"CounterT"],
 ", ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkGluonVertex", All, CellTags],
  ButtonData->"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkPropagator", All, CellTags],
  ButtonData->"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"CounterTerm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CounterTerm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CounterTerm", "[", "Zm", "]"}]], "Input",
 CellTags->"CounterTerm",
 CellLabel->"In[304]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["C", "F"], " ", 
     SubsuperscriptBox["g", "s", "4"], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"4", " ", 
         RowBox[{"(", 
          RowBox[{
           FractionBox[
            RowBox[{"11", " ", 
             SubscriptBox["C", "A"]}], "2"], "+", 
           FractionBox[
            RowBox[{"9", " ", 
             SubscriptBox["C", "F"]}], "2"], "-", 
           RowBox[{"2", " ", 
            SubscriptBox["N", "f"], " ", 
            SubscriptBox["T", "f"]}]}], ")"}]}], 
        SuperscriptBox[
         TagBox["\[CurlyEpsilon]",
          TraditionalForm], "2"]], "+", 
       FractionBox[
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{
           FractionBox[
            RowBox[{"97", " ", 
             SubscriptBox["C", "A"]}], "12"], "+", 
           FractionBox[
            RowBox[{"3", " ", 
             SubscriptBox["C", "F"]}], "4"], "-", 
           FractionBox[
            RowBox[{"5", " ", 
             SubscriptBox["N", "f"], " ", 
             SubscriptBox["T", "f"]}], "3"]}], ")"}]}], 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]]}], ")"}]}], 
    RowBox[{"256", " ", 
     SuperscriptBox["\[Pi]", "4"]}]], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     SubscriptBox["C", "F"], " ", 
     SubsuperscriptBox["g", "s", "2"]}], 
    RowBox[{"8", " ", 
     SuperscriptBox["\[Pi]", "2"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"CounterTerm",
 CellLabel->"Out[304]="]
}, Open  ]]
}, Open  ]],

Cell["CouplingConstant", "ObjectName",
 CellTags->"CouplingConstant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CouplingConstant"],

Cell["\<\
In general, CouplingConstant is an option for several Feynman rule functions \
and for CovariantD and FieldStrength.
In the convention of the subpackage PHI, CouplingConstant is also the head of \
coupling constants.  CouplingConstant takes three extra optional arguments, \
with head RenormalizationState, RenormalizationScheme and ExpansionState \
respectively.  E.g. CouplingConstant[QED[1]] is the unit charge, \
CouplingConstant[ChPT2[4],1] is the first of the coupling constants of the \
lagrangian ChPT2[4].  CouplingConstant[a_,b_,c___][i_] := \
CouplingConstant[a,b,RenormalizationState[i],c].\
\>", "Text",
 CellTags->"CouplingConstant"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CovariantD", All, CellTags],
  ButtonData->"CovariantD",
  ButtonNote->"CovariantD"],
 ", ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FieldStrength", All, CellTags],
  ButtonData->"FieldStrength",
  ButtonNote->"FieldStrength"],
 "."
}], "Text",
 CellTags->"CouplingConstant"]
}, Open  ]],

Cell["CovariantD", "ObjectName",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CovariantD"],

Cell["\<\
CovariantD[mu] is a generic covariant derivative with Lorentz index mu. With \
the option-setting Explicit \[Rule] True, an explicit expression for a \
fermionic field is returned, depending on the setting on the other options.
CovariantD[x, mu] is a generic covariant derivative with respect to x^mu.
CovariantD[mu, a, b] is a covariant derivative for a bosonic field; acting on \
QuantumField[f,{},{a,b}], where f is some field name and a and b are two \
SU(N) indices. Again, with the option-setting Explicit \[Rule] True, an \
explicit expression is returned, depending on the setting on the other options.
CovariantD[OPEDelta, a, b] is a short form for  \
CovariantD[mu,a,b]*FourVector[OPEDelta, mu]. CovariantD[{OPEDelta, a, b}, \
{n}] yields the product of n operators, where n is an integer.  \
CovariantD[OPEDelta, a, b, {m, n}]  gives the expanded form of \
CovariantD[OPEDelta, a, b]^m up to order g^n for the gluon, where n is an \
integer and g the coupling constant  indicated by the setting of the option \
CouplingConstant. CovariantD[OPEDelta, {m, n}] gives the expanded form of  \
CovariantD[OPEDelta]^m up to order g^n of the fermionic field.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "CovariantD", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[305]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"DummyIndex", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
    RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[305]="]
}, Open  ]],

Cell["\<\
 Possible settings of PartialD are: LeftPartialD, LeftRigthPartialD, \
RightPartialD. The default setting of QuantumField is GaugeField.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftPartialD", All, CellTags],
  ButtonData->"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD", All, CellTags],
  ButtonData->"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RightPartialD", All, CellTags],
  ButtonData->"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"CovariantD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[306]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["D", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[306]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[307]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["D", "\[Mu]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[307]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"\[Mu]", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[308]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "1"],
         Subscripted[
          $CellContext`c[1]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "1"],
         Subscripted[
          $CellContext`c[1]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[308]="]
}, Open  ]],

Cell["\<\
The first argument of CovariantD is intepreted as type LorentzIndex, except \
for OPEDelta, which is type Momentum.\
\>", "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", "OPEDelta", "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[309]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["D", 
   FormBox["\[CapitalDelta]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[309]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[310]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["D", 
   FormBox["\[CapitalDelta]",
    TraditionalForm], 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[310]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[311]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "b"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox[
       InterpretationBox[
        SubscriptBox["c", "2"],
        Subscripted[
         $CellContext`c[2]],
        Editable->False],
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b", 
      FormBox[
       InterpretationBox[
        SubscriptBox["c", "2"],
        Subscripted[
         $CellContext`c[2]],
        Editable->False],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[311]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[312]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "3"],
         Subscripted[
          $CellContext`c[3]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox[
        InterpretationBox[
         SubscriptBox["c", "3"],
         Subscripted[
          $CellContext`c[3]],
         Editable->False],
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[312]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", "2", "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[313]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Delta]", 
       RowBox[{"a", "c23"}]], " ", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]]}], "-", 
     RowBox[{
      SubscriptBox["g", "s"], " ", 
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["e24",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubscriptBox["f", 
       RowBox[{"a", "c23", "e24"}]]}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[Delta]", 
       RowBox[{"b", "c23"}]], " ", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]]}], "-", 
     RowBox[{
      SubscriptBox["g", "s"], " ", 
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["e25",
         TraditionalForm],
        TraditionalForm]], " ", 
      SubscriptBox["f", 
       RowBox[{"c23", "b", "e25"}]]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"CovariantD",
 CellLabel->"Out[313]="]
}, Open  ]],

Cell[TextData[{
 "This gives",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"m", " ", "times", " ", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], "\[CapitalDelta]"]}], 
     ",", " "}]}], TraditionalForm]]],
 "the partial derivative ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "contracted with ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[CapitalDelta]", "\[Mu]"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "0"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[314]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ")"}], "m"]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[314]="]
}, Open  ]],

Cell[TextData[{
 "The expansion up to first order in the coupling constant ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{
     RowBox[{"(", "thesumistheFeynCalcOPESum", ")"}], "."}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[315]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], ")"}], "m"]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "m"}]], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], "i"], ".", 
         SubsuperscriptBox[
          FormBox["A",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", "i", "+", "m"}]]}], " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]}]]}]}],
      HoldForm],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[315]="]
}, Open  ]],

Cell[TextData[{
 "The expansion up to second order in the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["g", "s"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"CovariantD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", "a", ",", "b", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[316]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["g", "s", "2"], " ", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "2"}], "+", "m"}]], 
        RowBox[{"(", 
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "j"], 
            RowBox[{
             RowBox[{
              SuperscriptBox[
               RowBox[{"(", 
                SubscriptBox[
                 OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                 FormBox["\[CapitalDelta]",
                  TraditionalForm]], ")"}], "i"], ".", 
              SubsuperscriptBox[
               FormBox["A",
                TraditionalForm], 
               FormBox["\[CapitalDelta]",
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "1"],
                  Subscripted[
                   $CellContext`c[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]], ".", 
              SuperscriptBox[
               RowBox[{"(", 
                SubscriptBox[
                 OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                 FormBox["\[CapitalDelta]",
                  TraditionalForm]], ")"}], 
               RowBox[{
                RowBox[{"-", "i"}], "+", "j"}]], ".", 
              SubsuperscriptBox[
               FormBox["A",
                TraditionalForm], 
               FormBox["\[CapitalDelta]",
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[
                   $CellContext`c[2]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]], ".", 
              SuperscriptBox[
               RowBox[{"(", 
                SubscriptBox[
                 OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                 FormBox["\[CapitalDelta]",
                  TraditionalForm]], ")"}], 
               RowBox[{
                RowBox[{"-", "2"}], "-", "j", "+", "m"}]]}], " ", 
             SubscriptBox["f", 
              RowBox[{
               FormBox[
                FormBox["a",
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "1"],
                  Subscripted[
                   $CellContext`c[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["e", "1"],
                  Subscripted[
                   $CellContext`e[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]}]], " ", 
             SubscriptBox["f", 
              RowBox[{
               FormBox[
                FormBox["b",
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["c", "2"],
                  Subscripted[
                   $CellContext`c[2]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm], 
               FormBox[
                FormBox[
                 InterpretationBox[
                  SubscriptBox["e", "1"],
                  Subscripted[
                   $CellContext`e[1]],
                  Editable->False],
                 TraditionalForm],
                TraditionalForm]}]]}]}],
           HoldForm],
          TraditionalForm], ")"}]}],
       HoldForm],
      TraditionalForm]}]}], "-", 
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "m"}]], 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], "i"], ".", 
         SubsuperscriptBox[
          FormBox["A",
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox[
          RowBox[{"(", 
           SubscriptBox[
            OverscriptBox["\[PartialD]", "\[RightArrow]"], 
            FormBox["\[CapitalDelta]",
             TraditionalForm]], ")"}], 
          RowBox[{
           RowBox[{"-", "1"}], "-", "i", "+", "m"}]]}], " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox[
            InterpretationBox[
             SubscriptBox["c", "1"],
             Subscripted[
              $CellContext`c[1]],
             Editable->False],
            TraditionalForm],
           TraditionalForm]}]]}]}],
      HoldForm],
     TraditionalForm]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[RightArrow]"], 
       FormBox["\[CapitalDelta]",
        TraditionalForm]], ")"}], "m"]}]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[316]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 SuperscriptBox[
  RowBox[{"CovariantD", "[", 
   RowBox[{"OPEDelta", ",", "a", ",", "b"}], "]"}], "OPEm"]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[317]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    SubsuperscriptBox["D", 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     RowBox[{"a", "b"}]], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[317]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CovariantD", "[", 
  RowBox[{"OPEDelta", ",", 
   RowBox[{"{", 
    RowBox[{"OPEm", ",", "2"}], "}"}]}], "]"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[318]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["g", "s", "2"], " ", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"j", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "2"}], "+", "m"}]], 
        RowBox[{"(", 
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], "j"], 
            RowBox[{
             SubscriptBox["T", 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "1"],
                 Subscripted[
                  $CellContext`c[1]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SubscriptBox["T", 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "2"],
                 Subscripted[
                  $CellContext`c[2]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SuperscriptBox[
              RowBox[{"(", 
               SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                 TraditionalForm]], ")"}], "i"], ".", 
             SubsuperscriptBox[
              FormBox["A",
               TraditionalForm], 
              FormBox["\[CapitalDelta]",
               TraditionalForm], 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "1"],
                 Subscripted[
                  $CellContext`c[1]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SuperscriptBox[
              RowBox[{"(", 
               SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                 TraditionalForm]], ")"}], 
              RowBox[{
               RowBox[{"-", "i"}], "+", "j"}]], ".", 
             SubsuperscriptBox[
              FormBox["A",
               TraditionalForm], 
              FormBox["\[CapitalDelta]",
               TraditionalForm], 
              FormBox[
               FormBox[
                InterpretationBox[
                 SubscriptBox["c", "2"],
                 Subscripted[
                  $CellContext`c[2]],
                 Editable->False],
                TraditionalForm],
               TraditionalForm]], ".", 
             SuperscriptBox[
              RowBox[{"(", 
               SubscriptBox[
                OverscriptBox["\[PartialD]", "\[RightArrow]"], 
                FormBox["\[CapitalDelta]",
                 TraditionalForm]], ")"}], 
              RowBox[{
               RowBox[{"-", "2"}], "-", "j", "+", "m"}]]}]}],
           HoldForm],
          TraditionalForm], ")"}]}],
       HoldForm],
      TraditionalForm]}]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["g", "s"], " ", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "1"}], "+", "m"}]], 
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox[
           InterpretationBox[
            SubscriptBox["c", "1"],
            Subscripted[
             $CellContext`c[1]],
            Editable->False],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox[
         RowBox[{"(", 
          SubscriptBox[
           OverscriptBox["\[PartialD]", "\[RightArrow]"], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]], ")"}], "i"], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox[
           InterpretationBox[
            SubscriptBox["c", "1"],
            Subscripted[
             $CellContext`c[1]],
            Editable->False],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox[
         RowBox[{"(", 
          SubscriptBox[
           OverscriptBox["\[PartialD]", "\[RightArrow]"], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]], ")"}], 
         RowBox[{
          RowBox[{"-", "1"}], "-", "i", "+", "m"}]]}]}],
      HoldForm],
     TraditionalForm]}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ")"}], "m"]}], TraditionalForm]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[318]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CovariantD", "[", 
   RowBox[{"OPEDelta", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[319]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", "Gstrong", " ", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"SUNIndex", "[", 
      InterpretationBox[
       SubscriptBox["c", "4"],
       Subscripted[
        $CellContext`c[4]],
       Editable->False], "]"}], "]"}], ".", 
    RowBox[{"QuantumField", "[", 
     RowBox[{"GaugeField", ",", 
      RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
      RowBox[{"SUNIndex", "[", 
       InterpretationBox[
        SubscriptBox["c", "4"],
        Subscripted[
         $CellContext`c[4]],
        Editable->False], "]"}]}], "]"}]}]}], "+", 
  RowBox[{"RightPartialD", "[", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}]}]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[319]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"CovariantD", "[", 
   RowBox[{"\[Mu]", ",", "a", ",", "b", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"CovariantD",
 CellLabel->"In[320]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"RightPartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}]}], "-", 
  RowBox[{"Gstrong", " ", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"GaugeField", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"SUNIndex", "[", 
      InterpretationBox[
       SubscriptBox["c", "5"],
       Subscripted[
        $CellContext`c[5]],
       Editable->False], "]"}]}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", 
     InterpretationBox[
      SubscriptBox["c", "5"],
      Subscripted[
       $CellContext`c[5]],
      Editable->False]}], "]"}]}]}]], "Output",
 CellTags->"CovariantD",
 CellLabel->"Out[320]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["CrossProduct", "ObjectName",
 CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"CrossProduct"],

Cell["\<\
CrossProduct[a, b] denotes the three-dimensional cross-product of the \
three-vectors a and b.\
\>", "Text",
 CellTags->"CrossProduct"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DotProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotProduct", All, CellTags],
  ButtonData->"DotProduct",
  ButtonNote->"DotProduct"],
 ", ",
 ButtonBox["ThreeVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ThreeVector", All, CellTags],
  ButtonData->"ThreeVector",
  ButtonNote->"ThreeVector"],
 "."
}], "Text",
 CellTags->"CrossProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"CrossProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"CrossProduct", "[", 
  RowBox[{
   RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
   RowBox[{"CrossProduct", "[", 
    RowBox[{
     RowBox[{"ThreeVector", "[", "b", "]"}], ",", 
     RowBox[{"ThreeVector", "[", "c", "]"}]}], "]"}]}], "]"}]], "Input",
 CellTags->"CrossProduct",
 CellLabel->"In[321]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    OverscriptBox["b", "\[RightVector]"], " ", 
    RowBox[{
     FormBox[
      OverscriptBox["a", "\[RightVector]"],
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      OverscriptBox["c", "\[RightVector]"],
      TraditionalForm]}]}], "-", 
   RowBox[{
    OverscriptBox["c", "\[RightVector]"], " ", 
    RowBox[{
     FormBox[
      OverscriptBox["a", "\[RightVector]"],
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      OverscriptBox["b", "\[RightVector]"],
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"CrossProduct",
 CellLabel->"Out[321]="]
}, Open  ]]
}, Open  ]],

Cell["C0", "ObjectName",
 CellTags->"C0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"C0"],

Cell[TextData[{
 "C0[p10, p12, p20, m1^2, m2^2, m3^2] is the scalar Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["C", "0"], TraditionalForm]]],
 "function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2]), the \
first three arguments of C0 are the scalar products p10 = p1^2, p12 = \
(p1-p2).(p1-p2), p20 = p2^2."
}], "Text",
 CellTags->"C0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "D0", All, CellTags],
  ButtonData->"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeOrder", All, CellTags],
  ButtonData->"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"C0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"C0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"C0", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", " ", "m12", ",", "m22", ",", "m32"}], 
  "]"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[322]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "m12", ",", "m22", 
   ",", "m32", ")"}], TraditionalForm]], "Output",
 CellTags->"C0",
 CellLabel->"Out[322]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"C0", "[", 
   RowBox[{"b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
   "]"}], "//", "PaVeOrder"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[323]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "m12", ",", "m22", 
   ",", "m32", ")"}], TraditionalForm]], "Output",
 CellTags->"C0",
 CellLabel->"Out[323]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"C0", "[", 
    RowBox[{"b", ",", "a", ",", "c", ",", "m32", ",", "m22", ",", "m12"}], 
    "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"c", ",", "a"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"C0",
 CellLabel->"In[324]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "b", ",", "c", ",", "a", ",", "m22", ",", "m32", 
   ",", "m12", ")"}], TraditionalForm]], "Output",
 CellTags->"C0",
 CellLabel->"Out[324]="]
}, Open  ]]
}, Open  ]],

Cell["DataType", "ObjectName",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DataType"],

Cell["\<\
DataType[exp, type] = True defines the object exp to have data-type type. \
DataType[exp1, exp2, ..., type] defines the objects exp1, exp2, ...to have \
data-type type. The default setting is DataType[__, _] := False. To assign a \
certain data-type, do, e.g., DataType[x, PositiveInteger] = True.\
\>", "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "Currently used DataTypes: ",
 ButtonBox["NonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "NonCommutative", All, CellTags],
  ButtonData->"NonCommutative",
  ButtonNote->"NonCommutative"],
 ", ",
 ButtonBox["PositiveInteger",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PositiveInteger", All, CellTags],
  ButtonData->"PositiveInteger",
  ButtonNote->"PositiveInteger"],
 ", ",
 ButtonBox["NegativeInteger",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "NegativeInteger", All, CellTags],
  ButtonData->"NegativeInteger",
  ButtonNote->"NegativeInteger"],
 ", ",
 ButtonBox["PositiveNumber",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PositiveNumber", All, CellTags],
  ButtonData->"PositiveNumber",
  ButtonNote->"PositiveNumber"],
 ", ",
 ButtonBox["FreeIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FreeIndex", All, CellTags],
  ButtonData->"FreeIndex",
  ButtonNote->"FreeIndex"],
 ", ",
 ButtonBox["GrassmannParity",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GrassmannParity", All, CellTags],
  ButtonData->"GrassmannParity",
  ButtonNote->"GrassmannParity"]
}], "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "PHI adds the DataTypes: ",
 ButtonBox["UMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "UMatrix", All, CellTags],
  ButtonData->"UMatrix",
  ButtonNote->"UMatrix"],
 ", ",
 ButtonBox["UScalar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "UScalar", All, CellTags],
  ButtonData->"UScalar",
  ButtonNote->"UScalar"],
 "."
}], "Text",
 CellTags->"DataType"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeclareNonCommutative", All, CellTags],
  ButtonData->"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DataType"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DataType"],

Cell["NonCommutative is just a data-type.", "Text",
 CellTags->"DataType"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", "=", " ",
    "True"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[325]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"g", ".", 
    RowBox[{"(", 
     RowBox[{"2", "a"}], ")"}], ".", "f"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[326]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"g", ".", 
    RowBox[{"(", 
     RowBox[{"2", " ", "a"}], ")"}], ".", "f"}]}], TraditionalForm]], "Output",\

 CellTags->"DataType",
 CellLabel->"Out[326]="]
}, Open  ]],

Cell["\<\
Since \"f \"and \"g\" have DataType NonCommutative the function DotSimplify \
extracts only \"a\" out of the noncommutative product.\
\>", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[327]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", ".", "g"}], "-", 
   RowBox[{"2", " ", "a", " ", 
    RowBox[{"g", ".", "f"}]}]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[327]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "odd"}], "]"}], "=", 
   RowBox[{
    RowBox[{"DataType", "[", 
     RowBox[{"a", ",", "even"}], "]"}], "=", "True"}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[328]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ptest1", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
      RowBox[{"DataType", "[", 
       RowBox[{"n", ",", "odd"}], "]"}]}], "\[RuleDelayed]", 
     RowBox[{"-", "1"}]}]}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[329]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ptest2", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n_"], "/;", 
      RowBox[{"DataType", "[", 
       RowBox[{"n", ",", "even"}], "]"}]}], "\[RuleDelayed]", "1"}]}]}], 
  ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[330]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "a"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "z"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[331]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "z"]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[331]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest1", "[", "t", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[332]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "z"], "-", "1"}], TraditionalForm]], "Output",\

 CellTags->"DataType",
 CellLabel->"Out[332]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest2", "[", "%", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[333]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "z"], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[333]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"ptest1", ",", "ptest2", ",", "t", ",", "a", ",", "m"}], "]"}], 
  ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[334]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "ganzeZahl"}], "]"}], "=", "True"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[335]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"f", "[", "x_", "]"}], ":=", 
   RowBox[{"x", "/.", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "^", "p_"}], "/;", 
       RowBox[{"DataType", "[", 
        RowBox[{"p", ",", "ganzeZahl"}], "]"}]}], "\[RuleDelayed]", "1"}], 
     "}"}]}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[336]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"test", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "m"}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", "n"}], " ", "x"}]}]}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[337]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}]}], 
  TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[337]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f", "[", "test", "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[338]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "n"], " ", "x"}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[338]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"f", ",", "test"}], "]"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[339]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"f", ",", "g", ",", " ", "NonCommutative"}], "]"}], " ", "=", " ",
    "False"}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[340]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"DataType", "[", 
    RowBox[{"m", ",", "odd"}], "]"}], "=", 
   RowBox[{
    RowBox[{"DataType", "[", 
     RowBox[{"a", ",", "even"}], "]"}], "=", "False"}]}], ";"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[341]:="],

Cell["\<\
Certain FeynCalc objects have DataType PositiveInteger set to True.\
\>", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[342]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[342]="]
}, Open  ]],

Cell["PowerSimplify uses the DataType information.", "Text",
 CellTags->"DataType"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[343]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[343]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", " ", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", " ", 
     RowBox[{"SO", "[", "q", "]"}]}], ")"}], "^", "OPEm"}], "]"}]], "Input",
 CellTags->"DataType",
 CellLabel->"In[344]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["q",
       TraditionalForm]}], ")"}], "m"]}], TraditionalForm]], "Output",
 CellTags->"DataType",
 CellLabel->"Out[344]="]
}, Open  ]]
}, Open  ]],

Cell["DB0", "ObjectName",
 CellTags->"DB0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DB0"],

Cell["\<\
DB0[p2, m1^2, m2^2] is the derivative of the two-point function B0[p2, m1^2, \
m2^2] with respect to p2.\
\>", "Text",
 CellTags->"DB0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 "."
}], "Text",
 CellTags->"DB0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DB0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"B0", "[", 
    RowBox[{
     SubscriptBox["p", "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "1"], "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
   SubscriptBox["p", "2"]}], "]"}]], "Input",
 CellTags->"DB0",
 CellLabel->"In[345]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"DB0", "(", 
   RowBox[{
    SubscriptBox["p", "2"], ",", 
    SubsuperscriptBox["m", "1", "2"], ",", 
    SubsuperscriptBox["m", "2", "2"]}], ")"}], TraditionalForm]], "Output",
 CellTags->"DB0",
 CellLabel->"Out[345]="]
}, Open  ]]
}, Open  ]],

Cell["DB1", "ObjectName",
 CellTags->"DB1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DB1"],

Cell["\<\
DB1[p2,m1^2,m2^2] is the derivative of B1[p2,m1^2,m2^2] with respect to p2.\
\>", "Text",
 CellTags->"DB1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B1", All, CellTags],
  ButtonData->"B1",
  ButtonNote->"B1"],
 "."
}], "Text",
 CellTags->"DB1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DB1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"B1", "[", 
    RowBox[{
     SubscriptBox["p", "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "1"], "2"], ",", 
     SuperscriptBox[
      SubscriptBox["m", "2"], "2"]}], "]"}], ",", 
   SubscriptBox["p", "2"]}], "]"}]], "Input",
 CellTags->"DB1",
 CellLabel->"In[346]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["m", "2", "2"], "-", 
        SubsuperscriptBox["m", "1", "2"]}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         SubscriptBox["p", "2"], ",", 
         SubsuperscriptBox["m", "1", "2"], ",", 
         SubsuperscriptBox["m", "2", "2"], ")"}], "-", 
        RowBox[{
         SubscriptBox["B", "0"], "(", "0", ",", 
         SubsuperscriptBox["m", "1", "2"], ",", 
         SubsuperscriptBox["m", "2", "2"], ")"}]}], ")"}]}], 
     RowBox[{"2", " ", 
      SubsuperscriptBox["p", "2", "2"]}]]}], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubsuperscriptBox["m", "2", "2"], "-", 
       SubsuperscriptBox["m", "1", "2"]}], ")"}], " ", 
     RowBox[{"DB0", "(", 
      RowBox[{
       SubscriptBox["p", "2"], ",", 
       SubsuperscriptBox["m", "1", "2"], ",", 
       SubsuperscriptBox["m", "2", "2"]}], ")"}]}], 
    RowBox[{"2", " ", 
     SubscriptBox["p", "2"]}]], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"DB0", "(", 
     RowBox[{
      SubscriptBox["p", "2"], ",", 
      SubsuperscriptBox["m", "1", "2"], ",", 
      SubsuperscriptBox["m", "2", "2"]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"DB1",
 CellLabel->"Out[346]="]
}, Open  ]]
}, Open  ]],

Cell["DeclareNonCommutative", "ObjectName",
 CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeclareNonCommutative"],

Cell["\<\
DeclareNonCommutative[a, b, ...] declares a,b, ... to be non-commutative, \
i.e., DataType[a,b, ..., NonCommutative] is set to True.\
\>", "Text",
 CellTags->"DeclareNonCommutative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["UnDeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "UnDeclareNonCommutative", All, CellTags],
  ButtonData->"UnDeclareNonCommutative",
  ButtonNote->"UnDeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[347]:="],

Cell["\<\
As a side effect of DeclareNonCommutative x is declared to be of data type \
NonCommutative.\
\>", "Text",
 CellTags->"DeclareNonCommutative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[348]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[348]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[349]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
  "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[350]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[350]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"x", ",", "y", ",", "z"}], "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[351]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"a", ",", "x", ",", "y", ",", "z", ",", "NonCommutative"}], 
  "]"}]], "Input",
 CellTags->"DeclareNonCommutative",
 CellLabel->"In[352]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DeclareNonCommutative",
 CellLabel->"Out[352]="]
}, Open  ]]
}, Open  ]],

Cell["DeltaFunction", "ObjectName",
 CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunction"],

Cell[TextData[{
 " DeltaFunction[x] is the Dirac delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 ".\n*** After version 4 of ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ", there is a built-in function, DiracDelta, with comparable properties. \
***"
}], "Text",
 CellTags->"DeltaFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Convolute", All, CellTags],
  ButtonData->"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeltaFunctionPrime", All, CellTags],
  ButtonData->"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Integrate2", All, CellTags],
  ButtonData->"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["SimplifyDeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SimplifyDeltaFunction", All, CellTags],
  ButtonData->"SimplifyDeltaFunction",
  ButtonNote->"SimplifyDeltaFunction"],
 "."
}], "Text",
 CellTags->"DeltaFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeltaFunction", "[", 
  RowBox[{"1", "-", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[353]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Delta]", "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[353]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[354]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[354]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", "x", "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[355]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "0", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[355]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[356]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[356]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Convolute", "[", 
  RowBox[{
   RowBox[{"DeltaFunction", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunction",
 CellLabel->"In[357]:="],

Cell[BoxData[
 FormBox["x", TraditionalForm]], "Output",
 CellTags->"DeltaFunction",
 CellLabel->"Out[357]="]
}, Open  ]]
}, Open  ]],

Cell["DeltaFunctionDoublePrime", "ObjectName",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
 " DeltaFunctionDoublePrime[1-x] is the second derivative of the Dirac \
delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DeltaFunctionDoublePrime"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeltaFunctionPrime", All, CellTags],
  ButtonData->"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 "."
}], "Text",
 CellTags->"DeltaFunctionDoublePrime"]
}, Open  ]],

Cell["DeltaFunctionPrime", "ObjectName",
 CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
 " DeltaFunctionPrime[1-x] is the derivative of the Dirac delta-function ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Delta]", "(", "x", ")"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DeltaFunctionPrime"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Convolute", All, CellTags],
  ButtonData->"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeltaFunction", All, CellTags],
  ButtonData->"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Integrate2", All, CellTags],
  ButtonData->"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["SimplifyDeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SimplifyDeltaFunction", All, CellTags],
  ButtonData->"SimplifyDeltaFunction",
  ButtonNote->"SimplifyDeltaFunction"],
 "."
}], "Text",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DeltaFunctionPrime"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DeltaFunctionPrime", "[", 
  RowBox[{"1", "-", "x"}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[358]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[358]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[359]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["f", "\[Prime]",
    MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[359]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    SuperscriptBox["x", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"In[360]:="],

Cell[BoxData[
 FormBox["2", TraditionalForm]], "Output",
 CellTags->"DeltaFunctionPrime",
 CellLabel->"Out[360]="]
}, Open  ]]
}, Open  ]],

Cell["DenominatorOrder", "ObjectName",
 CellTags->"DenominatorOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DenominatorOrder"],

Cell["\<\
DenominatorOrder is an option for OneLoop, if set to True the \
PropagatorDenominator will be ordered in a standard way.\
\>", "Text",
 CellTags->"DenominatorOrder"],

Cell[TextData[{
 "See also: ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PropagatorDenominator", All, CellTags],
  ButtonData->"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"DenominatorOrder"]
}, Open  ]],

Cell["Dimension", "ObjectName",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Dimension"],

Cell["\<\
Dimension is an option of several functions and denotes the number of \
space-time dimensions. Possible settings are: 4, n, d, D, ... ,the variable \
does not matter, but it should have Head Symbol.\
\>", "Text",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "MetricTensor", "]"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[361]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[361]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"m", ",", "n", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[362]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{"m", "n"}]]}], TraditionalForm]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[362]="]
}, Open  ]],

Cell["\<\
The dimension of the indices is not shown by default but can be inspected \
easily.\
\>", "Text",
 CellTags->"Dimension"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MetricTensor", "[", 
    RowBox[{"m", ",", "n", ",", 
     RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}], 
   RowBox[{"DiracMatrix", "[", 
    RowBox[{"\[Alpha]", ",", 
     RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[363]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "d"}], "]"}], ",", "d"}], "]"}], " ", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"m", ",", "n", ",", 
    RowBox[{"Dimension", "\[Rule]", "d"}]}], "]"}]}]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[363]//StandardForm="]
}, Open  ]],

Cell["\<\
Setting the global variable $LorentzIndices to True will display the \
dimension (if different from 4) as a subscript.\
\>", "Text",
 CellTags->"Dimension"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[364]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[365]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Alpha]",
       TraditionalForm], 
      FormBox["n",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[365]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[366]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Output",
 CellTags->"Dimension",
 CellLabel->"Out[366]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
 CellTags->"Dimension",
 CellLabel->"In[367]:="]
}, Open  ]],

Cell["DimensionalReduction", "ObjectName",
 CellTags->"DimensionalReduction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DimensionalReduction"],

Cell["\<\
DimensionalReduction is an option for TID and OneLoopSimplify.\
\>", "Text",
 CellTags->"DimensionalReduction"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TID", All, CellTags],
  ButtonData->"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"DimensionalReduction"]
}, Open  ]],

Cell["DiracBasis", "ObjectName",
 CellTags->"DiracBasis"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracBasis"],

Cell["\<\
DiracBasis[any] is a head which is wrapped around Dirac structures (and the \
1) as a result of the function DiracReduce. Eventually you want to substitute \
DiracBasis by Identity (or set: DiracBasis[1] = S; \
DiracBasis[DiracMatrix[mu]] = P; etc.).\
\>", "Text",
 CellTags->"DiracBasis"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracReduce", All, CellTags],
  ButtonData->"DiracReduce",
  ButtonNote->"DiracReduce"],
 "."
}], "Text",
 CellTags->"DiracBasis"]
}, Open  ]],

Cell["DiracCanonical", "ObjectName",
 CellTags->"DiracCanonical"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracCanonical"],

Cell["\<\
DiracCanonical is an option for DiracSimplify. If set to True DiracSimplify \
uses the function DiracOrder internally.\
\>", "Text",
 CellTags->"DiracCanonical"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracCanonical"]
}, Open  ]],

Cell["DiracEquation", "ObjectName",
 CellTags->"DiracEquation"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracEquation"],

Cell["\<\
DiracEquation[exp] applies the Dirac equation without expanding exp. If that \
is needed, use DiracSimplify.\
\>", "Text",
 CellTags->"DiracEquation"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracEquation"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracEquation"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "p", "]"}], "]"}]}]], "Input",
 CellTags->"DiracEquation",
 CellLabel->"In[368]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracEquation",
 CellLabel->"Out[368]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracEquation", "[", "%", "]"}]], "Input",
 CellTags->"DiracEquation",
 CellLabel->"In[369]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"m", " ", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracEquation",
 CellLabel->"Out[369]="]
}, Open  ]]
}, Open  ]],

Cell["DiracGamma", "ObjectName",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGamma"],

Cell[TextData[{
 "DiracGamma[x, dim] is the head of all Dirac matrices and slashes (in the \
internal representation). Use DiracMatrix (or GA, GAD) and DiracSlash (or GS, \
GSD) for manual (short) input. DiracGamma[x, 4] simplifies to DiracGamma[x]. \
DiracGamma[5] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ". DiracGamma[6] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "+", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], 
   TraditionalForm]]],
 " DiracGamma[7] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["\[Gamma]", "5"]}], ")"}], "/", "2."}], TraditionalForm]]]
}], "Text",
 CellTags->"DiracGamma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGammaExpand", All, CellTags],
  ButtonData->"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSlash", All, CellTags],
  ButtonData->"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGamma"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGamma", "[", "5", "]"}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[370]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[370]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[371]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Alpha]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[371]="]
}, Open  ]],

Cell[TextData[{
 "A Dirac-slash, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    SubscriptBox["q", "\[Mu]"]}], TraditionalForm]]],
 ", is displayed as ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DiracGamma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], " "}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[372]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[372]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[373]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[373]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"q", ",", "D"}], "]"}], ",", "D"}], "]"}], " "}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[374]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[374]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}]}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[375]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[375]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", "=", 
  RowBox[{"DiracGammaExpand", "[", "a1", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[376]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[376]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", "=", 
  RowBox[{
   RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"GSD", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ".", 
   RowBox[{"GSD", "[", "q", "]"}], ".", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}]}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[377]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[377]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a4", "=", 
  RowBox[{"DiracTrick", "[", "a3", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[378]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"D", "-", "4"}], ")"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", "(", 
       FormBox[
        RowBox[{"p", "-", "q"}],
        TraditionalForm], ")"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")", "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[378]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a5", "=", 
  RowBox[{"DiracSimplify", "[", "a4", "]"}]}]], "Input",
 CellTags->"DiracGamma",
 CellLabel->"In[379]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"D", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}]}]}], "-", 
   RowBox[{"D", " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracGamma",
 CellLabel->"Out[379]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "a3", ",", "a4", ",", "a5"}], "]"}]], "Input",\

 CellTags->"DiracGamma",
 CellLabel->"In[380]:="]
}, Open  ]],

Cell["DiracGammaCombine", "ObjectName",
 CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaCombine"],

Cell["\<\
DiracGammaCombine[exp] is (nearly) the inverse operation to DiracGammaExpand.\
\
\>", "Text",
 CellTags->"DiracGammaCombine"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGammaExpand", All, CellTags],
  ButtonData->"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSlash", All, CellTags],
  ButtonData->"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGammaCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaCombine", "[", 
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], " ", "+", " ", 
   RowBox[{"GS", "[", "q", "]"}]}], "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[381]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[381]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[382]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "+", "q"}], "]"}], "]"}]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[382]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaCombine", "[", 
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"GS", "[", "p", "]"}]}], " ", "-", " ", 
   RowBox[{"2", " ", 
    RowBox[{"GS", "[", "q", "]"}]}]}], "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[383]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "-", 
     RowBox[{"2", " ", "q"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[383]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[384]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", 
   RowBox[{
    RowBox[{"2", " ", "p"}], "-", 
    RowBox[{"2", " ", "q"}]}], "]"}], "]"}]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[384]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaExpand", "[", "%%", "]"}]], "Input",
 CellTags->"DiracGammaCombine",
 CellLabel->"In[385]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], "-", 
   RowBox[{"2", " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaCombine",
 CellLabel->"Out[385]="]
}, Open  ]]
}, Open  ]],

Cell["DiracGammaExpand", "ObjectName",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaExpand"],

Cell["\<\
DiracGammaExpand[exp] expands all DiracGamma[Momentum[a+b+..]] in exp into \
(DiracGamma[Momentum[a]] + DiracGamma[Momentum[b]] + ...).\
\>", "Text",
 CellTags->"DiracGammaExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracGammaCombine",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGammaCombine", All, CellTags],
  ButtonData->"DiracGammaCombine",
  ButtonNote->"DiracGammaCombine"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSlash", All, CellTags],
  ButtonData->"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracGammaExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "q", "]"}], "]"}], " ", ".", " ", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", 
     RowBox[{"p", "-", "q"}], "]"}], "]"}]}]}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[386]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[386]="]
}, Open  ]],

Cell["\<\
Momentum is the head of p-q, i.e., it is treated as one four-momentum.\
\>", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[387]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[387]//StandardForm="]
}, Open  ]],

Cell["With DiracGammaExpand the Momentum[p-q] gets expanded.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaExpand", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[388]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[388]="]
}, Open  ]],

Cell["The inverse operation is DiracGammaCombine.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"DiracGammaCombine", "[", 
   RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}], "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[389]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], "]"}]}]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[389]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"DiracGammaExpand", "[", "t", "]"}], "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[390]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"Momentum", "[", "q", "]"}], "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "p", "]"}], "]"}], "-", 
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "q", "]"}], "]"}]}], ")"}]}]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[390]//StandardForm="]
}, Open  ]],

Cell["In order to do non-commutative expansion use DiracSimplify.", "Text",
 CellTags->"DiracGammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[391]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"DiracGammaExpand",
 CellLabel->"Out[391]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"DiracGammaExpand",
 CellLabel->"In[392]:="]
}, Open  ]],

Cell["DiracGammaT", "ObjectName",
 CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracGammaT"],

Cell["\<\
DiracGammaT[x] denotes the transpose of DiracGamma[x]. \
Transpose[DiracGammaT[x]] gives DiracGamma[x]. Note that x must have Head \
LorentzIndex or Momentum.\
\>", "Text",
 CellTags->"DiracGammaT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 "."
}], "Text",
 CellTags->"DiracGammaT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracGammaT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracGammaT", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"DiracGammaT",
 CellLabel->"In[393]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], "T"], TraditionalForm]], "Output",
 CellTags->"DiracGammaT",
 CellLabel->"Out[393]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Transpose", "[", "%", "]"}]], "Input",
 CellTags->"DiracGammaT",
 CellLabel->"In[394]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"DiracGammaT",
 CellLabel->"Out[394]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", "Transpose"}]], "Input",\

 CellTags->"DiracGammaT",
 CellLabel->"In[395]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", "\[Gamma]", "\[CenterDot]", 
    FormBox["p",
     TraditionalForm], ")"}], "T"], TraditionalForm]], "Output",
 CellTags->"DiracGammaT",
 CellLabel->"Out[395]="]
}, Open  ]]
}, Open  ]],

Cell["DiracMatrix", "ObjectName",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracMatrix"],

Cell[TextData[{
 "DiracMatrix[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes a Dirac gamma matrix with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ". DiracMatrix[",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Mu]", ",", "\[Nu]", ","}], " "}], TraditionalForm]]],
 "...] is a product of ",
 Cell[BoxData[
  FormBox["\[Gamma]", TraditionalForm]]],
 " matrices with Lorentz indices ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
 " DiracMatrix[5] is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ". DiracMatrix[6] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", "2"}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"+", " ", 
      SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm]]],
 ". DiracMatrix[7] is",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"1", "/", "2"}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     RowBox[{"-", " ", 
      SuperscriptBox["\[Gamma]", "5"]}], "/", "2"}]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"DiracMatrix"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGammaExpand", All, CellTags],
  ButtonData->"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSlash", All, CellTags],
  ButtonData->"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GA", All, CellTags],
  ButtonData->"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GAD", All, CellTags],
  ButtonData->"GAD",
  ButtonNote->"GAD"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GS", All, CellTags],
  ButtonData->"GS",
  ButtonNote->"GS"],
 ", ",
 ButtonBox["GSD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GSD", All, CellTags],
  ButtonData->"GSD",
  ButtonNote->"GSD"],
 "."
}], "Text",
 CellTags->"DiracMatrix"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[396]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[396]="]
}, Open  ]],

Cell[TextData[{
 "This is how to enter the non-commutative product of two ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], TraditionalForm]]],
 ". The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Dot \".\" is used as non-commutative multiplication operator."
}], "Text",
 CellTags->"DiracMatrix"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[397]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[397]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Alpha]", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[398]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], "]"}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[398]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], "//", "FCE"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[399]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[399]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[400]:="],

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[400]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[401]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[401]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[402]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[402]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[403]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[403]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", 
   RowBox[{"a", " ", ".", " ", "b"}], "]"}], "//", "FCI"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[404]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[404]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracMatrix",
 CellLabel->"In[405]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "a", "]"}], "]"}], ".", 
  RowBox[{"DiracGamma", "[", 
   RowBox[{"LorentzIndex", "[", "b", "]"}], "]"}]}]], "Output",
 CellTags->"DiracMatrix",
 CellLabel->"Out[405]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracOrder", "ObjectName",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracOrder"],

Cell["\<\
DiracOrder[expr] orders the Dirac matrices in expr alphabetically. \
DiracOrder[expr,orderlist] orders the Dirac matrices in expr according to \
orderlist.\
\>", "Text",
 CellTags->"DiracOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracOrder"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Beta]", ",", "\[Alpha]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[406]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[406]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[407]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[407]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[408]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[408]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t1", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[409]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[409]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[410]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[410]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", 
    RowBox[{"5", " "}]], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t3", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[411]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[411]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[412]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[412]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "t4", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[413]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[413]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]"}], "]"}]}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[414]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[414]="]
}, Open  ]],

Cell[TextData[{
 "This orders the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Alpha]"], 
    SuperscriptBox["\[Gamma]", "\[Beta]"], 
    SuperscriptBox["\[Gamma]", "\[Delta]"]}], TraditionalForm]]],
 " in reverse order."
}], "Text",
 CellTags->"DiracOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", 
  RowBox[{"t5", ",", 
   RowBox[{"{", 
    RowBox[{"\[Delta]", ",", "\[Beta]", ",", "\[Alpha]"}], "}"}]}], 
  "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[415]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Delta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Delta]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Delta]"}]]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Beta]", "\[Delta]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[415]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracOrder", "[", "%", "]"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[416]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Delta]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracOrder",
 CellLabel->"Out[416]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5"}], "]"}], 
  ";"}]], "Input",
 CellTags->"DiracOrder",
 CellLabel->"In[417]:="]
}, Open  ]],

Cell["DiracReduce", "ObjectName",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracReduce"],

Cell["\<\
DiracReduce[exp] reduces all four-dimensional Dirac matrices in exp to the \
standard basis (S,P,V,A,T) using the Chisholm identity (see Chisholm). In the \
result the basic Dirac structures are wrapped with a head DiracBasis. I.e., S \
corresponds to DiracBasis[1], P : DiracBasis[DiracMatrix[5]], V: \
DiracBasis[DiracMatrix[mu]], A: DiracBasis[DiracMatrix[mu, 5]], T: \
DiracBasis[DiracSigma[DiracMatrix[mu, nu]]]. By default DiracBasis is \
substituted to Identity. Notice that the result of DiracReduce is given in \
the FeynCalcExternal-way, i.e.,evtl. you may have to use FeynCalcInternal on \
the result.\
\>", "Text",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracReduce", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[418]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"DiracBasis", "\[Rule]", "Identity"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[418]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[419]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[419]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracReduce", "[", "t1", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[420]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[420]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[421]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[421]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracReduce", "[", "t2", "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[422]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      RowBox[{"$MU", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}]], ".", 
     SuperscriptBox["\[Gamma]", "5"]}], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", 
      FormBox[
       RowBox[{"$MU", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Rho]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[422]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
   "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[423]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], 
   SuperscriptBox["\[Gamma]", "\[Nu]"], 
   SuperscriptBox["\[Gamma]", "\[Rho]"], 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[423]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"DiracReduce", "[", "t3", "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[424]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["\[Gamma]", "5"], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]]}]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
    SuperscriptBox["\[Sigma]", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[424]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", " ", "=", 
  RowBox[{"Calc", "[", 
   RowBox[{"DiracSimplify", "[", 
    RowBox[{"DiracSigmaExplicit", "[", 
     RowBox[{"t4", ".", "t4"}], "]"}], "]"}], "]"}]}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[425]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "128"}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[425]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"t4", ".", "t4"}], "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[426]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "128"}], TraditionalForm]], "Output",
 CellTags->"DiracReduce",
 CellLabel->"Out[426]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4"}], "]"}]], "Input",
 CellTags->"DiracReduce",
 CellLabel->"In[427]:="]
}, Open  ]],

Cell["DiracSigma", "ObjectName",
 CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSigma"],

Cell["\<\
DiracSigma[a, b] stands for i/2*(a . b - b . a) in 4 dimensions. a and b must \
have Head DiracGamma, DiracMatrix or DiracSlash. Only antisymmetry is \
implemented.\
\>", "Text",
 CellTags->"DiracSigma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSigmaExplicit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSigmaExplicit", All, CellTags],
  ButtonData->"DiracSigmaExplicit",
  ButtonNote->"DiracSigmaExplicit"],
 "."
}], "Text",
 CellTags->"DiracSigma"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSigma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
    RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[428]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[428]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[429]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[429]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GA", "[", "\[Beta]", "]"}], ",", 
    RowBox[{"GA", "[", "\[Alpha]", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[430]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox["\[Sigma]", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[430]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"DiracSigma", "[", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], ",", 
    RowBox[{"GS", "[", "q", "]"}]}], "]"}]}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[431]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"p", "q"}]], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[431]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "t3", "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[432]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}]}], "-", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSigma",
 CellLabel->"Out[432]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"DiracSigma",
 CellLabel->"In[433]:="]
}, Open  ]],

Cell["DiracSigmaExplicit", "ObjectName",
 CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSigmaExplicit"],

Cell["\<\
DiracSigmaExplicit[exp] inserts in exp for all DiracSigma its definition. \
DiracSigmaExplict is also an option of DiracSimplify.\
\>", "Text",
 CellTags->"DiracSigmaExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSigma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSigma", All, CellTags],
  ButtonData->"DiracSigma",
  ButtonNote->"DiracSigma"],
 "."
}], "Text",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSigmaExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigma", "[", 
  RowBox[{
   RowBox[{"GA", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"GA", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"In[434]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Sigma]", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"Out[434]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSigmaExplicit", "[", "%", "]"}]], "Input",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"In[435]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Beta]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSigmaExplicit",
 CellLabel->"Out[435]="]
}, Open  ]]
}, Open  ]],

Cell["DiracSimpCombine", "ObjectName",
 CellTags->"DiracSimpCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimpCombine"],

Cell["\<\
DiracSimpCombine is an option for DiracSimplify. If set to True, sums of \
DiracGamma's will be merged as much as possible in DiracGamma[ .. + .. + ]'s.\
\
\>", "Text",
 CellTags->"DiracSimpCombine"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSimpCombine"]
}, Open  ]],

Cell["DiracSimplify", "ObjectName",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimplify"],

Cell["\<\
DiracSimplify[expr] simplifies products of Dirac matrices in expr and expands \
non-commutative products. Double Lorentz indices and four vectors are \
contracted. The Dirac equation is applied. All DiracMatrix[5], DiracMatrix[6] \
and DiracMatrix[7] are moved to the right. The order of the other Dirac \
matrices is not changed.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracSimplify", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[436]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracCanonical", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSigmaExplicit", "\[Rule]", "True"}], ",", 
    RowBox[{"DiracSimpCombine", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSubstitute67", "\[Rule]", "False"}], ",", 
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "False"}], ",", 
    RowBox[{"InsideDiracTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[436]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGammaExpand", All, CellTags],
  ButtonData->"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"DiracSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSimplify"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[437]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[437]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[438]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[438]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[439]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[439]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[440]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[440]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[441]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[441]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t3", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[442]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[442]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[443]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[443]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t4", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[444]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[444]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", 
    RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[445]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[445]="]
}, Open  ]],

Cell["\<\
Contrary to DiracTrick DiracSimplify does non-commutative expansion.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t5", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[446]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[446]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[447]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", "(", 
      FormBox[
       RowBox[{"a", "+", "b"}],
       TraditionalForm], ")"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", "(", 
      FormBox[
       RowBox[{"c", "+", "d"}],
       TraditionalForm], ")"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[447]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", " ", "=", " ", 
  RowBox[{
   RowBox[{"SpinorVBar", "[", "p", "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], " ", ".", " ", 
   RowBox[{"SpinorUBar", "[", "q", "]"}]}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[448]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"v\"\>", "_"],
     TraditionalForm], "(", "p", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", "q", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[448]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "t6", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[449]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[449]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "@@", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", "\[Mu]", "}"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      SubscriptBox["\[Nu]", "i"], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[450]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[450]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", "%", "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[451]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"12", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "1"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "2"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "3"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "4"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "5"],
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox[
         SubscriptBox["\[Nu]", "6"],
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{"D", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "2"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "3"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "1"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "3"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "2"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "4"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "3"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], 
      SubscriptBox["\[Nu]", "5"]}]]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "4"], 
      SubscriptBox["\[Nu]", "6"]}]]}], "+", 
   RowBox[{"4", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{
      SubscriptBox["\[Nu]", "5"], 
      SubscriptBox["\[Nu]", "6"]}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[451]="]
}, Open  ]],

Cell["\<\
 With the option DiracCanonical an alphabetic ordering is done.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}], ",", 
   RowBox[{"DiracCanonical", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[452]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[452]="]
}, Open  ]],

Cell["\<\
Setting InsideDiracTrace\[Rule]True assumes that a trace is still to be taken \
later on.\
\>", "Text",
 CellTags->"DiracSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
   ",", 
   RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[453]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[453]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], ",", 
   RowBox[{"InsideDiracTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[454]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"DiracSimplify",
 CellLabel->"Out[454]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
  "]"}]], "Input",
 CellTags->"DiracSimplify",
 CellLabel->"In[455]:="]
}, Open  ]],

Cell["DiracSimplify2", "ObjectName",
 CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSimplify2"],

Cell[TextData[{
 "DiracSimplify2[exp] simplifies the Dirac structure but leaves any ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " untouched."
}], "Text",
 CellTags->"DiracSimplify2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSimplify2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSimplify2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{
  "\[Mu]", ",", "\[Nu]", ",", "\[Mu]", ",", "5", ",", "\[Alpha]", ",", 
   "\[Beta]", ",", "\[Alpha]"}], "]"}]], "Input",
 CellTags->"DiracSimplify2",
 CellLabel->"In[456]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify2",
 CellLabel->"Out[456]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify2", "[", "%", "]"}]], "Input",
 CellTags->"DiracSimplify2",
 CellLabel->"In[457]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracSimplify2",
 CellLabel->"Out[457]="]
}, Open  ]]
}, Open  ]],

Cell["DiracSlash", "ObjectName",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSlash"],

Cell[TextData[{
 "DiracSlash[p] is the contraction ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], 
    SubscriptBox["\[Gamma]", 
     RowBox[{"\[Mu]", " "}]]}], TraditionalForm]]],
 "(FourVector[p, ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] DiracMatrix[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]). Products of those can be entered in the form DiracSlash[p1, p2, ...]."
}], "Text",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracSlash", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[458]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[458]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGammaExpand", All, CellTags],
  ButtonData->"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GS", All, CellTags],
  ButtonData->"GS",
  ButtonNote->"GS"],
 ", ",
 ButtonBox["GSD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GSD", All, CellTags],
  ButtonData->"GSD",
  ButtonNote->"GSD"],
 "."
}], "Text",
 CellTags->"DiracSlash"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracSlash"],

Cell[TextData[{
 "This is q-slash, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], 
    RowBox[{
     SubscriptBox["q", "\[Mu]"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"DiracSlash"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", "q", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[459]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[459]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
  RowBox[{"DiracSlash", "[", "q", "]"}]}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[460]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[460]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[461]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[461]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", "p", "]"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[462]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[462]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", "q", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[463]:="],

Cell[BoxData[
 RowBox[{"DiracSlash", "[", "q", "]"}]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[463]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", 
   RowBox[{"q", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"DiracSlash",
 CellLabel->"In[464]:="],

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"q", ",", 
   RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]], "Output",
 CellTags->"DiracSlash",
 CellLabel->"Out[464]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracSpinor", "ObjectName",
 CellTags->"DiracSpinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSpinor"],

Cell["\<\
DiracSpinor is simply a quantity defined as noncommutative (with \
DeclareNonCommutative[DiracSpinor]). The convention intended is that \
DiracSpinor[p, m, ind] is a Dirac spinor for a fermion with momentum p and \
mass m and indices ind.\
\>", "Text",
 CellTags->"DiracSpinor"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Spinor", All, CellTags],
  ButtonData->"Spinor",
  ButtonNote->"Spinor"],
 "."
}], "Text",
 CellTags->"DiracSpinor"]
}, Open  ]],

Cell["DiracSubstitute67", "ObjectName",
 CellTags->"DiracSubstitute67"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracSubstitute67"],

Cell["\<\
DiracSubstitute67 is an option for DiracSimplify. If set to True the \
chirality-projectors DiracGamma[6] and DiracGamma[7] are substituted by their \
definitions.\
\>", "Text",
 CellTags->"DiracSubstitute67"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracSubstitute67"]
}, Open  ]],

Cell["DiracTrace", "ObjectName",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTrace"],

Cell["\<\
DiracTrace[expr] is the head of Dirac traces. Whether the trace is evaluated \
depends on the option DiracTraceEvaluate. Direct trace evaluation should be \
performed with TR (or Tr). The argument expr may be a product of Dirac \
matrices or slashes separated by the Mathematica Dot (.).\
\>", "Text",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracTrace", "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[465]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsContract", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PairCollect", "\[Rule]", "True"}], ",", 
    RowBox[{"DiracTraceEvaluate", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "0"}], ",", 
    RowBox[{"LeviCivitaSign", "\[RuleDelayed]", "$LeviCivitaSign"}], ",", 
    RowBox[{"TraceOfOne", "\[Rule]", "4"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[465]="]
}, Open  ]],

Cell[TextData[{
 "For comments regarding ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "schemes see the notes for ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TR", All, CellTags],
  ButtonData->"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTrace"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TR", All, CellTags],
  ButtonData->"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTrace"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[466]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[466]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[467]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[467]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"DiracTrace", "\[Rule]", "Tr"}]}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[468]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[468]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}],
     "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[469]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[469]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{
    "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Delta]", 
     ",", "\[Tau]", ",", "5"}], "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[470]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Mu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Nu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Delta]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Delta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[470]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[471]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[471]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
   RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[472]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[472]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"DiracTrace",
 CellLabel->"In[473]:="],

Cell[BoxData[
 RowBox[{"4", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Output",
 CellTags->"DiracTrace",
 CellLabel->"Out[473]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["DiracTraceEvaluate", "ObjectName",
 CellTags->"DiracTraceEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTraceEvaluate"],

Cell["\<\
DiracTraceEvaluate is an option for DiracTrace and Tr. If set to False, \
DiracTrace remains unevaluated.\
\>", "Text",
 CellTags->"DiracTraceEvaluate"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TR", All, CellTags],
  ButtonData->"TR",
  ButtonNote->"TR"],
 "."
}], "Text",
 CellTags->"DiracTraceEvaluate"]
}, Open  ]],

Cell["DiracTrick", "ObjectName",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DiracTrick"],

Cell["\<\
DiracTrick[exp] contracts gamma matrices with each other and performs several \
simplifications, but no expansion, use Calc or DiracSimplify for \
non-commutative expansion.\
\>", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DiracTrick", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[474]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Expanding", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[474]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["DiracGammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGammaExpand", All, CellTags],
  ButtonData->"DiracGammaExpand",
  ButtonNote->"DiracGammaExpand"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"DiracTrick"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DiracTrick"],

Cell["This is a string of Dirac matrices in four dimensions. ", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[475]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[475]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t1", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[476]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[476]="]
}, Open  ]],

Cell["This is a string of Dirac matrices in D dimensions. ", "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[477]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[477]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t2", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[478]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[478]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"5", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[479]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "5"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[479]="]
}, Open  ]],

Cell[TextData[{
 "By default ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "is moved to the right."
}], "Text",
 CellTags->"DiracTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t3", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[480]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[480]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"GA", "[", 
   RowBox[{"6", ",", "\[Mu]", ",", "7"}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[481]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "6"], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[481]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t4", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[482]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "7"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[482]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"GS", "[", 
    RowBox[{"a", "+", "b"}], "]"}], " ", ".", " ", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", "p", "]"}], ".", 
   RowBox[{"GS", "[", 
    RowBox[{"c", "+", "d"}], "]"}]}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[483]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"a", "+", "b"}],
      TraditionalForm], ")"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"c", "+", "d"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[483]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "t5", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[484]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", "(", 
      FormBox[
       RowBox[{"a", "+", "b"}],
       TraditionalForm], ")"}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", "(", 
      FormBox[
       RowBox[{"c", "+", "d"}],
       TraditionalForm], ")"}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[484]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "t5", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[485]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["a",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["c",
        TraditionalForm]}], ")"}]}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["b",
        TraditionalForm]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["d",
        TraditionalForm]}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[485]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "@@", 
  RowBox[{"Join", "[", 
   RowBox[{
    RowBox[{"{", "\[Mu]", "}"}], ",", 
    RowBox[{"Table", "[", 
     RowBox[{
      SubscriptBox["\[Nu]", "i"], ",", 
      RowBox[{"{", 
       RowBox[{"i", ",", "6"}], "}"}]}], "]"}], ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}]}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[486]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "1"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "2"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "3"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "4"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "5"]], ".", 
   SuperscriptBox["\[Gamma]", 
    SubscriptBox["\[Nu]", "6"]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[486]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrick", "[", "%", "]"}]], "Input",
 CellTags->"DiracTrick",
 CellLabel->"In[487]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"D", "-", "12"}], ")"}], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "1"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "2"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "3"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "4"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "5"],
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox[
        SubscriptBox["\[Nu]", "6"],
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Gamma]", 
          FormBox[
           FormBox[
            SubscriptBox["\[Nu]", "3"],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox["\[Gamma]", 
          FormBox[
           FormBox[
            SubscriptBox["\[Nu]", "4"],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox["\[Gamma]", 
          FormBox[
           FormBox[
            SubscriptBox["\[Nu]", "5"],
            TraditionalForm],
           TraditionalForm]], ".", 
         SuperscriptBox["\[Gamma]", 
          FormBox[
           FormBox[
            SubscriptBox["\[Nu]", "6"],
            TraditionalForm],
           TraditionalForm]]}], " ", 
        SuperscriptBox["g", 
         RowBox[{
          SubscriptBox["\[Nu]", "1"], 
          SubscriptBox["\[Nu]", "2"]}]]}]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "1"], 
         SubscriptBox["\[Nu]", "3"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "1"], 
         SubscriptBox["\[Nu]", "4"]}]]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "1"], 
         SubscriptBox["\[Nu]", "5"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "1"], 
         SubscriptBox["\[Nu]", "6"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "2"], 
         SubscriptBox["\[Nu]", "3"]}]]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "2"], 
         SubscriptBox["\[Nu]", "4"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "2"], 
         SubscriptBox["\[Nu]", "5"]}]]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "2"], 
         SubscriptBox["\[Nu]", "6"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "3"], 
         SubscriptBox["\[Nu]", "4"]}]]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "3"], 
         SubscriptBox["\[Nu]", "5"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "3"], 
         SubscriptBox["\[Nu]", "6"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "6"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "4"], 
         SubscriptBox["\[Nu]", "5"]}]]}], "+", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "5"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "4"], 
         SubscriptBox["\[Nu]", "6"]}]]}], "-", 
      RowBox[{
       RowBox[{
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "1"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "2"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "3"],
           TraditionalForm],
          TraditionalForm]], ".", 
        SuperscriptBox["\[Gamma]", 
         FormBox[
          FormBox[
           SubscriptBox["\[Nu]", "4"],
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox["g", 
        RowBox[{
         SubscriptBox["\[Nu]", "5"], 
         SubscriptBox["\[Nu]", "6"]}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"DiracTrick",
 CellLabel->"Out[487]="]
}, Open  ]]
}, Open  ]],

Cell["Divideout", "ObjectName",
 CellTags->"Divideout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Divideout"],

Cell["\<\
Divideout is an option for OPEInt and OPEInsert. The setting is divided out \
at the end.\
\>", "Text",
 CellTags->"Divideout"],

Cell[TextData[{
 "See also: ",
 ButtonBox["OPEInt",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPEInt", All, CellTags],
  ButtonData->"OPEInt",
  ButtonNote->"OPEInt"],
 ", ",
 ButtonBox["OPEInsert",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPEInsert", All, CellTags],
  ButtonData->"OPEInsert",
  ButtonNote->"OPEInsert"],
 "."
}], "Text",
 CellTags->"Divideout"]
}, Open  ]],

Cell["DOT", "ObjectName",
 CellTags->"DOT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DOT"],

Cell["\<\
DOT[a, b, ...] is the FeynCalc function for non-commutative multiplication. \
By default it is set to the Mathematica Dot functions. By setting
DOT=.
this can be disabled. Note that then non-commutative products should to be \
entered like DOT[ DiracMatrix[mu], m + DiracSlash[p], DiracMatrix[mu] ], etc.\
\
\>", "Text",
 CellTags->"DOT"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DOT"]
}, Open  ]],

Cell["DotExpand", "ObjectName",
 CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotExpand"],

Cell["DotExpand[expr] expands DOT products in expr.", "Text",
 CellTags->"DotExpand"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DOT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DOT", All, CellTags],
  ButtonData->"DOT",
  ButtonNote->"DOT"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeclareNonCommutative", All, CellTags],
  ButtonData->"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 ", ",
 ButtonBox["UnDeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "UnDeclareNonCommutative", All, CellTags],
  ButtonData->"UnDeclareNonCommutative",
  ButtonNote->"UnDeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"DotExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[488]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", "d", " ", "x"}], "+", 
   RowBox[{"a", " ", "e", " ", "x"}], "+", 
   RowBox[{"a", " ", "f", " ", "x"}], "+", 
   RowBox[{"b", " ", "d", " ", "y"}], "+", 
   RowBox[{"b", " ", "e", " ", "y"}], "+", 
   RowBox[{"b", " ", "f", " ", "y"}], "+", 
   RowBox[{"c", " ", "d", " ", "z"}], "+", 
   RowBox[{"c", " ", "e", " ", "z"}], "+", 
   RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
 CellTags->"DotExpand",
 CellLabel->"Out[488]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeclareNonCommutative", "/@", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}]}], 
  ";"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[489]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[490]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "d"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "e"}]}], "+", 
   RowBox[{"x", " ", 
    RowBox[{"a", ".", "f"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "d"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "e"}]}], "+", 
   RowBox[{"y", " ", 
    RowBox[{"b", ".", "f"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "d"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "e"}]}], "+", 
   RowBox[{"z", " ", 
    RowBox[{"c", ".", "f"}]}]}], TraditionalForm]], "Output",
 CellTags->"DotExpand",
 CellLabel->"Out[490]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"UnDeclareNonCommutative", "/@", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}]}], 
  ";"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[491]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotExpand", "[", 
  RowBox[{"DOT", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", " ", "x"}], "+", 
     RowBox[{"b", " ", "y"}], "+", 
     RowBox[{"c", " ", "z"}]}], ",", 
    RowBox[{"d", "+", "e", "+", "f"}]}], "]"}], "]"}]], "Input",
 CellTags->"DotExpand",
 CellLabel->"In[492]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", "d", " ", "x"}], "+", 
   RowBox[{"a", " ", "e", " ", "x"}], "+", 
   RowBox[{"a", " ", "f", " ", "x"}], "+", 
   RowBox[{"b", " ", "d", " ", "y"}], "+", 
   RowBox[{"b", " ", "e", " ", "y"}], "+", 
   RowBox[{"b", " ", "f", " ", "y"}], "+", 
   RowBox[{"c", " ", "d", " ", "z"}], "+", 
   RowBox[{"c", " ", "e", " ", "z"}], "+", 
   RowBox[{"c", " ", "f", " ", "z"}]}], TraditionalForm]], "Output",
 CellTags->"DotExpand",
 CellLabel->"Out[492]="]
}, Open  ]]
}, Open  ]],

Cell["DotPower", "ObjectName",
 CellTags->"DotPower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotPower"],

Cell["\<\
DotPower is an option for DotSimplify. It determines whether non-commutative \
powers are represented by succesive multiplication or by Power.\
\>", "Text",
 CellTags->"DotPower"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DotPower"]
}, Open  ]],

Cell["DotProduct", "ObjectName",
 CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotProduct"],

Cell["\<\
DotProduct[x, y] denotes the three-dimensional dot-product. If x and y have \
Head List, DotProduct[x, a] (where a is a vector) performs Sum[ x[[k]] \
a[[k]], {k, 0, 3}].\
\>", "Text",
 CellTags->"DotProduct"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CrossProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CrossProduct", All, CellTags],
  ButtonData->"CrossProduct",
  ButtonNote->"CrossProduct"],
 ", ",
 ButtonBox["ThreeVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ThreeVector", All, CellTags],
  ButtonData->"ThreeVector",
  ButtonNote->"ThreeVector"],
 "."
}], "Text",
 CellTags->"DotProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotProduct", "[", 
  RowBox[{
   RowBox[{"ThreeVector", "[", "a", "]"}], ",", 
   RowBox[{"3", 
    RowBox[{"ThreeVector", "[", "b", "]"}]}]}], "]"}]], "Input",
 CellTags->"DotProduct",
 CellLabel->"In[493]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"3", " ", 
   RowBox[{
    FormBox[
     OverscriptBox["a", "\[RightVector]"],
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     OverscriptBox["b", "\[RightVector]"],
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"DotProduct",
 CellLabel->"Out[493]="]
}, Open  ]]
}, Open  ]],

Cell["DotSimplifyRelations", "ObjectName",
 CellTags->"DotSimplifyRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotSimplifyRelations"],

Cell["\<\
DotSimplifyRelations is an option for DotSimplify. Its setting may be a list \
of substitution rules of the form DotSimplifyRelations \[Rule] {a . b \[Rule] \
c, b^2 \[Rule] 0, ...}.\
\>", "Text",
 CellTags->"DotSimplifyRelations"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 "."
}], "Text",
 CellTags->"DotSimplifyRelations"]
}, Open  ]],

Cell["DotSimplify", "ObjectName",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DotSimplify"],

Cell["\<\
DotSimplify[expr] expands and reorders noncommutative terms in expr. \
Simplifying relations may be specified by the option DotSimplifyRelations or \
by Commutator and AntiCommutator definitions. Whether expr is expanded \
noncommutatively depends on the option Expanding.\
\>", "Text",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "DotSimplify", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[494]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Expanding", "\[Rule]", "True"}], ",", 
    RowBox[{"DotSimplifyRelations", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"DotPower", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcInternal", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[494]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiCommutator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "AntiCommutator", All, CellTags],
  ButtonData->"AntiCommutator",
  ButtonNote->"AntiCommutator"],
 ", ",
 ButtonBox["Commutator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Commutator", All, CellTags],
  ButtonData->"Commutator",
  ButtonNote->"Commutator"],
 ", ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 "."
}], "Text",
 CellTags->"DotSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DotSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"GS", "[", "p", "]"}]}], "-", 
     RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
   RowBox[{"GA", "[", "\[Nu]", "]"}]}]}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[495]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], "-", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "q"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[495]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[496]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[496]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[497]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"a", ".", 
   RowBox[{"(", 
    RowBox[{"b", "-", 
     RowBox[{"z", " ", "c"}]}], ")"}], ".", "a"}]}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[498]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"a", ".", 
   RowBox[{"(", 
    RowBox[{"b", "-", 
     RowBox[{"c", " ", "z"}]}], ")"}], ".", "a"}], TraditionalForm]], "Output",\

 CellTags->"DotSimplify",
 CellLabel->"Out[498]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[499]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[499]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "=", "1"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[500]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[500]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[501]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"c", ".", "a", ".", "a"}], "+", "a"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[501]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Commutator", "[", 
   RowBox[{"a", ",", "c"}], "]"}], "=", "."}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[502]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[503]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[503]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"b", ",", "a"}], "]"}], "=", "c"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[504]:="],

Cell[BoxData[
 FormBox["c", TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[504]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[505]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"a", ".", "a", ".", "b"}]}], "-", 
   RowBox[{"z", " ", 
    RowBox[{"a", ".", "c", ".", "a"}]}], "+", 
   RowBox[{"a", ".", "c"}]}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[505]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"AntiCommutator", "[", 
   RowBox[{"b", ",", "a"}], "]"}], "=", "."}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[506]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"DotSimplifyRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a", ".", "c"}], "\[Rule]", 
      RowBox[{"1", "/", "z"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[507]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", ".", "b", ".", "a"}], "-", "a"}], TraditionalForm]], "Output",\

 CellTags->"DotSimplify",
 CellLabel->"Out[507]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[508]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{"x", ".", "x", ".", "x"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[509]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[509]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DotSimplify", "[", 
  RowBox[{
   RowBox[{"x", ".", "x", ".", "x"}], ",", 
   RowBox[{"DotPower", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[510]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", ".", "x", ".", "x"}], TraditionalForm]], "Output",
 CellTags->"DotSimplify",
 CellLabel->"Out[510]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "x"}], "]"}]], "Input",
 CellTags->"DotSimplify",
 CellLabel->"In[511]:="]
}, Open  ]],

Cell["DummyIndex", "ObjectName",
 CellTags->"DummyIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DummyIndex"],

Cell["\<\
DummyIndex is an option of CovariantD specifying an index to use as dummy \
summation index. If set to Automatic, unique indices are generated\
\>", "Text",
 CellTags->"DummyIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CovariantD", All, CellTags],
  ButtonData->"CovariantD",
  ButtonNote->"CovariantD"],
 "."
}], "Text",
 CellTags->"DummyIndex"]
}, Open  ]],

Cell["D0", "ObjectName",
 CellTags->"D0"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"D0"],

Cell[TextData[{
 "D0[ p10, p12, p23, p30, p20, p13, m1^2, m2^2, m3^2, m4^2 ] is the \
Passarino-Veltman ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["D", "0"], TraditionalForm]]],
 " function. The convention for the arguments is that if the denominator of \
the integrand has the form ([q^2-m1^2] [(q+p1)^2-m2^2] [(q+p2)^2-m3^2] \
[(q+p3)^2-m4^2] ), the first six arguments of D0 are the scalar products p10 \
= p1^2, p12 = (p1-p2)^2, p23 = (p2-p3)^2, p30 = p3^2, p20 = p2^2, p13 = \
(p1-p3)^2."
}], "Text",
 CellTags->"D0"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "C0", All, CellTags],
  ButtonData->"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeOrder", All, CellTags],
  ButtonData->"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"D0"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"D0"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D0", "[", 
  RowBox[{
  "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", ",", 
   RowBox[{"m1", "^", "2"}], ",", 
   RowBox[{"m2", "^", "2"}], ",", 
   RowBox[{"m3", "^", "2"}], ",", 
   RowBox[{"m4", "^", "2"}]}], "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[512]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p12", ",", "p23", ",", "p30", ",", 
   "p20", ",", "p13", ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"D0",
 CellLabel->"Out[512]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "p10", ",", "p12", ",", "p23", ",", "p30", ",", "p20", ",", "p13", ",", 
     RowBox[{"m1", "^", "2"}], ",", 
     RowBox[{"m2", "^", "2"}], ",", 
     RowBox[{"m3", "^", "2"}], ",", 
     RowBox[{"m4", "^", "2"}]}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"p13", ",", "p20"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[513]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p30", ",", "p23", ",", "p12", ",", 
   "p13", ",", "p20", ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"D0",
 CellLabel->"Out[513]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", "%", "]"}]], "Input",
 CellTags->"D0",
 CellLabel->"In[514]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "p10", ",", "p12", ",", "p23", ",", "p30", ",", 
   "p20", ",", "p13", ",", 
   FormBox[
    SuperscriptBox["m1", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m2", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m3", "2"],
    TraditionalForm], ",", 
   FormBox[
    SuperscriptBox["m4", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"D0",
 CellLabel->"Out[514]="]
}, Open  ]]
}, Open  ]],

Cell["D0Convention", "ObjectName",
 CellTags->"D0Convention"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"D0Convention"],

Cell["\<\
D0Convention is an option for Write2. If set to 1, the convention for the \
arguments of D0 is changed when writing a Fortran file with Write2: The fifth \
and sixth argument of D0 are interchanged and the square root is taken of the \
last four arguments.\
\>", "Text",
 CellTags->"D0Convention"],

Cell[TextData[{
 "See also: ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "D0", All, CellTags],
  ButtonData->"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Write2", All, CellTags],
  ButtonData->"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"D0Convention"]
}, Open  ]],

Cell["Eps", "ObjectName",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Eps"],

Cell[TextData[{
 "Eps[a, b, c, d] is the head of the totally antisymmetric ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", "\[Epsilon]"}], TraditionalForm]]],
 " (Levi-Civita) tensor. The a,b, ... may have head LorentzIndex, Momentum or \
Integer. In case of integers the Levi-Civita tensor is evaluated immediately. "
}], "Text",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Eps", "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[515]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[515]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["EpsEvaluate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "EpsEvaluate", All, CellTags],
  ButtonData->"EpsEvaluate",
  ButtonNote->"EpsEvaluate"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LC", All, CellTags],
  ButtonData->"LC",
  ButtonNote->"LC"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LCD", All, CellTags],
  ButtonData->"LCD",
  ButtonNote->"LCD"],
 ", ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeviCivita", All, CellTags],
  ButtonData->"LeviCivita",
  ButtonNote->"LeviCivita"],
 "."
}], "Text",
 CellTags->"Eps"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Eps"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[516]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[516]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[517]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[517]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Eps", "[", 
   RowBox[{"b", ",", "a", ",", "c", ",", "d"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[518]:="],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Eps", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]}]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[518]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{"0", ",", "1", ",", "2", ",", "3"}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[519]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[519]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{"1", ",", "0", ",", "2", ",", "3"}], "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[520]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[520]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[521]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[522]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[522]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a1a1", "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[523]:="],

Cell[BoxData[
 FormBox["a1a1", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[523]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[524]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[525]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[525]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", "a2a2", "]"}], "//", "Factor2"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[526]:="],

Cell[BoxData[
 FormBox["a2a2", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[526]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{
   RowBox[{"LCD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], "]"}], 
   ".", 
   RowBox[{"GAD", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], 
    "]"}]}]}]], "Input",
 CellLabel->"In[527]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Alpha]"}]], ".", 
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Alpha]"]}], TraditionalForm]], "Output",
 CellLabel->"Out[527]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellLabel->"In[528]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]}]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellLabel->"Out[528]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g5", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "24"]}], 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], "]"}], 
     ".", 
     RowBox[{"GAD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Alpha]"}], 
      "]"}]}]}], "//", "FCI"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[529]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm]}]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[529]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g5p", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     FractionBox["\[ImaginaryI]", "24"]}], " ", 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None]}], "]"}], ".", 
     RowBox[{"GAD", "[", 
      RowBox[{
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None], ",", 
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None]}], "]"}]}]}], "//", "FCI"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[530]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox[
        SuperscriptBox["\[Mu]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Nu]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Rho]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox[
        SuperscriptBox["\[Alpha]", "\[Prime]",
         MultilineFunction->None],
        TraditionalForm],
       TraditionalForm]}]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Mu]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Nu]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Rho]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Alpha]", "\[Prime]",
        MultilineFunction->None],
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[530]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g52", "=", 
  RowBox[{"Factor2", "[", 
   RowBox[{"Calc", "[", 
    RowBox[{"g5", ".", "g5p"}], "]"}], "]"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[531]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "24"]}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[531]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g52", "/.", 
  RowBox[{"D", "\[Rule]", "4"}]}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[532]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"Eps",
 CellLabel->"Out[532]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "g5", ",", "g5p", ",", "g52"}], 
  "]"}]], "Input",
 CellTags->"Eps",
 CellLabel->"In[533]:="]
}, Open  ]],

Cell["EpsChisholm", "ObjectName",
 CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsChisholm"],

Cell["\<\
EpsChisholm[expr] substitutes for a gamma matrix contracted with a \
Levi-Civita tensor (Eps) the Chisholm identity.\
\>", "Text",
 CellTags->"EpsChisholm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Chisholm", All, CellTags],
  ButtonData->"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"EpsChisholm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsChisholm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Chisholm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"EpsChisholm",
 CellLabel->"In[534]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"\[ImaginaryI]", " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["$MU$5058",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", "5"]}], " ", 
     SuperscriptBox["\[Epsilon]", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["$MU$5058",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], ".", 
     SuperscriptBox["\[Gamma]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Rho]"}]]}]}], TraditionalForm]], "Output",
 CellTags->"EpsChisholm",
 CellLabel->"Out[534]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EpsChisholm", "[", "%", "]"}]], "Input",
 CellTags->"EpsChisholm",
 CellLabel->"In[535]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"EpsChisholm",
 CellLabel->"Out[535]="]
}, Open  ]]
}, Open  ]],

Cell["EpsContract", "ObjectName",
 CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsContract"],

Cell["\<\
EpsContract is an option of Contract specifying whether Levi-Civita tensors \
Eps[...] will be contracted, i.e., products of two  Eps are replaced via the \
determinant formula.\
\>", "Text",
 CellTags->"EpsContract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Eps", All, CellTags],
  ButtonData->"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"EpsContract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsContract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", 
  RowBox[{"Eps", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[536]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"EpsContract",
 CellLabel->"Out[536]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"a1", " ", "a1"}], ",", 
   RowBox[{"EpsContract", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[537]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]], ")"}], "2"], TraditionalForm]], "Output",
 CellTags->"EpsContract",
 CellLabel->"Out[537]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"a1", " ", "a1"}], ",", 
   RowBox[{"EpsContract", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"EpsContract",
 CellLabel->"In[538]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["D", "4"]}], "+", 
   RowBox[{"6", " ", 
    SuperscriptBox["D", "3"]}], "-", 
   RowBox[{"11", " ", 
    SuperscriptBox["D", "2"]}], "+", 
   RowBox[{"6", " ", "D"}]}], TraditionalForm]], "Output",
 CellTags->"EpsContract",
 CellLabel->"Out[538]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "a1", "]"}]], "Input",
 CellLabel->"In[539]:="]
}, Open  ]],

Cell["EpsDiscard", "ObjectName",
 CellTags->"EpsDiscard"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsDiscard"],

Cell["\<\
EpsDiscard is an option for FeynCalc2FORM. If set to True all Levi-Civita \
tensors are replaced by 0 after contraction.\
\>", "Text",
 CellTags->"EpsDiscard"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc2FORM", All, CellTags],
  ButtonData->"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"EpsDiscard"]
}, Open  ]],

Cell["EpsEvaluate", "ObjectName",
 CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsEvaluate"],

Cell["\<\
EpsEvaluate[expr] applies total antisymmetry and linearity (w.r.t. \
Momentum's) to all Levi-Civita tensors (Eps') in expr.\
\>", "Text",
 CellTags->"EpsEvaluate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Eps", All, CellTags],
  ButtonData->"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeviCivita", All, CellTags],
  ButtonData->"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Trick", All, CellTags],
  ButtonData->"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"EpsEvaluate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"EpsEvaluate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
   " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{
     RowBox[{"p", "+", "q"}], ",", "\[Sigma]"}], "]"}]}], "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[540]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"EpsEvaluate",
 CellLabel->"Out[540]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"EpsEvaluate", "[", "%", "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[541]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"EpsEvaluate",
 CellLabel->"Out[541]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"EpsEvaluate",
 CellLabel->"In[542]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}]}], "]"}]], "Output",
 CellTags->"EpsEvaluate",
 CellLabel->"Out[542]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["EpsUncontract", "ObjectName",
 CellTags->"EpsUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsUncontract"],

Cell["\<\
EpsUncontract does Uncontract on scalar products involving Eps.\
\>", "Text",
 CellTags->"EpsUncontract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Eps", All, CellTags],
  ButtonData->"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Uncontract", All, CellTags],
  ButtonData->"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"EpsUncontract"]
}, Open  ]],

Cell["Epsilon", "ObjectName",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Epsilon"],

Cell[TextData[{
 "Epsilon is (",
 StyleBox["n",
  FontSlant->"Italic"],
 "-4), where ",
 StyleBox["n",
  FontSlant->"Italic"],
 " is the space-time dimension. Epsilon stands for a small positive number."
}], "Text",
 CellTags->"Epsilon"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Series2", All, CellTags],
  ButtonData->"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"Epsilon"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData["Epsilon"], "Input",
 CellTags->"Epsilon",
 CellLabel->"In[543]:="],

Cell[BoxData[
 FormBox[
  TagBox["\[CurlyEpsilon]",
   TraditionalForm], TraditionalForm]], "Output",
 CellTags->"Epsilon",
 CellLabel->"Out[543]="]
}, Open  ]],

Cell["\<\
Epsilon has no functional properties, but some upvalues are changed:\
\>", "Text",
 CellTags->"Epsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "4"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "3"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "2"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", 
    RowBox[{"-", "1"}]}], ",", 
   RowBox[{
    RowBox[{"Re", "[", "Epsilon", "]"}], " ", ">", " ", "0"}]}], 
  "}"}]], "Input",
 CellTags->"Epsilon",
 CellLabel->"In[544]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True", ",", "True", ",", "True", ",", "True"}], 
   "}"}], TraditionalForm]], "Output",
 CellTags->"Epsilon",
 CellLabel->"Out[544]="]
}, Open  ]]
}, Open  ]],

Cell["EpsilonOrder", "ObjectName",
 CellTags->"EpsilonOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"EpsilonOrder"],

Cell[TextData[{
 "EpsilonOrder is an option of OPEIntDelta and RHI. The setting determines \
the order ",
 "n",
 " (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["Epsilon", 
    StyleBox["n",
     FontSlant->"Plain"]], TraditionalForm]]],
 ") which should be kept."
}], "Text",
 CellTags->"EpsilonOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEIntDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPEIntDelta", All, CellTags],
  ButtonData->"OPEIntDelta",
  ButtonNote->"OPEIntDelta"],
 ", ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"EpsilonOrder"]
}, Open  ]],

Cell["Expanding", "ObjectName",
 CellTags->"Expanding"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Expanding"],

Cell["\<\
Expanding is an option for Calc, Contract, DiracSimplify,DotSimplify, \
SUNSimplify, etc. As option for Contract it specifies whether expansion \
w.r.t. LorentzIndex is done BEFORE contraction. If set to False in \
DiracSimplify or SUNSimplify, only a limited set of simplifications \
(multiplicative linearity etc.) is performed.\
\>", "Text",
 CellTags->"Expanding"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"Expanding"]
}, Open  ]],

Cell["ExpandPartialD", "ObjectName",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExpandPartialD"],

Cell["\<\
ExpandPartialD[exp] expands all products of QuantumField's and partial \
differentiation operators in exp and applies the Leibniz rule.\
\>", "Text",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ExpandPartialD", "]"}]], "Input",
 CellLabel->"In[545]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PartialDRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"a$___", ".", 
       RowBox[{"\[PartialD]", "/", "\[PartialD]", 
        SuperscriptBox[
         FormBox["x$_",
          TraditionalForm], 
         FormBox[
          FormBox["mu$_",
           TraditionalForm],
          TraditionalForm]]}], ".", "b$___"}], "\[RuleDelayed]", 
      RowBox[{"a$", ".", 
       RowBox[{
        SubscriptBox[
         TagBox[
          StyleBox["\<\"\[PartialD]\"\>",
           FontSlant->"Italic"],
          StyleForm[#, FontSlant -> "Italic"]& ], 
         TagBox[
          FormBox[
           FormBox["mu$",
            TraditionalForm],
           TraditionalForm],
          TraditionalForm,
          Editable->True]], "(", 
        TagBox[
         FormBox[
          RowBox[{"Dot", "[", "b$", "]"}],
          TraditionalForm],
         TraditionalForm,
         Editable->True], ")"}]}]}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[545]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExplicitPartialD", All, CellTags],
  ButtonData->"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftPartialD", All, CellTags],
  ButtonData->"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD", All, CellTags],
  ButtonData->"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["PartialDRelations",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialDRelations", All, CellTags],
  ButtonData->"PartialDRelations",
  ButtonNote->"PartialDRelations"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RightPartialD", All, CellTags],
  ButtonData->"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"ExpandPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExpandPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[546]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[546]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[547]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[547]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[548]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]}]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[548]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[549]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[549]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[550]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[Null]"]}], "-", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubscriptBox["A", "\[Nu]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[550]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "2"}], ")"}], 
  ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[551]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "2"], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[551]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[552]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", "\[Mu]"], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm]}], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}], ")"}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       RowBox[{
        SubscriptBox["\[PartialD]", 
         FormBox["\[CapitalDelta]",
          TraditionalForm]], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}], ".", 
       RowBox[{
        SubscriptBox["\[PartialD]", 
         FormBox["\[CapitalDelta]",
          TraditionalForm]], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}]}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm], 
         FormBox[
          SubscriptBox["\[PartialD]", 
           FormBox["\[CapitalDelta]",
            TraditionalForm]],
          TraditionalForm]}], 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], "\[Null]"]}], ")"}], ".", 
      SubscriptBox["A", "\[Rho]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[552]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"8", " ", 
  RowBox[{
   RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "3"}]}]], "Input",\

 CellTags->"ExpandPartialD",
 CellLabel->"In[553]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"8", " ", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "3"]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[553]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[554]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}], "3"], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[554]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[555]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "+", 
   RowBox[{"3", " ", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "-", 
   RowBox[{"3", " ", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], ".", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[555]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Tau]"}], "]"}], " ", 
  
  RowBox[{"RightPartialD", "[", 
   RowBox[{"\[Alpha]", ",", "\[Mu]", ",", "\[Beta]", ",", "\[Nu]"}], 
   "]"}]}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[556]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Tau]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[556]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExpandPartialD",
 CellLabel->"In[557]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"ExpandPartialD",
 CellLabel->"Out[557]="]
}, Open  ]]
}, Open  ]],

Cell["ExpandScalarProduct, ScalarProductExpand", "ObjectName",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell["\<\
ExpandScalarProduct[expr] expands scalar products of sums of momenta in expr. \
ExpandScalarProduct does not use Expand on expr.
ScalarProductExpand=ExpandScalarProduct.\
\>", "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "//", "Options"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[558]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"FeynCalcInternal", "\[Rule]", "True"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[558]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumExpand", All, CellTags],
  ButtonData->"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumCombine", All, CellTags],
  ButtonData->"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p1", "+", "p2", "+", "p3"}], ",", 
    RowBox[{"p4", "+", "p5", "+", "p6"}]}], "]"}]}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[559]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p1", "+", "p2", "+", "p3"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p4", "+", "p5", "+", "p6"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[559]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "ScalarProductExpand"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[560]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p1",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p4",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p1",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p5",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p1",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p6",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p2",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p4",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p2",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p5",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p2",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p6",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p3",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p4",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p3",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p5",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["p3",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p6",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[560]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"p", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[561]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[561]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[562]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[562]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[563]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[563]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", "%", "]"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[564]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[564]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "-", "r"}]}], "]"}], "//", "FCI"}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[565]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"q", "-", "r"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[565]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"Pair", "\[Rule]", "ExpandScalarProduct"}]}]], "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"In[566]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"},
 CellLabel->"Out[566]="]
}, Open  ]]
}, Open  ]],

Cell["Expand2", "ObjectName",
 CellTags->"Expand2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Expand2"],

Cell["\<\
Expand2[exp, x] expands all sums containing x. Expand2[exp, {x1, x2, ...}]  \
expands all sums containing x1, x2, ....\
\>", "Text",
 CellTags->"Expand2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Expand2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Expand2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"x1", "+", "x2", "+", "x3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "x1"}], "+", 
       RowBox[{"3", "x2"}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}], 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", "y1"}], "+", 
       RowBox[{"3", "y2"}]}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x1", ",", "x2"}], "}"}]}], "]"}]], "Input",
 CellTags->"Expand2",
 CellLabel->"In[567]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["x1", "2"]}], "+", 
   RowBox[{"5", " ", "x1", " ", "x2"}], "+", 
   RowBox[{"2", " ", "x1", " ", "x3"}], "+", 
   RowBox[{"3", " ", 
    SuperscriptBox["x2", "2"]}], "+", 
   RowBox[{"3", " ", "x2", " ", "x3"}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"2", " ", "y1"}], "+", 
      RowBox[{"3", " ", "y2"}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"y1", "+", "y2", "+", "y3"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Expand2",
 CellLabel->"Out[567]="]
}, Open  ]]
}, Open  ]],

Cell["Explicit", "ObjectName",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Explicit"],

Cell[TextData[{
 "Explicit is an option for FieldStrength, GluonVertex, SUNF, and \
Twist2GluonOperator. If set to True the full form of the operator is \
inserted. ",
 StyleBox["Explicit[exp]",
  FontVariations->{"CompatibilityType"->0}],
 " inserts explicit expressions of GluonVertex, Twist2GluonOperator, etc., in \
exp. SUNF's are replaced by SUNTrace objects."
}], "Text",
 CellTags->"Explicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2GluonOperator", All, CellTags],
  ButtonData->"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "//", "Options"}]], "Input",
 CellLabel->"In[568]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[568]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Explicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellLabel->"In[569]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", "g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellLabel->"Out[569]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[570]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellLabel->"Out[570]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{
   RowBox[{"GluonPropagator", "[", 
    RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellLabel->"In[571]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellLabel->"Out[571]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
  "p", ",", "\[Mu]", ",", "a", ",", " ", "q", ",", "\[Nu]", ",", "b", ",", 
   "r", ",", "\[Rho]", ",", "c"}], "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[572]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[572]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[573]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"Explicit",
 CellLabel->"Out[573]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
  "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[574]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[574]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[575]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[575]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[576]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "a"], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[576]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellTags->"Explicit",
 CellLabel->"In[577]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b27",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c28",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b27",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c28",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Explicit",
 CellLabel->"Out[577]="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitLorentzIndex", "ObjectName",
 CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],

Cell["\<\
ExplicitLorentzIndex[ind] is an explicit Lorentz index, i.e., ind is an \
integer.\
\>", "Text",
 CellTags->"ExplicitLorentzIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LorentzIndex", All, CellTags],
  ButtonData->"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "1", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Input",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"In[578]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{
    FormBox["1",
     TraditionalForm], "\[Mu]"}]], TraditionalForm]], "Output",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"Out[578]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"In[579]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"ExplicitLorentzIndex", "[", "1", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"ExplicitLorentzIndex",
 CellLabel->"Out[579]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitPartialD", "ObjectName",
 CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitPartialD"],

Cell["\<\
ExplicitPartialD[exp] inserts in exp the definition for LeftRightPartialD[z] \
(and LeftRightPartialD2[z]).\
\>", "Text",
 CellTags->"ExplicitPartialD"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD", All, CellTags],
  ButtonData->"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD2", All, CellTags],
  ButtonData->"LeftRightPartialD2",
  ButtonNote->"LeftRightPartialD2"],
 "."
}], "Text",
 CellTags->"ExplicitPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[580]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[580]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[581]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "+", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[581]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", " ", 
  RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], " ", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[582]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[582]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"16", " ", 
  RowBox[{
   RowBox[{"LeftRightPartialD", "[", "OPEDelta", "]"}], "^", "4"}]}]], "Input",\

 CellTags->"ExplicitPartialD",
 CellLabel->"In[583]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"16", " ", 
   SubsuperscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], "4"]}], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[583]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"ExplicitPartialD",
 CellLabel->"In[584]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]]}], ")"}], "4"], TraditionalForm]], "Output",
 CellTags->"ExplicitPartialD",
 CellLabel->"Out[584]="]
}, Open  ]]
}, Open  ]],

Cell["ExplicitSUNIndex", "ObjectName",
 CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
 "ExplicitSUNIndex[ind] is a specific SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") index, i.e., ind is an integer."
}], "Text",
 CellTags->"ExplicitSUNIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNDelta", All, CellTags],
  ButtonData->"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNF", All, CellTags],
  ButtonData->"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNIndex", All, CellTags],
  ButtonData->"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ExplicitSUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitSUNIndex", "[", "1", "]"}]], "Input",
 CellTags->"ExplicitSUNIndex",
 CellLabel->"In[585]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"ExplicitSUNIndex",
 CellLabel->"Out[585]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"1", ",", "a"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellLabel->"In[586]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellLabel->"Out[586]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["ExtraFactor", "ObjectName",
 CellTags->"ExtraFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExtraFactor"],

Cell["\<\
ExtraFactor is an option for FermionSpinSum. The setting ExtraFactor \[Rule] \
fa  multiplies the whole amplitude with the factor fa before squaring.\
\>", "Text",
 CellTags->"ExtraFactor"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FermionSpinSum", All, CellTags],
  ButtonData->"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 "."
}], "Text",
 CellTags->"ExtraFactor"]
}, Open  ]],

Cell["ExtraVariables", "ObjectName",
 CellTags->"ExtraVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ExtraVariables"],

Cell["\<\
ExtraVariables is an option for OneLoopSum; it may be set to a list of \
variables which are also bracketed out in the result, just like B0, C0, D0 \
and  PaVe.\
\>", "Text",
 CellTags->"ExtraVariables"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"ExtraVariables"]
}, Open  ]],

Cell["FactorFull", "ObjectName",
 CellTags->"FactorFull"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FactorFull"],

Cell[TextData[{
 "FactorFull is an option of Factor2 (default False). If set to False, \
products like (",
 StyleBox["a",
  FontSlant->"Italic"],
 "-",
 StyleBox["b",
  FontSlant->"Italic"],
 ") (",
 StyleBox["a",
  FontSlant->"Italic"],
 "+",
 StyleBox["b",
  FontSlant->"Italic"],
 ") will be replaced by (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["a", "2"], TraditionalForm]]],
 "-",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["b", "2"], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"FactorFull"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Factor2", All, CellTags],
  ButtonData->"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"FactorFull"]
}, Open  ]],

Cell["Factoring", "ObjectName",
 CellTags->"Factoring"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factoring"],

Cell["\<\
Factoring is an option for Collect2, Contract, Tr and more functions. If set \
to True, the result will be factored, using Factor2. If set to any function \
f, this function will be used.\
\>", "Text",
 CellTags->"Factoring"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Collect2", All, CellTags],
  ButtonData->"Collect2",
  ButtonNote->"Collect2"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"Factoring"]
}, Open  ]],

Cell["Factorout", "ObjectName",
 CellTags->"Factorout"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factorout"],

Cell["Factorout is an option for OPEInt.", "Text",
 CellTags->"Factorout"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEInt",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPEInt", All, CellTags],
  ButtonData->"OPEInt",
  ButtonNote->"OPEInt"],
 "."
}], "Text",
 CellTags->"Factorout"]
}, Open  ]],

Cell["FactorTime", "ObjectName",
 CellTags->"FactorTime"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FactorTime"],

Cell["\<\
FactorTime is an option for Factor2. It denotes the maximum time (in seconds) \
during which Factor2 tries to factor.\
\>", "Text",
 CellTags->"FactorTime"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Factor2", All, CellTags],
  ButtonData->"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"FactorTime"]
}, Open  ]],

Cell["Factor1", "ObjectName",
 CellTags->"Factor1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factor1"],

Cell["\<\
Factor1[poly] factorizes common terms  in the summands of poly. It uses \
basically PolynomialGCD.\
\>", "Text",
 CellTags->"Factor1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Factor2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Factor2", All, CellTags],
  ButtonData->"Factor2",
  ButtonNote->"Factor2"],
 "."
}], "Text",
 CellTags->"Factor1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Factor1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[587]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[587]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factor1", "[", "t1", "]"}], ",", " ", 
    RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[588]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "x"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "x"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"a", "-", "x"}], ")"}]}], " ", 
     RowBox[{"(", 
      RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[588]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "b"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[589]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[589]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor", "[", "t3", "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[590]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[590]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor1", "[", "t3", "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[591]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor1",
 CellLabel->"Out[591]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"Factor1",
 CellLabel->"In[592]:="]
}, Open  ]],

Cell["Factor2", "ObjectName",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Factor2"],

Cell["\<\
Factor2[poly] factors a polynomial in a standard way. Factor2 works sometimes \
better than Factor on polynomials involving rationals with sums in the \
denominator. Factor2 uses Factor internally and is in general slower than \
Factor. There are four possible settings of the option Method (0,1,2,3). In \
general Factor will work faster than Factor2.\
\>", "Text",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Factor2", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[593]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FactorFull", "\[Rule]", "False"}], ",", 
    RowBox[{"Method", "\[Rule]", "3"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[593]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Collect2", All, CellTags],
  ButtonData->"Collect2",
  ButtonNote->"Collect2"],
 "."
}], "Text",
 CellTags->"Factor2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Factor2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[594]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "x"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"b", "-", "x"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[594]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factor2", "[", "t1", "]"}], ",", " ", 
    RowBox[{"Factor", "[", "t1", "]"}]}], "}"}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[595]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "x"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "x"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{"a", "-", "x"}], ")"}]}], " ", 
     RowBox[{"(", 
      RowBox[{"x", "-", "b"}], ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[595]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Expand", "[", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"a", "-", "b"}], ")"}], 
    RowBox[{"(", 
     RowBox[{"a", "+", "b"}], ")"}]}], "]"}]}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[596]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[596]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor", "[", "t3", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[597]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[597]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", "t3", "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[598]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["a", "2"], "-", 
   SuperscriptBox["b", "2"]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[598]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"t3", ",", 
   RowBox[{"FactorFull", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[599]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"a", "-", "b"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"a", "+", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Factor2",
 CellLabel->"Out[599]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2", ",", "t3"}], "]"}]], "Input",
 CellTags->"Factor2",
 CellLabel->"In[600]:="]
}, Open  ]],

Cell["FAD", "ObjectName",
 CellTags->"FAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FAD"],

Cell["\<\
FAD is the FeynCalc external form of FeynAmpDenominator and denotes an \
inverse propagator. FAD[q, q-p, ...] is 1/(q^2 (q-p)^2 ...). FAD[{q1,m}, \
{q1-p,m}, q2, ...] is 1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... ). \
Translation into FeynCalc internal form is performed by FeynCalcInternal.\
\>", "Text",
 CellTags->"FAD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FAD", All, CellTags],
  ButtonData->"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominator", All, CellTags],
  ButtonData->"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorSimplify", All, CellTags],
  ButtonData->"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PropagatorDenominator", All, CellTags],
  ButtonData->"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"FAD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FAD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[601]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"p", "-", "q"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", ($CellContext`p - $CellContext`q)^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"FAD",
 CellLabel->"Out[601]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"p", "-", "q"}], ",", "m"}], "}"}]}], "]"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[602]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["p", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`p^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"p", "-", "q"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm[
      "[", ($CellContext`p - $CellContext`q)^2, "-", $CellContext`m^2, "]"],
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"FAD",
 CellLabel->"Out[602]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{"q", ",", 
      RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[603]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{"p", "-", "q"}]}], "]"}]], "Output",
 CellTags->"FAD",
 CellLabel->"Out[603]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q", ",", 
     RowBox[{"p", "-", "q"}]}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[604]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", 
      RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"p", ",", "D"}], "]"}], "-", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], "]"}]], "Output",\

 CellTags->"FAD",
 CellLabel->"Out[604]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", "p", "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{"p", "-", "q"}], "]"}]}], " ", "//", " ", 
   "FeynAmpDenominatorCombine"}], "//", "StandardForm"}]], "Input",
 CellTags->"FAD",
 CellLabel->"In[605]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", 
      RowBox[{"p", ",", "D"}], "]"}], ",", "0"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"p", ",", "D"}], "]"}], "-", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}]}], ",", "0"}], "]"}]}], "]"}]], "Output",\

 CellTags->"FAD",
 CellLabel->"Out[605]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FC", "ObjectName",
 CellTags->"FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC"],

Cell["\<\
FC changes the output format to FeynCalcForm. To change to InputForm use FI.\
\>", "Text",
 CellTags->"FC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcForm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcForm", All, CellTags],
  ButtonData->"FeynCalcForm",
  ButtonNote->"FeynCalcForm"],
 ", ",
 ButtonBox["FI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FI", All, CellTags],
  ButtonData->"FI",
  ButtonNote->"FI"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcInternal", All, CellTags],
  ButtonData->"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC"],

Cell[BoxData["FI"], "Input",
 CellTags->"FC",
 CellLabel->"In[606]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FC",
 CellLabel->"In[607]:="],

Cell["{DiracGamma[5], DiracGamma[Momentum[p]]}", "Output",
 CellTags->"FC",
 CellLabel->"Out[607]="]
}, Open  ]],

Cell[BoxData["FC"], "Input",
 CellTags->"FC",
 CellLabel->"In[608]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FC",
 CellLabel->"In[609]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FC",
 CellLabel->"Out[609]="]
}, Open  ]]
}, Open  ]],

Cell["FCE", "ObjectName",
 CellTags->"FCE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCE"],

Cell["\<\
FCE[exp] translates exp from the internal FeynCalc representation to a short \
form.\
\>", "Text",
 CellTags->"FCE"],

Cell["FCE is equivalent to FeynCalcExternal.", "Text",
 CellTags->"FCE"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcInternal", All, CellTags],
  ButtonData->"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FCE"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FCE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCE", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracGamma", "[", "5", "]"}], ",", 
    RowBox[{"DiracGamma", "[", 
     RowBox[{"Momentum", "[", "p", "]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[610]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[610]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[611]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "5", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[611]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[612]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[612]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[613]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[613]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[614]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[614]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[615]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[615]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FCE",
 CellLabel->"In[616]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCE",
 CellLabel->"Out[616]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FCI", "ObjectName",
 CellTags->"FCI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCI"],

Cell["\<\
FCI[exp] translates exp into the internal FeynCalc \
(datatype-)representation.FCI is equivalent to FeynCalcInternal.\
\>", "Text",
 CellTags->"FCI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcInternal", All, CellTags],
  ButtonData->"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 "."
}], "Text",
 CellTags->"FCI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FCI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[617]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[617]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[618]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[618]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[619]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[619]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[620]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[620]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", "%", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FCI",
 CellLabel->"In[621]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FCI",
 CellLabel->"Out[621]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FCIntegral", "ObjectName",
 CellTags->"FCIntegral"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCIntegral"],

Cell["\<\
FCIntegral is the head of integrals in a setting of the option IntegralTable \
of FeynAmpDenominatorSimplify. Currently implemented only for 2-loop \
integrals.\
\>", "Text",
 CellTags->"FCIntegral"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IntegralTable",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IntegralTable", All, CellTags],
  ButtonData->"IntegralTable",
  ButtonNote->"IntegralTable"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorSimplify", All, CellTags],
  ButtonData->"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FCIntegral"]
}, Open  ]],

Cell["FCIntegrate", "ObjectName",
 CellTags->"FCIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCIntegrate"],

Cell["\<\
FCIntegrate is an option of certain Feynman integral related functions. It \
determines which integration function is used to evaluate analytic integrals. \
Possible settings include Integrate, NIntegrate, (Dot[Integratedx@@#2, #1] &).\
\>", "Text",
 CellTags->"FCIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCNIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCNIntegrate", All, CellTags],
  ButtonData->"FCNIntegrate",
  ButtonNote->"FCNIntegrate"],
 "."
}], "Text",
 CellTags->"FCIntegrate"]
}, Open  ]],

Cell["FCNIntegrate", "ObjectName",
 CellTags->"FCNIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FCNIntegrate"],

Cell["\<\
FCNIntegrate is an option of certain Feynman integral related functions which \
may return output containing both integrals that can be evaluated and \
integrals that can only be evaluated numerically. It then determines which \
integration function is used to evaluate numeric integrals. Possible settings \
include NIntegrate, (0*#1)&, (Dot[Integratedx@@#2, #1] &).\
\>", "Text",
 CellTags->"FCNIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCIntegrate", All, CellTags],
  ButtonData->"FCIntegrate",
  ButtonNote->"FCIntegrate"],
 "."
}], "Text",
 CellTags->"FCNIntegrate"]
}, Open  ]],

Cell["FC2RHI ", "ObjectName",
 CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC2RHI"],

Cell["\<\
FC2RHI[exp, k1, k2] transforms all 2-loop OPE-integrals in FeynAmpDenominator \
form to the RHI-integrals.  FC2RHI[exp] is equivalent to FC2RHI[exp,q1,q2]. \
The option IncludePair governs the inclusion  of scalar products p.k1, p.k2 \
and k1.k2 (setting True).\
\>", "Text",
 CellTags->"FC2RHI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FC2RHI", "]"}]], "Input",
 CellTags->"FC2RHI",
 CellLabel->"In[622]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
    RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FC2RHI",
 CellLabel->"Out[622]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{"q1", ",", 
      RowBox[{"q1", "-", "q2"}], ",", 
      RowBox[{"q2", "-", "p"}]}], "]"}], " ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"q1", ",", "OPEDelta"}], "]"}], "^", "OPEm"}]}], "//", 
   "FCI"}]}]], "Input",
 CellLabel->"In[623]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[623]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2RHI", "[", 
  RowBox[{"t", ",", "q1", ",", "q2"}], "]"}]], "Input",
 CellLabel->"In[624]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"1", "0", "0", "1", "1"}],
    FontColor->RGBColor[0, 0, 1]], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellLabel->"Out[624]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[625]:="],

Cell["RHI[{0, 0, OPEm, 0, 0}, {0, 1, 1, 0, 1}]", "Output",
 CellLabel->"Out[625]//InputForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"FC2RHI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC2RHI"],

Cell[BoxData[""], "Input",
 CellTags->"FC2RHI",
 CellLabel->"In[626]:="]
}, Open  ]],

Cell["FC2TLI ", "ObjectName",
 CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FC2TLI"],

Cell["\<\
FC2TLI[exp, k1, k2] transforms all 2-loop OPE-integrals in FeynAmpDenominator \
form to the TLI-integrals.  The option IncludePair governs the inclusion  of \
scalar products p.k1, p.k2 and k1.k2 (setting True).\
\>", "Text",
 CellTags->"FC2TLI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FC2TLI", "]"}]], "Input",
 CellTags->"FC2TLI",
 CellLabel->"In[627]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"IncludePair", "\[Rule]", "True"}], ",", 
    RowBox[{"Do", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FC2TLI",
 CellLabel->"Out[627]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "FAD"}]], "Input",
 CellLabel->"In[628]:="],

Cell[BoxData[
 StyleBox["\<\"FAD[q, q-p, ...] denotes 1/(q^2 (q-p)^2 ...).\\nFAD[{q1,m}, \
{q1-p,m}, q2, ...] is\\n1/( (q1^2 - m^2) ( (q1-p)^2 - m^2 ) q2^2 ... \
).\\n(Translation into FeynCalc internal form is performed \
by\\nFeynCalcInternal.)\"\>", "MSG"]], "Print", "PrintUsage",
 CellTags->"Info3535453193-5096475"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q1", ",", "m1"}], "}"}], ",", 
     RowBox[{"q2", "-", "q1"}], ",", 
     RowBox[{"q2", "-", "p"}]}], "]"}], " ", 
   RowBox[{
    RowBox[{"SP", "[", 
     RowBox[{"q1", ",", "OPEDelta"}], "]"}], "^", "OPEm"}]}]}]], "Input",
 CellLabel->"In[629]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}],
      TraditionalForm], ")"}], "m"], " ", 
   FormBox[
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q2", "-", "p"}], ")"}], "2"], 
         "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", (-$CellContext`p + $CellContext`q2)^2, "]"],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q2", "-", "q1"}], ")"}], "2"], 
         "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", (-$CellContext`q1 + $CellContext`q2)^2, "]"],
        Editable->False], ")"}], " ", 
      RowBox[{"(", 
       InterpretationBox[
        RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
         SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
         "\[InvisibleSpace]", 
         SuperscriptBox["m1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
        SequenceForm["[", $CellContext`q1^2, "-", $CellContext`m1^2, "]"],
        Editable->False], ")"}]}]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellLabel->"Out[629]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2TLI", "[", 
  RowBox[{"t", ",", "q1", ",", "q2"}], "]"}]], "Input",
 CellLabel->"In[630]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   RowBox[{
    StyleBox["1",
     FontWeight->"Bold",
     FontColor->RGBColor[1, 0, 0]], 
    StyleBox["0",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["0",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["1",
     FontColor->RGBColor[0, 0, 1]], 
    StyleBox["1",
     FontColor->RGBColor[0, 0, 1]]}], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellLabel->"Out[630]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[631]:="],

Cell["TLI[{OPEm, 0, 0, 0, 0}, {{1, m1}, 0, 0, 1, 1}]", "Output",
 CellLabel->"Out[631]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TLI2FC", "[", "%", "]"}]], "Input",
 CellLabel->"In[632]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m1", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[632]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TLI", All, CellTags],
  ButtonData->"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"FC2TLI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FC2TLI"],

Cell[BoxData[""], "Input",
 CellTags->"FC2TLI",
 CellLabel->"In[633]:="]
}, Open  ]],

Cell["FDS", "ObjectName",
 CellTags->"FDS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FDS"],

Cell["FDS is shorthand for FeynAmpDenominatorSimplify.", "Text",
 CellTags->"FDS"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorSimplify", All, CellTags],
  ButtonData->"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FDS"]
}, Open  ]],

Cell["FermionSpinSum", "ObjectName",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FermionSpinSum"],

Cell["\<\
FermionSpinSum[x] constructs the Traces out of squared ampliudes.\
\>", "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FermionSpinSum", "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[634]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"SpinPolarizationSum", "\[Rule]", "Identity"}], ",", 
    RowBox[{"SpinorCollect", "\[Rule]", "False"}], ",", 
    RowBox[{"ExtraFactor", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[634]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Spinor", All, CellTags],
  ButtonData->"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["ComplexConjugate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ComplexConjugate", All, CellTags],
  ButtonData->"ComplexConjugate",
  ButtonNote->"ComplexConjugate"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"FermionSpinSum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FermionSpinSum"],

Cell[TextData[{
 "Spinors of fermions of mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " are normalized to have square 2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 " or -2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
  RowBox[{"SpinorU", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[635]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"u\"\>", "_"],
     TraditionalForm], "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"u", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[635]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[636]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{"m", "+", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[636]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[637]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "m"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[637]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorVBar", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}], ".", 
  RowBox[{"SpinorV", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[638]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     OverscriptBox["\<\"v\"\>", "_"],
     TraditionalForm], "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"v", "(", 
    FormBox["p",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[638]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "FermionSpinSum"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[639]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], "-", "m"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[639]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[640]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "m"}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[640]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{"k1", ",", "m"}], "]"}], ".", 
   RowBox[{"DiracSlash", "[", "p", "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}], ".", 
   RowBox[{"Spinor", "[", 
    RowBox[{"p1", ",", "m"}], "]"}]}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[641]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["k1",
     TraditionalForm], ",", "m", ")"}], ".", 
   RowBox[{"(", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "5"], ".", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox["p1",
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[641]="]
}, Open  ]],

Cell["\<\
Notice that SpinorUBar and SpinorU are only input functions. Internally they \
are converted to Spinor objects.\
\>", "Text",
 CellTags->"FermionSpinSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ct", "=", 
  RowBox[{"ComplexConjugate", "[", "t", "]"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[642]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    RowBox[{"\[CurlyPhi]", "(", 
     FormBox["p1",
      TraditionalForm], ",", "m", ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"\[CurlyPhi]", "(", 
     FormBox["k1",
      TraditionalForm], ",", "m", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[642]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FermionSpinSum", "[", 
  RowBox[{"t", " ", "ct"}], "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[643]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox["\<\"\[Gamma]\"\>",
          TraditionalForm], "\[CenterDot]", 
         FormBox["k1",
          TraditionalForm]}], "+", "m"}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"(", 
       RowBox[{"m", "+", 
        RowBox[{
         FormBox["\<\"\[Gamma]\"\>",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p1",
          TraditionalForm]}]}], ")"}], ".", 
      SuperscriptBox["\[Gamma]", "5"], ".", 
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[Gamma]\"\>",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[643]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", 
  RowBox[{"DiracTrace", "\[Rule]", "TR"}]}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[644]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p1",
        TraditionalForm]}]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox["k1",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p1",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FermionSpinSum",
 CellLabel->"Out[644]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t", ",", " ", "ct"}], "]"}]], "Input",
 CellTags->"FermionSpinSum",
 CellLabel->"In[645]:="]
}, Open  ]],

Cell["FeynAmp", "ObjectName",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmp"],

Cell["\<\
FeynAmp[q, amp] is the head of a Feynman amplitude. amp denotes the \
analytical expression for the amplitude and q is the integration variable. \
FeynAmp[q1, q2, amp] denotes a two-loop amplitude.\
\>", "Text",
 CellTags->"FeynAmp"],

Cell["\<\
FeynAmp has no functional properties and serves just as a head. There are \
however special typesetting rules attached.\
\>", "Text",
 CellTags->"FeynAmp"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Amplitude", All, CellTags],
  ButtonData->"Amplitude",
  ButtonNote->"Amplitude"],
 "."
}], "Text",
 CellTags->"FeynAmp"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmp"],

Cell["\<\
This is a 1-loop gluon self-energy amplitude (ignoring factors of (2 \[Pi])).\
\
\>", "Text",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{"q", ",", 
   RowBox[{
    RowBox[{"GV", "[", 
     RowBox[{"p", ",", "\[Mu]", ",", "a", ",", " ", 
      RowBox[{"q", "-", "p"}], ",", "\[Alpha]", ",", "c", ",", " ", 
      RowBox[{"-", "q"}], ",", "\[Beta]", ",", "e"}], "]"}], " ", 
    RowBox[{"GP", "[", 
     RowBox[{
      RowBox[{"p", "-", "q"}], ",", " ", "\[Alpha]", ",", "c", ",", " ", 
      "\[Rho]", ",", "d"}], "]"}], 
    RowBox[{"GV", "[", 
     RowBox[{
      RowBox[{"-", "p"}], ",", "\[Nu]", ",", "b", ",", " ", 
      RowBox[{"p", "-", "q"}], ",", "\[Rho]", ",", "d", ",", " ", "q", ",", 
      "\[Sigma]", ",", "f"}], "]"}], " ", 
    RowBox[{"GP", "[", 
     RowBox[{
     "q", ",", " ", "\[Beta]", ",", "e", ",", " ", "\[Sigma]", ",", "f"}], 
     "]"}]}]}], "]"}]], "Input",
 CellTags->"FeynAmp",
 CellLabel->"In[646]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   StyleBox[
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], "Global`q"}],
    ZeroWidthTimes->True], "(", 
   FormBox[
    RowBox[{
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["f",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{
      SubsuperscriptBox["\[CapitalPi]", 
       RowBox[{"e", "f"}], 
       RowBox[{"\[Beta]", "\[Sigma]"}]], "(", "q", ")"}], " ", 
     RowBox[{
      SuperscriptBox["V", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Alpha]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Beta]",
          TraditionalForm],
         TraditionalForm]}]], "(", 
      RowBox[{
       FormBox["p",
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"q", "-", "p"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"-", "q"}],
        TraditionalForm]}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["V", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]], "(", 
      RowBox[{
       FormBox[
        RowBox[{"-", "p"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox[
        RowBox[{"p", "-", "q"}],
        TraditionalForm], 
       RowBox[{",", " "}], 
       FormBox["q",
        TraditionalForm]}], ")"}], " ", 
     RowBox[{
      SubsuperscriptBox["\[CapitalPi]", 
       RowBox[{"c", "d"}], 
       RowBox[{"\[Alpha]", "\[Rho]"}]], "(", 
      FormBox[
       RowBox[{"p", "-", "q"}],
       TraditionalForm], ")"}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FeynAmp",
 CellLabel->"Out[646]="]
}, Open  ]],

Cell["This is a generic 2-loop amplitude.", "Text",
 CellTags->"FeynAmp"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmp", "[", 
  RowBox[{
   SubscriptBox["q", "1"], ",", 
   SubscriptBox["q", "2"], ",", "anyexpression"}], "]"}]], "Input",
 CellTags->"FeynAmp",
 CellLabel->"In[647]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Integral]", 
   RowBox[{
    StyleBox[
     RowBox[{
      SuperscriptBox["\[DifferentialD]", "D"], 
      SubscriptBox["q", "1"]}],
     ZeroWidthTimes->True], "\[Integral]", 
    RowBox[{
     SuperscriptBox["\[DifferentialD]", "D"], 
     SubscriptBox["q", "2"]}]}], "(", "anyexpression", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmp",
 CellLabel->"Out[647]="]
}, Open  ]]
}, Open  ]],

Cell["FeynAmpDenominator", "ObjectName",
 CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominator"],

Cell["\<\
FeynAmpDenominator[ PropagatorDenominator[ ... ], PropagatorDenominator[ ... \
], ...] is the head of the denominators of the propagators, i.e., \
FeynAmpDenominator[x] is the representation of 1/x .\
\>", "Text",
 CellTags->"FeynAmpDenominator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FAD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FAD", All, CellTags],
  ButtonData->"FAD",
  ButtonNote->"FAD"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorSimplify", All, CellTags],
  ButtonData->"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[648]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["p", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[648]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{"p", ",", "m"}], "]"}], ",", 
   RowBox[{"PropagatorDenominator", "[", 
    RowBox[{
     RowBox[{"p", "-", "q"}], ",", "m"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[649]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["p", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"p", "-", "q"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[649]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"FeynAmpDenominator", "[", 
    RowBox[{"PropagatorDenominator", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "]"}]}], ";"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[650]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"t", "//", "FCI"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[651]:="],

Cell[BoxData[
 RowBox[{"FeynAmpDenominator", "[", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"p", ",", "D"}], "]"}], ",", "m"}], "]"}], "]"}]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[651]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"t", "//", "FCE"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[652]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"{", 
   RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Output",
 CellTags->"FeynAmpDenominator",
 CellLabel->"Out[652]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"FeynAmpDenominator",
 CellLabel->"In[653]:="]
}, Open  ]],

Cell["FeynAmpDenominatorCombine", "ObjectName",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],

Cell["\<\
FeynAmpDenominatorCombine[expr] expands expr with respect to \
FeynAmpDenominator and combines products of FeynAmpDenominator in expr into \
one FeynAmpDenominator.\
\>", "Text",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSplit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorSplit", All, CellTags],
  ButtonData->"FeynAmpDenominatorSplit",
  ButtonNote->"FeynAmpDenominatorSplit"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", " ", "=", " ", 
  RowBox[{
   RowBox[{"FAD", "[", "q", "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{"q", "-", "p"}], "]"}]}]}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[654]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    FractionBox["1", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q^2, "]"],
      Editable->False]],
    TraditionalForm], " ", 
   FormBox[
    FractionBox["1", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q)^2, "]"],
      Editable->False]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[654]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[655]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q", ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q"}]}], "]"}]], "Output",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[655]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorSplit", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"In[656]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", "q", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q"}], "]"}]}]], "Output",
 CellTags->"FeynAmpDenominatorCombine",
 CellLabel->"Out[656]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynAmpDenominatorSimplify", "ObjectName",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell["\<\
FeynAmpDenominatorSimplify[exp] tries to simplify each PropagatorDenominator \
in a canonical way. FeynAmpDenominatorSimplify[exp, q1] simplifies all \
FeynAmpDenominator's in exp in a canonical way, including some translation of \
momenta. FeynAmpDenominatorSimplify[exp, q1, q2] additionally removes \
integrals with no mass scale.\
\>", "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell["FDS can be used as an alias.", "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData["FDS"], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[657]:="],

Cell[BoxData[
 FormBox["FeynAmpDenominatorSimplify", TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[657]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[TextData[{
 "The cornerstone of dimensional regularization is that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Integral]", 
     RowBox[{
      SuperscriptBox["d", "n"], "k", " ", 
      RowBox[{
       RowBox[{"f", "(", "k", ")"}], "/", 
       SuperscriptBox["k", 
        RowBox[{"2", "m"}]]}]}]}], "=", " ", 
    RowBox[{"0", " ", "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "k", "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[658]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[658]="]
}, Open  ]],

Cell[TextData[{
 "This brings ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], ")"}]}], 
   TraditionalForm]]],
 " into a standard form."
}], "Text",
 CellTags->"FeynAmpDenominatorSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"k", "-", 
      SubscriptBox["p", "1"]}], ",", 
     RowBox[{"k", "-", 
      SubscriptBox["p", "2"]}]}], "]"}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[659]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"], "+", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[659]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", "k", ",", 
     RowBox[{"k", "-", "q"}]}], "]"}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[660]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "q"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[660]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "k", "]"}], 
    RowBox[{"FAD", "[", 
     RowBox[{"k", ",", 
      RowBox[{"k", "-", "q"}], ",", 
      RowBox[{"k", "-", "q"}]}], "]"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[661]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"f", "(", 
    RowBox[{"q", "-", "k"}], ")"}], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "q"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[661]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"FeynAmpDenominatorSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FAD", "[", 
      RowBox[{
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"]}], ",", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "2"]}]}], "]"}], " ", 
     RowBox[{"SPD", "[", 
      RowBox[{"k", ",", "k"}], "]"}]}], ",", "k"}], "]"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[662]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    SubsuperscriptBox["p", "2", "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[662]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r", "=", 
  RowBox[{"SPC", "[", 
   RowBox[{"t", ",", "k", ",", 
    RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[663]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SubsuperscriptBox["p", "2", "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "+", 
        SubscriptBox["p", "1"], "-", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[663]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{"r", ",", "k"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[664]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox[
     SubscriptBox["p", "1"],
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "2"],
     TraditionalForm]}], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"], "+", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[664]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FDS", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"k", "-", "p1"}], ",", 
      RowBox[{"k", "-", "p2"}]}], "]"}], 
    RowBox[{
     RowBox[{"SPD", "[", 
      RowBox[{"k", ",", "OPEDelta"}], "]"}], "^", "2"}]}], ",", "k"}], 
  "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[665]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p2",
       TraditionalForm]}]}], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p2",
        TraditionalForm]}],
      TraditionalForm], "2"], 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p1", "+", "p2"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"Out[665]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t", ",", "r"}], "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSimplify",
 CellLabel->"In[666]:="]
}, Open  ]],

Cell["FeynAmpDenominatorSplit", "ObjectName",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],

Cell["\<\
FeynAmpDenominatorSplit[expr] splits all FeynAmpDenominator[a,b, ...] in expr \
into FeynAmpDenominator[a]*FeynAmpDenominator[b] ... . \
FeynAmpDenominatorSplit[expr, q1] splits all FeynAmpDenominator in expr into \
a product of two, one containing q1 and other momenta, the second without q1.\
\
\>", "Text",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorCombine",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorCombine", All, CellTags],
  ButtonData->"FeynAmpDenominatorCombine",
  ButtonNote->"FeynAmpDenominatorCombine"],
 "."
}], "Text",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"q1", "-", "q2"}], ",", "q2", ",", 
     RowBox[{"q2", "-", "p"}]}], "]"}], "//", "FCI"}]}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[667]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"], ".", 
    SuperscriptBox["q2", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[667]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "Head"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[668]:="],

Cell[BoxData[
 FormBox["FeynAmpDenominator", TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[668]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSplit", "[", "t", "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[669]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q1", "2"], " ", 
    SuperscriptBox["q2", "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[669]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[670]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", "q1", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q1"}], "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{"q1", "-", "q2"}], "]"}], " ", 
  RowBox[{"FAD", "[", "q2", "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q2"}], "]"}]}]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[670]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorSplit", "[", 
    RowBox[{"t", ",", "q1"}], "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[671]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", 
   RowBox[{"q2", ",", 
    RowBox[{
     RowBox[{"-", "p"}], "+", "q2"}]}], "]"}], " ", 
  RowBox[{"FAD", "[", 
   RowBox[{"q1", ",", 
    RowBox[{
     RowBox[{"-", "p"}], "+", "q1"}], ",", 
    RowBox[{"q1", "-", "q2"}]}], "]"}]}]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[671]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FeynAmpDenominatorCombine", "[", "%", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[672]:="],

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q1", ",", "q2", ",", 
   RowBox[{"q1", "-", "q2"}], ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q1"}], ",", 
   RowBox[{
    RowBox[{"-", "p"}], "+", "q2"}]}], "]"}]], "Output",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"Out[672]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"FeynAmpDenominatorSplit",
 CellLabel->"In[673]:="]
}, Open  ]],

Cell["FeynAmpList", "ObjectName",
 CellTags->"FeynAmpList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList[info][FeynAmp[...], FeynAmp[...], ...] is a head of a list of \
Feynman amplitudes.\
\>", "Text",
 CellTags->"FeynAmpList"],

Cell["\<\
FeynAmpList has no functional properties and serves just as a head.\
\>", "Text",
 CellTags->"FeynAmpList"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmp",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmp", All, CellTags],
  ButtonData->"FeynAmp",
  ButtonNote->"FeynAmp"],
 "."
}], "Text",
 CellTags->"FeynAmpList"]
}, Open  ]],

Cell["FeynCalc", "ObjectName",
 CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalc"],

Cell["FeynCalc is simply a symbol with a usage definition.", "Text",
 CellTags->"FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "FeynCalc"}]], "Input",
 CellTags->"FeynCalc",
 CellLabel->"In[674]:="],

Cell[BoxData[
 StyleBox["\<\"For installation notes visit www.feyncalc.org\\n\\nFor a list \
of availabe objects type $FeynCalcStuff, which contains a list of all \
functions and options in StringForm. You can get on-line information by \
?function, e.g. ?Contract.\\n\\nThere are several useful functions for short \
input, type $FCS for a list of short commands. Then type, e.g., \
?GA.\\n\\n\\nTo enable/disable start-up messages, put the \
line\\n\\n$FeynCalcStartupMessages = \
True;\\n\\nor\\n\\n$FeynCalcStartupMessages = False;\\n\\n into your \
\\\"init.m\\\" file or into your \\\"FCConfig.m\\\" file.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellTags->{"FeynCalc", "Info3535453199-5096475"}]
}, Open  ]]
}, Open  ]],

Cell["FeynCalcExternal", "ObjectName",
 CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcExternal"],

Cell["\<\
FeynCalcExternal[exp] translates exp from the internal FeynCalc \
representation to a shorthand form.\
\>", "Text",
 CellTags->"FeynCalcExternal"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcInternal", All, CellTags],
  ButtonData->"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FeynCalcExternal"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcExternal"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynCalcExternal", "[", 
  RowBox[{"DiracGamma", "[", "5", "]"}], "]"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[675]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[675]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[676]:="],

Cell[BoxData[
 RowBox[{"GA", "[", "5", "]"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[676]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[677]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[677]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[678]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[678]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[679]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[679]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[680]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[680]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynCalcExternal",
 CellLabel->"In[681]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcExternal",
 CellLabel->"Out[681]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynCalcForm", "ObjectName",
 CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcForm"],

Cell[TextData[{
 "FeynCalcForm[expr] changes the printed output to a an easy-to-read form. It \
allows a readable output also when running a terminal based ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " session. Whether the result of FeynCalcForm[expr] is displayed or not, \
depends on the setting of $PrePrint. $PrePrint = FeynCalcForm forces \
displaying everything after applying FeynCalcForm. In order to change to the \
normal (internal) Mathematica OutputForm, do: ($PrePrint=.)."
}], "Text",
 CellTags->"FeynCalcForm"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FC", All, CellTags],
  ButtonData->"FC",
  ButtonNote->"FC"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcInternal", All, CellTags],
  ButtonData->"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FeynCalcForm"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcForm"],

Cell["This is the normal notebook display:", "Text",
 CellTags->"FeynCalcForm"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "b", "]"}], ".", 
   RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[682]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcForm",
 CellLabel->"Out[682]="]
}, Open  ]],

Cell["\<\
This is the shorthand (terminal) display (easy-to-read form):\
\>", "Text",
 CellTags->"FeynCalcForm"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PrePrint", " ", "=", " ", "FeynCalcForm"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[683]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Options", "[", 
         RowBox[{"$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
         "]"}], "/.", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], "\[Rule]", 
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "OutputForm"}], ")"}]}]}], 
       ")"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[684]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "b", "]"}], ".", 
   RowBox[{"SUNT", "[", "c", "]"}]}], "]"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[685]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\<\"tr\"\>", "(", 
   FormBox[
    InterpretationBox[
     RowBox[{
      RowBox[{"\<\"T\"\>", "(", "a", ")"}], "\<\" \"\>", 
      RowBox[{"\<\"T\"\>", "(", "b", ")"}], "\<\" \"\>", 
      RowBox[{"\<\"T\"\>", "(", "c", ")"}]}],
     Infix[
      HighEnergyPhysics`FeynCalc`FeynCalcForm`Private`fcdot2[
       "T"[$CellContext`a], 
       "T"[$CellContext`b], 
       "T"[$CellContext`c]], " "],
     Editable->False],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcForm",
 CellLabel->"Out[685]="]
}, Open  ]],

Cell["Reset to normal notebook display:", "Text",
 CellTags->"FeynCalcForm"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$PrePrint", "=."}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[686]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"$FrontEnd", ",", 
    RowBox[{"Evaluate", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"Options", "[", 
         RowBox[{"$FrontEnd", ",", "\"\<CommonDefaultFormatTypes\>\""}], 
         "]"}], "/.", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "_"}], ")"}], "\[Rule]", 
         RowBox[{"(", 
          RowBox[{"\"\<Output\>\"", "\[Rule]", "TraditionalForm"}], ")"}]}]}],
        ")"}], "[", 
      RowBox[{"[", "1", "]"}], "]"}], "]"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalcForm",
 CellLabel->"In[687]:="]
}, Open  ]],

Cell["FeynCalcInternal", "ObjectName",
 CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcInternal"],

Cell["\<\
FeynCalcInternal[exp] translates exp into the internal FeynCalc (abstract \
data-type) representation.\
\>", "Text",
 CellTags->"FeynCalcInternal"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 "."
}], "Text",
 CellTags->"FeynCalcInternal"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalcInternal"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[688]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ",", 
    SuperscriptBox["\[Gamma]", "\[Rho]"], ",", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ",", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox["p", "\[Mu]"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[688]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[689]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[689]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FeynCalcInternal"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[690]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ",", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}], ",", 
    SuperscriptBox["g", 
     RowBox[{"\[Alpha]", "\[Beta]"}]], ",", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[690]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[691]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"DiracGamma", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", "D"}], "]"}], ",", 
   RowBox[{"DiracGamma", "[", 
    RowBox[{"Momentum", "[", "p", "]"}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "p", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], ",", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[691]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalcExternal", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[692]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"GA", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"GAD", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"GS", "[", "p", "]"}], ",", 
   RowBox[{"SP", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[692]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FCI", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"SD", "[", 
     RowBox[{"a", ",", "b"}], "]"}], ",", 
    RowBox[{"SUND", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
    RowBox[{"FAD", "[", "q", "]"}], ",", 
    RowBox[{"LC", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}]}],
    "}"}], "]"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[693]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    SubscriptBox["d", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], ",", 
    FractionBox["1", 
     SuperscriptBox["q", "2"]], ",", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[693]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FeynCalcInternal",
 CellLabel->"In[694]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], ",", 
   RowBox[{"SUND", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
   RowBox[{"SUNF", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
     RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], ",", 
   RowBox[{"FeynAmpDenominator", "[", 
    RowBox[{"PropagatorDenominator", "[", 
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"q", ",", "D"}], "]"}], ",", "0"}], "]"}], "]"}], ",", 
   RowBox[{"Eps", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]}], 
  "}"}]], "Output",
 CellTags->"FeynCalcInternal",
 CellLabel->"Out[694]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FeynCalc2FORM", "ObjectName",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalc2FORM"],

Cell["\<\
FeynCalc2FORM[expr] displays expr in FORM syntax. FeynCalc2FORM[file, x] \
writes x in FORM syntax to a file. FeynCalc2FORM[file, x==y] writes x=y to a \
file in FORM syntax.\
\>", "Text",
 CellTags->"FeynCalc2FORM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FORM2FeynCalc", All, CellTags],
  ButtonData->"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynCalc2FORM", "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[695]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsDiscard", "\[Rule]", "False"}], ",", 
    RowBox[{"FORMEpilog", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"FORMProlog", "\[Rule]", "\<\"write statistics;\"\>"}], ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"\<\"\\\\[Alpha]\"\>", "\[Rule]", "\<\"al\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Beta]\"\>", "\[Rule]", "\<\"be\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Gamma]\"\>", "\[Rule]", "\<\"ga\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Delta]\"\>", "\[Rule]", "\<\"de\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Mu]\"\>", "\[Rule]", "\<\"mu\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Nu]\"\>", "\[Rule]", "\<\"nu\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Rho]\"\>", "\[Rule]", "\<\"ro\"\>"}], ",", 
       RowBox[{"\<\"\\\\[Sigma]\"\>", "\[Rule]", "\<\"si\"\>"}]}], "}"}]}], 
    ",", 
    RowBox[{"TraceDimension", "\[Rule]", "4"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[695]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynCalc2FORM"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Rho]"}], "]"}], " ", 
  RowBox[{
   RowBox[{"y", "^", "2"}], "/", "d"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[696]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["y", "2"], " ", 
    SuperscriptBox["p", "\[Rho]"], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "d"], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[696]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[697]:="],

Cell["(y^2*d_(mu,nu)*p(ro))/d", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"LC", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Delta]", ",", "\[Rho]"}], 
   "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[698]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Alpha]", "\[Beta]", "\[Delta]", "\[Rho]"}]], 
  TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[698]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[699]:="],

Cell["(-i_)*e_(al,be,de,ro)", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracTrace", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[700]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
     SuperscriptBox["\[Gamma]", "\[Sigma]"]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[700]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[701]:="],

Cell["\<\
g_(0,mu)*g_(0,nu)*g_(0,ro)*g_(0,si)\
\>", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracTrace", "[", 
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], 
  RowBox[{"DiracTrace", "[", 
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Rho]"}], "]"}], "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[702]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Nu]"]}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
      SuperscriptBox["\[Gamma]", "\[Rho]"]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[702]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", "%", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[703]:="],

Cell["\<\
g_(0,mu)*g_(0,nu)*g_(1,mu)*g_(1,ro)\
\>", "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", 
   RowBox[{"\"\<fc2ftest.f\>\"", ",", 
    RowBox[{
     RowBox[{"MT", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], 
     RowBox[{"FV", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[704]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReadList", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
      ",", "\"\<\>\""}], "]"}], "<>", "\"\<fc2ftest.f\>\""}], ",", "String"}],
   "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[705]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "\<\"d_(mu,nu)*p(mu)\"\>", "}"}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[705]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"TR", "[", 
   RowBox[{
    RowBox[{"GA", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
    ".", 
    RowBox[{"GS", "[", 
     RowBox[{"p", ",", "q"}], "]"}]}], "]"}]}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[706]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[706]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"FeynCalc2FORM", "[", 
   RowBox[{"\"\<fc2ftest.f\>\"", ",", 
    RowBox[{"L", " ", "==", "t"}]}], "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[707]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "\"\<fc2ftest.f\>\""}], ",", 
    "String"}], "]"}], "]"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[708]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"Indices \\\\[Mu],\\\\[Nu],\\\\[Rho],\\\\[Sigma];\"\>"},
      {"\<\"Vectors p,q;\"\>"},
      {"\<\"write statistics;\"\>"},
      {"\<\"Local L = ( \"\>"},
      {"\<\"4*(d_(mu,si)*d_(nu,ro)*q.p-d_(mu,ro)*d_(nu,si)*q.p+d_(mu,nu)*d_(\
ro,si)*q.p+\"\>"},
      {"\<\"d_(ro,si)*p(nu)*q(mu)-d_(nu,si)*p(ro)*q(mu)+d_(nu,ro)*p(si)*q(mu)-\
\"\>"},
      {"\<\"d_(ro,si)*p(mu)*q(nu)+d_(mu,si)*p(ro)*q(nu)-d_(mu,ro)*p(si)*q(nu)+\
\"\>"},
      {"\<\"d_(nu,si)*p(mu)*q(ro)-d_(mu,si)*p(nu)*q(ro)+d_(mu,nu)*p(si)*q(ro)-\
\"\>"},
      {"\<\"d_(nu,ro)*p(mu)*q(si)+d_(mu,ro)*p(nu)*q(si)-d_(mu,nu)*p(ro)*q(si))\
 ); \"\>"},
      {"\<\"   \"\>"},
      {"\<\"Print; \"\>"},
      {"\<\".end\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"FeynCalc2FORM",
 CellLabel->"Out[708]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"If", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"FileNames", "[", "\"\<fc2ftest.f\>\"", "]"}], "=!=", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"DeleteFile", "[", "\"\<fc2ftest.f\>\"", "]"}]}], "]"}], 
  ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[709]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"FeynCalc2FORM",
 CellLabel->"In[710]:="]
}, Open  ]],

Cell["FeynCalcToLaTeX", "ObjectName",
 CellTags->"FeynCalcToLaTeX"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynCalcToLaTeX"],

Cell["\<\
FeynCalcToLaTeX[expr] generates LaTeX with line-breaking  for expr. 
FeynCalcToLaTeX[expr, 500] generates LaTeX for expr where 500 is the Window \
width  setting for the Mathematica frontend. Increasing its value will \
generate less line breaks.\
\>", "Text",
 CellTags->"FeynCalcToLaTeX"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "1", ",", "2"}], "]"}], "//", "Explicit"}]], "Input",
 CellLabel->"In[711]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["ci1",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["ci2",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"li1", "li2"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellLabel->"Out[711]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynCalcToLaTeX", "[", "%", "]"}]], "Input",
 CellLabel->"In[712]:="],

Cell[BoxData[
 FormBox["\<\"%% AMS-LaTeX Created by Wolfram Mathematica 8.0 : \
www.wolfram.com\\\\documentclass{article}\\n\\\\usepackage{amsmath, amssymb, \
graphics, \
setspace}\\\\newcommand{\\\\mathsym}[1]{{}}\\n\\\\newcommand{\\\\unicode}[1]{{\
}}\\\\newcounter{mathematicapage}\\n\\\\begin{document}\\\\begin{doublespace}\
\\n\\\\noindent\\\\(-\\\\frac{i \\\\delta _{\\\\text{ci1}\\\\text{ci2}} g^{\\\
\\text{li1}\\\\text{li2}}}{p^2}\\\\)\\n\\\\end{doublespace}\"\>", 
  TraditionalForm]], "Output",
 CellLabel->"Out[712]="]
}, Open  ]]
}, Open  ]],

Cell["FeynmanParameterNames", "ObjectName",
 CellTags->"FeynmanParameterNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParameterNames"],

Cell["\<\
FeynmanParameterNames is an option for FeynmanParametrize and \
FeynmanParametrize.\
\>", "Text",
 CellTags->"FeynmanParameterNames"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynmanParametrize", All, CellTags],
  ButtonData->"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 ", ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynmanParametrize", All, CellTags],
  ButtonData->"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 "."
}], "Text",
 CellTags->"FeynmanParameterNames"]
}, Open  ]],

Cell["FeynmanParametrize ***unfinished***", "ObjectName",
 CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParametrize"],

Cell["\<\
FeynmanParametrize[exp,k] introduces feynman parameters for all one-loop \
integrals in exp (k = integration momentum).\
\>", "Text",
 CellTags->"FeynmanParametrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanParametrize", "]"}]], "Input",
 CellTags->"FeynmanParametrize",
 CellLabel->"In[713]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"FeynmanParameterNames", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynmanParametrize",
 CellLabel->"Out[713]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanParametrize"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanParametrize",
 CellLabel->"In[714]:="]
}, Open  ]],

Cell["FeynRule", "ObjectName",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynRule"],

Cell["\<\
FeynRule[lag, {fields}] derives the Feynman rule corresponding to the field \
configuration fields of the lagrangian lag.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynRule", "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[715]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Anti5", "\[Rule]", 
     RowBox[{"-", "\[Infinity]"}]}], ",", 
    RowBox[{"Contract", "\[Rule]", "False"}], ",", 
    RowBox[{"Factor1", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PartialD", "\[Rule]", "RightPartialD"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}], ",", 
    RowBox[{"InitialFunction", "\[Rule]", "Identity"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[715]="]
}, Open  ]],

Cell["FeynRule does not calculate propagator Feynman rules.", "Text",
 CellTags->"FeynRule"],

Cell["\<\
The option ZeroMomentumInsertion can be used for twist-2 and higher twist \
operators.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Lagrangian",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Lagrangian", All, CellTags],
  ButtonData->"Lagrangian",
  ButtonNote->"Lagrangian"],
 "."
}], "Text",
 CellTags->"FeynRule"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gou", "=", 
  RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[716]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[716]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"gop", "=", 
  RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[717]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Beta]", "\[Gamma]"}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[717]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "gop", "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[718]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Beta]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox[
         FormBox["\[Gamma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Gamma]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox[
         FormBox["\[Beta]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SubscriptBox["g", "s"], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b31",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c32",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Beta]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b31",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Gamma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c32",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Alpha]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox["\[CapitalDelta]",
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["A",
         TraditionalForm], 
        FormBox[
         FormBox["\[Alpha]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SubscriptBox["g", "s"], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b33",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c34",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox[
          FormBox["\[Alpha]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b33",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox[
         FormBox["A",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["c34",
           TraditionalForm],
          TraditionalForm]]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[718]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Cases2", "[", 
  RowBox[{"%", ",", "QuantumField"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[719]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b33",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b31",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c32",
       TraditionalForm],
      TraditionalForm]], ",", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["c34",
       TraditionalForm],
      TraditionalForm]], ",", 
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Alpha]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], ",", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[719]="]
}, Open  ]],

Cell["2-gluon Feynman rules (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[720]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[720]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f2u", "=", 
  RowBox[{"FullSimplify", "/@", 
   RowBox[{"Factor2", "[", 
    RowBox[{"FeynRule", "[", 
     RowBox[{"gou", ",", "fi", ",", 
      RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[721]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], "m"]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], "m"]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}],
      TraditionalForm], "2"], " ", 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}],
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[721]="]
}, Open  ]],

Cell["2-gluon Feynman rules (polarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[722]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[722]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"f2p", "=", 
  RowBox[{"FullSimplify", "/@", 
   RowBox[{"Factor2", "[", 
    RowBox[{"FeynRule", "[", 
     RowBox[{"gop", ",", "fi", ",", 
      RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}], 
    "]"}]}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[723]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "m"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["\[Mu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}], "+", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}]}], ")"}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "m"], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["\[Nu]",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["q",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}], "+", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           SuperscriptBox["\[Epsilon]", 
            RowBox[{
             FormBox[
              FormBox["\[Mu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox[
              FormBox["\[Nu]",
               TraditionalForm],
              TraditionalForm], 
             FormBox["p",
              TraditionalForm], 
             FormBox["\[CapitalDelta]",
              TraditionalForm]}]]}]}], ")"}]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}],
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[723]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Calc", "[", 
   RowBox[{"f2p", "/.", 
    RowBox[{"p", "\[Rule]", 
     RowBox[{"-", "q"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[724]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["q",
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[724]="]
}, Open  ]],

Cell["\<\
Compare with the Feynman rule tabulated in Twist2GluonOperator.\
\>", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"q", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[725]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["q",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[725]="]
}, Open  ]],

Cell["quark-quark Feynman rule (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qo", "=", 
  RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[726]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[726]="]
}, Open  ]],

Cell["quark-quark -gluon-gluon Feynman rule (unpolarized)", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qo", "=", 
  RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[727]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[727]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"qggf", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", "QuarkField", "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}], "[", "q", "]"}], 
    ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "r", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "s", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[728]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     FormBox["\[Psi]",
      TraditionalForm], "(", "p", ")"}], ",", 
    RowBox[{
     FormBox[
      OverscriptBox["\[Psi]", "_"],
      TraditionalForm], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "s", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[728]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"n4", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{"qo", ",", "qggf", ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[729]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "j"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "j"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[729]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[730]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["s",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[730]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"n4", "-", "t4"}], "/.", 
     RowBox[{"OPEm", "\[Rule]", "5"}]}], "/.", 
    RowBox[{"s", "\[Rule]", 
     RowBox[{
      RowBox[{"-", "p"}], "-", "q", "-", "r"}]}]}], "/.", 
   RowBox[{"D", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[731]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[731]="]
}, Open  ]],

Cell["\<\
 In general equality can be shown by Timing[Factor2[ \
FCE[Calc[ChangeDimension[FCE[OPESumExplicit[n4-t4]],4]/.s\[Rule]-p-q-r]]]] \
but it is a little bit slow ...\
\>", "Text",
 CellTags->"FeynRule"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"qggf", ",", "n2", ",", "n4"}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[732]:="],

Cell["QCD vertices", "Text",
 CellTags->"FeynRule"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fii", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Rho]", "}"}], ",", 
       RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[733]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[733]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g3", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fii"}], 
   "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[734]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"FeynRule",
 CellLabel->"Out[734]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[735]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"r", "-", "p"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"q", "-", "r"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[735]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"g3", "-", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[736]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[736]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"fi4", "=", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Mu]", "}"}], ",", 
       RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Nu]", "}"}], ",", 
       RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Rho]", "}"}], ",", 
       RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}], ",", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", 
       RowBox[{"{", "\[Sigma]", "}"}], ",", 
       RowBox[{"{", "d", "}"}]}], "]"}], "[", "s", "]"}]}], "}"}]}]], "Input",\

 CellTags->"FeynRule",
 CellLabel->"In[737]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], "(", "p", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], "(", "q", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], "(", "r", ")"}], ",", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], "(", "s", ")"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[737]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"g4", "=", 
  RowBox[{"FeynRule", "[", 
   RowBox[{
    RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}], ",", "fi4"}], 
   "]"}]}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[738]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[ImaginaryI]", " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si40",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si40",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si40",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si40",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si40",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si40",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[738]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[739]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u100",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u100",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u100",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u100",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u100",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u100",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[739]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", 
  RowBox[{"g4", "-", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{"%", ",", "4"}], "]"}]}], "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[740]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FeynRule",
 CellLabel->"Out[740]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "f2p", ",", "f2u", ",", "f3", ",", "f32", ",", "fi", ",", "fi4", ",", "fii",
    ",", "g3", ",", "g4", ",", "gop", ",", "gou", ",", "n3", ",", "nf3", ",", 
   "n4", ",", "np2", ",", "npf3", ",", "p33", ",", "pf3", ",", "pn3", ",", 
   "pqo", ",", "qf", ",", "qp", ",", "qgf", ",", "qo", ",", "t4"}], 
  "]"}]], "Input",
 CellTags->"FeynRule",
 CellLabel->"In[741]:="]
}, Open  ]],

Cell["FI", "ObjectName",
 CellTags->"FI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FI"],

Cell["\<\
FI changes the output format to InputForm. This is useful to see the internal \
representation of FeynCalc objects. To change back to FeynCalcForm use FC.\
\>", "Text",
 CellTags->"FI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcForm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcForm", All, CellTags],
  ButtonData->"FeynCalcForm",
  ButtonNote->"FeynCalcForm"],
 ", ",
 ButtonBox["FC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FC", All, CellTags],
  ButtonData->"FC",
  ButtonNote->"FC"],
 ", ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FeynCalcInternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcInternal", All, CellTags],
  ButtonData->"FeynCalcInternal",
  ButtonNote->"FeynCalcInternal"],
 "."
}], "Text",
 CellTags->"FI"]
}, Open  ]],

Cell["FieldDerivative", "ObjectName",
 CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FieldDerivative"],

Cell["\<\
FieldDerivative[f[x],x,li1,li2,...] is the derivative of f[x] with respect to \
space-time variables x and with Lorentz indices li1, li2, ..., where li1, \
li2, ... have head LorentzIndex. FieldDerivative[f[x],x,li1,li2,...] can be \
given as FieldDerivative[f[x],x,{l1,l2,...}], where l1 is li1 without the \
head, ... FieldDerivative is defined only for objects with head \
QuantumField[...]. If the space-time derivative of other objects is wanted, \
the corresponding rule must be specified.\
\>", "Text",
 CellTags->"FieldDerivative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialD", All, CellTags],
  ButtonData->"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 "."
}], "Text",
 CellTags->"FieldDerivative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FieldDerivative"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"B", ",", 
     RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "[", "y", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"C", ",", 
     RowBox[{"{", "\[Rho]", "}"}]}], "]"}], "[", "x", "]"}], ".", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"D", ",", 
     RowBox[{"{", "\[Sigma]", "}"}]}], "]"}], "[", "y", "]"}]}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[742]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
   RowBox[{
    SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
   RowBox[{
    SubscriptBox[
     FormBox["C",
      TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
   RowBox[{
    SubscriptBox[
     FormBox["D",
      TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"FieldDerivative",
 CellLabel->"Out[742]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FieldDerivative", "[", 
   RowBox[{"%", ",", "x", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[743]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["C",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FieldDerivative",
 CellLabel->"Out[743]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FieldDerivative", "[", 
   RowBox[{"%", ",", "y", ",", 
    RowBox[{"{", "\[Nu]", "}"}]}], "]"}], "//", "DotExpand"}]], "Input",
 CellTags->"FieldDerivative",
 CellLabel->"In[744]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["C",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["D",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["A", "\[Mu]"], "(", "x", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["B", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["C",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     SubscriptBox["B", "\[Nu]"], "(", "y", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox[
        FormBox["D",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}]}], "+", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "x", ")"}]}], ".", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["B", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ")"}], "\[InvisibleApplication]", 
     RowBox[{"(", "y", ")"}]}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["C",
       TraditionalForm], "\[Rho]"], "(", "x", ")"}], ".", 
    RowBox[{
     SubscriptBox[
      FormBox["D",
       TraditionalForm], "\[Sigma]"], "(", "y", ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FieldDerivative",
 CellLabel->"Out[744]="]
}, Open  ]]
}, Open  ]],

Cell["FieldStrength", "ObjectName",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FieldStrength"],

Cell[TextData[{
 "FieldStrength[\[Mu], \[Nu], a] is the field strength tensor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SubsuperscriptBox["A", "\[Nu]", "a"]}], "-", 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Nu]"], 
     SubsuperscriptBox["A", "\[Mu]", "a"]}], "+", 
    RowBox[{
     SubscriptBox["g", "s"], " ", 
     SubsuperscriptBox["A", "\[Mu]", "b1"], 
     SubsuperscriptBox["A", "\[Nu]", "c1"]}]}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["f", 
     RowBox[{"a", " ", "b1", " ", "c1"}]]}], TraditionalForm]]],
 ". FieldStrength[\[Mu], \[Nu]] is the field strength tensor ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     SubsuperscriptBox["A", "\[Nu]", "\[Null]"]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", "\[Nu]"], 
      SubsuperscriptBox["A", "\[Mu]", "\[Null]"]}], "."}]}], 
   TraditionalForm]]],
 " The name of the field (A) and the coupling constant (g) can be set through \
the options or by additional arguments. The first two indices are interpreted \
as type LorentzIndex, except OPEDelta, which is converted to \
Momentum[OPEDelta]."
}], "Text",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FieldStrength", "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[745]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{
    "HighEnergyPhysics`fctools`IndexPosition`IndexPosition", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"0", ",", "0"}], "}"}]}], ",", 
    RowBox[{"Symbol", "\[Rule]", "\<\"F\"\>"}], ",", 
    RowBox[{"QuantumField", "\[Rule]", "A"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[745]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuantumField", All, CellTags],
  ButtonData->"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"FieldStrength"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FieldStrength"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[746]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "\[Null]"], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[746]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[747]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{"\[Mu]", "\[Nu]"}], "a"], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[747]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[748]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], "\[Null]"]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "\[Null]"]}]}], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[748]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[749]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b109", "c110"}]], " ", 
    RowBox[{
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b109",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c110",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[749]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[750]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], "-", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[750]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "OPEDelta", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[751]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"Momentum", "[", "OPEDelta", "]"}]}], "]"}], "-", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
    RowBox[{"PartialD", "[", 
     RowBox[{"Momentum", "[", "OPEDelta", "]"}], "]"}], ",", "GaugeField", 
    ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[751]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FieldStrength", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
   RowBox[{"CouplingConstant", "\[Rule]", 
    RowBox[{"-", "Gstrong"}]}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"FieldStrength",
 CellLabel->"In[752]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubscriptBox["g", "s"], " ", 
     SubscriptBox["f", 
      RowBox[{"a", "b111", "c112"}]], " ", 
     RowBox[{
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b111",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubsuperscriptBox[
       FormBox["A",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c112",
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FieldStrength",
 CellLabel->"Out[752]="]
}, Open  ]]
}, Open  ]],

Cell["FinalFunction", "ObjectName",
 CellTags->"FinalFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FinalFunction"],

Cell["FinalFunction is an option for OneLoopSum.", "Text",
 CellTags->"FinalFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"FinalFunction"]
}, Open  ]],

Cell["FinalSubstitutions", "ObjectName",
 CellTags->"FinalSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FinalSubstitutions"],

Cell["\<\
FinalSubstitutions is an option for OneLoop and OneLoopSum and Write2. All \
substitutions indicated hereby are done at the end of the calculation.\
\>", "Text",
 CellTags->"FinalSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Write2", All, CellTags],
  ButtonData->"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"FinalSubstitutions"]
}, Open  ]],

Cell["FORM", "ObjectName",
 CellTags->"FORM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORM"],

Cell["\<\
FORM is an option for RHI. If set to True a FORM file is generated and run \
from Mathematica (provided R. Hamberg's FORM-program is installed correctly \
... ).\
\>", "Text",
 CellTags->"FORM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 ", ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc2FORM", All, CellTags],
  ButtonData->"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"FORM"]
}, Open  ]],

Cell["FORMEpilog", "ObjectName",
 CellTags->"FORMEpilog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORMEpilog"],

Cell["\<\
FORMEpilog is an option for FeynCalc2FORM. It may be set to a string which is \
put at the end of the FORM-file.\
\>", "Text",
 CellTags->"FORMEpilog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc2FORM", All, CellTags],
  ButtonData->"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 ", ",
 ButtonBox["FORMProlog",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FORMProlog", All, CellTags],
  ButtonData->"FORMProlog",
  ButtonNote->"FORMProlog"],
 "."
}], "Text",
 CellTags->"FORMEpilog"]
}, Open  ]],

Cell["FORMProlog", "ObjectName",
 CellTags->"FORMProlog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORMProlog"],

Cell["\<\
FORMProlog is an option for FeynCalc2FORM. It may be set to a string which is \
put after the type declarations of the FORM-file.\
\>", "Text",
 CellTags->"FORMProlog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc2FORM", All, CellTags],
  ButtonData->"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 ", ",
 ButtonBox["FORMEpilog",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FORMEpilog", All, CellTags],
  ButtonData->"FORMEpilog",
  ButtonNote->"FORMEpilog"],
 "."
}], "Text",
 CellTags->"FORMProlog"]
}, Open  ]],

Cell["FORM2FeynCalc", "ObjectName",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FORM2FeynCalc"],

Cell["\<\
FORM2FeynCalc[expr] translates the FORM expr into FeynCalc notation. \
FORM2FeynCalc[file] translates the FORM expresssions in file into \
FeynCalcnotation. FORM2FeynCalc[file, x1, x2, ...] reads in a file in \
FORM-format and translates the assignments for the variables a, b, ... into \
FeynCalc syntax. If the option Set is True, the variables x1, x2 are assigned \
to the right hand sides defined in the FORM-file.\
\>", "Text",
 CellTags->"FORM2FeynCalc"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc2FORM", All, CellTags],
  ButtonData->"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FORM2FeynCalc", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[753]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Dot", "\[Rule]", "Dot"}], ",", 
    RowBox[{"HoldForm", "\[Rule]", "True"}], ",", 
    RowBox[{"LorentzIndex", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"mu", ",", "nu", ",", "al", ",", "be"}], "}"}]}], ",", 
    RowBox[{"Set", "\[Rule]", "False"}], ",", 
    RowBox[{"Replace", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Vectors", "\[Rule]", "Automatic"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[753]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<p.q + 2*x m^2\>\"", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[754]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"x", ".", 
     SuperscriptBox["m", "2"]}]}], "+", 
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[754]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[755]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"2", " ", 
   RowBox[{"x", ".", 
    SuperscriptBox["m", "2"]}]}], "+", 
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[755]//StandardForm="]
}, Open  ]],

Cell["\<\
Functions are automatically converted right, but bracketed expressions need \
to be substituted explicitly.\
\>", "Text",
 CellTags->"FORM2FeynCalc"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  RowBox[{"\"\<x +f(z)+ log(x)^2+[li2(1-x)]\>\"", ",", 
   RowBox[{"Replace", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"\"\<[li2(1-x)]\>\"", "\[Rule]", "\"\<PolyLog[2,1-x]\>\""}], 
     "}"}]}]}], "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[756]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "z", ")"}], "+", 
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "+", "x", "+", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "x", ")"}]}], TraditionalForm]], "Output",\

 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[756]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[757]:="],

Cell[BoxData[
 RowBox[{"x", "+", 
  RowBox[{"f", "[", "z", "]"}], "+", 
  SuperscriptBox[
   RowBox[{"Log", "[", "x", "]"}], "2"], "+", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}]}]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[757]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<x + [(1)]*y -[(-1)^m]\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[758]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Hold", "[", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "]"}]}], "+", 
   RowBox[{
    RowBox[{"Hold", "[", "1", "]"}], ".", "y"}], "+", "x"}], 
  TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[758]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ReleaseHold", "[", "%", "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[759]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], "+", "x", "+", 
   RowBox[{"1", ".", "y"}]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[759]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", "\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[760]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"mu", "nu"}]], "+", 
   RowBox[{
    SuperscriptBox["p", "mu"], ".", 
    SuperscriptBox["q", "nu"]}]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[760]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[761]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "mu"}], "]"}], ".", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "nu"}], "]"}]}], "+", 
  RowBox[{"MT", "[", 
   RowBox[{"mu", ",", "nu"}], "]"}]}]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[761]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  RowBox[{"\"\<p(mu)*q(nu)+d_(mu,nu)\>\"", ",", 
   RowBox[{"Replace", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"mu", "\[Rule]", "\[Mu]"}], ",", 
      RowBox[{"nu", "\[Rule]", "\[Nu]"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[762]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], "+", 
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], ".", 
    SuperscriptBox["q", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[762]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FORM2FeynCalc", "[", 
  "\"\<i_*az*bz*aM^2*D1*[(1)]*b_G1 * ( 4*eperp(mu,nu)*avec.bvec*blam )\>\"", 
  "]"}]], "Input",
 CellTags->"FORM2FeynCalc",
 CellLabel->"In[763]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"4", " ", "\[ImaginaryI]"}], ")"}], ".", "az", ".", "bz", ".", 
   SuperscriptBox["aM", "2"], ".", 
   RowBox[{"(", 
    TagBox[
     RowBox[{
      RowBox[{
       SuperscriptBox["q", "\[Mu]"], ".", 
       SuperscriptBox["p", "\[Nu]"], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["p", "\[Mu]"], ".", 
       SuperscriptBox["q", "\[Nu]"], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
      RowBox[{
       SuperscriptBox["q", "\[Mu]"], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], ".", 
       SuperscriptBox["p", "\[Rho]"]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], ".", 
       SuperscriptBox["q", "\[Nu]"], ".", 
       SuperscriptBox["p", "\[Rho]"]}], "+", 
      RowBox[{
       SuperscriptBox["p", "\[Mu]"], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], ".", 
       SuperscriptBox["q", "\[Rho]"]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], ".", 
       SuperscriptBox["p", "\[Nu]"], ".", 
       SuperscriptBox["q", "\[Rho]"]}], "+", 
      RowBox[{
       SuperscriptBox["q", "\[Mu]"], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], ".", 
       SuperscriptBox["p", "\[Sigma]"]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], ".", 
       SuperscriptBox["q", "\[Nu]"], ".", 
       SuperscriptBox["p", "\[Sigma]"]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
       SuperscriptBox["q", "\[Rho]"], ".", 
       SuperscriptBox["p", "\[Sigma]"]}], "-", 
      RowBox[{
       SuperscriptBox["p", "\[Mu]"], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], ".", 
       SuperscriptBox["q", "\[Sigma]"]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], ".", 
       SuperscriptBox["p", "\[Nu]"], ".", 
       SuperscriptBox["q", "\[Sigma]"]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
       SuperscriptBox["p", "\[Rho]"], ".", 
       SuperscriptBox["q", "\[Sigma]"]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], ".", 
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], ".", 
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], ".", 
       SuperscriptBox["g", 
        RowBox[{"\[Rho]", "\[Sigma]"}]], ".", 
       RowBox[{"(", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], ")"}]}]}],
     HoldForm], ")"}], ".", 
   RowBox[{"Hold", "[", "1", "]"}], ".", "b$G1", ".", 
   RowBox[{"eperp", "(", 
    RowBox[{"mu", ",", "nu"}], ")"}], ".", 
   RowBox[{"(", 
    FormBox[
     RowBox[{
      FormBox["avec",
       TraditionalForm], "\[CenterDot]", 
      FormBox["bvec",
       TraditionalForm]}],
     TraditionalForm], ")"}], ".", "blam"}], TraditionalForm]], "Output",
 CellTags->"FORM2FeynCalc",
 CellLabel->"Out[763]="]
}, Open  ]]
}, Open  ]],

Cell["FourDivergence", "ObjectName",
 CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourDivergence"],

Cell["\<\
FourDivergence[exp, FourVector[p, mu]] calculates the partial derivative of \
exp w.r.t. p(mu). FourDivergence[exp, FourVector[p, mu], FourVector[p,nu], \
...]gives the multiple derivative.\
\>", "Text",
 CellTags->"FourDivergence"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RussianTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RussianTrick", All, CellTags],
  ButtonData->"RussianTrick",
  ButtonNote->"RussianTrick"],
 "."
}], "Text",
 CellTags->"FourDivergence"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourDivergence"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[764]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[764]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourDivergence", "[", 
  RowBox[{"t", ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[765]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[765]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "-", "k"}], ",", "q"}], "]"}]}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[766]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "k"}],
    TraditionalForm], ")", "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[766]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourDivergence", "[", 
  RowBox[{"t", ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[767]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FourDivergence",
 CellLabel->"Out[767]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"FourDivergence",
 CellLabel->"In[768]:="]
}, Open  ]],

Cell["FourLaplacian", "ObjectName",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourLaplacian"],

Cell[TextData[{
 "FourLaplacian[exp, p, q] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PartialD]", 
    RowBox[{"/", 
     RowBox[{"\[PartialD]", 
      SubscriptBox["p", "\[Mu]"]}]}], " "}], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{"\[PartialD]", 
    RowBox[{"/", 
     RowBox[{"\[PartialD]", 
      SubscriptBox["q", "\[Mu]"]}]}], " "}], TraditionalForm]]],
 "exp."
}], "Text",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FourLaplacian", "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[769]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[769]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourDivergence",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourDivergence", All, CellTags],
  ButtonData->"FourDivergence",
  ButtonNote->"FourDivergence"],
 ", ",
 ButtonBox["RussianTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RussianTrick", All, CellTags],
  ButtonData->"RussianTrick",
  ButtonNote->"RussianTrick"],
 "."
}], "Text",
 CellTags->"FourLaplacian"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourLaplacian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"q", ",", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[770]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["q",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[770]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourLaplacian", "[", 
  RowBox[{"%", ",", "q", ",", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[771]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "D"}], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[771]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SOD", "[", "q", "]"}], "^", "OPEm"}], 
   RowBox[{"FAD", "[", 
    RowBox[{"q", ",", 
     RowBox[{"q", "-", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[772]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"FourLaplacian",
 CellLabel->"Out[772]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourLaplacian", "[", 
  RowBox[{"%", ",", "q", ",", " ", "q"}], "]"}]], "Input",
 CellTags->"FourLaplacian",
 CellLabel->"In[773]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", "D", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], "m"]}], 
     RowBox[{
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox["q", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], "-", 
   FractionBox[
    RowBox[{"2", " ", "D", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "m", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", "m", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{"12", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox["q", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellTags->"FourLaplacian",
 CellLabel->"Out[773]="]
}, Open  ]]
}, Open  ]],

Cell["FourVector", "ObjectName",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FourVector"],

Cell[TextData[{
 "FourVector[p, ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the four-dimensional vector p with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ". A vector with space-time Dimension D is obtained by supplying the option \
Dimension \[Rule] D."
}], "Text",
 CellTags->"FourVector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FV", All, CellTags],
  ButtonData->"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FVD", All, CellTags],
  ButtonData->"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FourVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[774]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[774]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[775]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{"p", "-", "q"}],
      HoldForm],
     TraditionalForm], ")"}], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[775]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[776]:="],

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[776]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[777]:="],

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", 
   RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[777]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[778]:="],

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[778]//StandardForm="]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FourVector. Since \
FourVector is turned into Pair internally ExpandScalarProduct may be used.\
\>", "Text",
 CellTags->"FourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FourVector", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FourVector",
 CellLabel->"In[779]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FourVector",
 CellLabel->"Out[779]="]
}, Open  ]]
}, Open  ]],

Cell["FreeIndex", "ObjectName",
 CellTags->"FreeIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FreeIndex"],

Cell["\<\
FreeIndex is a datatype which is recognized by Contract. Possible use: \
DataType[mu, FreeIndex] = True.\
\>", "Text",
 CellTags->"FreeIndex"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"FreeIndex"]
}, Open  ]],

Cell["FreeQ2", "ObjectName",
 CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FreeQ2"],

Cell["\<\
FreeQ2[expr, {form1, form2, ...}] yields True if expr does not contain any \
occurence of form1, form2, ... and False otherwise. FreeQ2[expr, form] is the \
same as FreeQ[expr, form].\
\>", "Text",
 CellTags->"FreeQ2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SelectFree",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SelectFree", All, CellTags],
  ButtonData->"SelectFree",
  ButtonNote->"SelectFree"],
 ", ",
 ButtonBox["SelectNotFree",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SelectNotFree", All, CellTags],
  ButtonData->"SelectNotFree",
  ButtonNote->"SelectNotFree"],
 "."
}], "Text",
 CellTags->"FreeQ2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FreeQ2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"x", "+", 
    RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"a", ",", "x"}], "}"}]}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[780]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[780]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"x", "+", 
    RowBox[{"f", "[", "x", "]"}], "+", "y"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[781]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[781]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{"x", ",", " ", "y"}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[782]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[782]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FreeQ2", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", " ", "f"}], "]"}]], "Input",
 CellTags->"FreeQ2",
 CellLabel->"In[783]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"FreeQ2",
 CellLabel->"Out[783]="]
}, Open  ]]
}, Open  ]],

Cell["FRH", "ObjectName",
 CellTags->"FRH"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FRH"],

Cell["\<\
FRH[exp_] := FixedPoint[ReleaseHold, exp], i.e., FRH removes all HoldForm and \
Hold in exp.\
\>", "Text",
 CellTags->"FRH"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"FRH"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FRH"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hold", "[", 
  RowBox[{"1", "-", "1", " ", "-", " ", 
   RowBox[{"Hold", "[", 
    RowBox[{"2", "-", "2"}], "]"}]}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[784]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Hold", "[", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Hold", "[", 
      RowBox[{"2", "-", "2"}], "]"}]}], "+", "1", "-", "1"}], "]"}], 
  TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[784]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", "%", "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[785]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[785]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{"Solve", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", "3"}], "-", "x", "-", "1"}], "==", "0"}], "]"}], ",", 
   "x", ",", 
   RowBox[{"IsolateNames", "\[Rule]", "HH"}]}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[786]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "115", ")"}],
       HoldForm]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "118", ")"}],
       HoldForm]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", "\[Rule]", 
      TagBox[
       RowBox[{"HH", "(", "119", ")"}],
       HoldForm]}], "}"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[786]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", 
  RowBox[{"HH", "[", "3", "]"}], "]"}]], "Input",
 CellTags->"FRH",
 CellLabel->"In[787]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"HH", "(", "3", ")"}], TraditionalForm]], "Output",
 CellTags->"FRH",
 CellLabel->"Out[787]="]
}, Open  ]]
}, Open  ]],

Cell["FromTFI", "ObjectName",
 CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FromTFi"],

Cell["\<\
FromTFi[expr, q1, q2, p] translates the TFi notatation from the TARCER \
package to the usual FeynCalc notation. See TFi for details on the \
conventions.\
\>", "Text",
 CellTags->"FromTFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TFI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TFI", All, CellTags],
  ButtonData->"TFI",
  ButtonNote->"TFI"],
 ", ",
 ButtonBox["ToTFI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ToTFI", All, CellTags],
  ButtonData->"ToTFI",
  ButtonNote->"ToTFI"],
 "."
}], "Text",
 CellTags->"FromTFI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FromTFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToTFI", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}], ",", "q1", ",", "q2", ",", "p"}], 
  "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[788]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFI", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["m", "2"], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0"},
       {"1", "M"},
       {"1", "0"},
       {"1", "m"},
       {"1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"FromTFi",
 CellLabel->"Out[788]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromTFI", "[", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"D", ",", 
     RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "p"}], "]"}], ",", 
     RowBox[{"SOD", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "M"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1", ",", 
   "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[789]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFI", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["m", "2"], ",", 
    RowBox[{
     FormBox["\<\"\[CapitalDelta]\"\>",
      TraditionalForm], 
     FormBox["\<\"\[CenterDot]\"\>",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "0"},
       {"1", "M"},
       {"1", "0"},
       {"1", "m"},
       {"1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"FromTFi",
 CellLabel->"Out[789]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FromTFI", "[", 
  RowBox[{
   RowBox[{"TFI", "[", 
    RowBox[{"D", ",", 
     RowBox[{"SPD", "[", 
      RowBox[{"p", ",", "p"}], "]"}], ",", 
     RowBox[{"SOD", "[", "p", "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"0", ",", "1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "M"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "m"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{"1", ",", "0"}], "}"}]}], "}"}]}], "]"}], ",", "q1", ",", 
   "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"FromTFi",
 CellLabel->"In[790]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFI", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["m", "2"], ",", 
    RowBox[{
     FormBox["\<\"\[CapitalDelta]\"\>",
      TraditionalForm], 
     FormBox["\<\"\[CenterDot]\"\>",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}], ",", 
    RowBox[{"{", 
     RowBox[{"0", ",", "1"}], "}"}], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"1", "m"},
       {"1", "M"},
       {"1", "0"},
       {"1", "m"},
       {"1", "0"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"FromTFi",
 CellLabel->"Out[790]="]
}, Open  ]]
}, Open  ]],

Cell["FUNCTION", "ObjectName",
 CellTags->"FUNCTION"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FUNCTION"],

Cell["\<\
FUNCTION[exp, string] is a head of an expression to be declared a function \
(of type string), if used in Write2.\
\>", "Text",
 CellTags->"FUNCTION"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Write2", All, CellTags],
  ButtonData->"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"FUNCTION"]
}, Open  ]],

Cell["FunctionalD", "ObjectName",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FunctionalD"],

Cell["\<\
FunctionalD[expr, {QuantumField[name, {mu}, {a}][p], ...}] calculates the \
functional derivative of expr with respect to the field list (with incoming \
momenta p, etc.) and does the fourier transform. FunctionalD[expr, \
{QuantumField[name, {mu},{a}], ...}] calculates the functional derivative and \
does partial integration but omits the x-space delta functions.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell["FunctionalD is a low level function used in FeynRule.", "Text",
 CellTags->"FunctionalD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynRule", All, CellTags],
  ButtonData->"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuantumField", All, CellTags],
  ButtonData->"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"FunctionalD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FunctionalD"],

Cell[TextData[{
 "Instead of the usual ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"\[Delta]\[Phi]", "(", "x", ")"}], "/", 
     RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
    RowBox[{
     SuperscriptBox["\[Delta]", 
      RowBox[{"(", "D", ")"}]], "(", 
     RowBox[{"x", "-", "y"}], ")"}]}], TraditionalForm]]],
 " the arguments and the \[Delta] function are omitted, i.e., for the program \
for simplicity: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Delta]\[Phi]", "/", "\[Delta]\[Phi]"}], "=", "1"}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[791]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[791]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"QuantumField", "[", "\[Phi]", "]"}], "^", "2"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[792]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "\[Phi]"}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[792]="]
}, Open  ]],

Cell[TextData[{
 "Instead of the usual ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"\[Delta]", " ", 
       RowBox[{
        SubscriptBox["\[PartialD]", "\[Mu]"], 
        RowBox[{"\[Phi]", "(", "x", ")"}]}]}], ")"}], "/", 
     RowBox[{"\[Delta]\[Phi]", "(", "y", ")"}]}], "=", 
    RowBox[{
     SubscriptBox["\[PartialD]", "\[Mu]"], 
     RowBox[{
      SuperscriptBox["\[Delta]", 
       RowBox[{"(", "D", ")"}]], "(", 
      RowBox[{"x", "-", "y"}], ")"}]}]}], TraditionalForm]]],
 " the arguments are omitted, and the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
 "operator is specified by default to be an integration by parts operator, \
i.e., the right hand side will be just ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"-", 
     FormBox[
      SubscriptBox["\[PartialD]", "\[Mu]"],
      TraditionalForm]}], ","}], TraditionalForm]]],
 " or, more precisely (by default) ",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], TraditionalForm]]],
 "."
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[793]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[793]="]
}, Open  ]],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"S", "[", "\[Phi]", "]"}], " ", "=", 
   RowBox[{
    RowBox[{"1", "/", "2"}], 
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      SuperscriptBox["d", "D"], 
      RowBox[{"x", " ", "[", " ", 
       RowBox[{
        RowBox[{
         RowBox[{
          SubscriptBox["\[PartialD]", "\[Mu]"], 
          RowBox[{"\[Phi]", "(", "x", ")"}]}], " ", 
         RowBox[{
          SuperscriptBox["\[PartialD]", "\[Mu]"], 
          RowBox[{"\[Phi]", "(", "x", ")"}]}]}], "-", 
        RowBox[{
         SuperscriptBox["m", "2"], 
         RowBox[{"\[Phi]", "(", "x", ")"}], " ", 
         RowBox[{"\[Phi]", "(", "y", ")"}]}]}], "]"}]}]}]}]}], 
  TraditionalForm]]]], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"s", "[", "\[Phi]", "]"}], "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}], ".", 
      
      RowBox[{"QuantumField", "[", 
       RowBox[{
        RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "\[Phi]"}], "]"}]}], 
     "-", 
     RowBox[{
      RowBox[{"m", "^", "2"}], " ", 
      RowBox[{
       RowBox[{"QuantumField", "[", "\[Phi]", "]"}], ".", 
       RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}]}]}], ")"}], "/", 
   "2"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[794]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}], ".", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], "-", 
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      RowBox[{"\[Phi]", ".", "\[Phi]"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[794]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{
   RowBox[{"s", "[", "\[Phi]", "]"}], ",", 
   RowBox[{"QuantumField", "[", "\[Phi]", "]"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[795]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["m", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"-", "\[Phi]"}], ")"}]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["\[Phi]", "\[Null]", "\[Null]"]}]}], 
  TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[795]="]
}, Open  ]],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"S", "[", "A", "]"}], " ", "=", " ", 
   RowBox[{"-", 
    RowBox[{"\[Integral]", " ", 
     RowBox[{
      SuperscriptBox["d", "D"], "x", " ", 
      FractionBox["1", "4"], " ", 
      RowBox[{
       SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], "(", "x", ")"}], " ", 
      RowBox[{
       SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}]}]}]}], 
  TraditionalForm]]]], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[TextData[StyleBox["First approach",
 FontWeight->"Bold"]], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F1", "=", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
    RowBox[{"{", 
     RowBox[{"A", ",", "b", ",", "c"}], "}"}], ",", "1", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[796]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "c"}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[796]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"F2", "=", 
  RowBox[{"FieldStrength", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "a", ",", 
    RowBox[{"{", 
     RowBox[{"A", ",", "d", ",", "e"}], "}"}], ",", "1", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[797]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "d", "e"}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[797]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"S", "[", "A", "]"}], "=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"1", "/", "4"}]}], 
   RowBox[{"F1", ".", "F2"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[798]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "b", "c"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}]}], "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "d", "e"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]]}]}], "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[798]="]
}, Open  ]],

Cell[TextData[{
 "In order to derive the equation of motion the functional derivative of ",
 Cell[BoxData[
  FormBox["S", TraditionalForm]]],
 " with respect to ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["A", "\[Sigma]", "g"], TraditionalForm]]],
 "has to be set to zero. Bearing in mind that for FeynCalc we have to be \
precise as to where which operators (coming from the substitution of the \
derivative of the \[Delta] function) act:, act with the functional derivative \
operator on the first field strength:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     FormBox[
      RowBox[{
       FormBox[
        RowBox[{
         FormBox[
          RowBox[{"0", " ", "=", " ", 
           RowBox[{
            FormBox[
             RowBox[{
              RowBox[{"(", "\[Delta]S", ")"}], "/", 
              RowBox[{"(", 
               RowBox[{"\[Delta]", 
                RowBox[{
                 SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", ")"}]}], 
               ")"}]}],
             TraditionalForm], "=", 
            RowBox[{"-", 
             RowBox[{"2", "/", "4"}]}]}]}],
          TraditionalForm], 
         RowBox[{"\[Integral]", 
          RowBox[{
           SuperscriptBox["d", "D"], "x"}]}]}],
        TraditionalForm], " ", 
       RowBox[{"(", 
        RowBox[{"\[Delta]", "/", 
         RowBox[{"(", 
          RowBox[{"\[Delta]", 
           RowBox[{
            SubsuperscriptBox["A", "\[Sigma]", "g"], "(", "y", ")"}]}], 
          ")"}]}]}]}],
      TraditionalForm], " ", 
     RowBox[{
      SubscriptBox["F", "\[Mu]\[Nu]a"], "(", "x", ")"}]}], ")"}], 
   TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    FormBox[
     RowBox[{" ", 
      SubsuperscriptBox["F", "a", 
       RowBox[{",", "\[Mu]\[Nu]"}]]}],
     TraditionalForm], 
    RowBox[{"(", "x", ")"}]}], TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"FunctionalD"],

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{"See", " ", "what", " ", "happens", " ", "with", " ", "just", 
   RowBox[{
    FormBox[
     RowBox[{
      RowBox[{"(", 
       RowBox[{"\[Delta]", 
        RowBox[{"S", "[", "A", "]"}]}], ")"}], "/", 
      RowBox[{"(", 
       RowBox[{"\[Delta]", 
        SubsuperscriptBox["A", "\[Sigma]", "g"]}], ")"}]}],
     TraditionalForm], "."}]}], TraditionalForm]]]], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Ag", "=", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Sigma]", "}"}], ",", 
    RowBox[{"{", "g", "}"}]}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[799]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["A", 
   FormBox[
    FormBox["\[Sigma]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["g",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[799]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"FunctionalD", "[", 
   RowBox[{"F1", ",", "Ag"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[800]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], ")"}], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[800]="]
}, Open  ]],

Cell["\<\
In order to minimize the number of dummy indices, replace b \[Rule] c.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{"t1", " ", "/.", " ", 
   RowBox[{"b", "\[Rule]", "c"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[801]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]]}], ")"}], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[801]="]
}, Open  ]],

Cell[TextData[{
 "Instead of inserting the definition for the second ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["F", "a", "\[Mu]\[Nu]"], TraditionalForm]]],
 ", introduce a QuantumField object with antisymmetry built into the Lorentz \
indices:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{"F", "/:", " ", 
  RowBox[{"QuantumField", "[", 
   RowBox[{
   "pard___", ",", "F", ",", "\[Beta]_", ",", "\[Alpha]_", ",", "s_"}], "]"}],
   ":=", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"QuantumField", "[", 
     RowBox[{"pard", ",", "F", ",", "\[Alpha]", ",", "\[Beta]", ",", "s"}], 
     "]"}]}], "/;", 
   RowBox[{"!", 
    RowBox[{"OrderedQ", "[", 
     RowBox[{"{", 
      RowBox[{"\[Beta]", ",", "\[Alpha]"}], "}"}], "]"}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[802]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"F", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[803]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["F", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]}], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[803]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{"\[Mu]", "\[RuleDelayed]", "\[Nu]"}], ",", 
    RowBox[{"\[Nu]", "\[RuleDelayed]", "\[Mu]"}]}], "}"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[804]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubsuperscriptBox["F", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]}], 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[804]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{
   RowBox[{"Contract", "[", 
    RowBox[{"ExpandPartialD", "[", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"1", "/", "2"}]}], " ", 
      RowBox[{"t1", ".", 
       RowBox[{"QuantumField", "[", 
        RowBox[{"F", ",", 
         RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
         RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
         RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]}]}], "]"}], "]"}], "/.", 
   RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[805]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubsuperscriptBox["F", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["F", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["F", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]}], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[805]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", " ", "=", " ", 
  RowBox[{"t2", " ", "/.", " ", 
   RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[806]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[806]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{
   RowBox[{"FCE", "[", "t3", "]"}], "/.", " ", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "c", ",", "g"}], "]"}], "\[Rule]", 
    RowBox[{"-", 
     RowBox[{"SUNF", "[", 
      RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[807]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[807]="]
}, Open  ]],

Cell[TextData[{
 "Since the variational derivative vanishes t4 implies that 0 = ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["D", "\[Mu]"], 
    SubsuperscriptBox["F", "g", "\[Mu]\[Sigma]"]}], TraditionalForm]]],
 " ."
}], "Text",
 CellTags->"FunctionalD"],

Cell[TextData[StyleBox["Second approach",
 FontWeight->"Bold"]], "Text",
 CellTags->"FunctionalD"],

Cell["\<\
It is of course also possible to do the functional deriviate on the S[A] with \
both field strength tensors inserted.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"S", "[", "A", "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[808]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "b", "c"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}]}], "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["f", 
        RowBox[{"a", "d", "e"}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]]}]}], "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[808]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r1", "=", 
  RowBox[{"FunctionalD", "[", 
   RowBox[{
    RowBox[{"S", "[", "A", "]"}], ",", "Ag"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[809]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "4"], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]]}]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]]}]}]}], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]]}]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]]}], ".", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[809]="]
}, Open  ]],

Cell["\<\
This is just funcional derivation and partial integration and simple \
contraction of indices. No attempt is made to rename dummy indices (since \
this is difficult in general).\
\>", "Text",
 CellTags->"FunctionalD"],

Cell["\<\
With a general replacement rule only valid for commuting fields the color \
indices can be canonicalized a bit more. The idea is to use the commutative \
properties of the vector fields, and canonicalize the color indices by a \
trick. This function will work on any commuting product of fields.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "symfun", "]"}], ";"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[810]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"symfun", "[", 
   RowBox[{"z_", ",", "fieldname_Symbol"}], "]"}], ":=", 
  RowBox[{"Expand", "[", 
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"FixedPoint", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"Collect2", "[", 
        RowBox[{
         RowBox[{
          RowBox[{
           RowBox[{"DotSimplify", "[", 
            RowBox[{"#1", "/.", 
             RowBox[{"Times", "\[Rule]", "Dot"}]}], "]"}], "/.", 
           RowBox[{
            RowBox[{"(", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{
                RowBox[{"(", "qi___", ")"}], ".", 
                RowBox[{"QuantumField", "[", 
                 RowBox[{
                 "par1___", ",", "fieldname", ",", "li1_", ",", "sui1_"}], 
                 "]"}], ".", 
                RowBox[{"QuantumField", "[", 
                 RowBox[{
                 "par2___", ",", "fieldname", ",", "li2_", ",", "sui2_"}], 
                 "]"}], ".", "qf___"}], " ", ")"}], "any_"}], ")"}], 
            "\[RuleDelayed]", 
            RowBox[{
             RowBox[{"(", 
              RowBox[{
               RowBox[{"(", 
                RowBox[{"qi", ".", 
                 RowBox[{"QuantumField", "[", 
                  RowBox[{"par1", ",", "A", ",", "li1", ",", "sui2"}], "]"}], 
                 ".", 
                 RowBox[{"QuantumField", "[", 
                  RowBox[{"par2", ",", "fieldname", ",", "li2", ",", "sui1"}],
                   "]"}], ".", "qf"}], " ", ")"}], 
               RowBox[{"(", 
                RowBox[{"any", "/.", 
                 RowBox[{"{", 
                  RowBox[{
                   RowBox[{"sui1", "\[RuleDelayed]", "sui2"}], ",", 
                   RowBox[{"sui2", "\[RuleDelayed]", "sui1"}]}], "}"}]}], 
                ")"}]}], ")"}], "/;", 
             RowBox[{
              RowBox[{"(", 
               RowBox[{"!", 
                RowBox[{"(", 
                 RowBox[{"FreeQ2", "[", 
                  RowBox[{"any", ",", 
                   RowBox[{"{", 
                    RowBox[{"sui1", ",", "su2"}], "}"}]}], "]"}], ")"}]}], 
               ")"}], "&&", 
              RowBox[{"!", 
               RowBox[{"(", 
                RowBox[{"OrderedQ", "[", 
                 RowBox[{"{", 
                  RowBox[{"sui1", ",", "sui2"}], "}"}], "]"}], 
                ")"}]}]}]}]}]}], "/.", 
          RowBox[{"Dot", "\[Rule]", "Times"}]}], ",", "QuantumField"}], "]"}],
        "&"}], ",", "z", ",", "42"}], "]"}], "]"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[811]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r2", " ", "=", " ", 
  RowBox[{"symfun", "[", 
   RowBox[{"r1", ",", "A"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[812]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "4"], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm], 
       FormBox[
        SubscriptBox["\[PartialD]", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]],
        TraditionalForm]}], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[812]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r3", "=", 
  RowBox[{"r2", "/.", 
   RowBox[{"\[Nu]", "\[Rule]", "\[Mu]"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[813]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["3", "4"], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[813]="]
}, Open  ]],

Cell[TextData[{
 "Inspection reveals that still terms are the same. Gather the terms with two \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"f", " "}], TraditionalForm]]],
 "'s:"
}], "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twof", "=", 
  RowBox[{"Select", "[", 
   RowBox[{"r3", ",", 
    RowBox[{
     RowBox[{
      RowBox[{"Count", "[", 
       RowBox[{"#", ",", 
        RowBox[{"SUNF", "[", "__", "]"}]}], "]"}], "===", "2"}], "&"}]}], 
   "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[814]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["3", "4"], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]]}]}], "-", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[814]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"twofnew", "=", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"twof", "\[LeftDoubleBracket]", "1", "\[RightDoubleBracket]"}], 
     "+", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"twof", "[", 
        RowBox[{"[", "2", "]"}], "]"}], "/.", 
       RowBox[{"{", 
        RowBox[{
         RowBox[{"b", "\[RuleDelayed]", "c"}], ",", 
         RowBox[{"c", "\[RuleDelayed]", "b"}]}], "}"}]}], ")"}]}], ")"}], "/.", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"a", "\[RuleDelayed]", "c"}], ",", 
     RowBox[{"c", "\[RuleDelayed]", "a"}]}], "}"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[815]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["e",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[815]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"r4", " ", "=", " ", 
  RowBox[{"r3", "-", "twof", "+", "twofnew"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[816]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]]}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[816]="]
}, Open  ]],

Cell["\<\
Check that this is now indeed the same as the t4 result from the first \
attempt.\
\>", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData["t4"], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[817]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["F", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]}], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[817]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w0", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"FieldStrength", "[", 
     RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "g", ",", 
      RowBox[{"{", 
       RowBox[{"A", ",", "a", ",", "b"}], "}"}], ",", "1"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"QuantumField", "[", 
     RowBox[{"A", ",", 
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
      RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}], 
    RowBox[{"FieldStrength", "[", 
     RowBox[{"\[Mu]", ",", "\[Sigma]", ",", "a", ",", 
      RowBox[{"{", 
       RowBox[{"A", ",", "b", ",", "d"}], "}"}], ",", "1"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"g", ",", "c", ",", "a"}], "]"}]}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[818]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{"g", "c", "a"}]], " ", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Mu]", "\[Sigma]"}], 
     RowBox[{"a", 
      FormBox[
       RowBox[{"{", 
        RowBox[{"A", ",", "b", ",", "d"}], "}"}],
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Mu]", "\[Sigma]"}], 
     RowBox[{"g", 
      FormBox[
       RowBox[{"{", 
        RowBox[{"A", ",", "a", ",", "b"}], "}"}],
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[818]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w1", "=", 
  RowBox[{"Explicit", "[", "w0", "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[819]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[RightArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["g",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}]}], "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["g",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["g",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubsuperscriptBox["A", 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]]}]}], "+", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[819]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"w2", "=", 
  RowBox[{
   RowBox[{"ExpandPartialD", "[", "w1", "]"}], "/.", 
   RowBox[{"Dot", "\[Rule]", "Times"}]}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[820]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[820]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dif1", " ", "=", " ", 
  RowBox[{"w2", "-", "r4"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[821]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["g",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox[
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]],
       TraditionalForm], 
      FormBox[
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]],
       TraditionalForm]}], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]]}], ")"}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[821]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"dif2", "=", 
  RowBox[{"symfun", "[", 
   RowBox[{"dif1", ",", "A"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[822]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "+", 
   RowBox[{"2", " ", 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]]}]}], "-", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox[
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]],
       TraditionalForm], 
      FormBox[
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]],
       TraditionalForm]}], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["g",
        TraditionalForm],
       TraditionalForm]]}], ")"}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Sigma]",
         TraditionalForm],
        TraditionalForm]],
      TraditionalForm]}], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["g",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[822]="]
}, Open  ]],

Cell["quod erat demonstrandum.", "Text",
 CellTags->"FunctionalD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Unset", "[", 
   RowBox[{"s", "[", "\[Phi]", "]"}], "]"}], ";", 
  RowBox[{"Unset", "[", 
   RowBox[{"S", "[", "A", "]"}], "]"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{
   "Ag", ",", "F1", ",", "F2", ",", "t1", ",", "t2", ",", "t3", ",", "t4", 
    ",", "F", ",", "r1", ",", "r2", ",", "r3", ",", "r4", ",", "symfun", ",", 
    "twof", ",", "twofnew", ",", "w0", ",", "w1", ",", "w2", ",", "dif1", ",",
     "dif2", ",", "dif3"}], "]"}]}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[823]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples of funcional differentiation as used in FeynRule", \
"Subsubsection",
 CellTags->"FunctionalD"],

Cell["This is a part of the QCD Lagrangian.", "Text",
 CellTags->"FunctionalD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"(", 
   RowBox[{"Gstrong", "*", 
    RowBox[{
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", " ", 
       RowBox[{"LorentzIndex", "[", "li1", "]"}], ",", "  ", 
       RowBox[{"SUNIndex", "[", "si2", "]"}]}], "]"}], ".", "  ", 
     RowBox[{"QuantumField", "[", 
      RowBox[{"GaugeField", ",", " ", 
       RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", "   ", 
       RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}], " ", ".", "  ", 
     RowBox[{"QuantumField", "[", 
      RowBox[{
       RowBox[{"PartialD", "[", 
        RowBox[{"LorentzIndex", "[", "li1", "]"}], "]"}], ",", " ", 
       "GaugeField", ",", "   ", 
       RowBox[{"LorentzIndex", "[", "li2", "]"}], ",", " ", 
       RowBox[{"SUNIndex", "[", "si1", "]"}]}], "]"}]}], "*", "  ", 
    RowBox[{"SUNF", "[", 
     RowBox[{
      RowBox[{"SUNIndex", "[", "si1", "]"}], ",", " ", 
      RowBox[{"SUNIndex", "[", "si2", "]"}], ",", " ", 
      RowBox[{"SUNIndex", "[", "si4", "]"}]}], "]"}]}], ")"}], "/", 
  "4"}]], "Input",
 CellLabel->"In[824]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["si1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["li1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubsuperscriptBox[
     FormBox["A",
      TraditionalForm], 
     FormBox[
      FormBox["li2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["A",
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["si1",
        TraditionalForm],
       TraditionalForm]]}]}]}], TraditionalForm]], "Output",
 CellLabel->"Out[824]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FunctionalD", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Mu]", "}"}], ",", 
        RowBox[{"{", "a", "}"}]}], "]"}], "[", "p", "]"}], ",", " ", 
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Nu]", "}"}], ",", 
        RowBox[{"{", "b", "}"}]}], "]"}], "[", "q", "]"}], ",", " ", 
     RowBox[{
      RowBox[{"QuantumField", "[", 
       RowBox[{"GaugeField", ",", 
        RowBox[{"{", "\[Rho]", "}"}], ",", 
        RowBox[{"{", "c", "}"}]}], "]"}], "[", "r", "]"}]}], "}"}]}], 
  "]"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[825]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["si1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["si4",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Rho]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["b",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si2",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li1", "\[Nu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["a",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si4",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Mu]"}]]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", "\[ImaginaryI]"}], " ", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["si1",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SuperscriptBox["g", 
         RowBox[{"li2", "\[Rho]"}]]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"FunctionalD",
 CellLabel->"Out[825]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Calc", "[", 
   RowBox[{"%", "//", "Calc"}], "]"}], "//", "Factor"}]], "Input",
 CellTags->"FunctionalD",
 CellLabel->"In[826]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
     RowBox[{
      SuperscriptBox[
       FormBox["r",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["r",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"FunctionalD",
 CellLabel->"Out[826]="]
}, Open  ]]
}, Open  ]],

Cell["FV", "ObjectName",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FV"],

Cell[TextData[{
 "FV[",
 StyleBox["p",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the four-dimensional vector ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
 ". "
}], "Text",
 CellTags->"FV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FVD", All, CellTags],
  ButtonData->"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourVector", All, CellTags],
  ButtonData->"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FV"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[827]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FV",
 CellLabel->"Out[827]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[828]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FV",
 CellLabel->"Out[828]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[829]:="],

Cell[BoxData[
 RowBox[{"FV", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FV",
 CellLabel->"Out[829]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[830]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellTags->"FV",
 CellLabel->"Out[830]//StandardForm="]
}, Open  ]],

Cell["\<\
There is no special function to expand momenta in FV; ExpandScalarProduct \
does the job.\
\>", "Text",
 CellTags->"FV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[831]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FV",
 CellLabel->"Out[831]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"FV",
 CellLabel->"In[832]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "-", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]], "Output",
 CellTags->"FV",
 CellLabel->"Out[832]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["FVD", "ObjectName",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FVD"],

Cell[TextData[{
 "FVD[",
 StyleBox["p",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] is the D-dimensional vector ",
 StyleBox["p",
  FontSlant->"Italic"],
 " with Lorentz index ",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "."
}], "Text",
 CellTags->"FVD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FV", All, CellTags],
  ButtonData->"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourVector", All, CellTags],
  ButtonData->"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"FVD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[833]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["p", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[833]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[834]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[834]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FVD", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[835]:="],

Cell[BoxData[
 RowBox[{"FVD", "[", 
  RowBox[{"p", ",", "\[Mu]"}], "]"}]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[835]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"FVD", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[836]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[836]//StandardForm="]
}, Open  ]],

Cell["There is no special function to expand momenta in FVD.", "Text",
 CellTags->"FVD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"FVD", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[837]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[837]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"FVD",
 CellLabel->"In[838]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"p", ",", "D"}], "]"}]}], "]"}], "-", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"q", ",", "D"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"FVD",
 CellLabel->"Out[838]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GA", "ObjectName",
 CellTags->"GA"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GA"],

Cell[TextData[{
 "GA[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] can be used as input for a 4-dimensional ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "and is transformed into DiracGamma[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] by FeynCalcInternal (=FCI).GA[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", "\[Nu]", ","}], TraditionalForm]]],
 " ...] is a short form for GA[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "].GA[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "]. ... ."
}], "Text",
 CellTags->"GA"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GAD", All, CellTags],
  ButtonData->"GAD",
  ButtonNote->"GAD"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GS", All, CellTags],
  ButtonData->"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GA"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GA"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[839]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[839]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"GA",
 CellLabel->"In[840]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[840]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", 
   RowBox[{"GA", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[841]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellTags->"GA",
 CellLabel->"Out[841]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[842]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[842]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"GA",
 CellLabel->"In[843]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Rho]", "]"}], ".", 
  RowBox[{"GA", "[", "\[Sigma]", "]"}]}]], "Output",
 CellTags->"GA",
 CellLabel->"Out[843]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GA", "[", "\[Beta]", "]"}]}]], "Input",
 CellTags->"GA",
 CellLabel->"In[844]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
 CellTags->"GA",
 CellLabel->"Out[844]="]
}, Open  ]]
}, Open  ]],

Cell["GAD", "ObjectName",
 CellTags->"GAD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GAD"],

Cell[TextData[{
 "GAD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] can be used as input for a D-dimensional ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", 
    RowBox[{"\[Mu]", " "}]], TraditionalForm]]],
 "and is transformed into DiracGamma[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 ",D],D] by FeynCalcInternal (=FCI).GAD[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", " ", "..."}], TraditionalForm]]],
 "] is a short form for GAD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "].GAD[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "]. ... ."
}], "Text",
 CellTags->"GAD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracMatrix",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracMatrix", All, CellTags],
  ButtonData->"DiracMatrix",
  ButtonNote->"DiracMatrix"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GA", All, CellTags],
  ButtonData->"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GS", All, CellTags],
  ButtonData->"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GAD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GAD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[845]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[845]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "-", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Nu]", ",", "\[Mu]"}], "]"}]}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[846]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}], "-", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[846]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[847]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[847]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GAD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[848]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Rho]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Sigma]"]}], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[848]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"GAD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], "]"}], 
  "]"}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[849]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Rho]", "]"}], ".", 
  RowBox[{"GAD", "[", "\[Sigma]", "]"}]}]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[849]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GAD", "[", "\[Alpha]", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GAD", "[", "\[Beta]", "]"}]}]], "Input",
 CellTags->"GAD",
 CellLabel->"In[850]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Alpha]"], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   SuperscriptBox["\[Gamma]", "\[Beta]"]}], TraditionalForm]], "Output",
 CellTags->"GAD",
 CellLabel->"Out[850]="]
}, Open  ]]
}, Open  ]],

Cell["GammaEpsilon", "ObjectName",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GammaEpsilon"],

Cell["\<\
GammaEpsilon[exp] gives a series expansion of Gamma[exp] in Epsilon up to \
order 6 (where EulerGamma is neglected).\
\>", "Text",
 CellTags->"GammaEpsilon"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GammaExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GammaExpand", All, CellTags],
  ButtonData->"GammaExpand",
  ButtonNote->"GammaExpand"],
 ", ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Series2", All, CellTags],
  ButtonData->"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"GammaEpsilon"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GammaEpsilon"],

Cell["\<\
If the argument is of the form (1+a Epsilon) the result is not calculated but \
tabulated.\
\>", "Text",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", 
  RowBox[{"1", "+", 
   RowBox[{"a", " ", "Epsilon"}]}], "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[851]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "5"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["a", "5"], " ", 
         TemplateBox[{"5"},
          "Zeta"]}], "5"]}], "-", 
      RowBox[{
       FractionBox["1", "36"], " ", 
       SuperscriptBox["\[Pi]", "2"], " ", 
       SuperscriptBox["a", "5"], " ", 
       TemplateBox[{"3"},
        "Zeta"]}]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "160"], " ", 
    SuperscriptBox["\[Pi]", "4"], " ", 
    SuperscriptBox["a", "4"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "4"]}], "-", 
   RowBox[{
    FractionBox["1", "3"], " ", 
    SuperscriptBox["a", "3"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"3"},
     "Zeta"]}], "+", 
   RowBox[{
    FractionBox["1", "12"], " ", 
    SuperscriptBox["\[Pi]", "2"], " ", 
    SuperscriptBox["a", "2"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"]}], "+", 
   RowBox[{"C$7569", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[851]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", 
  RowBox[{"1", "-", 
   RowBox[{"Epsilon", "/", "2"}]}], "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[852]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"C$7589", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "4"]}], "2560"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"]}], "48"], "+", 
   RowBox[{
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "5"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        SuperscriptBox["\[Pi]", "2"], " ", 
        TemplateBox[{"3"},
         "Zeta"]}], "1152"], "+", 
      FractionBox[
       TemplateBox[{"5"},
        "Zeta"], "160"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"], " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "24"], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[852]="]
}, Open  ]],

Cell["For other arguments the expansion is calculated.", "Text",
 CellTags->"GammaEpsilon"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", "Epsilon", "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[853]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"C$8683", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "6"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "4"], " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"]}], "160"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}], "12"], "+", 
   FractionBox["1", 
    FormBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm],
     TraditionalForm]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "5"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"61", " ", 
        SuperscriptBox["\[Pi]", "6"]}], "+", 
       RowBox[{"1680", " ", 
        SuperscriptBox[
         TemplateBox[{"2","1"},
          "PolyGamma2"], "2"]}]}], ")"}]}], "120960"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     TemplateBox[{"2","1"},
      "PolyGamma2"]}], "6"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "6"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "2016"}], " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        TemplateBox[{"5"},
         "Zeta"]}], "+", 
       RowBox[{"126", " ", 
        SuperscriptBox["\[Pi]", "4"], " ", 
        TemplateBox[{"2","1"},
         "PolyGamma2"]}], "+", 
       RowBox[{"24", " ", 
        TemplateBox[{"6","1"},
         "PolyGamma2"]}]}], ")"}]}], "120960"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "4"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"20", " ", 
        SuperscriptBox["\[Pi]", "2"], " ", 
        TemplateBox[{"2","1"},
         "PolyGamma2"]}], "-", 
       RowBox[{"288", " ", 
        TemplateBox[{"5"},
         "Zeta"]}]}], ")"}]}], "1440"]}], TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[853]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaEpsilon", "[", "x", "]"}]], "Input",
 CellTags->"GammaEpsilon",
 CellLabel->"In[854]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"x"},
   "Gamma"], TraditionalForm]], "Output",
 CellTags->"GammaEpsilon",
 CellLabel->"Out[854]="]
}, Open  ]]
}, Open  ]],

Cell["GammaExpand", "ObjectName",
 CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GammaExpand"],

Cell["\<\
GammaExpand[exp] rewrites Gamma[n + m] in exp (where n has Head Integer).\
\>", "Text",
 CellTags->"GammaExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GammaEpsilon",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GammaEpsilon", All, CellTags],
  ButtonData->"GammaEpsilon",
  ButtonNote->"GammaEpsilon"],
 "."
}], "Text",
 CellTags->"GammaExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GammaExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{"2", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[855]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "+", "1"}], ")"}], " ", 
   TemplateBox[{RowBox[{
       TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
    "Gamma"]}], TraditionalForm]], "Output",
 CellTags->"GammaExpand",
 CellLabel->"Out[855]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{
    RowBox[{"-", "3"}], "+", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[856]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   TemplateBox[{RowBox[{
       TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
    "Gamma"], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "3"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "2"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "-", "1"}], ")"}], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"GammaExpand",
 CellLabel->"Out[856]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GammaExpand", "[", 
  RowBox[{"Gamma", "[", 
   RowBox[{"1", " ", "+", " ", "Epsilon"}], "]"}], "]"}]], "Input",
 CellTags->"GammaExpand",
 CellLabel->"In[857]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{RowBox[{
      TagBox["\[CurlyEpsilon]", TraditionalForm], "+", "1"}]},
   "Gamma"], TraditionalForm]], "Output",
 CellTags->"GammaExpand",
 CellLabel->"Out[857]="]
}, Open  ]]
}, Open  ]],

Cell["Gamma1", "ObjectName",
 CellTags->"Gamma1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma1"],

Cell["\<\
Gamma1[al,ga, be,de] is a special product of Gamma functions expanded up to \
order Epsilon^2.\
\>", "Text",
 CellTags->"Gamma1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gamma2", All, CellTags],
  ButtonData->"Gamma2",
  ButtonNote->"Gamma2"],
 ", ",
 ButtonBox["Gamma3",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gamma3", All, CellTags],
  ButtonData->"Gamma3",
  ButtonNote->"Gamma3"],
 "."
}], "Text",
 CellTags->"Gamma1"]
}, Open  ]],

Cell["Gamma2", "ObjectName",
 CellTags->"Gamma2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma2"],

Cell["\<\
Gamma2[x,y] is a special product of Gamma functions expanded up to order \
Epsilon^3 when positive integer arguments are given.\
\>", "Text",
 CellTags->"Gamma2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gamma1", All, CellTags],
  ButtonData->"Gamma1",
  ButtonNote->"Gamma1"],
 ", ",
 ButtonBox["Gamma3",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gamma3", All, CellTags],
  ButtonData->"Gamma3",
  ButtonNote->"Gamma3"],
 "."
}], "Text",
 CellTags->"Gamma2"]
}, Open  ]],

Cell["Gamma3", "ObjectName",
 CellTags->"Gamma3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gamma3"],

Cell["\<\
Gamma3[al,be,ga,ep] is a special product of Gamma functions expanded up to \
order Epsilon^n when positive integer arguments are given (the order n is \
determined by the option EpsilonOrder).\
\>", "Text",
 CellTags->"Gamma3"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gamma1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gamma1", All, CellTags],
  ButtonData->"Gamma1",
  ButtonNote->"Gamma1"],
 ", ",
 ButtonBox["Gamma2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gamma2", All, CellTags],
  ButtonData->"Gamma2",
  ButtonNote->"Gamma2"],
 "."
}], "Text",
 CellTags->"Gamma3"]
}, Open  ]],

Cell["Gauge", "ObjectName",
 CellTags->"Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gauge"],

Cell["\<\
Gauge is an option for GluonProgagator. If set to 1 the 't Hooft Feynman \
gauge is used.\
\>", "Text",
 CellTags->"Gauge"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$Gauge",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$Gauge", All, CellTags],
  ButtonData->"$Gauge",
  ButtonNote->"$Gauge"],
 ", ",
 ButtonBox["GluonProgagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonProgagator", All, CellTags],
  ButtonData->"GluonProgagator",
  ButtonNote->"GluonProgagator"],
 "."
}], "Text",
 CellTags->"Gauge"]
}, Open  ]],

Cell["GaugeField", "ObjectName",
 CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GaugeField"],

Cell["\<\
GaugeField is just a name. No functional properties are associated with it. \
GaugeField is used as default setting for the option QuantumField of \
FieldStrength.\
\>", "Text",
 CellTags->"GaugeField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FieldStrength", All, CellTags],
  ButtonData->"FieldStrength",
  ButtonNote->"FieldStrength"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuantumField", All, CellTags],
  ButtonData->"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"GaugeField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GaugeField"],

Cell[CellGroupData[{

Cell[BoxData["GaugeField"], "Input",
 CellTags->"GaugeField",
 CellLabel->"In[858]:="],

Cell[BoxData[
 FormBox["A", TraditionalForm]], "Output",
 CellTags->"GaugeField",
 CellLabel->"Out[858]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
 CellTags->"GaugeField",
 CellLabel->"In[859]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"GaugeField",
 CellLabel->"Out[859]="]
}, Open  ]]
}, Open  ]],

Cell["GaugeXi", "ObjectName",
 CellTags->"GaugeXi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GaugeXi"],

Cell["GaugeXi is a head for gauge parameters.", "Text",
 CellTags->"GaugeXi"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gauge",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gauge", All, CellTags],
  ButtonData->"Gauge",
  ButtonNote->"Gauge"],
 ", ",
 ButtonBox["GaugeField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GaugeField", All, CellTags],
  ButtonData->"GaugeField",
  ButtonNote->"GaugeField"],
 "."
}], "Text",
 CellTags->"GaugeXi"]
}, Open  ]],

Cell["GA5", "ObjectName",
 CellTags->"GA5"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GA5"],

Cell["GA5 is equivalent to DiracGamma[5] and denotes gamma5.", "Text",
 CellTags->"GA5"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GA", All, CellTags],
  ButtonData->"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GS", All, CellTags],
  ButtonData->"GS",
  ButtonNote->"GS"],
 "."
}], "Text",
 CellTags->"GA5"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GA5"],

Cell[CellGroupData[{

Cell[BoxData["GA5"], "Input",
 CellTags->"GA5",
 CellLabel->"In[860]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "5"], TraditionalForm]], "Output",
 CellTags->"GA5",
 CellLabel->"Out[860]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Tr"}]], "Input",
 CellTags->"GA5",
 CellLabel->"In[861]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"GA5",
 CellLabel->"Out[861]="]
}, Open  ]]
}, Open  ]],

Cell["GGV", "ObjectName",
 CellTags->"GGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GGV"],

Cell["GGV is equivalent to GluonGhostVertex.", "Text",
 CellTags->"GGV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonGhostVertex", All, CellTags],
  ButtonData->"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GGV"]
}, Open  ]],

Cell["GhostPropagator", "ObjectName",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GhostPropagator"],

Cell[TextData[{
 "GhostPropagator[p, a, b] gives the ghost propagator where \"a\" and \"b\" \
are the color indices. GhostPropagator[p] omits the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"GhostPropagator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GHP",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GHP", All, CellTags],
  ButtonData->"GHP",
  ButtonNote->"GHP"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonGhostVertex", All, CellTags],
  ButtonData->"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GhostPropagator", "]"}]], "Input",
 CellLabel->"In[862]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[862]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GhostPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", 
  RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[863]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[863]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", 
  RowBox[{"p", ",", "a", ",", "b", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[864]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[864]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GhostPropagator", "[", "p", "]"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[865]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "u"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[865]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GhostPropagator", "[", 
   RowBox[{"p", ",", "1", ",", "2"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[866]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["ci1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ci2",
        TraditionalForm],
       TraditionalForm]}]]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[866]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"GhostPropagator", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], "//",
    " ", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"GhostPropagator",
 CellLabel->"In[867]:="],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellTags->"GhostPropagator",
 CellLabel->"Out[867]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GHP", "ObjectName",
 CellTags->"GHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GHP"],

Cell[TextData[{
 "GHP[p, a, b] gives the ghost propagator where \"a\" and \"b\" are the color \
indices. GHP[p] omits the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[Delta]", 
     RowBox[{"a", "\[NoBreak]", "b"}]], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"GHP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GhostPropagator", All, CellTags],
  ButtonData->"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonGhostVertex", All, CellTags],
  ButtonData->"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GHP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GHP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GHP", "[", 
  RowBox[{"p", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[868]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[868]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GHP", "[", "p", "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[869]:="],

Cell[BoxData[
 FormBox[
  FractionBox["\[ImaginaryI]", 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[869]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GHP", "[", 
  RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[870]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", 
    RowBox[{"ci1", "ci2"}]], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[870]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{
    RowBox[{"GHP", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], 
   "]"}], "]"}]], "Input",
 CellTags->"GHP",
 CellLabel->"In[871]:="],

Cell[BoxData[
 RowBox[{"\[ImaginaryI]", " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellTags->"GHP",
 CellLabel->"Out[871]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["GluonField", "ObjectName",
 CellTags->"GluonField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonField"],

Cell["GluonField is a name of a gauge field.", "Text",
 CellTags->"GluonField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GaugeField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GaugeField", All, CellTags],
  ButtonData->"GaugeField",
  ButtonNote->"GaugeField"],
 "."
}], "Text",
 CellTags->"GluonField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonField"],

Cell[CellGroupData[{

Cell[BoxData["GluonField"], "Input",
 CellTags->"GluonField",
 CellLabel->"In[872]:="],

Cell[BoxData[
 FormBox["A", TraditionalForm]], "Output",
 CellTags->"GluonField",
 CellLabel->"Out[872]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GluonField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Input",
 CellTags->"GluonField",
 CellLabel->"In[873]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"GluonField",
 CellLabel->"Out[873]="]
}, Open  ]]
}, Open  ]],

Cell["GluonGhostVertex", "ObjectName",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonGhostVertex"],

Cell["\<\
GluonGhostVertex[{p,mu,a}, {q,nu,b}, {k,rho,c}] or GluonGhostVertex[ p,mu,a , \
q,nu,b , k,rho,c ] yields the Gluon-Ghost vertex. The first argument \
represents the gluon and the third argument the outgoing ghost field (but \
incoming four-momentum). The dimension and the name of the coupling constant \
are determined by the options Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"GluonGhostVertex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonSelfEnergy", All, CellTags],
  ButtonData->"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GhostPropagator", All, CellTags],
  ButtonData->"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonGhostVertex", "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[874]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[874]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonGhostVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[875]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", "k", ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[875]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[876]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[876]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input",
 CellLabel->"In[877]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    OverscriptBox["\[CapitalLambda]", "~"], "\[Mu]"], "(", "k", ")"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[877]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", "%", "]"}]], "Input",
 CellLabel->"In[878]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SubscriptBox["g", "s"]}], " ", 
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellLabel->"Out[878]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "Explicit"}], "//", "StandardForm"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[879]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "Gstrong"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"k", ",", "D"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[879]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonGhostVertex", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "k", ",", "3"}], 
  "]"}]], "Input",
 CellTags->"GluonGhostVertex",
 CellLabel->"In[880]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     OverscriptBox["\[CapitalLambda]", "~"], "li1"], "(", "k", ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"GluonGhostVertex",
 CellLabel->"Out[880]="]
}, Open  ]]
}, Open  ]],

Cell["GluonPropagator", "ObjectName",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonPropagator"],

Cell[TextData[{
 "GluonPropagator[p, {mu, a}, {nu, b}] or GluonPropagator[p, mu, a , nu, b ] \
yields the gluon propagator. GluonPropagator[p, {mu}, {nu}] or \
GluonPropagator[p, mu, nu] omits the SUNDelta. The gauge and the dimension is \
determined by the option Gauge and Dimension. The following settings of Gauge \
are possible: 1 for the Feynman gauge; ",
 Cell[BoxData[
  FormBox["\[Alpha]", TraditionalForm]]],
 " for the general covariant gauge; {Momentum[n] ,1} for the axial gauge."
}], "Text",
 CellTags->"GluonPropagator"],

Cell["GP can be used as an abbreviation of GluonPropagator.", "Text",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData["GP"], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[881]:="],

Cell[BoxData[
 FormBox["GluonPropagator", TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[881]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonPropagator", "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[882]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[882]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonSelfEnergy", All, CellTags],
  ButtonData->"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GhostPropagator", All, CellTags],
  ButtonData->"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonGhostVertex", All, CellTags],
  ButtonData->"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GluonPropagator"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonPropagator", "[", 
  RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], 
  "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[883]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CapitalPi]", 
    RowBox[{"a", "b"}], 
    RowBox[{"\[Mu]", "\[Nu]"}]], "(", "p", ")"}], TraditionalForm]], "Output",\

 CellTags->"GluonPropagator",
 CellLabel->"Out[883]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b"}], "]"}], "//",
   "Explicit"}]], "Input",
 CellLabel->"In[884]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellLabel->"Out[884]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"Gauge", "\[Rule]", "\[Alpha]"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[885]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Alpha]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[885]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"Gauge", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"Momentum", "[", "n", "]"}], ",", "1"}], "}"}]}]}], "]"}], "//",
   "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[886]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], " ", 
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["n",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}],
         TraditionalForm], "2"]], "+", 
      FractionBox[
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["n",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], 
       RowBox[{
        FormBox["n",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[886]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "\[Nu]"}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[887]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]]}], 
    SuperscriptBox["p", "2"]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[887]="]
}, Open  ]],

Cell["\<\
This is a convenient way to enter amplitudes by hand (GP is an abbreviation \
GluonPropagator).\
\>", "Text",
 CellTags->"GluonPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{
   RowBox[{"GP", "[", 
    RowBox[{"p", ",", "1", ",", "2"}], "]"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[888]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["ci1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ci2",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li1",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["li2",
           TraditionalForm],
          TraditionalForm]]}], 
       SuperscriptBox["p", "2"]], "-", 
      SuperscriptBox["g", 
       RowBox[{"li1", "li2"}]]}], ")"}]}], 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[888]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"GP", "[", 
     RowBox[{
      RowBox[{"-", "k"}], ",", "3", ",", "4"}], "]"}], "//", "Explicit"}], "//",
    "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[889]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", "\[ImaginaryI]"}], " ", 
  RowBox[{"FAD", "[", "k", "]"}], " ", 
  RowBox[{"MTD", "[", 
   RowBox[{"li3", ",", "li4"}], "]"}], " ", 
  RowBox[{"SD", "[", 
   RowBox[{"ci3", ",", "ci4"}], "]"}]}]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[889]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "1"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[890]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"11", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["19", "6"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[890]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "2"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[891]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         FractionBox["1", "6"], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], "2"], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]]}]}], "-", 
       RowBox[{
        FractionBox["1", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[891]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "3"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[892]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", 
     SubscriptBox["T", "f"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[892]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "4"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[893]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"10", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["10", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[893]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonPropagator", "[", 
   RowBox[{"p", ",", "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", 
    RowBox[{"CounterTerm", "\[Rule]", " ", "5"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonPropagator",
 CellLabel->"In[894]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"10", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"], "-", 
       RowBox[{
        FractionBox["10", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", 
     SubscriptBox["T", "f"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], "3"]}], ")"}]}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"GluonPropagator",
 CellLabel->"Out[894]="]
}, Open  ]]
}, Open  ]],

Cell["GluonSelfEnergy", "ObjectName",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonSelfEnergy"],

Cell["\<\
GluonSelfEnergy[{mu, a}, {nu,b}] yields the 1-loop Gluon selfenergy.\
\>", "Text",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonSelfEnergy", "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[895]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"log", "(", 
         RowBox[{"_", " ", 
          SuperscriptBox["\[Mu]", "2"]}], ")"}], "\[RuleDelayed]", "0"}], ",", 
       RowBox[{
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]], "\[RuleDelayed]", 
        RowBox[{"log", "(", 
         RowBox[{"4", " ", "\[Pi]"}], ")"}]}]}], "}"}]}], ",", 
    RowBox[{"Gauge", "\[Rule]", "1"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[895]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 ", ",
 ButtonBox["GhostPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GhostPropagator", All, CellTags],
  ButtonData->"GhostPropagator",
  ButtonNote->"GhostPropagator"],
 ", ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonGhostVertex", All, CellTags],
  ButtonData->"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 "."
}], "Text",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"GluonSelfEnergy"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonSelfEnergy", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[896]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]], "-", 
      FractionBox["62", "9"]}], ")"}], " ", 
    SubscriptBox["C", "A"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", "9"], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]]}], ")"}], " ", 
    SubscriptBox["T", "f"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[896]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonSelfEnergy", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Gauge", "\[Rule]", "\[Xi]"}]}], "]"}]], "Input",
 CellTags->"GluonSelfEnergy",
 CellLabel->"In[897]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"2", " ", 
        RowBox[{"(", 
         RowBox[{
          FractionBox["13", "3"], "-", "\[Xi]"}], ")"}]}], 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]], "-", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"1", "-", "\[Xi]"}], ")"}], "2"]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "\[Xi]"}], ")"}]}], "-", 
      FractionBox["62", "9"]}], ")"}], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], "+", 
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["20", "9"], "-", 
      FractionBox["8", 
       RowBox[{"3", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}]]}], ")"}], " ", 
    SubscriptBox["T", "f"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"GluonSelfEnergy",
 CellLabel->"Out[897]="]
}, Open  ]]
}, Open  ]],

Cell["GluonVertex", "ObjectName",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GluonVertex"],

Cell["Note: All momenta are flowing into the vertex.", "Text",
 CellTags->"GluonVertex"],

Cell["\<\
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}] or GluonVertex[p,mu,a , q,nu,b , \
k,la,c ] yields the 3-gluon vertex. GluonVertex[{p,mu}, {q,nu}, {k,la}] \
yields the 3-gluon vertex without color structure and the coupling constant. \
GluonVertex[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or GluonVertex[{mu,a}, \
{nu,b}, {la,c}, {si,d}] or GluonVertex[p,mu,a , q,nu,b , k,la,c , s,si,d] or \
GluonVertex[ mu,a , nu,b , la,c , si,d ] yields the 4-gluon vertex. The \
dimension and the name of the coupling constant are determined by the options \
Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"GluonVertex"],

Cell["GV can be used as an abbreviation of GluonVertex.", "Text",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData["GV"], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[898]:="],

Cell[BoxData[
 FormBox["GluonVertex", TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[898]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "GluonVertex", "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[899]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[899]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonGhostVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonGhostVertex", All, CellTags],
  ButtonData->"GluonGhostVertex",
  ButtonNote->"GluonGhostVertex"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 "."
}], "Text",
 CellTags->"GluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[900]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[900]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[901]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"GluonVertex",
 CellLabel->"Out[901]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[902]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["V", 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]"}]], "(", 
   RowBox[{"p", 
    RowBox[{",", " "}], "q", 
    RowBox[{",", " "}], "r"}], ")"}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[902]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[903]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], "-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"GluonVertex",
 CellLabel->"Out[903]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[904]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["V", 
    RowBox[{"a", "b", "c", "d"}], 
    RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], "(", 
   RowBox[{"p", 
    RowBox[{",", " "}], "q", 
    RowBox[{",", " "}], "r", 
    RowBox[{",", " "}], "s"}], ")"}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[904]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}], "//", 
  "Explicit"}]], "Input",
 CellLabel->"In[905]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u132",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u132",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u132",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u132",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u132",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u132",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[905]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[906]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["W", 
   RowBox[{"a", "b", "c", "d"}]], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[906]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GluonVertex", "[", 
   RowBox[{
   "\[Mu]", ",", "a", ",", "\[Nu]", ",", "b", ",", "\[Rho]", ",", "c", ",", 
    "\[Sigma]", ",", "d"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[907]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u133",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u133",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u133",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u133",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Rho]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u133",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u133",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Sigma]"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[907]="]
}, Open  ]],

Cell["\<\
A very convenient way to enter diagrams by hand is to label each line hitting \
a vertex by a number and put this number after the inflowing momentum.\
\>", "Text",
 CellTags->"GluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GV", "[", 
  RowBox[{"p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3"}], 
  "]"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[908]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["ci1",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci2",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["ci3",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{
    SuperscriptBox["V", 
     RowBox[{
      FormBox[
       FormBox["li1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["li3",
        TraditionalForm],
       TraditionalForm]}]], "(", 
    RowBox[{
     FormBox["p",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["q",
      TraditionalForm], 
     RowBox[{",", " "}], 
     FormBox["r",
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[908]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GV", "[", 
   RowBox[{
   "p", ",", "1", ",", "q", ",", "2", ",", "r", ",", "3", ",", "s", ",", 
    "4"}], "]"}], "//", "Explicit"}]], "Input",
 CellTags->"GluonVertex",
 CellLabel->"In[909]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u134",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u134",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li2"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li3", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li3"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li4"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u134",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u134",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li2"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li3", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li4"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li3"}]]}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci1",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci2",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u134",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["ci3",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["ci4",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["u134",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li3"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li4"}]]}], "-", 
        RowBox[{
         SuperscriptBox["g", 
          RowBox[{"li1", "li4"}]], " ", 
         SuperscriptBox["g", 
          RowBox[{"li2", "li3"}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GluonVertex",
 CellLabel->"Out[909]="]
}, Open  ]]
}, Open  ]],

Cell["GO", "ObjectName",
 CellTags->"GO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GO"],

Cell["GO is equivalent to Twist2GluonOperator.", "Text",
 CellTags->"GO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2GluonOperator", All, CellTags],
  ButtonData->"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"GO"]
}, Open  ]],

Cell["GP", "ObjectName",
 CellTags->"GP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GP"],

Cell["GP is equivalent to GluonPropagator.", "Text",
 CellTags->"GP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 "."
}], "Text",
 CellTags->"GP"]
}, Open  ]],

Cell["GrassmannParity", "ObjectName",
 CellTags->"GrassmannParity"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GrassmannParity"],

Cell["\<\
GrassmannParity is a data type. E.g. DataType[F, GrassmannParity] = 1 \
declares F to be of bosonic type and DataType[F, GrassmannParity] = -1 of \
fermionic one.\
\>", "Text",
 CellTags->"GrassmannParity"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"GrassmannParity"]
}, Open  ]],

Cell["GS", "ObjectName",
 CellTags->"GS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GS"],

Cell[TextData[{
 "GS[p] can be used as input for a 4-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "p", " \[NegativeMediumSpace]", 
    "\[NegativeMediumSpace]\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
    "/", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"=", 
       RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], TraditionalForm]]],
 " and is transformed into DiracGamma[Momentum[p]] by FeynCalcInternal \
(=FCI). GS[p,q, ...] is a short form for GS[p].GS[q]. ... ."
}], "Text",
 CellTags->"GS"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSlash", All, CellTags],
  ButtonData->"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GA", All, CellTags],
  ButtonData->"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GAD", All, CellTags],
  ButtonData->"GAD",
  ButtonNote->"GAD"],
 "."
}], "Text",
 CellTags->"GS"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", "p", "]"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[910]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellTags->"GS",
 CellLabel->"Out[910]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GS", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[911]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{"Momentum", "[", "p", "]"}], "]"}]], "Output",
 CellTags->"GS",
 CellLabel->"Out[911]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GS", "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[912]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GS",
 CellLabel->"Out[912]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GS",
 CellLabel->"In[913]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", "p", "]"}], ".", 
  RowBox[{"GS", "[", "q", "]"}], ".", 
  RowBox[{"GS", "[", "r", "]"}], ".", 
  RowBox[{"GS", "[", "s", "]"}]}]], "Output",
 CellTags->"GS",
 CellLabel->"Out[913]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GS", "[", "q", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GS", "[", "q", "]"}]}]], "Input",
 CellTags->"GS",
 CellLabel->"In[914]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GS",
 CellLabel->"Out[914]="]
}, Open  ]]
}, Open  ]],

Cell["GSD", "ObjectName",
 CellTags->"GSD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GSD"],

Cell[TextData[{
 "GSD[p] can be used as input for a D-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "p", " \[NegativeMediumSpace]", 
    "\[NegativeMediumSpace]\[NegativeMediumSpace]", "\[NegativeMediumSpace]", 
    "/", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"=", 
       RowBox[{"\[Gamma]", ".", "p"}]}], " ", "=", " ", 
      RowBox[{
       SubscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["p", "\[Mu]"]}]}], ")"}]}], TraditionalForm]]],
 " and is transformed into DiracGamma[Momentum[p,D],D] by FeynCalcInternal \
(=FCI). GSD[p,q, ...] is a short form for GSD[p].GSD[q]. ... ."
}], "Text",
 CellTags->"GSD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["DiracSlash",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSlash", All, CellTags],
  ButtonData->"DiracSlash",
  ButtonNote->"DiracSlash"],
 ", ",
 ButtonBox["GA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GA", All, CellTags],
  ButtonData->"GA",
  ButtonNote->"GA"],
 ", ",
 ButtonBox["GAD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GAD", All, CellTags],
  ButtonData->"GAD",
  ButtonNote->"GAD"],
 "."
}], "Text",
 CellTags->"GSD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"GSD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GSD", "[", "p", "]"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[915]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], TraditionalForm]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[915]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"GSD", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[916]:="],

Cell[BoxData[
 RowBox[{"DiracGamma", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", "D"}], "]"}]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[916]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GSD", "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[917]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "p"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "r"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "s"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[917]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[918]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "p", "]"}], ".", 
  RowBox[{"GSD", "[", "q", "]"}], ".", 
  RowBox[{"GSD", "[", "r", "]"}], ".", 
  RowBox[{"GSD", "[", "s", "]"}]}]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[918]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "q", "]"}], ".", 
  RowBox[{"(", 
   RowBox[{
    RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
  RowBox[{"GSD", "[", "q", "]"}]}]], "Input",
 CellTags->"GSD",
 CellLabel->"In[919]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"m", "+", 
     RowBox[{"\[Gamma]", "\[CenterDot]", "p"}]}], ")"}], ".", 
   RowBox[{"(", 
    RowBox[{"\[Gamma]", "\[CenterDot]", "q"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"GSD",
 CellLabel->"Out[919]="]
}, Open  ]]
}, Open  ]],

Cell["Gstrong", "ObjectName",
 CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Gstrong"],

Cell["Gstrong denotes the strong coupling constant.", "Text",
 CellTags->"Gstrong"],

Cell[TextData[{
 "See also: ",
 ButtonBox["CovariantD",
  BaseStyle->"AddOnsLink",
  ButtonData:>"CovariantD"],
 ", ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonData:>"FieldStrength"],
 ", ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonData:>"GluonVertex"],
 "."
}], "Text",
 CellTags->"Gstrong"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Gstrong"],

Cell["\<\
Gstrong has no functional properties. Only a typesetting rule is defined.\
\>", "Text",
 CellTags->"Gstrong"],

Cell[CellGroupData[{

Cell[BoxData["Gstrong"], "Input",
 CellTags->"Gstrong",
 CellLabel->"In[920]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["g", "s"], TraditionalForm]], "Output",
 CellTags->"Gstrong",
 CellLabel->"Out[920]="]
}, Open  ]]
}, Open  ]],

Cell["GTI", "ObjectName",
 CellTags->"GTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GTI"],

Cell["GTI is like RHI, but with no functional properties.", "Text",
 CellTags->"GTI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"GTI"]
}, Open  ]],

Cell["GV", "ObjectName",
 CellTags->"GV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"GV"],

Cell["GV is equivalent to GluonVertex.", "Text",
 CellTags->"GV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"GV"]
}, Open  ]],

Cell["Hill", "ObjectName",
 CellTags->"Hill"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Hill"],

Cell["\<\
Hill[x, y] gives the Hill identity with arguments x and y. The returned \
object is 0.\
\>", "Text",
 CellTags->"Hill"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SimplifyPolyLog", All, CellTags],
  ButtonData->"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Hill"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Hill"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hill", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[921]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",FractionBox[
      RowBox[{"1", "-", "a"}], 
      RowBox[{"1", "-", "b"}]]},
    "PolyLog"], "+", 
   TemplateBox[{"2",FractionBox["b", "a"]},
    "PolyLog"], "-", 
   TemplateBox[{"2",FractionBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
      RowBox[{"a", " ", 
        RowBox[{"(", 
          RowBox[{"1", "-", "b"}], ")"}]}]]},
    "PolyLog"], "+", 
   RowBox[{
    RowBox[{"log", "(", "a", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "a"}], ")"}], "-", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"log", "(", 
     FractionBox[
      RowBox[{"1", "-", "a"}], 
      RowBox[{"1", "-", "b"}]], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"log", "(", 
        FractionBox[
         RowBox[{"a", "-", "b"}], "a"], ")"}]}], "+", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], 
        RowBox[{"1", "-", "b"}]], ")"}], "-", 
      RowBox[{"log", "(", "a", ")"}], "+", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"log", "(", 
        FractionBox[
         RowBox[{"a", "-", "b"}], "a"], ")"}]}], "+", 
      RowBox[{"log", "(", 
       FractionBox[
        RowBox[{"a", "-", "b"}], 
        RowBox[{"a", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "b"}], ")"}]}]], ")"}], "+", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "b"}], ")"}]}], ")"}], " ", 
    RowBox[{"log", "(", 
     FractionBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{"1", "-", "a"}], ")"}], " ", "b"}], 
      RowBox[{"a", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "b"}], ")"}]}]], ")"}]}], "+", 
   TemplateBox[{"2","a"},
    "PolyLog"], "-", 
   TemplateBox[{"2","b"},
    "PolyLog"], "-", 
   FractionBox[
    SuperscriptBox["\[Pi]", "2"], "6"]}], TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[921]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"%", " ", "/.", " ", 
    RowBox[{"a", "\[RuleDelayed]", " ", ".123"}]}], " ", "/.", " ", 
   RowBox[{"b", "\[RuleDelayed]", " ", ".656"}]}], " ", "//", " ", 
  "Chop"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[922]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[922]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Hill", "[", 
     RowBox[{"x", ",", 
      RowBox[{"x", " ", "y"}]}], "]"}], "//", "PowerExpand"}], "//", 
   "SimplifyPolyLog"}], "//", "Expand"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[923]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
   TemplateBox[{"2",RowBox[{"x", " ", "y"}]},
    "PolyLog"], "+", 
   TemplateBox[{"2",FractionBox[
      RowBox[{"1", "-", "x"}], 
      RowBox[{"1", "-", 
        RowBox[{"x", " ", "y"}]}]]},
    "PolyLog"], "-", 
   TemplateBox[{"2",FractionBox[
      RowBox[{
        RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "y"}], 
      RowBox[{"1", "-", 
        RowBox[{"x", " ", "y"}]}]]},
    "PolyLog"], "-", 
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], "-", 
   TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
    "PolyLog"], "-", 
   RowBox[{
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", 
      RowBox[{"x", " ", "y"}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "y"}], ")"}], " ", 
    RowBox[{"log", "(", "y", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[923]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"%", " ", "/.", " ", 
     RowBox[{"x", "\[RuleDelayed]", " ", ".34"}]}], "/.", " ", 
    RowBox[{"y", "\[Rule]", " ", ".6"}]}], "//", "N"}], "//", 
  "Chop"}]], "Input",
 CellTags->"Hill",
 CellLabel->"In[924]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Hill",
 CellLabel->"Out[924]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricAC", "ObjectName",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricAC"],

Cell["\<\
HypergeometricAC[n][exp] analytically continues Hypergeometric2F1 functions \
in exp. The second argument n refers to the equation number (n) in chapter \
2.10 of \"Higher Transcendental Functions\" by Ergelyi, Magnus, \
Oberhettinger, Tricomi. In case of eq. (6) (p.109) the last line is returned \
for HypergeometricAC[6][exp], while the first equality is given by \
HypergeometricAC[61][exp]. ((2.10.1) is identical to eq. (9.5.7) of \"Special \
Functions & their Applications\" by N.N.Lebedev).\
\>", "Text",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricAC", "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[925]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Collect2", "\[Rule]", "True"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"HypergeometricAC",
 CellLabel->"Out[925]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypExplicit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypExplicit", All, CellTags],
  ButtonData->"HypExplicit",
  ButtonNote->"HypExplicit"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricIR", All, CellTags],
  ButtonData->"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricSE", All, CellTags],
  ButtonData->"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 ", ",
 ButtonBox["ToHypergeometric",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ToHypergeometric", All, CellTags],
  ButtonData->"ToHypergeometric",
  ButtonNote->"ToHypergeometric"],
 "."
}], "Text",
 CellTags->"HypergeometricAC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricAC"],

Cell["These are all transformation rules currently built in.", "Text",
 CellTags->"HypergeometricAC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "1", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[926]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]], " ", 
     TemplateBox[{
      RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
       RowBox[{"\[Gamma]", "-", "\[Beta]"}],RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]", "+", 
         "1"}],RowBox[{"1", "-", "z"}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{"\[Beta]"},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]","\[Beta]",
       RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"1", "-", "z"}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[926]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "2", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[927]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "z"}], ")"}], 
      RowBox[{"-", "\[Alpha]"}]], " ", 
     TemplateBox[{RowBox[{"\[Beta]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]",RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],FractionBox["1", "z"]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Beta]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "z"}], ")"}], 
      RowBox[{"-", "\[Beta]"}]], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "-", "\[Beta]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Beta]",RowBox[{"\[Beta]", "-", "\[Gamma]", "+", "1"}],RowBox[{
         RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],FractionBox[
       "1", "z"]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[927]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "3", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[928]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{"-", "\[Alpha]"}]], " ", 
     TemplateBox[{RowBox[{"\[Beta]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]",RowBox[{"\[Gamma]", "-", "\[Beta]"}],
       RowBox[{"\[Alpha]", "-", "\[Beta]", "+", "1"}],FractionBox["1", 
        RowBox[{"1", "-", "z"}]]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Beta]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{"-", "\[Beta]"}]], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "-", "\[Beta]"}]},
      "Gamma"], " ", 
     TemplateBox[{"\[Beta]",RowBox[{"\[Gamma]", "-", "\[Alpha]"}],RowBox[{
         RowBox[{"-", "\[Alpha]"}], "+", "\[Beta]", "+", "1"}],FractionBox[
       "1", 
        RowBox[{"1", "-", "z"}]]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[928]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "4", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[929]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox["z", 
      RowBox[{"-", "\[Alpha]"}]], " ", 
     TemplateBox[{RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]},
      "Gamma"], " ", 
     TemplateBox[{
      "\[Alpha]",RowBox[{"\[Alpha]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]", "+", "1"}],
       RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "z"}], "z"]}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Alpha]"}]},
      "Gamma"], " ", 
     TemplateBox[{RowBox[{"\[Gamma]", "-", "\[Beta]"}]},
      "Gamma"]}]], "+", 
   FractionBox[
    RowBox[{
     TemplateBox[{"\[Gamma]"},
      "Gamma"], " ", 
     SuperscriptBox["z", 
      RowBox[{"\[Alpha]", "-", "\[Gamma]"}]], " ", 
     TemplateBox[{RowBox[{"\[Alpha]", "+", "\[Beta]", "-", "\[Gamma]"}]},
      "Gamma"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "z"}], ")"}], 
      RowBox[{
       RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]"}]], " ", 
     TemplateBox[{
      RowBox[{"1", "-", "\[Alpha]"}],RowBox[{"\[Gamma]", "-", "\[Alpha]"}],
       RowBox[{
         RowBox[{"-", "\[Alpha]"}], "-", "\[Beta]", "+", "\[Gamma]", "+", 
         "1"}],RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "z"}], "z"]}]},
      "Hypergeometric2F1"]}], 
    RowBox[{
     TemplateBox[{"\[Alpha]"},
      "Gamma"], " ", 
     TemplateBox[{"\[Beta]"},
      "Gamma"]}]]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[929]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "6", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[930]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "z"}], ")"}], 
    RowBox[{"-", "\[Beta]"}]], " ", 
   TemplateBox[{
    "\[Beta]",RowBox[{"\[Gamma]", "-", "\[Alpha]"}],"\[Gamma]",RowBox[{"-", 
       FractionBox["z", 
        RowBox[{"1", "-", "z"}]]}]},
    "Hypergeometric2F1"]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[930]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"HypergeometricAC", "[", "61", "]"}], "[", 
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "z"}], "]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricAC",
 CellLabel->"In[931]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "z"}], ")"}], 
    RowBox[{"-", "\[Alpha]"}]], " ", 
   TemplateBox[{
    "\[Alpha]",RowBox[{"\[Gamma]", "-", "\[Beta]"}],"\[Gamma]",RowBox[{"-", 
       FractionBox["z", 
        RowBox[{"1", "-", "z"}]]}]},
    "Hypergeometric2F1"]}], TraditionalForm]], "Output",
 CellTags->"HypergeometricAC",
 CellLabel->"Out[931]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricIR", "ObjectName",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricIR"],

Cell["\<\
HypergeometricIR[exp, t] substitutes for all Hypergeometric2F1[a,b,c,z] in \
exp by its Euler integral reprentation. The factor Integratedx[t, 0, 1] can \
be omitted by setting the option Integratedx \[Rule] False.\
\>", "Text",
 CellTags->"HypergeometricIR"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricAC", All, CellTags],
  ButtonData->"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricSE", All, CellTags],
  ButtonData->"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 ", ",
 ButtonBox["ToHypergeometric",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ToHypergeometric", All, CellTags],
  ButtonData->"ToHypergeometric",
  ButtonNote->"ToHypergeometric"],
 "."
}], "Text",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricIR", "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[932]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Integratedx", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[932]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricIR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricIR", "[", 
  RowBox[{
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[933]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]]}], 
   RowBox[{
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[933]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"t", "^", "b"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", "c"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "^", "a"}]}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[934]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{RowBox[{"b", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{
     RowBox[{"-", "a"}],RowBox[{"b", "+", "1"}],
      RowBox[{"b", "+", "c", "+", "2"}],RowBox[{"-", "z"}]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{RowBox[{"b", "+", "c", "+", "2"}]},
    "Gamma"]], TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[934]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricIR", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"HypergeometricIR",
 CellLabel->"In[935]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["t", "b"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{"t", " ", "z"}], "+", "1"}], ")"}], "a"]}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricIR",
 CellLabel->"Out[935]="]
}, Open  ]]
}, Open  ]],

Cell["HypergeometricSE", "ObjectName",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypergeometricSE"],

Cell[TextData[{
 "HypergeometricSE[exp, ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "] expresses Hypergeometric functions by their series expansion in terms of \
a sum (the Sum is omitted and ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 ", running from 0 to ",
 Cell[BoxData[
  FormBox["\[Infinity]", TraditionalForm]]],
 ", is the summation index)."
}], "Text",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "HypergeometricSE", "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[936]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Simplify", "\[Rule]", "FullSimplify"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"HypergeometricSE",
 CellLabel->"Out[936]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricIR", All, CellTags],
  ButtonData->"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 "."
}], "Text",
 CellTags->"HypergeometricSE"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypergeometricSE"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricSE", "[", 
  RowBox[{
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], ",", "\[Nu]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[937]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellTags->"HypergeometricSE",
 CellLabel->"Out[937]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricSE", "[", 
  RowBox[{
   RowBox[{"HypergeometricPFQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}], ",", "\[Nu]"}], 
  "]"}]], "Input",
 CellTags->"HypergeometricSE",
 CellLabel->"In[938]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"d"},
     "Gamma"], " ", 
    TemplateBox[{"e"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"d", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"e", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellTags->"HypergeometricSE",
 CellLabel->"Out[938]="]
}, Open  ]]
}, Open  ]],

Cell["HypExplicit", "ObjectName",
 CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypExplicit"],

Cell[TextData[{
 "HypExplicit[exp, ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "] expresses Hypergeometric functions in exp by their definition in terms of \
a sum (the Sum is omitted and ",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 " is the summation index)."
}], "Text",
 CellTags->"HypExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricIR", All, CellTags],
  ButtonData->"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 "."
}], "Text",
 CellTags->"HypExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[939]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"a","b","c","z"},
   "Hypergeometric2F1"], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[939]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypExplicit", "[", 
  RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[940]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[940]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypergeometricPFQ", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"d", ",", "e"}], "}"}], ",", "z"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[941]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[
    RowBox[{
     RowBox[{
      SubscriptBox["\[InvisiblePrefixScriptBase]", "3"], 
      SubscriptBox["F", "2"]}], "\[InvisibleApplication]", 
     RowBox[{"(", 
      RowBox[{
       TagBox[
        TagBox[
         RowBox[{
          TagBox["a",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["b",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["c",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        HypergeometricPFQ,
        Editable->False,
        Selectable->False], ";", 
       TagBox[
        TagBox[
         RowBox[{
          TagBox["d",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True], ",", 
          TagBox["e",
           HypergeometricPFQ,
           Editable->True,
           Selectable->True]}],
         InterpretTemplate[{
           SlotSequence[1]}& ]],
        HypergeometricPFQ,
        Editable->False,
        Selectable->False], ";", 
       TagBox["z",
        HypergeometricPFQ,
        Editable->True,
        Selectable->True]}], ")"}]}],
    InterpretTemplate[HypergeometricPFQ[#, #2, #3]& ],
    Editable->False,
    Selectable->False],
   HypergeometricPFQ], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[941]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypExplicit", "[", 
  RowBox[{"%", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"HypExplicit",
 CellLabel->"In[942]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{"d"},
     "Gamma"], " ", 
    TemplateBox[{"e"},
     "Gamma"], " ", 
    SuperscriptBox["z", "\[Nu]"], " ", 
    TemplateBox[{RowBox[{"a", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"b", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "\[Nu]"}]},
     "Gamma"]}], 
   RowBox[{
    TemplateBox[{"a"},
     "Gamma"], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"\[Nu]", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"d", "+", "\[Nu]"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"e", "+", "\[Nu]"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellTags->"HypExplicit",
 CellLabel->"Out[942]="]
}, Open  ]]
}, Open  ]],

Cell["HypInt", "ObjectName",
 CellTags->"HypInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"HypInt"],

Cell["\<\
HypInt[exp, t]  substitutes for all  Hypergeometric2F1[a,b,c,z] in exp \
Gamma[c]/(Gamma[b] Gamma[c-b]) Integratedx[t,0,1]  t^(b-1) (1-t)^(c-b-1) (1-t \
z)^(-a).\
\>", "Text",
 CellTags->"HypInt"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"HypInt"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}]], "Input",
 CellTags->"HypInt",
 CellLabel->"In[943]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"a","b","c","z"},
   "Hypergeometric2F1"], TraditionalForm]], "Output",
 CellTags->"HypInt",
 CellLabel->"Out[943]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"HypInt", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"HypInt",
 CellLabel->"In[944]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    TemplateBox[{"c"},
     "Gamma"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    RowBox[{
     SubsuperscriptBox["\[Integral]", 
      FormBox["0",
       TraditionalForm], 
      FormBox["1",
       TraditionalForm]], 
     RowBox[{"\[DifferentialD]", 
      TagBox[
       FormBox["t",
        TraditionalForm],
       TraditionalForm,
       Editable->True], "\[VeryThinSpace]"}]}]}], 
   RowBox[{
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"]}]], TraditionalForm]], "Output",
 CellTags->"HypInt",
 CellLabel->"Out[944]="]
}, Open  ]]
}, Open  ]],

Cell["IFPD", "ObjectName",
 CellTags->"IFPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPD"],

Cell["IFPD[p, m] denotes (p^2 - m^2).", "Text",
 CellTags->"IFPD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PropagatorDenominator", All, CellTags],
  ButtonData->"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"IFPD"]
}, Open  ]],

Cell["IFPDOff", "ObjectName",
 CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPDOff"],

Cell["\<\
IFPDOff[exp_,q1_, q2_, ...] changes from IFPD representation to \
FeynAmpDenominator[ ...]. The q1, q2, ... are the integration momenta.\
\>", "Text",
 CellTags->"IFPDOff"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IFPD", All, CellTags],
  ButtonData->"IFPD",
  ButtonNote->"IFPD"],
 ", ",
 ButtonBox["IFPDOn",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IFPDOn", All, CellTags],
  ButtonData->"IFPDOn",
  ButtonNote->"IFPDOn"],
 "."
}], "Text",
 CellTags->"IFPDOff"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"IFPDOff"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IFPD", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[945]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    SuperscriptBox["p", "2"],
    TraditionalForm], 
   FormBox["\<\" - \"\>",
    TraditionalForm], 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[945]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[946]:="],

Cell[BoxData[
 RowBox[{"IFPD", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m"}], "]"}]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[946]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IFPDOff", "[", 
  RowBox[{"%", ",", "p"}], "]"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[947]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "-", 
   SuperscriptBox["m", "2"]}], TraditionalForm]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[947]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"IFPDOff",
 CellLabel->"In[948]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"-", 
   SuperscriptBox["m", "2"]}], "+", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Output",
 CellTags->"IFPDOff",
 CellLabel->"Out[948]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["IFPDOn", "ObjectName",
 CellTags->"IFPDOn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IFPDOn"],

Cell["\<\
IFPDOn[exp_,q1_, q2_, ...] changes from FeynAmpDenominator[ ...] \
representation to the IFPD one (Inverse Feynman Propagator Denominator). \
I.e., FeynAmpDenominator[PropagatorDenominator[a,b]] is replaced by \
1/IFPD[a,b] and The q1, q2, ... are the integration momenta.\
\>", "Text",
 CellTags->"IFPDOn"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IFPD", All, CellTags],
  ButtonData->"IFPD",
  ButtonNote->"IFPD"],
 ", ",
 ButtonBox["IFPDOff",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IFPDOff", All, CellTags],
  ButtonData->"IFPDOff",
  ButtonNote->"IFPDOff"],
 "."
}], "Text",
 CellTags->"IFPDOn"]
}, Open  ]],

Cell["IncludePair", "ObjectName",
 CellTags->"IncludePair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IncludePair"],

Cell["\<\
IncludePair is an option for FC2RHI. Possible settings are True and False.\
\>", "Text",
 CellTags->"IncludePair"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FC2RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FC2RHI", All, CellTags],
  ButtonData->"FC2RHI",
  ButtonNote->"FC2RHI"],
 "."
}], "Text",
 CellTags->"IncludePair"]
}, Open  ]],

Cell["IndexPosition", "ObjectName",
 CellTags->"IndexPosition"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IndexPosition"],

Cell["IndexPosition is an option for FieldStrength.", "Text",
 CellTags->"IndexPosition"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FieldStrength",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FieldStrength", All, CellTags],
  ButtonData->"FieldStrength",
  ButtonNote->"FieldStrength"],
 "."
}], "Text",
 CellTags->"IndexPosition"]
}, Open  ]],

Cell["Indices", "ObjectName",
 CellTags->"Indices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Indices"],

Cell["\<\
Indices is an option for FORM2FeynCalc. Its default setting is Automatic. It \
may be set to a list, if the FORM-file does not contain a I(ndices) statement.\
\>", "Text",
 CellTags->"Indices"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FORM2FeynCalc", All, CellTags],
  ButtonData->"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"Indices"]
}, Open  ]],

Cell["InitialFunction", "ObjectName",
 CellTags->"InitialFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InitialFunction"],

Cell["\<\
InitialFunction is an option of FeynRule the setting of which is applied to \
the first argument of FeynRule before anything else\
\>", "Text",
 CellTags->"InitialFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynRule", All, CellTags],
  ButtonData->"FeynRule",
  ButtonNote->"FeynRule"],
 "."
}], "Text",
 CellTags->"InitialFunction"]
}, Open  ]],

Cell["InitialSubstitutions", "ObjectName",
 CellTags->"InitialSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InitialSubstitutions"],

Cell["\<\
InitialSubstitutions is an option for OneLoop and OneLoopSum and Write2. All \
substitutions indicated hereby are done at the end of the calculation.\
\>", "Text",
 CellTags->"InitialSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Write2", All, CellTags],
  ButtonData->"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"InitialSubstitutions"]
}, Open  ]],

Cell["InsideDiracTrace", "ObjectName",
 CellTags->"InsideDiracTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InsideDiracTrace"],

Cell["\<\
InsideDiracTrace is and option of DiracSimplify. If set to True, \
DiracSimplify assumes to operate inside a DiracTrace, i.e., products of an \
odd number of Dirac matrices are discarded. Furthermore simple traces are \
calculated (but divided by a factor 4, i.e. :  \
DiracSimplify[DiracMatrix[a,b], InsideDiracTrace\[Rule]True]  yields \
ScalarProduct[a,b]) Traces involving more than four DiracGamma's and \
DiracGamma[5] are not performed.\
\>", "Text",
 CellTags->"InsideDiracTrace"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"InsideDiracTrace"]
}, Open  ]],

Cell["IntegralTable", "ObjectName",
 CellTags->"IntegralTable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntegralTable"],

Cell["\<\
IntegralTable is an option of OneLoopSimplify, TwoLoopSimplify and \
FeynAmpDenominatorSimplify. It may be set to a list of the form : \
{FCIntegral[ ... ] \[RuleDelayed] bla, ...}.\
\>", "Text",
 CellTags->"IntegralTable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TwoLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TwoLoopSimplify", All, CellTags],
  ButtonData->"TwoLoopSimplify",
  ButtonNote->"TwoLoopSimplify"],
 ", ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorSimplify", All, CellTags],
  ButtonData->"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"IntegralTable"]
}, Open  ]],

Cell["IntegrateByParts", "ObjectName",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntegrateByParts"],

Cell["\<\
IntegrateByParts[(1-t)^(a Epsilon -1) g[t], deriv, t] does an integration by \
parts of the definite integral over t from 0 to 1.\
\>", "Text",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "IntegrateByParts", "]"}]], "Input",
 CellTags->"IntegrateByParts",
 CellLabel->"In[949]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Hold", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",
 CellTags->"IntegrateByParts",
 CellLabel->"Out[949]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"IntegrateByParts"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"IntegrateByParts", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
    RowBox[{"g", "[", "t", "]"}]}], ",", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "t"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], ",", "t"}], 
  "]"}]], "Input",
 CellTags->"IntegrateByParts",
 CellLabel->"In[950]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{"a", " ", 
       TagBox["\[CurlyEpsilon]",
        TraditionalForm]}]], " ", 
     RowBox[{
      SuperscriptBox["g", "\[Prime]",
       MultilineFunction->None], "(", "t", ")"}]}], 
    RowBox[{"a", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]], "+", 
   FractionBox[
    RowBox[{"g", "(", "0", ")"}], 
    RowBox[{"a", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"IntegrateByParts",
 CellLabel->"Out[950]="]
}, Open  ]]
}, Open  ]],

Cell["Integratedx", "ObjectName",
 CellTags->"Integratedx"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integratedx"],

Cell["\<\
Integratedx[x, low, up] is a variable representing the integration operator \
Integrate[#, {x,low,up}]&.\
\>", "Text",
 CellTags->"Integratedx"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanParametrize",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynmanParametrize", All, CellTags],
  ButtonData->"FeynmanParametrize",
  ButtonNote->"FeynmanParametrize"],
 ", ",
 ButtonBox["CalculateCounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CalculateCounterTerm", All, CellTags],
  ButtonData->"CalculateCounterTerm",
  ButtonNote->"CalculateCounterTerm"],
 ", ",
 ButtonBox["TimedIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TimedIntegrate", All, CellTags],
  ButtonData->"TimedIntegrate",
  ButtonNote->"TimedIntegrate"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricIR", All, CellTags],
  ButtonData->"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypInt",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypInt", All, CellTags],
  ButtonData->"HypInt",
  ButtonNote->"HypInt"],
 ", ",
 ButtonBox["TimedIntegrate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TimedIntegrate", All, CellTags],
  ButtonData->"TimedIntegrate",
  ButtonNote->"TimedIntegrate"],
 "."
}], "Text",
 CellTags->"Integratedx"]
}, Open  ]],

Cell["Integrate2", "ObjectName",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integrate2"],

Cell[TextData[{
 "Integrate2 is like Integrate, but : Integrate2[a_Plus, b__] := \
Map[Integrate2[#, b]&, a] ( more linear algebra and partial fraction \
decomposition is done) Integrate2[f[x] DeltaFunction[x], x] \[Rule] f[0] \
Integrate2[f[x] DeltaFunction[x0-x], x] \[Rule] f[x0]. Integrate2[f[x] \
DeltaFunction[a + b x], x] \[Rule] Integrate[f[x] (1/Abs[b]) \
DeltaFunction[a/b + x], x], where abs[b] \[Rule] b, if b is a Symbol, and if \
b = -c, then abs[-c] \[Rule] c, i.e., the variable contained in b is supposed \
to be positive. ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox["\[Pi]", "2"]}], TraditionalForm]]],
 " is replaced by 6 Zeta2. Integrate2[1/(1-y),{y,x,1}] is intepreted as \
distribution, i.e. as Integrate2[-1/(1-y)],{y, 0, x}] \[Rule] Log[1-y]. \
Integrate2[1/(1-x),{x,0,1}] \[Rule] 0."
}], "Text",
 CellTags->"Integrate2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeltaFunction", All, CellTags],
  ButtonData->"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate3",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Integrate3", All, CellTags],
  ButtonData->"Integrate3",
  ButtonNote->"Integrate3"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumS", All, CellTags],
  ButtonData->"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumT", All, CellTags],
  ButtonData->"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"Integrate2"],

Cell["\<\
NOTE: Since Integrate2 does do a reordering and partial fraction \
decomposition before calling the integral table of Integrate3 it will in \
general be slower compared to Integrate3 for sums of integrals. I.e., if the \
integrand has already an expanded form and if partial fraction decomposition \
is not necessary it is more effective to use Integrate3.\
\>", "Text",
 CellTags->"Integrate2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], 
     RowBox[{
      RowBox[{"Log", "[", "x", "]"}], "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[951]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.024001000000001937`", ",", 
    RowBox[{
     TemplateBox[{"3"},
      "Zeta"], "-", 
     RowBox[{
      FractionBox["3", "2"], " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}]}]}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Integrate2",
 CellLabel->"Out[951]="]
}, Open  ]],

Cell[TextData[{
 "Since Integrate2 uses table-look-up methods it is much faster than ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 "'s Integrate."
}], "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"2", ",", 
     RowBox[{"x", "^", "2"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[952]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Zeta]", "(", "2", ")"}], "-", "4", "+", 
   RowBox[{"4", " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[952]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"3", ",", 
     RowBox[{"-", "x"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[953]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "-", 
   FractionBox[
    RowBox[{"3", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "4"], "+", "1", "-", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[953]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PolyLog", "[", 
    RowBox[{"3", ",", 
     RowBox[{"1", "/", 
      RowBox[{"(", 
       RowBox[{"1", "+", "x"}], ")"}]}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[954]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "2", ")"}]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"3", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "4"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "3"], "(", "2", ")"}], "3"], "-", 
   RowBox[{
    SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[954]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[955]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[955]="]
}, Open  ]],

Cell[TextData[{
 "Integrate2 does integration in a Hadamard sense, i.e., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], 
     RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
 " means acutally expanding the result of ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Integral]", "\[Delta]", 
     RowBox[{"1", "-", "\[Delta]"}]], 
    RowBox[{
     RowBox[{"f", "(", "x", ")"}], 
     RowBox[{"\[DifferentialD]", "x"}]}], " "}], TraditionalForm]]],
 "up do ",
 Cell[BoxData[
  FormBox[
   RowBox[{"O", "(", "\[Delta]", ")"}], TraditionalForm]]],
 " and neglecting all ",
 Cell[BoxData[
  FormBox["\[Delta]", TraditionalForm]]],
 "-dependent terms. E.g. ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubsuperscriptBox["\[Integral]", "\[Delta]", 
      RowBox[{"1", "-", "\[Delta]"}]], 
     RowBox[{
      RowBox[{"1", "/", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], 
      RowBox[{"\[DifferentialD]", "x"}]}]}], " ", "=", " ", 
    RowBox[{
     RowBox[{
      RowBox[{
       RowBox[{"-", "log"}], 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], 
      SubsuperscriptBox["\[VerticalSeparator]", "\[Delta]", 
       RowBox[{"1", "-", "\[Delta]"}]]}], "=", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        RowBox[{"log", "(", "\[Delta]", ")"}]}], "+", 
       RowBox[{"log", "(", "1", ")"}]}], "\[Implies]", "0."}]}], " "}], 
   TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->False,
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[956]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[956]="]
}, Open  ]],

Cell[TextData[{
 "In the physics literature sometimes the \"+\" notation is used. In FeynCalc \
the ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]]],
 " is represented by ",
 Cell[BoxData[
  FormBox[
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], TraditionalForm]]],
 " or just ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[957]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[957]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{"1", "-", "x"}]}], "]"}], "/", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[958]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", "-", 
   RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[958]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"Log", "[", "x", "]"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"1", "+", "x"}], "]"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[959]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    TemplateBox[{"3"},
     "Zeta"], "8"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"Log", "[", "x", "]"}], "^", "2"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[960]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[960]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{"-", "x"}]}], "]"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[961]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    TemplateBox[{"3"},
     "Zeta"], "4"], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[961]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Log", "[", "x", "]"}], " ", 
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[962]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"3", "-", 
   RowBox[{"2", " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[962]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"PolyLog", "[", 
     RowBox[{"3", ",", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[963]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], "4"]}], "+", 
   FractionBox[
    TemplateBox[{"3"},
     "Zeta"], "2"], "+", 
   FractionBox["3", "16"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[963]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"Log", "[", "x", "]"}], "^", "2"}], " ", 
      RowBox[{"Log", "[", 
       RowBox[{"1", "-", "x"}], "]"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"1", "+", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[964]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "2", ")"}]}], "-", 
   RowBox[{"4", " ", 
    TemplateBox[{"4",FractionBox["1", "2"]},
     "PolyLog"]}], "+", 
   FractionBox[
    SuperscriptBox["\[Pi]", "4"], "90"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "4"], "(", "2", ")"}], "6"]}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[964]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PolyLog", "[", 
     RowBox[{"2", ",", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{
           RowBox[{"x", " ", 
            RowBox[{"(", 
             RowBox[{"1", "-", "z"}], ")"}]}], "+", "z"}], ")"}], " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "x", "+", 
           RowBox[{"x", " ", "z"}]}], ")"}]}], ")"}], "/", "z"}]}], "]"}], 
    "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x", "+", 
      RowBox[{"x", " ", "z"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[965]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
     TemplateBox[{"2",RowBox[{"-", "z"}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",RowBox[{"1", "-", "z"}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"3",RowBox[{"-", "z"}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",FractionBox["1", 
        RowBox[{"z", "+", "1"}]]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "z"}], 
          RowBox[{"z", "+", "1"}]]}]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"3",FractionBox[
        RowBox[{"1", "-", "z"}], 
        RowBox[{"z", "+", "1"}]]},
      "PolyLog"]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "z"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"-", "z"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     TemplateBox[{"2",RowBox[{"-", "z"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "z"}],
       TraditionalForm], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "3"], "(", "z", ")"}], 
    RowBox[{"6", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"z", "+", "1"}], ")"}]}], 
    RowBox[{"3", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "-", 
   FractionBox[
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "z", ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "z"}], ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]], "+", 
   FractionBox[
    RowBox[{"2", " ", "\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"log", "(", 
      RowBox[{"z", "+", "1"}], ")"}], " ", 
     RowBox[{"log", "(", "z", ")"}]}], 
    RowBox[{"1", "-", "z"}]]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[965]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Apart", "[", 
  RowBox[{
   RowBox[{"Integrate2", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"x", "^", 
       RowBox[{"(", 
        RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"1", "-", "x"}]}], "]"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], ",", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[966]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], 
     SuperscriptBox["m", "2"]]}], "-", 
   FractionBox[
    RowBox[{"\[Zeta]", "(", "2", ")"}], 
    RowBox[{"m", "-", "1"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
     RowBox[{
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{
         SubscriptBox["S", "1"], "(", 
         FormBox[
          RowBox[{"m", "-", "2"}],
          TraditionalForm], ")"}]}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["S", "12"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], "+", 
     TemplateBox[{"3"},
      "Zeta"]}], "m"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[966]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"x", "^", 
      RowBox[{"(", 
       RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], " ", 
     RowBox[{"Log", "[", "x", "]"}], " ", 
     FractionBox[
      RowBox[{"Log", "[", 
       RowBox[{"1", "+", "x"}], "]"}], 
      RowBox[{"1", "+", "x"}]]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[967]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "360"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"450", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"90", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubsuperscriptBox["S", 
        RowBox[{"-", "1"}], "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"270", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "2"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
        RowBox[{
         RowBox[{"log", "(", "4", ")"}], " ", 
         RowBox[{
          SubscriptBox["S", 
           RowBox[{"-", "1"}]], "(", 
          FormBox[
           RowBox[{"m", "-", "1"}],
           TraditionalForm], ")"}]}], "-", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "2", ")"}]}], ")"}]}], "-", 
     RowBox[{"540", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"180", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
         RowBox[{"log", "(", "8", ")"}]}], "-", 
        RowBox[{"2", " ", 
         TemplateBox[{"3"},
          "Zeta"]}]}], ")"}]}], "+", 
     RowBox[{"585", " ", 
      TemplateBox[{"3"},
       "Zeta"], " ", 
      RowBox[{
       SubscriptBox["S", "1"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "2"}], "-", "1", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "2", "-", "1"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{
         RowBox[{"-", "1"}], "-", "1", "-", "2"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "21"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"1", "-", "12"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"2", "-", "11"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "2"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", "3"], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "21"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"360", " ", 
      RowBox[{"log", "(", "2", ")"}], " ", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"-", "12"}]], "(", 
       FormBox[
        RowBox[{"m", "-", "1"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"720", " ", 
      TemplateBox[{"4",FractionBox["1", "2"]},
       "PolyLog"]}], "-", 
     RowBox[{"945", " ", 
      TemplateBox[{"3"},
       "Zeta"], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "+", 
     RowBox[{"8", " ", 
      SuperscriptBox["\[Pi]", "4"]}], "-", 
     RowBox[{"30", " ", 
      RowBox[{
       SuperscriptBox["log", "4"], "(", "2", ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[967]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", " ", "/.", " ", 
  RowBox[{"OPEm", "\[Rule]", "2"}]}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[968]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "360"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"720", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
     RowBox[{"450", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"180", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "2", ")"}], "+", 
        RowBox[{"log", "(", "4", ")"}]}], ")"}]}], "-", 
     RowBox[{"540", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "-", 
     RowBox[{"720", " ", 
      TemplateBox[{"4",FractionBox["1", "2"]},
       "PolyLog"]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
         RowBox[{"log", "(", "8", ")"}]}], "-", 
        RowBox[{"2", " ", 
         TemplateBox[{"3"},
          "Zeta"]}]}], ")"}]}], "+", 
     RowBox[{"585", " ", 
      TemplateBox[{"3"},
       "Zeta"]}], "-", 
     RowBox[{"945", " ", 
      TemplateBox[{"3"},
       "Zeta"], " ", 
      RowBox[{"log", "(", "2", ")"}]}], "+", 
     RowBox[{"8", " ", 
      SuperscriptBox["\[Pi]", "4"]}], "-", "2160", "-", 
     RowBox[{"30", " ", 
      RowBox[{
       SuperscriptBox["log", "4"], "(", "2", ")"}]}], "-", 
     RowBox[{"180", " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "2", ")"}]}], "+", 
     RowBox[{"1080", " ", 
      RowBox[{"log", "(", "2", ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"Integrate2",
 CellLabel->"Out[968]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[969]:="],

Cell[BoxData[
 FormBox["0.05051383540274483`", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[969]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", " ", 
  RowBox[{
   RowBox[{"x", " ", 
    RowBox[{"Log", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"Log", "[", "x", "]"}], " ", 
    FractionBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], 
     RowBox[{"1", "+", "x"}]]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[970]:="],

Cell[BoxData[
 FormBox["0.050513835399059064`", TraditionalForm]], "Output",
 CellLabel->"Out[970]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        FractionBox[
         RowBox[{"1", "-", "x"}], 
         RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "x"}], 
          RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[971]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{"3", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "2", ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "1"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "1"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "3"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "2"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "3"], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "21"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       RowBox[{"-", "1"}], "-", "2"}]], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "12"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "21"], "(", 
     FormBox["m",
      TraditionalForm], ")"}], "m"], "-", 
   FractionBox[
    RowBox[{"7", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     TemplateBox[{"3"},
      "Zeta"]}], 
    RowBox[{"8", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{"21", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], 
    RowBox[{"8", " ", "m"}]]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[971]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"OPEm", "\[Rule]", "3"}]}]], "Input",
 CellLabel->"In[972]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"\[Zeta]", "(", "2", ")"}], "2"]}], "+", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"log", "(", "2", ")"}]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"7", " ", 
     TemplateBox[{"3"},
      "Zeta"]}], "6"], "+", 
   FractionBox["2", "3"]}], TraditionalForm]], "Output",
 CellLabel->"Out[972]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellLabel->"In[973]:="],

Cell[BoxData[
 FormBox["0.10641794295256102`", TraditionalForm]], "Output",
 CellLabel->"Out[973]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"3", "-", "1"}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        FractionBox[
         RowBox[{"1", "-", "x"}], 
         RowBox[{"1", "+", "x"}]]}], "]"}], "-", 
      RowBox[{"PolyLog", "[", 
       RowBox[{"3", ",", 
        RowBox[{"-", 
         FractionBox[
          RowBox[{"1", "-", "x"}], 
          RowBox[{"1", "+", "x"}]]}]}], "]"}]}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[974]:="],

Cell[BoxData[
 FormBox["0.10641794295771256`", TraditionalForm]], "Output",
 CellLabel->"Out[974]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"OPEm", ",", "PositiveInteger"}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[975]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[975]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], " ", 
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[976]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[976]="]
}, Open  ]],

Cell["\<\
This is the polarized non-singlet spin splitting function whose first moment \
vanishes.\
\>", "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[977]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "8"}], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], "-", 
      FractionBox[
       SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "-", 
          RowBox[{"4", " ", 
           TemplateBox[{"2",RowBox[{"-", "x"}]},
            "PolyLog"]}], "+", 
          RowBox[{
           SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
          RowBox[{"4", " ", 
           RowBox[{"log", "(", 
            RowBox[{"x", "+", "1"}], ")"}], " ", 
           RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], 
       RowBox[{"x", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "+", 
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["536", "9"], "-", 
         RowBox[{"16", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"88", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
         RowBox[{"24", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         FractionBox["17", "3"]}], ")"}]}], "+", 
      RowBox[{
       FractionBox["4", "9"], " ", 
       RowBox[{"(", 
        RowBox[{"53", "-", 
         RowBox[{"187", " ", "x"}]}], ")"}]}], "-", 
      RowBox[{
       FractionBox["4", "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"5", " ", "x"}], "-", 
         FractionBox["22", 
          RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"8", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"3", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}]]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"16", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
         FractionBox["2", "3"]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"88", " ", "x"}], "9"], "-", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
      FractionBox["8", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"1", "-", "x"}]]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"24", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
         RowBox[{"48", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", "3"}], ")"}]}], "-", 
      RowBox[{"40", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "x"}], "+", 
         FractionBox["3", 
          RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[977]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate2", "[", 
   RowBox[{"t", ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[978]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.07200399999999973`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[978]="]
}, Open  ]],

Cell["\<\
Expanding t with respect to x yields a form already suitable for Integrate3 \
and therefore the following is faster:\
\>", "Text",
 CellTags->"Integrate2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Integrate3", "[", 
    RowBox[{
     RowBox[{"Expand", "[", 
      RowBox[{"t", ",", "x"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", "Expand"}], "//",
   "Timing"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[979]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0.024000999999998385`", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[979]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[980]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"DeltaFunction", "[", 
     RowBox[{"1", "-", "x"}], "]"}], " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[981]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[981]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "5"], 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[982]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"46", " ", 
     RowBox[{"log", "(", "2", ")"}]}], "45"], "-", 
   FractionBox["6959", "10800"]}], TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[982]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "@", "%"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[983]:="],

Cell[BoxData[
 FormBox["0.06419859938720329`", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[983]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "5"], 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "^", "2"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[984]:="],

Cell[BoxData[
 FormBox["0.06419859938720252`", TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[984]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"OPEm", "-", "1"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate2",
 CellLabel->"In[985]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], " ", 
      RowBox[{
       SubsuperscriptBox["S", "1", "2"], "(", 
       FormBox["m",
        TraditionalForm], ")"}]}], "m"]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      FractionBox[
       RowBox[{"m", "-", "1"}], "2"],
      TraditionalForm], ")"}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", "m"}]], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      FractionBox["m", "2"],
      TraditionalForm], ")"}], 
    RowBox[{"2", " ", "m"}]], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", "2"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{"-", "11"}]], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{"4", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox["m",
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox[
        RowBox[{"m", "-", "1"}], "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"log", "(", "2", ")"}], " ", 
     RowBox[{
      SubscriptBox["S", "1"], "(", 
      FormBox[
       FractionBox["m", "2"],
       TraditionalForm], ")"}]}], "m"], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "2", ")"}]}], "m"], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["log", "2"], "(", "2", ")"}], "m"]}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate2",
 CellLabel->"Out[985]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"OPEm", "\[Rule]", "4"}]}]], "Input",
 CellLabel->"In[986]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"4", " ", 
     RowBox[{"log", "(", "2", ")"}]}], "3"], "-", 
   FractionBox["241", "288"]}], TraditionalForm]], "Output",
 CellLabel->"Out[986]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "%", "]"}]], "Input",
 CellLabel->"In[987]:="],

Cell[BoxData[
 FormBox["0.0873906851910381`", TraditionalForm]], "Output",
 CellLabel->"Out[987]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"x", "^", 
     RowBox[{"(", 
      RowBox[{"4", "-", "1"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "+", "x"}], "]"}], "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellLabel->"In[988]:="],

Cell[BoxData[
 FormBox["0.0873906851910375`", TraditionalForm]], "Output",
 CellLabel->"Out[988]="]
}, Open  ]]
}, Open  ]],

Cell["Integrate3", "ObjectName",
 CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Integrate3"],

Cell["\<\
Integrate3 contains the integral table used by Integrate2. Integration is \
performed in a distributional sense. Integrate3 works more effectively on a \
sum of expressions if they are expanded or collected with respect to the \
integration variable. See the examples in Integrate2.\
\>", "Text",
 CellTags->"Integrate3"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Integrate2", All, CellTags],
  ButtonData->"Integrate2",
  ButtonNote->"Integrate2"],
 "."
}], "Text",
 CellTags->"Integrate3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Integrate3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["x", "OPEm"], " ", 
    RowBox[{"Log", "[", "x", "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[989]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "1"}], ")"}], "2"]]}], TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[989]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["x", "OPEm"], " ", 
     RowBox[{"Log", "[", "x", "]"}], " ", 
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}]}], 
    RowBox[{"1", "-", "x"}]], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[990]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox["m",
      TraditionalForm], ")"}]}], "-", 
   RowBox[{
    SubscriptBox["S", "12"], "(", 
    FormBox["m",
     TraditionalForm], ")"}], "-", 
   RowBox[{
    SubscriptBox["S", "21"], "(", 
    FormBox["m",
     TraditionalForm], ")"}], "+", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[990]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"a", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "OPEm"], " ", 
       RowBox[{"Log", "[", "x", "]"}], " ", 
       RowBox[{"Log", "[", 
        RowBox[{"1", "-", "x"}], "]"}]}], 
      RowBox[{"1", "-", "x"}]]}], "+", 
    RowBox[{"b", 
     FractionBox[
      RowBox[{
       SuperscriptBox["x", "OPEm"], 
       RowBox[{"PolyLog", "[", 
        RowBox[{"3", ",", 
         RowBox[{"-", "x"}]}], "]"}]}], 
      RowBox[{"1", "+", "x"}]]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[991]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"a", " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{
        SubscriptBox["S", "1"], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       SubscriptBox["S", "12"], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "-", 
      RowBox[{
       SubscriptBox["S", "21"], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "+", 
      TemplateBox[{"3"},
       "Zeta"]}], ")"}]}], "+", 
   RowBox[{"b", " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       SuperscriptBox[
        RowBox[{"\[Zeta]", "(", "2", ")"}], "2"], "8"], "+", 
      RowBox[{
       FractionBox["1", "2"], " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"-", "2"}]], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "-", 
      RowBox[{
       FractionBox["3", "4"], " ", 
       TemplateBox[{"3"},
        "Zeta"], " ", 
       RowBox[{
        SubscriptBox["S", 
         RowBox[{"-", "1"}]], "(", 
        FormBox["m",
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       SubscriptBox["S", 
        RowBox[{"3", "-", "1"}]], "(", 
       FormBox["m",
        TraditionalForm], ")"}], "+", 
      RowBox[{
       RowBox[{"log", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SubscriptBox["S", "3"], "(", 
          FormBox["m",
           TraditionalForm], ")"}], "-", 
         RowBox[{
          SubscriptBox["S", 
           RowBox[{"-", "3"}]], "(", 
          FormBox["m",
           TraditionalForm], ")"}]}], ")"}]}], "-", 
      RowBox[{
       FractionBox["3", "4"], " ", 
       TemplateBox[{"3"},
        "Zeta"], " ", 
       RowBox[{"log", "(", "2", ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[991]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{"DeltaFunctionPrime", "[", 
    RowBox[{"1", "-", "x"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[992]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[992]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], " ", 
    RowBox[{"DeltaFunctionPrime", "[", 
     RowBox[{"1", "-", "x"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[993]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["f", "\[Prime]",
    MultilineFunction->None], "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[993]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate3", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Integrate3",
 CellLabel->"In[994]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Integrate3",
 CellLabel->"Out[994]="]
}, Open  ]]
}, Open  ]],

Cell["IntermediateSubstitutions", "ObjectName",
 CellTags->"IntermediateSubstitutions"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IntermediateSubstitutions"],

Cell["\<\
IntermediateSubstitutions is an option for OneLoop. All substitutions \
indicated hereby are done at an intermediate stage of the calculation.\
\>", "Text",
 CellTags->"IntermediateSubstitutions"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"IntermediateSubstitutions"]
}, Open  ]],

Cell["InverseMellin", "ObjectName",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"InverseMellin"],

Cell[TextData[{
 "InverseMellin[exp, y] performs the inverse Mellin transform of polynomials \
in OPEm. The inverse transforms are not calculated but a table-lookup is \
done. WARNING: do not \"trust\" the results for the inverse Mellin transform \
involving SumT's; there is an unresolved inconsistency here (related to ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "m"}], ")"}], "."}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"InverseMellin"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeltaFunction", All, CellTags],
  ButtonData->"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Integrate2", All, CellTags],
  ButtonData->"Integrate2",
  ButtonNote->"Integrate2"],
 ", ",
 ButtonBox["OPEm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEi",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumS", All, CellTags],
  ButtonData->"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumT", All, CellTags],
  ButtonData->"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"InverseMellin"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[995]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["y", 
   RowBox[{"m", "-", "1"}]], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[995]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"(", 
     RowBox[{"OPEm", "+", "3"}], ")"}]}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[996]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["y", 
   RowBox[{"m", "+", "2"}]], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[996]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{"1", ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[997]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "y"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[997]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{"1", "/", 
    RowBox[{"OPEm", "^", "4"}]}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[998]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "6"]}], " ", 
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SuperscriptBox["log", "3"], "(", "y", ")"}]}], TraditionalForm]], "Output",\

 CellTags->"InverseMellin",
 CellLabel->"Out[998]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "/", "OPEm"}], "+", "1"}], ",", "y"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[999]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"m", "-", "1"}]], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "y"}],
      TraditionalForm], ")"}]}], "+", 
   SuperscriptBox["y", 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[999]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"InverseMellin", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"1", "/", "i"}], "+", "1"}], ",", "y", ",", "i"}], "]"}]], "Input",\

 CellTags->"InverseMellin",
 CellLabel->"In[1000]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["y", 
     RowBox[{"i", "-", "1"}]], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "y"}],
      TraditionalForm], ")"}]}], "+", 
   SuperscriptBox["y", 
    RowBox[{"i", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1000]="]
}, Open  ]],

Cell["\<\
The inverse operation to InverseMellin is done by Integrate2.\
\>", "Text",
 CellTags->"InverseMellin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"InverseMellin", "[", 
    RowBox[{
     RowBox[{"1", "/", "OPEm"}], ",", "y"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"y", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1001]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", "m"], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1001]="]
}, Open  ]],

Cell["\<\
Below is a list of all built-in basic inverse Mellin transforms .\
\>", "Text",
 CellTags->"InverseMellin"],

Cell[BoxData[
 RowBox[{
  RowBox[{"list", "=", 
   RowBox[{"{", 
    RowBox[{"1", ",", 
     FractionBox["1", 
      RowBox[{"OPEm", "+", "n"}]], ",", 
     FractionBox["1", 
      RowBox[{
       RowBox[{"-", "OPEm"}], "+", "n"}]], ",", 
     RowBox[{"PolyGamma", "[", 
      RowBox[{"0", ",", "OPEm"}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"OPEm", "-", "1"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"1", "-", "OPEm"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      RowBox[{"OPEm", "+", "1"}]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], 
      SuperscriptBox["OPEm", "2"]], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
     FractionBox[
      SuperscriptBox[
       RowBox[{"SumS", "[", 
        RowBox[{"1", ",", 
         RowBox[{
          RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "2"], "OPEm"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"2", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     FractionBox[
      RowBox[{"SumS", "[", 
       RowBox[{"2", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "OPEm"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"3", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{"OPEm", "-", "1"}]}], "]"}], "2"], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "2", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"2", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], ",", 
     SuperscriptBox[
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "3"], ",", 
     RowBox[{
      RowBox[{"SumS", "[", 
       RowBox[{"1", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], " ", 
      RowBox[{"SumS", "[", 
       RowBox[{"2", ",", 
        RowBox[{
         RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], ",", 
     RowBox[{"SumS", "[", 
      RowBox[{"1", ",", "1", ",", "1", ",", 
       RowBox[{
        RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}]}], "}"}]}], ";"}]], "Input",\

 CellTags->"InverseMellin",
 CellLabel->"In[1002]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"im", "[", "z_", "]"}], ":=", 
  RowBox[{"z", " ", "\[LongRightArrow]", 
   RowBox[{"InverseMellin", "[", 
    RowBox[{"z", ",", "y"}], "]"}]}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1003]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"OPEm", "^", 
   RowBox[{"(", 
    RowBox[{"-", "3"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1004]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "3"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "y", ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1004]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"OPEm", "^", 
   RowBox[{"(", 
    RowBox[{"-", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1005]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], " ", 
     RowBox[{"log", "(", "y", ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1005]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"PolyGamma", "[", 
   RowBox[{"0", ",", "OPEm"}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1006]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"0","m"},
    "PolyGamma2"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], " ", 
       SuperscriptBox["y", 
        RowBox[{"m", "-", "1"}]], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "y"}],
         TraditionalForm], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "y"}]], ")"}], "+"], " ", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1006]="]
}, Open  ]],

Cell[BoxData[" "], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1007]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1008]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      RowBox[{"(", 
       FractionBox["1", 
        RowBox[{"1", "-", "y"}]], ")"}], "+"], " ", 
     RowBox[{"(", 
      RowBox[{"-", 
       SuperscriptBox["y", 
        RowBox[{"m", "-", "1"}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1008]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"OPEm", "-", "1"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1009]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    RowBox[{"m", "-", "1"}]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "2"}]]}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1009]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"OPEm", "+", "1"}], ")"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1010]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    RowBox[{"m", "+", "1"}]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["y", 
       RowBox[{"m", "-", "1"}]]}], "+", 
     SuperscriptBox["y", "m"], "-", 
     RowBox[{
      SuperscriptBox["y", "m"], " ", 
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "y"}], ")"}]}], "+", 
     RowBox[{
      SuperscriptBox["y", "m"], " ", 
      RowBox[{"log", "(", "y", ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"InverseMellin",
 CellLabel->"Out[1010]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", 
   RowBox[{"OPEm", "^", "2"}]}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1011]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], 
    SuperscriptBox["m", "2"]], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
       TemplateBox[{"2","y"},
        "PolyLog"], "-", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1011]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", 
     RowBox[{
      RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "/", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1012]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"log", "(", "y", ")"}], "-", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "y"}], ")"}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1012]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SumS", "[", 
     RowBox[{"1", ",", 
      RowBox[{
       RowBox[{"-", "1"}], "+", "OPEm"}]}], "]"}], "^", "2"}], "/", "OPEm"}], 
  "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1013]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["S", "1", "2"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{"3", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
       TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
        "PolyLog"], "+", 
       RowBox[{"2", " ", 
        TemplateBox[{"2","y"},
         "PolyLog"]}], "+", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "y"}], ")"}], "+", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}], "2"]}], ")"}]}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1013]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"2", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1014]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "2"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}], "+", 
       FractionBox[
        RowBox[{"log", "(", "y", ")"}], 
        RowBox[{"1", "-", "y"}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1014]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{
   RowBox[{"SumS", "[", 
    RowBox[{"2", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "/", "OPEm"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1015]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubscriptBox["S", "2"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}], "m"], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
       TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
        "PolyLog"], "-", 
       RowBox[{
        FractionBox["1", "2"], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1015]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"3", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1016]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        TemplateBox[{"3"},
         "Zeta"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}], "-", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", "y", ")"}], 
        RowBox[{"2", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "y"}], ")"}]}]]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1016]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", "1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1017]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "11"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{"log", "(", 
         RowBox[{"1", "-", "y"}], ")"}], 
        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1017]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"im", "[", 
   RowBox[{"SumS", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1018]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "12"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        TemplateBox[{"2",RowBox[{"1", "-", "y"}]},
         "PolyLog"], 
        RowBox[{"1", "-", "y"}]], "-", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "-", 
       RowBox[{
        TemplateBox[{"3"},
         "Zeta"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1018]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"im", "[", 
   RowBox[{"SumS", "[", 
    RowBox[{"2", ",", "1", ",", 
     RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1019]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "21"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        TemplateBox[{"2","y"},
         "PolyLog"], 
        RowBox[{"1", "-", "y"}]], "-", 
       RowBox[{
        RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox["1", 
           RowBox[{"1", "-", "y"}]], ")"}], "+"]}], "+", 
       RowBox[{"2", " ", 
        TemplateBox[{"3"},
         "Zeta"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "y"}],
          TraditionalForm], ")"}]}]}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1019]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"im", "[", 
  RowBox[{"SumS", "[", 
   RowBox[{"1", ",", "1", ",", "1", ",", 
    RowBox[{"OPEm", "-", "1"}]}], "]"}], "]"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1020]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "111"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "\[LongRightArrow]", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox["1", "2"]}], " ", 
     SuperscriptBox["y", 
      RowBox[{"m", "-", "1"}]], " ", 
     SubscriptBox[
      RowBox[{"(", 
       FractionBox[
        RowBox[{
         SuperscriptBox["log", "2"], "(", 
         RowBox[{"1", "-", "y"}], ")"}], 
        RowBox[{"1", "-", "y"}]], ")"}], "+"]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"InverseMellin",
 CellLabel->"Out[1020]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"im", ",", "list"}], "]"}], ";"}]], "Input",
 CellTags->"InverseMellin",
 CellLabel->"In[1021]:="]
}, Open  ]],

Cell["Isolate", "ObjectName",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Isolate"],

Cell["\<\
Isolate[expr] substitutes abbreviations KK[i] for all Plus[...] (sub-sums) in \
expr. The inserted KK[i] have head HoldForm. Isolate[expr, varlist] \
substitutes KK[i] for all subsums in expr which are free of any occurence of \
a member of the list varlist. Instead of KK any other head or a list of names \
of the abbreviations may be specified with the option IsolateNames.\
\>", "Text",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Isolate", "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1022]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"IsolateNames", "\[Rule]", "KK"}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`IsolatePrint`IsolatePrint", "\[Rule]", 
     "False"}], ",", 
    RowBox[{
    "HighEnergyPhysics`FeynCalc`IsolateSplit`IsolateSplit", "\[Rule]", 
     "\[Infinity]"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1022]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Isolate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t0", "=", 
  RowBox[{"Isolate", "[", 
   RowBox[{"a", "+", "b"}], "]"}]}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1023]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"KK", "(", "135", ")"}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1023]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Isolate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"a", "+", "b"}], ")"}], " ", "f"}], " ", "+", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"c", "+", "d"}], ")"}], " ", "f"}], " ", "+", " ", "e"}], ",", 
    "f"}], "]"}]}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1024]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"e", "+", 
   RowBox[{"f", " ", 
    TagBox[
     RowBox[{"KK", "(", "135", ")"}],
     HoldForm]}], "+", 
   RowBox[{"f", " ", 
    TagBox[
     RowBox[{"KK", "(", "136", ")"}],
     HoldForm]}]}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1024]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "t1", "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1025]:="],

Cell[BoxData[
 RowBox[{"e", "+", 
  RowBox[{"f", " ", 
   TagBox[
    RowBox[{"KK", "[", "135", "]"}],
    HoldForm]}], "+", 
  RowBox[{"f", " ", 
   TagBox[
    RowBox[{"KK", "[", "136", "]"}],
    HoldForm]}]}]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1025]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{"t0", ",", " ", "t1", ",", " ", 
   RowBox[{"ReleaseHold", "[", "t1", "]"}]}], "}"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1026]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    TagBox[
     RowBox[{"KK", "(", "135", ")"}],
     HoldForm], ",", 
    RowBox[{"e", "+", 
     RowBox[{"f", " ", 
      TagBox[
       RowBox[{"KK", "(", "135", ")"}],
       HoldForm]}], "+", 
     RowBox[{"f", " ", 
      TagBox[
       RowBox[{"KK", "(", "136", ")"}],
       HoldForm]}]}], ",", 
    RowBox[{
     RowBox[{"f", " ", 
      RowBox[{"(", 
       RowBox[{"a", "+", "b"}], ")"}]}], "+", 
     RowBox[{"f", " ", 
      RowBox[{"(", 
       RowBox[{"c", "+", "d"}], ")"}]}], "+", "e"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1026]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"a", "[", "z", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "+", 
       RowBox[{"c", " ", 
        RowBox[{"(", 
         RowBox[{"y", "+", "z"}], ")"}]}]}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"d", "[", "z", "]"}], " ", 
     RowBox[{"(", 
      RowBox[{"y", "+", "z"}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "d"}], "}"}], ",", 
   RowBox[{"IsolateNames", "\[Rule]", "fF"}]}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1027]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    TagBox[
     RowBox[{"fF", "(", "138", ")"}],
     HoldForm], " ", 
    RowBox[{"a", "(", "z", ")"}]}], "+", 
   RowBox[{
    TagBox[
     RowBox[{"fF", "(", "137", ")"}],
     HoldForm], " ", 
    RowBox[{"d", "(", "z", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1027]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"??", "fF"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1028]:="],

Cell["Global`fF", "Print", "PrintUsage",
 CellTags->{"Info3535453244-5096475", "Isolate"}],

Cell[BoxData[
 InterpretationBox[GridBox[{
    {GridBox[{
       {
        RowBox[{
         RowBox[{"fF", "[", "137", "]"}], "=", 
         RowBox[{"y", "+", "z"}]}]},
       {" "},
       {
        RowBox[{
         RowBox[{"fF", "[", "138", "]"}], "=", 
         RowBox[{"b", "+", 
          RowBox[{"c", " ", 
           RowBox[{"\<\"HoldForm\"\>", "[", 
            RowBox[{"fF", "[", "137", "]"}], "]"}]}]}]}]}
      },
      BaselinePosition->{Baseline, {1, 1}},
      GridBoxAlignment->{
       "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
        "RowsIndexed" -> {}},
      GridBoxItemSize->{"Columns" -> {{
           Scaled[0.999]}}, "ColumnsIndexed" -> {}, "Rows" -> {{1.}}, 
        "RowsIndexed" -> {}}]}
   },
   BaselinePosition->{Baseline, {1, 1}},
   GridBoxAlignment->{
    "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
     "RowsIndexed" -> {}}],
  Definition[$CellContext`fF],
  Editable->False]], "Print",
 CellTags->{"Info3535453244-5096475", "Isolate"}]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Isolate", "[", 
  RowBox[{
   RowBox[{"a", "-", "b", "-", "c", "-", "d", "-", "e"}], ",", 
   RowBox[{"IsolateNames", "\[Rule]", "L"}], ",", 
   RowBox[{"IsolateSplit", "\[Rule]", "15"}]}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1029]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"L", "(", "140", ")"}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1029]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"L", "[", "2", "]"}], ",", 
   RowBox[{"L", "[", "1", "]"}]}], "}"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1030]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"L", "(", "2", ")"}], ",", 
    RowBox[{"L", "(", "1", ")"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Isolate",
 CellLabel->"Out[1030]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t0", ",", "t1", ",", "L", ",", "fF"}], "]"}]], "Input",
 CellTags->"Isolate",
 CellLabel->"In[1031]:="]
}, Open  ]],

Cell["IsolateHead", "ObjectName",
 CellTags->"IsolateHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateHead"],

Cell["IsolateHead is equivalent to IsolateNames.", "Text",
 CellTags->"IsolateHead"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["IsolateNames",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IsolateNames", All, CellTags],
  ButtonData->"IsolateNames",
  ButtonNote->"IsolateNames"],
 "."
}], "Text",
 CellTags->"IsolateHead"]
}, Open  ]],

Cell["IsolateNames", "ObjectName",
 CellTags->"IsolateNames"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateNames"],

Cell["\<\
IsolateNames is an option for Isolate and Collect2. Its default setting is \
KK. Instead of a symbol the setting may also be a list with the names of the \
abbrevations.\
\>", "Text",
 CellTags->"IsolateNames"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["Collect2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Collect2", All, CellTags],
  ButtonData->"Collect2",
  ButtonNote->"Collect2"],
 "."
}], "Text",
 CellTags->"IsolateNames"]
}, Open  ]],

Cell["IsolatePrint", "ObjectName",
 CellTags->"IsolatePrint"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolatePrint"],

Cell["\<\
IsolatePrint is an option of Isolate. If it is set to OutputForm (or any \
other *Form) the definitions of the abbreviations are printed during the \
operation of Isolate. The setting IsolatePrint \[Rule] False suppresses \
printing.\
\>", "Text",
 CellTags->"IsolatePrint"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"IsolatePrint"]
}, Open  ]],

Cell["IsolateSplit", "ObjectName",
 CellTags->"IsolateSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"IsolateSplit"],

Cell["\<\
IsolateSplit is an option for Isolate. Its setting determines the maximum \
number of characters of FortranForm[expr] which are abbreviated by Isolate. \
If the expression is larger than the indicated number, it is split into \
smaller pieces and onto each subsum Isolate is applied. With the default \
setting IsolateSplit \[Rule] Infinity no splitting is done.\
\>", "Text",
 CellTags->"IsolateSplit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 "."
}], "Text",
 CellTags->"IsolateSplit"]
}, Open  ]],

Cell["KeepOnly", "ObjectName",
 CellTags->"KeepOnly"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"KeepOnly"],

Cell["\<\
KeepOnly is an option of OneLoop. It may be set to B0, C0, D0 keeping only \
the corresponding coefficients. The default setting is False. If KeepOnly is \
set to {} then the part of the amplitude which is not coefficient of B0, C0, \
D0 is kept.\
\>", "Text",
 CellTags->"KeepOnly"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "C0", All, CellTags],
  ButtonData->"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "D0", All, CellTags],
  ButtonData->"D0",
  ButtonNote->"D0"],
 "."
}], "Text",
 CellTags->"KeepOnly"]
}, Open  ]],

Cell["KK", "ObjectName",
 CellTags->"KK"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"KK"],

Cell["\<\
KK[i] is the default setting of IsolateNames, which is the head of \
abbreviations used by Isolate. A KK[i] returned by Isolate is given in \
HoldForm and can be recovered by ReleaseHold[KK[i]].\
\>", "Text",
 CellTags->"KK"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["IsolateNames",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IsolateNames", All, CellTags],
  ButtonData->"IsolateNames",
  ButtonNote->"IsolateNames"],
 "."
}], "Text",
 CellTags->"KK"]
}, Open  ]],

Cell["Kummer", "ObjectName",
 CellTags->"Kummer"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Kummer"],

Cell["\<\
Kummer[i][exp] applies Kummer relation number i (i =1, ... 24, 94,95,96) to \
all Hypergeometric2F1 in exp. i = 94 corresponds to eq. 9.131.2, i = 95 to \
eq. 9.132.1 and i = 96 to eq. 9.132.2 in Gradsteyn & Ryzhik.\
\>", "Text",
 CellTags->"Kummer"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricAC", All, CellTags],
  ButtonData->"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 "."
}], "Text",
 CellTags->"Kummer"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Kummer"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "2", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1032]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c","z"},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1032]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "3", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1033]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",RowBox[{"-", 
        FractionBox["z", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1033]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "4", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1034]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",RowBox[{"-", 
        FractionBox["z", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1034]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "6", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1035]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",
      RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1035]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "6", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1036]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"1", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
      RowBox[{"a", "+", "b", "-", "c", "+", "1"}],RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1036]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "7", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1037]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1037]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "7", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1038]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{
     "a",RowBox[{"a", "-", "c", "+", "1"}],
      RowBox[{"a", "+", "b", "-", "c", "+", "1"}],RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1038]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "8", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1039]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1039]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "8", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"a", "+", "b", "+", "1", "-", "c"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1040]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{
     "b",RowBox[{"b", "-", "c", "+", "1"}],
      RowBox[{"a", "+", "b", "-", "c", "+", "1"}],RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1040]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "10", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1041]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",FractionBox[
      "1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1041]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "10", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1042]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a",RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"a", "-", "b", "+", "1"}],
     FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "b"}],RowBox[{"c", "-", "b"}],
      RowBox[{"a", "-", "b", "+", "1"}],FractionBox["1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1042]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "11", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1043]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1043]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "11", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1044]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a",RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"a", "-", "b", "+", "1"}],
     FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", 
       FractionBox[
        RowBox[{"1", "-", "z"}], "z"]}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TemplateBox[{
     "a",RowBox[{"c", "-", "b"}],RowBox[{"a", "-", "b", "+", "1"}],
      FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1044]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "12", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1045]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"b", "-", "a"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1045]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", 
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"a", "+", "1", "-", "b"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "12", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", 
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"a", "+", "1", "-", "b"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1046]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    "a",RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"a", "-", "b", "+", "1"}],
     FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"1", "-", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "b"}],RowBox[{"a", "-", "c", "+", "1"}],
      RowBox[{"a", "-", "b", "+", "1"}],FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1046]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "14", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1047]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",FractionBox[
      "1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1047]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "14", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1048]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
       RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"a", "+", "b", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "a"}],RowBox[{"c", "-", "a"}],RowBox[{
        RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1048]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "15", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1049]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "b"], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1049]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "15", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1050]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
       RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], 
     RowBox[{"b", "-", "c", "+", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "a"}],RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
        RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1050]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "16", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1051]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "a"], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1051]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
    RowBox[{"b", "+", "1", "-", "a"}], ",", 
    RowBox[{"z", "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "16", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"b", "+", "1", "-", "c"}], ",", "b", ",", 
     RowBox[{"b", "+", "1", "-", "a"}], ",", 
     RowBox[{"z", "^", 
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}]}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1052]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
       RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "z"}], ")"}], "b"], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],RowBox[{
        RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", 
       RowBox[{"1", "-", "z"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1052]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "18", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1053]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
     RowBox[{"2", "-", "c"}],"z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "a"}],RowBox[{"1", "-", "b"}],RowBox[{"2", "-", "c"}],
      "z"},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1053]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "18", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1054]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c","z"},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1054]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "19", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1055]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
     RowBox[{"2", "-", "c"}],"z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "a"}], "+", "c", "-", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "b"}],RowBox[{"a", "-", "c", "+", "1"}],
      RowBox[{"2", "-", "c"}],FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1055]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "19", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1056]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1056]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"a", "+", "1", "-", "c"}], ",", 
    RowBox[{"b", "+", "1", "-", "c"}], ",", 
    RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "20", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"a", "+", "1", "-", "c"}], ",", 
     RowBox[{"b", "+", "1", "-", "c"}], ",", 
     RowBox[{"2", "-", "c"}], ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1057]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{
    RowBox[{"a", "-", "c", "+", "1"}],RowBox[{"b", "-", "c", "+", "1"}],
     RowBox[{"2", "-", "c"}],"z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{
      RowBox[{"-", "b"}], "+", "c", "-", "1"}]], " ", 
    TemplateBox[{
     RowBox[{"1", "-", "a"}],RowBox[{"b", "-", "c", "+", "1"}],
      RowBox[{"2", "-", "c"}],FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1057]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "20", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1058]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",FractionBox["z", 
       RowBox[{"z", "-", "1"}]]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1058]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "22", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1059]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
       RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"1", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "a"}],RowBox[{"1", "-", "b"}],RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
      RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1059]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "22", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1060]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{
      RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
    TemplateBox[{
     RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],"c",
      RowBox[{"1", "-", "z"}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1060]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "23", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1061]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
       RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"a", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "a"}],RowBox[{"c", "-", "a"}],RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],RowBox[{"1", "-", 
        FractionBox["1", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1061]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "23", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1062]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "z"}], ")"}], 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"a",RowBox[{"c", "-", "b"}],"c",RowBox[{"1", "-", 
        FractionBox["1", 
         RowBox[{"1", "-", "z"}]]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1062]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{
    RowBox[{"c", "-", "a"}], ",", 
    RowBox[{"c", "-", "b"}], ",", 
    RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "24", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{
     RowBox[{"c", "-", "a"}], ",", 
     RowBox[{"c", "-", "b"}], ",", 
     RowBox[{"c", "+", "1", "-", "a", "-", "b"}], ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1063]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
       RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
     RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"b", "-", "c"}]], " ", 
    TemplateBox[{RowBox[{"1", "-", "b"}],RowBox[{"c", "-", "b"}],RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1063]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"1", "-", "z"}]}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "24", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", 
     RowBox[{"1", "-", "z"}]}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1064]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c",RowBox[{"1", "-", "z"}]},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    SuperscriptBox["z", 
     RowBox[{"-", "b"}]], " ", 
    TemplateBox[{"b",RowBox[{"c", "-", "a"}],"c",RowBox[{"-", 
        FractionBox[
         RowBox[{"1", "-", "z"}], "z"]}]},
     "Hypergeometric2F1"]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1064]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "94", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1065]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      TemplateBox[{"c"},
       "Gamma"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{
        RowBox[{"-", "a"}], "-", "b", "+", "c"}]], " ", 
      TemplateBox[{RowBox[{"a", "+", "b", "-", "c"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "a"}],RowBox[{"c", "-", "b"}],RowBox[{
          RowBox[{"-", "a"}], "-", "b", "+", "c", "+", "1"}],
        RowBox[{"1", "-", "z"}]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"a"},
       "Gamma"], " ", 
      TemplateBox[{"b"},
       "Gamma"]}]], "+", 
    FractionBox[
     RowBox[{
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{
          RowBox[{"-", "a"}], "-", "b", "+", "c"}]},
       "Gamma"], " ", 
      TemplateBox[{
       "a","b",RowBox[{"a", "+", "b", "-", "c", "+", "1"}],
        RowBox[{"1", "-", "z"}]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{RowBox[{"c", "-", "a"}]},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "b"}]},
       "Gamma"]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1065]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "95", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1066]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{"-", "a"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"b", "-", "a"}]},
       "Gamma"], " ", 
      TemplateBox[{
       "a",RowBox[{"c", "-", "b"}],RowBox[{"a", "-", "b", "+", "1"}],
        FractionBox["1", 
         RowBox[{"1", "-", "z"}]]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"b"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "a"}]},
       "Gamma"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"1", "-", "z"}], ")"}], 
       RowBox[{"-", "b"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"a", "-", "b"}]},
       "Gamma"], " ", 
      TemplateBox[{"b",RowBox[{"c", "-", "a"}],RowBox[{
          RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", 
         RowBox[{"1", "-", "z"}]]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"a"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "b"}]},
       "Gamma"]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1066]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Hypergeometric2F1", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "==", 
  RowBox[{
   RowBox[{"Kummer", "[", "96", "]"}], "[", 
   RowBox[{"Hypergeometric2F1", "[", 
    RowBox[{"a", ",", "b", ",", "c", ",", "z"}], "]"}], "]"}]}]], "Input",
 CellTags->"Kummer",
 CellLabel->"In[1067]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"a","b","c","z"},
    "Hypergeometric2F1"], "\[LongEqual]", 
   RowBox[{
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "a"], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", "a"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"b", "-", "a"}]},
       "Gamma"], " ", 
      TemplateBox[{
       "a",RowBox[{"a", "-", "c", "+", "1"}],
        RowBox[{"a", "-", "b", "+", "1"}],FractionBox["1", "z"]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"b"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "a"}]},
       "Gamma"]}]], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "b"], " ", 
      SuperscriptBox["z", 
       RowBox[{"-", "b"}]], " ", 
      TemplateBox[{"c"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"a", "-", "b"}]},
       "Gamma"], " ", 
      TemplateBox[{"b",RowBox[{"b", "-", "c", "+", "1"}],RowBox[{
          RowBox[{"-", "a"}], "+", "b", "+", "1"}],FractionBox["1", "z"]},
       "Hypergeometric2F1"]}], 
     RowBox[{
      TemplateBox[{"a"},
       "Gamma"], " ", 
      TemplateBox[{RowBox[{"c", "-", "b"}]},
       "Gamma"]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Kummer",
 CellLabel->"Out[1067]="]
}, Open  ]]
}, Open  ]],

Cell["Lagrangian", "ObjectName",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Lagrangian"],

Cell["\<\
Lagrangian[\"oqu\"] gives the unpolarized OPE quark operator. \
Lagrangian[\"oqp\"] gives the polarized quark OPE operator. \
Lagrangian[\"ogu\"] gives the unpolarized gluon OPE operator. \
Lagrangian[\"ogp\"] gives the polarized gluon OPE operator. \
Lagrangian[\"ogd\"] gives the sigma-term part of the QCD lagrangian. \
Lagrangian[\"QCD\"] gives the gluon self interaction part of the QCD \
lagrangian.\
\>", "Text",
 CellTags->"Lagrangian"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynRule", All, CellTags],
  ButtonData->"FeynRule",
  ButtonNote->"FeynRule"],
 "."
}], "Text",
 CellTags->"Lagrangian"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<QCD\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1068]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "4"]}], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", "\[Beta]"}], "a"], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", "\[Beta]"}], "a"]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1068]="]
}, Open  ]],

Cell["Twist-2 operator product expansion operators", "Text",
 CellTags->"Lagrangian"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<ogu\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1069]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"m", "-", "1"}]], " ", 
   RowBox[{
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1069]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<ogp\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1070]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Alpha]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Beta]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["\[Gamma]",
        TraditionalForm],
       TraditionalForm], 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Beta]", "\[Gamma]"}], "a"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      SubsuperscriptBox["D", 
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       RowBox[{"a", "b"}]], ")"}], 
     RowBox[{"m", "-", "2"}]], ".", 
    SubsuperscriptBox["F", 
     RowBox[{"\[Alpha]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], "b"]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1070]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<oqu\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1071]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1071]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Lagrangian", "[", "\"\<oqp\>\"", "]"}]], "Input",
 CellTags->"Lagrangian",
 CellLabel->"In[1072]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[ImaginaryI]", "m"], " ", 
   RowBox[{
    FormBox[
     OverscriptBox["\[Psi]", "_"],
     TraditionalForm], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox[
     SubscriptBox["D", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]], 
     RowBox[{"m", "-", "1"}]], ".", 
    FormBox["\[Psi]",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Lagrangian",
 CellLabel->"Out[1072]="]
}, Open  ]]
}, Open  ]],

Cell["LC", "ObjectName",
 CellTags->"LC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LC"],

Cell["\<\
LC[m,n,r,s] evaluates to 4-dimensional LeviCivita[m,n,r,s] by virtue of \
applying FeynCalcInternal. LC[m,...][p, ...] evaluates to 4-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",
 CellTags->"LC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeviCivita", All, CellTags],
  ButtonData->"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LCD", All, CellTags],
  ButtonData->"LCD",
  ButtonNote->"LCD"],
 "."
}], "Text",
 CellTags->"LC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LC", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1073]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], 
  TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1073]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1074]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1074]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1075]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Sigma]", "]"}]}], "]"}]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1075]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LC", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1076]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "p", "q"}]], TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1076]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1077]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1077]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p", "]"}], 
    " ", 
    RowBox[{
     RowBox[{"LC", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q", 
     "]"}]}], "]"}], " "}]], "Input",
 CellTags->"LC",
 CellLabel->"In[1078]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"18", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "-", 
   RowBox[{"24", " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}]}], TraditionalForm]], "Output",
 CellTags->"LC",
 CellLabel->"Out[1078]="]
}, Open  ]]
}, Open  ]],

Cell["LCD", "ObjectName",
 CellTags->"LCD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LCD"],

Cell["\<\
LCD[m,n,r,s] evaluates to D-dimensional LeviCivita[m,n,r,s] by virtue of \
FeynCalcInternal. LCD[m,...][p, ...] evaluates to D-dimensional \
LeviCivita[m,...][p,...] applying FeynCalcInternal.\
\>", "Text",
 CellTags->"LCD"],

Cell["\<\
You need to do SetOptions[Eps, Dimension\[Rule]D] before LCD can be used as \
D-dimensional Levi-Civita input function.\
\>", "Text",
 CellTags->"LCD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LeviCivita",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeviCivita", All, CellTags],
  ButtonData->"LeviCivita",
  ButtonNote->"LeviCivita"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LC", All, CellTags],
  ButtonData->"LC",
  ButtonNote->"LC"],
 "."
}], "Text",
 CellTags->"LCD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LCD"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1079]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LCD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]"}], 
  "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1080]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "\[Rho]", "\[Sigma]"}]], 
  TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1080]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "FCI"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1081]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Sigma]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1081]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1082]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Rho]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Sigma]", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1082]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LCD", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1083]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Mu]", "\[Nu]", "p", "q"}]], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1083]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1084]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1084]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Factor2", "[", 
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "p", "]"}], 
    " ", 
    RowBox[{
     RowBox[{"LCD", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]"}], "]"}], "[", "q", 
     "]"}]}], "]"}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1085]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1085]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Eps", ",", 
   RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}]], "Input",
 CellTags->"LCD",
 CellLabel->"In[1086]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellTags->"LCD",
 CellLabel->"Out[1086]="]
}, Open  ]]
}, Open  ]],

Cell["LeftPartialD", "ObjectName",
 CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftPartialD"],

Cell[TextData[{
 "LeftPartialD[\[Mu]] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 "acting to the left."
}], "Text",
 CellTags->"LeftPartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialD", All, CellTags],
  ButtonData->"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD", All, CellTags],
  ButtonData->"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RightPartialD", All, CellTags],
  ButtonData->"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"LeftPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1087]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1087]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1088]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1088]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1089]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1089]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"LeftPartialD", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1090]:="],

Cell[BoxData[
 RowBox[{"LeftPartialD", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1090]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Rho]", "]"}]}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1091]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["A", "\[Nu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Rho]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1091]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1092]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1092]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"LeftPartialD",
 CellLabel->"In[1093]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}], "+", 
  RowBox[{
   RowBox[{"QuantumField", "[", 
    RowBox[{
     RowBox[{"PartialD", "[", 
      RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}], "]"}], ",", "A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
   RowBox[{"QuantumField", "[", 
    RowBox[{"A", ",", 
     RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]}]], "Output",
 CellTags->"LeftPartialD",
 CellLabel->"Out[1093]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["LeftRightPartialD", "ObjectName",
 CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftRightPartialD"],

Cell[TextData[{
 "LeftRightPartialD[mu] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 ", acting to the left and right. ExplicitPartialD[LeftRightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] gives 1/2 (RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] - LeftPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "])."
}], "Text",
 CellTags->"LeftRightPartialD"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExplicitPartialD", All, CellTags],
  ButtonData->"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialD", All, CellTags],
  ButtonData->"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftPartialD", All, CellTags],
  ButtonData->"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD2", All, CellTags],
  ButtonData->"LeftRightPartialD2",
  ButtonNote->"LeftRightPartialD2"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RightPartialD", All, CellTags],
  ButtonData->"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftRightPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeftRightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1094]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1094]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1095]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[RightArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "-", 
     SubscriptBox[
      OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1095]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1096]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1096]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1097]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["\[PartialD]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], 
      SubsuperscriptBox["A", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[Null]"]}], "-", 
     RowBox[{
      SubscriptBox[
       OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SubscriptBox["A", "\[Nu]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1097]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftRightPartialD", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1098]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1098]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD",
 CellLabel->"In[1099]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SubscriptBox["A", "\[Mu]"], ".", 
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}]}], "-", 
     RowBox[{
      RowBox[{
       SubscriptBox["\[PartialD]", 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], 
       SubsuperscriptBox["A", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], "\[Null]"]}], ".", 
      SubscriptBox["A", "\[Rho]"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD",
 CellLabel->"Out[1099]="]
}, Open  ]]
}, Open  ]],

Cell["LeftRightPartialD2", "ObjectName",
 CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeftRightPartialD2"],

Cell[TextData[{
 "LeftRightPartialD2[mu] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], TraditionalForm]]],
 ", acting to the left and right. ExplicitPartialD[LeftRightPartialD2[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "]] gives (RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] + LeftPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "])."
}], "Text",
 CellTags->"LeftRightPartialD2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExplicitPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExplicitPartialD", All, CellTags],
  ButtonData->"ExplicitPartialD",
  ButtonNote->"ExplicitPartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialD", All, CellTags],
  ButtonData->"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftPartialD", All, CellTags],
  ButtonData->"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RightPartialD", All, CellTags],
  ButtonData->"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeftRightPartialD2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1100]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1100]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExplicitPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1101]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "+", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1101]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeftRightPartialD2", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1102]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1102]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1103]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    SubsuperscriptBox["A", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], "\[Null]"]}], "+", 
   RowBox[{
    SubscriptBox[
     OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SubscriptBox["A", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1103]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}], ".", 
  RowBox[{"LeftRightPartialD2", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Rho]", "]"}]}], "]"}]}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1104]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftRightArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Rho]"]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1104]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"LeftRightPartialD2",
 CellLabel->"In[1105]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["A", "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["A", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["A", "\[Rho]"]}]}], TraditionalForm]], "Output",
 CellTags->"LeftRightPartialD2",
 CellLabel->"Out[1105]="]
}, Open  ]]
}, Open  ]],

Cell["LeviCivita ", "ObjectName",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeviCivita"],

Cell[TextData[{
 "LeviCivita[",
 Cell[BoxData[
  FormBox[
   RowBox[{
   "\[Mu]", ",", " ", "\[Nu]", ",", " ", "\[Rho]", ",", " ", "\[Sigma]"}], 
   TraditionalForm]]],
 "] is an input function for the totally antisymmetric Levi-Civita tensor. It \
evaluates automatically to the internal representation Eps[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox[
   FormBox["\[Rho]",
    TraditionalForm], TraditionalForm]]],
 "], LorentzIndex[\[Sigma]]] (or with a second argument in LorentzIndex for \
the Dimension, if the option Dimension of LeviCivita is changed). LeviCivita[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", 
    RowBox[{"\[Nu]", " ", "..."}]}], TraditionalForm]]],
 "][ ",
 Cell[BoxData[
  FormBox[
   RowBox[{"p", ",", " ", ".."}], TraditionalForm]]],
 ".] evaluates to Eps[LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "], LorentzIndex[",
 Cell[BoxData[
  FormBox["\[Nu]", TraditionalForm]]],
 "], ..., Momentum[",
 Cell[BoxData[
  FormBox["p", TraditionalForm]]],
 "], ...]."
}], "Text",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "LeviCivita", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1106]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1106]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Eps", All, CellTags],
  ButtonData->"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["LC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LC", All, CellTags],
  ButtonData->"LC",
  ButtonNote->"LC"],
 ", ",
 ButtonBox["LCD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LCD", All, CellTags],
  ButtonData->"LCD",
  ButtonNote->"LCD"],
 "."
}], "Text",
 CellTags->"LeviCivita"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LeviCivita"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeviCivita", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1107]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Alpha]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Beta]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Gamma]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Delta]",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1107]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
  RowBox[{"OPEDelta", ",", "p"}], "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1108]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox["p",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1108]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"LeviCivita", "[", "]"}], "[", 
  RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1109]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox["p",
     TraditionalForm], 
    FormBox["q",
     TraditionalForm], 
    FormBox["r",
     TraditionalForm], 
    FormBox["s",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1109]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LeviCivita", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Beta]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1110]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1110]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
   RowBox[{"OPEDelta", ",", "p"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1111]:="],

Cell[BoxData[
 RowBox[{"Eps", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], ",", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1111]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
    "]"}], ".", 
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
    "]"}]}], " "}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1112]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]], ".", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Alpha]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Beta]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Gamma]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1112]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "t1", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1113]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "24"}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1113]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";", 
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], ";"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1114]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", " ", 
   RowBox[{
    RowBox[{"LeviCivita", "[", 
     RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
     "]"}], ".", 
    RowBox[{"LeviCivita", "[", 
     RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Rho]"}], 
     "]"}]}], "]"}], "//", "Factor2"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1115]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"2", "-", "D"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"3", "-", "D"}], ")"}], " ", "D"}], TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1115]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"LeviCivita", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";", 
  RowBox[{"SetOptions", "[", 
   RowBox[{"Eps", ",", 
    RowBox[{"Dimension", "\[Rule]", "4"}]}], "]"}], ";"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1116]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LC", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", "\[Gamma]", ",", "\[Delta]"}], 
  "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1117]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{"\[Alpha]", "\[Beta]", "\[Gamma]", "\[Delta]"}]], 
  TraditionalForm]], "Output",
 CellTags->"LeviCivita",
 CellLabel->"Out[1117]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"LeviCivita",
 CellLabel->"In[1118]:="]
}, Open  ]],

Cell["LeviCivitaSign", "ObjectName",
 CellTags->"LeviCivitaSign"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LeviCivitaSign"],

Cell["\<\
LeviCivitaSign is an option for DiracTrace and EpsChisholm. It determines the \
sign in the result of a Dirac trace of four gamma matrices and gamma5.\
\>", "Text",
 CellTags->"LeviCivitaSign"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["EpsChisholm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "EpsChisholm", All, CellTags],
  ButtonData->"EpsChisholm",
  ButtonNote->"EpsChisholm"],
 "."
}], "Text",
 CellTags->"LeviCivitaSign"]
}, Open  ]],

Cell["ListIntegrals  ***unfinished***", "ObjectName",
 CellTags->"ListIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ListIntegrals"],

Cell["\<\
ListIntegrals[exp, {q1, q2}] gives a list of basic integrals  \
(FeynAmpDenominator's multiplied by scalar products involving q1, q2).  Any \
non-integer exponent (e.g. the (OPEm-1) in  \
ScalarProduct[OPEDelta,q1]^(OPEm-1)) is replaced by OPEm.  If the options \
Pair is set to True only integrals involving   scalar products (not counting \
those with OPEDelta) are given; with Pair \[Rule] False those are excluded.\
\>", "Text",
 CellTags->"ListIntegrals"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ListIntegrals"],

Cell[BoxData[""], "Input",
 CellTags->"ListIntegrals",
 CellLabel->"In[1119]:="]
}, Open  ]],

Cell["Li2 ", "ObjectName",
 CellTags->"Li2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Li2"],

Cell["\<\
 Li2 is an abbreviation for the dilog function, i.e., Li2 = PolyLog[2, #]&.\
\>", "Text",
 CellTags->"Li2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Li3",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Li3", All, CellTags],
  ButtonData->"Li3",
  ButtonNote->"Li3"],
 "."
}], "Text",
 CellTags->"Li2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Li2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li2", "[", "x", "]"}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1120]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"2","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellTags->"Li2",
 CellLabel->"Out[1120]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li2", "//", "StandardForm"}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1121]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", "#1"}], "]"}], "&"}]], "Output",
 CellTags->"Li2",
 CellLabel->"Out[1121]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"\[Integral]", 
   RowBox[{
    FractionBox[
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], "x"], 
    RowBox[{"\[DifferentialD]", "x"}]}]}]}]], "Input",
 CellTags->"Li2",
 CellLabel->"In[1122]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"2","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellTags->"Li2",
 CellLabel->"Out[1122]="]
}, Open  ]]
}, Open  ]],

Cell["Li3 ", "ObjectName",
 CellTags->"Li3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Li3"],

Cell["\<\
 Li3 is an abbreviation for the trilog function, i.e., Li3 = PolyLog[3, #]&.\
\>", "Text",
 CellTags->"Li3"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Li2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Li2", All, CellTags],
  ButtonData->"Li2",
  ButtonNote->"Li2"],
 "."
}], "Text",
 CellTags->"Li3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Li3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li3", "[", "x", "]"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1123]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"3","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1123]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Li3", "//", "StandardForm"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1124]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", "#1"}], "]"}], "&"}]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1124]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"D", "[", 
  RowBox[{
   RowBox[{"Li3", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1125]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   TemplateBox[{"2","x"},
    "PolyLog"], "x"], TraditionalForm]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1125]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"\[Integral]", 
  RowBox[{
   FractionBox[
    RowBox[{"Li2", "[", "x", "]"}], "x"], 
   RowBox[{"\[DifferentialD]", "x"}]}]}]], "Input",
 CellTags->"Li3",
 CellLabel->"In[1126]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"3","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellTags->"Li3",
 CellLabel->"Out[1126]="]
}, Open  ]]
}, Open  ]],

Cell["Loop", "ObjectName",
 CellTags->"Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Loop"],

Cell["Loop is an option indicating the number of (virtual) loops.", "Text",
 CellTags->"Loop"]
}, Open  ]],

Cell["LorentzIndex", "ObjectName",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"LorentzIndex"],

Cell["\<\
LorentzIndex[mu] denotes a four dimensional Lorentz index. For other than \
four dimensions: LorentzIndex[mu, D] or LorentzIndex[mu], etc. \
LorentzIndex[mu, 4] simplifies to LorentzIndex[mu].\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ChangeDimension",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ChangeDimension", All, CellTags],
  ButtonData->"ChangeDimension",
  ButtonNote->"ChangeDimension"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Momentum", All, CellTags],
  ButtonData->"Momentum",
  ButtonNote->"Momentum"],
 "."
}], "Text",
 CellTags->"LorentzIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"LorentzIndex"],

Cell["This denotes a four-dimensional Lorentz index.", "Text",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1127]:="],

Cell[BoxData[
 FormBox["\[Alpha]", TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1127]="]
}, Open  ]],

Cell["\<\
An optional second argument can be given for a dimension different from 4.\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"LorentzIndex", "[", 
  RowBox[{"\[Alpha]", ",", "n"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1128]:="],

Cell[BoxData[
 FormBox["\[Alpha]", TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1128]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["LorentzIndex[ComplexIndex[\[Alpha]]]", "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1129]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Alpha]", "*"], TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1129]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1130]:="],

Cell[BoxData[
 FormBox["\[Mu]", TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1130]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ComplexConjugate", "[", 
  RowBox[{"LorentzIndex", "[", 
   RowBox[{"ComplexIndex", "[", "\[Beta]", "]"}], "]"}], "]"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1131]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Beta]", "*"], TraditionalForm]], "Output",
 CellTags->"LorentzIndex",
 CellLabel->"Out[1131]="]
}, Open  ]],

Cell["\<\
Setting $LorentzIndices=True displays the dimension (if different from 4) as \
a subindex.\
\>", "Text",
 CellTags->"LorentzIndex"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1132]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Nu]", ",", "n"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "4"}], "]"}]}], "}"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1133]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SubscriptBox[
     FormBox["\[Alpha]",
      TraditionalForm], 
     FormBox["D",
      TraditionalForm]], ",", 
    SubscriptBox[
     FormBox["\[Nu]",
      TraditionalForm], 
     FormBox["n",
      TraditionalForm]], ",", "\[Beta]"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"LorentzIndex",
 CellLabel->"Out[1133]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", " ", "=", " ", "False"}], ";"}]], "Input",
 CellTags->"LorentzIndex",
 CellLabel->"In[1134]:="]
}, Open  ]],

Cell["Lower", "ObjectName",
 CellTags->"Lower"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Lower"],

Cell["\<\
Lower may be used inside LorentzIndex to indicate an covariant LorentzIndex.\
\>", "Text",
 CellTags->"Lower"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LorentzIndex", All, CellTags],
  ButtonData->"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Upper",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Upper", All, CellTags],
  ButtonData->"Upper",
  ButtonNote->"Upper"],
 ", ",
 ButtonBox["Contract1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract1", All, CellTags],
  ButtonData->"Contract1",
  ButtonNote->"Contract1"],
 "."
}], "Text",
 CellTags->"Lower"]
}, Open  ]],

Cell["MakeContext", "ObjectName",
 CellTags->"MakeContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MakeContext"],

Cell["\<\
MakeContext[string] constructs the context path of string. MakeContext is \
invoked at startup of FeynCalc. MakeContext[a, b] construct the context path \
of b defined in the context of a.\
\>", "Text",
 CellTags->"MakeContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc", All, CellTags],
  ButtonData->"FeynCalc",
  ButtonNote->"FeynCalc"],
 ", ",
 ButtonBox["CheckContext",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CheckContext", All, CellTags],
  ButtonData->"CheckContext",
  ButtonNote->"CheckContext"],
 ", ",
 ButtonBox["$FeynCalcStuff",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$FeynCalcStuff", All, CellTags],
  ButtonData->"$FeynCalcStuff",
  ButtonNote->"$FeynCalcStuff"],
 ", ",
 ButtonBox["Load",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Load", All, CellTags],
  ButtonData->"Load",
  ButtonNote->"Load"],
 "."
}], "Text",
 CellTags->"MakeContext"]
}, Open  ]],

Cell["Mandelstam", "ObjectName",
 CellTags->"Mandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Mandelstam"],

Cell[TextData[{
 "Mandelstam is an option for DiracTrace, OneLoop, OneLoopSum, Tr and \
TrickMandelstam.  A typical setting is Mandelstam \[Rule] {",
 "s",
 ", ",
 "t",
 ", ",
 "u",
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
 "}, which stands for  ",
 "s",
 " + ",
 "t",
 " + ",
 "u",
 " = ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "1"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "2"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "3"], "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox[
     StyleBox["m",
      FontSlant->"Plain"], "4"], "2"], TraditionalForm]]],
 ". If other than four-particle processes are calculated the setting should \
be: Mandelstam \[Rule] {}."
}], "Text",
 CellTags->"Mandelstam"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["TrickMandelstam",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TrickMandelstam", All, CellTags],
  ButtonData->"TrickMandelstam",
  ButtonNote->"TrickMandelstam"],
 ", ",
 ButtonBox["SetMandelstam",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SetMandelstam", All, CellTags],
  ButtonData->"SetMandelstam",
  ButtonNote->"SetMandelstam"],
 "."
}], "Text",
 CellTags->"Mandelstam"]
}, Open  ]],

Cell["Map2", "ObjectName",
 CellTags->"Map2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Map2"],

Cell["\<\
Map2[f, exp] is equivalent to Map if Nterms[exp] > 0, otherwise Map2[f, exp] \
gives f[exp].\
\>", "Text",
 CellTags->"Map2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["NTerms",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "NTerms", All, CellTags],
  ButtonData->"NTerms",
  ButtonNote->"NTerms"],
 "."
}], "Text",
 CellTags->"Map2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Map2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", 
   RowBox[{"a", "-", "b"}]}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1135]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "a", ")"}], "+", 
   RowBox[{"f", "(", 
    RowBox[{"-", "b"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1135]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", "x"}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1136]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1136]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1137]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1137]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Map2", "[", 
  RowBox[{"f", ",", "1"}], "]"}]], "Input",
 CellTags->"Map2",
 CellLabel->"In[1138]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "1", ")"}], TraditionalForm]], "Output",
 CellTags->"Map2",
 CellLabel->"Out[1138]="]
}, Open  ]]
}, Open  ]],

Cell["MemoryAvailable", "ObjectName",
 CellTags->"MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MemoryAvailable"],

Cell["\<\
MemoryAvailable is an option of MemSet. It can be set to an integer n, where \
n is the available amount of main memory in Mega Byte. The default setting is \
$MemoryAvailable.\
\>", "Text",
 CellTags->"MemoryAvailable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MemSet",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MemSet", All, CellTags],
  ButtonData->"MemSet",
  ButtonNote->"MemSet"],
 ", ",
 ButtonBox["$MemoryAvailable",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$MemoryAvailable", All, CellTags],
  ButtonData->"$MemoryAvailable",
  ButtonNote->"$MemoryAvailable"],
 "."
}], "Text",
 CellTags->"MemoryAvailable"]
}, Open  ]],

Cell["MemSet", "ObjectName",
 CellTags->"MemSet"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MemSet"],

Cell[TextData[{
 "MemSet[f[x_], body] is like f[x_] := f[x] = body, but dependend on the \
value of the setting of MemoryAvailable \[Rule] memorycut (memorycut - \
MemoryInUse[]/",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["10", "6"], TraditionalForm]]],
 ") MemSet[f[x_], body] may evaluate as f[x_] := body."
}], "Text",
 CellTags->"MemSet"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["MemoryAvailable",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MemoryAvailable", All, CellTags],
  ButtonData->"MemoryAvailable",
  ButtonNote->"MemoryAvailable"],
 "."
}], "Text",
 CellTags->"MemSet"]
}, Open  ]],

Cell["MetricTensor", "ObjectName",
 CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MetricTensor"],

Cell[TextData[{
 "MetricTensor[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor. The default dimension is 4."
}], "Text",
 CellTags->"MetricTensor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MT", All, CellTags],
  ButtonData->"MT",
  ButtonNote->"MT"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MTD", All, CellTags],
  ButtonData->"MTD",
  ButtonNote->"MTD"],
 "."
}], "Text",
 CellTags->"MetricTensor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MetricTensor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1139]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1139]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "*", " ", "%"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1140]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1140]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
   RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1141]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1141]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "*", " ", "%"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1142]:="],

Cell[BoxData[
 FormBox["n", TraditionalForm]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1142]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1143]:="],

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1143]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"a", ",", "b", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1144]:="],

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"a", ",", "b", ",", 
   RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1144]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"MetricTensor", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1145]:="],

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1145]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", 
   RowBox[{"MetricTensor", "[", 
    RowBox[{"a", ",", "b", ",", 
     RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "]"}], "]"}]], "Input",
 CellTags->"MetricTensor",
 CellLabel->"In[1146]:="],

Cell[BoxData[
 RowBox[{"MetricTensor", "[", 
  RowBox[{"a", ",", "b", ",", 
   RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}]], "Output",
 CellTags->"MetricTensor",
 CellLabel->"Out[1146]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MLimit", "ObjectName",
 CellTags->"MLimit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MLimit"],

Cell["\<\
MLimit[expr, lims] takes multiple limits of expr using the limits lims.\
\>", "Text",
 CellTags->"MLimit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MLimit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MLimit", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"y", " ", 
     RowBox[{"Log", "[", "y", "]"}]}], "+", 
    RowBox[{
     RowBox[{"Sin", "[", 
      RowBox[{"x", "-", "1"}], "]"}], "/", 
     RowBox[{"(", 
      RowBox[{"x", "-", "1"}], ")"}]}]}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", "1"}], ",", 
     RowBox[{"y", "\[Rule]", "0"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"MLimit",
 CellLabel->"In[1147]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"MLimit",
 CellLabel->"Out[1147]="]
}, Open  ]]
}, Open  ]],

Cell["Momentum", "ObjectName",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Momentum"],

Cell["\<\
Momentum[p] is the head of a four momentum (p). The internal representation \
of a four-dimensional p is Momentum[p]. For other than four dimensions: \
Momentum[p, dim]. Momentum[p, 4] simplifies to Momentum[p].\
\>", "Text",
 CellTags->"Momentum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracGamma",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracGamma", All, CellTags],
  ButtonData->"DiracGamma",
  ButtonNote->"DiracGamma"],
 ", ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Eps", All, CellTags],
  ButtonData->"Eps",
  ButtonNote->"Eps"],
 ", ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LorentzIndex", All, CellTags],
  ButtonData->"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumExpand", All, CellTags],
  ButtonData->"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 "."
}], "Text",
 CellTags->"Momentum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Momentum"],

Cell["This is a four dimensional momentum.", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Momentum", "[", "p", "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1148]:="],

Cell[BoxData[
 FormBox["p", TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1148]="]
}, Open  ]],

Cell["\<\
As an optional second argument the dimension must be specified if it is \
different from 4.\
\>", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "D"}], "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1149]:="],

Cell[BoxData[
 FormBox["p", TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1149]="]
}, Open  ]],

Cell["The dimension index is supressed in the output.", "Text",
 CellTags->"Momentum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", ",", "d"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1150]:="],

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "d"}], "]"}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1150]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "=", 
  RowBox[{"Momentum", "[", 
   RowBox[{"-", "q"}], "]"}]}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1151]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "q"}], TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1151]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1152]:="],

Cell[BoxData[
 RowBox[{"-", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1152]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], " ", "+", " ", 
   RowBox[{"Momentum", "[", 
    RowBox[{"2", "q"}], "]"}]}]}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1153]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"p", "-", "q"}], ")"}], "+", 
   RowBox[{"2", " ", "q"}]}], TraditionalForm]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1153]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1154]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1154]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a2", "//", "MomentumExpand"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1155]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "+", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1155]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"a2", "//", "MomentumCombine"}], "//", "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1156]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1156]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", "d"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1157]:="],

Cell[BoxData[
 RowBox[{"Momentum", "[", 
  RowBox[{"p", ",", "d"}], "]"}]], "Output",
 CellTags->"Momentum",
 CellLabel->"Out[1157]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2"}], "]"}]], "Input",
 CellTags->"Momentum",
 CellLabel->"In[1158]:="]
}, Open  ]],

Cell["MomentumCombine", "ObjectName",
 CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumCombine"],

Cell["\<\
MomentumCombine[expr] is the inverse operation to MomentumExpand and \
ExpandScalarProduct. MomentumCombine combines also Pair`s.\
\>", "Text",
 CellTags->"MomentumCombine"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandScalarProduct", All, CellTags],
  ButtonData->"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["Momentum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Momentum", All, CellTags],
  ButtonData->"Momentum",
  ButtonNote->"Momentum"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumExpand", All, CellTags],
  ButtonData->"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["MomentumCombine2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumCombine2", All, CellTags],
  ButtonData->"MomentumCombine2",
  ButtonNote->"MomentumCombine2"],
 "."
}], "Text",
 CellTags->"MomentumCombine"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumCombine"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"Momentum", "[", "q", "]"}]}]}], " ", "//", " ", 
   "MomentumCombine"}], " ", "//", " ", "StandardForm"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1159]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "-", 
  RowBox[{"2", " ", 
   RowBox[{"Momentum", "[", "q", "]"}]}]}]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1159]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "+", " ", 
   RowBox[{"2", " ", 
    RowBox[{"FV", "[", 
     RowBox[{"q", ",", "\[Mu]"}], "]"}]}], " "}]}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1160]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1160]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumCombine", "[", "t1", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1161]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["p", "\[Mu]"], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox["q", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1161]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1162]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{"p", ",", "\[Mu]"}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"FV", "[", 
    RowBox[{"q", ",", "\[Mu]"}], "]"}]}]}]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1162]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine", "[", "t1", "]"}], "//", 
  "ExpandScalarProduct"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1163]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1163]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1164]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]}]], "Output",
 CellTags->"MomentumCombine",
 CellLabel->"Out[1164]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"MomentumCombine",
 CellLabel->"In[1165]:="]
}, Open  ]],

Cell["MomentumCombine2", "ObjectName",
 CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumCombine2"],

Cell["\<\
MomentumCombine2[expr]  is the inverse operation to MomentumExpand and \
ExpandScalarProduct. MomentumCombine2 combines also FourVectors.\
\>", "Text",
 CellTags->"MomentumCombine2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumCombine", All, CellTags],
  ButtonData->"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 ", ",
 ButtonBox["MomentumExpand",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumExpand", All, CellTags],
  ButtonData->"MomentumExpand",
  ButtonNote->"MomentumExpand"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandScalarProduct", All, CellTags],
  ButtonData->"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourVector", All, CellTags],
  ButtonData->"FourVector",
  ButtonNote->"FourVector"],
 "."
}], "Text",
 CellTags->"MomentumCombine2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumCombine2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"3", " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}], "+", 
  RowBox[{"2", " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]}]], "Input",
 CellTags->"MomentumCombine2",
 CellLabel->"In[1166]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"3", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumCombine2",
 CellLabel->"Out[1166]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumCombine2", "[", "%", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MomentumCombine2",
 CellLabel->"In[1167]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{
     RowBox[{"3", " ", "p"}], "+", 
     RowBox[{"2", " ", "q"}]}], "]"}]}], "]"}]], "Output",
 CellTags->"MomentumCombine2",
 CellLabel->"Out[1167]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MomentumExpand", "ObjectName",
 CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MomentumExpand"],

Cell["\<\
MomentumExpand[expr] expands Momentum[a+b+ ...] in expr into Momentum[a] + \
Momentum[b] + ... .\
\>", "Text",
 CellTags->"MomentumExpand"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandScalarProduct", All, CellTags],
  ButtonData->"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["MomentumCombine",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MomentumCombine", All, CellTags],
  ButtonData->"MomentumCombine",
  ButtonNote->"MomentumCombine"],
 "."
}], "Text",
 CellTags->"MomentumExpand"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MomentumExpand"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MomentumExpand", "[", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1168]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Momentum", "[", "p", "]"}], "+", 
  RowBox[{"Momentum", "[", "q", "]"}]}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1168]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{
   RowBox[{"p", "+", "q"}], ",", "r"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1169]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "+", "q"}],
    TraditionalForm], ")", "\[CenterDot]", 
   FormBox["r",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1169]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1170]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "+", "q"}], "]"}], ",", 
   RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1170]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumExpand", "[", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "+", "q"}], ",", "r"}], "]"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1171]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   TagBox[
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")", ".", "(", 
   TagBox[
    FormBox["r",
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1171]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1172]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "+", 
    RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
   RowBox[{"Momentum", "[", "r", "]"}]}], "]"}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1172]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MomentumExpand", "[", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{"p", "+", "q"}], ",", 
    RowBox[{"r", "-", "p"}]}], "]"}], "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1173]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   TagBox[
    FormBox[
     RowBox[{"p", "+", "q"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")", ".", "(", 
   TagBox[
    FormBox[
     RowBox[{"r", "-", "p"}],
     TraditionalForm],
    TraditionalForm,
    Editable->True], ")"}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1173]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1174]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], "+", 
    RowBox[{"Momentum", "[", "q", "]"}]}], ",", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"Momentum", "[", "p", "]"}]}], "+", 
    RowBox[{"Momentum", "[", "r", "]"}]}]}], "]"}]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1174]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "%", "]"}]], "Input",
 CellTags->"MomentumExpand",
 CellLabel->"In[1175]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "+", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"MomentumExpand",
 CellLabel->"Out[1175]="]
}, Open  ]]
}, Open  ]],

Cell["MT", "ObjectName",
 CellTags->"MT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MT"],

Cell[TextData[{
 "MT[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor in 4 dimensions."
}], "Text",
 CellTags->"MT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MetricTensor", All, CellTags],
  ButtonData->"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MTD", All, CellTags],
  ButtonData->"MTD",
  ButtonNote->"MTD"],
 "."
}], "Text",
 CellTags->"MT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1176]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1176]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"MT", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1177]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1177]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1178]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1178]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"MT", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1179]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1179]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"FCI", "[", 
    RowBox[{"MT", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MT",
 CellLabel->"In[1180]:="],

Cell[BoxData[
 RowBox[{"MT", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"MT",
 CellLabel->"Out[1180]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["MTD", "ObjectName",
 CellTags->"MTD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"MTD"],

Cell[TextData[{
 "MTD[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", "\[Nu]"}], TraditionalForm]]],
 "] is the metric tensor in D dimensions."
}], "Text",
 CellTags->"MTD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalcExternal",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalcExternal", All, CellTags],
  ButtonData->"FeynCalcExternal",
  ButtonNote->"FeynCalcExternal"],
 ", ",
 ButtonBox["FCE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCE", All, CellTags],
  ButtonData->"FCE",
  ButtonNote->"FCE"],
 ", ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MetricTensor", All, CellTags],
  ButtonData->"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MT", All, CellTags],
  ButtonData->"MT",
  ButtonNote->"MT"],
 "."
}], "Text",
 CellTags->"MTD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"MTD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1181]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1181]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}], "]"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1182]:="],

Cell[BoxData[
 FormBox["D", TraditionalForm]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1182]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MTD", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1183]:="],

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1183]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCI", "[", 
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1184]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1184]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"FCI", "[", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"MTD",
 CellLabel->"In[1185]:="],

Cell[BoxData[
 RowBox[{"MTD", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Output",
 CellTags->"MTD",
 CellLabel->"Out[1185]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["NegativeInteger", "ObjectName",
 CellTags->"NegativeInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NegativeInteger"],

Cell["\<\
NegativeInteger is a data type. E.g. DataType[n, NegativeInteger] can be set \
to True.\
\>", "Text",
 CellTags->"NegativeInteger"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"NegativeInteger"]
}, Open  ]],

Cell["Nf", "ObjectName",
 CellTags->"Nf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Nf"],

Cell["Nf denotes the number of flavors.", "Text",
 CellTags->"Nf"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Amplitude", All, CellTags],
  ButtonData->"Amplitude",
  ButtonNote->"Amplitude"],
 ", ",
 ButtonBox["CounterTerm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CounterTerm", All, CellTags],
  ButtonData->"CounterTerm",
  ButtonNote->"CounterTerm"],
 ", ",
 ButtonBox["Convolute",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Convolute", All, CellTags],
  ButtonData->"Convolute",
  ButtonNote->"Convolute"],
 ", ",
 ButtonBox["AnomalousDimension",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "AnomalousDimension", All, CellTags],
  ButtonData->"AnomalousDimension",
  ButtonNote->"AnomalousDimension"],
 ", ",
 ButtonBox["SplittingFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SplittingFunction", All, CellTags],
  ButtonData->"SplittingFunction",
  ButtonNote->"SplittingFunction"],
 "."
}], "Text",
 CellTags->"Nf"]
}, Open  ]],

Cell["Nielsen", "ObjectName",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Nielsen"],

Cell["Nielsen[i,j, x] denotes Nielsen's polylogarithm.", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Nielsen", "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1186]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PolyLog", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"Nielsen",
 CellLabel->"Out[1186]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SimplifyPolyLog", All, CellTags],
  ButtonData->"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Nielsen"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "2", ",", "x"}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1187]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", 
    RowBox[{
     FormBox["1",
      TraditionalForm], 
     FormBox["2",
      TraditionalForm]}]], "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1187]="]
}, Open  ]],

Cell["\<\
Numerical evaluation is done via N[Nielsen[n_,p_,x_]] := \
(-1)^(n+p-1)/(n-1)!/p! NIntegrate[Log[1-x t]^p Log[t]^(n-1)/t,{t,0,1}];\
\>", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", 
  RowBox[{"Nielsen", "[", 
   RowBox[{"1", ",", "2", ",", ".45"}], "]"}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1188]:="],

Cell[BoxData[
 FormBox["0.07287162612104658`", TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1188]="]
}, Open  ]],

Cell["Some special values are built in.", "Text",
 CellTags->"Nielsen"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", "0"}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"-", "1"}]}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"1", "/", "2"}]}], "]"}], ",", 
   RowBox[{"Nielsen", "[", 
    RowBox[{"1", ",", "2", ",", "1"}], "]"}]}], "}"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1189]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"0", ",", 
    FractionBox[
     TemplateBox[{"3"},
      "Zeta"], "8"], ",", 
    FractionBox[
     TemplateBox[{"3"},
      "Zeta"], "8"], ",", 
    TemplateBox[{"3"},
     "Zeta"]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1189]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "2", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1190]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   TemplateBox[{"3"},
    "Zeta"]}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1190]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"1", ",", "3", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1191]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    TemplateBox[{"4",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    RowBox[{"log", "(", "x", ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   FractionBox[
    SuperscriptBox["\[Pi]", "4"], "90"]}], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1191]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Nielsen", "[", 
  RowBox[{"3", ",", "1", ",", "x", ",", 
   RowBox[{"PolyLog", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Nielsen",
 CellLabel->"In[1192]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"4","x"},
   "PolyLog"], TraditionalForm]], "Output",
 CellTags->"Nielsen",
 CellLabel->"Out[1192]="]
}, Open  ]]
}, Open  ]],

Cell["NonCommFreeQ", "ObjectName",
 CellTags->"NonCommFreeQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommFreeQ"],

Cell["\<\
NonCommFreeQ[exp] yields True if exp contains no non-commutative objects \
(i.e. those objects which are listed in $NonComm) or only non-commutative \
objects inside DiracTrace's or SUNTrace's.\
\>", "Text",
 CellTags->"NonCommFreeQ"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$NonComm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$NonComm", All, CellTags],
  ButtonData->"$NonComm",
  ButtonNote->"$NonComm"],
 ", ",
 ButtonBox["NonCommQ",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "NonCommQ", All, CellTags],
  ButtonData->"NonCommQ",
  ButtonNote->"NonCommQ"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNTrace", All, CellTags],
  ButtonData->"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"NonCommFreeQ"]
}, Open  ]],

Cell["NonCommQ", "ObjectName",
 CellTags->"NonCommQ"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommQ"],

Cell["\<\
NonCommQ[exp] yields True if exp contains non-commutative objects (i.e. those \
objects which are listed in $NonComm) not inside DiracTrace's or SUNTrace's.\
\>", "Text",
 CellTags->"NonCommQ"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$NonComm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$NonComm", All, CellTags],
  ButtonData->"$NonComm",
  ButtonNote->"$NonComm"],
 ", ",
 ButtonBox["NonCommFreeQ",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "NonCommFreeQ", All, CellTags],
  ButtonData->"NonCommFreeQ",
  ButtonNote->"NonCommFreeQ"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNTrace", All, CellTags],
  ButtonData->"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"NonCommQ"]
}, Open  ]],

Cell["NonCommutative", "ObjectName",
 CellTags->"NonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NonCommutative"],

Cell["\<\
NonCommutative is a data type which may be used, e.g.,  as: DataType[x, \
NonCommutative] = True.\
\>", "Text",
 CellTags->"NonCommutative"],

Cell[TextData[{
 "See also:",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeclareNonCommutative", All, CellTags],
  ButtonData->"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"NonCommutative"]
}, Open  ]],

Cell["NTerms", "ObjectName",
 CellTags->"NTerms"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NTerms"],

Cell["\<\
NTerms[x] is equivalent to Length if x is a sum; otherwise NTerms[x] returns \
1, except NTerms[0] \[Rule] 0.\
\>", "Text",
 CellTags->"NTerms"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"NTerms"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", 
  RowBox[{"a", "-", "b"}], "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1193]:="],

Cell[BoxData[
 FormBox["2", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1193]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", 
  RowBox[{"a", " ", "b", " ", "c"}], "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1194]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1194]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", "9", "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1195]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1195]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NTerms", "[", "0", "]"}]], "Input",
 CellTags->"NTerms",
 CellLabel->"In[1196]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"NTerms",
 CellLabel->"Out[1196]="]
}, Open  ]]
}, Open  ]],

Cell["NumberOfMetricTensors", "ObjectName",
 CellTags->"NumberOfMetricTensors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumberOfMetricTensors"],

Cell["NumberOfMetricTensors is an option of Tdec.", "Text",
 CellTags->"NumberOfMetricTensors"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Tdec",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tdec", All, CellTags],
  ButtonData->"Tdec",
  ButtonNote->"Tdec"],
 "."
}], "Text",
 CellTags->"NonCommQ"]
}, Open  ]],

Cell["NumericalFactor", "ObjectName",
 CellTags->"NumericalFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumericalFactor"],

Cell["\<\
NumericalFactor[expr] gives the overall numerical factor of expr.\
\>", "Text",
 CellTags->"NumericalFactor"]
}, Open  ]],

Cell["NumericQ1", "ObjectName",
 CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"NumericQ1"],

Cell["\<\
NumericQ1[x,{a,b,..}] is like NumericQ, but assumes that {a,b,..} are numeric \
quantities.\
\>", "Text",
 CellTags->"NumericQ1"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"NumericQ1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ", "[", 
  RowBox[{
   RowBox[{"3", " ", "a"}], "+", 
   RowBox[{"Log", "[", "b", "]"}], "+", 
   SuperscriptBox["c", "2"]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1197]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"NumericQ1",
 CellLabel->"Out[1197]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ1", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"Log", "[", "b", "]"}], "+", 
    SuperscriptBox["c", "2"]}], ",", 
   RowBox[{"{", "}"}]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1198]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"NumericQ1",
 CellLabel->"Out[1198]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"NumericQ1", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"3", " ", "a"}], "+", 
    RowBox[{"Log", "[", "b", "]"}], "+", 
    SuperscriptBox["c", "2"]}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"NumericQ1",
 CellLabel->"In[1199]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"NumericQ1",
 CellLabel->"Out[1199]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["OneLoop  ***unfinished***", "ObjectName",
 CellTags->"OneLoop"],

Cell[TextData[{
 "See also the old FeynCalc documentation at ",
 ButtonBox["http://www.mertig.com/oldfc/",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.mertig.com/oldfc/"], None}],
 ". NOTICE: While OneLoop is restricted to 't Hooft Feynman gauge the \
function ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 " does not have this restriction (but is usually slower). OneLoop handles \
selfenergies, vertex and box-graphs (those only up to 3rd rank tensor in the \
integration variable)."
}], "Text",
 CellTags->"OneLoop"],

Cell[TextData[{
 "WARNING: If you encounter anomalies: ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
    "The", " ", "default", " ", "setting", " ", "of", " ", "$West", " ", "is",
      " ", "True"}], ",", " ", 
    RowBox[{"i", ".", "e", "."}], ",", " ", 
    RowBox[{"the", " ", "way", " ", 
     RowBox[{"tr", "(", 
      RowBox[{
       SuperscriptBox["\[Gamma]", "\[Mu]"], 
       SuperscriptBox["\[Gamma]", "\[Nu]"], 
       SuperscriptBox["\[Gamma]", "\[Rho]"], 
       SuperscriptBox["\[Gamma]", "\[Sigma]"], 
       SuperscriptBox["\[Gamma]", "\[Tau]"], 
       SuperscriptBox["\[Gamma]", "\[Lambda]"], 
       SuperscriptBox["\[Gamma]", "5"]}], ")"}]}]}], TraditionalForm]]],
 " is calculated in D dimensions has changed compared to the old FeynCalc \
version. Please keep in mind that the issue of ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "schemes is inherintly tricky."
}], "Text",
 CellTags->"OneLoop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoop"],

Cell[TextData[{
 "OneLoop[",
 "q",
 ", amplitude] calculates the one-loop Feynman diagram amplitude (",
 StyleBox["n",
  FontSlant->"Italic"],
 "-point, where ",
 StyleBox["n",
  FontSlant->"Italic"],
 "\[LessEqual]4 and the highest tensor rank of the integration momenta (after \
cancellation of scalar products) may be 3; unless OneLoopSimplify is used)."
}], "Text",
 CellTags->"OneLoop"],

Cell["\<\
The argument q denotes the integration variable, i.e., the loop momentum. \
OneLoop[name, q, amplitude] has as first argument a name of the amplitude. If \
the second argument has head FeynAmp then OneLoop[q, FeynAmp[name, k, expr]] \
and OneLoop[FeynAmp[name, k, expr]] tranform to OneLoop[name, k, expr].\
\>", "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OneLoop", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1200]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Apart2", "\[Rule]", "True"}], ",", 
    RowBox[{"CancelQP", "\[Rule]", "True"}], ",", 
    RowBox[{"DenominatorOrder", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
    RowBox[{"InitialSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IntermediateSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"OneLoopSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"Prefactor", "\[Rule]", "1"}], ",", 
    RowBox[{"ReduceGamma", "\[Rule]", "False"}], ",", 
    RowBox[{"ReduceToScalars", "\[Rule]", "False"}], ",", 
    RowBox[{"SmallVariables", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"WriteOut", "\[Rule]", "False"}], ",", 
    RowBox[{"WriteOutPaVe", "\[Rule]", "False"}], ",", 
    RowBox[{"Sum", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1200]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["B0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "B0", All, CellTags],
  ButtonData->"B0",
  ButtonNote->"B0"],
 ", ",
 ButtonBox["C0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "C0", All, CellTags],
  ButtonData->"C0",
  ButtonNote->"C0"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "D0", All, CellTags],
  ButtonData->"D0",
  ButtonNote->"D0"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TID", All, CellTags],
  ButtonData->"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TIDL", All, CellTags],
  ButtonData->"TIDL",
  ButtonNote->"TIDL"],
 ", ",
 ButtonBox["$LimitTo4",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$LimitTo4", All, CellTags],
  ButtonData->"$LimitTo4",
  ButtonNote->"$LimitTo4"],
 "."
}], "Text",
 CellTags->"OneLoop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"-", "I"}], "/", 
    RowBox[{"Pi", "^", "2"}]}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{"{", 
     RowBox[{"q", ",", "m"}], "}"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1201]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["\[ImaginaryI]", 
    RowBox[{
     SuperscriptBox["\[Pi]", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1201]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q", ",", "%"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1202]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", 
   FormBox[
    SuperscriptBox["m", "2"],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1202]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"mf", "/:", 
   RowBox[{"MakeBoxes", "[", 
    RowBox[{"mf", ",", "TraditionalForm"}], "]"}], "=", 
   RowBox[{"InterpretationBox", "[", 
    RowBox[{
     RowBox[{"SubscriptBox", "[", 
      RowBox[{"\"\<m\>\"", ",", "\"\<f\>\""}], "]"}], ",", "mf"}], "]"}]}], 
  ";"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1203]:="],

Cell[TextData[{
 "Remember that FAD[{q,mf},{q-k,mf}] is a fast possibility to enter ",
 Cell[BoxData[
  FormBox[
   RowBox[{"1", "/", 
    RowBox[{
     RowBox[{"(", " ", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["q", "2"], "-", 
         SuperscriptBox["mf", "2"]}], ")"}], " ", 
       RowBox[{"(", " ", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
         SuperscriptBox["mf", "2"]}], ")"}]}], " ", ")"}], "."}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"I", " ", 
    FractionBox[
     SuperscriptBox["el", "2"], 
     RowBox[{"16", " ", 
      SuperscriptBox["Pi", "4"], 
      RowBox[{"(", 
       RowBox[{"1", "-", "D"}], ")"}]}]], 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "mf"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q", "-", "k"}], ",", "mf"}], "}"}]}], "]"}], 
    RowBox[{"DiracTrace", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"mf", "+", 
        RowBox[{"GS", "[", 
         RowBox[{"q", "-", "k"}], "]"}]}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
      RowBox[{"(", 
       RowBox[{"mf", "+", 
        RowBox[{"GS", "[", "q", "]"}]}], ")"}], ".", 
      RowBox[{"GA", "[", "\[Mu]", "]"}]}], "]"}]}], " ", "//", " ", 
   "FCI"}]}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1204]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    SuperscriptBox["el", "2"], " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "+", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", "(", 
          FormBox[
           RowBox[{"q", "-", "k"}],
           TraditionalForm], ")"}]}], ")"}], ".", 
       SuperscriptBox["\[Gamma]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], ".", 
       RowBox[{"(", 
        RowBox[{
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], "+", 
         RowBox[{
          FormBox["\<\"\[Gamma]\"\>",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], ")"}], ".", 
       SuperscriptBox["\[Gamma]", 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}], 
   RowBox[{"16", " ", 
    SuperscriptBox["\[Pi]", "4"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", "D"}], ")"}], " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], ")"}], "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q", "-", "k"}], ")"}], "2"], "-", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], ")"}], "2"]}], ")"}]}]}]], 
  TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1204]="]
}, Open  ]],

Cell["\<\
The input to OneLoop may be in 4 dimensions, since the function changes the \
dimension of the objects automatically to the setting of the Dimension option \
(D by default).\
\>", "Text",
 CellTags->"OneLoop"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"q", ",", "t"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1205]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["el", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"-", 
        RowBox[{
         FractionBox["8", "3"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"], " ", 
         RowBox[{
          SubscriptBox["B", "0"], "(", 
          SuperscriptBox[
           FormBox["k",
            TraditionalForm], "2"], ",", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"], ",", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"], ")"}]}]}], "-", 
       RowBox[{
        FractionBox["4", "3"], " ", 
        SuperscriptBox[
         FormBox["k",
          TraditionalForm], "2"], " ", 
        RowBox[{
         SubscriptBox["B", "0"], "(", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"], ",", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"], ",", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"], ")"}]}], "+", 
       RowBox[{
        FractionBox["8", "3"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
          TraditionalForm], "(", 
         FormBox[
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"],
          TraditionalForm], ")"}]}], "-", 
       FractionBox[
        RowBox[{"8", " ", 
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"]}], "3"], "+", 
       FractionBox[
        RowBox[{"4", " ", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"]}], "9"]}], ")"}]}], 
    RowBox[{"16", " ", 
     SuperscriptBox["\[Pi]", "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1205]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullSimplify", "[", "%", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1206]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["el", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"3", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", 
          SuperscriptBox[
           RowBox[{"(", 
            InterpretationBox[
             SubscriptBox["m", "f"],
             $CellContext`mf], ")"}], "2"]}], "+", 
         SuperscriptBox[
          FormBox["k",
           TraditionalForm], "2"]}], ")"}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["k",
          TraditionalForm], "2"], ",", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], ")"}], "2"], ",", 
        SuperscriptBox[
         RowBox[{"(", 
          InterpretationBox[
           SubscriptBox["m", "f"],
           $CellContext`mf], ")"}], "2"], ")"}]}], "-", 
      RowBox[{"6", " ", 
       RowBox[{
        FormBox[
         SubscriptBox["\<\"A\"\>", "\<\"0\"\>"],
         TraditionalForm], "(", 
        FormBox[
         SuperscriptBox[
          RowBox[{"(", 
           InterpretationBox[
            SubscriptBox["m", "f"],
            $CellContext`mf], ")"}], "2"],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{"6", " ", 
       SuperscriptBox[
        RowBox[{"(", 
         InterpretationBox[
          SubscriptBox["m", "f"],
          $CellContext`mf], ")"}], "2"]}], "-", 
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], "2"]}], ")"}]}], 
   RowBox[{"36", " ", 
    SuperscriptBox["\[Pi]", "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1206]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"k", ",", "r"}], "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1207]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1207]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoop", "[", 
  RowBox[{"k", ",", "%"}], "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1208]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SuperscriptBox["\[Pi]", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        RowBox[{
         SubscriptBox["B", "0"], "(", "0", ",", 
         SuperscriptBox["m", "2"], ",", 
         SuperscriptBox["m", "2"], ")"}], " ", 
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}], 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"]}]]}], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        SubscriptBox["B", "0"], "(", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], ",", "0", ",", 
        SuperscriptBox["m", "2"], ")"}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]], "+", 
     RowBox[{
      FractionBox["1", "2"], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], ",", "0", ",", 
       SuperscriptBox["m", "2"], ")"}]}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"OneLoop",
 CellLabel->"Out[1208]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"OneLoop",
 CellLabel->"In[1209]:="]
}, Open  ]],

Cell["OneLoopSimplify", "ObjectName",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoopSimplify"],

Cell["\<\
OneLoopSimplify[amp, q] simplifies the one-loop amplitude amp. The second \
argument denotes the integration momentum.\
\>", "Text",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OneLoopSimplify", "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1210]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"DiracSimplify", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"IntegralTable", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"OPE1Loop", "\[Rule]", "False"}], ",", 
    RowBox[{"ScalarProductCancel", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1210]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TID", All, CellTags],
  ButtonData->"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TIDL", All, CellTags],
  ButtonData->"TIDL",
  ButtonNote->"TIDL"],
 "."
}], "Text",
 CellTags->"OneLoopSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OneLoopSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"k", ",", "r"}], "]"}], " ", 
   RowBox[{"FAD", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"k", ",", "m"}], "}"}], " ", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1211]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["k", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1211]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1212]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "+", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], ")"}], " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["r",
       TraditionalForm]}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["k", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["r",
      TraditionalForm]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["k", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1212]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{
   RowBox[{"%", "/.", 
    RowBox[{"m", "\[Rule]", "0"}]}], ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1213]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], 
   RowBox[{"2", " ", 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], TraditionalForm]], "Output",\

 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1213]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", "k", ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "1"]}], ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "2"]}]}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1214]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1214]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", " ", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1215]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SubsuperscriptBox["p", 
        FormBox["2",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SubsuperscriptBox["p", 
        FormBox["1",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}]}], 
     RowBox[{"2", " ", 
      RowBox[{
       SuperscriptBox["k", "2"], ".", 
       SuperscriptBox["k", "2"], ".", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"k", "-", 
          SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SubsuperscriptBox["p", "1", "2"], " ", 
         SubsuperscriptBox["p", "2", "2"]}], "-", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], ")"}]}]]}], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubsuperscriptBox["p", "1", "2"], " ", 
       SubsuperscriptBox["p", "2", "2"], " ", 
       SubsuperscriptBox["p", 
        FormBox["1",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1215]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FCE", "[", "%", "]"}], "/.", 
   RowBox[{
    RowBox[{"SPD", "[", 
     SubscriptBox["p", "1"], "]"}], "\[Rule]", "0"}]}], "//", 
  "FCI"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1216]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "2", "2"], " ", 
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]], "-", 
   FractionBox[
    SubsuperscriptBox["p", 
     FormBox["1",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"]}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{"2", " ", 
     RowBox[{
      SuperscriptBox["k", "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"k", "-", 
         SubscriptBox["p", "2"]}], ")"}], "2"]}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1216]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OneLoopSimplify", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ",", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}]}], "]"}], " ", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"k", ",", "l"}], "]"}], "^", "2"}]}], ",", "k"}], 
  "]"}]], "Input",
 CellTags->"OneLoopSimplify",
 CellLabel->"In[1217]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"D", " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["l",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}],
       TraditionalForm], "2"]}], "+", 
    RowBox[{"2", " ", "D", " ", 
     RowBox[{
      FormBox["l",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox["l",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}]}], "+", 
    RowBox[{"D", " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["l",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"]}], "-", 
    RowBox[{
     SuperscriptBox[
      FormBox["l",
       TraditionalForm], "2"], " ", 
     SubsuperscriptBox["p", "1", "2"]}], "-", 
    RowBox[{
     SuperscriptBox[
      FormBox["l",
       TraditionalForm], "2"], " ", 
     SubsuperscriptBox["p", "2", "2"]}], "-", 
    RowBox[{"4", " ", 
     RowBox[{
      FormBox["l",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}], " ", 
     RowBox[{
      FormBox["l",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}]}], "+", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["l",
       TraditionalForm], "2"], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]}], 
   RowBox[{"4", " ", 
    RowBox[{"(", 
     RowBox[{"D", "-", "1"}], ")"}], " ", 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", 
        SubscriptBox["p", "1"], "+", 
        SubscriptBox["p", "2"]}], ")"}], "2"]}]}]], 
  TraditionalForm]], "Output",
 CellTags->"OneLoopSimplify",
 CellLabel->"Out[1217]="]
}, Open  ]]
}, Open  ]],

Cell["OneLoopSum", "ObjectName",
 CellTags->"OneLoopSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OneLoopSum"],

Cell["\<\
OneLoopSum[ FeynAmp[ ... ], FeynAmp[ ... ] , ...] will calculate a list of \
Feynman amplitudes by replacing FeynAmp step by step by OneLoop.\
\>", "Text",
 CellTags->"OneLoopSum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"OneLoopSum"]
}, Open  ]],

Cell["OPE", "ObjectName",
 CellTags->"OPE"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE"],

Cell["\<\
OPE is a convenience variable to separate OPE insertions.
OPE is also an option of several input functions like GluonPropagator.\
\>", "Text",
 CellTags->"OPE"],

Cell["\<\
The OPE variable has the property to vanish for higher powers:\
\>", "Text"],

Cell[CellGroupData[{

Cell["{OPE,OPE^2,OPE^3}", "Input",
 CellLabel->"In[1218]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\[CapitalOmega]", ",", "0", ",", "0"}], "}"}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[1218]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPE1Loop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPE1Loop", All, CellTags],
  ButtonData->"OPE1Loop",
  ButtonNote->"OPE1Loop"],
 "."
}], "Text",
 CellTags->"OPE"]
}, Open  ]],

Cell["OPEDelta", "ObjectName",
 CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEDelta"],

Cell["\<\
OPEDelta is a lightlike axial vector as used e.g. in the operator product \
expansion in QCD.\
\>", "Text",
 CellTags->"OPEDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2QuarkOperator", All, CellTags],
  ButtonData->"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"OPEDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FourVector", "[", 
  RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1219]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   FormBox["\[CapitalDelta]",
    TraditionalForm], "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"OPEDelta",
 CellLabel->"Out[1219]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{"%", " ", "%"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1220]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"OPEDelta",
 CellLabel->"Out[1220]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"OPEDelta", ",", "OPEDelta"}], "]"}]], "Input",
 CellTags->"OPEDelta",
 CellLabel->"In[1221]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"OPEDelta",
 CellLabel->"Out[1221]="]
}, Open  ]]
}, Open  ]],

Cell["OPEi, OPEj, OPEk, OPEl, OPEm, OPEn, OPEo", "ObjectName",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEi"],

Cell["\<\
OPEi, etc. are variables with DataType PositiveInteger which are used in \
functions relating to the operator product expansion.\
\>", "Text",
 CellTags->"OPEi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData["OPEi"], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1222]:="],

Cell[BoxData[
 FormBox["i", TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1222]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{
  "OPEi", ",", " ", "OPEj", ",", "OPEk", ",", "OPEl", ",", " ", "OPEm", ",", 
   " ", "OPEn", ",", " ", "OPEo", ",", " ", "PositiveInteger"}], 
  "]"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1223]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "True", ",", "True", ",", "True", ",", "True", ",", "True", ",", "True", 
    ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1223]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEi"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEj"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEk"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEl"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEm"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEn"}], ")"}]}], ",", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{"2", "OPEo"}], ")"}]}]}], "}"}], "]"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1224]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1", ",", "1"}], 
   "}"}], TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1224]="]
}, Open  ]],

Cell["Re has been changed:", "Text",
 CellTags->"OPEi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "3"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "2"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", 
    RowBox[{"-", "1"}]}], ",", "   ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEi", "]"}], ">", "1"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1225]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "3"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "2"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", 
     RowBox[{"-", "1"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "i", ")"}], ">", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1225]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "1"}], ",", 
   RowBox[{
    RowBox[{"Re", "[", 
     RowBox[{
      RowBox[{"-", "OPEi"}], " ", "+", " ", "OPEm"}], "]"}], " ", ">", " ", 
    "2"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1226]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "1"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", 
      RowBox[{"m", "-", "i"}], ")"}], ">", "2"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1226]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "3"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "2"}]}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", 
    RowBox[{"-", "1"}]}], ",", "   ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", "0"}], ",", " ", 
   RowBox[{
    RowBox[{"Re", "[", "OPEm", "]"}], ">", "1"}]}], "}"}]], "Input",
 CellTags->"OPEi",
 CellLabel->"In[1227]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "3"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "2"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", 
     RowBox[{"-", "1"}]}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", "0"}], ",", 
    RowBox[{
     RowBox[{"Re", "(", "m", ")"}], ">", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPEi",
 CellLabel->"Out[1227]="]
}, Open  ]]
}, Open  ]],

Cell["OPEInsert  ***unfinished***", "ObjectName",
 CellTags->"OPEInsert"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEInsert"],

Cell["\<\
OPEInsert[diagram_String, name_, rhifile_String] or OPEInsert[diagram_String, \
name_]. The setting of the option EpsContract (4 or D) determines whether the \
Levi-Civita tensors are contracted in 4 or D dimensions.\
\>", "Text",
 CellTags->"OPEInsert"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEInsert"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEInsert"],

Cell[BoxData[""], "Input",
 CellTags->"OPEInsert",
 CellLabel->"In[1228]:="]
}, Open  ]],

Cell["OPEInt  ***unfinished***", "ObjectName",
 CellTags->"OPEInt"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEInt"],

Cell["\<\
OPEInt[expr, q, p, x].  The dimension is changed to the one indicated by the \
option  Dimension. The setting of the option EpsContract determines  the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPEInt"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEInt"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEInt"],

Cell[BoxData[""], "Input",
 CellTags->"OPEInt",
 CellLabel->"In[1229]:="]
}, Open  ]],

Cell["OPEIntDelta  ***unfinished***", "ObjectName",
 CellTags->"OPEIntDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntDelta"],

Cell[TextData[{
 "OPEIntDelta[expr, x, m] introduces the delta(1-x) (DeltaFunction[1-x]). The \
",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntDelta by (4+Epsilon)."
}], "Text",
 CellTags->"OPEIntDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntDelta"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntDelta",
 CellLabel->"In[1230]:="]
}, Open  ]],

Cell["OPEIntegrate  ***unfinished***", "ObjectName",
 CellTags->"OPEIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntegrate"],

Cell["\<\
OPEIntegrate[expr, q, x].  The dimension is changed to the one indicated by \
the option  Dimension. The setting of the option EpsContract determines  the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPEIntegrate"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntegrate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntegrate"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntegrate",
 CellLabel->"In[1231]:="]
}, Open  ]],

Cell["OPEIntegrateDelta  ***unfinished***", "ObjectName",
 CellTags->"OPEIntegrateDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
 "OPEIntegrateDelta[expr, x, m] introduces the \[Delta](1-x) \
(DeltaFunction[1-x]). The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " Integrate function is called and each integration  (from 0 to 1) is \
recorded for reference (and bug-checking) in the global list $MIntegrate. \n\
Notice that the dimension specified by the option should also be the \
dimension used in expr. It is replaced in OPEIntegrateDelta by (4+Epsilon)."
}], "Text",
 CellTags->"OPEIntegrateDelta"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPEIntegrateDelta"],

Cell[BoxData[""], "Input",
 CellTags->"OPEIntegrateDelta",
 CellLabel->"In[1232]:="]
}, Open  ]],

Cell["OPESumExplicit", "ObjectName",
 CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESumExplicit"],

Cell["OPESumExplicit[exp] calculates OPESum's.", "Text",
 CellTags->"OPESumExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESum", All, CellTags],
  ButtonData->"OPESum",
  ButtonNote->"OPESum"],
 ", ",
 ButtonBox["OPESumSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESumSimplify", All, CellTags],
  ButtonData->"OPESumSimplify",
  ButtonNote->"OPESumSimplify"],
 "."
}], "Text",
 CellTags->"OPESumExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESumExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"A", "^", "i"}], 
     RowBox[{"B", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "i", "-", "3"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", 
      RowBox[{"m", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1233]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], 
     RowBox[{
      RowBox[{"-", "3"}], "+", "m"}]], 
    RowBox[{
     SuperscriptBox["A", "i"], " ", 
     SuperscriptBox["B", 
      RowBox[{
       RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1233]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1234]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox["A", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{"A", "-", "B"}]], "-", 
   FractionBox[
    FormBox[
     SuperscriptBox["B", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{"A", "-", "B"}]]}], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1234]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "^", "i"}], 
     RowBox[{"b", "^", 
      RowBox[{"(", 
       RowBox[{"j", "-", "i"}], ")"}]}], 
     RowBox[{"c", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", 
      RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1235]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"j", "=", "0"}], 
       RowBox[{
        RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"j", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["a", "i"], " ", 
     SuperscriptBox["b", 
      RowBox[{"j", "-", "i"}]], " ", 
     SuperscriptBox["c", 
      RowBox[{
       RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1235]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1236]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox["a", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"c", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OPESumExplicit",
 CellLabel->"Out[1236]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
 CellTags->"OPESumExplicit",
 CellLabel->"In[1237]:="]
}, Open  ]],

Cell["OPESum", "ObjectName",
 CellTags->"OPESum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESum"],

Cell["\<\
OPESum[exp, {i, 0, m}] denotes a symbolic sum.The syntax is the same as for \
Sum.\
\>", "Text",
 CellTags->"OPESum"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESumExplicit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESumExplicit", All, CellTags],
  ButtonData->"OPESumExplicit",
  ButtonNote->"OPESumExplicit"],
 ", ",
 ButtonBox["OPESumSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESumSimplify", All, CellTags],
  ButtonData->"OPESumSimplify",
  ButtonNote->"OPESumSimplify"],
 "."
}], "Text",
 CellTags->"OPESum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{
      RowBox[{"SO", "[", "p", "]"}], "^", "OPEi"}], 
     RowBox[{
      RowBox[{"SO", "[", "k", "]"}], "^", 
      RowBox[{"(", 
       RowBox[{"OPEm", "-", "OPEi", "-", "3"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", 
      RowBox[{"OPEm", "-", "3"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1238]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], 
     RowBox[{
      RowBox[{"-", "3"}], "+", "m"}]], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["k",
         TraditionalForm]}], ")"}], 
      RowBox[{
       RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm]}], ")"}], "i"]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1238]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1239]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["k",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["k",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["p",
       TraditionalForm]}]}]], "-", 
   FractionBox[
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["k",
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["\<\"\[CapitalDelta]\"\>",
       TraditionalForm], 
      FormBox["\<\"\[CenterDot]\"\>",
       TraditionalForm], 
      FormBox["p",
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1239]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"a", "^", "i"}], 
     RowBox[{"b", "^", 
      RowBox[{"(", 
       RowBox[{"j", "-", "i"}], ")"}]}], 
     RowBox[{"c", "^", 
      RowBox[{"(", 
       RowBox[{"m", "-", "j", "-", "4"}], ")"}]}]}], ",", 
    RowBox[{"{", 
     RowBox[{"i", ",", "0", ",", "j"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"j", ",", "0", ",", 
      RowBox[{"m", "-", "4"}]}], "}"}]}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1240]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"j", "=", "0"}], 
       RowBox[{
        RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"j", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["a", "i"], " ", 
     SuperscriptBox["b", 
      RowBox[{"j", "-", "i"}]], " ", 
     SuperscriptBox["c", 
      RowBox[{
       RowBox[{"-", "j"}], "+", "m", "-", "4"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1240]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumExplicit", "[", "t2", "]"}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1241]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    FormBox[
     SuperscriptBox["a", 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"c", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["b", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "-", 
   FractionBox[
    RowBox[{"b", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{"a", " ", 
     FormBox[
      SuperscriptBox["c", 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"a", "-", "b"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"a", "-", "c"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{"b", "-", "c"}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"OPESum",
 CellLabel->"Out[1241]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}]}]], "Input",
 CellTags->"OPESum",
 CellLabel->"In[1242]:="]
}, Open  ]],

Cell["OPESumSimplify", "ObjectName",
 CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPESumSimplify"],

Cell["OPESumSimplify[exp] simplifies OPESum's in exp.", "Text",
 CellTags->"OPESumSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESum", All, CellTags],
  ButtonData->"OPESum",
  ButtonNote->"OPESum"],
 ", ",
 ButtonBox["OPESumExplicit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESumExplicit", All, CellTags],
  ButtonData->"OPESumExplicit",
  ButtonNote->"OPESumExplicit"],
 "."
}], "Text",
 CellTags->"OPESumSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPESumSimplify"],

Cell[CellGroupData[{

Cell["\<\
OPESum[(-SOD[p])^(OPEi+1) SOD[p-q]^(OPEm-OPEi-2),{OPEi,0,OPEm}]\
\>", "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1243]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "m"], 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", 
        RowBox[{"(", 
         RowBox[{
          FormBox["\<\"\[CapitalDelta]\"\>",
           TraditionalForm], 
          FormBox["\<\"\[CenterDot]\"\>",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}], ")"}]}], ")"}], 
      RowBox[{"1", "+", "i"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\<\"\[CapitalDelta]\"\>",
         TraditionalForm], 
        FormBox["\<\"\[CenterDot]\"\>",
         TraditionalForm], 
        FormBox["\<\"(\"\>",
         TraditionalForm], 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], 
        FormBox["\<\")\"\>",
         TraditionalForm]}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], "-", "i", "+", "m"}]]}]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1243]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["OPESumSimplify[%]", "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1244]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["\[CapitalDelta]",
     TraditionalForm], "\[CenterDot]", 
    FormBox["p",
     TraditionalForm]}], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], "m"], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", "(", 
            FormBox[
             RowBox[{"p", "-", "q"}],
             TraditionalForm], ")"}], ")"}], 
          RowBox[{
           RowBox[{"-", "2"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1244]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumSimplify", "[", 
  RowBox[{
   RowBox[{"OPESum", "[", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", "OPEm"}], "}"}], "]"}], " ", 
   RowBox[{"a", "^", "OPEi"}]}], "]"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1245]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "0"}], "m"], 
    FormBox[
     SuperscriptBox["a", "i"],
     TraditionalForm]}],
   HoldForm], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1245]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"OPESumSimplify", "[", 
  RowBox[{
   RowBox[{"OPESum", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"j", ",", "0", ",", "i"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"i", ",", "0", ",", "m"}], "}"}]}], "]"}], " ", 
   RowBox[{"a", "^", 
    RowBox[{"(", 
     RowBox[{"j", "-", "i"}], ")"}]}], " ", 
   RowBox[{"b", "^", "i"}]}], "]"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1246]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TagBox[
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"i", "=", "0"}], "m"], "\<\"\"\>"}],
     HoldForm],
    TraditionalForm], 
   FormBox[
    RowBox[{"\<\"\"\>", " ", 
     RowBox[{"(", 
      RowBox[{"i", "+", "1"}], ")"}]}],
    TraditionalForm], 
   FormBox[
    RowBox[{
     SuperscriptBox["b", "i"], " ", 
     SuperscriptBox["a", 
      RowBox[{"j", "-", "i"}]]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1246]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"OPESumSimplify",
 CellLabel->"In[1247]:="],

Cell[BoxData[
 RowBox[{"OPESum", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["a", 
     RowBox[{
      RowBox[{"-", "i"}], "+", "j"}]], " ", 
    SuperscriptBox["b", "i"]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "0", ",", "m"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "0", ",", "i"}], "}"}]}], "]"}]], "Output",
 CellTags->"OPESumSimplify",
 CellLabel->"Out[1247]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["OPE1Loop ", "ObjectName",
 CellTags->"OPE1Loop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE1Loop"],

Cell["\<\
OPE1Loop[q1, amp].  OPE1Loop[{q1,q2}, amp] does sub-loop  decomposition.\
\>", "Text",
 CellTags->"OPE1Loop"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESum", All, CellTags],
  ButtonData->"OPESum",
  ButtonNote->"OPESum"],
 "."
}], "Text",
 CellTags->"OPE1Loop"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPE1Loop"],

Cell[BoxData[""], "Input",
 CellTags->"OPE1Loop",
 CellLabel->"In[1248]:="]
}, Open  ]],

Cell["OPE2TID  ***unfinished***", "ObjectName",
 CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OPE2TID"],

Cell["\<\
OPE2TID[exp, k1, k2, p]. The setting of the option EpsContract determines the \
dimension in which the Levi-Civita tensors are contracted.\
\>", "Text",
 CellTags->"OPE2TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "OPE2TID", "]"}]], "Input",
 CellTags->"OPE2TID",
 CellLabel->"In[1249]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Uncontract", "\[Rule]", "False"}], ",", 
    RowBox[{"Contract", "\[Rule]", "True"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"EpsContract", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"OPE2TID",
 CellLabel->"Out[1249]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPESum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPESum", All, CellTags],
  ButtonData->"OPESum",
  ButtonNote->"OPESum"],
 "."
}], "Text",
 CellTags->"OPE2TID"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"OPE2TID"],

Cell[BoxData[""], "Input",
 CellTags->"OPE2TID",
 CellLabel->"In[1250]:="]
}, Open  ]],

Cell["OptionsSelect", "ObjectName",
 CellTags->"OptionsSelect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"OptionsSelect"],

Cell["\<\
OptionsSelect[function,opts] returns the option settings of opts accepted by \
function.  When an option occurs several times in opts, the first setting is \
selected.\
\>", "Text",
 CellTags->"OptionsSelect"]
}, Open  ]],

Cell["Pair", "ObjectName",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Pair"],

Cell[TextData[{
 "Pair[x, y] is the head of a special pairing used in the internal \
representation: x and y may have heads LorentzIndex or Momentum. If both x \
and y have head LorentzIndex, the metric tensor is understood. If x and y \
have head Momentum, a scalar product is meant. If one of x and y has head \
LorentzIndex and the other Momentum, a Lorentz vector ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["p", "\[Mu]"], TraditionalForm]]],
 " is understood."
}], "Text",
 CellTags->"Pair"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourVector", All, CellTags],
  ButtonData->"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["FV",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FV", All, CellTags],
  ButtonData->"FV",
  ButtonNote->"FV"],
 ", ",
 ButtonBox["FVD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FVD", All, CellTags],
  ButtonData->"FVD",
  ButtonNote->"FVD"],
 ", ",
 ButtonBox["MetricTensor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MetricTensor", All, CellTags],
  ButtonData->"MetricTensor",
  ButtonNote->"MetricTensor"],
 ", ",
 ButtonBox["MT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MT", All, CellTags],
  ButtonData->"MT",
  ButtonNote->"MT"],
 ", ",
 ButtonBox["MTD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "MTD", All, CellTags],
  ButtonData->"MTD",
  ButtonNote->"MTD"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProduct", All, CellTags],
  ButtonData->"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SP", All, CellTags],
  ButtonData->"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SPD", All, CellTags],
  ButtonData->"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"Pair"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Pair"],

Cell["This represents a four-dimensional metric tensor.", "Text",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1251]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1251]="]
}, Open  ]],

Cell["This is a D-dimensional metric tensor.", "Text",
 CellTags->"Pair"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
   RowBox[{"LorentzIndex", "[", 
    RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1252]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["g", 
   RowBox[{"\[Alpha]", "\[Beta]"}]], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1252]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "\[Alpha]", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1253]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], 
   FormBox[
    FormBox["\[Alpha]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1253]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "q", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1254]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1254]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1255]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1255]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", "-", "q"}], "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1256]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1256]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "2"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1257]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1257]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "p", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "^", "3"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1258]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox[
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"],
    TraditionalForm], "3"], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1258]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", 
      RowBox[{"n", "-", "4"}]}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", "\[Beta]", "]"}]}], "]"}], "//", 
  "Contract"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1259]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1259]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"p", "-", "q"}], "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], "]"}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1260]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1260]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", 
     RowBox[{"-", "q"}], "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", "+", " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"Momentum", "[", "q", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]}]], "Input",
 CellTags->"Pair",
 CellLabel->"In[1261]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Pair",
 CellLabel->"Out[1261]="]
}, Open  ]]
}, Open  ]],

Cell["PairCollect", "ObjectName",
 CellTags->"PairCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairCollect"],

Cell["\<\
PairCollect is an option for DiracTrace specifying if the result is collected \
with respect to Pair's.\
\>", "Text",
 CellTags->"PairCollect"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 "."
}], "Text",
 CellTags->"PairCollect"]
}, Open  ]],

Cell["PairContract", "ObjectName",
 CellTags->"PairContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract"],

Cell["\<\
PairContract is like Pair, but with (local) contraction properties.\
\>", "Text",
 CellTags->"PairContract"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"PairContract"]
}, Open  ]],

Cell["PairContract2", "ObjectName",
 CellTags->"PairContract2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract2"],

Cell["\<\
PairContract2 is like Pair, but with local contraction properties among \
PairContract2's.\
\>", "Text",
 CellTags->"PairContract2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["PairContract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PairContract", All, CellTags],
  ButtonData->"PairContract",
  ButtonNote->"PairContract"],
 "."
}], "Text",
 CellTags->"PairContract2"]
}, Open  ]],

Cell["PairContract3", "ObjectName",
 CellTags->"PairContract3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PairContract3"],

Cell["\<\
PairContract3 is like Pair, but with local contraction properties among \
PairContract3's.\
\>", "Text",
 CellTags->"PairContract3"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["PairContract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PairContract", All, CellTags],
  ButtonData->"PairContract",
  ButtonNote->"PairContract"],
 "."
}], "Text",
 CellTags->"PairContract3"]
}, Open  ]],

Cell["PartialD", "ObjectName",
 CellTags->"PartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialD"],

Cell[TextData[{
 "PartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes the four-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "."}], TraditionalForm]]],
 " PartialD is used to denote derivative fields. PartialD[LorentzIndex[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", 
    StyleBox["D",
     FontSlant->"Plain"]}], TraditionalForm]]],
 "]] denotes the ",
 Cell[BoxData[
  FormBox[
   StyleBox["D",
    FontSlant->"Plain"], TraditionalForm]]],
 "-dimensional ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[PartialD]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "."}], TraditionalForm]]]
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftPartialD", All, CellTags],
  ButtonData->"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD", All, CellTags],
  ButtonData->"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RightPartialD", All, CellTags],
  ButtonData->"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"PartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1262]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"PartialD",
 CellLabel->"Out[1262]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1263]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"PartialD",
 CellLabel->"Out[1263]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"PartialD",
 CellLabel->"In[1264]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Nu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"PartialD",
 CellLabel->"Out[1264]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["PartialDRelations", "ObjectName",
 CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialDRelations"],

Cell["\<\
PartialDRelations is an option for ExpandPartialD. It is a list of rules \
applied by ExpandPartialD at the end.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialDRelations"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialD", All, CellTags],
  ButtonData->"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftPartialD", All, CellTags],
  ButtonData->"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 ", ",
 ButtonBox["LeftRightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftRightPartialD", All, CellTags],
  ButtonData->"LeftRightPartialD",
  ButtonNote->"LeftRightPartialD"],
 ", ",
 ButtonBox["RightPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RightPartialD", All, CellTags],
  ButtonData->"RightPartialD",
  ButtonNote->"RightPartialD"],
 "."
}], "Text",
 CellTags->"PartialDRelations"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialDRelations"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"B", ",", 
    RowBox[{"{", "\[Mu]", "}"}]}], "]"}], ".", 
  RowBox[{"LeftPartialD", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"PartialDRelations",
 CellLabel->"In[1265]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["A", "\[Mu]"], ".", 
   SubscriptBox["B", "\[Mu]"], ".", 
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[LeftArrow]"], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"PartialDRelations",
 CellLabel->"Out[1265]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", 
  RowBox[{"%", ",", " ", 
   RowBox[{"PartialDRelations", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"A", "\[Rule]", "C"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"PartialDRelations",
 CellLabel->"In[1266]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     FormBox["C",
      TraditionalForm], "\[Mu]"], ".", 
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox["B", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}]}], "+", 
   RowBox[{
    RowBox[{
     SubscriptBox["\[PartialD]", 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], 
     SubsuperscriptBox[
      FormBox["C",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], "\[Null]"]}], ".", 
    SubscriptBox["B", "\[Mu]"]}]}], TraditionalForm]], "Output",
 CellTags->"PartialDRelations",
 CellLabel->"Out[1266]="]
}, Open  ]]
}, Open  ]],

Cell["PartialFourVector", "ObjectName",
 CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialFourVector"],

Cell["\<\
PartialFourVector[exp, FourVector[p, mu]]  calculates the partial derivative \
of exp w.r.t. p(mu). PartialFourVector[exp, FourVector[p, mu], \
FourVector[p,nu], ...]  gives the multiple derivative.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialFourVector"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourVector", All, CellTags],
  ButtonData->"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"PartialFourVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialFourVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialFourVector", "[", 
  RowBox[{
   RowBox[{"a", " ", 
    SuperscriptBox["\[ExponentialE]", 
     SuperscriptBox[
      RowBox[{"FourVector", "[", 
       RowBox[{"p", ",", "\[Nu]"}], "]"}], "2"]]}], ",", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}], "]"}]], "Input",
 CellTags->"PartialFourVector",
 CellLabel->"In[1267]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "a", " ", 
   SuperscriptBox["\[ExponentialE]", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"PartialFourVector",
 CellLabel->"Out[1267]="]
}, Open  ]]
}, Open  ]],

Cell["PartialIntegrate", "ObjectName",
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartialIntegrate"],

Cell["\<\
 PartialIntegrate[exp, ap, t] does a partial integration of the definite \
integral Integrate[exp,{t,0,1}], with ap the factor that is to be integrated \
and exp/ap the factor that is to be differentiated.\
\>", "Text",
 ZeroWidthTimes->True,
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PartialIntegrate", "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1268]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Integrate", "\[Rule]", "Integrate"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1268]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"PartialIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PartialIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"f", "[", "x", "]"}], 
    RowBox[{"g", "[", "x", "]"}]}], ",", 
   RowBox[{"g", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1269]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     TagBox[
      RowBox[{
       RowBox[{
        RowBox[{"f", "(", "x", ")"}], " ", 
        RowBox[{"\[Integral]", 
         RowBox[{
          RowBox[{"g", "(", "x", ")"}], 
          RowBox[{"\[DifferentialD]", "x"}]}]}]}], "/.", "\[VeryThinSpace]", 
       RowBox[{"x", "\[Rule]", "0"}]}],
      HoldForm], ")"}]}], "+", 
   RowBox[{"(", 
    TagBox[
     RowBox[{
      RowBox[{
       RowBox[{"f", "(", "x", ")"}], " ", 
       RowBox[{"\[Integral]", 
        RowBox[{
         RowBox[{"g", "(", "x", ")"}], 
         RowBox[{"\[DifferentialD]", "x"}]}]}]}], "/.", "\[VeryThinSpace]", 
      RowBox[{"x", "\[Rule]", "1"}]}],
     HoldForm], ")"}], "-", 
   RowBox[{
    SubsuperscriptBox["\[Integral]", "0", "1"], 
    RowBox[{
     RowBox[{
      RowBox[{
       SuperscriptBox["f", "\[Prime]",
        MultilineFunction->None], "(", "x", ")"}], " ", 
      RowBox[{"(", 
       RowBox[{"\[Integral]", 
        RowBox[{
         RowBox[{"g", "(", "x", ")"}], 
         RowBox[{"\[DifferentialD]", "x"}]}]}], ")"}]}], 
     RowBox[{"\[DifferentialD]", "x"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1269]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "InputForm"}]], "Input",
 CellLabel->"In[1270]:="],

Cell["\<\
-HoldForm[f[x]*Integrate[g[x], x] /. x -> 0] + HoldForm[f[x]*Integrate[g[x], \
x] /. x -> 1] - 
 Integrate[Integrate[g[x], x]*Derivative[1][f][x], {x, 0, 1}]\
\>", "Output",
 CellLabel->"Out[1270]//InputForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"f", "[", "x_", "]"}], "=", 
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"Log", "[", 
     RowBox[{
      RowBox[{"3", "x"}], "+", "2"}], "]"}], ",", "x"}], "]"}]}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1271]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "3"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], "-", "1"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1271]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x_", "]"}], "=", 
  RowBox[{"D", "[", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"Log", "[", 
      RowBox[{
       RowBox[{"3", "x"}], "+", "2"}], "]"}]], ",", "x"}], "]"}]}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1272]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["3", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{
       RowBox[{"3", " ", "x"}], "+", "2"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1272]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{"PartialIntegrate", "[", 
     RowBox[{
      RowBox[{
       RowBox[{"f", "[", "x", "]"}], 
       RowBox[{"g", "[", "x", "]"}]}], ",", 
      RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1273]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["2", 
    RowBox[{"log", "(", "8", ")"}]], "-", 
   FractionBox["5", 
    RowBox[{"log", "(", "125", ")"}]]}], TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1273]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Integrate", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"f", "[", "x", "]"}], 
     RowBox[{"g", "[", "x", "]"}]}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}], "//", 
  "Simplify"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1274]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["2", 
    RowBox[{"log", "(", "8", ")"}]], "-", 
   FractionBox["5", 
    RowBox[{"log", "(", "125", ")"}]]}], TraditionalForm]], "Output",
 CellTags->"PartialIntegrate",
 CellLabel->"Out[1274]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"f", ",", "g"}], "]"}]], "Input",
 CellTags->"PartialIntegrate",
 CellLabel->"In[1275]:="]
}, Open  ]],

Cell["PartitHead", "ObjectName",
 CellTags->"PartitHead"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PartitHead"],

Cell["\<\
PartitHead[expr, h] returns a list {ex1, h[ex2]} with ex1 free of expressions \
with head h, and h[ex2] having head h.\
\>", "Text",
 CellTags->"PartitHead"]
}, Open  ]],

Cell["PauliSigma", "ObjectName",
 CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PauliSigma"],

Cell["\<\
PauliSigma denotes the vector of the 3 Pauli matrices. PauliSigma[1], \
PauliSigma[2], PauliSigma[3] give the explicit Pauli matrices. PauliSigma[] \
yields {PauliSigma[1], PauliSigma[2], PauliSigma[3]}.\
\>", "Text",
 CellTags->"PauliSigma"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PauliSigma", "[", "]"}], "//", "MatrixForm"}]], "Input",
 CellTags->"PauliSigma",
 CellLabel->"In[1276]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{"(", "\[NoBreak]", GridBox[{
      {
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {"0"},
           {"1"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}], 
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {"1"},
           {"0"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}]},
      {
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {"0"},
           {
            RowBox[{"-", "\[ImaginaryI]"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}], 
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {"\[ImaginaryI]"},
           {"0"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}]},
      {
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {"1"},
           {"0"}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}], 
       RowBox[{"(", "\[NoBreak]", 
        TagBox[GridBox[{
           {"0"},
           {
            RowBox[{"-", "1"}]}
          },
          
          GridBoxAlignment->{
           "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
            "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
          GridBoxSpacings->{"Columns" -> {
              Offset[0.27999999999999997`], {
               Offset[0.5599999999999999]}, 
              Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
              Offset[0.2], {
               Offset[0.4]}, 
              Offset[0.2]}, "RowsIndexed" -> {}}],
         Column], "\[NoBreak]", ")"}]}
     },
     GridBoxAlignment->{
      "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}},
        "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.7]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}],
   Function[BoxForm`e$, 
    MatrixForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"PauliSigma",
 CellLabel->"Out[1276]//MatrixForm="]
}, Open  ]]
}, Open  ]],

Cell["PaVe", "ObjectName",
 CellTags->"PaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVe"],

Cell["\<\
 PaVe[ i,j,... {p10,p12,...},{m1^2, mw^2, ...} ] denotes the invariant (and \
scalar) Passarino-Veltman integrals, i.e. the coefficient functions of the \
tensor integral decomposition. Joining plist and mlist gives the same \
conventions as for A0, B0, C0, D0. Automatic simlifications are performed for \
the coefficient functions of two-point integrals and for the scalar integrals.\
\>", "Text",
 CellTags->"PaVe"],

Cell[TextData[{
 "For a more detailed description see the manual (no changes from there \
basically): ",
 ButtonBox["http://www.mertig.com/oldfc",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://www.mertig.com/oldfc"], None}]
}], "Text",
 CellTags->"PaVe"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"PaVe"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVe"],

Cell["Some of the PaVe's reduce to special cases.", "Text",
 CellTags->"PaVe"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVe", "[", 
  RowBox[{"0", ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"m1", ",", "m2", ",", "m3", ",", "m4"}], "}"}]}], "]"}]], "Input",\

 CellTags->"PaVe",
 CellLabel->"In[1277]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", 
   "f", ",", "m1", ",", "m2", ",", "m3", ",", "m4", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"PaVe",
 CellLabel->"Out[1277]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVe", "[", 
  RowBox[{"0", ",", "0", ",", 
   RowBox[{"{", "pp", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"m", "^", "2"}], ",", 
     RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"PaVe",
 CellLabel->"In[1278]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"PaVe", "(", 
   RowBox[{"0", ",", "0", ",", 
    RowBox[{"{", "pp", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"]}], "}"}]}], ")"}], TraditionalForm]], "Output",\

 CellTags->"PaVe",
 CellLabel->"Out[1278]="]
}, Open  ]]
}, Open  ]],

Cell["PaVeOrderList", "ObjectName",
 CellTags->"PaVeOrderList"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeOrderList"],

Cell["\<\
PaVeOrderList is an option for PaVeOrder and PaVeReduce, specifying in which \
order the arguments of D0 are to be permuted.\
\>", "Text",
 CellTags->"PaVeOrderList"],

Cell[TextData[{
 "See also: ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeOrder", All, CellTags],
  ButtonData->"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["D0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "D0", All, CellTags],
  ButtonData->"D0",
  ButtonNote->"D0"],
 "."
}], "Text",
 CellTags->"PaVeOrderList"]
}, Open  ]],

Cell["PaVeOrder", "ObjectName",
 CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeOrder"],

Cell[TextData[{
 "PaVeOrder[expr] orders the arguments of all D0 in expr in a standard way. \
PaVeOrder[expr, PaVeOrderList \[Rule] { {..., s, u, ...}, {... ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["m", "1"], "2"], TraditionalForm]]],
 ", ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["m", "2"], "2"], TraditionalForm]]],
 " ...}, ...}] orders the arguments of all D0 in expr according to the \
specified ordering lists. The lists may contain only a subsequence of the \
D0-variables."
}], "Text",
 CellTags->"PaVeOrder"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"PaVeOrder"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVeOrder"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", "me2",
      ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
  "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1279]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2", 
   ",", "me2", ",", "me2", ",", 
   FormBox["0",
    TraditionalForm], ",", 
   FormBox["0",
    TraditionalForm], ",", "me2", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeOrder",
 CellLabel->"Out[1279]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{"D0", "[", 
    RowBox[{
    "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", "me2",
      ",", "0", ",", "me2", ",", "0"}], "]"}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}]}]}], 
  "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1280]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
    TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2", 
   ",", "me2", ",", "me2", ",", 
   FormBox["0",
    TraditionalForm], ",", 
   FormBox["0",
    TraditionalForm], ",", "me2", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeOrder",
 CellLabel->"Out[1280]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeOrder", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"D0", "[", 
     RowBox[{
     "a", ",", "b", ",", "c", ",", "d", ",", "e", ",", "f", ",", "m12", ",", 
      "m22", ",", "m32", ",", "m42"}], "]"}], "+", 
    RowBox[{"D0", "[", 
     RowBox[{
     "me2", ",", "me2", ",", "mw2", ",", "mw2", ",", "t", ",", "s", ",", 
      "me2", ",", "0", ",", "me2", ",", "0"}], "]"}]}], ",", 
   RowBox[{"PaVeOrderList", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"me2", ",", "me2", ",", "0", ",", "0"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"f", ",", "e"}], "}"}]}], "}"}]}]}], "]"}]], "Input",
 CellTags->"PaVeOrder",
 CellLabel->"In[1281]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox[
     SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
     TraditionalForm], "(", "a", ",", "d", ",", "c", ",", "b", ",", "f", ",", 
    "e", ",", "m22", ",", "m12", ",", "m42", ",", "m32", ")"}], "+", 
   RowBox[{
    FormBox[
     SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
     TraditionalForm], "(", "me2", ",", "s", ",", "mw2", ",", "t", ",", "mw2",
     ",", "me2", ",", "me2", ",", 
    FormBox["0",
     TraditionalForm], ",", 
    FormBox["0",
     TraditionalForm], ",", "me2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"PaVeOrder",
 CellLabel->"Out[1281]="]
}, Open  ]]
}, Open  ]],

Cell["PaVeReduce", "ObjectName",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PaVeReduce"],

Cell["\<\
PaVeReduce[expr] reduces all Passarino-Veltman integrals (i.e. all PaVe's) in \
expr down to scalar A0, B0, C0 and D0.\
\>", "Text",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PaVeReduce", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1282]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "True"}], ",", 
    RowBox[{"IsolateNames", "\[Rule]", "False"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PaVeOrderList", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"WriteOutPaVe", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1282]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "WriteOutPaVe"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1283]:="],

Cell[BoxData[
 StyleBox["\<\"WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If \
set to a string, the results of all Passarino-Veltman PaVe's are stored in \
files with names generated from this string and the arguments of PaVe.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellTags->{"Info3535453273-5096475", "PaVeReduce"}]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FRH",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FRH", All, CellTags],
  ButtonData->"FRH",
  ButtonNote->"FRH"],
 ", ",
 ButtonBox["PaVeOrder",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeOrder", All, CellTags],
  ButtonData->"PaVeOrder",
  ButtonNote->"PaVeOrder"],
 "."
}], "Text",
 CellTags->"PaVeReduce"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PaVeReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeReduce", "[", 
  RowBox[{
   RowBox[{"PaVe", "[", 
    RowBox[{"1", ",", "2", ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", 
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"m", "^", "2"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"m", "^", "2"}], ",", 
       RowBox[{"M", "^", "2"}]}], "}"}]}], "]"}], ",", 
   RowBox[{"IsolateHead", "\[Rule]", "L"}]}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1284]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["M", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"3", " ", 
        SuperscriptBox["M", "2"]}], "-", 
       RowBox[{"2", " ", "s"}]}], ")"}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", "s", ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["M", "2"],
       TraditionalForm], ",", 
      FormBox[
       SuperscriptBox["m", "2"],
       TraditionalForm], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["m", "2"], "-", 
       SuperscriptBox["M", "2"]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "0", ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox["m", "2"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", 
       RowBox[{"6", " ", 
        SuperscriptBox["M", "2"]}], "-", "s"}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "s", ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{"8", " ", 
        SuperscriptBox["m", "4"]}], "-", 
       RowBox[{"10", " ", 
        SuperscriptBox["m", "2"], " ", 
        SuperscriptBox["M", "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox["m", "2"], " ", "s"}], "+", 
       RowBox[{
        SuperscriptBox["M", "2"], " ", "s"}]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["m", "2"], ",", 
      SuperscriptBox["M", "2"], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox["m", "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["m", "2"]}], "-", "s"}], ")"}], "2"]}]], "+", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["m", "2"]}], "-", "s"}], ")"}]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1284]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"L", "[", "10", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1285]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1285]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FRH", "[", "%", "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1286]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"L", "(", "10", ")"}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1286]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PaVeReduce", "[", 
  RowBox[{
   RowBox[{"PaVe", "[", 
    RowBox[{"2", ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "mw2", ",", "t"}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"SmallVariable", "[", "me2", "]"}], ",", "0", ",", "mw2"}], 
      "}"}]}], "]"}], ",", 
   RowBox[{"WriteOutPaVe", "\[Rule]", "\"\<p\>\""}]}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1287]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SubscriptBox["B", "0"], "(", "t", ",", "mw2", ",", "me2", ")"}], 
     RowBox[{"mw2", "-", "t"}]]}], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["B", "0"], "(", "0", ",", "mw2", ",", "mw2", ")"}], 
    RowBox[{"mw2", "-", "t"}]], "+", 
   FractionBox["2", 
    RowBox[{"mw2", "-", "t"}]]}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1287]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", 
     "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\""}], ",", "String"}], "]"}], 
  "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1288]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"( 2/(mw2 - t) + B0[0, mw2, mw2]/(mw2 - t) - B0[t, mw2, \
SmallVariable[me2]]/\"\>"},
      {"\<\"  (mw2 - t)\"\>"},
      {"\<\"  ) \"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1288]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "/@", 
   RowBox[{
   "FileNames", "[", "\"\<pPaVe2Csmame2mw2tCsmame20mw2.s\>\"", "]"}]}], 
  ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1289]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"se", "=", 
   RowBox[{"SmallVariable", "[", "ME2", "]"}]}], ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1290]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"d122", "=", 
  RowBox[{"PaVeReduce", "[", 
   RowBox[{
    RowBox[{"PaVe", "[", 
     RowBox[{"1", ",", "2", ",", "2", ",", 
      RowBox[{"{", 
       RowBox[{"se", ",", "MW2", ",", "MW2", ",", "se", ",", "S", ",", "T"}], 
       "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"0", ",", "se", ",", "0", ",", "se"}], "}"}]}], "]"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"S", ",", "T", ",", "U", ",", 
       RowBox[{"2", " ", "MW2"}]}], "}"}]}], ",", 
    RowBox[{"IsolateHead", "\[Rule]", "F"}]}], "]"}]}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1291]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["S", "2"], " ", 
      SuperscriptBox["T", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", "T", ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["0",
        TraditionalForm], ")"}]}], 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["S", "2"], " ", "T", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], "2"], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "MW2", ",", "S", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"S", " ", "U"}]}], ")"}], "3"]], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"MW2", "+", "S"}], ")"}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", 
         SuperscriptBox["MW2", "6"]}], "-", 
        RowBox[{"8", " ", 
         SuperscriptBox["MW2", "5"], " ", "S"}], "-", 
        RowBox[{"2", " ", 
         SuperscriptBox["MW2", "5"], " ", "T"}], "+", 
        RowBox[{"12", " ", 
         SuperscriptBox["MW2", "4"], " ", 
         SuperscriptBox["S", "2"]}], "+", 
        RowBox[{"20", " ", 
         SuperscriptBox["MW2", "4"], " ", "S", " ", "T"}], "-", 
        RowBox[{"8", " ", 
         SuperscriptBox["MW2", "3"], " ", 
         SuperscriptBox["S", "3"]}], "-", 
        RowBox[{"36", " ", 
         SuperscriptBox["MW2", "3"], " ", 
         SuperscriptBox["S", "2"], " ", "T"}], "-", 
        RowBox[{"6", " ", 
         SuperscriptBox["MW2", "3"], " ", "S", " ", 
         SuperscriptBox["T", "2"]}], "+", 
        RowBox[{"2", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "4"]}], "+", 
        RowBox[{"20", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "3"], " ", "T"}], "+", 
        RowBox[{"6", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["T", "2"]}], "-", 
        RowBox[{"2", " ", "MW2", " ", 
         SuperscriptBox["S", "4"], " ", "T"}], "-", 
        RowBox[{"6", " ", "MW2", " ", 
         SuperscriptBox["S", "3"], " ", 
         SuperscriptBox["T", "2"]}], "+", 
        RowBox[{"4", " ", "MW2", " ", 
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["T", "3"]}], "-", 
        RowBox[{
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["T", "4"]}]}], ")"}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["\<\"C\"\>", "\<\"0\"\>"],
        TraditionalForm], "(", "MW2", ",", "MW2", ",", "T", ",", 
       FormBox["ME2",
        TraditionalForm], ",", 
       FormBox["0",
        TraditionalForm], ",", 
       FormBox["ME2",
        TraditionalForm], ")"}]}], ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "3"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["S", "3"], " ", 
     SuperscriptBox["T", "2"], " ", 
     RowBox[{"(", 
      RowBox[{"MW2", "-", "S"}], ")"}], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["\<\"D\"\>", "\<\"0\"\>"],
       TraditionalForm], "(", "MW2", ",", "MW2", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", "T", ",", "S", ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ",", 
      FormBox["ME2",
       TraditionalForm], ",", 
      FormBox["0",
       TraditionalForm], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "3"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"MW2", "+", "S"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", 
        SuperscriptBox["MW2", "3"]}], "-", 
       RowBox[{"9", " ", 
        SuperscriptBox["MW2", "2"], " ", "S"}], "-", 
       RowBox[{
        SuperscriptBox["MW2", "2"], " ", "U"}], "-", 
       RowBox[{"4", " ", "MW2", " ", "S", " ", "U"}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "3"]}], "+", 
       RowBox[{"5", " ", 
        SuperscriptBox["S", "2"], " ", "U"}], "+", 
       RowBox[{"3", " ", "S", " ", 
        SuperscriptBox["U", "2"]}]}], ")"}], " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "T", ",", "ME2", ",", "ME2", ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "5"]}], "-", 
        RowBox[{"5", " ", 
         SuperscriptBox["MW2", "4"], " ", "S"}], "-", 
        RowBox[{
         SuperscriptBox["MW2", "4"], " ", "U"}], "-", 
        RowBox[{"16", " ", 
         SuperscriptBox["MW2", "3"], " ", 
         SuperscriptBox["S", "2"]}], "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "3"]}], "-", 
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "2"], " ", 
         SuperscriptBox["S", "2"], " ", "U"}], "+", 
        RowBox[{"4", " ", 
         SuperscriptBox["MW2", "2"], " ", "S", " ", 
         SuperscriptBox["U", "2"]}], "+", 
        RowBox[{"4", " ", "MW2", " ", 
         SuperscriptBox["S", "4"]}], "+", 
        RowBox[{"8", " ", "MW2", " ", 
         SuperscriptBox["S", "3"], " ", "U"}], "+", 
        RowBox[{
         SuperscriptBox["S", "3"], " ", 
         SuperscriptBox["U", "2"]}], "+", 
        RowBox[{
         SuperscriptBox["S", "2"], " ", 
         SuperscriptBox["U", "3"]}]}], ")"}], " ", 
      RowBox[{
       SubscriptBox["B", "0"], "(", "MW2", ",", "0", ",", "ME2", ")"}]}], 
     ")"}], "/", 
    RowBox[{"(", 
     RowBox[{"2", " ", 
      RowBox[{"(", 
       RowBox[{"MW2", "-", "S"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], "2"], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox["MW2", "2"], "-", 
         RowBox[{"S", " ", "U"}]}], ")"}], "2"]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{"S", " ", 
     RowBox[{
      SubscriptBox["B", "0"], "(", "S", ",", "0", ",", "0", ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"4", " ", "MW2", " ", "S"}], "+", 
       RowBox[{"2", " ", 
        SuperscriptBox["S", "2"]}], "+", 
       RowBox[{"S", " ", "U"}]}], ")"}]}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{"MW2", "-", "S"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["MW2", "2"], "-", 
        RowBox[{"S", " ", "U"}]}], ")"}], "2"]}]], "-", 
   FractionBox[
    RowBox[{"MW2", "+", "S"}], 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"4", " ", "MW2"}], "-", "T"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["MW2", "2"], "-", 
       RowBox[{"S", " ", "U"}]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1291]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"\"\<fctd122.for\>\"", ",", 
    RowBox[{"d122res", "==", "d122"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], ";"}]], "Input",\

 CellTags->"PaVeReduce",
 CellLabel->"In[1292]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "\"\<fctd122.for\>\""}], ",", 
    "String"}], "]"}], "]"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1293]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"        d122res = (-5D0D-1*(MW2 + S))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)*(MW2**2 - 1D0*S*U)) - \"\>"},
      {"\<\"     &  (5.D-1*(4D0*MW2**5 - 5D0*MW2**4*S - \"\>"},
      {"\<\"     &       16D1*MW2**3*S**2 + 4D0*MW2**2*S**3 + \"\>"},
      {"\<\"     &       4D0*MW2*S**4 - 1D0*MW2**4*U - \"\>"},
      {"\<\"     &       4D0*MW2**2*S**2*U + 8D0*MW2*S**3*U + \"\>"},
      {"\<\"     &       4D0*MW2**2*S*U**2 + S**3*U**2 + S**2*U**3)*\"\>"},
      {"\<\"     &     B0(MW2,0D0,ME2))/\"\>"},
      {"\<\"     &   ((MW2 - 1D0*S)*(4D0*MW2 - 1D0*T)**2*\"\>"},
      {"\<\"     &     (MW2**2 - 1D0*S*U)**2) + \"\>"},
      {"\<\"     &  (5.D-1*S*(MW2**2 - 4D0*MW2*S + 2D0*S**2 + S*U)*\"\>"},
      {"\<\"     &     B0(S,0D0,0D0))/\"\>"},
      {"\<\"     &   ((MW2 - 1D0*S)*(MW2**2 - 1D0*S*U)**2) + \"\>"},
      {"\<\"     &  (5.D-1*(MW2 + S)*\"\>"},
      {"\<\"     &     (4D0*MW2**3 - 9D0*MW2**2*S + 2D0*S**3 - \"\>"},
      {"\<\"     &       1D0*MW2**2*U - 4D0*MW2*S*U + 5D0*S**2*U + \"\>"},
      {"\<\"     &       3D0*S*U**2)*B0(T,ME2,ME2))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)**2*(MW2**2 - 1D0*S*U)**2) - \"\>"},
      {"\<\"     &  (5.D-1*(MW2 + S)*\"\>"},
      {"\<\"     &     (2D0*MW2**6 - 8D0*MW2**5*S + 12D1*MW2**4*S**2 - \
\"\>"},
      {"\<\"     &       8D0*MW2**3*S**3 + 2D0*MW2**2*S**4 - \"\>"},
      {"\<\"     &       2D0*MW2**5*T + 20D1*MW2**4*S*T - \"\>"},
      {"\<\"     &       36D1*MW2**3*S**2*T + 20D1*MW2**2*S**3*T - \"\>"},
      {"\<\"     &       2D0*MW2*S**4*T - 6D0*MW2**3*S*T**2 + \"\>"},
      {"\<\"     &       6D0*MW2**2*S**2*T**2 - 6D0*MW2*S**3*T**2 + \"\>"},
      {"\<\"     &       4D0*MW2*S**2*T**3 - 1D0*S**2*T**4)*\"\>"},
      {"\<\"     &     C0(MW2,MW2,T,ME2,0D0,ME2))/\"\>"},
      {"\<\"     &   ((4D0*MW2 - 1D0*T)**2*(MW2**2 - 1D0*S*U)**3) + \"\>"},
      {"\<\"     &  ((MW2 - 1D0*S)**2*S**2*T*C0(MW2,S,ME2,ME2,0D0,0D0))/\"\>"},
      {"\<\"     &   (MW2**2 - 1D0*S*U)**3 - \"\>"},
      {"\<\"     &  (5.D-1*(MW2 - 1D0*S)*S**2*T**2*\"\>"},
      {"\<\"     &     C0(T,ME2,ME2,ME2,ME2,0D0))/(MW2**2 - 1D0*S*U)**3 + \
\"\>"},
      {"\<\"     &  (5.D-1*(MW2 - 1D0*S)*S**3*T**2*\"\>"},
      {"\<\"     &     D0(MW2,MW2,ME2,ME2,T,S,ME2,0D0,ME2,0D0))/\"\>"},
      {"\<\"     &   (MW2**2 - 1D0*S*U)**3\"\>"},
      {"\<\"                  \"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"PaVeReduce",
 CellLabel->"Out[1293]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "/@", 
   RowBox[{"FileNames", "[", "\"\<fctd122.for\>\"", "]"}]}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"d122", ",", "se"}], "]"}], ";"}]], "Input",
 CellTags->"PaVeReduce",
 CellLabel->"In[1294]:="]
}, Open  ]],

Cell["PD", "ObjectName",
 CellTags->"PD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PD"],

Cell["PD is an abbreviation for PropagatorDenominator.", "Text",
 CellTags->"PD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PropagatorDenominator", All, CellTags],
  ButtonData->"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 ", ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IFPD", All, CellTags],
  ButtonData->"IFPD",
  ButtonNote->"IFPD"],
 "."
}], "Text",
 CellTags->"PD"]
}, Open  ]],

Cell["PlusDistribution", "ObjectName",
 CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PlusDistribution"],

Cell["\<\
PlusDistribution[1/(1-x)] denotes a distribution (in the sense of the \"+\" \
prescription).\
\>", "Text",
 CellTags->"PlusDistribution"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Integrate2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Integrate2", All, CellTags],
  ButtonData->"Integrate2",
  ButtonNote->"Integrate2"],
 "."
}], "Text",
 CellTags->"PlusDistribution"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PlusDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{"1", "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1295]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FractionBox["1", 
     RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1295]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"1", "-", "x"}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1296]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   RowBox[{"(", 
    FractionBox[
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], 
     RowBox[{"1", "-", "x"}]], ")"}], "+"], TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1296]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1297]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1297]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{
     RowBox[{"Log", "[", 
      RowBox[{"1", "-", "x"}], "]"}], "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1298]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1298]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"PlusDistribution", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"Log", "[", 
       RowBox[{"1", "-", "x"}], "]"}], "^", "2"}], "/", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1299]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1299]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PlusDistribution", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"x", " ", 
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "]"}], "/", 
   RowBox[{"(", 
    RowBox[{"1", "-", "x"}], ")"}]}], "]"}]], "Input",
 CellTags->"PlusDistribution",
 CellLabel->"In[1300]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"log", "(", "x", ")"}], 
    RowBox[{"1", "-", "x"}]], "+", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox[
      RowBox[{"log", "(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], TraditionalForm]], "Output",
 CellTags->"PlusDistribution",
 CellLabel->"Out[1300]="]
}, Open  ]]
}, Open  ]],

Cell["Polarization", "ObjectName",
 CellTags->"Polarization"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Polarization"],

Cell["\<\
Polarization[k] = Polarization[k, I] is the head of a polarization momentum \
with (incoming) momentum k. A slashed polarization vector (e1(k) slash) has \
to be entered as DiracSlash[Polarization[k]]. The internal representation for \
a polarization vector e1 corresponding to a boson with four momentum k is: \
Momentum[ Polarization[ k, I ] ]. With this notation transversality of \
polarization vectors is provided, i.e. , Pair[ Momentum[k], Momentum[ \
Polarization[k, I] ] ] yields 0. Polarization[k,-I] denotes the complex \
conjugate polarization. Polarization is also an option of various functions \
related to the operator product expansion. The setting 0 denotes the \
unpolarized and 1 the polarized case.\
\>", "Text",
 CellTags->"Polarization"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PolarizationVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PolarizationVector", All, CellTags],
  ButtonData->"PolarizationVector",
  ButtonNote->"PolarizationVector"],
 ", ",
 ButtonBox["PolarizationSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PolarizationSum", All, CellTags],
  ButtonData->"PolarizationSum",
  ButtonNote->"PolarizationSum"],
 ", ",
 ButtonBox["DoPolarizationSums",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DoPolarizationSums", All, CellTags],
  ButtonData->"DoPolarizationSums",
  ButtonNote->"DoPolarizationSums"],
 "."
}], "Text",
 CellTags->"Polarization"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Polarization"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Polarization", "[", "k", "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1301]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyEpsilon]", "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1301]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Polarization", "[", "k", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1302]:="],

Cell[BoxData[
 RowBox[{"Polarization", "[", 
  RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1302]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Polarization", "[", "k", "]"}], "//", 
  "ComplexConjugate"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1303]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    TemplateBox[{"\"\[CurlyEpsilon]\"","\"*\""},
     "Superscript"],
    TraditionalForm], "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1303]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Polarization", "[", "k", "]"}], "//", "ComplexConjugate"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1304]:="],

Cell[BoxData[
 RowBox[{"Polarization", "[", 
  RowBox[{"k", ",", 
   RowBox[{"-", "\[ImaginaryI]"}]}], "]"}]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1304]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"Polarization", "[", "k", "]"}], "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1305]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", "k", ")"}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1305]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracSlash", "[", 
   RowBox[{"Polarization", "[", "k", "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1306]:="],

Cell[BoxData[
 RowBox[{"DiracSlash", "[", 
  RowBox[{"Polarization", "[", 
   RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1306]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Pair", "[", " ", 
  RowBox[{
   RowBox[{"Momentum", "[", "k", "]"}], ",", " ", 
   RowBox[{"Momentum", "[", " ", 
    RowBox[{"Polarization", "[", 
     RowBox[{"k", ",", " ", "I"}], "]"}], " ", "]"}]}], " ", "]"}]], "Input",
 CellTags->"Polarization",
 CellLabel->"In[1307]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Polarization",
 CellLabel->"Out[1307]="]
}, Open  ]]
}, Open  ]],

Cell["PolarizationSum", "ObjectName",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationSum"],

Cell["\<\
PolarizationSum[ mu,nu, ... ] defines (as abbreviations) several polarization \
sums. The first two arguments are the interpreted as Lorentz indices, all \
further ones are momenta. PolarizationSum performs no calculations.\
\>", "Text",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "PolarizationSum", "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1308]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", "4"}], "}"}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1308]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Polarization", All, CellTags],
  ButtonData->"Polarization",
  ButtonNote->"Polarization"],
 "."
}], "Text",
 CellTags->"PolarizationSum"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PolarizationSum"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1309]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1309]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1310]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], "2"]], "-", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1310]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", "n"}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1311]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["n",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox["n",
        TraditionalForm]}],
      TraditionalForm], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["n",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["n",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["n",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1311]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationSum", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]", ",", "k", ",", 
   RowBox[{
    SubscriptBox["p", "1"], "-", 
    SubscriptBox["p", "2"]}]}], "]"}]], "Input",
 CellTags->"PolarizationSum",
 CellLabel->"In[1312]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "1", "2"], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{"2", " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", "2", "2"], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm]}], "-", 
       RowBox[{
        FormBox["k",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}]}], ")"}], "2"]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          SubscriptBox["p", "1"], "-", 
          SubscriptBox["p", "2"]}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["k",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{
          SubscriptBox["p", "1"], "-", 
          SubscriptBox["p", "2"]}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "1"],
       TraditionalForm]}], "-", 
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       SubscriptBox["p", "2"],
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellTags->"PolarizationSum",
 CellLabel->"Out[1312]="]
}, Open  ]]
}, Open  ]],

Cell["PolarizationUncontract", "ObjectName",
 CellTags->"PolarizationUncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationUncontract"],

Cell["\<\
PolarizationUncontract does Uncontract on scalar products involving \
polarization vectors.\
\>", "Text",
 CellTags->"PolarizationUncontract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Polarization", All, CellTags],
  ButtonData->"Polarization",
  ButtonNote->"Polarization"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Uncontract", All, CellTags],
  ButtonData->"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"PolarizationUncontract"]
}, Open  ]],

Cell["PolarizationVector", "ObjectName",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PolarizationVector"],

Cell["PolarizationVector[p, mu] gives a polarization vector.", "Text",
 CellTags->"PolarizationVector"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FourVector",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourVector", All, CellTags],
  ButtonData->"FourVector",
  ButtonNote->"FourVector"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Polarization", All, CellTags],
  ButtonData->"Polarization",
  ButtonNote->"Polarization"],
 "."
}], "Text",
 CellTags->"PolarizationVector"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PolarizationVector", "[", 
  RowBox[{"k", ",", "\[Mu]"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1313]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CurlyEpsilon]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1313]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Conjugate", "[", 
  RowBox[{"PolarizationVector", "[", 
   RowBox[{"k", ",", "\[Mu]"}], "]"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1314]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[CurlyEpsilon]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "*"], "(", "k", ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1314]="]
}, Open  ]],

Cell[TextData[{
 "A polarization vector ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}], 
   TraditionalForm]]],
 "is a special four-vector."
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolarizationVector", "[", 
   RowBox[{"k", ",", "mu"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1315]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"LorentzIndex", "[", "mu", "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"Polarization", "[", 
     RowBox[{"k", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], "]"}]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1315]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 "The transverality property ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      FormBox["k",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", "k", ")"}]}], "=", "0"}],
    TraditionalForm]]],
 " is built in."
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a1", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1316]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["k", "\[Mu]"], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", "k", ")"}]}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1316]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a1", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1317]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1317]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a2", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1318]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["k", "\[Mu]"], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", 
    FormBox[
     RowBox[{"k", "-", "p"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1318]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a2", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1319]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["k",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", 
     FormBox[
      RowBox[{"k", "-", "p"}],
      TraditionalForm], ")"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1319]="]
}, Open  ]],

Cell[TextData[{
 "Depending on the alphabetical ordering of the momenta simplifcations are \
done, e.g., ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{
      SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
      FormBox[
       RowBox[{"k", "-", "p"}],
       TraditionalForm], ")"}]}], "=", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"k", "-", "p", "-", "k"}], ")"}], "\[Mu]"]}], 
      RowBox[{
       SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
       FormBox[
        RowBox[{"k", "-", "p"}],
        TraditionalForm], ")"}]}], "=", 
     RowBox[{
      RowBox[{
       SuperscriptBox["k", "\[Mu]"], 
       RowBox[{
        SubscriptBox["\[CurlyEpsilon]", "\[Mu]"], "(", 
        FormBox[
         RowBox[{"k", "-", "p"}],
         TraditionalForm], ")"}]}], "=", 
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[NoBreak]", "\[CenterDot]", "\[NoBreak]", 
       RowBox[{
        RowBox[{"(", "\[NoBreak]", 
         FormBox[
          RowBox[{"\[CurlyEpsilon]", "\[NoBreak]", "(", "\[NoBreak]", 
           FormBox[
            RowBox[{"k", "-", "p"}],
            TraditionalForm], "\[NoBreak]", ")"}],
          TraditionalForm], "\[NoBreak]", ")"}], "."}]}]}]}]}], 
   TraditionalForm]]]
}], "Text",
 CellTags->"PolarizationVector"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"a3", " ", "=", " ", 
  RowBox[{
   RowBox[{"PolarizationVector", "[", 
    RowBox[{
     RowBox[{"k", "-", "p"}], ",", "\[Mu]"}], "]"}], " ", 
   RowBox[{"FourVector", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}]}]}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1320]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["p", "\[Mu]"], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", 
    FormBox[
     RowBox[{"k", "-", "p"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1320]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", "a3", "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1321]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["k",
    TraditionalForm], "\[CenterDot]", "(", 
   FormBox[
    RowBox[{"\[CurlyEpsilon]", "(", 
     FormBox[
      RowBox[{"k", "-", "p"}],
      TraditionalForm], ")"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"PolarizationVector",
 CellLabel->"Out[1321]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"a1", ",", "a2", ",", "a3"}], "]"}]], "Input",
 CellTags->"PolarizationVector",
 CellLabel->"In[1322]:="]
}, Open  ]],

Cell["PositiveInteger", "ObjectName",
 CellTags->"PositiveInteger"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PositiveInteger"],

Cell["\<\
PositiveInteger is a data type. E.g. DataType[OPEm, PositiveInteger] gives \
True.\
\>", "Text",
 CellTags->"PositiveInteger"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"PositiveInteger"]
}, Open  ]],

Cell["PositiveNumber", "ObjectName",
 CellTags->"PositiveNumber"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PositiveNumber"],

Cell["\<\
PositiveNumber is a data type. E.g. DataType[Epsilon, PositiveNumber] = True \
(by default). \
\>", "Text",
 CellTags->"PositiveNumber"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 "."
}], "Text",
 CellTags->"PositiveNumber"]
}, Open  ]],

Cell["PowerFactor", "ObjectName",
 CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PowerFactor"],

Cell[TextData[{
 "PowerFactor[exp] replaces ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "a"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox["y", "a"], TraditionalForm]]],
 "with ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"PowerFactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PowerSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PowerSimplify", All, CellTags],
  ButtonData->"PowerSimplify",
  ButtonNote->"PowerSimplify"],
 "."
}], "Text",
 CellTags->"PowerFactor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PowerFactor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  SuperscriptBox["x", "a"], 
  SuperscriptBox["y", "a"]}]], "Input",
 CellTags->"PowerFactor",
 CellLabel->"In[1323]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["x", "a"], " ", 
   SuperscriptBox["y", "a"]}], TraditionalForm]], "Output",
 CellTags->"PowerFactor",
 CellLabel->"Out[1323]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerFactor", "[", "%", "]"}]], "Input",
 CellTags->"PowerFactor",
 CellLabel->"In[1324]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"x", " ", "y"}], ")"}], "a"], TraditionalForm]], "Output",
 CellTags->"PowerFactor",
 CellLabel->"Out[1324]="]
}, Open  ]]
}, Open  ]],

Cell["PowerSimplify", "ObjectName",
 CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PowerSimplify"],

Cell[TextData[{
 "PowerSimplify[exp] simplifies",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "x"}], ")"}], "a"]}], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "a"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["x", "a"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"y", "-", "x"}], ")"}], "n"], TraditionalForm]]],
 " to ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", "-", "y"}], ")"}], "n"], TraditionalForm]]],
 "; thus assuming that the exponent is an integer (even if it is symbolic). \
Furthermore ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"a", "+", "n"}]], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"a", "+", "n"}]], TraditionalForm]]],
 " are expanded and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ImaginaryI]", 
    RowBox[{"2", " ", "m"}]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
 " and (-1)^(",
 StyleBox["n",
  FontSlant->"Italic"],
 " ",
 StyleBox["m",
  FontSlant->"Italic"],
 ") \[Rule] 1 and (-1)^(",
 StyleBox["n",
  FontSlant->"Italic"],
 " ",
 StyleBox["m",
  FontSlant->"Italic"],
 ") \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    FormBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}],
     TraditionalForm], "m"], TraditionalForm]]],
 " for ",
 StyleBox["n",
  FontSlant->"Italic"],
 " even and odd respectively and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], 
    RowBox[{"-", "n"}]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "n"], TraditionalForm]]],
 " and ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[ExponentialE]", 
    StyleBox[
     RowBox[{"\[ImaginaryI]", 
      StyleBox["m",
       FontSlant->"Italic"], "\[Pi]"}]]], TraditionalForm]]],
 " \[Rule] ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"PowerSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["OPEm",
  BaseStyle->"AddOnsLink",
  ButtonData:>"OPEi",
  ButtonNote->"OPEm"],
 "."
}], "Text",
 CellTags->"PowerSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PowerSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1325]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1325]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"OPEm", "+", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1326]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1326]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "^", 
   RowBox[{"(", 
    RowBox[{"OPEm", "-", "2"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1327]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1327]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PowerSimplify", "[", 
  RowBox[{"I", "^", 
   RowBox[{"(", 
    RowBox[{"2", "OPEm"}], ")"}]}], "]"}]], "Input",
 CellTags->"PowerSimplify",
 CellLabel->"In[1328]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "1"}], ")"}], "m"], TraditionalForm]], "Output",
 CellTags->"PowerSimplify",
 CellLabel->"Out[1328]="]
}, Open  ]]
}, Open  ]],

Cell["Power2", "ObjectName",
 CellTags->"Power2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Power2"],

Cell["\<\
Power2[x, y] represents x^y.  Sometimes Power2 is more useful than the \
Mathematica Power. Power2[-a,b] simplifies to (-1)^b Power2[a,b] (if no \
Epsilon is in b ...).\
\>", "Text",
 CellTags->"Power2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PowerFactor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PowerFactor", All, CellTags],
  ButtonData->"PowerFactor",
  ButtonNote->"PowerFactor"],
 "."
}], "Text",
 CellTags->"Power2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Power2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Power", "[", 
  RowBox[{
   RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
 CellTags->"Power2",
 CellLabel->"In[1329]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   RowBox[{"(", 
    RowBox[{"-", "a"}], ")"}], "b"], TraditionalForm]], "Output",
 CellTags->"Power2",
 CellLabel->"Out[1329]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Power2", "[", 
  RowBox[{
   RowBox[{"-", "a"}], ",", "b"}], "]"}]], "Input",
 CellTags->"Power2",
 CellLabel->"In[1330]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "b"], " ", 
   FormBox[
    SuperscriptBox["a", "b"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Power2",
 CellLabel->"Out[1330]="]
}, Open  ]]
}, Open  ]],

Cell["PostFortranFile", "ObjectName",
 CellTags->"PostFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PostFortranFile"],

Cell["\<\
PostFortranFile is an option for Write2 which may be set to a file name (or a \
list of file names) or a string,  which will be put at the end of the \
generated Fortran file.\
\>", "Text",
 CellTags->"PostFortranFile"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Write2", All, CellTags],
  ButtonData->"Write2",
  ButtonNote->"Write2"],
 ", ",
 ButtonBox["PreFortranFile",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PreFortranFile", All, CellTags],
  ButtonData->"PreFortranFile",
  ButtonNote->"PreFortranFile"],
 "."
}], "Text",
 CellTags->"PostFortranFile"]
}, Open  ]],

Cell["Prefactor", "ObjectName",
 CellTags->"Prefactor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Prefactor"],

Cell["\<\
Prefactor is an option for OneLoop and OneLoopSum. If set as option of \
OneLoop, the amplitude is multiplied by Prefactor before calculation; if \
specified as option of OneLoopSum, after calculation in the final result as a \
global factor.\
\>", "Text",
 CellTags->"Prefactor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"Prefactor"]
}, Open  ]],

Cell["PreFortranFile", "ObjectName",
 CellTags->"PreFortranFile"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PreFortranFile"],

Cell["\<\
PreFortranFile is an option for Write2 which may be set to a file name (or a \
list of file names) or a string, which will be put at the beginning of the \
generated Fortran file.\
\>", "Text",
 CellTags->"PreFortranFile"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Write2", All, CellTags],
  ButtonData->"Write2",
  ButtonNote->"Write2"],
 ", ",
 ButtonBox["PostFortranFile",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PostFortranFile", All, CellTags],
  ButtonData->"PostFortranFile",
  ButtonNote->"PostFortranFile"],
 "."
}], "Text",
 CellTags->"PreFortranFile"]
}, Open  ]],

Cell["PropagatorDenominator", "ObjectName",
 CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PropagatorDenominator"],

Cell[TextData[{
 "PropagatorDenominator[q, m] is a factor of the denominator of a propagator. \
If p is supposed to be D-dimensional enter: PropagatorDenominator[Momentum[q, \
D], m]. What is meant is ",
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{"1", "/", 
     RowBox[{"(", 
      RowBox[{"q", "^", "2"}]}]}]}], TraditionalForm]]],
 " - ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 "). PropagatorDenominator[p] evaluates to PropagatorDenominator[p, 0]. "
}], "Text",
 CellTags->"PropagatorDenominator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominator", All, CellTags],
  ButtonData->"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["PropagatorDenominatorExplicit",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PropagatorDenominatorExplicit", All, CellTags],
  ButtonData->"PropagatorDenominatorExplicit",
  ButtonNote->"PropagatorDenominatorExplicit"],
 ", ",
 ButtonBox["IFPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "IFPD", All, CellTags],
  ButtonData->"IFPD",
  ButtonNote->"IFPD"],
 "."
}], "Text",
 CellTags->"PropagatorDenominator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PropagatorDenominator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"p", ",", "m"}], "]"}]}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1331]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["p", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1331]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", "p", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1332]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   SuperscriptBox["p", "2"]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1332]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"PropagatorDenominator", "[", 
   RowBox[{"q", ",", "m"}], "]"}], " "}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1333]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox["q", "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1333]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCI", "[", "t1", "]"}], "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1334]:="],

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1334]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"ChangeDimension", "[", 
   RowBox[{"t1", ",", "D"}], "]"}], "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1335]:="],

Cell[BoxData[
 RowBox[{"PropagatorDenominator", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}], ",", "m"}], "]"}]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1335]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"PropagatorDenominatorExplicit", "[", "t1", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1336]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"], "-", 
    SuperscriptBox["m", "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1336]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", "%", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1337]:="],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox["m", "2"]}], "+", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"Momentum", "[", "q", "]"}], ",", 
     RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]}]]], "Output",
 CellTags->"PropagatorDenominator",
 CellLabel->"Out[1337]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t1", "]"}]], "Input",
 CellTags->"PropagatorDenominator",
 CellLabel->"In[1338]:="]
}, Open  ]],

Cell["PropagatorDenominatorExplicit", "ObjectName",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],

Cell["\<\
PropagatorDenominatorExplicit[exp] changes each occurence of \
PropagatorDenominator[a,b] in exp into 1/(ScalarProduct[a,a]-b^2) and \
replaces FeynAmpDenominator by Identity.\
\>", "Text",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominator", All, CellTags],
  ButtonData->"FeynAmpDenominator",
  ButtonNote->"FeynAmpDenominator"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PropagatorDenominator", All, CellTags],
  ButtonData->"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FAD", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"q", ",", "m"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}], "//", 
  "FCI"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1339]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1339]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PropagatorDenominatorExplicit", "[", "%", "]"}], "//", 
  "FCE"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1340]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm], "-", 
      SuperscriptBox["m", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["m", "2"], "-", 
      RowBox[{"2", " ", 
       FormBox[
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}],
        TraditionalForm]}], "+", 
      FormBox[
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], "2"],
       TraditionalForm]}], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1340]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"In[1341]:="],

Cell[BoxData[
 FractionBox["1", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      SuperscriptBox["m", "2"]}], "+", 
     RowBox[{"SPD", "[", 
      RowBox[{"q", ",", "q"}], "]"}]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["m", "2"], "-", 
     RowBox[{"2", " ", 
      RowBox[{"SPD", "[", 
       RowBox[{"p", ",", "q"}], "]"}]}], "+", 
     RowBox[{"SPD", "[", 
      RowBox[{"q", ",", "q"}], "]"}]}], ")"}]}]]], "Output",
 CellTags->"PropagatorDenominatorExplicit",
 CellLabel->"Out[1341]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["QGV", "ObjectName",
 CellTags->"QGV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QGV"],

Cell["QGV is equivalent to QuarkGluonVertex.", "Text",
 CellTags->"QGV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkGluonVertex", All, CellTags],
  ButtonData->"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 "."
}], "Text",
 CellTags->"QGV"]
}, Open  ]],

Cell["QO", "ObjectName",
 CellTags->"QO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QO"],

Cell["QO is equivalent to Twist2QuarkOperator.", "Text",
 CellTags->"QO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2QuarkOperator", All, CellTags],
  ButtonData->"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"QO"]
}, Open  ]],

Cell["QP", "ObjectName",
 CellTags->"QP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QP"],

Cell["\<\
QP is an alias for QuarkPropagator. QP[p] is the massless quark propagator. \
QP[{p,m}] gives the  quark propagator with mass m.\
\>", "Text",
 CellTags->"QP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["QuarkPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkPropagator", All, CellTags],
  ButtonData->"QuarkPropagator",
  ButtonNote->"QuarkPropagator"],
 "."
}], "Text",
 CellTags->"QP"]
}, Open  ]],

Cell["QuantumField", "ObjectName",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuantumField"],

Cell[TextData[{
 "QuantumField is the head of quantized fields and their derivatives. \
QuantumField[par, ftype, {lorind}, {sunind}] denotes a quantum field of type \
ftype with (possible) Lorentz-indices lorind and SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") indices sunind. The optional first argument par denotes a partial \
derivative acting on the field."
}], "Text",
 CellTags->"QuantumField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynRule", All, CellTags],
  ButtonData->"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialD", All, CellTags],
  ButtonData->"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 "."
}], "Text",
 CellTags->"QuantumField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuantumField"],

Cell["This denotes a scalar field.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "S", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1342]:="],

Cell[BoxData[
 FormBox["S", TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1342]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "AntiQuarkField", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1343]:="],

Cell[BoxData[
 FormBox[
  OverscriptBox["\[Psi]", "_"], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1343]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", "QuarkField", "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1344]:="],

Cell[BoxData[
 FormBox["\[Psi]", TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1344]="]
}, Open  ]],

Cell["This is a field with a Lorentz index.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"B", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1345]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["B", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1345]="]
}, Open  ]],

Cell["Color indices should be put after the Lorentz ones.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1346]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   FormBox["A",
    TraditionalForm], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1346]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1347]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"GaugeField", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1347]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["A", "\[CapitalDelta]", "a"], TraditionalForm]]],
 " is a short form for ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalDelta]", "^", "\[Mu]"}], " ", 
    RowBox[{
     SubsuperscriptBox["A", "\[Mu]", "a"], "."}]}], TraditionalForm]]],
 " "
}], "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"A", ",", 
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1348]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["A", 
   FormBox["\[CapitalDelta]",
    TraditionalForm], 
   FormBox[
    FormBox["a",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1348]="]
}, Open  ]],

Cell["\<\
 The first list of indices is usually interpreted as type LorentzIndex, \
except for OPEDelta, which gets converted to type Momentum. \
\>", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"{", "OPEDelta", "}"}], ",", 
    RowBox[{"{", "a", "}"}]}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1349]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{"A", ",", 
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1349]//StandardForm="]
}, Open  ]],

Cell["Derivatives of fields are denoted as follows.", "Text",
 CellTags->"QuantumField"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "\[Mu]", "]"}], ",", "A", ",", 
   RowBox[{"{", "\[Mu]", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1350]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1350]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "S"}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1351]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox["\[CapitalDelta]",
     TraditionalForm]], 
   SubsuperscriptBox["S", "\[Null]", "\[Null]"]}], TraditionalForm]], "Output",\

 CellTags->"QuantumField",
 CellLabel->"Out[1351]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", "OPEDelta", "]"}], ",", "A", ",", 
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1352]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SubscriptBox["\[PartialD]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]],
    TraditionalForm], 
   SubsuperscriptBox["A", 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1352]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"PartialD", "[", "OPEDelta", "]"}], "^", "OPEm"}], ",", "A", ",", 
   
   RowBox[{"{", "OPEDelta", "}"}], ",", 
   RowBox[{"{", "a", "}"}]}], "]"}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1353]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["\[PartialD]", 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox["m",
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1353]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"QuantumField", "[", 
   RowBox[{"QuantumField", "[", "A", "]"}], "]"}], " ", "===", " ", 
  RowBox[{"QuantumField", "[", "A", "]"}]}]], "Input",
 CellTags->"QuantumField",
 CellLabel->"In[1354]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"QuantumField",
 CellLabel->"Out[1354]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkField", "ObjectName",
 CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkField"],

Cell["\<\
QuarkField is the name of a fermionic field. QuarkField is just a name with \
no functional properties. Only typesetting rules are attached.\
\>", "Text",
 CellTags->"QuarkField"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AntiQuarkField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "AntiQuarkField", All, CellTags],
  ButtonData->"AntiQuarkField",
  ButtonNote->"AntiQuarkField"],
 ", ",
 ButtonBox["QuantumField",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuantumField", All, CellTags],
  ButtonData->"QuantumField",
  ButtonNote->"QuantumField"],
 "."
}], "Text",
 CellTags->"QuarkField"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkField"],

Cell[CellGroupData[{

Cell[BoxData["QuarkField"], "Input",
 CellTags->"QuarkField",
 CellLabel->"In[1355]:="],

Cell[BoxData[
 FormBox["\[Psi]", TraditionalForm]], "Output",
 CellTags->"QuarkField",
 CellLabel->"Out[1355]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkGluonVertex", "ObjectName",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkGluonVertex"],

Cell[TextData[{
 "QuarkGluonVertex[",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Mu]", ",", " ", 
    StyleBox["a",
     FontSlant->"Plain"]}], TraditionalForm]]],
 "] gives the Feynman rule for the quark-gluon vertex. "
}], "Text",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QuarkGluonVertex", "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1356]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1356]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonVertex", All, CellTags],
  ButtonData->"GluonVertex",
  ButtonNote->"GluonVertex"],
 "."
}], "Text",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a"}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1357]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1357]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1358]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1358]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "2"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1359]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1359]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{"\[Mu]", ",", "a", ",", 
   RowBox[{"CounterTerm", " ", "\[Rule]", "3"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1360]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["Q", "a", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1360]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", "k", "}"}], ",", 
   RowBox[{"OPE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1361]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuarkGluonVertex", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", "k", "}"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "True"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1361]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", "k", "}"}], ",", 
   RowBox[{"OPE", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"QuarkGluonVertex",
 CellLabel->"In[1362]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"QuarkGluonVertex", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", "k", "}"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkGluonVertex",
 CellLabel->"Out[1362]="]
}, Open  ]]
}, Open  ]],

Cell["QuarkMass", "ObjectName",
 CellTags->"QuarkMass"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkMass"],

Cell["QuarkMass is an option of Amplitude.", "Text",
 CellTags->"QuarkMass"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Amplitude",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Amplitude", All, CellTags],
  ButtonData->"Amplitude",
  ButtonNote->"Amplitude"],
 "."
}], "Text",
 CellTags->"QuarkMass"]
}, Open  ]],

Cell["QuarkPropagator", "ObjectName",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"QuarkPropagator"],

Cell["\<\
QuarkPropagator[p] is the massless quark propagator. QuarkPropagator[{p,m}] \
or gives the quark propagator with mass m.\
\>", "Text",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "QuarkPropagator", "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1363]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CounterTerm", "\[Rule]", "False"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Loop", "\[Rule]", "0"}], ",", 
    RowBox[{"\[CapitalOmega]", "\[Rule]", "False"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1363]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["QuarkGluonVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "QuarkGluonVertex", All, CellTags],
  ButtonData->"QuarkGluonVertex",
  ButtonNote->"QuarkGluonVertex"],
 "."
}], "Text",
 CellTags->"QuarkPropagator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"QuarkPropagator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", "p", "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1364]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1364]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", 
  RowBox[{"{", 
   RowBox[{"p", ",", "m"}], "}"}], "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1365]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1365]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"QuarkPropagator", "[", 
  RowBox[{"p", ",", "1", ",", "2"}], "]"}]], "Input",
 CellTags->"QuarkPropagator",
 CellLabel->"In[1366]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[CapitalPi]", "q"], "(", "p", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"QuarkPropagator",
 CellLabel->"Out[1366]="]
}, Open  ]]
}, Open  ]],

Cell["ReduceGamma", "ObjectName",
 CellTags->"ReduceGamma"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ReduceGamma"],

Cell["\<\
ReduceGamma is an option of OneLoop. If set to True all DiracMatrix[6] and \
DiracMatrix[7] (i.e. all ChiralityProjector) are reduced to Gamma5.\
\>", "Text",
 CellTags->"ReduceGamma"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"ReduceGamma"]
}, Open  ]],

Cell["ReduceToScalars", "ObjectName",
 CellTags->"ReduceToScalars"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ReduceToScalars"],

Cell["\<\
ReduceToScalars is an option for OneLoop  and OneLoopSum that specifies \
whether the result will be reduced to scalar A0, B0, C0 and D0 scalar \
integrals.\
\>", "Text",
 CellTags->"ReduceToScalars"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"ReduceToScalars"]
}, Open  ]],

Cell["Rename", "ObjectName",
 CellTags->"Rename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Rename"],

Cell["\<\
Rename is an option for Contract. If set to True, dummy indices in Eps are \
renamed, using $MU[i].\
\>", "Text",
 CellTags->"Rename"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"Rename"]
}, Open  ]],

Cell["RHI  ***unfinished***", "ObjectName",
 CellTags->"RHI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHI"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, mark1 \
\[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). The \
exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"RHI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI2FC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI2FC", All, CellTags],
  ButtonData->"RHI2FC",
  ButtonNote->"RHI2FC"],
 "."
}], "Text",
 CellTags->"RHI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHI"],

Cell[BoxData[""], "Input",
 CellTags->"RHI",
 CellLabel->"In[1367]:="]
}, Open  ]],

Cell["RHI2FC  ***unfinished***", "ObjectName",
 CellTags->"RHI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHI2FC"],

Cell["\<\
RHI[{v,w,x,y,z},{a,b,c,d,e,f,g}, {al,be,ga,de,ep}]. (sn \[Rule] 1, mark1 \
\[Rule] 1, mark2 \[Rule] 1, mark3 \[Rule] 1, eph \[Rule] Epsilon/2 ). The \
exponents of the numerator scalar product are (dl = OPEDelta): 
v: k1.k1, w: k2.k2,  x: p.k1, y: p.k2, z: k1.k2. 
a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: dl.(k1-k2), f: \
dl.(p+k1-k2), g: dl.(p-k1-k2) 
RHI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  RHI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}]; 
RHI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
RHI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"RHI2FC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI2FC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI2FC", All, CellTags],
  ButtonData->"RHI2FC",
  ButtonNote->"RHI2FC"],
 "."
}], "Text",
 CellTags->"RHI2FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHI2FC"],

Cell[BoxData[""], "Input",
 CellTags->"RHI2FC",
 CellLabel->"In[1368]:="]
}, Open  ]],

Cell["RHM  ***unfinished***", "ObjectName",
 CellTags->"RHM"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHM"],

Cell["RHM[] is like RHI[], gives Gamma functions.", "Text",
 CellTags->"RHM"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"RHM"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHM"],

Cell[BoxData[""], "Input",
 CellTags->"RHM",
 CellLabel->"In[1369]:="]
}, Open  ]],

Cell["RHO  ***unfinished***", "ObjectName",
 CellTags->"RHO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHO"],

Cell["\<\
RHO[i] with i from 1 to 4 is an abbreviation for the 4 operators (eq. \
(3.2.17) -- (3.2.20)) defined in R.Hambergs thesis. The Lorentz indices are \
suppressed. The explicit expressions may be recovered by RHO[i, mu, nu, p].\
\>", "Text",
 CellTags->"RHO"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHI", All, CellTags],
  ButtonData->"RHI",
  ButtonNote->"RHI"],
 "."
}], "Text",
 CellTags->"RHO"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHO"],

Cell[BoxData[""], "Input",
 CellTags->"RHO",
 CellLabel->"In[1370]:="]
}, Open  ]],

Cell["RHP  ", "ObjectName",
 CellTags->"RHP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RHP"],

Cell["\<\
RHP[i, mu, nu, p] with i from 1 to 4 gives the projectors for RHO[i]. RHP[mu, \
nu, p] gives Sum[RHP[i,mu, nu, p] RHO[i], {i, 4}] collected with respect to \
mu and nu. See the thesis of Roelof Hamberg (but correcting a misprint).\
\>", "Text",
 CellTags->"RHP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["RHO",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "RHO", All, CellTags],
  ButtonData->"RHO",
  ButtonNote->"RHO"],
 "."
}], "Text",
 CellTags->"RHP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RHP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"1", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1371]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", "\[Nu]"}]], 
    RowBox[{"D", "-", "2"}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "2"}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}],
       TraditionalForm], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"2", "-", "D"}], ")"}], " ", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[1371]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"2", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1372]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], 
   SuperscriptBox[
    FormBox[
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}],
     TraditionalForm], "2"]], TraditionalForm]], "Output",
 CellLabel->"Out[1372]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"3", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1373]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]]}], 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]], "-", 
   FractionBox[
    RowBox[{
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], 
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[1373]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RHP", "[", 
  RowBox[{"4", ",", "\[Mu]", ",", "\[Nu]", ",", "p"}], "]"}]], "Input",
 CellLabel->"In[1374]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]]}], 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}]], TraditionalForm]], "Output",
 CellLabel->"Out[1374]="]
}, Open  ]],

Cell[BoxData[""], "Input",
 CellTags->"RHP",
 CellLabel->"In[1375]:="]
}, Open  ]],

Cell["RTL  ***unfinished***", "ObjectName",
 CellTags->"RTL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RTL"],

Cell["\<\
RTL[exp] inserts the list of known TLI integrals into exp, substitutes D\
\[RightArrow]4+Epsilon and expands around Epsilon up to the finite part.\
\>", "Text",
 CellTags->"RTL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TLI", All, CellTags],
  ButtonData->"TLI",
  ButtonNote->"TLI"],
 ", ",
 ButtonBox["Epsilon",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Epsilon", All, CellTags],
  ButtonData->"Epsilon",
  ButtonNote->"Epsilon"],
 "."
}], "Text",
 CellTags->"RTL"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RTL"],

Cell[BoxData[""], "Input",
 CellTags->"RTL",
 CellLabel->"In[1376]:="]
}, Open  ]],

Cell["RightPartialD", "ObjectName",
 CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RightPartialD"],

Cell[TextData[{
 "RightPartialD[",
 Cell[BoxData[
  FormBox["\[Mu]", TraditionalForm]]],
 "] denotes ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[PartialD]", "\[Mu]"], TraditionalForm]]],
 ", acting to the right."
}], "Text",
 CellTags->"RightPartialD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ExpandPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandPartialD", All, CellTags],
  ButtonData->"ExpandPartialD",
  ButtonNote->"ExpandPartialD"],
 ", ",
 ButtonBox["PartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PartialD", All, CellTags],
  ButtonData->"PartialD",
  ButtonNote->"PartialD"],
 ", ",
 ButtonBox["LeftPartialD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LeftPartialD", All, CellTags],
  ButtonData->"LeftPartialD",
  ButtonNote->"LeftPartialD"],
 "."
}], "Text",
 CellTags->"RightPartialD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RightPartialD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RightPartialD", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1377]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox[
   OverscriptBox["\[PartialD]", "\[RightArrow]"], 
   FormBox[
    FormBox["\[Mu]",
     TraditionalForm],
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1377]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"QuantumField", "[", 
   RowBox[{"A", ",", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1378]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["\[PartialD]", "\[RightArrow]"], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SubscriptBox["A", "\[Mu]"]}], TraditionalForm]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1378]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandPartialD", "[", "%", "]"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1379]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[PartialD]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   SubsuperscriptBox["A", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], "\[Null]"]}], TraditionalForm]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1379]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1380]:="],

Cell[BoxData[
 RowBox[{"QuantumField", "[", 
  RowBox[{
   RowBox[{"PartialD", "[", 
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}], ",", "A", ",", 
   RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}]}], "]"}]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1380]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"RightPartialD", "[", "\[Mu]", "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"RightPartialD",
 CellLabel->"In[1381]:="],

Cell[BoxData[
 RowBox[{"RightPartialD", "[", 
  RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], "]"}]], "Output",
 CellTags->"RightPartialD",
 CellLabel->"Out[1381]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["RussianTrick", "ObjectName",
 CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"RussianTrick"],

Cell["\<\
RussianTrick[exp, k, {q1,q2,p}] (=RussianTrick[exp,p,p,{q1,q2,p}]) does the \
integration by parts where p is the external momentum. RussianTrick[exp, k,l, \
{q1,q2,p}] (=RussianTrick[exp,k,l]) does integration by parts where l is the \
momentum to be differentiated.\
\>", "Text",
 CellTags->"RussianTrick"],

Cell["The result is an expression which is vanishing.", "Text",
 CellTags->"RussianTrick"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FourDivergence",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourDivergence", All, CellTags],
  ButtonData->"FourDivergence",
  ButtonNote->"FourDivergence"],
 ", ",
 ButtonBox["FourLaplacian",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FourLaplacian", All, CellTags],
  ButtonData->"FourLaplacian",
  ButtonNote->"FourLaplacian"],
 "."
}], "Text",
 CellTags->"RussianTrick"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"RussianTrick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"RHI", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"OPEm", ",", "0", ",", " ", "0", ",", "0", ",", " ", "0"}], 
     "}"}], ",", " ", 
    RowBox[{"{", 
     RowBox[{
     "1", ",", " ", "1", ",", " ", "1", ",", " ", "1", ",", " ", "1"}], 
     "}"}]}], "]"}]}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1382]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox[
   StyleBox["T",
    FontFamily->"Times",
    FontWeight->"Bold"], 
   StyleBox[
    RowBox[{"1", "1", "1", "1", "1"}],
    FontColor->RGBColor[0, 0, 1]], 
   RowBox[{
    FormBox["m",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm], 
    FormBox["0",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1382]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "RHI2FC"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1383]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q1",
       TraditionalForm]}], ")"}], "m"], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox["q2", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1383]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"RussianTrick", "[", 
  RowBox[{
   RowBox[{"%", "//", "RHI2FC"}], ",", "q2"}], "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1384]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "4"}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q1",
         TraditionalForm]}], ")"}], "m"]}], 
    RowBox[{
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "-", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"]}]], "+", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q1",
        TraditionalForm]}], ")"}], "m"], 
    RowBox[{
     SuperscriptBox["q2", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1384]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FC2RHI", "[", "%", "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1385]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"D", "-", "4"}], ")"}], " ", 
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "1", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "2", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm]}], "+", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"0", "1", "1", "1", "2"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm], "-", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"1", "0", "1", "1", "2"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm], "-", 
   FormBox[
    SubsuperscriptBox[
     StyleBox["T",
      FontFamily->"Times",
      FontWeight->"Bold"], 
     StyleBox[
      RowBox[{"1", "0", "1", "2", "1"}],
      FontColor->RGBColor[0, 0, 1]], 
     RowBox[{
      FormBox["m",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm], 
      FormBox["0",
       TraditionalForm]}]],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1385]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{"%", ",", "t"}], "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1386]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    FormBox[
     SubsuperscriptBox[
      StyleBox["T",
       FontFamily->"Times",
       FontWeight->"Bold"], 
      StyleBox[
       RowBox[{"1", "1", "1", "1", "1"}],
       FontColor->RGBColor[0, 0, 1]], 
      RowBox[{
       FormBox["m",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm], 
       FormBox["0",
        TraditionalForm]}]],
     TraditionalForm], "\[Rule]", 
    FractionBox[
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       FormBox[
        SubsuperscriptBox[
         StyleBox["T",
          FontFamily->"Times",
          FontWeight->"Bold"], 
         StyleBox[
          RowBox[{"1", "1", "1", "2", "1"}],
          FontColor->RGBColor[0, 0, 1]], 
         RowBox[{
          FormBox["m",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm], 
          FormBox["0",
           TraditionalForm]}]],
        TraditionalForm]}], "+", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"0", "1", "1", "1", "2"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm], "-", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"1", "0", "1", "1", "2"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm], "-", 
      FormBox[
       SubsuperscriptBox[
        StyleBox["T",
         FontFamily->"Times",
         FontWeight->"Bold"], 
        StyleBox[
         RowBox[{"1", "0", "1", "2", "1"}],
         FontColor->RGBColor[0, 0, 1]], 
        RowBox[{
         FormBox["m",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm], 
         FormBox["0",
          TraditionalForm]}]],
       TraditionalForm]}], 
     RowBox[{"4", "-", "D"}]]}], "}"}], TraditionalForm]], "Output",
 CellTags->"RussianTrick",
 CellLabel->"Out[1386]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellTags->"RussianTrick",
 CellLabel->"In[1387]:="]
}, Open  ]],

Cell["ScalarGluonVertex", "ObjectName",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarGluonVertex"],

Cell["\<\
ScalarGluonVertex[{p}, {q}, {mu,a}] or ScalarGluonVertex[ p,  q,  mu, a ] \
yields the scalar-scalar-gluon vertex (p and q are incoming momenta).
ScalarGluonVertex[{mu,a}, {nu,b}] yields the scalar-scalar-gluon-gluon vertex \
(p and q are incoming momenta).
The dimension  and the name of the coupling constant are determined by the \
options Dimension and CouplingConstant.\
\>", "Text",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ScalarGluonVertex", "]"}]], "Input",
 CellTags->"ScalarGluonVertex",
 CellLabel->"In[1388]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"ScalarGluonVertex",
 CellLabel->"Out[1388]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarGluonVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarGluonVertex", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", " ", 
   RowBox[{"{", "q", "}"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"mu", ",", " ", "a"}], "}"}]}], "]"}]], "Input",
 CellTags->"ScalarGluonVertex",
 CellLabel->"In[1389]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   SubscriptBox["T", 
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     FormBox[
      RowBox[{"p", "-", "q"}],
      TraditionalForm], ")"}], 
    FormBox[
     FormBox["mu",
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"ScalarGluonVertex",
 CellLabel->"Out[1389]="]
}, Open  ]]
}, Open  ]],

Cell["ScalarProduct", "ObjectName",
 CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarProduct"],

Cell[TextData[{
 "ScalarProduct[p, q] is the input for scalar product. ScalarProduct[p] is \
equivalent to ScalarProduct[p, p]. Expansion of sums of momenta in \
ScalarProduct is done with ExpandScalarProduct. Scalar product may be set, \
e.g., ScalarProduct[a, b] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 "; but a and b must not contain sums. Note that ScalarProduct[a, b] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 " actually sets also: Pair[Momentum[a, ___], Momentum[b, ___]] = ",
 Cell[BoxData[
  FormBox[
   RowBox[{"m", "^", "2"}], TraditionalForm]]],
 ". It is encouraged to always set ScalarProduct's ",
 StyleBox["before",
  FontWeight->"Bold"],
 " any calculation. This improves the performance of FeynCalc ."
}], "Text",
 CellTags->"ScalarProduct"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ClearScalarProducts",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ClearScalarProducts", All, CellTags],
  ButtonData->"ClearScalarProducts",
  ButtonNote->"ClearScalarProducts"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandScalarProduct", All, CellTags],
  ButtonData->"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProductCancel", All, CellTags],
  ButtonData->"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SP", All, CellTags],
  ButtonData->"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SPD", All, CellTags],
  ButtonData->"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ScalarProduct"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarProduct"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1390]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1390]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{
   RowBox[{"p", "+", "q"}], ",", 
   RowBox[{"-", "q"}]}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1391]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"(", 
    RowBox[{
     FormBox["q",
      TraditionalForm], "\[CenterDot]", "(", 
     FormBox[
      RowBox[{"p", "+", "q"}],
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1391]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", 
  RowBox[{"p", ",", "p"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1392]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["p",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1392]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProduct", "[", "q", "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1393]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox[
   FormBox["q",
    TraditionalForm], "2"], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1393]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1394]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", 
   RowBox[{"Momentum", "[", "q", "]"}]}], "]"}]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1394]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{"p", ",", "q", ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1395]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"q", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1395]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    SubscriptBox["p", "1"], ",", 
    SubscriptBox["p", "2"]}], "]"}], " ", "=", " ", 
  RowBox[{"s", "/", "2"}]}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1396]:="],

Cell[BoxData[
 FormBox[
  FractionBox["s", "2"], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1396]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", " ", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "-", "q"}], ",", 
    RowBox[{
     SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1397]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      SubscriptBox["p", "1"],
      TraditionalForm]}]}], "+", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "2"],
     TraditionalForm]}], "+", 
   FractionBox["s", "2"]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1397]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"ScalarProduct", "[", 
   RowBox[{
    RowBox[{
     SubscriptBox["p", "1"], "-", "q"}], ",", 
    RowBox[{
     SubscriptBox["p", "2"], "-", "k"}]}], "]"}], "]"}]], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1398]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox[
      SubscriptBox["p", "1"],
      TraditionalForm]}]}], "+", 
   RowBox[{
    FormBox["k",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox[
     SubscriptBox["p", "2"],
     TraditionalForm]}], "+", 
   FractionBox["s", "2"]}], TraditionalForm]], "Output",
 CellTags->"ScalarProduct",
 CellLabel->"Out[1398]="]
}, Open  ]],

Cell[BoxData["ClearScalarProducts"], "Input",
 CellTags->"ScalarProduct",
 CellLabel->"In[1399]:="]
}, Open  ]],

Cell["ScalarProductCancel", "ObjectName",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScalarProductCancel"],

Cell["\<\
ScalarProductCancel[exp, q1, q2, ...] cancels scalar products with \
propagators. ScalarProductCancel[exp] cancels simple cases.\
\>", "Text",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData["SPC"], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1400]:="],

Cell[BoxData[
 FormBox["ScalarProductCancel", TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1400]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProductCancel", "//", "Options"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1401]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1401]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "SPC"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1402]:="],

Cell[BoxData[
 StyleBox["\<\"SPC is an abbreviation for ScalarProductCancel.\"\>", 
  "MSG"]], "Print", "PrintUsage",
 CellTags->{"Info3535453289-5096475", "ScalarProductCancel"}]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ClearScalarProducts",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ClearScalarProducts", All, CellTags],
  ButtonData->"ClearScalarProducts",
  ButtonNote->"ClearScalarProducts"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandScalarProduct", All, CellTags],
  ButtonData->"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["SP",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SP", All, CellTags],
  ButtonData->"SP",
  ButtonNote->"SP"],
 ", ",
 ButtonBox["SPD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SPD", All, CellTags],
  ButtonData->"SPD",
  ButtonNote->"SPD"],
 "."
}], "Text",
 CellTags->"ScalarProductCancel"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ScalarProductCancel"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynAmpDenominatorSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynAmpDenominatorSimplify", All, CellTags],
  ButtonData->"FeynAmpDenominatorSimplify",
  ButtonNote->"FeynAmpDenominatorSimplify"],
 "."
}], "Text",
 CellTags->"ScalarProductCancel"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"q", "-", "p"}], ",", "0"}], "}"}]}], "]"}]}], "//", 
   "FCI"}]}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1403]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1403]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ScalarProductCancel", "[", 
  RowBox[{"t1", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1404]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]], "+", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]]}], TraditionalForm]], "Output",\

 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1404]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FeynAmpDenominatorSimplify", "[", 
  RowBox[{"%", ",", "q"}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1405]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1405]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPC", "[", 
  RowBox[{"t1", ",", "q", ",", 
   RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1406]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q", "-", "p"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}]}]]}], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1406]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", " ", "=", " ", 
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"q2", ",", "p"}], "]"}], 
    RowBox[{"SPD", "[", 
     RowBox[{"q1", ",", "p"}], "]"}], " ", 
    RowBox[{"FAD", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q1", ",", "m"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q2", ",", "m"}], "}"}], ",", 
      RowBox[{"q1", "-", "p"}], ",", 
      RowBox[{"q2", "-", "p"}], ",", 
      RowBox[{"q2", "-", "q1"}]}], "]"}]}], "//", "FCI"}]}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1407]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q1", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1407]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPC", "[", 
  RowBox[{"t2", ",", "q1", ",", "q2", ",", 
   RowBox[{"FDS", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1408]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{"-", 
      FractionBox[
       SuperscriptBox["m", "2"], "2"]}], "-", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "2"], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], "2"]}], 
    RowBox[{
     SuperscriptBox["q1", "2"], ".", 
     SuperscriptBox["q2", "2"], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "p"}], ")"}], "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]], "-", 
   FractionBox["1", 
    RowBox[{"2", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q1", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]}]], "+", 
   FractionBox["1", 
    RowBox[{"4", " ", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q1", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["q2", "2"], "-", 
        SuperscriptBox["m", "2"]}], ")"}], ".", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     FractionBox[
      SuperscriptBox["m", "4"], "4"], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"]}], "2"], "+", 
     FractionBox[
      SuperscriptBox[
       FormBox[
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"],
        TraditionalForm], "2"], "4"]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q1", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox["q2", "2"], "-", 
       SuperscriptBox["m", "2"]}], ")"}], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "p"}], ")"}], "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"q2", "-", "q1"}], ")"}], "2"]}]]}], 
  TraditionalForm]], "Output",
 CellTags->"ScalarProductCancel",
 CellLabel->"Out[1408]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2"}], "]"}], ";"}]], "Input",
 CellTags->"ScalarProductCancel",
 CellLabel->"In[1409]:="]
}, Open  ]],

Cell["ScaleMu", "ObjectName",
 CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ScaleMu"],

Cell["\<\
ScaleMu is the mass scale used for dimensional regularization of loop \
integrals.\
\>", "Text",
 CellTags->"ScaleMu"],

Cell[CellGroupData[{

Cell[BoxData["ScaleMu"], "Input",
 CellTags->"ScaleMu",
 CellLabel->"In[1410]:="],

Cell[BoxData[
 FormBox["\[Mu]", TraditionalForm]], "Output",
 CellTags->"ScaleMu",
 CellLabel->"Out[1410]="]
}, Open  ]]
}, Open  ]],

Cell["Schouten", "ObjectName",
 CellTags->"Schouten"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Schouten"],

Cell["\<\
Schouten[expr] applies the Schouten identity on at most 42 terms in a sum. If \
Schouten should operate on larger expression you can give a second argument, \
e.g.: Schouten[expr, 4711] which will work on sums with less than 4711 terms.
Schouten is also an option of Contract and DiracTrace. It may be set to an \
integer indicating the maximum number of terms onto which the function \
Schouten will be applied .\
\>", "Text",
 CellTags->"Schouten"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"Schouten"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Schouten"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      UnderoverscriptBox["\[Sum]", 
       RowBox[{"a", "=", "1"}], "4"], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SP", "[", 
         RowBox[{"k", ",", 
          RowBox[{"q", "[", "a", "]"}]}], "]"}], 
        RowBox[{"(", 
         RowBox[{
          FractionBox["1", "6"], 
          RowBox[{"Eps", "[", 
           RowBox[{
            RowBox[{"LorentzIndex", "[", "a", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "b", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "c", "]"}], ",", 
            RowBox[{"LorentzIndex", "[", "d", "]"}]}], "]"}], 
          RowBox[{"Eps", "[", 
           RowBox[{
            RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "b", "]"}], "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "c", "]"}], "]"}], ",", 
            RowBox[{"Momentum", "[", 
             RowBox[{"q", "[", "d", "]"}], "]"}]}], "]"}]}], ")"}]}], ")"}]}],
      ",", 
     RowBox[{"{", 
      RowBox[{"b", ",", "1", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"c", ",", "1", ",", "4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"d", ",", "1", ",", "4"}], "}"}]}], "]"}], "-", 
   RowBox[{
    RowBox[{"Eps", "[", 
     RowBox[{
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "1", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "2", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "3", "]"}], "]"}], ",", 
      RowBox[{"Momentum", "[", 
       RowBox[{"q", "[", "4", "]"}], "]"}]}], "]"}], "*", 
    RowBox[{"Pair", "[", 
     RowBox[{
      RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
      RowBox[{"Momentum", "[", "k", "]"}]}], "]"}]}]}]}]], "Input",
 CellTags->"Schouten",
 CellLabel->"In[1411]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["k",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         RowBox[{"q", "(", "1", ")"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"q", "(", "2", ")"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"q", "(", "3", ")"}],
         TraditionalForm], 
        FormBox[
         RowBox[{"q", "(", "4", ")"}],
         TraditionalForm]}]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    FormBox[
     RowBox[{
      FormBox["k",
       TraditionalForm], "\[CenterDot]", 
      FormBox[
       RowBox[{"q", "(", "4", ")"}],
       TraditionalForm]}],
     TraditionalForm], " ", 
    SuperscriptBox["\[Epsilon]", 
     RowBox[{
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "3", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "2", ")"}],
       TraditionalForm], 
      FormBox[
       RowBox[{"q", "(", "1", ")"}],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Schouten",
 CellLabel->"Out[1411]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "//", "Schouten"}]], "Input",
 CellTags->"Schouten",
 CellLabel->"In[1412]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"Schouten",
 CellLabel->"Out[1412]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "t", "]"}]], "Input",
 CellLabel->"In[1413]:="]
}, Open  ]],

Cell["SD", "ObjectName",
 CellTags->"SD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SD"],

Cell[TextData[{
 "SD[i, j] is the (FeynCalc-external) Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with color indices i and j. SD[i,j] is transformed into \
SUNDelta[SUNIndex[i],SUNIndex[j]] by FeynCalcInternal."
}], "Text",
 CellTags->"SD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNDelta", All, CellTags],
  ButtonData->"SUNDelta",
  ButtonNote->"SUNDelta"],
 "."
}], "Text",
 CellTags->"SD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1414]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"SD",
 CellLabel->"Out[1414]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCI"}], "//", "StandardForm"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1415]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SD",
 CellLabel->"Out[1415]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"SD",
 CellLabel->"In[1416]:="],

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SD",
 CellLabel->"Out[1416]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SelectFree", "ObjectName",
 CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectFree"],

Cell["\<\
SelectFree[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, {a,b, \
...}]&], except the special cases: SelectFree[a, b] returns a and \
SelectFree[a,a] returns 1 (where a is not a product or a sum).\
\>", "Text",
 CellTags->"SelectFree"],

Cell["SelectFree is equivalent to Select1.", "Text",
 CellTags->"SelectFree"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FreeQ2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FreeQ2", All, CellTags],
  ButtonData->"FreeQ2",
  ButtonNote->"FreeQ2"],
 ", ",
 ButtonBox["SelectNotFree",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SelectNotFree", All, CellTags],
  ButtonData->"SelectNotFree",
  ButtonNote->"SelectNotFree"],
 "."
}], "Text",
 CellTags->"SelectFree"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectFree"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "b", "+", 
    RowBox[{"f", "[", "a", "]"}], "+", "d"}], ",", "a"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1417]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"b", "+", "d"}], TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1417]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"x", " ", "y"}], ",", " ", "x"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1418]:="],

Cell[BoxData[
 FormBox["y", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1418]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"2", " ", "x", " ", "y", " ", "z", " ", 
    RowBox[{"f", "[", "x", "]"}]}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1419]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "z"}], TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1419]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1420]:="],

Cell[BoxData[
 FormBox["a", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1420]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"a", ",", "a"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1421]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1421]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{"1", ",", "c"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1422]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1422]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectFree", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"SelectFree",
 CellLabel->"In[1423]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectFree",
 CellLabel->"Out[1423]="]
}, Open  ]]
}, Open  ]],

Cell["SelectGraphs", "ObjectName",
 CellTags->"SelectGraphs"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectGraphs"],

Cell["\<\
SelectGraphs is an option for OneLoopSum indicating that only a slected set \
of graphs of the list provided to OneLoopSum is to be calculated. Possible \
settings are: SelectGraphs\[RightArrow]{ i, j,  ... } or SelectGraphs\
\[RightArrow]{ a, {b, c}, ... } which indicates the graphs to be taken from \
the list provided to OneLoopSum. In the second setting the list {b, c} \
indicates that all amplitudes from b to c should be taken.\
\>", "Text",
 CellTags->"SelectGraphs"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"SelectGraphs"]
}, Open  ]],

Cell["SelectNotFree", "ObjectName",
 CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, x] returns that part of expr which is not free of any \
occurance of x.\
\>", "Text",
 CellTags->"SelectNotFree"],

Cell["\<\
SelectNotFree[expr, a, b, ...] is equivalent to Select[expr, !FreeQ2[#, {a, \
b, ...}]&], except the special cases: SelectNotFree[a, b] returns 1 and \
SelectNotFree[a, a] returns a (where a is not a product or a sum).\
\>", "Text",
 CellTags->"SelectNotFree"],

Cell["SelectNotFree is equivalent to Select2.", "Text",
 CellTags->"SelectNotFree"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FreeQ2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FreeQ2", All, CellTags],
  ButtonData->"FreeQ2",
  ButtonNote->"FreeQ2"],
 ", ",
 ButtonBox["SelectFree",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SelectFree", All, CellTags],
  ButtonData->"SelectFree",
  ButtonNote->"SelectFree"],
 "."
}], "Text",
 CellTags->"SelectNotFree"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectNotFree"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "b", "+", 
    RowBox[{"f", "[", "a", "]"}]}], ",", "a"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1424]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", "a", ")"}], "+", "a"}], TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1424]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"2", " ", "x", " ", "y", " ", 
    RowBox[{"f", "[", "x", "]"}], " ", "z"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1425]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", " ", "y", " ", 
   RowBox[{"f", "(", "x", ")"}]}], TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1425]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1426]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1426]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"a", "+", "x"}], ",", "b"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1427]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1427]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"a", ",", "a"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1428]:="],

Cell[BoxData[
 FormBox["a", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1428]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{"1", ",", "c"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1429]:="],

Cell[BoxData[
 FormBox["1", TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1429]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectNotFree", "[", 
  RowBox[{
   RowBox[{"f", "[", "x", "]"}], ",", "x"}], "]"}]], "Input",
 CellTags->"SelectNotFree",
 CellLabel->"In[1430]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"f", "(", "x", ")"}], TraditionalForm]], "Output",
 CellTags->"SelectNotFree",
 CellLabel->"Out[1430]="]
}, Open  ]]
}, Open  ]],

Cell["SelectSplit", "ObjectName",
 CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SelectSplit"],

Cell["\<\
SelectSplit[l, p] Construct list of mutually exclusive subsets from l in \
which every element li satisfies a criterium pj[li] with pj from p and \
appends the subset of remaining unmatched elements.\
\>", "Text",
 CellTags->"SelectSplit"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SelectSplit"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectSplit", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["a", "2"], ",", 
     SuperscriptBox["b", "3"], ",", 
     SuperscriptBox["c", "4"], ",", 
     SuperscriptBox["d", "5"], ",", 
     SuperscriptBox["e", "6"], ",", 
     RowBox[{"f", "+", "g"}], ",", 
     SuperscriptBox["h", "4"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"MatchQ", "[", 
       RowBox[{"#", ",", 
        RowBox[{"_", "^", "2"}]}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"MatchQ", "[", 
       RowBox[{"#", ",", 
        RowBox[{"_", "^", "4"}]}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectSplit",
 CellLabel->"In[1431]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     SuperscriptBox["a", "2"], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["c", "4"], ",", 
      SuperscriptBox["h", "4"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"f", "+", "g"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["b", "3"], ",", 
      SuperscriptBox["d", "5"], ",", 
      SuperscriptBox["e", "6"]}], "}"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"SelectSplit",
 CellLabel->"Out[1431]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SelectSplit", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     SuperscriptBox["a", "2"], ",", 
     SuperscriptBox["b", "3"], ",", 
     SuperscriptBox["c", "4"], ",", 
     SuperscriptBox["d", "5"], ",", 
     SuperscriptBox["e", "6"], ",", 
     RowBox[{"f", "+", "g"}], ",", 
     SuperscriptBox["h", "4"]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Plus"}], "]"}], "&"}], ",", 
     RowBox[{
      RowBox[{"FreeQ", "[", 
       RowBox[{"#", ",", "Power"}], "]"}], "&"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"SelectSplit",
 CellLabel->"In[1432]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      SuperscriptBox["a", "2"], ",", 
      SuperscriptBox["b", "3"], ",", 
      SuperscriptBox["c", "4"], ",", 
      SuperscriptBox["d", "5"], ",", 
      SuperscriptBox["e", "6"], ",", 
      SuperscriptBox["h", "4"]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"f", "+", "g"}], "}"}], ",", 
    RowBox[{"{", "}"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"SelectSplit",
 CellLabel->"Out[1432]="]
}, Open  ]]
}, Open  ]],

Cell["Select1", "ObjectName",
 CellTags->"Select1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Select1"],

Cell["\<\
Select1[expr, a, b, ...] is equivalent to Select[expr, FreeQ2[#, {a,b, \
...}]&], except the special cases: Select1[a, b] returns a and Select1[a,a] \
returns 1 (where a is not a product or a sum).\
\>", "Text",
 CellTags->"Select1"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Select2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Select2", All, CellTags],
  ButtonData->"Select2",
  ButtonNote->"Select2"],
 "."
}], "Text",
 CellTags->"Select1"]
}, Open  ]],

Cell["Select2", "ObjectName",
 CellTags->"Select2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Select2"],

Cell["\<\
Select2[expr, a, b, ...] is equivalent to Select[expr, !FreeQ2[#, {a,b, \
...}]&], except the special cases: Select2[a, b] returns 1 and  Select2[a,a] \
returns a (where a is not a product or a sum).\
\>", "Text",
 CellTags->"Select2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Select1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Select1", All, CellTags],
  ButtonData->"Select1",
  ButtonNote->"Select1"],
 "."
}], "Text",
 CellTags->"Select2"]
}, Open  ]],

Cell["Series2", "ObjectName",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Series2"],

Cell[TextData[{
 " Series2 performs a series expansion around 0. Series2 is (up to the \
Gamma-bug in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " versions smaller than 5.0) equivalent to Series, except that it applies \
Normal on the result and has an option FinalSubstitutions. Series2[f, e, n] \
is equivalent to Series2[f, {e, 0, n}]."
}], "Text",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Series2", "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1433]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "\[Rule]", "0"}], "}"}]}], ",", 
    RowBox[{"Print", "\[Rule]", "False"}], ",", 
    RowBox[{"SimplifyPolyLog", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1433]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", "1"}], 
  "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1434]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", "x", ")"}]}], "+", "1"}], TraditionalForm]], "Output",\

 CellTags->"Series2",
 CellLabel->"Out[1434]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1435]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
   FractionBox["1", "x"]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1435]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1436]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{
    FractionBox["1", "x"], "-", 
    TagBox["\[DoubledGamma]",
     Function[{}, EulerGamma]], "+", 
    RowBox[{
     FractionBox["1", "12"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"6", " ", 
        SuperscriptBox[
         TagBox["\[DoubledGamma]",
          Function[{}, EulerGamma]], "2"]}], "+", 
       SuperscriptBox["\[Pi]", "2"]}], ")"}], " ", "x"}], "+", 
    InterpretationBox[
     RowBox[{"O", "(", 
      SuperscriptBox["x", "2"], ")"}],
     SeriesData[$CellContext`x, 0, {}, -1, 2, 1],
     Editable->False]}],
   SeriesData[$CellContext`x, 0, {
    1, -EulerGamma, Rational[1, 12] (6 EulerGamma^2 + Pi^2)}, -1, 2, 1],
   Editable->False], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1436]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1437]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["x", "2"], " ", 
      TemplateBox[{"3"},
       "Zeta"]}], "3"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
   FractionBox["1", "x"]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1437]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series2", "[", 
   RowBox[{
    RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "2", ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "]"}], "//", "FullSimplify"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1438]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"3", " ", 
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
          SuperscriptBox["x", "2"]}], "+", "2"}], ")"}]}]}], "-", 
     RowBox[{"2", " ", 
      SuperscriptBox["x", "2"], " ", 
      TemplateBox[{"3"},
       "Zeta"]}], "-", 
     RowBox[{
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "3"], " ", 
      SuperscriptBox["x", "2"]}], "+", 
     RowBox[{"3", " ", 
      RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
     RowBox[{"3", " ", 
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "2"], " ", "x"}], "+", 
     FractionBox["6", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1438]="]
}, Open  ]],

Cell[TextData[{
 "In earlier ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " versions the Gamma functions was not expanded far enough, in 5.0 this got \
finally fixed:"
}], "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"Series", "[", 
     RowBox[{
      RowBox[{"Gamma", "[", "x", "]"}], ",", 
      RowBox[{"{", 
       RowBox[{"x", ",", "0", ",", 
        RowBox[{"If", "[", 
         RowBox[{
          RowBox[{"$VersionNumber", "<", "5"}], ",", "4", ",", "2"}], "]"}]}],
        "}"}]}], "]"}], "//", "Normal"}], "//", "Expand"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1439]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "12"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        TagBox["\[DoubledGamma]",
         Function[{}, EulerGamma]], "3"], " ", 
       SuperscriptBox["x", "2"]}]}], "-", 
     RowBox[{
      TagBox["\[DoubledGamma]",
       Function[{}, EulerGamma]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox["\[Pi]", "2"], " ", 
         SuperscriptBox["x", "2"]}], "+", "12"}], ")"}]}], "+", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{
        SuperscriptBox["\[Pi]", "2"], "-", 
        RowBox[{"4", " ", "x", " ", 
         TemplateBox[{"3"},
          "Zeta"]}]}], ")"}]}], "+", 
     RowBox[{"6", " ", 
      SuperscriptBox[
       TagBox["\[DoubledGamma]",
        Function[{}, EulerGamma]], "2"], " ", "x"}], "+", 
     FractionBox["12", "x"]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1439]="]
}, Open  ]],

Cell["\<\
There is a table of expansions of special hypergeometric functions.\
\>", "Text",
 CellTags->"Series2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"HypergeometricPFQ", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"1", ",", 
       RowBox[{"OPEm", "-", "1"}], ",", 
       RowBox[{
        RowBox[{"Epsilon", "/", "2"}], "+", "OPEm"}]}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"OPEm", ",", 
       RowBox[{"OPEm", "+", "Epsilon"}]}], "}"}], ",", "1"}], "]"}], ",", 
   "Epsilon", ",", "1"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1440]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["2", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]]}], "+", 
   FractionBox[
    RowBox[{"2", " ", "m"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    TemplateBox[{"1","m"},
     "PolyGamma2"]}], "-", 
   FractionBox[
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     TemplateBox[{"1","m"},
      "PolyGamma2"]}], "2"], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1440]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{" ", 
  RowBox[{"Series2", "[", 
   RowBox[{
    RowBox[{"HypergeometricPFQ", "[", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"1", ",", " ", "OPEm", ",", " ", 
        RowBox[{
         RowBox[{"Epsilon", "/", "2"}], " ", "+", " ", "OPEm"}]}], "}"}], ",",
       " ", 
      RowBox[{"{", 
       RowBox[{
        RowBox[{"1", " ", "+", " ", "OPEm"}], ",", " ", 
        RowBox[{"Epsilon", " ", "+", " ", "OPEm"}]}], "}"}], ",", "  ", "1"}],
      "]"}], ",", "Epsilon", ",", "1"}], "]"}]}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1441]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "m"}], "+", 
   FractionBox[
    RowBox[{"2", " ", "m"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    SuperscriptBox[
     TemplateBox[{"0","m"},
      "PolyGamma2"], "2"]}], "+", 
   RowBox[{
    FractionBox["3", "4"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    TemplateBox[{"1","m"},
     "PolyGamma2"]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", "m", " ", 
    RowBox[{
     SubscriptBox["S", "11"], "(", 
     FormBox[
      RowBox[{"m", "-", "1"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1441]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Hypergeometric2F1", "[", 
  RowBox[{"1", ",", " ", "Epsilon", ",", " ", 
   RowBox[{"1", " ", "+", " ", 
    RowBox[{"2", " ", "Epsilon"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1442]:="],

Cell[BoxData[
 FormBox[
  TemplateBox[{"1",TagBox["\[CurlyEpsilon]", TraditionalForm],RowBox[{
      RowBox[{"2", " ", 
        TagBox["\[CurlyEpsilon]", TraditionalForm]}], "+", "1"}],"x"},
   "Hypergeometric2F1"], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1442]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{"%", ",", "Epsilon", ",", "3"}], "]"}]], "Input",
 CellTags->"Series2",
 CellLabel->"In[1443]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"2", " ", 
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"]}], "-", 
   RowBox[{"4", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{"4", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{
     SuperscriptBox["log", "3"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "-", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    RowBox[{
     SuperscriptBox["log", "2"], "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}], " ", 
    RowBox[{"log", "(", "x", ")"}]}], "-", 
   RowBox[{
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    RowBox[{"log", "(", 
     RowBox[{"1", "-", "x"}], ")"}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    TemplateBox[{"3"},
     "Zeta"]}], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Series2",
 CellLabel->"Out[1443]="]
}, Open  ]],

Cell[TextData[{
 "There are over 100 more special expansions of ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[ThinSpace]", 
    RowBox[{
     SubscriptBox["", "2"], 
     SubscriptBox["F", "1"]}]}], TraditionalForm]]],
 " tabulated in Series2.m. The interested user can consult the source code \
(search for HYPERLIST)."
}], "Text",
 CellTags->"Series2"]
}, Open  ]],

Cell["Series3", "ObjectName",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Series3"],

Cell["\<\
Series3 performs a series expansion around 0. Series3 is equivalent to \
Series, except that it applies Normal on the result and that some Series bugs \
are fixed. Series3[f, e, n] is equivalent to  Series3[f, {e, 0, n}].\
\>", "Text",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Series3", "]"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1444]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "True"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Series3",
 CellLabel->"Out[1444]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Series2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Series2", All, CellTags],
  ButtonData->"Series2",
  ButtonNote->"Series2"],
 "."
}], "Text",
 CellTags->"Series3"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Series3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series3", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"x", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"\[Delta]", "/", "2"}], ")"}]}], ",", "\[Delta]", ",", "1"}], 
  "]"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1445]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", "\[Delta]", " ", 
    RowBox[{"log", "(", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "1"}], 
  TraditionalForm]], "Output",
 CellTags->"Series3",
 CellLabel->"Out[1445]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Series3", "[", 
   RowBox[{
    RowBox[{"Gamma", "[", "x", "]"}], ",", "x", ",", "1"}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"Series3",
 CellLabel->"In[1446]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "x"], "-", 
   TagBox["\[DoubledGamma]",
    Function[{}, EulerGamma]], "+", "1"}], TraditionalForm]], "Output",
 CellTags->"Series3",
 CellLabel->"Out[1446]="]
}, Open  ]]
}, Open  ]],

Cell["SetMandelstam", "ObjectName",
 CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SetMandelstam"],

Cell["\<\
SetMandelstam[s, t, u, p1, p2, p3, p4, m1, m2, m3, m4] defines the Mandelstam \
variables  s=(p1+p2)^2, t=(p1+p3)^2, u=(p1+p4)^2 and sets the pi on-shell: \
p1^2=m1^2, p2^2=m2^2, p3^2=m3^2, p4^2=m4^2. Note that p1 +  p2 + p3 + p4 = 0 \
is assumed.
SetMandelstam[x, {p1, p2, p3, p4, p5}, {m1, m2, m3, m4, m5}] defines x[i, j] \
= (pi+pj)^2 and sets the pi on-shell. The pi satisfy: p1 + p2 + p3 + p4 + p5 \
= 0.\
\>", "Text",
 CellTags->"SetMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SetMandelstam", "]"}]], "Input",
 CellTags->"SetMandelstam",
 CellLabel->"In[1447]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Dimension", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"4", ",", "D", ",", "___"}], "}"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SetMandelstam",
 CellLabel->"Out[1447]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Mandelstam",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Mandelstam", All, CellTags],
  ButtonData->"Mandelstam",
  ButtonNote->"Mandelstam"],
 "."
}], "Text",
 CellTags->"SetMandelstam"]
}, Open  ]],

Cell["SimplifyDeltaFunction", "ObjectName",
 CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],

Cell["\<\
SimplifyDeltaFunction[exp, x] simplifies f[x]*DeltaFunction[1-x] \
toLimit[f[x],x\[Rule]1] DeltaFunction[1-x] and applies a list of \
transformation rules for DeltaFunctionPrime[1-x]*x^(OPEm-1)*f[x] where \
x^(OPEm-1) is suppressed in exp.\
\>", "Text",
 CellTags->"SimplifyDeltaFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DeltaFunction",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeltaFunction", All, CellTags],
  ButtonData->"DeltaFunction",
  ButtonNote->"DeltaFunction"],
 ", ",
 ButtonBox["DeltaFunctionPrime",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeltaFunctionPrime", All, CellTags],
  ButtonData->"DeltaFunctionPrime",
  ButtonNote->"DeltaFunctionPrime"],
 "."
}], "Text",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x", "]"}], " ", 
  RowBox[{"DeltaFunction", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1448]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"g", "(", "x", ")"}], " ", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1448]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1449]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     UnderscriptBox["lim", 
      RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
     RowBox[{"g", "(", "x", ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1449]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"g", "[", "x", "]"}], 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1450]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"g", "(", "x", ")"}], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1450]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1451]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      UnderscriptBox["lim", 
       RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
      RowBox[{
       SuperscriptBox["g", "\[Prime]",
        MultilineFunction->None], "(", "x", ")"}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{
     SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      UnderscriptBox["lim", 
       RowBox[{"x", "\[Rule]", "1"}]], "\[ThinSpace]", 
      RowBox[{"g", "(", "x", ")"}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1451]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"x", " ", 
  RowBox[{"Log", "[", "x", "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1452]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"x", " ", 
   RowBox[{"log", "(", "x", ")"}], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1452]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1453]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Delta]", "(", 
   FormBox[
    RowBox[{"1", "-", "x"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1453]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1454]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1454]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1455]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1455]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Log", "[", "x", "]"}], 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1456]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
    "PolyLog"], " ", 
   RowBox[{"log", "(", "x", ")"}], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1456]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1457]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1457]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], " ", 
  RowBox[{"DeltaFunctionPrime", "[", 
   RowBox[{"1", "-", "x"}], "]"}]}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1458]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
    "PolyLog"], " ", 
   RowBox[{
    SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1458]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyDeltaFunction", "[", " ", 
  RowBox[{"%", ",", "x"}], "]"}]], "Input",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"In[1459]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{"\[Delta]", "(", 
    FormBox[
     RowBox[{"1", "-", "x"}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyDeltaFunction",
 CellLabel->"Out[1459]="]
}, Open  ]]
}, Open  ]],

Cell["SimplifyGTI  ***unfinished***", "ObjectName",
 CellTags->"SimplifyGTI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyGTI"],

Cell["SimplifyGTI simplifies GTI's.", "Text",
 CellTags->"SimplifyGTI"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["GTI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GTI", All, CellTags],
  ButtonData->"GTI",
  ButtonNote->"GTI"],
 "."
}], "Text",
 CellTags->"SimplifyGTI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyGTI"],

Cell[BoxData[""], "Input",
 CellTags->"SimplifyGTI",
 CellLabel->"In[1460]:="]
}, Open  ]],

Cell["SimplifyPolyLog", "ObjectName",
 CellTags->"SimplifyPolyLog"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SimplifyPolyLog"],

Cell["\<\
SimplifyPolyLog[y] performs several simplifications assuming that the \
variables occuring in the Log and PolyLog functions are between 0 and 1. \
\>", "Text",
 CellTags->"SimplifyPolyLog"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Nielsen",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Nielsen", All, CellTags],
  ButtonData->"Nielsen",
  ButtonNote->"Nielsen"],
 "."
}], "Text",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SimplifyPolyLog"],

Cell[BoxData[
 RowBox[{
  RowBox[{"sip", "[", "y_", "]"}], " ", ":=", " ", 
  RowBox[{"y", " ", "==", " ", 
   RowBox[{"SimplifyPolyLog", "[", "y", "]"}]}]}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1461]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "/", "x"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1462]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",FractionBox["1", "x"]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], "+", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{"\[ImaginaryI]", " ", "\[Pi]", " ", 
     RowBox[{"log", "(", "x", ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1462]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", "x"}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1463]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2","x"},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"\[Zeta]", "(", "2", ")"}], "-", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1463]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"1", "-", 
     RowBox[{"x", "^", "2"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1464]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"1", "-", 
       SuperscriptBox["x", "2"]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"x", "+", "1"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1464]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1465]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",SuperscriptBox["x", "2"]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"2", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"]}], "+", 
    RowBox[{"2", " ", 
     TemplateBox[{"2",RowBox[{"-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1465]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1466]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",RowBox[{"-", 
       FractionBox["x", 
        RowBox[{"1", "-", "x"}]]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1466]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"2", ",", 
    RowBox[{"x", "/", 
     RowBox[{"(", 
      RowBox[{"x", "-", "1"}], ")"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1467]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"2",FractionBox["x", 
      RowBox[{"x", "-", "1"}]]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     RowBox[{"\[Zeta]", "(", "2", ")"}]}], "+", 
    TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
     "PolyLog"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1467]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Nielsen", "[", 
   RowBox[{"1", ",", "2", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1468]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", 
     RowBox[{
      FormBox["1",
       TraditionalForm], 
      FormBox["2",
       TraditionalForm]}]], "(", 
    FormBox[
     RowBox[{"-", 
      FractionBox["x", 
       RowBox[{"1", "-", "x"}]]}],
     TraditionalForm], ")"}], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "-", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1468]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", 
     RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1469]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"-", 
       FractionBox["1", "x"]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    TemplateBox[{"3",RowBox[{"-", "x"}]},
     "PolyLog"], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["log", "3"], "(", "x", ")"}], "6"]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1469]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", "x"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1470]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1470]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"x", "^", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1471]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",SuperscriptBox["x", "2"]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"4", " ", 
     TemplateBox[{"3",RowBox[{"-", "x"}]},
      "PolyLog"]}], "-", 
    RowBox[{"4", " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{"4", " ", 
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}]}], "+", 
    RowBox[{"4", " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{"2", " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
    RowBox[{"4", " ", 
     TemplateBox[{"3"},
      "Zeta"]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1471]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1472]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"-", 
       FractionBox["x", 
        RowBox[{"1", "-", "x"}]]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     TemplateBox[{"3",RowBox[{"1", "-", "x"}]},
      "PolyLog"]}], "+", 
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "6"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1472]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"3", ",", 
    RowBox[{"1", "-", 
     RowBox[{"1", "/", "x"}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1473]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"3",RowBox[{"1", "-", 
       FractionBox["1", "x"]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "+", 
    FractionBox[
     RowBox[{
      SuperscriptBox["log", "3"], "(", "x", ")"}], "6"], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}]}], "-", 
    TemplateBox[{"3"},
     "Zeta"]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1473]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"PolyLog", "[", 
   RowBox[{"4", ",", 
    RowBox[{"-", 
     RowBox[{"x", "/", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}]}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1474]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   TemplateBox[{"4",RowBox[{"-", 
       FractionBox["x", 
        RowBox[{"1", "-", "x"}]]}]},
    "PolyLog"], "\[LongEqual]", 
   RowBox[{
    RowBox[{"-", 
     TemplateBox[{"4","x"},
      "PolyLog"]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     TemplateBox[{"2",RowBox[{"1", "-", "x"}]},
      "PolyLog"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["1",
        TraditionalForm], 
       FormBox["3",
        TraditionalForm]}]], "(", "x", ")"}], "+", 
    RowBox[{
     SubscriptBox["S", 
      RowBox[{
       FormBox["2",
        TraditionalForm], 
       FormBox["2",
        TraditionalForm]}]], "(", "x", ")"}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", 
       RowBox[{
        FormBox["1",
         TraditionalForm], 
        FormBox["2",
         TraditionalForm]}]], "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     TemplateBox[{"3"},
      "Zeta"], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "24"], " ", 
     RowBox[{
      SuperscriptBox["log", "4"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "+", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{"log", "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], "-", 
    RowBox[{
     FractionBox["1", "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", "x", ")"}], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1474]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Log", "[", 
   RowBox[{"a", "+", 
    RowBox[{"b", "/", "c"}]}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1475]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    RowBox[{"a", "+", 
     FractionBox["b", "c"]}], ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    FractionBox[
     RowBox[{
      RowBox[{"a", " ", "c"}], "+", "b"}], "c"], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1475]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"Log", "[", 
   RowBox[{"1", "/", "x"}], "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1476]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"log", "(", 
    FractionBox["1", "x"], ")"}], "\[LongEqual]", 
   RowBox[{"-", 
    RowBox[{"log", "(", "x", ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1476]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcTanh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1477]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["tanh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"log", "(", 
     RowBox[{"-", 
      FractionBox[
       RowBox[{"x", "+", "1"}], 
       RowBox[{"1", "-", "x"}]]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1477]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcSinh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1478]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["sinh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    RowBox[{
     SqrtBox[
      RowBox[{
       SuperscriptBox["x", "2"], "+", "1"}]], "+", "x"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1478]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"sip", "[", 
  RowBox[{"ArcCosh", "[", "x", "]"}], "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1479]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox["cosh", 
     RowBox[{"-", "1"}]], "(", "x", ")"}], "\[LongEqual]", 
   RowBox[{"log", "(", 
    RowBox[{
     SqrtBox[
      RowBox[{
       SuperscriptBox["x", "2"], "-", "1"}]], "+", "x"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SimplifyPolyLog",
 CellLabel->"Out[1479]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "sip", "]"}]], "Input",
 CellTags->"SimplifyPolyLog",
 CellLabel->"In[1480]:="]
}, Open  ]],

Cell["Simplify2  ***unfinished***", "ObjectName",
 CellTags->"Simplify2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Simplify2"],

Cell["Simplify2 is a special ordering function.", "Text",
 CellTags->"Simplify2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Simplify2"],

Cell[BoxData[""], "Input",
 CellTags->"Simplify2",
 CellLabel->"In[1481]:="]
}, Open  ]],

Cell["SmallDelta", "ObjectName",
 CellTags->"SmallDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallDelta"],

Cell["SmallDelta denotes some small positive number.", "Text",
 CellTags->"SmallDelta"]
}, Open  ]],

Cell["SmallEpsilon", "ObjectName",
 CellTags->"SmallEpsilon"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallEpsilon"],

Cell["SmallEpsilon denotes some small positive number.", "Text",
 CellTags->"SmallEpsilon"]
}, Open  ]],

Cell["SmallVariable", "ObjectName",
 CellTags->"SmallVariable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallVariable"],

Cell["\<\
SmallVariable[me] is the head of small (negligible) variables. This means any \
mass with this head can be neglected if it appears in a sum, but not as an \
argument of Passarino-Veltman (PaVe) functions or PropagatorDenominator.\
\>", "Text",
 CellTags->"SmallVariable"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PropagatorDenominator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PropagatorDenominator", All, CellTags],
  ButtonData->"PropagatorDenominator",
  ButtonNote->"PropagatorDenominator"],
 "."
}], "Text",
 CellTags->"SmallVariable"]
}, Open  ]],

Cell["SmallVariables", "ObjectName",
 CellTags->"SmallVariables"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SmallVariables"],

Cell["\<\
SmallVariables is an option for OneLoop. \"SmallVariables\[Rule]{Melectron}\" \
i.e. will substitute \"SmallVariable[Melectron]\"  for all Melectron's in the \
calculation.\
\>", "Text",
 CellTags->"SmallVariables"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"SmallVariables"]
}, Open  ]],

Cell["SMP", "ObjectName",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SMP"],

Cell["\<\
SMP[par] substitutes a symbol for the  Standard Model parameter par.  SMP[] \
gives the list of substitutions. par should be a string; e.g., SMP[\"SW\"] \
gives  sw (in the Global` context).\
\>", "Text",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SMP", "[", "]"}], "//", "InputForm"}]], "Input",
 CellTags->"SMP",
 CellLabel->"In[1482]:="],

Cell["\<\
{\"EL\" :> e, \"CW\" :> cw, \"ME\" :> me, \"MH\" :> mh, \"MW\" :> mw, \"SW\" \
:> sw}\
\>", "Output",
 CellTags->"SMP",
 CellLabel->"Out[1482]//InputForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["SMVertex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SMVertex", All, CellTags],
  ButtonData->"SMVertex",
  ButtonNote->"SMVertex"],
 "."
}], "Text",
 CellTags->"SMP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SMP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMP", "[", "\"\<ME\>\"", "]"}]], "Input",
 CellTags->"SMP",
 CellLabel->"In[1483]:="],

Cell[BoxData[
 FormBox["me", TraditionalForm]], "Output",
 CellTags->"SMP",
 CellLabel->"Out[1483]="]
}, Open  ]]
}, Open  ]],

Cell["SMVertex", "ObjectName",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SMVertex"],

Cell["\<\
SMVertex[\"AWW\", p,mu, q,nu, k,rho] gives the photon-W-W vertex (p,mu \
correspond to the photon, q,nu to the (incoming) W+ and k,rho to the \
(incoming) W-. All momenta are flowing into the vertex.  SMVertex[\"HHH\", \
___] give the three-higgs coupling. \
\>", "Text",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMVertex", "//", "Options"}]], "Input",
 CellTags->"SMVertex",
 CellLabel->"In[1484]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "4"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SMVertex",
 CellLabel->"Out[1484]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SMVertex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SMVertex", "[", 
  RowBox[{
  "\"\<AWW\>\"", ",", "p", ",", "\[Mu]", ",", " ", "q", ",", "\[Nu]", ",", 
   " ", "k", ",", "\[Rho]"}], "]"}]], "Input",
 CellTags->"SMVertex",
 CellLabel->"In[1485]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "\[ImaginaryI]"}], " ", "e", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         FormBox[
          RowBox[{"k", "-", "p"}],
          TraditionalForm], ")"}], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"k", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        FormBox[
         RowBox[{"p", "-", "q"}],
         TraditionalForm], ")"}], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SMVertex",
 CellLabel->"Out[1485]="]
}, Open  ]]
}, Open  ]],

Cell["Smu  ***unfinished***", "ObjectName",
 CellTags->"Smu"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Smu"],

Cell["Smu  is ...", "Text",
 CellTags->"Smu"]
}, Open  ]],

Cell["Sn", "ObjectName",
 CellTags->"Sn"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Sn"],

Cell[TextData[{
 "Sn is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["\[Pi]", 
    RowBox[{"n", "/", "2"}]], TraditionalForm]]],
 "/",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"2", " ", "\[Pi]"}], ")"}], "n"], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Sn"]
}, Open  ]],

Cell["SO", "ObjectName",
 CellTags->"SO"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SO"],

Cell["\<\
SO[q] is a four-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q], Momentum[OPEDelta] by FCI.\
\>", "Text",
 CellTags->"SO"],

Cell[TextData[{
 "See also: ",
 ButtonBox["FCI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FCI", All, CellTags],
  ButtonData->"FCI",
  ButtonNote->"FCI"],
 ", ",
 ButtonBox["OPEDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPEDelta", All, CellTags],
  ButtonData->"OPEDelta",
  ButtonNote->"OPEDelta"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProduct", All, CellTags],
  ButtonData->"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SOD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SOD", All, CellTags],
  ButtonData->"SOD",
  ButtonNote->"SOD"],
 "."
}], "Text",
 CellTags->"SO"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SO"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SO", "[", "p", "]"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1486]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SO",
 CellLabel->"Out[1486]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SO", "[", 
  RowBox[{"p", "-", "q"}], "]"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1487]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SO",
 CellLabel->"Out[1487]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SO", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SO",
 CellLabel->"In[1488]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "OPEDelta", "]"}], ",", 
   RowBox[{"Momentum", "[", "p", "]"}]}], "]"}]], "Output",
 CellTags->"SO",
 CellLabel->"Out[1488]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SOD", "ObjectName",
 CellTags->"SOD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SOD"],

Cell["\<\
SOD[q] is a D-dimensional scalar product of OPEDelta with q. It is \
transformed into Pair[Momentum[q,D], Momentum[OPEDelta,D] by FeynCalcInternal.\
\>", "Text",
 CellTags->"SOD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPEDelta", All, CellTags],
  ButtonData->"OPEDelta",
  ButtonNote->"OPEDelta"],
 ", ",
 ButtonBox["Pair",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Pair", All, CellTags],
  ButtonData->"Pair",
  ButtonNote->"Pair"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProduct", All, CellTags],
  ButtonData->"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 ", ",
 ButtonBox["SOD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SOD", All, CellTags],
  ButtonData->"SOD",
  ButtonNote->"SOD"],
 "."
}], "Text",
 CellTags->"SOD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SOD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SOD", "[", "p", "]"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1489]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SOD",
 CellLabel->"Out[1489]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SOD", "[", 
  RowBox[{"p", "-", "q"}], "]"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1490]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[CapitalDelta]\"\>",
    TraditionalForm], 
   FormBox["\<\"\[CenterDot]\"\>",
    TraditionalForm], 
   FormBox["\<\"(\"\>",
    TraditionalForm], 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], 
   FormBox["\<\")\"\>",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SOD",
 CellLabel->"Out[1490]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SOD", "[", "p", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SOD",
 CellLabel->"In[1491]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"OPEDelta", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"p", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"SOD",
 CellLabel->"Out[1491]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["Solve2", "ObjectName",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Solve2"],

Cell["\<\
Solve2 is equivalent to Solve, except that it works only for linear equations \
(and returns just a list) and accepts the options Factoring and \
FinalSubstitutions. Solve2 uses the \"high school algorithm\" and factors \
intermediate results. Therefore it can be drastically more useful than Solve.\
\
\>", "Text",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Solve2", "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1492]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1492]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1493]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["1", "4"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1493]="]
}, Open  ]],

Cell["\<\
If no equation sign is given the polynomials are supposed to be 0.\
\>", "Text",
 CellTags->"Solve2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"x", "+", "y"}], ",", "x"}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1494]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", 
    RowBox[{"-", "y"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1494]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"x", "+", "y"}], ",", "x", ",", 
   RowBox[{"FinalSubstitutions", "\[Rule]", 
    RowBox[{"{", 
     RowBox[{"y", "\[Rule]", "h"}], "}"}]}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1495]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"x", "\[Rule]", 
    RowBox[{"-", "h"}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1495]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve2", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}], ",", 
   RowBox[{"Factoring", "\[Rule]", "Expand"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1496]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["b", "2"], "-", 
      FractionBox["w", "4"]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1496]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve2",
 CellLabel->"In[1497]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"x", "\[Rule]", 
      RowBox[{
       FractionBox["1", "4"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
     RowBox[{"y", "\[Rule]", 
      RowBox[{"d", "+", "p"}]}]}], "}"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve2",
 CellLabel->"Out[1497]="]
}, Open  ]]
}, Open  ]],

Cell["Solve3", "ObjectName",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Solve3"],

Cell["\<\
Solve3 is equivalent to Solve, except that it works only for linear equations \
(and returns just a list) and uses the \"high school algorithm\" and is \
sometimes better than Solve for systems involving rational polyonomials.\
\>", "Text",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Solve3", "]"}]], "Input",
 CellTags->"Solve3",
 CellLabel->"In[1498]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"ParallelMap", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Solve3",
 CellLabel->"Out[1498]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Solve2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Solve2", All, CellTags],
  ButtonData->"Solve2",
  ButtonNote->"Solve2"],
 "."
}], "Text",
 CellTags->"SMP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Solve3"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Solve3", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{
      RowBox[{"2", " ", "x"}], "==", 
      RowBox[{"b", "-", 
       RowBox[{"w", "/", "2"}]}]}], ",", 
     RowBox[{
      RowBox[{"y", "-", "d"}], "==", "p"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y"}], "}"}]}], "]"}]], "Input",
 CellTags->"Solve3",
 CellLabel->"In[1499]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"x", "\[Rule]", 
     RowBox[{
      FractionBox["1", "4"], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "b"}], "-", "w"}], ")"}]}]}], ",", 
    RowBox[{"y", "\[Rule]", 
     RowBox[{"d", "+", "p"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Solve3",
 CellLabel->"Out[1499]="]
}, Open  ]]
}, Open  ]],

Cell["SP", "ObjectName",
 CellTags->"SP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SP"],

Cell[TextData[{
 "SP[a, b] denotes a four-dimensional scalar product. SP[a, b] is transformed \
into ScalarProduct[a, b] by FeynCalcInternal. SP[p] is the same as SP[p, p] \
",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"=", 
     SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"SP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandScalarProduct", All, CellTags],
  ButtonData->"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProduct", All, CellTags],
  ButtonData->"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 "."
}], "Text",
 CellTags->"SP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
  RowBox[{"SP", "[", "q", "]"}]}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1500]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], "+", 
   FormBox[
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1500]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", 
   RowBox[{"q", "+", 
    RowBox[{"2", "p"}]}]}], "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1501]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1501]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"SP", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "+", 
     RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1502]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1502]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"SP", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1503]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}]}], "+", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1503]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1504]:="],

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1504]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SP", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1505]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "a", "]"}], ",", 
   RowBox[{"Momentum", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1505]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SP", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SP",
 CellLabel->"In[1506]:="],

Cell[BoxData[
 RowBox[{"SP", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SP",
 CellLabel->"Out[1506]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SPC", "ObjectName",
 CellTags->"SPC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPC"],

Cell["SPC is an abbreviation for ScalarProductCancel.", "Text",
 CellTags->"SPC"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["ScalarProductCancel",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProductCancel", All, CellTags],
  ButtonData->"ScalarProductCancel",
  ButtonNote->"ScalarProductCancel"],
 "."
}], "Text",
 CellTags->"SPC"]
}, Open  ]],

Cell["SPD", "ObjectName",
 CellTags->"SPD"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPD"],

Cell[TextData[{
 "SPD[a, b] denotes a D-dimensional scalar product. SPD[a, b] is transformed \
into ScalarProduct[a, b,Dimension\[Rule]D] by FeynCalcInternal. SPD[p] is the \
same as SPD[p,p] ",
 Cell[BoxData[
  FormBox[
   RowBox[{"(", 
    RowBox[{"=", 
     SuperscriptBox["p", "2"]}]}], TraditionalForm]]],
 ")."
}], "Text",
 CellTags->"SPD"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PD", All, CellTags],
  ButtonData->"PD",
  ButtonNote->"PD"],
 ", ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["ExpandScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExpandScalarProduct", All, CellTags],
  ButtonData->"ExpandScalarProduct",
  ButtonNote->"ExpandScalarProduct"],
 ", ",
 ButtonBox["ScalarProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ScalarProduct", All, CellTags],
  ButtonData->"ScalarProduct",
  ButtonNote->"ScalarProduct"],
 "."
}], "Text",
 CellTags->"SPD"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SPD"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SPD", "[", 
   RowBox[{"p", ",", "q"}], "]"}], " ", "+", " ", 
  RowBox[{"SPD", "[", "q", "]"}]}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1507]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], "+", 
   FormBox[
    SuperscriptBox[
     FormBox["q",
      TraditionalForm], "2"],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1507]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{
   RowBox[{"p", "-", "q"}], ",", 
   RowBox[{"q", "+", 
    RowBox[{"2", "p"}]}]}], "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1508]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", 
   FormBox[
    RowBox[{"p", "-", "q"}],
    TraditionalForm], ")", "\[CenterDot]", "(", 
   FormBox[
    RowBox[{
     RowBox[{"2", " ", "p"}], "+", "q"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1508]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", " ", 
  RowBox[{"SPD", "[", 
   RowBox[{
    RowBox[{"p", "-", "q"}], ",", 
    RowBox[{"q", "+", 
     RowBox[{"2", "p"}]}]}], "]"}], " ", "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1509]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], "+", 
   RowBox[{"2", " ", 
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"]}], "-", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1509]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ExpandScalarProduct", "[", 
  RowBox[{"SPD", "[", 
   RowBox[{"p", "-", "q"}], "]"}], "]"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1510]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"2", " ", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}]}], "+", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1510]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SPD", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1511]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1511]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SPD", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1512]:="],

Cell[BoxData[
 RowBox[{"Pair", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", 
    RowBox[{"a", ",", "D"}], "]"}], ",", 
   RowBox[{"Momentum", "[", 
    RowBox[{"b", ",", "D"}], "]"}]}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1512]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SPD", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1513]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1513]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FCE", "[", 
   RowBox[{"ChangeDimension", "[", 
    RowBox[{
     RowBox[{"SP", "[", 
      RowBox[{"p", ",", "q"}], "]"}], ",", " ", "D"}], "]"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SPD",
 CellLabel->"In[1514]:="],

Cell[BoxData[
 RowBox[{"SPD", "[", 
  RowBox[{"p", ",", "q"}], "]"}]], "Output",
 CellTags->"SPD",
 CellLabel->"Out[1514]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["Spinor", "ObjectName",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Spinor"],

Cell[TextData[{
 "Spinor[p, m, o] is the head of Dirac spinors. Which of the spinors ",
 StyleBox["u",
  FontSlant->"Italic"],
 ", ",
 StyleBox["v",
  FontSlant->"Italic"],
 ", ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{",", " "}], TraditionalForm]]],
 "or ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", 
    RowBox[{"_", " "}]], TraditionalForm]]],
 "is understood, depends on the sign of the momentum (p) argument and the \
relative position of DiracSlash[p]: Spinor[sign p, mass] is that spinor which \
yields: sign*mass*Spinor[p, mass] if the Dirac equation is applied (by \
DiracEquation or DiracSimplify).\nThe optional argument o can be used for \
additional degrees of freedom. If no optional argument o is supplied, a 1 is \
subsituted in.\nSpinors of fermions of mass ",
 StyleBox["m",
  FontSlant->"Italic"],
 " are normalized to have square ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 StyleBox["u",
  FontSlant->"Italic"],
 "=2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 " and  ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", 
    RowBox[{"_", " "}]], TraditionalForm]]],
 StyleBox["v=",
  FontSlant->"Italic"],
 "-2 ",
 StyleBox["m",
  FontSlant->"Italic"],
 "."
}], "Text",
 CellTags->"Spinor"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FermionSpinSum", All, CellTags],
  ButtonData->"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorU", All, CellTags],
  ButtonData->"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorV", All, CellTags],
  ButtonData->"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorUBar", All, CellTags],
  ButtonData->"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorVBar", All, CellTags],
  ButtonData->"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"Spinor"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Spinor", "[", "p", "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1515]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyPhi]", "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1515]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1516]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[CurlyPhi]", "(", 
   FormBox["p",
    TraditionalForm], ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1516]="]
}, Open  ]],

Cell["\<\
FeynCalc uses covariant normalization (as opposed to e.g. the normalization \
used in Bjorken&Drell).\
\>", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{"p", ",", "m"}], "]"}], ".", 
   RowBox[{"Spinor", "[", 
    RowBox[{"p", ",", "m"}], "]"}]}], "//", "DiracSimplify"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1517]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "m"}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1517]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DiracSimplify", "[", 
  RowBox[{
   RowBox[{"Spinor", "[", 
    RowBox[{
     RowBox[{"-", "p"}], ",", "m"}], "]"}], ".", 
   RowBox[{"DiracSlash", "[", "p", "]"}]}], "]"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1518]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "m"}], " ", 
   RowBox[{"\[CurlyPhi]", "(", 
    FormBox[
     RowBox[{"-", "p"}],
     TraditionalForm], ",", "m", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Spinor",
 CellLabel->"Out[1518]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", "p", "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1519]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1519]//StandardForm="]
}, Open  ]],

Cell["\<\
By convention, ChangeDimension does not operate on momenta in Spinor's (but \
on e.g.  DiracSlash[Momentum[p]]). \
\>", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"ChangeDimension", "[", 
   RowBox[{
    RowBox[{"Spinor", "[", "p", "]"}], ",", "D"}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1520]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1520]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1521]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1521]//StandardForm="]
}, Open  ]],

Cell["SmallVariable's are discarded by Spinor.", "Text",
 CellTags->"Spinor"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Spinor", "[", 
   RowBox[{"p", ",", 
    RowBox[{"SmallVariable", "[", "m", "]"}]}], "]"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Spinor",
 CellLabel->"In[1522]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "0", ",", "1"}], "]"}]], "Output",\

 CellTags->"Spinor",
 CellLabel->"Out[1522]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorCollect", "ObjectName",
 CellTags->"SpinorCollect"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorCollect"],

Cell["\<\
SpinorCollect is an option for FermionSpinSum. If set to False the  argument \
of FermionSpinSum has to be already collected w.r.t. Spinor.\
\>", "Text",
 CellTags->"SpinorCollect"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FermionSpinSum", All, CellTags],
  ButtonData->"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Spinor", All, CellTags],
  ButtonData->"Spinor",
  ButtonNote->"Spinor"],
 "."
}], "Text",
 CellTags->"SpinorCollect"]
}, Open  ]],

Cell["SpinorUBar", "ObjectName",
 CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorUBar"],

Cell[TextData[{
 "SpinorUBar[p, m] denotes a ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["u", "_"], TraditionalForm]]],
 "-spinor."
}], "Text",
 CellTags->"SpinorUBar"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Spinor", All, CellTags],
  ButtonData->"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorU", All, CellTags],
  ButtonData->"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorV", All, CellTags],
  ButtonData->"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorVBar", All, CellTags],
  ButtonData->"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorUBar"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorUBar"],

Cell["One argument only assumes a massless spinor.", "Text",
 CellTags->"SpinorUBar"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", "p", "]"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1523]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"u\"\>", "_"],
    TraditionalForm], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorUBar",
 CellLabel->"Out[1523]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1524]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"u\"\>", "_"],
    TraditionalForm], "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorUBar",
 CellLabel->"Out[1524]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorUBar", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1525]:="],

Cell[BoxData[
 RowBox[{"SpinorUBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorUBar",
 CellLabel->"Out[1525]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorUBar", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1526]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorUBar",
 CellLabel->"Out[1526]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorUBar", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorUBar",
 CellLabel->"In[1527]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorUBar",
 CellLabel->"Out[1527]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorU", "ObjectName",
 CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorU"],

Cell["SpinorU[p, m, optarg] denotes a u-spinor.", "Text",
 CellTags->"SpinorU"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Spinor", All, CellTags],
  ButtonData->"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorUBar", All, CellTags],
  ButtonData->"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorV", All, CellTags],
  ButtonData->"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorVBar", All, CellTags],
  ButtonData->"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorU"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorU"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorU", "[", "p", "]"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1528]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"u", "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorU",
 CellLabel->"Out[1528]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorU", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1529]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"u", "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorU",
 CellLabel->"Out[1529]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorU", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1530]:="],

Cell[BoxData[
 RowBox[{"SpinorU", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorU",
 CellLabel->"Out[1530]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorU", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1531]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorU",
 CellLabel->"Out[1531]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorU", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorU",
 CellLabel->"In[1532]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"Momentum", "[", "p", "]"}], ",", "m", ",", "1"}], "]"}]], "Output",\

 CellTags->"SpinorU",
 CellLabel->"Out[1532]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorVBar", "ObjectName",
 CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorVBar"],

Cell[TextData[{
 "SpinorVBar[p, m] denotes a ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["v", "_"], TraditionalForm]]],
 "(p,m)-spinor."
}], "Text",
 CellTags->"SpinorVBar"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Spinor", All, CellTags],
  ButtonData->"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorU", All, CellTags],
  ButtonData->"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorV",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorV", All, CellTags],
  ButtonData->"SpinorV",
  ButtonNote->"SpinorV"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorUBar", All, CellTags],
  ButtonData->"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 "."
}], "Text",
 CellTags->"SpinorVBar"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorVBar"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", "p", "]"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1533]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"v\"\>", "_"],
    TraditionalForm], "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1533]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1534]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    OverscriptBox["\<\"v\"\>", "_"],
    TraditionalForm], "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1534]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorVBar", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1535]:="],

Cell[BoxData[
 RowBox[{"SpinorVBar", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1535]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorVBar", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1536]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1536]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorVBar", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorVBar",
 CellLabel->"In[1537]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorVBar",
 CellLabel->"Out[1537]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinorV", "ObjectName",
 CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinorV"],

Cell["SpinorV[p, m, optarg] denotes a v-spinor.", "Text",
 CellTags->"SpinorV"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Spinor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Spinor", All, CellTags],
  ButtonData->"Spinor",
  ButtonNote->"Spinor"],
 ", ",
 ButtonBox["SpinorUBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorUBar", All, CellTags],
  ButtonData->"SpinorUBar",
  ButtonNote->"SpinorUBar"],
 ", ",
 ButtonBox["SpinorU",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorU", All, CellTags],
  ButtonData->"SpinorU",
  ButtonNote->"SpinorU"],
 ", ",
 ButtonBox["SpinorVBar",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SpinorVBar", All, CellTags],
  ButtonData->"SpinorVBar",
  ButtonNote->"SpinorVBar"],
 "."
}], "Text",
 CellTags->"SpinorV"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SpinorV"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorV", "[", "p", "]"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1538]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"v", "(", "p", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1538]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SpinorV", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1539]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"v", "(", "p", ",", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1539]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SpinorV", "[", 
   RowBox[{"p", ",", "m"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1540]:="],

Cell[BoxData[
 RowBox[{"SpinorV", "[", 
  RowBox[{"p", ",", "m"}], "]"}]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1540]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SpinorV", "[", 
    RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1541]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1541]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SpinorV", "[", 
     RowBox[{"p", ",", "m"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SpinorV",
 CellLabel->"In[1542]:="],

Cell[BoxData[
 RowBox[{"Spinor", "[", 
  RowBox[{
   RowBox[{"-", 
    RowBox[{"Momentum", "[", "p", "]"}]}], ",", "m", ",", "1"}], 
  "]"}]], "Output",
 CellTags->"SpinorV",
 CellLabel->"Out[1542]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SpinPolarizationSum", "ObjectName",
 CellTags->"SpinPolarizationSum"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SpinPolarizationSum"],

Cell["\<\
SpinPolarizationSum is an option for FermionSpinSum. The set (pure) function \
acts on the usual spin sum.\
\>", "Text",
 CellTags->"SpinPolarizationSum"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FermionSpinSum", All, CellTags],
  ButtonData->"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 "."
}], "Text",
 CellTags->"SpinPolarizationSum"]
}, Open  ]],

Cell["SPL", "ObjectName",
 CellTags->"SPL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SPL"],

Cell["SPL is an abbreviation for SimplifyPolyLog.", "Text",
 CellTags->"SPL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SimplifyPolyLog", All, CellTags],
  ButtonData->"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"SPL"]
}, Open  ]],

Cell["SplittingFunction", "ObjectName",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SplittingFunction"],

Cell[TextData[{
 "SplittingFunction[pxy] is a database of splitting functions in the ",
 Cell[BoxData[
  FormBox[
   OverscriptBox["MS", "_"], TraditionalForm]]],
 " scheme."
}], "Text",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SplittingFunction", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1543]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Polarization", "\[Rule]", "1"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1543]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["AnomalousDimension",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "AnomalousDimension", All, CellTags],
  ButtonData->"AnomalousDimension",
  ButtonNote->"AnomalousDimension"],
 "."
}], "Text",
 CellTags->"SplittingFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SplittingFunction"],

Cell["Unpolarized case:", "Text",
 CellTags->"SplittingFunction"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SplittingFunction", ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}], ";"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1544]:="],

Cell["\<\
In general the argument should be a string, but if the variables Pqq, etc. \
have no value, you can omit the \"\".\
\>", "Text",
 CellTags->"SplittingFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1545]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"4", " ", "x"}], "+", 
     RowBox[{"8", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1545]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1546]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", 
      SuperscriptBox["x", "2"]}], "-", 
     RowBox[{"16", " ", "x"}], "+", "8"}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1546]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1547]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"4", " ", "x"}], "+", 
     FractionBox["8", "x"], "-", "8"}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1547]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1548]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"8", " ", 
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["x", "2"]}], "+", 
      RowBox[{
       FractionBox["11", "12"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", "x", "+", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"], "+", 
      FractionBox["1", "x"], "-", "2"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["8", "3"], " ", 
    SubscriptBox["N", "f"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1548]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1549]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"7", "-", 
        RowBox[{"4", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"2", " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
     RowBox[{"4", " ", 
      RowBox[{"(", 
       RowBox[{
        FractionBox[
         RowBox[{"log", "(", "x", ")"}], 
         RowBox[{"1", "-", "x"}]], "+", 
        SubscriptBox[
         RowBox[{"(", 
          FractionBox[
           RowBox[{"log", "(", 
            RowBox[{"1", "-", "x"}], ")"}], 
           RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1549]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1550]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"2", " ", "x"}]}], "-", 
     FractionBox["4", "x"], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"2", " ", "x"}]}], "-", 
        FractionBox["4", "x"], "+", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1550]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1551]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"8", " ", 
          SuperscriptBox["x", "2"]}]}], "+", 
        RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1551]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1552]:="],

Cell["Polarized case:", "Text",
 CellTags->"SplittingFunction"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SplittingFunction", ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "]"}], ";"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1553]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1554]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"6", " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "-", 
     RowBox[{"4", " ", "x"}], "+", 
     RowBox[{"8", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1554]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1555]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", "x"}], "-", "8"}], ")"}], " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1555]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1556]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"8", "-", 
     RowBox[{"4", " ", "x"}]}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1556]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "Pgg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1557]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["22", "3"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "-", 
      RowBox[{"16", " ", "x"}], "+", 
      RowBox[{"8", " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", "8"}], ")"}]}], "-", 
   RowBox[{
    FractionBox["8", "3"], " ", 
    SubscriptBox["N", "f"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1557]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1558]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"7", "-", 
        RowBox[{"4", " ", 
         RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
      RowBox[{"\[Delta]", "(", 
       FormBox[
        RowBox[{"1", "-", "x"}],
        TraditionalForm], ")"}]}], "+", 
     RowBox[{"8", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], "+", 
     RowBox[{"2", " ", "x"}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "+", "2"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
     RowBox[{"4", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox["1", 
         RowBox[{"1", "-", "x"}]], ")"}], "+"], " ", 
      RowBox[{"log", "(", "x", ")"}]}], "-", 
     RowBox[{"4", " ", 
      SubscriptBox[
       RowBox[{"(", 
        FractionBox[
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], 
         RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", "4"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1558]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agq", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1559]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{"4", " ", "x"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "+", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1559]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agqd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1560]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"2", " ", "x"}], "-", "4"}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "2"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1560]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1561]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", 
        RowBox[{"8", " ", "x"}]}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1561]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aqgd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1562]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"4", "-", 
        RowBox[{"8", " ", "x"}]}], ")"}], " ", 
      RowBox[{"log", "(", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", "4"}], 
    ")"}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1562]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "agg", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1563]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["67", "9"], "-", 
         RowBox[{"4", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
         SubscriptBox[
          RowBox[{"(", 
           FractionBox[
            RowBox[{"log", "(", 
             RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "+", "2"}], 
     ")"}]}], "-", 
   RowBox[{
    FractionBox["20", "9"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1563]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "aggd", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1564]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["67", "9"], "-", 
         RowBox[{"4", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"log", "(", "x", ")"}], 
          RowBox[{"1", "-", "x"}]], "+", 
         SubscriptBox[
          RowBox[{"(", 
           FractionBox[
            RowBox[{"log", "(", 
             RowBox[{"1", "-", "x"}], ")"}], 
            RowBox[{"1", "-", "x"}]], ")"}], "+"]}], ")"}]}], "+", "2"}], 
     ")"}]}], "-", 
   RowBox[{
    FractionBox["20", "9"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1564]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQQS", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1565]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "F"], " ", 
   SubscriptBox["T", "f"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}]}], "-", 
     RowBox[{"16", " ", 
      RowBox[{"(", 
       RowBox[{"x", "+", "1"}], ")"}], " ", 
      RowBox[{
       SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{
        RowBox[{"48", " ", "x"}], "-", "16"}], ")"}], " ", 
      RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"SplittingFunction",
 CellLabel->"Out[1565]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQQNS", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1566]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", "8"}], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], "-", 
      FractionBox[
       SubscriptBox["C", "A"], "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           RowBox[{"2", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "-", 
          RowBox[{"4", " ", 
           TemplateBox[{"2",RowBox[{"-", "x"}]},
            "PolyLog"]}], "+", 
          RowBox[{
           SuperscriptBox["log", "2"], "(", "x", ")"}], "-", 
          RowBox[{"4", " ", 
           RowBox[{"log", "(", 
            RowBox[{"x", "+", "1"}], ")"}], " ", 
           RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], 
       RowBox[{"x", "+", "1"}]], "+", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{"2", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["C", "F"], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox[
       RowBox[{"4", " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
        RowBox[{
         SuperscriptBox["log", "2"], "(", "x", ")"}]}], 
       RowBox[{"1", "-", "x"}]], "+", 
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["536", "9"], "-", 
         RowBox[{"16", " ", 
          RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], ")"}], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"88", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"], "-", 
         RowBox[{"24", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", 
         FractionBox["17", "3"]}], ")"}]}], "+", 
      RowBox[{
       FractionBox["4", "9"], " ", 
       RowBox[{"(", 
        RowBox[{"53", "-", 
         RowBox[{"187", " ", "x"}]}], ")"}]}], "-", 
      RowBox[{
       FractionBox["4", "3"], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"5", " ", "x"}], "-", 
         FractionBox["22", 
          RowBox[{"1", "-", "x"}]], "+", "5"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["N", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"8", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"3", " ", 
         RowBox[{"(", 
          RowBox[{"1", "-", "x"}], ")"}]}]]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"16", " ", 
            RowBox[{"\[Zeta]", "(", "2", ")"}]}], "3"]}], "-", 
         FractionBox["2", "3"]}], ")"}], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      FractionBox[
       RowBox[{"88", " ", "x"}], "9"], "-", 
      RowBox[{
       FractionBox["80", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "-", 
      FractionBox["8", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"16", " ", 
         RowBox[{"(", 
          RowBox[{
           SuperscriptBox["x", "2"], "+", "1"}], ")"}], " ", 
         RowBox[{"log", "(", 
          RowBox[{"1", "-", "x"}], ")"}], " ", 
         RowBox[{"log", "(", "x", ")"}]}], 
        RowBox[{"1", "-", "x"}]]}], "+", 
      RowBox[{
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"24", " ", 
           RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
         RowBox[{"48", " ", 
          TemplateBox[{"3"},
           "Zeta"]}], "+", "3"}], ")"}]}], "-", 
      RowBox[{"40", " ", 
       RowBox[{"(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{"4", " ", 
       RowBox[{"(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "-", 
      RowBox[{"8", " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"2", " ", "x"}], "+", 
         FractionBox["3", 
          RowBox[{"1", "-", "x"}]]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1566]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PQG", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1567]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"8", " ", 
        RowBox[{"\[Zeta]", "(", "2", ")"}]}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"16", " ", "x"}]}], "-", "8"}], ")"}], " ", 
       TemplateBox[{"2",RowBox[{"-", "x"}]},
        "PolyLog"]}], "-", 
      RowBox[{"44", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"4", "-", 
         RowBox[{"8", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"8", " ", "x"}]}], "-", "4"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"32", " ", "x"}], "+", "4"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"16", " ", "x"}]}], "-", "8"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", "48"}], ")"}]}], "+", 
   RowBox[{"4", " ", 
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"8", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}]}], "-", 
      RowBox[{"16", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
      RowBox[{"54", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "4"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"4", " ", "x"}], "-", "2"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"16", "-", 
         RowBox[{"16", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"8", "-", 
         RowBox[{"16", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{"18", " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "44"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1567]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PGQ", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1568]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "A"], " ", 
      SubscriptBox["C", "F"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
       TemplateBox[{"2",RowBox[{"-", "x"}]},
        "PolyLog"]}], "+", 
      RowBox[{"16", " ", 
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", "x"}], "+", 
      FractionBox[
       RowBox[{"280", " ", "x"}], "9"], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"16", "-", 
         RowBox[{"8", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "+", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"8", " ", "x"}], "3"], "+", 
         FractionBox["80", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "32"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"32", "-", 
         RowBox[{"104", " ", "x"}]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "+", "32"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}]}], "+", 
      FractionBox["328", "9"]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SubscriptBox["C", "F"], " ", 
      SubscriptBox["T", "f"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       FractionBox[
        RowBox[{"32", " ", "x"}], "9"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{"32", " ", "x"}], "3"], "-", 
         FractionBox["64", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      FractionBox["128", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "F", "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"32", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"8", " ", "x"}], "-", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{"8", "-", 
         RowBox[{"4", " ", "x"}]}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"8", " ", "x"}]}], "-", "16"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}]}], "-", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"32", " ", "x"}], "+", "64"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"36", " ", "x"}], "+", "48"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "68"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1568]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SplittingFunction", "[", "PGG", "]"}]], "Input",
 CellTags->"SplittingFunction",
 CellLabel->"In[1569]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["C", "A"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        FractionBox["32", "3"], " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "x"}],
          TraditionalForm], ")"}]}]}], "+", 
      FractionBox[
       RowBox[{"608", " ", "x"}], "9"], "-", 
      RowBox[{
       FractionBox["160", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox[
           RowBox[{"32", " ", "x"}], "3"]}], "-", 
         FractionBox["32", "3"]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      FractionBox["448", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubsuperscriptBox["C", "A", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "+", 
         FractionBox["32", 
          RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
       TemplateBox[{"2",RowBox[{"-", "x"}]},
        "PolyLog"]}], "+", 
      RowBox[{
       FractionBox["64", "3"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "+", 
      RowBox[{
       RowBox[{"\[Zeta]", "(", "2", ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "-", 
         RowBox[{"16", " ", 
          SubscriptBox[
           RowBox[{"(", 
            FractionBox["1", 
             RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
         FractionBox["16", 
          RowBox[{"x", "+", "1"}]]}], ")"}]}], "+", 
      RowBox[{"24", " ", 
       TemplateBox[{"3"},
        "Zeta"], " ", 
       RowBox[{"\[Delta]", "(", 
        FormBox[
         RowBox[{"1", "-", "x"}],
         TraditionalForm], ")"}]}], "-", 
      FractionBox[
       RowBox[{"388", " ", "x"}], "9"], "+", 
      RowBox[{
       FractionBox["536", "9"], " ", 
       SubscriptBox[
        RowBox[{"(", 
         FractionBox["1", 
          RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          FractionBox["8", 
           RowBox[{"x", "+", "1"}]]}], "+", 
         FractionBox["8", 
          RowBox[{"1", "-", "x"}]], "+", "32"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["232", "3"], "-", 
         FractionBox[
          RowBox[{"536", " ", "x"}], "3"]}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "-", 
         FractionBox["32", 
          RowBox[{"1", "-", "x"}]], "-", "32"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"64", " ", "x"}], "+", 
         FractionBox["32", 
          RowBox[{"x", "+", "1"}]], "+", "32"}], ")"}], " ", 
       RowBox[{"log", "(", 
        RowBox[{"x", "+", "1"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", 
      FractionBox["148", "9"]}], ")"}]}], "+", 
   RowBox[{
    SubscriptBox["C", "F"], " ", 
    SubscriptBox["T", "f"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{"8", " ", 
        RowBox[{"\[Delta]", "(", 
         FormBox[
          RowBox[{"1", "-", "x"}],
          TraditionalForm], ")"}]}]}], "+", 
      RowBox[{"80", " ", "x"}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"-", 
          RowBox[{"16", " ", "x"}]}], "-", "16"}], ")"}], " ", 
       RowBox[{
        SuperscriptBox["log", "2"], "(", "x", ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         RowBox[{"16", " ", "x"}], "-", "80"}], ")"}], " ", 
       RowBox[{"log", "(", "x", ")"}]}], "-", "80"}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"SplittingFunction",
 CellLabel->"Out[1569]="]
}, Open  ]]
}, Open  ]],

Cell["StandardMatrixElement", "ObjectName",
 CellTags->"StandardMatrixElement"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"StandardMatrixElement"],

Cell["\<\
StandardMatrixElement[ ... ] is the head for matrix element abbreviations.\
\>", "Text",
 CellTags->"StandardMatrixElement"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"StandardMatrixElement"]
}, Open  ]],

Cell["SubContext", "ObjectName",
 CellTags->"SubContext"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SubContext"],

Cell["\<\
SubContext[fun] gives the sub-directory (context) in HighEnergyPhysics.\
\>", "Text",
 CellTags->"SubContext"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc", All, CellTags],
  ButtonData->"FeynCalc",
  ButtonNote->"FeynCalc"],
 ", ",
 ButtonBox["Load",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Load", All, CellTags],
  ButtonData->"Load",
  ButtonNote->"Load"],
 "."
}], "Text",
 CellTags->"SubContext"]
}, Open  ]],

Cell["SubLoop", "ObjectName",
 CellTags->"SubLoop"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SubLoop"],

Cell["\<\
SubLoop is an option for OPE1Loop. If set to True, sub 1-loop tensorintegral \
decomposition is performed.\
\>", "Text",
 CellTags->"SubLoop"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPE1Loop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPE1Loop", All, CellTags],
  ButtonData->"OPE1Loop",
  ButtonNote->"OPE1Loop"],
 "."
}], "Text",
 CellTags->"SubLoop"]
}, Open  ]],

Cell["SumP", "ObjectName",
 CellTags->"SumP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumP"],

Cell[TextData[{
 "SumP[k, m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["2", 
     RowBox[{
      StyleBox["k",
       FontSlant->"Plain"], "-", "1"}]], 
    RowBox[{
     SubsuperscriptBox["\[Sum]", 
      RowBox[{
       StyleBox["i",
        FontSlant->"Plain"], "=", "1"}], 
      RowBox[{"2", 
       StyleBox["m",
        FontSlant->"Plain"]}]], 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "+", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], 
         StyleBox["i",
          FontSlant->"Plain"]]}], ")"}], "/", 
      SuperscriptBox[
       StyleBox["i",
        FontSlant->"Plain"], 
       StyleBox["k",
        FontSlant->"Plain"]]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SumP"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumS", All, CellTags],
  ButtonData->"SumS",
  ButtonNote->"SumS"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumT", All, CellTags],
  ButtonData->"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"SumP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1570]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1570]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1571]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "2", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1571]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", "m"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1572]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["S", "1", "'"], "\[NoBreak]", "(", "\[NoBreak]", 
   FormBox["m",
    TraditionalForm], "\[NoBreak]", ")"}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1572]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumP", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1573]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1573]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "8"], 
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "+", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "i"}]}], ")"}], "/", "i"}]}]], "Input",\

 CellTags->"SumP",
 CellLabel->"In[1574]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1574]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Explicit", "[", 
  RowBox[{"SumP", "[", 
   RowBox[{"1", ",", 
    RowBox[{"n", "/", "2"}]}], "]"}], "]"}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1575]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"]}], ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      FractionBox[
       RowBox[{"n", "-", "1"}], "2"],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "n"], "+", "1"}], ")"}], " ", 
    RowBox[{
     SubscriptBox["S", "1"], "(", 
     FormBox[
      FractionBox["n", "2"],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1575]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "/.", 
  RowBox[{"n", "\[Rule]", "8"}]}]], "Input",
 CellTags->"SumP",
 CellLabel->"In[1576]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumP",
 CellLabel->"Out[1576]="]
}, Open  ]]
}, Open  ]],

Cell["SumS", "ObjectName",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumS"],

Cell[TextData[{
 "SumS[1, m] is the harmonic number ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{
      StyleBox["i",
       FontSlant->"Italic"], "=", "1"}], 
     StyleBox["m",
      FontSlant->"Italic"]], 
    StyleBox[" ",
     FontSlant->"Italic"], 
    RowBox[{
     StyleBox["i",
      FontSlant->"Italic"], "^", 
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"=", 
      RowBox[{
       SubscriptBox["S", "1"], "(", "m", ")"}]}], " ", ")"}], "."}], 
   TraditionalForm]]],
 " SumS[1,1,m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{
      StyleBox["i",
       FontSlant->"Italic"], "=", "1"}], 
     StyleBox["m",
      FontSlant->"Italic"]], 
    SubscriptBox["S", "1"]}], TraditionalForm]]],
 "(",
 StyleBox["i",
  FontSlant->"Italic"],
 ")/",
 StyleBox["i",
  FontSlant->"Italic"],
 ". SumS[k,l,m] is ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], " ", 
    RowBox[{
     RowBox[{
      SubscriptBox["S", "l"], "(", "i", ")"}], "/", 
     SuperscriptBox["i", "k"]}]}], TraditionalForm]]],
 ". SumS[r, n] represents Sum[Sign[r]^i/i^Abs[r], {i, 1, n}]. SumS[r,s, n] is \
Sum[Sign[r]^k/k^Abs[r] Sign[s]^j/j^Abs[s], {k, 1, n}, {j, 1, k}], etc."
}], "Text",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SumS", "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1577]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Reduce", "\[Rule]", "False"}], "}"}], TraditionalForm]], "Output",\

 CellTags->"SumS",
 CellLabel->"Out[1577]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumP",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumP", All, CellTags],
  ButtonData->"SumP",
  ButtonNote->"SumP"],
 ", ",
 ButtonBox["SumT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumT", All, CellTags],
  ButtonData->"SumT",
  ButtonNote->"SumT"],
 "."
}], "Text",
 CellTags->"SumS"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumS"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1578]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "1"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1578]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1579]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "2"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1579]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "m"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1580]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", 
    RowBox[{"-", "1"}]], "(", "m", ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1580]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "m", ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1581]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", "m"]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1581]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"3", ",", 
   RowBox[{"m", "+", "2"}], ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1582]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "+", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1582]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SumS", ",", 
    RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}], ";"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1583]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"3", ",", 
   RowBox[{"m", "+", "2"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1584]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    SuperscriptBox["m", "3"]], "+", 
   RowBox[{
    SubscriptBox["S", "3"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "1"}], ")"}], "3"]], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"m", "+", "2"}], ")"}], "3"]]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1584]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"SumS", ",", 
    RowBox[{"Reduce", "\[Rule]", "False"}]}], "]"}], ";"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1585]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1586]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1586]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "4"], 
  RowBox[{"1", "/", "i"}]}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1587]:="],

Cell[BoxData[
 FormBox[
  FractionBox["25", "12"], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1587]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1588]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["S", "12"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1588]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", "1", ",", "1", ",", "11"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1589]:="],

Cell[BoxData[
 FormBox[
  FractionBox["31276937512951", "4260000729600"], TraditionalForm]], "Output",\

 CellTags->"SumS",
 CellLabel->"Out[1589]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1590]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1590]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumS",
 CellLabel->"In[1591]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumS",
 CellLabel->"Out[1591]="]
}, Open  ]]
}, Open  ]],

Cell["SumT", "ObjectName",
 CellTags->"SumT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SumT"],

Cell[TextData[{
 "SumT[1, m] is the alternative harmonic number ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubsuperscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "m"], " ", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i", " "}]}], 
   TraditionalForm]]],
 " SumT[r, n] represents Sum[(-1)^i/i^r, {i,1,n}], SumT[r,s, n] is Sum[1/k^r \
(-1)^j/j^s, {k, 1, n}, {j, 1, k}]."
}], "Text",
 CellTags->"SumT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SumP",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumP", All, CellTags],
  ButtonData->"SumP",
  ButtonNote->"SumP"],
 ", ",
 ButtonBox["SumS",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SumS", All, CellTags],
  ButtonData->"SumS",
  ButtonNote->"SumS"],
 "."
}], "Text",
 CellTags->"SumT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SumT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1592]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "1"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1592]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1593]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "2"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1593]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "m"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1594]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "1"], "(", "m", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1594]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "m", ",", 
   RowBox[{"Reduce", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1595]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox[
     OverscriptBox["S", "~"], "1"], "(", 
    FormBox[
     RowBox[{"m", "-", "1"}],
     TraditionalForm], ")"}], "+", 
   FractionBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"], "m"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1595]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1596]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1596]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], "4"], 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1597]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1597]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", 
   RowBox[{"m", "-", "1"}]}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1598]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox[
    OverscriptBox["S", "~"], "12"], "(", 
   FormBox[
    RowBox[{"m", "-", "1"}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1598]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", "42"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1599]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["38987958697055013360489864298703621429610152138683927", 
    "10512121660702378405316004964483761080879190528000000"]}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1599]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1600]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1600]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{
   RowBox[{"-", "1"}], ",", "4"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1601]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["7", "12"]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1601]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  UnderoverscriptBox["\[Sum]", 
   RowBox[{"i", "=", "1"}], 
   RowBox[{"m", "-", "1"}]], 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "^", "i"}], "/", "i"}]}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1602]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
    TemplateBox[{RowBox[{"-", "1"}],"1","m"},
     "LerchPhi"]}], "-", 
   RowBox[{"log", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1602]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumT", "[", 
  RowBox[{"1", ",", "2", ",", "12"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1603]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["57561743656913", "21300003648000"]}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1603]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SumS", "[", 
  RowBox[{"1", ",", 
   RowBox[{"-", "2"}], ",", "42"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1604]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox["38987958697055013360489864298703621429610152138683927", 
    "10512121660702378405316004964483761080879190528000000"]}], 
  TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1604]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Array", "[", 
  RowBox[{"SumT", ",", "6"}], "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1605]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", 
     FractionBox["5", "8"]}], ",", 
    RowBox[{"-", 
     FractionBox["179", "216"]}], ",", 
    RowBox[{"-", 
     FractionBox["1207", "1728"]}], ",", 
    RowBox[{"-", 
     FractionBox["170603", "216000"]}], ",", 
    RowBox[{"-", 
     FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1605]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Array", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"SumS", "[", 
     RowBox[{
      RowBox[{"-", "2"}], ",", "1", ",", "#1"}], "]"}], "&"}], ",", "6"}], 
  "]"}]], "Input",
 CellTags->"SumT",
 CellLabel->"In[1606]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"-", "1"}], ",", 
    RowBox[{"-", 
     FractionBox["5", "8"]}], ",", 
    RowBox[{"-", 
     FractionBox["179", "216"]}], ",", 
    RowBox[{"-", 
     FractionBox["1207", "1728"]}], ",", 
    RowBox[{"-", 
     FractionBox["170603", "216000"]}], ",", 
    RowBox[{"-", 
     FractionBox["155903", "216000"]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"SumT",
 CellLabel->"Out[1606]="]
}, Open  ]]
}, Open  ]],

Cell["SUND", "ObjectName",
 CellTags->"SUND"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUND"],

Cell[TextData[{
 "SUND[a,b,c] are the symmetric SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["d", "abc"], "."}], TraditionalForm]]]
}], "Text",
 CellTags->"SUND"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNDelta", All, CellTags],
  ButtonData->"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNF", All, CellTags],
  ButtonData->"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUND"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUND"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1607]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "c"}]], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1607]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"tt", "=", 
  RowBox[{"SUND", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1608]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       SubscriptBox["T", 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{"2", " ", 
    RowBox[{"tr", "(", 
     FormBox[
      RowBox[{
       SubscriptBox["T", 
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm]], ".", 
       SubscriptBox["T", 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm]]}],
      TraditionalForm], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1608]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"c", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1609]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "c"}]], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1609]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "b"}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1610]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["d", 
   RowBox[{"a", "b", "b"}]], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1610]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1611]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", 
   RowBox[{"(", 
    RowBox[{"4", "-", 
     SubsuperscriptBox["C", "A", "2"]}], ")"}], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1611]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1612]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["N", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1612]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1613]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{"4", "-", 
       SuperscriptBox["N", "2"]}], ")"}], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["e",
         TraditionalForm],
        TraditionalForm]}]]}], "N"]}], TraditionalForm]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1613]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUND", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], "Input",\

 CellTags->"SUND",
 CellLabel->"In[1614]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1614]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1615]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1615]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUND", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
   "StandardForm"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1616]:="],

Cell[BoxData[
 RowBox[{"SUND", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUND",
 CellLabel->"Out[1616]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", "tt", "]"}]], "Input",
 CellTags->"SUND",
 CellLabel->"In[1617]:="]
}, Open  ]],

Cell["SUNDelta", "ObjectName",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNDelta"],

Cell[TextData[{
 "SUNDelta[a, b] is the Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with color indices a and b."
}], "Text",
 CellTags->"SUNDelta"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitSUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExplicitSUNIndex", All, CellTags],
  ButtonData->"ExplicitSUNIndex",
  ButtonNote->"ExplicitSUNIndex"],
 ", ",
 ButtonBox["SD",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SD", All, CellTags],
  ButtonData->"SD",
  ButtonNote->"SD"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNF", All, CellTags],
  ButtonData->"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNIndex", All, CellTags],
  ButtonData->"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Trick", All, CellTags],
  ButtonData->"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"SUNDelta"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1618]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{"a", "b"}]], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1618]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", " ", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"b", ",", "c"}], "]"}]}], " ", "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1619]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1619]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1620]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1620]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNDelta", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1621]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1621]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1622]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1622]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNDelta", "[", 
     RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1623]:="],

Cell[BoxData[
 RowBox[{"SD", "[", 
  RowBox[{"a", ",", "b"}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1623]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SD", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1624]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1624]//StandardForm="]
}, Open  ]],

Cell["\<\
Symbolic arguments to SUNDelta are transformed into the data type SUNIndex \
and integer arguments are transformed to ExplicitSUNIndex. The difference is \
that SUNSimplify will only sum over symbolic indices.\
\>", "Text",
 CellTags->"SUNDelta"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "2"}], "]"}], 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"c", ",", "2"}], "]"}]}], "//", "SUNSimplify"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1625]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox["2",
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox["2",
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1625]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1626]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}], " ", 
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"ExplicitSUNIndex", "[", "2", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1626]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"1", ",", "2"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNDelta",
 CellLabel->"In[1627]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"ExplicitSUNIndex", "[", "1", "]"}], ",", 
   RowBox[{"ExplicitSUNIndex", "[", "2", "]"}]}], "]"}]], "Output",
 CellTags->"SUNDelta",
 CellLabel->"Out[1627]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNDeltaContract", "ObjectName",
 CellTags->"SUNDeltaContract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNDeltaContract"],

Cell[TextData[{
 "SUNDeltaContract[expr] substitues for all SUNDelta in expr \
SUNDeltaContract, contracts the SUN(N) indices and resubstitutes SUNDelta. \
SUNDeltaContract[i, j] is the Kronecker-delta for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") with contraction properties. SUNDeltaContract wraps also the head \
SUNIndex around its arguments."
}], "Text",
 CellTags->"SUNDeltaContract"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNDelta", All, CellTags],
  ButtonData->"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNIndex", All, CellTags],
  ButtonData->"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"SUNDeltaContract"]
}, Open  ]],

Cell["SUNF", "ObjectName",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNF"],

Cell[TextData[{
 "SUNF[a,b,c] are the structure constants of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 "). The arguments a,b,c should be of symbolic type."
}], "Text",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNF", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1628]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1628]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUND",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUND", All, CellTags],
  ButtonData->"SUND",
  ButtonNote->"SUND"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNDelta", All, CellTags],
  ButtonData->"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNIndex", All, CellTags],
  ButtonData->"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["SUNT",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNT", All, CellTags],
  ButtonData->"SUNT",
  ButtonNote->"SUNT"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Trick", All, CellTags],
  ButtonData->"Trick",
  ButtonNote->"Trick"],
 "."
}], "Text",
 CellTags->"SUNF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "x"}], "+", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"b", ",", "a", ",", "c"}], "]"}], "y"}]}]}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1629]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    SubscriptBox["f", 
     RowBox[{"a", "b", "c"}]]}], "+", 
   RowBox[{"y", " ", 
    SubscriptBox["f", 
     RowBox[{"b", "a", "c"}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1629]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Calc", "[", "t1", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1630]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"x", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{"y", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1630]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1631]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"x", "-", "y"}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1631]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "a", ",", "b"}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1632]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{"a", "a", "b"}]], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1632]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "a", ",", "b"}], "]"}], "//", "Calc"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1633]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1633]="]
}, Open  ]],

Cell[TextData[{
 "This is a consequence of the usual choice for the normalization of the ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SubscriptBox["T", "a"], " ", "generatorsin", " ", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"see", "  ", "SUNT"}], ")"}], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SUNF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1634]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1634]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1635]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1635]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1636]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1636]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "StandardForm"}]], "Input",\

 CellTags->"SUNF",
 CellLabel->"In[1637]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1637]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1638]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "b", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "c", "]"}]}], "]"}]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1638]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "c"}], "]"}], "//", "FCI"}], "//", "FCE"}], "//",
   "StandardForm"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1639]:="],

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"a", ",", "b", ",", "c"}], "]"}]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1639]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNF", "[", 
  RowBox[{"b", ",", "a", ",", "c"}], "]"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1640]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["f", 
   RowBox[{"b", "a", "c"}]], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1640]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNF", "[", 
   RowBox[{"b", ",", "a", ",", "c"}], "]"}], "//", "FCI"}]], "Input",
 CellTags->"SUNF",
 CellLabel->"In[1641]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNF",
 CellLabel->"Out[1641]="]
}, Open  ]]
}, Open  ]],

Cell["SUNFJacobi", "ObjectName",
 CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNFJacobi"],

Cell["\<\
SUNFJacobi is an option for SUNSimplify, indicating whether the Jacobi \
identity should be used.\
\>", "Text",
 CellTags->"SUNFJacobi"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNF", All, CellTags],
  ButtonData->"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUNFJacobi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNFJacobi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
  RowBox[{
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"#", ",", 
     RowBox[{"SUNFJacobi", "\[Rule]", "False"}]}], "]"}], "&"}]}]], "Input",
 CellTags->"SUNFJacobi",
 CellLabel->"In[1642]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["f",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNFJacobi",
 CellLabel->"Out[1642]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"e", ",", "f", ",", "c"}], "]"}]}], "//", 
  RowBox[{
   RowBox[{"SUNSimplify", "[", 
    RowBox[{"#", ",", 
     RowBox[{"SUNFJacobi", "\[Rule]", "True"}]}], "]"}], "&"}]}]], "Input",
 CellTags->"SUNFJacobi",
 CellLabel->"In[1643]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["f",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["f",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNFJacobi",
 CellLabel->"Out[1643]="]
}, Open  ]]
}, Open  ]],

Cell["SUNIndex", "ObjectName",
 CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNIndex"],

Cell[TextData[{
 "SUNIndex is the head of SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") indices."
}], "Text",
 CellTags->"SUNIndex"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ExplicitSUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ExplicitSUNIndex", All, CellTags],
  ButtonData->"ExplicitSUNIndex",
  ButtonNote->"ExplicitSUNIndex"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNDelta", All, CellTags],
  ButtonData->"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNF", All, CellTags],
  ButtonData->"SUNF",
  ButtonNote->"SUNF"],
 "."
}], "Text",
 CellTags->"SUNIndex"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNIndex"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNIndex", "[", "i", "]"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1644]:="],

Cell[BoxData[
 FormBox["i", TraditionalForm]], "Output",
 CellTags->"SUNIndex",
 CellLabel->"Out[1644]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1645]:="],

Cell[BoxData[
 RowBox[{"SUNIndex", "[", "i", "]"}]], "Output",
 CellTags->"SUNIndex",
 CellLabel->"Out[1645]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"i", ",", "j"}], "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNIndex",
 CellLabel->"In[1646]:="],

Cell[BoxData[
 RowBox[{"SUNDelta", "[", 
  RowBox[{
   RowBox[{"SUNIndex", "[", "i", "]"}], ",", 
   RowBox[{"SUNIndex", "[", "j", "]"}]}], "]"}]], "Output",
 CellTags->"SUNIndex",
 CellLabel->"Out[1646]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNIndexRename", "ObjectName",
 CellTags->"SUNIndexRename"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNIndexRename"],

Cell["\<\
SUNIndexRename is an option of SUNSimplify. If set to False, no automatic \
renaming of dummy indices is done.\
\>", "Text",
 CellTags->"SUNIndexRename"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNIndex", All, CellTags],
  ButtonData->"SUNIndex",
  ButtonNote->"SUNIndex"],
 "."
}], "Text",
 CellTags->"SUNIndexRename"]
}, Open  ]],

Cell["SUNN", "ObjectName",
 CellTags->"SUNN"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNN"],

Cell[TextData[{
 "SUNN denotes the number of colors. Trick[SUNDelta[a, a]] yields (",
 Cell[BoxData[
  FormBox[
   SuperscriptBox["SUNN", "2"], TraditionalForm]]],
 " -1)."
}], "Text",
 CellTags->"SUNN"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Trick", All, CellTags],
  ButtonData->"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["SUNIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNIndex", All, CellTags],
  ButtonData->"SUNIndex",
  ButtonNote->"SUNIndex"],
 ", ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CA", All, CellTags],
  ButtonData->"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CF", All, CellTags],
  ButtonData->"CF",
  ButtonNote->"CF"],
 "."
}], "Text",
 CellTags->"SUNN"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNN"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
     RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNN",
 CellLabel->"In[1647]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["N", "2"], "-", "1"}], TraditionalForm]], "Output",
 CellTags->"SUNN",
 CellLabel->"Out[1647]="]
}, Open  ]]
}, Open  ]],

Cell["SUNNToCACF", "ObjectName",
 CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNNToCACF"],

Cell["\<\
SUNNToCACF is an option of SUNSimplify. If set to True, the Casimir operator \
eigenvalues CA (=N) and CF (=(N^2-1)/(2 N)) are introduced.\
\>", "Text",
 CellTags->"SUNNToCACF"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Trick", All, CellTags],
  ButtonData->"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["SUNN",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNN", All, CellTags],
  ButtonData->"SUNN",
  ButtonNote->"SUNN"],
 ", ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CA", All, CellTags],
  ButtonData->"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CF", All, CellTags],
  ButtonData->"CF",
  ButtonNote->"CF"],
 "."
}], "Text",
 CellTags->"SUNNToCACF"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNNToCACF"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{
     RowBox[{"SUNIndex", "[", "a", "]"}], ",", " ", 
     RowBox[{"SUNIndex", "[", "a", "]"}]}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNNToCACF",
 CellLabel->"In[1648]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUNNToCACF",
 CellLabel->"Out[1648]="]
}, Open  ]]
}, Open  ]],

Cell["SUNSimplify", "ObjectName",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNSimplify"],

Cell["\<\
SUNSimplify simplifies products of SUNT (and complex conjugated) matrices. \
Basic renaming of dummy indices is done. If the option SUNTrace is set to \
False, then any SUNT-matrices are taken out of DiracTrace[...]; otherwise a \
color-trace is taken (by SUNTrace) before taking the SUN-objects in front of \
DiracTrace[...].\
\>", "Text",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNSimplify", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1649]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Expanding", "\[Rule]", "False"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"SUNIndexRename", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNFJacobi", "\[Rule]", "False"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1649]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["Trick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Trick", All, CellTags],
  ButtonData->"Trick",
  ButtonNote->"Trick"],
 ", ",
 ButtonBox["CalcColorFactor",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CalcColorFactor", All, CellTags],
  ButtonData->"CalcColorFactor",
  ButtonNote->"CalcColorFactor"],
 "."
}], "Text",
 CellTags->"SUNSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNSimplify"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{
   RowBox[{"SUNDelta", "[", 
    RowBox[{"a", ",", "b"}], "]"}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1650]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{"a", "b"}]], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{"b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1650]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t1", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1651]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Delta]", 
   RowBox[{
    FormBox[
     FormBox["a",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["c",
      TraditionalForm],
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1651]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"SUNT", "[", "a", "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1652]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1652]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t2", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1653]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["C", "F"], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1653]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1654]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["N", "2"], "-", "1"}], 
   RowBox[{"2", " ", "N"}]], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1654]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "r", ",", "s"}], "]"}], 
   RowBox[{"SUNF", "[", 
    RowBox[{"b", ",", "r", ",", "s"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1655]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "r", "s"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"b", "r", "s"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1655]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t3", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1656]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1656]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", ".", " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1657]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]], ".", 
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1657]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t4", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1658]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubsuperscriptBox["C", "A", "2"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1658]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNF", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1659]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"d", "b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1659]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", "t5", "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1660]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1660]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", "=", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1661]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["d", 
    RowBox[{"b", "c", "d"}]], " ", 
   SubscriptBox["f", 
    RowBox[{"a", "b", "c"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1661]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t6", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1662]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1662]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1663]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", 
      SuperscriptBox["N", "2"]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SuperscriptBox["N", "2"]}], ")"}]}], "N"], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1663]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNSimplify", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"SUND", "[", 
      RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
     RowBox[{"SUND", "[", 
      RowBox[{"e", ",", "b", ",", "c"}], "]"}]}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}], "//", 
  "FullSimplify"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1664]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["N", "2"], "-", "4"}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "N"], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1664]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1665]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "-", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1665]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1666]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}], "+", 
     RowBox[{"tr", "(", 
      FormBox[
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm]]}],
       TraditionalForm], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1666]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUNT", "[", 
    RowBox[{"c", ",", "b", ",", "a"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1667]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
   SubscriptBox["C", "A"], " ", 
   SubscriptBox["C", "F"]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1667]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t7", "=", 
  RowBox[{
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "e"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"c", ",", "d", ",", "e"}], "]"}]}], "+", 
   RowBox[{
    RowBox[{"SUNF", "[", 
     RowBox[{"a", ",", "b", ",", "z"}], "]"}], 
    RowBox[{"SUNF", "[", 
     RowBox[{"c", ",", "d", ",", "z"}], "]"}]}]}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1668]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "e"}]], " ", 
    SubscriptBox["f", 
     RowBox[{"c", "d", "e"}]]}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{"a", "b", "z"}]], " ", 
    SubscriptBox["f", 
     RowBox[{"c", "d", "z"}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1668]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t7", ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1669]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1669]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t7", ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNIndexRename", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1670]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["z",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["z",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1670]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"1", "-", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"i", ",", "i"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1671]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"2", "-", 
   SubsuperscriptBox["C", "A", "2"]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1671]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t8", "=", 
  RowBox[{"DiracTrace", "[", 
   RowBox[{
    RowBox[{"f", "[", 
     RowBox[{"SUNIndex", "[", "a", "]"}], "]"}], 
    RowBox[{
     RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
     RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}]}]}], "]"}]}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1672]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{"f", "(", 
      FormBox["a",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1672]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t8", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1673]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     RowBox[{"f", "(", 
      FormBox["a",
       TraditionalForm], ")"}], " ", 
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1673]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t8", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1674]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"f", "(", 
    FormBox["a",
     TraditionalForm], ")"}], " ", 
   SubscriptBox["C", "A"], " ", 
   RowBox[{"tr", "(", 
    FormBox[
     RowBox[{
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], ".", 
      SuperscriptBox["\[Gamma]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNSimplify",
 CellLabel->"Out[1674]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "b", ",", "a"}], "]"}], "+", 
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "a", ",", "b"}], "]"}]}], "]"}]], "Input",
 CellLabel->"In[1675]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   SubscriptBox["C", "F"], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["C", "A"], "-", 
     RowBox[{"4", " ", 
      SubscriptBox["C", "F"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellLabel->"Out[1675]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{
  "t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6", ",", "t7", ",",
    "t8"}], "]"}]], "Input",
 CellTags->"SUNSimplify",
 CellLabel->"In[1676]:="]
}, Open  ]],

Cell["SUNT", "ObjectName",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNT"],

Cell[TextData[{
 "SUNT[a] is the SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "a"], TraditionalForm]]],
 " generator in the fundamental representation."
}], "Text",
 CellTags->"SUNT"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["CA",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CA", All, CellTags],
  ButtonData->"CA",
  ButtonNote->"CA"],
 ", ",
 ButtonBox["CF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CF", All, CellTags],
  ButtonData->"CF",
  ButtonNote->"CF"],
 ", ",
 ButtonBox["SUND",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUND", All, CellTags],
  ButtonData->"SUND",
  ButtonNote->"SUND"],
 ", ",
 ButtonBox["SUNDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNDelta", All, CellTags],
  ButtonData->"SUNDelta",
  ButtonNote->"SUNDelta"],
 ", ",
 ButtonBox["SUNF",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNF", All, CellTags],
  ButtonData->"SUNF",
  ButtonNote->"SUNF"],
 ", ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 "."
}], "Text",
 CellTags->"SUNT"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNT", "[", "a", "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1677]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["T", 
   FormBox["a",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1677]="]
}, Open  ]],

Cell[TextData[{
 "Since ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "a"], TraditionalForm]]],
 " is a noncommutative object, products have to separated by a dot (.)."
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNT", "[", "a", "]"}], ".", " ", 
  RowBox[{"SUNT", "[", "b", "]"}], ".", " ", 
  RowBox[{"SUNT", "[", "c", "]"}]}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1678]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["b",
     TraditionalForm]], ".", 
   SubscriptBox["T", 
    FormBox["c",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1678]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNT", "[", 
  RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1679]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", 
    FormBox["a",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["b",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["c",
     TraditionalForm]], 
   SubscriptBox["T", 
    FormBox["d",
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1679]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1680]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    SubscriptBox["T", 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1680]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1681]:="],

Cell[BoxData[
 FormBox[
  SubsuperscriptBox["C", "F", "2"], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1681]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "a"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1682]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   SubscriptBox["T", 
    FormBox[
     FormBox["b",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["C", "A"], "-", 
     RowBox[{"2", " ", 
      SubscriptBox["C", "F"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1682]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "a"}], "]"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1683]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    SubscriptBox["T", 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1683]="]
}, Open  ]],

Cell[TextData[{
 "The normalizaton of the generators is chosen in the standard way, therefore \
",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"tr", "(", 
     RowBox[{
      SubscriptBox["T", "a"], 
      SubscriptBox["T", "b"]}], ")"}], " ", "=", " ", 
    RowBox[{
     RowBox[{"1", "/", "2"}], " ", 
     RowBox[{
      SubscriptBox["\[Delta]", "ab"], "."}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1684]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], "2"], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1684]="]
}, Open  ]],

Cell[TextData[{
 "In case you want ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "f"], TraditionalForm]]],
 ", you need to include a factor 2Tf inside the trace."
}], "Text",
 CellTags->"SUNT"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"2", " ", "Tf", " ", 
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1685]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["T", "f"], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1685]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SUNTrace", "[", 
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b"}], "]"}], "]"}], "//", "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1686]:="],

Cell[BoxData[
 RowBox[{
  FractionBox["1", "2"], " ", 
  RowBox[{"SUNDelta", "[", 
   RowBox[{
    RowBox[{"SUNIndex", "[", "a", "]"}], ",", 
    RowBox[{"SUNIndex", "[", "b", "]"}]}], "]"}]}]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1686]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1687]:="],

Cell[BoxData[
 RowBox[{"SUNT", "[", 
  RowBox[{"SUNIndex", "[", "a", "]"}], "]"}]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1687]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"SUNT", "[", "a", "]"}], "//", "FCI"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"SUNT",
 CellLabel->"In[1688]:="],

Cell[BoxData[
 RowBox[{"SUNT", "[", "a", "]"}]], "Output",
 CellTags->"SUNT",
 CellLabel->"Out[1688]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["SUNTrace", "ObjectName",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SUNTrace"],

Cell["SUNTrace[expr] calculates the color-trace.", "Text",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "SUNTrace", "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1689]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Explicit", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1689]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["SUNSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNSimplify", All, CellTags],
  ButtonData->"SUNSimplify",
  ButtonNote->"SUNSimplify"],
 ", ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"SUNTrace"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"SUNTrace"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1690]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], "2"], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1690]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1691]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1691]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{
   RowBox[{"SUNT", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1692]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    SubscriptBox["d", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]], "4"], "+", 
   RowBox[{
    FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1692]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNTrace", "[", 
  RowBox[{"SUNT", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1693]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1693]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"SUNTrace", "[", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1694]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
     SubscriptBox["f", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm], "e"}]], " ", 
     SubscriptBox["d", 
      RowBox[{"e", 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]]}]}], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], "e"}]]}], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "-", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    FractionBox["1", "8"], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["d", 
     RowBox[{"e", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]], "-", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["c",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["d",
         TraditionalForm],
        TraditionalForm]}]]}], 
    RowBox[{"4", " ", "N"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1694]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t1", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1695]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1695]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"SUNTrace", "[", 
   RowBox[{
    RowBox[{"SUNT", "[", 
     RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "]"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1696]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FractionBox["1", "2"], " ", 
    SubscriptBox["d", 
     RowBox[{"c215", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], "c216"}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm], "c216"}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c216", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c215",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c215",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["c215",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]]}], ")"}]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], "c215"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], "c217"}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", "\[ImaginaryI]", " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm], "c217"}]]}], "-", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]]}], "+", 
      RowBox[{
       FractionBox["1", "8"], " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["e",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["d", 
        RowBox[{"c217", 
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c215",
           TraditionalForm],
          TraditionalForm]}]]}], "-", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c215",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c215",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]], "+", 
      FractionBox[
       RowBox[{
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["c215",
            TraditionalForm],
           TraditionalForm]}]], " ", 
        SubscriptBox["\[Delta]", 
         RowBox[{
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}], 
       RowBox[{"4", " ", "N"}]]}], ")"}]}], "+", 
   FractionBox[
    RowBox[{
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SubscriptBox["d", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]], "4"], "+", 
       RowBox[{
        FractionBox["1", "4"], " ", "\[ImaginaryI]", " ", 
        SubscriptBox["f", 
         RowBox[{
          FormBox[
           FormBox["c",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["d",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["e",
            TraditionalForm],
           TraditionalForm]}]]}]}], ")"}]}], 
    RowBox[{"2", " ", "N"}]]}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1696]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{"t2", ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1697]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"tr", "(", 
   FormBox[
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["d",
        TraditionalForm],
       TraditionalForm]], ".", 
     SubscriptBox["T", 
      FormBox[
       FormBox["e",
        TraditionalForm],
       TraditionalForm]]}],
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1697]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1698]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1698]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "d"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1699]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1699]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUNF", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "d", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1700]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1700]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SUNSimplify", "[", 
  RowBox[{
   RowBox[{"SUND", "[", 
    RowBox[{"a", ",", "b", ",", "c"}], "]"}], " ", 
   RowBox[{"SUND", "[", 
    RowBox[{"d", ",", "b", ",", "c"}], "]"}]}], "]"}]], "Input",
 CellTags->"SUNTrace",
 CellLabel->"In[1701]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{"(", 
     RowBox[{"4", "-", 
      SubsuperscriptBox["C", "A", "2"]}], ")"}]}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubscriptBox["C", "A"], "-", 
     RowBox[{"2", " ", 
      SubscriptBox["C", "F"]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"SUNTrace",
 CellLabel->"Out[1701]="]
}, Open  ]]
}, Open  ]],

Cell["SymbolicSum2", "ObjectName",
 CellTags->"SymbolicSum2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SymbolicSum2"],

Cell["\<\
SymbolicSum2 is similar to SymbolicSum (Algegra`SymbolicSum`SymbolicSum was a \
function to do symbolic summation. It was obsolete from version 3 - all \
functionality is now autoloaded by Sum), but extended to several double sums.\
\
\>", "Text",
 CellTags->"SymbolicSum2"]
}, Open  ]],

Cell["SymbolicSum3", "ObjectName",
 CellTags->"SymbolicSum3"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"SymbolicSum3"],

Cell["\<\
SymbolicSymbolicSum3 is similar to SymbolicSum \
(Algegra`SymbolicSum`SymbolicSum was a function to do symbolic summation. It \
was obsolete from version 3 - all functionality is now autoloaded by Sum), \
but extended to several double sums.\
\>", "Text",
 CellTags->"SymbolicSum3"]
}, Open  ]],

Cell["Symmetrize", "ObjectName",
 CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Symmetrize"],

Cell["\<\
Symmetrize[expr, {a1, a2, ...}] antisymmetrizes expr with respect to the \
variables a1,a2, ... \
\>", "Text",
 CellTags->"Symmetrize"],

Cell[TextData[{
 "See also: ",
 ButtonBox["AntiSymmetrize",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "AntiSymmetrize", All, CellTags],
  ButtonData->"AntiSymmetrize",
  ButtonNote->"AntiSymmetrize"],
 "."
}], "Text",
 CellTags->"Symmetrize"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Symmetrize"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Symmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"a", ",", "b"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"a", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Symmetrize",
 CellLabel->"In[1702]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"a", ",", "b"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"b", ",", "a"}], ")"}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Symmetrize",
 CellLabel->"Out[1702]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Symmetrize", "[", 
  RowBox[{
   RowBox[{"f", "[", 
    RowBox[{"x", ",", "y", ",", "z"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "y", ",", "z"}], "}"}]}], "]"}]], "Input",
 CellTags->"Symmetrize",
 CellLabel->"In[1703]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "6"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "y", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"x", ",", "z", ",", "y"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "x", ",", "z"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"y", ",", "z", ",", "x"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "x", ",", "y"}], ")"}], "+", 
     RowBox[{"f", "(", 
      RowBox[{"z", ",", "y", ",", "x"}], ")"}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Symmetrize",
 CellLabel->"Out[1703]="]
}, Open  ]]
}, Open  ]],

Cell["TBox, Tbox", "ObjectName",
 CellTags->{"TBox", "Tbox"}],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->{"TBox", "Tbox"}],

Cell["\<\
TBox[a, b, ...] or Tbox[a, b, ...] produces a RowBox[{a,b, ...}] where a, b, \
... are boxed in TraditionalForm. TBox and Tbox are used internally by \
FeynCalc to produce the typeset output in TraditionalForm\
\>", "Text",
 CellTags->{"TBox", "Tbox"}]
}, Open  ]],

Cell["Tdec", "ObjectName",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tdec"],

Cell["\<\
Tdec[{q,mu}, {p}]; Tdec[{{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] or \
Tdec[exp, {{qi, mu}, {qj, nu}, ...}, {p1, p2, ...}] calculates the tensorial \
decomposition formulas. The more common ones are saved in TIDL.\
\>", "Text",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Tdec", "]"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1704]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "Factor2"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "True"}], ",", 
    RowBox[{"List", "\[Rule]", "True"}], ",", 
    RowBox[{"NumberOfMetricTensors", "\[Rule]", "\[Infinity]"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1704]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TID", All, CellTags],
  ButtonData->"TID",
  ButtonNote->"TID"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TIDL", All, CellTags],
  ButtonData->"TIDL",
  ButtonNote->"TIDL"],
 ", ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"Tdec"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Tdec"],

Cell[TextData[{
 "Check that ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[Integral]", 
     RowBox[{
      SuperscriptBox["d", "D"], 
      RowBox[{"f", "(", 
       RowBox[{"p", ",", "q"}], ")"}], 
      SuperscriptBox["q", "\[Mu]"]}]}], "=", " ", 
    RowBox[{
     FractionBox[
      SuperscriptBox["p", "\[Mu]"], 
      SuperscriptBox["p", "2"]], 
     RowBox[{"\[Integral]", 
      RowBox[{
       SuperscriptBox["d", "D"], 
       RowBox[{"f", "(", 
        RowBox[{"p", ",", "q"}], ")"}], 
       RowBox[{"p", "\[CenterDot]", "q"}]}]}]}]}], TraditionalForm]]]
}], "Text",
 CellTags->"Tdec"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tdec", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1705]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"X1", "\[Rule]", 
       SuperscriptBox["m", "2"]}], ",", 
      RowBox[{"X2", "\[Rule]", 
       FormBox[
        RowBox[{
         FormBox["p",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}],
        TraditionalForm]}]}], "}"}], ",", 
    FractionBox[
     RowBox[{"X2", " ", 
      SuperscriptBox["p", "\[Mu]"]}], "X1"]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1705]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "/.", 
  RowBox[{"%", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1706]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["p", "\[Mu]"], " ", 
    FormBox[
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}],
     TraditionalForm]}], 
   SuperscriptBox["m", "2"]], TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1706]="]
}, Open  ]],

Cell[TextData[{
 "This calculates integral transformation for any ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Integral]", 
    RowBox[{
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "1"], 
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "2"], 
     SuperscriptBox["d", "D"], 
     SubscriptBox["q", "3"]}]}], TraditionalForm]]],
 " ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"f", "(", 
     RowBox[{"p", ",", 
      RowBox[{
       SubscriptBox["q", 
        RowBox[{"1", ","}]], 
       SubscriptBox["q", "2"]}], ",", 
      SubscriptBox["q", "3"]}], ")"}], " ", 
    SubsuperscriptBox["q", "1", "\[Mu]"], 
    SubsuperscriptBox["q", "2", "\[Nu]"], 
    SubsuperscriptBox["q", "3", "\[Rho]"]}], TraditionalForm]]],
 "."
}], "Text",
 CellTags->"Tdec"],

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "=", 
   RowBox[{"Tdec", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "1"], ",", "\[Mu]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "2"], ",", "\[Nu]"}], "}"}], ",", 
       RowBox[{"{", 
        RowBox[{
         SubscriptBox["q", "3"], ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
     RowBox[{"{", "p", "}"}]}], "]"}]}], ";"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1707]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"t", "[", 
   RowBox[{"[", "2", "]"}], "]"}], "/.", 
  RowBox[{"t", "[", 
   RowBox[{"[", "1", "]"}], "]"}]}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1708]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["p", "\[Rho]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "3"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["p", "\[Nu]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "2"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox["p", "\[Mu]"], " ", 
     SuperscriptBox["g", 
      RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
     FormBox[
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["q", "1"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox["m", "4"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox["p", "\[Mu]"], " ", 
     SuperscriptBox["p", "\[Nu]"], " ", 
     SuperscriptBox["p", "\[Rho]"], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}], "+", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           FormBox[
            RowBox[{
             FormBox["p",
              TraditionalForm], "\[CenterDot]", 
             FormBox[
              SubscriptBox["q", "3"],
              TraditionalForm]}],
            TraditionalForm], ")"}]}], ")"}]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm]}], "-", 
       RowBox[{
        SuperscriptBox["m", "2"], " ", 
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm]}], "+", 
       RowBox[{"2", " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "1"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "2"],
           TraditionalForm]}],
         TraditionalForm], " ", 
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["q", "3"],
           TraditionalForm]}],
         TraditionalForm]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox["m", "6"]}]]}], TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1708]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Contract", "[", 
   RowBox[{
    RowBox[{"%", " ", 
     RowBox[{"FVD", "[", 
      RowBox[{"p", ",", "\[Mu]"}], "]"}]}], 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "\[Nu]"}], "]"}], " ", 
    RowBox[{"FVD", "[", 
     RowBox[{"p", ",", "\[Rho]"}], "]"}]}], "]"}], "//", "Factor"}]], "Input",\

 CellTags->"Tdec",
 CellLabel->"In[1709]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "1"}], ")"}], " ", 
     SuperscriptBox["m", "6"]}]], 
   RowBox[{
    SuperscriptBox[
     FormBox[
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"],
      TraditionalForm], "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"D", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["m", "4"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}]}], "-", 
      RowBox[{"3", " ", 
       SuperscriptBox["m", "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["m", "2"], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}]}], "+", 
      RowBox[{"2", " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "1"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "2"],
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["q", "3"],
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Tdec",
 CellLabel->"Out[1709]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", "t", "]"}], ";"}]], "Input",
 CellTags->"Tdec",
 CellLabel->"In[1710]:="]
}, Open  ]],

Cell["TensorFunction", "ObjectName",
 CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TensorFunction"],

Cell["\<\
Tensorfunction[t, mu, nu, ...] transform into t[LorentzIndex[mu], \
LorentzIndex[nu], ...], i.e., it can be used as an unspecified tensoriell \
function t. A symmetric tensor can be obtained by Tensorfunction[{t, \"S\"}, \
mu, nu, ...], and an antisymmteric one by Tensorfunction[{t, \"A\"}, mu, nu, \
...].\
\>", "Text",
 CellTags->"TensorFunction"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Symmetrize",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Symmetrize", All, CellTags],
  ButtonData->"Symmetrize",
  ButtonNote->"Symmetrize"],
 "."
}], "Text",
 CellTags->"TensorFunction"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TensorFunction"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1711]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1711]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1712]:="],

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], "]"}]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1712]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Contract", "[", 
  RowBox[{
   RowBox[{"FV", "[", 
    RowBox[{"p", ",", "\[Mu]"}], "]"}], " ", "%"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1713]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"Tensorfunction", "(", 
    RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1713]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1714]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", "p", "]"}]}], "]"}], " ", 
  RowBox[{"Tensorfunction", "[", 
   RowBox[{"t", ",", "\[Mu]", ",", "\[Nu]", ",", "\[Tau]"}], 
   "]"}]}]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1714]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Alpha]", ",", "\[Beta]"}],
   "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1715]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Alpha]", ",", 
    "\[Beta]"}], ")"}], TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1715]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\"\<S\>\""}], "}"}], ",", "\[Beta]", ",", "\[Alpha]"}],
   "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1716]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"Tensorfunction", "(", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", 
    "\[Alpha]"}], ")"}], TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1716]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1717]:="],

Cell[BoxData[
 RowBox[{"Tensorfunction", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"f", ",", "\<\"S\"\>"}], "}"}], ",", "\[Beta]", ",", "\[Alpha]"}],
   "]"}]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1717]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "f", "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1718]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellTags->"TensorFunction",
 CellLabel->"Out[1718]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"ClearAttributes", "[", 
  RowBox[{"f", ",", "Orderless"}], "]"}]], "Input",
 CellTags->"TensorFunction",
 CellLabel->"In[1719]:="]
}, Open  ]],

Cell["Tf", "ObjectName",
 CellTags->"Tf"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tf"],

Cell[TextData[{
 "Tf is the color factor ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["T", "f"], TraditionalForm]]],
 ". It is 1/2 for SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"Tf"],

Cell[TextData[{
 "See also: ",
 ButtonBox["Tr2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr2", All, CellTags],
  ButtonData->"Tr2",
  ButtonNote->"Tr2"],
 ", ",
 ButtonBox["GluonPropagator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonPropagator", All, CellTags],
  ButtonData->"GluonPropagator",
  ButtonNote->"GluonPropagator"],
 ", ",
 ButtonBox["GluonSelfEnergy",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "GluonSelfEnergy", All, CellTags],
  ButtonData->"GluonSelfEnergy",
  ButtonNote->"GluonSelfEnergy"],
 "."
}], "Text",
 CellTags->"Tf"]
}, Open  ]],

Cell["TFi", "ObjectName",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "TFi"}]], "Input",
 CellLabel->"In[1720]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   StyleBox[
    RowBox[{"Information", "::", "notfound"}], "MessageName"], 
   RowBox[{
   ":", " "}], "\<\"Symbol \[NoBreak]\\!\\(TraditionalForm\\`\\\"TFi\\\"\\)\
\[NoBreak] not found. \\!\\(\\*ButtonBox[\\\"\[RightSkeleton]\\\", \
ButtonStyle->\\\"Link\\\", ButtonFrame->None, \
ButtonData:>\\\"paclet:ref/message/General/notfound\\\", ButtonNote -> \
\\\"Information::notfound\\\"]\\)\"\>"}], TraditionalForm]], "Message", "MSG"]
}, Open  ]],

Cell["\<\
TFi[d, pp, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] is the 2-loop \
d-dimensional integral 1/( (q1^2 - m1^2)^n1 (q2^2 - m2^2)^n2 ((q1-p)^2 - \
m3^2)^n3 *((q2-p)^2 - m4^2)^n4 * ((q1-q2)^2 - m5^2)^n5 ) . TFi[d, pp, \
{x,y,z,v,w}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as additional \
factors in the numerator (q1^2)^x*(q2^2)^y*(q1.p)^z* (q2.p)^v*(q1.q2)^w. \
TFi[d, pp, dp, {a,b}, {{n1,m1},{n2,m2},{n3,m3},{n4,m4},{n5,m5}}] has as \
additional factors in the numerator (OPEDelta.q1)^a * (OPEDelta.q2)^b; dp is \
(OPEDelta.p).\
\>", "Text",
 CellTags->"TFi"],

Cell[TextData[{
 "TFi is similar to TFI from the TARCER package, see ",
 " ",
 ButtonBox["hep-ph/9801383.",
  BaseStyle->"Hyperlink",
  ButtonData:>{
    URL["http://xxx.lanl.gov/abs/hep-ph/9801383"], None}]
}], "Text",
 CellTags->"TFi"],

Cell["\<\
The function TarcerRecurse from the TARCER package recognize TFi (as well as \
TFI, which is defined in the HighEnergyPhysics`Tarcer` context).\
\>", "Text",
 CellTags->"TFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["ToTFi",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "ToTFi", All, CellTags],
  ButtonData->"ToTFi",
  ButtonNote->"ToTFi"],
 ", ",
 ButtonBox["FromTFi",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FromTFi", All, CellTags],
  ButtonData->"FromTFi",
  ButtonNote->"FromTFi"],
 "."
}], "Text",
 CellTags->"TFi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TFi", "[", 
  RowBox[{"D", ",", 
   RowBox[{"M", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "m1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m2"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"3", ",", "m3"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m4"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"1", ",", "m5"}], "}"}]}], "}"}]}], "]"}]], "Input",
 CellTags->"TFi",
 CellLabel->"In[1721]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFi", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["M", "2"], ",", 
    RowBox[{"(", "\[NoBreak]", GridBox[{
       {"2", "m1"},
       {"1", "m2"},
       {"3", "m3"},
       {"1", "m4"},
       {"1", "m5"}
      },
      GridBoxAlignment->{
       "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, 
        "Rows" -> {{Baseline}}, "RowsIndexed" -> {}},
      GridBoxSpacings->{"Columns" -> {
          Offset[0.27999999999999997`], {
           Offset[0.7]}, 
          Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
          Offset[0.2], {
           Offset[0.4]}, 
          Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}]}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"TFi",
 CellLabel->"Out[1721]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TFi", "[", 
  RowBox[{"D", ",", 
   RowBox[{"M", "^", "2"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", ",", 
     "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"TFi",
 CellLabel->"In[1722]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"TFi", "(", 
   RowBox[{"D", ",", 
    SuperscriptBox["M", "2"], ",", 
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"2", ",", "M"}], "}"}], ",", "1", ",", "1", ",", "1", ",", 
      "1"}], "}"}]}], ")"}], TraditionalForm]], "Output",
 CellTags->"TFi",
 CellLabel->"Out[1722]="]
}, Open  ]]
}, Open  ]],

Cell["ThreeVector", "ObjectName",
 CellTags->"ThreeVector"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ThreeVector"],

Cell["ThreeVector[p] is the three dimensional vector p.", "Text",
 CellTags->"ThreeVector"],

Cell[TextData[{
 "See also: ",
 ButtonBox["DotProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotProduct", All, CellTags],
  ButtonData->"DotProduct",
  ButtonNote->"DotProduct"],
 ", ",
 ButtonBox["CrossProduct",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "CrossProduct", All, CellTags],
  ButtonData->"CrossProduct",
  ButtonNote->"CrossProduct"],
 "."
}], "Text",
 CellTags->"ThreeVector"]
}, Open  ]],

Cell["TID", "ObjectName",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TID"],

Cell["TID[amp,q] performs a tensorial decomposition.", "Text",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TID", "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1723]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Collecting", "\[Rule]", "True"}], ",", 
    RowBox[{"Contract", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"ChangeDimension", "\[Rule]", "D"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynAmpDenominatorCombine", "\[Rule]", "True"}], ",", 
    RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "False"}], ",", 
    RowBox[{"Isolate", "\[Rule]", "False"}], ",", 
    RowBox[{"ScalarProductCancel", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1723]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 ", ",
 ButtonBox["TIDL",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TIDL", All, CellTags],
  ButtonData->"TIDL",
  ButtonNote->"TIDL"],
 "."
}], "Text",
 CellTags->"TID"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TID"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "1"]}], ",", " ", 
     RowBox[{"k", " ", "-", " ", 
      SubscriptBox["p", "2"]}]}], "]"}], 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1724]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "1"]}], ")"}], "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", 
       SubscriptBox["p", "2"]}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1724]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Factor", "/@", 
   RowBox[{"TID", "[", 
    RowBox[{"%", ",", "k"}], "]"}]}], "//", "FCE"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1725]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox[
         SubsuperscriptBox["p", "1", "2"],
         TraditionalForm], " ", 
        SubsuperscriptBox["p", 
         FormBox["2",
          TraditionalForm], "\[Mu]"]}], "-", 
       FractionBox[
        RowBox[{"s", " ", 
         SubsuperscriptBox["p", 
          FormBox["1",
           TraditionalForm], "\[Mu]"]}], "2"]}], ")"}], " ", 
     FormBox[
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox["k", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm["[", $CellContext`k^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 1])^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 2])^2, "]"],
          Editable->False], ")"}]}]],
      TraditionalForm]}], 
    RowBox[{
     RowBox[{
      FormBox[
       SubsuperscriptBox["p", "1", "2"],
       TraditionalForm], " ", 
      FormBox[
       SubsuperscriptBox["p", "2", "2"],
       TraditionalForm]}], "-", 
     FractionBox[
      SuperscriptBox["s", "2"], "4"]}]], "+", 
   FractionBox[
    RowBox[{
     FormBox[
      RowBox[{
       FormBox["k",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}],
      TraditionalForm], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        RowBox[{"s", " ", 
         SubsuperscriptBox["p", 
          FormBox["2",
           TraditionalForm], "\[Mu]"]}], "2"], "-", 
       RowBox[{
        FormBox[
         SubsuperscriptBox["p", "2", "2"],
         TraditionalForm], " ", 
        SubsuperscriptBox["p", 
         FormBox["1",
          TraditionalForm], "\[Mu]"]}]}], ")"}], " ", 
     FormBox[
      FractionBox["1", 
       RowBox[{
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox["k", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm["[", $CellContext`k^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "1"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 1])^2, "]"],
          Editable->False], ")"}], " ", 
        RowBox[{"(", 
         InterpretationBox[
          RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"k", "-", 
              SubscriptBox["p", "2"]}], ")"}], "2"], 
           "\[InvisibleSpace]", "\<\"]\"\>"}],
          SequenceForm[
          "[", ($CellContext`k - Subscript[$CellContext`p, 2])^2, "]"],
          Editable->False], ")"}]}]],
      TraditionalForm]}], 
    RowBox[{
     FractionBox[
      SuperscriptBox["s", "2"], "4"], "-", 
     RowBox[{
      FormBox[
       SubsuperscriptBox["p", "1", "2"],
       TraditionalForm], " ", 
      FormBox[
       SubsuperscriptBox["p", "2", "2"],
       TraditionalForm]}]}]]}], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1725]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"k", ",", " ", 
     RowBox[{"k", " ", "-", " ", "p"}]}], "]"}], 
   RowBox[{"FVD", "[", 
    RowBox[{"k", ",", "\[Mu]"}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1726]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   SuperscriptBox[
    FormBox["k",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], 
   RowBox[{
    SuperscriptBox["k", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"k", "-", "p"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"TID",
 CellLabel->"Out[1726]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TID", "[", 
  RowBox[{"%", ",", "k"}], "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1727]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], TraditionalForm]], "Output",\

 CellTags->"TID",
 CellLabel->"Out[1727]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TID", "[", 
  RowBox[{"%%", ",", "k", ",", 
   RowBox[{"FeynAmpDenominatorSimplify", "\[Rule]", "True"}]}], 
  "]"}]], "Input",
 CellTags->"TID",
 CellLabel->"In[1728]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["k",
      TraditionalForm], "\[CenterDot]", 
     FormBox["p",
      TraditionalForm]}]}], 
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], "2"], " ", 
    RowBox[{
     SuperscriptBox["k", "2"], ".", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"k", "-", "p"}], ")"}], "2"]}]}]], TraditionalForm]], "Output",\

 CellTags->"TID",
 CellLabel->"Out[1728]="]
}, Open  ]]
}, Open  ]],

Cell["TIDL", "ObjectName",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TIDL"],

Cell[" TIDL is a database of tensorial reduction formalae.", "Text",
 CellTags->"TIDL"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TID",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TID", All, CellTags],
  ButtonData->"TID",
  ButtonNote->"TID"]
}], "Text",
 CellTags->"TIDL"]
}, Open  ]],

Cell["Examples", "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[TextData[{
 "In any n-dimensional integral ",
 Cell[BoxData[
  FormBox[
   RowBox[{"\[Integral]", 
    RowBox[{
     SuperscriptBox["d", "n"], "\[NegativeVeryThinSpace]", 
     SuperscriptBox["q", "\[Mu]"], " ", 
     RowBox[{"f", "(", 
      RowBox[{"q", ",", "p"}], ")"}], " ", "the", " ", 
     SuperscriptBox["q", "\[Mu]"]}], " "}], TraditionalForm]]],
 "can be replaced by"
}], "Text",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1729]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox[
     FormBox["p",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}]}], 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"]], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1729]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["B", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", "p", "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1730]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], "2"]}]], "-", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["p",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["p",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "D"}], ")"}], " ", 
     SuperscriptBox[
      FormBox[
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], "2"],
       TraditionalForm], "2"]}]]}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1730]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1731]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["2",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", 
       FormBox["1",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm]}]}]}], 
    RowBox[{
     SuperscriptBox[
      FormBox[
       RowBox[{
        FormBox[
         SubscriptBox["p", "1"],
         TraditionalForm], "\[CenterDot]", 
        FormBox[
         SubscriptBox["p", "2"],
         TraditionalForm]}],
       TraditionalForm], "2"], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}]}]]}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1731]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1732]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SubsuperscriptBox["p", "2", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{"D", "-", "2"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        FormBox[
         RowBox[{
          FormBox[
           SubscriptBox["p", "1"],
           TraditionalForm], "\[CenterDot]", 
          FormBox[
           SubscriptBox["p", "2"],
           TraditionalForm]}],
         TraditionalForm], "2"], "-", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", "D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "1", "2"],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "1", "2"],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "1", "2"],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "3"]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["q",
           TraditionalForm], "2"]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "3"]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "-", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}], "+", 
   RowBox[{
    FractionBox["1", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"D", "-", "2"}], ")"}], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          FormBox[
           RowBox[{
            FormBox[
             SubscriptBox["p", "1"],
             TraditionalForm], "\[CenterDot]", 
            FormBox[
             SubscriptBox["p", "2"],
             TraditionalForm]}],
           TraditionalForm], "2"], "-", 
         RowBox[{
          SubsuperscriptBox["p", "1", "2"], " ", 
          SubsuperscriptBox["p", "2", "2"]}]}], ")"}], "2"]}]], 
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Nu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{"2", " ", "D", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"2", " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "+", 
       RowBox[{"D", " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "2", "2"],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "2", "2"],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SuperscriptBox[
         FormBox[
          SubsuperscriptBox["p", "2", "2"],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}]}], ")"}]}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1732]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1733]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["C", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1734]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TIDL", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     SubscriptBox["p", "1"], ",", 
     SubscriptBox["p", "2"], ",", 
     SubscriptBox["p", "3"]}], "}"}]}], "]"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1735]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["3",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "3", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", "3", "2"]}]}]], "-", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["2",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         RowBox[{"(", 
          RowBox[{
           FormBox["q",
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm]}], ")"}]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "1"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "3"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "1", "2"], " ", 
        SubsuperscriptBox["p", "3", "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "3", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", "3", "2"]}]}]], "+", 
   FractionBox[
    RowBox[{
     SubsuperscriptBox["p", 
      FormBox["1",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        SuperscriptBox[
         FormBox[
          RowBox[{
           FormBox[
            SubscriptBox["p", "2"],
            TraditionalForm], "\[CenterDot]", 
           FormBox[
            SubscriptBox["p", "3"],
            TraditionalForm]}],
          TraditionalForm], "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"]}], "+", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "3", "2"]}], "-", 
       RowBox[{
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm]}], " ", 
        SubsuperscriptBox["p", "2", "2"], " ", 
        SubsuperscriptBox["p", "3", "2"]}]}], ")"}]}], 
    RowBox[{
     RowBox[{
      SubsuperscriptBox["p", "3", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "-", 
     RowBox[{"2", " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "3"],
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox[
        SubscriptBox["p", "1"],
        TraditionalForm], "\[CenterDot]", 
       FormBox[
        SubscriptBox["p", "2"],
        TraditionalForm]}]}], "+", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "2"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox[
          SubscriptBox["p", "1"],
          TraditionalForm], "\[CenterDot]", 
         FormBox[
          SubscriptBox["p", "3"],
          TraditionalForm]}],
        TraditionalForm], "2"], " ", 
      SubsuperscriptBox["p", "2", "2"]}], "-", 
     RowBox[{
      SubsuperscriptBox["p", "1", "2"], " ", 
      SubsuperscriptBox["p", "2", "2"], " ", 
      SubsuperscriptBox["p", "3", "2"]}]}]]}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1735]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1736]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]\[Rho]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1737]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell[TextData[Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["D", "\[Mu]\[Nu]\[Rho]\[Sigma]"], "-", "type"}], 
  TraditionalForm]]]], "Subsubsection",
 CellTags->"TIDL"],

Cell[BoxData[
 RowBox[{
  RowBox[{"TIDL", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Mu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Rho]"}], "}"}], ",", 
      RowBox[{"{", 
       RowBox[{"q", ",", "\[Sigma]"}], "}"}]}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"p", ",", "k", ",", "r", ",", "s"}], "}"}]}], "]"}], 
  ";"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1738]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"Length", "[", "%", "]"}], ",", " ", 
   RowBox[{"LeafCount", "[", "%", "]"}], ",", 
   RowBox[{"ByteCount", "[", "%", "]"}]}], "}"}]], "Input",
 CellTags->"TIDL",
 CellLabel->"In[1739]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"4", ",", "29", ",", "744"}], "}"}], TraditionalForm]], "Output",
 CellTags->"TIDL",
 CellLabel->"Out[1739]="]
}, Open  ]]
}, Open  ]],

Cell["TimedIntegrate", "ObjectName",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TimedIntegrate"],

Cell["\<\
 TimedIntegrate[exp, vars] is like Integrate, but stops after the number of \
seconds specified by the option Timing. Options of Integrate can be given and \
are passed on.\
\>", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TimedIntegrate", "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1740]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Timing", "\[Rule]", "10"}], ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], ">", "0"}]}], ",", 
    RowBox[{"Integrate", "\[Rule]", "Integrate"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1740]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TimedIntegrate"],

Cell["This should reach to be done", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimedIntegrate", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"x", "^", "5"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"Timing", "\[Rule]", "1"}]}], "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1741]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "5"}], TraditionalForm]], "Output",
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1741]="]
}, Open  ]],

Cell["This shouldn't", "Text",
 CellTags->"TimedIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimedIntegrate", "[", 
  RowBox[{
   RowBox[{"Log", "[", 
    RowBox[{"Cos", "[", 
     RowBox[{"x", "^", "5"}], "]"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
   RowBox[{"Timing", "\[Rule]", "10"}], ",", 
   RowBox[{"Integrate", "\[Rule]", "int"}]}], "]"}]], "Input",
 CellTags->"TimedIntegrate",
 CellLabel->"In[1742]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"int", "(", 
   RowBox[{
    RowBox[{"log", "(", 
     RowBox[{"cos", "(", 
      SuperscriptBox["x", "5"], ")"}], ")"}], ",", 
    RowBox[{"{", 
     RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
    RowBox[{"Assumptions", "\[Rule]", 
     RowBox[{
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], ">", "0"}]}]}], ")"}], TraditionalForm]], "Output",
 CellTags->"TimedIntegrate",
 CellLabel->"Out[1742]="]
}, Open  ]]
}, Open  ]],

Cell["TLI  ***unfinished***", "ObjectName",
 CellTags->"TLI"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI"],

Cell["\<\
TLI[{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}].  The exponents of the \
numerator scalar product are (dl = OPEDelta):  v: k1.k1, w: k2.k2,  x: p.k1, \
y: p.k2, z: k1.k2.  a: dl.k1, b: dl.k2,  c: dl.(p-k1), d: dl.(p-k2), e: \
dl.(k1-k2).
TLI[{mu1, ...}, {nu1, ...}][{v,w,x,y,z},{a,b,c,d,e}, {al,be,ga,de,ep}] is a \
tensor integral, where mu1 belongs to k1 and nu1 to k2.
TLI[any___,{a,b,c,d,e,0,0}, {al,be,ga,de,ep}] simplifies to  TLI[any, \
{a,b,c,d,e}, {al,be,ga,de,ep}].
TLI[{0,0,0,0,0},{a,b,c,d,e}, {al,be,ga,de,ep}] simplifies to  \
TLI[{a,b,c,d,e}, {al,be,ga,de,ep}].\
\>", "Text",
 CellTags->"TLI"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLI", "]"}]], "Input",
 CellTags->"TLI",
 CellLabel->"In[1743]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"EpsilonOrder", "\[Rule]", "0"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"TLI",
 CellLabel->"Out[1743]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI2FC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TLI2FC", All, CellTags],
  ButtonData->"TLI2FC",
  ButtonNote->"TLI2FC"],
 "."
}], "Text",
 CellTags->"TLI"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLI"],

Cell[BoxData[""], "Input",
 CellTags->"TLI",
 CellLabel->"In[1744]:="]
}, Open  ]],

Cell["TLIFP  ***unfinished***", "ObjectName",
 CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLIFP"],

Cell["TLIFP[exp] does Feynman-Parametrizations of TLI's in exp.", "Text",
 CellTags->"TLIFP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLIFP", "]"}]], "Input",
 CellTags->"TLIFP",
 CellLabel->"In[1745]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"x", ",", "t", ",", "u", ",", "s", ",", "y"}], "}"}]}], ",", 
    RowBox[{"GammaExpand", "\[Rule]", "True"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}], ",", 
    RowBox[{"Print", "\[Rule]", "True"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"TLIFP",
 CellLabel->"Out[1745]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TLI", All, CellTags],
  ButtonData->"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLIFP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLIFP"],

Cell[BoxData[""], "Input",
 CellTags->"TLIFP",
 CellLabel->"In[1746]:="]
}, Open  ]],

Cell["TLIHYP  ***unfinished***", "ObjectName",
 CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLIHYP"],

Cell["\<\
TLIHYP[exp] expresses TLI's in exp. in terms of hypergeometric functions, \
where possible.\
\>", "Text",
 CellTags->"TLIHYP"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLIHYP", "]"}]], "Input",
 CellTags->"TLIHYP",
 CellLabel->"In[1747]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", "x"}], ",", 
    RowBox[{"Momentum", "\[Rule]", "p"}]}], "}"}], TraditionalForm]], "Output",\

 CellTags->"TLIHYP",
 CellLabel->"Out[1747]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TLI", All, CellTags],
  ButtonData->"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLIHYP"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLIHYP"],

Cell[BoxData[""], "Input",
 CellTags->"TLIHYP",
 CellLabel->"In[1748]:="]
}, Open  ]],

Cell["TLI2", "ObjectName",
 CellTags->"TLI2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI2"],

Cell["like TLI, but without any functional properties.", "Text",
 CellTags->"TLI2"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TLI", All, CellTags],
  ButtonData->"TLI",
  ButtonNote->"TLI"],
 "."
}], "Text",
 CellTags->"TLI2"]
}, Open  ]],

Cell["TLI2FC  ***unfinished***", "ObjectName",
 CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TLI2FC"],

Cell["\<\
TLI2FC[exp] transforms all TLI-integrals in exp to the FAD form.\
\>", "Text",
 CellTags->"TLI2FC"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TLI2FC", "]"}]], "Input",
 CellTags->"TLI2FC",
 CellLabel->"In[1749]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"Momentum", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"q1", ",", "q2", ",", "p"}], "}"}]}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TLI2FC",
 CellLabel->"Out[1749]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["TLI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TLI", All, CellTags],
  ButtonData->"TLI",
  ButtonNote->"TLI"],
 ".  ",
 ButtonBox["FC2TLI",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FC2TLI", All, CellTags],
  ButtonData->"FC2TLI",
  ButtonNote->"FC2TLI"],
 "."
}], "Text",
 CellTags->"TLI2FC"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TLI2FC"],

Cell[BoxData[""], "Input",
 CellTags->"TLI2FC",
 CellLabel->"In[1750]:="]
}, Open  ]],

Cell["ToDistribution", "ObjectName",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToDistribution"],

Cell["\<\
ToDistribution[exp,x] replaces (1-x)^(a Epsilon - 1) in exp by 1/(a Epsilon) \
DeltaFunction[1-x] + 1/(1-x) + a Epsilon Log[1-x]/(1-x) + 1/2 a^2 Epsilon^2 \
Log[1-x]^2/(1-x)] and (1-x)^(a Epsilon - 2) in exp by -1/(a Epsilon) \
DeltaFunctionPrime[1-x] + 1/(1-x)^2 + (a Epsilon) Log[1-x]/(1-x)^2 + a^2 \
Epsilon^2/2 Log[1-x]^2/(1-x)^2 + a^3 Epsilon^3/6 Log[1-x]^3/(1-x)^2.\
\>", "Text",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "ToDistribution", "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1751]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1751]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PlusDistribution",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PlusDistribution", All, CellTags],
  ButtonData->"PlusDistribution",
  ButtonNote->"PlusDistribution"],
 "."
}], "Text",
 CellTags->"ToDistribution"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToDistribution"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"ToDistribution", ",", 
   RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}]}], 
  "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1752]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "PlusDistribution"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1752]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToDistribution", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"Epsilon", "-", "1"}], ")"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1753]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox[
    RowBox[{"\[Delta]", "(", 
     FormBox[
      RowBox[{"1", "-", "x"}],
      TraditionalForm], ")"}], 
    TagBox["\[CurlyEpsilon]",
     TraditionalForm]], "+", 
   RowBox[{
    FractionBox["1", "6"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox["log", "3"], "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
   RowBox[{
    FractionBox["1", "2"], " ", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "2"], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{
        SuperscriptBox["log", "2"], "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
   RowBox[{
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], " ", 
    SubscriptBox[
     RowBox[{"(", 
      FractionBox[
       RowBox[{"log", "(", 
        RowBox[{"1", "-", "x"}], ")"}], 
       RowBox[{"1", "-", "x"}]], ")"}], "+"]}], "+", 
   SubscriptBox[
    RowBox[{"(", 
     FractionBox["1", 
      RowBox[{"1", "-", "x"}]], ")"}], "+"]}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1753]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"ToDistribution", ",", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}]}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1754]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"PlusDistribution", "\[Rule]", "Identity"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1754]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToDistribution", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{"1", "-", "x"}], ")"}], "^", 
    RowBox[{"(", 
     RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1755]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox[
     RowBox[{
      SuperscriptBox["\[Delta]", "\[Prime]"], "(", 
      FormBox[
       RowBox[{"1", "-", "x"}],
       TraditionalForm], ")"}], 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]]}], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "3"], " ", 
     RowBox[{
      SuperscriptBox["log", "3"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"6", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     SuperscriptBox[
      TagBox["\[CurlyEpsilon]",
       TraditionalForm], "2"], " ", 
     RowBox[{
      SuperscriptBox["log", "2"], "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    RowBox[{"2", " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "2"]}]], "+", 
   FractionBox[
    RowBox[{
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], " ", 
     RowBox[{"log", "(", 
      RowBox[{"1", "-", "x"}], ")"}]}], 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "2"]], "+", 
   FractionBox["1", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "x"}], ")"}], "2"]]}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1755]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Series2", "[", 
  RowBox[{
   RowBox[{"Integrate", "[", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], 
      RowBox[{"Epsilon", "-", "2"}]], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", "0", ",", "1"}], "}"}], ",", 
     RowBox[{"GenerateConditions", "\[Rule]", "False"}]}], "]"}], ",", 
   "Epsilon", ",", "3"}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1756]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "2"], "-", 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1756]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "@", "Integrate"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1757]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Assumptions", "\[RuleDelayed]", "$Assumptions"}], ",", 
    RowBox[{"GenerateConditions", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"PrincipalValue", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1757]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Integrate2", "[", 
  RowBox[{
   RowBox[{"ToDistribution", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       RowBox[{"1", "-", "x"}], ")"}], "^", 
      RowBox[{"(", 
       RowBox[{"Epsilon", "-", "2"}], ")"}]}], ",", "x"}], "]"}], ",", 
   RowBox[{"{", 
    RowBox[{"x", ",", "0", ",", "1"}], "}"}]}], "]"}]], "Input",
 CellTags->"ToDistribution",
 CellLabel->"In[1758]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     TagBox["\[CurlyEpsilon]",
      TraditionalForm], "3"]}], "-", 
   SuperscriptBox[
    TagBox["\[CurlyEpsilon]",
     TraditionalForm], "2"], "-", 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm], "-", "1"}], TraditionalForm]], "Output",
 CellTags->"ToDistribution",
 CellLabel->"Out[1758]="]
}, Open  ]]
}, Open  ]],

Cell["ToHypergeometric", "ObjectName",
 CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToHypergeometric"],

Cell[TextData[{
 "ToHypergeometric[",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox[
     StyleBox["t",
      FontSlant->"Plain"], 
     StyleBox["b",
      FontSlant->"Plain"]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       StyleBox["t",
        FontSlant->"Plain"]}], ")"}], 
     StyleBox["c",
      FontSlant->"Plain"]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{
        StyleBox["t",
         FontSlant->"Plain"], " ", 
        StyleBox["z",
         FontSlant->"Plain"]}]}], ")"}], 
     StyleBox["a",
      FontSlant->"Plain"]]}], TraditionalForm]],
  FontVariations->{"CompatibilityType"->0}],
 ", ",
 "t",
 "] returns ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{"\[CapitalGamma]", "(", 
     RowBox[{"b", "+", "1"}], ")"}], " ", 
    RowBox[{
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"c", "+", "1"}], ")"}], "/", 
     RowBox[{"\[CapitalGamma]", "(", 
      RowBox[{"b", "+", "c", "+", "2"}], ")"}]}], " ", 
    FormBox[
     RowBox[{
      SubscriptBox["\[Null]", "2"], 
      RowBox[{
       SubscriptBox["F", "1"], "(", 
       RowBox[{
        RowBox[{
         RowBox[{"-", "a"}], ",", 
         RowBox[{"b", "+", "1"}]}], ";", 
        RowBox[{"b", "+", "c", "+", "2"}], ";", 
        RowBox[{"-", "z"}]}], ")"}]}],
     TraditionalForm]}], TraditionalForm]]],
 ". Remember that Re ",
 StyleBox["b",
  FontSlant->"Italic"],
 " >0 and Re (",
 StyleBox["c",
  FontSlant->"Italic"],
 "-",
 StyleBox["b",
  FontSlant->"Italic"],
 ") > 0 should hold (need not be set in ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 ")."
}], "Text",
 CellTags->"ToHypergeometric"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["HypergeometricAC",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricAC", All, CellTags],
  ButtonData->"HypergeometricAC",
  ButtonNote->"HypergeometricAC"],
 ", ",
 ButtonBox["HypergeometricIR",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricIR", All, CellTags],
  ButtonData->"HypergeometricIR",
  ButtonNote->"HypergeometricIR"],
 ", ",
 ButtonBox["HypergeometricSE",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "HypergeometricSE", All, CellTags],
  ButtonData->"HypergeometricSE",
  ButtonNote->"HypergeometricSE"],
 "."
}], "Text",
 CellTags->"ToHypergeometric"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToHypergeometric"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["t", "b"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1759]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    TemplateBox[{RowBox[{"b", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{
     RowBox[{"-", "a"}],RowBox[{"b", "+", "1"}],
      RowBox[{"b", "+", "c", "+", "2"}],RowBox[{"-", "z"}]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{RowBox[{"b", "+", "c", "+", "2"}]},
    "Gamma"]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1759]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{"w", " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{"c", "-", "b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1760]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"w", " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"], " ", 
    TemplateBox[{"a","b","c","z"},
     "Hypergeometric2F1"]}], 
   TemplateBox[{"c"},
    "Gamma"]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1760]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{
    SuperscriptBox["t", "b"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "c"], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "+", 
       RowBox[{"t", " ", "z"}]}], ")"}], "a"]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1761]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    SuperscriptBox["u", "a"], " ", 
    TemplateBox[{RowBox[{"b", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "+", "1"}]},
     "Gamma"], " ", 
    TemplateBox[{
     RowBox[{"-", "a"}],RowBox[{"b", "+", "1"}],
      RowBox[{"b", "+", "c", "+", "2"}],RowBox[{"-", 
        FractionBox["z", "u"]}]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{RowBox[{"b", "+", "c", "+", "2"}]},
    "Gamma"]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1761]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToHypergeometric", "[", 
  RowBox[{
   RowBox[{"w", " ", 
    SuperscriptBox["t", 
     RowBox[{"b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], 
     RowBox[{"c", "-", "b", "-", "1"}]], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"u", "-", 
       RowBox[{"t", " ", "z"}]}], ")"}], 
     RowBox[{"-", "a"}]]}], ",", "t"}], "]"}]], "Input",
 CellTags->"ToHypergeometric",
 CellLabel->"In[1762]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"w", " ", 
    SuperscriptBox["u", 
     RowBox[{"-", "a"}]], " ", 
    TemplateBox[{"b"},
     "Gamma"], " ", 
    TemplateBox[{RowBox[{"c", "-", "b"}]},
     "Gamma"], " ", 
    TemplateBox[{"a","b","c",FractionBox["z", "u"]},
     "Hypergeometric2F1"]}], 
   TemplateBox[{"c"},
    "Gamma"]], TraditionalForm]], "Output",
 CellTags->"ToHypergeometric",
 CellLabel->"Out[1762]="]
}, Open  ]]
}, Open  ]],

Cell["ToLarin", "ObjectName",
 CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToLarin"],

Cell[TextData[{
 "ToLarin[exp] translates all ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
    SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]]],
 " in exp into ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     RowBox[{"-", "\[ImaginaryI]"}], "/", "6"}], " ", 
    SuperscriptBox["\[CurlyEpsilon]", "\[Mu]\[Nu]\[Lambda]\[Sigma]"], 
    SuperscriptBox["\[Gamma]", "\[Nu]"], " ", 
    SuperscriptBox["\[Gamma]", "\[Lambda]"], " ", 
    RowBox[{
     SuperscriptBox["\[Gamma]", "\[Sigma]"], "."}]}], TraditionalForm]]]
}], "Text",
 CellTags->"ToLarin"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Eps",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Eps", All, CellTags],
  ButtonData->"Eps",
  ButtonNote->"Eps"],
 "."
}], "Text",
 CellTags->"ToLarin"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToLarin"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracMatrix", "[", "\[Nu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "\[Mu]", "]"}], ".", 
  RowBox[{"DiracMatrix", "[", "5", "]"}]}]], "Input",
 CellTags->"ToLarin",
 CellLabel->"In[1763]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], ".", 
   SuperscriptBox["\[Gamma]", "5"]}], TraditionalForm]], "Output",
 CellTags->"ToLarin",
 CellLabel->"Out[1763]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToLarin", "[", "%", "]"}]], "Input",
 CellTags->"ToLarin",
 CellLabel->"In[1764]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "6"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du232",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du233",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["du234",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du232",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du233",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["du234",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"ToLarin",
 CellLabel->"Out[1764]="]
}, Open  ]]
}, Open  ]],

Cell["ToTFi", "ObjectName",
 CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ToTFi"],

Cell["\<\
ToTFi[expr, q1, q2, p] translates FeynCalc 2-loop self energy type integrals \
into the TFi notatation, which can be used as input for the function \
TarcerRecurse from the TARCER package. See TFi for details on the conventions.\
\>", "Text",
 CellTags->"ToTFi"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TFi",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "TFi", All, CellTags],
  ButtonData->"TFi",
  ButtonNote->"TFi"],
 ", ",
 ButtonBox["FromTFi",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FromTFi", All, CellTags],
  ButtonData->"FromTFi",
  ButtonNote->"FromTFi"],
 "."
}], "Text",
 CellTags->"ToTFi"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"ToTFi"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FAD", "[", 
  RowBox[{"q1", ",", 
   RowBox[{"q1", "-", "p"}], ",", 
   RowBox[{"{", 
    RowBox[{"q2", ",", "M"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{
     RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
   RowBox[{"q1", "-", "q2"}]}], "]"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1765]:="],

Cell[BoxData[
 FormBox[
  FractionBox["1", 
   RowBox[{
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q1^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", (-$CellContext`p + $CellContext`q1)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q1", "-", "q2"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", ($CellContext`q1 - $CellContext`q2)^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["q2", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
       "\[InvisibleSpace]", 
       SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm["[", $CellContext`q2^2, "-", $CellContext`M^2, "]"],
      Editable->False], ")"}], " ", 
    RowBox[{"(", 
     InterpretationBox[
      RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"q2", "-", "p"}], ")"}], "2"], 
       "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
       SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
      SequenceForm[
      "[", (-$CellContext`p + $CellContext`q2)^2, "-", $CellContext`m^2, "]"],
      
      Editable->False], ")"}]}]], TraditionalForm]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1765]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ToTFi", "[", 
  RowBox[{"%", ",", "q1", ",", "q2", ",", "p"}], "]"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1766]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"ToTFi", "(", 
   RowBox[{
    FormBox[
     FractionBox["1", 
      RowBox[{
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q1", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q1^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q1", "-", "p"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", (-$CellContext`p + $CellContext`q1)^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q1", "-", "q2"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", ($CellContext`q1 - $CellContext`q2)^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["q2", "2"], "\[InvisibleSpace]", "\<\"-\"\>", 
          "\[InvisibleSpace]", 
          SuperscriptBox["M", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm["[", $CellContext`q2^2, "-", $CellContext`M^2, "]"],
         Editable->False], ")"}], " ", 
       RowBox[{"(", 
        InterpretationBox[
         RowBox[{"\<\"[\"\>", "\[InvisibleSpace]", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"q2", "-", "p"}], ")"}], "2"], 
          "\[InvisibleSpace]", "\<\"-\"\>", "\[InvisibleSpace]", 
          SuperscriptBox["m", "2"], "\[InvisibleSpace]", "\<\"]\"\>"}],
         SequenceForm[
         "[", (-$CellContext`p + $CellContext`q2)^2, "-", $CellContext`m^2, 
          "]"],
         Editable->False], ")"}]}]],
     TraditionalForm], ",", "q1", ",", "q2", ",", "p"}], ")"}], 
  TraditionalForm]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1766]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1767]:="],

Cell[BoxData[
 RowBox[{"ToTFi", "[", 
  RowBox[{
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{
      RowBox[{"-", "p"}], "+", "q1"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"-", "p"}], "+", "q2"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}], ",", "q1", ",", "q2", ",", "p"}], 
  "]"}]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1767]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"SOD", "[", "q1", "]"}], " ", 
   RowBox[{"SOD", "[", "q2", "]"}], 
   RowBox[{"FAD", "[", 
    RowBox[{"q1", ",", 
     RowBox[{"q1", "-", "p"}], ",", 
     RowBox[{"{", 
      RowBox[{"q2", ",", "M"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"q2", "-", "p"}], ",", "m"}], "}"}], ",", 
     RowBox[{"q1", "-", "q2"}]}], "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"ToTFi",
 CellLabel->"In[1768]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q1",
      TraditionalForm]}], " ", 
    RowBox[{
     FormBox["\[CapitalDelta]",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q2",
      TraditionalForm]}]}], 
   RowBox[{
    SuperscriptBox["q1", "2"], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "p"}], ")"}], "2"], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["q2", "2"], "-", 
      SuperscriptBox["M", "2"]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"q2", "-", "p"}], ")"}], "2"], "-", 
      SuperscriptBox["m", "2"]}], ")"}], ".", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"q1", "-", "q2"}], ")"}], "2"]}]], TraditionalForm]], "Output",
 CellTags->"ToTFi",
 CellLabel->"Out[1768]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["TR", "ObjectName",
 CellTags->"TR"],

Cell[BoxData[
 FormBox["TR", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1959]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TR"],

Cell[TextData[{
 "TR[exp] calculates the Dirac trace of exp. Depending on the setting of the \
option SUNTrace also a trace over SU(",
 StyleBox["N",
  FontSlant->"Italic"],
 ") objects is performed. The ",
 StyleBox["Mathematica",
  FontSlant->"Italic"],
 " build-in function Tr is overloaded to call TR if ",
 "any of DiracGamma, DiracSlash, GA, GAD, GS or GSD",
 " are in the expression."
}], "Text",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "TR", "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1769]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"DiracTraceEvaluate", "\[Rule]", "True"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Factoring", "\[Rule]", "False"}], ",", 
    RowBox[{"FeynCalcExternal", "\[Rule]", "False"}], ",", 
    RowBox[{"LeviCivitaSign", "\[RuleDelayed]", "$LeviCivitaSign"}], ",", 
    RowBox[{"Mandelstam", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"PairCollect", "\[Rule]", "False"}], ",", 
    RowBox[{"Schouten", "\[Rule]", "442"}], ",", 
    RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
    RowBox[{"TraceOfOne", "\[Rule]", "4"}], ",", 
    RowBox[{"SUNNToCACF", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1769]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["FermionSpinSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FermionSpinSum", All, CellTags],
  ButtonData->"FermionSpinSum",
  ButtonNote->"FermionSpinSum"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNTrace", All, CellTags],
  ButtonData->"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"TR"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TR"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", 
  RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1770]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1770]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", "%", "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1771]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1771]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"(", 
    RowBox[{
     RowBox[{"GSD", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
   RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"GSD", "[", "q", "]"}], "-", "m"}], ")"}], ".", 
   RowBox[{"GAD", "[", "\[Nu]", "]"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1772]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["m", "2"], " ", 
      RowBox[{"(", 
       RowBox[{"-", 
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]]}], ")"}]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1772]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GA", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "5"}], 
   "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1773]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1773]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GAD", "[", 
   RowBox[{
   "\[Mu]", ",", "\[Nu]", ",", "\[Rho]", ",", "\[Sigma]", ",", "\[Tau]", ",", 
    "\[Xi]", ",", "5"}], "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1774]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Xi]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Xi]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Nu]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Rho]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Xi]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Sigma]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Tau]",
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Rho]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Sigma]", "\[Tau]"}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Xi]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1774]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"GAD", "[", 
    RowBox[{
     SubscriptBox["\[Nu]", "1"], ",", 
     SubscriptBox["\[Nu]", "2"], ",", 
     SubscriptBox["\[Nu]", "3"], ",", 
     SubscriptBox["\[Nu]", "4"], ",", 
     SubscriptBox["\[Nu]", "5"], ",", 
     SubscriptBox["\[Nu]", "6"]}], "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1775]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        SubscriptBox["\[Nu]", "2"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        SubscriptBox["\[Nu]", "3"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        SubscriptBox["\[Nu]", "4"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        SubscriptBox["\[Nu]", "5"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "1"], 
        SubscriptBox["\[Nu]", "6"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        SubscriptBox["\[Nu]", "3"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        SubscriptBox["\[Nu]", "4"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        SubscriptBox["\[Nu]", "5"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "2"], 
        SubscriptBox["\[Nu]", "6"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        SubscriptBox["\[Nu]", "4"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        SubscriptBox["\[Nu]", "5"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "3"], 
        SubscriptBox["\[Nu]", "6"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], 
        SubscriptBox["\[Nu]", "5"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "6"],
          TraditionalForm],
         TraditionalForm]}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "4"], 
        SubscriptBox["\[Nu]", "6"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "5"],
          TraditionalForm],
         TraditionalForm]}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{
        SubscriptBox["\[Nu]", "5"], 
        SubscriptBox["\[Nu]", "6"]}]], " ", 
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "1"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "2"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "3"],
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox[
          SubscriptBox["\[Nu]", "4"],
          TraditionalForm],
         TraditionalForm]}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1775]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"GS", "[", 
   RowBox[{"p", ",", "q", ",", "r", ",", "s"}], "]"}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1776]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["q",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}]}], "+", 
     RowBox[{
      RowBox[{
       FormBox["p",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], " ", 
      RowBox[{
       FormBox["r",
        TraditionalForm], "\[CenterDot]", 
       FormBox["s",
        TraditionalForm]}]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1776]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{"GS", "[", "p", "]"}], "+", "m"}], ")"}], ".", 
    RowBox[{"GA", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"GS", "[", "q", "]"}], "+", "m"}], ")"}], ".", 
    RowBox[{"GA", "[", "\[Mu]", "]"}]}], ",", 
   RowBox[{"Factoring", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1777]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"8", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"2", " ", 
      SuperscriptBox["m", "2"]}], "-", 
     RowBox[{
      FormBox["p",
       TraditionalForm], "\[CenterDot]", 
      FormBox["q",
       TraditionalForm]}]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1777]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1778]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1778]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1779]:="],

Cell[BoxData[
 RowBox[{"4", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1779]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"GAD", "[", "\[Mu]", "]"}], ".", 
    RowBox[{"GA", "[", "\[Alpha]", "]"}], ".", 
    RowBox[{"GA", "[", "\[Beta]", "]"}], ".", 
    RowBox[{"GAD", "[", "\[Mu]", "]"}]}], ",", 
   RowBox[{"FCE", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1780]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "D", " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Alpha]", "\[Beta]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1780]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1781]:="],

Cell[BoxData[
 RowBox[{"4", " ", "D", " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}]}]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1781]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{
   RowBox[{"GA", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
   RowBox[{
    RowBox[{"SUNT", "[", "b", "]"}], ".", 
    RowBox[{"SUNT", "[", "c", "]"}]}], " ", 
   RowBox[{"SUNDelta", "[", 
    RowBox[{"c", ",", "b"}], "]"}]}]}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1782]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["\[Delta]", 
    RowBox[{"b", "c"}]], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox["b",
      TraditionalForm]], ".", 
    SubscriptBox["T", 
     FormBox["c",
      TraditionalForm]]}], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
    SuperscriptBox["\[Gamma]", "\[Nu]"]}]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1782]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"t", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1783]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SubscriptBox["C", "F"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1783]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"t", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1784]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SubscriptBox["C", "F"], " ", 
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]]}], TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1784]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1785]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1785]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1786]:="],

Cell[BoxData[
 FormBox["4", TraditionalForm]], "Output",
 CellTags->"TR",
 CellLabel->"Out[1786]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tr", "[", " ", 
  RowBox[{"GA", "[", 
   RowBox[{"m", ",", "n"}], "]"}], "]"}]], "Input",
 CellLabel->"In[1787]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   SuperscriptBox["g", 
    RowBox[{"m", "n"}]]}], TraditionalForm]], "Output",
 CellLabel->"Out[1787]="]
}, Open  ]],

Cell["Tr has been overloaded to use TR:", "Text"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", "Tr"}]], "Input",
 CellLabel->"In[1788]:="],

Cell[BoxData[
 RowBox[{
  StyleBox["\<\"FeynCalc extension: Tr[list] finds the trace of the matrix or \
tensor list. Tr[list, f] finds a\\n   generalized trace, combining terms with \
f instead of Plus. Tr[list, f, n]\\n   goes down to level n in list. \\n \
\\n\\nTr[ expression ] calculates the DiracTrace, i.e.,  TR[ expression \
],\\nif any of DiracGamma, DiracSlash, GA, GAD, GS or GSD are present in \
expression.\"\>", "MSG"], "\[NonBreakingSpace]", 
  ButtonBox[
   StyleBox["\[RightSkeleton]", "SR"],
   Active->True,
   BaseStyle->"Link",
   ButtonData->"paclet:ref/Tr"]}]], "Print", "PrintUsage",
 CellTags->"Info3535453335-5096475"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"T", "[", "n_", "]"}], ":=", 
   RowBox[{"Block", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"gamma", ",", "calc"}], "}"}], ",", 
     RowBox[{
      RowBox[{"gamma", "=", 
       RowBox[{"Dot", "@@", 
        RowBox[{"Table", "[", 
         RowBox[{
          RowBox[{"DiracMatrix", "[", 
           RowBox[{
            RowBox[{"a", "[", "i", "]"}], ",", 
            RowBox[{"Dimension", "\[Rule]", 
             RowBox[{"(", 
              RowBox[{"d", "-", "4"}], ")"}]}]}], "]"}], ",", 
          RowBox[{"{", 
           RowBox[{"i", ",", "1", ",", "n"}], "}"}]}], "]"}]}]}], ";", 
      "\[IndentingNewLine]", 
      RowBox[{"calc", "=", 
       RowBox[{"Timing", "[", 
        RowBox[{
         RowBox[{"Tr", "[", 
          RowBox[{"gamma", ".", "gamma"}], "]"}], "//", "Factor"}], "]"}]}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"Print", "[", 
       RowBox[{"\"\<Time=\>\"", ",", 
        RowBox[{
         RowBox[{"calc", "[", 
          RowBox[{"[", "1", "]"}], "]"}], "//", "FeynCalcForm"}]}], "]"}], 
      ";", "\[IndentingNewLine]", 
      RowBox[{"calc", "[", 
       RowBox[{"[", "2", "]"}], "]"}]}]}], "]"}]}], ";"}]], "Input",
 CellLabel->"In[1789]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"T", "[", "3", "]"}]], "Input",
 CellLabel->"In[1790]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Time=\"\>", "\[InvisibleSpace]", "0.02800200000000075`"}],
   SequenceForm["Time=", 0.02800200000000075],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", 
   RowBox[{"(", 
    RowBox[{"d", "-", "4"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["d", "2"], "-", 
     RowBox[{"14", " ", "d"}], "+", "44"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[1790]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"T", "[", "8", "]"}]], "Input",
 CellLabel->"In[1791]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Time=\"\>", "\[InvisibleSpace]", "0.02800200000000075`"}],
   SequenceForm["Time=", 0.02800200000000075],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{"d", "-", "6"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"d", "-", "4"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["d", "6"], "-", 
     RowBox[{"78", " ", 
      SuperscriptBox["d", "5"]}], "+", 
     RowBox[{"2276", " ", 
      SuperscriptBox["d", "4"]}], "-", 
     RowBox[{"32264", " ", 
      SuperscriptBox["d", "3"]}], "+", 
     RowBox[{"238176", " ", 
      SuperscriptBox["d", "2"]}], "-", 
     RowBox[{"880384", " ", "d"}], "+", "1286144"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[1791]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"T", "[", "10", "]"}]], "Input",
 CellLabel->"In[1792]:="],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"Time=\"\>", "\[InvisibleSpace]", "0.2840180000000032`"}],
   SequenceForm["Time=", 0.2840180000000032],
   Editable->False], TraditionalForm]], "Print"],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "4"}], " ", 
   RowBox[{"(", 
    RowBox[{"d", "-", "6"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{"d", "-", "4"}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox["d", "8"], "-", 
     RowBox[{"120", " ", 
      SuperscriptBox["d", "7"]}], "+", 
     RowBox[{"5736", " ", 
      SuperscriptBox["d", "6"]}], "-", 
     RowBox[{"144000", " ", 
      SuperscriptBox["d", "5"]}], "+", 
     RowBox[{"2099856", " ", 
      SuperscriptBox["d", "4"]}], "-", 
     RowBox[{"18411648", " ", 
      SuperscriptBox["d", "3"]}], "+", 
     RowBox[{"95670016", " ", 
      SuperscriptBox["d", "2"]}], "-", 
     RowBox[{"271300608", " ", "d"}], "+", "323158016"}], ")"}]}], 
  TraditionalForm]], "Output",
 CellLabel->"Out[1792]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t", ",", "T"}], "]"}], ";"}]], "Input",
 CellTags->"TR",
 CellLabel->"In[1793]:="]
}, Open  ]],

Cell["TraceDimension", "ObjectName",
 CellTags->"TraceDimension"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TraceDimension"],

Cell["\<\
TraceDimension is an option for FeynCalc2FORM. If set to 4: trace, if set to \
n: tracen.\
\>", "Text",
 CellTags->"TraceDimension"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynCalc2FORM",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynCalc2FORM", All, CellTags],
  ButtonData->"FeynCalc2FORM",
  ButtonNote->"FeynCalc2FORM"],
 "."
}], "Text",
 CellTags->"TraceDimension"]
}, Open  ]],

Cell["TraceOfOne", "ObjectName",
 CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TraceOfOne"],

Cell["\<\
TraceOfOne is an option for Tr and DiracTrace. Its setting determines the \
value of the unit trace.\
\>", "Text",
 CellTags->"TraceOfOne"],

Cell[TextData[{
 "See also: ",
 ButtonBox["TR",
  BaseStyle->"AddOnsLink",
  ButtonData:>"TR",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 "."
}], "Text",
 CellTags->"TraceOfOne"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TraceOfOne"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "False"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
   RowBox[{"TraceOfOne", "\[Rule]", "tr1"}]}], "]"}]], "Input",
 CellTags->"TraceOfOne",
 CellLabel->"In[1794]:="],

Cell[BoxData[
 FormBox["tr1", TraditionalForm]], "Output",
 CellTags->"TraceOfOne",
 CellLabel->"Out[1794]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{"1", ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}], ",", 
   RowBox[{"SUNNToCACF", "\[Rule]", "True"}], ",", 
   RowBox[{"TraceOfOne", "\[Rule]", "tr2"}]}], "]"}]], "Input",
 CellTags->"TraceOfOne",
 CellLabel->"In[1795]:="],

Cell[BoxData[
 FormBox["tr2", TraditionalForm]], "Output",
 CellTags->"TraceOfOne",
 CellLabel->"Out[1795]="]
}, Open  ]]
}, Open  ]],

Cell["Trick", "ObjectName",
 CellTags->"Trick"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Trick"],

Cell["\<\
Trick[exp] performs several basic simplifications without expansion. \
Trick[exp] uses Contract, DotSimplify and SUNDeltaContract.\
\>", "Text",
 CellTags->"Trick"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Calc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Calc", All, CellTags],
  ButtonData->"Calc",
  ButtonNote->"Calc"],
 ", ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 ", ",
 ButtonBox["DotSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DotSimplify", All, CellTags],
  ButtonData->"DotSimplify",
  ButtonNote->"DotSimplify"],
 ", ",
 ButtonBox["DiracTrick",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrick", All, CellTags],
  ButtonData->"DiracTrick",
  ButtonNote->"DiracTrick"],
 "."
}], "Text",
 CellTags->"Trick"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData["Examples"], "Subsubsection",
 CellTags->"Trick"],

Cell[TextData[{
 "This calculates ",
 Cell[BoxData[
  FormBox[
   RowBox[{
    SuperscriptBox["g", 
     RowBox[{"\[Mu]", " ", "\[Nu]"}]], 
    SubscriptBox["\[Gamma]", "\[Mu]"]}], TraditionalForm]]],
 "and ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["g", "\[Nu]", "\[Nu]"], TraditionalForm]]],
 " in D dimensions."
}], "Text",
 ZeroWidthTimes->True,
 CellTags->"Trick"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{"{", 
   RowBox[{
    RowBox[{
     RowBox[{"GA", "[", "\[Mu]", "]"}], " ", 
     RowBox[{"MT", "[", 
      RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}]}], ",", " ", 
    RowBox[{"MTD", "[", 
     RowBox[{"\[Nu]", ",", "\[Nu]"}], "]"}]}], "}"}], "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1796]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], ",", "D"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1796]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"p", "+", "r"}], ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"MT", "[", 
   RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], " ", 
  RowBox[{"FV", "[", 
   RowBox[{
    RowBox[{"q", "-", "p"}], ",", "\[Nu]"}], "]"}]}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1797]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"q", "-", "p"}],
      TraditionalForm], "\[Nu]"], ")"}], " ", 
   RowBox[{"(", 
    SuperscriptBox[
     FormBox[
      RowBox[{"p", "+", "r"}],
      TraditionalForm], "\[Mu]"], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1797]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", "%", "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1798]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["q",
     TraditionalForm]}], "-", 
   RowBox[{
    FormBox["p",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}], "-", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], "2"], "+", 
   RowBox[{
    FormBox["q",
     TraditionalForm], "\[CenterDot]", 
    FormBox["r",
     TraditionalForm]}]}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1798]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Trick", "[", 
  RowBox[{"c", ".", "b", ".", "a", " ", ".", " ", 
   RowBox[{"GA", "[", "d", "]"}], ".", 
   RowBox[{"GA", "[", "e", "]"}]}], "]"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1799]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"a", " ", "b", " ", "c", " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["d",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", 
     FormBox[
      FormBox["e",
       TraditionalForm],
      TraditionalForm]]}]}], TraditionalForm]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1799]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"%", "//", "FCE"}], "//", "StandardForm"}]], "Input",
 CellTags->"Trick",
 CellLabel->"In[1800]:="],

Cell[BoxData[
 RowBox[{"a", " ", "b", " ", "c", " ", 
  RowBox[{
   RowBox[{"GA", "[", "d", "]"}], ".", 
   RowBox[{"GA", "[", "e", "]"}]}]}]], "Output",
 CellTags->"Trick",
 CellLabel->"Out[1800]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell["TrickIntegrate", "ObjectName",
 CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TrickIntegrate"],

Cell["\<\
 TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] does an integration trick for \
the definite integral of ((1-t)^(a Epsilon -1) g[t])  from 0 to 1. \
TrickIntegrate[(1-t)^(a Epsilon -1) g[t], t] gives  g[1]/a/Epsilon + \
Hold[Integrate][(1-t)^(a Epsilon-1) (g[t]-g[1]),{t,0,1}]. TrickIntegrate[t^(a \
Epsilon -1) f[t], t] gives f[0]/a/Epsilon + Hold[Integrate][t^(a Epsilon-1) \
(f[t]-f[0]),{t,0,1}], provided g[1] and f[0] do exist.\
\>", "Text",
 CellTags->"TrickIntegrate"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TrickIntegrate"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrickIntegrate", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"1", "-", "t"}], ")"}], "^", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"a", " ", "Epsilon"}], "-", "1"}], ")"}]}], " ", 
    RowBox[{"g", "[", "t", "]"}]}], ",", "t"}], "]"}]], "Input",
 CellTags->"TrickIntegrate",
 CellLabel->"In[1801]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"Hold", "[", "Integrate", "]"}], "[", 
   RowBox[{
    RowBox[{
     RowBox[{"g", "(", "t", ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"1", "-", "t"}], ")"}], 
      RowBox[{
       RowBox[{"a", " ", 
        TagBox["\[CurlyEpsilon]",
         TraditionalForm]}], "-", "1"}]]}], ",", 
    RowBox[{"{", 
     RowBox[{"t", ",", "0", ",", "1"}], "}"}]}], "]"}], 
  TraditionalForm]], "Output",
 CellTags->"TrickIntegrate",
 CellLabel->"Out[1801]="]
}, Open  ]]
}, Open  ]],

Cell["TrickMandelstam", "ObjectName",
 CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TrickMandelstam"],

Cell[TextData[{
 "TrickMandelstam[expr, {s, t, u, ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "1", "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "2", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "3", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Plain"], "4", "2"], TraditionalForm]]],
 "}] simplifies all sums in expr so that one of the Mandelstam variables s, t \
or u is eliminated by the relation ",
 StyleBox["s",
  FontSlant->"Italic"],
 " + ",
 StyleBox["t",
  FontSlant->"Italic"],
 " + ",
 StyleBox["u",
  FontSlant->"Italic"],
 " = ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox[
    StyleBox["m",
     FontSlant->"Italic"], "1", "2"], TraditionalForm]]],
 " + ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "2", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "3", "2"], TraditionalForm]]],
 "+ ",
 Cell[BoxData[
  FormBox[
   SubsuperscriptBox["m", "4", "2"], TraditionalForm]]],
 " . The trick is that the resulting sum has the most short number of terms."
}], "Text",
 CellTags->"TrickMandelstam"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TrickMandelstam"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"TrickMandelstam", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{"s", "+", "t", "-", "u"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"2", 
        SubsuperscriptBox["M", "W", "2"]}], "-", "t", "-", "u"}], ")"}]}], 
    ",", 
    RowBox[{"{", 
     RowBox[{"s", ",", "t", ",", "u", ",", 
      RowBox[{"2", 
       SubsuperscriptBox["M", "W", "2"]}]}], "}"}]}], "]"}], "//", 
  "Factor2"}]], "Input",
 CellTags->"TrickMandelstam",
 CellLabel->"In[1802]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", "2"}], " ", "s", " ", 
   RowBox[{"(", 
    RowBox[{"u", "-", 
     SubsuperscriptBox["M", "W", "2"]}], ")"}]}], TraditionalForm]], "Output",\

 CellTags->"TrickMandelstam",
 CellLabel->"Out[1802]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TrickMandelstam", "[", 
  RowBox[{
   RowBox[{
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "s"}], " ", "-", " ", 
    RowBox[{"s", "^", "2"}], " ", "+", " ", 
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "t"}], " ", "-", " ", "st", " ", "+", " ", 
    
    RowBox[{
     RowBox[{"M", "^", "2"}], " ", "u"}], " ", "-", " ", "su"}], ",", " ", 
   RowBox[{"{", 
    RowBox[{"s", ",", "t", ",", "u", ",", 
     RowBox[{"2", 
      RowBox[{"M", "^", "2"}]}]}], "}"}]}], "]"}]], "Input",
 CellTags->"TrickMandelstam",
 CellLabel->"In[1803]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"2", " ", 
    SuperscriptBox["M", "4"]}], "-", 
   SuperscriptBox["s", "2"], "-", "st", "-", "su"}], 
  TraditionalForm]], "Output",
 CellTags->"TrickMandelstam",
 CellLabel->"Out[1803]="]
}, Open  ]]
}, Open  ]],

Cell["Tr2", "ObjectName",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Tr2"],

Cell["\<\
If exp contains DiracTrace's, Tr2[exp] simplifies exp and does the Dirac \
traces unless more that 4 gamma matrices and DiracGamma[5] occur. Tr2[exp] \
also separates the color-strucure, and takes the color trace if Tf occurs in \
exp. If exp does not contain DiracTrace's, Tr2[exp] takes the Dirac trace.\
\>", "Text",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Tr2", "]"}]], "Input",
 CellTags->"Tr2",
 CellLabel->"In[1804]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"Factoring", "\[Rule]", "False"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Tr2",
 CellLabel->"Out[1804]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["Tf",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tf", All, CellTags],
  ButtonData->"Tf",
  ButtonNote->"Tf"],
 ", ",
 ButtonBox["DiracTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracTrace", All, CellTags],
  ButtonData->"DiracTrace",
  ButtonNote->"DiracTrace"],
 ", ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 ", ",
 ButtonBox["SUNTrace",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SUNTrace", All, CellTags],
  ButtonData->"SUNTrace",
  ButtonNote->"SUNTrace"],
 "."
}], "Text",
 CellTags->"Tr2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Tr2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Tr2", "[", 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"a", ",", "b", ",", "c", ",", "d"}], "]"}], "]"}]], "Input",
 CellLabel->"In[1805]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "d"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"b", "c"}]]}], "-", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "c"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"b", "d"}]]}], "+", 
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"a", "b"}]], " ", 
      SuperscriptBox["g", 
       RowBox[{"c", "d"}]]}]}], ")"}]}], TraditionalForm]], "Output",
 CellLabel->"Out[1805]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TR", "[", 
  RowBox[{
   RowBox[{"a", " ", "Tf"}], ",", 
   RowBox[{"SUNTrace", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Tr2",
 CellLabel->"In[1806]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"4", " ", "a", " ", 
   SubscriptBox["T", "f"]}], TraditionalForm]], "Output",
 CellTags->"Tr2",
 CellLabel->"Out[1806]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2AlienOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
 CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2AlienOperator"],

Cell["\<\
Twist2AlienOperator[p, 0] : (7);   Twist2AlienOperator[p1,p2,{p3,mu,a}, 0] \
(p1: incoming quark momentum, p3: incoming gluon (count1)).\
\>", "Text",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist2AlienOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2AlienOperator", "[", 
  RowBox[{"p", ",", " ", "0"}], "]"}]], "Input",
 CellTags->"Twist2AlienOperator",
 CellLabel->"In[1807]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"2", " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["2", "m"], "-", 
      FractionBox["1", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["2", 
       RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
    SubscriptBox["C", "F"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Twist2AlienOperator",
 CellLabel->"Out[1807]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2AlienOperator", "[", 
  RowBox[{"p1", ",", "p2", ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", " ", "0"}], 
  "]"}]], "Input",
 CellTags->"Twist2AlienOperator",
 CellLabel->"In[1808]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{"\[ImaginaryI]", " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
    SubsuperscriptBox["g", "s", "3"], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["mu",
       TraditionalForm],
      TraditionalForm]], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], ".", 
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}]}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["2", "m"], "-", 
         FractionBox["1", 
          RowBox[{"m", "+", "1"}]], "-", 
         FractionBox["2", 
          RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p1",
              TraditionalForm]}], ")"}], 
           RowBox[{"m", "-", "1"}]], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p1",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}]}]], "-", 
         FractionBox[
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p3",
                TraditionalForm]}], ")"}]}], ")"}], 
           RowBox[{"m", "-", "1"}]], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p1",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}]}]]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{"(", 
        RowBox[{
         FractionBox["1", 
          RowBox[{"m", "-", "1"}]], "-", 
         FractionBox["1", "m"]}], ")"}], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", 
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p3",
             TraditionalForm]}], ")"}]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}]}], ")"}]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Twist2AlienOperator",
 CellLabel->"Out[1808]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2CounterOperator  ***unfinished*** (unclear usage def.)", \
"ObjectName",
 CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2CounterOperator"],

Cell["\<\
Twist2CounterOperator[p,mu,nu,a,b,5]; Twist2CounterOperator[p, 7] : (7);  \
Twist2CounterOperator[p1,p2,{p3,mu,a}, 1] (p1: incoming quark momentum, p3: \
incoming gluon (count1)).\
\>", "Text",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist2CounterOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p", ",", "mu", ",", "nu", ",", "a", ",", "b", ",", "5"}], 
  "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1809]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   RowBox[{
    FractionBox["1", 
     RowBox[{"2", " ", 
      TagBox["\[CurlyEpsilon]",
       TraditionalForm]}]], 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
     SubscriptBox["C", "A"], " ", 
     SubsuperscriptBox["g", "s", "2"], " ", 
     SubscriptBox["S", "n"], " ", 
     SubscriptBox["\[Delta]", 
      RowBox[{
       FormBox[
        FormBox["a",
         TraditionalForm],
        TraditionalForm], 
       FormBox[
        FormBox["b",
         TraditionalForm],
        TraditionalForm]}]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["8", "m"], "-", 
          FractionBox["12", 
           RowBox[{"m", "+", "1"}]], "+", 
          FractionBox["8", 
           RowBox[{"m", "+", "2"}]], "-", 
          FractionBox["8", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox["g", 
         RowBox[{"mu", "nu"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], "m"]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          FractionBox["8", "m"], "-", 
          FractionBox["24", 
           RowBox[{"m", "+", "1"}]], "+", 
          FractionBox["24", 
           RowBox[{"m", "+", "2"}]], "-", 
          FractionBox["4", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox[
         FormBox["p",
          TraditionalForm], "2"], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["mu",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["nu",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "2"}]]}], "+", 
       RowBox[{
        RowBox[{"(", 
         RowBox[{
          RowBox[{"-", 
           FractionBox["6", "m"]}], "+", 
          FractionBox["16", 
           RowBox[{"m", "+", "1"}]], "-", 
          FractionBox["16", 
           RowBox[{"m", "+", "2"}]], "+", 
          FractionBox["6", 
           RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["mu",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["nu",
              TraditionalForm],
             TraditionalForm]]}], "+", 
          RowBox[{
           SuperscriptBox[
            FormBox["\[CapitalDelta]",
             TraditionalForm], 
            FormBox[
             FormBox["mu",
              TraditionalForm],
             TraditionalForm]], " ", 
           SuperscriptBox[
            FormBox["p",
             TraditionalForm], 
            FormBox[
             FormBox["nu",
              TraditionalForm],
             TraditionalForm]]}]}], ")"}]}]}], ")"}]}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1809]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p", ",", "7"}], "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1810]:="],

Cell[BoxData[
 FormBox[
  FractionBox[
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      FractionBox["2", "m"], "-", 
      FractionBox["1", 
       RowBox[{"m", "+", "1"}]], "-", 
      FractionBox["2", 
       RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
    SubscriptBox["C", "F"], " ", 
    SubsuperscriptBox["g", "s", "2"], " ", 
    SubscriptBox["S", "n"], " ", 
    RowBox[{
     FormBox["\<\"\[Gamma]\"\>",
      TraditionalForm], "\[CenterDot]", 
     FormBox["\[CapitalDelta]",
      TraditionalForm]}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]]}], 
   TagBox["\[CurlyEpsilon]",
    TraditionalForm]], TraditionalForm]], "Output",
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1810]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2CounterOperator", "[", 
  RowBox[{"p1", ",", "p2", ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "mu", ",", "a"}], "}"}], ",", "1"}], "]"}]], "Input",
 CellTags->"Twist2CounterOperator",
 CellLabel->"In[1811]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   FractionBox[
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox["2", "m"], "-", 
       FractionBox["1", 
        RowBox[{"m", "+", "1"}]], "-", 
       FractionBox["2", 
        RowBox[{"m", "-", "1"}]]}], ")"}], " ", 
     SubscriptBox["T", 
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubsuperscriptBox["g", "s", "3"], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["mu",
        TraditionalForm],
       TraditionalForm]], " ", 
     SubscriptBox["S", "n"], " ", 
     RowBox[{"(", 
      RowBox[{
       SubscriptBox["C", "A"], "-", 
       RowBox[{"2", " ", 
        SubscriptBox["C", "F"]}]}], ")"}], " ", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], " ", 
     RowBox[{"(", 
      RowBox[{
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p1",
            TraditionalForm]}], ")"}], 
         RowBox[{"m", "-", "1"}]], 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p1",
           TraditionalForm]}], "+", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p2",
           TraditionalForm]}]}]], "-", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p2",
              TraditionalForm]}], ")"}]}], ")"}], 
         RowBox[{"m", "-", "1"}]], 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p1",
           TraditionalForm]}], "+", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p2",
           TraditionalForm]}]}]]}], ")"}]}], 
    RowBox[{"2", " ", 
     TagBox["\[CurlyEpsilon]",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2CounterOperator",
 CellLabel->"Out[1811]="]
}, Open  ]]
}, Open  ]],

Cell["Twist2GluonOperator", "ObjectName",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell["\<\
Twist2GluonOperator[{p, mu, a}, {nu, b}] or Twist2GluonOperator[p, {mu, a}, \
{nu, b}] or Twist2GluonOperator[p, mu,a, nu,b] yields the 2-gluon operator (p \
is ingoing momentum corresponding to Lorentz index mu). \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}] or Twist2GluonOperator[ \
p,mu,a , q,nu,b , k,la,c ] gives the 3-gluon operator. \
Twist2GluonOperator[{p,mu,a}, {q,nu,b}, {k,la,c}, {s,si,d}] or \
Twist2GluonOperator[p,mu,a , q,nu,b , k,la,c , s,si,d] yields the 4-Gluon \
operator. The dimension is determined by the option and Dimension. The \
setting of the option Polarization (unpolarized: 0; polarized: 1) determines \
whether the unpolarized or polarized feynman rule is returned. With the \
setting Explicit to False the color-structure and the (1+(-1)^OPEm) (for \
polarized: (1-(-1)^OPEm)) is extracted and the color indices are omitted in \
the arguments of Twist2GluonOperator.\
\>", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2QuarkOperator", All, CellTags],
  ButtonData->"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion ", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}], ",", " ", 
   RowBox[{"ZeroMomentumInsertion", " ", "\[Rule]", " ", "True"}]}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1812]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1812]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1813]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1813]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1814]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1814]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1815]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm]}], "G2"], "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1815]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1816]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "1"}]],
    TraditionalForm], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox["p",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1816]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1817]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"(", 
    RowBox[{"1", "-", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm]}], ")"}], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]}], "G3"], "(", 
     FormBox["q",
      TraditionalForm], ",", 
     FormBox["r",
      TraditionalForm], ",", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1817]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1818]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Mu]",
      TraditionalForm]}], "G3"], "(", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1818]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1819]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}], "+", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}], "+", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}], "-", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", "1"}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]], " ", 
    RowBox[{"(", 
     RowBox[{"-", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]}]]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm]}]]}]}], ")"}]}], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]]}]}], ")"}]}], "-", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["r",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1819]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1820]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c241",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c241",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c241",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c241",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c241",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c241",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1820]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1821]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Sigma]",
      TraditionalForm]}], "G4"], "(", 
   FormBox["p",
    TraditionalForm], ",", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["s",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1821]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1822]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["r",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["r",
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["s",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["q",
           TraditionalForm], 
          FormBox["s",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["s",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["p",
           TraditionalForm], 
          FormBox["r",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["q",
           TraditionalForm], 
          FormBox["r",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1822]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1823]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "False"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1823]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1824]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]}], "G2"], "(", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1824]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1825]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1825]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1826]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm]}], "G2"], "(", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1826]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{"p", ",", 
   RowBox[{"{", "\[Mu]", "}"}], ",", 
   RowBox[{"{", "\[Nu]", "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1827]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], ")"}], 
     RowBox[{"m", "-", "2"}]],
    TraditionalForm], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      SuperscriptBox["g", 
       RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
      SuperscriptBox[
       FormBox[
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}],
        TraditionalForm], "2"]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["p",
        TraditionalForm], "2"], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]]}], "+", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1827]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1828]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{"-", "1"}], ")"}], "m"],
      TraditionalForm], "+", "1"}], ")"}], " ", 
   SubscriptBox["g", "s"], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     SubsuperscriptBox["O", 
      RowBox[{
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm], "\[VeryThinSpace]", 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]}], "G3"], "(", 
     FormBox["q",
      TraditionalForm], ",", 
     FormBox["r",
      TraditionalForm], ",", 
     FormBox["p",
      TraditionalForm], ")"}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1828]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1829]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Mu]",
      TraditionalForm]}], "G3"], "(", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["p",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1829]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1830]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{"-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}]}], "+", 
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Rho]"}]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["p",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Nu]"}]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["q",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}]}], ")"}]}], "+", 
   RowBox[{
    FormBox[
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], ")"}], 
      RowBox[{"m", "-", "2"}]],
     TraditionalForm], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Nu]", "\[Rho]"}]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox["g", 
           RowBox[{"\[Mu]", "\[Rho]"}]]}]}], ")"}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["r",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]]}], "-", 
         RowBox[{
          SuperscriptBox[
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm]], " ", 
          SuperscriptBox[
           FormBox["r",
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm]]}]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1830]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1831]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c242",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c242",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c242",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c242",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c242",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c242",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1831]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1832]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubsuperscriptBox["O", 
    RowBox[{
     FormBox["\[Mu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Nu]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Rho]",
      TraditionalForm], "\[VeryThinSpace]", 
     FormBox["\[Sigma]",
      TraditionalForm]}], "G4"], "(", 
   FormBox["p",
    TraditionalForm], ",", 
   FormBox["q",
    TraditionalForm], ",", 
   FormBox["r",
    TraditionalForm], ",", 
   FormBox["s",
    TraditionalForm], ")"}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1832]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1833]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}]}], ")"}], " ", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], ")"}], 
     RowBox[{"m", "-", "2"}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["r",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     FormBox[
      TagBox[
       RowBox[{
        UnderoverscriptBox["\[Sum]", 
         RowBox[{"i", "=", "0"}], 
         RowBox[{
          RowBox[{"-", "3"}], "+", "m"}]], 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "3"}], "-", "i", "+", "m"}]]}]}],
       HoldForm],
      TraditionalForm], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["s",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["s",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["s",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Sigma]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["r",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}]}]}], ")"}]}], "-", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox[
       TagBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        HoldForm],
       TraditionalForm], 
      FormBox[
       RowBox[{"\<\"\"\>", " ", 
        RowBox[{"(", 
         RowBox[{"j", "+", "1"}], ")"}]}],
       TraditionalForm], 
      FormBox[
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["s",
             TraditionalForm]}]}], ")"}], 
         RowBox[{"j", "-", "i"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", 
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}], ")"}]}], ")"}], "i"]}],
       TraditionalForm]}], ")"}], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], " ", 
    SuperscriptBox[
     FormBox["\[CapitalDelta]",
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox[
         FormBox["s",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["q",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Sigma]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["q",
         TraditionalForm], "\[CenterDot]", 
        FormBox["s",
         TraditionalForm]}]}]}], ")"}]}]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1833]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Advanced Examples", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell["The setting All for Explicit performs the sums.", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Twist2GluonOperator", ",", " ", 
    RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
    RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1834]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"Twist2GluonOperator", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1835]:="]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1836]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1836]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1837]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", "\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      FormBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]]}], "-", 
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]]}]}], ")"}]}], "+", 
     RowBox[{
      FormBox[
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]],
       TraditionalForm], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm]}]]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         SuperscriptBox["\[Epsilon]", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], 
           FormBox["\[CapitalDelta]",
            TraditionalForm], 
           FormBox["p",
            TraditionalForm], 
           FormBox["q",
            TraditionalForm]}]]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1837]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1838]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SubscriptBox["g", "s"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{
      RowBox[{"-", 
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]]}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["q",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "-", 
     RowBox[{
      SuperscriptBox["\[Epsilon]", 
       RowBox[{
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm], 
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox["p",
         TraditionalForm], 
        FormBox["r",
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       FormBox[
        TagBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         HoldForm],
        TraditionalForm], ")"}], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Rho]",
         TraditionalForm],
        TraditionalForm]]}], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["p",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["p",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["q",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "+", 
     RowBox[{
      FractionBox["1", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], "+", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}], ")"}], "2"]], 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{"(", 
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], "+", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}]}], ")"}], "m"], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "3"]}], "+", 
         RowBox[{"2", " ", 
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}],
            TraditionalForm], "2"]}], "+", 
         RowBox[{
          RowBox[{"(", 
           FormBox[
            TagBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             HoldForm],
            TraditionalForm], ")"}], " ", 
          SuperscriptBox[
           FormBox[
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}],
            TraditionalForm], "2"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}]}]}], ")"}]}]}], "-", 
     FractionBox[
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["p",
              TraditionalForm]}], ")"}], "m"]}], "+", 
         RowBox[{
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], " ", 
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["r",
              TraditionalForm]}], ")"}], "m"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}]}], "+", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["q",
              TraditionalForm]}], ")"}], "m"], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["r",
            TraditionalForm]}], " ", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}]}]}], ")"}]}], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["r",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]]}], ")"}], " ", 
   SubscriptBox["f", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["c",
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1838]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1839]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[ImaginaryI]", " ", 
   RowBox[{"(", 
    RowBox[{"1", "-", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"]}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c243",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c243",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c243",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c243",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c243",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c243",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1839]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2GluonOperator", ",", " ", 
   RowBox[{"Polarization", " ", "\[Rule]", " ", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1840]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1840]="]
}, Open  ]],

Cell["2-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"\[Nu]", ",", "b"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1841]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["b",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"\[Mu]", "\[Nu]"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Nu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["q",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["\[Mu]",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1841]="]
}, Open  ]],

Cell["3-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Twist2GluonOperator", "[", 
   RowBox[{
    RowBox[{"{", 
     RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1842]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1843]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["\[Mu]",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["\[Mu]",
         TraditionalForm]}], " ", 
       SuperscriptBox["g", 
        RowBox[{"q", "r"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[Mu]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["r",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1843]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1844]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    FractionBox["1", "2"]}], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubscriptBox["\[Delta]", 
    RowBox[{
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Rho]",
       TraditionalForm],
      TraditionalForm]}]], " ", 
   RowBox[{"(", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["\[Mu]",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm], "+", 
     FormBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "2"}]],
      TraditionalForm]}], ")"}], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["\[Mu]",
         TraditionalForm]}], " ", 
       SuperscriptBox["g", 
        RowBox[{"q", "r"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[Mu]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["r",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}]}], "+", 
     RowBox[{
      SuperscriptBox[
       FormBox["\[CapitalDelta]",
        TraditionalForm], 
       FormBox[
        FormBox["q",
         TraditionalForm],
        TraditionalForm]], " ", 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         SuperscriptBox[
          FormBox["p",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}], "-", 
        RowBox[{
         SuperscriptBox[
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox[
           FormBox["r",
            TraditionalForm],
           TraditionalForm]], " ", 
         RowBox[{
          FormBox["p",
           TraditionalForm], "\[CenterDot]", 
          FormBox["\[Mu]",
           TraditionalForm]}]}]}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1844]="]
}, Open  ]],

Cell["4-gluon legs", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"s", ",", "\[Sigma]", ",", "d"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1845]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FractionBox["1", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{"-", "1"}], ")"}], "m"], "+", "1"}], ")"}], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{"(", 
    RowBox[{
     RowBox[{"-", 
      RowBox[{
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c244",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       SubscriptBox["f", 
        RowBox[{
         FormBox[
          FormBox["c",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["c244",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["d",
           TraditionalForm],
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        RowBox[{
         SubsuperscriptBox["O", 
          RowBox[{
           FormBox[
            FormBox["\[Mu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Nu]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Rho]",
             TraditionalForm],
            TraditionalForm], "\[VeryThinSpace]", 
           FormBox[
            FormBox["\[Sigma]",
             TraditionalForm],
            TraditionalForm]}], "G4"], "(", 
         FormBox["p",
          TraditionalForm], ",", 
         FormBox["q",
          TraditionalForm], ",", 
         FormBox["r",
          TraditionalForm], ",", 
         FormBox["s",
          TraditionalForm], ")"}], ")"}]}]}], "-", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c244",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c244",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}], "+", 
     RowBox[{
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["a",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c244",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["d",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      SubscriptBox["f", 
       RowBox[{
        FormBox[
         FormBox["b",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c",
          TraditionalForm],
         TraditionalForm], 
        FormBox[
         FormBox["c244",
          TraditionalForm],
         TraditionalForm]}]], " ", 
      RowBox[{"(", 
       RowBox[{
        SubsuperscriptBox["O", 
         RowBox[{
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], "\[VeryThinSpace]", 
          FormBox[
           FormBox["\[Sigma]",
            TraditionalForm],
           TraditionalForm]}], "G4"], "(", 
        FormBox["r",
         TraditionalForm], ",", 
        FormBox["q",
         TraditionalForm], ",", 
        FormBox["p",
         TraditionalForm], ",", 
        FormBox["s",
         TraditionalForm], ")"}], ")"}]}]}], ")"}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1845]="]
}, Open  ]],

Cell["Suppress the lengthy output.", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Twist2GluonOperator", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", ",", "\[Mu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "\[Rho]"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"s", ",", "\[Sigma]"}], "}"}], ",", 
     RowBox[{"Explicit", "\[Rule]", "True"}]}], "]"}], ",", "6"}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1846]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Sigma]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Mu]", "\[Rho]"}]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]]}]}], ")"}], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], "+", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}], ")"}], 
      RowBox[{"m", "-", "2"}]]}], "+", 
    RowBox[{"\[LeftSkeleton]", "8", "\[RightSkeleton]"}], "+", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"\<\"\"\>", " ", 
         RowBox[{"(", 
          RowBox[{"j", "+", "1"}], ")"}]}],
        TraditionalForm], 
       FormBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}]}], ")"}], "i"], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{"j", "-", "i"}]]}],
        TraditionalForm]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Sigma]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["r",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Rho]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}]}]}], ")"}]}], "-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox[
        RowBox[{
         UnderoverscriptBox["\[Sum]", 
          RowBox[{"j", "=", "0"}], 
          RowBox[{
           RowBox[{"-", "4"}], "+", "m"}]], "\<\"\"\>"}],
        TraditionalForm], 
       FormBox[
        RowBox[{"\<\"\"\>", " ", 
         RowBox[{"(", 
          RowBox[{"j", "+", "1"}], ")"}]}],
        TraditionalForm], 
       FormBox[
        RowBox[{
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], ")"}], 
          RowBox[{
           RowBox[{"-", "j"}], "+", "m", "-", "4"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{
            RowBox[{"-", 
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}], ")"}]}], "-", 
            RowBox[{
             FormBox["\[CapitalDelta]",
              TraditionalForm], "\[CenterDot]", 
             FormBox["s",
              TraditionalForm]}]}], ")"}], 
          RowBox[{"j", "-", "i"}]], " ", 
         SuperscriptBox[
          RowBox[{"(", 
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["s",
               TraditionalForm]}], ")"}]}], ")"}], "i"]}],
        TraditionalForm]}], ")"}], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], " ", 
     SuperscriptBox[
      FormBox["\[CapitalDelta]",
       TraditionalForm], 
      FormBox[
       FormBox["\[Rho]",
        TraditionalForm],
       TraditionalForm]], " ", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{
        RowBox[{"-", 
         SuperscriptBox[
          FormBox["s",
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm]]}], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox["g", 
         RowBox[{"\[Nu]", "\[Sigma]"}]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}]}], "-", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["q",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}], "+", 
       RowBox[{
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm]], " ", 
        SuperscriptBox[
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox[
          FormBox["\[Sigma]",
           TraditionalForm],
          TraditionalForm]], " ", 
        RowBox[{
         FormBox["q",
          TraditionalForm], "\[CenterDot]", 
         FormBox["s",
          TraditionalForm]}]}]}], ")"}]}]}],
   Short[#, 6]& ], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1846]//Short="]
}, Open  ]],

Cell["\<\
The setting Explicit \[Rule] All performs the sums.\
\>", "Text",
 CellTags->"Twist2GluonOperator"],

Cell[BoxData[
 RowBox[{
  RowBox[{"SetOptions", "[", 
   RowBox[{"Twist2GluonOperator", ",", " ", 
    RowBox[{"Polarization", " ", "\[Rule]", " ", "1"}], ",", " ", 
    RowBox[{"Explicit", "\[Rule]", "All"}]}], "]"}], ";"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1847]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Short", "[", 
  RowBox[{
   RowBox[{"Twist2GluonOperator", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"p", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"q", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"r", ",", "\[Rho]", ",", "c"}], "}"}]}], "]"}], ",", "5"}], 
  "]"}]], "Input",
 CellTags->"Twist2GluonOperator",
 CellLabel->"In[1848]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   RowBox[{
    SubscriptBox["g", "s"], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{"-", 
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}]]}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["p",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "+", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["q",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Mu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox[
          FormBox["\[Nu]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Rho]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["r",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "2"}]]}], "-", 
      RowBox[{
       SuperscriptBox["\[Epsilon]", 
        RowBox[{
         FormBox[
          FormBox["\[Rho]",
           TraditionalForm],
          TraditionalForm], 
         FormBox["\[CapitalDelta]",
          TraditionalForm], 
         FormBox["p",
          TraditionalForm], 
         FormBox["r",
          TraditionalForm]}]], " ", 
       RowBox[{"(", 
        FormBox[
         RowBox[{
          UnderoverscriptBox["\[Sum]", 
           RowBox[{"i", "=", "0"}], 
           RowBox[{
            RowBox[{"-", "3"}], "+", "m"}]], 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], "i"]}]}],
         TraditionalForm], ")"}], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Mu]",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["\[Nu]",
          TraditionalForm],
         TraditionalForm]]}], "+", 
      RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
      RowBox[{"\[LeftSkeleton]", "14", "\[RightSkeleton]"}], "+", 
      RowBox[{
       FractionBox["1", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}]}], ")"}], "2"]], 
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm], 
          FormBox["p",
           TraditionalForm]}]], " ", 
        RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
        RowBox[{"(", 
         RowBox[{
          SuperscriptBox[
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}], "m"], "+", 
          RowBox[{
           RowBox[{"(", 
            FormBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], "+", 
                  RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}]}], 
                 ")"}], "i"]}]}],
             TraditionalForm], ")"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
             TraditionalForm], "3"]}], "+", 
          RowBox[{"2", " ", 
           RowBox[{"(", 
            FormBox[
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
             TraditionalForm], ")"}], " ", 
           RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{"\[LeftSkeleton]", "1", "\[RightSkeleton]"}],
             TraditionalForm], "2"]}], "+", 
          RowBox[{
           RowBox[{"(", 
            FormBox[
             RowBox[{
              UnderoverscriptBox["\[Sum]", 
               RowBox[{"i", "=", "0"}], 
               RowBox[{
                RowBox[{"-", "3"}], "+", "m"}]], 
              RowBox[{
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  FormBox["\[CapitalDelta]",
                   TraditionalForm], "\[CenterDot]", 
                  FormBox["r",
                   TraditionalForm]}], ")"}], 
                RowBox[{
                 RowBox[{"-", "3"}], "-", "i", "+", "m"}]], " ", 
               SuperscriptBox[
                RowBox[{"(", 
                 RowBox[{
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["q",
                    TraditionalForm]}], "+", 
                  RowBox[{
                   FormBox["\[CapitalDelta]",
                    TraditionalForm], "\[CenterDot]", 
                   FormBox["r",
                    TraditionalForm]}]}], ")"}], "i"]}]}],
             TraditionalForm], ")"}], " ", 
           SuperscriptBox[
            FormBox[
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}],
             TraditionalForm], "2"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}]}]}], ")"}]}]}], "-", 
      FractionBox[
       RowBox[{
        SuperscriptBox["\[Epsilon]", 
         RowBox[{
          FormBox[
           FormBox["\[Mu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Nu]",
            TraditionalForm],
           TraditionalForm], 
          FormBox[
           FormBox["\[Rho]",
            TraditionalForm],
           TraditionalForm], 
          FormBox["\[CapitalDelta]",
           TraditionalForm]}]], " ", 
        RowBox[{"(", 
         RowBox[{
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], "m"]}], "+", 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}], ")"}], "m"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}]}], "+", 
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "m"], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}]}]}], ")"}]}], 
       RowBox[{
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["q",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["r",
          TraditionalForm]}], " ", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}]}]]}], ")"}], " ", 
    SubscriptBox["f", 
     RowBox[{
      FormBox[
       FormBox["a",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["b",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["c",
        TraditionalForm],
       TraditionalForm]}]]}],
   Short[#, 5]& ], TraditionalForm]], "Output",
 CellTags->"Twist2GluonOperator",
 CellLabel->"Out[1848]//Short="]
}, Open  ]]
}, Open  ]],

Cell["Twist2QuarkOperator", "ObjectName",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell["\<\
Twist2QuarkOperator[p] or Twist2QuarkOperator[p,_,_] yields the \
quark-antiquark operator (p is momentum in the direction of the incoming \
quark). Twist2QuarkOperator[{p,q}] yields the 2-quark operator for non-zero \
momentum insertion (p is momentum in the direction of the incoming quark). \
Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a] is the quark-antiquark-gluon \
operator, where p1 is the incoming quark, p2 the incoming antiquark and p3 \
denotes the incoming gluon momentum. Twist2QuarkOperator[{p1}, {p2}, {p3, mu, \
a}, {p4, nu, b}] or Twist2QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}, {p4, \
nu, b}] or Twist2QuarkOperator[p1,_,_, p2,_,_, p3,mu,a, p4, nu, b] gives the \
Quark-Quark-Gluon-Gluon-operator. The setting of the option Polarization \
(unpolarized: 0; polarized: 1) determines whether the unpolarized or \
polarized operator is returned\
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2GluonOperator", All, CellTags],
  ButtonData->"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],

Cell["Examples", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell["Polarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1849]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1849]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1850]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    RowBox[{
     RowBox[{"(", 
      RowBox[{
       FormBox["\<\"\[Gamma]\"\>",
        TraditionalForm], "\[CenterDot]", 
       FormBox["\[CapitalDelta]",
        TraditionalForm]}], ")"}], ".", 
     SuperscriptBox["\[Gamma]", "5"]}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1850]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1851]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubscriptBox["g", "s"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1851]="]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t3", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1852]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubsuperscriptBox["g", "s", "2"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"j", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"j", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], "i"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"j", "-", "i"}]]}],
          TraditionalForm]}], ")"}]}], "-", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], " ", 
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"j", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"j", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", "1"}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "i"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], ")"}], 
            RowBox[{
             RowBox[{"-", "j"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"j", "-", "i"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1852]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, zero-momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", " ", 
   RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1853]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "0"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1853]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", "p", "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1854]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    FormBox["\<\"\[Gamma]\"\>",
     TraditionalForm], "\[CenterDot]", 
    FormBox["\[CapitalDelta]",
     TraditionalForm]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1854]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1855]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{"-", "1"}], ")"}], "i"], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["p",
            TraditionalForm]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           FormBox["\[CapitalDelta]",
            TraditionalForm], "\[CenterDot]", 
           FormBox["q",
            TraditionalForm]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1855]="]
}, Open  ]],

Cell["Quark-antiquark-gluon-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t6", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1856]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"-", 
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"]}], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubsuperscriptBox["g", "s", "2"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["k",
                TraditionalForm], "\[CenterDot]", 
               FormBox["\[CapitalDelta]",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FormBox[
          TagBox[
           RowBox[{
            UnderoverscriptBox["\[Sum]", 
             RowBox[{"i", "=", "0"}], 
             RowBox[{
              RowBox[{"-", "3"}], "+", "m"}]], "\<\"\"\>"}],
           HoldForm],
          TraditionalForm], 
         FormBox[
          RowBox[{"\<\"\"\>", " ", 
           RowBox[{"(", 
            RowBox[{"i", "+", "1"}], ")"}]}],
          TraditionalForm], 
         FormBox[
          RowBox[{
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], "j"], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{"-", 
              RowBox[{"(", 
               RowBox[{
                FormBox["\[CapitalDelta]",
                 TraditionalForm], "\[CenterDot]", 
                FormBox["p",
                 TraditionalForm]}], ")"}]}], ")"}], 
            RowBox[{
             RowBox[{"-", "i"}], "+", "m", "-", "3"}]], " ", 
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"i", "-", "j"}]]}],
          TraditionalForm]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1856]="]
}, Open  ]],

Cell["This shows the FeynCalcExternal (FCE) form.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"Twist2QuarkOperator", "[", "p", "]"}], "//", "FCE"}], "//", 
  "StandardForm"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1857]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"GSD", "[", "OPEDelta", "]"}], " ", 
  SuperscriptBox[
   RowBox[{"SOD", "[", "p", "]"}], 
   RowBox[{
    RowBox[{"-", "1"}], "+", "OPEm"}]]}]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1857]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2QuarkOperator", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1858]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubscriptBox["g", "s"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]]}], " ", 
   RowBox[{"(", 
    RowBox[{
     FractionBox[
      SuperscriptBox[
       RowBox[{"(", 
        RowBox[{
         FormBox["\[CapitalDelta]",
          TraditionalForm], "\[CenterDot]", 
         FormBox["p",
          TraditionalForm]}], ")"}], 
       RowBox[{"m", "-", "1"}]], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]], "+", 
     FractionBox[
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"-", "1"}], ")"}], "m"], " ", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{
          FormBox["\[CapitalDelta]",
           TraditionalForm], "\[CenterDot]", 
          FormBox["q",
           TraditionalForm]}], ")"}], 
        RowBox[{"m", "-", "1"}]]}], 
      RowBox[{
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p",
         TraditionalForm]}], "+", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["q",
         TraditionalForm]}]}]]}], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1858]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist2QuarkOperator", "[", 
  RowBox[{
   RowBox[{"{", "p", "}"}], ",", 
   RowBox[{"{", "q", "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"r", ",", "\[Nu]", ",", "b"}], "}"}], ",", 
   RowBox[{"Explicit", "\[Rule]", "All"}], ",", 
   RowBox[{"Polarization", "\[Rule]", "0"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1859]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{"-", "1"}], ")"}], "m"],
    TraditionalForm], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   RowBox[{"(", 
    RowBox[{"-", 
     SubsuperscriptBox["g", "s", "2"]}], ")"}], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm], " ", 
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm]}], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "-", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "+", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["k",
                TraditionalForm], "\[CenterDot]", 
               FormBox["\[CapitalDelta]",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]]}], ")"}]}], "+", 
      RowBox[{
       RowBox[{
        SubscriptBox["T", 
         FormBox[
          FormBox["b",
           TraditionalForm],
          TraditionalForm]], ".", 
        SubscriptBox["T", 
         FormBox[
          FormBox["a",
           TraditionalForm],
          TraditionalForm]]}], " ", 
       RowBox[{"(", 
        RowBox[{
         FractionBox[
          RowBox[{
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{"-", "1"}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm], " ", 
           FormBox[
            SuperscriptBox[
             RowBox[{"(", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["p",
                TraditionalForm]}], ")"}], 
             RowBox[{"m", "-", "1"}]],
            TraditionalForm]}], 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}]}]], "-", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}]}], ")"}]}]], "+", 
         FractionBox[
          FormBox[
           SuperscriptBox[
            RowBox[{"(", 
             RowBox[{
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["q",
                TraditionalForm]}], "+", 
              RowBox[{
               FormBox["\[CapitalDelta]",
                TraditionalForm], "\[CenterDot]", 
               FormBox["r",
                TraditionalForm]}]}], ")"}], 
            RowBox[{"m", "-", "1"}]],
           TraditionalForm], 
          RowBox[{
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["r",
             TraditionalForm]}], " ", 
           RowBox[{"(", 
            RowBox[{
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["p",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["q",
               TraditionalForm]}], "+", 
             RowBox[{
              FormBox["\[CapitalDelta]",
               TraditionalForm], "\[CenterDot]", 
              FormBox["r",
               TraditionalForm]}]}], ")"}]}]]}], ")"}]}]}], ")"}]}]}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1859]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Polarized case, non-zero momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1860]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1860]="]
}, Open  ]],

Cell["\<\
With the setting ZeroMomentumInsertion \[Rule] False a non-zero momentum is \
assumed to flow into the operator vertex. \
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell["\<\
This is the Feynman rule associated with the quark-antiquark operator, where \
p is the momentum of the incoming quark and q the momentum of the antiquark. \
The momentum flowing into the operator vertex is thus -p-q.\
\>", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1861]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "-", "m"}]], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], "-", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1861]="]
}, Open  ]],

Cell["This is the quark-antiquark-gluon operator vertex.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1862]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "-", "m"}]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], ")"}]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1862]="]
}, Open  ]],

Cell["This shows the FeynCalcExternal form.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"StandardForm", "[", 
  RowBox[{"FCE", "[", "t2", "]"}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1863]:="],

Cell[BoxData[
 RowBox[{
  SuperscriptBox["2", 
   RowBox[{"2", "-", "OPEm"}]], " ", "Gstrong", " ", 
  RowBox[{
   RowBox[{"SUNT", "[", "a", "]"}], ".", 
   RowBox[{"GA", "[", "5", "]"}], ".", 
   RowBox[{"GSD", "[", "OPEDelta", "]"}]}], " ", 
  RowBox[{"FVD", "[", 
   RowBox[{"OPEDelta", ",", "\[Mu]"}], "]"}], " ", 
  RowBox[{"OPESum", "[", 
   RowBox[{
    RowBox[{
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"-", 
         RowBox[{"SOD", "[", "k", "]"}]}], "+", 
        RowBox[{"SOD", "[", "p", "]"}], "-", 
        RowBox[{"SOD", "[", "q", "]"}]}], ")"}], 
      RowBox[{
       RowBox[{"-", "2"}], "-", "OPEi", "+", "OPEm"}]], " ", 
     SuperscriptBox[
      RowBox[{"(", 
       RowBox[{
        RowBox[{"SOD", "[", "k", "]"}], "+", 
        RowBox[{"SOD", "[", "p", "]"}], "-", 
        RowBox[{"SOD", "[", "q", "]"}]}], ")"}], "OPEi"]}], ",", 
    RowBox[{"{", 
     RowBox[{"OPEi", ",", "0", ",", 
      RowBox[{
       RowBox[{"-", "2"}], "+", "OPEm"}]}], "}"}]}], "]"}]}]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1863]//StandardForm="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Unpolarized case, non-zero momentum insertion", "Subsubsection",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SetOptions", "[", 
  RowBox[{"Twist2QuarkOperator", ",", 
   RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
   RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "]"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1864]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Explicit", "\[Rule]", "True"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}], ",", 
    RowBox[{"ZeroMomentumInsertion", "\[Rule]", "False"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1864]="]
}, Open  ]],

Cell["Quark-antiquark operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t4", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{"{", 
    RowBox[{"p", ",", "q"}], "}"}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1865]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"1", "-", "m"}]], " ", 
   RowBox[{
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["p",
        TraditionalForm]}], "-", 
      RowBox[{
       FormBox["\[CapitalDelta]",
        TraditionalForm], "\[CenterDot]", 
       FormBox["q",
        TraditionalForm]}]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1865]="]
}, Open  ]],

Cell["Quark-antiquark-gluon operator.", "Text",
 CellTags->"Twist2QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t5", "=", 
  RowBox[{"Twist2QuarkOperator", "[", 
   RowBox[{
    RowBox[{"{", "p", "}"}], ",", 
    RowBox[{"{", "q", "}"}], ",", 
    RowBox[{"{", 
     RowBox[{"k", ",", "\[Mu]", ",", "a"}], "}"}]}], "]"}]}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1866]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["2", 
    RowBox[{"2", "-", "m"}]], " ", 
   SuperscriptBox[
    FormBox["\[CapitalDelta]",
     TraditionalForm], 
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm]], " ", 
   SubscriptBox["g", "s"], " ", 
   RowBox[{
    SubscriptBox["T", 
     FormBox[
      FormBox["a",
       TraditionalForm],
      TraditionalForm]], ".", 
    SuperscriptBox["\[Gamma]", "5"], ".", 
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}]}], " ", 
   RowBox[{"(", 
    FormBox[
     TagBox[
      RowBox[{
       UnderoverscriptBox["\[Sum]", 
        RowBox[{"i", "=", "0"}], 
        RowBox[{
         RowBox[{"-", "2"}], "+", "m"}]], 
       RowBox[{
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{"-", 
            RowBox[{"(", 
             RowBox[{
              FormBox["k",
               TraditionalForm], "\[CenterDot]", 
              FormBox["\[CapitalDelta]",
               TraditionalForm]}], ")"}]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], 
         RowBox[{
          RowBox[{"-", "2"}], "-", "i", "+", "m"}]], " ", 
        SuperscriptBox[
         RowBox[{"(", 
          RowBox[{
           RowBox[{
            FormBox["k",
             TraditionalForm], "\[CenterDot]", 
            FormBox["\[CapitalDelta]",
             TraditionalForm]}], "+", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["p",
             TraditionalForm]}], "-", 
           RowBox[{
            FormBox["\[CapitalDelta]",
             TraditionalForm], "\[CenterDot]", 
            FormBox["q",
             TraditionalForm]}]}], ")"}], "i"]}]}],
      HoldForm],
     TraditionalForm], ")"}]}], TraditionalForm]], "Output",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"Out[1866]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Clear", "[", 
   RowBox[{"t1", ",", "t2", ",", "t3", ",", "t4", ",", "t5", ",", "t6"}], 
   "]"}], ";"}]], "Input",
 CellTags->"Twist2QuarkOperator",
 CellLabel->"In[1867]:="]
}, Open  ]],

Cell["Twist3QuarkOperator", "ObjectName",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist3QuarkOperator"],

Cell["\<\
Twist3QuarkOperator[p] or  Twist3QuarkOperator[p,_,_]  yields the  2-quark \
operator (p is momentum in the direction of the fermion number flow). \
Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, a}] or \
Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a] Quark-Quark-Gluon-operator, \
where p1 is the incoming quark, p2 the incoming antiquark and p3 denotes the \
(incoming) gluon momentum.  Twist3QuarkOperator[{p1,___}, {p2,___}, {p3, mu, \
a}, {p4, nu, b}] or Twist3QuarkOperator[p1,_,_,  p2,_,_,  p3,mu,a, p4, nu, b] \
 gives the Quark-Quark-Gluon-Gluon-operator. The setting of the option \
Polarization (unpolarized: 0; polarized: 1) determines whether the \
uppolarized or polarized operator is returned.\
\>", "Text",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist3QuarkOperator", "//", "Options"}]], "Input",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"In[1868]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"CouplingConstant", "\[Rule]", 
     SubscriptBox["g", "s"]}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Polarization", "\[Rule]", "1"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"Out[1868]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2QuarkOperator", All, CellTags],
  ButtonData->"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2GluonOperator", All, CellTags],
  ButtonData->"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist3QuarkOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist3QuarkOperator", "[", "p", "]"}]], "Input",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"In[1869]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"-", "1"}], ")"}], "m"], " ", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      FormBox["\<\"\[Gamma]\"\>",
       TraditionalForm], "\[CenterDot]", 
      FormBox["\[CapitalDelta]",
       TraditionalForm]}], ")"}], ".", 
    SuperscriptBox["\[Gamma]", "5"]}], " ", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      FormBox["\[CapitalDelta]",
       TraditionalForm], "\[CenterDot]", 
      FormBox["p",
       TraditionalForm]}], ")"}], 
    RowBox[{"m", "-", "1"}]]}], TraditionalForm]], "Output",
 CellTags->"Twist3QuarkOperator",
 CellLabel->"Out[1869]="]
}, Open  ]]
}, Open  ]],

Cell["Twist4GluonOperator", "ObjectName",
 CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Twist4GluonOperator"],

Cell["\<\
Twist4GluonOperator[{oa, ob, oc, od}, {p1,la1,a1}, {p2,la2,a2}, {p3,la3,a3}, \
{p4,la4,a4}].\
\>", "Text",
 CellTags->"Twist4GluonOperator"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2QuarkOperator", All, CellTags],
  ButtonData->"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 ", ",
 ButtonBox["Twist3QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist3QuarkOperator", All, CellTags],
  ButtonData->"Twist3QuarkOperator",
  ButtonNote->"Twist3QuarkOperator"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2GluonOperator", All, CellTags],
  ButtonData->"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 "."
}], "Text",
 CellTags->"Twist4GluonOperator"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Twist4GluonOperator"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Twist4GluonOperator", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"oa", ",", " ", "ob", ",", " ", "oc", ",", " ", "od"}], "}"}], 
   ",", 
   RowBox[{"{", 
    RowBox[{"p1", ",", "la1", ",", "a1"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p2", ",", "la2", ",", "a2"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p3", ",", "la3", ",", "a3"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"p4", ",", "la4", ",", "a4"}], "}"}]}], "]"}]], "Input",
 CellTags->"Twist4GluonOperator",
 CellLabel->"In[1870]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la3"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la2", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p2",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}], "+", 
   RowBox[{
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p2",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la1", "la2"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p1",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p1",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la1",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la2",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p1",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p2",
         TraditionalForm]}]}]}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      RowBox[{
       SuperscriptBox[
        FormBox["p4",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox["g", 
        RowBox[{"la3", "la4"}]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p3",
         TraditionalForm]}]}], "+", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["p3",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["\[CapitalDelta]",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}], "-", 
      RowBox[{
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la3",
          TraditionalForm],
         TraditionalForm]], " ", 
       SuperscriptBox[
        FormBox["\[CapitalDelta]",
         TraditionalForm], 
        FormBox[
         FormBox["la4",
          TraditionalForm],
         TraditionalForm]], " ", 
       RowBox[{
        FormBox["p3",
         TraditionalForm], "\[CenterDot]", 
        FormBox["p4",
         TraditionalForm]}]}]}], ")"}], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a1",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oa",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a2",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["ob",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a3",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["oc",
        TraditionalForm],
       TraditionalForm]}]], " ", 
    SubscriptBox["\[Delta]", 
     RowBox[{
      FormBox[
       FormBox["a4",
        TraditionalForm],
       TraditionalForm], 
      FormBox[
       FormBox["od",
        TraditionalForm],
       TraditionalForm]}]]}]}], TraditionalForm]], "Output",
 CellTags->"Twist4GluonOperator",
 CellLabel->"Out[1870]="]
}, Open  ]]
}, Open  ]],

Cell["TwoLoopSimplify  ***unfinished*** (examples missing)", "ObjectName",
 CellTags->"TwoLoopSimplify"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"TwoLoopSimplify"],

Cell["\<\
TwoLoopSimplify[amplitude,{qu1,qu2}] simplifies the 2-loop amplitude (qu1 and \
qu2 denote the integration momenta). TwoLoopSimplify[amplitude] transforms to \
TwoLoopSimplify[amplitude, {Global`q1, Global`q2}], i.e., the integration \
momenta in amplitude must be named q1 and q2.\
\>", "Text",
 CellTags->"TwoLoopSimplify"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OneLoopSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSimplify", All, CellTags],
  ButtonData->"OneLoopSimplify",
  ButtonNote->"OneLoopSimplify"],
 "."
}], "Text",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"TwoLoopSimplify"],

Cell[BoxData[""], "Input",
 CellTags->"TwoLoopSimplify",
 CellLabel->"In[1871]:="]
}, Open  ]],

Cell["Uncontract", "ObjectName",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Uncontract"],

Cell["\<\
Uncontract[exp, q1, q2, ...] uncontracts Eps and DiracGamma. Uncontract[exp, \
q1, q2, Pair \[Rule] {p}] uncontracts also p.q1 and p.q2; the option Pair \
\[Rule] All uncontracts all momenta except OPEDelta.\
\>", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Uncontract", "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1872]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "Automatic"}], ",", 
    RowBox[{"DimensionalReduction", "\[Rule]", "False"}], ",", 
    RowBox[{"Pair", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"Unique", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1872]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Contract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract", All, CellTags],
  ButtonData->"Contract",
  ButtonNote->"Contract"],
 "."
}], "Text",
 CellTags->"Uncontract"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t1", " ", "=", " ", 
  RowBox[{
   RowBox[{"LeviCivita", "[", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}], "]"}], "[", 
   RowBox[{"p", ",", "q"}], "]"}]}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1873]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Epsilon]", 
   RowBox[{
    FormBox[
     FormBox["\[Mu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox[
     FormBox["\[Nu]",
      TraditionalForm],
     TraditionalForm], 
    FormBox["p",
     TraditionalForm], 
    FormBox["q",
     TraditionalForm]}]], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1873]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t1", ",", " ", "p"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1874]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25292", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$25292", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm], 
     FormBox["q",
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1874]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"DiracSlash", "[", "p", "]"}]}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1875]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1875]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t2", ",", " ", "p"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1876]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["\<\"\[Gamma]\"\>",
    TraditionalForm], "\[CenterDot]", 
   FormBox["p",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1876]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{"t1", ",", " ", "p", ",", "q"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1877]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25302", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25301", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox["\[Epsilon]", 
    RowBox[{
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox["\[Nu]",
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$25302", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm], 
     FormBox[
      FormBox[
       RowBox[{"$AL$25301", "\[InvisibleApplication]", 
        RowBox[{"(", "1", ")"}]}],
       TraditionalForm],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1877]="]
}, Open  ]],

Cell["By default scalar products are not uncontracted.", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", " ", "q"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1878]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox["p",
    TraditionalForm], "\[CenterDot]", 
   FormBox["q",
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1878]="]
}, Open  ]],

Cell["\<\
With the option Pair\[Rule]All they are \[OpenCurlyDoubleQuote]uncontracted \
\[CloseCurlyDoubleQuote].\
\>", "Text",
 CellTags->"Uncontract"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1879]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25309", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25309", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1879]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{
    RowBox[{"ScalarProduct", "[", 
     RowBox[{"p", ",", "q"}], "]"}], "^", "2"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1880]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25314", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25314", "\[InvisibleApplication]", 
       RowBox[{"(", "2", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25314", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25314", "\[InvisibleApplication]", 
       RowBox[{"(", "2", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"Uncontract",
 CellLabel->"Out[1880]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"t1", ",", "t2"}], "]"}]], "Input",
 CellTags->"Uncontract",
 CellLabel->"In[1881]:="]
}, Open  ]],

Cell["UnDeclareNonCommutative", "ObjectName",
 CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],

Cell["\<\
UnDeclareNonCommutative[a, b, ...] undeclares a,b, ... to be noncommutative, \
i.e., DataType[a,b, ..., NonCommutative] is set to False.\
\>", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DataType",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DataType", All, CellTags],
  ButtonData->"DataType",
  ButtonNote->"DataType"],
 ", ",
 ButtonBox["DeclareNonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DeclareNonCommutative", All, CellTags],
  ButtonData->"DeclareNonCommutative",
  ButtonNote->"DeclareNonCommutative"],
 "."
}], "Text",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1882]:="],

Cell["\<\
As a side-effect of DeclareNonCommutative x is declared to be of DataType \
NonCommutative.\
\>", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[CellGroupData[{

Cell["DataType[x,NonCommutative]", "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1883]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1883]="]
}, Open  ]],

Cell["The inverse operation is UnDeclareNonCommutative.", "Text",
 CellTags->"UnDeclareNonCommutative"],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", "x", "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1884]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"x", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1885]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1885]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1886]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1887]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1887]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"UnDeclareNonCommutative", "[", 
  RowBox[{"y", ",", "z"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1888]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"DataType", "[", 
  RowBox[{"y", ",", "z", ",", "NonCommutative"}], "]"}]], "Input",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"In[1889]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"True", ",", "True"}], "}"}], TraditionalForm]], "Output",
 CellTags->"UnDeclareNonCommutative",
 CellLabel->"Out[1889]="]
}, Open  ]]
}, Open  ]],

Cell["Upper", "ObjectName",
 CellTags->"Upper"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Upper"],

Cell["\<\
Upper may be used inside LorentzIndex to indicate an contravariant \
LorentzIndex.\
\>", "Text",
 CellTags->"Upper"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["LorentzIndex",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "LorentzIndex", All, CellTags],
  ButtonData->"LorentzIndex",
  ButtonNote->"LorentzIndex"],
 ", ",
 ButtonBox["Lower",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Lower", All, CellTags],
  ButtonData->"Lower",
  ButtonNote->"Lower"],
 ", ",
 ButtonBox["Contract1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Contract1", All, CellTags],
  ButtonData->"Contract1",
  ButtonNote->"Contract1"],
 "."
}], "Text",
 CellTags->"Upper"]
}, Open  ]],

Cell["UVPart  ***unfinished*** (examples missing)", "ObjectName",
 CellTags->"UVPart"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"UVPart"],

Cell["\<\
UVPart[exp, q] discards ultraviolet finite integrals  (q = integration \
momentum).\
\>", "Text",
 CellTags->"UVPart"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"UVPart"],

Cell[BoxData[""], "Input",
 CellTags->"UVPart",
 CellLabel->"In[1890]:="]
}, Open  ]],

Cell["Vectors", "ObjectName",
 CellTags->"Vectors"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Vectors"],

Cell["\<\
Vectors is an option for FORM2FeynCalc. Its default setting is Automatic. It \
may be set to a list, if the FORM-file does not contain a V(ectors) statement.\
\>", "Text",
 CellTags->"Vectors"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FORM2FeynCalc",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FORM2FeynCalc", All, CellTags],
  ButtonData->"FORM2FeynCalc",
  ButtonNote->"FORM2FeynCalc"],
 "."
}], "Text",
 CellTags->"Vectors"]
}, Open  ]],

Cell["WriteOut", "ObjectName",
 CellTags->"WriteOut"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"WriteOut"],

Cell["\<\
WriteOut is an option for OneLoop.  If set to True, the result of OneLoop \
will be written to a file called \"name.res\", where name is the first \
argument of OneLoop.\
\>", "Text",
 CellTags->"WriteOut"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"WriteOut"]
}, Open  ]],

Cell["WriteOutPaVe", "ObjectName",
 CellTags->"WriteOutPaVe"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"WriteOutPaVe"],

Cell["\<\
WriteOutPaVe is an option for PaVeReduce and OneLoopSum. If set to a string, \
the results of all Passarino-Veltman PaVe's are stored in files with names \
generated from this string and the arguments of PaVe.\
\>", "Text",
 CellTags->"WriteOutPaVe"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["OneLoopSum",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoopSum", All, CellTags],
  ButtonData->"OneLoopSum",
  ButtonNote->"OneLoopSum"],
 "."
}], "Text",
 CellTags->"WriteOutPaVe"]
}, Open  ]],

Cell["Write2", "ObjectName",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Write2"],

Cell["\<\
Write2[file, val1 = expr1, val2 = expr2, ...] writes the settings val1 = \
expr1, val2 = expr2 in sequence followed by a newline, to the specified \
output file. Setting the option FormatType of Write2 to FortranForm results \
in FORTRAN syntax output.\
\>", "Text",
 CellTags->"Write2"],

Cell["\<\
The continuation character for FORTRAN output can be controlled by changing \
$FortranContinuationCharacter.\
\>", "Text",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"FullForm", "[", "$FortranContinuationCharacter", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1891]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   StyleBox["\"\<&\>\"",
    ShowSpecialCharacters->False,
    ShowStringCharacters->True,
    NumberMarks->True],
   FullForm], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1891]//FullForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "Write2", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1892]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"D0Convention", "\[Rule]", "0"}], ",", 
    RowBox[{"FinalSubstitutions", "\[Rule]", 
     RowBox[{"{", "}"}]}], ",", 
    RowBox[{"FormatType", "\[Rule]", "InputForm"}], ",", 
    RowBox[{"FortranFormatDoublePrecision", "\[Rule]", "True"}], ",", 
    RowBox[{"PageWidth", "\[Rule]", "62"}], ",", 
    RowBox[{"PostFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"PreFortranFile", "\[Rule]", "\<\"\"\>"}], ",", 
    RowBox[{"StringReplace", "\[Rule]", 
     RowBox[{"{", "}"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1892]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Attributes", "[", "Write2", "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1893]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "HoldRest", "}"}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1893]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Isolate",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Isolate", All, CellTags],
  ButtonData->"Isolate",
  ButtonNote->"Isolate"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 "."
}], "Text",
 CellTags->"Write2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t", "=", 
  RowBox[{"Collect", "[", 
   RowBox[{
    SuperscriptBox[
     RowBox[{"(", 
      RowBox[{
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"a", "-", "c"}], ")"}], "2"], "+", 
       SuperscriptBox[
        RowBox[{"(", 
         RowBox[{"a", "-", "b"}], ")"}], "2"]}], ")"}], "2"], ",", "a", ",", 
    "Factor"}], "]"}]}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1894]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["a", "4"]}], "-", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "3"], " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}]}], "+", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "2"], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      RowBox[{"b", " ", "c"}], "+", 
      SuperscriptBox["c", "2"]}], ")"}]}], "-", 
   RowBox[{"4", " ", "a", " ", 
    RowBox[{"(", 
     RowBox[{"b", "+", "c"}], ")"}], " ", 
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      SuperscriptBox["c", "2"]}], ")"}]}], "+", 
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{
      SuperscriptBox["b", "2"], "+", 
      SuperscriptBox["c", "2"]}], ")"}], "2"]}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1894]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"tempfilename", "=", 
   RowBox[{
    RowBox[{"ToString", "[", "$SessionID", "]"}], "<>", "\"\<.s\>\""}]}], 
  ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1895]:="],

Cell["This writes the assignment r=t to a file.", "Text",
 CellTags->"Write2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t"}]}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1896]:="],

Cell["This shows the contents of the file.", "Text",
 CellTags->"Write2"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1897]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"r = ( 4*a^4 - 8*a^3*(b + c) - 4*a*(b + c)*(b^2 + c^2) + \"\>"},
      {"\<\" (b^2 + c^2)^2 + 8*a^2*(b^2 + b*c + c^2)\"\>"},
      {"\<\"       );\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1897]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
     ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1898]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"t2", "=", 
  RowBox[{"x", "+", 
   RowBox[{"Isolate", "[", 
    RowBox[{"t", ",", "a", ",", 
     RowBox[{"IsolateNames", "\[Rule]", "W"}]}], "]"}]}]}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1899]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{"4", " ", 
    SuperscriptBox["a", "4"]}], "-", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "3"], " ", 
    TagBox[
     RowBox[{"W", "(", "245", ")"}],
     HoldForm]}], "+", 
   RowBox[{"8", " ", 
    SuperscriptBox["a", "2"], " ", 
    TagBox[
     RowBox[{"W", "(", "247", ")"}],
     HoldForm]}], "-", 
   RowBox[{"4", " ", "a", " ", 
    TagBox[
     RowBox[{"W", "(", "245", ")"}],
     HoldForm], " ", 
    TagBox[
     RowBox[{"W", "(", "246", ")"}],
     HoldForm]}], "+", 
   SuperscriptBox[
    TagBox[
     RowBox[{"W", "(", "246", ")"}],
     HoldForm], "2"], "+", "x"}], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1899]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t2"}]}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1900]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1901]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"W[245] = (b + c\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"W[246] = (b^2 + c^2\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"W[247] = (b^2 + b*c + c^2\"\>"},
      {"\<\"       );\"\>"},
      {"\<\"r = ( 4*a^4 + x - 8*a^3*HoldForm[W[245]] - \"\>"},
      {"\<\" 4*a*HoldForm[W[245]]*HoldForm[W[246]] + HoldForm[W[246]]^2 + \
\"\>"},
      {"\<\" 8*a^2*HoldForm[W[247]]\"\>"},
      {"\<\"       );\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1901]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"DeleteFile", "[", 
  RowBox[{
   RowBox[{"If", "[", 
    RowBox[{
     RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
     ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1902]:="],

Cell["\<\
This is how to write out the expression t2 in Fortran format.\
\>", "Text",
 CellTags->"Write2"],

Cell[BoxData[
 RowBox[{
  RowBox[{"Write2", "[", 
   RowBox[{"tempfilename", ",", 
    RowBox[{"r", "=", "t2"}], ",", 
    RowBox[{"FormatType", "\[Rule]", "FortranForm"}]}], "]"}], ";"}]], "Input",\

 CellTags->"Write2",
 CellLabel->"In[1903]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TableForm", "[", 
  RowBox[{"ReadList", "[", 
   RowBox[{
    RowBox[{
     RowBox[{"If", "[", 
      RowBox[{
       RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"",
        ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], ",", "String"}], 
   "]"}], "]"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1904]:="],

Cell[BoxData[
 FormBox[
  TagBox[
   TagBox[GridBox[{
      {"\<\"        W(245D2)= W(245D2)\"\>"},
      {"\<\"        W(246D2)= W(246D2)\"\>"},
      {"\<\"        W(247D2)= W(247D2)\"\>"},
      {"\<\"        r = 4D0*a**4 + x - 8D0*a**3*W(245D2) - \"\>"},
      {"\<\"     &  4D0*a*W(245D2)*W(246D2) + W(246D2)**2 + \"\>"},
      {"\<\"     &  8D0*a**2*W(247D2)\"\>"}
     },
     GridBoxAlignment->{
      "Columns" -> {{Left}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
       "RowsIndexed" -> {}},
     GridBoxSpacings->{"Columns" -> {
         Offset[0.27999999999999997`], {
          Offset[0.5599999999999999]}, 
         Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
         Offset[0.2], {
          Offset[0.4]}, 
         Offset[0.2]}, "RowsIndexed" -> {}}],
    Column],
   Function[BoxForm`e$, 
    TableForm[BoxForm`e$]]], TraditionalForm]], "Output",
 CellTags->"Write2",
 CellLabel->"Out[1904]//TableForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"DeleteFile", "[", 
   RowBox[{
    RowBox[{"If", "[", 
     RowBox[{
      RowBox[{"$OperatingSystem", "===", "\"\<MacOS\>\""}], ",", "\"\<:\>\"", 
      ",", "\"\<\>\""}], "]"}], "<>", "tempfilename"}], "]"}], ";", 
  RowBox[{"Clear", "[", 
   RowBox[{"t", ",", "t2", ",", "tempfilename"}], "]"}], ";"}]], "Input",
 CellTags->"Write2",
 CellLabel->"In[1905]:="]
}, Open  ]],

Cell["XYT", "ObjectName",
 CellTags->"XYT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"XYT"],

Cell[TextData[{
 "XYT[exp, x,y] transforms  ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    RowBox[{"(", 
     RowBox[{"x", " ", "y"}], ")"}], "m"], TraditionalForm]]],
 " away ..."
}], "Text",
 CellTags->"XYT"]
}, Open  ]],

Cell["ZeroMomentumInsertion", "ObjectName",
 CellTags->"ZeroMomentumInsertion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"ZeroMomentumInsertion"],

Cell["\<\
ZeroMomentumInsertion is an option of FeynRule, Twist2GluonOperator and \
Twist2QuarkOperator.\
\>", "Text",
 CellTags->"ZeroMomentumInsertion"],

Cell[TextData[{
 "See also:  ",
 ButtonBox["FeynRule",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynRule", All, CellTags],
  ButtonData->"FeynRule",
  ButtonNote->"FeynRule"],
 ", ",
 ButtonBox["Twist2GluonOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2GluonOperator", All, CellTags],
  ButtonData->"Twist2GluonOperator",
  ButtonNote->"Twist2GluonOperator"],
 ", ",
 ButtonBox["Twist2QuarkOperator",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Twist2QuarkOperator", All, CellTags],
  ButtonData->"Twist2QuarkOperator",
  ButtonNote->"Twist2QuarkOperator"],
 "."
}], "Text",
 CellTags->"ZeroMomentumInsertion"]
}, Open  ]],

Cell["Zeta2", "ObjectName",
 CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"Zeta2"],

Cell["Zeta2 denotes Zeta[2]. N[Zeta[2]] evaluates to N[Zeta[2]].", "Text",
 CellTags->"Zeta2"],

Cell[TextData[{
 "See also: ",
 ButtonBox["SimplifyPolyLog",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "SimplifyPolyLog", All, CellTags],
  ButtonData->"SimplifyPolyLog",
  ButtonNote->"SimplifyPolyLog"],
 "."
}], "Text",
 CellTags->"Zeta2"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"Zeta2"],

Cell[CellGroupData[{

Cell[BoxData["Zeta2"], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1906]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"\[Zeta]", "(", "2", ")"}], TraditionalForm]], "Output",
 CellTags->"Zeta2",
 CellLabel->"Out[1906]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"N", "[", "Zeta2", "]"}]], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1907]:="],

Cell[BoxData[
 FormBox["1.6449340668482262`", TraditionalForm]], "Output",
 CellTags->"Zeta2",
 CellLabel->"Out[1907]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"SimplifyPolyLog", "[", 
  RowBox[{"Pi", "^", "2"}], "]"}]], "Input",
 CellTags->"Zeta2",
 CellLabel->"In[1908]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"6", " ", 
   RowBox[{"\[Zeta]", "(", "2", ")"}]}], TraditionalForm]], "Output",
 CellTags->"Zeta2",
 CellLabel->"Out[1908]="]
}, Open  ]]
}, Open  ]],

Cell["$Abbreviations", "ObjectName",
 CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Abbreviations"],

Cell["\<\
$Abbreviations are a list of string substitution rules used when generating \
names for storing intermediate results. It is used by OneLoop and \
PaVeReduce.The elements of the list should be of the form \"name\" \[Rule] \
\"abbreviation\".\
\>", "Text",
 CellTags->"$Abbreviations"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$Abbreviations", "//", "StandardForm"}]], "Input",
 CellTags->"$Abbreviations",
 CellLabel->"In[1909]:="],

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"\<\", \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"^\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"{\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"/\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"Subscript\"\>", "\[Rule]", "\<\"su\"\>"}], ",", 
   RowBox[{"\<\"SmallVariable\"\>", "\[Rule]", "\<\"sma\"\>"}], ",", 
   RowBox[{"\<\"}\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"[\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"]\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"*\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\" \"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"\\n\"\>", "\[Rule]", "\<\"\"\>"}], ",", 
   RowBox[{"\<\"\\r\"\>", "\[Rule]", "\<\"\"\>"}]}], "}"}]], "Output",
 CellTags->"$Abbreviations",
 CellLabel->"Out[1909]//StandardForm="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Abbreviation",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Abbreviation", All, CellTags],
  ButtonData->"Abbreviation",
  ButtonNote->"Abbreviation"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["WriteOut",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "WriteOut", All, CellTags],
  ButtonData->"WriteOut",
  ButtonNote->"WriteOut"],
 ", ",
 ButtonBox["WriteOutPaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "WriteOutPaVe", All, CellTags],
  ButtonData->"WriteOutPaVe",
  ButtonNote->"WriteOutPaVe"],
 "."
}], "Text",
 CellTags->"$Abbreviations"]
}, Open  ]],

Cell["$AL", "ObjectName",
 CellTags->"$AL"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$AL"],

Cell["\<\
$AL is the head for dummy indices which may be introduced by Uncontract.\
\>", "Text",
 CellTags->"$AL"],

Cell[CellGroupData[{

Cell[BoxData["$AL"], "Input",
 CellTags->"$AL",
 CellLabel->"In[1910]:="],

Cell[BoxData[
 FormBox["$AL", TraditionalForm]], "Output",
 CellTags->"$AL",
 CellLabel->"Out[1910]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1911]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25394", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"$AL$25394", "\[InvisibleApplication]", 
       RowBox[{"(", "1", ")"}]}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"$AL",
 CellLabel->"Out[1911]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$AL", "=", "\[Mu]"}], ";"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1912]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Uncontract", "[", 
  RowBox[{
   RowBox[{"ScalarProduct", "[", 
    RowBox[{"p", ",", "q"}], "]"}], ",", "q", ",", 
   RowBox[{"Pair", "\[Rule]", "All"}]}], "]"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1913]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"\[Mu]$25398", "(", "1", ")"}],
      TraditionalForm],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["q",
     TraditionalForm], 
    FormBox[
     FormBox[
      RowBox[{"\[Mu]$25398", "(", "1", ")"}],
      TraditionalForm],
     TraditionalForm]]}], TraditionalForm]], "Output",
 CellTags->"$AL",
 CellLabel->"Out[1913]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$AL", "=."}], ";"}]], "Input",
 CellTags->"$AL",
 CellLabel->"In[1914]:="]
}, Open  ]],

Cell["$BreitMaison", "ObjectName",
 CellTags->"$BreitMaison"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$BreitMaison"],

Cell[TextData[{
 "The Breitenlohner-Maison ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " scheme is currently not supported by the Dirac algebra functions.Use \
Tracer if you need it (PHI supplies the functions FCToTracer and \
TracerToFC)."
}], "Text",
 CellTags->"$BreitMaison"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["$Larin",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$Larin", All, CellTags],
  ButtonData->"$Larin",
  ButtonNote->"$Larin"],
 ", ",
 ButtonBox["$West",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$West", All, CellTags],
  ButtonData->"$West",
  ButtonNote->"$West"],
 "."
}], "Text",
 CellTags->"$BreitMaison"]
}, Open  ]],

Cell["$Color", "ObjectName",
 CellTags->"$Color"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Color"],

Cell["\<\
$Color is False by default. If set to True, some special variables will be \
colored.\
\>", "Text",
 CellTags->"$Color"]
}, Open  ]],

Cell["$Covariant", "ObjectName",
 CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Covariant"],

Cell["\<\
The boolean setting of $Covariant determines whether lorentz indices are \
displayed as lower indices (True) or as upper ones (False).\
\>", "Text",
 CellTags->"$Covariant"],

Cell[CellGroupData[{

Cell[BoxData["$Covariant"], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1915]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$Covariant",
 CellLabel->"Out[1915]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1916]:="],

Cell[BoxData[
 FormBox[
  SuperscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"$Covariant",
 CellLabel->"Out[1916]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Covariant", "=", "True"}], ";"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1917]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"GA", "[", "\[Mu]", "]"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1918]:="],

Cell[BoxData[
 FormBox[
  SubscriptBox["\[Gamma]", "\[Mu]"], TraditionalForm]], "Output",
 CellTags->"$Covariant",
 CellLabel->"Out[1918]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$Covariant", "=", "False"}], ";"}]], "Input",
 CellTags->"$Covariant",
 CellLabel->"In[1919]:="]
}, Open  ]],

Cell["$FCS", "ObjectName",
 CellTags->"$FCS"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FCS"],

Cell["\<\
$FCS is a list of functions with a short name. E.g. GA[nu] can be used \
instead of DiracGamma[nu].\
\>", "Text",
 CellTags->"$FCS"],

Cell[CellGroupData[{

Cell[BoxData["$FCS"], "Input",
 CellTags->"$FCS",
 CellLabel->"In[1920]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"\<\"FAD\"\>", ",", "\<\"FV\"\>", ",", "\<\"FVD\"\>", 
    ",", "\<\"GA\"\>", ",", "\<\"GA5\"\>", ",", "\<\"GS\"\>", 
    ",", "\<\"GSD\"\>", ",", "\<\"LC\"\>", ",", "\<\"LCD\"\>", 
    ",", "\<\"MT\"\>", ",", "\<\"MTD\"\>", ",", "\<\"SD\"\>", 
    ",", "\<\"SOD\"\>", ",", "\<\"SP\"\>", ",", "\<\"SPC\"\>", 
    ",", "\<\"SPD\"\>", ",", "\<\"SPL\"\>", ",", "\<\"FCI\"\>", 
    ",", "\<\"FCE\"\>", ",", "\<\"FI\"\>", ",", "\<\"FC\"\>", 
    ",", "\<\"GGV\"\>", ",", "\<\"GP\"\>", ",", "\<\"QGV\"\>", 
    ",", "\<\"QO\"\>", ",", "\<\"FDr\"\>", ",", "\<\"CDr\"\>"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"$FCS",
 CellLabel->"Out[1920]="]
}, Open  ]]
}, Open  ]],

Cell["$FCT", "ObjectName",
 CellTags->"$FCT"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FCT"],

Cell["\<\
Setting the global variable $FCT to True changes the default typesetting \
rules for the noncommutative multiplication operator Dot[].\
\>", "Text",
 CellTags->"$FCT"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$FCT", " ", "=", " ", "True"}], ";"}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1921]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
  RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1922]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], " ", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"$FCT",
 CellLabel->"Out[1922]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$FCT", " ", "=", " ", "False"}], ";"}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1923]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"GA", "[", "\[Mu]", "]"}], " ", ".", " ", 
  RowBox[{"GA", "[", "\[Nu]", "]"}]}]], "Input",
 CellTags->"$FCT",
 CellLabel->"In[1924]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", "\[Mu]"], ".", 
   SuperscriptBox["\[Gamma]", "\[Nu]"]}], TraditionalForm]], "Output",
 CellTags->"$FCT",
 CellLabel->"Out[1924]="]
}, Open  ]]
}, Open  ]],

Cell["$FeynCalcStuff", "ObjectName",
 CellTags->"$FeynCalcStuff"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FeynCalcStuff"],

Cell["$FeynCalcStuff is the list of availabe stuff in FeynCalc.", "Text",
 CellTags->"$FeynCalcStuff"]
}, Open  ]],

Cell["$FeynCalcVersion", "ObjectName",
 CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FeynCalcVersion"],

Cell["\<\
$FeynCalcVersion is a string that represents the version of FeynCalc.\
\>", "Text",
 CellTags->"$FeynCalcVersion"],

Cell[CellGroupData[{

Cell[BoxData["$FeynCalcVersion"], "Input",
 CellTags->"$FeynCalcVersion",
 CellLabel->"In[1925]:="],

Cell[BoxData[
 FormBox["\<\"8.1.0\"\>", TraditionalForm]], "Output",
 CellTags->"$FeynCalcVersion",
 CellLabel->"Out[1925]="]
}, Open  ]]
}, Open  ]],

Cell["$FortranContinuationCharacter", "ObjectName",
 CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$FortranContinuationCharacter"],

Cell["\<\
$FortranContinuationCharacter is the continuation character used in Write2.\
\>", "Text",
 CellTags->"$FortranContinuationCharacter"],

Cell[CellGroupData[{

Cell[BoxData["$FortranContinuationCharacter"], "Input",
 CellTags->"$FortranContinuationCharacter",
 CellLabel->"In[1926]:="],

Cell[BoxData[
 FormBox["\<\"&\"\>", TraditionalForm]], "Output",
 CellTags->"$FortranContinuationCharacter",
 CellLabel->"Out[1926]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Write2",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Write2", All, CellTags],
  ButtonData->"Write2",
  ButtonNote->"Write2"],
 "."
}], "Text",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],

Cell["$Gauge", "ObjectName",
 CellTags->"$Gauge"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Gauge"],

Cell["\<\
$Gauge(= 1/xi) is a constant specifying the gauge fixing parameter of QED in \
Lorentz gauge.  The usual choice is Feynman gauge, $Gauge=1. Notice that \
$Gauge is used by some functions, the option Gauge by others.\
\>", "Text",
 CellTags->"$Gauge"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Gauge",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Gauge", All, CellTags],
  ButtonData->"Gauge",
  ButtonNote->"Gauge"],
 "."
}], "Text",
 CellTags->"$Gauge"]
}, Open  ]],

Cell["$Larin", "ObjectName",
 CellTags->"$Larin"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$Larin"],

Cell[TextData[{
 "If set to True, the Larin-Gorishny-Atkyampo-DelBurgo-scheme for ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensions is used, i.e. before evaluating traces (but after moving ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " anticommuting in all dimensions to the right of the Dirac string) a \
product  gamma[mu].gamma5 is substituted to  -I/6 Eps[mu,al,be,si] \
gamma[al,be,si], where all indices live in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensions now. Especially the Levic-Civita tensor is taken to be ",
 StyleBox["D",
  FontSlant->"Italic"],
 "-dimensional, i.e., contraction of two Eps's results in ",
 StyleBox["D",
  FontSlant->"Italic"],
 "'s. This has (FOR ONE AXIAL-VECTOR-CURRENT ONLY, it is not so clear if this \
scheme also works for more than one fermion line involving ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 ") the same effect as the Breitenlohner-Maison-'t Hooft-Veltman scheme."
}], "Text",
 CellTags->"$Larin"],

Cell[CellGroupData[{

Cell[BoxData["$Larin"], "Input",
 CellTags->"$Larin",
 CellLabel->"In[1927]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$Larin",
 CellLabel->"Out[1927]="]
}, Open  ]],

Cell[TextData[{
 "See also:  ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 ", ",
 ButtonBox["$BreitMaison",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$BreitMaison", All, CellTags],
  ButtonData->"$BreitMaison",
  ButtonNote->"$BreitMaison"],
 ", ",
 ButtonBox["$West",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$West", All, CellTags],
  ButtonData->"$West",
  ButtonNote->"$West"],
 "."
}], "Text",
 CellTags->"$Larin"]
}, Open  ]],

Cell["$LimitTo4", "ObjectName",
 CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$LimitTo4"],

Cell["\<\
$LimitTo4 is a global variable with default setting True. If set to False, no \
limit Dimension \[Rule] 4 is performed after tensor integral decomposition.\
\>", "Text",
 CellTags->"$LimitTo4"],

Cell[CellGroupData[{

Cell[BoxData["$LimitTo4"], "Input",
 CellTags->"$LimitTo4",
 CellLabel->"In[1928]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"$LimitTo4",
 CellLabel->"Out[1928]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["PaVe",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVe", All, CellTags],
  ButtonData->"PaVe",
  ButtonNote->"PaVe"],
 ", ",
 ButtonBox["PaVeReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "PaVeReduce", All, CellTags],
  ButtonData->"PaVeReduce",
  ButtonNote->"PaVeReduce"],
 ", ",
 ButtonBox["OneLoop",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OneLoop", All, CellTags],
  ButtonData->"OneLoop",
  ButtonNote->"OneLoop"],
 "."
}], "Text",
 CellTags->"$LimitTo4"]
}, Open  ]],

Cell["$LorentzIndices", "ObjectName",
 CellTags->"$LorentzIndices"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$LorentzIndices"],

Cell["\<\
$LorentzIndices is a global variable. If set to True the dimension of \
LorentzIndex is displayed as an index.\
\>", "Text",
 CellTags->"$LorentzIndices"],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "True"}], ";"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1929]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}], 
  RowBox[{"DiracMatrix", "[", 
   RowBox[{"\[Alpha]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1930]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Alpha]",
       TraditionalForm], 
      FormBox["n",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["n",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1930]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1931]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "n"}], "]"}], ",", "n"}], "]"}], " ", 
  RowBox[{"MetricTensor", "[", 
   RowBox[{"\[Alpha]", ",", "\[Beta]", ",", 
    RowBox[{"Dimension", "\[Rule]", "n"}]}], "]"}]}]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1931]//StandardForm="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"MTD", "[", 
    RowBox[{"\[Alpha]", ",", "\[Beta]"}], "]"}], " ", 
   RowBox[{"FVD", "[", 
    RowBox[{"p", ",", "\[Beta]"}], "]"}], 
   RowBox[{"GAD", "[", "\[Mu]", "]"}]}], "//", "FCI"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1932]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["\[Gamma]", 
    FormBox[
     SubscriptBox[
      FormBox["\[Mu]",
       TraditionalForm], 
      FormBox["D",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox[
    FormBox["p",
     TraditionalForm], 
    FormBox[
     SubscriptBox[
      FormBox["\[Beta]",
       TraditionalForm], 
      FormBox["D",
       TraditionalForm]],
     TraditionalForm]], " ", 
   SuperscriptBox["g", 
    RowBox[{
     FormBox[
      SubscriptBox[
       FormBox["\[Alpha]",
        TraditionalForm], 
       FormBox["D",
        TraditionalForm]],
      TraditionalForm], 
     FormBox[
      SubscriptBox[
       FormBox["\[Beta]",
        TraditionalForm], 
       FormBox["D",
        TraditionalForm]],
      TraditionalForm]}]]}], TraditionalForm]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1932]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "StandardForm"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1933]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"DiracGamma", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Mu]", ",", "D"}], "]"}], ",", "D"}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Alpha]", ",", "D"}], "]"}], ",", 
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "D"}], "]"}]}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", 
     RowBox[{"\[Beta]", ",", "D"}], "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"p", ",", "D"}], "]"}]}], "]"}]}]], "Output",
 CellTags->"$LorentzIndices",
 CellLabel->"Out[1933]//StandardForm="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$LorentzIndices", "=", "False"}], ";"}]], "Input",
 CellTags->"$LorentzIndices",
 CellLabel->"In[1934]:="]
}, Open  ]],

Cell["$MIntegrate", "ObjectName",
 CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MIntegrate"],

Cell["\<\
$MIntegrate is a global list of integrations done by Mathematica inside \
OPEIntDelta.\
\>", "Text",
 CellTags->"$MIntegrate"],

Cell[CellGroupData[{

Cell[BoxData["$MIntegrate"], "Input",
 CellTags->"$MIntegrate",
 CellLabel->"In[1935]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", "}"}], TraditionalForm]], "Output",
 CellTags->"$MIntegrate",
 CellLabel->"Out[1935]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["OPEIntDelta",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "OPEIntDelta", All, CellTags],
  ButtonData->"OPEIntDelta",
  ButtonNote->"OPEIntDelta"],
 "."
}], "Text",
 CellTags->"$MIntegrate"]
}, Open  ]],

Cell["$MU", "ObjectName",
 CellTags->"$MU"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MU"],

Cell["\<\
$MU is the head for dummy indices which may be introduced by Chisholm (and \
evtl. Contract and DiracReduce).\
\>", "Text",
 CellTags->"$MU"],

Cell[CellGroupData[{

Cell[BoxData["$MU"], "Input",
 CellTags->"$MU",
 CellLabel->"In[1936]:="],

Cell[BoxData[
 FormBox["$MU", TraditionalForm]], "Output",
 CellTags->"$MU",
 CellLabel->"Out[1936]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Chisholm",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Chisholm", All, CellTags],
  ButtonData->"Chisholm",
  ButtonNote->"Chisholm"],
 "."
}], "Text",
 CellTags->"$MU"]
}, Open  ]],

Cell["$MemoryAvailable", "ObjectName",
 CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$MemoryAvailable"],

Cell[TextData[{
 "$MemoryAvailable is  a global variable which is set to an integer ",
 StyleBox["n",
  FontSlant->"Italic"],
 ", where ",
 StyleBox["n",
  FontSlant->"Italic"],
 " is the available amount of main memory in MB. The default is 128. It \
should be increased if possible. The higher $MemoryAvailable can be, the more \
intermediate steps do not have to be repeated by FeynCalc."
}], "Text",
 CellTags->"$MemoryAvailable"],

Cell[CellGroupData[{

Cell[BoxData["$MemoryAvailable"], "Input",
 CellTags->"$MemoryAvailable",
 CellLabel->"In[1937]:="],

Cell[BoxData[
 FormBox["1024", TraditionalForm]], "Output",
 CellTags->"$MemoryAvailable",
 CellLabel->"Out[1937]="]
}, Open  ]]
}, Open  ]],

Cell["$NonComm", "ObjectName",
 CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$NonComm"],

Cell["\<\
$NonComm contains a list of all non-commutative heads present.\
\>", "Text",
 CellTags->"$NonComm"],

Cell[CellGroupData[{

Cell[BoxData["$NonComm"], "Input",
 CellTags->"$NonComm",
 CellLabel->"In[1938]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "ChiralityProjector", ",", "DiracGamma", ",", "DiracGammaT", ",", 
    "DiracMatrix", ",", "DiracSigma", ",", "DiracSlash", ",", "Spinor", ",", 
    "GAD", ",", "GA", ",", "GSD", ",", "GS", ",", "LeftPartialD", ",", 
    "LeftRightPartialD2", ",", "LeftRightPartialD", ",", "PartialD", ",", 
    "PauliSigma", ",", "QuantumField", ",", "RightPartialD", ",", 
    "SpinorUBar", ",", "SpinorU", ",", "SpinorVBar", ",", "SpinorV", ",", 
    "SUNT", ",", "FieldStrength", ",", "QuarkGluonVertex", ",", 
    "QuarkPropagator", ",", "C", ",", "Twist2QuarkOperator", ",", 
    "CovariantD", ",", "OPESum", ",", "Twist2AlienOperator", ",", 
    "Twist2CounterOperator"}], "}"}], TraditionalForm]], "Output",
 CellTags->"$NonComm",
 CellLabel->"Out[1938]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["NonCommQ",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "NonCommQ", All, CellTags],
  ButtonData->"NonCommQ",
  ButtonNote->"NonCommQ"],
 ", ",
 ButtonBox["NonCommutative",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "NonCommutative", All, CellTags],
  ButtonData->"NonCommutative",
  ButtonNote->"NonCommutative"],
 "."
}], "Text",
 CellTags->"$NonComm"]
}, Open  ]],

Cell["$PairBrackets", "ObjectName",
 CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$PairBrackets"],

Cell["\<\
$PairBrackets determines whether brackets are drawn around scalar products in \
the notebook interface.\
\>", "Text",
 CellTags->"$PairBrackets"],

Cell[CellGroupData[{

Cell[BoxData["$PairBrackets"], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1939]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1939]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], 
  RowBox[{"SP", "[", 
   RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1940]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm]}],
    TraditionalForm], " ", 
   FormBox[
    RowBox[{
     FormBox["r",
      TraditionalForm], "\[CenterDot]", 
     FormBox["s",
      TraditionalForm]}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1940]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$PairBrackets", "=", "True"}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1941]:="],

Cell[BoxData[
 FormBox["True", TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1941]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"SP", "[", 
   RowBox[{"p", ",", "q"}], "]"}], 
  RowBox[{"SP", "[", 
   RowBox[{"r", ",", "s"}], "]"}]}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1942]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   FormBox[
    RowBox[{"(", 
     FormBox["p",
      TraditionalForm], "\[CenterDot]", 
     FormBox["q",
      TraditionalForm], ")"}],
    TraditionalForm], " ", 
   FormBox[
    RowBox[{"(", 
     FormBox["r",
      TraditionalForm], "\[CenterDot]", 
     FormBox["s",
      TraditionalForm], ")"}],
    TraditionalForm]}], TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1942]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"$PairBrackets", "=", "False"}]], "Input",
 CellTags->"$PairBrackets",
 CellLabel->"In[1943]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$PairBrackets",
 CellLabel->"Out[1943]="]
}, Open  ]]
}, Open  ]],

Cell["$SpinorMinimal", "ObjectName",
 CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$SpinorMinimal"],

Cell["\<\
$SpinorMinimal is a global switch for an additional simplification attempt in \
DiracSimplify for more than one Spinor-line. The default is False, since \
otherwise it costs too much time.\
\>", "Text",
 CellTags->"$SpinorMinimal"],

Cell[CellGroupData[{

Cell[BoxData["$SpinorMinimal"], "Input",
 CellTags->"$SpinorMinimal",
 CellLabel->"In[1944]:="],

Cell[BoxData[
 FormBox["False", TraditionalForm]], "Output",
 CellTags->"$SpinorMinimal",
 CellLabel->"Out[1944]="]
}, Open  ]],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["DiracSimplify",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "DiracSimplify", All, CellTags],
  ButtonData->"DiracSimplify",
  ButtonNote->"DiracSimplify"],
 "."
}], "Text",
 CellTags->"$SpinorMinimal"]
}, Open  ]],

Cell["$VeryVerbose", "ObjectName",
 CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$VeryVerbose"],

Cell["\<\
$VeryVerbose is a global variable with default setting 0. If set to 1, 2, \
..., less and more intermediate comments and informations are displayed \
during calculations.\
\>", "Text",
 CellTags->"$VeryVerbose"],

Cell[CellGroupData[{

Cell[BoxData["$VeryVerbose"], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1945]:="],

Cell[BoxData[
 FormBox["0", TraditionalForm]], "Output",
 CellTags->"$VeryVerbose",
 CellLabel->"Out[1945]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$VeryVerbose", " ", "=", " ", "3"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1946]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Collect2", "[", 
   RowBox[{
    RowBox[{"Expand", "[", 
     RowBox[{
      RowBox[{"(", 
       RowBox[{"x", "-", "y", "-", "z"}], ")"}], "^", "6"}], "]"}], ",", 
    "x"}], "]"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1947]:="],

Cell["\<\
expanding. inhomogeneous part; LeafCount = 43; factored. #1#2#3#4#5#6\
\>", "Print",
 CellTags->"$VeryVerbose"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"length \"\>", "\[InvisibleSpace]", "21", 
    "\[InvisibleSpace]", "\<\".\"\>"}],
   SequenceForm["length ", 21, "."],
   Editable->False], TraditionalForm]], "Print",
 CellTags->"$VeryVerbose"],

Cell[BoxData[
 FormBox[
  InterpretationBox[
   RowBox[{"\<\"collecting \"\>", "\[InvisibleSpace]", "6", 
    "\[InvisibleSpace]", "\<\" terms.\"\>"}],
   SequenceForm["collecting ", 6, " terms."],
   Editable->False], TraditionalForm]], "Print",
 CellTags->"$VeryVerbose"]
}, Open  ]],

Cell[BoxData[
 RowBox[{
  RowBox[{"$VeryVerbose", "=", "0"}], ";"}]], "Input",
 CellTags->"$VeryVerbose",
 CellLabel->"In[1948]:="],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["$V0",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "$V0", All, CellTags],
  ButtonData->"$V0",
  ButtonNote->"$V0"],
 "."
}], "Text",
 CellTags->"$V0"]
}, Open  ]],

Cell["$West", "ObjectName",
 CellTags->"$West"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"$West"],

Cell[TextData[{
 "If $West is set to True (which is the default), traces involving more than \
4 Dirac matrices and ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 " are calculated recursively according to formula (A.5) from Comp. Phys. \
Comm 77 (1993) 286-298, which is based on the Breitenlohner Maison ",
 Cell[BoxData[
  FormBox[
   SubscriptBox["\[Gamma]", "5"], TraditionalForm]]],
 "-scheme.\nFrom FeynCalc 8.0.2 onwards this variable cannot be set to False \
(since it is not working at the moment)."
}], "Text",
 CellTags->"$West"],

Cell[TextData[{
 "See examples in ",
 ButtonBox["Tr",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Tr", All, CellTags],
  ButtonData->"Tr",
  ButtonNote->"Tr"],
 "."
}], "Text",
 CellTags->"$West"],

Cell["Evaluation time and memory usage", "Commentary"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"AbsoluteTime", "[", "]"}], "-", "absolutestarttime"}]], "Input",
 CellLabel->"In[1949]:="],

Cell[BoxData[
 FormBox["224.955713`8.803642020527986", TraditionalForm]], "Output",
 CellLabel->"Out[1949]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MemoryInUse", "[", "]"}]], "Input",
 CellLabel->"In[1950]:="],

Cell[BoxData[
 FormBox["140171088", TraditionalForm]], "Output",
 CellLabel->"Out[1950]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"MaxMemoryUsed", "[", "]"}]], "Input",
 CellLabel->"In[1951]:="],

Cell[BoxData[
 FormBox["140463664", TraditionalForm]], "Output",
 CellLabel->"Out[1951]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"TimeUsed", "[", "]"}]], "Input",
 CellLabel->"In[1952]:="],

Cell[BoxData[
 FormBox["56.363523`", TraditionalForm]], "Output",
 CellLabel->"Out[1952]="]
}, Open  ]]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Functions (experimental)", "Section",
 CellTags->"Experimental"],

Cell["DoPolarizationSums", "ObjectName",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"DoPolarizationSums"],

Cell["\<\
***EXPERIMENTAL***
DoPolarizationSums[exp] sums over vector polarizations for expressions with a \
factor of the form
Pair[LorentzIndex[rho1_], Momentum[Polarization[p_, -I]]]*
Pair[LorentzIndex[rho2_], Momentum[Polarization[p_, I]]].\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["Polarization",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Polarization", All, CellTags],
  ButtonData->"Polarization",
  ButtonNote->"Polarization"],
 ", ",
 ButtonBox["Uncontract",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "Uncontract", All, CellTags],
  ButtonData->"Uncontract",
  ButtonNote->"Uncontract"],
 "."
}], "Text",
 CellTags->"DoPolarizationSums"]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"DoPolarizationSums"],

Cell["\<\
In the following (and indeed everywhere else within FeynCalc), notice that \
Lorentz indices written as super or subscripts are not to be taken as such. \
Instead, by convention, when two indices are contracted one is always lower \
and the other upper.\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell["\<\
FeynCalc uses the following normalization of Polarization vectors:\
\>", "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"Polarization", "[", 
      RowBox[{"p4", ",", 
       RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], " ", 
  RowBox[{"Pair", "[", 
   RowBox[{
    RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
    RowBox[{"Momentum", "[", 
     RowBox[{"Polarization", "[", 
      RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], "]"}]}]], "Input",\

 CellTags->"DoPolarizationSums",
 CellLabel->"In[1953]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   RowBox[{
    SubsuperscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "*"], "(", "p4", ")"}], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm]], "(", "p4", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1953]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1954]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1954]="]
}, Open  ]],

Cell[TextData[{
 "DoPolarizationSums is chosen in such a way as to be consistent with this \
normalization and with ",
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", "Polarizations"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{
    RowBox[{
     SubscriptBox["\[CurlyEpsilon]", 
      FormBox[
       FormBox["\[Mu]",
        TraditionalForm],
       TraditionalForm]], "(", 
     FormBox[
      SubscriptBox[
       TagBox[
        StyleBox["\<\"p\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ], "4"],
      TraditionalForm], ")"}], " ", 
    RowBox[{
     SubscriptBox[
      RowBox[{"(", 
       SuperscriptBox["\[CurlyEpsilon]", "\[Nu]"], ")"}], 
      FormBox["",
       TraditionalForm]], "(", 
     FormBox[
      SubscriptBox[
       TagBox[
        StyleBox["\<\"p\"\>",
         FontSlant->"Italic"],
        StyleForm[#, FontSlant -> "Italic"]& ], "4"],
      TraditionalForm], ")"}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "\[Mu]"], 
      RowBox[{"\[NoBreak]", 
       FormBox["\[Nu]",
        TraditionalForm]}]],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 "We can choose e.g. the following polarization vectors, labeled with a \
subscript, {",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "1"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "2"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "3"], TraditionalForm]]],
 ",",
 Cell[BoxData[
  FormBox[
   SubscriptBox["e", "4"], TraditionalForm]]],
 "} = {(\[ImaginaryI],0,0,0), (0,\[ImaginaryI],0,0), (0,0,\[ImaginaryI],0), \
(0,0,0,\[ImaginaryI])}."
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ee", ",", "tt"}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1955]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"ev", "[", "i_", "]"}], ":=", 
   RowBox[{"ReplacePart", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"0", ",", "0", ",", "0", ",", "0"}], "}"}], ",", "I", ",", 
     "i"}], "]"}]}], ";"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1956]:="],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
   RowBox[{"ev", "[", "1", "]"}], ",", 
   RowBox[{"ev", "[", "2", "]"}], ",", 
   RowBox[{"ev", "[", "3", "]"}], ",", 
   RowBox[{"ev", "[", "4", "]"}]}], "}"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1957]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {"\[ImaginaryI]", "0", "0", "0"},
     {"0", "\[ImaginaryI]", "0", "0"},
     {"0", "0", "\[ImaginaryI]", "0"},
     {"0", "0", "0", "\[ImaginaryI]"}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1957]="]
}, Open  ]],

Cell[TextData[{
 StyleBox["ee[i][\[Mu]]", "Input"],
 " is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{
    RowBox[{"ee", "[", "i_", "]"}], "[", "mu_", "]"}], ":=", 
   RowBox[{
    RowBox[{"ev", "[", "i", "]"}], "[", 
    RowBox[{"[", "mu", "]"}], "]"}]}], ";"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1958]:="],

Cell[BoxData[
 RowBox[{
  RowBox[{"tt", "[", 
   RowBox[{"i_", ",", "j_", ",", "mu_", ",", "nu_"}], "]"}], ":=", 
  RowBox[{
   RowBox[{
    RowBox[{"ee", "[", "i", "]"}], "[", "mu", "]"}], 
   RowBox[{
    RowBox[{"ee", "[", "j", "]"}], "[", "nu", "]"}]}]}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1959]:="],

Cell[TextData[{
 StyleBox["tt[i,j,\[Mu],\[Nu]]", "Input"],
 " is ",
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "i"], "\[Mu]"], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   SuperscriptBox[
    SubscriptBox["e", "j"], "\[Nu]"], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", 
    RowBox[{"i", ",", "j"}]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       SubscriptBox["e", "i"], "\[Mu]"],
      TraditionalForm], 
     FormBox[
      SubscriptBox["e", 
       RowBox[{"j", ",", "\[Nu]"}]],
      TraditionalForm]}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "\[Mu]"], 
      RowBox[{"\[NoBreak]", 
       FormBox["\[Nu]",
        TraditionalForm]}]],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"i", "=", "1"}], "4"], 
    RowBox[{"tt", "[", 
     RowBox[{"i", ",", "i", ",", "mu", ",", "nu"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"mu", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"nu", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1960]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1960]="]
}, Open  ]],

Cell[TextData[{
 Cell[BoxData[
  FormBox[
   UnderscriptBox["\[Sum]", 
    RowBox[{"\[Mu]", ",", "\[Nu]"}]], TraditionalForm]]],
 Cell[BoxData[
  FormBox[
   RowBox[{" ", 
    RowBox[{
     FormBox[
      SuperscriptBox[
       SubscriptBox["e", "i"], "\[Mu]"],
      TraditionalForm], 
     FormBox[
      SubscriptBox["e", 
       RowBox[{"j", ",", "\[Nu]"}]],
      TraditionalForm]}]}], TraditionalForm]]],
 "=",
 Cell[BoxData[
  FormBox[
   RowBox[{"-", 
    FormBox[
     SuperscriptBox[
      SubscriptBox["g", "i"], "j"],
     TraditionalForm]}], TraditionalForm]]],
 ":"
}], "Text",
 CellTags->"DoPolarizationSums"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Table", "[", 
  RowBox[{
   RowBox[{
    UnderoverscriptBox["\[Sum]", 
     RowBox[{"mu", "=", "1"}], "4"], 
    RowBox[{"tt", "[", 
     RowBox[{"i", ",", "j", ",", "mu", ",", "mu"}], "]"}]}], ",", 
   RowBox[{"{", 
    RowBox[{"i", ",", "1", ",", "4"}], "}"}], ",", 
   RowBox[{"{", 
    RowBox[{"j", ",", "1", ",", "4"}], "}"}]}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1961]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"(", "\[NoBreak]", GridBox[{
     {
      RowBox[{"-", "1"}], "0", "0", "0"},
     {"0", 
      RowBox[{"-", "1"}], "0", "0"},
     {"0", "0", 
      RowBox[{"-", "1"}], "0"},
     {"0", "0", "0", 
      RowBox[{"-", "1"}]}
    },
    GridBoxAlignment->{
     "Columns" -> {{Center}}, "ColumnsIndexed" -> {}, "Rows" -> {{Baseline}}, 
      "RowsIndexed" -> {}},
    GridBoxSpacings->{"Columns" -> {
        Offset[0.27999999999999997`], {
         Offset[0.7]}, 
        Offset[0.27999999999999997`]}, "ColumnsIndexed" -> {}, "Rows" -> {
        Offset[0.2], {
         Offset[0.4]}, 
        Offset[0.2]}, "RowsIndexed" -> {}}], "\[NoBreak]", ")"}], 
  TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1961]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "=", 
  RowBox[{
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}]}], "]"}], " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", "\[Mu]", "]"}], ",", 
     RowBox[{"Momentum", "[", 
      RowBox[{"Polarization", "[", 
       RowBox[{"p4", ",", 
        RowBox[{"-", "\[ImaginaryI]"}]}], "]"}], "]"}]}], "]"}], " ", 
   RowBox[{"Pair", "[", 
    RowBox[{
     RowBox[{"LorentzIndex", "[", 
      RowBox[{"ComplexIndex", "[", "\[Mu]", "]"}], "]"}], ",", 
     RowBox[{"Momentum", "[", 
      RowBox[{"Polarization", "[", 
       RowBox[{"p4", ",", "\[ImaginaryI]"}], "]"}], "]"}]}], 
    "]"}]}]}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1962]:="],

Cell[BoxData[
 FormBox[
  RowBox[{
   SuperscriptBox["g", 
    RowBox[{"\[Mu]", 
     FormBox[
      SuperscriptBox["\[Mu]", "*"],
      TraditionalForm]}]], " ", 
   RowBox[{
    SubsuperscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox["\[Mu]",
       TraditionalForm],
      TraditionalForm], "*"], "(", "p4", ")"}], " ", 
   RowBox[{
    SubscriptBox["\[CurlyEpsilon]", 
     FormBox[
      FormBox[
       SuperscriptBox["\[Mu]", "*"],
       TraditionalForm],
      TraditionalForm]], "(", "p4", ")"}]}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1962]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "//", "DoPolarizationSums"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1963]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", 
   SuperscriptBox[
    FormBox[
     SuperscriptBox["g", 
      RowBox[{"\[Mu]", 
       FormBox[
        SuperscriptBox["\[Mu]", "*"],
        TraditionalForm]}]],
     TraditionalForm], "2"]}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1963]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1964]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "4"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1964]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ptest", "//", "Contract"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1965]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "1"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1965]="]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"%", "//", "DoPolarizationSums"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1966]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"-", "4"}], TraditionalForm]], "Output",
 CellTags->"DoPolarizationSums",
 CellLabel->"Out[1966]="]
}, Open  ]],

Cell[BoxData[
 RowBox[{"Clear", "[", 
  RowBox[{"ee", ",", "tt", ",", "ev", ",", "ptest"}], "]"}]], "Input",
 CellTags->"DoPolarizationSums",
 CellLabel->"In[1967]:="]
}, Open  ]],

Cell["FeynmanDoIntegrals ***unfinished***", "ObjectName",
 CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanDoIntegrals[exp, moms, vars] attempts to evaluate integrals over \
Feynman parameters vars in an expression exp as produced e.g. with \
FeynmanReduce. The variables given must be atomic quantities (AtomQ). If vars \
is omitted all variables in the integrals will be integrated. If vars is \
given, only the variables in vars will be integrated. moms is a list of all \
external momenta. The integrals in exp must be given in the form \
Integratedx[x, low, up].int, where x is the integration variable, low and up \
are the integration limits and int the integrand. The interval [low,up] is \
assumed to include integration bounds put by possible DeltaFunctions and \
moreover it is assumed that up \[GreaterEqual] 0 and that up > low. The two \
options FCIntegrate and FCNIntegrate determine which integration will be \
applied to integrals judged respectively analytically and numerically doable. \
This judging is a very rough one. Using (TimedIntegrate[##, \
Integrate\[Rule]Integrate]&) or (TimedIntegrate[##, \
Integrate\[Rule]NIntegrate]&) as the setting of one or both allows for finer \
judging. Those that are judged neither analytically nor numerically doable \
are left unevaluated, but can of course be attempted evaluated by a simple \
sustitution. Beside the explicit options of FeynmanDoIntegrals options of the \
integration functions specified (FCIntegrate and FCNIntegrate) may be given \
and are passed on to these.\
\>", "Text",
 CellTags->"FeynmanDoIntegrals"],

Cell[TextData[{
 "See also: ",
 " ",
 ButtonBox["FeynmanReduce",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynmanReduce", All, CellTags],
  ButtonData->"FeynmanReduce",
  ButtonNote->"FeynmanReduce"],
 ", ",
 ButtonBox["FeynmanParametrize1",
  BaseStyle->"AddOnsLink",
  ButtonFunction:>NotebookFind[
    ButtonNotebook[], "FeynmanParametrize1", All, CellTags],
  ButtonData->"FeynmanParametrize1",
  ButtonNote->"FeynmanParametrize1"],
 "."
}], "Text",
 CellTags->"FeynmanDoIntegrals"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanDoIntegrals", "]"}]], "Input",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"In[1968]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"NIntegrate", "\[Rule]", "False"}], ",", 
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"EpsilonOrder", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"-", "2"}], ",", "0"}], "}"}]}], ",", 
    RowBox[{"FCIntegrate", "\[Rule]", "TimedIntegrate"}], ",", 
    RowBox[{"Simplify", "\[Rule]", "True"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}], ",", 
    RowBox[{"Series", "\[Rule]", "True"}], ",", 
    RowBox[{"FCNIntegrate", "\[Rule]", 
     RowBox[{"(", 
      RowBox[{
       RowBox[{"DOT", "(", 
        RowBox[{
         RowBox[{"Sequence", "@@", 
          RowBox[{
           RowBox[{"(", 
            RowBox[{
             RowBox[{
             "HighEnergyPhysics`fcdevel`FeynmanDoIntegrals`Private`\
Integratedx", "@@", "#1"}], "&"}], ")"}], "/@", 
           RowBox[{"{", "##2", "}"}]}]}], ",", "#1"}], ")"}], "&"}], 
      ")"}]}]}], "}"}], TraditionalForm]], "Output",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"Out[1968]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanDoIntegrals",
 CellLabel->"In[1969]:="]
}, Open  ]],

Cell["FeynmanParametrize1 ***unfinished***", "ObjectName",
 CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanParametrize1"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanParametrize1[exp,k,Method->Denominator] introduces Feynman parameters \
for all one-loop integrals in exp (k = integration momentum) using formula \
(11.A.1) from \"The Quantum Theory of Fields\" vol. 1 by Steven Weinberg.  \
FeynmanParametrize1[exp,k,Method->Exp] introduces Feynman parameters for all \
one-loop integrals in exp (k = integration momentum) using 1/(A-I eps) = I \
Integrate[Exp[-I x (A-I eps)],{x,0,Infinity}, \
Assumptions->{Arg[A]==0,Arg[eps]==0}]. In this case, when the option \
Integrate is set to True, odd factors of k-tensors are dropped and even \
factors are replaced according to Itzykson&Zuber (8-117).\
\>", "Text",
 CellTags->"FeynmanParametrize1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanParametrize1", "]"}]], "Input",
 CellTags->"FeynmanParametrize1",
 CellLabel->"In[1970]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"FeynmanParameterNames", "\[Rule]", 
     RowBox[{"{", 
      RowBox[{"a", ",", "b", ",", "c", ",", "d", ",", "e"}], "}"}]}], ",", 
    RowBox[{"Method", "\[Rule]", "Denominator"}], ",", 
    RowBox[{"Integrate", "\[Rule]", "True"}], ",", 
    RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
    RowBox[{"CompleteSquare", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynmanParametrize1",
 CellLabel->"Out[1970]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanParametrize1"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanParametrize1",
 CellLabel->"In[1971]:="]
}, Open  ]],

Cell["FeynmanReduce ***unfinished***", "ObjectName",
 CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell["Description", "Subsubsection",
 CellTags->"FeynmanReduce"],

Cell["\<\
***EXPERIMENTAL***
 FeynmanReduce[exp,params] takes a Feynman parameterized expression exp (as \
e.g. generated with FeynmanParametrize1) and a list of Feynman parameters as \
input and attempts to simplify the expression. If no parameters are given, \
Integratedx variables in the expression will be used. Currently, reduction of \
exponentials is implemented. This will work on terms of the form \
E^p1[a,b,c,...]*p2[a,b,c,...], where p1 and p2 are fractions of polynomials \
in the Feynman parameters a,b,c,... If the option Expand is set to True, \
FeynmanReduce will attempt to bring the expression exp into a sum of such \
terms and operate on the terms one by one.\
\>", "Text",
 CellTags->"FeynmanReduce"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Options", "[", "FeynmanReduce", "]"}]], "Input",
 CellTags->"FeynmanReduce",
 CellLabel->"In[1972]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
    RowBox[{"Dimension", "\[Rule]", "D"}], ",", 
    RowBox[{"Flatten", "\[Rule]", "True"}], ",", 
    RowBox[{"Expand", "\[Rule]", "True"}]}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"FeynmanReduce",
 CellLabel->"Out[1972]="]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Examples", "Subsubsection",
 CellTags->"FeynmanReduce"],

Cell[BoxData[""], "Input",
 CellTags->"FeynmanReduce",
 CellLabel->"In[1973]:="],

Cell["\<\
Other objects that are considered to be ok or under active development:\
\>", "Text",
 CellTags->"Experimental"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "ChargeConjugationMatrix", ",", " ", "DiracGammaT", ",", " ", 
   "SquareAmplitude"}], "}"}]], "Input",
 CellTags->"Experimental",
 CellLabel->"In[1974]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{"C", ",", "DiracGammaT", ",", "SquareAmplitude"}], "}"}], 
  TraditionalForm]], "Output",
 CellTags->"Experimental",
 CellLabel->"Out[1974]="]
}, Open  ]],

Cell["Objects that are under consideration for being discarded:", "Text",
 CellTags->"Experimental"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"{", 
  RowBox[{
  "$OPEWard", ",", "$AchmedRoss", ",", "Contract2", ",", 
   "SpinSumExternalMomentum", ",", "SelectedGraphs"}], "}"}]], "Input",
 CellTags->"Experimental",
 CellLabel->"In[1975]:="],

Cell[BoxData[
 FormBox[
  RowBox[{"{", 
   RowBox[{
   "False", ",", "False", ",", "Contract2", ",", "SpinSumExternalMomentum", 
    ",", "SelectedGraphs"}], "}"}], TraditionalForm]], "Output",
 CellTags->"Experimental",
 CellLabel->"Out[1975]="]
}, Open  ]],

Cell[CellGroupData[{

Cell["\<\
AbsoluteTime[]-absolutestarttime\
\>", "Input",
 CellLabel->"In[1976]:="],

Cell[BoxData[
 FormBox["227.299517`8.808143506368072", TraditionalForm]], "Output",
 CellLabel->"Out[1976]="]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
},
ScreenStyleEnvironment->"AllBrackets",
WindowToolbars->"EditBar",
EvaluationCompletionAction->{},
WindowSize->{909, 939},
WindowMargins->{{0, Automatic}, {0, Automatic}},
PrintingStartingPageNumber->1,
PrintingPageRange->{Automatic, Automatic},
PageHeaders->{{Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["FileName"]], "Header"], Inherited}},
PageFooters->{{Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}, {Inherited, 
   Cell[
    TextData[
     ValueBox["DateLong"]], "Footer"], Inherited}},
PrintingOptions->{"FacingPages"->True,
"FirstPageFace"->Left,
"FirstPageFooter"->True,
"FirstPageHeader"->False,
"GraphicsPrintingFormat"->"Automatic",
"Magnification"->1.,
"PaperOrientation"->"Portrait",
"PaperSize"->{597.5625, 842.375},
"PrintCellBrackets"->False,
"PrintMultipleHorizontalPages"->False,
"PrintRegistrationMarks"->True,
"PrintSelectionHighlighting"->False,
"PrintingMargins"->{{54., 54.}, {72., 72.}}},
PrivateNotebookOptions->{"ShortBoxForm"->False},
ShowSelection->True,
TaggingRules:>{
 "MakeContentsNonDefaultOptions" -> {$CellContext`AutomaticCellTagging -> \
$CellContext`FromContent, $CellContext`PageNumbers -> 
    False, $CellContext`SaveOptions -> 
    True, $CellContext`SelectCellStyles -> {
     "Section", "Subsection"}, $CellContext`TOCCellStyles -> {
     "Contents", "Index"}}},
CellLabelAutoDelete->True,
FrontEndVersion->"8.0 for Linux x86 (64-bit) (October 10, 2011)",
StyleDefinitions->"HelpBrowser.nb"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{
 "Functions"->{
  Cell[1770, 69, 52, 1, 66, "Section",
   CellTags->"Functions"]},
 "Abbreviation"->{
  Cell[3623, 140, 84, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"Abbreviation"],
  Cell[3732, 146, 63, 1, 36, "Subsubsection",
   CellTags->"Abbreviation"],
  Cell[3798, 149, 329, 6, 41, "Text",
   CellTags->"Abbreviation"],
  Cell[4130, 157, 1716, 59, 41, "Text",
   CellTags->"Abbreviation"],
  Cell[5883, 221, 60, 1, 36, "Subsubsection",
   CellTags->"Abbreviation"],
  Cell[5968, 226, 264, 8, 26, "Input",
   CellTags->"Abbreviation"],
  Cell[6235, 236, 228, 8, 28, "Output",
   CellTags->"Abbreviation"]},
 "Amplitude"->{
  Cell[6490, 248, 55, 1, 39, "ObjectName",
   CellTags->"Amplitude"],
  Cell[6570, 253, 60, 1, 36, "Subsubsection",
   CellTags->"Amplitude"],
  Cell[6633, 256, 529, 9, 75, "Text",
   CellTags->"Amplitude"],
  Cell[7187, 269, 115, 3, 26, "Input",
   CellTags->"Amplitude"],
  Cell[7305, 274, 328, 10, 26, "Output",
   CellTags->"Amplitude"],
  Cell[7648, 287, 258, 11, 24, "Text",
   CellTags->"Amplitude"],
  Cell[7943, 303, 57, 1, 36, "Subsubsection",
   CellTags->"Amplitude"],
  Cell[8025, 308, 133, 4, 26, "Input",
   CellTags->"Amplitude"],
  Cell[8161, 314, 104, 3, 26, "Output",
   CellTags->"Amplitude"],
  Cell[8280, 320, 93, 1, 24, "Text",
   CellTags->"Amplitude"],
  Cell[8398, 325, 121, 3, 26, "Input",
   CellTags->"Amplitude"],
  Cell[8522, 330, 2216, 95, 30, "Output",
   CellTags->"Amplitude"],
  Cell[10775, 430, 108, 3, 26, "Input",
   CellTags->"Amplitude"],
  Cell[10886, 435, 5133, 189, 73, "Output",
   CellTags->"Amplitude"],
  Cell[16034, 627, 211, 8, 24, "Text",
   CellTags->"Amplitude"],
  Cell[16270, 639, 306, 9, 26, "Input",
   CellTags->"Amplitude"],
  Cell[16579, 650, 13529, 441, 136, "Output",
   CellTags->"Amplitude"]},
 "Amputate"->{
  Cell[30135, 1095, 53, 1, 39, "ObjectName",
   CellTags->"Amputate"],
  Cell[30213, 1100, 59, 1, 36, "Subsubsection",
   CellTags->"Amputate"],
  Cell[30275, 1103, 215, 5, 41, "Text",
   CellTags->"Amputate"],
  Cell[30515, 1112, 114, 3, 26, "Input",
   CellTags->"Amputate"],
  Cell[30632, 1117, 296, 10, 26, "Output",
   CellTags->"Amputate"],
  Cell[30943, 1130, 1080, 38, 24, "Text",
   CellTags->"Amputate"],
  Cell[32060, 1173, 56, 1, 36, "Subsubsection",
   CellTags->"Amputate"],
  Cell[32141, 1178, 170, 5, 26, "Input",
   CellTags->"Amputate"],
  Cell[32314, 1185, 428, 16, 28, "Output",
   CellTags->"Amputate"],
  Cell[32779, 1206, 131, 4, 26, "Input",
   CellTags->"Amputate"],
  Cell[32913, 1212, 721, 27, 28, "Output",
   CellTags->"Amputate"]},
 "AnomalousDimension"->{
  Cell[33661, 1243, 73, 1, 39, "ObjectName",
   CellTags->"AnomalousDimension"],
  Cell[33759, 1248, 69, 1, 36, "Subsubsection",
   CellTags->"AnomalousDimension"],
  Cell[33831, 1251, 143, 4, 24, "Text",
   CellTags->"AnomalousDimension"],
  Cell[33999, 1259, 134, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[34136, 1264, 254, 8, 26, "Output",
   CellTags->"AnomalousDimension"],
  Cell[34405, 1275, 653, 24, 24, "Text",
   CellTags->"AnomalousDimension"],
  Cell[35095, 1304, 66, 1, 36, "Subsubsection",
   CellTags->"AnomalousDimension"],
  Cell[35164, 1307, 63, 1, 24, "Text",
   CellTags->"AnomalousDimension"],
  Cell[35252, 1312, 206, 5, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[35461, 1319, 254, 8, 26, "Output",
   CellTags->"AnomalousDimension"],
  Cell[35730, 1330, 241, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[35996, 1343, 133, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[36132, 1348, 462, 17, 43, "Output",
   CellTags->"AnomalousDimension"],
  Cell[36609, 1368, 240, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[36874, 1381, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[37009, 1386, 284, 10, 45, "Output",
   CellTags->"AnomalousDimension"],
  Cell[37308, 1399, 240, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[37573, 1412, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[37708, 1417, 283, 10, 43, "Output",
   CellTags->"AnomalousDimension"],
  Cell[38006, 1430, 240, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[38271, 1443, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[38406, 1448, 594, 21, 46, "Output",
   CellTags->"AnomalousDimension"],
  Cell[39015, 1472, 241, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[39281, 1485, 133, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[39417, 1490, 689, 23, 47, "Output",
   CellTags->"AnomalousDimension"],
  Cell[40121, 1516, 241, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[40387, 1529, 133, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[40523, 1534, 6606, 218, 249, "Output",
   CellTags->"AnomalousDimension"],
  Cell[47144, 1755, 240, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[47409, 1768, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[47544, 1773, 4116, 140, 150, "Output",
   CellTags->"AnomalousDimension"],
  Cell[51675, 1916, 240, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[51940, 1929, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[52075, 1934, 6222, 212, 235, "Output",
   CellTags->"AnomalousDimension"],
  Cell[58312, 2149, 240, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[58577, 2162, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[58712, 2167, 4910, 166, 235, "Output",
   CellTags->"AnomalousDimension"],
  Cell[63637, 2336, 267, 9, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[63929, 2349, 132, 3, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[64064, 2354, 4379, 139, 151, "Output",
   CellTags->"AnomalousDimension"],
  Cell[68458, 2496, 326, 10, 29, "Text",
   CellTags->"AnomalousDimension"],
  Cell[68809, 2510, 295, 9, 26, "Input",
   CellTags->"AnomalousDimension"],
  Cell[69107, 2521, 239, 7, 26, "Output",
   CellTags->"AnomalousDimension"]},
 "AntiCommutator"->{
  Cell[69373, 2532, 65, 1, 39, "ObjectName",
   CellTags->"AntiCommutator"],
  Cell[69463, 2537, 65, 1, 36, "Subsubsection",
   CellTags->"AntiCommutator"],
  Cell[69531, 2540, 145, 4, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[69679, 2546, 955, 32, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[70671, 2583, 62, 1, 36, "Subsubsection",
   CellTags->"AntiCommutator"],
  Cell[70736, 2586, 200, 9, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[70939, 2597, 150, 4, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[71114, 2605, 143, 4, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[71260, 2611, 163, 5, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[71460, 2621, 124, 3, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[71587, 2626, 180, 6, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[71804, 2637, 265, 8, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[72072, 2647, 438, 16, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[72547, 2668, 258, 8, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[72808, 2678, 318, 11, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[73141, 2692, 218, 6, 31, "Input",
   CellTags->"AntiCommutator"],
  Cell[73362, 2700, 104, 1, 24, "Text",
   CellTags->"AntiCommutator"],
  Cell[73491, 2705, 177, 5, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[73671, 2712, 109, 3, 27, "Output",
   CellTags->"AntiCommutator"],
  Cell[73817, 2720, 182, 5, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[74002, 2727, 190, 6, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[74229, 2738, 250, 8, 31, "Input",
   CellTags->"AntiCommutator"],
  Cell[74482, 2748, 132, 4, 26, "Output",
   CellTags->"AntiCommutator"],
  Cell[74651, 2757, 252, 7, 31, "Input",
   CellTags->"AntiCommutator"],
  Cell[74906, 2766, 315, 11, 29, "Output",
   CellTags->"AntiCommutator"],
  Cell[75236, 2780, 240, 6, 31, "Input",
   CellTags->"AntiCommutator"],
  Cell[75479, 2788, 176, 5, 26, "Input",
   CellTags->"AntiCommutator"],
  Cell[75658, 2795, 226, 7, 31, "Input",
   CellTags->"AntiCommutator"]},
 "AntiQuarkField"->{
  Cell[75899, 2805, 65, 1, 39, "ObjectName",
   CellTags->"AntiQuarkField"],
  Cell[75989, 2810, 65, 1, 36, "Subsubsection",
   CellTags->"AntiQuarkField"],
  Cell[76057, 2813, 197, 4, 24, "Text",
   CellTags->"AntiQuarkField"],
  Cell[76257, 2819, 483, 18, 24, "Text",
   CellTags->"AntiQuarkField"],
  Cell[76777, 2842, 62, 1, 36, "Subsubsection",
   CellTags->"AntiQuarkField"],
  Cell[76864, 2847, 93, 2, 26, "Input",
   CellTags->"AntiQuarkField"],
  Cell[76960, 2851, 137, 4, 26, "Output",
   CellTags->"AntiQuarkField"]},
 "AntiSymmetrize"->{
  Cell[77124, 2859, 65, 1, 39, "ObjectName",
   CellTags->"AntiSymmetrize"],
  Cell[77214, 2864, 65, 1, 36, "Subsubsection",
   CellTags->"AntiSymmetrize"],
  Cell[77282, 2867, 153, 4, 24, "Text",
   CellTags->"AntiSymmetrize"],
  Cell[77438, 2873, 269, 10, 24, "Text",
   CellTags->"AntiSymmetrize"],
  Cell[77744, 2888, 62, 1, 36, "Subsubsection",
   CellTags->"AntiSymmetrize"],
  Cell[77831, 2893, 246, 8, 26, "Input",
   CellTags->"AntiSymmetrize"],
  Cell[78080, 2903, 319, 11, 42, "Output",
   CellTags->"AntiSymmetrize"],
  Cell[78436, 2919, 266, 8, 26, "Input",
   CellTags->"AntiSymmetrize"],
  Cell[78705, 2929, 654, 20, 42, "Output",
   CellTags->"AntiSymmetrize"]},
 "Anti5"->{
  Cell[79386, 2953, 47, 1, 39, "ObjectName",
   CellTags->"Anti5"],
  Cell[79458, 2958, 56, 1, 36, "Subsubsection",
   CellTags->"Anti5"],
  Cell[79517, 2961, 487, 16, 41, "Text",
   CellTags->"Anti5"],
  Cell[80007, 2979, 163, 7, 24, "Text",
   CellTags->"Anti5"],
  Cell[80173, 2988, 678, 25, 24, "Text",
   CellTags->"Anti5"],
  Cell[80888, 3018, 53, 1, 36, "Subsubsection",
   CellTags->"Anti5"],
  Cell[80966, 3023, 154, 5, 26, "Input",
   CellTags->"Anti5"],
  Cell[81123, 3030, 186, 6, 26, "Output",
   CellTags->"Anti5"],
  Cell[81346, 3041, 102, 3, 26, "Input",
   CellTags->"Anti5"],
  Cell[81451, 3046, 290, 11, 26, "Output",
   CellTags->"Anti5"],
  Cell[81778, 3062, 144, 5, 26, "Input",
   CellTags->"Anti5"],
  Cell[81925, 3069, 265, 10, 26, "Output",
   CellTags->"Anti5"],
  Cell[82227, 3084, 194, 6, 26, "Input",
   CellTags->"Anti5"],
  Cell[82424, 3092, 320, 9, 26, "Output",
   CellTags->"Anti5"],
  Cell[82781, 3106, 125, 4, 26, "Input",
   CellTags->"Anti5"],
  Cell[82909, 3112, 320, 9, 26, "Output",
   CellTags->"Anti5"],
  Cell[83266, 3126, 133, 4, 26, "Input",
   CellTags->"Anti5"],
  Cell[83402, 3132, 320, 9, 26, "Output",
   CellTags->"Anti5"],
  Cell[83759, 3146, 151, 5, 26, "Input",
   CellTags->"Anti5"],
  Cell[83913, 3153, 320, 9, 26, "Output",
   CellTags->"Anti5"],
  Cell[84248, 3165, 351, 14, 24, "Text",
   CellTags->"Anti5"],
  Cell[84624, 3183, 127, 4, 26, "Input",
   CellTags->"Anti5"],
  Cell[84754, 3189, 191, 6, 26, "Output",
   CellTags->"Anti5"],
  Cell[84982, 3200, 102, 3, 26, "Input",
   CellTags->"Anti5"],
  Cell[85087, 3205, 290, 11, 26, "Output",
   CellTags->"Anti5"]},
 "Apart2"->{
  Cell[85404, 3220, 49, 1, 39, "ObjectName",
   CellTags->"Apart2"],
  Cell[85478, 3225, 57, 1, 36, "Subsubsection",
   CellTags->"Apart2"],
  Cell[85538, 3228, 108, 3, 24, "Text",
   CellTags->"Apart2"],
  Cell[85649, 3233, 471, 18, 24, "Text",
   CellTags->"Apart2"],
  Cell[86157, 3256, 54, 1, 36, "Subsubsection",
   CellTags->"Apart2"],
  Cell[86236, 3261, 256, 9, 26, "Input",
   CellTags->"Apart2"],
  Cell[86495, 3272, 1188, 30, 48, "Output",
   CellTags->"Apart2"],
  Cell[87720, 3307, 104, 3, 26, "Input",
   CellTags->"Apart2"],
  Cell[87827, 3312, 720, 26, 53, "Output",
   CellTags->"Apart2"],
  Cell[88584, 3343, 140, 4, 26, "Input",
   CellTags->"Apart2"],
  Cell[88727, 3349, 517, 19, 56, "Output",
   CellTags->"Apart2"]},
 "A0"->{
  Cell[89271, 3372, 64, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"A0"],
  Cell[89360, 3378, 53, 1, 36, "Subsubsection",
   CellTags->"A0"],
  Cell[89416, 3381, 291, 11, 24, "Text",
   CellTags->"A0"],
  Cell[89732, 3396, 107, 3, 26, "Input",
   CellTags->"A0"],
  Cell[89842, 3401, 161, 6, 26, "Output",
   CellTags->"A0"],
  Cell[90018, 3410, 743, 32, 24, "Text",
   CellTags->"A0"],
  Cell[90798, 3447, 50, 1, 36, "Subsubsection",
   CellTags->"A0"],
  Cell[90851, 3450, 246, 11, 24, "Text",
   CellTags->"A0"],
  Cell[91122, 3465, 120, 4, 29, "Input",
   CellTags->"A0"],
  Cell[91245, 3471, 259, 10, 29, "Output",
   CellTags->"A0"],
  Cell[91519, 3484, 186, 6, 26, "Input",
   CellTags->"A0"],
  Cell[91730, 3494, 120, 4, 29, "Input",
   CellTags->"A0"],
  Cell[91853, 3500, 324, 11, 29, "Output",
   CellTags->"A0"],
  Cell[92192, 3514, 187, 6, 26, "Input",
   CellTags->"A0"],
  Cell[92382, 3522, 222, 8, 24, "Text",
   CellTags->"A0"],
  Cell[92629, 3534, 96, 3, 26, "Input",
   CellTags->"A0"],
  Cell[92728, 3539, 97, 3, 26, "Output",
   CellTags->"A0"],
  Cell[92862, 3547, 160, 5, 26, "Input",
   CellTags->"A0"],
  Cell[93025, 3554, 97, 3, 26, "Output",
   CellTags->"A0"]},
 "A0ToB0"->{
  Cell[93149, 3561, 72, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"A0ToB0"],
  Cell[93246, 3567, 57, 1, 36, "Subsubsection",
   CellTags->"A0ToB0"],
  Cell[93306, 3570, 614, 27, 24, "Text",
   CellTags->"A0ToB0"],
  Cell[93945, 3601, 111, 3, 26, "Input",
   CellTags->"A0ToB0"],
  Cell[94059, 3606, 165, 6, 26, "Output",
   CellTags->"A0ToB0"],
  Cell[94239, 3615, 915, 39, 22, "Text",
   CellTags->"A0ToB0"]},
 "BackgroundGluonVertex"->{
  Cell[95169, 3657, 79, 1, 39, "ObjectName",
   CellTags->"BackgroundGluonVertex"],
  Cell[95273, 3662, 72, 1, 36, "Subsubsection",
   CellTags->"BackgroundGluonVertex"],
  Cell[95348, 3665, 1042, 19, 126, "Text",
   CellTags->"BackgroundGluonVertex"],
  Cell[96415, 3688, 140, 3, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[96558, 3693, 318, 9, 26, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[96891, 3705, 286, 11, 24, "Text",
   CellTags->"BackgroundGluonVertex"],
  Cell[97214, 3721, 69, 1, 36, "Subsubsection",
   CellTags->"BackgroundGluonVertex"],
  Cell[97308, 3726, 360, 10, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[97671, 3738, 1486, 58, 29, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[99194, 3801, 436, 12, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[99633, 3815, 3846, 142, 51, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[103516, 3962, 411, 11, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[103930, 3975, 1559, 60, 47, "Output",
   CellTags->"BackgroundGluonVertex"],
  Cell[105526, 4040, 487, 13, 26, "Input",
   CellTags->"BackgroundGluonVertex"],
  Cell[106016, 4055, 3954, 145, 92, "Output",
   CellTags->"BackgroundGluonVertex"]},
 "Bracket"->{
  Cell[109997, 4204, 51, 1, 39, "ObjectName",
   CellTags->"Bracket"],
  Cell[110073, 4209, 58, 1, 36, "Subsubsection",
   CellTags->"Bracket"],
  Cell[110134, 4212, 72, 1, 24, "Text",
   CellTags->"Bracket"],
  Cell[110209, 4215, 264, 11, 24, "Text",
   CellTags->"Bracket"]},
 "BReduce"->{
  Cell[110488, 4229, 74, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"BReduce"],
  Cell[110587, 4235, 58, 1, 36, "Subsubsection",
   CellTags->"BReduce"],
  Cell[110648, 4238, 148, 4, 24, "Text",
   CellTags->"BReduce"],
  Cell[110799, 4244, 916, 39, 24, "Text",
   CellTags->"BReduce"],
  Cell[111752, 4288, 55, 1, 36, "Subsubsection",
   CellTags->"BReduce"],
  Cell[111810, 4291, 251, 11, 24, "Text",
   CellTags->"BReduce"],
  Cell[112086, 4306, 134, 4, 26, "Input",
   CellTags->"BReduce"],
  Cell[112223, 4312, 176, 6, 26, "Output",
   CellTags->"BReduce"],
  Cell[112798, 4338, 94, 3, 24, "Text",
   CellTags->"BReduce"],
  Cell[112917, 4345, 183, 5, 26, "Input",
   CellTags->"BReduce"],
  Cell[113103, 4352, 260, 10, 44, "Output",
   CellTags->"BReduce"]},
 "B0"->{
  Cell[113390, 4366, 41, 1, 39, "ObjectName",
   CellTags->"B0"],
  Cell[113456, 4371, 53, 1, 36, "Subsubsection",
   CellTags->"B0"],
  Cell[113512, 4374, 390, 9, 41, "Text",
   CellTags->"B0"],
  Cell[113927, 4387, 102, 3, 26, "Input",
   CellTags->"B0"],
  Cell[114032, 4392, 280, 9, 26, "Output",
   CellTags->"B0"],
  Cell[114327, 4404, 751, 32, 24, "Text",
   CellTags->"B0"],
  Cell[115115, 4441, 50, 1, 36, "Subsubsection",
   CellTags->"B0"],
  Cell[115190, 4446, 234, 8, 29, "Input",
   CellTags->"B0"],
  Cell[115427, 4456, 316, 12, 29, "Output",
   CellTags->"B0"],
  Cell[115758, 4471, 237, 7, 26, "Input",
   CellTags->"B0"],
  Cell[116020, 4482, 154, 5, 29, "Input",
   CellTags->"B0"],
  Cell[116177, 4489, 245, 8, 29, "Output",
   CellTags->"B0"],
  Cell[116459, 4502, 179, 6, 29, "Input",
   CellTags->"B0"],
  Cell[116641, 4510, 245, 8, 29, "Output",
   CellTags->"B0"],
  Cell[116923, 4523, 177, 6, 26, "Input",
   CellTags->"B0"],
  Cell[117103, 4531, 218, 7, 29, "Output",
   CellTags->"B0"]},
 "B0Real"->{
  Cell[117348, 4542, 72, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"B0Real"],
  Cell[117445, 4548, 57, 1, 36, "Subsubsection",
   CellTags->"B0Real"],
  Cell[117505, 4551, 186, 4, 22, "Text",
   CellTags->"B0Real"],
  Cell[117694, 4557, 229, 10, 22, "Text",
   CellTags->"B0Real"],
  Cell[117960, 4572, 54, 1, 34, "Subsubsection",
   CellTags->"B0Real"],
  Cell[118017, 4575, 83, 1, 24, "Text",
   CellTags->"B0Real"],
  Cell[118125, 4580, 133, 4, 26, "Input",
   CellTags->"B0Real"],
  Cell[118261, 4586, 200, 7, 26, "Output",
   CellTags->"B0Real"],
  Cell[118498, 4598, 102, 3, 26, "Input",
   CellTags->"B0Real"],
  Cell[118603, 4603, 282, 9, 26, "Output",
   CellTags->"B0Real"],
  Cell[118900, 4615, 92, 3, 24, "Text",
   CellTags->"B0Real"],
  Cell[119017, 4622, 181, 5, 26, "Input",
   CellTags->"B0Real"],
  Cell[119201, 4629, 200, 7, 26, "Output",
   CellTags->"B0Real"]},
 "B0Unique"->{
  Cell[119428, 4640, 76, 2, 39, "ObjectName",
   PageBreakAbove->True,
   CellTags->"B0Unique"],
  Cell[119529, 4646, 59, 1, 36, "Subsubsection",
   CellTags->"B0Unique"],
  Cell[119591, 4649, 184, 4, 24, "Text",
   CellTags->"B0Unique"],
  Cell[119778, 4655, 231, 10, 24, "Text",
   CellTags->"B0Unique"],
  Cell[120046, 4670, 56, 1, 36, "Subsubsection",
   CellTags->"B0Unique"],
  Cell[120105, 4673, 73, 1, 24, "Text",
   CellTags->"B0Unique"],
  Cell[120203, 4678, 135, 4, 26, "Input",
   CellTags->"B0Unique"],
  Cell[120341, 4684, 202, 7, 26, "Output",
   CellTags->"B0Unique"],
  Cell[120580, 4696, 104, 3, 26, "Input",
   CellTags->"B0Unique"],
  Cell[120687, 4701, 284, 9, 26, "Output",
   CellTags->"B0Unique"],
  Cell[120986, 4713, 88, 3, 24, "Text",
   CellTags->"B0Unique"],
  Cell[121099, 4720, 186, 5, 26, "Input",
   CellTags->"B0Unique"],
  Cell[121288, 4727, 177, 6, 26, "Output",
   CellTags->"B0Unique"]},
 "B00"->{
  Cell[121492, 4737, 43, 1, 39, "ObjectName",
   CellTags->"B00"],
  Cell[121560, 4742, 54, 1, 36, "Subsubsection",
   CellTags->"B00"],
  Cell[121617, 4745, 290, 8, 41, "Text",
   CellTags->"B00"],
  Cell[121932, 4757, 104, 3, 26, "Input",
   CellTags->"B00"],
  Cell[122039, 4762, 162, 6, 26, "Output",
   CellTags->"B00"],
  Cell[122216, 4771, 576, 25, 24, "Text",
   CellTags->"B00"],
  Cell[122829, 4801, 51, 1, 36, "Subsubsection",
   CellTags->"B00"],
  Cell[122883, 4804, 202, 7, 26, "Text",
   CellTags->"B00"],
  Cell[123110, 4815, 97, 3, 26, "Input",
   CellTags->"B00"],
  Cell[123210, 4820, 156, 6, 28, "Output",
   CellTags->"B00"],
  Cell[123403, 4831, 211, 7, 29, "Input",
   CellTags->"B00"],
  Cell[123617, 4840, 2675, 90, 88, "Output",
   CellTags->"B00"],
  Cell[126329, 4935, 211, 7, 29, "Input",
   CellTags->"B00"],
  Cell[126543, 4944, 1452, 54, 43, "Output",
   CellTags->"B00"],
  Cell[128032, 5003, 261, 8, 29, "Input",
   CellTags->"B00"],
  Cell[128296, 5013, 516, 22, 29, "Output",
   CellTags->"B00"],
  Cell[128849, 5040, 181, 6, 29, "Input",
   CellTags->"B00"],
  Cell[129033, 5048, 614, 22, 44, "Output",
   CellTags->"B00"],
  Cell[129684, 5075, 248, 8, 29, "Input",
   CellTags->"B00"],
  Cell[129935, 5085, 641, 23, 44, "Output",
   CellTags->"B00"]},
 "B1"->{
  Cell[130603, 5112, 41, 1, 39, "ObjectName",
   CellTags->"B1"],
  Cell[130669, 5117, 53, 1, 36, "Subsubsection",
   CellTags->"B1"],
  Cell[130725, 5120, 230, 7, 24, "Text",
   CellTags->"B1"],
  Cell[130980, 5131, 102, 3, 26, "Input",
   CellTags->"B1"],
  Cell[131085, 5136, 161, 6, 26, "Output",
   CellTags->"B1"],
  Cell[131261, 5145, 951, 39, 24, "Text",
   CellTags->"B1"],
  Cell[132249, 5189, 50, 1, 36, "Subsubsection",
   CellTags->"B1"],
  Cell[132324, 5194, 209, 7, 29, "Input",
   CellTags->"B1"],
  Cell[132536, 5203, 1153, 42, 49, "Output",
   CellTags->"B1"],
  Cell[133726, 5250, 259, 8, 29, "Input",
   CellTags->"B1"],
  Cell[133988, 5260, 316, 12, 29, "Output",
   CellTags->"B1"],
  Cell[134319, 5275, 187, 6, 26, "Input",
   CellTags->"B1"],
  Cell[134531, 5285, 209, 7, 29, "Input",
   CellTags->"B1"],
  Cell[134743, 5294, 390, 15, 42, "Output",
   CellTags->"B1"],
  Cell[135170, 5314, 179, 6, 29, "Input",
   CellTags->"B1"],
  Cell[135352, 5322, 403, 14, 42, "Output",
   CellTags->"B1"],
  Cell[135792, 5341, 154, 5, 29, "Input",
   CellTags->"B1"],
  Cell[135949, 5348, 403, 14, 42, "Output",
   CellTags->"B1"],
  Cell[136389, 5367, 238, 7, 29, "Input",
   CellTags->"B1"],
  Cell[136630, 5376, 791, 25, 48, "Output",
   CellTags->"B1"],
  Cell[137458, 5406, 279, 8, 29, "Input",
   CellTags->"B1"],
  Cell[137740, 5416, 419, 14, 42, "Output",
   CellTags->"B1"]},
 "B11"->{
  Cell[138186, 5434, 43, 1, 39, "ObjectName",
   CellTags->"B11"],
  Cell[138254, 5439, 54, 1, 36, "Subsubsection",
   CellTags->"B11"],
  Cell[138311, 5442, 495, 17, 43, "Text",
   CellTags->"B11"],
  Cell[138831, 5463, 104, 3, 26, "Input",
   CellTags->"B11"],
  Cell[138938, 5468, 162, 6, 26, "Output",
   CellTags->"B11"],
  Cell[139115, 5477, 748, 32, 24, "Text",
   CellTags->"B11"],
  Cell[139900, 5514, 51, 1, 36, "Subsubsection",
   CellTags->"B11"],
  Cell[139954, 5517, 215, 7, 26, "Text",
   CellTags->"B11"],
  Cell[140194, 5528, 97, 3, 26, "Input",
   CellTags->"B11"],
  Cell[140294, 5533, 156, 6, 28, "Output",
   CellTags->"B11"],
  Cell[140487, 5544, 212, 7, 29, "Input",
   CellTags->"B11"],
  Cell[140702, 5553, 2848, 97, 95, "Output",
   CellTags->"B11"],
  Cell[143587, 5655, 212, 7, 29, "Input",
   CellTags->"B11"],
  Cell[143802, 5664, 1206, 47, 53, "Output",
   CellTags->"B11"],
  Cell[145045, 5716, 171, 5, 26, "Input",
   CellTags->"B11"],
  Cell[145219, 5723, 164, 6, 26, "Output",
   CellTags->"B11"],
  Cell[145420, 5734, 212, 7, 29, "Input",
   CellTags->"B11"],
  Cell[145635, 5743, 517, 22, 29, "Output",
   CellTags->"B11"],
  Cell[146189, 5770, 170, 5, 26, "Input",
   CellTags->"B11"],
  Cell[146362, 5777, 163, 6, 26, "Output",
   CellTags->"B11"],
  Cell[146562, 5788, 182, 6, 29, "Input",
   CellTags->"B11"],
  Cell[146747, 5796, 269, 9, 42, "Output",
   CellTags->"B11"],
  Cell[147053, 5810, 249, 8, 29, "Input",
   CellTags->"B11"],
  Cell[147305, 5820, 295, 10, 42, "Output",
   CellTags->"B11"]},
 "CA"->{
  Cell[147627, 5834, 41, 1, 39, "ObjectName",
   CellTags->"CA"],
  Cell[147693, 5839, 53, 1, 36, "Subsubsection",
   CellTags->"CA"],
  Cell[147749, 5842, 198, 9, 24, "Text",
   CellTags->"CA"],
  Cell[147950, 5853, 435, 18, 24, "Text",
   CellTags->"CA"],
  Cell[148422, 5876, 50, 1, 36, "Subsubsection",
   CellTags->"CA"],
  Cell[148497, 5881, 70, 2, 26, "Input",
   CellTags->"CA"],
  Cell[148570, 5885, 120, 4, 26, "Output",
   CellTags->"CA"],
  Cell[148727, 5894, 173, 5, 26, "Input",
   CellTags->"CA"],
  Cell[148903, 5901, 98, 3, 27, "Output",
   CellTags->"CA"],
  Cell[149038, 5909, 72, 2, 26, "Input",
   CellTags->"CA"],
  Cell[149113, 5913, 98, 3, 27, "Output",
   CellTags->"CA"]},
 "Calc"->{
  Cell[149238, 5920, 45, 1, 39, "ObjectName",
   CellTags->"Calc"],
  Cell[149308, 5925, 55, 1, 36, "Subsubsection",
   CellTags->"Calc"],
  Cell[149366, 5928, 233, 5, 41, "Text",
   CellTags->"Calc"],
  Cell[149602, 5935, 657, 25, 24, "Text",
   CellTags->"Calc"],
  Cell[150296, 5965, 61, 1, 33, "Subsubsection",
   CellTags->"Calc"],
  Cell[150360, 5968, 346, 13, 27, "Text",
   CellTags->"Calc"],
  Cell[150731, 5985, 284, 9, 26, "Input",
   CellTags->"Calc"],
  Cell[151018, 5996, 147, 5, 26, "Output",
   CellTags->"Calc"],
  Cell[151180, 6004, 213, 9, 24, "Text",
   CellTags->"Calc"],
  Cell[151418, 6017, 258, 8, 26, "Input",
   CellTags->"Calc"],
  Cell[151679, 6027, 352, 15, 26, "Output",
   CellTags->"Calc"],
  Cell[152068, 6047, 328, 11, 26, "Input",
   CellTags->"Calc"],
  Cell[152399, 6060, 424, 16, 28, "Output",
   CellTags->"Calc"],
  Cell[152860, 6081, 101, 3, 26, "Input",
   CellTags->"Calc"],
  Cell[152964, 6086, 525, 22, 28, "Output",
   CellTags->"Calc"],
  Cell[153526, 6113, 215, 6, 26, "Input",
   CellTags->"Calc"],
  Cell[153744, 6121, 985, 44, 28, "Output",
   CellTags->"Calc"],
  Cell[154766, 6170, 382, 13, 26, "Input",
   CellTags->"Calc"],
  Cell[155151, 6185, 100, 3, 26, "Output",
   CellTags->"Calc"]},
 "CalcColorFactor"->{
  Cell[155278, 6192, 67, 1, 39, "ObjectName",
   CellTags->"CalcColorFactor"],
  Cell[155370, 6197, 66, 1, 36, "Subsubsection",
   CellTags->"CalcColorFactor"],
  Cell[155439, 6200, 470, 9, 108, "Text",
   CellTags->"CalcColorFactor"],
  Cell[155912, 6211, 280, 11, 24, "Text",
   CellTags->"CalcColorFactor"]},
 "Collecting"->{
  Cell[161101, 6403, 662, 24, 24, "Text",
   CellTags->"Collecting"],
  Cell[259084, 10278, 57, 1, 39, "ObjectName",
   CellTags->"Collecting"],
  Cell[259166, 10283, 61, 1, 36, "Subsubsection",
   CellTags->"Collecting"],
  Cell[259230, 10286, 207, 5, 41, "Text",
   CellTags->"Collecting"],
  Cell[259440, 10293, 662, 24, 24, "Text",
   CellTags->"Collecting"]},
 "CalculateCounterTerm"->{
  Cell[161778, 6430, 94, 1, 39, "ObjectName",
   CellTags->"CalculateCounterTerm"],
  Cell[161897, 6435, 71, 1, 36, "Subsubsection",
   CellTags->"CalculateCounterTerm"],
  Cell[161971, 6438, 110, 1, 24, "Text",
   CellTags->"CalculateCounterTerm"],
  Cell[162118, 6444, 77, 1, 33, "Subsubsection",
   CellTags->"CalculateCounterTerm"],
  Cell[162220, 6449, 123, 3, 26, "Input",
   CellTags->"CalculateCounterTerm"],
  Cell[162346, 6454, 191, 3, 40, "Print",
   CellTags->{"CalculateCounterTerm", "Info3535453139-5096475"}]},
 "Info3535453139-5096475"->{
  Cell[162346, 6454, 191, 3, 40, "Print",
   CellTags->{"CalculateCounterTerm", "Info3535453139-5096475"}]},
 "CancelQP"->{
  Cell[162564, 6461, 53, 1, 39, "ObjectName",
   CellTags->"CancelQP"],
  Cell[162642, 6466, 59, 1, 36, "Subsubsection",
   CellTags->"CancelQP"],
  Cell[162704, 6469, 314, 13, 26, "Text",
   CellTags->"CancelQP"],
  Cell[163055, 6487, 65, 1, 33, "Subsubsection",
   CellTags->"CancelQP"],
  Cell[163145, 6492, 549, 19, 26, "Input",
   CellTags->"CancelQP"],
  Cell[163697, 6513, 667, 24, 47, "Output",
   CellTags->"CancelQP"],
  Cell[164401, 6542, 182, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[164586, 6549, 2183, 73, 47, "Output",
   CellTags->"CancelQP"],
  Cell[166806, 6627, 183, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[166992, 6634, 6421, 209, 189, "Output",
   CellTags->"CancelQP"],
  Cell[173450, 6848, 555, 18, 26, "Input",
   CellTags->"CancelQP"],
  Cell[174008, 6868, 886, 33, 47, "Output",
   CellTags->"CancelQP"],
  Cell[174931, 6906, 182, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[175116, 6913, 6471, 225, 96, "Output",
   CellTags->"CancelQP"],
  Cell[181624, 7143, 183, 5, 26, "Input",
   CellTags->"CancelQP"],
  Cell[181810, 7150, 3734, 126, 49, "Output",
   CellTags->"CancelQP"],
  Cell[185559, 7279, 106, 3, 26, "Input",
   CellTags->"CancelQP"]},
 "Cases2"->{
  Cell[185680, 7285, 49, 1, 39, "ObjectName",
   CellTags->"Cases2"],
  Cell[185754, 7290, 57, 1, 36, "Subsubsection",
   CellTags->"Cases2"],
  Cell[185814, 7293, 293, 6, 41, "Text",
   CellTags->"Cases2"],
  Cell[186144, 7304, 63, 1, 33, "Subsubsection",
   CellTags->"Cases2"],
  Cell[186232, 7309, 305, 10, 29, "Input",
   CellTags->"Cases2"],
  Cell[186540, 7321, 275, 9, 26, "Output",
   CellTags->"Cases2"],
  Cell[186852, 7335, 312, 10, 26, "Input",
   CellTags->"Cases2"],
  Cell[187167, 7347, 279, 9, 26, "Output",
   CellTags->"Cases2"],
  Cell[187483, 7361, 427, 14, 26, "Input",
   CellTags->"Cases2"],
  Cell[187913, 7377, 355, 11, 26, "Output",
   CellTags->"Cases2"],
  Cell[188305, 7393, 327, 10, 26, "Input",
   CellTags->"Cases2"],
  Cell[188635, 7405, 463, 17, 28, "Output",
   CellTags->"Cases2"]},
 "CF"->{
  Cell[189125, 7426, 41, 1, 39, "ObjectName",
   CellTags->"CF"],
  Cell[189191, 7431, 53, 1, 36, "Subsubsection",
   CellTags->"CF"],
  Cell[189247, 7434, 287, 13, 24, "Text",
   CellTags->"CF"],
  Cell[189537, 7449, 435, 18, 24, "Text",
   CellTags->"CF"],
  Cell[190009, 7472, 50, 1, 36, "Subsubsection",
   CellTags->"CF"],
  Cell[190084, 7477, 70, 2, 26, "Input",
   CellTags->"CF"],
  Cell[190157, 7481, 120, 4, 26, "Output",
   CellTags->"CF"],
  Cell[190314, 7490, 178, 5, 26, "Input",
   CellTags->"CF"],
  Cell[190495, 7497, 193, 7, 44, "Output",
   CellTags->"CF"],
  Cell[190924, 7519, 72, 2, 26, "Input",
   CellTags->"CF"],
  Cell[190999, 7523, 98, 3, 27, "Output",
   CellTags->"CF"],
  Cell[191134, 7531, 233, 7, 26, "Input",
   CellTags->"CF"],
  Cell[191370, 7540, 177, 6, 26, "Output",
   CellTags->"CF"]},
 "ChangeDimension"->{
  Cell[191574, 7550, 67, 1, 39, "ObjectName",
   CellTags->"ChangeDimension"],
  Cell[191666, 7555, 66, 1, 28, "Subsubsection",
   CellTags->"ChangeDimension"],
  Cell[191735, 7558, 210, 5, 22, "Text",
   CellTags->"ChangeDimension"],
  Cell[191948, 7565, 848, 32, 22, "Text",
   CellTags->"ChangeDimension"],
  Cell[192833, 7602, 63, 1, 34, "Subsubsection",
   CellTags->"ChangeDimension"],
  Cell[192899, 7605, 219, 5, 41, "Text",
   CellTags->"ChangeDimension"],
  Cell[193143, 7614, 212, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[193358, 7622, 198, 7, 28, "Output",
   CellTags->"ChangeDimension"],
  Cell[193593, 7634, 151, 4, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[193747, 7640, 198, 7, 28, "Output",
   CellTags->"ChangeDimension"],
  Cell[193982, 7652, 116, 3, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[194101, 7657, 272, 8, 38, "Output",
   CellTags->"ChangeDimension"],
  Cell[194388, 7668, 119, 3, 24, "Text",
   CellTags->"ChangeDimension"],
  Cell[194532, 7675, 201, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[194736, 7683, 222, 6, 38, "Output",
   CellTags->"ChangeDimension"],
  Cell[194973, 7692, 105, 1, 24, "Text",
   CellTags->"ChangeDimension"],
  Cell[195103, 7697, 482, 15, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[195588, 7714, 571, 18, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[196196, 7737, 153, 4, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[196352, 7743, 214, 7, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[196603, 7755, 222, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[196828, 7763, 214, 7, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[197057, 7773, 132, 3, 24, "Text",
   CellTags->"ChangeDimension"],
  Cell[197192, 7778, 208, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[197403, 7786, 219, 5, 41, "Text",
   CellTags->"ChangeDimension"],
  Cell[197625, 7793, 201, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[197851, 7803, 220, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[198074, 7811, 489, 21, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[198600, 7837, 182, 5, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[198785, 7844, 489, 21, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[199311, 7870, 176, 5, 29, "Input",
   CellTags->"ChangeDimension"],
  Cell[199490, 7877, 301, 10, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[199828, 7892, 141, 4, 29, "Input",
   CellTags->"ChangeDimension"],
  Cell[199972, 7898, 130, 4, 26, "Output",
   CellTags->"ChangeDimension"],
  Cell[200117, 7905, 201, 6, 26, "Input",
   CellTags->"ChangeDimension"],
  Cell[200321, 7913, 157, 5, 26, "Input",
   CellTags->"ChangeDimension"]},
 "ChargeConjugationMatrix"->{
  Cell[200493, 7921, 83, 1, 39, "ObjectName",
   CellTags->"ChargeConjugationMatrix"],
  Cell[200601, 7926, 74, 1, 36, "Subsubsection",
   CellTags->"ChargeConjugationMatrix"],
  Cell[200678, 7929, 176, 6, 24, "Text",
   CellTags->"ChargeConjugationMatrix"],
  Cell[200857, 7937, 348, 11, 24, "Text",
   CellTags->"ChargeConjugationMatrix"],
  Cell[201242, 7953, 71, 1, 36, "Subsubsection",
   CellTags->"ChargeConjugationMatrix"],
  Cell[201338, 7958, 214, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[201555, 7965, 306, 11, 26, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[201898, 7981, 120, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202021, 7986, 255, 9, 30, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202313, 8000, 209, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202525, 8007, 228, 7, 26, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202790, 8019, 120, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[202913, 8024, 192, 6, 28, "Output",
   CellTags->"ChargeConjugationMatrix"],
  Cell[203142, 8035, 134, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrix"],
  Cell[203279, 8040, 137, 4, 26, "Output",
   CellTags->"ChargeConjugationMatrix"]},
 "ChargeConjugationMatrixInv"->{
  Cell[203443, 8048, 89, 1, 39, "ObjectName",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203557, 8053, 77, 1, 36, "Subsubsection",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203637, 8056, 195, 4, 24, "Text",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[203835, 8062, 339, 11, 24, "Text",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[204211, 8078, 74, 1, 36, "Subsubsection",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[204310, 8083, 217, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[204530, 8090, 309, 11, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[204876, 8106, 123, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205002, 8111, 258, 9, 30, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205297, 8125, 212, 5, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205512, 8132, 231, 7, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205780, 8144, 123, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[205906, 8149, 195, 6, 28, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206138, 8160, 165, 4, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206306, 8166, 189, 6, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206532, 8177, 119, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206654, 8182, 164, 5, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206855, 8192, 140, 3, 26, "Input",
   CellTags->"ChargeConjugationMatrixInv"],
  Cell[206998, 8197, 140, 4, 26, "Output",
   CellTags->"ChargeConjugationMatrixInv"]},
 "CheckContext"->{
  Cell[207165, 8205, 61, 1, 39, "ObjectName",
   CellTags->"CheckContext"],
  Cell[207251, 8210, 63, 1, 36, "Subsubsection",
   CellTags->"CheckContext"],
  Cell[207317, 8213, 164, 4, 24, "Text",
   CellTags->"CheckContext"],
  Cell[207484, 8219, 493, 18, 24, "Text",
   CellTags->"CheckContext"],
  Cell[208014, 8242, 60, 1, 36, "Subsubsection",
   CellTags->"CheckContext"],
  Cell[208077, 8245, 142, 4, 24, "Text",
   CellTags->"CheckContext"],
  Cell[208244, 8253, 138, 3, 26, "Input",
   CellTags->"CheckContext"],
  Cell[208385, 8258, 112, 3, 26, "Output",
   CellTags->"CheckContext"],
  Cell[208512, 8264, 87, 1, 24, "Text",
   CellTags->"CheckContext"],
  Cell[208624, 8269, 92, 2, 26, "Input",
   CellTags->"CheckContext"],
  Cell[208719, 8273, 121, 3, 26, "Output",
   CellTags->"CheckContext"],
  Cell[208877, 8281, 138, 3, 26, "Input",
   CellTags->"CheckContext"],
  Cell[209018, 8286, 111, 3, 26, "Output",
   CellTags->"CheckContext"]},
 "CheckDB"->{
  Cell[209156, 8293, 51, 1, 39, "ObjectName",
   CellTags->"CheckDB"],
  Cell[209232, 8298, 106, 3, 36, "Subsubsection",
   CellTags->"CheckDB"],
  Cell[210447, 8322, 113, 3, 26, "Input",
   CellTags->"CheckDB"],
  Cell[210563, 8327, 471, 14, 26, "Output",
   CellTags->"CheckDB"],
  Cell[211071, 8346, 116, 3, 26, "Input",
   CellTags->"CheckDB"],
  Cell[211190, 8351, 134, 4, 26, "Output",
   CellTags->"CheckDB"],
  Cell[212482, 8406, 383, 13, 26, "Input",
   CellTags->"CheckDB"],
  Cell[212868, 8421, 150, 5, 26, "Output",
   CellTags->"CheckDB"],
  Cell[213118, 8433, 303, 9, 29, "Input",
   CellTags->"CheckDB"],
  Cell[213424, 8444, 116, 3, 26, "Output",
   CellTags->"CheckDB"],
  Cell[213555, 8450, 268, 8, 29, "Input",
   CellTags->"CheckDB"]},
 "ChiralityProjector"->{
  Cell[213838, 8461, 73, 1, 39, "ObjectName",
   CellTags->"ChiralityProjector"],
  Cell[213936, 8466, 69, 1, 36, "Subsubsection",
   CellTags->"ChiralityProjector"],
  Cell[214008, 8469, 499, 19, 24, "Text",
   CellTags->"ChiralityProjector"],
  Cell[214510, 8490, 655, 25, 24, "Text",
   CellTags->"ChiralityProjector"],
  Cell[215202, 8520, 66, 1, 36, "Subsubsection",
   CellTags->"ChiralityProjector"],
  Cell[215293, 8525, 262, 8, 26, "Input",
   CellTags->"ChiralityProjector"],
  Cell[215558, 8535, 227, 7, 29, "Output",
   CellTags->"ChiralityProjector"],
  Cell[215822, 8547, 114, 3, 26, "Input",
   CellTags->"ChiralityProjector"],
  Cell[215939, 8552, 227, 7, 29, "Output",
   CellTags->"ChiralityProjector"],
  Cell[216203, 8564, 196, 5, 26, "Input",
   CellTags->"ChiralityProjector"],
  Cell[216402, 8571, 380, 14, 46, "Output",
   CellTags->"ChiralityProjector"]},
 "Chisholm"->{
  Cell[216809, 8589, 53, 1, 39, "ObjectName",
   CellTags->"Chisholm"],
  Cell[216887, 8594, 59, 1, 36, "Subsubsection",
   CellTags->"Chisholm"],
  Cell[216949, 8597, 142, 4, 24, "Text",
   CellTags->"Chisholm"],
  Cell[217094, 8603, 489, 18, 24, "Text",
   CellTags->"Chisholm"],
  Cell[217620, 8626, 56, 1, 36, "Subsubsection",
   CellTags->"Chisholm"],
  Cell[217701, 8631, 206, 6, 26, "Input",
   CellTags->"Chisholm"],
  Cell[217910, 8639, 2020, 78, 26, "Output",
   CellTags->"Chisholm"],
  Cell[219967, 8722, 183, 5, 26, "Input",
   CellTags->"Chisholm"],
  Cell[220153, 8729, 236, 7, 26, "Output",
   CellTags->"Chisholm"],
  Cell[220426, 8741, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[220568, 8747, 1374, 54, 26, "Output",
   CellTags->"Chisholm"],
  Cell[221957, 8804, 77, 1, 24, "Text",
   CellTags->"Chisholm"],
  Cell[222059, 8809, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[222192, 8815, 105, 3, 26, "Output",
   CellTags->"Chisholm"],
  Cell[222334, 8823, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[222476, 8829, 1374, 54, 26, "Output",
   CellTags->"Chisholm"],
  Cell[223887, 8888, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[224020, 8894, 105, 3, 26, "Output",
   CellTags->"Chisholm"],
  Cell[224162, 8902, 169, 5, 26, "Input",
   CellTags->"Chisholm"],
  Cell[224334, 8909, 590, 22, 26, "Output",
   CellTags->"Chisholm"],
  Cell[224961, 8936, 110, 3, 26, "Input",
   CellTags->"Chisholm"],
  Cell[225074, 8941, 1439, 57, 26, "Output",
   CellTags->"Chisholm"],
  Cell[226550, 9003, 232, 7, 26, "Input",
   CellTags->"Chisholm"],
  Cell[226785, 9012, 391, 10, 26, "Output",
   CellTags->"Chisholm"],
  Cell[227213, 9027, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[227355, 9033, 11544, 439, 87, "Output",
   CellTags->"Chisholm"],
  Cell[238936, 9477, 139, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[239078, 9483, 11544, 439, 87, "Output",
   CellTags->"Chisholm"],
  Cell[250637, 9925, 85, 1, 24, "Text",
   CellTags->"Chisholm"],
  Cell[250747, 9930, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[250880, 9936, 125, 4, 26, "Output",
   CellTags->"Chisholm"],
  Cell[251042, 9945, 130, 4, 26, "Input",
   CellTags->"Chisholm"],
  Cell[251175, 9951, 125, 4, 26, "Output",
   CellTags->"Chisholm"],
  Cell[251315, 9958, 192, 6, 26, "Input",
   CellTags->"Chisholm"]},
 "ChisholmSpinor"->{
  Cell[251522, 9967, 65, 1, 39, "ObjectName",
   CellTags->"ChisholmSpinor"],
  Cell[251612, 9972, 65, 1, 36, "Subsubsection",
   CellTags->"ChisholmSpinor"],
  Cell[251680, 9975, 296, 6, 41, "Text",
   CellTags->"ChisholmSpinor"],
  Cell[251979, 9983, 471, 18, 24, "Text",
   CellTags->"ChisholmSpinor"],
  Cell[252487, 10006, 62, 1, 36, "Subsubsection",
   CellTags->"ChisholmSpinor"],
  Cell[252574, 10011, 312, 9, 26, "Input",
   CellTags->"ChisholmSpinor"],
  Cell[252889, 10022, 427, 15, 26, "Output",
   CellTags->"ChisholmSpinor"],
  Cell[253353, 10042, 117, 3, 26, "Input",
   CellTags->"ChisholmSpinor"],
  Cell[253473, 10047, 2686, 99, 90, "Output",
   CellTags->"ChisholmSpinor"]},
 "ClearScalarProducts"->{
  Cell[256186, 10150, 75, 1, 39, "ObjectName",
   CellTags->"ClearScalarProducts"],
  Cell[256286, 10155, 70, 1, 36, "Subsubsection",
   CellTags->"ClearScalarProducts"],
  Cell[256359, 10158, 145, 4, 24, "Text",
   CellTags->"ClearScalarProducts"],
  Cell[256507, 10164, 802, 31, 24, "Text",
   CellTags->"ClearScalarProducts"],
  Cell[257346, 10200, 67, 1, 36, "Subsubsection",
   CellTags->"ClearScalarProducts"],
  Cell[257438, 10205, 195, 6, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[257636, 10213, 139, 4, 26, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[257812, 10222, 211, 6, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[258026, 10230, 139, 4, 26, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[258180, 10237, 104, 2, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[258309, 10243, 211, 6, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[258523, 10251, 173, 6, 28, "Output",
   CellTags->"ClearScalarProducts"],
  Cell[258733, 10262, 148, 4, 26, "Input",
   CellTags->"ClearScalarProducts"],
  Cell[258884, 10268, 173, 6, 28, "Output",
   CellTags->"ClearScalarProducts"]},
 "Collect2"->{
  Cell[260117, 10320, 53, 1, 39, "ObjectName",
   CellTags->"Collect2"],
  Cell[260195, 10325, 59, 1, 36, "Subsubsection",
   CellTags->"Collect2"],
  Cell[260257, 10328, 613, 10, 75, "Text",
   CellTags->"Collect2"],
  Cell[260895, 10342, 115, 3, 26, "Input",
   CellTags->"Collect2"],
  Cell[261013, 10347, 398, 11, 26, "Output",
   CellTags->"Collect2"],
  Cell[261426, 10361, 257, 11, 24, "Text",
   CellTags->"Collect2"],
  Cell[261720, 10377, 56, 1, 36, "Subsubsection",
   CellTags->"Collect2"],
  Cell[261801, 10382, 440, 14, 40, "Input",
   CellTags->"Collect2"],
  Cell[262244, 10398, 451, 16, 42, "Output",
   CellTags->"Collect2"],
  Cell[262732, 10419, 185, 5, 26, "Input",
   CellTags->"Collect2"],
  Cell[262920, 10426, 399, 14, 42, "Output",
   CellTags->"Collect2"],
  Cell[263356, 10445, 186, 5, 26, "Input",
   CellTags->"Collect2"],
  Cell[263545, 10452, 451, 16, 42, "Output",
   CellTags->"Collect2"],
  Cell[264033, 10473, 525, 18, 29, "Input",
   CellTags->"Collect2"],
  Cell[264561, 10493, 537, 18, 26, "Output",
   CellTags->"Collect2"],
  Cell[265135, 10516, 320, 10, 29, "Input",
   CellTags->"Collect2"],
  Cell[265458, 10528, 820, 31, 26, "Output",
   CellTags->"Collect2"],
  Cell[266315, 10564, 112, 3, 26, "Input",
   CellTags->"Collect2"],
  Cell[266430, 10569, 825, 27, 26, "Output",
   CellTags->"Collect2"],
  Cell[267270, 10599, 130, 4, 26, "Input",
   CellTags->"Collect2"],
  Cell[267425, 10607, 305, 9, 29, "Input",
   CellTags->"Collect2"],
  Cell[267733, 10618, 935, 31, 29, "Output",
   CellTags->"Collect2"],
  Cell[268705, 10654, 149, 4, 26, "Input",
   CellTags->"Collect2"],
  Cell[268857, 10660, 671, 21, 29, "Output",
   CellTags->"Collect2"]},
 "Collect3"->{
  Cell[269555, 10685, 53, 1, 39, "ObjectName",
   CellTags->"Collect3"],
  Cell[269633, 10690, 59, 1, 36, "Subsubsection",
   CellTags->"Collect3"],
  Cell[269695, 10693, 719, 23, 41, "Text",
   CellTags->"Collect3"],
  Cell[270439, 10720, 115, 3, 26, "Input",
   CellTags->"Collect3"],
  Cell[270557, 10725, 233, 8, 26, "Output",
   CellTags->"Collect3"],
  Cell[270805, 10736, 449, 18, 24, "Text",
   CellTags->"Collect3"],
  Cell[271291, 10759, 56, 1, 36, "Subsubsection",
   CellTags->"Collect3"],
  Cell[271372, 10764, 525, 18, 29, "Input",
   CellTags->"Collect3"],
  Cell[271900, 10784, 548, 19, 26, "Output",
   CellTags->"Collect3"],
  Cell[272485, 10808, 289, 9, 29, "Input",
   CellTags->"Collect3"],
  Cell[272777, 10819, 4614, 134, 139, "Output",
   CellTags->"Collect3"]},
 "Combine"->{
  Cell[278574, 10999, 51, 1, 39, "ObjectName",
   CellTags->"Combine"],
  Cell[278650, 11004, 58, 1, 36, "Subsubsection",
   CellTags->"Combine"],
  Cell[278711, 11007, 315, 6, 41, "Text",
   CellTags->"Combine"],
  Cell[279051, 11017, 113, 3, 26, "Input",
   CellTags->"Combine"],
  Cell[279167, 11022, 171, 6, 26, "Output",
   CellTags->"Combine"],
  Cell[279353, 11031, 256, 11, 24, "Text",
   CellTags->"Combine"],
  Cell[279646, 11047, 55, 1, 36, "Subsubsection",
   CellTags->"Combine"],
  Cell[279726, 11052, 283, 10, 43, "Input",
   CellTags->"Combine"],
  Cell[280012, 11064, 304, 11, 46, "Output",
   CellTags->"Combine"],
  Cell[280331, 11078, 122, 3, 24, "Text",
   CellTags->"Combine"],
  Cell[280478, 11085, 284, 10, 43, "Input",
   CellTags->"Combine"],
  Cell[280765, 11097, 303, 10, 46, "Output",
   CellTags->"Combine"],
  Cell[281083, 11110, 175, 4, 24, "Text",
   CellTags->"Combine"],
  Cell[281283, 11118, 354, 12, 43, "Input",
   CellTags->"Combine"],
  Cell[281640, 11132, 303, 10, 46, "Output",
   CellTags->"Combine"]},
 "CombineGraphs"->{
  Cell[281970, 11146, 63, 1, 39, "ObjectName",
   CellTags->"CombineGraphs"],
  Cell[282058, 11151, 64, 1, 36, "Subsubsection",
   CellTags->"CombineGraphs"],
  Cell[282125, 11154, 86, 1, 24, "Text",
   CellTags->"CombineGraphs"],
  Cell[282214, 11157, 269, 10, 24, "Text",
   CellTags->"CombineGraphs"]},
 "Commutator"->{
  Cell[282498, 11170, 57, 1, 39, "ObjectName",
   CellTags->"Commutator"],
  Cell[282580, 11175, 61, 1, 36, "Subsubsection",
   CellTags->"Commutator"],
  Cell[282644, 11178, 138, 4, 24, "Text",
   CellTags->"Commutator"],
  Cell[282785, 11184, 967, 32, 24, "Text",
   CellTags->"Commutator"],
  Cell[283789, 11221, 58, 1, 36, "Subsubsection",
   CellTags->"Commutator"],
  Cell[283850, 11224, 167, 4, 26, "Input",
   CellTags->"Commutator"],
  Cell[284042, 11232, 136, 4, 26, "Input",
   CellTags->"Commutator"],
  Cell[284181, 11238, 184, 5, 26, "Output",
   CellTags->"Commutator"],
  Cell[284402, 11248, 121, 3, 26, "Input",
   CellTags->"Commutator"],
  Cell[284526, 11253, 177, 6, 26, "Output",
   CellTags->"Commutator"],
  Cell[284740, 11264, 247, 8, 26, "Input",
   CellTags->"Commutator"],
  Cell[284990, 11274, 381, 12, 26, "Output",
   CellTags->"Commutator"],
  Cell[285386, 11289, 169, 4, 26, "Input",
   CellTags->"Commutator"],
  Cell[285558, 11295, 68, 1, 24, "Text",
   CellTags->"Commutator"],
  Cell[285629, 11298, 228, 6, 26, "Input",
   CellTags->"Commutator"],
  Cell[285882, 11308, 463, 15, 26, "Input",
   CellTags->"Commutator"],
  Cell[286348, 11325, 727, 19, 26, "Output",
   CellTags->"Commutator"],
  Cell[287112, 11349, 114, 3, 26, "Input",
   CellTags->"Commutator"],
  Cell[287229, 11354, 106, 3, 26, "Output",
   CellTags->"Commutator"],
  Cell[287350, 11360, 113, 3, 26, "Input",
   CellTags->"Commutator"],
  Cell[287466, 11365, 159, 4, 26, "Input",
   CellTags->"Commutator"]},
 "CommutatorExplicit"->{
  Cell[287640, 11372, 73, 1, 39, "ObjectName",
   CellTags->"CommutatorExplicit"],
  Cell[287738, 11377, 69, 1, 36, "Subsubsection",
   CellTags->"CommutatorExplicit"],
  Cell[287810, 11380, 157, 4, 24, "Text",
   CellTags->"CommutatorExplicit"],
  Cell[287970, 11386, 459, 18, 24, "Text",
   CellTags->"CommutatorExplicit"],
  Cell[288466, 11409, 66, 1, 36, "Subsubsection",
   CellTags->"CommutatorExplicit"],
  Cell[288535, 11412, 175, 4, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[288735, 11420, 144, 4, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[288882, 11426, 192, 5, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[289111, 11436, 129, 3, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[289243, 11441, 185, 6, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[289465, 11452, 196, 6, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[289664, 11460, 283, 10, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[289984, 11475, 129, 3, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[290116, 11480, 389, 14, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[290542, 11499, 167, 5, 26, "Input",
   CellTags->"CommutatorExplicit"],
  Cell[290712, 11506, 389, 12, 26, "Output",
   CellTags->"CommutatorExplicit"],
  Cell[291116, 11521, 177, 4, 26, "Input",
   CellTags->"CommutatorExplicit"]},
 "CompleteSquare"->{
  Cell[291308, 11528, 65, 1, 39, "ObjectName",
   CellTags->"CompleteSquare"],
  Cell[291398, 11533, 65, 1, 36, "Subsubsection",
   CellTags->"CompleteSquare"],
  Cell[291466, 11536, 1107, 82, 41, "Text",
   CellTags->"CompleteSquare"],
  Cell[292610, 11623, 62, 1, 36, "Subsubsection",
   CellTags->"CompleteSquare"],
  Cell[293351, 11657, 170, 5, 26, "Input",
   CellTags->"CompleteSquare"],
  Cell[293524, 11664, 685, 27, 46, "Output",
   CellTags->"CompleteSquare"],
  Cell[294246, 11696, 180, 6, 26, "Input",
   CellTags->"CompleteSquare"],
  Cell[294429, 11704, 110, 3, 26, "Output",
   CellTags->"CompleteSquare"],
  Cell[294576, 11712, 155, 4, 26, "Input",
   CellTags->"CompleteSquare"],
  Cell[294734, 11718, 515, 20, 46, "Output",
   CellTags->"CompleteSquare"],
  Cell[295264, 11741, 137, 4, 26, "Input",
   CellTags->"CompleteSquare"]},
 "ComplexConjugate"->{
  Cell[295416, 11748, 69, 1, 39, "ObjectName",
   CellTags->"ComplexConjugate"],
  Cell[295510, 11753, 67, 1, 36, "Subsubsection",
   CellTags->"ComplexConjugate"],
  Cell[295580, 11756, 414, 7, 58, "Text",
   CellTags->"ComplexConjugate"],
  Cell[295997, 11765, 709, 25, 24, "Text",
   CellTags->"ComplexConjugate"],
  Cell[296743, 11795, 64, 1, 36, "Subsubsection",
   CellTags->"ComplexConjugate"],
  Cell[296832, 11800, 196, 5, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[297031, 11807, 163, 5, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[297231, 11817, 121, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[297355, 11822, 239, 6, 38, "Output",
   CellTags->"ComplexConjugate"],
  Cell[297631, 11833, 152, 4, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[297786, 11839, 249, 7, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[298072, 11851, 125, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[298200, 11856, 427, 16, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[298664, 11877, 182, 5, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[298849, 11884, 531, 22, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[299417, 11911, 125, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[299545, 11916, 531, 22, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[300113, 11943, 190, 5, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[300306, 11950, 380, 17, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[300723, 11972, 270, 7, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[300996, 11981, 157, 4, 38, "Output",
   CellTags->"ComplexConjugate"],
  Cell[301190, 11990, 273, 8, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[301466, 12000, 335, 10, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[301838, 12015, 125, 3, 26, "Input",
   CellTags->"ComplexConjugate"],
  Cell[301966, 12020, 431, 15, 26, "Output",
   CellTags->"ComplexConjugate"],
  Cell[302412, 12038, 166, 4, 24, "Text",
   CellTags->"ComplexConjugate"]},
 "Contract"->{
  Cell[302593, 12045, 53, 1, 39, "ObjectName",
   CellTags->"Contract"],
  Cell[302671, 12050, 59, 1, 36, "Subsubsection",
   CellTags->"Contract"],
  Cell[302733, 12053, 411, 7, 58, "Text",
   CellTags->"Contract"],
  Cell[303169, 12064, 115, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[303287, 12069, 612, 15, 47, "Output",
   CellTags->"Contract"],
  Cell[304055, 12092, 677, 25, 24, "Text",
   CellTags->"Contract"],
  Cell[304769, 12122, 56, 1, 36, "Subsubsection",
   CellTags->"Contract"],
  Cell[304850, 12127, 233, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[305086, 12136, 207, 7, 29, "Output",
   CellTags->"Contract"],
  Cell[305330, 12148, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[305442, 12153, 236, 10, 28, "Output",
   CellTags->"Contract"],
  Cell[305715, 12168, 199, 6, 26, "Input",
   CellTags->"Contract"],
  Cell[305917, 12176, 264, 10, 29, "Output",
   CellTags->"Contract"],
  Cell[306218, 12191, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[306330, 12196, 221, 8, 28, "Output",
   CellTags->"Contract"],
  Cell[306588, 12209, 144, 4, 26, "Input",
   CellTags->"Contract"],
  Cell[306735, 12215, 155, 5, 26, "Output",
   CellTags->"Contract"],
  Cell[306905, 12223, 83, 1, 24, "Text",
   CellTags->"Contract"],
  Cell[307013, 12228, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[307125, 12233, 104, 3, 26, "Output",
   CellTags->"Contract"],
  Cell[307244, 12239, 208, 5, 41, "Text",
   CellTags->"Contract"],
  Cell[307477, 12248, 231, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[307711, 12257, 233, 8, 26, "Output",
   CellTags->"Contract"],
  Cell[307981, 12270, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[308093, 12275, 104, 3, 27, "Output",
   CellTags->"Contract"],
  Cell[308234, 12283, 231, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[308468, 12292, 233, 8, 26, "Output",
   CellTags->"Contract"],
  Cell[308738, 12305, 227, 7, 26, "Input",
   CellTags->"Contract"],
  Cell[308968, 12314, 181, 6, 29, "Output",
   CellTags->"Contract"],
  Cell[309186, 12325, 114, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[309303, 12330, 206, 8, 28, "Output",
   CellTags->"Contract"],
  Cell[309546, 12343, 277, 9, 26, "Input",
   CellTags->"Contract"],
  Cell[309826, 12354, 427, 18, 30, "Output",
   CellTags->"Contract"],
  Cell[310290, 12377, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[310402, 12382, 575, 24, 28, "Output",
   CellTags->"Contract"],
  Cell[311014, 12411, 271, 8, 26, "Input",
   CellTags->"Contract"],
  Cell[311288, 12421, 555, 23, 28, "Output",
   CellTags->"Contract"],
  Cell[311880, 12449, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[311992, 12454, 439, 19, 26, "Output",
   CellTags->"Contract"],
  Cell[312468, 12478, 313, 9, 26, "Input",
   CellTags->"Contract"],
  Cell[312784, 12489, 922, 40, 26, "Output",
   CellTags->"Contract"],
  Cell[313743, 12534, 109, 3, 26, "Input",
   CellTags->"Contract"],
  Cell[313855, 12539, 204, 7, 26, "Output",
   CellTags->"Contract"],
  Cell[314096, 12551, 426, 13, 26, "Input",
   CellTags->"Contract"],
  Cell[314525, 12566, 301, 9, 26, "Output",
   CellTags->"Contract"],
  Cell[314863, 12580, 139, 4, 26, "Input",
   CellTags->"Contract"],
  Cell[315005, 12586, 352, 12, 26, "Output",
   CellTags->"Contract"],
  Cell[315372, 12601, 194, 6, 26, "Input",
   CellTags->"Contract"]},
 "Contract1"->{
  Cell[315581, 12610, 55, 1, 39, "ObjectName",
   CellTags->"Contract1"],
  Cell[315661, 12615, 60, 1, 28, "Subsubsection",
   CellTags->"Contract1"],
  Cell[315724, 12618, 459, 8, 83, "Text",
   CellTags->"Contract1"],
  Cell[316186, 12628, 633, 24, 22, "Text",
   CellTags->"Contract1"],
  Cell[316856, 12657, 57, 1, 36, "Subsubsection",
   CellTags->"Contract1"],
  Cell[316938, 12662, 349, 12, 26, "Input",
   CellTags->"Contract1"],
  Cell[317290, 12676, 207, 8, 28, "Output",
   CellTags->"Contract1"],
  Cell[317534, 12689, 349, 12, 26, "Input",
   CellTags->"Contract1"],
  Cell[317886, 12703, 478, 20, 28, "Output",
   CellTags->"Contract1"],
  Cell[318401, 12728, 269, 9, 26, "Input",
   CellTags->"Contract1"],
  Cell[318673, 12739, 105, 3, 26, "Output",
   CellTags->"Contract1"],
  Cell[318815, 12747, 269, 9, 26, "Input",
   CellTags->"Contract1"],
  Cell[319087, 12758, 298, 11, 26, "Output",
   CellTags->"Contract1"]},
 "Convolute"->{
  Cell[319412, 12773, 55, 1, 39, "ObjectName",
   CellTags->"Convolute"],
  Cell[319492, 12778, 60, 1, 28, "Subsubsection",
   CellTags->"Convolute"],
  Cell[319555, 12781, 1363, 49, 64, "Text",
   CellTags->"Convolute"],
  Cell[320943, 12834, 117, 3, 26, "Input",
   CellTags->"Convolute"],
  Cell[321063, 12839, 400, 13, 26, "Output",
   CellTags->"Convolute"],
  Cell[321478, 12855, 510, 18, 22, "Text",
   CellTags->"Convolute"],
  Cell[322025, 12878, 57, 1, 36, "Subsubsection",
   CellTags->"Convolute"],
  Cell[322107, 12883, 134, 4, 26, "Input",
   CellTags->"Convolute"],
  Cell[322244, 12889, 154, 5, 26, "Output",
   CellTags->"Convolute"],
  Cell[322435, 12899, 134, 4, 26, "Input",
   CellTags->"Convolute"],
  Cell[322572, 12905, 178, 6, 26, "Output",
   CellTags->"Convolute"],
  Cell[322787, 12916, 134, 4, 26, "Input",
   CellTags->"Convolute"],
  Cell[322924, 12922, 128, 4, 26, "Output",
   CellTags->"Convolute"],
  Cell[323089, 12931, 181, 6, 42, "Input",
   CellTags->"Convolute"],
  Cell[323273, 12939, 215, 7, 26, "Output",
   CellTags->"Convolute"],
  Cell[323525, 12951, 227, 7, 42, "Input",
   CellTags->"Convolute"],
  Cell[323755, 12960, 215, 7, 26, "Output",
   CellTags->"Convolute"],
  Cell[324007, 12972, 181, 6, 42, "Input",
   CellTags->"Convolute"],
  Cell[324191, 12980, 307, 11, 26, "Output",
   CellTags->"Convolute"],
  Cell[324535, 12996, 228, 8, 42, "Input",
   CellTags->"Convolute"],
  Cell[324766, 13006, 536, 19, 43, "Output",
   CellTags->"Convolute"],
  Cell[325339, 13030, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[325532, 13038, 321, 11, 26, "Output",
   CellTags->"Convolute"],
  Cell[325890, 13054, 216, 7, 26, "Input",
   CellTags->"Convolute"],
  Cell[326109, 13063, 273, 10, 26, "Output",
   CellTags->"Convolute"],
  Cell[326419, 13078, 237, 8, 42, "Input",
   CellTags->"Convolute"],
  Cell[326659, 13088, 389, 13, 26, "Output",
   CellTags->"Convolute"],
  Cell[327085, 13106, 263, 9, 42, "Input",
   CellTags->"Convolute"],
  Cell[327351, 13117, 617, 21, 26, "Output",
   CellTags->"Convolute"],
  Cell[328005, 13143, 239, 8, 42, "Input",
   CellTags->"Convolute"],
  Cell[328247, 13153, 717, 24, 42, "Output",
   CellTags->"Convolute"],
  Cell[329001, 13182, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[329194, 13190, 245, 9, 26, "Output",
   CellTags->"Convolute"],
  Cell[329476, 13204, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[329669, 13212, 317, 11, 26, "Output",
   CellTags->"Convolute"],
  Cell[330023, 13228, 260, 9, 42, "Input",
   CellTags->"Convolute"],
  Cell[330286, 13239, 452, 16, 48, "Output",
   CellTags->"Convolute"],
  Cell[330775, 13260, 165, 5, 26, "Input",
   CellTags->"Convolute"],
  Cell[330943, 13267, 229, 9, 42, "Output",
   CellTags->"Convolute"],
  Cell[331209, 13281, 195, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[331407, 13289, 239, 9, 42, "Output",
   CellTags->"Convolute"],
  Cell[331683, 13303, 212, 7, 42, "Input",
   CellTags->"Convolute"],
  Cell[331898, 13312, 416, 14, 42, "Output",
   CellTags->"Convolute"],
  Cell[332351, 13331, 213, 7, 42, "Input",
   CellTags->"Convolute"],
  Cell[332567, 13340, 316, 12, 42, "Output",
   CellTags->"Convolute"],
  Cell[332920, 13357, 237, 8, 42, "Input",
   CellTags->"Convolute"],
  Cell[333160, 13367, 550, 18, 42, "Output",
   CellTags->"Convolute"],
  Cell[333747, 13390, 213, 7, 42, "Input",
   CellTags->"Convolute"],
  Cell[333963, 13399, 409, 13, 42, "Output",
   CellTags->"Convolute"],
  Cell[334409, 13417, 190, 6, 26, "Input",
   CellTags->"Convolute"],
  Cell[334602, 13425, 245, 9, 26, "Output",
   CellTags->"Convolute"],
  Cell[334884, 13439, 165, 5, 26, "Input",
   CellTags->"Convolute"],
  Cell[335052, 13446, 188, 6, 26, "Output",
   CellTags->"Convolute"],
  Cell[335277, 13457, 286, 10, 42, "Input",
   CellTags->"Convolute"],
  Cell[335566, 13469, 790, 29, 48, "Output",
   CellTags->"Convolute"],
  Cell[336393, 13503, 295, 10, 42, "Input",
   CellTags->"Convolute"],
  Cell[336691, 13515, 1193, 41, 64, "Output",
   CellTags->"Convolute"],
  Cell[340438, 13673, 244, 8, 42, "Input",
   CellTags->"Convolute"],
  Cell[340685, 13683, 463, 16, 42, "Output",
   CellTags->"Convolute"]},
 "ConvoluteTable"->{
  Cell[337911, 13560, 66, 1, 39, "ObjectName",
   CellTags->"ConvoluteTable"],
  Cell[338002, 13565, 65, 1, 36, "Subsubsection",
   CellTags->"ConvoluteTable"],
  Cell[338070, 13568, 152, 4, 24, "Text",
   CellTags->"ConvoluteTable"],
  Cell[338247, 13576, 122, 3, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[338372, 13581, 405, 13, 26, "Output",
   CellTags->"ConvoluteTable"],
  Cell[338792, 13597, 495, 18, 24, "Text",
   CellTags->"ConvoluteTable"],
  Cell[339324, 13620, 62, 1, 36, "Subsubsection",
   CellTags->"ConvoluteTable"],
  Cell[339411, 13625, 144, 4, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[339558, 13631, 159, 5, 26, "Output",
   CellTags->"ConvoluteTable"],
  Cell[339754, 13641, 144, 4, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[339901, 13647, 183, 6, 26, "Output",
   CellTags->"ConvoluteTable"],
  Cell[340121, 13658, 144, 4, 26, "Input",
   CellTags->"ConvoluteTable"],
  Cell[340268, 13664, 133, 4, 26, "Output",
   CellTags->"ConvoluteTable"]},
 "CovariantFieldDerivative"->{
  Cell[341175, 13703, 85, 1, 39, "ObjectName",
   CellTags->"CovariantFieldDerivative"],
  Cell[341285, 13708, 75, 1, 36, "Subsubsection",
   CellTags->"CovariantFieldDerivative"],
  Cell[341363, 13711, 372, 6, 58, "Text",
   CellTags->"CovariantFieldDerivative"],
  Cell[341738, 13719, 721, 25, 24, "Text",
   CellTags->"CovariantFieldDerivative"],
  Cell[342496, 13749, 72, 1, 36, "Subsubsection",
   CellTags->"CovariantFieldDerivative"],
  Cell[342593, 13754, 329, 9, 26, "Input",
   CellTags->"CovariantFieldDerivative"],
  Cell[342925, 13765, 543, 21, 29, "Output",
   CellTags->"CovariantFieldDerivative"]},
 "CounterT"->{
  Cell[343495, 13790, 53, 1, 39, "ObjectName",
   CellTags->"CounterT"],
  Cell[343573, 13795, 59, 1, 36, "Subsubsection",
   CellTags->"CounterT"],
  Cell[343635, 13798, 146, 4, 24, "Text",
   CellTags->"CounterT"],
  Cell[343784, 13804, 713, 25, 24, "Text",
   CellTags->"CounterT"],
  Cell[344534, 13834, 56, 1, 36, "Subsubsection",
   CellTags->"CounterT"],
  Cell[344615, 13839, 284, 6, 26, "Input",
   CellTags->"CounterT"],
  Cell[344902, 13847, 3127, 106, 63, "Output",
   CellTags->"CounterT"]},
 "CounterTerm"->{
  Cell[348056, 13957, 59, 1, 39, "ObjectName",
   CellTags->"CounterTerm"],
  Cell[348140, 13962, 62, 1, 36, "Subsubsection",
   CellTags->"CounterTerm"],
  Cell[348205, 13965, 216, 5, 41, "Text",
   CellTags->"CounterTerm"],
  Cell[348424, 13972, 928, 32, 24, "Text",
   CellTags->"CounterTerm"],
  Cell[349389, 14009, 59, 1, 36, "Subsubsection",
   CellTags->"CounterTerm"],
  Cell[349473, 14014, 116, 3, 26, "Input",
   CellTags->"CounterTerm"],
  Cell[349592, 14019, 1632, 52, 66, "Output",
   CellTags->"CounterTerm"]},
 "CouplingConstant"->{
  Cell[351251, 14075, 69, 1, 39, "ObjectName",
   CellTags->"CouplingConstant"],
  Cell[351345, 14080, 67, 1, 36, "Subsubsection",
   CellTags->"CouplingConstant"],
  Cell[351415, 14083, 657, 11, 100, "Text",
   CellTags->"CouplingConstant"],
  Cell[352075, 14096, 489, 18, 24, "Text",
   CellTags->"CouplingConstant"]},
 "CovariantD"->{
  Cell[352579, 14117, 57, 1, 39, "ObjectName",
   CellTags->"CovariantD"],
  Cell[352661, 14122, 61, 1, 36, "Subsubsection",
   CellTags->"CovariantD"],
  Cell[352725, 14125, 1210, 18, 201, "Text",
   CellTags->"CovariantD"],
  Cell[353960, 14147, 119, 3, 26, "Input",
   CellTags->"CovariantD"],
  Cell[354082, 14152, 439, 12, 26, "Output",
   CellTags->"CovariantD"],
  Cell[354536, 14167, 185, 4, 24, "Text",
   CellTags->"CovariantD"],
  Cell[354724, 14173, 719, 25, 24, "Text",
   CellTags->"CovariantD"],
  Cell[355480, 14203, 58, 1, 36, "Subsubsection",
   CellTags->"CovariantD"],
  Cell[355563, 14208, 117, 3, 26, "Input",
   CellTags->"CovariantD"],
  Cell[355683, 14213, 132, 4, 29, "Output",
   CellTags->"CovariantD"],
  Cell[355852, 14222, 150, 4, 26, "Input",
   CellTags->"CovariantD"],
  Cell[356005, 14228, 161, 5, 30, "Output",
   CellTags->"CovariantD"],
  Cell[356203, 14238, 180, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[356386, 14245, 992, 39, 34, "Output",
   CellTags->"CovariantD"],
  Cell[357393, 14287, 164, 4, 24, "Text",
   CellTags->"CovariantD"],
  Cell[357582, 14295, 120, 3, 26, "Input",
   CellTags->"CovariantD"],
  Cell[357705, 14300, 176, 6, 26, "Output",
   CellTags->"CovariantD"],
  Cell[357918, 14311, 153, 4, 26, "Input",
   CellTags->"CovariantD"],
  Cell[358074, 14317, 205, 7, 28, "Output",
   CellTags->"CovariantD"],
  Cell[358316, 14329, 203, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[358522, 14336, 939, 36, 33, "Output",
   CellTags->"CovariantD"],
  Cell[359498, 14377, 183, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[359684, 14384, 932, 35, 33, "Output",
   CellTags->"CovariantD"],
  Cell[360653, 14424, 187, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[360843, 14431, 1345, 49, 36, "Output",
   CellTags->"CovariantD"],
  Cell[362203, 14483, 602, 22, 31, "Text",
   CellTags->"CovariantD"],
  Cell[362830, 14509, 215, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[363048, 14517, 513, 20, 30, "Output",
   CellTags->"CovariantD"],
  Cell[363576, 14540, 285, 10, 24, "Text",
   CellTags->"CovariantD"],
  Cell[363886, 14554, 215, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[364104, 14562, 2287, 81, 54, "Output",
   CellTags->"CovariantD"],
  Cell[366406, 14646, 189, 7, 24, "Text",
   CellTags->"CovariantD"],
  Cell[366620, 14657, 215, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[366838, 14665, 6316, 201, 58, "Output",
   CellTags->"CovariantD"],
  Cell[373191, 14871, 181, 5, 26, "Input",
   CellTags->"CovariantD"],
  Cell[373375, 14878, 261, 9, 31, "Output",
   CellTags->"CovariantD"],
  Cell[373673, 14892, 195, 6, 26, "Input",
   CellTags->"CovariantD"],
  Cell[373871, 14900, 4816, 151, 58, "Output",
   CellTags->"CovariantD"],
  Cell[378724, 15056, 223, 7, 26, "Input",
   CellTags->"CovariantD"],
  Cell[378950, 15065, 788, 24, 58, "Output",
   CellTags->"CovariantD"],
  Cell[379775, 15094, 240, 7, 26, "Input",
   CellTags->"CovariantD"],
  Cell[380018, 15103, 803, 25, 58, "Output",
   CellTags->"CovariantD"]},
 "CrossProduct"->{
  Cell[380848, 15132, 61, 1, 39, "ObjectName",
   CellTags->"CrossProduct"],
  Cell[380934, 15137, 63, 1, 36, "Subsubsection",
   CellTags->"CrossProduct"],
  Cell[381000, 15140, 145, 4, 24, "Text",
   CellTags->"CrossProduct"],
  Cell[381148, 15146, 477, 18, 24, "Text",
   CellTags->"CrossProduct"],
  Cell[381662, 15169, 60, 1, 36, "Subsubsection",
   CellTags->"CrossProduct"],
  Cell[381747, 15174, 326, 9, 26, "Input",
   CellTags->"CrossProduct"],
  Cell[382076, 15185, 639, 22, 29, "Output",
   CellTags->"CrossProduct"]},
 "C0"->{
  Cell[382742, 15211, 41, 1, 39, "ObjectName",
   CellTags->"C0"],
  Cell[382808, 15216, 53, 1, 36, "Subsubsection",
   CellTags->"C0"],
  Cell[382864, 15219, 448, 10, 58, "Text",
   CellTags->"C0"],
  Cell[383315, 15231, 771, 32, 24, "Text",
   CellTags->"C0"],
  Cell[384123, 15268, 50, 1, 36, "Subsubsection",
   CellTags->"C0"],
  Cell[384198, 15273, 174, 5, 26, "Input",
   CellTags->"C0"],
  Cell[384375, 15280, 260, 8, 26, "Output",
   CellTags->"C0"],
  Cell[384672, 15293, 203, 6, 26, "Input",
   CellTags->"C0"],
  Cell[384878, 15301, 260, 8, 26, "Output",
   CellTags->"C0"],
  Cell[385175, 15314, 326, 10, 26, "Input",
   CellTags->"C0"],
  Cell[385504, 15326, 260, 8, 26, "Output",
   CellTags->"C0"]},
 "DataType"->{
  Cell[385791, 15338, 53, 1, 39, "ObjectName",
   CellTags->"DataType"],
  Cell[385869, 15343, 59, 1, 36, "Subsubsection",
   CellTags->"DataType"],
  Cell[385931, 15346, 344, 6, 41, "Text",
   CellTags->"DataType"],
  Cell[386278, 15354, 1361, 44, 40, "Text",
   CellTags->"DataType"],
  Cell[387642, 15400, 453, 17, 24, "Text",
   CellTags->"DataType"],
  Cell[388098, 15419, 313, 11, 24, "Text",
   CellTags->"DataType"],
  Cell[388448, 15435, 56, 1, 36, "Subsubsection",
   CellTags->"DataType"],
  Cell[388507, 15438, 74, 1, 24, "Text",
   CellTags->"DataType"],
  Cell[388584, 15441, 221, 7, 26, "Input",
   CellTags->"DataType"],
  Cell[388830, 15452, 224, 8, 26, "Input",
   CellTags->"DataType"],
  Cell[389057, 15462, 238, 9, 26, "Output",
   CellTags->"DataType"],
  Cell[389310, 15474, 179, 4, 24, "Text",
   CellTags->"DataType"],
  Cell[389514, 15482, 112, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[389629, 15487, 210, 7, 26, "Output",
   CellTags->"DataType"],
  Cell[389854, 15497, 271, 9, 26, "Input",
   CellTags->"DataType"],
  Cell[390128, 15508, 412, 14, 26, "Input",
   CellTags->"DataType"],
  Cell[390543, 15524, 395, 14, 26, "Input",
   CellTags->"DataType"],
  Cell[390963, 15542, 340, 13, 26, "Input",
   CellTags->"DataType"],
  Cell[391306, 15557, 350, 13, 26, "Output",
   CellTags->"DataType"],
  Cell[391693, 15575, 107, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[391803, 15580, 281, 11, 26, "Output",
   CellTags->"DataType"],
  Cell[392121, 15596, 107, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[392231, 15601, 172, 6, 26, "Output",
   CellTags->"DataType"],
  Cell[392418, 15610, 191, 6, 26, "Input",
   CellTags->"DataType"],
  Cell[392612, 15618, 187, 6, 26, "Input",
   CellTags->"DataType"],
  Cell[392802, 15626, 429, 15, 26, "Input",
   CellTags->"DataType"],
  Cell[393256, 15645, 290, 11, 26, "Input",
   CellTags->"DataType"],
  Cell[393549, 15658, 299, 12, 26, "Output",
   CellTags->"DataType"],
  Cell[393885, 15675, 105, 3, 26, "Input",
   CellTags->"DataType"],
  Cell[393993, 15680, 228, 9, 26, "Output",
   CellTags->"DataType"],
  Cell[394236, 15692, 151, 5, 26, "Input",
   CellTags->"DataType"],
  Cell[394390, 15699, 222, 7, 26, "Input",
   CellTags->"DataType"],
  Cell[394615, 15708, 272, 9, 26, "Input",
   CellTags->"DataType"],
  Cell[394890, 15719, 114, 3, 24, "Text",
   CellTags->"DataType"],
  Cell[395029, 15726, 149, 4, 26, "Input",
   CellTags->"DataType"],
  Cell[395181, 15732, 107, 3, 26, "Output",
   CellTags->"DataType"],
  Cell[395303, 15738, 83, 1, 24, "Text",
   CellTags->"DataType"],
  Cell[395411, 15743, 233, 8, 26, "Input",
   CellTags->"DataType"],
  Cell[395647, 15753, 104, 3, 26, "Output",
   CellTags->"DataType"],
  Cell[395788, 15761, 226, 7, 26, "Input",
   CellTags->"DataType"],
  Cell[396017, 15770, 435, 16, 28, "Output",
   CellTags->"DataType"]},
 "DB0"->{
  Cell[396479, 15790, 43, 1, 39, "ObjectName",
   CellTags->"DB0"],
  Cell[396547, 15795, 54, 1, 36, "Subsubsection",
   CellTags->"DB0"],
  Cell[396604, 15798, 146, 4, 24, "Text",
   CellTags->"DB0"],
  Cell[396753, 15804, 232, 11, 24, "Text",
   CellTags->"DB0"],
  Cell[397022, 15820, 51, 1, 36, "Subsubsection",
   CellTags->"DB0"],
  Cell[397098, 15825, 339, 12, 29, "Input",
   CellTags->"DB0"],
  Cell[397440, 15839, 255, 8, 29, "Output",
   CellTags->"DB0"]},
 "DB1"->{
  Cell[397722, 15851, 43, 1, 39, "ObjectName",
   CellTags->"DB1"],
  Cell[397790, 15856, 54, 1, 36, "Subsubsection",
   CellTags->"DB1"],
  Cell[397847, 15859, 117, 3, 24, "Text",
   CellTags->"DB1"],
  Cell[397967, 15864, 232, 11, 24, "Text",
   CellTags->"DB1"],
  Cell[398236, 15880, 51, 1, 36, "Subsubsection",
   CellTags->"DB1"],
  Cell[398312, 15885, 339, 12, 29, "Input",
   CellTags->"DB1"],
  Cell[398654, 15899, 1410, 45, 49, "Output",
   CellTags->"DB1"]},
 "DeclareNonCommutative"->{
  Cell[400091, 15948, 79, 1, 39, "ObjectName",
   CellTags->"DeclareNonCommutative"],
  Cell[400195, 15953, 72, 1, 36, "Subsubsection",
   CellTags->"DeclareNonCommutative"],
  Cell[400270, 15956, 192, 4, 24, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[400465, 15962, 526, 18, 24, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[401028, 15985, 69, 1, 36, "Subsubsection",
   CellTags->"DeclareNonCommutative"],
  Cell[401100, 15988, 135, 3, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[401238, 15993, 152, 4, 24, "Text",
   CellTags->"DeclareNonCommutative"],
  Cell[401415, 16001, 158, 4, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[401576, 16007, 120, 3, 26, "Output",
   CellTags->"DeclareNonCommutative"],
  Cell[401711, 16013, 158, 4, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[401894, 16021, 191, 5, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[402088, 16028, 199, 6, 26, "Output",
   CellTags->"DeclareNonCommutative"],
  Cell[402302, 16037, 170, 4, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[402497, 16045, 191, 5, 26, "Input",
   CellTags->"DeclareNonCommutative"],
  Cell[402691, 16052, 199, 6, 26, "Output",
   CellTags->"DeclareNonCommutative"]},
 "DeltaFunction"->{
  Cell[402917, 16062, 63, 1, 39, "ObjectName",
   CellTags->"DeltaFunction"],
  Cell[403005, 16067, 64, 1, 36, "Subsubsection",
   CellTags->"DeltaFunction"],
  Cell[403072, 16070, 353, 11, 49, "Text",
   CellTags->"DeltaFunction"],
  Cell[403428, 16083, 946, 32, 24, "Text",
   CellTags->"DeltaFunction"],
  Cell[404411, 16120, 61, 1, 36, "Subsubsection",
   CellTags->"DeltaFunction"],
  Cell[404497, 16125, 142, 4, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[404642, 16131, 203, 7, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[404882, 16143, 320, 10, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[405205, 16155, 137, 4, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[405379, 16164, 294, 9, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[405676, 16175, 137, 4, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[405850, 16184, 320, 10, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[406173, 16196, 137, 4, 26, "Output",
   CellTags->"DeltaFunction"],
  Cell[406347, 16205, 204, 6, 26, "Input",
   CellTags->"DeltaFunction"],
  Cell[406554, 16213, 109, 3, 27, "Output",
   CellTags->"DeltaFunction"]},
 "DeltaFunctionDoublePrime"->{
  Cell[406690, 16220, 85, 1, 39, "ObjectName",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[406800, 16225, 75, 1, 36, "Subsubsection",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[406878, 16228, 248, 8, 24, "Text",
   CellTags->"DeltaFunctionDoublePrime"],
  Cell[407129, 16238, 311, 10, 24, "Text",
   CellTags->"DeltaFunctionDoublePrime"]},
 "DeltaFunctionPrime"->{
  Cell[407455, 16251, 73, 1, 39, "ObjectName",
   CellTags->"DeltaFunctionPrime"],
  Cell[407553, 16256, 69, 1, 36, "Subsubsection",
   CellTags->"DeltaFunctionPrime"],
  Cell[407625, 16259, 227, 7, 24, "Text",
   CellTags->"DeltaFunctionPrime"],
  Cell[407855, 16268, 925, 31, 24, "Text",
   CellTags->"DeltaFunctionPrime"],
  Cell[408817, 16304, 66, 1, 36, "Subsubsection",
   CellTags->"DeltaFunctionPrime"],
  Cell[408908, 16309, 152, 4, 26, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[409063, 16315, 240, 8, 26, "Output",
   CellTags->"DeltaFunctionPrime"],
  Cell[409340, 16328, 330, 10, 26, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[409673, 16340, 203, 6, 26, "Output",
   CellTags->"DeltaFunctionPrime"],
  Cell[409913, 16351, 326, 10, 29, "Input",
   CellTags->"DeltaFunctionPrime"],
  Cell[410242, 16363, 114, 3, 26, "Output",
   CellTags->"DeltaFunctionPrime"]},
 "DenominatorOrder"->{
  Cell[410383, 16370, 69, 1, 39, "ObjectName",
   CellTags->"DenominatorOrder"],
  Cell[410477, 16375, 67, 1, 36, "Subsubsection",
   CellTags->"DenominatorOrder"],
  Cell[410547, 16378, 175, 4, 24, "Text",
   CellTags->"DenominatorOrder"],
  Cell[410725, 16384, 503, 17, 24, "Text",
   CellTags->"DenominatorOrder"]},
 "Dimension"->{
  Cell[411243, 16404, 55, 1, 39, "ObjectName",
   CellTags->"Dimension"],
  Cell[411323, 16409, 60, 1, 36, "Subsubsection",
   CellTags->"Dimension"],
  Cell[411386, 16412, 246, 5, 41, "Text",
   CellTags->"Dimension"],
  Cell[411657, 16421, 120, 3, 26, "Input",
   CellTags->"Dimension"],
  Cell[411780, 16426, 244, 8, 26, "Output",
   CellTags->"Dimension"],
  Cell[412061, 16439, 313, 9, 26, "Input",
   CellTags->"Dimension"],
  Cell[412377, 16450, 286, 11, 26, "Output",
   CellTags->"Dimension"],
  Cell[412678, 16464, 131, 4, 24, "Text",
   CellTags->"Dimension"],
  Cell[412834, 16472, 357, 11, 26, "Input",
   CellTags->"Dimension"],
  Cell[413194, 16485, 360, 10, 38, "Output",
   CellTags->"Dimension"],
  Cell[413569, 16498, 166, 4, 24, "Text",
   CellTags->"Dimension"],
  Cell[413738, 16504, 133, 4, 26, "Input",
   CellTags->"Dimension"],
  Cell[413896, 16512, 326, 9, 26, "Input",
   CellTags->"Dimension"],
  Cell[414225, 16523, 662, 28, 26, "Output",
   CellTags->"Dimension"],
  Cell[414924, 16556, 110, 3, 26, "Input",
   CellTags->"Dimension"],
  Cell[415037, 16561, 373, 10, 38, "Output",
   CellTags->"Dimension"],
  Cell[415425, 16574, 134, 4, 26, "Input",
   CellTags->"Dimension"]},
 "DimensionalReduction"->{
  Cell[415574, 16581, 77, 1, 39, "ObjectName",
   CellTags->"DimensionalReduction"],
  Cell[415676, 16586, 71, 1, 36, "Subsubsection",
   CellTags->"DimensionalReduction"],
  Cell[415750, 16589, 121, 3, 24, "Text",
   CellTags->"DimensionalReduction"],
  Cell[415874, 16594, 467, 17, 24, "Text",
   CellTags->"DimensionalReduction"]},
 "DiracBasis"->{
  Cell[416356, 16614, 57, 1, 39, "ObjectName",
   CellTags->"DiracBasis"],
  Cell[416438, 16619, 61, 1, 36, "Subsubsection",
   CellTags->"DiracBasis"],
  Cell[416502, 16622, 299, 6, 41, "Text",
   CellTags->"DiracBasis"],
  Cell[416804, 16630, 269, 10, 24, "Text",
   CellTags->"DiracBasis"]},
 "DiracCanonical"->{
  Cell[417088, 16643, 65, 1, 39, "ObjectName",
   CellTags->"DiracCanonical"],
  Cell[417178, 16648, 65, 1, 36, "Subsubsection",
   CellTags->"DiracCanonical"],
  Cell[417246, 16651, 171, 4, 24, "Text",
   CellTags->"DiracCanonical"],
  Cell[417420, 16657, 281, 10, 24, "Text",
   CellTags->"DiracCanonical"]},
 "DiracEquation"->{
  Cell[417716, 16670, 63, 1, 39, "ObjectName",
   CellTags->"DiracEquation"],
  Cell[417804, 16675, 64, 1, 36, "Subsubsection",
   CellTags->"DiracEquation"],
  Cell[417871, 16678, 160, 4, 24, "Text",
   CellTags->"DiracEquation"],
  Cell[418034, 16684, 286, 11, 24, "Text",
   CellTags->"DiracEquation"],
  Cell[418357, 16700, 61, 1, 36, "Subsubsection",
   CellTags->"DiracEquation"],
  Cell[418443, 16705, 279, 8, 26, "Input",
   CellTags->"DiracEquation"],
  Cell[418725, 16715, 368, 13, 28, "Output",
   CellTags->"DiracEquation"],
  Cell[419130, 16733, 119, 3, 26, "Input",
   CellTags->"DiracEquation"],
  Cell[419252, 16738, 217, 7, 28, "Output",
   CellTags->"DiracEquation"]},
 "DiracGamma"->{
  Cell[419496, 16749, 57, 1, 39, "ObjectName",
   CellTags->"DiracGamma"],
  Cell[419578, 16754, 61, 1, 36, "Subsubsection",
   CellTags->"DiracGamma"],
  Cell[419642, 16757, 762, 24, 58, "Text",
   CellTags->"DiracGamma"],
  Cell[420407, 16783, 1111, 39, 24, "Text",
   CellTags->"DiracGamma"],
  Cell[421555, 16827, 58, 1, 36, "Subsubsection",
   CellTags->"DiracGamma"],
  Cell[421638, 16832, 113, 3, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[421754, 16837, 137, 4, 26, "Output",
   CellTags->"DiracGamma"],
  Cell[421928, 16846, 159, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[422090, 16852, 214, 8, 26, "Output",
   CellTags->"DiracGamma"],
  Cell[422319, 16863, 332, 13, 27, "Text",
   CellTags->"DiracGamma"],
  Cell[422676, 16880, 167, 5, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[422846, 16887, 223, 8, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[423106, 16900, 280, 8, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[423389, 16910, 468, 17, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[423894, 16932, 218, 7, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[424115, 16941, 223, 8, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[424375, 16954, 208, 7, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[424586, 16963, 341, 12, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[424964, 16980, 144, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[425111, 16986, 593, 22, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[425741, 17013, 300, 9, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[426044, 17024, 430, 13, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[426511, 17042, 138, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[426652, 17048, 767, 28, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[427456, 17081, 141, 4, 26, "Input",
   CellTags->"DiracGamma"],
  Cell[427600, 17087, 1123, 42, 28, "Output",
   CellTags->"DiracGamma"],
  Cell[428738, 17132, 168, 5, 26, "Input",
   CellTags->"DiracGamma"]},
 "DiracGammaCombine"->{
  Cell[428921, 17140, 71, 1, 39, "ObjectName",
   CellTags->"DiracGammaCombine"],
  Cell[429017, 17145, 68, 1, 36, "Subsubsection",
   CellTags->"DiracGammaCombine"],
  Cell[429088, 17148, 135, 4, 24, "Text",
   CellTags->"DiracGammaCombine"],
  Cell[429226, 17154, 1318, 46, 24, "Text",
   CellTags->"DiracGammaCombine"],
  Cell[430581, 17205, 65, 1, 36, "Subsubsection",
   CellTags->"DiracGammaCombine"],
  Cell[430671, 17210, 220, 6, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[430894, 17218, 265, 9, 28, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[431196, 17232, 122, 3, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[431321, 17237, 194, 5, 38, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[431552, 17247, 270, 8, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[431825, 17257, 317, 11, 28, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[432179, 17273, 122, 3, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[432304, 17278, 244, 7, 38, "Output",
   CellTags->"DiracGammaCombine"],
  Cell[432585, 17290, 127, 3, 26, "Input",
   CellTags->"DiracGammaCombine"],
  Cell[432715, 17295, 438, 16, 28, "Output",
   CellTags->"DiracGammaCombine"]},
 "DiracGammaExpand"->{
  Cell[433180, 17315, 69, 1, 39, "ObjectName",
   CellTags->"DiracGammaExpand"],
  Cell[433274, 17320, 67, 1, 36, "Subsubsection",
   CellTags->"DiracGammaExpand"],
  Cell[433344, 17323, 190, 4, 41, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[433537, 17329, 1321, 46, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[434895, 17380, 64, 1, 36, "Subsubsection",
   CellTags->"DiracGammaExpand"],
  Cell[434984, 17385, 314, 9, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[435301, 17396, 474, 17, 28, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[435790, 17416, 125, 3, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[435940, 17423, 121, 3, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[436064, 17428, 291, 8, 38, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[436370, 17439, 101, 1, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[436496, 17444, 125, 3, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[436624, 17449, 599, 22, 28, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[437238, 17474, 90, 1, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[437353, 17479, 209, 5, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[437565, 17486, 291, 8, 38, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[437893, 17499, 164, 4, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[438060, 17505, 394, 11, 38, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[438469, 17519, 106, 1, 24, "Text",
   CellTags->"DiracGammaExpand"],
  Cell[438600, 17524, 122, 3, 26, "Input",
   CellTags->"DiracGammaExpand"],
  Cell[438725, 17529, 534, 20, 28, "Output",
   CellTags->"DiracGammaExpand"],
  Cell[439274, 17552, 114, 3, 26, "Input",
   CellTags->"DiracGammaExpand"]},
 "DiracGammaT"->{
  Cell[439403, 17558, 59, 1, 39, "ObjectName",
   CellTags->"DiracGammaT"],
  Cell[439487, 17563, 62, 1, 36, "Subsubsection",
   CellTags->"DiracGammaT"],
  Cell[439552, 17566, 208, 5, 41, "Text",
   CellTags->"DiracGammaT"],
  Cell[439763, 17573, 272, 11, 24, "Text",
   CellTags->"DiracGammaT"],
  Cell[440072, 17589, 59, 1, 36, "Subsubsection",
   CellTags->"DiracGammaT"],
  Cell[440156, 17594, 158, 4, 26, "Input",
   CellTags->"DiracGammaT"],
  Cell[440317, 17600, 220, 8, 30, "Output",
   CellTags->"DiracGammaT"],
  Cell[440574, 17613, 113, 3, 26, "Input",
   CellTags->"DiracGammaT"],
  Cell[440690, 17618, 212, 8, 26, "Output",
   CellTags->"DiracGammaT"],
  Cell[440939, 17631, 167, 6, 26, "Input",
   CellTags->"DiracGammaT"],
  Cell[441109, 17639, 219, 7, 28, "Output",
   CellTags->"DiracGammaT"]},
 "DiracMatrix"->{
  Cell[441355, 17650, 59, 1, 39, "ObjectName",
   CellTags->"DiracMatrix"],
  Cell[441439, 17655, 62, 1, 36, "Subsubsection",
   CellTags->"DiracMatrix"],
  Cell[441504, 17658, 1220, 46, 41, "Text",
   CellTags->"DiracMatrix"],
  Cell[442727, 17706, 1788, 67, 24, "Text",
   CellTags->"DiracMatrix"],
  Cell[444552, 17778, 59, 1, 36, "Subsubsection",
   CellTags->"DiracMatrix"],
  Cell[444636, 17783, 119, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[444758, 17788, 212, 8, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[444985, 17799, 539, 20, 24, "Text",
   CellTags->"DiracMatrix"],
  Cell[445549, 17823, 184, 5, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[445736, 17830, 350, 14, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[446123, 17849, 160, 5, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[446286, 17856, 175, 4, 38, "Output",
   CellTags->"DiracMatrix"],
  Cell[446498, 17865, 145, 4, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[446646, 17871, 142, 4, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[446825, 17880, 112, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[446940, 17885, 125, 3, 38, "Output",
   CellTags->"DiracMatrix"],
  Cell[447102, 17893, 111, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[447216, 17898, 142, 4, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[447395, 17907, 138, 4, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[447536, 17913, 221, 6, 38, "Output",
   CellTags->"DiracMatrix"],
  Cell[447794, 17924, 152, 4, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[447949, 17930, 249, 7, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[448235, 17942, 166, 5, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[448404, 17949, 342, 14, 26, "Output",
   CellTags->"DiracMatrix"],
  Cell[448783, 17968, 112, 3, 26, "Input",
   CellTags->"DiracMatrix"],
  Cell[448898, 17973, 269, 7, 38, "Output",
   CellTags->"DiracMatrix"]},
 "DiracOrder"->{
  Cell[449194, 17984, 57, 1, 39, "ObjectName",
   CellTags->"DiracOrder"],
  Cell[449276, 17989, 61, 1, 36, "Subsubsection",
   CellTags->"DiracOrder"],
  Cell[449340, 17992, 204, 5, 41, "Text",
   CellTags->"DiracOrder"],
  Cell[449547, 17999, 483, 18, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[450067, 18022, 58, 1, 36, "Subsubsection",
   CellTags->"DiracOrder"],
  Cell[450150, 18027, 176, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[450329, 18034, 206, 6, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[450572, 18045, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[450689, 18050, 472, 18, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[451176, 18071, 93, 1, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[451294, 18076, 176, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[451473, 18083, 247, 7, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[451757, 18095, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[451874, 18100, 472, 18, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[452383, 18123, 171, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[452557, 18130, 243, 7, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[452815, 18140, 196, 8, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[453036, 18152, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[453153, 18157, 391, 15, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[453581, 18177, 167, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[453751, 18184, 239, 7, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[454027, 18196, 114, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[454144, 18201, 271, 10, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[454452, 18216, 183, 5, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[454638, 18223, 255, 7, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[454908, 18233, 288, 10, 24, "Text",
   CellTags->"DiracOrder"],
  Cell[455221, 18247, 219, 7, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[455443, 18256, 1207, 45, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[456687, 18306, 113, 3, 26, "Input",
   CellTags->"DiracOrder"],
  Cell[456803, 18311, 477, 19, 26, "Output",
   CellTags->"DiracOrder"],
  Cell[457295, 18333, 188, 6, 26, "Input",
   CellTags->"DiracOrder"]},
 "DiracReduce"->{
  Cell[457498, 18342, 59, 1, 39, "ObjectName",
   CellTags->"DiracReduce"],
  Cell[457582, 18347, 62, 1, 36, "Subsubsection",
   CellTags->"DiracReduce"],
  Cell[457647, 18350, 662, 11, 92, "Text",
   CellTags->"DiracReduce"],
  Cell[458334, 18365, 121, 3, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[458458, 18370, 323, 10, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[458796, 18383, 284, 11, 22, "Text",
   CellTags->"DiracReduce"],
  Cell[459117, 18399, 59, 1, 36, "Subsubsection",
   CellTags->"DiracReduce"],
  Cell[459201, 18404, 172, 5, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[459376, 18411, 202, 6, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[459615, 18422, 116, 3, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[459734, 18427, 281, 9, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[460052, 18441, 186, 5, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[460241, 18448, 239, 7, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[460517, 18460, 116, 3, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[460636, 18465, 919, 28, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[461592, 18498, 207, 6, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[461802, 18506, 283, 8, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[462122, 18519, 140, 4, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[462265, 18525, 1814, 54, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[464116, 18584, 253, 7, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[464372, 18593, 127, 4, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[464536, 18602, 133, 4, 26, "Input",
   CellTags->"DiracReduce"],
  Cell[464672, 18608, 127, 4, 26, "Output",
   CellTags->"DiracReduce"],
  Cell[464814, 18615, 156, 4, 26, "Input",
   CellTags->"DiracReduce"]},
 "DiracSigma"->{
  Cell[464985, 18622, 57, 1, 39, "ObjectName",
   CellTags->"DiracSigma"],
  Cell[465067, 18627, 61, 1, 36, "Subsubsection",
   CellTags->"DiracSigma"],
  Cell[465131, 18630, 213, 5, 41, "Text",
   CellTags->"DiracSigma"],
  Cell[465347, 18637, 303, 11, 24, "Text",
   CellTags->"DiracSigma"],
  Cell[465687, 18653, 58, 1, 36, "Subsubsection",
   CellTags->"DiracSigma"],
  Cell[465770, 18658, 236, 7, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[466009, 18667, 169, 5, 26, "Output",
   CellTags->"DiracSigma"],
  Cell[466215, 18677, 122, 3, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[466340, 18682, 784, 29, 42, "Output",
   CellTags->"DiracSigma"],
  Cell[467161, 18716, 236, 7, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[467400, 18725, 189, 6, 26, "Output",
   CellTags->"DiracSigma"],
  Cell[467626, 18736, 223, 7, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[467852, 18745, 156, 5, 26, "Output",
   CellTags->"DiracSigma"],
  Cell[468045, 18755, 122, 3, 26, "Input",
   CellTags->"DiracSigma"],
  Cell[468170, 18760, 950, 33, 42, "Output",
   CellTags->"DiracSigma"],
  Cell[469135, 18796, 144, 4, 26, "Input",
   CellTags->"DiracSigma"]},
 "DiracSigmaExplicit"->{
  Cell[469294, 18803, 73, 1, 39, "ObjectName",
   CellTags->"DiracSigmaExplicit"],
  Cell[469392, 18808, 69, 1, 36, "Subsubsection",
   CellTags->"DiracSigmaExplicit"],
  Cell[469464, 18811, 186, 4, 24, "Text",
   CellTags->"DiracSigmaExplicit"],
  Cell[469653, 18817, 279, 11, 24, "Text",
   CellTags->"DiracSigmaExplicit"],
  Cell[469969, 18833, 66, 1, 36, "Subsubsection",
   CellTags->"DiracSigmaExplicit"],
  Cell[470060, 18838, 217, 6, 26, "Input",
   CellTags->"DiracSigmaExplicit"],
  Cell[470280, 18846, 177, 5, 26, "Output",
   CellTags->"DiracSigmaExplicit"],
  Cell[470494, 18856, 129, 3, 26, "Input",
   CellTags->"DiracSigmaExplicit"],
  Cell[470626, 18861, 792, 29, 42, "Output",
   CellTags->"DiracSigmaExplicit"]},
 "DiracSimpCombine"->{
  Cell[471445, 18894, 69, 1, 39, "ObjectName",
   CellTags->"DiracSimpCombine"],
  Cell[471539, 18899, 67, 1, 36, "Subsubsection",
   CellTags->"DiracSimpCombine"],
  Cell[471609, 18902, 209, 5, 22, "Text",
   CellTags->"DiracSimpCombine"],
  Cell[471821, 18909, 283, 10, 22, "Text",
   CellTags->"DiracSimpCombine"]},
 "DiracSimplify"->{
  Cell[472119, 18922, 63, 1, 39, "ObjectName",
   CellTags->"DiracSimplify"],
  Cell[472207, 18927, 64, 1, 28, "Subsubsection",
   CellTags->"DiracSimplify"],
  Cell[472274, 18930, 382, 7, 58, "Text",
   CellTags->"DiracSimplify"],
  Cell[472681, 18941, 125, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[472809, 18946, 604, 14, 47, "Output",
   CellTags->"DiracSimplify"],
  Cell[473428, 18963, 674, 25, 22, "Text",
   CellTags->"DiracSimplify"],
  Cell[474139, 18993, 61, 1, 34, "Subsubsection",
   CellTags->"DiracSimplify"],
  Cell[474203, 18996, 99, 1, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[474327, 19001, 178, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[474508, 19008, 250, 7, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[474795, 19020, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[474918, 19025, 261, 10, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[475194, 19038, 96, 1, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[475315, 19043, 179, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[475497, 19050, 250, 7, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[475784, 19062, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[475907, 19067, 261, 10, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[476205, 19082, 174, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[476382, 19089, 246, 7, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[476643, 19099, 179, 7, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[476847, 19110, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[476970, 19115, 394, 15, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[477401, 19135, 170, 5, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[477574, 19142, 242, 7, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[477853, 19154, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[477976, 19159, 274, 10, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[478287, 19174, 326, 10, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[478616, 19186, 570, 18, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[479201, 19207, 120, 3, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[479346, 19214, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[479469, 19219, 1908, 72, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[481414, 19296, 117, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[481534, 19301, 607, 22, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[482178, 19328, 272, 7, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[482453, 19337, 407, 14, 28, "Output",
   CellTags->"DiracSimplify"],
  Cell[482897, 19356, 120, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[483020, 19361, 109, 3, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[483166, 19369, 373, 12, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[483542, 19383, 624, 18, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[484203, 19406, 119, 3, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[484325, 19411, 13964, 531, 87, "Output",
   CellTags->"DiracSimplify"],
  Cell[498304, 19945, 115, 3, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[498444, 19952, 251, 7, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[498698, 19961, 465, 18, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[499178, 19982, 141, 4, 24, "Text",
   CellTags->"DiracSimplify"],
  Cell[499344, 19990, 289, 8, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[499636, 20000, 556, 19, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[500229, 20024, 268, 7, 26, "Input",
   CellTags->"DiracSimplify"],
  Cell[500500, 20033, 109, 3, 26, "Output",
   CellTags->"DiracSimplify"],
  Cell[500624, 20039, 183, 5, 26, "Input",
   CellTags->"DiracSimplify"]},
 "DiracSimplify2"->{
  Cell[500822, 20047, 65, 1, 39, "ObjectName",
   CellTags->"DiracSimplify2"],
  Cell[500912, 20052, 65, 1, 36, "Subsubsection",
   CellTags->"DiracSimplify2"],
  Cell[500980, 20055, 224, 7, 24, "Text",
   CellTags->"DiracSimplify2"],
  Cell[501207, 20064, 287, 11, 24, "Text",
   CellTags->"DiracSimplify2"],
  Cell[501531, 20080, 62, 1, 36, "Subsubsection",
   CellTags->"DiracSimplify2"],
  Cell[501618, 20085, 222, 6, 26, "Input",
   CellTags->"DiracSimplify2"],
  Cell[501843, 20093, 439, 11, 26, "Output",
   CellTags->"DiracSimplify2"],
  Cell[502319, 20109, 121, 3, 26, "Input",
   CellTags->"DiracSimplify2"],
  Cell[502443, 20114, 883, 35, 26, "Output",
   CellTags->"DiracSimplify2"]},
 "DiracSlash"->{
  Cell[503353, 20153, 57, 1, 39, "ObjectName",
   CellTags->"DiracSlash"],
  Cell[503435, 20158, 61, 1, 36, "Subsubsection",
   CellTags->"DiracSlash"],
  Cell[503499, 20161, 469, 16, 27, "Text",
   CellTags->"DiracSlash"],
  Cell[503993, 20181, 119, 3, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[504115, 20186, 245, 8, 26, "Output",
   CellTags->"DiracSlash"],
  Cell[504375, 20197, 1451, 53, 24, "Text",
   CellTags->"DiracSlash"],
  Cell[505863, 20255, 58, 1, 36, "Subsubsection",
   CellTags->"DiracSlash"],
  Cell[505924, 20258, 234, 9, 27, "Text",
   CellTags->"DiracSlash"],
  Cell[506183, 20271, 113, 3, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[506299, 20276, 223, 8, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[506559, 20289, 173, 5, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[506735, 20296, 431, 16, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[507203, 20317, 136, 4, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[507342, 20323, 431, 16, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[507810, 20344, 105, 3, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[507918, 20349, 147, 4, 28, "Output",
   CellTags->"DiracSlash"],
  Cell[508102, 20358, 148, 4, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[508253, 20364, 128, 3, 38, "Output",
   CellTags->"DiracSlash"],
  Cell[508418, 20372, 214, 7, 26, "Input",
   CellTags->"DiracSlash"],
  Cell[508635, 20381, 189, 5, 38, "Output",
   CellTags->"DiracSlash"]},
 "DiracSpinor"->{
  Cell[508851, 20390, 59, 1, 39, "ObjectName",
   CellTags->"DiracSpinor"],
  Cell[508935, 20395, 62, 1, 36, "Subsubsection",
   CellTags->"DiracSpinor"],
  Cell[509000, 20398, 288, 6, 41, "Text",
   CellTags->"DiracSpinor"],
  Cell[509291, 20406, 250, 10, 24, "Text",
   CellTags->"DiracSpinor"]},
 "DiracSubstitute67"->{
  Cell[509556, 20419, 71, 1, 39, "ObjectName",
   CellTags->"DiracSubstitute67"],
  Cell[509652, 20424, 68, 1, 36, "Subsubsection",
   CellTags->"DiracSubstitute67"],
  Cell[509723, 20427, 219, 5, 41, "Text",
   CellTags->"DiracSubstitute67"],
  Cell[509945, 20434, 484, 17, 24, "Text",
   CellTags->"DiracSubstitute67"]},
 "DiracTrace"->{
  Cell[510444, 20454, 57, 1, 39, "ObjectName",
   CellTags->"DiracTrace"],
  Cell[510526, 20459, 61, 1, 36, "Subsubsection",
   CellTags->"DiracTrace"],
  Cell[510590, 20462, 336, 6, 41, "Text",
   CellTags->"DiracTrace"],
  Cell[510951, 20472, 119, 3, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[511073, 20477, 668, 16, 47, "Output",
   CellTags->"DiracTrace"],
  Cell[511756, 20496, 359, 14, 24, "Text",
   CellTags->"DiracTrace"],
  Cell[512118, 20512, 239, 11, 24, "Text",
   CellTags->"DiracTrace"],
  Cell[512394, 20528, 58, 1, 36, "Subsubsection",
   CellTags->"DiracTrace"],
  Cell[512477, 20533, 174, 5, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[512654, 20540, 270, 9, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[512961, 20554, 209, 6, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[513173, 20562, 370, 11, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[513580, 20578, 149, 4, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[513732, 20584, 635, 22, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[514404, 20611, 296, 8, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[514703, 20621, 566, 23, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[515306, 20649, 334, 9, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[515643, 20660, 8415, 321, 52, "Output",
   CellTags->"DiracTrace"],
  Cell[524095, 20986, 186, 5, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[524284, 20993, 498, 15, 28, "Output",
   CellTags->"DiracTrace"],
  Cell[524819, 21013, 294, 8, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[525116, 21023, 182, 6, 26, "Output",
   CellTags->"DiracTrace"],
  Cell[525335, 21034, 111, 3, 26, "Input",
   CellTags->"DiracTrace"],
  Cell[525449, 21039, 175, 5, 38, "Output",
   CellTags->"DiracTrace"]},
 "DiracTraceEvaluate"->{
  Cell[525651, 21048, 73, 1, 39, "ObjectName",
   CellTags->"DiracTraceEvaluate"],
  Cell[525749, 21053, 69, 1, 36, "Subsubsection",
   CellTags->"DiracTraceEvaluate"],
  Cell[525821, 21056, 162, 4, 24, "Text",
   CellTags->"DiracTraceEvaluate"],
  Cell[525986, 21062, 441, 17, 24, "Text",
   CellTags->"DiracTraceEvaluate"]},
 "DiracTrick"->{
  Cell[526442, 21082, 57, 1, 39, "ObjectName",
   CellTags->"DiracTrick"],
  Cell[526524, 21087, 61, 1, 36, "Subsubsection",
   CellTags->"DiracTrick"],
  Cell[526588, 21090, 222, 5, 41, "Text",
   CellTags->"DiracTrick"],
  Cell[526835, 21099, 119, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[526957, 21104, 174, 6, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[527146, 21113, 683, 25, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[527866, 21143, 58, 1, 36, "Subsubsection",
   CellTags->"DiracTrick"],
  Cell[527927, 21146, 96, 1, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[528048, 21151, 175, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[528226, 21158, 247, 7, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[528510, 21170, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[528627, 21175, 258, 10, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[528900, 21188, 93, 1, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[529018, 21193, 176, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[529197, 21200, 247, 7, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[529481, 21212, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[529598, 21217, 288, 11, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[529923, 21233, 171, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[530097, 21240, 243, 7, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[530355, 21250, 176, 7, 24, "Text",
   CellTags->"DiracTrick"],
  Cell[530556, 21261, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[530673, 21266, 391, 15, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[531101, 21286, 167, 5, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[531271, 21293, 239, 7, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[531547, 21305, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[531664, 21310, 271, 10, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[531972, 21325, 323, 10, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[532298, 21337, 567, 18, 28, "Output",
   CellTags->"DiracTrick"],
  Cell[532902, 21360, 114, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[533019, 21365, 604, 22, 28, "Output",
   CellTags->"DiracTrick"],
  Cell[533660, 21392, 108, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[533771, 21397, 1905, 72, 28, "Output",
   CellTags->"DiracTrick"],
  Cell[535713, 21474, 370, 12, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[536086, 21488, 621, 18, 26, "Output",
   CellTags->"DiracTrick"],
  Cell[536744, 21511, 113, 3, 26, "Input",
   CellTags->"DiracTrick"],
  Cell[536860, 21516, 14288, 499, 87, "Output",
   CellTags->"DiracTrick"]},
 "Divideout"->{
  Cell[551175, 22019, 55, 1, 39, "ObjectName",
   CellTags->"Divideout"],
  Cell[551255, 22024, 60, 1, 36, "Subsubsection",
   CellTags->"Divideout"],
  Cell[551318, 22027, 137, 4, 24, "Text",
   CellTags->"Divideout"],
  Cell[551458, 22033, 444, 17, 24, "Text",
   CellTags->"Divideout"]},
 "DOT"->{
  Cell[551917, 22053, 43, 1, 39, "ObjectName",
   CellTags->"DOT"],
  Cell[551985, 22058, 54, 1, 36, "Subsubsection",
   CellTags->"DOT"],
  Cell[552042, 22061, 348, 8, 91, "Text",
   CellTags->"DOT"],
  Cell[552393, 22071, 262, 10, 24, "Text",
   CellTags->"DOT"]},
 "DotExpand"->{
  Cell[552670, 22084, 55, 1, 39, "ObjectName",
   CellTags->"DotExpand"],
  Cell[552750, 22089, 60, 1, 36, "Subsubsection",
   CellTags->"DotExpand"],
  Cell[552813, 22092, 85, 1, 24, "Text",
   CellTags->"DotExpand"],
  Cell[552901, 22095, 936, 31, 24, "Text",
   CellTags->"DotExpand"],
  Cell[553874, 22131, 57, 1, 36, "Subsubsection",
   CellTags->"DotExpand"],
  Cell[553956, 22136, 313, 10, 26, "Input",
   CellTags->"DotExpand"],
  Cell[554272, 22148, 504, 13, 26, "Output",
   CellTags->"DotExpand"],
  Cell[554791, 22164, 229, 7, 26, "Input",
   CellTags->"DotExpand"],
  Cell[555045, 22175, 313, 10, 26, "Input",
   CellTags->"DotExpand"],
  Cell[555361, 22187, 639, 22, 26, "Output",
   CellTags->"DotExpand"],
  Cell[556015, 22212, 231, 7, 26, "Input",
   CellTags->"DotExpand"],
  Cell[556271, 22223, 313, 10, 26, "Input",
   CellTags->"DotExpand"],
  Cell[556587, 22235, 504, 13, 26, "Output",
   CellTags->"DotExpand"]},
 "DotPower"->{
  Cell[557118, 22252, 53, 1, 39, "ObjectName",
   CellTags->"DotPower"],
  Cell[557196, 22257, 59, 1, 36, "Subsubsection",
   CellTags->"DotPower"],
  Cell[557258, 22260, 189, 4, 24, "Text",
   CellTags->"DotPower"],
  Cell[557450, 22266, 267, 10, 24, "Text",
   CellTags->"DotPower"]},
 "DotProduct"->{
  Cell[557732, 22279, 57, 1, 39, "ObjectName",
   CellTags->"DotProduct"],
  Cell[557814, 22284, 61, 1, 36, "Subsubsection",
   CellTags->"DotProduct"],
  Cell[557878, 22287, 219, 5, 41, "Text",
   CellTags->"DotProduct"],
  Cell[558100, 22294, 483, 18, 24, "Text",
   CellTags->"DotProduct"],
  Cell[558620, 22317, 58, 1, 36, "Subsubsection",
   CellTags->"DotProduct"],
  Cell[558703, 22322, 234, 7, 26, "Input",
   CellTags->"DotProduct"],
  Cell[558940, 22331, 314, 11, 29, "Output",
   CellTags->"DotProduct"]},
 "DotSimplifyRelations"->{
  Cell[559281, 22346, 77, 1, 39, "ObjectName",
   CellTags->"DotSimplifyRelations"],
  Cell[559383, 22351, 71, 1, 36, "Subsubsection",
   CellTags->"DotSimplifyRelations"],
  Cell[559457, 22354, 241, 5, 41, "Text",
   CellTags->"DotSimplifyRelations"],
  Cell[559701, 22361, 279, 10, 24, "Text",
   CellTags->"DotSimplifyRelations"]},
 "DotSimplify"->{
  Cell[559995, 22374, 59, 1, 39, "ObjectName",
   CellTags->"DotSimplify"],
  Cell[560079, 22379, 62, 1, 36, "Subsubsection",
   CellTags->"DotSimplify"],
  Cell[560144, 22382, 322, 6, 58, "Text",
   CellTags->"DotSimplify"],
  Cell[560491, 22392, 121, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[560615, 22397, 381, 11, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[561011, 22411, 664, 25, 24, "Text",
   CellTags->"DotSimplify"],
  Cell[561712, 22441, 59, 1, 36, "Subsubsection",
   CellTags->"DotSimplify"],
  Cell[561796, 22446, 342, 11, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[562141, 22459, 376, 11, 28, "Output",
   CellTags->"DotSimplify"],
  Cell[562554, 22475, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[562673, 22480, 1013, 39, 28, "Output",
   CellTags->"DotSimplify"],
  Cell[563701, 22522, 158, 4, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[563884, 22530, 209, 7, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[564096, 22539, 217, 8, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[564350, 22552, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[564469, 22557, 223, 7, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[564729, 22569, 161, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[564893, 22576, 107, 3, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[565037, 22584, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[565156, 22589, 279, 10, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[565450, 22602, 161, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[565636, 22611, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[565755, 22616, 223, 7, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[566015, 22628, 165, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[566183, 22635, 107, 3, 27, "Output",
   CellTags->"DotSimplify"],
  Cell[566327, 22643, 116, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[566446, 22648, 277, 9, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[566738, 22660, 165, 5, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[566928, 22669, 299, 9, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[567230, 22680, 166, 6, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[567411, 22689, 125, 3, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[567561, 22696, 148, 4, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[567712, 22702, 140, 4, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[567889, 22711, 213, 6, 26, "Input",
   CellTags->"DotSimplify"],
  Cell[568105, 22719, 140, 4, 26, "Output",
   CellTags->"DotSimplify"],
  Cell[568260, 22726, 170, 4, 26, "Input",
   CellTags->"DotSimplify"]},
 "DummyIndex"->{
  Cell[568445, 22733, 57, 1, 39, "ObjectName",
   CellTags->"DummyIndex"],
  Cell[568527, 22738, 61, 1, 36, "Subsubsection",
   CellTags->"DummyIndex"],
  Cell[568591, 22741, 191, 4, 24, "Text",
   CellTags->"DummyIndex"],
  Cell[568785, 22747, 265, 10, 24, "Text",
   CellTags->"DummyIndex"]},
 "D0"->{
  Cell[569065, 22760, 41, 1, 39, "ObjectName",
   CellTags->"D0"],
  Cell[569131, 22765, 53, 1, 36, "Subsubsection",
   CellTags->"D0"],
  Cell[569187, 22768, 524, 12, 58, "Text",
   CellTags->"D0"],
  Cell[569714, 22782, 771, 32, 24, "Text",
   CellTags->"D0"],
  Cell[570522, 22819, 50, 1, 36, "Subsubsection",
   CellTags->"D0"],
  Cell[570597, 22824, 315, 9, 26, "Input",
   CellTags->"D0"],
  Cell[570915, 22835, 550, 20, 29, "Output",
   CellTags->"D0"],
  Cell[571502, 22860, 484, 14, 26, "Input",
   CellTags->"D0"],
  Cell[571989, 22876, 550, 20, 29, "Output",
   CellTags->"D0"],
  Cell[572576, 22901, 104, 3, 26, "Input",
   CellTags->"D0"],
  Cell[572683, 22906, 550, 20, 29, "Output",
   CellTags->"D0"]},
 "D0Convention"->{
  Cell[573260, 22930, 61, 1, 39, "ObjectName",
   CellTags->"D0Convention"],
  Cell[573346, 22935, 63, 1, 36, "Subsubsection",
   CellTags->"D0Convention"],
  Cell[573412, 22938, 307, 6, 41, "Text",
   CellTags->"D0Convention"],
  Cell[573722, 22946, 419, 17, 24, "Text",
   CellTags->"D0Convention"]},
 "Eps"->{
  Cell[574156, 22966, 43, 1, 39, "ObjectName",
   CellTags->"Eps"],
  Cell[574224, 22971, 54, 1, 36, "Subsubsection",
   CellTags->"Eps"],
  Cell[574281, 22974, 345, 8, 41, "Text",
   CellTags->"Eps"],
  Cell[574651, 22986, 105, 3, 26, "Input",
   CellTags->"Eps"],
  Cell[574759, 22991, 160, 5, 26, "Output",
   CellTags->"Eps"],
  Cell[574934, 22999, 808, 32, 24, "Text",
   CellTags->"Eps"],
  Cell[575779, 23036, 51, 1, 36, "Subsubsection",
   CellTags->"Eps"],
  Cell[575855, 23041, 322, 8, 26, "Input",
   CellTags->"Eps"],
  Cell[576180, 23051, 477, 21, 26, "Output",
   CellTags->"Eps"],
  Cell[576694, 23077, 314, 8, 26, "Input",
   CellTags->"Eps"],
  Cell[577011, 23087, 435, 19, 26, "Output",
   CellTags->"Eps"],
  Cell[577483, 23111, 181, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[577667, 23119, 176, 5, 38, "Output",
   CellTags->"Eps"],
  Cell[577880, 23129, 142, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[578025, 23135, 99, 3, 26, "Output",
   CellTags->"Eps"],
  Cell[578161, 23143, 142, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[578306, 23149, 117, 4, 26, "Output",
   CellTags->"Eps"],
  Cell[578438, 23156, 189, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[578652, 23166, 460, 13, 26, "Input",
   CellTags->"Eps"],
  Cell[579115, 23181, 477, 21, 26, "Output",
   CellTags->"Eps"],
  Cell[579629, 23207, 107, 3, 26, "Input",
   CellTags->"Eps"],
  Cell[579739, 23212, 102, 3, 26, "Output",
   CellTags->"Eps"],
  Cell[579856, 23218, 189, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[580070, 23228, 460, 13, 26, "Input",
   CellTags->"Eps"],
  Cell[580533, 23243, 477, 21, 26, "Output",
   CellTags->"Eps"],
  Cell[581047, 23269, 137, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[581187, 23275, 102, 3, 26, "Output",
   CellTags->"Eps"],
  Cell[583116, 23358, 433, 14, 43, "Input",
   CellTags->"Eps"],
  Cell[583552, 23374, 1119, 45, 42, "Output",
   CellTags->"Eps"],
  Cell[584708, 23424, 978, 28, 43, "Input",
   CellTags->"Eps"],
  Cell[585689, 23454, 1679, 61, 42, "Output",
   CellTags->"Eps"],
  Cell[587405, 23520, 188, 6, 26, "Input",
   CellTags->"Eps"],
  Cell[587596, 23528, 342, 12, 42, "Output",
   CellTags->"Eps"],
  Cell[587975, 23545, 124, 4, 26, "Input",
   CellTags->"Eps"],
  Cell[588102, 23551, 99, 3, 26, "Output",
   CellTags->"Eps"],
  Cell[588216, 23557, 164, 5, 26, "Input",
   CellTags->"Eps"]},
 "EpsChisholm"->{
  Cell[588395, 23565, 59, 1, 39, "ObjectName",
   CellTags->"EpsChisholm"],
  Cell[588479, 23570, 62, 1, 36, "Subsubsection",
   CellTags->"EpsChisholm"],
  Cell[588544, 23573, 166, 4, 24, "Text",
   CellTags->"EpsChisholm"],
  Cell[588713, 23579, 264, 11, 24, "Text",
   CellTags->"EpsChisholm"],
  Cell[589014, 23595, 59, 1, 36, "Subsubsection",
   CellTags->"EpsChisholm"],
  Cell[589098, 23600, 208, 6, 26, "Input",
   CellTags->"EpsChisholm"],
  Cell[589309, 23608, 2019, 78, 26, "Output",
   CellTags->"EpsChisholm"],
  Cell[591365, 23691, 115, 3, 26, "Input",
   CellTags->"EpsChisholm"],
  Cell[591483, 23696, 594, 24, 26, "Output",
   CellTags->"EpsChisholm"]},
 "EpsContract"->{
  Cell[592104, 23724, 59, 1, 39, "ObjectName",
   CellTags->"EpsContract"],
  Cell[592188, 23729, 62, 1, 36, "Subsubsection",
   CellTags->"EpsContract"],
  Cell[592253, 23732, 227, 5, 41, "Text",
   CellTags->"EpsContract"],
  Cell[592483, 23739, 436, 18, 24, "Text",
   CellTags->"EpsContract"],
  Cell[592956, 23762, 59, 1, 36, "Subsubsection",
   CellTags->"EpsContract"],
  Cell[593040, 23767, 468, 13, 26, "Input",
   CellTags->"EpsContract"],
  Cell[593511, 23782, 485, 21, 26, "Output",
   CellTags->"EpsContract"],
  Cell[594033, 23808, 205, 6, 26, "Input",
   CellTags->"EpsContract"],
  Cell[594241, 23816, 569, 23, 26, "Output",
   CellTags->"EpsContract"],
  Cell[594847, 23844, 204, 6, 26, "Input",
   CellTags->"EpsContract"],
  Cell[595054, 23852, 320, 11, 26, "Output",
   CellTags->"EpsContract"]},
 "EpsDiscard"->{
  Cell[595488, 23871, 57, 1, 39, "ObjectName",
   CellTags->"EpsDiscard"],
  Cell[595570, 23876, 61, 1, 36, "Subsubsection",
   CellTags->"EpsDiscard"],
  Cell[595634, 23879, 169, 4, 24, "Text",
   CellTags->"EpsDiscard"],
  Cell[595806, 23885, 283, 11, 24, "Text",
   CellTags->"EpsDiscard"]},
 "EpsEvaluate"->{
  Cell[596104, 23899, 59, 1, 39, "ObjectName",
   CellTags->"EpsEvaluate"],
  Cell[596188, 23904, 62, 1, 36, "Subsubsection",
   CellTags->"EpsEvaluate"],
  Cell[596253, 23907, 173, 4, 24, "Text",
   CellTags->"EpsEvaluate"],
  Cell[596429, 23913, 816, 32, 24, "Text",
   CellTags->"EpsEvaluate"],
  Cell[597282, 23950, 59, 1, 36, "Subsubsection",
   CellTags->"EpsEvaluate"],
  Cell[597366, 23955, 335, 10, 26, "Input",
   CellTags->"EpsEvaluate"],
  Cell[597704, 23967, 466, 20, 26, "Output",
   CellTags->"EpsEvaluate"],
  Cell[598207, 23992, 115, 3, 26, "Input",
   CellTags->"EpsEvaluate"],
  Cell[598325, 23997, 466, 20, 26, "Output",
   CellTags->"EpsEvaluate"],
  Cell[598828, 24022, 116, 3, 26, "Input",
   CellTags->"EpsEvaluate"],
  Cell[598947, 24027, 434, 12, 38, "Output",
   CellTags->"EpsEvaluate"]},
 "EpsUncontract"->{
  Cell[599408, 24043, 63, 1, 39, "ObjectName",
   CellTags->"EpsUncontract"],
  Cell[599496, 24048, 64, 1, 36, "Subsubsection",
   CellTags->"EpsUncontract"],
  Cell[599563, 24051, 115, 3, 24, "Text",
   CellTags->"EpsUncontract"],
  Cell[599681, 24056, 446, 18, 24, "Text",
   CellTags->"EpsUncontract"]},
 "Epsilon"->{
  Cell[600142, 24077, 51, 1, 39, "ObjectName",
   CellTags->"Epsilon"],
  Cell[600218, 24082, 58, 1, 36, "Subsubsection",
   CellTags->"Epsilon"],
  Cell[600279, 24085, 237, 9, 24, "Text",
   CellTags->"Epsilon"],
  Cell[600519, 24096, 256, 11, 24, "Text",
   CellTags->"Epsilon"],
  Cell[600812, 24112, 55, 1, 36, "Subsubsection",
   CellTags->"Epsilon"],
  Cell[600892, 24117, 80, 2, 26, "Input",
   CellTags->"Epsilon"],
  Cell[600975, 24121, 148, 5, 26, "Output",
   CellTags->"Epsilon"],
  Cell[601138, 24129, 114, 3, 24, "Text",
   CellTags->"Epsilon"],
  Cell[601277, 24136, 587, 19, 26, "Input",
   CellTags->"Epsilon"],
  Cell[601867, 24157, 199, 6, 26, "Output",
   CellTags->"Epsilon"]},
 "EpsilonOrder"->{
  Cell[602093, 24167, 61, 1, 39, "ObjectName",
   CellTags->"EpsilonOrder"],
  Cell[602179, 24172, 63, 1, 36, "Subsubsection",
   CellTags->"EpsilonOrder"],
  Cell[602245, 24175, 305, 12, 24, "Text",
   CellTags->"EpsilonOrder"],
  Cell[602553, 24189, 449, 18, 24, "Text",
   CellTags->"EpsilonOrder"]},
 "Expanding"->{
  Cell[603017, 24210, 55, 1, 39, "ObjectName",
   CellTags->"Expanding"],
  Cell[603097, 24215, 60, 1, 36, "Subsubsection",
   CellTags->"Expanding"],
  Cell[603160, 24218, 378, 7, 58, "Text",
   CellTags->"Expanding"],
  Cell[603541, 24227, 1058, 39, 24, "Text",
   CellTags->"Expanding"]},
 "ExpandPartialD"->{
  Cell[604614, 24269, 65, 1, 39, "ObjectName",
   CellTags->"ExpandPartialD"],
  Cell[604704, 24274, 65, 1, 36, "Subsubsection",
   CellTags->"ExpandPartialD"],
  Cell[604772, 24277, 188, 4, 24, "Text",
   CellTags->"ExpandPartialD"],
  Cell[606142, 24328, 1169, 38, 24, "Text",
   CellTags->"ExpandPartialD"],
  Cell[607348, 24371, 62, 1, 36, "Subsubsection",
   CellTags->"ExpandPartialD"],
  Cell[607435, 24376, 373, 10, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[607811, 24388, 345, 12, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[608193, 24405, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[608317, 24410, 773, 30, 30, "Output",
   CellTags->"ExpandPartialD"],
  Cell[609127, 24445, 115, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[609245, 24450, 798, 21, 58, "Output",
   CellTags->"ExpandPartialD"],
  Cell[610080, 24476, 259, 7, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[610342, 24485, 312, 11, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[610691, 24501, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[610815, 24506, 704, 26, 42, "Output",
   CellTags->"ExpandPartialD"],
  Cell[611556, 24537, 433, 13, 46, "Input",
   CellTags->"ExpandPartialD"],
  Cell[611992, 24552, 332, 10, 35, "Output",
   CellTags->"ExpandPartialD"],
  Cell[612361, 24567, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[612485, 24572, 1940, 67, 42, "Output",
   CellTags->"ExpandPartialD"],
  Cell[614462, 24644, 180, 6, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[614645, 24652, 267, 8, 34, "Output",
   CellTags->"ExpandPartialD"],
  Cell[614949, 24665, 123, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[615075, 24670, 438, 14, 38, "Output",
   CellTags->"ExpandPartialD"],
  Cell[615550, 24689, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[615674, 24694, 1877, 59, 30, "Output",
   CellTags->"ExpandPartialD"],
  Cell[617588, 24758, 314, 9, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[617905, 24769, 1192, 47, 33, "Output",
   CellTags->"ExpandPartialD"],
  Cell[619134, 24821, 121, 3, 26, "Input",
   CellTags->"ExpandPartialD"],
  Cell[619258, 24826, 110, 3, 26, "Output",
   CellTags->"ExpandPartialD"]},
 "ScalarProductCancel"->{
  Cell[619395, 24833, 121, 1, 39, "ObjectName",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[619541, 24838, 95, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[619639, 24841, 252, 5, 49, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[619916, 24850, 158, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[620077, 24855, 214, 6, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[620306, 24864, 717, 25, 24, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621060, 24894, 92, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621177, 24899, 257, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621437, 24908, 338, 10, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621812, 24923, 152, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621967, 24928, 1214, 49, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623218, 24982, 186, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623407, 24989, 277, 9, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623721, 25003, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623880, 25008, 331, 12, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624248, 25025, 190, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624441, 25032, 227, 7, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624705, 25044, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624864, 25049, 441, 18, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[625342, 25072, 239, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[625584, 25081, 312, 10, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[625933, 25096, 184, 4, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[626120, 25102, 565, 22, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[1662773, 67094, 75, 1, 39, "ObjectName",
   CellTags->"ScalarProductCancel"],
  Cell[1662873, 67099, 70, 1, 36, "Subsubsection",
   CellTags->"ScalarProductCancel"],
  Cell[1662946, 67102, 186, 4, 24, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1663157, 67110, 89, 2, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1663249, 67114, 134, 3, 26, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1663420, 67122, 134, 3, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1663557, 67127, 392, 10, 26, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1663986, 67142, 106, 3, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1664095, 67147, 179, 3, 38, "Print",
   CellTags->{"Info3535453289-5096475", "ScalarProductCancel"}],
  Cell[1664289, 67153, 1238, 45, 24, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1665564, 67203, 67, 1, 36, "Subsubsection",
   CellTags->"ScalarProductCancel"],
  Cell[1665634, 67206, 344, 11, 24, "Text",
   CellTags->"ScalarProductCancel"],
  Cell[1666003, 67221, 429, 15, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1666435, 67238, 454, 17, 45, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1666926, 67260, 156, 4, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1667085, 67266, 849, 32, 56, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1667971, 67303, 162, 4, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1668136, 67309, 708, 26, 56, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1668881, 67340, 185, 5, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1669069, 67347, 708, 26, 56, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1669814, 67378, 578, 18, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1670395, 67398, 898, 33, 47, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1671330, 67436, 197, 5, 26, "Input",
   CellTags->"ScalarProductCancel"],
  Cell[1671530, 67443, 3108, 111, 112, "Output",
   CellTags->"ScalarProductCancel"],
  Cell[1674653, 67557, 162, 5, 26, "Input",
   CellTags->"ScalarProductCancel"]},
 "ExpandScalarProduct"->{
  Cell[619395, 24833, 121, 1, 39, "ObjectName",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[619541, 24838, 95, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[619639, 24841, 252, 5, 49, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[619916, 24850, 158, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[620077, 24855, 214, 6, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[620306, 24864, 717, 25, 24, "Text",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621060, 24894, 92, 1, 36, "Subsubsection",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621177, 24899, 257, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621437, 24908, 338, 10, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621812, 24923, 152, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[621967, 24928, 1214, 49, 26, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623218, 24982, 186, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623407, 24989, 277, 9, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623721, 25003, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[623880, 25008, 331, 12, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624248, 25025, 190, 5, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624441, 25032, 227, 7, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624705, 25044, 156, 3, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[624864, 25049, 441, 18, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[625342, 25072, 239, 7, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[625584, 25081, 312, 10, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[625933, 25096, 184, 4, 26, "Input",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
  Cell[626120, 25102, 565, 22, 28, "Output",
   CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]},
 "Expand2"->{
  Cell[626712, 25128, 51, 1, 39, "ObjectName",
   CellTags->"Expand2"],
  Cell[626788, 25133, 58, 1, 36, "Subsubsection",
   CellTags->"Expand2"],
  Cell[626849, 25136, 164, 4, 24, "Text",
   CellTags->"Expand2"],
  Cell[627050, 25145, 55, 1, 36, "Subsubsection",
   CellTags->"Expand2"],
  Cell[627130, 25150, 579, 21, 26, "Input",
   CellTags->"Expand2"],
  Cell[627712, 25173, 574, 19, 26, "Output",
   CellTags->"Expand2"]},
 "Explicit"->{
  Cell[628313, 25196, 53, 1, 39, "ObjectName",
   CellTags->"Explicit"],
  Cell[628391, 25201, 59, 1, 36, "Subsubsection",
   CellTags->"Explicit"],
  Cell[628453, 25204, 401, 9, 41, "Text",
   CellTags->"Explicit"],
  Cell[628857, 25215, 509, 18, 24, "Text",
   CellTags->"Explicit"],
  Cell[629876, 25257, 56, 1, 36, "Subsubsection",
   CellTags->"Explicit"],
  Cell[631730, 25336, 230, 6, 26, "Input",
   CellTags->"Explicit"],
  Cell[631963, 25344, 940, 42, 28, "Output",
   CellTags->"Explicit"],
  Cell[632940, 25391, 109, 3, 26, "Input",
   CellTags->"Explicit"],
  Cell[633052, 25396, 1971, 79, 28, "Output",
   CellTags->"Explicit"],
  Cell[635060, 25480, 184, 5, 26, "Input",
   CellTags->"Explicit"],
  Cell[635247, 25487, 892, 36, 42, "Output",
   CellTags->"Explicit"],
  Cell[636176, 25528, 109, 3, 26, "Input",
   CellTags->"Explicit"],
  Cell[636288, 25533, 2644, 102, 42, "Output",
   CellTags->"Explicit"],
  Cell[638969, 25640, 155, 4, 26, "Input",
   CellTags->"Explicit"],
  Cell[639127, 25646, 163, 5, 30, "Output",
   CellTags->"Explicit"],
  Cell[639327, 25656, 109, 3, 26, "Input",
   CellTags->"Explicit"],
  Cell[639439, 25661, 1744, 77, 30, "Output",
   CellTags->"Explicit"]},
 "ExplicitLorentzIndex"->{
  Cell[641210, 25742, 77, 1, 39, "ObjectName",
   CellTags->"ExplicitLorentzIndex"],
  Cell[641312, 25747, 71, 1, 36, "Subsubsection",
   CellTags->"ExplicitLorentzIndex"],
  Cell[641386, 25750, 141, 4, 24, "Text",
   CellTags->"ExplicitLorentzIndex"],
  Cell[641530, 25756, 459, 17, 24, "Text",
   CellTags->"ExplicitLorentzIndex"],
  Cell[642026, 25778, 68, 1, 36, "Subsubsection",
   CellTags->"ExplicitLorentzIndex"],
  Cell[642119, 25783, 224, 6, 26, "Input",
   CellTags->"ExplicitLorentzIndex"],
  Cell[642346, 25791, 199, 7, 26, "Output",
   CellTags->"ExplicitLorentzIndex"],
  Cell[642582, 25803, 121, 3, 26, "Input",
   CellTags->"ExplicitLorentzIndex"],
  Cell[642706, 25808, 247, 6, 38, "Output",
   CellTags->"ExplicitLorentzIndex"]},
 "ExplicitPartialD"->{
  Cell[642980, 25818, 69, 1, 39, "ObjectName",
   CellTags->"ExplicitPartialD"],
  Cell[643074, 25823, 67, 1, 36, "Subsubsection",
   CellTags->"ExplicitPartialD"],
  Cell[643144, 25826, 162, 4, 24, "Text",
   CellTags->"ExplicitPartialD"],
  Cell[643309, 25832, 747, 24, 24, "Text",
   CellTags->"ExplicitPartialD"],
  Cell[644093, 25861, 64, 1, 36, "Subsubsection",
   CellTags->"ExplicitPartialD"],
  Cell[644182, 25866, 183, 4, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[644368, 25872, 525, 19, 42, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[644930, 25896, 184, 4, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[645117, 25902, 430, 16, 33, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[645584, 25923, 186, 4, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[645773, 25929, 461, 15, 42, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[646271, 25949, 183, 6, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[646457, 25957, 270, 8, 34, "Output",
   CellTags->"ExplicitPartialD"],
  Cell[646764, 25970, 125, 3, 26, "Input",
   CellTags->"ExplicitPartialD"],
  Cell[646892, 25975, 440, 14, 38, "Output",
   CellTags->"ExplicitPartialD"]},
 "ExplicitSUNIndex"->{
  Cell[647359, 25993, 69, 1, 39, "ObjectName",
   CellTags->"ExplicitSUNIndex"],
  Cell[647453, 25998, 67, 1, 36, "Subsubsection",
   CellTags->"ExplicitSUNIndex"],
  Cell[647523, 26001, 178, 6, 24, "Text",
   CellTags->"ExplicitSUNIndex"],
  Cell[647704, 26009, 803, 31, 24, "Text",
   CellTags->"ExplicitSUNIndex"],
  Cell[648544, 26045, 64, 1, 36, "Subsubsection",
   CellTags->"ExplicitSUNIndex"],
  Cell[648633, 26050, 125, 3, 26, "Input",
   CellTags->"ExplicitSUNIndex"],
  Cell[648761, 26055, 112, 3, 26, "Output",
   CellTags->"ExplicitSUNIndex"]},
 "ExtraFactor"->{
  Cell[649320, 26080, 59, 1, 39, "ObjectName",
   CellTags->"ExtraFactor"],
  Cell[649404, 26085, 62, 1, 36, "Subsubsection",
   CellTags->"ExtraFactor"],
  Cell[649469, 26088, 199, 4, 24, "Text",
   CellTags->"ExtraFactor"],
  Cell[649671, 26094, 282, 10, 24, "Text",
   CellTags->"ExtraFactor"]},
 "ExtraVariables"->{
  Cell[649968, 26107, 65, 1, 39, "ObjectName",
   CellTags->"ExtraVariables"],
  Cell[650058, 26112, 65, 1, 36, "Subsubsection",
   CellTags->"ExtraVariables"],
  Cell[650126, 26115, 213, 5, 41, "Text",
   CellTags->"ExtraVariables"],
  Cell[650342, 26122, 463, 18, 24, "Text",
   CellTags->"ExtraVariables"]},
 "FactorFull"->{
  Cell[650820, 26143, 57, 1, 39, "ObjectName",
   CellTags->"FactorFull"],
  Cell[650902, 26148, 61, 1, 36, "Subsubsection",
   CellTags->"FactorFull"],
  Cell[650966, 26151, 509, 24, 24, "Text",
   CellTags->"FactorFull"],
  Cell[651478, 26177, 259, 11, 24, "Text",
   CellTags->"FactorFull"]},
 "Factoring"->{
  Cell[651752, 26191, 55, 1, 39, "ObjectName",
   CellTags->"Factoring"],
  Cell[651832, 26196, 60, 1, 36, "Subsubsection",
   CellTags->"Factoring"],
  Cell[651895, 26199, 235, 5, 41, "Text",
   CellTags->"Factoring"],
  Cell[652133, 26206, 622, 25, 24, "Text",
   CellTags->"Factoring"]},
 "Factorout"->{
  Cell[652770, 26234, 55, 1, 39, "ObjectName",
   CellTags->"Factorout"],
  Cell[652850, 26239, 60, 1, 36, "Subsubsection",
   CellTags->"Factorout"],
  Cell[652913, 26242, 74, 1, 24, "Text",
   CellTags->"Factorout"],
  Cell[652990, 26245, 254, 11, 24, "Text",
   CellTags->"Factorout"]},
 "FactorTime"->{
  Cell[653259, 26259, 57, 1, 39, "ObjectName",
   CellTags->"FactorTime"],
  Cell[653341, 26264, 61, 1, 36, "Subsubsection",
   CellTags->"FactorTime"],
  Cell[653405, 26267, 166, 4, 24, "Text",
   CellTags->"FactorTime"],
  Cell[653574, 26273, 259, 11, 24, "Text",
   CellTags->"FactorTime"]},
 "Factor1"->{
  Cell[653848, 26287, 51, 1, 39, "ObjectName",
   CellTags->"Factor1"],
  Cell[653924, 26292, 58, 1, 36, "Subsubsection",
   CellTags->"Factor1"],
  Cell[653985, 26295, 144, 4, 24, "Text",
   CellTags->"Factor1"],
  Cell[654132, 26301, 256, 11, 24, "Text",
   CellTags->"Factor1"],
  Cell[654425, 26317, 55, 1, 36, "Subsubsection",
   CellTags->"Factor1"],
  Cell[654505, 26322, 213, 8, 26, "Input",
   CellTags->"Factor1"],
  Cell[654721, 26332, 224, 8, 26, "Output",
   CellTags->"Factor1"],
  Cell[654982, 26345, 222, 7, 26, "Input",
   CellTags->"Factor1"],
  Cell[655207, 26354, 435, 16, 26, "Output",
   CellTags->"Factor1"],
  Cell[655679, 26375, 251, 9, 26, "Input",
   CellTags->"Factor1"],
  Cell[655933, 26386, 176, 6, 26, "Output",
   CellTags->"Factor1"],
  Cell[656146, 26397, 107, 3, 26, "Input",
   CellTags->"Factor1"],
  Cell[656256, 26402, 224, 8, 26, "Output",
   CellTags->"Factor1"],
  Cell[656517, 26415, 108, 3, 26, "Input",
   CellTags->"Factor1"],
  Cell[656628, 26420, 176, 6, 26, "Output",
   CellTags->"Factor1"],
  Cell[656819, 26429, 141, 4, 26, "Input",
   CellTags->"Factor1"]},
 "Factor2"->{
  Cell[656975, 26436, 51, 1, 39, "ObjectName",
   CellTags->"Factor2"],
  Cell[657051, 26441, 58, 1, 36, "Subsubsection",
   CellTags->"Factor2"],
  Cell[657112, 26444, 398, 7, 58, "Text",
   CellTags->"Factor2"],
  Cell[657535, 26455, 113, 3, 26, "Input",
   CellTags->"Factor2"],
  Cell[657651, 26460, 230, 7, 26, "Output",
   CellTags->"Factor2"],
  Cell[657896, 26470, 260, 11, 24, "Text",
   CellTags->"Factor2"],
  Cell[658193, 26486, 55, 1, 36, "Subsubsection",
   CellTags->"Factor2"],
  Cell[658273, 26491, 213, 8, 26, "Input",
   CellTags->"Factor2"],
  Cell[658489, 26501, 224, 8, 26, "Output",
   CellTags->"Factor2"],
  Cell[658750, 26514, 222, 7, 26, "Input",
   CellTags->"Factor2"],
  Cell[658975, 26523, 435, 16, 26, "Output",
   CellTags->"Factor2"],
  Cell[659447, 26544, 251, 9, 26, "Input",
   CellTags->"Factor2"],
  Cell[659701, 26555, 176, 6, 26, "Output",
   CellTags->"Factor2"],
  Cell[659914, 26566, 107, 3, 26, "Input",
   CellTags->"Factor2"],
  Cell[660024, 26571, 224, 8, 26, "Output",
   CellTags->"Factor2"],
  Cell[660285, 26584, 108, 3, 26, "Input",
   CellTags->"Factor2"],
  Cell[660396, 26589, 176, 6, 26, "Output",
   CellTags->"Factor2"],
  Cell[660609, 26600, 173, 5, 26, "Input",
   CellTags->"Factor2"],
  Cell[660785, 26607, 224, 8, 26, "Output",
   CellTags->"Factor2"],
  Cell[661024, 26618, 141, 4, 26, "Input",
   CellTags->"Factor2"]},
 "FAD"->{
  Cell[661180, 26625, 43, 1, 39, "ObjectName",
   CellTags->"FAD"],
  Cell[661248, 26630, 54, 1, 36, "Subsubsection",
   CellTags->"FAD"],
  Cell[661305, 26633, 335, 6, 58, "Text",
   CellTags->"FAD"],
  Cell[661643, 26641, 1320, 46, 24, "Text",
   CellTags->"FAD"],
  Cell[663000, 26692, 51, 1, 36, "Subsubsection",
   CellTags->"FAD"],
  Cell[663076, 26697, 146, 5, 26, "Input",
   CellTags->"FAD"],
  Cell[663225, 26704, 700, 20, 48, "Output",
   CellTags->"FAD"],
  Cell[663962, 26729, 197, 7, 26, "Input",
   CellTags->"FAD"],
  Cell[664162, 26738, 819, 22, 48, "Output",
   CellTags->"FAD"],
  Cell[665018, 26765, 245, 9, 26, "Input",
   CellTags->"FAD"],
  Cell[665266, 26776, 161, 5, 38, "Output",
   CellTags->"FAD"],
  Cell[665464, 26786, 215, 8, 26, "Input",
   CellTags->"FAD"],
  Cell[665682, 26796, 527, 16, 58, "Output",
   CellTags->"FAD"],
  Cell[666246, 26817, 280, 9, 26, "Input",
   CellTags->"FAD"],
  Cell[666529, 26828, 527, 16, 58, "Output",
   CellTags->"FAD"]},
 "FC"->{
  Cell[667083, 26848, 41, 1, 39, "ObjectName",
   CellTags->"FC"],
  Cell[667149, 26853, 53, 1, 36, "Subsubsection",
   CellTags->"FC"],
  Cell[667205, 26856, 117, 3, 24, "Text",
   CellTags->"FC"],
  Cell[667325, 26861, 887, 32, 24, "Text",
   CellTags->"FC"],
  Cell[668249, 26898, 50, 1, 36, "Subsubsection",
   CellTags->"FC"],
  Cell[668302, 26901, 70, 2, 26, "Input",
   CellTags->"FC"],
  Cell[668397, 26907, 227, 7, 26, "Input",
   CellTags->"FC"],
  Cell[668627, 26916, 100, 2, 25, "Output",
   CellTags->"FC"],
  Cell[668742, 26921, 70, 2, 26, "Input",
   CellTags->"FC"],
  Cell[668837, 26927, 227, 7, 26, "Input",
   CellTags->"FC"],
  Cell[669067, 26936, 305, 11, 29, "Output",
   CellTags->"FC"]},
 "FCE"->{
  Cell[669399, 26951, 43, 1, 39, "ObjectName",
   CellTags->"FCE"],
  Cell[669467, 26956, 54, 1, 36, "Subsubsection",
   CellTags->"FCE"],
  Cell[669524, 26959, 126, 4, 24, "Text",
   CellTags->"FCE"],
  Cell[669653, 26965, 72, 1, 24, "Text",
   CellTags->"FCE"],
  Cell[669728, 26968, 684, 25, 24, "Text",
   CellTags->"FCE"],
  Cell[670449, 26998, 51, 1, 36, "Subsubsection",
   CellTags->"FCE"],
  Cell[670525, 27003, 262, 8, 26, "Input",
   CellTags->"FCE"],
  Cell[670790, 27013, 225, 8, 29, "Output",
   CellTags->"FCE"],
  Cell[671052, 27026, 104, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[671159, 27031, 190, 6, 38, "Output",
   CellTags->"FCE"],
  Cell[671386, 27042, 437, 13, 26, "Input",
   CellTags->"FCE"],
  Cell[671826, 27057, 547, 18, 29, "Output",
   CellTags->"FCE"],
  Cell[672410, 27080, 104, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[672517, 27085, 452, 13, 38, "Output",
   CellTags->"FCE"],
  Cell[673006, 27103, 95, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[673104, 27108, 862, 34, 29, "Output",
   CellTags->"FCE"],
  Cell[674003, 27147, 104, 3, 26, "Input",
   CellTags->"FCE"],
  Cell[674110, 27152, 893, 24, 77, "Output",
   CellTags->"FCE"],
  Cell[675040, 27181, 134, 4, 26, "Input",
   CellTags->"FCE"],
  Cell[675177, 27187, 452, 13, 38, "Output",
   CellTags->"FCE"]},
 "FCI"->{
  Cell[675656, 27204, 43, 1, 39, "ObjectName",
   CellTags->"FCI"],
  Cell[675724, 27209, 54, 1, 36, "Subsubsection",
   CellTags->"FCI"],
  Cell[675781, 27212, 159, 4, 24, "Text",
   CellTags->"FCI"],
  Cell[675943, 27218, 684, 25, 24, "Text",
   CellTags->"FCI"],
  Cell[676664, 27248, 51, 1, 36, "Subsubsection",
   CellTags->"FCI"],
  Cell[676740, 27253, 437, 13, 26, "Input",
   CellTags->"FCI"],
  Cell[677180, 27268, 547, 18, 29, "Output",
   CellTags->"FCI"],
  Cell[677764, 27291, 104, 3, 26, "Input",
   CellTags->"FCI"],
  Cell[677871, 27296, 452, 13, 38, "Output",
   CellTags->"FCI"],
  Cell[678360, 27314, 95, 3, 26, "Input",
   CellTags->"FCI"],
  Cell[678458, 27319, 862, 34, 29, "Output",
   CellTags->"FCI"],
  Cell[679357, 27358, 104, 3, 26, "Input",
   CellTags->"FCI"],
  Cell[679464, 27363, 893, 24, 77, "Output",
   CellTags->"FCI"],
  Cell[680394, 27392, 134, 4, 26, "Input",
   CellTags->"FCI"],
  Cell[680531, 27398, 452, 13, 38, "Output",
   CellTags->"FCI"]},
 "FCIntegral"->{
  Cell[681010, 27415, 57, 1, 39, "ObjectName",
   CellTags->"FCIntegral"],
  Cell[681092, 27420, 61, 1, 36, "Subsubsection",
   CellTags->"FCIntegral"],
  Cell[681156, 27423, 209, 5, 41, "Text",
   CellTags->"FCIntegral"],
  Cell[681368, 27430, 547, 18, 24, "Text",
   CellTags->"FCIntegral"]},
 "FCIntegrate"->{
  Cell[681930, 27451, 59, 1, 39, "ObjectName",
   CellTags->"FCIntegrate"],
  Cell[682014, 27456, 62, 1, 36, "Subsubsection",
   CellTags->"FCIntegrate"],
  Cell[682079, 27459, 285, 5, 41, "Text",
   CellTags->"FCIntegrate"],
  Cell[682367, 27466, 280, 11, 24, "Text",
   CellTags->"FCIntegrate"]},
 "FCNIntegrate"->{
  Cell[682662, 27480, 61, 1, 39, "ObjectName",
   CellTags->"FCNIntegrate"],
  Cell[682748, 27485, 63, 1, 36, "Subsubsection",
   CellTags->"FCNIntegrate"],
  Cell[682814, 27488, 418, 7, 58, "Text",
   CellTags->"FCNIntegrate"],
  Cell[683235, 27497, 277, 11, 24, "Text",
   CellTags->"FCNIntegrate"]},
 "FC2RHI"->{
  Cell[683527, 27511, 50, 1, 39, "ObjectName",
   CellTags->"FC2RHI"],
  Cell[683602, 27516, 57, 1, 36, "Subsubsection",
   CellTags->"FC2RHI"],
  Cell[683662, 27519, 306, 6, 41, "Text",
   CellTags->"FC2RHI"],
  Cell[683993, 27529, 111, 3, 26, "Input",
   CellTags->"FC2RHI"],
  Cell[684107, 27534, 277, 8, 26, "Output",
   CellTags->"FC2RHI"],
  Cell[686143, 27622, 239, 11, 24, "Text",
   CellTags->"FC2RHI"],
  Cell[686419, 27638, 54, 1, 36, "Subsubsection",
   CellTags->"FC2RHI"],
  Cell[686476, 27641, 72, 2, 26, "Input",
   CellTags->"FC2RHI"]},
 "FC2TLI"->{
  Cell[686563, 27646, 50, 1, 39, "ObjectName",
   CellTags->"FC2TLI"],
  Cell[686638, 27651, 57, 1, 36, "Subsubsection",
   CellTags->"FC2TLI"],
  Cell[686698, 27654, 256, 5, 41, "Text",
   CellTags->"FC2TLI"],
  Cell[686979, 27663, 111, 3, 26, "Input",
   CellTags->"FC2TLI"],
  Cell[687093, 27668, 277, 8, 26, "Output",
   CellTags->"FC2TLI"],
  Cell[691498, 27833, 239, 11, 24, "Text",
   CellTags->"FC2TLI"],
  Cell[691774, 27849, 54, 1, 36, "Subsubsection",
   CellTags->"FC2TLI"],
  Cell[691831, 27852, 72, 2, 26, "Input",
   CellTags->"FC2TLI"]},
 "Info3535453193-5096475"->{
  Cell[687481, 27685, 321, 5, 112, "Print",
   CellTags->"Info3535453193-5096475"]},
 "FDS"->{
  Cell[691918, 27857, 43, 1, 39, "ObjectName",
   CellTags->"FDS"],
  Cell[691986, 27862, 54, 1, 36, "Subsubsection",
   CellTags->"FDS"],
  Cell[692043, 27865, 82, 1, 24, "Text",
   CellTags->"FDS"],
  Cell[692128, 27868, 328, 11, 24, "Text",
   CellTags->"FDS"]},
 "FermionSpinSum"->{
  Cell[692471, 27882, 65, 1, 39, "ObjectName",
   CellTags->"FermionSpinSum"],
  Cell[692561, 27887, 65, 1, 36, "Subsubsection",
   CellTags->"FermionSpinSum"],
  Cell[692629, 27890, 118, 3, 24, "Text",
   CellTags->"FermionSpinSum"],
  Cell[692772, 27897, 127, 3, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[692902, 27902, 314, 9, 26, "Output",
   CellTags->"FermionSpinSum"],
  Cell[693231, 27914, 851, 32, 24, "Text",
   CellTags->"FermionSpinSum"],
  Cell[694119, 27951, 62, 1, 36, "Subsubsection",
   CellTags->"FermionSpinSum"],
  Cell[694184, 27954, 261, 12, 24, "Text",
   CellTags->"FermionSpinSum"],
  Cell[694470, 27970, 296, 9, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[694769, 27981, 353, 13, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[695159, 27999, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[695305, 28005, 330, 12, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[695672, 28022, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[695818, 28028, 133, 4, 26, "Output",
   CellTags->"FermionSpinSum"],
  Cell[695988, 28037, 296, 9, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[696287, 28048, 353, 13, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[696677, 28066, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[696823, 28072, 330, 12, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[697190, 28089, 143, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[697336, 28095, 152, 5, 26, "Output",
   CellTags->"FermionSpinSum"],
  Cell[697525, 28105, 334, 10, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[697862, 28117, 508, 17, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[698385, 28137, 164, 4, 24, "Text",
   CellTags->"FermionSpinSum"],
  Cell[698574, 28145, 147, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[698724, 28151, 540, 18, 28, "Output",
   CellTags->"FermionSpinSum"],
  Cell[699301, 28174, 145, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[699449, 28180, 1074, 36, 29, "Output",
   CellTags->"FermionSpinSum"],
  Cell[700560, 28221, 148, 4, 26, "Input",
   CellTags->"FermionSpinSum"],
  Cell[700711, 28227, 837, 32, 29, "Output",
   CellTags->"FermionSpinSum"],
  Cell[701563, 28262, 141, 4, 26, "Input",
   CellTags->"FermionSpinSum"]},
 "FeynAmp"->{
  Cell[701719, 28269, 51, 1, 39, "ObjectName",
   CellTags->"FeynAmp"],
  Cell[701795, 28274, 58, 1, 36, "Subsubsection",
   CellTags->"FeynAmp"],
  Cell[701856, 28277, 243, 5, 41, "Text",
   CellTags->"FeynAmp"],
  Cell[702102, 28284, 165, 4, 24, "Text",
   CellTags->"FeynAmp"],
  Cell[702270, 28290, 264, 11, 24, "Text",
   CellTags->"FeynAmp"],
  Cell[702571, 28306, 55, 1, 36, "Subsubsection",
   CellTags->"FeynAmp"],
  Cell[702629, 28309, 125, 4, 24, "Text",
   CellTags->"FeynAmp"],
  Cell[702779, 28317, 846, 22, 46, "Input",
   CellTags->"FeynAmp"],
  Cell[703628, 28341, 2586, 102, 42, "Output",
   CellTags->"FeynAmp"],
  Cell[706229, 28446, 73, 1, 24, "Text",
   CellTags->"FeynAmp"],
  Cell[706327, 28451, 198, 6, 26, "Input",
   CellTags->"FeynAmp"],
  Cell[706528, 28459, 414, 14, 42, "Output",
   CellTags->"FeynAmp"]},
 "FeynAmpDenominator"->{
  Cell[706969, 28477, 73, 1, 39, "ObjectName",
   CellTags->"FeynAmpDenominator"],
  Cell[707067, 28482, 69, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominator"],
  Cell[707139, 28485, 256, 5, 41, "Text",
   CellTags->"FeynAmpDenominator"],
  Cell[707398, 28492, 515, 18, 24, "Text",
   CellTags->"FeynAmpDenominator"],
  Cell[707950, 28515, 66, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominator"],
  Cell[708041, 28520, 201, 5, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[708245, 28527, 211, 7, 48, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[708493, 28539, 326, 9, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[708822, 28550, 430, 15, 48, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[709267, 28568, 247, 7, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[709539, 28579, 149, 4, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[709691, 28585, 279, 7, 38, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[710007, 28597, 149, 4, 26, "Input",
   CellTags->"FeynAmpDenominator"],
  Cell[710159, 28603, 176, 5, 38, "Output",
   CellTags->"FeynAmpDenominator"],
  Cell[710350, 28611, 134, 4, 26, "Input",
   CellTags->"FeynAmpDenominator"]},
 "FeynAmpDenominatorCombine"->{
  Cell[710499, 28618, 87, 1, 39, "ObjectName",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[710611, 28623, 76, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[710690, 28626, 228, 5, 41, "Text",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[710921, 28633, 338, 11, 24, "Text",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[711296, 28649, 73, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[711394, 28654, 234, 7, 26, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[711631, 28663, 765, 23, 48, "Output",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[712433, 28691, 208, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[712644, 28699, 203, 6, 38, "Output",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[712884, 28710, 206, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorCombine"],
  Cell[713093, 28718, 233, 7, 38, "Output",
   CellTags->"FeynAmpDenominatorCombine"]},
 "FeynAmpDenominatorSimplify"->{
  Cell[713353, 28729, 89, 1, 39, "ObjectName",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[713467, 28734, 77, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[713547, 28737, 398, 7, 58, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[713948, 28746, 85, 1, 24, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[714058, 28751, 95, 2, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[714156, 28755, 147, 3, 26, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[714318, 28761, 307, 11, 24, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[714662, 28777, 74, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[714739, 28780, 432, 14, 27, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[715196, 28798, 280, 8, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[715479, 28808, 122, 3, 26, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[715616, 28814, 468, 18, 27, "Text",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[716109, 28836, 328, 10, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[716440, 28848, 336, 11, 49, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[716813, 28864, 260, 7, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[717076, 28873, 308, 10, 49, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[717421, 28888, 339, 10, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[717763, 28900, 362, 12, 49, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[718162, 28917, 449, 14, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[718614, 28933, 1091, 40, 50, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[719742, 28978, 215, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[719960, 28986, 795, 29, 50, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[720792, 29020, 157, 4, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[720952, 29026, 493, 18, 46, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[721482, 29049, 373, 13, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[721858, 29064, 1361, 50, 51, "Output",
   CellTags->"FeynAmpDenominatorSimplify"],
  Cell[723234, 29117, 147, 4, 26, "Input",
   CellTags->"FeynAmpDenominatorSimplify"]},
 "FeynAmpDenominatorSplit"->{
  Cell[723396, 29124, 83, 1, 39, "ObjectName",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[723504, 29129, 74, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[723581, 29132, 357, 7, 58, "Text",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[723941, 29141, 344, 11, 24, "Text",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[724322, 29157, 71, 1, 36, "Subsubsection",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[724418, 29162, 308, 9, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[724729, 29173, 489, 16, 48, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[725255, 29194, 116, 3, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[725374, 29199, 136, 3, 26, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[725547, 29207, 139, 3, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[725689, 29212, 489, 16, 48, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[726215, 29233, 150, 4, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[726368, 29239, 425, 13, 38, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[726830, 29257, 230, 7, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[727063, 29266, 368, 12, 38, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[727468, 29283, 206, 6, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[727677, 29291, 305, 9, 38, "Output",
   CellTags->"FeynAmpDenominatorSplit"],
  Cell[727997, 29303, 121, 3, 26, "Input",
   CellTags->"FeynAmpDenominatorSplit"]},
 "FeynAmpList"->{
  Cell[728133, 29309, 59, 1, 39, "ObjectName",
   CellTags->"FeynAmpList"],
  Cell[728217, 29314, 62, 1, 36, "Subsubsection",
   CellTags->"FeynAmpList"],
  Cell[728282, 29317, 145, 4, 24, "Text",
   CellTags->"FeynAmpList"],
  Cell[728430, 29323, 117, 3, 24, "Text",
   CellTags->"FeynAmpList"],
  Cell[728550, 29328, 260, 11, 24, "Text",
   CellTags->"FeynAmpList"]},
 "FeynCalc"->{
  Cell[728825, 29342, 53, 1, 39, "ObjectName",
   CellTags->"FeynCalc"],
  Cell[728903, 29347, 59, 1, 36, "Subsubsection",
   CellTags->"FeynCalc"],
  Cell[728965, 29350, 91, 1, 24, "Text",
   CellTags->"FeynCalc"],
  Cell[729081, 29355, 99, 3, 26, "Input",
   CellTags->"FeynCalc"],
  Cell[729183, 29360, 703, 11, 319, "Print",
   CellTags->{"FeynCalc", "Info3535453199-5096475"}]},
 "Info3535453199-5096475"->{
  Cell[729183, 29360, 703, 11, 319, "Print",
   CellTags->{"FeynCalc", "Info3535453199-5096475"}]},
 "FeynCalcExternal"->{
  Cell[729913, 29375, 69, 1, 39, "ObjectName",
   CellTags->"FeynCalcExternal"],
  Cell[730007, 29380, 67, 1, 36, "Subsubsection",
   CellTags->"FeynCalcExternal"],
  Cell[730077, 29383, 156, 4, 24, "Text",
   CellTags->"FeynCalcExternal"],
  Cell[730236, 29389, 301, 11, 24, "Text",
   CellTags->"FeynCalcExternal"],
  Cell[730574, 29405, 64, 1, 36, "Subsubsection",
   CellTags->"FeynCalcExternal"],
  Cell[730663, 29410, 162, 4, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[730828, 29416, 143, 4, 26, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[731008, 29425, 117, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[731128, 29430, 126, 3, 38, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[731291, 29438, 450, 13, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[731744, 29453, 560, 18, 29, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[732341, 29476, 117, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[732461, 29481, 465, 13, 38, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[732963, 29499, 121, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[733087, 29504, 875, 34, 29, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[733999, 29543, 117, 3, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[734119, 29548, 906, 24, 77, "Output",
   CellTags->"FeynCalcExternal"],
  Cell[735062, 29577, 163, 5, 26, "Input",
   CellTags->"FeynCalcExternal"],
  Cell[735228, 29584, 465, 13, 38, "Output",
   CellTags->"FeynCalcExternal"]},
 "FeynCalcForm"->{
  Cell[735720, 29601, 61, 1, 39, "ObjectName",
   CellTags->"FeynCalcForm"],
  Cell[735806, 29606, 63, 1, 36, "Subsubsection",
   CellTags->"FeynCalcForm"],
  Cell[735872, 29609, 534, 10, 75, "Text",
   CellTags->"FeynCalcForm"],
  Cell[736409, 29621, 689, 25, 24, "Text",
   CellTags->"FeynCalcForm"],
  Cell[737135, 29651, 60, 1, 36, "Subsubsection",
   CellTags->"FeynCalcForm"],
  Cell[737198, 29654, 79, 1, 24, "Text",
   CellTags->"FeynCalcForm"],
  Cell[737302, 29659, 242, 7, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[737547, 29668, 527, 22, 26, "Output",
   CellTags->"FeynCalcForm"],
  Cell[738089, 29693, 112, 3, 24, "Text",
   CellTags->"FeynCalcForm"],
  Cell[738204, 29698, 148, 4, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[738355, 29704, 647, 19, 73, "Input",
   CellTags->"FeynCalcForm"],
  Cell[739027, 29727, 242, 7, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[739272, 29736, 564, 17, 26, "Output",
   CellTags->"FeynCalcForm"],
  Cell[739851, 29756, 76, 1, 24, "Text",
   CellTags->"FeynCalcForm"],
  Cell[739930, 29759, 123, 4, 26, "Input",
   CellTags->"FeynCalcForm"],
  Cell[740056, 29765, 652, 19, 51, "Input",
   CellTags->"FeynCalcForm"]},
 "FeynCalcInternal"->{
  Cell[740723, 29787, 69, 1, 39, "ObjectName",
   CellTags->"FeynCalcInternal"],
  Cell[740817, 29792, 67, 1, 36, "Subsubsection",
   CellTags->"FeynCalcInternal"],
  Cell[740887, 29795, 157, 4, 24, "Text",
   CellTags->"FeynCalcInternal"],
  Cell[741047, 29801, 645, 25, 24, "Text",
   CellTags->"FeynCalcInternal"],
  Cell[741729, 29831, 64, 1, 36, "Subsubsection",
   CellTags->"FeynCalcInternal"],
  Cell[741818, 29836, 450, 13, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[742271, 29851, 560, 18, 29, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[742868, 29874, 117, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[742988, 29879, 465, 13, 38, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[743490, 29897, 121, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[743614, 29902, 875, 34, 29, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[744526, 29941, 117, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[744646, 29946, 906, 24, 77, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[745589, 29975, 163, 5, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[745755, 29982, 465, 13, 38, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[746257, 30000, 517, 15, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[746777, 30017, 1461, 63, 48, "Output",
   CellTags->"FeynCalcInternal"],
  Cell[748275, 30085, 117, 3, 26, "Input",
   CellTags->"FeynCalcInternal"],
  Cell[748395, 30090, 1128, 30, 96, "Output",
   CellTags->"FeynCalcInternal"]},
 "FeynCalc2FORM"->{
  Cell[749550, 30124, 63, 1, 39, "ObjectName",
   CellTags->"FeynCalc2FORM"],
  Cell[749638, 30129, 64, 1, 36, "Subsubsection",
   CellTags->"FeynCalc2FORM"],
  Cell[749705, 30132, 226, 5, 41, "Text",
   CellTags->"FeynCalc2FORM"],
  Cell[749934, 30139, 286, 11, 24, "Text",
   CellTags->"FeynCalc2FORM"],
  Cell[750245, 30154, 125, 3, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[750373, 30159, 1022, 22, 67, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[751444, 30187, 61, 1, 36, "Subsubsection",
   CellTags->"FeynCalc2FORM"],
  Cell[751530, 30192, 273, 9, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[751806, 30203, 276, 9, 45, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[752119, 30217, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[752259, 30223, 68, 1, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[752364, 30229, 199, 6, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[752566, 30237, 199, 6, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[752802, 30248, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[752942, 30254, 66, 1, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[753045, 30260, 212, 6, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[753260, 30268, 373, 11, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[753670, 30284, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[753810, 30290, 88, 3, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[753935, 30298, 298, 9, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[754236, 30309, 473, 16, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[754746, 30330, 137, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[754886, 30336, 88, 3, 21, "Print",
   CellTags->"FeynCalc2FORM"],
  Cell[754989, 30342, 328, 10, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[755342, 30356, 332, 10, 29, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[755677, 30368, 154, 4, 26, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[755868, 30377, 314, 10, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[756185, 30389, 6179, 240, 50, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[762379, 30632, 209, 6, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[762613, 30642, 376, 11, 29, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[762992, 30655, 1338, 36, 225, "Output",
   CellTags->"FeynCalc2FORM"],
  Cell[764345, 30694, 313, 10, 26, "Input",
   CellTags->"FeynCalc2FORM"],
  Cell[764661, 30706, 129, 4, 26, "Input",
   CellTags->"FeynCalc2FORM"]},
 "FeynCalcToLaTeX"->{
  Cell[764805, 30713, 67, 1, 39, "ObjectName",
   CellTags->"FeynCalcToLaTeX"],
  Cell[764897, 30718, 66, 1, 36, "Subsubsection",
   CellTags->"FeynCalcToLaTeX"],
  Cell[764966, 30721, 300, 6, 66, "Text",
   CellTags->"FeynCalcToLaTeX"]},
 "FeynmanParameterNames"->{
  Cell[766620, 30777, 79, 1, 39, "ObjectName",
   CellTags->"FeynmanParameterNames"],
  Cell[766724, 30782, 72, 1, 36, "Subsubsection",
   CellTags->"FeynmanParameterNames"],
  Cell[766799, 30785, 143, 4, 24, "Text",
   CellTags->"FeynmanParameterNames"],
  Cell[766945, 30791, 546, 18, 24, "Text",
   CellTags->"FeynmanParameterNames"]},
 "FeynmanParametrize"->{
  Cell[767506, 30812, 90, 1, 39, "ObjectName",
   CellTags->"FeynmanParametrize"],
  Cell[767621, 30817, 69, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize"],
  Cell[767693, 30820, 176, 4, 24, "Text",
   CellTags->"FeynmanParametrize"],
  Cell[767894, 30828, 135, 3, 26, "Input",
   CellTags->"FeynmanParametrize"],
  Cell[768032, 30833, 251, 8, 26, "Output",
   CellTags->"FeynmanParametrize"],
  Cell[768332, 30847, 66, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize"],
  Cell[768401, 30850, 84, 2, 26, "Input",
   CellTags->"FeynmanParametrize"]},
 "FeynRule"->{
  Cell[768500, 30855, 53, 1, 39, "ObjectName",
   CellTags->"FeynRule"],
  Cell[768578, 30860, 59, 1, 36, "Subsubsection",
   CellTags->"FeynRule"],
  Cell[768640, 30863, 168, 4, 24, "Text",
   CellTags->"FeynRule"],
  Cell[768833, 30871, 115, 3, 26, "Input",
   CellTags->"FeynRule"],
  Cell[768951, 30876, 630, 16, 47, "Output",
   CellTags->"FeynRule"],
  Cell[769596, 30895, 92, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[769691, 30898, 133, 4, 24, "Text",
   CellTags->"FeynRule"],
  Cell[769827, 30904, 269, 11, 24, "Text",
   CellTags->"FeynRule"],
  Cell[770133, 30920, 56, 1, 36, "Subsubsection",
   CellTags->"FeynRule"],
  Cell[770214, 30925, 146, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[770363, 30931, 680, 23, 42, "Output",
   CellTags->"FeynRule"],
  Cell[771080, 30959, 146, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[771229, 30965, 1003, 36, 42, "Output",
   CellTags->"FeynRule"],
  Cell[772269, 31006, 111, 3, 26, "Input",
   CellTags->"FeynRule"],
  Cell[772383, 31011, 4554, 176, 42, "Output",
   CellTags->"FeynRule"],
  Cell[776974, 31192, 141, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[777118, 31198, 2656, 113, 30, "Output",
   CellTags->"FeynRule"],
  Cell[779789, 31314, 74, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[779888, 31319, 503, 16, 26, "Input",
   CellTags->"FeynRule"],
  Cell[780394, 31337, 700, 30, 30, "Output",
   CellTags->"FeynRule"],
  Cell[781131, 31372, 307, 9, 26, "Input",
   CellTags->"FeynRule"],
  Cell[781441, 31383, 3795, 138, 49, "Output",
   CellTags->"FeynRule"],
  Cell[785251, 31524, 72, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[785348, 31529, 503, 16, 26, "Input",
   CellTags->"FeynRule"],
  Cell[785854, 31547, 700, 30, 30, "Output",
   CellTags->"FeynRule"],
  Cell[786591, 31582, 307, 9, 26, "Input",
   CellTags->"FeynRule"],
  Cell[786901, 31593, 4465, 151, 49, "Output",
   CellTags->"FeynRule"],
  Cell[791403, 31749, 218, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[791624, 31758, 1044, 42, 28, "Output",
   CellTags->"FeynRule"],
  Cell[792683, 31803, 110, 3, 24, "Text",
   CellTags->"FeynRule"],
  Cell[792818, 31810, 360, 10, 26, "Input",
   CellTags->"FeynRule"],
  Cell[793181, 31822, 1108, 45, 28, "Output",
   CellTags->"FeynRule"],
  Cell[794304, 31870, 77, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[794406, 31875, 145, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[794554, 31881, 618, 22, 26, "Output",
   CellTags->"FeynRule"],
  Cell[795187, 31906, 90, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[795302, 31911, 145, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[795450, 31917, 618, 22, 26, "Output",
   CellTags->"FeynRule"],
  Cell[796105, 31944, 696, 21, 46, "Input",
   CellTags->"FeynRule"],
  Cell[796804, 31967, 893, 37, 30, "Output",
   CellTags->"FeynRule"],
  Cell[797734, 32009, 227, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[797964, 32018, 4957, 177, 114, "Output",
   CellTags->"FeynRule"],
  Cell[802958, 32200, 420, 12, 26, "Input",
   CellTags->"FeynRule"],
  Cell[803381, 32214, 4764, 159, 78, "Output",
   CellTags->"FeynRule"],
  Cell[808182, 32378, 362, 12, 26, "Input",
   CellTags->"FeynRule"],
  Cell[808547, 32392, 104, 3, 26, "Output",
   CellTags->"FeynRule"],
  Cell[808666, 32398, 210, 5, 41, "Text",
   CellTags->"FeynRule"],
  Cell[808879, 32405, 144, 4, 26, "Input",
   CellTags->"FeynRule"],
  Cell[809026, 32411, 51, 1, 24, "Text",
   CellTags->"FeynRule"],
  Cell[809102, 32416, 693, 21, 46, "Input",
   CellTags->"FeynRule"],
  Cell[809798, 32439, 984, 42, 30, "Output",
   CellTags->"FeynRule"],
  Cell[810819, 32486, 212, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[811034, 32495, 1971, 79, 28, "Output",
   CellTags->"FeynRule"],
  Cell[813042, 32579, 385, 11, 26, "Input",
   CellTags->"FeynRule"],
  Cell[813430, 32592, 1419, 57, 28, "Output",
   CellTags->"FeynRule"],
  Cell[814886, 32654, 197, 6, 26, "Input",
   CellTags->"FeynRule"],
  Cell[815086, 32662, 104, 3, 26, "Output",
   CellTags->"FeynRule"],
  Cell[815227, 32670, 884, 26, 46, "Input",
   CellTags->"FeynRule"],
  Cell[816114, 32698, 1270, 54, 32, "Output",
   CellTags->"FeynRule"],
  Cell[817421, 32757, 212, 7, 26, "Input",
   CellTags->"FeynRule"],
  Cell[817636, 32766, 3305, 131, 28, "Output",
   CellTags->"FeynRule"],
  Cell[820978, 32902, 461, 13, 26, "Input",
   CellTags->"FeynRule"],
  Cell[821442, 32917, 3476, 132, 28, "Output",
   CellTags->"FeynRule"],
  Cell[824955, 33054, 197, 6, 26, "Input",
   CellTags->"FeynRule"],
  Cell[825155, 33062, 104, 3, 26, "Output",
   CellTags->"FeynRule"],
  Cell[825274, 33068, 418, 9, 46, "Input",
   CellTags->"FeynRule"]},
 "FI"->{
  Cell[825707, 33080, 41, 1, 39, "ObjectName",
   CellTags->"FI"],
  Cell[825773, 33085, 53, 1, 36, "Subsubsection",
   CellTags->"FI"],
  Cell[825829, 33088, 195, 4, 41, "Text",
   CellTags->"FI"],
  Cell[826027, 33094, 887, 32, 24, "Text",
   CellTags->"FI"]},
 "FieldDerivative"->{
  Cell[826929, 33129, 67, 1, 39, "ObjectName",
   CellTags->"FieldDerivative"],
  Cell[827021, 33134, 66, 1, 36, "Subsubsection",
   CellTags->"FieldDerivative"],
  Cell[827090, 33137, 549, 9, 75, "Text",
   CellTags->"FieldDerivative"],
  Cell[827642, 33148, 484, 18, 24, "Text",
   CellTags->"FieldDerivative"],
  Cell[828163, 33171, 63, 1, 36, "Subsubsection",
   CellTags->"FieldDerivative"],
  Cell[828251, 33176, 620, 19, 26, "Input",
   CellTags->"FieldDerivative"],
  Cell[828874, 33197, 465, 17, 29, "Output",
   CellTags->"FieldDerivative"],
  Cell[829376, 33219, 218, 6, 26, "Input",
   CellTags->"FieldDerivative"],
  Cell[829597, 33227, 1529, 55, 30, "Output",
   CellTags->"FieldDerivative"],
  Cell[831163, 33287, 218, 6, 26, "Input",
   CellTags->"FieldDerivative"],
  Cell[831384, 33295, 4269, 153, 54, "Output",
   CellTags->"FieldDerivative"]},
 "FieldStrength"->{
  Cell[835680, 33452, 63, 1, 39, "ObjectName",
   CellTags->"FieldStrength"],
  Cell[835768, 33457, 64, 1, 36, "Subsubsection",
   CellTags->"FieldStrength"],
  Cell[835835, 33460, 1285, 37, 62, "Text",
   CellTags->"FieldStrength"],
  Cell[837145, 33501, 125, 3, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[837273, 33506, 529, 15, 47, "Output",
   CellTags->"FieldStrength"],
  Cell[837817, 33524, 282, 11, 24, "Text",
   CellTags->"FieldStrength"],
  Cell[838136, 33540, 61, 1, 36, "Subsubsection",
   CellTags->"FieldStrength"],
  Cell[838222, 33545, 150, 4, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[838375, 33551, 174, 5, 30, "Output",
   CellTags->"FieldStrength"],
  Cell[838586, 33561, 160, 4, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[838749, 33567, 168, 5, 30, "Output",
   CellTags->"FieldStrength"],
  Cell[838954, 33577, 200, 5, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[839157, 33584, 722, 30, 30, "Output",
   CellTags->"FieldStrength"],
  Cell[839916, 33619, 210, 5, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[840129, 33626, 1507, 65, 30, "Output",
   CellTags->"FieldStrength"],
  Cell[841673, 33696, 241, 6, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[841917, 33704, 547, 15, 58, "Output",
   CellTags->"FieldStrength"],
  Cell[842501, 33724, 244, 6, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[842748, 33732, 545, 15, 58, "Output",
   CellTags->"FieldStrength"],
  Cell[843330, 33752, 291, 7, 26, "Input",
   CellTags->"FieldStrength"],
  Cell[843624, 33761, 1553, 66, 30, "Output",
   CellTags->"FieldStrength"]},
 "FinalFunction"->{
  Cell[845204, 33831, 63, 1, 39, "ObjectName",
   CellTags->"FinalFunction"],
  Cell[845292, 33836, 64, 1, 36, "Subsubsection",
   CellTags->"FinalFunction"],
  Cell[845359, 33839, 86, 1, 24, "Text",
   CellTags->"FinalFunction"],
  Cell[845448, 33842, 274, 11, 24, "Text",
   CellTags->"FinalFunction"]},
 "FinalSubstitutions"->{
  Cell[845737, 33856, 73, 1, 39, "ObjectName",
   CellTags->"FinalSubstitutions"],
  Cell[845835, 33861, 69, 1, 36, "Subsubsection",
   CellTags->"FinalSubstitutions"],
  Cell[845907, 33864, 204, 4, 41, "Text",
   CellTags->"FinalSubstitutions"],
  Cell[846114, 33870, 651, 25, 24, "Text",
   CellTags->"FinalSubstitutions"]},
 "FORM"->{
  Cell[846780, 33898, 45, 1, 39, "ObjectName",
   CellTags->"FORM"],
  Cell[846850, 33903, 55, 1, 36, "Subsubsection",
   CellTags->"FORM"],
  Cell[846908, 33906, 204, 5, 41, "Text",
   CellTags->"FORM"],
  Cell[847115, 33913, 449, 18, 24, "Text",
   CellTags->"FORM"]},
 "FORMEpilog"->{
  Cell[847579, 33934, 57, 1, 39, "ObjectName",
   CellTags->"FORMEpilog"],
  Cell[847661, 33939, 61, 1, 36, "Subsubsection",
   CellTags->"FORMEpilog"],
  Cell[847725, 33942, 161, 4, 24, "Text",
   CellTags->"FORMEpilog"],
  Cell[847889, 33948, 483, 18, 24, "Text",
   CellTags->"FORMEpilog"]},
 "FORMProlog"->{
  Cell[848387, 33969, 57, 1, 39, "ObjectName",
   CellTags->"FORMProlog"],
  Cell[848469, 33974, 61, 1, 36, "Subsubsection",
   CellTags->"FORMProlog"],
  Cell[848533, 33977, 178, 4, 24, "Text",
   CellTags->"FORMProlog"],
  Cell[848714, 33983, 483, 18, 24, "Text",
   CellTags->"FORMProlog"]},
 "FORM2FeynCalc"->{
  Cell[849212, 34004, 63, 1, 39, "ObjectName",
   CellTags->"FORM2FeynCalc"],
  Cell[849300, 34009, 64, 1, 36, "Subsubsection",
   CellTags->"FORM2FeynCalc"],
  Cell[849367, 34012, 470, 8, 58, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[849840, 34022, 286, 11, 24, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[850151, 34037, 125, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[850279, 34042, 662, 18, 47, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[850990, 34066, 61, 1, 36, "Subsubsection",
   CellTags->"FORM2FeynCalc"],
  Cell[851076, 34071, 139, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[851218, 34076, 354, 14, 28, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[851609, 34095, 114, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[851726, 34100, 243, 8, 38, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[851984, 34111, 159, 4, 24, "Text",
   CellTags->"FORM2FeynCalc"],
  Cell[852168, 34119, 314, 8, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[852485, 34129, 298, 10, 26, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[852820, 34144, 114, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[852937, 34149, 303, 9, 38, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[853277, 34163, 150, 4, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[853430, 34169, 331, 12, 26, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[853798, 34186, 117, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[853918, 34191, 259, 9, 26, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[854214, 34205, 150, 4, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[854367, 34211, 257, 9, 28, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[854661, 34225, 114, 3, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[854778, 34230, 307, 10, 38, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[855122, 34245, 333, 9, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[855458, 34256, 269, 9, 28, "Output",
   CellTags->"FORM2FeynCalc"],
  Cell[855764, 34270, 195, 5, 26, "Input",
   CellTags->"FORM2FeynCalc"],
  Cell[855962, 34277, 3922, 121, 71, "Output",
   CellTags->"FORM2FeynCalc"]},
 "FourDivergence"->{
  Cell[859911, 34402, 65, 1, 39, "ObjectName",
   CellTags->"FourDivergence"],
  Cell[860001, 34407, 65, 1, 36, "Subsubsection",
   CellTags->"FourDivergence"],
  Cell[860069, 34410, 242, 5, 41, "Text",
   CellTags->"FourDivergence"],
  Cell[860314, 34417, 283, 11, 24, "Text",
   CellTags->"FourDivergence"],
  Cell[860634, 34433, 62, 1, 36, "Subsubsection",
   CellTags->"FourDivergence"],
  Cell[860721, 34438, 167, 5, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[860891, 34445, 212, 8, 28, "Output",
   CellTags->"FourDivergence"],
  Cell[861140, 34458, 211, 6, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[861354, 34466, 242, 10, 28, "Output",
   CellTags->"FourDivergence"],
  Cell[861633, 34481, 192, 6, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[861828, 34489, 247, 9, 28, "Output",
   CellTags->"FourDivergence"],
  Cell[862112, 34503, 237, 7, 26, "Input",
   CellTags->"FourDivergence"],
  Cell[862352, 34512, 110, 3, 26, "Output",
   CellTags->"FourDivergence"],
  Cell[862477, 34518, 112, 3, 26, "Input",
   CellTags->"FourDivergence"]},
 "FourLaplacian"->{
  Cell[862604, 34524, 63, 1, 39, "ObjectName",
   CellTags->"FourLaplacian"],
  Cell[862692, 34529, 64, 1, 36, "Subsubsection",
   CellTags->"FourLaplacian"],
  Cell[862759, 34532, 426, 16, 27, "Text",
   CellTags->"FourLaplacian"],
  Cell[863210, 34552, 125, 3, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[863338, 34557, 170, 5, 26, "Output",
   CellTags->"FourLaplacian"],
  Cell[863523, 34565, 498, 18, 24, "Text",
   CellTags->"FourLaplacian"],
  Cell[864058, 34588, 61, 1, 36, "Subsubsection",
   CellTags->"FourLaplacian"],
  Cell[864144, 34593, 131, 4, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[864278, 34599, 167, 6, 28, "Output",
   CellTags->"FourLaplacian"],
  Cell[864482, 34610, 152, 4, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[864637, 34616, 132, 4, 26, "Output",
   CellTags->"FourLaplacian"],
  Cell[864806, 34625, 264, 9, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[865073, 34636, 440, 16, 48, "Output",
   CellTags->"FourLaplacian"],
  Cell[865550, 34657, 157, 4, 26, "Input",
   CellTags->"FourLaplacian"],
  Cell[865710, 34663, 3857, 136, 96, "Output",
   CellTags->"FourLaplacian"]},
 "FourVector"->{
  Cell[869594, 34803, 57, 1, 39, "ObjectName",
   CellTags->"FourVector"],
  Cell[869676, 34808, 61, 1, 36, "Subsubsection",
   CellTags->"FourVector"],
  Cell[869740, 34811, 339, 10, 41, "Text",
   CellTags->"FourVector"],
  Cell[870082, 34823, 587, 25, 24, "Text",
   CellTags->"FourVector"],
  Cell[870706, 34853, 58, 1, 36, "Subsubsection",
   CellTags->"FourVector"],
  Cell[870789, 34858, 140, 4, 26, "Input",
   CellTags->"FourVector"],
  Cell[870932, 34864, 132, 4, 29, "Output",
   CellTags->"FourVector"],
  Cell[871101, 34873, 164, 5, 26, "Input",
   CellTags->"FourVector"],
  Cell[871268, 34880, 249, 10, 30, "Output",
   CellTags->"FourVector"],
  Cell[871554, 34895, 180, 5, 26, "Input",
   CellTags->"FourVector"],
  Cell[871737, 34902, 155, 4, 38, "Output",
   CellTags->"FourVector"],
  Cell[871929, 34911, 229, 6, 26, "Input",
   CellTags->"FourVector"],
  Cell[872161, 34919, 203, 5, 38, "Output",
   CellTags->"FourVector"],
  Cell[872401, 34929, 212, 6, 26, "Input",
   CellTags->"FourVector"],
  Cell[872616, 34937, 155, 4, 38, "Output",
   CellTags->"FourVector"],
  Cell[872786, 34944, 193, 4, 24, "Text",
   CellTags->"FourVector"],
  Cell[873004, 34952, 212, 6, 26, "Input",
   CellTags->"FourVector"],
  Cell[873219, 34960, 407, 18, 28, "Output",
   CellTags->"FourVector"]},
 "FreeIndex"->{
  Cell[873653, 34982, 55, 1, 39, "ObjectName",
   CellTags->"FreeIndex"],
  Cell[873733, 34987, 60, 1, 36, "Subsubsection",
   CellTags->"FreeIndex"],
  Cell[873796, 34990, 152, 4, 24, "Text",
   CellTags->"FreeIndex"],
  Cell[873951, 34996, 454, 18, 24, "Text",
   CellTags->"FreeIndex"]},
 "FreeQ2"->{
  Cell[874420, 35017, 49, 1, 39, "ObjectName",
   CellTags->"FreeQ2"],
  Cell[874494, 35022, 57, 1, 36, "Subsubsection",
   CellTags->"FreeQ2"],
  Cell[874554, 35025, 228, 5, 41, "Text",
   CellTags->"FreeQ2"],
  Cell[874785, 35032, 479, 18, 24, "Text",
   CellTags->"FreeQ2"],
  Cell[875301, 35055, 54, 1, 36, "Subsubsection",
   CellTags->"FreeQ2"],
  Cell[875380, 35060, 246, 8, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[875629, 35070, 106, 3, 26, "Output",
   CellTags->"FreeQ2"],
  Cell[875772, 35078, 241, 8, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[876016, 35088, 105, 3, 26, "Output",
   CellTags->"FreeQ2"],
  Cell[876158, 35096, 133, 4, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[876294, 35102, 105, 3, 26, "Output",
   CellTags->"FreeQ2"],
  Cell[876436, 35110, 162, 5, 26, "Input",
   CellTags->"FreeQ2"],
  Cell[876601, 35117, 106, 3, 26, "Output",
   CellTags->"FreeQ2"]},
 "FRH"->{
  Cell[876734, 35124, 43, 1, 39, "ObjectName",
   CellTags->"FRH"],
  Cell[876802, 35129, 54, 1, 36, "Subsubsection",
   CellTags->"FRH"],
  Cell[876859, 35132, 134, 4, 24, "Text",
   CellTags->"FRH"],
  Cell[876996, 35138, 252, 11, 24, "Text",
   CellTags->"FRH"],
  Cell[877285, 35154, 51, 1, 36, "Subsubsection",
   CellTags->"FRH"],
  Cell[877361, 35159, 200, 6, 26, "Input",
   CellTags->"FRH"],
  Cell[877564, 35167, 248, 9, 26, "Output",
   CellTags->"FRH"],
  Cell[877849, 35181, 99, 3, 26, "Input",
   CellTags->"FRH"],
  Cell[877951, 35186, 99, 3, 26, "Output",
   CellTags->"FRH"],
  Cell[878087, 35194, 304, 10, 26, "Input",
   CellTags->"FRH"],
  Cell[878394, 35206, 531, 20, 26, "Output",
   CellTags->"FRH"],
  Cell[878962, 35231, 128, 4, 26, "Input",
   CellTags->"FRH"],
  Cell[879093, 35237, 128, 4, 26, "Output",
   CellTags->"FRH"]},
 "FromTFi"->{
  Cell[879248, 35245, 51, 1, 39, "ObjectName",
   CellTags->"FromTFi"],
  Cell[879324, 35250, 58, 1, 36, "Subsubsection",
   CellTags->"FromTFi"],
  Cell[879385, 35253, 200, 5, 41, "Text",
   CellTags->"FromTFi"],
  Cell[880039, 35282, 55, 1, 36, "Subsubsection",
   CellTags->"FromTFi"],
  Cell[880119, 35287, 431, 14, 26, "Input",
   CellTags->"FromTFi"],
  Cell[880553, 35303, 776, 24, 92, "Output",
   CellTags->"FromTFi"],
  Cell[881366, 35332, 693, 22, 26, "Input",
   CellTags->"FromTFi"],
  Cell[882062, 35356, 966, 31, 92, "Output",
   CellTags->"FromTFi"],
  Cell[883065, 35392, 756, 24, 26, "Input",
   CellTags->"FromTFi"],
  Cell[883824, 35418, 1027, 33, 92, "Output",
   CellTags->"FromTFi"]},
 "FromTFI"->{
  Cell[879588, 35260, 414, 17, 24, "Text",
   CellTags->"FromTFI"]},
 "FUNCTION"->{
  Cell[884878, 35455, 53, 1, 39, "ObjectName",
   CellTags->"FUNCTION"],
  Cell[884956, 35460, 59, 1, 36, "Subsubsection",
   CellTags->"FUNCTION"],
  Cell[885018, 35463, 160, 4, 24, "Text",
   CellTags->"FUNCTION"],
  Cell[885181, 35469, 253, 11, 24, "Text",
   CellTags->"FUNCTION"]},
 "FunctionalD"->{
  Cell[885449, 35483, 59, 1, 39, "ObjectName",
   CellTags->"FunctionalD"],
  Cell[885533, 35488, 62, 1, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[885598, 35491, 416, 7, 58, "Text",
   CellTags->"FunctionalD"],
  Cell[886017, 35500, 95, 1, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[886115, 35503, 472, 18, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[886624, 35526, 59, 1, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[886686, 35529, 610, 20, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[887321, 35553, 228, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[887552, 35561, 107, 3, 26, "Output",
   CellTags->"FunctionalD"],
  Cell[887696, 35569, 253, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[887952, 35578, 135, 4, 26, "Output",
   CellTags->"FunctionalD"],
  Cell[888102, 35585, 1269, 44, 51, "Text",
   CellTags->"FunctionalD"],
  Cell[889396, 35633, 295, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[889694, 35643, 273, 10, 33, "Output",
   CellTags->"FunctionalD"],
  Cell[889982, 35656, 778, 24, 27, "Text",
   CellTags->"FunctionalD"],
  Cell[890785, 35684, 668, 22, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[891456, 35708, 735, 26, 42, "Output",
   CellTags->"FunctionalD"],
  Cell[892228, 35739, 217, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[892448, 35747, 652, 26, 29, "Output",
   CellTags->"FunctionalD"],
  Cell[893115, 35776, 489, 15, 28, "Text",
   CellTags->"FunctionalD"],
  Cell[893607, 35793, 97, 2, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[893729, 35799, 315, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[894047, 35809, 1303, 56, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[895387, 35870, 315, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[895705, 35880, 1303, 56, 32, "Output",
   CellTags->"FunctionalD"],
  Cell[897045, 35941, 216, 8, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[897264, 35951, 2956, 115, 42, "Output",
   CellTags->"FunctionalD"],
  Cell[900235, 36069, 560, 13, 61, "Text",
   CellTags->"FunctionalD"],
  Cell[900798, 36084, 1398, 50, 27, "Text",
   CellTags->"FunctionalD"],
  Cell[902199, 36136, 445, 13, 26, "Text",
   CellTags->"FunctionalD"],
  Cell[902669, 36153, 231, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[902903, 36162, 286, 12, 28, "Output",
   CellTags->"FunctionalD"],
  Cell[903226, 36179, 165, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[903394, 36186, 2288, 96, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[905697, 36285, 120, 3, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[905842, 36292, 176, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[906021, 36299, 2288, 96, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[908324, 36398, 290, 8, 26, "Text",
   CellTags->"FunctionalD"],
  Cell[908617, 36408, 523, 16, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[909165, 36428, 230, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[909398, 36437, 384, 17, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[909819, 36459, 240, 7, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[910062, 36468, 416, 18, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[910515, 36491, 579, 16, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[911097, 36509, 2958, 127, 42, "Output",
   CellTags->"FunctionalD"],
  Cell[914092, 36641, 184, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[914279, 36648, 1405, 62, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[915721, 36715, 352, 11, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[916076, 36728, 1159, 50, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[917250, 36781, 269, 9, 27, "Text",
   CellTags->"FunctionalD"],
  Cell[917522, 36792, 98, 2, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[917623, 36796, 167, 4, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[917815, 36804, 105, 3, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[917923, 36809, 2956, 115, 42, "Output",
   CellTags->"FunctionalD"],
  Cell[920916, 36929, 194, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[921113, 36937, 23483, 1005, 156, "Output",
   CellTags->"FunctionalD"],
  Cell[944611, 37945, 227, 5, 41, "Text",
   CellTags->"FunctionalD"],
  Cell[944841, 37952, 345, 6, 58, "Text",
   CellTags->"FunctionalD"],
  Cell[945189, 37960, 132, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[945324, 37966, 2583, 67, 122, "Input",
   CellTags->"FunctionalD"],
  Cell[947932, 38037, 169, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[948104, 38044, 12210, 526, 80, "Output",
   CellTags->"FunctionalD"],
  Cell[960351, 38575, 164, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[960518, 38582, 6535, 286, 42, "Output",
   CellTags->"FunctionalD"],
  Cell[967068, 38871, 212, 8, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[967305, 38883, 315, 11, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[967623, 38896, 2734, 119, 42, "Output",
   CellTags->"FunctionalD"],
  Cell[970394, 39020, 647, 20, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[971044, 39042, 1310, 60, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[972391, 39107, 154, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[972548, 39113, 5227, 228, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[977790, 39344, 131, 4, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[977946, 39352, 79, 2, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[978028, 39356, 1159, 50, 30, "Output",
   CellTags->"FunctionalD"],
  Cell[979224, 39411, 798, 21, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[980025, 39434, 1064, 41, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[981126, 39480, 137, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[981266, 39486, 4174, 167, 34, "Output",
   CellTags->"FunctionalD"],
  Cell[985477, 39658, 204, 6, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[985684, 39666, 6098, 267, 32, "Output",
   CellTags->"FunctionalD"],
  Cell[991819, 39938, 138, 4, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[991960, 39944, 11189, 491, 56, "Output",
   CellTags->"FunctionalD"],
  Cell[1003186, 40440, 163, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[1003352, 40447, 10243, 451, 56, "Output",
   CellTags->"FunctionalD"],
  Cell[1013610, 40901, 66, 1, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[1013679, 40904, 539, 13, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[1014255, 40922, 110, 2, 36, "Subsubsection",
   CellTags->"FunctionalD"],
  Cell[1014368, 40926, 79, 1, 24, "Text",
   CellTags->"FunctionalD"],
  Cell[1016922, 41023, 765, 22, 46, "Input",
   CellTags->"FunctionalD"],
  Cell[1017690, 41047, 8897, 329, 91, "Output",
   CellTags->"FunctionalD"],
  Cell[1026624, 41381, 165, 5, 26, "Input",
   CellTags->"FunctionalD"],
  Cell[1026792, 41388, 2097, 84, 42, "Output",
   CellTags->"FunctionalD"]},
 "FV"->{
  Cell[1028916, 41476, 41, 1, 39, "ObjectName",
   CellTags->"FV"],
  Cell[1028982, 41481, 53, 1, 36, "Subsubsection",
   CellTags->"FV"],
  Cell[1029038, 41484, 275, 13, 26, "Text",
   CellTags->"FV"],
  Cell[1029316, 41499, 955, 39, 24, "Text",
   CellTags->"FV"],
  Cell[1030308, 41543, 50, 1, 36, "Subsubsection",
   CellTags->"FV"],
  Cell[1030383, 41548, 124, 4, 26, "Input",
   CellTags->"FV"],
  Cell[1030510, 41554, 126, 4, 28, "Output",
   CellTags->"FV"],
  Cell[1030673, 41563, 148, 5, 26, "Input",
   CellTags->"FV"],
  Cell[1030824, 41570, 185, 7, 28, "Output",
   CellTags->"FV"],
  Cell[1031046, 41582, 160, 5, 26, "Input",
   CellTags->"FV"],
  Cell[1031209, 41589, 139, 4, 38, "Output",
   CellTags->"FV"],
  Cell[1031385, 41598, 195, 7, 26, "Input",
   CellTags->"FV"],
  Cell[1031583, 41607, 217, 6, 38, "Output",
   CellTags->"FV"],
  Cell[1031815, 41616, 130, 4, 24, "Text",
   CellTags->"FV"],
  Cell[1031970, 41624, 196, 6, 26, "Input",
   CellTags->"FV"],
  Cell[1032169, 41632, 399, 18, 28, "Output",
   CellTags->"FV"],
  Cell[1032605, 41655, 107, 3, 26, "Input",
   CellTags->"FV"],
  Cell[1032715, 41660, 380, 11, 38, "Output",
   CellTags->"FV"]},
 "FVD"->{
  Cell[1033122, 41675, 43, 1, 39, "ObjectName",
   CellTags->"FVD"],
  Cell[1033190, 41680, 54, 1, 36, "Subsubsection",
   CellTags->"FVD"],
  Cell[1033247, 41683, 312, 15, 22, "Text",
   CellTags->"FVD"],
  Cell[1033562, 41700, 952, 39, 22, "Text",
   CellTags->"FVD"],
  Cell[1034551, 41744, 51, 1, 36, "Subsubsection",
   CellTags->"FVD"],
  Cell[1034627, 41749, 126, 4, 26, "Input",
   CellTags->"FVD"],
  Cell[1034756, 41755, 127, 4, 28, "Output",
   CellTags->"FVD"],
  Cell[1034920, 41764, 150, 5, 26, "Input",
   CellTags->"FVD"],
  Cell[1035073, 41771, 186, 7, 28, "Output",
   CellTags->"FVD"],
  Cell[1035296, 41783, 162, 5, 26, "Input",
   CellTags->"FVD"],
  Cell[1035461, 41790, 141, 4, 38, "Output",
   CellTags->"FVD"],
  Cell[1035639, 41799, 197, 7, 26, "Input",
   CellTags->"FVD"],
  Cell[1035839, 41808, 268, 8, 38, "Output",
   CellTags->"FVD"],
  Cell[1036122, 41819, 88, 1, 24, "Text",
   CellTags->"FVD"],
  Cell[1036235, 41824, 198, 6, 26, "Input",
   CellTags->"FVD"],
  Cell[1036436, 41832, 400, 18, 28, "Output",
   CellTags->"FVD"],
  Cell[1036873, 41855, 108, 3, 26, "Input",
   CellTags->"FVD"],
  Cell[1036984, 41860, 485, 15, 38, "Output",
   CellTags->"FVD"]},
 "GA"->{
  Cell[1037496, 41879, 41, 1, 39, "ObjectName",
   CellTags->"GA"],
  Cell[1037562, 41884, 53, 1, 28, "Subsubsection",
   CellTags->"GA"],
  Cell[1037618, 41887, 654, 24, 40, "Text",
   CellTags->"GA"],
  Cell[1038275, 41913, 607, 25, 22, "Text",
   CellTags->"GA"],
  Cell[1038919, 41943, 50, 1, 36, "Subsubsection",
   CellTags->"GA"],
  Cell[1038994, 41948, 101, 3, 26, "Input",
   CellTags->"GA"],
  Cell[1039098, 41953, 133, 4, 26, "Output",
   CellTags->"GA"],
  Cell[1039268, 41962, 213, 7, 26, "Input",
   CellTags->"GA"],
  Cell[1039484, 41971, 317, 10, 26, "Output",
   CellTags->"GA"],
  Cell[1039838, 41986, 171, 5, 26, "Input",
   CellTags->"GA"],
  Cell[1040012, 41993, 163, 4, 38, "Output",
   CellTags->"GA"],
  Cell[1040212, 42002, 163, 5, 26, "Input",
   CellTags->"GA"],
  Cell[1040378, 42009, 289, 8, 26, "Output",
   CellTags->"GA"],
  Cell[1040704, 42022, 203, 6, 26, "Input",
   CellTags->"GA"],
  Cell[1040910, 42030, 262, 7, 38, "Output",
   CellTags->"GA"],
  Cell[1041209, 42042, 250, 8, 26, "Input",
   CellTags->"GA"],
  Cell[1041462, 42052, 301, 9, 28, "Output",
   CellTags->"GA"]},
 "GAD"->{
  Cell[1041790, 42065, 43, 1, 39, "ObjectName",
   CellTags->"GAD"],
  Cell[1041858, 42070, 54, 1, 28, "Subsubsection",
   CellTags->"GAD"],
  Cell[1041915, 42073, 671, 24, 40, "Text",
   CellTags->"GAD"],
  Cell[1042589, 42099, 604, 25, 22, "Text",
   CellTags->"GAD"],
  Cell[1043230, 42129, 51, 1, 36, "Subsubsection",
   CellTags->"GAD"],
  Cell[1043306, 42134, 103, 3, 26, "Input",
   CellTags->"GAD"],
  Cell[1043412, 42139, 134, 4, 26, "Output",
   CellTags->"GAD"],
  Cell[1043583, 42148, 216, 7, 26, "Input",
   CellTags->"GAD"],
  Cell[1043802, 42157, 318, 10, 26, "Output",
   CellTags->"GAD"],
  Cell[1044157, 42172, 173, 5, 26, "Input",
   CellTags->"GAD"],
  Cell[1044333, 42179, 213, 6, 38, "Output",
   CellTags->"GAD"],
  Cell[1044583, 42190, 165, 5, 26, "Input",
   CellTags->"GAD"],
  Cell[1044751, 42197, 290, 8, 26, "Output",
   CellTags->"GAD"],
  Cell[1045078, 42210, 205, 6, 26, "Input",
   CellTags->"GAD"],
  Cell[1045286, 42218, 267, 7, 38, "Output",
   CellTags->"GAD"],
  Cell[1045590, 42230, 254, 8, 26, "Input",
   CellTags->"GAD"],
  Cell[1045847, 42240, 302, 9, 28, "Output",
   CellTags->"GAD"]},
 "GammaEpsilon"->{
  Cell[1046176, 42253, 61, 1, 39, "ObjectName",
   CellTags->"GammaEpsilon"],
  Cell[1046262, 42258, 63, 1, 36, "Subsubsection",
   CellTags->"GammaEpsilon"],
  Cell[1046328, 42261, 167, 4, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1046498, 42267, 465, 18, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1047000, 42290, 60, 1, 36, "Subsubsection",
   CellTags->"GammaEpsilon"],
  Cell[1047063, 42293, 141, 4, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1047229, 42301, 170, 5, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1047402, 42308, 1361, 48, 48, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1048800, 42361, 170, 5, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1048973, 42368, 1109, 41, 48, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1050097, 42412, 91, 1, 24, "Text",
   CellTags->"GammaEpsilon"],
  Cell[1050213, 42417, 123, 3, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1050339, 42422, 2148, 77, 93, "Output",
   CellTags->"GammaEpsilon"],
  Cell[1052524, 42504, 117, 3, 26, "Input",
   CellTags->"GammaEpsilon"],
  Cell[1052644, 42509, 138, 5, 26, "Output",
   CellTags->"GammaEpsilon"]},
 "GammaExpand"->{
  Cell[1052809, 42518, 59, 1, 39, "ObjectName",
   CellTags->"GammaExpand"],
  Cell[1052893, 42523, 62, 1, 36, "Subsubsection",
   CellTags->"GammaExpand"],
  Cell[1052958, 42526, 123, 3, 24, "Text",
   CellTags->"GammaExpand"],
  Cell[1053084, 42531, 280, 11, 24, "Text",
   CellTags->"GammaExpand"],
  Cell[1053401, 42547, 59, 1, 36, "Subsubsection",
   CellTags->"GammaExpand"],
  Cell[1053485, 42552, 187, 5, 26, "Input",
   CellTags->"GammaExpand"],
  Cell[1053675, 42559, 329, 11, 26, "Output",
   CellTags->"GammaExpand"],
  Cell[1054041, 42575, 197, 6, 26, "Input",
   CellTags->"GammaExpand"],
  Cell[1054241, 42583, 632, 22, 45, "Output",
   CellTags->"GammaExpand"],
  Cell[1054910, 42610, 187, 5, 26, "Input",
   CellTags->"GammaExpand"],
  Cell[1055100, 42617, 203, 6, 26, "Output",
   CellTags->"GammaExpand"]},
 "Gamma1"->{
  Cell[1055330, 42627, 49, 1, 39, "ObjectName",
   CellTags->"Gamma1"],
  Cell[1055404, 42632, 57, 1, 36, "Subsubsection",
   CellTags->"Gamma1"],
  Cell[1055464, 42635, 139, 4, 24, "Text",
   CellTags->"Gamma1"],
  Cell[1055606, 42641, 435, 18, 24, "Text",
   CellTags->"Gamma1"]},
 "Gamma2"->{
  Cell[1056056, 42662, 49, 1, 39, "ObjectName",
   CellTags->"Gamma2"],
  Cell[1056130, 42667, 57, 1, 36, "Subsubsection",
   CellTags->"Gamma2"],
  Cell[1056190, 42670, 172, 4, 24, "Text",
   CellTags->"Gamma2"],
  Cell[1056365, 42676, 435, 18, 24, "Text",
   CellTags->"Gamma2"]},
 "Gamma3"->{
  Cell[1056815, 42697, 49, 1, 39, "ObjectName",
   CellTags->"Gamma3"],
  Cell[1056889, 42702, 57, 1, 36, "Subsubsection",
   CellTags->"Gamma3"],
  Cell[1056949, 42705, 237, 5, 41, "Text",
   CellTags->"Gamma3"],
  Cell[1057189, 42712, 435, 18, 24, "Text",
   CellTags->"Gamma3"]},
 "Gauge"->{
  Cell[1057639, 42733, 47, 1, 39, "ObjectName",
   CellTags->"Gauge"],
  Cell[1057711, 42738, 56, 1, 36, "Subsubsection",
   CellTags->"Gauge"],
  Cell[1057770, 42741, 133, 4, 24, "Text",
   CellTags->"Gauge"],
  Cell[1057906, 42747, 470, 18, 24, "Text",
   CellTags->"Gauge"]},
 "GaugeField"->{
  Cell[1058391, 42768, 57, 1, 39, "ObjectName",
   CellTags->"GaugeField"],
  Cell[1058473, 42773, 61, 1, 36, "Subsubsection",
   CellTags->"GaugeField"],
  Cell[1058537, 42776, 212, 5, 41, "Text",
   CellTags->"GaugeField"],
  Cell[1058752, 42783, 491, 18, 24, "Text",
   CellTags->"GaugeField"],
  Cell[1059280, 42806, 58, 1, 36, "Subsubsection",
   CellTags->"GaugeField"],
  Cell[1059363, 42811, 86, 2, 26, "Input",
   CellTags->"GaugeField"],
  Cell[1059452, 42815, 106, 3, 27, "Output",
   CellTags->"GaugeField"],
  Cell[1059595, 42823, 237, 6, 26, "Input",
   CellTags->"GaugeField"],
  Cell[1059835, 42831, 316, 14, 30, "Output",
   CellTags->"GaugeField"]},
 "GaugeXi"->{
  Cell[1060178, 42849, 51, 1, 39, "ObjectName",
   CellTags->"GaugeXi"],
  Cell[1060254, 42854, 58, 1, 36, "Subsubsection",
   CellTags->"GaugeXi"],
  Cell[1060315, 42857, 77, 1, 24, "Text",
   CellTags->"GaugeXi"],
  Cell[1060395, 42860, 448, 18, 24, "Text",
   CellTags->"GaugeXi"]},
 "GA5"->{
  Cell[1060858, 42881, 43, 1, 39, "ObjectName",
   CellTags->"GA5"],
  Cell[1060926, 42886, 54, 1, 36, "Subsubsection",
   CellTags->"GA5"],
  Cell[1060983, 42889, 88, 1, 24, "Text",
   CellTags->"GA5"],
  Cell[1061074, 42892, 600, 25, 24, "Text",
   CellTags->"GA5"],
  Cell[1061711, 42922, 51, 1, 36, "Subsubsection",
   CellTags->"GA5"],
  Cell[1061787, 42927, 72, 2, 26, "Input",
   CellTags->"GA5"],
  Cell[1061862, 42931, 130, 4, 26, "Output",
   CellTags->"GA5"],
  Cell[1062029, 42940, 94, 3, 26, "Input",
   CellTags->"GA5"],
  Cell[1062126, 42945, 99, 3, 26, "Output",
   CellTags->"GA5"]},
 "GGV"->{
  Cell[1062252, 42952, 43, 1, 39, "ObjectName",
   CellTags->"GGV"],
  Cell[1062320, 42957, 54, 1, 36, "Subsubsection",
   CellTags->"GGV"],
  Cell[1062377, 42960, 72, 1, 24, "Text",
   CellTags->"GGV"],
  Cell[1062452, 42963, 288, 11, 24, "Text",
   CellTags->"GGV"]},
 "GhostPropagator"->{
  Cell[1062755, 42977, 67, 1, 39, "ObjectName",
   CellTags->"GhostPropagator"],
  Cell[1062847, 42982, 66, 1, 36, "Subsubsection",
   CellTags->"GhostPropagator"],
  Cell[1062916, 42985, 324, 9, 24, "Text",
   CellTags->"GhostPropagator"],
  Cell[1063243, 42996, 692, 25, 24, "Text",
   CellTags->"GhostPropagator"],
  Cell[1064259, 43040, 63, 1, 36, "Subsubsection",
   CellTags->"GhostPropagator"],
  Cell[1064347, 43045, 156, 4, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1064506, 43051, 193, 6, 28, "Output",
   CellTags->"GhostPropagator"],
  Cell[1064736, 43062, 206, 5, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1064945, 43069, 409, 16, 48, "Output",
   CellTags->"GhostPropagator"],
  Cell[1065391, 43090, 123, 3, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1065517, 43095, 176, 6, 28, "Output",
   CellTags->"GhostPropagator"],
  Cell[1065730, 43106, 188, 5, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1065921, 43113, 413, 16, 48, "Output",
   CellTags->"GhostPropagator"],
  Cell[1066371, 43134, 285, 9, 26, "Input",
   CellTags->"GhostPropagator"],
  Cell[1066659, 43145, 228, 6, 38, "Output",
   CellTags->"GhostPropagator"]},
 "GHP"->{
  Cell[1066914, 43155, 43, 1, 39, "ObjectName",
   CellTags->"GHP"],
  Cell[1066982, 43160, 54, 1, 36, "Subsubsection",
   CellTags->"GHP"],
  Cell[1067039, 43163, 288, 9, 24, "Text",
   CellTags->"GHP"],
  Cell[1067330, 43174, 728, 25, 24, "Text",
   CellTags->"GHP"],
  Cell[1068095, 43204, 51, 1, 36, "Subsubsection",
   CellTags->"GHP"],
  Cell[1068171, 43209, 132, 4, 26, "Input",
   CellTags->"GHP"],
  Cell[1068306, 43215, 181, 6, 28, "Output",
   CellTags->"GHP"],
  Cell[1068524, 43226, 130, 4, 26, "Input",
   CellTags->"GHP"],
  Cell[1068657, 43232, 157, 5, 48, "Output",
   CellTags->"GHP"],
  Cell[1068851, 43242, 132, 4, 26, "Input",
   CellTags->"GHP"],
  Cell[1068986, 43248, 185, 6, 28, "Output",
   CellTags->"GHP"],
  Cell[1069208, 43259, 264, 9, 26, "Input",
   CellTags->"GHP"],
  Cell[1069475, 43270, 216, 6, 38, "Output",
   CellTags->"GHP"]},
 "GluonField"->{
  Cell[1069718, 43280, 57, 1, 39, "ObjectName",
   CellTags->"GluonField"],
  Cell[1069800, 43285, 61, 1, 36, "Subsubsection",
   CellTags->"GluonField"],
  Cell[1069864, 43288, 79, 1, 24, "Text",
   CellTags->"GluonField"],
  Cell[1069946, 43291, 271, 11, 24, "Text",
   CellTags->"GluonField"],
  Cell[1070254, 43307, 58, 1, 36, "Subsubsection",
   CellTags->"GluonField"],
  Cell[1070337, 43312, 86, 2, 26, "Input",
   CellTags->"GluonField"],
  Cell[1070426, 43316, 106, 3, 27, "Output",
   CellTags->"GluonField"],
  Cell[1070569, 43324, 237, 6, 26, "Input",
   CellTags->"GluonField"],
  Cell[1070809, 43332, 316, 14, 30, "Output",
   CellTags->"GluonField"]},
 "GluonGhostVertex"->{
  Cell[1071152, 43350, 69, 1, 39, "ObjectName",
   CellTags->"GluonGhostVertex"],
  Cell[1071246, 43355, 67, 1, 28, "Subsubsection",
   CellTags->"GluonGhostVertex"],
  Cell[1071316, 43358, 422, 7, 52, "Text",
   CellTags->"GluonGhostVertex"],
  Cell[1071741, 43367, 935, 31, 24, "Text",
   CellTags->"GluonGhostVertex"],
  Cell[1072701, 43402, 131, 3, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1072835, 43407, 324, 10, 26, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1073208, 43423, 64, 1, 36, "Subsubsection",
   CellTags->"GluonGhostVertex"],
  Cell[1073297, 43428, 348, 10, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1073648, 43440, 517, 21, 26, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1075660, 43538, 148, 4, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1075811, 43544, 333, 10, 38, "Output",
   CellTags->"GluonGhostVertex"],
  Cell[1076181, 43559, 191, 5, 26, "Input",
   CellTags->"GluonGhostVertex"],
  Cell[1076375, 43566, 521, 21, 30, "Output",
   CellTags->"GluonGhostVertex"]},
 "GluonPropagator"->{
  Cell[1076923, 43591, 67, 1, 39, "ObjectName",
   CellTags->"GluonPropagator"],
  Cell[1077015, 43596, 66, 1, 36, "Subsubsection",
   CellTags->"GluonPropagator"],
  Cell[1077084, 43599, 535, 10, 58, "Text",
   CellTags->"GluonPropagator"],
  Cell[1077622, 43611, 99, 1, 24, "Text",
   CellTags->"GluonPropagator"],
  Cell[1077746, 43616, 83, 2, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1077832, 43620, 125, 3, 26, "Output",
   CellTags->"GluonPropagator"],
  Cell[1077994, 43628, 129, 3, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1078126, 43633, 482, 13, 26, "Output",
   CellTags->"GluonPropagator"],
  Cell[1078623, 43649, 944, 32, 24, "Text",
   CellTags->"GluonPropagator"],
  Cell[1079604, 43686, 72, 1, 33, "Subsubsection",
   CellTags->"GluonPropagator"],
  Cell[1079701, 43691, 187, 5, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1079891, 43698, 233, 8, 28, "Output",
   CellTags->"GluonPropagator"],
  Cell[1080867, 43741, 271, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1081141, 43750, 1035, 39, 58, "Output",
   CellTags->"GluonPropagator"],
  Cell[1082213, 43794, 357, 10, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1082573, 43806, 2748, 104, 59, "Output",
   CellTags->"GluonPropagator"],
  Cell[1085358, 43915, 199, 6, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1085560, 43923, 271, 9, 48, "Output",
   CellTags->"GluonPropagator"],
  Cell[1085846, 43935, 149, 4, 24, "Text",
   CellTags->"GluonPropagator"],
  Cell[1086020, 43943, 242, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1086265, 43952, 1028, 39, 60, "Output",
   CellTags->"GluonPropagator"],
  Cell[1087330, 43996, 267, 9, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1087600, 44007, 314, 9, 38, "Output",
   CellTags->"GluonPropagator"],
  Cell[1087951, 44021, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1088229, 44030, 1274, 46, 53, "Output",
   CellTags->"GluonPropagator"],
  Cell[1089540, 44081, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1089818, 44090, 1288, 47, 49, "Output",
   CellTags->"GluonPropagator"],
  Cell[1091143, 44142, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1091421, 44151, 1282, 46, 53, "Output",
   CellTags->"GluonPropagator"],
  Cell[1092740, 44202, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1093018, 44211, 1274, 46, 53, "Output",
   CellTags->"GluonPropagator"],
  Cell[1094329, 44262, 275, 7, 26, "Input",
   CellTags->"GluonPropagator"],
  Cell[1094607, 44271, 2422, 88, 53, "Output",
   CellTags->"GluonPropagator"]},
 "GluonSelfEnergy"->{
  Cell[1097056, 44363, 67, 1, 39, "ObjectName",
   CellTags->"GluonSelfEnergy"],
  Cell[1097148, 44368, 66, 1, 36, "Subsubsection",
   CellTags->"GluonSelfEnergy"],
  Cell[1097217, 44371, 122, 3, 24, "Text",
   CellTags->"GluonSelfEnergy"],
  Cell[1097364, 44378, 129, 3, 26, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1097496, 44383, 794, 23, 29, "Output",
   CellTags->"GluonSelfEnergy"],
  Cell[1098305, 44409, 944, 32, 24, "Text",
   CellTags->"GluonSelfEnergy"],
  Cell[1099286, 44446, 72, 1, 33, "Subsubsection",
   CellTags->"GluonSelfEnergy"],
  Cell[1099383, 44451, 252, 8, 26, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1099638, 44461, 2391, 92, 45, "Output",
   CellTags->"GluonSelfEnergy"],
  Cell[1102066, 44558, 300, 9, 26, "Input",
   CellTags->"GluonSelfEnergy"],
  Cell[1102369, 44569, 2734, 103, 55, "Output",
   CellTags->"GluonSelfEnergy"]},
 "GluonVertex"->{
  Cell[1105130, 44676, 59, 1, 39, "ObjectName",
   CellTags->"GluonVertex"],
  Cell[1105214, 44681, 62, 1, 28, "Subsubsection",
   CellTags->"GluonVertex"],
  Cell[1105279, 44684, 88, 1, 22, "Text",
   CellTags->"GluonVertex"],
  Cell[1105370, 44687, 621, 10, 67, "Text",
   CellTags->"GluonVertex"],
  Cell[1105994, 44699, 91, 1, 24, "Text",
   CellTags->"GluonVertex"],
  Cell[1106110, 44704, 79, 2, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1106192, 44708, 117, 3, 26, "Output",
   CellTags->"GluonVertex"],
  Cell[1106346, 44716, 121, 3, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1106470, 44721, 378, 11, 26, "Output",
   CellTags->"GluonVertex"],
  Cell[1106863, 44735, 516, 18, 24, "Text",
   CellTags->"GluonVertex"],
  Cell[1107416, 44758, 59, 1, 36, "Subsubsection",
   CellTags->"GluonVertex"],
  Cell[1107500, 44763, 338, 10, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1107841, 44775, 943, 42, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1108821, 44822, 379, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1109203, 44836, 1974, 79, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1111214, 44920, 308, 10, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1111525, 44932, 272, 9, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1111834, 44946, 349, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1112186, 44960, 1682, 65, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1113905, 45030, 414, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1114322, 45044, 352, 11, 29, "Output",
   CellTags->"GluonVertex"],
  Cell[1118635, 45211, 374, 12, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1119012, 45225, 160, 5, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1119209, 45235, 259, 7, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1119471, 45244, 3479, 132, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1122965, 45379, 200, 4, 41, "Text",
   CellTags->"GluonVertex"],
  Cell[1123190, 45387, 172, 5, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1123365, 45394, 942, 42, 28, "Output",
   CellTags->"GluonVertex"],
  Cell[1124344, 45441, 230, 7, 26, "Input",
   CellTags->"GluonVertex"],
  Cell[1124577, 45450, 3431, 132, 52, "Output",
   CellTags->"GluonVertex"]},
 "GO"->{
  Cell[1128035, 45586, 41, 1, 39, "ObjectName",
   CellTags->"GO"],
  Cell[1128101, 45591, 53, 1, 36, "Subsubsection",
   CellTags->"GO"],
  Cell[1128157, 45594, 73, 1, 22, "Text",
   CellTags->"GO"],
  Cell[1128233, 45597, 299, 11, 22, "Text",
   CellTags->"GO"]},
 "GP"->{
  Cell[1128547, 45611, 41, 1, 39, "ObjectName",
   CellTags->"GP"],
  Cell[1128613, 45616, 53, 1, 28, "Subsubsection",
   CellTags->"GP"],
  Cell[1128669, 45619, 69, 1, 22, "Text",
   CellTags->"GP"],
  Cell[1128741, 45622, 283, 11, 22, "Text",
   CellTags->"GP"]},
 "GrassmannParity"->{
  Cell[1129039, 45636, 67, 1, 39, "ObjectName",
   CellTags->"GrassmannParity"],
  Cell[1129131, 45641, 66, 1, 28, "Subsubsection",
   CellTags->"GrassmannParity"],
  Cell[1129200, 45644, 216, 5, 22, "Text",
   CellTags->"GrassmannParity"],
  Cell[1129419, 45651, 268, 11, 22, "Text",
   CellTags->"GrassmannParity"]},
 "GS"->{
  Cell[1129702, 45665, 41, 1, 39, "ObjectName",
   CellTags->"GS"],
  Cell[1129768, 45670, 53, 1, 28, "Subsubsection",
   CellTags->"GS"],
  Cell[1129824, 45673, 635, 18, 40, "Text",
   CellTags->"GS"],
  Cell[1130462, 45693, 803, 32, 22, "Text",
   CellTags->"GS"],
  Cell[1131302, 45730, 50, 1, 36, "Subsubsection",
   CellTags->"GS"],
  Cell[1131377, 45735, 97, 3, 26, "Input",
   CellTags->"GS"],
  Cell[1131477, 45740, 139, 4, 28, "Output",
   CellTags->"GS"],
  Cell[1131653, 45749, 162, 6, 26, "Input",
   CellTags->"GS"],
  Cell[1131818, 45757, 155, 4, 38, "Output",
   CellTags->"GS"],
  Cell[1132010, 45766, 140, 4, 26, "Input",
   CellTags->"GS"],
  Cell[1132153, 45772, 412, 13, 28, "Output",
   CellTags->"GS"],
  Cell[1132602, 45790, 179, 6, 26, "Input",
   CellTags->"GS"],
  Cell[1132784, 45798, 242, 7, 38, "Output",
   CellTags->"GS"],
  Cell[1133063, 45810, 237, 8, 26, "Input",
   CellTags->"GS"],
  Cell[1133303, 45820, 361, 12, 28, "Output",
   CellTags->"GS"]},
 "GSD"->{
  Cell[1133691, 45836, 43, 1, 39, "ObjectName",
   CellTags->"GSD"],
  Cell[1133759, 45841, 54, 1, 28, "Subsubsection",
   CellTags->"GSD"],
  Cell[1133816, 45844, 644, 18, 40, "Text",
   CellTags->"GSD"],
  Cell[1134463, 45864, 804, 32, 22, "Text",
   CellTags->"GSD"],
  Cell[1135304, 45901, 51, 1, 36, "Subsubsection",
   CellTags->"GSD"],
  Cell[1135380, 45906, 99, 3, 26, "Input",
   CellTags->"GSD"],
  Cell[1135482, 45911, 140, 4, 28, "Output",
   CellTags->"GSD"],
  Cell[1135659, 45920, 164, 6, 26, "Input",
   CellTags->"GSD"],
  Cell[1135826, 45928, 205, 6, 38, "Output",
   CellTags->"GSD"],
  Cell[1136068, 45939, 142, 4, 26, "Input",
   CellTags->"GSD"],
  Cell[1136213, 45945, 413, 13, 28, "Output",
   CellTags->"GSD"],
  Cell[1136663, 45963, 181, 6, 26, "Input",
   CellTags->"GSD"],
  Cell[1136847, 45971, 247, 7, 38, "Output",
   CellTags->"GSD"],
  Cell[1137131, 45983, 241, 8, 26, "Input",
   CellTags->"GSD"],
  Cell[1137375, 45993, 362, 12, 28, "Output",
   CellTags->"GSD"]},
 "Gstrong"->{
  Cell[1137764, 46009, 51, 1, 39, "ObjectName",
   CellTags->"Gstrong"],
  Cell[1137840, 46014, 58, 1, 28, "Subsubsection",
   CellTags->"Gstrong"],
  Cell[1137901, 46017, 83, 1, 22, "Text",
   CellTags->"Gstrong"],
  Cell[1137987, 46020, 334, 15, 22, "Text",
   CellTags->"Gstrong"],
  Cell[1138358, 46040, 55, 1, 34, "Subsubsection",
   CellTags->"Gstrong"],
  Cell[1138416, 46043, 119, 3, 24, "Text",
   CellTags->"Gstrong"],
  Cell[1138560, 46050, 80, 2, 26, "Input",
   CellTags->"Gstrong"],
  Cell[1138643, 46054, 125, 4, 26, "Output",
   CellTags->"Gstrong"]},
 "GTI"->{
  Cell[1138795, 46062, 43, 1, 39, "ObjectName",
   CellTags->"GTI"],
  Cell[1138863, 46067, 54, 1, 28, "Subsubsection",
   CellTags->"GTI"],
  Cell[1138920, 46070, 85, 1, 22, "Text",
   CellTags->"GTI"],
  Cell[1139008, 46073, 236, 11, 22, "Text",
   CellTags->"GTI"]},
 "GV"->{
  Cell[1139259, 46087, 41, 1, 39, "ObjectName",
   CellTags->"GV"],
  Cell[1139325, 46092, 53, 1, 28, "Subsubsection",
   CellTags->"GV"],
  Cell[1139381, 46095, 65, 1, 22, "Text",
   CellTags->"GV"],
  Cell[1139449, 46098, 267, 11, 22, "Text",
   CellTags->"GV"]},
 "Hill"->{
  Cell[1139731, 46112, 45, 1, 39, "ObjectName",
   CellTags->"Hill"],
  Cell[1139801, 46117, 55, 1, 36, "Subsubsection",
   CellTags->"Hill"],
  Cell[1139859, 46120, 129, 4, 24, "Text",
   CellTags->"Hill"],
  Cell[1139991, 46126, 285, 11, 24, "Text",
   CellTags->"Hill"],
  Cell[1140313, 46142, 52, 1, 36, "Subsubsection",
   CellTags->"Hill"],
  Cell[1140390, 46147, 124, 4, 26, "Input",
   CellTags->"Hill"],
  Cell[1140517, 46153, 2145, 73, 132, "Output",
   CellTags->"Hill"],
  Cell[1142699, 46231, 270, 8, 26, "Input",
   CellTags->"Hill"],
  Cell[1142972, 46241, 100, 3, 26, "Output",
   CellTags->"Hill"],
  Cell[1143109, 46249, 262, 9, 26, "Input",
   CellTags->"Hill"],
  Cell[1143374, 46260, 985, 32, 45, "Output",
   CellTags->"Hill"],
  Cell[1144396, 46297, 273, 9, 26, "Input",
   CellTags->"Hill"],
  Cell[1144672, 46308, 100, 3, 26, "Output",
   CellTags->"Hill"]},
 "HypergeometricAC"->{
  Cell[1144799, 46315, 69, 1, 39, "ObjectName",
   CellTags->"HypergeometricAC"],
  Cell[1144893, 46320, 67, 1, 36, "Subsubsection",
   CellTags->"HypergeometricAC"],
  Cell[1144963, 46323, 553, 9, 75, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1145541, 46336, 131, 3, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1145675, 46341, 177, 6, 26, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1145867, 46350, 953, 32, 24, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1146857, 46387, 64, 1, 36, "Subsubsection",
   CellTags->"HypergeometricAC"],
  Cell[1146924, 46390, 101, 1, 24, "Text",
   CellTags->"HypergeometricAC"],
  Cell[1147050, 46395, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1147317, 46404, 1411, 43, 47, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1148765, 46452, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1149032, 46461, 1375, 43, 52, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1150444, 46509, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1150711, 46518, 1416, 44, 52, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1152164, 46567, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1152431, 46576, 1695, 51, 100, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1154163, 46632, 264, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1154430, 46641, 415, 13, 40, "Output",
   CellTags->"HypergeometricAC"],
  Cell[1154882, 46659, 265, 7, 26, "Input",
   CellTags->"HypergeometricAC"],
  Cell[1155150, 46668, 416, 13, 40, "Output",
   CellTags->"HypergeometricAC"]},
 "HypergeometricIR"->{
  Cell[1155593, 46685, 69, 1, 39, "ObjectName",
   CellTags->"HypergeometricIR"],
  Cell[1155687, 46690, 67, 1, 36, "Subsubsection",
   CellTags->"HypergeometricIR"],
  Cell[1155757, 46693, 269, 5, 41, "Text",
   CellTags->"HypergeometricIR"],
  Cell[1156029, 46700, 749, 25, 24, "Text",
   CellTags->"HypergeometricIR"],
  Cell[1156803, 46729, 131, 3, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1156937, 46734, 182, 6, 26, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1157168, 46746, 64, 1, 36, "Subsubsection",
   CellTags->"HypergeometricIR"],
  Cell[1157257, 46751, 241, 7, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1157501, 46760, 640, 24, 48, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1158178, 46789, 394, 14, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1158575, 46805, 488, 15, 46, "Output",
   CellTags->"HypergeometricIR"],
  Cell[1159100, 46825, 148, 4, 26, "Input",
   CellTags->"HypergeometricIR"],
  Cell[1159251, 46831, 352, 13, 26, "Output",
   CellTags->"HypergeometricIR"]},
 "HypergeometricSE"->{
  Cell[1159630, 46848, 69, 1, 39, "ObjectName",
   CellTags->"HypergeometricSE"],
  Cell[1159724, 46853, 67, 1, 28, "Subsubsection",
   CellTags->"HypergeometricSE"],
  Cell[1159794, 46856, 420, 13, 41, "Text",
   CellTags->"HypergeometricSE"],
  Cell[1160239, 46873, 131, 3, 26, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1160373, 46878, 186, 6, 26, "Output",
   CellTags->"HypergeometricSE"],
  Cell[1160574, 46887, 301, 11, 22, "Text",
   CellTags->"HypergeometricSE"],
  Cell[1160912, 46903, 64, 1, 36, "Subsubsection",
   CellTags->"HypergeometricSE"],
  Cell[1161001, 46908, 245, 7, 26, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1161249, 46917, 585, 21, 46, "Output",
   CellTags->"HypergeometricSE"],
  Cell[1161871, 46943, 351, 11, 26, "Input",
   CellTags->"HypergeometricSE"],
  Cell[1162225, 46956, 809, 29, 49, "Output",
   CellTags->"HypergeometricSE"]},
 "HypExplicit"->{
  Cell[1163061, 46989, 59, 1, 39, "ObjectName",
   CellTags->"HypExplicit"],
  Cell[1163145, 46994, 62, 1, 28, "Subsubsection",
   CellTags->"HypExplicit"],
  Cell[1163210, 46997, 325, 10, 22, "Text",
   CellTags->"HypExplicit"],
  Cell[1163538, 47009, 296, 11, 22, "Text",
   CellTags->"HypExplicit"],
  Cell[1163871, 47025, 59, 1, 36, "Subsubsection",
   CellTags->"HypExplicit"],
  Cell[1163955, 47030, 164, 4, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1164122, 47036, 161, 5, 26, "Output",
   CellTags->"HypExplicit"],
  Cell[1164320, 47046, 142, 4, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1164465, 47052, 580, 21, 46, "Output",
   CellTags->"HypExplicit"],
  Cell[1165082, 47078, 262, 8, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1165347, 47088, 1495, 55, 26, "Output",
   CellTags->"HypExplicit"],
  Cell[1166879, 47148, 142, 4, 26, "Input",
   CellTags->"HypExplicit"],
  Cell[1167024, 47154, 804, 29, 49, "Output",
   CellTags->"HypExplicit"]},
 "HypInt"->{
  Cell[1167855, 47187, 49, 1, 39, "ObjectName",
   CellTags->"HypInt"],
  Cell[1167929, 47192, 57, 1, 28, "Subsubsection",
   CellTags->"HypInt"],
  Cell[1167989, 47195, 206, 5, 22, "Text",
   CellTags->"HypInt"],
  Cell[1168232, 47205, 54, 1, 36, "Subsubsection",
   CellTags->"HypInt"],
  Cell[1168311, 47210, 159, 4, 26, "Input",
   CellTags->"HypInt"],
  Cell[1168473, 47216, 156, 5, 26, "Output",
   CellTags->"HypInt"],
  Cell[1168666, 47226, 128, 4, 26, "Input",
   CellTags->"HypInt"],
  Cell[1168797, 47232, 945, 36, 53, "Output",
   CellTags->"HypInt"]},
 "IFPD"->{
  Cell[1169769, 47272, 45, 1, 39, "ObjectName",
   CellTags->"IFPD"],
  Cell[1169839, 47277, 55, 1, 28, "Subsubsection",
   CellTags->"IFPD"],
  Cell[1169897, 47280, 66, 1, 22, "Text",
   CellTags->"IFPD"],
  Cell[1169966, 47283, 309, 11, 22, "Text",
   CellTags->"IFPD"]},
 "IFPDOff"->{
  Cell[1170290, 47297, 51, 1, 39, "ObjectName",
   CellTags->"IFPDOff"],
  Cell[1170366, 47302, 58, 1, 28, "Subsubsection",
   CellTags->"IFPDOff"],
  Cell[1170427, 47305, 182, 4, 22, "Text",
   CellTags->"IFPDOff"],
  Cell[1170612, 47311, 428, 18, 22, "Text",
   CellTags->"IFPDOff"],
  Cell[1171077, 47334, 55, 1, 36, "Subsubsection",
   CellTags->"IFPDOff"],
  Cell[1171157, 47339, 163, 5, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1171323, 47346, 384, 16, 28, "Output",
   CellTags->"IFPDOff"],
  Cell[1171744, 47367, 108, 3, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1171855, 47372, 178, 5, 38, "Output",
   CellTags->"IFPDOff"],
  Cell[1172070, 47382, 130, 4, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1172203, 47388, 212, 8, 28, "Output",
   CellTags->"IFPDOff"],
  Cell[1172452, 47401, 108, 3, 26, "Input",
   CellTags->"IFPDOff"],
  Cell[1172563, 47406, 283, 9, 38, "Output",
   CellTags->"IFPDOff"]},
 "IFPDOn"->{
  Cell[1172873, 47419, 49, 1, 39, "ObjectName",
   CellTags->"IFPDOn"],
  Cell[1172947, 47424, 57, 1, 28, "Subsubsection",
   CellTags->"IFPDOn"],
  Cell[1173007, 47427, 317, 6, 37, "Text",
   CellTags->"IFPDOn"],
  Cell[1173327, 47435, 431, 18, 22, "Text",
   CellTags->"IFPDOn"]},
 "IncludePair"->{
  Cell[1173773, 47456, 59, 1, 39, "ObjectName",
   CellTags->"IncludePair"],
  Cell[1173857, 47461, 62, 1, 28, "Subsubsection",
   CellTags->"IncludePair"],
  Cell[1173922, 47464, 124, 3, 22, "Text",
   CellTags->"IncludePair"],
  Cell[1174049, 47469, 256, 11, 22, "Text",
   CellTags->"IncludePair"]},
 "IndexPosition"->{
  Cell[1174320, 47483, 63, 1, 39, "ObjectName",
   CellTags->"IndexPosition"],
  Cell[1174408, 47488, 64, 1, 28, "Subsubsection",
   CellTags->"IndexPosition"],
  Cell[1174475, 47491, 89, 1, 22, "Text",
   CellTags->"IndexPosition"],
  Cell[1174567, 47494, 286, 11, 22, "Text",
   CellTags->"IndexPosition"]},
 "Indices"->{
  Cell[1174868, 47508, 51, 1, 39, "ObjectName",
   CellTags->"Indices"],
  Cell[1174944, 47513, 58, 1, 28, "Subsubsection",
   CellTags->"Indices"],
  Cell[1175005, 47516, 203, 4, 22, "Text",
   CellTags->"Indices"],
  Cell[1175211, 47522, 280, 11, 24, "Text",
   CellTags->"Indices"]},
 "InitialFunction"->{
  Cell[1175506, 47536, 67, 1, 39, "ObjectName",
   CellTags->"InitialFunction"],
  Cell[1175598, 47541, 66, 1, 36, "Subsubsection",
   CellTags->"InitialFunction"],
  Cell[1175667, 47544, 183, 4, 24, "Text",
   CellTags->"InitialFunction"],
  Cell[1175853, 47550, 268, 11, 24, "Text",
   CellTags->"InitialFunction"]},
 "InitialSubstitutions"->{
  Cell[1176136, 47564, 77, 1, 39, "ObjectName",
   CellTags->"InitialSubstitutions"],
  Cell[1176238, 47569, 71, 1, 36, "Subsubsection",
   CellTags->"InitialSubstitutions"],
  Cell[1176312, 47572, 208, 4, 41, "Text",
   CellTags->"InitialSubstitutions"],
  Cell[1176523, 47578, 653, 25, 24, "Text",
   CellTags->"InitialSubstitutions"]},
 "InsideDiracTrace"->{
  Cell[1177191, 47606, 69, 1, 39, "ObjectName",
   CellTags->"InsideDiracTrace"],
  Cell[1177285, 47611, 67, 1, 36, "Subsubsection",
   CellTags->"InsideDiracTrace"],
  Cell[1177355, 47614, 498, 9, 58, "Text",
   CellTags->"InsideDiracTrace"],
  Cell[1177856, 47625, 489, 18, 24, "Text",
   CellTags->"InsideDiracTrace"]},
 "IntegralTable"->{
  Cell[1178360, 47646, 63, 1, 39, "ObjectName",
   CellTags->"IntegralTable"],
  Cell[1178448, 47651, 64, 1, 36, "Subsubsection",
   CellTags->"IntegralTable"],
  Cell[1178515, 47654, 233, 5, 41, "Text",
   CellTags->"IntegralTable"],
  Cell[1178751, 47661, 778, 25, 24, "Text",
   CellTags->"IntegralTable"]},
 "IntegrateByParts"->{
  Cell[1179544, 47689, 69, 1, 39, "ObjectName",
   CellTags->"IntegrateByParts"],
  Cell[1179638, 47694, 67, 1, 36, "Subsubsection",
   CellTags->"IntegrateByParts"],
  Cell[1179708, 47697, 184, 4, 24, "Text",
   CellTags->"IntegrateByParts"],
  Cell[1179917, 47705, 131, 3, 26, "Input",
   CellTags->"IntegrateByParts"],
  Cell[1180051, 47710, 172, 5, 26, "Output",
   CellTags->"IntegrateByParts"],
  Cell[1180272, 47721, 64, 1, 36, "Subsubsection",
   CellTags->"IntegrateByParts"],
  Cell[1180361, 47726, 539, 19, 26, "Input",
   CellTags->"IntegrateByParts"],
  Cell[1180903, 47747, 648, 23, 43, "Output",
   CellTags->"IntegrateByParts"]},
 "Integratedx"->{
  Cell[1181578, 47774, 59, 1, 39, "ObjectName",
   CellTags->"Integratedx"],
  Cell[1181662, 47779, 62, 1, 36, "Subsubsection",
   CellTags->"Integratedx"],
  Cell[1181727, 47782, 154, 4, 24, "Text",
   CellTags->"Integratedx"],
  Cell[1181884, 47788, 1384, 46, 24, "Text",
   CellTags->"Integratedx"]},
 "Integrate2"->{
  Cell[1183283, 47837, 57, 1, 39, "ObjectName",
   CellTags->"Integrate2"],
  Cell[1183365, 47842, 61, 1, 36, "Subsubsection",
   CellTags->"Integrate2"],
  Cell[1183429, 47845, 866, 17, 92, "Text",
   CellTags->"Integrate2"],
  Cell[1184298, 47864, 835, 32, 22, "Text",
   CellTags->"Integrate2"],
  Cell[1185136, 47898, 405, 7, 52, "Text",
   CellTags->"Integrate2"],
  Cell[1185578, 47910, 58, 1, 36, "Subsubsection",
   CellTags->"Integrate2"],
  Cell[1185661, 47915, 425, 15, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1186089, 47932, 368, 13, 44, "Output",
   CellTags->"Integrate2"],
  Cell[1186472, 47948, 190, 6, 24, "Text",
   CellTags->"Integrate2"],
  Cell[1186687, 47958, 281, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1186971, 47969, 230, 7, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1187238, 47981, 276, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1187517, 47992, 345, 12, 42, "Output",
   CellTags->"Integrate2"],
  Cell[1187899, 48009, 336, 11, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1188238, 48022, 575, 20, 45, "Output",
   CellTags->"Integrate2"],
  Cell[1188850, 48047, 317, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1189170, 48059, 134, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1189319, 48066, 1589, 53, 53, "Text",
   CellTags->"Integrate2"],
  Cell[1190933, 48123, 270, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1191206, 48134, 106, 3, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1191327, 48140, 649, 24, 44, "Text",
   CellTags->"Integrate2"],
  Cell[1192001, 48168, 317, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1192321, 48180, 106, 3, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1192464, 48188, 386, 13, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1192853, 48203, 164, 5, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1193054, 48213, 417, 14, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1193474, 48229, 179, 7, 42, "Output",
   CellTags->"Integrate2"],
  Cell[1193690, 48241, 328, 11, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1194021, 48254, 160, 6, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1194218, 48265, 354, 12, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1194575, 48279, 309, 11, 42, "Output",
   CellTags->"Integrate2"],
  Cell[1194921, 48295, 313, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1195237, 48307, 189, 6, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1195463, 48318, 281, 9, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1195747, 48329, 296, 11, 42, "Output",
   CellTags->"Integrate2"],
  Cell[1196080, 48345, 445, 15, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1196528, 48362, 487, 17, 46, "Output",
   CellTags->"Integrate2"],
  Cell[1197052, 48384, 721, 24, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1197776, 48410, 4351, 147, 139, "Output",
   CellTags->"Integrate2"],
  Cell[1202164, 48562, 471, 16, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1202638, 48580, 819, 29, 44, "Output",
   CellTags->"Integrate2"],
  Cell[1203494, 48614, 567, 19, 42, "Input",
   CellTags->"Integrate2"],
  Cell[1204064, 48635, 5252, 177, 146, "Output",
   CellTags->"Integrate2"],
  Cell[1209353, 48817, 142, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1209498, 48823, 1696, 53, 80, "Output",
   CellTags->"Integrate2"],
  Cell[1211231, 48881, 104, 3, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1211338, 48886, 125, 3, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1212059, 48916, 684, 23, 42, "Input",
   CellTags->"Integrate2"],
  Cell[1212746, 48941, 3678, 139, 125, "Output",
   CellTags->"Integrate2"],
  Cell[1218070, 49153, 151, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1218224, 49159, 109, 3, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1218370, 49167, 368, 12, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1218741, 49181, 106, 3, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1218862, 49187, 137, 4, 24, "Text",
   CellTags->"Integrate2"],
  Cell[1219024, 49195, 147, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1219174, 49201, 5377, 174, 230, "Output",
   CellTags->"Integrate2"],
  Cell[1224588, 49380, 230, 8, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1224821, 49390, 175, 6, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1225011, 49399, 165, 4, 24, "Text",
   CellTags->"Integrate2"],
  Cell[1225201, 49407, 326, 11, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1225530, 49420, 176, 6, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1225721, 49429, 126, 4, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1225872, 49437, 317, 10, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1226192, 49449, 134, 4, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1226363, 49458, 325, 11, 29, "Input",
   CellTags->"Integrate2"],
  Cell[1226691, 49471, 240, 8, 43, "Output",
   CellTags->"Integrate2"],
  Cell[1226968, 49484, 99, 3, 26, "Input",
   CellTags->"Integrate2"],
  Cell[1227070, 49489, 125, 3, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1227232, 49497, 325, 11, 29, "Input",
   CellTags->"Integrate2"],
  Cell[1227560, 49510, 125, 3, 26, "Output",
   CellTags->"Integrate2"],
  Cell[1227722, 49518, 381, 13, 29, "Input",
   CellTags->"Integrate2"],
  Cell[1228106, 49533, 4862, 184, 139, "Output",
   CellTags->"Integrate2"]},
 "Integrate3"->{
  Cell[1234062, 49770, 57, 1, 39, "ObjectName",
   CellTags->"Integrate3"],
  Cell[1234144, 49775, 61, 1, 28, "Subsubsection",
   CellTags->"Integrate3"],
  Cell[1234208, 49778, 331, 6, 37, "Text",
   CellTags->"Integrate3"],
  Cell[1234542, 49786, 271, 11, 24, "Text",
   CellTags->"Integrate3"],
  Cell[1234850, 49802, 58, 1, 36, "Subsubsection",
   CellTags->"Integrate3"],
  Cell[1234933, 49807, 280, 9, 29, "Input",
   CellTags->"Integrate3"],
  Cell[1235216, 49818, 225, 8, 47, "Output",
   CellTags->"Integrate3"],
  Cell[1235478, 49831, 400, 13, 44, "Input",
   CellTags->"Integrate3"],
  Cell[1235881, 49846, 513, 20, 26, "Output",
   CellTags->"Integrate3"],
  Cell[1236431, 49871, 676, 23, 44, "Input",
   CellTags->"Integrate3"],
  Cell[1237110, 49896, 2035, 73, 68, "Output",
   CellTags->"Integrate3"],
  Cell[1239182, 49974, 266, 8, 26, "Input",
   CellTags->"Integrate3"],
  Cell[1239451, 49984, 106, 3, 26, "Output",
   CellTags->"Integrate3"],
  Cell[1239594, 49992, 322, 10, 26, "Input",
   CellTags->"Integrate3"],
  Cell[1239919, 50004, 195, 6, 26, "Output",
   CellTags->"Integrate3"],
  Cell[1240151, 50015, 270, 9, 26, "Input",
   CellTags->"Integrate3"],
  Cell[1240424, 50026, 106, 3, 26, "Output",
   CellTags->"Integrate3"]},
 "IntermediateSubstitutions"->{
  Cell[1240557, 50033, 87, 1, 39, "ObjectName",
   CellTags->"IntermediateSubstitutions"],
  Cell[1240669, 50038, 76, 1, 36, "Subsubsection",
   CellTags->"IntermediateSubstitutions"],
  Cell[1240748, 50041, 206, 4, 24, "Text",
   CellTags->"IntermediateSubstitutions"],
  Cell[1240957, 50047, 274, 11, 24, "Text",
   CellTags->"IntermediateSubstitutions"]},
 "InverseMellin"->{
  Cell[1241246, 50061, 63, 1, 39, "ObjectName",
   CellTags->"InverseMellin"],
  Cell[1241334, 50066, 64, 1, 36, "Subsubsection",
   CellTags->"InverseMellin"],
  Cell[1241401, 50069, 531, 14, 58, "Text",
   CellTags->"InverseMellin"],
  Cell[1241935, 50085, 936, 37, 24, "Text",
   CellTags->"InverseMellin"],
  Cell[1242908, 50127, 61, 1, 36, "Subsubsection",
   CellTags->"InverseMellin"],
  Cell[1242994, 50132, 169, 5, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1243166, 50139, 157, 5, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1243360, 50149, 220, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1243583, 50158, 157, 5, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1243777, 50168, 142, 4, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1243922, 50174, 280, 10, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1244239, 50189, 194, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1244436, 50197, 293, 11, 42, "Output",
   CellTags->"InverseMellin"],
  Cell[1244766, 50213, 194, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1244963, 50221, 360, 13, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1245360, 50239, 204, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1245567, 50248, 361, 13, 26, "Output",
   CellTags->"InverseMellin"],
  Cell[1245943, 50264, 113, 3, 24, "Text",
   CellTags->"InverseMellin"],
  Cell[1246081, 50271, 294, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1246378, 50282, 131, 4, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1246524, 50289, 117, 3, 24, "Text",
   CellTags->"InverseMellin"],
  Cell[1246644, 50294, 3132, 100, 166, "Input",
   CellTags->"InverseMellin"],
  Cell[1249779, 50396, 244, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1250048, 50407, 179, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1250230, 50415, 398, 14, 43, "Output",
   CellTags->"InverseMellin"],
  Cell[1250665, 50434, 179, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1250847, 50442, 356, 12, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1251240, 50459, 172, 5, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1251415, 50466, 726, 25, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1252156, 50494, 81, 2, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1252262, 50500, 192, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1252457, 50508, 529, 20, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1253023, 50533, 270, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1253296, 50544, 502, 18, 42, "Output",
   CellTags->"InverseMellin"],
  Cell[1253835, 50567, 270, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1254108, 50578, 695, 25, 42, "Output",
   CellTags->"InverseMellin"],
  Cell[1254840, 50608, 267, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1255110, 50619, 693, 25, 43, "Output",
   CellTags->"InverseMellin"],
  Cell[1255840, 50649, 246, 9, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1256089, 50660, 549, 20, 44, "Output",
   CellTags->"InverseMellin"],
  Cell[1256675, 50685, 274, 10, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1256952, 50697, 914, 31, 49, "Output",
   CellTags->"InverseMellin"],
  Cell[1257903, 50733, 192, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1258098, 50741, 700, 25, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1258835, 50771, 221, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1259059, 50780, 687, 24, 43, "Output",
   CellTags->"InverseMellin"],
  Cell[1259783, 50809, 192, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1259978, 50817, 793, 29, 48, "Output",
   CellTags->"InverseMellin"],
  Cell[1260808, 50851, 202, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1261013, 50859, 544, 20, 47, "Output",
   CellTags->"InverseMellin"],
  Cell[1261594, 50884, 223, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1261820, 50893, 931, 33, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1262788, 50931, 223, 7, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1263014, 50940, 921, 33, 46, "Output",
   CellTags->"InverseMellin"],
  Cell[1263972, 50978, 212, 6, 26, "Input",
   CellTags->"InverseMellin"],
  Cell[1264187, 50986, 632, 23, 49, "Output",
   CellTags->"InverseMellin"],
  Cell[1264834, 51012, 158, 5, 26, "Input",
   CellTags->"InverseMellin"]},
 "Isolate"->{
  Cell[1265007, 51020, 51, 1, 39, "ObjectName",
   CellTags->"Isolate"],
  Cell[1265083, 51025, 58, 1, 36, "Subsubsection",
   CellTags->"Isolate"],
  Cell[1265144, 51028, 422, 7, 58, "Text",
   CellTags->"Isolate"],
  Cell[1265591, 51039, 114, 3, 26, "Input",
   CellTags->"Isolate"],
  Cell[1265708, 51044, 404, 12, 47, "Output",
   CellTags->"Isolate"],
  Cell[1266161, 51062, 55, 1, 36, "Subsubsection",
   CellTags->"Isolate"],
  Cell[1266241, 51067, 156, 5, 26, "Input",
   CellTags->"Isolate"],
  Cell[1266400, 51074, 160, 6, 26, "Output",
   CellTags->"Isolate"],
  Cell[1266597, 51085, 378, 13, 26, "Input",
   CellTags->"Isolate"],
  Cell[1266978, 51100, 309, 12, 26, "Output",
   CellTags->"Isolate"],
  Cell[1267324, 51117, 114, 3, 26, "Input",
   CellTags->"Isolate"],
  Cell[1267441, 51122, 286, 11, 38, "Output",
   CellTags->"Isolate"],
  Cell[1267764, 51138, 182, 5, 26, "Input",
   CellTags->"Isolate"],
  Cell[1267949, 51145, 646, 25, 26, "Output",
   CellTags->"Isolate"],
  Cell[1268632, 51175, 561, 19, 26, "Input",
   CellTags->"Isolate"],
  Cell[1269196, 51196, 359, 14, 26, "Output",
   CellTags->"Isolate"],
  Cell[1269592, 51215, 94, 3, 26, "Input",
   CellTags->"Isolate"],
  Cell[1269689, 51220, 90, 1, 40, "Print",
   CellTags->{"Info3535453244-5096475", "Isolate"}],
  Cell[1269782, 51223, 1031, 30, 52, "Print",
   CellTags->{"Info3535453244-5096475", "Isolate"}],
  Cell[1270850, 51258, 278, 7, 26, "Input",
   CellTags->"Isolate"],
  Cell[1271131, 51267, 159, 6, 26, "Output",
   CellTags->"Isolate"],
  Cell[1271327, 51278, 178, 6, 26, "Input",
   CellTags->"Isolate"],
  Cell[1271508, 51286, 211, 7, 26, "Output",
   CellTags->"Isolate"],
  Cell[1271734, 51296, 152, 4, 26, "Input",
   CellTags->"Isolate"]},
 "Info3535453244-5096475"->{
  Cell[1269689, 51220, 90, 1, 40, "Print",
   CellTags->{"Info3535453244-5096475", "Isolate"}],
  Cell[1269782, 51223, 1031, 30, 52, "Print",
   CellTags->{"Info3535453244-5096475", "Isolate"}]},
 "IsolateHead"->{
  Cell[1271901, 51303, 59, 1, 39, "ObjectName",
   CellTags->"IsolateHead"],
  Cell[1271985, 51308, 62, 1, 36, "Subsubsection",
   CellTags->"IsolateHead"],
  Cell[1272050, 51311, 84, 1, 24, "Text",
   CellTags->"IsolateHead"],
  Cell[1272137, 51314, 280, 11, 24, "Text",
   CellTags->"IsolateHead"]},
 "IsolateNames"->{
  Cell[1272432, 51328, 61, 1, 39, "ObjectName",
   CellTags->"IsolateNames"],
  Cell[1272518, 51333, 63, 1, 36, "Subsubsection",
   CellTags->"IsolateNames"],
  Cell[1272584, 51336, 220, 5, 41, "Text",
   CellTags->"IsolateNames"],
  Cell[1272807, 51343, 453, 18, 24, "Text",
   CellTags->"IsolateNames"]},
 "IsolatePrint"->{
  Cell[1273275, 51364, 61, 1, 39, "ObjectName",
   CellTags->"IsolatePrint"],
  Cell[1273361, 51369, 63, 1, 36, "Subsubsection",
   CellTags->"IsolatePrint"],
  Cell[1273427, 51372, 284, 6, 41, "Text",
   CellTags->"IsolatePrint"],
  Cell[1273714, 51380, 261, 11, 24, "Text",
   CellTags->"IsolatePrint"]},
 "IsolateSplit"->{
  Cell[1273990, 51394, 61, 1, 39, "ObjectName",
   CellTags->"IsolateSplit"],
  Cell[1274076, 51399, 63, 1, 36, "Subsubsection",
   CellTags->"IsolateSplit"],
  Cell[1274142, 51402, 413, 7, 58, "Text",
   CellTags->"IsolateSplit"],
  Cell[1274558, 51411, 261, 11, 24, "Text",
   CellTags->"IsolateSplit"]},
 "KeepOnly"->{
  Cell[1274834, 51425, 53, 1, 39, "ObjectName",
   CellTags->"KeepOnly"],
  Cell[1274912, 51430, 59, 1, 36, "Subsubsection",
   CellTags->"KeepOnly"],
  Cell[1274974, 51433, 293, 6, 41, "Text",
   CellTags->"KeepOnly"],
  Cell[1275270, 51441, 761, 32, 24, "Text",
   CellTags->"KeepOnly"]},
 "KK"->{
  Cell[1276046, 51476, 41, 1, 39, "ObjectName",
   CellTags->"KK"],
  Cell[1276112, 51481, 53, 1, 36, "Subsubsection",
   CellTags->"KK"],
  Cell[1276168, 51484, 235, 5, 41, "Text",
   CellTags->"KK"],
  Cell[1276406, 51491, 459, 18, 24, "Text",
   CellTags->"KK"]},
 "Kummer"->{
  Cell[1276880, 51512, 49, 1, 39, "ObjectName",
   CellTags->"Kummer"],
  Cell[1276954, 51517, 57, 1, 36, "Subsubsection",
   CellTags->"Kummer"],
  Cell[1277014, 51520, 260, 5, 41, "Text",
   CellTags->"Kummer"],
  Cell[1277277, 51527, 291, 11, 24, "Text",
   CellTags->"Kummer"],
  Cell[1277605, 51543, 54, 1, 36, "Subsubsection",
   CellTags->"Kummer"],
  Cell[1277684, 51548, 337, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1278024, 51559, 452, 14, 26, "Output",
   CellTags->"Kummer"],
  Cell[1278513, 51578, 337, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1278853, 51589, 467, 15, 40, "Output",
   CellTags->"Kummer"],
  Cell[1279357, 51609, 337, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1279697, 51620, 467, 15, 40, "Output",
   CellTags->"Kummer"],
  Cell[1280201, 51640, 388, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1280592, 51653, 452, 14, 26, "Output",
   CellTags->"Kummer"],
  Cell[1281081, 51672, 479, 13, 26, "Input",
   CellTags->"Kummer"],
  Cell[1281563, 51687, 531, 15, 26, "Output",
   CellTags->"Kummer"],
  Cell[1282131, 51707, 388, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1282522, 51720, 434, 13, 43, "Output",
   CellTags->"Kummer"],
  Cell[1282993, 51738, 479, 13, 26, "Input",
   CellTags->"Kummer"],
  Cell[1283475, 51753, 548, 17, 43, "Output",
   CellTags->"Kummer"],
  Cell[1284060, 51775, 388, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1284451, 51788, 434, 13, 43, "Output",
   CellTags->"Kummer"],
  Cell[1284922, 51806, 479, 13, 26, "Input",
   CellTags->"Kummer"],
  Cell[1285404, 51821, 548, 17, 43, "Output",
   CellTags->"Kummer"],
  Cell[1285989, 51843, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1286479, 51860, 621, 20, 43, "Output",
   CellTags->"Kummer"],
  Cell[1287137, 51885, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1287769, 51906, 722, 22, 43, "Output",
   CellTags->"Kummer"],
  Cell[1288528, 51933, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1289018, 51950, 593, 19, 44, "Output",
   CellTags->"Kummer"],
  Cell[1289648, 51974, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1290280, 51995, 707, 23, 44, "Output",
   CellTags->"Kummer"],
  Cell[1291024, 52023, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1291514, 52040, 569, 18, 43, "Output",
   CellTags->"Kummer"],
  Cell[1292120, 52063, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1292752, 52084, 750, 23, 43, "Output",
   CellTags->"Kummer"],
  Cell[1293539, 52112, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1294029, 52129, 621, 20, 43, "Output",
   CellTags->"Kummer"],
  Cell[1294687, 52154, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1295319, 52175, 745, 20, 43, "Output",
   CellTags->"Kummer"],
  Cell[1296101, 52200, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1296591, 52217, 543, 17, 43, "Output",
   CellTags->"Kummer"],
  Cell[1297171, 52239, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1297803, 52260, 789, 22, 43, "Output",
   CellTags->"Kummer"],
  Cell[1298629, 52287, 487, 15, 26, "Input",
   CellTags->"Kummer"],
  Cell[1299119, 52304, 543, 17, 43, "Output",
   CellTags->"Kummer"],
  Cell[1299699, 52326, 629, 19, 46, "Input",
   CellTags->"Kummer"],
  Cell[1300331, 52347, 680, 19, 43, "Output",
   CellTags->"Kummer"],
  Cell[1301048, 52371, 531, 15, 46, "Input",
   CellTags->"Kummer"],
  Cell[1301582, 52388, 576, 18, 26, "Output",
   CellTags->"Kummer"],
  Cell[1302195, 52411, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1302536, 52422, 452, 14, 26, "Output",
   CellTags->"Kummer"],
  Cell[1303025, 52441, 531, 15, 46, "Input",
   CellTags->"Kummer"],
  Cell[1303559, 52458, 632, 19, 40, "Output",
   CellTags->"Kummer"],
  Cell[1304228, 52482, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1304569, 52493, 441, 14, 40, "Output",
   CellTags->"Kummer"],
  Cell[1305047, 52512, 531, 15, 46, "Input",
   CellTags->"Kummer"],
  Cell[1305581, 52529, 632, 19, 40, "Output",
   CellTags->"Kummer"],
  Cell[1306250, 52553, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1306591, 52564, 441, 14, 40, "Output",
   CellTags->"Kummer"],
  Cell[1307069, 52583, 582, 17, 46, "Input",
   CellTags->"Kummer"],
  Cell[1307654, 52602, 587, 15, 26, "Output",
   CellTags->"Kummer"],
  Cell[1308278, 52622, 389, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1308670, 52635, 452, 14, 26, "Output",
   CellTags->"Kummer"],
  Cell[1309159, 52654, 582, 17, 46, "Input",
   CellTags->"Kummer"],
  Cell[1309744, 52673, 690, 18, 43, "Output",
   CellTags->"Kummer"],
  Cell[1310471, 52696, 389, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1310863, 52709, 492, 15, 43, "Output",
   CellTags->"Kummer"],
  Cell[1311392, 52729, 582, 17, 46, "Input",
   CellTags->"Kummer"],
  Cell[1311977, 52748, 632, 16, 43, "Output",
   CellTags->"Kummer"],
  Cell[1312646, 52769, 389, 11, 26, "Input",
   CellTags->"Kummer"],
  Cell[1313038, 52782, 434, 13, 43, "Output",
   CellTags->"Kummer"],
  Cell[1313509, 52800, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1313850, 52811, 1310, 43, 71, "Output",
   CellTags->"Kummer"],
  Cell[1315197, 52859, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1315538, 52870, 1370, 46, 52, "Output",
   CellTags->"Kummer"],
  Cell[1316945, 52921, 338, 9, 26, "Input",
   CellTags->"Kummer"],
  Cell[1317286, 52932, 1396, 46, 52, "Output",
   CellTags->"Kummer"]},
 "Lagrangian"->{
  Cell[1318709, 52982, 57, 1, 39, "ObjectName",
   CellTags->"Lagrangian"],
  Cell[1318791, 52987, 61, 1, 36, "Subsubsection",
   CellTags->"Lagrangian"],
  Cell[1318855, 52990, 454, 9, 58, "Text",
   CellTags->"Lagrangian"],
  Cell[1319312, 53001, 263, 11, 24, "Text",
   CellTags->"Lagrangian"],
  Cell[1319612, 53017, 58, 1, 36, "Subsubsection",
   CellTags->"Lagrangian"],
  Cell[1319695, 53022, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1319822, 53027, 332, 11, 42, "Output",
   CellTags->"Lagrangian"],
  Cell[1320169, 53041, 85, 1, 24, "Text",
   CellTags->"Lagrangian"],
  Cell[1320279, 53046, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1320406, 53051, 683, 23, 42, "Output",
   CellTags->"Lagrangian"],
  Cell[1321126, 53079, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1321253, 53084, 1006, 36, 42, "Output",
   CellTags->"Lagrangian"],
  Cell[1322296, 53125, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1322423, 53130, 621, 22, 26, "Output",
   CellTags->"Lagrangian"],
  Cell[1323081, 53157, 124, 3, 26, "Input",
   CellTags->"Lagrangian"],
  Cell[1323208, 53162, 664, 23, 26, "Output",
   CellTags->"Lagrangian"]},
 "LC"->{
  Cell[1323899, 53189, 41, 1, 39, "ObjectName",
   CellTags->"LC"],
  Cell[1323965, 53194, 53, 1, 36, "Subsubsection",
   CellTags->"LC"],
  Cell[1324021, 53197, 240, 5, 41, "Text",
   CellTags->"LC"],
  Cell[1324264, 53204, 435, 18, 24, "Text",
   CellTags->"LC"],
  Cell[1324736, 53227, 50, 1, 36, "Subsubsection",
   CellTags->"LC"],
  Cell[1324811, 53232, 164, 5, 26, "Input",
   CellTags->"LC"],
  Cell[1324978, 53239, 184, 6, 26, "Output",
   CellTags->"LC"],
  Cell[1325199, 53250, 95, 3, 26, "Input",
   CellTags->"LC"],
  Cell[1325297, 53255, 477, 21, 26, "Output",
   CellTags->"LC"],
  Cell[1325811, 53281, 104, 3, 26, "Input",
   CellTags->"LC"],
  Cell[1325918, 53286, 337, 8, 38, "Output",
   CellTags->"LC"],
  Cell[1326292, 53299, 181, 6, 26, "Input",
   CellTags->"LC"],
  Cell[1326476, 53307, 169, 5, 26, "Output",
   CellTags->"LC"],
  Cell[1326682, 53317, 130, 4, 26, "Input",
   CellTags->"LC"],
  Cell[1326815, 53323, 367, 10, 38, "Output",
   CellTags->"LC"],
  Cell[1327219, 53338, 381, 13, 26, "Input",
   CellTags->"LC"],
  Cell[1327603, 53353, 396, 16, 28, "Output",
   CellTags->"LC"]},
 "LCD"->{
  Cell[1328026, 53373, 43, 1, 39, "ObjectName",
   CellTags->"LCD"],
  Cell[1328094, 53378, 54, 1, 36, "Subsubsection",
   CellTags->"LCD"],
  Cell[1328151, 53381, 234, 5, 41, "Text",
   CellTags->"LCD"],
  Cell[1328388, 53388, 161, 4, 24, "Text",
   CellTags->"LCD"],
  Cell[1328552, 53394, 432, 18, 24, "Text",
   CellTags->"LCD"],
  Cell[1329021, 53417, 51, 1, 36, "Subsubsection",
   CellTags->"LCD"],
  Cell[1329075, 53420, 190, 6, 26, "Input",
   CellTags->"LCD"],
  Cell[1329290, 53430, 166, 5, 26, "Input",
   CellTags->"LCD"],
  Cell[1329459, 53437, 185, 6, 26, "Output",
   CellTags->"LCD"],
  Cell[1329681, 53448, 96, 3, 26, "Input",
   CellTags->"LCD"],
  Cell[1329780, 53453, 478, 21, 26, "Output",
   CellTags->"LCD"],
  Cell[1330295, 53479, 105, 3, 26, "Input",
   CellTags->"LCD"],
  Cell[1330403, 53484, 438, 12, 38, "Output",
   CellTags->"LCD"],
  Cell[1330878, 53501, 183, 6, 26, "Input",
   CellTags->"LCD"],
  Cell[1331064, 53509, 170, 5, 26, "Output",
   CellTags->"LCD"],
  Cell[1331271, 53519, 131, 4, 26, "Input",
   CellTags->"LCD"],
  Cell[1331405, 53525, 418, 12, 38, "Output",
   CellTags->"LCD"],
  Cell[1331860, 53542, 400, 13, 26, "Input",
   CellTags->"LCD"],
  Cell[1332263, 53557, 397, 15, 28, "Output",
   CellTags->"LCD"],
  Cell[1332697, 53577, 170, 5, 26, "Input",
   CellTags->"LCD"],
  Cell[1332870, 53584, 161, 5, 26, "Output",
   CellTags->"LCD"]},
 "LeftPartialD"->{
  Cell[1333058, 53593, 61, 1, 39, "ObjectName",
   CellTags->"LeftPartialD"],
  Cell[1333144, 53598, 63, 1, 28, "Subsubsection",
   CellTags->"LeftPartialD"],
  Cell[1333210, 53601, 307, 12, 30, "Text",
   CellTags->"LeftPartialD"],
  Cell[1333520, 53615, 921, 32, 22, "Text",
   CellTags->"LeftPartialD"],
  Cell[1334478, 53652, 69, 1, 33, "Subsubsection",
   CellTags->"LeftPartialD"],
  Cell[1334572, 53657, 253, 7, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1334828, 53666, 306, 11, 33, "Output",
   CellTags->"LeftPartialD"],
  Cell[1335171, 53682, 120, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1335294, 53687, 355, 14, 30, "Output",
   CellTags->"LeftPartialD"],
  Cell[1335686, 53706, 118, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1335807, 53711, 291, 7, 38, "Output",
   CellTags->"LeftPartialD"],
  Cell[1336135, 53723, 161, 4, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1336299, 53729, 176, 4, 38, "Output",
   CellTags->"LeftPartialD"],
  Cell[1336512, 53738, 372, 10, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1336887, 53750, 344, 12, 33, "Output",
   CellTags->"LeftPartialD"],
  Cell[1337268, 53767, 120, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1337391, 53772, 774, 30, 30, "Output",
   CellTags->"LeftPartialD"],
  Cell[1338202, 53807, 118, 3, 26, "Input",
   CellTags->"LeftPartialD"],
  Cell[1338323, 53812, 799, 21, 58, "Output",
   CellTags->"LeftPartialD"]},
 "LeftRightPartialD"->{
  Cell[1339149, 53837, 71, 1, 39, "ObjectName",
   CellTags->"LeftRightPartialD"],
  Cell[1339245, 53842, 68, 1, 28, "Subsubsection",
   CellTags->"LeftRightPartialD"],
  Cell[1339316, 53845, 592, 21, 30, "Text",
   CellTags->"LeftRightPartialD"],
  Cell[1339911, 53868, 1356, 45, 22, "Text",
   CellTags->"LeftRightPartialD"],
  Cell[1341304, 53918, 65, 1, 36, "Subsubsection",
   CellTags->"LeftRightPartialD"],
  Cell[1341394, 53923, 132, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1341529, 53928, 260, 9, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1341826, 53942, 127, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1341956, 53947, 527, 19, 42, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1342520, 53971, 263, 7, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1342786, 53980, 316, 11, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1343139, 53996, 125, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1343267, 54001, 708, 26, 42, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1344012, 54032, 382, 10, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1344397, 54044, 354, 12, 33, "Output",
   CellTags->"LeftRightPartialD"],
  Cell[1344788, 54061, 125, 3, 26, "Input",
   CellTags->"LeftRightPartialD"],
  Cell[1344916, 54066, 902, 33, 42, "Output",
   CellTags->"LeftRightPartialD"]},
 "LeftRightPartialD2"->{
  Cell[1345845, 54103, 73, 1, 39, "ObjectName",
   CellTags->"LeftRightPartialD2"],
  Cell[1345943, 54108, 69, 1, 28, "Subsubsection",
   CellTags->"LeftRightPartialD2"],
  Cell[1346015, 54111, 591, 21, 30, "Text",
   CellTags->"LeftRightPartialD2"],
  Cell[1346609, 54134, 1131, 39, 22, "Text",
   CellTags->"LeftRightPartialD2"],
  Cell[1347777, 54178, 66, 1, 36, "Subsubsection",
   CellTags->"LeftRightPartialD2"],
  Cell[1347868, 54183, 134, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1348005, 54188, 261, 9, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1348303, 54202, 128, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1348434, 54207, 433, 16, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1348904, 54228, 265, 7, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1349172, 54237, 317, 11, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1349526, 54253, 126, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1349655, 54258, 600, 23, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1350292, 54286, 384, 10, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1350679, 54298, 355, 12, 33, "Output",
   CellTags->"LeftRightPartialD2"],
  Cell[1351071, 54315, 126, 3, 26, "Input",
   CellTags->"LeftRightPartialD2"],
  Cell[1351200, 54320, 780, 30, 30, "Output",
   CellTags->"LeftRightPartialD2"]},
 "LeviCivita"->{
  Cell[1352007, 54354, 58, 1, 39, "ObjectName",
   CellTags->"LeviCivita"],
  Cell[1352090, 54359, 61, 1, 36, "Subsubsection",
   CellTags->"LeviCivita"],
  Cell[1352154, 54362, 1210, 40, 80, "Text",
   CellTags->"LeviCivita"],
  Cell[1353389, 54406, 120, 3, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1353512, 54411, 168, 5, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1353695, 54419, 775, 32, 24, "Text",
   CellTags->"LeviCivita"],
  Cell[1354507, 54456, 58, 1, 36, "Subsubsection",
   CellTags->"LeviCivita"],
  Cell[1354590, 54461, 187, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1354780, 54468, 492, 21, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1355309, 54494, 204, 6, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1355516, 54502, 411, 17, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1355964, 54524, 180, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1356147, 54531, 313, 13, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1356497, 54549, 186, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1356686, 54556, 107, 3, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1356830, 54564, 242, 7, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1357075, 54573, 332, 8, 38, "Output",
   CellTags->"LeviCivita"],
  Cell[1357444, 54586, 365, 10, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1357812, 54598, 932, 40, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1358781, 54643, 113, 3, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1358897, 54648, 126, 4, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1359038, 54655, 316, 9, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1359379, 54668, 403, 11, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1359785, 54681, 297, 10, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1360097, 54694, 316, 9, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1360438, 54707, 179, 5, 26, "Input",
   CellTags->"LeviCivita"],
  Cell[1360620, 54714, 199, 6, 26, "Output",
   CellTags->"LeviCivita"],
  Cell[1360834, 54723, 110, 3, 26, "Input",
   CellTags->"LeviCivita"]},
 "LeviCivitaSign"->{
  Cell[1360959, 54729, 65, 1, 39, "ObjectName",
   CellTags->"LeviCivitaSign"],
  Cell[1361049, 54734, 65, 1, 28, "Subsubsection",
   CellTags->"LeviCivitaSign"],
  Cell[1361117, 54737, 203, 4, 22, "Text",
   CellTags->"LeviCivitaSign"],
  Cell[1361323, 54743, 479, 18, 22, "Text",
   CellTags->"LeviCivitaSign"]},
 "ListIntegrals"->{
  Cell[1361817, 54764, 81, 1, 39, "ObjectName",
   CellTags->"ListIntegrals"],
  Cell[1361923, 54769, 64, 1, 28, "Subsubsection",
   CellTags->"ListIntegrals"],
  Cell[1361990, 54772, 470, 8, 52, "Text",
   CellTags->"ListIntegrals"],
  Cell[1362497, 54785, 61, 1, 36, "Subsubsection",
   CellTags->"ListIntegrals"],
  Cell[1362561, 54788, 80, 2, 26, "Input",
   CellTags->"ListIntegrals"]},
 "Li2"->{
  Cell[1362656, 54793, 44, 1, 39, "ObjectName",
   CellTags->"Li2"],
  Cell[1362725, 54798, 54, 1, 28, "Subsubsection",
   CellTags->"Li2"],
  Cell[1362782, 54801, 117, 3, 22, "Text",
   CellTags->"Li2"],
  Cell[1362902, 54806, 230, 10, 24, "Text",
   CellTags->"Li2"],
  Cell[1363169, 54821, 51, 1, 36, "Subsubsection",
   CellTags->"Li2"],
  Cell[1363245, 54826, 100, 3, 26, "Input",
   CellTags->"Li2"],
  Cell[1363348, 54831, 136, 5, 26, "Output",
   CellTags->"Li2"],
  Cell[1363521, 54841, 107, 3, 26, "Input",
   CellTags->"Li2"],
  Cell[1363631, 54846, 162, 5, 38, "Output",
   CellTags->"Li2"],
  Cell[1363830, 54856, 253, 9, 42, "Input",
   CellTags->"Li2"],
  Cell[1364086, 54867, 136, 5, 26, "Output",
   CellTags->"Li2"]},
 "Li3"->{
  Cell[1364249, 54876, 44, 1, 39, "ObjectName",
   CellTags->"Li3"],
  Cell[1364318, 54881, 54, 1, 36, "Subsubsection",
   CellTags->"Li3"],
  Cell[1364375, 54884, 118, 3, 24, "Text",
   CellTags->"Li3"],
  Cell[1364496, 54889, 230, 10, 24, "Text",
   CellTags->"Li3"],
  Cell[1364763, 54904, 51, 1, 36, "Subsubsection",
   CellTags->"Li3"],
  Cell[1364839, 54909, 100, 3, 26, "Input",
   CellTags->"Li3"],
  Cell[1364942, 54914, 136, 5, 26, "Output",
   CellTags->"Li3"],
  Cell[1365115, 54924, 107, 3, 26, "Input",
   CellTags->"Li3"],
  Cell[1365225, 54929, 162, 5, 38, "Output",
   CellTags->"Li3"],
  Cell[1365424, 54939, 152, 5, 26, "Input",
   CellTags->"Li3"],
  Cell[1365579, 54946, 159, 6, 44, "Output",
   CellTags->"Li3"],
  Cell[1365775, 54957, 204, 7, 44, "Input",
   CellTags->"Li3"],
  Cell[1365982, 54966, 136, 5, 26, "Output",
   CellTags->"Li3"]},
 "Loop"->{
  Cell[1366145, 54975, 45, 1, 39, "ObjectName",
   CellTags->"Loop"],
  Cell[1366215, 54980, 55, 1, 36, "Subsubsection",
   CellTags->"Loop"],
  Cell[1366273, 54983, 94, 1, 24, "Text",
   CellTags->"Loop"]},
 "LorentzIndex"->{
  Cell[1366382, 54987, 61, 1, 39, "ObjectName",
   CellTags->"LorentzIndex"],
  Cell[1366468, 54992, 63, 1, 36, "Subsubsection",
   CellTags->"LorentzIndex"],
  Cell[1366534, 54995, 243, 5, 41, "Text",
   CellTags->"LorentzIndex"],
  Cell[1366780, 55002, 485, 18, 24, "Text",
   CellTags->"LorentzIndex"],
  Cell[1367302, 55025, 60, 1, 34, "Subsubsection",
   CellTags->"LorentzIndex"],
  Cell[1367365, 55028, 89, 1, 24, "Text",
   CellTags->"LorentzIndex"],
  Cell[1367479, 55033, 125, 3, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1367607, 55038, 116, 3, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1367738, 55044, 125, 3, 24, "Text",
   CellTags->"LorentzIndex"],
  Cell[1367888, 55051, 148, 4, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1368039, 55057, 116, 3, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1368192, 55065, 106, 2, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1368301, 55069, 140, 4, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1368478, 55078, 165, 4, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1368646, 55084, 113, 3, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1368796, 55092, 207, 5, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1369006, 55099, 139, 4, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1369160, 55106, 141, 4, 24, "Text",
   CellTags->"LorentzIndex"],
  Cell[1369304, 55112, 137, 4, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1369466, 55120, 347, 10, 26, "Input",
   CellTags->"LorentzIndex"],
  Cell[1369816, 55132, 385, 16, 26, "Output",
   CellTags->"LorentzIndex"],
  Cell[1370216, 55151, 148, 4, 26, "Input",
   CellTags->"LorentzIndex"]},
 "Lower"->{
  Cell[1370379, 55158, 47, 1, 39, "ObjectName",
   CellTags->"Lower"],
  Cell[1370451, 55163, 56, 1, 28, "Subsubsection",
   CellTags->"Lower"],
  Cell[1370510, 55166, 120, 3, 22, "Text",
   CellTags->"Lower"],
  Cell[1370633, 55171, 650, 25, 22, "Text",
   CellTags->"Lower"]},
 "MakeContext"->{
  Cell[1371298, 55199, 59, 1, 39, "ObjectName",
   CellTags->"MakeContext"],
  Cell[1371382, 55204, 62, 1, 28, "Subsubsection",
   CellTags->"MakeContext"],
  Cell[1371447, 55207, 238, 5, 37, "Text",
   CellTags->"MakeContext"],
  Cell[1371688, 55214, 864, 32, 22, "Text",
   CellTags->"MakeContext"]},
 "Mandelstam"->{
  Cell[1372567, 55249, 57, 1, 39, "ObjectName",
   CellTags->"Mandelstam"],
  Cell[1372649, 55254, 61, 1, 28, "Subsubsection",
   CellTags->"Mandelstam"],
  Cell[1372713, 55257, 1575, 73, 37, "Text",
   CellTags->"Mandelstam"],
  Cell[1374291, 55332, 1259, 46, 22, "Text",
   CellTags->"Mandelstam"]},
 "Map2"->{
  Cell[1375565, 55381, 45, 1, 39, "ObjectName",
   CellTags->"Map2"],
  Cell[1375635, 55386, 55, 1, 28, "Subsubsection",
   CellTags->"Map2"],
  Cell[1375693, 55389, 135, 4, 22, "Text",
   CellTags->"Map2"],
  Cell[1375831, 55395, 249, 11, 22, "Text",
   CellTags->"Map2"],
  Cell[1376117, 55411, 52, 1, 36, "Subsubsection",
   CellTags->"Map2"],
  Cell[1376194, 55416, 149, 5, 26, "Input",
   CellTags->"Map2"],
  Cell[1376346, 55423, 202, 7, 26, "Output",
   CellTags->"Map2"],
  Cell[1376585, 55435, 125, 4, 26, "Input",
   CellTags->"Map2"],
  Cell[1376713, 55441, 129, 4, 26, "Output",
   CellTags->"Map2"],
  Cell[1376879, 55450, 184, 6, 26, "Input",
   CellTags->"Map2"],
  Cell[1377066, 55458, 191, 7, 26, "Output",
   CellTags->"Map2"],
  Cell[1377294, 55470, 125, 4, 26, "Input",
   CellTags->"Map2"],
  Cell[1377422, 55476, 129, 4, 26, "Output",
   CellTags->"Map2"]},
 "MemoryAvailable"->{
  Cell[1377578, 55484, 67, 1, 39, "ObjectName",
   CellTags->"MemoryAvailable"],
  Cell[1377670, 55489, 66, 1, 28, "Subsubsection",
   CellTags->"MemoryAvailable"],
  Cell[1377739, 55492, 230, 5, 37, "Text",
   CellTags->"MemoryAvailable"],
  Cell[1377972, 55499, 484, 18, 24, "Text",
   CellTags->"MemoryAvailable"]},
 "MemSet"->{
  Cell[1378471, 55520, 49, 1, 39, "ObjectName",
   CellTags->"MemSet"],
  Cell[1378545, 55525, 57, 1, 36, "Subsubsection",
   CellTags->"MemSet"],
  Cell[1378605, 55528, 345, 9, 41, "Text",
   CellTags->"MemSet"],
  Cell[1378953, 55539, 287, 11, 24, "Text",
   CellTags->"MemSet"]},
 "MetricTensor"->{
  Cell[1379255, 55553, 61, 1, 39, "ObjectName",
   CellTags->"MetricTensor"],
  Cell[1379341, 55558, 63, 1, 36, "Subsubsection",
   CellTags->"MetricTensor"],
  Cell[1379407, 55561, 214, 7, 24, "Text",
   CellTags->"MetricTensor"],
  Cell[1379624, 55570, 981, 39, 24, "Text",
   CellTags->"MetricTensor"],
  Cell[1380642, 55614, 60, 1, 36, "Subsubsection",
   CellTags->"MetricTensor"],
  Cell[1380727, 55619, 154, 4, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1380884, 55625, 165, 5, 26, "Output",
   CellTags->"MetricTensor"],
  Cell[1381086, 55635, 147, 4, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1381236, 55641, 109, 3, 26, "Output",
   CellTags->"MetricTensor"],
  Cell[1381382, 55649, 202, 5, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1381587, 55656, 165, 5, 26, "Output",
   CellTags->"MetricTensor"],
  Cell[1381789, 55666, 147, 4, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1381939, 55672, 109, 3, 27, "Output",
   CellTags->"MetricTensor"],
  Cell[1382085, 55680, 181, 5, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1382269, 55687, 156, 4, 38, "Output",
   CellTags->"MetricTensor"],
  Cell[1382462, 55696, 230, 6, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1382695, 55704, 204, 5, 38, "Output",
   CellTags->"MetricTensor"],
  Cell[1382936, 55714, 213, 6, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1383152, 55722, 156, 4, 38, "Output",
   CellTags->"MetricTensor"],
  Cell[1383345, 55731, 263, 7, 26, "Input",
   CellTags->"MetricTensor"],
  Cell[1383611, 55740, 204, 5, 38, "Output",
   CellTags->"MetricTensor"]},
 "MLimit"->{
  Cell[1383842, 55749, 49, 1, 39, "ObjectName",
   CellTags->"MLimit"],
  Cell[1383916, 55754, 57, 1, 36, "Subsubsection",
   CellTags->"MLimit"],
  Cell[1383976, 55757, 116, 3, 24, "Text",
   CellTags->"MLimit"],
  Cell[1384129, 55765, 54, 1, 36, "Subsubsection",
   CellTags->"MLimit"],
  Cell[1384208, 55770, 462, 16, 26, "Input",
   CellTags->"MLimit"],
  Cell[1384673, 55788, 103, 3, 26, "Output",
   CellTags->"MLimit"]},
 "Momentum"->{
  Cell[1384803, 55795, 53, 1, 39, "ObjectName",
   CellTags->"Momentum"],
  Cell[1384881, 55800, 59, 1, 36, "Subsubsection",
   CellTags->"Momentum"],
  Cell[1384943, 55803, 258, 5, 41, "Text",
   CellTags->"Momentum"],
  Cell[1385204, 55810, 865, 32, 24, "Text",
   CellTags->"Momentum"],
  Cell[1386106, 55847, 56, 1, 36, "Subsubsection",
   CellTags->"Momentum"],
  Cell[1386165, 55850, 75, 1, 24, "Text",
   CellTags->"Momentum"],
  Cell[1386265, 55855, 110, 3, 26, "Input",
   CellTags->"Momentum"],
  Cell[1386378, 55860, 105, 3, 29, "Output",
   CellTags->"Momentum"],
  Cell[1386498, 55866, 138, 4, 24, "Text",
   CellTags->"Momentum"],
  Cell[1386661, 55874, 133, 4, 26, "Input",
   CellTags->"Momentum"],
  Cell[1386797, 55880, 105, 3, 29, "Output",
   CellTags->"Momentum"],
  Cell[1386917, 55886, 86, 1, 24, "Text",
   CellTags->"Momentum"],
  Cell[1387028, 55891, 169, 5, 26, "Input",
   CellTags->"Momentum"],
  Cell[1387200, 55898, 148, 4, 38, "Output",
   CellTags->"Momentum"],
  Cell[1387385, 55907, 153, 5, 26, "Input",
   CellTags->"Momentum"],
  Cell[1387541, 55914, 123, 4, 28, "Output",
   CellTags->"Momentum"],
  Cell[1387701, 55923, 111, 3, 26, "Input",
   CellTags->"Momentum"],
  Cell[1387815, 55928, 143, 4, 38, "Output",
   CellTags->"Momentum"],
  Cell[1387995, 55937, 259, 8, 26, "Input",
   CellTags->"Momentum"],
  Cell[1388257, 55947, 201, 7, 28, "Output",
   CellTags->"Momentum"],
  Cell[1388495, 55959, 111, 3, 26, "Input",
   CellTags->"Momentum"],
  Cell[1388609, 55964, 231, 7, 38, "Output",
   CellTags->"Momentum"],
  Cell[1388877, 55976, 148, 4, 26, "Input",
   CellTags->"Momentum"],
  Cell[1389028, 55982, 183, 5, 38, "Output",
   CellTags->"Momentum"],
  Cell[1389248, 55992, 149, 4, 26, "Input",
   CellTags->"Momentum"],
  Cell[1389400, 55998, 231, 7, 38, "Output",
   CellTags->"Momentum"],
  Cell[1389668, 56010, 216, 7, 26, "Input",
   CellTags->"Momentum"],
  Cell[1389887, 56019, 148, 4, 38, "Output",
   CellTags->"Momentum"],
  Cell[1390050, 56026, 132, 4, 26, "Input",
   CellTags->"Momentum"]},
 "MomentumCombine"->{
  Cell[1390197, 56033, 67, 1, 39, "ObjectName",
   CellTags->"MomentumCombine"],
  Cell[1390289, 56038, 66, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine"],
  Cell[1390358, 56041, 183, 4, 24, "Text",
   CellTags->"MomentumCombine"],
  Cell[1390544, 56047, 938, 31, 24, "Text",
   CellTags->"MomentumCombine"],
  Cell[1391519, 56083, 63, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine"],
  Cell[1391607, 56088, 303, 9, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1391913, 56099, 214, 6, 38, "Output",
   CellTags->"MomentumCombine"],
  Cell[1392164, 56110, 288, 9, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1392455, 56121, 218, 7, 28, "Output",
   CellTags->"MomentumCombine"],
  Cell[1392710, 56133, 125, 3, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1392838, 56138, 218, 7, 28, "Output",
   CellTags->"MomentumCombine"],
  Cell[1393093, 56150, 163, 5, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1393259, 56157, 259, 8, 38, "Output",
   CellTags->"MomentumCombine"],
  Cell[1393555, 56170, 170, 5, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1393728, 56177, 444, 19, 28, "Output",
   CellTags->"MomentumCombine"],
  Cell[1394209, 56201, 121, 3, 26, "Input",
   CellTags->"MomentumCombine"],
  Cell[1394333, 56206, 421, 12, 38, "Output",
   CellTags->"MomentumCombine"],
  Cell[1394769, 56221, 115, 3, 26, "Input",
   CellTags->"MomentumCombine"]},
 "MomentumCombine2"->{
  Cell[1394899, 56227, 69, 1, 39, "ObjectName",
   CellTags->"MomentumCombine2"],
  Cell[1394993, 56232, 67, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine2"],
  Cell[1395063, 56235, 192, 4, 41, "Text",
   CellTags->"MomentumCombine2"],
  Cell[1395258, 56241, 943, 31, 24, "Text",
   CellTags->"MomentumCombine2"],
  Cell[1396238, 56277, 64, 1, 36, "Subsubsection",
   CellTags->"MomentumCombine2"],
  Cell[1396327, 56282, 434, 13, 26, "Input",
   CellTags->"MomentumCombine2"],
  Cell[1396764, 56297, 476, 20, 28, "Output",
   CellTags->"MomentumCombine2"],
  Cell[1397277, 56322, 164, 5, 26, "Input",
   CellTags->"MomentumCombine2"],
  Cell[1397444, 56329, 309, 9, 38, "Output",
   CellTags->"MomentumCombine2"]},
 "MomentumExpand"->{
  Cell[1397780, 56342, 65, 1, 39, "ObjectName",
   CellTags->"MomentumExpand"],
  Cell[1397870, 56347, 65, 1, 36, "Subsubsection",
   CellTags->"MomentumExpand"],
  Cell[1397938, 56350, 149, 4, 24, "Text",
   CellTags->"MomentumExpand"],
  Cell[1398090, 56356, 531, 18, 24, "Text",
   CellTags->"MomentumExpand"],
  Cell[1398658, 56379, 62, 1, 36, "Subsubsection",
   CellTags->"MomentumExpand"],
  Cell[1398745, 56384, 218, 6, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1398966, 56392, 189, 5, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1399192, 56402, 168, 5, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1399363, 56409, 248, 9, 28, "Output",
   CellTags->"MomentumExpand"],
  Cell[1399648, 56423, 116, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1399767, 56428, 247, 7, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1400051, 56440, 211, 6, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1400265, 56448, 358, 15, 28, "Output",
   CellTags->"MomentumExpand"],
  Cell[1400660, 56468, 116, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1400779, 56473, 284, 8, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1401100, 56486, 236, 7, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1401339, 56495, 384, 16, 28, "Output",
   CellTags->"MomentumExpand"],
  Cell[1401760, 56516, 116, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1401879, 56521, 367, 11, 38, "Output",
   CellTags->"MomentumExpand"],
  Cell[1402283, 56537, 112, 3, 26, "Input",
   CellTags->"MomentumExpand"],
  Cell[1402398, 56542, 560, 23, 28, "Output",
   CellTags->"MomentumExpand"]},
 "MT"->{
  Cell[1402985, 56569, 41, 1, 39, "ObjectName",
   CellTags->"MT"],
  Cell[1403051, 56574, 53, 1, 36, "Subsubsection",
   CellTags->"MT"],
  Cell[1403107, 56577, 182, 7, 24, "Text",
   CellTags->"MT"],
  Cell[1403292, 56586, 1011, 39, 24, "Text",
   CellTags->"MT"],
  Cell[1404340, 56630, 50, 1, 36, "Subsubsection",
   CellTags->"MT"],
  Cell[1404415, 56635, 134, 4, 26, "Input",
   CellTags->"MT"],
  Cell[1404552, 56641, 155, 5, 26, "Output",
   CellTags->"MT"],
  Cell[1404744, 56651, 263, 8, 26, "Input",
   CellTags->"MT"],
  Cell[1405010, 56661, 99, 3, 26, "Output",
   CellTags->"MT"],
  Cell[1405146, 56669, 157, 5, 26, "Input",
   CellTags->"MT"],
  Cell[1405306, 56676, 136, 4, 38, "Output",
   CellTags->"MT"],
  Cell[1405479, 56685, 189, 6, 26, "Input",
   CellTags->"MT"],
  Cell[1405671, 56693, 218, 6, 38, "Output",
   CellTags->"MT"],
  Cell[1405926, 56704, 225, 8, 26, "Input",
   CellTags->"MT"],
  Cell[1406154, 56714, 136, 4, 38, "Output",
   CellTags->"MT"]},
 "MTD"->{
  Cell[1406317, 56722, 43, 1, 39, "ObjectName",
   CellTags->"MTD"],
  Cell[1406385, 56727, 54, 1, 36, "Subsubsection",
   CellTags->"MTD"],
  Cell[1406442, 56730, 184, 7, 24, "Text",
   CellTags->"MTD"],
  Cell[1406629, 56739, 1008, 39, 24, "Text",
   CellTags->"MTD"],
  Cell[1407674, 56783, 51, 1, 36, "Subsubsection",
   CellTags->"MTD"],
  Cell[1407750, 56788, 136, 4, 26, "Input",
   CellTags->"MTD"],
  Cell[1407889, 56794, 156, 5, 26, "Output",
   CellTags->"MTD"],
  Cell[1408082, 56804, 266, 8, 26, "Input",
   CellTags->"MTD"],
  Cell[1408351, 56814, 100, 3, 27, "Output",
   CellTags->"MTD"],
  Cell[1408488, 56822, 175, 6, 26, "Input",
   CellTags->"MTD"],
  Cell[1408666, 56830, 151, 4, 38, "Output",
   CellTags->"MTD"],
  Cell[1408854, 56839, 207, 7, 26, "Input",
   CellTags->"MTD"],
  Cell[1409064, 56848, 282, 8, 38, "Output",
   CellTags->"MTD"],
  Cell[1409383, 56861, 235, 8, 26, "Input",
   CellTags->"MTD"],
  Cell[1409621, 56871, 146, 4, 38, "Output",
   CellTags->"MTD"]},
 "NegativeInteger"->{
  Cell[1409794, 56879, 67, 1, 39, "ObjectName",
   CellTags->"NegativeInteger"],
  Cell[1409886, 56884, 66, 1, 36, "Subsubsection",
   CellTags->"NegativeInteger"],
  Cell[1409955, 56887, 141, 4, 24, "Text",
   CellTags->"NegativeInteger"],
  Cell[1410099, 56893, 268, 11, 24, "Text",
   CellTags->"NegativeInteger"]},
 "Nf"->{
  Cell[1410382, 56907, 41, 1, 39, "ObjectName",
   CellTags->"Nf"],
  Cell[1410448, 56912, 53, 1, 36, "Subsubsection",
   CellTags->"Nf"],
  Cell[1410504, 56915, 66, 1, 24, "Text",
   CellTags->"Nf"],
  Cell[1410573, 56918, 1119, 39, 24, "Text",
   CellTags->"Nf"]},
 "Nielsen"->{
  Cell[1411707, 56960, 51, 1, 39, "ObjectName",
   CellTags->"Nielsen"],
  Cell[1411783, 56965, 58, 1, 36, "Subsubsection",
   CellTags->"Nielsen"],
  Cell[1411844, 56968, 86, 1, 24, "Text",
   CellTags->"Nielsen"],
  Cell[1411955, 56973, 114, 3, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1412072, 56978, 169, 6, 26, "Output",
   CellTags->"Nielsen"],
  Cell[1412256, 56987, 288, 11, 24, "Text",
   CellTags->"Nielsen"],
  Cell[1412581, 57003, 55, 1, 36, "Subsubsection",
   CellTags->"Nielsen"],
  Cell[1412661, 57008, 141, 4, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1412805, 57014, 251, 10, 26, "Output",
   CellTags->"Nielsen"],
  Cell[1413071, 57027, 176, 4, 24, "Text",
   CellTags->"Nielsen"],
  Cell[1413272, 57035, 172, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1413447, 57042, 123, 3, 26, "Output",
   CellTags->"Nielsen"],
  Cell[1413585, 57048, 71, 1, 24, "Text",
   CellTags->"Nielsen"],
  Cell[1413681, 57053, 467, 14, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1414151, 57069, 320, 13, 44, "Output",
   CellTags->"Nielsen"],
  Cell[1414508, 57087, 190, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1414701, 57094, 573, 20, 42, "Output",
   CellTags->"Nielsen"],
  Cell[1415311, 57119, 190, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1415504, 57126, 807, 27, 45, "Output",
   CellTags->"Nielsen"],
  Cell[1416348, 57158, 190, 5, 26, "Input",
   CellTags->"Nielsen"],
  Cell[1416541, 57165, 140, 5, 26, "Output",
   CellTags->"Nielsen"]},
 "NonCommFreeQ"->{
  Cell[1416708, 57174, 61, 1, 39, "ObjectName",
   CellTags->"NonCommFreeQ"],
  Cell[1416794, 57179, 63, 1, 36, "Subsubsection",
   CellTags->"NonCommFreeQ"],
  Cell[1416860, 57182, 244, 5, 41, "Text",
   CellTags->"NonCommFreeQ"],
  Cell[1417107, 57189, 849, 32, 24, "Text",
   CellTags->"NonCommFreeQ"]},
 "NonCommQ"->{
  Cell[1417971, 57224, 53, 1, 39, "ObjectName",
   CellTags->"NonCommQ"],
  Cell[1418049, 57229, 59, 1, 36, "Subsubsection",
   CellTags->"NonCommQ"],
  Cell[1418111, 57232, 203, 4, 41, "Text",
   CellTags->"NonCommQ"],
  Cell[1418317, 57238, 861, 32, 24, "Text",
   CellTags->"NonCommQ"],
  Cell[1421752, 57394, 245, 11, 24, "Text",
   CellTags->"NonCommQ"]},
 "NonCommutative"->{
  Cell[1419193, 57273, 65, 1, 39, "ObjectName",
   CellTags->"NonCommutative"],
  Cell[1419283, 57278, 65, 1, 36, "Subsubsection",
   CellTags->"NonCommutative"],
  Cell[1419351, 57281, 150, 4, 24, "Text",
   CellTags->"NonCommutative"],
  Cell[1419504, 57287, 510, 18, 24, "Text",
   CellTags->"NonCommutative"]},
 "NTerms"->{
  Cell[1420029, 57308, 49, 1, 39, "ObjectName",
   CellTags->"NTerms"],
  Cell[1420103, 57313, 57, 1, 36, "Subsubsection",
   CellTags->"NTerms"],
  Cell[1420163, 57316, 154, 4, 24, "Text",
   CellTags->"NTerms"],
  Cell[1420354, 57325, 54, 1, 36, "Subsubsection",
   CellTags->"NTerms"],
  Cell[1420433, 57330, 129, 4, 26, "Input",
   CellTags->"NTerms"],
  Cell[1420565, 57336, 103, 3, 26, "Output",
   CellTags->"NTerms"],
  Cell[1420705, 57344, 139, 4, 26, "Input",
   CellTags->"NTerms"],
  Cell[1420847, 57350, 103, 3, 26, "Output",
   CellTags->"NTerms"],
  Cell[1420987, 57358, 106, 3, 26, "Input",
   CellTags->"NTerms"],
  Cell[1421096, 57363, 103, 3, 26, "Output",
   CellTags->"NTerms"],
  Cell[1421236, 57371, 106, 3, 26, "Input",
   CellTags->"NTerms"],
  Cell[1421345, 57376, 103, 3, 26, "Output",
   CellTags->"NTerms"]},
 "NumberOfMetricTensors"->{
  Cell[1421475, 57383, 79, 1, 39, "ObjectName",
   CellTags->"NumberOfMetricTensors"],
  Cell[1421579, 57388, 72, 1, 36, "Subsubsection",
   CellTags->"NumberOfMetricTensors"],
  Cell[1421654, 57391, 95, 1, 24, "Text",
   CellTags->"NumberOfMetricTensors"]},
 "NumericalFactor"->{
  Cell[1422012, 57408, 67, 1, 39, "ObjectName",
   CellTags->"NumericalFactor"],
  Cell[1422104, 57413, 66, 1, 36, "Subsubsection",
   CellTags->"NumericalFactor"],
  Cell[1422173, 57416, 119, 3, 24, "Text",
   CellTags->"NumericalFactor"]},
 "NumericQ1"->{
  Cell[1422307, 57422, 55, 1, 39, "ObjectName",
   CellTags->"NumericQ1"],
  Cell[1422387, 57427, 60, 1, 36, "Subsubsection",
   CellTags->"NumericQ1"],
  Cell[1422450, 57430, 139, 4, 24, "Text",
   CellTags->"NumericQ1"],
  Cell[1422626, 57439, 57, 1, 36, "Subsubsection",
   CellTags->"NumericQ1"],
  Cell[1422708, 57444, 224, 7, 29, "Input",
   CellTags->"NumericQ1"],
  Cell[1422935, 57453, 110, 3, 26, "Output",
   CellTags->"NumericQ1"],
  Cell[1423082, 57461, 271, 9, 29, "Input",
   CellTags->"NumericQ1"],
  Cell[1423356, 57472, 110, 3, 26, "Output",
   CellTags->"NumericQ1"],
  Cell[1423503, 57480, 311, 10, 29, "Input",
   CellTags->"NumericQ1"],
  Cell[1423817, 57492, 109, 3, 26, "Output",
   CellTags->"NumericQ1"]},
 "OneLoop"->{
  Cell[1423975, 57501, 69, 1, 39, "ObjectName",
   CellTags->"OneLoop"],
  Cell[1424047, 57504, 708, 18, 58, "Text",
   CellTags->"OneLoop"],
  Cell[1424758, 57524, 968, 26, 41, "Text",
   CellTags->"OneLoop"],
  Cell[1425763, 57555, 58, 1, 36, "Subsubsection",
   CellTags->"OneLoop"],
  Cell[1425824, 57558, 391, 12, 41, "Text",
   CellTags->"OneLoop"],
  Cell[1426218, 57572, 352, 6, 58, "Text",
   CellTags->"OneLoop"],
  Cell[1426595, 57582, 114, 3, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1426712, 57587, 1275, 29, 87, "Output",
   CellTags->"OneLoop"],
  Cell[1428002, 57619, 1336, 53, 24, "Text",
   CellTags->"OneLoop"],
  Cell[1429375, 57677, 55, 1, 36, "Subsubsection",
   CellTags->"OneLoop"],
  Cell[1429455, 57682, 275, 10, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1429733, 57694, 323, 11, 48, "Output",
   CellTags->"OneLoop"],
  Cell[1430093, 57710, 131, 4, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1430227, 57716, 266, 10, 29, "Output",
   CellTags->"OneLoop"],
  Cell[1430508, 57729, 363, 11, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1430874, 57742, 588, 20, 27, "Text",
   CellTags->"OneLoop"],
  Cell[1431487, 57766, 908, 30, 49, "Input",
   CellTags->"OneLoop"],
  Cell[1432398, 57798, 1871, 65, 55, "Output",
   CellTags->"OneLoop"],
  Cell[1434284, 57866, 219, 5, 41, "Text",
   CellTags->"OneLoop"],
  Cell[1434528, 57875, 131, 4, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1434662, 57881, 2406, 79, 59, "Output",
   CellTags->"OneLoop"],
  Cell[1437105, 57965, 113, 3, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1437221, 57970, 1707, 58, 49, "Output",
   CellTags->"OneLoop"],
  Cell[1438965, 58033, 333, 11, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1439301, 58046, 442, 17, 50, "Output",
   CellTags->"OneLoop"],
  Cell[1439780, 58068, 131, 4, 26, "Input",
   CellTags->"OneLoop"],
  Cell[1439914, 58074, 1917, 67, 49, "Output",
   CellTags->"OneLoop"],
  Cell[1441846, 58144, 106, 3, 26, "Input",
   CellTags->"OneLoop"]},
 "OneLoopSimplify"->{
  Cell[1441967, 58150, 67, 1, 39, "ObjectName",
   CellTags->"OneLoopSimplify"],
  Cell[1442059, 58155, 66, 1, 36, "Subsubsection",
   CellTags->"OneLoopSimplify"],
  Cell[1442128, 58158, 172, 4, 24, "Text",
   CellTags->"OneLoopSimplify"],
  Cell[1442325, 58166, 130, 3, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1442458, 58171, 737, 18, 47, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1443210, 58192, 612, 25, 24, "Text",
   CellTags->"OneLoopSimplify"],
  Cell[1443859, 58222, 63, 1, 36, "Subsubsection",
   CellTags->"OneLoopSimplify"],
  Cell[1443947, 58227, 341, 11, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1444291, 58240, 450, 17, 50, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1444778, 58262, 147, 4, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1444928, 58268, 1129, 43, 50, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1446094, 58316, 203, 6, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1446300, 58324, 400, 16, 46, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1446737, 58345, 401, 12, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1447141, 58359, 580, 22, 50, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1447758, 58386, 152, 4, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1447913, 58392, 7168, 261, 98, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1455118, 58658, 253, 9, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1455374, 58669, 4274, 162, 50, "Output",
   CellTags->"OneLoopSimplify"],
  Cell[1459685, 58836, 424, 15, 26, "Input",
   CellTags->"OneLoopSimplify"],
  Cell[1460112, 58853, 2174, 83, 52, "Output",
   CellTags->"OneLoopSimplify"]},
 "OneLoopSum"->{
  Cell[1462313, 58940, 57, 1, 39, "ObjectName",
   CellTags->"OneLoopSum"],
  Cell[1462395, 58945, 61, 1, 36, "Subsubsection",
   CellTags->"OneLoopSum"],
  Cell[1462459, 58948, 190, 4, 24, "Text",
   CellTags->"OneLoopSum"],
  Cell[1462652, 58954, 259, 11, 24, "Text",
   CellTags->"OneLoopSum"]},
 "OPE"->{
  Cell[1462926, 58968, 43, 1, 39, "ObjectName",
   CellTags->"OPE"],
  Cell[1462994, 58973, 54, 1, 36, "Subsubsection",
   CellTags->"OPE"],
  Cell[1463051, 58976, 170, 4, 49, "Text",
   CellTags->"OPE"],
  Cell[1463569, 58999, 256, 11, 24, "Text",
   CellTags->"OPE"]},
 "OPEDelta"->{
  Cell[1463840, 59013, 53, 1, 39, "ObjectName",
   CellTags->"OPEDelta"],
  Cell[1463918, 59018, 59, 1, 36, "Subsubsection",
   CellTags->"OPEDelta"],
  Cell[1463980, 59021, 140, 4, 24, "Text",
   CellTags->"OPEDelta"],
  Cell[1464123, 59027, 305, 11, 24, "Text",
   CellTags->"OPEDelta"],
  Cell[1464465, 59043, 56, 1, 36, "Subsubsection",
   CellTags->"OPEDelta"],
  Cell[1464546, 59048, 146, 4, 26, "Input",
   CellTags->"OPEDelta"],
  Cell[1464695, 59054, 179, 6, 29, "Output",
   CellTags->"OPEDelta"],
  Cell[1464911, 59065, 133, 4, 26, "Input",
   CellTags->"OPEDelta"],
  Cell[1465047, 59071, 105, 3, 26, "Output",
   CellTags->"OPEDelta"],
  Cell[1465189, 59079, 141, 4, 26, "Input",
   CellTags->"OPEDelta"],
  Cell[1465333, 59085, 105, 3, 26, "Output",
   CellTags->"OPEDelta"]},
 "OPEi"->{
  Cell[1465465, 59092, 81, 1, 39, "ObjectName",
   CellTags->"OPEi"],
  Cell[1465571, 59097, 55, 1, 36, "Subsubsection",
   CellTags->"OPEi"],
  Cell[1465629, 59100, 171, 4, 24, "Text",
   CellTags->"OPEi"],
  Cell[1465837, 59109, 52, 1, 36, "Subsubsection",
   CellTags->"OPEi"],
  Cell[1465914, 59114, 75, 2, 26, "Input",
   CellTags->"OPEi"],
  Cell[1465992, 59118, 101, 3, 27, "Output",
   CellTags->"OPEi"],
  Cell[1466130, 59126, 259, 7, 26, "Input",
   CellTags->"OPEi"],
  Cell[1466392, 59135, 228, 7, 26, "Output",
   CellTags->"OPEi"],
  Cell[1466657, 59147, 1076, 40, 46, "Input",
   CellTags->"OPEi"],
  Cell[1467736, 59189, 202, 6, 26, "Output",
   CellTags->"OPEi"],
  Cell[1467953, 59198, 55, 1, 24, "Text",
   CellTags->"OPEi"],
  Cell[1468033, 59203, 519, 17, 26, "Input",
   CellTags->"OPEi"],
  Cell[1468555, 59222, 529, 19, 26, "Output",
   CellTags->"OPEi"],
  Cell[1469121, 59246, 534, 19, 26, "Input",
   CellTags->"OPEi"],
  Cell[1469658, 59267, 413, 15, 26, "Output",
   CellTags->"OPEi"],
  Cell[1470108, 59287, 519, 17, 26, "Input",
   CellTags->"OPEi"],
  Cell[1470630, 59306, 529, 19, 26, "Output",
   CellTags->"OPEi"]},
 "OPEInsert"->{
  Cell[1471186, 59329, 73, 1, 39, "ObjectName",
   CellTags->"OPEInsert"],
  Cell[1471284, 59334, 60, 1, 36, "Subsubsection",
   CellTags->"OPEInsert"],
  Cell[1471347, 59337, 264, 5, 41, "Text",
   CellTags->"OPEInsert"],
  Cell[1471614, 59344, 242, 11, 24, "Text",
   CellTags->"OPEInsert"],
  Cell[1471893, 59360, 57, 1, 36, "Subsubsection",
   CellTags->"OPEInsert"],
  Cell[1471953, 59363, 76, 2, 26, "Input",
   CellTags->"OPEInsert"]},
 "OPEInt"->{
  Cell[1472044, 59368, 67, 1, 39, "ObjectName",
   CellTags->"OPEInt"],
  Cell[1472136, 59373, 57, 1, 36, "Subsubsection",
   CellTags->"OPEInt"],
  Cell[1472196, 59376, 257, 5, 41, "Text",
   CellTags->"OPEInt"],
  Cell[1472456, 59383, 239, 11, 24, "Text",
   CellTags->"OPEInt"],
  Cell[1472732, 59399, 54, 1, 36, "Subsubsection",
   CellTags->"OPEInt"],
  Cell[1472789, 59402, 73, 2, 26, "Input",
   CellTags->"OPEInt"]},
 "OPEIntDelta"->{
  Cell[1472877, 59407, 77, 1, 39, "ObjectName",
   CellTags->"OPEIntDelta"],
  Cell[1472979, 59412, 62, 1, 36, "Subsubsection",
   CellTags->"OPEIntDelta"],
  Cell[1473044, 59415, 478, 10, 66, "Text",
   CellTags->"OPEIntDelta"],
  Cell[1473525, 59427, 244, 11, 24, "Text",
   CellTags->"OPEIntDelta"],
  Cell[1473806, 59443, 59, 1, 36, "Subsubsection",
   CellTags->"OPEIntDelta"],
  Cell[1473868, 59446, 78, 2, 26, "Input",
   CellTags->"OPEIntDelta"]},
 "OPEIntegrate"->{
  Cell[1473961, 59451, 79, 1, 39, "ObjectName",
   CellTags->"OPEIntegrate"],
  Cell[1474065, 59456, 63, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrate"],
  Cell[1474131, 59459, 266, 5, 41, "Text",
   CellTags->"OPEIntegrate"],
  Cell[1474400, 59466, 245, 11, 24, "Text",
   CellTags->"OPEIntegrate"],
  Cell[1474682, 59482, 60, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrate"],
  Cell[1474745, 59485, 79, 2, 26, "Input",
   CellTags->"OPEIntegrate"]},
 "OPEIntegrateDelta"->{
  Cell[1474839, 59490, 89, 1, 39, "ObjectName",
   CellTags->"OPEIntegrateDelta"],
  Cell[1474953, 59495, 68, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrateDelta"],
  Cell[1475024, 59498, 499, 10, 66, "Text",
   CellTags->"OPEIntegrateDelta"],
  Cell[1475526, 59510, 250, 11, 24, "Text",
   CellTags->"OPEIntegrateDelta"],
  Cell[1475813, 59526, 65, 1, 36, "Subsubsection",
   CellTags->"OPEIntegrateDelta"],
  Cell[1475881, 59529, 84, 2, 26, "Input",
   CellTags->"OPEIntegrateDelta"]},
 "OPESumExplicit"->{
  Cell[1475980, 59534, 65, 1, 39, "ObjectName",
   CellTags->"OPESumExplicit"],
  Cell[1476070, 59539, 65, 1, 36, "Subsubsection",
   CellTags->"OPESumExplicit"],
  Cell[1476138, 59542, 85, 1, 24, "Text",
   CellTags->"OPESumExplicit"],
  Cell[1476226, 59545, 475, 18, 24, "Text",
   CellTags->"OPESumExplicit"],
  Cell[1476738, 59568, 62, 1, 36, "Subsubsection",
   CellTags->"OPESumExplicit"],
  Cell[1476825, 59573, 387, 13, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1477215, 59588, 407, 15, 54, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1477659, 59608, 123, 3, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1477785, 59613, 408, 16, 44, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1478230, 59634, 554, 18, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1478787, 59654, 702, 27, 56, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1479526, 59686, 123, 3, 26, "Input",
   CellTags->"OPESumExplicit"],
  Cell[1479652, 59691, 1747, 66, 47, "Output",
   CellTags->"OPESumExplicit"],
  Cell[1481414, 59760, 157, 5, 26, "Input",
   CellTags->"OPESumExplicit"]},
 "OPESum"->{
  Cell[1481586, 59768, 49, 1, 39, "ObjectName",
   CellTags->"OPESum"],
  Cell[1481660, 59773, 57, 1, 36, "Subsubsection",
   CellTags->"OPESum"],
  Cell[1481720, 59776, 127, 4, 24, "Text",
   CellTags->"OPESum"],
  Cell[1481850, 59782, 499, 18, 24, "Text",
   CellTags->"OPESum"],
  Cell[1482386, 59805, 54, 1, 36, "Subsubsection",
   CellTags->"OPESum"],
  Cell[1482465, 59810, 460, 15, 26, "Input",
   CellTags->"OPESum"],
  Cell[1482928, 59827, 855, 31, 54, "Output",
   CellTags->"OPESum"],
  Cell[1483820, 59863, 115, 3, 26, "Input",
   CellTags->"OPESum"],
  Cell[1483938, 59868, 1604, 60, 49, "Output",
   CellTags->"OPESum"],
  Cell[1485579, 59933, 546, 18, 26, "Input",
   CellTags->"OPESum"],
  Cell[1486128, 59953, 694, 27, 56, "Output",
   CellTags->"OPESum"],
  Cell[1486859, 59985, 115, 3, 26, "Input",
   CellTags->"OPESum"],
  Cell[1486977, 59990, 1739, 66, 47, "Output",
   CellTags->"OPESum"],
  Cell[1488731, 60059, 149, 5, 26, "Input",
   CellTags->"OPESum"]},
 "OPESumSimplify"->{
  Cell[1488895, 60067, 65, 1, 39, "ObjectName",
   CellTags->"OPESumSimplify"],
  Cell[1488985, 60072, 65, 1, 36, "Subsubsection",
   CellTags->"OPESumSimplify"],
  Cell[1489053, 60075, 92, 1, 24, "Text",
   CellTags->"OPESumSimplify"],
  Cell[1489148, 60078, 475, 18, 24, "Text",
   CellTags->"OPESumSimplify"],
  Cell[1489660, 60101, 62, 1, 36, "Subsubsection",
   CellTags->"OPESumSimplify"],
  Cell[1489747, 60106, 143, 4, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1489893, 60112, 1052, 37, 53, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1490982, 60154, 89, 2, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1491074, 60158, 1152, 39, 54, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1492263, 60202, 273, 8, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1492539, 60212, 283, 11, 53, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1492859, 60228, 439, 14, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1493301, 60244, 553, 22, 53, "Output",
   CellTags->"OPESumSimplify"],
  Cell[1493891, 60271, 116, 3, 26, "Input",
   CellTags->"OPESumSimplify"],
  Cell[1494010, 60276, 404, 13, 43, "Output",
   CellTags->"OPESumSimplify"]},
 "OPE1Loop"->{
  Cell[1494441, 60293, 54, 1, 39, "ObjectName",
   CellTags->"OPE1Loop"],
  Cell[1494520, 60298, 59, 1, 36, "Subsubsection",
   CellTags->"OPE1Loop"],
  Cell[1494582, 60301, 119, 3, 24, "Text",
   CellTags->"OPE1Loop"],
  Cell[1494704, 60306, 253, 11, 24, "Text",
   CellTags->"OPE1Loop"],
  Cell[1494994, 60322, 56, 1, 36, "Subsubsection",
   CellTags->"OPE1Loop"],
  Cell[1495053, 60325, 75, 2, 26, "Input",
   CellTags->"OPE1Loop"]},
 "OPE2TID"->{
  Cell[1495143, 60330, 69, 1, 39, "ObjectName",
   CellTags->"OPE2TID"],
  Cell[1495237, 60335, 58, 1, 28, "Subsubsection",
   CellTags->"OPE2TID"],
  Cell[1495298, 60338, 184, 4, 24, "Text",
   CellTags->"OPE2TID"],
  Cell[1495507, 60346, 114, 3, 26, "Input",
   CellTags->"OPE2TID"],
  Cell[1495624, 60351, 343, 10, 26, "Output",
   CellTags->"OPE2TID"],
  Cell[1495982, 60364, 252, 11, 22, "Text",
   CellTags->"OPE2TID"],
  Cell[1496271, 60380, 55, 1, 36, "Subsubsection",
   CellTags->"OPE2TID"],
  Cell[1496329, 60383, 74, 2, 26, "Input",
   CellTags->"OPE2TID"]},
 "OptionsSelect"->{
  Cell[1496418, 60388, 63, 1, 39, "ObjectName",
   CellTags->"OptionsSelect"],
  Cell[1496506, 60393, 64, 1, 28, "Subsubsection",
   CellTags->"OptionsSelect"],
  Cell[1496573, 60396, 219, 5, 41, "Text",
   CellTags->"OptionsSelect"]},
 "Pair"->{
  Cell[1496807, 60404, 45, 1, 39, "ObjectName",
   CellTags->"Pair"],
  Cell[1496877, 60409, 55, 1, 36, "Subsubsection",
   CellTags->"Pair"],
  Cell[1496935, 60412, 499, 11, 60, "Text",
   CellTags->"Pair"],
  Cell[1497437, 60425, 1705, 67, 24, "Text",
   CellTags->"Pair"],
  Cell[1499179, 60497, 52, 1, 36, "Subsubsection",
   CellTags->"Pair"],
  Cell[1499234, 60500, 84, 1, 24, "Text",
   CellTags->"Pair"],
  Cell[1499343, 60505, 218, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1499564, 60513, 157, 5, 26, "Output",
   CellTags->"Pair"],
  Cell[1499736, 60521, 73, 1, 24, "Text",
   CellTags->"Pair"],
  Cell[1499834, 60526, 268, 8, 26, "Input",
   CellTags->"Pair"],
  Cell[1500105, 60536, 157, 5, 26, "Output",
   CellTags->"Pair"],
  Cell[1500299, 60546, 208, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1500510, 60554, 236, 10, 28, "Output",
   CellTags->"Pair"],
  Cell[1500783, 60569, 197, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1500983, 60577, 203, 8, 28, "Output",
   CellTags->"Pair"],
  Cell[1501223, 60590, 197, 6, 26, "Input",
   CellTags->"Pair"],
  Cell[1501423, 60598, 159, 6, 28, "Output",
   CellTags->"Pair"],
  Cell[1501619, 60609, 222, 7, 26, "Input",
   CellTags->"Pair"],
  Cell[1501844, 60618, 238, 9, 28, "Output",
   CellTags->"Pair"],
  Cell[1502119, 60632, 223, 7, 26, "Input",
   CellTags->"Pair"],
  Cell[1502345, 60641, 223, 9, 30, "Output",
   CellTags->"Pair"],
  Cell[1502605, 60655, 223, 7, 26, "Input",
   CellTags->"Pair"],
  Cell[1502831, 60664, 223, 9, 30, "Output",
   CellTags->"Pair"],
  Cell[1503091, 60678, 308, 10, 26, "Input",
   CellTags->"Pair"],
  Cell[1503402, 60690, 101, 3, 26, "Output",
   CellTags->"Pair"],
  Cell[1503540, 60698, 272, 8, 26, "Input",
   CellTags->"Pair"],
  Cell[1503815, 60708, 292, 12, 28, "Output",
   CellTags->"Pair"],
  Cell[1504144, 60725, 383, 12, 26, "Input",
   CellTags->"Pair"],
  Cell[1504530, 60739, 101, 3, 26, "Output",
   CellTags->"Pair"]},
 "PairCollect"->{
  Cell[1504658, 60746, 59, 1, 39, "ObjectName",
   CellTags->"PairCollect"],
  Cell[1504742, 60751, 62, 1, 36, "Subsubsection",
   CellTags->"PairCollect"],
  Cell[1504807, 60754, 153, 4, 24, "Text",
   CellTags->"PairCollect"],
  Cell[1504963, 60760, 448, 18, 24, "Text",
   CellTags->"PairCollect"]},
 "PairContract"->{
  Cell[1505426, 60781, 61, 1, 39, "ObjectName",
   CellTags->"PairContract"],
  Cell[1505512, 60786, 63, 1, 36, "Subsubsection",
   CellTags->"PairContract"],
  Cell[1505578, 60789, 118, 3, 24, "Text",
   CellTags->"PairContract"],
  Cell[1505699, 60794, 435, 17, 24, "Text",
   CellTags->"PairContract"]},
 "PairContract2"->{
  Cell[1506149, 60814, 63, 1, 39, "ObjectName",
   CellTags->"PairContract2"],
  Cell[1506237, 60819, 64, 1, 36, "Subsubsection",
   CellTags->"PairContract2"],
  Cell[1506304, 60822, 142, 4, 24, "Text",
   CellTags->"PairContract2"],
  Cell[1506449, 60828, 644, 24, 24, "Text",
   CellTags->"PairContract2"]},
 "PairContract3"->{
  Cell[1507108, 60855, 63, 1, 39, "ObjectName",
   CellTags->"PairContract3"],
  Cell[1507196, 60860, 64, 1, 36, "Subsubsection",
   CellTags->"PairContract3"],
  Cell[1507263, 60863, 142, 4, 24, "Text",
   CellTags->"PairContract3"],
  Cell[1507408, 60869, 644, 24, 24, "Text",
   CellTags->"PairContract3"]},
 "PartialD"->{
  Cell[1508067, 60896, 53, 1, 39, "ObjectName",
   CellTags->"PartialD"],
  Cell[1508145, 60901, 59, 1, 36, "Subsubsection",
   CellTags->"PartialD"],
  Cell[1508207, 60904, 864, 35, 47, "Text",
   CellTags->"PartialD"],
  Cell[1509074, 60941, 933, 32, 24, "Text",
   CellTags->"PartialD"],
  Cell[1510044, 60978, 65, 1, 33, "Subsubsection",
   CellTags->"PartialD"],
  Cell[1510134, 60983, 233, 7, 26, "Input",
   CellTags->"PartialD"],
  Cell[1510370, 60992, 302, 11, 33, "Output",
   CellTags->"PartialD"],
  Cell[1510709, 61008, 116, 3, 26, "Input",
   CellTags->"PartialD"],
  Cell[1510828, 61013, 351, 14, 30, "Output",
   CellTags->"PartialD"],
  Cell[1511216, 61032, 114, 3, 26, "Input",
   CellTags->"PartialD"],
  Cell[1511333, 61037, 287, 7, 38, "Output",
   CellTags->"PartialD"]},
 "PartialDRelations"->{
  Cell[1511647, 61048, 71, 1, 39, "ObjectName",
   CellTags->"PartialDRelations"],
  Cell[1511743, 61053, 68, 1, 36, "Subsubsection",
   CellTags->"PartialDRelations"],
  Cell[1511814, 61056, 191, 5, 24, "Text",
   CellTags->"PartialDRelations"],
  Cell[1512008, 61063, 1134, 39, 24, "Text",
   CellTags->"PartialDRelations"],
  Cell[1513179, 61107, 74, 1, 33, "Subsubsection",
   CellTags->"PartialDRelations"],
  Cell[1513278, 61112, 344, 10, 26, "Input",
   CellTags->"PartialDRelations"],
  Cell[1513625, 61124, 348, 12, 33, "Output",
   CellTags->"PartialDRelations"],
  Cell[1514010, 61141, 256, 7, 26, "Input",
   CellTags->"PartialDRelations"],
  Cell[1514269, 61150, 855, 34, 30, "Output",
   CellTags->"PartialDRelations"]},
 "PartialFourVector"->{
  Cell[1515151, 61188, 71, 1, 39, "ObjectName",
   CellTags->"PartialFourVector"],
  Cell[1515247, 61193, 68, 1, 36, "Subsubsection",
   CellTags->"PartialFourVector"],
  Cell[1515318, 61196, 277, 6, 41, "Text",
   CellTags->"PartialFourVector"],
  Cell[1515598, 61204, 646, 25, 24, "Text",
   CellTags->"PartialFourVector"],
  Cell[1516281, 61234, 74, 1, 33, "Subsubsection",
   CellTags->"PartialFourVector"],
  Cell[1516380, 61239, 382, 11, 35, "Input",
   CellTags->"PartialFourVector"],
  Cell[1516765, 61252, 399, 15, 30, "Output",
   CellTags->"PartialFourVector"]},
 "PartialIntegrate"->{
  Cell[1517191, 61271, 69, 1, 39, "ObjectName",
   CellTags->"PartialIntegrate"],
  Cell[1517285, 61276, 67, 1, 36, "Subsubsection",
   CellTags->"PartialIntegrate"],
  Cell[1517355, 61279, 283, 6, 41, "Text",
   CellTags->"PartialIntegrate"],
  Cell[1517663, 61289, 132, 3, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1517798, 61294, 185, 6, 26, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1518032, 61306, 73, 1, 33, "Subsubsection",
   CellTags->"PartialIntegrate"],
  Cell[1518130, 61311, 326, 10, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1518459, 61323, 1229, 40, 45, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1520069, 61382, 275, 9, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1520347, 61393, 383, 14, 42, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1520767, 61412, 293, 10, 44, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1521063, 61424, 378, 14, 48, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1521478, 61443, 443, 14, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1521924, 61459, 246, 8, 46, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1522207, 61472, 320, 11, 26, "Input",
   CellTags->"PartialIntegrate"],
  Cell[1522530, 61485, 246, 8, 46, "Output",
   CellTags->"PartialIntegrate"],
  Cell[1522791, 61496, 138, 4, 26, "Input",
   CellTags->"PartialIntegrate"]},
 "PartitHead"->{
  Cell[1522944, 61503, 57, 1, 39, "ObjectName",
   CellTags->"PartitHead"],
  Cell[1523026, 61508, 61, 1, 28, "Subsubsection",
   CellTags->"PartitHead"],
  Cell[1523090, 61511, 167, 4, 22, "Text",
   CellTags->"PartitHead"]},
 "PauliSigma"->{
  Cell[1523272, 61518, 57, 1, 39, "ObjectName",
   CellTags->"PauliSigma"],
  Cell[1523354, 61523, 61, 1, 28, "Subsubsection",
   CellTags->"PauliSigma"],
  Cell[1523418, 61526, 252, 5, 41, "Text",
   CellTags->"PauliSigma"],
  Cell[1523695, 61535, 142, 4, 26, "Input",
   CellTags->"PauliSigma"],
  Cell[1523840, 61541, 4559, 125, 122, "Output",
   CellTags->"PauliSigma"]},
 "PaVe"->{
  Cell[1528426, 61670, 45, 1, 39, "ObjectName",
   CellTags->"PaVe"],
  Cell[1528496, 61675, 55, 1, 28, "Subsubsection",
   CellTags->"PaVe"],
  Cell[1528554, 61678, 428, 7, 58, "Text",
   CellTags->"PaVe"],
  Cell[1528985, 61687, 266, 8, 24, "Text",
   CellTags->"PaVe"],
  Cell[1529254, 61697, 265, 11, 24, "Text",
   CellTags->"PaVe"],
  Cell[1529556, 61713, 52, 1, 36, "Subsubsection",
   CellTags->"PaVe"],
  Cell[1529611, 61716, 78, 1, 24, "Text",
   CellTags->"PaVe"],
  Cell[1529714, 61721, 303, 10, 26, "Input",
   CellTags->"PaVe"],
  Cell[1530020, 61733, 304, 9, 26, "Output",
   CellTags->"PaVe"],
  Cell[1530361, 61747, 271, 9, 26, "Input",
   CellTags->"PaVe"],
  Cell[1530635, 61758, 311, 11, 29, "Output",
   CellTags->"PaVe"]},
 "PaVeOrderList"->{
  Cell[1530973, 61773, 63, 1, 39, "ObjectName",
   CellTags->"PaVeOrderList"],
  Cell[1531061, 61778, 64, 1, 36, "Subsubsection",
   CellTags->"PaVeOrderList"],
  Cell[1531128, 61781, 176, 4, 24, "Text",
   CellTags->"PaVeOrderList"],
  Cell[1531307, 61787, 808, 31, 24, "Text",
   CellTags->"PaVeOrderList"]},
 "PaVeOrder"->{
  Cell[1532130, 61821, 55, 1, 39, "ObjectName",
   CellTags->"PaVeOrder"],
  Cell[1532210, 61826, 60, 1, 36, "Subsubsection",
   CellTags->"PaVeOrder"],
  Cell[1532273, 61829, 564, 16, 41, "Text",
   CellTags->"PaVeOrder"],
  Cell[1532840, 61847, 270, 11, 24, "Text",
   CellTags->"PaVeOrder"],
  Cell[1533147, 61863, 57, 1, 36, "Subsubsection",
   CellTags->"PaVeOrder"],
  Cell[1533229, 61868, 413, 12, 26, "Input",
   CellTags->"PaVeOrder"],
  Cell[1533645, 61882, 382, 12, 26, "Output",
   CellTags->"PaVeOrder"],
  Cell[1534064, 61899, 413, 12, 26, "Input",
   CellTags->"PaVeOrder"],
  Cell[1534480, 61913, 382, 12, 26, "Output",
   CellTags->"PaVeOrder"],
  Cell[1534899, 61930, 706, 20, 46, "Input",
   CellTags->"PaVeOrder"],
  Cell[1535608, 61952, 623, 18, 26, "Output",
   CellTags->"PaVeOrder"]},
 "PaVeReduce"->{
  Cell[1536258, 61974, 57, 1, 39, "ObjectName",
   CellTags->"PaVeReduce"],
  Cell[1536340, 61979, 61, 1, 36, "Subsubsection",
   CellTags->"PaVeReduce"],
  Cell[1536404, 61982, 167, 4, 24, "Text",
   CellTags->"PaVeReduce"],
  Cell[1536596, 61990, 120, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1536719, 61995, 446, 13, 26, "Output",
   CellTags->"PaVeReduce"],
  Cell[1537202, 62013, 106, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1537311, 62018, 332, 5, 58, "Print",
   CellTags->{"Info3535453273-5096475", "PaVeReduce"}],
  Cell[1537658, 62026, 439, 18, 24, "Text",
   CellTags->"PaVeReduce"],
  Cell[1538134, 62049, 58, 1, 36, "Subsubsection",
   CellTags->"PaVeReduce"],
  Cell[1538217, 62054, 520, 16, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1538740, 62072, 3016, 104, 96, "Output",
   CellTags->"PaVeReduce"],
  Cell[1541793, 62181, 106, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1541902, 62186, 136, 4, 26, "Output",
   CellTags->"PaVeReduce"],
  Cell[1542075, 62195, 107, 3, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1542185, 62200, 136, 4, 26, "Output",
   CellTags->"PaVeReduce"],
  Cell[1542358, 62209, 491, 15, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1542852, 62226, 468, 15, 42, "Output",
   CellTags->"PaVeReduce"],
  Cell[1543357, 62246, 398, 12, 51, "Input",
   CellTags->"PaVeReduce"],
  Cell[1543758, 62260, 792, 23, 72, "Output",
   CellTags->"PaVeReduce"],
  Cell[1544565, 62286, 209, 7, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1544777, 62295, 162, 5, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1544964, 62304, 633, 17, 46, "Input",
   CellTags->"PaVeReduce"],
  Cell[1545600, 62323, 8399, 260, 305, "Output",
   CellTags->"PaVeReduce"],
  Cell[1554014, 62586, 267, 8, 26, "Input",
   CellTags->"PaVeReduce"],
  Cell[1554306, 62598, 375, 11, 29, "Input",
   CellTags->"PaVeReduce"],
  Cell[1554684, 62611, 2976, 57, 653, "Output",
   CellTags->"PaVeReduce"],
  Cell[1557675, 62671, 253, 7, 26, "Input",
   CellTags->"PaVeReduce"]},
 "Info3535453273-5096475"->{
  Cell[1537311, 62018, 332, 5, 58, "Print",
   CellTags->{"Info3535453273-5096475", "PaVeReduce"}]},
 "PD"->{
  Cell[1557943, 62681, 41, 1, 39, "ObjectName",
   CellTags->"PD"],
  Cell[1558009, 62686, 53, 1, 36, "Subsubsection",
   CellTags->"PD"],
  Cell[1558065, 62689, 81, 1, 24, "Text",
   CellTags->"PD"],
  Cell[1558149, 62692, 483, 18, 24, "Text",
   CellTags->"PD"]},
 "PlusDistribution"->{
  Cell[1558647, 62713, 69, 1, 39, "ObjectName",
   CellTags->"PlusDistribution"],
  Cell[1558741, 62718, 67, 1, 36, "Subsubsection",
   CellTags->"PlusDistribution"],
  Cell[1558811, 62721, 147, 4, 24, "Text",
   CellTags->"PlusDistribution"],
  Cell[1558961, 62727, 277, 11, 24, "Text",
   CellTags->"PlusDistribution"],
  Cell[1559275, 62743, 64, 1, 36, "Subsubsection",
   CellTags->"PlusDistribution"],
  Cell[1559364, 62748, 198, 6, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1559565, 62756, 208, 7, 44, "Output",
   CellTags->"PlusDistribution"],
  Cell[1559810, 62768, 254, 8, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1560067, 62778, 268, 9, 46, "Output",
   CellTags->"PlusDistribution"],
  Cell[1560372, 62792, 329, 10, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1560704, 62804, 113, 3, 26, "Output",
   CellTags->"PlusDistribution"],
  Cell[1560854, 62812, 389, 12, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1561246, 62826, 113, 3, 26, "Output",
   CellTags->"PlusDistribution"],
  Cell[1561396, 62834, 417, 13, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1561816, 62849, 113, 3, 26, "Output",
   CellTags->"PlusDistribution"],
  Cell[1561966, 62857, 307, 10, 26, "Input",
   CellTags->"PlusDistribution"],
  Cell[1562276, 62869, 376, 13, 46, "Output",
   CellTags->"PlusDistribution"]},
 "Polarization"->{
  Cell[1562679, 62886, 61, 1, 39, "ObjectName",
   CellTags->"Polarization"],
  Cell[1562765, 62891, 63, 1, 36, "Subsubsection",
   CellTags->"Polarization"],
  Cell[1562831, 62894, 769, 12, 92, "Text",
   CellTags->"Polarization"],
  Cell[1563603, 62908, 757, 25, 24, "Text",
   CellTags->"Polarization"],
  Cell[1564397, 62938, 60, 1, 36, "Subsubsection",
   CellTags->"Polarization"],
  Cell[1564482, 62943, 118, 3, 26, "Input",
   CellTags->"Polarization"],
  Cell[1564603, 62948, 151, 4, 26, "Output",
   CellTags->"Polarization"],
  Cell[1564791, 62957, 153, 4, 26, "Input",
   CellTags->"Polarization"],
  Cell[1564947, 62963, 168, 4, 38, "Output",
   CellTags->"Polarization"],
  Cell[1565152, 62972, 160, 5, 26, "Input",
   CellTags->"Polarization"],
  Cell[1565315, 62979, 237, 8, 26, "Output",
   CellTags->"Polarization"],
  Cell[1565589, 62992, 196, 6, 26, "Input",
   CellTags->"Polarization"],
  Cell[1565788, 63000, 187, 5, 38, "Output",
   CellTags->"Polarization"],
  Cell[1566012, 63010, 155, 4, 26, "Input",
   CellTags->"Polarization"],
  Cell[1566170, 63016, 270, 9, 26, "Output",
   CellTags->"Polarization"],
  Cell[1566477, 63030, 194, 6, 26, "Input",
   CellTags->"Polarization"],
  Cell[1566674, 63038, 206, 5, 38, "Output",
   CellTags->"Polarization"],
  Cell[1566917, 63048, 302, 8, 26, "Input",
   CellTags->"Polarization"],
  Cell[1567222, 63058, 109, 3, 26, "Output",
   CellTags->"Polarization"]},
 "PolarizationSum"->{
  Cell[1567358, 63065, 67, 1, 39, "ObjectName",
   CellTags->"PolarizationSum"],
  Cell[1567450, 63070, 66, 1, 36, "Subsubsection",
   CellTags->"PolarizationSum"],
  Cell[1567519, 63073, 277, 5, 41, "Text",
   CellTags->"PolarizationSum"],
  Cell[1567821, 63082, 130, 3, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1567954, 63087, 173, 5, 26, "Output",
   CellTags->"PolarizationSum"],
  Cell[1568142, 63095, 284, 11, 24, "Text",
   CellTags->"PolarizationSum"],
  Cell[1568463, 63111, 63, 1, 36, "Subsubsection",
   CellTags->"PolarizationSum"],
  Cell[1568551, 63116, 155, 4, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1568709, 63122, 183, 6, 26, "Output",
   CellTags->"PolarizationSum"],
  Cell[1568929, 63133, 165, 4, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1569097, 63139, 605, 25, 45, "Output",
   CellTags->"PolarizationSum"],
  Cell[1569739, 63169, 175, 4, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1569917, 63175, 1686, 71, 47, "Output",
   CellTags->"PolarizationSum"],
  Cell[1571640, 63251, 247, 7, 26, "Input",
   CellTags->"PolarizationSum"],
  Cell[1571890, 63260, 4044, 162, 50, "Output",
   CellTags->"PolarizationSum"]},
 "PolarizationUncontract"->{
  Cell[1575961, 63426, 81, 1, 39, "ObjectName",
   CellTags->"PolarizationUncontract"],
  Cell[1576067, 63431, 73, 1, 36, "Subsubsection",
   CellTags->"PolarizationUncontract"],
  Cell[1576143, 63434, 152, 4, 24, "Text",
   CellTags->"PolarizationUncontract"],
  Cell[1576298, 63440, 491, 18, 24, "Text",
   CellTags->"PolarizationUncontract"]},
 "PolarizationVector"->{
  Cell[1576804, 63461, 73, 1, 39, "ObjectName",
   CellTags->"PolarizationVector"],
  Cell[1576902, 63466, 69, 1, 36, "Subsubsection",
   CellTags->"PolarizationVector"],
  Cell[1576974, 63469, 103, 1, 24, "Text",
   CellTags->"PolarizationVector"],
  Cell[1577080, 63472, 657, 24, 24, "Text",
   CellTags->"PolarizationVector"],
  Cell[1577774, 63501, 66, 1, 36, "Subsubsection",
   CellTags->"PolarizationVector"],
  Cell[1577865, 63506, 157, 4, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1578025, 63512, 258, 9, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1578320, 63526, 194, 5, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1578517, 63533, 268, 9, 30, "Output",
   CellTags->"PolarizationVector"],
  Cell[1578800, 63545, 241, 9, 27, "Text",
   CellTags->"PolarizationVector"],
  Cell[1579066, 63558, 190, 5, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1579259, 63565, 310, 8, 38, "Output",
   CellTags->"PolarizationVector"],
  Cell[1579584, 63576, 440, 18, 27, "Text",
   CellTags->"PolarizationVector"],
  Cell[1580049, 63598, 284, 8, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1580336, 63608, 314, 11, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1580687, 63624, 121, 3, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1580811, 63629, 115, 3, 26, "Output",
   CellTags->"PolarizationVector"],
  Cell[1580963, 63637, 310, 9, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1581276, 63648, 376, 14, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1581689, 63667, 121, 3, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1581813, 63672, 336, 12, 28, "Output",
   CellTags->"PolarizationVector"],
  Cell[1582164, 63687, 1511, 51, 43, "Text",
   CellTags->"PolarizationVector"],
  Cell[1583700, 63742, 310, 9, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1584013, 63753, 376, 14, 29, "Output",
   CellTags->"PolarizationVector"],
  Cell[1584426, 63772, 121, 3, 26, "Input",
   CellTags->"PolarizationVector"],
  Cell[1584550, 63777, 336, 12, 28, "Output",
   CellTags->"PolarizationVector"],
  Cell[1584901, 63792, 153, 4, 26, "Input",
   CellTags->"PolarizationVector"]},
 "PositiveInteger"->{
  Cell[1585069, 63799, 67, 1, 39, "ObjectName",
   CellTags->"PositiveInteger"],
  Cell[1585161, 63804, 66, 1, 36, "Subsubsection",
   CellTags->"PositiveInteger"],
  Cell[1585230, 63807, 136, 4, 24, "Text",
   CellTags->"PositiveInteger"],
  Cell[1585369, 63813, 268, 11, 24, "Text",
   CellTags->"PositiveInteger"]},
 "PositiveNumber"->{
  Cell[1585652, 63827, 65, 1, 39, "ObjectName",
   CellTags->"PositiveNumber"],
  Cell[1585742, 63832, 65, 1, 36, "Subsubsection",
   CellTags->"PositiveNumber"],
  Cell[1585810, 63835, 146, 4, 24, "Text",
   CellTags->"PositiveNumber"],
  Cell[1585959, 63841, 267, 11, 24, "Text",
   CellTags->"PositiveNumber"]},
 "PowerFactor"->{
  Cell[1586241, 63855, 59, 1, 39, "ObjectName",
   CellTags->"PowerFactor"],
  Cell[1586325, 63860, 62, 1, 36, "Subsubsection",
   CellTags->"PowerFactor"],
  Cell[1586390, 63863, 375, 16, 24, "Text",
   CellTags->"PowerFactor"],
  Cell[1586768, 63881, 284, 11, 24, "Text",
   CellTags->"PowerFactor"],
  Cell[1587089, 63897, 59, 1, 36, "Subsubsection",
   CellTags->"PowerFactor"],
  Cell[1587173, 63902, 144, 5, 26, "Input",
   CellTags->"PowerFactor"],
  Cell[1587320, 63909, 181, 6, 26, "Output",
   CellTags->"PowerFactor"],
  Cell[1587538, 63920, 116, 3, 26, "Input",
   CellTags->"PowerFactor"],
  Cell[1587657, 63925, 181, 6, 26, "Output",
   CellTags->"PowerFactor"]},
 "PowerSimplify"->{
  Cell[1587865, 63935, 63, 1, 39, "ObjectName",
   CellTags->"PowerSimplify"],
  Cell[1587953, 63940, 64, 1, 36, "Subsubsection",
   CellTags->"PowerSimplify"],
  Cell[1588020, 63943, 2559, 112, 58, "Text",
   CellTags->"PowerSimplify"],
  Cell[1590582, 64057, 364, 16, 24, "Text",
   CellTags->"PowerSimplify"],
  Cell[1590983, 64078, 61, 1, 36, "Subsubsection",
   CellTags->"PowerSimplify"],
  Cell[1591069, 64083, 234, 8, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1591306, 64093, 110, 3, 26, "Output",
   CellTags->"PowerSimplify"],
  Cell[1591453, 64101, 239, 8, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1591695, 64111, 178, 6, 26, "Output",
   CellTags->"PowerSimplify"],
  Cell[1591910, 64122, 239, 8, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1592152, 64132, 178, 6, 26, "Output",
   CellTags->"PowerSimplify"],
  Cell[1592367, 64143, 190, 6, 26, "Input",
   CellTags->"PowerSimplify"],
  Cell[1592560, 64151, 178, 6, 26, "Output",
   CellTags->"PowerSimplify"]},
 "Power2"->{
  Cell[1592765, 64161, 49, 1, 39, "ObjectName",
   CellTags->"Power2"],
  Cell[1592839, 64166, 57, 1, 36, "Subsubsection",
   CellTags->"Power2"],
  Cell[1592899, 64169, 213, 5, 41, "Text",
   CellTags->"Power2"],
  Cell[1593115, 64176, 271, 11, 24, "Text",
   CellTags->"Power2"],
  Cell[1593423, 64192, 54, 1, 36, "Subsubsection",
   CellTags->"Power2"],
  Cell[1593502, 64197, 147, 5, 26, "Input",
   CellTags->"Power2"],
  Cell[1593652, 64204, 171, 6, 26, "Output",
   CellTags->"Power2"],
  Cell[1593860, 64215, 148, 5, 26, "Input",
   CellTags->"Power2"],
  Cell[1594011, 64222, 257, 10, 26, "Output",
   CellTags->"Power2"]},
 "PostFortranFile"->{
  Cell[1594295, 64236, 67, 1, 39, "ObjectName",
   CellTags->"PostFortranFile"],
  Cell[1594387, 64241, 66, 1, 36, "Subsubsection",
   CellTags->"PostFortranFile"],
  Cell[1594456, 64244, 229, 5, 41, "Text",
   CellTags->"PostFortranFile"],
  Cell[1594688, 64251, 476, 18, 24, "Text",
   CellTags->"PostFortranFile"]},
 "Prefactor"->{
  Cell[1595179, 64272, 55, 1, 39, "ObjectName",
   CellTags->"Prefactor"],
  Cell[1595259, 64277, 60, 1, 36, "Subsubsection",
   CellTags->"Prefactor"],
  Cell[1595322, 64280, 290, 6, 41, "Text",
   CellTags->"Prefactor"],
  Cell[1595615, 64288, 458, 18, 24, "Text",
   CellTags->"Prefactor"]},
 "PreFortranFile"->{
  Cell[1596088, 64309, 65, 1, 39, "ObjectName",
   CellTags->"PreFortranFile"],
  Cell[1596178, 64314, 65, 1, 36, "Subsubsection",
   CellTags->"PreFortranFile"],
  Cell[1596246, 64317, 232, 5, 41, "Text",
   CellTags->"PreFortranFile"],
  Cell[1596481, 64324, 479, 18, 24, "Text",
   CellTags->"PreFortranFile"]},
 "PropagatorDenominator"->{
  Cell[1596975, 64345, 79, 1, 39, "ObjectName",
   CellTags->"PropagatorDenominator"],
  Cell[1597079, 64350, 72, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominator"],
  Cell[1597154, 64353, 549, 16, 60, "Text",
   CellTags->"PropagatorDenominator"],
  Cell[1597706, 64371, 766, 25, 24, "Text",
   CellTags->"PropagatorDenominator"],
  Cell[1598509, 64401, 69, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominator"],
  Cell[1598603, 64406, 178, 5, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1598784, 64413, 215, 7, 48, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1599036, 64425, 136, 3, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1599175, 64430, 164, 5, 48, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1599376, 64440, 189, 5, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1599568, 64447, 215, 7, 48, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1599820, 64459, 158, 4, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1599981, 64465, 235, 6, 38, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1600253, 64476, 194, 5, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1600450, 64483, 235, 6, 38, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1600722, 64494, 145, 3, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1600870, 64499, 253, 9, 48, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1601160, 64513, 127, 3, 26, "Input",
   CellTags->"PropagatorDenominator"],
  Cell[1601290, 64518, 325, 10, 57, "Output",
   CellTags->"PropagatorDenominator"],
  Cell[1601630, 64531, 121, 3, 26, "Input",
   CellTags->"PropagatorDenominator"]},
 "PropagatorDenominatorExplicit"->{
  Cell[1601766, 64537, 95, 1, 39, "ObjectName",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1601886, 64542, 80, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1601969, 64545, 243, 5, 41, "Text",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1602215, 64552, 566, 18, 24, "Text",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1602818, 64575, 77, 1, 36, "Subsubsection",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1602920, 64580, 308, 11, 26, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1603231, 64593, 357, 12, 48, "Output",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1603625, 64610, 181, 5, 26, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1603809, 64617, 766, 29, 48, "Output",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1604612, 64651, 131, 3, 26, "Input",
   CellTags->"PropagatorDenominatorExplicit"],
  Cell[1604746, 64656, 547, 18, 62, "Output",
   CellTags->"PropagatorDenominatorExplicit"]},
 "QGV"->{
  Cell[1605320, 64678, 43, 1, 39, "ObjectName",
   CellTags->"QGV"],
  Cell[1605388, 64683, 54, 1, 36, "Subsubsection",
   CellTags->"QGV"],
  Cell[1605445, 64686, 72, 1, 24, "Text",
   CellTags->"QGV"],
  Cell[1605520, 64689, 288, 11, 24, "Text",
   CellTags->"QGV"]},
 "QO"->{
  Cell[1605823, 64703, 41, 1, 39, "ObjectName",
   CellTags->"QO"],
  Cell[1605889, 64708, 53, 1, 36, "Subsubsection",
   CellTags->"QO"],
  Cell[1605945, 64711, 73, 1, 24, "Text",
   CellTags->"QO"],
  Cell[1606021, 64714, 299, 11, 24, "Text",
   CellTags->"QO"]},
 "QP"->{
  Cell[1606335, 64728, 41, 1, 39, "ObjectName",
   CellTags->"QP"],
  Cell[1606401, 64733, 53, 1, 36, "Subsubsection",
   CellTags->"QP"],
  Cell[1606457, 64736, 169, 4, 24, "Text",
   CellTags->"QP"],
  Cell[1606629, 64742, 283, 11, 24, "Text",
   CellTags->"QP"]},
 "QuantumField"->{
  Cell[1606927, 64756, 61, 1, 39, "ObjectName",
   CellTags->"QuantumField"],
  Cell[1607013, 64761, 63, 1, 36, "Subsubsection",
   CellTags->"QuantumField"],
  Cell[1607079, 64764, 404, 9, 58, "Text",
   CellTags->"QuantumField"],
  Cell[1607486, 64775, 673, 25, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1608196, 64805, 60, 1, 36, "Subsubsection",
   CellTags->"QuantumField"],
  Cell[1608259, 64808, 71, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1608355, 64813, 118, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1608476, 64818, 109, 3, 27, "Output",
   CellTags->"QuantumField"],
  Cell[1608622, 64826, 131, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1608756, 64831, 137, 4, 26, "Output",
   CellTags->"QuantumField"],
  Cell[1608930, 64840, 127, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1609060, 64845, 114, 3, 26, "Output",
   CellTags->"QuantumField"],
  Cell[1609189, 64851, 80, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1609294, 64856, 169, 5, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1609466, 64863, 135, 4, 29, "Output",
   CellTags->"QuantumField"],
  Cell[1609616, 64870, 94, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1609735, 64875, 212, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1609950, 64883, 319, 14, 30, "Output",
   CellTags->"QuantumField"],
  Cell[1610306, 64902, 114, 3, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1610423, 64907, 255, 6, 38, "Output",
   CellTags->"QuantumField"],
  Cell[1610693, 64916, 358, 13, 28, "Text",
   CellTags->"QuantumField"],
  Cell[1611076, 64933, 206, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1611285, 64941, 259, 10, 28, "Output",
   CellTags->"QuantumField"],
  Cell[1611559, 64954, 185, 4, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1611769, 64962, 244, 7, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1612016, 64971, 245, 6, 38, "Output",
   CellTags->"QuantumField"],
  Cell[1612276, 64980, 88, 1, 24, "Text",
   CellTags->"QuantumField"],
  Cell[1612389, 64985, 219, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1612611, 64993, 355, 14, 30, "Output",
   CellTags->"QuantumField"],
  Cell[1613003, 65012, 184, 5, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1613190, 65019, 257, 9, 28, "Output",
   CellTags->"QuantumField"],
  Cell[1613484, 65033, 259, 7, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1613746, 65042, 404, 16, 28, "Output",
   CellTags->"QuantumField"],
  Cell[1614187, 65063, 291, 9, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1614481, 65074, 413, 16, 28, "Output",
   CellTags->"QuantumField"],
  Cell[1614931, 65095, 232, 6, 26, "Input",
   CellTags->"QuantumField"],
  Cell[1615166, 65103, 112, 3, 26, "Output",
   CellTags->"QuantumField"]},
 "QuarkField"->{
  Cell[1615305, 65110, 57, 1, 39, "ObjectName",
   CellTags->"QuarkField"],
  Cell[1615387, 65115, 61, 1, 36, "Subsubsection",
   CellTags->"QuarkField"],
  Cell[1615451, 65118, 189, 4, 24, "Text",
   CellTags->"QuarkField"],
  Cell[1615643, 65124, 495, 18, 24, "Text",
   CellTags->"QuarkField"],
  Cell[1616175, 65147, 58, 1, 36, "Subsubsection",
   CellTags->"QuarkField"],
  Cell[1616258, 65152, 87, 2, 26, "Input",
   CellTags->"QuarkField"],
  Cell[1616348, 65156, 112, 3, 26, "Output",
   CellTags->"QuarkField"]},
 "QuarkGluonVertex"->{
  Cell[1616487, 65163, 69, 1, 39, "ObjectName",
   CellTags->"QuarkGluonVertex"],
  Cell[1616581, 65168, 67, 1, 36, "Subsubsection",
   CellTags->"QuarkGluonVertex"],
  Cell[1616651, 65171, 260, 9, 24, "Text",
   CellTags->"QuarkGluonVertex"],
  Cell[1616936, 65184, 132, 3, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1617071, 65189, 491, 13, 26, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1617577, 65205, 281, 11, 24, "Text",
   CellTags->"QuarkGluonVertex"],
  Cell[1617895, 65221, 64, 1, 36, "Subsubsection",
   CellTags->"QuarkGluonVertex"],
  Cell[1617984, 65226, 153, 4, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1618140, 65232, 149, 4, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1618326, 65241, 208, 5, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1618537, 65248, 149, 4, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1618723, 65257, 208, 5, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1618934, 65264, 149, 4, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1619120, 65273, 208, 5, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1619331, 65280, 149, 4, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1619517, 65289, 315, 9, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1619835, 65300, 366, 11, 28, "Output",
   CellTags->"QuarkGluonVertex"],
  Cell[1620238, 65316, 316, 9, 26, "Input",
   CellTags->"QuarkGluonVertex"],
  Cell[1620557, 65327, 367, 11, 28, "Output",
   CellTags->"QuarkGluonVertex"]},
 "QuarkMass"->{
  Cell[1620951, 65342, 55, 1, 39, "ObjectName",
   CellTags->"QuarkMass"],
  Cell[1621031, 65347, 60, 1, 36, "Subsubsection",
   CellTags->"QuarkMass"],
  Cell[1621094, 65350, 76, 1, 24, "Text",
   CellTags->"QuarkMass"],
  Cell[1621173, 65353, 266, 11, 24, "Text",
   CellTags->"QuarkMass"]},
 "QuarkPropagator"->{
  Cell[1621454, 65367, 67, 1, 39, "ObjectName",
   CellTags->"QuarkPropagator"],
  Cell[1621546, 65372, 66, 1, 36, "Subsubsection",
   CellTags->"QuarkPropagator"],
  Cell[1621615, 65375, 174, 4, 24, "Text",
   CellTags->"QuarkPropagator"],
  Cell[1621814, 65383, 130, 3, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1621947, 65388, 534, 14, 26, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1622496, 65405, 520, 18, 24, "Text",
   CellTags->"QuarkPropagator"],
  Cell[1623053, 65428, 63, 1, 36, "Subsubsection",
   CellTags->"QuarkPropagator"],
  Cell[1623141, 65433, 124, 3, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1623268, 65438, 177, 6, 30, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1623482, 65449, 171, 5, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1623656, 65456, 177, 6, 30, "Output",
   CellTags->"QuarkPropagator"],
  Cell[1623870, 65467, 157, 4, 26, "Input",
   CellTags->"QuarkPropagator"],
  Cell[1624030, 65473, 177, 6, 30, "Output",
   CellTags->"QuarkPropagator"]},
 "ReduceGamma"->{
  Cell[1624234, 65483, 59, 1, 39, "ObjectName",
   CellTags->"ReduceGamma"],
  Cell[1624318, 65488, 62, 1, 36, "Subsubsection",
   CellTags->"ReduceGamma"],
  Cell[1624383, 65491, 194, 4, 24, "Text",
   CellTags->"ReduceGamma"],
  Cell[1624580, 65497, 260, 11, 24, "Text",
   CellTags->"ReduceGamma"]},
 "ReduceToScalars"->{
  Cell[1624855, 65511, 67, 1, 39, "ObjectName",
   CellTags->"ReduceToScalars"],
  Cell[1624947, 65516, 66, 1, 36, "Subsubsection",
   CellTags->"ReduceToScalars"],
  Cell[1625016, 65519, 210, 5, 41, "Text",
   CellTags->"ReduceToScalars"],
  Cell[1625229, 65526, 464, 18, 24, "Text",
   CellTags->"ReduceToScalars"]},
 "Rename"->{
  Cell[1625708, 65547, 49, 1, 39, "ObjectName",
   CellTags->"Rename"],
  Cell[1625782, 65552, 57, 1, 36, "Subsubsection",
   CellTags->"Rename"],
  Cell[1625842, 65555, 144, 4, 24, "Text",
   CellTags->"Rename"],
  Cell[1625989, 65561, 259, 11, 24, "Text",
   CellTags->"Rename"]},
 "RHI"->{
  Cell[1626263, 65575, 61, 1, 39, "ObjectName",
   CellTags->"RHI"],
  Cell[1626349, 65580, 54, 1, 36, "Subsubsection",
   CellTags->"RHI"],
  Cell[1626406, 65583, 611, 12, 141, "Text",
   CellTags->"RHI"],
  Cell[1627020, 65597, 248, 11, 24, "Text",
   CellTags->"RHI"],
  Cell[1627305, 65613, 51, 1, 36, "Subsubsection",
   CellTags->"RHI"],
  Cell[1627359, 65616, 70, 2, 26, "Input",
   CellTags->"RHI"]},
 "RHI2FC"->{
  Cell[1627444, 65621, 67, 1, 39, "ObjectName",
   CellTags->"RHI2FC"],
  Cell[1627536, 65626, 57, 1, 36, "Subsubsection",
   CellTags->"RHI2FC"],
  Cell[1627596, 65629, 614, 12, 141, "Text",
   CellTags->"RHI2FC"],
  Cell[1628213, 65643, 251, 11, 24, "Text",
   CellTags->"RHI2FC"],
  Cell[1628501, 65659, 54, 1, 36, "Subsubsection",
   CellTags->"RHI2FC"],
  Cell[1628558, 65662, 73, 2, 26, "Input",
   CellTags->"RHI2FC"]},
 "RHM"->{
  Cell[1628646, 65667, 61, 1, 39, "ObjectName",
   CellTags->"RHM"],
  Cell[1628732, 65672, 54, 1, 36, "Subsubsection",
   CellTags->"RHM"],
  Cell[1628789, 65675, 77, 1, 24, "Text",
   CellTags->"RHM"],
  Cell[1628869, 65678, 236, 11, 24, "Text",
   CellTags->"RHM"],
  Cell[1629142, 65694, 51, 1, 36, "Subsubsection",
   CellTags->"RHM"],
  Cell[1629196, 65697, 70, 2, 26, "Input",
   CellTags->"RHM"]},
 "RHO"->{
  Cell[1629281, 65702, 61, 1, 39, "ObjectName",
   CellTags->"RHO"],
  Cell[1629367, 65707, 54, 1, 36, "Subsubsection",
   CellTags->"RHO"],
  Cell[1629424, 65710, 267, 5, 41, "Text",
   CellTags->"RHO"],
  Cell[1629694, 65717, 236, 11, 24, "Text",
   CellTags->"RHO"],
  Cell[1629967, 65733, 51, 1, 36, "Subsubsection",
   CellTags->"RHO"],
  Cell[1630021, 65736, 70, 2, 26, "Input",
   CellTags->"RHO"]},
 "RHP"->{
  Cell[1630106, 65741, 45, 1, 39, "ObjectName",
   CellTags->"RHP"],
  Cell[1630176, 65746, 54, 1, 36, "Subsubsection",
   CellTags->"RHP"],
  Cell[1630233, 65749, 272, 5, 41, "Text",
   CellTags->"RHP"],
  Cell[1630508, 65756, 236, 11, 24, "Text",
   CellTags->"RHP"],
  Cell[1630781, 65772, 51, 1, 36, "Subsubsection",
   CellTags->"RHP"],
  Cell[1636099, 66002, 70, 2, 26, "Input",
   CellTags->"RHP"]},
 "RTL"->{
  Cell[1636184, 66007, 61, 1, 39, "ObjectName",
   CellTags->"RTL"],
  Cell[1636270, 66012, 54, 1, 36, "Subsubsection",
   CellTags->"RTL"],
  Cell[1636327, 66015, 188, 4, 24, "Text",
   CellTags->"RTL"],
  Cell[1636518, 66021, 424, 18, 24, "Text",
   CellTags->"RTL"],
  Cell[1636979, 66044, 51, 1, 36, "Subsubsection",
   CellTags->"RTL"],
  Cell[1637033, 66047, 70, 2, 26, "Input",
   CellTags->"RTL"]},
 "RightPartialD"->{
  Cell[1637118, 66052, 63, 1, 39, "ObjectName",
   CellTags->"RightPartialD"],
  Cell[1637206, 66057, 64, 1, 36, "Subsubsection",
   CellTags->"RightPartialD"],
  Cell[1637273, 66060, 256, 10, 27, "Text",
   CellTags->"RightPartialD"],
  Cell[1637532, 66072, 690, 25, 24, "Text",
   CellTags->"RightPartialD"],
  Cell[1638259, 66102, 61, 1, 36, "Subsubsection",
   CellTags->"RightPartialD"],
  Cell[1638345, 66107, 124, 3, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1638472, 66112, 252, 9, 33, "Output",
   CellTags->"RightPartialD"],
  Cell[1638761, 66126, 255, 7, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1639019, 66135, 308, 11, 33, "Output",
   CellTags->"RightPartialD"],
  Cell[1639364, 66151, 121, 3, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1639488, 66156, 356, 14, 30, "Output",
   CellTags->"RightPartialD"],
  Cell[1639881, 66175, 115, 3, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1639999, 66180, 292, 7, 38, "Output",
   CellTags->"RightPartialD"],
  Cell[1640328, 66192, 162, 5, 26, "Input",
   CellTags->"RightPartialD"],
  Cell[1640493, 66199, 178, 4, 38, "Output",
   CellTags->"RightPartialD"]},
 "RussianTrick"->{
  Cell[1640698, 66207, 61, 1, 39, "ObjectName",
   CellTags->"RussianTrick"],
  Cell[1640784, 66212, 63, 1, 36, "Subsubsection",
   CellTags->"RussianTrick"],
  Cell[1640850, 66215, 318, 6, 41, "Text",
   CellTags->"RussianTrick"],
  Cell[1641171, 66223, 90, 1, 24, "Text",
   CellTags->"RussianTrick"],
  Cell[1641264, 66226, 501, 18, 24, "Text",
   CellTags->"RussianTrick"],
  Cell[1641802, 66249, 60, 1, 36, "Subsubsection",
   CellTags->"RussianTrick"],
  Cell[1641887, 66254, 374, 12, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1642264, 66268, 501, 21, 28, "Output",
   CellTags->"RussianTrick"],
  Cell[1642802, 66294, 108, 3, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1642913, 66299, 661, 23, 49, "Output",
   CellTags->"RussianTrick"],
  Cell[1643611, 66327, 172, 5, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1643786, 66334, 3469, 121, 141, "Output",
   CellTags->"RussianTrick"],
  Cell[1647292, 66460, 112, 3, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1647407, 66465, 2675, 111, 28, "Output",
   CellTags->"RussianTrick"],
  Cell[1650119, 66581, 135, 4, 26, "Input",
   CellTags->"RussianTrick"],
  Cell[1650257, 66587, 2947, 112, 45, "Output",
   CellTags->"RussianTrick"],
  Cell[1653219, 66702, 111, 3, 26, "Input",
   CellTags->"RussianTrick"]},
 "ScalarGluonVertex"->{
  Cell[1653345, 66708, 71, 1, 39, "ObjectName",
   CellTags->"ScalarGluonVertex"],
  Cell[1653441, 66713, 68, 1, 36, "Subsubsection",
   CellTags->"ScalarGluonVertex"],
  Cell[1653512, 66716, 430, 8, 74, "Text",
   CellTags->"ScalarGluonVertex"],
  Cell[1653967, 66728, 134, 3, 26, "Input",
   CellTags->"ScalarGluonVertex"],
  Cell[1654104, 66733, 271, 8, 26, "Output",
   CellTags->"ScalarGluonVertex"],
  Cell[1654424, 66747, 65, 1, 36, "Subsubsection",
   CellTags->"ScalarGluonVertex"],
  Cell[1654514, 66752, 274, 8, 26, "Input",
   CellTags->"ScalarGluonVertex"],
  Cell[1654791, 66762, 479, 19, 28, "Output",
   CellTags->"ScalarGluonVertex"]},
 "ScalarProduct"->{
  Cell[1655297, 66785, 63, 1, 39, "ObjectName",
   CellTags->"ScalarProduct"],
  Cell[1655385, 66790, 64, 1, 36, "Subsubsection",
   CellTags->"ScalarProduct"],
  Cell[1655452, 66793, 836, 21, 75, "Text",
   CellTags->"ScalarProduct"],
  Cell[1656291, 66816, 1468, 52, 24, "Text",
   CellTags->"ScalarProduct"],
  Cell[1657796, 66873, 61, 1, 36, "Subsubsection",
   CellTags->"ScalarProduct"],
  Cell[1657882, 66878, 143, 4, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1658028, 66884, 212, 8, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1658277, 66897, 186, 6, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1658466, 66905, 301, 11, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1658804, 66921, 143, 4, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1658950, 66927, 168, 6, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1659155, 66938, 120, 3, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1659278, 66943, 168, 6, 28, "Output",
   CellTags->"ScalarProduct"],
  Cell[1659483, 66954, 179, 5, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1659665, 66961, 221, 6, 38, "Output",
   CellTags->"ScalarProduct"],
  Cell[1659923, 66972, 231, 7, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1660157, 66981, 271, 8, 38, "Output",
   CellTags->"ScalarProduct"],
  Cell[1660465, 66994, 248, 8, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1660716, 67004, 131, 4, 40, "Output",
   CellTags->"ScalarProduct"],
  Cell[1660884, 67013, 295, 9, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1661182, 67024, 571, 23, 40, "Output",
   CellTags->"ScalarProduct"],
  Cell[1661790, 67052, 280, 9, 26, "Input",
   CellTags->"ScalarProduct"],
  Cell[1662073, 67063, 571, 23, 40, "Output",
   CellTags->"ScalarProduct"],
  Cell[1662659, 67089, 99, 2, 26, "Input",
   CellTags->"ScalarProduct"]},
 "Info3535453289-5096475"->{
  Cell[1664095, 67147, 179, 3, 38, "Print",
   CellTags->{"Info3535453289-5096475", "ScalarProductCancel"}]},
 "ScaleMu"->{
  Cell[1674830, 67565, 51, 1, 39, "ObjectName",
   CellTags->"ScaleMu"],
  Cell[1674906, 67570, 58, 1, 36, "Subsubsection",
   CellTags->"ScaleMu"],
  Cell[1674967, 67573, 128, 4, 24, "Text",
   CellTags->"ScaleMu"],
  Cell[1675120, 67581, 81, 2, 26, "Input",
   CellTags->"ScaleMu"],
  Cell[1675204, 67585, 108, 3, 26, "Output",
   CellTags->"ScaleMu"]},
 "Schouten"->{
  Cell[1675339, 67592, 53, 1, 39, "ObjectName",
   CellTags->"Schouten"],
  Cell[1675417, 67597, 59, 1, 36, "Subsubsection",
   CellTags->"Schouten"],
  Cell[1675479, 67600, 460, 8, 83, "Text",
   CellTags->"Schouten"],
  Cell[1675942, 67610, 455, 17, 24, "Text",
   CellTags->"Schouten"],
  Cell[1676434, 67632, 56, 1, 36, "Subsubsection",
   CellTags->"Schouten"],
  Cell[1676515, 67637, 1938, 54, 138, "Input",
   CellTags->"Schouten"],
  Cell[1678456, 67693, 15696, 628, 194, "Output",
   CellTags->"Schouten"],
  Cell[1694189, 68326, 106, 3, 26, "Input",
   CellTags->"Schouten"],
  Cell[1694298, 68331, 105, 3, 26, "Output",
   CellTags->"Schouten"]},
 "SD"->{
  Cell[1694517, 68342, 41, 1, 39, "ObjectName",
   CellTags->"SD"],
  Cell[1694583, 68347, 53, 1, 36, "Subsubsection",
   CellTags->"SD"],
  Cell[1694639, 68350, 265, 7, 41, "Text",
   CellTags->"SD"],
  Cell[1694907, 68359, 255, 11, 24, "Text",
   CellTags->"SD"],
  Cell[1695199, 68375, 50, 1, 36, "Subsubsection",
   CellTags->"SD"],
  Cell[1695274, 68380, 121, 4, 26, "Input",
   CellTags->"SD"],
  Cell[1695398, 68386, 147, 5, 26, "Output",
   CellTags->"SD"],
  Cell[1695582, 68396, 130, 4, 26, "Input",
   CellTags->"SD"],
  Cell[1695715, 68402, 214, 6, 38, "Output",
   CellTags->"SD"],
  Cell[1695966, 68413, 130, 4, 26, "Input",
   CellTags->"SD"],
  Cell[1696099, 68419, 136, 4, 38, "Output",
   CellTags->"SD"]},
 "SelectFree"->{
  Cell[1696262, 68427, 57, 1, 39, "ObjectName",
   CellTags->"SelectFree"],
  Cell[1696344, 68432, 61, 1, 36, "Subsubsection",
   CellTags->"SelectFree"],
  Cell[1696408, 68435, 255, 5, 41, "Text",
   CellTags->"SelectFree"],
  Cell[1696666, 68442, 77, 1, 24, "Text",
   CellTags->"SelectFree"],
  Cell[1696746, 68445, 467, 18, 24, "Text",
   CellTags->"SelectFree"],
  Cell[1697250, 68468, 58, 1, 36, "Subsubsection",
   CellTags->"SelectFree"],
  Cell[1697333, 68473, 211, 6, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1697547, 68481, 130, 4, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1697714, 68490, 166, 5, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1697883, 68497, 107, 3, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1698027, 68505, 275, 8, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1698305, 68515, 130, 4, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1698472, 68524, 137, 4, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1698612, 68530, 107, 3, 27, "Output",
   CellTags->"SelectFree"],
  Cell[1698756, 68538, 137, 4, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1698896, 68544, 107, 3, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1699040, 68552, 137, 4, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1699180, 68558, 107, 3, 26, "Output",
   CellTags->"SelectFree"],
  Cell[1699324, 68566, 166, 5, 26, "Input",
   CellTags->"SelectFree"],
  Cell[1699493, 68573, 107, 3, 26, "Output",
   CellTags->"SelectFree"]},
 "SelectGraphs"->{
  Cell[1699627, 68580, 61, 1, 39, "ObjectName",
   CellTags->"SelectGraphs"],
  Cell[1699713, 68585, 63, 1, 36, "Subsubsection",
   CellTags->"SelectGraphs"],
  Cell[1699779, 68588, 486, 8, 60, "Text",
   CellTags->"SelectGraphs"],
  Cell[1700268, 68598, 273, 11, 24, "Text",
   CellTags->"SelectGraphs"]},
 "SelectNotFree"->{
  Cell[1700556, 68612, 63, 1, 39, "ObjectName",
   CellTags->"SelectNotFree"],
  Cell[1700644, 68617, 64, 1, 36, "Subsubsection",
   CellTags->"SelectNotFree"],
  Cell[1700711, 68620, 143, 4, 24, "Text",
   CellTags->"SelectNotFree"],
  Cell[1700857, 68626, 270, 5, 41, "Text",
   CellTags->"SelectNotFree"],
  Cell[1701130, 68633, 83, 1, 24, "Text",
   CellTags->"SelectNotFree"],
  Cell[1701216, 68636, 458, 18, 24, "Text",
   CellTags->"SelectNotFree"],
  Cell[1701711, 68659, 61, 1, 36, "Subsubsection",
   CellTags->"SelectNotFree"],
  Cell[1701797, 68664, 207, 6, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1702007, 68672, 162, 5, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1702206, 68682, 276, 8, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1702485, 68692, 172, 5, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1702694, 68702, 143, 4, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1702840, 68708, 110, 3, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1702987, 68716, 167, 5, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1703157, 68723, 110, 3, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1703304, 68731, 143, 4, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1703450, 68737, 110, 3, 27, "Output",
   CellTags->"SelectNotFree"],
  Cell[1703597, 68745, 143, 4, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1703743, 68751, 110, 3, 26, "Output",
   CellTags->"SelectNotFree"],
  Cell[1703890, 68759, 172, 5, 26, "Input",
   CellTags->"SelectNotFree"],
  Cell[1704065, 68766, 138, 4, 26, "Output",
   CellTags->"SelectNotFree"]},
 "SelectSplit"->{
  Cell[1704230, 68774, 59, 1, 39, "ObjectName",
   CellTags->"SelectSplit"],
  Cell[1704314, 68779, 62, 1, 36, "Subsubsection",
   CellTags->"SelectSplit"],
  Cell[1704379, 68782, 249, 5, 41, "Text",
   CellTags->"SelectSplit"],
  Cell[1704665, 68792, 59, 1, 36, "Subsubsection",
   CellTags->"SelectSplit"],
  Cell[1704749, 68797, 804, 26, 29, "Input",
   CellTags->"SelectSplit"],
  Cell[1705556, 68825, 532, 19, 29, "Output",
   CellTags->"SelectSplit"],
  Cell[1706125, 68849, 652, 21, 29, "Input",
   CellTags->"SelectSplit"],
  Cell[1706780, 68872, 495, 16, 29, "Output",
   CellTags->"SelectSplit"]},
 "Select1"->{
  Cell[1707302, 68892, 51, 1, 39, "ObjectName",
   CellTags->"Select1"],
  Cell[1707378, 68897, 58, 1, 36, "Subsubsection",
   CellTags->"Select1"],
  Cell[1707439, 68900, 243, 5, 41, "Text",
   CellTags->"Select1"],
  Cell[1707685, 68907, 256, 11, 24, "Text",
   CellTags->"Select1"]},
 "Select2"->{
  Cell[1707956, 68921, 51, 1, 39, "ObjectName",
   CellTags->"Select2"],
  Cell[1708032, 68926, 58, 1, 36, "Subsubsection",
   CellTags->"Select2"],
  Cell[1708093, 68929, 245, 5, 41, "Text",
   CellTags->"Select2"],
  Cell[1708341, 68936, 256, 11, 24, "Text",
   CellTags->"Select2"]},
 "Series2"->{
  Cell[1708612, 68950, 51, 1, 39, "ObjectName",
   CellTags->"Series2"],
  Cell[1708688, 68955, 58, 1, 36, "Subsubsection",
   CellTags->"Series2"],
  Cell[1708749, 68958, 383, 9, 41, "Text",
   CellTags->"Series2"],
  Cell[1709157, 68971, 114, 3, 26, "Input",
   CellTags->"Series2"],
  Cell[1709274, 68976, 528, 15, 26, "Output",
   CellTags->"Series2"],
  Cell[1709851, 68997, 55, 1, 36, "Subsubsection",
   CellTags->"Series2"],
  Cell[1709931, 69002, 339, 12, 26, "Input",
   CellTags->"Series2"],
  Cell[1710273, 69016, 357, 12, 42, "Output",
   CellTags->"Series2"],
  Cell[1710667, 69033, 174, 5, 26, "Input",
   CellTags->"Series2"],
  Cell[1710844, 69040, 242, 8, 44, "Output",
   CellTags->"Series2"],
  Cell[1711123, 69053, 222, 7, 26, "Input",
   CellTags->"Series2"],
  Cell[1711348, 69062, 806, 25, 44, "Output",
   CellTags->"Series2"],
  Cell[1712191, 69092, 174, 5, 26, "Input",
   CellTags->"Series2"],
  Cell[1712368, 69099, 385, 14, 46, "Output",
   CellTags->"Series2"],
  Cell[1712790, 69118, 290, 8, 26, "Input",
   CellTags->"Series2"],
  Cell[1713083, 69128, 994, 32, 45, "Output",
   CellTags->"Series2"],
  Cell[1714304, 69173, 460, 15, 29, "Input",
   CellTags->"Series2"],
  Cell[1714767, 69190, 1000, 33, 43, "Output",
   CellTags->"Series2"],
  Cell[1715782, 69226, 113, 3, 24, "Text",
   CellTags->"Series2"],
  Cell[1715920, 69233, 484, 15, 26, "Input",
   CellTags->"Series2"],
  Cell[1716407, 69250, 648, 24, 45, "Output",
   CellTags->"Series2"],
  Cell[1717092, 69279, 578, 17, 46, "Input",
   CellTags->"Series2"],
  Cell[1717673, 69298, 988, 35, 42, "Output",
   CellTags->"Series2"],
  Cell[1718698, 69338, 242, 6, 26, "Input",
   CellTags->"Series2"],
  Cell[1718943, 69346, 292, 7, 26, "Output",
   CellTags->"Series2"],
  Cell[1719272, 69358, 147, 4, 26, "Input",
   CellTags->"Series2"],
  Cell[1719422, 69364, 3114, 106, 102, "Output",
   CellTags->"Series2"],
  Cell[1722551, 69473, 351, 11, 41, "Text",
   CellTags->"Series2"]},
 "Series3"->{
  Cell[1722917, 69487, 51, 1, 39, "ObjectName",
   CellTags->"Series3"],
  Cell[1722993, 69492, 58, 1, 36, "Subsubsection",
   CellTags->"Series3"],
  Cell[1723054, 69495, 267, 5, 41, "Text",
   CellTags->"Series3"],
  Cell[1723346, 69504, 114, 3, 26, "Input",
   CellTags->"Series3"],
  Cell[1723463, 69509, 262, 8, 26, "Output",
   CellTags->"Series3"],
  Cell[1723740, 69520, 256, 11, 24, "Text",
   CellTags->"Series3"],
  Cell[1724033, 69536, 55, 1, 36, "Subsubsection",
   CellTags->"Series3"],
  Cell[1724113, 69541, 339, 12, 26, "Input",
   CellTags->"Series3"],
  Cell[1724455, 69555, 307, 11, 42, "Output",
   CellTags->"Series3"],
  Cell[1724799, 69571, 214, 7, 26, "Input",
   CellTags->"Series3"],
  Cell[1725016, 69580, 215, 7, 44, "Output",
   CellTags->"Series3"]},
 "SetMandelstam"->{
  Cell[1725258, 69591, 63, 1, 39, "ObjectName",
   CellTags->"SetMandelstam"],
  Cell[1725346, 69596, 64, 1, 36, "Subsubsection",
   CellTags->"SetMandelstam"],
  Cell[1725413, 69599, 462, 9, 83, "Text",
   CellTags->"SetMandelstam"],
  Cell[1725900, 69612, 126, 3, 26, "Input",
   CellTags->"SetMandelstam"],
  Cell[1726029, 69617, 237, 8, 26, "Output",
   CellTags->"SetMandelstam"],
  Cell[1726281, 69628, 274, 11, 24, "Text",
   CellTags->"SetMandelstam"]},
 "SimplifyDeltaFunction"->{
  Cell[1726570, 69642, 79, 1, 39, "ObjectName",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1726674, 69647, 72, 1, 36, "Subsubsection",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1726749, 69650, 299, 6, 41, "Text",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1727051, 69658, 526, 18, 24, "Text",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1727614, 69681, 69, 1, 36, "Subsubsection",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1727708, 69686, 203, 6, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1727914, 69694, 268, 9, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1728219, 69708, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1728386, 69714, 392, 13, 37, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1728815, 69732, 203, 6, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1729021, 69740, 301, 10, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1729359, 69755, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1729526, 69761, 812, 27, 37, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1730375, 69793, 220, 6, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1730598, 69801, 313, 10, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1730948, 69816, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1731115, 69822, 212, 7, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1731364, 69834, 263, 8, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1731630, 69844, 330, 11, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1731997, 69860, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1732164, 69866, 234, 8, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1732435, 69879, 298, 9, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1732736, 69890, 371, 12, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733144, 69907, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733311, 69913, 118, 3, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733466, 69921, 263, 8, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1733732, 69931, 330, 11, 26, "Output",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1734099, 69947, 164, 4, 26, "Input",
   CellTags->"SimplifyDeltaFunction"],
  Cell[1734266, 69953, 234, 8, 26, "Output",
   CellTags->"SimplifyDeltaFunction"]},
 "SimplifyGTI"->{
  Cell[1734527, 69965, 77, 1, 39, "ObjectName",
   CellTags->"SimplifyGTI"],
  Cell[1734629, 69970, 62, 1, 36, "Subsubsection",
   CellTags->"SimplifyGTI"],
  Cell[1734694, 69973, 71, 1, 24, "Text",
   CellTags->"SimplifyGTI"],
  Cell[1734768, 69976, 244, 11, 24, "Text",
   CellTags->"SimplifyGTI"],
  Cell[1735049, 69992, 59, 1, 36, "Subsubsection",
   CellTags->"SimplifyGTI"],
  Cell[1735111, 69995, 78, 2, 26, "Input",
   CellTags->"SimplifyGTI"]},
 "SimplifyPolyLog"->{
  Cell[1735204, 70000, 67, 1, 39, "ObjectName",
   CellTags->"SimplifyPolyLog"],
  Cell[1735296, 70005, 66, 1, 36, "Subsubsection",
   CellTags->"SimplifyPolyLog"],
  Cell[1735365, 70008, 199, 4, 24, "Text",
   CellTags->"SimplifyPolyLog"],
  Cell[1735567, 70014, 264, 11, 24, "Text",
   CellTags->"SimplifyPolyLog"],
  Cell[1735868, 70030, 63, 1, 36, "Subsubsection",
   CellTags->"SimplifyPolyLog"],
  Cell[1735934, 70033, 224, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1736183, 70043, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1736381, 70051, 664, 20, 43, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1737082, 70076, 170, 5, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1737255, 70083, 432, 14, 26, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1737724, 70102, 221, 7, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1737948, 70111, 631, 20, 29, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1738616, 70136, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1738814, 70144, 608, 19, 29, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1739459, 70168, 271, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1739733, 70179, 670, 22, 42, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1740440, 70206, 248, 8, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1740691, 70216, 645, 21, 42, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1741373, 70242, 281, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1741657, 70253, 732, 29, 42, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1742426, 70287, 216, 7, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1742645, 70296, 496, 17, 46, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1743178, 70318, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1743376, 70326, 115, 3, 26, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1743528, 70334, 195, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1743726, 70342, 1063, 36, 29, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1744826, 70383, 271, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1745100, 70394, 1480, 50, 77, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1746617, 70449, 221, 7, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1746841, 70458, 1255, 45, 46, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1748133, 70508, 271, 9, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1748407, 70519, 2740, 96, 115, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1751184, 70620, 191, 6, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1751378, 70628, 336, 12, 45, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1751751, 70645, 166, 5, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1751920, 70652, 251, 8, 43, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1752208, 70665, 146, 4, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1752357, 70671, 406, 14, 43, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1752800, 70690, 146, 4, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1752949, 70696, 355, 13, 42, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1753341, 70714, 146, 4, 26, "Input",
   CellTags->"SimplifyPolyLog"],
  Cell[1753490, 70720, 355, 13, 42, "Output",
   CellTags->"SimplifyPolyLog"],
  Cell[1753860, 70736, 116, 3, 26, "Input",
   CellTags->"SimplifyPolyLog"]},
 "Simplify2"->{
  Cell[1753991, 70742, 73, 1, 39, "ObjectName",
   CellTags->"Simplify2"],
  Cell[1754089, 70747, 60, 1, 36, "Subsubsection",
   CellTags->"Simplify2"],
  Cell[1754152, 70750, 81, 1, 24, "Text",
   CellTags->"Simplify2"],
  Cell[1754270, 70756, 57, 1, 36, "Subsubsection",
   CellTags->"Simplify2"],
  Cell[1754330, 70759, 76, 2, 26, "Input",
   CellTags->"Simplify2"]},
 "SmallDelta"->{
  Cell[1754421, 70764, 57, 1, 39, "ObjectName",
   CellTags->"SmallDelta"],
  Cell[1754503, 70769, 61, 1, 36, "Subsubsection",
   CellTags->"SmallDelta"],
  Cell[1754567, 70772, 87, 1, 24, "Text",
   CellTags->"SmallDelta"]},
 "SmallEpsilon"->{
  Cell[1754669, 70776, 61, 1, 39, "ObjectName",
   CellTags->"SmallEpsilon"],
  Cell[1754755, 70781, 63, 1, 36, "Subsubsection",
   CellTags->"SmallEpsilon"],
  Cell[1754821, 70784, 91, 1, 24, "Text",
   CellTags->"SmallEpsilon"]},
 "SmallVariable"->{
  Cell[1754927, 70788, 63, 1, 39, "ObjectName",
   CellTags->"SmallVariable"],
  Cell[1755015, 70793, 64, 1, 36, "Subsubsection",
   CellTags->"SmallVariable"],
  Cell[1755082, 70796, 281, 5, 41, "Text",
   CellTags->"SmallVariable"],
  Cell[1755366, 70803, 694, 25, 24, "Text",
   CellTags->"SmallVariable"]},
 "SmallVariables"->{
  Cell[1756075, 70831, 65, 1, 39, "ObjectName",
   CellTags->"SmallVariables"],
  Cell[1756165, 70836, 65, 1, 36, "Subsubsection",
   CellTags->"SmallVariables"],
  Cell[1756233, 70839, 225, 5, 41, "Text",
   CellTags->"SmallVariables"],
  Cell[1756461, 70846, 263, 11, 24, "Text",
   CellTags->"SmallVariables"]},
 "SMP"->{
  Cell[1756739, 70860, 43, 1, 39, "ObjectName",
   CellTags->"SMP"],
  Cell[1756807, 70865, 54, 1, 36, "Subsubsection",
   CellTags->"SMP"],
  Cell[1756864, 70868, 232, 5, 41, "Text",
   CellTags->"SMP"],
  Cell[1757121, 70877, 127, 4, 26, "Input",
   CellTags->"SMP"],
  Cell[1757251, 70883, 166, 5, 38, "Output",
   CellTags->"SMP"],
  Cell[1757432, 70891, 250, 10, 24, "Text",
   CellTags->"SMP"],
  Cell[1757719, 70906, 51, 1, 36, "Subsubsection",
   CellTags->"SMP"],
  Cell[1757795, 70911, 109, 3, 26, "Input",
   CellTags->"SMP"],
  Cell[1757907, 70916, 101, 3, 26, "Output",
   CellTags->"SMP"],
  Cell[1771751, 71529, 242, 10, 24, "Text",
   CellTags->"SMP"]},
 "SMVertex"->{
  Cell[1758035, 70923, 53, 1, 39, "ObjectName",
   CellTags->"SMVertex"],
  Cell[1758113, 70928, 59, 1, 36, "Subsubsection",
   CellTags->"SMVertex"],
  Cell[1758175, 70931, 304, 6, 41, "Text",
   CellTags->"SMVertex"],
  Cell[1758504, 70941, 112, 3, 26, "Input",
   CellTags->"SMVertex"],
  Cell[1758619, 70946, 235, 8, 26, "Output",
   CellTags->"SMVertex"],
  Cell[1758903, 70960, 56, 1, 36, "Subsubsection",
   CellTags->"SMVertex"],
  Cell[1758984, 70965, 223, 6, 26, "Input",
   CellTags->"SMVertex"],
  Cell[1759210, 70973, 1179, 44, 28, "Output",
   CellTags->"SMVertex"]},
 "Smu"->{
  Cell[1760416, 71021, 61, 1, 39, "ObjectName",
   CellTags->"Smu"],
  Cell[1760502, 71026, 54, 1, 36, "Subsubsection",
   CellTags->"Smu"],
  Cell[1760559, 71029, 45, 1, 24, "Text",
   CellTags->"Smu"]},
 "Sn"->{
  Cell[1760619, 71033, 41, 1, 39, "ObjectName",
   CellTags->"Sn"],
  Cell[1760685, 71038, 53, 1, 36, "Subsubsection",
   CellTags->"Sn"],
  Cell[1760741, 71041, 300, 14, 24, "Text",
   CellTags->"Sn"]},
 "SO"->{
  Cell[1761056, 71058, 41, 1, 39, "ObjectName",
   CellTags->"SO"],
  Cell[1761122, 71063, 53, 1, 36, "Subsubsection",
   CellTags->"SO"],
  Cell[1761178, 71066, 173, 4, 24, "Text",
   CellTags->"SO"],
  Cell[1761354, 71072, 981, 38, 24, "Text",
   CellTags->"SO"],
  Cell[1762372, 71115, 50, 1, 36, "Subsubsection",
   CellTags->"SO"],
  Cell[1762447, 71120, 98, 3, 26, "Input",
   CellTags->"SO"],
  Cell[1762548, 71125, 265, 10, 28, "Output",
   CellTags->"SO"],
  Cell[1762850, 71140, 121, 4, 26, "Input",
   CellTags->"SO"],
  Cell[1762974, 71146, 384, 15, 28, "Output",
   CellTags->"SO"],
  Cell[1763395, 71166, 163, 6, 26, "Input",
   CellTags->"SO"],
  Cell[1763561, 71174, 217, 6, 38, "Output",
   CellTags->"SO"]},
 "SOD"->{
  Cell[1763805, 71184, 43, 1, 39, "ObjectName",
   CellTags->"SOD"],
  Cell[1763873, 71189, 54, 1, 36, "Subsubsection",
   CellTags->"SOD"],
  Cell[1763930, 71192, 189, 4, 41, "Text",
   CellTags->"SOD"],
  Cell[1764122, 71198, 816, 32, 24, "Text",
   CellTags->"SOD"],
  Cell[1764975, 71235, 51, 1, 36, "Subsubsection",
   CellTags->"SOD"],
  Cell[1765051, 71240, 100, 3, 26, "Input",
   CellTags->"SOD"],
  Cell[1765154, 71245, 266, 10, 28, "Output",
   CellTags->"SOD"],
  Cell[1765457, 71260, 123, 4, 26, "Input",
   CellTags->"SOD"],
  Cell[1765583, 71266, 385, 15, 28, "Output",
   CellTags->"SOD"],
  Cell[1766005, 71286, 165, 6, 26, "Input",
   CellTags->"SOD"],
  Cell[1766173, 71294, 268, 8, 38, "Output",
   CellTags->"SOD"]},
 "Solve2"->{
  Cell[1766468, 71306, 49, 1, 39, "ObjectName",
   CellTags->"Solve2"],
  Cell[1766542, 71311, 57, 1, 36, "Subsubsection",
   CellTags->"Solve2"],
  Cell[1766602, 71314, 346, 7, 41, "Text",
   CellTags->"Solve2"],
  Cell[1766973, 71325, 112, 3, 26, "Input",
   CellTags->"Solve2"],
  Cell[1767088, 71330, 264, 8, 26, "Output",
   CellTags->"Solve2"],
  Cell[1767401, 71344, 54, 1, 36, "Subsubsection",
   CellTags->"Solve2"],
  Cell[1767480, 71349, 390, 14, 26, "Input",
   CellTags->"Solve2"],
  Cell[1767873, 71365, 373, 13, 44, "Output",
   CellTags->"Solve2"],
  Cell[1768261, 71381, 111, 3, 24, "Text",
   CellTags->"Solve2"],
  Cell[1768397, 71388, 153, 5, 26, "Input",
   CellTags->"Solve2"],
  Cell[1768553, 71395, 176, 6, 26, "Output",
   CellTags->"Solve2"],
  Cell[1768766, 71406, 267, 8, 26, "Input",
   CellTags->"Solve2"],
  Cell[1769036, 71416, 176, 6, 26, "Output",
   CellTags->"Solve2"],
  Cell[1769249, 71427, 443, 15, 26, "Input",
   CellTags->"Solve2"],
  Cell[1769695, 71444, 314, 11, 45, "Output",
   CellTags->"Solve2"],
  Cell[1770046, 71460, 389, 14, 26, "Input",
   CellTags->"Solve2"],
  Cell[1770438, 71476, 406, 14, 44, "Output",
   CellTags->"Solve2"]},
 "Solve3"->{
  Cell[1770871, 71494, 49, 1, 39, "ObjectName",
   CellTags->"Solve3"],
  Cell[1770945, 71499, 57, 1, 36, "Subsubsection",
   CellTags->"Solve3"],
  Cell[1771005, 71502, 271, 5, 41, "Text",
   CellTags->"Solve3"],
  Cell[1771301, 71511, 112, 3, 26, "Input",
   CellTags->"Solve3"],
  Cell[1771416, 71516, 320, 10, 26, "Output",
   CellTags->"Solve3"],
  Cell[1772030, 71544, 54, 1, 36, "Subsubsection",
   CellTags->"Solve3"],
  Cell[1772109, 71549, 390, 14, 26, "Input",
   CellTags->"Solve3"],
  Cell[1772502, 71565, 373, 13, 44, "Output",
   CellTags->"Solve3"]},
 "SP"->{
  Cell[1772902, 71582, 41, 1, 39, "ObjectName",
   CellTags->"SP"],
  Cell[1772968, 71587, 53, 1, 36, "Subsubsection",
   CellTags->"SP"],
  Cell[1773024, 71590, 327, 11, 43, "Text",
   CellTags->"SP"],
  Cell[1773354, 71603, 687, 25, 24, "Text",
   CellTags->"SP"],
  Cell[1774078, 71633, 50, 1, 36, "Subsubsection",
   CellTags->"SP"],
  Cell[1774153, 71638, 184, 6, 26, "Input",
   CellTags->"SP"],
  Cell[1774340, 71646, 366, 16, 28, "Output",
   CellTags->"SP"],
  Cell[1774743, 71667, 189, 7, 26, "Input",
   CellTags->"SP"],
  Cell[1774935, 71676, 297, 11, 28, "Output",
   CellTags->"SP"],
  Cell[1775269, 71692, 234, 8, 26, "Input",
   CellTags->"SP"],
  Cell[1775506, 71702, 412, 17, 28, "Output",
   CellTags->"SP"],
  Cell[1775955, 71724, 168, 5, 26, "Input",
   CellTags->"SP"],
  Cell[1776126, 71731, 415, 17, 28, "Output",
   CellTags->"SP"],
  Cell[1776578, 71753, 157, 5, 26, "Input",
   CellTags->"SP"],
  Cell[1776738, 71760, 136, 4, 38, "Output",
   CellTags->"SP"],
  Cell[1776911, 71769, 188, 7, 26, "Input",
   CellTags->"SP"],
  Cell[1777102, 71778, 210, 6, 38, "Output",
   CellTags->"SP"],
  Cell[1777349, 71789, 217, 8, 26, "Input",
   CellTags->"SP"],
  Cell[1777569, 71799, 136, 4, 38, "Output",
   CellTags->"SP"]},
 "SPC"->{
  Cell[1777732, 71807, 43, 1, 39, "ObjectName",
   CellTags->"SPC"],
  Cell[1777800, 71812, 54, 1, 36, "Subsubsection",
   CellTags->"SPC"],
  Cell[1777857, 71815, 81, 1, 24, "Text",
   CellTags->"SPC"],
  Cell[1777941, 71818, 300, 11, 24, "Text",
   CellTags->"SPC"]},
 "SPD"->{
  Cell[1778256, 71832, 43, 1, 39, "ObjectName",
   CellTags->"SPD"],
  Cell[1778324, 71837, 54, 1, 36, "Subsubsection",
   CellTags->"SPD"],
  Cell[1778381, 71840, 346, 11, 43, "Text",
   CellTags->"SPD"],
  Cell[1778730, 71853, 856, 32, 24, "Text",
   CellTags->"SPD"],
  Cell[1779623, 71890, 51, 1, 36, "Subsubsection",
   CellTags->"SPD"],
  Cell[1779699, 71895, 187, 6, 26, "Input",
   CellTags->"SPD"],
  Cell[1779889, 71903, 367, 16, 28, "Output",
   CellTags->"SPD"],
  Cell[1780293, 71924, 191, 7, 26, "Input",
   CellTags->"SPD"],
  Cell[1780487, 71933, 298, 11, 28, "Output",
   CellTags->"SPD"],
  Cell[1780822, 71949, 236, 8, 26, "Input",
   CellTags->"SPD"],
  Cell[1781061, 71959, 413, 17, 28, "Output",
   CellTags->"SPD"],
  Cell[1781511, 71981, 170, 5, 26, "Input",
   CellTags->"SPD"],
  Cell[1781684, 71988, 416, 17, 28, "Output",
   CellTags->"SPD"],
  Cell[1782137, 72010, 159, 5, 26, "Input",
   CellTags->"SPD"],
  Cell[1782299, 72017, 138, 4, 38, "Output",
   CellTags->"SPD"],
  Cell[1782474, 72026, 190, 7, 26, "Input",
   CellTags->"SPD"],
  Cell[1782667, 72035, 261, 8, 38, "Output",
   CellTags->"SPD"],
  Cell[1782965, 72048, 219, 8, 26, "Input",
   CellTags->"SPD"],
  Cell[1783187, 72058, 138, 4, 38, "Output",
   CellTags->"SPD"],
  Cell[1783362, 72067, 270, 9, 26, "Input",
   CellTags->"SPD"],
  Cell[1783635, 72078, 138, 4, 38, "Output",
   CellTags->"SPD"]},
 "Spinor"->{
  Cell[1783800, 72086, 49, 1, 39, "ObjectName",
   CellTags->"Spinor"],
  Cell[1783874, 72091, 57, 1, 36, "Subsubsection",
   CellTags->"Spinor"],
  Cell[1783934, 72094, 1326, 49, 108, "Text",
   CellTags->"Spinor"],
  Cell[1785263, 72145, 1271, 46, 24, "Text",
   CellTags->"Spinor"],
  Cell[1786571, 72196, 54, 1, 36, "Subsubsection",
   CellTags->"Spinor"],
  Cell[1786650, 72201, 106, 3, 26, "Input",
   CellTags->"Spinor"],
  Cell[1786759, 72206, 175, 6, 28, "Output",
   CellTags->"Spinor"],
  Cell[1786971, 72217, 129, 4, 26, "Input",
   CellTags->"Spinor"],
  Cell[1787103, 72223, 185, 6, 28, "Output",
   CellTags->"Spinor"],
  Cell[1787303, 72232, 146, 4, 24, "Text",
   CellTags->"Spinor"],
  Cell[1787474, 72240, 250, 8, 26, "Input",
   CellTags->"Spinor"],
  Cell[1787727, 72250, 126, 4, 26, "Output",
   CellTags->"Spinor"],
  Cell[1787890, 72259, 254, 8, 26, "Input",
   CellTags->"Spinor"],
  Cell[1788147, 72269, 251, 9, 28, "Output",
   CellTags->"Spinor"],
  Cell[1788435, 72283, 141, 4, 26, "Input",
   CellTags->"Spinor"],
  Cell[1788579, 72289, 192, 6, 38, "Output",
   CellTags->"Spinor"],
  Cell[1788786, 72298, 158, 4, 24, "Text",
   CellTags->"Spinor"],
  Cell[1788969, 72306, 212, 7, 26, "Input",
   CellTags->"Spinor"],
  Cell[1789184, 72315, 192, 6, 38, "Output",
   CellTags->"Spinor"],
  Cell[1789413, 72326, 165, 5, 26, "Input",
   CellTags->"Spinor"],
  Cell[1789581, 72333, 192, 6, 38, "Output",
   CellTags->"Spinor"],
  Cell[1789788, 72342, 77, 1, 24, "Text",
   CellTags->"Spinor"],
  Cell[1789890, 72347, 210, 7, 26, "Input",
   CellTags->"Spinor"],
  Cell[1790103, 72356, 192, 6, 38, "Output",
   CellTags->"Spinor"]},
 "SpinorCollect"->{
  Cell[1790322, 72366, 63, 1, 39, "ObjectName",
   CellTags->"SpinorCollect"],
  Cell[1790410, 72371, 64, 1, 36, "Subsubsection",
   CellTags->"SpinorCollect"],
  Cell[1790477, 72374, 191, 4, 24, "Text",
   CellTags->"SpinorCollect"],
  Cell[1790671, 72380, 469, 17, 24, "Text",
   CellTags->"SpinorCollect"]},
 "SpinorUBar"->{
  Cell[1791155, 72400, 57, 1, 39, "ObjectName",
   CellTags->"SpinorUBar"],
  Cell[1791237, 72405, 61, 1, 36, "Subsubsection",
   CellTags->"SpinorUBar"],
  Cell[1791301, 72408, 170, 7, 24, "Text",
   CellTags->"SpinorUBar"],
  Cell[1791474, 72417, 831, 32, 24, "Text",
   CellTags->"SpinorUBar"],
  Cell[1792342, 72454, 58, 1, 36, "Subsubsection",
   CellTags->"SpinorUBar"],
  Cell[1792403, 72457, 85, 1, 24, "Text",
   CellTags->"SpinorUBar"],
  Cell[1792513, 72462, 114, 3, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1792630, 72467, 202, 7, 28, "Output",
   CellTags->"SpinorUBar"],
  Cell[1792869, 72479, 137, 4, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1793009, 72485, 212, 7, 28, "Output",
   CellTags->"SpinorUBar"],
  Cell[1793258, 72497, 173, 5, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1793434, 72504, 152, 4, 38, "Output",
   CellTags->"SpinorUBar"],
  Cell[1793623, 72513, 204, 7, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1793830, 72522, 196, 6, 38, "Output",
   CellTags->"SpinorUBar"],
  Cell[1794063, 72533, 233, 8, 26, "Input",
   CellTags->"SpinorUBar"],
  Cell[1794299, 72543, 196, 6, 38, "Output",
   CellTags->"SpinorUBar"]},
 "SpinorU"->{
  Cell[1794522, 72553, 51, 1, 39, "ObjectName",
   CellTags->"SpinorU"],
  Cell[1794598, 72558, 58, 1, 36, "Subsubsection",
   CellTags->"SpinorU"],
  Cell[1794659, 72561, 79, 1, 24, "Text",
   CellTags->"SpinorU"],
  Cell[1794741, 72564, 840, 32, 24, "Text",
   CellTags->"SpinorU"],
  Cell[1795618, 72601, 55, 1, 36, "Subsubsection",
   CellTags->"SpinorU"],
  Cell[1795698, 72606, 108, 3, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1795809, 72611, 132, 4, 28, "Output",
   CellTags->"SpinorU"],
  Cell[1795978, 72620, 131, 4, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1796112, 72626, 142, 4, 28, "Output",
   CellTags->"SpinorU"],
  Cell[1796291, 72635, 167, 5, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1796461, 72642, 146, 4, 38, "Output",
   CellTags->"SpinorU"],
  Cell[1796644, 72651, 198, 7, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1796845, 72660, 193, 6, 38, "Output",
   CellTags->"SpinorU"],
  Cell[1797075, 72671, 227, 8, 26, "Input",
   CellTags->"SpinorU"],
  Cell[1797305, 72681, 193, 6, 38, "Output",
   CellTags->"SpinorU"]},
 "SpinorVBar"->{
  Cell[1797525, 72691, 57, 1, 39, "ObjectName",
   CellTags->"SpinorVBar"],
  Cell[1797607, 72696, 61, 1, 36, "Subsubsection",
   CellTags->"SpinorVBar"],
  Cell[1797671, 72699, 175, 7, 24, "Text",
   CellTags->"SpinorVBar"],
  Cell[1797849, 72708, 831, 32, 24, "Text",
   CellTags->"SpinorVBar"],
  Cell[1798717, 72745, 58, 1, 36, "Subsubsection",
   CellTags->"SpinorVBar"],
  Cell[1798800, 72750, 114, 3, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1798917, 72755, 202, 7, 28, "Output",
   CellTags->"SpinorVBar"],
  Cell[1799156, 72767, 137, 4, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1799296, 72773, 212, 7, 28, "Output",
   CellTags->"SpinorVBar"],
  Cell[1799545, 72785, 173, 5, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1799721, 72792, 152, 4, 38, "Output",
   CellTags->"SpinorVBar"],
  Cell[1799910, 72801, 204, 7, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1800117, 72810, 217, 7, 38, "Output",
   CellTags->"SpinorVBar"],
  Cell[1800371, 72822, 233, 8, 26, "Input",
   CellTags->"SpinorVBar"],
  Cell[1800607, 72832, 217, 7, 38, "Output",
   CellTags->"SpinorVBar"]},
 "SpinorV"->{
  Cell[1800851, 72843, 51, 1, 39, "ObjectName",
   CellTags->"SpinorV"],
  Cell[1800927, 72848, 58, 1, 36, "Subsubsection",
   CellTags->"SpinorV"],
  Cell[1800988, 72851, 79, 1, 24, "Text",
   CellTags->"SpinorV"],
  Cell[1801070, 72854, 840, 32, 24, "Text",
   CellTags->"SpinorV"],
  Cell[1801947, 72891, 55, 1, 36, "Subsubsection",
   CellTags->"SpinorV"],
  Cell[1802027, 72896, 108, 3, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1802138, 72901, 132, 4, 28, "Output",
   CellTags->"SpinorV"],
  Cell[1802307, 72910, 131, 4, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1802441, 72916, 142, 4, 28, "Output",
   CellTags->"SpinorV"],
  Cell[1802620, 72925, 167, 5, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1802790, 72932, 146, 4, 38, "Output",
   CellTags->"SpinorV"],
  Cell[1802973, 72941, 198, 7, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1803174, 72950, 214, 7, 38, "Output",
   CellTags->"SpinorV"],
  Cell[1803425, 72962, 227, 8, 26, "Input",
   CellTags->"SpinorV"],
  Cell[1803655, 72972, 214, 7, 38, "Output",
   CellTags->"SpinorV"]},
 "SpinPolarizationSum"->{
  Cell[1803896, 72983, 75, 1, 39, "ObjectName",
   CellTags->"SpinPolarizationSum"],
  Cell[1803996, 72988, 70, 1, 36, "Subsubsection",
   CellTags->"SpinPolarizationSum"],
  Cell[1804069, 72991, 164, 4, 24, "Text",
   CellTags->"SpinPolarizationSum"],
  Cell[1804236, 72997, 291, 10, 24, "Text",
   CellTags->"SpinPolarizationSum"]},
 "SPL"->{
  Cell[1804542, 73010, 43, 1, 39, "ObjectName",
   CellTags->"SPL"],
  Cell[1804610, 73015, 54, 1, 36, "Subsubsection",
   CellTags->"SPL"],
  Cell[1804667, 73018, 77, 1, 24, "Text",
   CellTags->"SPL"],
  Cell[1804747, 73021, 284, 11, 24, "Text",
   CellTags->"SPL"]},
 "SplittingFunction"->{
  Cell[1805046, 73035, 71, 1, 39, "ObjectName",
   CellTags->"SplittingFunction"],
  Cell[1805142, 73040, 68, 1, 36, "Subsubsection",
   CellTags->"SplittingFunction"],
  Cell[1805213, 73043, 218, 7, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1805456, 73054, 134, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1805593, 73059, 181, 6, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1805789, 73068, 310, 11, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1806136, 73084, 65, 1, 36, "Subsubsection",
   CellTags->"SplittingFunction"],
  Cell[1806204, 73087, 65, 1, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1806272, 73090, 221, 6, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1806496, 73098, 170, 4, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1806691, 73106, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1806824, 73111, 531, 19, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1807392, 73135, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1807525, 73140, 301, 10, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1807863, 73155, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1807996, 73160, 269, 9, 43, "Output",
   CellTags->"SplittingFunction"],
  Cell[1808302, 73174, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1808435, 73179, 859, 29, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1809331, 73213, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1809464, 73218, 1128, 38, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1810629, 73261, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1810762, 73266, 599, 21, 43, "Output",
   CellTags->"SplittingFunction"],
  Cell[1811398, 73292, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1811531, 73297, 541, 19, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1812087, 73319, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1812220, 73324, 63, 1, 24, "Text",
   CellTags->"SplittingFunction"],
  Cell[1812286, 73327, 221, 6, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1812532, 73337, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1812665, 73342, 531, 19, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1813233, 73366, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1813366, 73371, 236, 8, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1813639, 73384, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1813772, 73389, 235, 8, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1814044, 73402, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1814177, 73407, 811, 28, 44, "Output",
   CellTags->"SplittingFunction"],
  Cell[1815025, 73440, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1815158, 73445, 1247, 42, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1816442, 73492, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1816575, 73497, 503, 18, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1817115, 73520, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1817249, 73525, 445, 16, 26, "Output",
   CellTags->"SplittingFunction"],
  Cell[1817731, 73546, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1817864, 73551, 445, 16, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1818346, 73572, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1818480, 73577, 445, 16, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1818962, 73598, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1819095, 73603, 1362, 46, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1820494, 73654, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1820628, 73659, 1362, 46, 46, "Output",
   CellTags->"SplittingFunction"],
  Cell[1822027, 73710, 131, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1822161, 73715, 637, 22, 29, "Output",
   CellTags->"SplittingFunction"],
  Cell[1822835, 73742, 132, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1822970, 73747, 5385, 174, 230, "Output",
   CellTags->"SplittingFunction"],
  Cell[1828392, 73926, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1828525, 73931, 2906, 92, 98, "Output",
   CellTags->"SplittingFunction"],
  Cell[1831468, 74028, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1831601, 74033, 3584, 117, 149, "Output",
   CellTags->"SplittingFunction"],
  Cell[1835222, 74155, 130, 3, 26, "Input",
   CellTags->"SplittingFunction"],
  Cell[1835355, 74160, 4258, 139, 193, "Output",
   CellTags->"SplittingFunction"]},
 "StandardMatrixElement"->{
  Cell[1839640, 74303, 79, 1, 39, "ObjectName",
   CellTags->"StandardMatrixElement"],
  Cell[1839744, 74308, 72, 1, 36, "Subsubsection",
   CellTags->"StandardMatrixElement"],
  Cell[1839819, 74311, 134, 3, 24, "Text",
   CellTags->"StandardMatrixElement"],
  Cell[1839956, 74316, 270, 11, 24, "Text",
   CellTags->"StandardMatrixElement"]},
 "SubContext"->{
  Cell[1840241, 74330, 57, 1, 39, "ObjectName",
   CellTags->"SubContext"],
  Cell[1840323, 74335, 61, 1, 36, "Subsubsection",
   CellTags->"SubContext"],
  Cell[1840387, 74338, 120, 3, 24, "Text",
   CellTags->"SubContext"],
  Cell[1840510, 74343, 439, 18, 24, "Text",
   CellTags->"SubContext"]},
 "SubLoop"->{
  Cell[1840964, 74364, 51, 1, 39, "ObjectName",
   CellTags->"SubLoop"],
  Cell[1841040, 74369, 58, 1, 36, "Subsubsection",
   CellTags->"SubLoop"],
  Cell[1841101, 74372, 152, 4, 24, "Text",
   CellTags->"SubLoop"],
  Cell[1841256, 74378, 260, 11, 24, "Text",
   CellTags->"SubLoop"]},
 "SumP"->{
  Cell[1841531, 74392, 45, 1, 39, "ObjectName",
   CellTags->"SumP"],
  Cell[1841601, 74397, 55, 1, 36, "Subsubsection",
   CellTags->"SumP"],
  Cell[1841659, 74400, 767, 31, 27, "Text",
   CellTags->"SumP"],
  Cell[1842429, 74433, 417, 18, 24, "Text",
   CellTags->"SumP"],
  Cell[1842883, 74456, 52, 1, 36, "Subsubsection",
   CellTags->"SumP"],
  Cell[1842960, 74461, 149, 5, 26, "Input",
   CellTags->"SumP"],
  Cell[1843112, 74468, 263, 8, 28, "Output",
   CellTags->"SumP"],
  Cell[1843412, 74481, 149, 5, 26, "Input",
   CellTags->"SumP"],
  Cell[1843564, 74488, 263, 8, 28, "Output",
   CellTags->"SumP"],
  Cell[1843864, 74501, 125, 4, 26, "Input",
   CellTags->"SumP"],
  Cell[1843992, 74507, 238, 7, 28, "Output",
   CellTags->"SumP"],
  Cell[1844267, 74519, 125, 4, 26, "Input",
   CellTags->"SumP"],
  Cell[1844395, 74525, 124, 4, 43, "Output",
   CellTags->"SumP"],
  Cell[1844556, 74534, 301, 12, 53, "Input",
   CellTags->"SumP"],
  Cell[1844860, 74548, 124, 4, 43, "Output",
   CellTags->"SumP"],
  Cell[1845021, 74557, 186, 6, 26, "Input",
   CellTags->"SumP"],
  Cell[1845210, 74565, 764, 29, 43, "Output",
   CellTags->"SumP"],
  Cell[1846011, 74599, 124, 4, 26, "Input",
   CellTags->"SumP"],
  Cell[1846138, 74605, 124, 4, 43, "Output",
   CellTags->"SumP"]},
 "SumS"->{
  Cell[1846289, 74613, 45, 1, 39, "ObjectName",
   CellTags->"SumS"],
  Cell[1846359, 74618, 55, 1, 36, "Subsubsection",
   CellTags->"SumS"],
  Cell[1846417, 74621, 1438, 57, 44, "Text",
   CellTags->"SumS"],
  Cell[1847880, 74682, 108, 3, 26, "Input",
   CellTags->"SumS"],
  Cell[1847991, 74687, 165, 6, 26, "Output",
   CellTags->"SumS"],
  Cell[1848171, 74696, 417, 18, 24, "Text",
   CellTags->"SumS"],
  Cell[1848625, 74719, 52, 1, 36, "Subsubsection",
   CellTags->"SumS"],
  Cell[1848702, 74724, 149, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1848854, 74731, 211, 8, 26, "Output",
   CellTags->"SumS"],
  Cell[1849102, 74744, 149, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1849254, 74751, 211, 8, 26, "Output",
   CellTags->"SumS"],
  Cell[1849502, 74764, 144, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1849649, 74771, 172, 6, 26, "Output",
   CellTags->"SumS"],
  Cell[1849858, 74782, 173, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1850034, 74789, 261, 10, 44, "Output",
   CellTags->"SumS"],
  Cell[1850332, 74804, 197, 6, 26, "Input",
   CellTags->"SumS"],
  Cell[1850532, 74812, 340, 13, 47, "Output",
   CellTags->"SumS"],
  Cell[1850887, 74828, 192, 6, 26, "Input",
   CellTags->"SumS"],
  Cell[1851104, 74838, 149, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1851256, 74845, 509, 19, 47, "Output",
   CellTags->"SumS"],
  Cell[1851780, 74867, 193, 6, 26, "Input",
   CellTags->"SumS"],
  Cell[1851998, 74877, 125, 4, 26, "Input",
   CellTags->"SumS"],
  Cell[1852126, 74883, 124, 4, 43, "Output",
   CellTags->"SumS"],
  Cell[1852287, 74892, 174, 6, 53, "Input",
   CellTags->"SumS"],
  Cell[1852464, 74900, 124, 4, 43, "Output",
   CellTags->"SumS"],
  Cell[1852625, 74909, 159, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1852787, 74916, 212, 8, 26, "Output",
   CellTags->"SumS"],
  Cell[1853036, 74929, 146, 4, 26, "Input",
   CellTags->"SumS"],
  Cell[1853185, 74935, 149, 5, 44, "Output",
   CellTags->"SumS"],
  Cell[1853371, 74945, 144, 5, 26, "Input",
   CellTags->"SumS"],
  Cell[1853518, 74952, 142, 5, 42, "Output",
   CellTags->"SumS"],
  Cell[1853697, 74962, 125, 4, 26, "Input",
   CellTags->"SumS"],
  Cell[1853825, 74968, 142, 5, 42, "Output",
   CellTags->"SumS"]},
 "SumT"->{
  Cell[1853994, 74977, 45, 1, 39, "ObjectName",
   CellTags->"SumT"],
  Cell[1854064, 74982, 55, 1, 36, "Subsubsection",
   CellTags->"SumT"],
  Cell[1854122, 74985, 461, 15, 43, "Text",
   CellTags->"SumT"],
  Cell[1854586, 75002, 417, 18, 24, "Text",
   CellTags->"SumT"],
  Cell[1855040, 75025, 52, 1, 36, "Subsubsection",
   CellTags->"SumT"],
  Cell[1855117, 75030, 149, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1855269, 75037, 236, 9, 26, "Output",
   CellTags->"SumT"],
  Cell[1855542, 75051, 149, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1855694, 75058, 236, 9, 26, "Output",
   CellTags->"SumT"],
  Cell[1855967, 75072, 125, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1856095, 75078, 180, 7, 26, "Output",
   CellTags->"SumT"],
  Cell[1856312, 75090, 173, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1856488, 75097, 361, 14, 45, "Output",
   CellTags->"SumT"],
  Cell[1856886, 75116, 125, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1857014, 75122, 142, 5, 42, "Output",
   CellTags->"SumT"],
  Cell[1857193, 75132, 244, 9, 53, "Input",
   CellTags->"SumT"],
  Cell[1857440, 75143, 142, 5, 42, "Output",
   CellTags->"SumT"],
  Cell[1857619, 75153, 159, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1857781, 75160, 237, 9, 26, "Output",
   CellTags->"SumT"],
  Cell[1858055, 75174, 136, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1858194, 75180, 253, 7, 44, "Output",
   CellTags->"SumT"],
  Cell[1858484, 75192, 125, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1858612, 75198, 142, 5, 42, "Output",
   CellTags->"SumT"],
  Cell[1858791, 75208, 144, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1858938, 75215, 142, 5, 42, "Output",
   CellTags->"SumT"],
  Cell[1859117, 75225, 268, 10, 53, "Input",
   CellTags->"SumT"],
  Cell[1859388, 75237, 336, 12, 26, "Output",
   CellTags->"SumT"],
  Cell[1859761, 75254, 136, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1859900, 75260, 170, 6, 43, "Output",
   CellTags->"SumT"],
  Cell[1860107, 75271, 155, 5, 26, "Input",
   CellTags->"SumT"],
  Cell[1860265, 75278, 253, 7, 44, "Output",
   CellTags->"SumT"],
  Cell[1860555, 75290, 129, 4, 26, "Input",
   CellTags->"SumT"],
  Cell[1860687, 75296, 458, 16, 45, "Output",
   CellTags->"SumT"],
  Cell[1861182, 75317, 240, 9, 26, "Input",
   CellTags->"SumT"],
  Cell[1861425, 75328, 458, 16, 45, "Output",
   CellTags->"SumT"]},
 "SUND"->{
  Cell[1861910, 75348, 45, 1, 39, "ObjectName",
   CellTags->"SUND"],
  Cell[1861980, 75353, 55, 1, 36, "Subsubsection",
   CellTags->"SUND"],
  Cell[1862038, 75356, 224, 10, 24, "Text",
   CellTags->"SUND"],
  Cell[1862265, 75368, 637, 25, 24, "Text",
   CellTags->"SUND"],
  Cell[1862939, 75398, 52, 1, 36, "Subsubsection",
   CellTags->"SUND"],
  Cell[1863016, 75403, 135, 4, 26, "Input",
   CellTags->"SUND"],
  Cell[1863154, 75409, 147, 5, 26, "Output",
   CellTags->"SUND"],
  Cell[1863338, 75419, 211, 6, 26, "Input",
   CellTags->"SUND"],
  Cell[1863552, 75427, 1086, 44, 26, "Output",
   CellTags->"SUND"],
  Cell[1864675, 75476, 135, 4, 26, "Input",
   CellTags->"SUND"],
  Cell[1864813, 75482, 147, 5, 26, "Output",
   CellTags->"SUND"],
  Cell[1864997, 75492, 135, 4, 26, "Input",
   CellTags->"SUND"],
  Cell[1865135, 75498, 147, 5, 26, "Output",
   CellTags->"SUND"],
  Cell[1865319, 75508, 266, 8, 26, "Input",
   CellTags->"SUND"],
  Cell[1865588, 75518, 260, 9, 29, "Output",
   CellTags->"SUND"],
  Cell[1865885, 75532, 377, 12, 26, "Input",
   CellTags->"SUND"],
  Cell[1866265, 75546, 304, 11, 46, "Output",
   CellTags->"SUND"],
  Cell[1866606, 75562, 377, 12, 26, "Input",
   CellTags->"SUND"],
  Cell[1866986, 75576, 474, 19, 46, "Output",
   CellTags->"SUND"],
  Cell[1867497, 75600, 173, 6, 26, "Input",
   CellTags->"SUND"],
  Cell[1867673, 75608, 150, 4, 38, "Output",
   CellTags->"SUND"],
  Cell[1867860, 75617, 202, 7, 26, "Input",
   CellTags->"SUND"],
  Cell[1868065, 75626, 258, 7, 38, "Output",
   CellTags->"SUND"],
  Cell[1868360, 75638, 231, 8, 26, "Input",
   CellTags->"SUND"],
  Cell[1868594, 75648, 150, 4, 38, "Output",
   CellTags->"SUND"],
  Cell[1868759, 75655, 104, 3, 26, "Input",
   CellTags->"SUND"]},
 "SUNDelta"->{
  Cell[1868878, 75661, 53, 1, 39, "ObjectName",
   CellTags->"SUNDelta"],
  Cell[1868956, 75666, 59, 1, 36, "Subsubsection",
   CellTags->"SUNDelta"],
  Cell[1869018, 75669, 172, 6, 24, "Text",
   CellTags->"SUNDelta"],
  Cell[1869193, 75677, 1207, 45, 24, "Text",
   CellTags->"SUNDelta"],
  Cell[1870437, 75727, 56, 1, 36, "Subsubsection",
   CellTags->"SUNDelta"],
  Cell[1870518, 75732, 133, 4, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1870654, 75738, 153, 5, 26, "Output",
   CellTags->"SUNDelta"],
  Cell[1870844, 75748, 262, 8, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1871109, 75758, 301, 13, 26, "Output",
   CellTags->"SUNDelta"],
  Cell[1871447, 75776, 205, 6, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1871655, 75784, 301, 13, 26, "Output",
   CellTags->"SUNDelta"],
  Cell[1871993, 75802, 169, 5, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1872165, 75809, 148, 4, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1872350, 75818, 200, 7, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1872553, 75827, 220, 6, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1872810, 75838, 229, 8, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1873042, 75848, 142, 4, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1873221, 75857, 194, 7, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1873418, 75866, 220, 6, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1873653, 75875, 256, 5, 41, "Text",
   CellTags->"SUNDelta"],
  Cell[1873934, 75884, 315, 10, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1874252, 75896, 460, 20, 26, "Output",
   CellTags->"SUNDelta"],
  Cell[1874749, 75921, 110, 3, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1874862, 75926, 395, 11, 38, "Output",
   CellTags->"SUNDelta"],
  Cell[1875294, 75942, 200, 7, 26, "Input",
   CellTags->"SUNDelta"],
  Cell[1875497, 75951, 236, 6, 38, "Output",
   CellTags->"SUNDelta"]},
 "SUNDeltaContract"->{
  Cell[1875760, 75961, 69, 1, 39, "ObjectName",
   CellTags->"SUNDeltaContract"],
  Cell[1875854, 75966, 67, 1, 36, "Subsubsection",
   CellTags->"SUNDeltaContract"],
  Cell[1875924, 75969, 397, 9, 58, "Text",
   CellTags->"SUNDeltaContract"],
  Cell[1876324, 75980, 461, 18, 24, "Text",
   CellTags->"SUNDeltaContract"]},
 "SUNF"->{
  Cell[1876800, 76001, 45, 1, 39, "ObjectName",
   CellTags->"SUNF"],
  Cell[1876870, 76006, 55, 1, 36, "Subsubsection",
   CellTags->"SUNF"],
  Cell[1876928, 76009, 190, 6, 24, "Text",
   CellTags->"SUNF"],
  Cell[1877143, 76019, 108, 3, 26, "Input",
   CellTags->"SUNF"],
  Cell[1877254, 76024, 168, 6, 26, "Output",
   CellTags->"SUNF"],
  Cell[1877437, 76033, 1185, 46, 24, "Text",
   CellTags->"SUNF"],
  Cell[1878659, 76084, 52, 1, 36, "Subsubsection",
   CellTags->"SUNF"],
  Cell[1878736, 76089, 294, 10, 26, "Input",
   CellTags->"SUNF"],
  Cell[1879033, 76101, 272, 10, 26, "Output",
   CellTags->"SUNF"],
  Cell[1879342, 76116, 103, 3, 26, "Input",
   CellTags->"SUNF"],
  Cell[1879448, 76121, 764, 34, 26, "Output",
   CellTags->"SUNF"],
  Cell[1880249, 76160, 110, 3, 26, "Input",
   CellTags->"SUNF"],
  Cell[1880362, 76165, 455, 20, 26, "Output",
   CellTags->"SUNF"],
  Cell[1880854, 76190, 135, 4, 26, "Input",
   CellTags->"SUNF"],
  Cell[1880992, 76196, 147, 5, 26, "Output",
   CellTags->"SUNF"],
  Cell[1881176, 76206, 163, 5, 26, "Input",
   CellTags->"SUNF"],
  Cell[1881342, 76213, 101, 3, 26, "Output",
   CellTags->"SUNF"],
  Cell[1881458, 76219, 319, 10, 24, "Text",
   CellTags->"SUNF"],
  Cell[1881802, 76233, 185, 5, 26, "Input",
   CellTags->"SUNF"],
  Cell[1881990, 76240, 1147, 44, 26, "Output",
   CellTags->"SUNF"],
  Cell[1883174, 76289, 266, 8, 26, "Input",
   CellTags->"SUNF"],
  Cell[1883443, 76299, 353, 15, 28, "Output",
   CellTags->"SUNF"],
  Cell[1883833, 76319, 239, 7, 26, "Input",
   CellTags->"SUNF"],
  Cell[1884075, 76328, 1166, 45, 26, "Output",
   CellTags->"SUNF"],
  Cell[1885278, 76378, 173, 6, 26, "Input",
   CellTags->"SUNF"],
  Cell[1885454, 76386, 150, 4, 38, "Output",
   CellTags->"SUNF"],
  Cell[1885641, 76395, 202, 7, 26, "Input",
   CellTags->"SUNF"],
  Cell[1885846, 76404, 258, 7, 38, "Output",
   CellTags->"SUNF"],
  Cell[1886141, 76416, 231, 8, 26, "Input",
   CellTags->"SUNF"],
  Cell[1886375, 76426, 150, 4, 38, "Output",
   CellTags->"SUNF"],
  Cell[1886562, 76435, 135, 4, 26, "Input",
   CellTags->"SUNF"],
  Cell[1886700, 76441, 147, 5, 26, "Output",
   CellTags->"SUNF"],
  Cell[1886884, 76451, 162, 5, 26, "Input",
   CellTags->"SUNF"],
  Cell[1887049, 76458, 401, 18, 26, "Output",
   CellTags->"SUNF"]},
 "SUNFJacobi"->{
  Cell[1887477, 76480, 57, 1, 39, "ObjectName",
   CellTags->"SUNFJacobi"],
  Cell[1887559, 76485, 61, 1, 36, "Subsubsection",
   CellTags->"SUNFJacobi"],
  Cell[1887623, 76488, 146, 4, 24, "Text",
   CellTags->"SUNFJacobi"],
  Cell[1887772, 76494, 446, 17, 24, "Text",
   CellTags->"SUNFJacobi"],
  Cell[1888255, 76516, 58, 1, 36, "Subsubsection",
   CellTags->"SUNFJacobi"],
  Cell[1888338, 76521, 380, 12, 26, "Input",
   CellTags->"SUNFJacobi"],
  Cell[1888721, 76535, 694, 32, 29, "Output",
   CellTags->"SUNFJacobi"],
  Cell[1889452, 76572, 379, 12, 26, "Input",
   CellTags->"SUNFJacobi"],
  Cell[1889834, 76586, 1362, 62, 29, "Output",
   CellTags->"SUNFJacobi"]},
 "SUNIndex"->{
  Cell[1891223, 76652, 53, 1, 39, "ObjectName",
   CellTags->"SUNIndex"],
  Cell[1891301, 76657, 59, 1, 36, "Subsubsection",
   CellTags->"SUNIndex"],
  Cell[1891363, 76660, 135, 6, 24, "Text",
   CellTags->"SUNIndex"],
  Cell[1891501, 76668, 655, 24, 24, "Text",
   CellTags->"SUNIndex"],
  Cell[1892193, 76697, 56, 1, 36, "Subsubsection",
   CellTags->"SUNIndex"],
  Cell[1892274, 76702, 110, 3, 26, "Input",
   CellTags->"SUNIndex"],
  Cell[1892387, 76707, 105, 3, 27, "Output",
   CellTags->"SUNIndex"],
  Cell[1892529, 76715, 110, 3, 26, "Input",
   CellTags->"SUNIndex"],
  Cell[1892642, 76720, 125, 3, 38, "Output",
   CellTags->"SUNIndex"],
  Cell[1892804, 76728, 200, 7, 26, "Input",
   CellTags->"SUNIndex"],
  Cell[1893007, 76737, 220, 6, 38, "Output",
   CellTags->"SUNIndex"]},
 "SUNIndexRename"->{
  Cell[1893254, 76747, 65, 1, 39, "ObjectName",
   CellTags->"SUNIndexRename"],
  Cell[1893344, 76752, 65, 1, 36, "Subsubsection",
   CellTags->"SUNIndexRename"],
  Cell[1893412, 76755, 163, 4, 24, "Text",
   CellTags->"SUNIndexRename"],
  Cell[1893578, 76761, 471, 18, 24, "Text",
   CellTags->"SUNIndexRename"]},
 "SUNN"->{
  Cell[1894064, 76782, 45, 1, 39, "ObjectName",
   CellTags->"SUNN"],
  Cell[1894134, 76787, 55, 1, 36, "Subsubsection",
   CellTags->"SUNN"],
  Cell[1894192, 76790, 203, 7, 24, "Text",
   CellTags->"SUNN"],
  Cell[1894398, 76799, 977, 39, 24, "Text",
   CellTags->"SUNN"],
  Cell[1895412, 76843, 52, 1, 36, "Subsubsection",
   CellTags->"SUNN"],
  Cell[1895489, 76848, 317, 9, 26, "Input",
   CellTags->"SUNN"],
  Cell[1895809, 76859, 149, 5, 26, "Output",
   CellTags->"SUNN"]},
 "SUNNToCACF"->{
  Cell[1895985, 76868, 57, 1, 39, "ObjectName",
   CellTags->"SUNNToCACF"],
  Cell[1896067, 76873, 61, 1, 36, "Subsubsection",
   CellTags->"SUNNToCACF"],
  Cell[1896131, 76876, 187, 4, 24, "Text",
   CellTags->"SUNNToCACF"],
  Cell[1896321, 76882, 967, 39, 24, "Text",
   CellTags->"SUNNToCACF"],
  Cell[1897325, 76926, 58, 1, 36, "Subsubsection",
   CellTags->"SUNNToCACF"],
  Cell[1897408, 76931, 322, 9, 26, "Input",
   CellTags->"SUNNToCACF"],
  Cell[1897733, 76942, 186, 6, 26, "Output",
   CellTags->"SUNNToCACF"]},
 "SUNSimplify"->{
  Cell[1897946, 76952, 59, 1, 39, "ObjectName",
   CellTags->"SUNSimplify"],
  Cell[1898030, 76957, 62, 1, 36, "Subsubsection",
   CellTags->"SUNSimplify"],
  Cell[1898095, 76960, 376, 7, 58, "Text",
   CellTags->"SUNSimplify"],
  Cell[1898496, 76971, 122, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1898621, 76976, 512, 13, 47, "Output",
   CellTags->"SUNSimplify"],
  Cell[1899148, 76992, 467, 17, 24, "Text",
   CellTags->"SUNSimplify"],
  Cell[1899652, 77014, 59, 1, 36, "Subsubsection",
   CellTags->"SUNSimplify"],
  Cell[1899736, 77019, 247, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1899986, 77029, 231, 8, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1900254, 77042, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1900374, 77047, 304, 13, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1900715, 77065, 189, 6, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1900907, 77073, 249, 10, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1901193, 77088, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1901313, 77093, 130, 4, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1901480, 77102, 183, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1901666, 77109, 203, 7, 44, "Output",
   CellTags->"SUNSimplify"],
  Cell[1901906, 77121, 254, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1902163, 77131, 227, 8, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1902427, 77144, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1902547, 77149, 360, 15, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1902944, 77169, 269, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1903216, 77179, 227, 8, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1903480, 77192, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1903600, 77197, 197, 6, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1903834, 77208, 259, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1904096, 77218, 227, 8, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1904360, 77231, 117, 3, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1904480, 77236, 360, 15, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1904877, 77256, 259, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1905139, 77266, 227, 8, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1905403, 77279, 180, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1905586, 77286, 108, 3, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1905731, 77294, 384, 12, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1906118, 77308, 311, 11, 46, "Output",
   CellTags->"SUNSimplify"],
  Cell[1906466, 77324, 389, 12, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1906858, 77338, 448, 18, 46, "Output",
   CellTags->"SUNSimplify"],
  Cell[1907343, 77361, 246, 7, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1907592, 77370, 1173, 45, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1908802, 77420, 246, 7, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1909051, 77429, 1132, 44, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1910220, 77478, 273, 8, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1910496, 77488, 251, 8, 42, "Output",
   CellTags->"SUNSimplify"],
  Cell[1910784, 77501, 439, 14, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1911226, 77517, 379, 14, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1911642, 77536, 181, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1911826, 77543, 705, 32, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1912568, 77580, 238, 6, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1912809, 77588, 1363, 62, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1914209, 77655, 200, 6, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1914412, 77663, 164, 5, 28, "Output",
   CellTags->"SUNSimplify"],
  Cell[1914613, 77673, 341, 10, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1914957, 77685, 543, 21, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1915537, 77711, 181, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1915721, 77718, 543, 21, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1916301, 77744, 180, 5, 26, "Input",
   CellTags->"SUNSimplify"],
  Cell[1916484, 77751, 571, 22, 26, "Output",
   CellTags->"SUNSimplify"],
  Cell[1917683, 77801, 208, 6, 26, "Input",
   CellTags->"SUNSimplify"]},
 "SUNT"->{
  Cell[1917906, 77810, 45, 1, 39, "ObjectName",
   CellTags->"SUNT"],
  Cell[1917976, 77815, 55, 1, 36, "Subsubsection",
   CellTags->"SUNT"],
  Cell[1918034, 77818, 237, 10, 24, "Text",
   CellTags->"SUNT"],
  Cell[1918274, 77830, 1149, 46, 24, "Text",
   CellTags->"SUNT"],
  Cell[1919460, 77881, 52, 1, 36, "Subsubsection",
   CellTags->"SUNT"],
  Cell[1919537, 77886, 102, 3, 26, "Input",
   CellTags->"SUNT"],
  Cell[1919642, 77891, 157, 6, 26, "Output",
   CellTags->"SUNT"],
  Cell[1919814, 77900, 203, 7, 24, "Text",
   CellTags->"SUNT"],
  Cell[1920042, 77911, 207, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1920252, 77919, 311, 13, 26, "Output",
   CellTags->"SUNT"],
  Cell[1920600, 77937, 145, 4, 26, "Input",
   CellTags->"SUNT"],
  Cell[1920748, 77943, 365, 16, 28, "Output",
   CellTags->"SUNT"],
  Cell[1921150, 77964, 242, 7, 26, "Input",
   CellTags->"SUNT"],
  Cell[1921395, 77973, 268, 11, 42, "Output",
   CellTags->"SUNT"],
  Cell[1921700, 77989, 184, 5, 26, "Input",
   CellTags->"SUNT"],
  Cell[1921887, 77996, 133, 4, 28, "Output",
   CellTags->"SUNT"],
  Cell[1922057, 78005, 174, 5, 26, "Input",
   CellTags->"SUNT"],
  Cell[1922234, 78012, 399, 16, 42, "Output",
   CellTags->"SUNT"],
  Cell[1922670, 78033, 242, 7, 26, "Input",
   CellTags->"SUNT"],
  Cell[1922915, 78042, 268, 11, 42, "Output",
   CellTags->"SUNT"],
  Cell[1923198, 78056, 422, 15, 24, "Text",
   CellTags->"SUNT"],
  Cell[1923645, 78075, 161, 5, 26, "Input",
   CellTags->"SUNT"],
  Cell[1923809, 78082, 328, 14, 42, "Output",
   CellTags->"SUNT"],
  Cell[1924152, 78099, 197, 7, 27, "Text",
   CellTags->"SUNT"],
  Cell[1924374, 78110, 197, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1924574, 78118, 353, 15, 29, "Output",
   CellTags->"SUNT"],
  Cell[1924964, 78138, 198, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1925165, 78146, 263, 8, 54, "Output",
   CellTags->"SUNT"],
  Cell[1925465, 78159, 167, 6, 26, "Input",
   CellTags->"SUNT"],
  Cell[1925635, 78167, 152, 4, 38, "Output",
   CellTags->"SUNT"],
  Cell[1925824, 78176, 195, 7, 26, "Input",
   CellTags->"SUNT"],
  Cell[1926022, 78185, 117, 3, 38, "Output",
   CellTags->"SUNT"]},
 "SUNTrace"->{
  Cell[1926166, 78192, 53, 1, 39, "ObjectName",
   CellTags->"SUNTrace"],
  Cell[1926244, 78197, 59, 1, 36, "Subsubsection",
   CellTags->"SUNTrace"],
  Cell[1926306, 78200, 81, 1, 24, "Text",
   CellTags->"SUNTrace"],
  Cell[1926412, 78205, 116, 3, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1926531, 78210, 172, 6, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1926718, 78219, 441, 18, 24, "Text",
   CellTags->"SUNTrace"],
  Cell[1927196, 78242, 56, 1, 36, "Subsubsection",
   CellTags->"SUNTrace"],
  Cell[1927277, 78247, 165, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1927445, 78254, 332, 14, 42, "Output",
   CellTags->"SUNTrace"],
  Cell[1927814, 78273, 175, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1927992, 78280, 524, 22, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1928553, 78307, 240, 7, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1928796, 78316, 811, 35, 44, "Output",
   CellTags->"SUNTrace"],
  Cell[1929644, 78356, 185, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1929832, 78363, 641, 27, 28, "Output",
   CellTags->"SUNTrace"],
  Cell[1930510, 78395, 278, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1930791, 78405, 4270, 184, 43, "Output",
   CellTags->"SUNTrace"],
  Cell[1935098, 78594, 177, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1935278, 78601, 641, 27, 28, "Output",
   CellTags->"SUNTrace"],
  Cell[1935956, 78633, 288, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1936247, 78643, 11352, 436, 174, "Output",
   CellTags->"SUNTrace"],
  Cell[1947636, 79084, 177, 5, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1947816, 79091, 758, 32, 28, "Output",
   CellTags->"SUNTrace"],
  Cell[1948611, 79128, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1948884, 79138, 105, 3, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1949026, 79146, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1949299, 79156, 105, 3, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1949441, 79164, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1949714, 79174, 105, 3, 26, "Output",
   CellTags->"SUNTrace"],
  Cell[1949856, 79182, 270, 8, 26, "Input",
   CellTags->"SUNTrace"],
  Cell[1950129, 79192, 576, 23, 29, "Output",
   CellTags->"SUNTrace"]},
 "SymbolicSum2"->{
  Cell[1950732, 79219, 61, 1, 39, "ObjectName",
   CellTags->"SymbolicSum2"],
  Cell[1950818, 79224, 63, 1, 36, "Subsubsection",
   CellTags->"SymbolicSum2"],
  Cell[1950884, 79227, 284, 6, 41, "Text",
   CellTags->"SymbolicSum2"]},
 "SymbolicSum3"->{
  Cell[1951183, 79236, 61, 1, 39, "ObjectName",
   CellTags->"SymbolicSum3"],
  Cell[1951269, 79241, 63, 1, 36, "Subsubsection",
   CellTags->"SymbolicSum3"],
  Cell[1951335, 79244, 292, 6, 41, "Text",
   CellTags->"SymbolicSum3"]},
 "Symmetrize"->{
  Cell[1951642, 79253, 57, 1, 39, "ObjectName",
   CellTags->"Symmetrize"],
  Cell[1951724, 79258, 61, 1, 36, "Subsubsection",
   CellTags->"Symmetrize"],
  Cell[1951788, 79261, 145, 4, 24, "Text",
   CellTags->"Symmetrize"],
  Cell[1951936, 79267, 281, 10, 24, "Text",
   CellTags->"Symmetrize"],
  Cell[1952254, 79282, 58, 1, 36, "Subsubsection",
   CellTags->"Symmetrize"],
  Cell[1952337, 79287, 240, 8, 26, "Input",
   CellTags->"Symmetrize"],
  Cell[1952580, 79297, 317, 11, 42, "Output",
   CellTags->"Symmetrize"],
  Cell[1952934, 79313, 260, 8, 26, "Input",
   CellTags->"Symmetrize"],
  Cell[1953197, 79323, 652, 20, 42, "Output",
   CellTags->"Symmetrize"]},
 "TBox"->{
  Cell[1953876, 79347, 61, 1, 39, "ObjectName",
   CellTags->{"TBox", "Tbox"}],
  Cell[1953962, 79352, 65, 1, 36, "Subsubsection",
   CellTags->{"TBox", "Tbox"}],
  Cell[1954030, 79355, 262, 5, 41, "Text",
   CellTags->{"TBox", "Tbox"}]},
 "Tbox"->{
  Cell[1953876, 79347, 61, 1, 39, "ObjectName",
   CellTags->{"TBox", "Tbox"}],
  Cell[1953962, 79352, 65, 1, 36, "Subsubsection",
   CellTags->{"TBox", "Tbox"}],
  Cell[1954030, 79355, 262, 5, 41, "Text",
   CellTags->{"TBox", "Tbox"}]},
 "Tdec"->{
  Cell[1954307, 79363, 45, 1, 39, "ObjectName",
   CellTags->"Tdec"],
  Cell[1954377, 79368, 55, 1, 36, "Subsubsection",
   CellTags->"Tdec"],
  Cell[1954435, 79371, 256, 5, 41, "Text",
   CellTags->"Tdec"],
  Cell[1954716, 79380, 108, 3, 26, "Input",
   CellTags->"Tdec"],
  Cell[1954827, 79385, 412, 11, 26, "Output",
   CellTags->"Tdec"],
  Cell[1955254, 79399, 633, 25, 24, "Text",
   CellTags->"Tdec"],
  Cell[1955924, 79429, 52, 1, 36, "Subsubsection",
   CellTags->"Tdec"],
  Cell[1955979, 79432, 608, 22, 34, "Text",
   CellTags->"Tdec"],
  Cell[1956612, 79458, 202, 7, 26, "Input",
   CellTags->"Tdec"],
  Cell[1956817, 79467, 556, 21, 43, "Output",
   CellTags->"Tdec"],
  Cell[1957410, 79493, 199, 7, 26, "Input",
   CellTags->"Tdec"],
  Cell[1957612, 79502, 354, 14, 43, "Output",
   CellTags->"Tdec"],
  Cell[1957981, 79519, 776, 28, 27, "Text",
   CellTags->"Tdec"],
  Cell[1958760, 79549, 529, 18, 26, "Input",
   CellTags->"Tdec"],
  Cell[1959314, 79571, 199, 7, 26, "Input",
   CellTags->"Tdec"],
  Cell[1959516, 79580, 7304, 266, 93, "Output",
   CellTags->"Tdec"],
  Cell[1966857, 79851, 380, 13, 26, "Input",
   CellTags->"Tdec"],
  Cell[1967240, 79866, 4966, 181, 72, "Output",
   CellTags->"Tdec"],
  Cell[1972221, 80050, 121, 4, 26, "Input",
   CellTags->"Tdec"]},
 "TensorFunction"->{
  Cell[1972357, 80057, 65, 1, 39, "ObjectName",
   CellTags->"TensorFunction"],
  Cell[1972447, 80062, 65, 1, 36, "Subsubsection",
   CellTags->"TensorFunction"],
  Cell[1972515, 80065, 360, 7, 41, "Text",
   CellTags->"TensorFunction"],
  Cell[1972878, 80074, 275, 11, 24, "Text",
   CellTags->"TensorFunction"],
  Cell[1973190, 80090, 62, 1, 36, "Subsubsection",
   CellTags->"TensorFunction"],
  Cell[1973277, 80095, 178, 4, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1973458, 80101, 212, 6, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1973707, 80112, 116, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1973826, 80117, 193, 4, 38, "Output",
   CellTags->"TensorFunction"],
  Cell[1974056, 80126, 198, 6, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1974257, 80134, 376, 14, 28, "Output",
   CellTags->"TensorFunction"],
  Cell[1974670, 80153, 116, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1974789, 80158, 358, 10, 38, "Output",
   CellTags->"TensorFunction"],
  Cell[1975184, 80173, 228, 7, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1975415, 80182, 263, 8, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1975715, 80195, 228, 7, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1975946, 80204, 263, 8, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1976246, 80217, 116, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1976365, 80222, 243, 7, 38, "Output",
   CellTags->"TensorFunction"],
  Cell[1976645, 80234, 118, 3, 26, "Input",
   CellTags->"TensorFunction"],
  Cell[1976766, 80239, 129, 4, 26, "Output",
   CellTags->"TensorFunction"],
  Cell[1976910, 80246, 154, 4, 26, "Input",
   CellTags->"TensorFunction"]},
 "Tf"->{
  Cell[1977079, 80253, 41, 1, 39, "ObjectName",
   CellTags->"Tf"],
  Cell[1977145, 80258, 53, 1, 36, "Subsubsection",
   CellTags->"Tf"],
  Cell[1977201, 80261, 214, 10, 27, "Text",
   CellTags->"Tf"],
  Cell[1977418, 80273, 669, 24, 24, "Text",
   CellTags->"Tf"]},
 "TFi"->{
  Cell[1978102, 80300, 43, 1, 39, "ObjectName",
   CellTags->"TFi"],
  Cell[1978170, 80305, 54, 1, 36, "Subsubsection",
   CellTags->"TFi"],
  Cell[1978808, 80327, 576, 10, 75, "Text",
   CellTags->"TFi"],
  Cell[1979387, 80339, 237, 8, 24, "Text",
   CellTags->"TFi"],
  Cell[1979627, 80349, 185, 4, 24, "Text",
   CellTags->"TFi"],
  Cell[1979815, 80355, 426, 17, 24, "Text",
   CellTags->"TFi"],
  Cell[1980278, 80377, 51, 1, 36, "Subsubsection",
   CellTags->"TFi"],
  Cell[1980354, 80382, 506, 17, 26, "Input",
   CellTags->"TFi"],
  Cell[1980863, 80401, 778, 24, 93, "Output",
   CellTags->"TFi"],
  Cell[1981678, 80430, 295, 10, 26, "Input",
   CellTags->"TFi"],
  Cell[1981976, 80442, 333, 11, 29, "Output",
   CellTags->"TFi"]},
 "ThreeVector"->{
  Cell[1982336, 80457, 59, 1, 39, "ObjectName",
   CellTags->"ThreeVector"],
  Cell[1982420, 80462, 62, 1, 36, "Subsubsection",
   CellTags->"ThreeVector"],
  Cell[1982485, 80465, 91, 1, 24, "Text",
   CellTags->"ThreeVector"],
  Cell[1982579, 80468, 474, 17, 24, "Text",
   CellTags->"ThreeVector"]},
 "TID"->{
  Cell[1983068, 80488, 43, 1, 39, "ObjectName",
   CellTags->"TID"],
  Cell[1983136, 80493, 54, 1, 36, "Subsubsection",
   CellTags->"TID"],
  Cell[1983193, 80496, 80, 1, 24, "Text",
   CellTags->"TID"],
  Cell[1983298, 80501, 106, 3, 26, "Input",
   CellTags->"TID"],
  Cell[1983407, 80506, 655, 15, 47, "Output",
   CellTags->"TID"],
  Cell[1984077, 80524, 460, 18, 24, "Text",
   CellTags->"TID"],
  Cell[1984574, 80547, 51, 1, 36, "Subsubsection",
   CellTags->"TID"],
  Cell[1984650, 80552, 374, 12, 26, "Input",
   CellTags->"TID"],
  Cell[1985027, 80566, 532, 21, 50, "Output",
   CellTags->"TID"],
  Cell[1985596, 80592, 181, 6, 26, "Input",
   CellTags->"TID"],
  Cell[1985780, 80600, 4307, 136, 64, "Output",
   CellTags->"TID"],
  Cell[1990124, 80741, 271, 9, 26, "Input",
   CellTags->"TID"],
  Cell[1990398, 80752, 389, 16, 50, "Output",
   CellTags->"TID"],
  Cell[1990824, 80773, 123, 4, 26, "Input",
   CellTags->"TID"],
  Cell[1990950, 80779, 627, 27, 49, "Output",
   CellTags->"TID"],
  Cell[1991614, 80811, 195, 6, 26, "Input",
   CellTags->"TID"],
  Cell[1991812, 80819, 627, 27, 49, "Output",
   CellTags->"TID"]},
 "TIDL"->{
  Cell[1992466, 80850, 45, 1, 39, "ObjectName",
   CellTags->"TIDL"],
  Cell[1992536, 80855, 55, 1, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[1992594, 80858, 87, 1, 24, "Text",
   CellTags->"TIDL"],
  Cell[1992684, 80861, 231, 10, 24, "Text",
   CellTags->"TIDL"],
  Cell[1992930, 80874, 52, 1, 36, "Subsubsection",
   CellTags->"TIDL"],
  Cell[1993007, 80879, 154, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[1993164, 80886, 414, 13, 27, "Text",
   CellTags->"TIDL"],
  Cell[1993603, 80903, 202, 7, 26, "Input",
   CellTags->"TIDL"],
  Cell[1993808, 80912, 465, 20, 47, "Output",
   CellTags->"TIDL"],
  Cell[1994322, 80938, 159, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[1994506, 80947, 312, 11, 26, "Input",
   CellTags->"TIDL"],
  Cell[1994821, 80960, 1876, 74, 51, "Output",
   CellTags->"TIDL"],
  Cell[1996746, 81040, 154, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[1996925, 81049, 277, 10, 26, "Input",
   CellTags->"TIDL"],
  Cell[1997205, 81061, 2812, 108, 49, "Output",
   CellTags->"TIDL"],
  Cell[2000066, 81175, 159, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2000250, 81184, 387, 14, 26, "Input",
   CellTags->"TIDL"],
  Cell[2000640, 81200, 24393, 841, 325, "Output",
   CellTags->"TIDL"],
  Cell[2025082, 82047, 165, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2025250, 82054, 434, 15, 26, "Input",
   CellTags->"TIDL"],
  Cell[2025721, 82074, 173, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2025897, 82081, 506, 17, 26, "Input",
   CellTags->"TIDL"],
  Cell[2026440, 82103, 154, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2026619, 82112, 312, 11, 26, "Input",
   CellTags->"TIDL"],
  Cell[2026934, 82125, 14684, 522, 143, "Output",
   CellTags->"TIDL"],
  Cell[2041667, 82653, 159, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2041829, 82660, 374, 13, 26, "Input",
   CellTags->"TIDL"],
  Cell[2042240, 82678, 165, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2042408, 82685, 444, 15, 26, "Input",
   CellTags->"TIDL"],
  Cell[2042889, 82705, 173, 5, 38, "Subsubsection",
   CellTags->"TIDL"],
  Cell[2043065, 82712, 529, 18, 26, "Input",
   CellTags->"TIDL"],
  Cell[2043619, 82734, 240, 7, 26, "Input",
   CellTags->"TIDL"],
  Cell[2043862, 82743, 161, 5, 26, "Output",
   CellTags->"TIDL"]},
 "TimedIntegrate"->{
  Cell[2044050, 82752, 65, 1, 39, "ObjectName",
   CellTags->"TimedIntegrate"],
  Cell[2044140, 82757, 65, 1, 36, "Subsubsection",
   CellTags->"TimedIntegrate"],
  Cell[2044208, 82760, 225, 5, 41, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2044458, 82769, 128, 3, 26, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2044589, 82774, 424, 13, 26, "Output",
   CellTags->"TimedIntegrate"],
  Cell[2045062, 82793, 62, 1, 36, "Subsubsection",
   CellTags->"TimedIntegrate"],
  Cell[2045127, 82796, 73, 1, 24, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2045225, 82801, 305, 9, 26, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2045533, 82812, 129, 4, 26, "Output",
   CellTags->"TimedIntegrate"],
  Cell[2045677, 82819, 59, 1, 24, "Text",
   CellTags->"TimedIntegrate"],
  Cell[2045761, 82824, 389, 11, 26, "Input",
   CellTags->"TimedIntegrate"],
  Cell[2046153, 82837, 449, 14, 29, "Output",
   CellTags->"TimedIntegrate"]},
 "TLI"->{
  Cell[2046629, 82855, 61, 1, 39, "ObjectName",
   CellTags->"TLI"],
  Cell[2046715, 82860, 54, 1, 36, "Subsubsection",
   CellTags->"TLI"],
  Cell[2046772, 82863, 616, 12, 116, "Text",
   CellTags->"TLI"],
  Cell[2047413, 82879, 106, 3, 26, "Input",
   CellTags->"TLI"],
  Cell[2047522, 82884, 229, 8, 28, "Output",
   CellTags->"TLI"],
  Cell[2047766, 82895, 248, 11, 24, "Text",
   CellTags->"TLI"],
  Cell[2048051, 82911, 51, 1, 36, "Subsubsection",
   CellTags->"TLI"],
  Cell[2048105, 82914, 70, 2, 26, "Input",
   CellTags->"TLI"]},
 "TLIFP"->{
  Cell[2048190, 82919, 65, 1, 39, "ObjectName",
   CellTags->"TLIFP"],
  Cell[2048280, 82924, 56, 1, 36, "Subsubsection",
   CellTags->"TLIFP"],
  Cell[2048339, 82927, 93, 1, 24, "Text",
   CellTags->"TLIFP"],
  Cell[2048457, 82932, 110, 3, 26, "Input",
   CellTags->"TLIFP"],
  Cell[2048570, 82937, 425, 12, 28, "Output",
   CellTags->"TLIFP"],
  Cell[2049010, 82952, 238, 11, 24, "Text",
   CellTags->"TLIFP"],
  Cell[2049285, 82968, 53, 1, 36, "Subsubsection",
   CellTags->"TLIFP"],
  Cell[2049341, 82971, 72, 2, 26, "Input",
   CellTags->"TLIFP"]},
 "TLIHYP"->{
  Cell[2049428, 82976, 67, 1, 39, "ObjectName",
   CellTags->"TLIHYP"],
  Cell[2049520, 82981, 57, 1, 36, "Subsubsection",
   CellTags->"TLIHYP"],
  Cell[2049580, 82984, 136, 4, 24, "Text",
   CellTags->"TLIHYP"],
  Cell[2049741, 82992, 112, 3, 26, "Input",
   CellTags->"TLIHYP"],
  Cell[2049856, 82997, 241, 8, 28, "Output",
   CellTags->"TLIHYP"],
  Cell[2050112, 83008, 239, 11, 24, "Text",
   CellTags->"TLIHYP"],
  Cell[2050388, 83024, 54, 1, 36, "Subsubsection",
   CellTags->"TLIHYP"],
  Cell[2050445, 83027, 73, 2, 26, "Input",
   CellTags->"TLIHYP"]},
 "TLI2"->{
  Cell[2050533, 83032, 45, 1, 39, "ObjectName",
   CellTags->"TLI2"],
  Cell[2050603, 83037, 55, 1, 36, "Subsubsection",
   CellTags->"TLI2"],
  Cell[2050661, 83040, 83, 1, 24, "Text",
   CellTags->"TLI2"],
  Cell[2050747, 83043, 237, 11, 24, "Text",
   CellTags->"TLI2"]},
 "TLI2FC"->{
  Cell[2050999, 83057, 67, 1, 39, "ObjectName",
   CellTags->"TLI2FC"],
  Cell[2051091, 83062, 57, 1, 36, "Subsubsection",
   CellTags->"TLI2FC"],
  Cell[2051151, 83065, 109, 3, 24, "Text",
   CellTags->"TLI2FC"],
  Cell[2051285, 83072, 112, 3, 26, "Input",
   CellTags->"TLI2FC"],
  Cell[2051400, 83077, 355, 11, 28, "Output",
   CellTags->"TLI2FC"],
  Cell[2051770, 83091, 419, 17, 24, "Text",
   CellTags->"TLI2FC"],
  Cell[2052226, 83113, 54, 1, 36, "Subsubsection",
   CellTags->"TLI2FC"],
  Cell[2052283, 83116, 73, 2, 26, "Input",
   CellTags->"TLI2FC"]},
 "ToDistribution"->{
  Cell[2052371, 83121, 65, 1, 39, "ObjectName",
   CellTags->"ToDistribution"],
  Cell[2052461, 83126, 65, 1, 36, "Subsubsection",
   CellTags->"ToDistribution"],
  Cell[2052529, 83129, 424, 7, 58, "Text",
   CellTags->"ToDistribution"],
  Cell[2052978, 83140, 128, 3, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2053109, 83145, 189, 6, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2053313, 83154, 299, 11, 24, "Text",
   CellTags->"ToDistribution"],
  Cell[2053649, 83170, 62, 1, 36, "Subsubsection",
   CellTags->"ToDistribution"],
  Cell[2053736, 83175, 217, 6, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2053956, 83183, 197, 6, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2054190, 83194, 277, 9, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2054470, 83205, 1338, 48, 49, "Output",
   CellTags->"ToDistribution"],
  Cell[2055845, 83258, 206, 5, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2056054, 83265, 189, 6, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2056280, 83276, 277, 9, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2056560, 83287, 1380, 50, 50, "Output",
   CellTags->"ToDistribution"],
  Cell[2057977, 83342, 461, 14, 29, "Input",
   CellTags->"ToDistribution"],
  Cell[2058441, 83358, 371, 13, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2058849, 83376, 118, 3, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2058970, 83381, 334, 9, 26, "Output",
   CellTags->"ToDistribution"],
  Cell[2059341, 83395, 409, 13, 26, "Input",
   CellTags->"ToDistribution"],
  Cell[2059753, 83410, 371, 13, 26, "Output",
   CellTags->"ToDistribution"]},
 "ToHypergeometric"->{
  Cell[2060151, 83427, 69, 1, 39, "ObjectName",
   CellTags->"ToHypergeometric"],
  Cell[2060245, 83432, 67, 1, 36, "Subsubsection",
   CellTags->"ToHypergeometric"],
  Cell[2060315, 83435, 1686, 67, 42, "Text",
   CellTags->"ToHypergeometric"],
  Cell[2062004, 83504, 749, 25, 24, "Text",
   CellTags->"ToHypergeometric"],
  Cell[2062790, 83534, 64, 1, 36, "Subsubsection",
   CellTags->"ToHypergeometric"],
  Cell[2062879, 83539, 395, 13, 29, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2063277, 83554, 489, 15, 46, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2063803, 83574, 488, 16, 29, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2064294, 83592, 356, 13, 46, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2064687, 83610, 395, 13, 29, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2065085, 83625, 552, 17, 50, "Output",
   CellTags->"ToHypergeometric"],
  Cell[2065674, 83647, 488, 16, 29, "Input",
   CellTags->"ToHypergeometric"],
  Cell[2066165, 83665, 431, 15, 50, "Output",
   CellTags->"ToHypergeometric"]},
 "ToLarin"->{
  Cell[2066623, 83684, 51, 1, 39, "ObjectName",
   CellTags->"ToLarin"],
  Cell[2066699, 83689, 58, 1, 36, "Subsubsection",
   CellTags->"ToLarin"],
  Cell[2066760, 83692, 605, 19, 24, "Text",
   CellTags->"ToLarin"],
  Cell[2067368, 83713, 240, 11, 24, "Text",
   CellTags->"ToLarin"],
  Cell[2067645, 83729, 55, 1, 36, "Subsubsection",
   CellTags->"ToLarin"],
  Cell[2067725, 83734, 229, 6, 26, "Input",
   CellTags->"ToLarin"],
  Cell[2067957, 83742, 389, 15, 26, "Output",
   CellTags->"ToLarin"],
  Cell[2068383, 83762, 108, 3, 26, "Input",
   CellTags->"ToLarin"],
  Cell[2068494, 83767, 1097, 45, 42, "Output",
   CellTags->"ToLarin"]},
 "ToTFi"->{
  Cell[2069618, 83816, 47, 1, 39, "ObjectName",
   CellTags->"ToTFi"],
  Cell[2069690, 83821, 56, 1, 36, "Subsubsection",
   CellTags->"ToTFi"],
  Cell[2069749, 83824, 272, 5, 41, "Text",
   CellTags->"ToTFi"],
  Cell[2070024, 83831, 420, 17, 24, "Text",
   CellTags->"ToTFi"],
  Cell[2070481, 83853, 53, 1, 36, "Subsubsection",
   CellTags->"ToTFi"],
  Cell[2070559, 83858, 333, 11, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2070895, 83871, 1870, 49, 48, "Output",
   CellTags->"ToTFi"],
  Cell[2072802, 83925, 149, 4, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2072954, 83931, 2126, 54, 48, "Output",
   CellTags->"ToTFi"],
  Cell[2075117, 83990, 107, 3, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2075227, 83995, 491, 16, 38, "Output",
   CellTags->"ToTFi"],
  Cell[2075755, 84016, 468, 15, 26, "Input",
   CellTags->"ToTFi"],
  Cell[2076226, 84033, 916, 33, 48, "Output",
   CellTags->"ToTFi"]},
 "TR"->{
  Cell[2077191, 84072, 41, 1, 39, "ObjectName",
   CellTags->"TR"],
  Cell[2077235, 84075, 100, 3, 26, "Output",
   CellTags->"TR"],
  Cell[2077372, 84083, 53, 1, 36, "Subsubsection",
   CellTags->"TR"],
  Cell[2077428, 84086, 420, 12, 41, "Text",
   CellTags->"TR"],
  Cell[2077873, 84102, 104, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2077980, 84107, 761, 18, 47, "Output",
   CellTags->"TR"],
  Cell[2078756, 84128, 877, 31, 24, "Text",
   CellTags->"TR"],
  Cell[2079670, 84164, 50, 1, 36, "Subsubsection",
   CellTags->"TR"],
  Cell[2079745, 84169, 129, 4, 26, "Input",
   CellTags->"TR"],
  Cell[2079877, 84175, 194, 6, 26, "Output",
   CellTags->"TR"],
  Cell[2080108, 84186, 98, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2080209, 84191, 175, 6, 26, "Output",
   CellTags->"TR"],
  Cell[2080421, 84202, 375, 12, 26, "Input",
   CellTags->"TR"],
  Cell[2080799, 84216, 1257, 50, 29, "Output",
   CellTags->"TR"],
  Cell[2082093, 84271, 205, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2082301, 84279, 559, 23, 26, "Output",
   CellTags->"TR"],
  Cell[2082897, 84307, 240, 7, 26, "Input",
   CellTags->"TR"],
  Cell[2083140, 84316, 8363, 321, 52, "Output",
   CellTags->"TR"],
  Cell[2091540, 84642, 421, 13, 26, "Input",
   CellTags->"TR"],
  Cell[2091964, 84657, 10947, 412, 67, "Output",
   CellTags->"TR"],
  Cell[2102948, 85074, 171, 5, 26, "Input",
   CellTags->"TR"],
  Cell[2103122, 85081, 991, 39, 28, "Output",
   CellTags->"TR"],
  Cell[2104150, 85125, 444, 14, 26, "Input",
   CellTags->"TR"],
  Cell[2104597, 85141, 342, 13, 29, "Output",
   CellTags->"TR"],
  Cell[2104976, 85159, 224, 7, 26, "Input",
   CellTags->"TR"],
  Cell[2105203, 85168, 180, 6, 26, "Output",
   CellTags->"TR"],
  Cell[2105420, 85179, 104, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2105527, 85184, 173, 5, 38, "Output",
   CellTags->"TR"],
  Cell[2105737, 85194, 347, 10, 26, "Input",
   CellTags->"TR"],
  Cell[2106087, 85206, 190, 6, 26, "Output",
   CellTags->"TR"],
  Cell[2106314, 85217, 104, 3, 26, "Input",
   CellTags->"TR"],
  Cell[2106421, 85222, 183, 5, 38, "Output",
   CellTags->"TR"],
  Cell[2106641, 85232, 339, 11, 26, "Input",
   CellTags->"TR"],
  Cell[2106983, 85245, 428, 16, 26, "Output",
   CellTags->"TR"],
  Cell[2107448, 85266, 214, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2107665, 85274, 208, 7, 26, "Output",
   CellTags->"TR"],
  Cell[2107910, 85286, 213, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2108126, 85294, 208, 7, 26, "Output",
   CellTags->"TR"],
  Cell[2108371, 85306, 214, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2108588, 85314, 99, 3, 26, "Output",
   CellTags->"TR"],
  Cell[2108724, 85322, 213, 6, 26, "Input",
   CellTags->"TR"],
  Cell[2108940, 85330, 99, 3, 26, "Output",
   CellTags->"TR"],
  Cell[2114151, 85518, 143, 5, 26, "Input",
   CellTags->"TR"]},
 "Info3535453335-5096475"->{
  Cell[2109531, 85360, 644, 13, 156, "Print",
   CellTags->"Info3535453335-5096475"]},
 "TraceDimension"->{
  Cell[2114309, 85526, 65, 1, 39, "ObjectName",
   CellTags->"TraceDimension"],
  Cell[2114399, 85531, 65, 1, 36, "Subsubsection",
   CellTags->"TraceDimension"],
  Cell[2114467, 85534, 142, 4, 24, "Text",
   CellTags->"TraceDimension"],
  Cell[2114612, 85540, 287, 11, 24, "Text",
   CellTags->"TraceDimension"]},
 "TraceOfOne"->{
  Cell[2114914, 85554, 57, 1, 39, "ObjectName",
   CellTags->"TraceOfOne"],
  Cell[2114996, 85559, 61, 1, 36, "Subsubsection",
   CellTags->"TraceOfOne"],
  Cell[2115060, 85562, 149, 4, 24, "Text",
   CellTags->"TraceOfOne"],
  Cell[2115212, 85568, 357, 15, 24, "Text",
   CellTags->"TraceOfOne"],
  Cell[2115606, 85588, 58, 1, 36, "Subsubsection",
   CellTags->"TraceOfOne"],
  Cell[2115689, 85593, 273, 7, 26, "Input",
   CellTags->"TraceOfOne"],
  Cell[2115965, 85602, 109, 3, 26, "Output",
   CellTags->"TraceOfOne"],
  Cell[2116111, 85610, 272, 7, 26, "Input",
   CellTags->"TraceOfOne"],
  Cell[2116386, 85619, 109, 3, 26, "Output",
   CellTags->"TraceOfOne"]},
 "Trick"->{
  Cell[2116522, 85626, 47, 1, 39, "ObjectName",
   CellTags->"Trick"],
  Cell[2116594, 85631, 56, 1, 36, "Subsubsection",
   CellTags->"Trick"],
  Cell[2116653, 85634, 174, 4, 24, "Text",
   CellTags->"Trick"],
  Cell[2116830, 85640, 1038, 39, 24, "Text",
   CellTags->"Trick"],
  Cell[2117905, 85684, 62, 1, 33, "Subsubsection",
   CellTags->"Trick"],
  Cell[2117970, 85687, 376, 15, 27, "Text",
   CellTags->"Trick"],
  Cell[2118371, 85706, 350, 11, 26, "Input",
   CellTags->"Trick"],
  Cell[2118724, 85719, 264, 10, 26, "Output",
   CellTags->"Trick"],
  Cell[2119025, 85734, 330, 11, 26, "Input",
   CellTags->"Trick"],
  Cell[2119358, 85747, 426, 16, 28, "Output",
   CellTags->"Trick"],
  Cell[2119821, 85768, 104, 3, 26, "Input",
   CellTags->"Trick"],
  Cell[2119928, 85773, 527, 22, 28, "Output",
   CellTags->"Trick"],
  Cell[2120492, 85800, 227, 6, 26, "Input",
   CellTags->"Trick"],
  Cell[2120722, 85808, 391, 15, 28, "Output",
   CellTags->"Trick"],
  Cell[2121150, 85828, 133, 4, 26, "Input",
   CellTags->"Trick"],
  Cell[2121286, 85834, 213, 6, 38, "Output",
   CellTags->"Trick"]},
 "TrickIntegrate"->{
  Cell[2121526, 85844, 65, 1, 39, "ObjectName",
   CellTags->"TrickIntegrate"],
  Cell[2121616, 85849, 65, 1, 36, "Subsubsection",
   CellTags->"TrickIntegrate"],
  Cell[2121684, 85852, 483, 8, 58, "Text",
   CellTags->"TrickIntegrate"],
  Cell[2122204, 85865, 62, 1, 36, "Subsubsection",
   CellTags->"TrickIntegrate"],
  Cell[2122291, 85870, 364, 12, 26, "Input",
   CellTags->"TrickIntegrate"],
  Cell[2122658, 85884, 517, 18, 29, "Output",
   CellTags->"TrickIntegrate"]},
 "TrickMandelstam"->{
  Cell[2123202, 85906, 67, 1, 39, "ObjectName",
   CellTags->"TrickMandelstam"],
  Cell[2123294, 85911, 66, 1, 36, "Subsubsection",
   CellTags->"TrickMandelstam"],
  Cell[2123363, 85914, 1340, 55, 45, "Text",
   CellTags->"TrickMandelstam"],
  Cell[2124740, 85974, 63, 1, 36, "Subsubsection",
   CellTags->"TrickMandelstam"],
  Cell[2124828, 85979, 528, 18, 29, "Input",
   CellTags->"TrickMandelstam"],
  Cell[2125359, 85999, 250, 9, 29, "Output",
   CellTags->"TrickMandelstam"],
  Cell[2125646, 86013, 568, 17, 26, "Input",
   CellTags->"TrickMandelstam"],
  Cell[2126217, 86032, 235, 8, 26, "Output",
   CellTags->"TrickMandelstam"]},
 "Tr2"->{
  Cell[2126479, 86044, 43, 1, 39, "ObjectName",
   CellTags->"Tr2"],
  Cell[2126547, 86049, 54, 1, 36, "Subsubsection",
   CellTags->"Tr2"],
  Cell[2126604, 86052, 347, 6, 58, "Text",
   CellTags->"Tr2"],
  Cell[2126976, 86062, 106, 3, 26, "Input",
   CellTags->"Tr2"],
  Cell[2127085, 86067, 168, 6, 26, "Output",
   CellTags->"Tr2"],
  Cell[2127268, 86076, 998, 38, 24, "Text",
   CellTags->"Tr2"],
  Cell[2128303, 86119, 51, 1, 36, "Subsubsection",
   CellTags->"Tr2"],
  Cell[2129131, 86155, 187, 6, 26, "Input",
   CellTags->"Tr2"],
  Cell[2129321, 86163, 156, 5, 29, "Output",
   CellTags->"Tr2"]},
 "Twist2AlienOperator"->{
  Cell[2129504, 86172, 116, 2, 39, "ObjectName",
   CellTags->"Twist2AlienOperator"],
  Cell[2129645, 86178, 70, 1, 36, "Subsubsection",
   CellTags->"Twist2AlienOperator"],
  Cell[2129718, 86181, 194, 4, 24, "Text",
   CellTags->"Twist2AlienOperator"],
  Cell[2129949, 86190, 67, 1, 36, "Subsubsection",
   CellTags->"Twist2AlienOperator"],
  Cell[2130041, 86195, 160, 4, 26, "Input",
   CellTags->"Twist2AlienOperator"],
  Cell[2130204, 86201, 887, 30, 49, "Output",
   CellTags->"Twist2AlienOperator"],
  Cell[2131128, 86236, 246, 7, 26, "Input",
   CellTags->"Twist2AlienOperator"],
  Cell[2131377, 86245, 3113, 103, 53, "Output",
   CellTags->"Twist2AlienOperator"]},
 "Twist2CounterOperator"->{
  Cell[2134517, 86352, 120, 2, 39, "ObjectName",
   CellTags->"Twist2CounterOperator"],
  Cell[2134662, 86358, 72, 1, 28, "Subsubsection",
   CellTags->"Twist2CounterOperator"],
  Cell[2134737, 86361, 239, 5, 37, "Text",
   CellTags->"Twist2CounterOperator"],
  Cell[2135013, 86371, 69, 1, 36, "Subsubsection",
   CellTags->"Twist2CounterOperator"],
  Cell[2135107, 86376, 204, 5, 26, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2135314, 86383, 4002, 136, 85, "Output",
   CellTags->"Twist2CounterOperator"],
  Cell[2139353, 86524, 159, 4, 26, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2139515, 86530, 1020, 35, 49, "Output",
   CellTags->"Twist2CounterOperator"],
  Cell[2140572, 86570, 242, 6, 26, "Input",
   CellTags->"Twist2CounterOperator"],
  Cell[2140817, 86578, 2610, 89, 53, "Output",
   CellTags->"Twist2CounterOperator"]},
 "Twist2GluonOperator"->{
  Cell[2143454, 86671, 75, 1, 39, "ObjectName",
   CellTags->"Twist2GluonOperator"],
  Cell[2143554, 86676, 70, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2143627, 86679, 964, 15, 126, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2144594, 86696, 316, 11, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2144947, 86712, 99, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2145071, 86717, 352, 8, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2145426, 86727, 444, 12, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2145885, 86742, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2145972, 86747, 271, 8, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2146246, 86757, 894, 35, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2147177, 86797, 321, 9, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2147501, 86808, 1120, 45, 28, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2148658, 86858, 221, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2148882, 86866, 336, 12, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2149255, 86883, 271, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2149529, 86892, 705, 28, 28, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2150249, 86923, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2150336, 86928, 355, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2150694, 86940, 1193, 48, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2151924, 86993, 325, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2152252, 87005, 492, 18, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2152781, 87028, 375, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2153159, 87041, 11308, 411, 133, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2164482, 87455, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2164569, 87460, 431, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2165003, 87474, 4976, 185, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2170016, 87664, 391, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2170410, 87678, 606, 22, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2171053, 87705, 441, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2171497, 87720, 24305, 869, 387, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2195851, 88595, 100, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2195976, 88600, 268, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2196247, 88608, 444, 12, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2196706, 88623, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2196793, 88628, 271, 8, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2197067, 88638, 904, 36, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2198008, 88679, 321, 9, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2198332, 88690, 2656, 102, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2201025, 88797, 221, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2201249, 88805, 336, 12, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2201622, 88822, 271, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2201896, 88831, 2231, 84, 29, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2204142, 88918, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2204229, 88923, 355, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2204587, 88935, 1267, 50, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2205891, 88990, 325, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2206219, 89002, 492, 18, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2206748, 89025, 375, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2207126, 89038, 16342, 587, 207, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2223483, 89628, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2223570, 89633, 431, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2224004, 89647, 4986, 186, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2229027, 89838, 391, 12, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2229421, 89852, 606, 22, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2230064, 89879, 441, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2230508, 89894, 31861, 1141, 463, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2262418, 91041, 76, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2262497, 91044, 97, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2262597, 91047, 295, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2262895, 91056, 380, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2263312, 91072, 102, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2263439, 91077, 284, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2263726, 91085, 443, 12, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2264184, 91100, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2264271, 91105, 320, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2264594, 91117, 3609, 129, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2268218, 91249, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2268305, 91254, 355, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2268663, 91266, 54187, 1690, 848, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2322865, 92959, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2322952, 92964, 482, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2323437, 92979, 4976, 185, 30, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2328462, 93170, 104, 1, 36, "Subsubsection",
   CellTags->"Twist2GluonOperator"],
  Cell[2328591, 93175, 284, 6, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2328878, 93183, 443, 12, 26, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2329336, 93198, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2329423, 93203, 320, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2329746, 93215, 3074, 116, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2332835, 93334, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2332900, 93337, 380, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2333305, 93352, 325, 10, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2333633, 93364, 3075, 116, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2336745, 93485, 375, 11, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2337123, 93498, 3075, 116, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2340213, 93617, 62, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2340300, 93622, 482, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2340785, 93637, 4986, 186, 42, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2345786, 93826, 78, 1, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2345889, 93831, 520, 16, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2346412, 93849, 10907, 376, 150, "Output",
   CellTags->"Twist2GluonOperator"],
  Cell[2357334, 94228, 109, 3, 24, "Text",
   CellTags->"Twist2GluonOperator"],
  Cell[2357446, 94233, 295, 7, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2357766, 94244, 428, 13, 26, "Input",
   CellTags->"Twist2GluonOperator"],
  Cell[2358197, 94259, 12432, 394, 173, "Output",
   CellTags->"Twist2GluonOperator"]},
 "Twist2QuarkOperator"->{
  Cell[2370656, 94657, 75, 1, 39, "ObjectName",
   CellTags->"Twist2QuarkOperator"],
  Cell[2370756, 94662, 70, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2370829, 94665, 980, 15, 126, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2371812, 94682, 316, 11, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2372143, 94696, 67, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2372235, 94701, 98, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2372358, 94706, 268, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2372629, 94714, 443, 12, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2373087, 94729, 75, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2373187, 94734, 156, 4, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2373346, 94740, 600, 21, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2373961, 94764, 81, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2374067, 94769, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2374375, 94780, 1581, 57, 54, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2375971, 94840, 87, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2376083, 94845, 380, 11, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2376466, 94858, 4917, 163, 114, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2381432, 95027, 100, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2381557, 95032, 273, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2381833, 95040, 443, 12, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2382291, 95055, 75, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2382391, 95060, 156, 4, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2382550, 95066, 481, 17, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2383046, 95086, 81, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2383152, 95091, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2383460, 95102, 1492, 54, 54, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2384967, 95159, 87, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2385079, 95164, 380, 11, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2385462, 95177, 4776, 159, 78, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2390253, 95339, 93, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2390371, 95344, 197, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2390571, 95352, 263, 8, 38, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2390871, 95365, 376, 10, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2391250, 95377, 1971, 72, 49, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2393258, 95454, 449, 12, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2393710, 95468, 9405, 298, 96, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2403164, 95772, 102, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2403291, 95777, 269, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2403563, 95785, 444, 12, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2404022, 95800, 178, 4, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2404203, 95806, 275, 5, 41, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2404503, 95815, 205, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2404711, 95823, 800, 28, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2405526, 95854, 100, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2405651, 95859, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2405959, 95870, 2305, 78, 54, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2408279, 95951, 87, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2408391, 95956, 156, 4, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2408550, 95962, 1102, 33, 62, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2409701, 96001, 104, 1, 36, "Subsubsection",
   CellTags->"Twist2QuarkOperator"],
  Cell[2409830, 96006, 269, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2410102, 96014, 444, 12, 26, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2410561, 96029, 75, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2410661, 96034, 205, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2410869, 96042, 800, 28, 28, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2411684, 96073, 81, 1, 24, "Text",
   CellTags->"Twist2QuarkOperator"],
  Cell[2411790, 96078, 305, 9, 26, "Input",
   CellTags->"Twist2QuarkOperator"],
  Cell[2412098, 96089, 2305, 78, 54, "Output",
   CellTags->"Twist2QuarkOperator"],
  Cell[2414418, 96170, 210, 6, 26, "Input",
   CellTags->"Twist2QuarkOperator"]},
 "Twist3QuarkOperator"->{
  Cell[2414643, 96179, 75, 1, 39, "ObjectName",
   CellTags->"Twist3QuarkOperator"],
  Cell[2414743, 96184, 70, 1, 36, "Subsubsection",
   CellTags->"Twist3QuarkOperator"],
  Cell[2414816, 96187, 768, 12, 92, "Text",
   CellTags->"Twist3QuarkOperator"],
  Cell[2415609, 96203, 134, 3, 26, "Input",
   CellTags->"Twist3QuarkOperator"],
  Cell[2415746, 96208, 328, 10, 26, "Output",
   CellTags->"Twist3QuarkOperator"],
  Cell[2416089, 96221, 552, 18, 24, "Text",
   CellTags->"Twist3QuarkOperator"],
  Cell[2416678, 96244, 67, 1, 36, "Subsubsection",
   CellTags->"Twist3QuarkOperator"],
  Cell[2416770, 96249, 132, 3, 26, "Input",
   CellTags->"Twist3QuarkOperator"],
  Cell[2416905, 96254, 654, 23, 28, "Output",
   CellTags->"Twist3QuarkOperator"]},
 "Twist4GluonOperator"->{
  Cell[2417586, 96281, 75, 1, 39, "ObjectName",
   CellTags->"Twist4GluonOperator"],
  Cell[2417686, 96286, 70, 1, 36, "Subsubsection",
   CellTags->"Twist4GluonOperator"],
  Cell[2417759, 96289, 150, 4, 24, "Text",
   CellTags->"Twist4GluonOperator"],
  Cell[2417912, 96295, 788, 25, 24, "Text",
   CellTags->"Twist4GluonOperator"],
  Cell[2418737, 96325, 67, 1, 36, "Subsubsection",
   CellTags->"Twist4GluonOperator"],
  Cell[2418829, 96330, 529, 15, 26, "Input",
   CellTags->"Twist4GluonOperator"],
  Cell[2419361, 96347, 119831, 4588, 1110, "Output",
   CellTags->"Twist4GluonOperator"]},
 "TwoLoopSimplify"->{
  Cell[2539219, 100939, 104, 1, 39, "ObjectName",
   CellTags->"TwoLoopSimplify"],
  Cell[2539348, 100944, 66, 1, 36, "Subsubsection",
   CellTags->"TwoLoopSimplify"],
  Cell[2539417, 100947, 335, 6, 58, "Text",
   CellTags->"TwoLoopSimplify"],
  Cell[2539755, 100955, 296, 11, 24, "Text",
   CellTags->"TwoLoopSimplify"],
  Cell[2540088, 100971, 63, 1, 36, "Subsubsection",
   CellTags->"TwoLoopSimplify"],
  Cell[2540154, 100974, 82, 2, 26, "Input",
   CellTags->"TwoLoopSimplify"]},
 "Uncontract"->{
  Cell[2540251, 100979, 57, 1, 39, "ObjectName",
   CellTags->"Uncontract"],
  Cell[2540333, 100984, 61, 1, 36, "Subsubsection",
   CellTags->"Uncontract"],
  Cell[2540397, 100987, 256, 5, 41, "Text",
   CellTags->"Uncontract"],
  Cell[2540678, 100996, 120, 3, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2540801, 101001, 372, 11, 26, "Output",
   CellTags->"Uncontract"],
  Cell[2541188, 101015, 263, 11, 24, "Text",
   CellTags->"Uncontract"],
  Cell[2541488, 101031, 58, 1, 36, "Subsubsection",
   CellTags->"Uncontract"],
  Cell[2541571, 101036, 234, 7, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2541808, 101045, 397, 17, 26, "Output",
   CellTags->"Uncontract"],
  Cell[2542242, 101067, 143, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2542388, 101073, 781, 31, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2543206, 101109, 138, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2543347, 101115, 224, 8, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2543608, 101128, 143, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2543754, 101134, 224, 8, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2544015, 101147, 153, 4, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2544171, 101153, 1138, 44, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2545324, 101200, 89, 1, 24, "Text",
   CellTags->"Uncontract"],
  Cell[2545438, 101205, 208, 6, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2545649, 101213, 209, 8, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2545873, 101224, 152, 4, 24, "Text",
   CellTags->"Uncontract"],
  Cell[2546050, 101232, 248, 7, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2546301, 101241, 570, 22, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2546908, 101268, 274, 8, 26, "Input",
   CellTags->"Uncontract"],
  Cell[2547185, 101278, 1030, 40, 28, "Output",
   CellTags->"Uncontract"],
  Cell[2548230, 101321, 134, 4, 26, "Input",
   CellTags->"Uncontract"]},
 "UnDeclareNonCommutative"->{
  Cell[2548379, 101328, 83, 1, 39, "ObjectName",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2548487, 101333, 74, 1, 36, "Subsubsection",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2548564, 101336, 198, 4, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2548765, 101342, 520, 18, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2549322, 101365, 71, 1, 36, "Subsubsection",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2549396, 101368, 138, 3, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2549537, 101373, 153, 4, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2549715, 101381, 107, 2, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2549825, 101385, 123, 3, 26, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2549963, 101391, 103, 1, 24, "Text",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2550069, 101394, 140, 3, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2550234, 101401, 161, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2550398, 101407, 123, 3, 26, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2550536, 101413, 161, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2550722, 101421, 171, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2550896, 101427, 173, 5, 26, "Output",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2551084, 101435, 163, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2551272, 101443, 171, 4, 26, "Input",
   CellTags->"UnDeclareNonCommutative"],
  Cell[2551446, 101449, 173, 5, 26, "Output",
   CellTags->"UnDeclareNonCommutative"]},
 "Upper"->{
  Cell[2551646, 101458, 47, 1, 39, "ObjectName",
   CellTags->"Upper"],
  Cell[2551718, 101463, 56, 1, 28, "Subsubsection",
   CellTags->"Upper"],
  Cell[2551777, 101466, 126, 4, 22, "Text",
   CellTags->"Upper"],
  Cell[2551906, 101472, 645, 24, 22, "Text",
   CellTags->"Upper"]},
 "UVPart"->{
  Cell[2552566, 101499, 86, 1, 39, "ObjectName",
   CellTags->"UVPart"],
  Cell[2552677, 101504, 57, 1, 28, "Subsubsection",
   CellTags->"UVPart"],
  Cell[2552737, 101507, 128, 4, 22, "Text",
   CellTags->"UVPart"],
  Cell[2552902, 101516, 54, 1, 36, "Subsubsection",
   CellTags->"UVPart"],
  Cell[2552959, 101519, 73, 2, 26, "Input",
   CellTags->"UVPart"]},
 "Vectors"->{
  Cell[2553047, 101524, 51, 1, 39, "ObjectName",
   CellTags->"Vectors"],
  Cell[2553123, 101529, 58, 1, 28, "Subsubsection",
   CellTags->"Vectors"],
  Cell[2553184, 101532, 203, 4, 22, "Text",
   CellTags->"Vectors"],
  Cell[2553390, 101538, 275, 10, 22, "Text",
   CellTags->"Vectors"]},
 "WriteOut"->{
  Cell[2553680, 101551, 53, 1, 39, "ObjectName",
   CellTags->"WriteOut"],
  Cell[2553758, 101556, 59, 1, 28, "Subsubsection",
   CellTags->"WriteOut"],
  Cell[2553820, 101559, 216, 5, 22, "Text",
   CellTags->"WriteOut"],
  Cell[2554039, 101566, 252, 10, 22, "Text",
   CellTags->"WriteOut"]},
 "WriteOutPaVe"->{
  Cell[2554306, 101579, 61, 1, 39, "ObjectName",
   CellTags->"WriteOutPaVe"],
  Cell[2554392, 101584, 63, 1, 28, "Subsubsection",
   CellTags->"WriteOutPaVe"],
  Cell[2554458, 101587, 260, 5, 37, "Text",
   CellTags->"WriteOutPaVe"],
  Cell[2554721, 101594, 644, 24, 22, "Text",
   CellTags->"WriteOutPaVe"]},
 "Write2"->{
  Cell[2555380, 101621, 49, 1, 39, "ObjectName",
   CellTags->"Write2"],
  Cell[2555454, 101626, 57, 1, 28, "Subsubsection",
   CellTags->"Write2"],
  Cell[2555514, 101629, 297, 6, 37, "Text",
   CellTags->"Write2"],
  Cell[2555814, 101637, 153, 4, 24, "Text",
   CellTags->"Write2"],
  Cell[2555992, 101645, 136, 3, 29, "Input",
   CellTags->"Write2"],
  Cell[2556131, 101650, 248, 9, 39, "Output",
   CellTags->"Write2"],
  Cell[2556416, 101664, 112, 3, 26, "Input",
   CellTags->"Write2"],
  Cell[2556531, 101669, 637, 15, 47, "Output",
   CellTags->"Write2"],
  Cell[2557205, 101689, 115, 3, 26, "Input",
   CellTags->"Write2"],
  Cell[2557323, 101694, 133, 4, 26, "Output",
   CellTags->"Write2"],
  Cell[2557471, 101701, 455, 18, 22, "Text",
   CellTags->"Write2"],
  Cell[2557963, 101724, 54, 1, 36, "Subsubsection",
   CellTags->"Write2"],
  Cell[2558042, 101729, 429, 15, 35, "Input",
   CellTags->"Write2"],
  Cell[2558474, 101746, 855, 29, 31, "Output",
   CellTags->"Write2"],
  Cell[2559344, 101778, 208, 7, 29, "Input",
   CellTags->"Write2"],
  Cell[2559555, 101787, 78, 1, 24, "Text",
   CellTags->"Write2"],
  Cell[2559636, 101790, 184, 6, 26, "Input",
   CellTags->"Write2"],
  Cell[2559823, 101798, 73, 1, 24, "Text",
   CellTags->"Write2"],
  Cell[2559921, 101803, 363, 11, 29, "Input",
   CellTags->"Write2"],
  Cell[2560287, 101816, 801, 22, 71, "Output",
   CellTags->"Write2"],
  Cell[2561103, 101841, 286, 8, 29, "Input",
   CellTags->"Write2"],
  Cell[2561414, 101853, 233, 7, 26, "Input",
   CellTags->"Write2"],
  Cell[2561650, 101862, 710, 27, 26, "Output",
   CellTags->"Write2"],
  Cell[2562375, 101892, 185, 6, 26, "Input",
   CellTags->"Write2"],
  Cell[2562585, 101902, 363, 11, 29, "Input",
   CellTags->"Write2"],
  Cell[2562951, 101915, 1061, 30, 195, "Output",
   CellTags->"Write2"],
  Cell[2564027, 101948, 286, 8, 29, "Input",
   CellTags->"Write2"],
  Cell[2564316, 101958, 106, 3, 24, "Text",
   CellTags->"Write2"],
  Cell[2564425, 101963, 247, 8, 26, "Input",
   CellTags->"Write2"],
  Cell[2564697, 101975, 363, 11, 29, "Input",
   CellTags->"Write2"],
  Cell[2565063, 101988, 954, 25, 126, "Output",
   CellTags->"Write2"],
  Cell[2566032, 102016, 397, 11, 29, "Input",
   CellTags->"Write2"]},
 "XYT"->{
  Cell[2566444, 102030, 43, 1, 39, "ObjectName",
   CellTags->"XYT"],
  Cell[2566512, 102035, 54, 1, 36, "Subsubsection",
   CellTags->"XYT"],
  Cell[2566569, 102038, 215, 9, 24, "Text",
   CellTags->"XYT"]},
 "ZeroMomentumInsertion"->{
  Cell[2566799, 102050, 79, 1, 39, "ObjectName",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2566903, 102055, 72, 1, 36, "Subsubsection",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2566978, 102058, 154, 4, 24, "Text",
   CellTags->"ZeroMomentumInsertion"],
  Cell[2567135, 102064, 741, 24, 24, "Text",
   CellTags->"ZeroMomentumInsertion"]},
 "Zeta2"->{
  Cell[2567891, 102091, 47, 1, 39, "ObjectName",
   CellTags->"Zeta2"],
  Cell[2567963, 102096, 56, 1, 36, "Subsubsection",
   CellTags->"Zeta2"],
  Cell[2568022, 102099, 94, 1, 24, "Text",
   CellTags->"Zeta2"],
  Cell[2568119, 102102, 280, 10, 24, "Text",
   CellTags->"Zeta2"],
  Cell[2568436, 102117, 53, 1, 36, "Subsubsection",
   CellTags->"Zeta2"],
  Cell[2568514, 102122, 77, 2, 26, "Input",
   CellTags->"Zeta2"],
  Cell[2568594, 102126, 136, 4, 26, "Output",
   CellTags->"Zeta2"],
  Cell[2568767, 102135, 104, 3, 26, "Input",
   CellTags->"Zeta2"],
  Cell[2568874, 102140, 120, 3, 26, "Output",
   CellTags->"Zeta2"],
  Cell[2569031, 102148, 138, 4, 26, "Input",
   CellTags->"Zeta2"],
  Cell[2569172, 102154, 160, 5, 26, "Output",
   CellTags->"Zeta2"]},
 "$Abbreviations"->{
  Cell[2569359, 102163, 65, 1, 39, "ObjectName",
   CellTags->"$Abbreviations"],
  Cell[2569449, 102168, 65, 1, 28, "Subsubsection",
   CellTags->"$Abbreviations"],
  Cell[2569517, 102171, 293, 6, 41, "Text",
   CellTags->"$Abbreviations"],
  Cell[2569835, 102181, 129, 3, 26, "Input",
   CellTags->"$Abbreviations"],
  Cell[2569967, 102186, 867, 17, 77, "Output",
   CellTags->"$Abbreviations"],
  Cell[2570849, 102206, 1071, 39, 22, "Text",
   CellTags->"$Abbreviations"]},
 "$AL"->{
  Cell[2571935, 102248, 43, 1, 39, "ObjectName",
   CellTags->"$AL"],
  Cell[2572003, 102253, 54, 1, 28, "Subsubsection",
   CellTags->"$AL"],
  Cell[2572060, 102256, 114, 3, 24, "Text",
   CellTags->"$AL"],
  Cell[2572199, 102263, 73, 2, 26, "Input",
   CellTags->"$AL"],
  Cell[2572275, 102267, 102, 3, 26, "Output",
   CellTags->"$AL"],
  Cell[2572414, 102275, 241, 7, 26, "Input",
   CellTags->"$AL"],
  Cell[2572658, 102284, 563, 22, 28, "Output",
   CellTags->"$AL"],
  Cell[2573236, 102309, 117, 4, 26, "Input",
   CellTags->"$AL"],
  Cell[2573378, 102317, 241, 7, 26, "Input",
   CellTags->"$AL"],
  Cell[2573622, 102326, 477, 20, 28, "Output",
   CellTags->"$AL"],
  Cell[2574114, 102349, 109, 4, 26, "Input",
   CellTags->"$AL"]},
 "$BreitMaison"->{
  Cell[2574238, 102356, 61, 1, 39, "ObjectName",
   CellTags->"$BreitMaison"],
  Cell[2574324, 102361, 63, 1, 28, "Subsubsection",
   CellTags->"$BreitMaison"],
  Cell[2574390, 102364, 319, 9, 37, "Text",
   CellTags->"$BreitMaison"],
  Cell[2574712, 102375, 605, 25, 24, "Text",
   CellTags->"$BreitMaison"]},
 "$Color"->{
  Cell[2575332, 102403, 49, 1, 39, "ObjectName",
   CellTags->"$Color"],
  Cell[2575406, 102408, 57, 1, 36, "Subsubsection",
   CellTags->"$Color"],
  Cell[2575466, 102411, 130, 4, 24, "Text",
   CellTags->"$Color"]},
 "$Covariant"->{
  Cell[2575611, 102418, 57, 1, 39, "ObjectName",
   CellTags->"$Covariant"],
  Cell[2575693, 102423, 61, 1, 36, "Subsubsection",
   CellTags->"$Covariant"],
  Cell[2575757, 102426, 183, 4, 24, "Text",
   CellTags->"$Covariant"],
  Cell[2575965, 102434, 87, 2, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2576055, 102438, 111, 3, 26, "Output",
   CellTags->"$Covariant"],
  Cell[2576203, 102446, 110, 3, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2576316, 102451, 142, 4, 26, "Output",
   CellTags->"$Covariant"],
  Cell[2576473, 102458, 130, 4, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2576628, 102466, 110, 3, 26, "Input",
   CellTags->"$Covariant"],
  Cell[2576741, 102471, 140, 4, 29, "Output",
   CellTags->"$Covariant"],
  Cell[2576896, 102478, 131, 4, 26, "Input",
   CellTags->"$Covariant"]},
 "$FCS"->{
  Cell[2577042, 102485, 45, 1, 39, "ObjectName",
   CellTags->"$FCS"],
  Cell[2577112, 102490, 55, 1, 36, "Subsubsection",
   CellTags->"$FCS"],
  Cell[2577170, 102493, 142, 4, 24, "Text",
   CellTags->"$FCS"],
  Cell[2577337, 102501, 75, 2, 26, "Input",
   CellTags->"$FCS"],
  Cell[2577415, 102505, 700, 14, 47, "Output",
   CellTags->"$FCS"]},
 "$FCT"->{
  Cell[2578142, 102523, 45, 1, 39, "ObjectName",
   CellTags->"$FCT"],
  Cell[2578212, 102528, 55, 1, 36, "Subsubsection",
   CellTags->"$FCT"],
  Cell[2578270, 102531, 177, 4, 24, "Text",
   CellTags->"$FCT"],
  Cell[2578450, 102537, 128, 4, 26, "Input",
   CellTags->"$FCT"],
  Cell[2578603, 102545, 170, 5, 26, "Input",
   CellTags->"$FCT"],
  Cell[2578776, 102552, 196, 6, 26, "Output",
   CellTags->"$FCT"],
  Cell[2578987, 102561, 129, 4, 26, "Input",
   CellTags->"$FCT"],
  Cell[2579141, 102569, 170, 5, 26, "Input",
   CellTags->"$FCT"],
  Cell[2579314, 102576, 196, 6, 26, "Output",
   CellTags->"$FCT"]},
 "$FeynCalcStuff"->{
  Cell[2579537, 102586, 65, 1, 39, "ObjectName",
   CellTags->"$FeynCalcStuff"],
  Cell[2579627, 102591, 65, 1, 36, "Subsubsection",
   CellTags->"$FeynCalcStuff"],
  Cell[2579695, 102594, 102, 1, 24, "Text",
   CellTags->"$FeynCalcStuff"]},
 "$FeynCalcVersion"->{
  Cell[2579812, 102598, 69, 1, 39, "ObjectName",
   CellTags->"$FeynCalcVersion"],
  Cell[2579906, 102603, 67, 1, 36, "Subsubsection",
   CellTags->"$FeynCalcVersion"],
  Cell[2579976, 102606, 124, 3, 24, "Text",
   CellTags->"$FeynCalcVersion"],
  Cell[2580125, 102613, 99, 2, 26, "Input",
   CellTags->"$FeynCalcVersion"],
  Cell[2580227, 102617, 125, 3, 26, "Output",
   CellTags->"$FeynCalcVersion"]},
 "$FortranContinuationCharacter"->{
  Cell[2580379, 102624, 95, 1, 39, "ObjectName",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2580499, 102629, 80, 1, 36, "Subsubsection",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2580582, 102632, 143, 3, 24, "Text",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2580750, 102639, 125, 2, 26, "Input",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2580878, 102643, 134, 3, 26, "Output",
   CellTags->"$FortranContinuationCharacter"],
  Cell[2581027, 102649, 274, 11, 24, "Text",
   CellTags->"$FortranContinuationCharacter"]},
 "$Gauge"->{
  Cell[2581316, 102663, 49, 1, 39, "ObjectName",
   CellTags->"$Gauge"],
  Cell[2581390, 102668, 57, 1, 36, "Subsubsection",
   CellTags->"$Gauge"],
  Cell[2581450, 102671, 260, 5, 41, "Text",
   CellTags->"$Gauge"],
  Cell[2581713, 102678, 247, 11, 24, "Text",
   CellTags->"$Gauge"]},
 "$Larin"->{
  Cell[2581975, 102692, 49, 1, 39, "ObjectName",
   CellTags->"$Larin"],
  Cell[2582049, 102697, 57, 1, 36, "Subsubsection",
   CellTags->"$Larin"],
  Cell[2582109, 102700, 1133, 30, 92, "Text",
   CellTags->"$Larin"],
  Cell[2583267, 102734, 79, 2, 26, "Input",
   CellTags->"$Larin"],
  Cell[2583349, 102738, 107, 3, 26, "Output",
   CellTags->"$Larin"],
  Cell[2583471, 102744, 618, 24, 24, "Text",
   CellTags->"$Larin"]},
 "$LimitTo4"->{
  Cell[2584104, 102771, 55, 1, 39, "ObjectName",
   CellTags->"$LimitTo4"],
  Cell[2584184, 102776, 60, 1, 36, "Subsubsection",
   CellTags->"$LimitTo4"],
  Cell[2584247, 102779, 203, 4, 24, "Text",
   CellTags->"$LimitTo4"],
  Cell[2584475, 102787, 85, 2, 26, "Input",
   CellTags->"$LimitTo4"],
  Cell[2584563, 102791, 109, 3, 26, "Output",
   CellTags->"$LimitTo4"],
  Cell[2584687, 102797, 634, 25, 24, "Text",
   CellTags->"$LimitTo4"]},
 "$LorentzIndices"->{
  Cell[2585336, 102825, 67, 1, 39, "ObjectName",
   CellTags->"$LorentzIndices"],
  Cell[2585428, 102830, 66, 1, 36, "Subsubsection",
   CellTags->"$LorentzIndices"],
  Cell[2585497, 102833, 164, 4, 24, "Text",
   CellTags->"$LorentzIndices"],
  Cell[2585664, 102839, 140, 4, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2585829, 102847, 333, 9, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2586165, 102858, 669, 28, 26, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2586871, 102891, 117, 3, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2586991, 102896, 380, 10, 38, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2587408, 102911, 302, 9, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2587713, 102922, 884, 38, 28, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2588634, 102965, 117, 3, 26, "Input",
   CellTags->"$LorentzIndices"],
  Cell[2588754, 102970, 655, 19, 58, "Output",
   CellTags->"$LorentzIndices"],
  Cell[2589424, 102992, 141, 4, 26, "Input",
   CellTags->"$LorentzIndices"]},
 "$MIntegrate"->{
  Cell[2589580, 102999, 59, 1, 39, "ObjectName",
   CellTags->"$MIntegrate"],
  Cell[2589664, 103004, 62, 1, 36, "Subsubsection",
   CellTags->"$MIntegrate"],
  Cell[2589729, 103007, 136, 4, 24, "Text",
   CellTags->"$MIntegrate"],
  Cell[2589890, 103015, 89, 2, 26, "Input",
   CellTags->"$MIntegrate"],
  Cell[2589982, 103019, 126, 4, 26, "Output",
   CellTags->"$MIntegrate"],
  Cell[2590123, 103026, 276, 11, 24, "Text",
   CellTags->"$MIntegrate"]},
 "$MU"->{
  Cell[2590414, 103040, 43, 1, 39, "ObjectName",
   CellTags->"$MU"],
  Cell[2590482, 103045, 54, 1, 36, "Subsubsection",
   CellTags->"$MU"],
  Cell[2590539, 103048, 151, 4, 24, "Text",
   CellTags->"$MU"],
  Cell[2590715, 103056, 73, 2, 26, "Input",
   CellTags->"$MU"],
  Cell[2590791, 103060, 102, 3, 26, "Output",
   CellTags->"$MU"],
  Cell[2590908, 103066, 256, 11, 24, "Text",
   CellTags->"$MU"]},
 "$MemoryAvailable"->{
  Cell[2591179, 103080, 69, 1, 39, "ObjectName",
   CellTags->"$MemoryAvailable"],
  Cell[2591273, 103085, 67, 1, 36, "Subsubsection",
   CellTags->"$MemoryAvailable"],
  Cell[2591343, 103088, 434, 11, 41, "Text",
   CellTags->"$MemoryAvailable"],
  Cell[2591802, 103103, 99, 2, 26, "Input",
   CellTags->"$MemoryAvailable"],
  Cell[2591904, 103107, 116, 3, 26, "Output",
   CellTags->"$MemoryAvailable"]},
 "$NonComm"->{
  Cell[2592047, 103114, 53, 1, 39, "ObjectName",
   CellTags->"$NonComm"],
  Cell[2592125, 103119, 59, 1, 36, "Subsubsection",
   CellTags->"$NonComm"],
  Cell[2592187, 103122, 109, 3, 24, "Text",
   CellTags->"$NonComm"],
  Cell[2592321, 103129, 83, 2, 26, "Input",
   CellTags->"$NonComm"],
  Cell[2592407, 103133, 803, 15, 89, "Output",
   CellTags->"$NonComm"],
  Cell[2593225, 103151, 477, 18, 24, "Text",
   CellTags->"$NonComm"]},
 "$PairBrackets"->{
  Cell[2593717, 103172, 63, 1, 39, "ObjectName",
   CellTags->"$PairBrackets"],
  Cell[2593805, 103177, 64, 1, 36, "Subsubsection",
   CellTags->"$PairBrackets"],
  Cell[2593872, 103180, 155, 4, 24, "Text",
   CellTags->"$PairBrackets"],
  Cell[2594052, 103188, 93, 2, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2594148, 103192, 114, 3, 26, "Output",
   CellTags->"$PairBrackets"],
  Cell[2594299, 103200, 204, 7, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2594506, 103209, 425, 18, 28, "Output",
   CellTags->"$PairBrackets"],
  Cell[2594968, 103232, 118, 3, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2595089, 103237, 113, 3, 26, "Output",
   CellTags->"$PairBrackets"],
  Cell[2595239, 103245, 204, 7, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2595446, 103254, 445, 18, 28, "Output",
   CellTags->"$PairBrackets"],
  Cell[2595928, 103277, 119, 3, 26, "Input",
   CellTags->"$PairBrackets"],
  Cell[2596050, 103282, 114, 3, 26, "Output",
   CellTags->"$PairBrackets"]},
 "$SpinorMinimal"->{
  Cell[2596191, 103289, 65, 1, 39, "ObjectName",
   CellTags->"$SpinorMinimal"],
  Cell[2596281, 103294, 65, 1, 36, "Subsubsection",
   CellTags->"$SpinorMinimal"],
  Cell[2596349, 103297, 241, 5, 41, "Text",
   CellTags->"$SpinorMinimal"],
  Cell[2596615, 103306, 95, 2, 26, "Input",
   CellTags->"$SpinorMinimal"],
  Cell[2596713, 103310, 115, 3, 26, "Output",
   CellTags->"$SpinorMinimal"],
  Cell[2596843, 103316, 287, 11, 24, "Text",
   CellTags->"$SpinorMinimal"]},
 "$VeryVerbose"->{
  Cell[2597145, 103330, 61, 1, 39, "ObjectName",
   CellTags->"$VeryVerbose"],
  Cell[2597231, 103335, 63, 1, 36, "Subsubsection",
   CellTags->"$VeryVerbose"],
  Cell[2597297, 103338, 221, 5, 41, "Text",
   CellTags->"$VeryVerbose"],
  Cell[2597543, 103347, 91, 2, 26, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2597637, 103351, 109, 3, 26, "Output",
   CellTags->"$VeryVerbose"],
  Cell[2597761, 103357, 141, 4, 26, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2597927, 103365, 287, 10, 26, "Input",
   CellTags->"$VeryVerbose"],
  Cell[2598217, 103377, 121, 3, 21, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2598341, 103382, 255, 7, 22, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2598599, 103391, 273, 7, 22, "Print",
   CellTags->"$VeryVerbose"],
  Cell[2598887, 103401, 131, 4, 26, "Input",
   CellTags->"$VeryVerbose"]},
 "$V0"->{
  Cell[2599021, 103407, 236, 11, 24, "Text",
   CellTags->"$V0"]},
 "$West"->{
  Cell[2599272, 103421, 47, 1, 39, "ObjectName",
   CellTags->"$West"],
  Cell[2599344, 103426, 56, 1, 36, "Subsubsection",
   CellTags->"$West"],
  Cell[2599403, 103429, 577, 14, 66, "Text",
   CellTags->"$West"],
  Cell[2599983, 103445, 234, 10, 24, "Text",
   CellTags->"$West"]},
 "Experimental"->{
  Cell[2601242, 103508, 70, 1, 66, "Section",
   CellTags->"Experimental"],
  Cell[2622524, 104279, 122, 3, 24, "Text",
   CellTags->"Experimental"],
  Cell[2622671, 104286, 198, 6, 26, "Input",
   CellTags->"Experimental"],
  Cell[2622872, 104294, 193, 6, 26, "Output",
   CellTags->"Experimental"],
  Cell[2623080, 104303, 100, 1, 24, "Text",
   CellTags->"Experimental"],
  Cell[2623205, 104308, 222, 6, 29, "Input",
   CellTags->"Experimental"],
  Cell[2623430, 104316, 246, 7, 26, "Output",
   CellTags->"Experimental"]},
 "DoPolarizationSums"->{
  Cell[2601315, 103511, 73, 1, 39, "ObjectName",
   CellTags->"DoPolarizationSums"],
  Cell[2601413, 103516, 69, 1, 36, "Subsubsection",
   CellTags->"DoPolarizationSums"],
  Cell[2601485, 103519, 291, 7, 99, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2601779, 103528, 487, 18, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2602303, 103551, 66, 1, 36, "Subsubsection",
   CellTags->"DoPolarizationSums"],
  Cell[2602372, 103554, 310, 6, 41, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2602685, 103562, 123, 3, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2602833, 103569, 571, 17, 46, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2603407, 103588, 448, 16, 30, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2603892, 103609, 116, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2604011, 103614, 133, 4, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2604159, 103621, 1270, 48, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2605432, 103671, 562, 21, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2605997, 103694, 142, 4, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2606142, 103700, 311, 10, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2606478, 103714, 271, 8, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2606752, 103724, 735, 20, 75, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2607502, 103747, 213, 9, 24, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2607718, 103758, 276, 9, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2607997, 103769, 328, 10, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2608328, 103781, 323, 13, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2608654, 103796, 697, 30, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2609376, 103830, 434, 13, 53, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2609813, 103845, 775, 24, 75, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2610603, 103872, 624, 27, 27, "Text",
   CellTags->"DoPolarizationSums"],
  Cell[2611252, 103903, 433, 13, 53, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2611688, 103918, 775, 24, 75, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2612500, 103947, 859, 24, 65, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2613362, 103973, 606, 22, 30, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2614005, 104000, 130, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2614138, 104005, 323, 12, 29, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2614498, 104022, 116, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2614617, 104027, 133, 4, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2614787, 104036, 120, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2614910, 104041, 133, 4, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2615080, 104050, 126, 3, 26, "Input",
   CellTags->"DoPolarizationSums"],
  Cell[2615209, 104055, 133, 4, 26, "Output",
   CellTags->"DoPolarizationSums"],
  Cell[2615357, 104062, 167, 4, 26, "Input",
   CellTags->"DoPolarizationSums"]},
 "FeynmanDoIntegrals"->{
  Cell[2615539, 104069, 90, 1, 39, "ObjectName",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2615654, 104074, 69, 1, 36, "Subsubsection",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2615726, 104077, 1518, 23, 210, "Text",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2617247, 104102, 527, 18, 24, "Text",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2617799, 104124, 136, 3, 26, "Input",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2617938, 104129, 1037, 29, 67, "Output",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2619024, 104164, 66, 1, 36, "Subsubsection",
   CellTags->"FeynmanDoIntegrals"],
  Cell[2619093, 104167, 85, 2, 26, "Input",
   CellTags->"FeynmanDoIntegrals"]},
 "FeynmanParametrize1"->{
  Cell[2619193, 104172, 92, 1, 39, "ObjectName",
   CellTags->"FeynmanParametrize1"],
  Cell[2619310, 104177, 70, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize1"],
  Cell[2619383, 104180, 717, 12, 117, "Text",
   CellTags->"FeynmanParametrize1"],
  Cell[2620125, 104196, 138, 3, 26, "Input",
   CellTags->"FeynmanParametrize1"],
  Cell[2620266, 104201, 505, 13, 26, "Output",
   CellTags->"FeynmanParametrize1"],
  Cell[2620820, 104220, 67, 1, 36, "Subsubsection",
   CellTags->"FeynmanParametrize1"],
  Cell[2620890, 104223, 86, 2, 26, "Input",
   CellTags->"FeynmanParametrize1"]},
 "FeynmanReduce"->{
  Cell[2620991, 104228, 80, 1, 39, "ObjectName",
   CellTags->"FeynmanReduce"],
  Cell[2621096, 104233, 64, 1, 36, "Subsubsection",
   CellTags->"FeynmanReduce"],
  Cell[2621163, 104236, 723, 12, 117, "Text",
   CellTags->"FeynmanReduce"],
  Cell[2621911, 104252, 126, 3, 26, "Input",
   CellTags->"FeynmanReduce"],
  Cell[2622040, 104257, 288, 9, 26, "Output",
   CellTags->"FeynmanReduce"],
  Cell[2622377, 104272, 61, 1, 36, "Subsubsection",
   CellTags->"FeynmanReduce"],
  Cell[2622441, 104275, 80, 2, 26, "Input",
   CellTags->"FeynmanReduce"]}
 }
*)
(*CellTagsIndex
CellTagsIndex->{
 {"Functions", 2625641, 104393},
 {"Abbreviation", 2625727, 104396},
 {"Amplitude", 2626271, 104412},
 {"Amputate", 2627444, 104447},
 {"AnomalousDimension", 2628231, 104470},
 {"AntiCommutator", 2631607, 104557},
 {"AntiQuarkField", 2633699, 104614},
 {"AntiSymmetrize", 2634249, 104629},
 {"Anti5", 2634940, 104648},
 {"Apart2", 2636583, 104699},
 {"A0", 2637332, 104722},
 {"A0ToB0", 2638551, 104762},
 {"BackgroundGluonVertex", 2639007, 104776},
 {"Bracket", 2640265, 104807},
 {"BReduce", 2640557, 104816},
 {"B0", 2641347, 104840},
 {"B0Real", 2642376, 104873},
 {"B0Unique", 2643291, 104901},
 {"B00", 2644227, 104929},
 {"B1", 2645526, 104970},
 {"B11", 2646934, 105015},
 {"CA", 2648487, 105064},
 {"Calc", 2649197, 105087},
 {"CalcColorFactor", 2650464, 105126},
 {"Collecting", 2650793, 105135},
 {"CalculateCounterTerm", 2651185, 105146},
 {"Info3535453139-5096475", 2651740, 105159},
 {"CancelQP", 2651864, 105162},
 {"Cases2", 2653074, 105197},
 {"CF", 2653903, 105222},
 {"ChangeDimension", 2654751, 105249},
 {"ChargeConjugationMatrix", 2657510, 105322},
 {"ChargeConjugationMatrixInv", 2658812, 105353},
 {"CheckContext", 2660491, 105392},
 {"CheckDB", 2661458, 105419},
 {"ChiralityProjector", 2662239, 105442},
 {"Chisholm", 2663135, 105465},
 {"ChisholmSpinor", 2665513, 105534},
 {"ClearScalarProducts", 2666233, 105553},
 {"Collect2", 2667390, 105582},
 {"Collect3", 2669101, 105631},
 {"Combine", 2669902, 105654},
 {"CombineGraphs", 2670968, 105685},
 {"Commutator", 2671291, 105694},
 {"CommutatorExplicit", 2672831, 105737},
 {"CompleteSquare", 2674221, 105772},
 {"ComplexConjugate", 2675094, 105795},
 {"Contract", 2677144, 105848},
 {"Contract1", 2680517, 105945},
 {"Convolute", 2681469, 105972},
 {"ConvoluteTable", 2685687, 106091},
 {"CovariantFieldDerivative", 2686716, 106118},
 {"CounterT", 2687346, 106133},
 {"CounterTerm", 2687869, 106148},
 {"CouplingConstant", 2688417, 106163},
 {"CovariantD", 2688755, 106172},
 {"CrossProduct", 2691807, 106257},
 {"C0", 2692347, 106272},
 {"DataType", 2693078, 106295},
 {"DB0", 2695949, 106378},
 {"DB1", 2696427, 106393},
 {"DeclareNonCommutative", 2696925, 106408},
 {"DeltaFunction", 2698195, 106439},
 {"DeltaFunctionDoublePrime", 2699362, 106470},
 {"DeltaFunctionPrime", 2699738, 106479},
 {"DenominatorOrder", 2700653, 106502},
 {"Dimension", 2700988, 106511},
 {"DimensionalReduction", 2702224, 106546},
 {"DiracBasis", 2702576, 106555},
 {"DiracCanonical", 2702892, 106564},
 {"DiracEquation", 2703223, 106573},
 {"DiracGamma", 2703927, 106592},
 {"DiracGammaCombine", 2705905, 106647},
 {"DiracGammaExpand", 2707124, 106678},
 {"DiracGammaT", 2708860, 106723},
 {"DiracMatrix", 2709691, 106746},
 {"DiracOrder", 2711613, 106799},
 {"DiracReduce", 2713865, 106862},
 {"DiracSigma", 2715642, 106911},
 {"DiracSigmaExplicit", 2716828, 106944},
 {"DiracSimpCombine", 2717583, 106963},
 {"DiracSimplify", 2717922, 106972},
 {"DiracSimplify2", 2721691, 107073},
 {"DiracSlash", 2722405, 107092},
 {"DiracSpinor", 2723871, 107133},
 {"DiracSubstitute67", 2724194, 107142},
 {"DiracTrace", 2724534, 107151},
 {"DiracTraceEvaluate", 2726298, 107200},
 {"DiracTrick", 2726642, 107209},
 {"Divideout", 2729256, 107282},
 {"DOT", 2729557, 107291},
 {"DotExpand", 2729840, 107300},
 {"DotPower", 2730791, 107327},
 {"DotProduct", 2731095, 107336},
 {"DotSimplifyRelations", 2731639, 107351},
 {"DotSimplify", 2731992, 107360},
 {"DummyIndex", 2734638, 107433},
 {"D0", 2734942, 107442},
 {"D0Convention", 2735680, 107465},
 {"Eps", 2735993, 107474},
 {"EpsChisholm", 2738358, 107547},
 {"EpsContract", 2739047, 107566},
 {"EpsDiscard", 2739881, 107589},
 {"EpsEvaluate", 2740194, 107598},
 {"EpsUncontract", 2741031, 107621},
 {"Epsilon", 2741352, 107630},
 {"EpsilonOrder", 2742070, 107651},
 {"Expanding", 2742390, 107660},
 {"ExpandPartialD", 2742703, 107669},
 {"ScalarProductCancel", 2744950, 107728},
 {"ExpandScalarProduct", 2749516, 107825},
 {"Expand2", 2751980, 107872},
 {"Explicit", 2752424, 107885},
 {"ExplicitLorentzIndex", 2753657, 107920},
 {"ExplicitPartialD", 2754429, 107939},
 {"ExplicitSUNIndex", 2755634, 107970},
 {"ExtraFactor", 2756210, 107985},
 {"ExtraVariables", 2756530, 107994},
 {"FactorFull", 2756858, 108003},
 {"Factoring", 2757170, 108012},
 {"Factorout", 2757477, 108021},
 {"FactorTime", 2757784, 108030},
 {"Factor1", 2758093, 108039},
 {"Factor2", 2759219, 108072},
 {"FAD", 2760615, 108113},
 {"FC", 2761612, 108144},
 {"FCE", 2762334, 108167},
 {"FCI", 2763652, 108208},
 {"FCIntegral", 2764657, 108239},
 {"FCIntegrate", 2764970, 108248},
 {"FCNIntegrate", 2765288, 108257},
 {"FC2RHI", 2765604, 108266},
 {"FC2TLI", 2766171, 108283},
 {"Info3535453193-5096475", 2766754, 108300},
 {"FDS", 2766849, 108303},
 {"FermionSpinSum", 2767136, 108312},
 {"FeynAmp", 2769438, 108373},
 {"FeynAmpDenominator", 2770300, 108398},
 {"FeynAmpDenominatorCombine", 2771540, 108429},
 {"FeynAmpDenominatorSimplify", 2772541, 108452},
 {"FeynAmpDenominatorSplit", 2774957, 108507},
 {"FeynAmpList", 2776515, 108544},
 {"FeynCalc", 2776900, 108555},
 {"Info3535453199-5096475", 2777313, 108566},
 {"FeynCalcExternal", 2777436, 108569},
 {"FeynCalcForm", 2778949, 108608},
 {"FeynCalcInternal", 2780160, 108641},
 {"FeynCalc2FORM", 2781679, 108680},
 {"FeynCalcToLaTeX", 2784180, 108747},
 {"FeynmanParameterNames", 2784447, 108754},
 {"FeynmanParametrize", 2784811, 108763},
 {"FeynRule", 2785397, 108778},
 {"FI", 2790043, 108911},
 {"FieldDerivative", 2790328, 108920},
 {"FieldStrength", 2791212, 108943},
 {"FinalFunction", 2792822, 108986},
 {"FinalSubstitutions", 2793153, 108995},
 {"FORM", 2793491, 109004},
 {"FORMEpilog", 2793779, 109013},
 {"FORMProlog", 2794091, 109022},
 {"FORM2FeynCalc", 2794406, 109031},
 {"FourDivergence", 2796535, 109088},
 {"FourLaplacian", 2797628, 109117},
 {"FourVector", 2798783, 109148},
 {"FreeIndex", 2800103, 109185},
 {"FreeQ2", 2800407, 109194},
 {"FRH", 2801310, 109221},
 {"FromTFi", 2802178, 109248},
 {"FromTFI", 2802900, 109269},
 {"FUNCTION", 2802984, 109272},
 {"FunctionalD", 2803289, 109281},
 {"FV", 2810055, 109466},
 {"FVD", 2811243, 109503},
 {"GA", 2812447, 109540},
 {"GAD", 2813571, 109575},
 {"GammaEpsilon", 2814721, 109610},
 {"GammaExpand", 2815874, 109641},
 {"Gamma1", 2816713, 109664},
 {"Gamma2", 2817009, 109673},
 {"Gamma3", 2817305, 109682},
 {"Gauge", 2817600, 109691},
 {"GaugeField", 2817896, 109700},
 {"GaugeXi", 2818578, 109719},
 {"GA5", 2818874, 109728},
 {"GGV", 2819485, 109747},
 {"GhostPropagator", 2819777, 109756},
 {"GHP", 2820967, 109787},
 {"GluonField", 2821851, 109814},
 {"GluonGhostVertex", 2822541, 109833},
 {"GluonPropagator", 2823604, 109860},
 {"GluonSelfEnergy", 2826214, 109927},
 {"GluonVertex", 2827106, 109950},
 {"GO", 2829349, 110011},
 {"GP", 2829624, 110020},
 {"GrassmannParity", 2829912, 110029},
 {"GS", 2830240, 110038},
 {"GSD", 2831235, 110069},
 {"Gstrong", 2832249, 110100},
 {"GTI", 2832827, 110117},
 {"GV", 2833106, 110126},
 {"Hill", 2833383, 110135},
 {"HypergeometricAC", 2834290, 110162},
 {"HypergeometricIR", 2835908, 110203},
 {"HypergeometricSE", 2836971, 110230},
 {"HypExplicit", 2837872, 110253},
 {"HypInt", 2838861, 110280},
 {"IFPD", 2839437, 110297},
 {"IFPDOff", 2839725, 110306},
 {"IFPDOn", 2840658, 110333},
 {"IncludePair", 2840959, 110342},
 {"IndexPosition", 2841282, 110351},
 {"Indices", 2841606, 110360},
 {"InitialFunction", 2841915, 110369},
 {"InitialSubstitutions", 2842261, 110378},
 {"InsideDiracTrace", 2842623, 110387},
 {"IntegralTable", 2842966, 110396},
 {"IntegrateByParts", 2843300, 110405},
 {"Integratedx", 2843964, 110422},
 {"Integrate2", 2844285, 110431},
 {"Integrate3", 2849580, 110576},
 {"IntermediateSubstitutions", 2850866, 110611},
 {"InverseMellin", 2851245, 110620},
 {"Isolate", 2855598, 110735},
 {"Info3535453244-5096475", 2857372, 110784},
 {"IsolateHead", 2857586, 110789},
 {"IsolateNames", 2857907, 110798},
 {"IsolatePrint", 2858233, 110807},
 {"IsolateSplit", 2858559, 110816},
 {"KeepOnly", 2858881, 110825},
 {"KK", 2859181, 110834},
 {"Kummer", 2859461, 110843},
 {"Lagrangian", 2864834, 110998},
 {"LC", 2866022, 111031},
 {"LCD", 2867147, 111066},
 {"LeftPartialD", 2868559, 111109},
 {"LeftRightPartialD", 2870021, 111148},
 {"LeftRightPartialD2", 2871421, 111183},
 {"LeviCivita", 2872830, 111218},
 {"LeviCivitaSign", 2874904, 111275},
 {"ListIntegrals", 2875239, 111284},
 {"Li2", 2875641, 111295},
 {"Li3", 2876387, 111318},
 {"Loop", 2877265, 111345},
 {"LorentzIndex", 2877492, 111352},
 {"Lower", 2879154, 111397},
 {"MakeContext", 2879451, 111406},
 {"Mandelstam", 2879771, 111415},
 {"Map2", 2880084, 111424},
 {"MemoryAvailable", 2880986, 111451},
 {"MemSet", 2881318, 111460},
 {"MetricTensor", 2881620, 111469},
 {"MLimit", 2883213, 111512},
 {"Momentum", 2883656, 111525},
 {"MomentumCombine", 2885731, 111584},
 {"MomentumCombine2", 2887166, 111621},
 {"MomentumExpand", 2887911, 111640},
 {"MT", 2889546, 111683},
 {"MTD", 2890539, 111714},
 {"NegativeInteger", 2891560, 111745},
 {"Nf", 2891888, 111754},
 {"Nielsen", 2892169, 111763},
 {"NonCommFreeQ", 2893663, 111806},
 {"NonCommQ", 2893985, 111815},
 {"NonCommutative", 2894367, 111826},
 {"NTerms", 2894695, 111835},
 {"NumberOfMetricTensors", 2895561, 111860},
 {"NumericalFactor", 2895842, 111867},
 {"NumericQ1", 2896100, 111874},
 {"OneLoop", 2896846, 111895},
 {"OneLoopSimplify", 2898843, 111952},
 {"OneLoopSum", 2900520, 111995},
 {"OPE", 2900829, 112004},
 {"OPEDelta", 2901115, 112013},
 {"OPEi", 2901917, 112036},
 {"OPEInsert", 2903084, 112071},
 {"OPEInt", 2903540, 112084},
 {"OPEIntDelta", 2903983, 112097},
 {"OPEIntegrate", 2904458, 112110},
 {"OPEIntegrateDelta", 2904943, 112123},
 {"OPESumExplicit", 2905456, 112136},
 {"OPESum", 2906561, 112165},
 {"OPESumSimplify", 2907564, 112194},
 {"OPE1Loop", 2908748, 112225},
 {"OPE2TID", 2909199, 112238},
 {"OptionsSelect", 2909790, 112255},
 {"Pair", 2910037, 112262},
 {"PairCollect", 2912000, 112321},
 {"PairContract", 2912322, 112330},
 {"PairContract2", 2912649, 112339},
 {"PairContract3", 2912980, 112348},
 {"PartialD", 2913306, 112357},
 {"PartialDRelations", 2914124, 112380},
 {"PartialFourVector", 2914883, 112399},
 {"PartialIntegrate", 2915480, 112414},
 {"PartitHead", 2916855, 112449},
 {"PauliSigma", 2917099, 112456},
 {"PaVe", 2917486, 112467},
 {"PaVeOrderList", 2918251, 112490},
 {"PaVeOrder", 2918578, 112499},
 {"PaVeReduce", 2919404, 112522},
 {"Info3535453273-5096475", 2921449, 112577},
 {"PD", 2921559, 112580},
 {"PlusDistribution", 2921848, 112589},
 {"Polarization", 2923222, 112624},
 {"PolarizationSum", 2924676, 112663},
 {"PolarizationUncontract", 2925889, 112694},
 {"PolarizationVector", 2926261, 112703},
 {"PositiveInteger", 2928475, 112758},
 {"PositiveNumber", 2928815, 112767},
 {"PowerFactor", 2929148, 112776},
 {"PowerSimplify", 2929846, 112795},
 {"Power2", 2930859, 112822},
 {"PostFortranFile", 2931514, 112841},
 {"Prefactor", 2931849, 112850},
 {"PreFortranFile", 2932165, 112859},
 {"PropagatorDenominator", 2932508, 112868},
 {"PropagatorDenominatorExplicit", 2934231, 112909},
 {"QGV", 2935267, 112932},
 {"QO", 2935546, 112941},
 {"QP", 2935821, 112950},
 {"QuantumField", 2936107, 112959},
 {"QuarkField", 2938889, 113034},
 {"QuarkGluonVertex", 2939434, 113049},
 {"QuarkMass", 2940960, 113088},
 {"QuarkPropagator", 2941276, 113097},
 {"ReduceGamma", 2942318, 113124},
 {"ReduceToScalars", 2942643, 113133},
 {"Rename", 2942975, 113142},
 {"RHI", 2943268, 113151},
 {"RHI2FC", 2943690, 113164},
 {"RHM", 2944127, 113177},
 {"RHO", 2944543, 113190},
 {"RHP", 2944960, 113203},
 {"RTL", 2945377, 113216},
 {"RightPartialD", 2945804, 113229},
 {"RussianTrick", 2946974, 113260},
 {"ScalarGluonVertex", 2948290, 113295},
 {"ScalarProduct", 2948963, 113312},
 {"Info3535453289-5096475", 2950823, 113361},
 {"ScaleMu", 2950947, 113364},
 {"Schouten", 2951318, 113375},
 {"SD", 2951985, 113394},
 {"SelectFree", 2952727, 113417},
 {"SelectGraphs", 2954209, 113458},
 {"SelectNotFree", 2954536, 113467},
 {"SelectSplit", 2956151, 113510},
 {"Select1", 2956773, 113527},
 {"Select2", 2957074, 113536},
 {"Series2", 2957375, 113545},
 {"Series3", 2959366, 113602},
 {"SetMandelstam", 2960168, 113625},
 {"SimplifyDeltaFunction", 2960658, 113638},
 {"SimplifyGTI", 2963120, 113697},
 {"SimplifyPolyLog", 2963596, 113710},
 {"Simplify2", 2966980, 113797},
 {"SmallDelta", 2967368, 113808},
 {"SmallEpsilon", 2967613, 113815},
 {"SmallVariable", 2967865, 113822},
 {"SmallVariables", 2968197, 113831},
 {"SMP", 2968522, 113840},
 {"SMVertex", 2969207, 113861},
 {"Smu", 2969799, 113878},
 {"Sn", 2970013, 113885},
 {"SO", 2970226, 113892},
 {"SOD", 2970962, 113915},
 {"Solve2", 2971713, 113938},
 {"Solve3", 2972911, 113973},
 {"SP", 2973487, 113990},
 {"SPC", 2974743, 114029},
 {"SPD", 2975023, 114038},
 {"Spinor", 2976432, 114081},
 {"SpinorCollect", 2978110, 114130},
 {"SpinorUBar", 2978438, 114139},
 {"SpinorU", 2979625, 114172},
 {"SpinorVBar", 2980699, 114203},
 {"SpinorV", 2981816, 114234},
 {"SpinPolarizationSum", 2982899, 114265},
 {"SPL", 2983244, 114274},
 {"SplittingFunction", 2983538, 114283},
 {"StandardMatrixElement", 2988298, 114402},
 {"SubContext", 2988658, 114411},
 {"SubLoop", 2988971, 114420},
 {"SumP", 2989269, 114429},
 {"SumS", 2990562, 114468},
 {"SumT", 2992787, 114535},
 {"SUND", 2995147, 114606},
 {"SUNDelta", 2996913, 114659},
 {"SUNDeltaContract", 2998789, 114712},
 {"SUNF", 2999123, 114721},
 {"SUNFJacobi", 3001427, 114790},
 {"SUNIndex", 3002115, 114809},
 {"SUNIndexRename", 3002927, 114832},
 {"SUNN", 3003253, 114841},
 {"SUNNToCACF", 3003751, 114856},
 {"SUNSimplify", 3004292, 114871},
 {"SUNT", 3008603, 114988},
 {"SUNTrace", 3010767, 115053},
 {"SymbolicSum2", 3013001, 115116},
 {"SymbolicSum3", 3013253, 115123},
 {"Symmetrize", 3013503, 115130},
 {"TBox", 3014184, 115149},
 {"Tbox", 3014434, 115156},
 {"Tdec", 3014684, 115163},
 {"TensorFunction", 3015993, 115202},
 {"Tf", 3017702, 115247},
 {"TFi", 3017980, 115256},
 {"ThreeVector", 3018736, 115279},
 {"TID", 3019048, 115288},
 {"TIDL", 3020196, 115323},
 {"TimedIntegrate", 3022523, 115390},
 {"TLI", 3023465, 115415},
 {"TLIFP", 3024017, 115432},
 {"TLIHYP", 3024584, 115449},
 {"TLI2", 3025157, 115466},
 {"TLI2FC", 3025444, 115475},
 {"ToDistribution", 3026028, 115492},
 {"ToHypergeometric", 3027681, 115535},
 {"ToLarin", 3028741, 115562},
 {"ToTFi", 3029398, 115581},
 {"TR", 3030308, 115608},
 {"Info3535453335-5096475", 3033143, 115695},
 {"TraceDimension", 3033251, 115698},
 {"TraceOfOne", 3033583, 115707},
 {"Trick", 3034263, 115726},
 {"TrickIntegrate", 3035386, 115759},
 {"TrickMandelstam", 3035885, 115772},
 {"Tr2", 3036535, 115789},
 {"Twist2AlienOperator", 3037166, 115808},
 {"Twist2CounterOperator", 3037867, 115825},
 {"Twist2GluonOperator", 3038752, 115846},
 {"Twist2QuarkOperator", 3046555, 116035},
 {"Twist3QuarkOperator", 3051440, 116154},
 {"Twist4GluonOperator", 3052218, 116173},
 {"TwoLoopSimplify", 3052835, 116188},
 {"Uncontract", 3053338, 116201},
 {"UnDeclareNonCommutative", 3055298, 116254},
 {"Upper", 3056964, 116293},
 {"UVPart", 3057260, 116302},
 {"Vectors", 3057636, 116313},
 {"WriteOut", 3057942, 116322},
 {"WriteOutPaVe", 3058256, 116331},
 {"Write2", 3058580, 116340},
 {"XYT", 3060832, 116405},
 {"ZeroMomentumInsertion", 3061069, 116412},
 {"Zeta2", 3061428, 116421},
 {"$Abbreviations", 3062216, 116444},
 {"$AL", 3062702, 116457},
 {"$BreitMaison", 3063462, 116480},
 {"$Color", 3063786, 116489},
 {"$Covariant", 3064021, 116496},
 {"$FCS", 3064848, 116519},
 {"$FCT", 3065206, 116530},
 {"$FeynCalcStuff", 3065843, 116549},
 {"$FeynCalcVersion", 3066108, 116556},
 {"$FortranContinuationCharacter", 3066550, 116567},
 {"$Gauge", 3067127, 116580},
 {"$Larin", 3067427, 116589},
 {"$LimitTo4", 3067870, 116602},
 {"$LorentzIndices", 3068335, 116615},
 {"$MIntegrate", 3069386, 116642},
 {"$MU", 3069851, 116655},
 {"$MemoryAvailable", 3070281, 116668},
 {"$NonComm", 3070703, 116679},
 {"$PairBrackets", 3071161, 116692},
 {"$SpinorMinimal", 3072187, 116719},
 {"$VeryVerbose", 3072679, 116732},
 {"$V0", 3073526, 116755},
 {"$West", 3073605, 116758},
 {"Experimental", 3073908, 116767},
 {"DoPolarizationSums", 3074460, 116782},
 {"FeynmanDoIntegrals", 3077601, 116859},
 {"FeynmanParametrize1", 3078298, 116876},
 {"FeynmanReduce", 3078913, 116891}
 }
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[579, 22, 94, 2, 108, "Title"],
Cell[676, 26, 87, 1, 67, "Subtitle"],
Cell[766, 29, 123, 3, 67, "Subtitle",
 PageBreakBelow->True],
Cell[892, 34, 80, 1, 54, "Subsubtitle"],
Cell[975, 37, 244, 8, 54, "Subsubtitle"],
Cell[CellGroupData[{
Cell[1244, 49, 147, 4, 29, "Input"],
Cell[1394, 55, 339, 9, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1770, 69, 52, 1, 66, "Section",
 CellTags->"Functions"],
Cell[1825, 72, 54, 0, 29, "Commentary"],
Cell[1882, 74, 87, 2, 24, "Text"],
Cell[CellGroupData[{
Cell[1994, 80, 122, 3, 26, "Input"],
Cell[2119, 85, 118, 3, 26, "Output"]
}, Open  ]],
Cell[2252, 91, 69, 1, 26, "Input"],
Cell[2324, 94, 163, 3, 17, "Text"],
Cell[2490, 99, 641, 21, 17, "Text"],
Cell[3134, 122, 329, 11, 17, "Text"],
Cell[3466, 135, 154, 3, 17, "Text"],
Cell[3623, 140, 84, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"Abbreviation"],
Cell[CellGroupData[{
Cell[3732, 146, 63, 1, 36, "Subsubsection",
 CellTags->"Abbreviation"],
Cell[3798, 149, 329, 6, 41, "Text",
 CellTags->"Abbreviation"],
Cell[4130, 157, 1716, 59, 41, "Text",
 CellTags->"Abbreviation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[5883, 221, 60, 1, 36, "Subsubsection",
 CellTags->"Abbreviation"],
Cell[CellGroupData[{
Cell[5968, 226, 264, 8, 26, "Input",
 CellTags->"Abbreviation"],
Cell[6235, 236, 228, 8, 28, "Output",
 CellTags->"Abbreviation"]
}, Open  ]]
}, Open  ]],
Cell[6490, 248, 55, 1, 39, "ObjectName",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[6570, 253, 60, 1, 36, "Subsubsection",
 CellTags->"Amplitude"],
Cell[6633, 256, 529, 9, 75, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[7187, 269, 115, 3, 26, "Input",
 CellTags->"Amplitude"],
Cell[7305, 274, 328, 10, 26, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[7648, 287, 258, 11, 24, "Text",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[CellGroupData[{
Cell[7943, 303, 57, 1, 36, "Subsubsection",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[8025, 308, 133, 4, 26, "Input",
 CellTags->"Amplitude"],
Cell[8161, 314, 104, 3, 26, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[8280, 320, 93, 1, 24, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[8398, 325, 121, 3, 26, "Input",
 CellTags->"Amplitude"],
Cell[8522, 330, 2216, 95, 30, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[CellGroupData[{
Cell[10775, 430, 108, 3, 26, "Input",
 CellTags->"Amplitude"],
Cell[10886, 435, 5133, 189, 73, "Output",
 CellTags->"Amplitude"]
}, Open  ]],
Cell[16034, 627, 211, 8, 24, "Text",
 CellTags->"Amplitude"],
Cell[CellGroupData[{
Cell[16270, 639, 306, 9, 26, "Input",
 CellTags->"Amplitude"],
Cell[16579, 650, 13529, 441, 136, "Output",
 CellTags->"Amplitude"]
}, Open  ]]
}, Open  ]],
Cell[30135, 1095, 53, 1, 39, "ObjectName",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[30213, 1100, 59, 1, 36, "Subsubsection",
 CellTags->"Amputate"],
Cell[30275, 1103, 215, 5, 41, "Text",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[30515, 1112, 114, 3, 26, "Input",
 CellTags->"Amputate"],
Cell[30632, 1117, 296, 10, 26, "Output",
 CellTags->"Amputate"]
}, Open  ]],
Cell[30943, 1130, 1080, 38, 24, "Text",
 CellTags->"Amputate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32060, 1173, 56, 1, 36, "Subsubsection",
 CellTags->"Amputate"],
Cell[CellGroupData[{
Cell[32141, 1178, 170, 5, 26, "Input",
 CellTags->"Amputate"],
Cell[32314, 1185, 428, 16, 28, "Output",
 CellTags->"Amputate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32779, 1206, 131, 4, 26, "Input",
 CellTags->"Amputate"],
Cell[32913, 1212, 721, 27, 28, "Output",
 CellTags->"Amputate"]
}, Open  ]]
}, Open  ]],
Cell[33661, 1243, 73, 1, 39, "ObjectName",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[33759, 1248, 69, 1, 36, "Subsubsection",
 CellTags->"AnomalousDimension"],
Cell[33831, 1251, 143, 4, 24, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[33999, 1259, 134, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[34136, 1264, 254, 8, 26, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[34405, 1275, 653, 24, 24, "Text",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[35095, 1304, 66, 1, 36, "Subsubsection",
 CellTags->"AnomalousDimension"],
Cell[35164, 1307, 63, 1, 24, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[35252, 1312, 206, 5, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[35461, 1319, 254, 8, 26, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[35730, 1330, 241, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[35996, 1343, 133, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[36132, 1348, 462, 17, 43, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[36609, 1368, 240, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[36874, 1381, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[37009, 1386, 284, 10, 45, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[37308, 1399, 240, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[37573, 1412, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[37708, 1417, 283, 10, 43, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[38006, 1430, 240, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[38271, 1443, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[38406, 1448, 594, 21, 46, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[39015, 1472, 241, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[39281, 1485, 133, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[39417, 1490, 689, 23, 47, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[40121, 1516, 241, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[40387, 1529, 133, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[40523, 1534, 6606, 218, 249, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[47144, 1755, 240, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[47409, 1768, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[47544, 1773, 4116, 140, 150, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[51675, 1916, 240, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[51940, 1929, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[52075, 1934, 6222, 212, 235, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[58312, 2149, 240, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[58577, 2162, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[58712, 2167, 4910, 166, 235, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[63637, 2336, 267, 9, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[63929, 2349, 132, 3, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[64064, 2354, 4379, 139, 151, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]],
Cell[68458, 2496, 326, 10, 29, "Text",
 CellTags->"AnomalousDimension"],
Cell[CellGroupData[{
Cell[68809, 2510, 295, 9, 26, "Input",
 CellTags->"AnomalousDimension"],
Cell[69107, 2521, 239, 7, 26, "Output",
 CellTags->"AnomalousDimension"]
}, Open  ]]
}, Open  ]],
Cell[69373, 2532, 65, 1, 39, "ObjectName",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[69463, 2537, 65, 1, 36, "Subsubsection",
 CellTags->"AntiCommutator"],
Cell[69531, 2540, 145, 4, 24, "Text",
 CellTags->"AntiCommutator"],
Cell[69679, 2546, 955, 32, 24, "Text",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[70671, 2583, 62, 1, 36, "Subsubsection",
 CellTags->"AntiCommutator"],
Cell[70736, 2586, 200, 9, 24, "Text",
 CellTags->"AntiCommutator"],
Cell[70939, 2597, 150, 4, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[71114, 2605, 143, 4, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[71260, 2611, 163, 5, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71460, 2621, 124, 3, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[71587, 2626, 180, 6, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[71804, 2637, 265, 8, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[72072, 2647, 438, 16, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[72547, 2668, 258, 8, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[72808, 2678, 318, 11, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[73141, 2692, 218, 6, 31, "Input",
 CellTags->"AntiCommutator"],
Cell[73362, 2700, 104, 1, 24, "Text",
 CellTags->"AntiCommutator"],
Cell[CellGroupData[{
Cell[73491, 2705, 177, 5, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[73671, 2712, 109, 3, 27, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[73817, 2720, 182, 5, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[74002, 2727, 190, 6, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74229, 2738, 250, 8, 31, "Input",
 CellTags->"AntiCommutator"],
Cell[74482, 2748, 132, 4, 26, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[74651, 2757, 252, 7, 31, "Input",
 CellTags->"AntiCommutator"],
Cell[74906, 2766, 315, 11, 29, "Output",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[75236, 2780, 240, 6, 31, "Input",
 CellTags->"AntiCommutator"],
Cell[75479, 2788, 176, 5, 26, "Input",
 CellTags->"AntiCommutator"],
Cell[75658, 2795, 226, 7, 31, "Input",
 CellTags->"AntiCommutator"]
}, Open  ]],
Cell[75899, 2805, 65, 1, 39, "ObjectName",
 CellTags->"AntiQuarkField"],
Cell[CellGroupData[{
Cell[75989, 2810, 65, 1, 36, "Subsubsection",
 CellTags->"AntiQuarkField"],
Cell[76057, 2813, 197, 4, 24, "Text",
 CellTags->"AntiQuarkField"],
Cell[76257, 2819, 483, 18, 24, "Text",
 CellTags->"AntiQuarkField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[76777, 2842, 62, 1, 36, "Subsubsection",
 CellTags->"AntiQuarkField"],
Cell[CellGroupData[{
Cell[76864, 2847, 93, 2, 26, "Input",
 CellTags->"AntiQuarkField"],
Cell[76960, 2851, 137, 4, 26, "Output",
 CellTags->"AntiQuarkField"]
}, Open  ]]
}, Open  ]],
Cell[77124, 2859, 65, 1, 39, "ObjectName",
 CellTags->"AntiSymmetrize"],
Cell[CellGroupData[{
Cell[77214, 2864, 65, 1, 36, "Subsubsection",
 CellTags->"AntiSymmetrize"],
Cell[77282, 2867, 153, 4, 24, "Text",
 CellTags->"AntiSymmetrize"],
Cell[77438, 2873, 269, 10, 24, "Text",
 CellTags->"AntiSymmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[77744, 2888, 62, 1, 36, "Subsubsection",
 CellTags->"AntiSymmetrize"],
Cell[CellGroupData[{
Cell[77831, 2893, 246, 8, 26, "Input",
 CellTags->"AntiSymmetrize"],
Cell[78080, 2903, 319, 11, 42, "Output",
 CellTags->"AntiSymmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[78436, 2919, 266, 8, 26, "Input",
 CellTags->"AntiSymmetrize"],
Cell[78705, 2929, 654, 20, 42, "Output",
 CellTags->"AntiSymmetrize"]
}, Open  ]]
}, Open  ]],
Cell[79386, 2953, 47, 1, 39, "ObjectName",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[79458, 2958, 56, 1, 36, "Subsubsection",
 CellTags->"Anti5"],
Cell[79517, 2961, 487, 16, 41, "Text",
 CellTags->"Anti5"],
Cell[80007, 2979, 163, 7, 24, "Text",
 CellTags->"Anti5"],
Cell[80173, 2988, 678, 25, 24, "Text",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[80888, 3018, 53, 1, 36, "Subsubsection",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[80966, 3023, 154, 5, 26, "Input",
 CellTags->"Anti5"],
Cell[81123, 3030, 186, 6, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81346, 3041, 102, 3, 26, "Input",
 CellTags->"Anti5"],
Cell[81451, 3046, 290, 11, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[81778, 3062, 144, 5, 26, "Input",
 CellTags->"Anti5"],
Cell[81925, 3069, 265, 10, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82227, 3084, 194, 6, 26, "Input",
 CellTags->"Anti5"],
Cell[82424, 3092, 320, 9, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[82781, 3106, 125, 4, 26, "Input",
 CellTags->"Anti5"],
Cell[82909, 3112, 320, 9, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83266, 3126, 133, 4, 26, "Input",
 CellTags->"Anti5"],
Cell[83402, 3132, 320, 9, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[83759, 3146, 151, 5, 26, "Input",
 CellTags->"Anti5"],
Cell[83913, 3153, 320, 9, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[84248, 3165, 351, 14, 24, "Text",
 CellTags->"Anti5"],
Cell[CellGroupData[{
Cell[84624, 3183, 127, 4, 26, "Input",
 CellTags->"Anti5"],
Cell[84754, 3189, 191, 6, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[84982, 3200, 102, 3, 26, "Input",
 CellTags->"Anti5"],
Cell[85087, 3205, 290, 11, 26, "Output",
 CellTags->"Anti5"]
}, Open  ]]
}, Open  ]],
Cell[85404, 3220, 49, 1, 39, "ObjectName",
 CellTags->"Apart2"],
Cell[CellGroupData[{
Cell[85478, 3225, 57, 1, 36, "Subsubsection",
 CellTags->"Apart2"],
Cell[85538, 3228, 108, 3, 24, "Text",
 CellTags->"Apart2"],
Cell[85649, 3233, 471, 18, 24, "Text",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[86157, 3256, 54, 1, 36, "Subsubsection",
 CellTags->"Apart2"],
Cell[CellGroupData[{
Cell[86236, 3261, 256, 9, 26, "Input",
 CellTags->"Apart2"],
Cell[86495, 3272, 1188, 30, 48, "Output",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[87720, 3307, 104, 3, 26, "Input",
 CellTags->"Apart2"],
Cell[87827, 3312, 720, 26, 53, "Output",
 CellTags->"Apart2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[88584, 3343, 140, 4, 26, "Input",
 CellTags->"Apart2"],
Cell[88727, 3349, 517, 19, 56, "Output",
 CellTags->"Apart2"]
}, Open  ]]
}, Open  ]],
Cell[89271, 3372, 64, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[89360, 3378, 53, 1, 36, "Subsubsection",
 CellTags->"A0"],
Cell[89416, 3381, 291, 11, 24, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[89732, 3396, 107, 3, 26, "Input",
 CellTags->"A0"],
Cell[89842, 3401, 161, 6, 26, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[90018, 3410, 743, 32, 24, "Text",
 CellTags->"A0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[90798, 3447, 50, 1, 36, "Subsubsection",
 CellTags->"A0"],
Cell[90851, 3450, 246, 11, 24, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[91122, 3465, 120, 4, 29, "Input",
 CellTags->"A0"],
Cell[91245, 3471, 259, 10, 29, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[91519, 3484, 186, 6, 26, "Input",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[91730, 3494, 120, 4, 29, "Input",
 CellTags->"A0"],
Cell[91853, 3500, 324, 11, 29, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[92192, 3514, 187, 6, 26, "Input",
 CellTags->"A0"],
Cell[92382, 3522, 222, 8, 24, "Text",
 CellTags->"A0"],
Cell[CellGroupData[{
Cell[92629, 3534, 96, 3, 26, "Input",
 CellTags->"A0"],
Cell[92728, 3539, 97, 3, 26, "Output",
 CellTags->"A0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[92862, 3547, 160, 5, 26, "Input",
 CellTags->"A0"],
Cell[93025, 3554, 97, 3, 26, "Output",
 CellTags->"A0"]
}, Open  ]]
}, Open  ]],
Cell[93149, 3561, 72, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"A0ToB0"],
Cell[CellGroupData[{
Cell[93246, 3567, 57, 1, 36, "Subsubsection",
 CellTags->"A0ToB0"],
Cell[93306, 3570, 614, 27, 24, "Text",
 CellTags->"A0ToB0"],
Cell[CellGroupData[{
Cell[93945, 3601, 111, 3, 26, "Input",
 CellTags->"A0ToB0"],
Cell[94059, 3606, 165, 6, 26, "Output",
 CellTags->"A0ToB0"]
}, Open  ]],
Cell[94239, 3615, 915, 39, 22, "Text",
 CellTags->"A0ToB0"]
}, Open  ]],
Cell[95169, 3657, 79, 1, 39, "ObjectName",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[95273, 3662, 72, 1, 36, "Subsubsection",
 CellTags->"BackgroundGluonVertex"],
Cell[95348, 3665, 1042, 19, 126, "Text",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[96415, 3688, 140, 3, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[96558, 3693, 318, 9, 26, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[96891, 3705, 286, 11, 24, "Text",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[97214, 3721, 69, 1, 36, "Subsubsection",
 CellTags->"BackgroundGluonVertex"],
Cell[CellGroupData[{
Cell[97308, 3726, 360, 10, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[97671, 3738, 1486, 58, 29, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[99194, 3801, 436, 12, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[99633, 3815, 3846, 142, 51, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[103516, 3962, 411, 11, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[103930, 3975, 1559, 60, 47, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[105526, 4040, 487, 13, 26, "Input",
 CellTags->"BackgroundGluonVertex"],
Cell[106016, 4055, 3954, 145, 92, "Output",
 CellTags->"BackgroundGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[109997, 4204, 51, 1, 39, "ObjectName",
 CellTags->"Bracket"],
Cell[CellGroupData[{
Cell[110073, 4209, 58, 1, 36, "Subsubsection",
 CellTags->"Bracket"],
Cell[110134, 4212, 72, 1, 24, "Text",
 CellTags->"Bracket"],
Cell[110209, 4215, 264, 11, 24, "Text",
 CellTags->"Bracket"]
}, Open  ]],
Cell[110488, 4229, 74, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[110587, 4235, 58, 1, 36, "Subsubsection",
 CellTags->"BReduce"],
Cell[110648, 4238, 148, 4, 24, "Text",
 CellTags->"BReduce"],
Cell[110799, 4244, 916, 39, 24, "Text",
 CellTags->"BReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[111752, 4288, 55, 1, 36, "Subsubsection",
 CellTags->"BReduce"],
Cell[111810, 4291, 251, 11, 24, "Text",
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[112086, 4306, 134, 4, 26, "Input",
 CellTags->"BReduce"],
Cell[112223, 4312, 176, 6, 26, "Output",
 CellTags->"BReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[112436, 4323, 81, 2, 26, "Input"],
Cell[112520, 4327, 263, 8, 26, "Output"]
}, Open  ]],
Cell[112798, 4338, 94, 3, 24, "Text",
 CellTags->"BReduce"],
Cell[CellGroupData[{
Cell[112917, 4345, 183, 5, 26, "Input",
 CellTags->"BReduce"],
Cell[113103, 4352, 260, 10, 44, "Output",
 CellTags->"BReduce"]
}, Open  ]]
}, Open  ]],
Cell[113390, 4366, 41, 1, 39, "ObjectName",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[113456, 4371, 53, 1, 36, "Subsubsection",
 CellTags->"B0"],
Cell[113512, 4374, 390, 9, 41, "Text",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[113927, 4387, 102, 3, 26, "Input",
 CellTags->"B0"],
Cell[114032, 4392, 280, 9, 26, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[114327, 4404, 751, 32, 24, "Text",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[115115, 4441, 50, 1, 36, "Subsubsection",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[115190, 4446, 234, 8, 29, "Input",
 CellTags->"B0"],
Cell[115427, 4456, 316, 12, 29, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[115758, 4471, 237, 7, 26, "Input",
 CellTags->"B0"],
Cell[CellGroupData[{
Cell[116020, 4482, 154, 5, 29, "Input",
 CellTags->"B0"],
Cell[116177, 4489, 245, 8, 29, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[116459, 4502, 179, 6, 29, "Input",
 CellTags->"B0"],
Cell[116641, 4510, 245, 8, 29, "Output",
 CellTags->"B0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[116923, 4523, 177, 6, 26, "Input",
 CellTags->"B0"],
Cell[117103, 4531, 218, 7, 29, "Output",
 CellTags->"B0"]
}, Open  ]]
}, Open  ]],
Cell[117348, 4542, 72, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[117445, 4548, 57, 1, 36, "Subsubsection",
 CellTags->"B0Real"],
Cell[117505, 4551, 186, 4, 22, "Text",
 CellTags->"B0Real"],
Cell[117694, 4557, 229, 10, 22, "Text",
 CellTags->"B0Real"]
}, Open  ]],
Cell[CellGroupData[{
Cell[117960, 4572, 54, 1, 34, "Subsubsection",
 CellTags->"B0Real"],
Cell[118017, 4575, 83, 1, 24, "Text",
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[118125, 4580, 133, 4, 26, "Input",
 CellTags->"B0Real"],
Cell[118261, 4586, 200, 7, 26, "Output",
 CellTags->"B0Real"]
}, Open  ]],
Cell[CellGroupData[{
Cell[118498, 4598, 102, 3, 26, "Input",
 CellTags->"B0Real"],
Cell[118603, 4603, 282, 9, 26, "Output",
 CellTags->"B0Real"]
}, Open  ]],
Cell[118900, 4615, 92, 3, 24, "Text",
 CellTags->"B0Real"],
Cell[CellGroupData[{
Cell[119017, 4622, 181, 5, 26, "Input",
 CellTags->"B0Real"],
Cell[119201, 4629, 200, 7, 26, "Output",
 CellTags->"B0Real"]
}, Open  ]]
}, Open  ]],
Cell[119428, 4640, 76, 2, 39, "ObjectName",
 PageBreakAbove->True,
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[119529, 4646, 59, 1, 36, "Subsubsection",
 CellTags->"B0Unique"],
Cell[119591, 4649, 184, 4, 24, "Text",
 CellTags->"B0Unique"],
Cell[119778, 4655, 231, 10, 24, "Text",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[CellGroupData[{
Cell[120046, 4670, 56, 1, 36, "Subsubsection",
 CellTags->"B0Unique"],
Cell[120105, 4673, 73, 1, 24, "Text",
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[120203, 4678, 135, 4, 26, "Input",
 CellTags->"B0Unique"],
Cell[120341, 4684, 202, 7, 26, "Output",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[CellGroupData[{
Cell[120580, 4696, 104, 3, 26, "Input",
 CellTags->"B0Unique"],
Cell[120687, 4701, 284, 9, 26, "Output",
 CellTags->"B0Unique"]
}, Open  ]],
Cell[120986, 4713, 88, 3, 24, "Text",
 CellTags->"B0Unique"],
Cell[CellGroupData[{
Cell[121099, 4720, 186, 5, 26, "Input",
 CellTags->"B0Unique"],
Cell[121288, 4727, 177, 6, 26, "Output",
 CellTags->"B0Unique"]
}, Open  ]]
}, Open  ]],
Cell[121492, 4737, 43, 1, 39, "ObjectName",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[121560, 4742, 54, 1, 36, "Subsubsection",
 CellTags->"B00"],
Cell[121617, 4745, 290, 8, 41, "Text",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[121932, 4757, 104, 3, 26, "Input",
 CellTags->"B00"],
Cell[122039, 4762, 162, 6, 26, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[122216, 4771, 576, 25, 24, "Text",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[122829, 4801, 51, 1, 36, "Subsubsection",
 CellTags->"B00"],
Cell[122883, 4804, 202, 7, 26, "Text",
 CellTags->"B00"],
Cell[CellGroupData[{
Cell[123110, 4815, 97, 3, 26, "Input",
 CellTags->"B00"],
Cell[123210, 4820, 156, 6, 28, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[123403, 4831, 211, 7, 29, "Input",
 CellTags->"B00"],
Cell[123617, 4840, 2675, 90, 88, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[126329, 4935, 211, 7, 29, "Input",
 CellTags->"B00"],
Cell[126543, 4944, 1452, 54, 43, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[128032, 5003, 261, 8, 29, "Input",
 CellTags->"B00"],
Cell[128296, 5013, 516, 22, 29, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[128849, 5040, 181, 6, 29, "Input",
 CellTags->"B00"],
Cell[129033, 5048, 614, 22, 44, "Output",
 CellTags->"B00"]
}, Open  ]],
Cell[CellGroupData[{
Cell[129684, 5075, 248, 8, 29, "Input",
 CellTags->"B00"],
Cell[129935, 5085, 641, 23, 44, "Output",
 CellTags->"B00"]
}, Open  ]]
}, Open  ]],
Cell[130603, 5112, 41, 1, 39, "ObjectName",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[130669, 5117, 53, 1, 36, "Subsubsection",
 CellTags->"B1"],
Cell[130725, 5120, 230, 7, 24, "Text",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[130980, 5131, 102, 3, 26, "Input",
 CellTags->"B1"],
Cell[131085, 5136, 161, 6, 26, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[131261, 5145, 951, 39, 24, "Text",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[132249, 5189, 50, 1, 36, "Subsubsection",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[132324, 5194, 209, 7, 29, "Input",
 CellTags->"B1"],
Cell[132536, 5203, 1153, 42, 49, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[133726, 5250, 259, 8, 29, "Input",
 CellTags->"B1"],
Cell[133988, 5260, 316, 12, 29, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[134319, 5275, 187, 6, 26, "Input",
 CellTags->"B1"],
Cell[CellGroupData[{
Cell[134531, 5285, 209, 7, 29, "Input",
 CellTags->"B1"],
Cell[134743, 5294, 390, 15, 42, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[135170, 5314, 179, 6, 29, "Input",
 CellTags->"B1"],
Cell[135352, 5322, 403, 14, 42, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[135792, 5341, 154, 5, 29, "Input",
 CellTags->"B1"],
Cell[135949, 5348, 403, 14, 42, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[136389, 5367, 238, 7, 29, "Input",
 CellTags->"B1"],
Cell[136630, 5376, 791, 25, 48, "Output",
 CellTags->"B1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[137458, 5406, 279, 8, 29, "Input",
 CellTags->"B1"],
Cell[137740, 5416, 419, 14, 42, "Output",
 CellTags->"B1"]
}, Open  ]]
}, Open  ]],
Cell[138186, 5434, 43, 1, 39, "ObjectName",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[138254, 5439, 54, 1, 36, "Subsubsection",
 CellTags->"B11"],
Cell[138311, 5442, 495, 17, 43, "Text",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[138831, 5463, 104, 3, 26, "Input",
 CellTags->"B11"],
Cell[138938, 5468, 162, 6, 26, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[139115, 5477, 748, 32, 24, "Text",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[139900, 5514, 51, 1, 36, "Subsubsection",
 CellTags->"B11"],
Cell[139954, 5517, 215, 7, 26, "Text",
 CellTags->"B11"],
Cell[CellGroupData[{
Cell[140194, 5528, 97, 3, 26, "Input",
 CellTags->"B11"],
Cell[140294, 5533, 156, 6, 28, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[140487, 5544, 212, 7, 29, "Input",
 CellTags->"B11"],
Cell[140702, 5553, 2848, 97, 95, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[143587, 5655, 212, 7, 29, "Input",
 CellTags->"B11"],
Cell[143802, 5664, 1206, 47, 53, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[145045, 5716, 171, 5, 26, "Input",
 CellTags->"B11"],
Cell[145219, 5723, 164, 6, 26, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[145420, 5734, 212, 7, 29, "Input",
 CellTags->"B11"],
Cell[145635, 5743, 517, 22, 29, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146189, 5770, 170, 5, 26, "Input",
 CellTags->"B11"],
Cell[146362, 5777, 163, 6, 26, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[146562, 5788, 182, 6, 29, "Input",
 CellTags->"B11"],
Cell[146747, 5796, 269, 9, 42, "Output",
 CellTags->"B11"]
}, Open  ]],
Cell[CellGroupData[{
Cell[147053, 5810, 249, 8, 29, "Input",
 CellTags->"B11"],
Cell[147305, 5820, 295, 10, 42, "Output",
 CellTags->"B11"]
}, Open  ]]
}, Open  ]],
Cell[147627, 5834, 41, 1, 39, "ObjectName",
 CellTags->"CA"],
Cell[CellGroupData[{
Cell[147693, 5839, 53, 1, 36, "Subsubsection",
 CellTags->"CA"],
Cell[147749, 5842, 198, 9, 24, "Text",
 CellTags->"CA"],
Cell[147950, 5853, 435, 18, 24, "Text",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148422, 5876, 50, 1, 36, "Subsubsection",
 CellTags->"CA"],
Cell[CellGroupData[{
Cell[148497, 5881, 70, 2, 26, "Input",
 CellTags->"CA"],
Cell[148570, 5885, 120, 4, 26, "Output",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[148727, 5894, 173, 5, 26, "Input",
 CellTags->"CA"],
Cell[148903, 5901, 98, 3, 27, "Output",
 CellTags->"CA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[149038, 5909, 72, 2, 26, "Input",
 CellTags->"CA"],
Cell[149113, 5913, 98, 3, 27, "Output",
 CellTags->"CA"]
}, Open  ]]
}, Open  ]],
Cell[149238, 5920, 45, 1, 39, "ObjectName",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[149308, 5925, 55, 1, 36, "Subsubsection",
 CellTags->"Calc"],
Cell[149366, 5928, 233, 5, 41, "Text",
 CellTags->"Calc"],
Cell[149602, 5935, 657, 25, 24, "Text",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[150296, 5965, 61, 1, 33, "Subsubsection",
 CellTags->"Calc"],
Cell[150360, 5968, 346, 13, 27, "Text",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[150731, 5985, 284, 9, 26, "Input",
 CellTags->"Calc"],
Cell[151018, 5996, 147, 5, 26, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[151180, 6004, 213, 9, 24, "Text",
 CellTags->"Calc"],
Cell[CellGroupData[{
Cell[151418, 6017, 258, 8, 26, "Input",
 CellTags->"Calc"],
Cell[151679, 6027, 352, 15, 26, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[152068, 6047, 328, 11, 26, "Input",
 CellTags->"Calc"],
Cell[152399, 6060, 424, 16, 28, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[152860, 6081, 101, 3, 26, "Input",
 CellTags->"Calc"],
Cell[152964, 6086, 525, 22, 28, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[153526, 6113, 215, 6, 26, "Input",
 CellTags->"Calc"],
Cell[153744, 6121, 985, 44, 28, "Output",
 CellTags->"Calc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[154766, 6170, 382, 13, 26, "Input",
 CellTags->"Calc"],
Cell[155151, 6185, 100, 3, 26, "Output",
 CellTags->"Calc"]
}, Open  ]]
}, Open  ]],
Cell[155278, 6192, 67, 1, 39, "ObjectName",
 CellTags->"CalcColorFactor"],
Cell[CellGroupData[{
Cell[155370, 6197, 66, 1, 36, "Subsubsection",
 CellTags->"CalcColorFactor"],
Cell[155439, 6200, 470, 9, 108, "Text",
 CellTags->"CalcColorFactor"],
Cell[155912, 6211, 280, 11, 24, "Text",
 CellTags->"CalcColorFactor"],
Cell[CellGroupData[{
Cell[156217, 6226, 250, 7, 26, "Input"],
Cell[156470, 6235, 333, 14, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[156840, 6254, 99, 2, 26, "Input"],
Cell[156942, 6258, 1707, 55, 30, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[158686, 6318, 118, 3, 26, "Input"],
Cell[158807, 6323, 2279, 77, 59, "Output"]
}, Open  ]],
Cell[161101, 6403, 662, 24, 24, "Text",
 CellTags->"Collecting"]
}, Open  ]],
Cell[161778, 6430, 94, 1, 39, "ObjectName",
 CellTags->"CalculateCounterTerm"],
Cell[CellGroupData[{
Cell[161897, 6435, 71, 1, 36, "Subsubsection",
 CellTags->"CalculateCounterTerm"],
Cell[161971, 6438, 110, 1, 24, "Text",
 CellTags->"CalculateCounterTerm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[162118, 6444, 77, 1, 33, "Subsubsection",
 CellTags->"CalculateCounterTerm"],
Cell[CellGroupData[{
Cell[162220, 6449, 123, 3, 26, "Input",
 CellTags->"CalculateCounterTerm"],
Cell[162346, 6454, 191, 3, 40, "Print",
 CellTags->{"CalculateCounterTerm", "Info3535453139-5096475"}]
}, Open  ]]
}, Open  ]],
Cell[162564, 6461, 53, 1, 39, "ObjectName",
 CellTags->"CancelQP"],
Cell[CellGroupData[{
Cell[162642, 6466, 59, 1, 36, "Subsubsection",
 CellTags->"CancelQP"],
Cell[162704, 6469, 314, 13, 26, "Text",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[163055, 6487, 65, 1, 33, "Subsubsection",
 CellTags->"CancelQP"],
Cell[CellGroupData[{
Cell[163145, 6492, 549, 19, 26, "Input",
 CellTags->"CancelQP"],
Cell[163697, 6513, 667, 24, 47, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[164401, 6542, 182, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[164586, 6549, 2183, 73, 47, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[166806, 6627, 183, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[166992, 6634, 6421, 209, 189, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[173450, 6848, 555, 18, 26, "Input",
 CellTags->"CancelQP"],
Cell[174008, 6868, 886, 33, 47, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[174931, 6906, 182, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[175116, 6913, 6471, 225, 96, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[181624, 7143, 183, 5, 26, "Input",
 CellTags->"CancelQP"],
Cell[181810, 7150, 3734, 126, 49, "Output",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[185559, 7279, 106, 3, 26, "Input",
 CellTags->"CancelQP"]
}, Open  ]],
Cell[185680, 7285, 49, 1, 39, "ObjectName",
 CellTags->"Cases2"],
Cell[CellGroupData[{
Cell[185754, 7290, 57, 1, 36, "Subsubsection",
 CellTags->"Cases2"],
Cell[185814, 7293, 293, 6, 41, "Text",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[186144, 7304, 63, 1, 33, "Subsubsection",
 CellTags->"Cases2"],
Cell[CellGroupData[{
Cell[186232, 7309, 305, 10, 29, "Input",
 CellTags->"Cases2"],
Cell[186540, 7321, 275, 9, 26, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[186852, 7335, 312, 10, 26, "Input",
 CellTags->"Cases2"],
Cell[187167, 7347, 279, 9, 26, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[187483, 7361, 427, 14, 26, "Input",
 CellTags->"Cases2"],
Cell[187913, 7377, 355, 11, 26, "Output",
 CellTags->"Cases2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[188305, 7393, 327, 10, 26, "Input",
 CellTags->"Cases2"],
Cell[188635, 7405, 463, 17, 28, "Output",
 CellTags->"Cases2"]
}, Open  ]]
}, Open  ]],
Cell[189125, 7426, 41, 1, 39, "ObjectName",
 CellTags->"CF"],
Cell[CellGroupData[{
Cell[189191, 7431, 53, 1, 36, "Subsubsection",
 CellTags->"CF"],
Cell[189247, 7434, 287, 13, 24, "Text",
 CellTags->"CF"],
Cell[189537, 7449, 435, 18, 24, "Text",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190009, 7472, 50, 1, 36, "Subsubsection",
 CellTags->"CF"],
Cell[CellGroupData[{
Cell[190084, 7477, 70, 2, 26, "Input",
 CellTags->"CF"],
Cell[190157, 7481, 120, 4, 26, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190314, 7490, 178, 5, 26, "Input",
 CellTags->"CF"],
Cell[190495, 7497, 193, 7, 44, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190725, 7509, 83, 2, 26, "Input"],
Cell[190811, 7513, 76, 1, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[190924, 7519, 72, 2, 26, "Input",
 CellTags->"CF"],
Cell[190999, 7523, 98, 3, 27, "Output",
 CellTags->"CF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[191134, 7531, 233, 7, 26, "Input",
 CellTags->"CF"],
Cell[191370, 7540, 177, 6, 26, "Output",
 CellTags->"CF"]
}, Open  ]]
}, Open  ]],
Cell[191574, 7550, 67, 1, 39, "ObjectName",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[191666, 7555, 66, 1, 28, "Subsubsection",
 CellTags->"ChangeDimension"],
Cell[191735, 7558, 210, 5, 22, "Text",
 CellTags->"ChangeDimension"],
Cell[191948, 7565, 848, 32, 22, "Text",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[192833, 7602, 63, 1, 34, "Subsubsection",
 CellTags->"ChangeDimension"],
Cell[192899, 7605, 219, 5, 41, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[193143, 7614, 212, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[193358, 7622, 198, 7, 28, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[193593, 7634, 151, 4, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[193747, 7640, 198, 7, 28, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[193982, 7652, 116, 3, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[194101, 7657, 272, 8, 38, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[194388, 7668, 119, 3, 24, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[194532, 7675, 201, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[194736, 7683, 222, 6, 38, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[194973, 7692, 105, 1, 24, "Text",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[195103, 7697, 482, 15, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[195588, 7714, 571, 18, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196196, 7737, 153, 4, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[196352, 7743, 214, 7, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[196603, 7755, 222, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[196828, 7763, 214, 7, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[197057, 7773, 132, 3, 24, "Text",
 CellTags->"ChangeDimension"],
Cell[197192, 7778, 208, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[197403, 7786, 219, 5, 41, "Text",
 CellTags->"ChangeDimension"],
Cell[197625, 7793, 201, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[CellGroupData[{
Cell[197851, 7803, 220, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[198074, 7811, 489, 21, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[198600, 7837, 182, 5, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[198785, 7844, 489, 21, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[199311, 7870, 176, 5, 29, "Input",
 CellTags->"ChangeDimension"],
Cell[199490, 7877, 301, 10, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[199828, 7892, 141, 4, 29, "Input",
 CellTags->"ChangeDimension"],
Cell[199972, 7898, 130, 4, 26, "Output",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[200117, 7905, 201, 6, 26, "Input",
 CellTags->"ChangeDimension"],
Cell[200321, 7913, 157, 5, 26, "Input",
 CellTags->"ChangeDimension"]
}, Open  ]],
Cell[200493, 7921, 83, 1, 39, "ObjectName",
 CellTags->"ChargeConjugationMatrix"],
Cell[CellGroupData[{
Cell[200601, 7926, 74, 1, 36, "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],
Cell[200678, 7929, 176, 6, 24, "Text",
 CellTags->"ChargeConjugationMatrix"],
Cell[200857, 7937, 348, 11, 24, "Text",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201242, 7953, 71, 1, 36, "Subsubsection",
 CellTags->"ChargeConjugationMatrix"],
Cell[CellGroupData[{
Cell[201338, 7958, 214, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[201555, 7965, 306, 11, 26, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[201898, 7981, 120, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[202021, 7986, 255, 9, 30, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[202313, 8000, 209, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[202525, 8007, 228, 7, 26, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[202790, 8019, 120, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[202913, 8024, 192, 6, 28, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[203142, 8035, 134, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrix"],
Cell[203279, 8040, 137, 4, 26, "Output",
 CellTags->"ChargeConjugationMatrix"]
}, Open  ]]
}, Open  ]],
Cell[203443, 8048, 89, 1, 39, "ObjectName",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[CellGroupData[{
Cell[203557, 8053, 77, 1, 36, "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[203637, 8056, 195, 4, 24, "Text",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[203835, 8062, 339, 11, 24, "Text",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[204211, 8078, 74, 1, 36, "Subsubsection",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[CellGroupData[{
Cell[204310, 8083, 217, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[204530, 8090, 309, 11, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[204876, 8106, 123, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[205002, 8111, 258, 9, 30, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[205297, 8125, 212, 5, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[205512, 8132, 231, 7, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[205780, 8144, 123, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[205906, 8149, 195, 6, 28, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206138, 8160, 165, 4, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[206306, 8166, 189, 6, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206532, 8177, 119, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[206654, 8182, 164, 5, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]],
Cell[CellGroupData[{
Cell[206855, 8192, 140, 3, 26, "Input",
 CellTags->"ChargeConjugationMatrixInv"],
Cell[206998, 8197, 140, 4, 26, "Output",
 CellTags->"ChargeConjugationMatrixInv"]
}, Open  ]]
}, Open  ]],
Cell[207165, 8205, 61, 1, 39, "ObjectName",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[207251, 8210, 63, 1, 36, "Subsubsection",
 CellTags->"CheckContext"],
Cell[207317, 8213, 164, 4, 24, "Text",
 CellTags->"CheckContext"],
Cell[207484, 8219, 493, 18, 24, "Text",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[CellGroupData[{
Cell[208014, 8242, 60, 1, 36, "Subsubsection",
 CellTags->"CheckContext"],
Cell[208077, 8245, 142, 4, 24, "Text",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[208244, 8253, 138, 3, 26, "Input",
 CellTags->"CheckContext"],
Cell[208385, 8258, 112, 3, 26, "Output",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[208512, 8264, 87, 1, 24, "Text",
 CellTags->"CheckContext"],
Cell[CellGroupData[{
Cell[208624, 8269, 92, 2, 26, "Input",
 CellTags->"CheckContext"],
Cell[208719, 8273, 121, 3, 26, "Output",
 CellTags->"CheckContext"]
}, Open  ]],
Cell[CellGroupData[{
Cell[208877, 8281, 138, 3, 26, "Input",
 CellTags->"CheckContext"],
Cell[209018, 8286, 111, 3, 26, "Output",
 CellTags->"CheckContext"]
}, Open  ]]
}, Open  ]],
Cell[209156, 8293, 51, 1, 39, "ObjectName",
 CellTags->"CheckDB"],
Cell[CellGroupData[{
Cell[209232, 8298, 106, 3, 36, "Subsubsection",
 CellTags->"CheckDB"],
Cell[209341, 8303, 1081, 15, 126, "Text"],
Cell[CellGroupData[{
Cell[210447, 8322, 113, 3, 26, "Input",
 CellTags->"CheckDB"],
Cell[210563, 8327, 471, 14, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[CellGroupData[{
Cell[211071, 8346, 116, 3, 26, "Input",
 CellTags->"CheckDB"],
Cell[211190, 8351, 134, 4, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[211339, 8358, 115, 3, 24, "Text"],
Cell[CellGroupData[{
Cell[211479, 8365, 337, 11, 26, "Input"],
Cell[211819, 8378, 178, 5, 22, "Print"],
Cell[212000, 8385, 178, 5, 22, "Print"],
Cell[212181, 8392, 128, 4, 26, "Output"]
}, Open  ]],
Cell[212324, 8399, 133, 3, 24, "Text"],
Cell[CellGroupData[{
Cell[212482, 8406, 383, 13, 26, "Input",
 CellTags->"CheckDB"],
Cell[212868, 8421, 150, 5, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[213033, 8429, 60, 0, 24, "Text"],
Cell[CellGroupData[{
Cell[213118, 8433, 303, 9, 29, "Input",
 CellTags->"CheckDB"],
Cell[213424, 8444, 116, 3, 26, "Output",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[213555, 8450, 268, 8, 29, "Input",
 CellTags->"CheckDB"]
}, Open  ]],
Cell[213838, 8461, 73, 1, 39, "ObjectName",
 CellTags->"ChiralityProjector"],
Cell[CellGroupData[{
Cell[213936, 8466, 69, 1, 36, "Subsubsection",
 CellTags->"ChiralityProjector"],
Cell[214008, 8469, 499, 19, 24, "Text",
 CellTags->"ChiralityProjector"],
Cell[214510, 8490, 655, 25, 24, "Text",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[215202, 8520, 66, 1, 36, "Subsubsection",
 CellTags->"ChiralityProjector"],
Cell[CellGroupData[{
Cell[215293, 8525, 262, 8, 26, "Input",
 CellTags->"ChiralityProjector"],
Cell[215558, 8535, 227, 7, 29, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[215822, 8547, 114, 3, 26, "Input",
 CellTags->"ChiralityProjector"],
Cell[215939, 8552, 227, 7, 29, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[216203, 8564, 196, 5, 26, "Input",
 CellTags->"ChiralityProjector"],
Cell[216402, 8571, 380, 14, 46, "Output",
 CellTags->"ChiralityProjector"]
}, Open  ]]
}, Open  ]],
Cell[216809, 8589, 53, 1, 39, "ObjectName",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[216887, 8594, 59, 1, 36, "Subsubsection",
 CellTags->"Chisholm"],
Cell[216949, 8597, 142, 4, 24, "Text",
 CellTags->"Chisholm"],
Cell[217094, 8603, 489, 18, 24, "Text",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[217620, 8626, 56, 1, 36, "Subsubsection",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[217701, 8631, 206, 6, 26, "Input",
 CellTags->"Chisholm"],
Cell[217910, 8639, 2020, 78, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[219967, 8722, 183, 5, 26, "Input",
 CellTags->"Chisholm"],
Cell[220153, 8729, 236, 7, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[220426, 8741, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[220568, 8747, 1374, 54, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[221957, 8804, 77, 1, 24, "Text",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[222059, 8809, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[222192, 8815, 105, 3, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[222334, 8823, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[222476, 8829, 1374, 54, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[223887, 8888, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[224020, 8894, 105, 3, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[224162, 8902, 169, 5, 26, "Input",
 CellTags->"Chisholm"],
Cell[224334, 8909, 590, 22, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[224961, 8936, 110, 3, 26, "Input",
 CellTags->"Chisholm"],
Cell[225074, 8941, 1439, 57, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[226550, 9003, 232, 7, 26, "Input",
 CellTags->"Chisholm"],
Cell[226785, 9012, 391, 10, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[227213, 9027, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[227355, 9033, 11544, 439, 87, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[238936, 9477, 139, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[239078, 9483, 11544, 439, 87, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[250637, 9925, 85, 1, 24, "Text",
 CellTags->"Chisholm"],
Cell[CellGroupData[{
Cell[250747, 9930, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[250880, 9936, 125, 4, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[251042, 9945, 130, 4, 26, "Input",
 CellTags->"Chisholm"],
Cell[251175, 9951, 125, 4, 26, "Output",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[251315, 9958, 192, 6, 26, "Input",
 CellTags->"Chisholm"]
}, Open  ]],
Cell[251522, 9967, 65, 1, 39, "ObjectName",
 CellTags->"ChisholmSpinor"],
Cell[CellGroupData[{
Cell[251612, 9972, 65, 1, 36, "Subsubsection",
 CellTags->"ChisholmSpinor"],
Cell[251680, 9975, 296, 6, 41, "Text",
 CellTags->"ChisholmSpinor"],
Cell[251979, 9983, 471, 18, 24, "Text",
 CellTags->"ChisholmSpinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[252487, 10006, 62, 1, 36, "Subsubsection",
 CellTags->"ChisholmSpinor"],
Cell[CellGroupData[{
Cell[252574, 10011, 312, 9, 26, "Input",
 CellTags->"ChisholmSpinor"],
Cell[252889, 10022, 427, 15, 26, "Output",
 CellTags->"ChisholmSpinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[253353, 10042, 117, 3, 26, "Input",
 CellTags->"ChisholmSpinor"],
Cell[253473, 10047, 2686, 99, 90, "Output",
 CellTags->"ChisholmSpinor"]
}, Open  ]]
}, Open  ]],
Cell[256186, 10150, 75, 1, 39, "ObjectName",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[256286, 10155, 70, 1, 36, "Subsubsection",
 CellTags->"ClearScalarProducts"],
Cell[256359, 10158, 145, 4, 24, "Text",
 CellTags->"ClearScalarProducts"],
Cell[256507, 10164, 802, 31, 24, "Text",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257346, 10200, 67, 1, 36, "Subsubsection",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[257438, 10205, 195, 6, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[257636, 10213, 139, 4, 26, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[257812, 10222, 211, 6, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[258026, 10230, 139, 4, 26, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[258180, 10237, 104, 2, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[CellGroupData[{
Cell[258309, 10243, 211, 6, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[258523, 10251, 173, 6, 28, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]],
Cell[CellGroupData[{
Cell[258733, 10262, 148, 4, 26, "Input",
 CellTags->"ClearScalarProducts"],
Cell[258884, 10268, 173, 6, 28, "Output",
 CellTags->"ClearScalarProducts"]
}, Open  ]]
}, Open  ]],
Cell[259084, 10278, 57, 1, 39, "ObjectName",
 CellTags->"Collecting"],
Cell[CellGroupData[{
Cell[259166, 10283, 61, 1, 36, "Subsubsection",
 CellTags->"Collecting"],
Cell[259230, 10286, 207, 5, 41, "Text",
 CellTags->"Collecting"],
Cell[259440, 10293, 662, 24, 24, "Text",
 CellTags->"Collecting"]
}, Open  ]],
Cell[260117, 10320, 53, 1, 39, "ObjectName",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[260195, 10325, 59, 1, 36, "Subsubsection",
 CellTags->"Collect2"],
Cell[260257, 10328, 613, 10, 75, "Text",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[260895, 10342, 115, 3, 26, "Input",
 CellTags->"Collect2"],
Cell[261013, 10347, 398, 11, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[261426, 10361, 257, 11, 24, "Text",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[261720, 10377, 56, 1, 36, "Subsubsection",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[261801, 10382, 440, 14, 40, "Input",
 CellTags->"Collect2"],
Cell[262244, 10398, 451, 16, 42, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[262732, 10419, 185, 5, 26, "Input",
 CellTags->"Collect2"],
Cell[262920, 10426, 399, 14, 42, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[263356, 10445, 186, 5, 26, "Input",
 CellTags->"Collect2"],
Cell[263545, 10452, 451, 16, 42, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[264033, 10473, 525, 18, 29, "Input",
 CellTags->"Collect2"],
Cell[264561, 10493, 537, 18, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[265135, 10516, 320, 10, 29, "Input",
 CellTags->"Collect2"],
Cell[265458, 10528, 820, 31, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[266315, 10564, 112, 3, 26, "Input",
 CellTags->"Collect2"],
Cell[266430, 10569, 825, 27, 26, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[267270, 10599, 130, 4, 26, "Input",
 CellTags->"Collect2"],
Cell[CellGroupData[{
Cell[267425, 10607, 305, 9, 29, "Input",
 CellTags->"Collect2"],
Cell[267733, 10618, 935, 31, 29, "Output",
 CellTags->"Collect2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[268705, 10654, 149, 4, 26, "Input",
 CellTags->"Collect2"],
Cell[268857, 10660, 671, 21, 29, "Output",
 CellTags->"Collect2"]
}, Open  ]]
}, Open  ]],
Cell[269555, 10685, 53, 1, 39, "ObjectName",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[269633, 10690, 59, 1, 36, "Subsubsection",
 CellTags->"Collect3"],
Cell[269695, 10693, 719, 23, 41, "Text",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[270439, 10720, 115, 3, 26, "Input",
 CellTags->"Collect3"],
Cell[270557, 10725, 233, 8, 26, "Output",
 CellTags->"Collect3"]
}, Open  ]],
Cell[270805, 10736, 449, 18, 24, "Text",
 CellTags->"Collect3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[271291, 10759, 56, 1, 36, "Subsubsection",
 CellTags->"Collect3"],
Cell[CellGroupData[{
Cell[271372, 10764, 525, 18, 29, "Input",
 CellTags->"Collect3"],
Cell[271900, 10784, 548, 19, 26, "Output",
 CellTags->"Collect3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[272485, 10808, 289, 9, 29, "Input",
 CellTags->"Collect3"],
Cell[272777, 10819, 4614, 134, 139, "Output",
 CellTags->"Collect3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[277428, 10958, 314, 9, 29, "Input"],
Cell[277745, 10969, 802, 26, 26, "Output"]
}, Open  ]]
}, Open  ]],
Cell[278574, 10999, 51, 1, 39, "ObjectName",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[278650, 11004, 58, 1, 36, "Subsubsection",
 CellTags->"Combine"],
Cell[278711, 11007, 315, 6, 41, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[279051, 11017, 113, 3, 26, "Input",
 CellTags->"Combine"],
Cell[279167, 11022, 171, 6, 26, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[279353, 11031, 256, 11, 24, "Text",
 CellTags->"Combine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[279646, 11047, 55, 1, 36, "Subsubsection",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[279726, 11052, 283, 10, 43, "Input",
 CellTags->"Combine"],
Cell[280012, 11064, 304, 11, 46, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[280331, 11078, 122, 3, 24, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[280478, 11085, 284, 10, 43, "Input",
 CellTags->"Combine"],
Cell[280765, 11097, 303, 10, 46, "Output",
 CellTags->"Combine"]
}, Open  ]],
Cell[281083, 11110, 175, 4, 24, "Text",
 CellTags->"Combine"],
Cell[CellGroupData[{
Cell[281283, 11118, 354, 12, 43, "Input",
 CellTags->"Combine"],
Cell[281640, 11132, 303, 10, 46, "Output",
 CellTags->"Combine"]
}, Open  ]]
}, Open  ]],
Cell[281970, 11146, 63, 1, 39, "ObjectName",
 CellTags->"CombineGraphs"],
Cell[CellGroupData[{
Cell[282058, 11151, 64, 1, 36, "Subsubsection",
 CellTags->"CombineGraphs"],
Cell[282125, 11154, 86, 1, 24, "Text",
 CellTags->"CombineGraphs"],
Cell[282214, 11157, 269, 10, 24, "Text",
 CellTags->"CombineGraphs"]
}, Open  ]],
Cell[282498, 11170, 57, 1, 39, "ObjectName",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[282580, 11175, 61, 1, 36, "Subsubsection",
 CellTags->"Commutator"],
Cell[282644, 11178, 138, 4, 24, "Text",
 CellTags->"Commutator"],
Cell[282785, 11184, 967, 32, 24, "Text",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[283789, 11221, 58, 1, 36, "Subsubsection",
 CellTags->"Commutator"],
Cell[283850, 11224, 167, 4, 26, "Input",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[284042, 11232, 136, 4, 26, "Input",
 CellTags->"Commutator"],
Cell[284181, 11238, 184, 5, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[284402, 11248, 121, 3, 26, "Input",
 CellTags->"Commutator"],
Cell[284526, 11253, 177, 6, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[284740, 11264, 247, 8, 26, "Input",
 CellTags->"Commutator"],
Cell[284990, 11274, 381, 12, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[285386, 11289, 169, 4, 26, "Input",
 CellTags->"Commutator"],
Cell[285558, 11295, 68, 1, 24, "Text",
 CellTags->"Commutator"],
Cell[285629, 11298, 228, 6, 26, "Input",
 CellTags->"Commutator"],
Cell[CellGroupData[{
Cell[285882, 11308, 463, 15, 26, "Input",
 CellTags->"Commutator"],
Cell[286348, 11325, 727, 19, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[287112, 11349, 114, 3, 26, "Input",
 CellTags->"Commutator"],
Cell[287229, 11354, 106, 3, 26, "Output",
 CellTags->"Commutator"]
}, Open  ]],
Cell[287350, 11360, 113, 3, 26, "Input",
 CellTags->"Commutator"],
Cell[287466, 11365, 159, 4, 26, "Input",
 CellTags->"Commutator"]
}, Open  ]],
Cell[287640, 11372, 73, 1, 39, "ObjectName",
 CellTags->"CommutatorExplicit"],
Cell[CellGroupData[{
Cell[287738, 11377, 69, 1, 36, "Subsubsection",
 CellTags->"CommutatorExplicit"],
Cell[287810, 11380, 157, 4, 24, "Text",
 CellTags->"CommutatorExplicit"],
Cell[287970, 11386, 459, 18, 24, "Text",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[288466, 11409, 66, 1, 36, "Subsubsection",
 CellTags->"CommutatorExplicit"],
Cell[288535, 11412, 175, 4, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[CellGroupData[{
Cell[288735, 11420, 144, 4, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[288882, 11426, 192, 5, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[289111, 11436, 129, 3, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[289243, 11441, 185, 6, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[289465, 11452, 196, 6, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[289664, 11460, 283, 10, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[289984, 11475, 129, 3, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[290116, 11480, 389, 14, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[290542, 11499, 167, 5, 26, "Input",
 CellTags->"CommutatorExplicit"],
Cell[290712, 11506, 389, 12, 26, "Output",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[291116, 11521, 177, 4, 26, "Input",
 CellTags->"CommutatorExplicit"]
}, Open  ]],
Cell[291308, 11528, 65, 1, 39, "ObjectName",
 CellTags->"CompleteSquare"],
Cell[CellGroupData[{
Cell[291398, 11533, 65, 1, 36, "Subsubsection",
 CellTags->"CompleteSquare"],
Cell[291466, 11536, 1107, 82, 41, "Text",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[292610, 11623, 62, 1, 36, "Subsubsection",
 CellTags->"CompleteSquare"],
Cell[CellGroupData[{
Cell[292697, 11628, 277, 10, 26, "Input"],
Cell[292977, 11640, 337, 12, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[293351, 11657, 170, 5, 26, "Input",
 CellTags->"CompleteSquare"],
Cell[293524, 11664, 685, 27, 46, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[294246, 11696, 180, 6, 26, "Input",
 CellTags->"CompleteSquare"],
Cell[294429, 11704, 110, 3, 26, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[CellGroupData[{
Cell[294576, 11712, 155, 4, 26, "Input",
 CellTags->"CompleteSquare"],
Cell[294734, 11718, 515, 20, 46, "Output",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[295264, 11741, 137, 4, 26, "Input",
 CellTags->"CompleteSquare"]
}, Open  ]],
Cell[295416, 11748, 69, 1, 39, "ObjectName",
 CellTags->"ComplexConjugate"],
Cell[CellGroupData[{
Cell[295510, 11753, 67, 1, 36, "Subsubsection",
 CellTags->"ComplexConjugate"],
Cell[295580, 11756, 414, 7, 58, "Text",
 CellTags->"ComplexConjugate"],
Cell[295997, 11765, 709, 25, 24, "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[296743, 11795, 64, 1, 36, "Subsubsection",
 CellTags->"ComplexConjugate"],
Cell[CellGroupData[{
Cell[296832, 11800, 196, 5, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[297031, 11807, 163, 5, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[297231, 11817, 121, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[297355, 11822, 239, 6, 38, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[297631, 11833, 152, 4, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[297786, 11839, 249, 7, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[298072, 11851, 125, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[298200, 11856, 427, 16, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[298664, 11877, 182, 5, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[298849, 11884, 531, 22, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[299417, 11911, 125, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[299545, 11916, 531, 22, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[300113, 11943, 190, 5, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[300306, 11950, 380, 17, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[300723, 11972, 270, 7, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[300996, 11981, 157, 4, 38, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[301190, 11990, 273, 8, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[301466, 12000, 335, 10, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[301838, 12015, 125, 3, 26, "Input",
 CellTags->"ComplexConjugate"],
Cell[301966, 12020, 431, 15, 26, "Output",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[302412, 12038, 166, 4, 24, "Text",
 CellTags->"ComplexConjugate"]
}, Open  ]],
Cell[302593, 12045, 53, 1, 39, "ObjectName",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[302671, 12050, 59, 1, 36, "Subsubsection",
 CellTags->"Contract"],
Cell[302733, 12053, 411, 7, 58, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[303169, 12064, 115, 3, 26, "Input",
 CellTags->"Contract"],
Cell[303287, 12069, 612, 15, 47, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[303914, 12087, 138, 3, 24, "Text"],
Cell[304055, 12092, 677, 25, 24, "Text",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[304769, 12122, 56, 1, 36, "Subsubsection",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[304850, 12127, 233, 7, 26, "Input",
 CellTags->"Contract"],
Cell[305086, 12136, 207, 7, 29, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[305330, 12148, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[305442, 12153, 236, 10, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[305715, 12168, 199, 6, 26, "Input",
 CellTags->"Contract"],
Cell[305917, 12176, 264, 10, 29, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[306218, 12191, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[306330, 12196, 221, 8, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[306588, 12209, 144, 4, 26, "Input",
 CellTags->"Contract"],
Cell[306735, 12215, 155, 5, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[306905, 12223, 83, 1, 24, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[307013, 12228, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[307125, 12233, 104, 3, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[307244, 12239, 208, 5, 41, "Text",
 CellTags->"Contract"],
Cell[CellGroupData[{
Cell[307477, 12248, 231, 7, 26, "Input",
 CellTags->"Contract"],
Cell[307711, 12257, 233, 8, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[307981, 12270, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[308093, 12275, 104, 3, 27, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[308234, 12283, 231, 7, 26, "Input",
 CellTags->"Contract"],
Cell[308468, 12292, 233, 8, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[308738, 12305, 227, 7, 26, "Input",
 CellTags->"Contract"],
Cell[308968, 12314, 181, 6, 29, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[309186, 12325, 114, 3, 26, "Input",
 CellTags->"Contract"],
Cell[309303, 12330, 206, 8, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[309546, 12343, 277, 9, 26, "Input",
 CellTags->"Contract"],
Cell[309826, 12354, 427, 18, 30, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[310290, 12377, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[310402, 12382, 575, 24, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311014, 12411, 271, 8, 26, "Input",
 CellTags->"Contract"],
Cell[311288, 12421, 555, 23, 28, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[311880, 12449, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[311992, 12454, 439, 19, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[312468, 12478, 313, 9, 26, "Input",
 CellTags->"Contract"],
Cell[312784, 12489, 922, 40, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[313743, 12534, 109, 3, 26, "Input",
 CellTags->"Contract"],
Cell[313855, 12539, 204, 7, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[314096, 12551, 426, 13, 26, "Input",
 CellTags->"Contract"],
Cell[314525, 12566, 301, 9, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[314863, 12580, 139, 4, 26, "Input",
 CellTags->"Contract"],
Cell[315005, 12586, 352, 12, 26, "Output",
 CellTags->"Contract"]
}, Open  ]],
Cell[315372, 12601, 194, 6, 26, "Input",
 CellTags->"Contract"]
}, Open  ]],
Cell[315581, 12610, 55, 1, 39, "ObjectName",
 CellTags->"Contract1"],
Cell[CellGroupData[{
Cell[315661, 12615, 60, 1, 28, "Subsubsection",
 CellTags->"Contract1"],
Cell[315724, 12618, 459, 8, 83, "Text",
 CellTags->"Contract1"],
Cell[316186, 12628, 633, 24, 22, "Text",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[316856, 12657, 57, 1, 36, "Subsubsection",
 CellTags->"Contract1"],
Cell[CellGroupData[{
Cell[316938, 12662, 349, 12, 26, "Input",
 CellTags->"Contract1"],
Cell[317290, 12676, 207, 8, 28, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[317534, 12689, 349, 12, 26, "Input",
 CellTags->"Contract1"],
Cell[317886, 12703, 478, 20, 28, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318401, 12728, 269, 9, 26, "Input",
 CellTags->"Contract1"],
Cell[318673, 12739, 105, 3, 26, "Output",
 CellTags->"Contract1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[318815, 12747, 269, 9, 26, "Input",
 CellTags->"Contract1"],
Cell[319087, 12758, 298, 11, 26, "Output",
 CellTags->"Contract1"]
}, Open  ]]
}, Open  ]],
Cell[319412, 12773, 55, 1, 39, "ObjectName",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[319492, 12778, 60, 1, 28, "Subsubsection",
 CellTags->"Convolute"],
Cell[319555, 12781, 1363, 49, 64, "Text",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[320943, 12834, 117, 3, 26, "Input",
 CellTags->"Convolute"],
Cell[321063, 12839, 400, 13, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[321478, 12855, 510, 18, 22, "Text",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322025, 12878, 57, 1, 36, "Subsubsection",
 CellTags->"Convolute"],
Cell[CellGroupData[{
Cell[322107, 12883, 134, 4, 26, "Input",
 CellTags->"Convolute"],
Cell[322244, 12889, 154, 5, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322435, 12899, 134, 4, 26, "Input",
 CellTags->"Convolute"],
Cell[322572, 12905, 178, 6, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[322787, 12916, 134, 4, 26, "Input",
 CellTags->"Convolute"],
Cell[322924, 12922, 128, 4, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[323089, 12931, 181, 6, 42, "Input",
 CellTags->"Convolute"],
Cell[323273, 12939, 215, 7, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[323525, 12951, 227, 7, 42, "Input",
 CellTags->"Convolute"],
Cell[323755, 12960, 215, 7, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[324007, 12972, 181, 6, 42, "Input",
 CellTags->"Convolute"],
Cell[324191, 12980, 307, 11, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[324535, 12996, 228, 8, 42, "Input",
 CellTags->"Convolute"],
Cell[324766, 13006, 536, 19, 43, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325339, 13030, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[325532, 13038, 321, 11, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[325890, 13054, 216, 7, 26, "Input",
 CellTags->"Convolute"],
Cell[326109, 13063, 273, 10, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[326419, 13078, 237, 8, 42, "Input",
 CellTags->"Convolute"],
Cell[326659, 13088, 389, 13, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[327085, 13106, 263, 9, 42, "Input",
 CellTags->"Convolute"],
Cell[327351, 13117, 617, 21, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[328005, 13143, 239, 8, 42, "Input",
 CellTags->"Convolute"],
Cell[328247, 13153, 717, 24, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[329001, 13182, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[329194, 13190, 245, 9, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[329476, 13204, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[329669, 13212, 317, 11, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330023, 13228, 260, 9, 42, "Input",
 CellTags->"Convolute"],
Cell[330286, 13239, 452, 16, 48, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[330775, 13260, 165, 5, 26, "Input",
 CellTags->"Convolute"],
Cell[330943, 13267, 229, 9, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[331209, 13281, 195, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[331407, 13289, 239, 9, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[331683, 13303, 212, 7, 42, "Input",
 CellTags->"Convolute"],
Cell[331898, 13312, 416, 14, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[332351, 13331, 213, 7, 42, "Input",
 CellTags->"Convolute"],
Cell[332567, 13340, 316, 12, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[332920, 13357, 237, 8, 42, "Input",
 CellTags->"Convolute"],
Cell[333160, 13367, 550, 18, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[333747, 13390, 213, 7, 42, "Input",
 CellTags->"Convolute"],
Cell[333963, 13399, 409, 13, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[334409, 13417, 190, 6, 26, "Input",
 CellTags->"Convolute"],
Cell[334602, 13425, 245, 9, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[334884, 13439, 165, 5, 26, "Input",
 CellTags->"Convolute"],
Cell[335052, 13446, 188, 6, 26, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[335277, 13457, 286, 10, 42, "Input",
 CellTags->"Convolute"],
Cell[335566, 13469, 790, 29, 48, "Output",
 CellTags->"Convolute"]
}, Open  ]],
Cell[CellGroupData[{
Cell[336393, 13503, 295, 10, 42, "Input",
 CellTags->"Convolute"],
Cell[336691, 13515, 1193, 41, 64, "Output",
 CellTags->"Convolute"]
}, Open  ]]
}, Open  ]],
Cell[337911, 13560, 66, 1, 39, "ObjectName",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[338002, 13565, 65, 1, 36, "Subsubsection",
 CellTags->"ConvoluteTable"],
Cell[338070, 13568, 152, 4, 24, "Text",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[338247, 13576, 122, 3, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[338372, 13581, 405, 13, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[338792, 13597, 495, 18, 24, "Text",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339324, 13620, 62, 1, 36, "Subsubsection",
 CellTags->"ConvoluteTable"],
Cell[CellGroupData[{
Cell[339411, 13625, 144, 4, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[339558, 13631, 159, 5, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[339754, 13641, 144, 4, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[339901, 13647, 183, 6, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[340121, 13658, 144, 4, 26, "Input",
 CellTags->"ConvoluteTable"],
Cell[340268, 13664, 133, 4, 26, "Output",
 CellTags->"ConvoluteTable"]
}, Open  ]],
Cell[CellGroupData[{
Cell[340438, 13673, 244, 8, 42, "Input",
 CellTags->"Convolute"],
Cell[340685, 13683, 463, 16, 42, "Output",
 CellTags->"Convolute"]
}, Open  ]]
}, Open  ]],
Cell[341175, 13703, 85, 1, 39, "ObjectName",
 CellTags->"CovariantFieldDerivative"],
Cell[CellGroupData[{
Cell[341285, 13708, 75, 1, 36, "Subsubsection",
 CellTags->"CovariantFieldDerivative"],
Cell[341363, 13711, 372, 6, 58, "Text",
 CellTags->"CovariantFieldDerivative"],
Cell[341738, 13719, 721, 25, 24, "Text",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[342496, 13749, 72, 1, 36, "Subsubsection",
 CellTags->"CovariantFieldDerivative"],
Cell[CellGroupData[{
Cell[342593, 13754, 329, 9, 26, "Input",
 CellTags->"CovariantFieldDerivative"],
Cell[342925, 13765, 543, 21, 29, "Output",
 CellTags->"CovariantFieldDerivative"]
}, Open  ]]
}, Open  ]],
Cell[343495, 13790, 53, 1, 39, "ObjectName",
 CellTags->"CounterT"],
Cell[CellGroupData[{
Cell[343573, 13795, 59, 1, 36, "Subsubsection",
 CellTags->"CounterT"],
Cell[343635, 13798, 146, 4, 24, "Text",
 CellTags->"CounterT"],
Cell[343784, 13804, 713, 25, 24, "Text",
 CellTags->"CounterT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[344534, 13834, 56, 1, 36, "Subsubsection",
 CellTags->"CounterT"],
Cell[CellGroupData[{
Cell[344615, 13839, 284, 6, 26, "Input",
 CellTags->"CounterT"],
Cell[344902, 13847, 3127, 106, 63, "Output",
 CellTags->"CounterT"]
}, Open  ]]
}, Open  ]],
Cell[348056, 13957, 59, 1, 39, "ObjectName",
 CellTags->"CounterTerm"],
Cell[CellGroupData[{
Cell[348140, 13962, 62, 1, 36, "Subsubsection",
 CellTags->"CounterTerm"],
Cell[348205, 13965, 216, 5, 41, "Text",
 CellTags->"CounterTerm"],
Cell[348424, 13972, 928, 32, 24, "Text",
 CellTags->"CounterTerm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[349389, 14009, 59, 1, 36, "Subsubsection",
 CellTags->"CounterTerm"],
Cell[CellGroupData[{
Cell[349473, 14014, 116, 3, 26, "Input",
 CellTags->"CounterTerm"],
Cell[349592, 14019, 1632, 52, 66, "Output",
 CellTags->"CounterTerm"]
}, Open  ]]
}, Open  ]],
Cell[351251, 14075, 69, 1, 39, "ObjectName",
 CellTags->"CouplingConstant"],
Cell[CellGroupData[{
Cell[351345, 14080, 67, 1, 36, "Subsubsection",
 CellTags->"CouplingConstant"],
Cell[351415, 14083, 657, 11, 100, "Text",
 CellTags->"CouplingConstant"],
Cell[352075, 14096, 489, 18, 24, "Text",
 CellTags->"CouplingConstant"]
}, Open  ]],
Cell[352579, 14117, 57, 1, 39, "ObjectName",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[352661, 14122, 61, 1, 36, "Subsubsection",
 CellTags->"CovariantD"],
Cell[352725, 14125, 1210, 18, 201, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[353960, 14147, 119, 3, 26, "Input",
 CellTags->"CovariantD"],
Cell[354082, 14152, 439, 12, 26, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[354536, 14167, 185, 4, 24, "Text",
 CellTags->"CovariantD"],
Cell[354724, 14173, 719, 25, 24, "Text",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[355480, 14203, 58, 1, 36, "Subsubsection",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[355563, 14208, 117, 3, 26, "Input",
 CellTags->"CovariantD"],
Cell[355683, 14213, 132, 4, 29, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[355852, 14222, 150, 4, 26, "Input",
 CellTags->"CovariantD"],
Cell[356005, 14228, 161, 5, 30, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[356203, 14238, 180, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[356386, 14245, 992, 39, 34, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[357393, 14287, 164, 4, 24, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[357582, 14295, 120, 3, 26, "Input",
 CellTags->"CovariantD"],
Cell[357705, 14300, 176, 6, 26, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[357918, 14311, 153, 4, 26, "Input",
 CellTags->"CovariantD"],
Cell[358074, 14317, 205, 7, 28, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[358316, 14329, 203, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[358522, 14336, 939, 36, 33, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[359498, 14377, 183, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[359684, 14384, 932, 35, 33, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[360653, 14424, 187, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[360843, 14431, 1345, 49, 36, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[362203, 14483, 602, 22, 31, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[362830, 14509, 215, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[363048, 14517, 513, 20, 30, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[363576, 14540, 285, 10, 24, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[363886, 14554, 215, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[364104, 14562, 2287, 81, 54, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[366406, 14646, 189, 7, 24, "Text",
 CellTags->"CovariantD"],
Cell[CellGroupData[{
Cell[366620, 14657, 215, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[366838, 14665, 6316, 201, 58, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[373191, 14871, 181, 5, 26, "Input",
 CellTags->"CovariantD"],
Cell[373375, 14878, 261, 9, 31, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[373673, 14892, 195, 6, 26, "Input",
 CellTags->"CovariantD"],
Cell[373871, 14900, 4816, 151, 58, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[378724, 15056, 223, 7, 26, "Input",
 CellTags->"CovariantD"],
Cell[378950, 15065, 788, 24, 58, "Output",
 CellTags->"CovariantD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[379775, 15094, 240, 7, 26, "Input",
 CellTags->"CovariantD"],
Cell[380018, 15103, 803, 25, 58, "Output",
 CellTags->"CovariantD"]
}, Open  ]]
}, Open  ]],
Cell[380848, 15132, 61, 1, 39, "ObjectName",
 CellTags->"CrossProduct"],
Cell[CellGroupData[{
Cell[380934, 15137, 63, 1, 36, "Subsubsection",
 CellTags->"CrossProduct"],
Cell[381000, 15140, 145, 4, 24, "Text",
 CellTags->"CrossProduct"],
Cell[381148, 15146, 477, 18, 24, "Text",
 CellTags->"CrossProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[381662, 15169, 60, 1, 36, "Subsubsection",
 CellTags->"CrossProduct"],
Cell[CellGroupData[{
Cell[381747, 15174, 326, 9, 26, "Input",
 CellTags->"CrossProduct"],
Cell[382076, 15185, 639, 22, 29, "Output",
 CellTags->"CrossProduct"]
}, Open  ]]
}, Open  ]],
Cell[382742, 15211, 41, 1, 39, "ObjectName",
 CellTags->"C0"],
Cell[CellGroupData[{
Cell[382808, 15216, 53, 1, 36, "Subsubsection",
 CellTags->"C0"],
Cell[382864, 15219, 448, 10, 58, "Text",
 CellTags->"C0"],
Cell[383315, 15231, 771, 32, 24, "Text",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384123, 15268, 50, 1, 36, "Subsubsection",
 CellTags->"C0"],
Cell[CellGroupData[{
Cell[384198, 15273, 174, 5, 26, "Input",
 CellTags->"C0"],
Cell[384375, 15280, 260, 8, 26, "Output",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[384672, 15293, 203, 6, 26, "Input",
 CellTags->"C0"],
Cell[384878, 15301, 260, 8, 26, "Output",
 CellTags->"C0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[385175, 15314, 326, 10, 26, "Input",
 CellTags->"C0"],
Cell[385504, 15326, 260, 8, 26, "Output",
 CellTags->"C0"]
}, Open  ]]
}, Open  ]],
Cell[385791, 15338, 53, 1, 39, "ObjectName",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[385869, 15343, 59, 1, 36, "Subsubsection",
 CellTags->"DataType"],
Cell[385931, 15346, 344, 6, 41, "Text",
 CellTags->"DataType"],
Cell[386278, 15354, 1361, 44, 40, "Text",
 CellTags->"DataType"],
Cell[387642, 15400, 453, 17, 24, "Text",
 CellTags->"DataType"],
Cell[388098, 15419, 313, 11, 24, "Text",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[388448, 15435, 56, 1, 36, "Subsubsection",
 CellTags->"DataType"],
Cell[388507, 15438, 74, 1, 24, "Text",
 CellTags->"DataType"],
Cell[388584, 15441, 221, 7, 26, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[388830, 15452, 224, 8, 26, "Input",
 CellTags->"DataType"],
Cell[389057, 15462, 238, 9, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[389310, 15474, 179, 4, 24, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[389514, 15482, 112, 3, 26, "Input",
 CellTags->"DataType"],
Cell[389629, 15487, 210, 7, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[389854, 15497, 271, 9, 26, "Input",
 CellTags->"DataType"],
Cell[390128, 15508, 412, 14, 26, "Input",
 CellTags->"DataType"],
Cell[390543, 15524, 395, 14, 26, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[390963, 15542, 340, 13, 26, "Input",
 CellTags->"DataType"],
Cell[391306, 15557, 350, 13, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[391693, 15575, 107, 3, 26, "Input",
 CellTags->"DataType"],
Cell[391803, 15580, 281, 11, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[392121, 15596, 107, 3, 26, "Input",
 CellTags->"DataType"],
Cell[392231, 15601, 172, 6, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[392418, 15610, 191, 6, 26, "Input",
 CellTags->"DataType"],
Cell[392612, 15618, 187, 6, 26, "Input",
 CellTags->"DataType"],
Cell[392802, 15626, 429, 15, 26, "Input",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[393256, 15645, 290, 11, 26, "Input",
 CellTags->"DataType"],
Cell[393549, 15658, 299, 12, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[393885, 15675, 105, 3, 26, "Input",
 CellTags->"DataType"],
Cell[393993, 15680, 228, 9, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[394236, 15692, 151, 5, 26, "Input",
 CellTags->"DataType"],
Cell[394390, 15699, 222, 7, 26, "Input",
 CellTags->"DataType"],
Cell[394615, 15708, 272, 9, 26, "Input",
 CellTags->"DataType"],
Cell[394890, 15719, 114, 3, 24, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[395029, 15726, 149, 4, 26, "Input",
 CellTags->"DataType"],
Cell[395181, 15732, 107, 3, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[395303, 15738, 83, 1, 24, "Text",
 CellTags->"DataType"],
Cell[CellGroupData[{
Cell[395411, 15743, 233, 8, 26, "Input",
 CellTags->"DataType"],
Cell[395647, 15753, 104, 3, 26, "Output",
 CellTags->"DataType"]
}, Open  ]],
Cell[CellGroupData[{
Cell[395788, 15761, 226, 7, 26, "Input",
 CellTags->"DataType"],
Cell[396017, 15770, 435, 16, 28, "Output",
 CellTags->"DataType"]
}, Open  ]]
}, Open  ]],
Cell[396479, 15790, 43, 1, 39, "ObjectName",
 CellTags->"DB0"],
Cell[CellGroupData[{
Cell[396547, 15795, 54, 1, 36, "Subsubsection",
 CellTags->"DB0"],
Cell[396604, 15798, 146, 4, 24, "Text",
 CellTags->"DB0"],
Cell[396753, 15804, 232, 11, 24, "Text",
 CellTags->"DB0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[397022, 15820, 51, 1, 36, "Subsubsection",
 CellTags->"DB0"],
Cell[CellGroupData[{
Cell[397098, 15825, 339, 12, 29, "Input",
 CellTags->"DB0"],
Cell[397440, 15839, 255, 8, 29, "Output",
 CellTags->"DB0"]
}, Open  ]]
}, Open  ]],
Cell[397722, 15851, 43, 1, 39, "ObjectName",
 CellTags->"DB1"],
Cell[CellGroupData[{
Cell[397790, 15856, 54, 1, 36, "Subsubsection",
 CellTags->"DB1"],
Cell[397847, 15859, 117, 3, 24, "Text",
 CellTags->"DB1"],
Cell[397967, 15864, 232, 11, 24, "Text",
 CellTags->"DB1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[398236, 15880, 51, 1, 36, "Subsubsection",
 CellTags->"DB1"],
Cell[CellGroupData[{
Cell[398312, 15885, 339, 12, 29, "Input",
 CellTags->"DB1"],
Cell[398654, 15899, 1410, 45, 49, "Output",
 CellTags->"DB1"]
}, Open  ]]
}, Open  ]],
Cell[400091, 15948, 79, 1, 39, "ObjectName",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[400195, 15953, 72, 1, 36, "Subsubsection",
 CellTags->"DeclareNonCommutative"],
Cell[400270, 15956, 192, 4, 24, "Text",
 CellTags->"DeclareNonCommutative"],
Cell[400465, 15962, 526, 18, 24, "Text",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[401028, 15985, 69, 1, 36, "Subsubsection",
 CellTags->"DeclareNonCommutative"],
Cell[401100, 15988, 135, 3, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[401238, 15993, 152, 4, 24, "Text",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[401415, 16001, 158, 4, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[401576, 16007, 120, 3, 26, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[401711, 16013, 158, 4, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[401894, 16021, 191, 5, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[402088, 16028, 199, 6, 26, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]],
Cell[402302, 16037, 170, 4, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[CellGroupData[{
Cell[402497, 16045, 191, 5, 26, "Input",
 CellTags->"DeclareNonCommutative"],
Cell[402691, 16052, 199, 6, 26, "Output",
 CellTags->"DeclareNonCommutative"]
}, Open  ]]
}, Open  ]],
Cell[402917, 16062, 63, 1, 39, "ObjectName",
 CellTags->"DeltaFunction"],
Cell[CellGroupData[{
Cell[403005, 16067, 64, 1, 36, "Subsubsection",
 CellTags->"DeltaFunction"],
Cell[403072, 16070, 353, 11, 49, "Text",
 CellTags->"DeltaFunction"],
Cell[403428, 16083, 946, 32, 24, "Text",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[404411, 16120, 61, 1, 36, "Subsubsection",
 CellTags->"DeltaFunction"],
Cell[CellGroupData[{
Cell[404497, 16125, 142, 4, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[404642, 16131, 203, 7, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[404882, 16143, 320, 10, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[405205, 16155, 137, 4, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[405379, 16164, 294, 9, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[405676, 16175, 137, 4, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[405850, 16184, 320, 10, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[406173, 16196, 137, 4, 26, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[406347, 16205, 204, 6, 26, "Input",
 CellTags->"DeltaFunction"],
Cell[406554, 16213, 109, 3, 27, "Output",
 CellTags->"DeltaFunction"]
}, Open  ]]
}, Open  ]],
Cell[406690, 16220, 85, 1, 39, "ObjectName",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[CellGroupData[{
Cell[406800, 16225, 75, 1, 36, "Subsubsection",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[406878, 16228, 248, 8, 24, "Text",
 CellTags->"DeltaFunctionDoublePrime"],
Cell[407129, 16238, 311, 10, 24, "Text",
 CellTags->"DeltaFunctionDoublePrime"]
}, Open  ]],
Cell[407455, 16251, 73, 1, 39, "ObjectName",
 CellTags->"DeltaFunctionPrime"],
Cell[CellGroupData[{
Cell[407553, 16256, 69, 1, 36, "Subsubsection",
 CellTags->"DeltaFunctionPrime"],
Cell[407625, 16259, 227, 7, 24, "Text",
 CellTags->"DeltaFunctionPrime"],
Cell[407855, 16268, 925, 31, 24, "Text",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[408817, 16304, 66, 1, 36, "Subsubsection",
 CellTags->"DeltaFunctionPrime"],
Cell[CellGroupData[{
Cell[408908, 16309, 152, 4, 26, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[409063, 16315, 240, 8, 26, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[409340, 16328, 330, 10, 26, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[409673, 16340, 203, 6, 26, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]],
Cell[CellGroupData[{
Cell[409913, 16351, 326, 10, 29, "Input",
 CellTags->"DeltaFunctionPrime"],
Cell[410242, 16363, 114, 3, 26, "Output",
 CellTags->"DeltaFunctionPrime"]
}, Open  ]]
}, Open  ]],
Cell[410383, 16370, 69, 1, 39, "ObjectName",
 CellTags->"DenominatorOrder"],
Cell[CellGroupData[{
Cell[410477, 16375, 67, 1, 36, "Subsubsection",
 CellTags->"DenominatorOrder"],
Cell[410547, 16378, 175, 4, 24, "Text",
 CellTags->"DenominatorOrder"],
Cell[410725, 16384, 503, 17, 24, "Text",
 CellTags->"DenominatorOrder"]
}, Open  ]],
Cell[411243, 16404, 55, 1, 39, "ObjectName",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[411323, 16409, 60, 1, 36, "Subsubsection",
 CellTags->"Dimension"],
Cell[411386, 16412, 246, 5, 41, "Text",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[411657, 16421, 120, 3, 26, "Input",
 CellTags->"Dimension"],
Cell[411780, 16426, 244, 8, 26, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[412061, 16439, 313, 9, 26, "Input",
 CellTags->"Dimension"],
Cell[412377, 16450, 286, 11, 26, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[412678, 16464, 131, 4, 24, "Text",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[412834, 16472, 357, 11, 26, "Input",
 CellTags->"Dimension"],
Cell[413194, 16485, 360, 10, 38, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[413569, 16498, 166, 4, 24, "Text",
 CellTags->"Dimension"],
Cell[413738, 16504, 133, 4, 26, "Input",
 CellTags->"Dimension"],
Cell[CellGroupData[{
Cell[413896, 16512, 326, 9, 26, "Input",
 CellTags->"Dimension"],
Cell[414225, 16523, 662, 28, 26, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[CellGroupData[{
Cell[414924, 16556, 110, 3, 26, "Input",
 CellTags->"Dimension"],
Cell[415037, 16561, 373, 10, 38, "Output",
 CellTags->"Dimension"]
}, Open  ]],
Cell[415425, 16574, 134, 4, 26, "Input",
 CellTags->"Dimension"]
}, Open  ]],
Cell[415574, 16581, 77, 1, 39, "ObjectName",
 CellTags->"DimensionalReduction"],
Cell[CellGroupData[{
Cell[415676, 16586, 71, 1, 36, "Subsubsection",
 CellTags->"DimensionalReduction"],
Cell[415750, 16589, 121, 3, 24, "Text",
 CellTags->"DimensionalReduction"],
Cell[415874, 16594, 467, 17, 24, "Text",
 CellTags->"DimensionalReduction"]
}, Open  ]],
Cell[416356, 16614, 57, 1, 39, "ObjectName",
 CellTags->"DiracBasis"],
Cell[CellGroupData[{
Cell[416438, 16619, 61, 1, 36, "Subsubsection",
 CellTags->"DiracBasis"],
Cell[416502, 16622, 299, 6, 41, "Text",
 CellTags->"DiracBasis"],
Cell[416804, 16630, 269, 10, 24, "Text",
 CellTags->"DiracBasis"]
}, Open  ]],
Cell[417088, 16643, 65, 1, 39, "ObjectName",
 CellTags->"DiracCanonical"],
Cell[CellGroupData[{
Cell[417178, 16648, 65, 1, 36, "Subsubsection",
 CellTags->"DiracCanonical"],
Cell[417246, 16651, 171, 4, 24, "Text",
 CellTags->"DiracCanonical"],
Cell[417420, 16657, 281, 10, 24, "Text",
 CellTags->"DiracCanonical"]
}, Open  ]],
Cell[417716, 16670, 63, 1, 39, "ObjectName",
 CellTags->"DiracEquation"],
Cell[CellGroupData[{
Cell[417804, 16675, 64, 1, 36, "Subsubsection",
 CellTags->"DiracEquation"],
Cell[417871, 16678, 160, 4, 24, "Text",
 CellTags->"DiracEquation"],
Cell[418034, 16684, 286, 11, 24, "Text",
 CellTags->"DiracEquation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[418357, 16700, 61, 1, 36, "Subsubsection",
 CellTags->"DiracEquation"],
Cell[CellGroupData[{
Cell[418443, 16705, 279, 8, 26, "Input",
 CellTags->"DiracEquation"],
Cell[418725, 16715, 368, 13, 28, "Output",
 CellTags->"DiracEquation"]
}, Open  ]],
Cell[CellGroupData[{
Cell[419130, 16733, 119, 3, 26, "Input",
 CellTags->"DiracEquation"],
Cell[419252, 16738, 217, 7, 28, "Output",
 CellTags->"DiracEquation"]
}, Open  ]]
}, Open  ]],
Cell[419496, 16749, 57, 1, 39, "ObjectName",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[419578, 16754, 61, 1, 36, "Subsubsection",
 CellTags->"DiracGamma"],
Cell[419642, 16757, 762, 24, 58, "Text",
 CellTags->"DiracGamma"],
Cell[420407, 16783, 1111, 39, 24, "Text",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[421555, 16827, 58, 1, 36, "Subsubsection",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[421638, 16832, 113, 3, 26, "Input",
 CellTags->"DiracGamma"],
Cell[421754, 16837, 137, 4, 26, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[421928, 16846, 159, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[422090, 16852, 214, 8, 26, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[422319, 16863, 332, 13, 27, "Text",
 CellTags->"DiracGamma"],
Cell[CellGroupData[{
Cell[422676, 16880, 167, 5, 26, "Input",
 CellTags->"DiracGamma"],
Cell[422846, 16887, 223, 8, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[423106, 16900, 280, 8, 26, "Input",
 CellTags->"DiracGamma"],
Cell[423389, 16910, 468, 17, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[423894, 16932, 218, 7, 26, "Input",
 CellTags->"DiracGamma"],
Cell[424115, 16941, 223, 8, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[424375, 16954, 208, 7, 26, "Input",
 CellTags->"DiracGamma"],
Cell[424586, 16963, 341, 12, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[424964, 16980, 144, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[425111, 16986, 593, 22, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[425741, 17013, 300, 9, 26, "Input",
 CellTags->"DiracGamma"],
Cell[426044, 17024, 430, 13, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[426511, 17042, 138, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[426652, 17048, 767, 28, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[427456, 17081, 141, 4, 26, "Input",
 CellTags->"DiracGamma"],
Cell[427600, 17087, 1123, 42, 28, "Output",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[428738, 17132, 168, 5, 26, "Input",
 CellTags->"DiracGamma"]
}, Open  ]],
Cell[428921, 17140, 71, 1, 39, "ObjectName",
 CellTags->"DiracGammaCombine"],
Cell[CellGroupData[{
Cell[429017, 17145, 68, 1, 36, "Subsubsection",
 CellTags->"DiracGammaCombine"],
Cell[429088, 17148, 135, 4, 24, "Text",
 CellTags->"DiracGammaCombine"],
Cell[429226, 17154, 1318, 46, 24, "Text",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[430581, 17205, 65, 1, 36, "Subsubsection",
 CellTags->"DiracGammaCombine"],
Cell[CellGroupData[{
Cell[430671, 17210, 220, 6, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[430894, 17218, 265, 9, 28, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[431196, 17232, 122, 3, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[431321, 17237, 194, 5, 38, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[431552, 17247, 270, 8, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[431825, 17257, 317, 11, 28, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[432179, 17273, 122, 3, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[432304, 17278, 244, 7, 38, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[432585, 17290, 127, 3, 26, "Input",
 CellTags->"DiracGammaCombine"],
Cell[432715, 17295, 438, 16, 28, "Output",
 CellTags->"DiracGammaCombine"]
}, Open  ]]
}, Open  ]],
Cell[433180, 17315, 69, 1, 39, "ObjectName",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[433274, 17320, 67, 1, 36, "Subsubsection",
 CellTags->"DiracGammaExpand"],
Cell[433344, 17323, 190, 4, 41, "Text",
 CellTags->"DiracGammaExpand"],
Cell[433537, 17329, 1321, 46, 24, "Text",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[434895, 17380, 64, 1, 36, "Subsubsection",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[434984, 17385, 314, 9, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[435301, 17396, 474, 17, 28, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[435790, 17416, 125, 3, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[435940, 17423, 121, 3, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[436064, 17428, 291, 8, 38, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[436370, 17439, 101, 1, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[436496, 17444, 125, 3, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[436624, 17449, 599, 22, 28, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[437238, 17474, 90, 1, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[437353, 17479, 209, 5, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[437565, 17486, 291, 8, 38, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[437893, 17499, 164, 4, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[438060, 17505, 394, 11, 38, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[438469, 17519, 106, 1, 24, "Text",
 CellTags->"DiracGammaExpand"],
Cell[CellGroupData[{
Cell[438600, 17524, 122, 3, 26, "Input",
 CellTags->"DiracGammaExpand"],
Cell[438725, 17529, 534, 20, 28, "Output",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[439274, 17552, 114, 3, 26, "Input",
 CellTags->"DiracGammaExpand"]
}, Open  ]],
Cell[439403, 17558, 59, 1, 39, "ObjectName",
 CellTags->"DiracGammaT"],
Cell[CellGroupData[{
Cell[439487, 17563, 62, 1, 36, "Subsubsection",
 CellTags->"DiracGammaT"],
Cell[439552, 17566, 208, 5, 41, "Text",
 CellTags->"DiracGammaT"],
Cell[439763, 17573, 272, 11, 24, "Text",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[440072, 17589, 59, 1, 36, "Subsubsection",
 CellTags->"DiracGammaT"],
Cell[CellGroupData[{
Cell[440156, 17594, 158, 4, 26, "Input",
 CellTags->"DiracGammaT"],
Cell[440317, 17600, 220, 8, 30, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[440574, 17613, 113, 3, 26, "Input",
 CellTags->"DiracGammaT"],
Cell[440690, 17618, 212, 8, 26, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[440939, 17631, 167, 6, 26, "Input",
 CellTags->"DiracGammaT"],
Cell[441109, 17639, 219, 7, 28, "Output",
 CellTags->"DiracGammaT"]
}, Open  ]]
}, Open  ]],
Cell[441355, 17650, 59, 1, 39, "ObjectName",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[441439, 17655, 62, 1, 36, "Subsubsection",
 CellTags->"DiracMatrix"],
Cell[441504, 17658, 1220, 46, 41, "Text",
 CellTags->"DiracMatrix"],
Cell[442727, 17706, 1788, 67, 24, "Text",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[444552, 17778, 59, 1, 36, "Subsubsection",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[444636, 17783, 119, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[444758, 17788, 212, 8, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[444985, 17799, 539, 20, 24, "Text",
 CellTags->"DiracMatrix"],
Cell[CellGroupData[{
Cell[445549, 17823, 184, 5, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[445736, 17830, 350, 14, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[446123, 17849, 160, 5, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[446286, 17856, 175, 4, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[446498, 17865, 145, 4, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[446646, 17871, 142, 4, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[446825, 17880, 112, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[446940, 17885, 125, 3, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447102, 17893, 111, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[447216, 17898, 142, 4, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447395, 17907, 138, 4, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[447536, 17913, 221, 6, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[447794, 17924, 152, 4, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[447949, 17930, 249, 7, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[448235, 17942, 166, 5, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[448404, 17949, 342, 14, 26, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]],
Cell[CellGroupData[{
Cell[448783, 17968, 112, 3, 26, "Input",
 CellTags->"DiracMatrix"],
Cell[448898, 17973, 269, 7, 38, "Output",
 CellTags->"DiracMatrix"]
}, Open  ]]
}, Open  ]],
Cell[449194, 17984, 57, 1, 39, "ObjectName",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[449276, 17989, 61, 1, 36, "Subsubsection",
 CellTags->"DiracOrder"],
Cell[449340, 17992, 204, 5, 41, "Text",
 CellTags->"DiracOrder"],
Cell[449547, 17999, 483, 18, 24, "Text",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[450067, 18022, 58, 1, 36, "Subsubsection",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[450150, 18027, 176, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[450329, 18034, 206, 6, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[450572, 18045, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[450689, 18050, 472, 18, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[451176, 18071, 93, 1, 24, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[451294, 18076, 176, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[451473, 18083, 247, 7, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[451757, 18095, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[451874, 18100, 472, 18, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[452383, 18123, 171, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[452557, 18130, 243, 7, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[452815, 18140, 196, 8, 24, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[453036, 18152, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[453153, 18157, 391, 15, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[453581, 18177, 167, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[453751, 18184, 239, 7, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[454027, 18196, 114, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[454144, 18201, 271, 10, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[454452, 18216, 183, 5, 26, "Input",
 CellTags->"DiracOrder"],
Cell[454638, 18223, 255, 7, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[454908, 18233, 288, 10, 24, "Text",
 CellTags->"DiracOrder"],
Cell[CellGroupData[{
Cell[455221, 18247, 219, 7, 26, "Input",
 CellTags->"DiracOrder"],
Cell[455443, 18256, 1207, 45, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[456687, 18306, 113, 3, 26, "Input",
 CellTags->"DiracOrder"],
Cell[456803, 18311, 477, 19, 26, "Output",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[457295, 18333, 188, 6, 26, "Input",
 CellTags->"DiracOrder"]
}, Open  ]],
Cell[457498, 18342, 59, 1, 39, "ObjectName",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[457582, 18347, 62, 1, 36, "Subsubsection",
 CellTags->"DiracReduce"],
Cell[457647, 18350, 662, 11, 92, "Text",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[458334, 18365, 121, 3, 26, "Input",
 CellTags->"DiracReduce"],
Cell[458458, 18370, 323, 10, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[458796, 18383, 284, 11, 22, "Text",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[459117, 18399, 59, 1, 36, "Subsubsection",
 CellTags->"DiracReduce"],
Cell[CellGroupData[{
Cell[459201, 18404, 172, 5, 26, "Input",
 CellTags->"DiracReduce"],
Cell[459376, 18411, 202, 6, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[459615, 18422, 116, 3, 26, "Input",
 CellTags->"DiracReduce"],
Cell[459734, 18427, 281, 9, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[460052, 18441, 186, 5, 26, "Input",
 CellTags->"DiracReduce"],
Cell[460241, 18448, 239, 7, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[460517, 18460, 116, 3, 26, "Input",
 CellTags->"DiracReduce"],
Cell[460636, 18465, 919, 28, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[461592, 18498, 207, 6, 26, "Input",
 CellTags->"DiracReduce"],
Cell[461802, 18506, 283, 8, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[462122, 18519, 140, 4, 26, "Input",
 CellTags->"DiracReduce"],
Cell[462265, 18525, 1814, 54, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[464116, 18584, 253, 7, 26, "Input",
 CellTags->"DiracReduce"],
Cell[464372, 18593, 127, 4, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[464536, 18602, 133, 4, 26, "Input",
 CellTags->"DiracReduce"],
Cell[464672, 18608, 127, 4, 26, "Output",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[464814, 18615, 156, 4, 26, "Input",
 CellTags->"DiracReduce"]
}, Open  ]],
Cell[464985, 18622, 57, 1, 39, "ObjectName",
 CellTags->"DiracSigma"],
Cell[CellGroupData[{
Cell[465067, 18627, 61, 1, 36, "Subsubsection",
 CellTags->"DiracSigma"],
Cell[465131, 18630, 213, 5, 41, "Text",
 CellTags->"DiracSigma"],
Cell[465347, 18637, 303, 11, 24, "Text",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[465687, 18653, 58, 1, 36, "Subsubsection",
 CellTags->"DiracSigma"],
Cell[CellGroupData[{
Cell[465770, 18658, 236, 7, 26, "Input",
 CellTags->"DiracSigma"],
Cell[466009, 18667, 169, 5, 26, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[466215, 18677, 122, 3, 26, "Input",
 CellTags->"DiracSigma"],
Cell[466340, 18682, 784, 29, 42, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[467161, 18716, 236, 7, 26, "Input",
 CellTags->"DiracSigma"],
Cell[467400, 18725, 189, 6, 26, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[467626, 18736, 223, 7, 26, "Input",
 CellTags->"DiracSigma"],
Cell[467852, 18745, 156, 5, 26, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[CellGroupData[{
Cell[468045, 18755, 122, 3, 26, "Input",
 CellTags->"DiracSigma"],
Cell[468170, 18760, 950, 33, 42, "Output",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[469135, 18796, 144, 4, 26, "Input",
 CellTags->"DiracSigma"]
}, Open  ]],
Cell[469294, 18803, 73, 1, 39, "ObjectName",
 CellTags->"DiracSigmaExplicit"],
Cell[CellGroupData[{
Cell[469392, 18808, 69, 1, 36, "Subsubsection",
 CellTags->"DiracSigmaExplicit"],
Cell[469464, 18811, 186, 4, 24, "Text",
 CellTags->"DiracSigmaExplicit"],
Cell[469653, 18817, 279, 11, 24, "Text",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[469969, 18833, 66, 1, 36, "Subsubsection",
 CellTags->"DiracSigmaExplicit"],
Cell[CellGroupData[{
Cell[470060, 18838, 217, 6, 26, "Input",
 CellTags->"DiracSigmaExplicit"],
Cell[470280, 18846, 177, 5, 26, "Output",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[470494, 18856, 129, 3, 26, "Input",
 CellTags->"DiracSigmaExplicit"],
Cell[470626, 18861, 792, 29, 42, "Output",
 CellTags->"DiracSigmaExplicit"]
}, Open  ]]
}, Open  ]],
Cell[471445, 18894, 69, 1, 39, "ObjectName",
 CellTags->"DiracSimpCombine"],
Cell[CellGroupData[{
Cell[471539, 18899, 67, 1, 36, "Subsubsection",
 CellTags->"DiracSimpCombine"],
Cell[471609, 18902, 209, 5, 22, "Text",
 CellTags->"DiracSimpCombine"],
Cell[471821, 18909, 283, 10, 22, "Text",
 CellTags->"DiracSimpCombine"]
}, Open  ]],
Cell[472119, 18922, 63, 1, 39, "ObjectName",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[472207, 18927, 64, 1, 28, "Subsubsection",
 CellTags->"DiracSimplify"],
Cell[472274, 18930, 382, 7, 58, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[472681, 18941, 125, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[472809, 18946, 604, 14, 47, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[473428, 18963, 674, 25, 22, "Text",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[474139, 18993, 61, 1, 34, "Subsubsection",
 CellTags->"DiracSimplify"],
Cell[474203, 18996, 99, 1, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[474327, 19001, 178, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[474508, 19008, 250, 7, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[474795, 19020, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[474918, 19025, 261, 10, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[475194, 19038, 96, 1, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[475315, 19043, 179, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[475497, 19050, 250, 7, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[475784, 19062, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[475907, 19067, 261, 10, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[476205, 19082, 174, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[476382, 19089, 246, 7, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[476643, 19099, 179, 7, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[476847, 19110, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[476970, 19115, 394, 15, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[477401, 19135, 170, 5, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[477574, 19142, 242, 7, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[477853, 19154, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[477976, 19159, 274, 10, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[478287, 19174, 326, 10, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[478616, 19186, 570, 18, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[479201, 19207, 120, 3, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[479346, 19214, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[479469, 19219, 1908, 72, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[481414, 19296, 117, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[481534, 19301, 607, 22, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[482178, 19328, 272, 7, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[482453, 19337, 407, 14, 28, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[482897, 19356, 120, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[483020, 19361, 109, 3, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[483166, 19369, 373, 12, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[483542, 19383, 624, 18, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[484203, 19406, 119, 3, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[484325, 19411, 13964, 531, 87, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[498304, 19945, 115, 3, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[498444, 19952, 251, 7, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[498698, 19961, 465, 18, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[499178, 19982, 141, 4, 24, "Text",
 CellTags->"DiracSimplify"],
Cell[CellGroupData[{
Cell[499344, 19990, 289, 8, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[499636, 20000, 556, 19, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[500229, 20024, 268, 7, 26, "Input",
 CellTags->"DiracSimplify"],
Cell[500500, 20033, 109, 3, 26, "Output",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[500624, 20039, 183, 5, 26, "Input",
 CellTags->"DiracSimplify"]
}, Open  ]],
Cell[500822, 20047, 65, 1, 39, "ObjectName",
 CellTags->"DiracSimplify2"],
Cell[CellGroupData[{
Cell[500912, 20052, 65, 1, 36, "Subsubsection",
 CellTags->"DiracSimplify2"],
Cell[500980, 20055, 224, 7, 24, "Text",
 CellTags->"DiracSimplify2"],
Cell[501207, 20064, 287, 11, 24, "Text",
 CellTags->"DiracSimplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[501531, 20080, 62, 1, 36, "Subsubsection",
 CellTags->"DiracSimplify2"],
Cell[CellGroupData[{
Cell[501618, 20085, 222, 6, 26, "Input",
 CellTags->"DiracSimplify2"],
Cell[501843, 20093, 439, 11, 26, "Output",
 CellTags->"DiracSimplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[502319, 20109, 121, 3, 26, "Input",
 CellTags->"DiracSimplify2"],
Cell[502443, 20114, 883, 35, 26, "Output",
 CellTags->"DiracSimplify2"]
}, Open  ]]
}, Open  ]],
Cell[503353, 20153, 57, 1, 39, "ObjectName",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[503435, 20158, 61, 1, 36, "Subsubsection",
 CellTags->"DiracSlash"],
Cell[503499, 20161, 469, 16, 27, "Text",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[503993, 20181, 119, 3, 26, "Input",
 CellTags->"DiracSlash"],
Cell[504115, 20186, 245, 8, 26, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[504375, 20197, 1451, 53, 24, "Text",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[505863, 20255, 58, 1, 36, "Subsubsection",
 CellTags->"DiracSlash"],
Cell[505924, 20258, 234, 9, 27, "Text",
 CellTags->"DiracSlash"],
Cell[CellGroupData[{
Cell[506183, 20271, 113, 3, 26, "Input",
 CellTags->"DiracSlash"],
Cell[506299, 20276, 223, 8, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[506559, 20289, 173, 5, 26, "Input",
 CellTags->"DiracSlash"],
Cell[506735, 20296, 431, 16, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[507203, 20317, 136, 4, 26, "Input",
 CellTags->"DiracSlash"],
Cell[507342, 20323, 431, 16, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[507810, 20344, 105, 3, 26, "Input",
 CellTags->"DiracSlash"],
Cell[507918, 20349, 147, 4, 28, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[508102, 20358, 148, 4, 26, "Input",
 CellTags->"DiracSlash"],
Cell[508253, 20364, 128, 3, 38, "Output",
 CellTags->"DiracSlash"]
}, Open  ]],
Cell[CellGroupData[{
Cell[508418, 20372, 214, 7, 26, "Input",
 CellTags->"DiracSlash"],
Cell[508635, 20381, 189, 5, 38, "Output",
 CellTags->"DiracSlash"]
}, Open  ]]
}, Open  ]],
Cell[508851, 20390, 59, 1, 39, "ObjectName",
 CellTags->"DiracSpinor"],
Cell[CellGroupData[{
Cell[508935, 20395, 62, 1, 36, "Subsubsection",
 CellTags->"DiracSpinor"],
Cell[509000, 20398, 288, 6, 41, "Text",
 CellTags->"DiracSpinor"],
Cell[509291, 20406, 250, 10, 24, "Text",
 CellTags->"DiracSpinor"]
}, Open  ]],
Cell[509556, 20419, 71, 1, 39, "ObjectName",
 CellTags->"DiracSubstitute67"],
Cell[CellGroupData[{
Cell[509652, 20424, 68, 1, 36, "Subsubsection",
 CellTags->"DiracSubstitute67"],
Cell[509723, 20427, 219, 5, 41, "Text",
 CellTags->"DiracSubstitute67"],
Cell[509945, 20434, 484, 17, 24, "Text",
 CellTags->"DiracSubstitute67"]
}, Open  ]],
Cell[510444, 20454, 57, 1, 39, "ObjectName",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[510526, 20459, 61, 1, 36, "Subsubsection",
 CellTags->"DiracTrace"],
Cell[510590, 20462, 336, 6, 41, "Text",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[510951, 20472, 119, 3, 26, "Input",
 CellTags->"DiracTrace"],
Cell[511073, 20477, 668, 16, 47, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[511756, 20496, 359, 14, 24, "Text",
 CellTags->"DiracTrace"],
Cell[512118, 20512, 239, 11, 24, "Text",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[512394, 20528, 58, 1, 36, "Subsubsection",
 CellTags->"DiracTrace"],
Cell[CellGroupData[{
Cell[512477, 20533, 174, 5, 26, "Input",
 CellTags->"DiracTrace"],
Cell[512654, 20540, 270, 9, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[512961, 20554, 209, 6, 26, "Input",
 CellTags->"DiracTrace"],
Cell[513173, 20562, 370, 11, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[513580, 20578, 149, 4, 26, "Input",
 CellTags->"DiracTrace"],
Cell[513732, 20584, 635, 22, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[514404, 20611, 296, 8, 26, "Input",
 CellTags->"DiracTrace"],
Cell[514703, 20621, 566, 23, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[515306, 20649, 334, 9, 26, "Input",
 CellTags->"DiracTrace"],
Cell[515643, 20660, 8415, 321, 52, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[524095, 20986, 186, 5, 26, "Input",
 CellTags->"DiracTrace"],
Cell[524284, 20993, 498, 15, 28, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[524819, 21013, 294, 8, 26, "Input",
 CellTags->"DiracTrace"],
Cell[525116, 21023, 182, 6, 26, "Output",
 CellTags->"DiracTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[525335, 21034, 111, 3, 26, "Input",
 CellTags->"DiracTrace"],
Cell[525449, 21039, 175, 5, 38, "Output",
 CellTags->"DiracTrace"]
}, Open  ]]
}, Open  ]],
Cell[525651, 21048, 73, 1, 39, "ObjectName",
 CellTags->"DiracTraceEvaluate"],
Cell[CellGroupData[{
Cell[525749, 21053, 69, 1, 36, "Subsubsection",
 CellTags->"DiracTraceEvaluate"],
Cell[525821, 21056, 162, 4, 24, "Text",
 CellTags->"DiracTraceEvaluate"],
Cell[525986, 21062, 441, 17, 24, "Text",
 CellTags->"DiracTraceEvaluate"]
}, Open  ]],
Cell[526442, 21082, 57, 1, 39, "ObjectName",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[526524, 21087, 61, 1, 36, "Subsubsection",
 CellTags->"DiracTrick"],
Cell[526588, 21090, 222, 5, 41, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[526835, 21099, 119, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[526957, 21104, 174, 6, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[527146, 21113, 683, 25, 24, "Text",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[527866, 21143, 58, 1, 36, "Subsubsection",
 CellTags->"DiracTrick"],
Cell[527927, 21146, 96, 1, 24, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[528048, 21151, 175, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[528226, 21158, 247, 7, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[528510, 21170, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[528627, 21175, 258, 10, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[528900, 21188, 93, 1, 24, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[529018, 21193, 176, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[529197, 21200, 247, 7, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[529481, 21212, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[529598, 21217, 288, 11, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[529923, 21233, 171, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[530097, 21240, 243, 7, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[530355, 21250, 176, 7, 24, "Text",
 CellTags->"DiracTrick"],
Cell[CellGroupData[{
Cell[530556, 21261, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[530673, 21266, 391, 15, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[531101, 21286, 167, 5, 26, "Input",
 CellTags->"DiracTrick"],
Cell[531271, 21293, 239, 7, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[531547, 21305, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[531664, 21310, 271, 10, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[531972, 21325, 323, 10, 26, "Input",
 CellTags->"DiracTrick"],
Cell[532298, 21337, 567, 18, 28, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[532902, 21360, 114, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[533019, 21365, 604, 22, 28, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[533660, 21392, 108, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[533771, 21397, 1905, 72, 28, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[535713, 21474, 370, 12, 26, "Input",
 CellTags->"DiracTrick"],
Cell[536086, 21488, 621, 18, 26, "Output",
 CellTags->"DiracTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[536744, 21511, 113, 3, 26, "Input",
 CellTags->"DiracTrick"],
Cell[536860, 21516, 14288, 499, 87, "Output",
 CellTags->"DiracTrick"]
}, Open  ]]
}, Open  ]],
Cell[551175, 22019, 55, 1, 39, "ObjectName",
 CellTags->"Divideout"],
Cell[CellGroupData[{
Cell[551255, 22024, 60, 1, 36, "Subsubsection",
 CellTags->"Divideout"],
Cell[551318, 22027, 137, 4, 24, "Text",
 CellTags->"Divideout"],
Cell[551458, 22033, 444, 17, 24, "Text",
 CellTags->"Divideout"]
}, Open  ]],
Cell[551917, 22053, 43, 1, 39, "ObjectName",
 CellTags->"DOT"],
Cell[CellGroupData[{
Cell[551985, 22058, 54, 1, 36, "Subsubsection",
 CellTags->"DOT"],
Cell[552042, 22061, 348, 8, 91, "Text",
 CellTags->"DOT"],
Cell[552393, 22071, 262, 10, 24, "Text",
 CellTags->"DOT"]
}, Open  ]],
Cell[552670, 22084, 55, 1, 39, "ObjectName",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[552750, 22089, 60, 1, 36, "Subsubsection",
 CellTags->"DotExpand"],
Cell[552813, 22092, 85, 1, 24, "Text",
 CellTags->"DotExpand"],
Cell[552901, 22095, 936, 31, 24, "Text",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[553874, 22131, 57, 1, 36, "Subsubsection",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[553956, 22136, 313, 10, 26, "Input",
 CellTags->"DotExpand"],
Cell[554272, 22148, 504, 13, 26, "Output",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[554791, 22164, 229, 7, 26, "Input",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[555045, 22175, 313, 10, 26, "Input",
 CellTags->"DotExpand"],
Cell[555361, 22187, 639, 22, 26, "Output",
 CellTags->"DotExpand"]
}, Open  ]],
Cell[556015, 22212, 231, 7, 26, "Input",
 CellTags->"DotExpand"],
Cell[CellGroupData[{
Cell[556271, 22223, 313, 10, 26, "Input",
 CellTags->"DotExpand"],
Cell[556587, 22235, 504, 13, 26, "Output",
 CellTags->"DotExpand"]
}, Open  ]]
}, Open  ]],
Cell[557118, 22252, 53, 1, 39, "ObjectName",
 CellTags->"DotPower"],
Cell[CellGroupData[{
Cell[557196, 22257, 59, 1, 36, "Subsubsection",
 CellTags->"DotPower"],
Cell[557258, 22260, 189, 4, 24, "Text",
 CellTags->"DotPower"],
Cell[557450, 22266, 267, 10, 24, "Text",
 CellTags->"DotPower"]
}, Open  ]],
Cell[557732, 22279, 57, 1, 39, "ObjectName",
 CellTags->"DotProduct"],
Cell[CellGroupData[{
Cell[557814, 22284, 61, 1, 36, "Subsubsection",
 CellTags->"DotProduct"],
Cell[557878, 22287, 219, 5, 41, "Text",
 CellTags->"DotProduct"],
Cell[558100, 22294, 483, 18, 24, "Text",
 CellTags->"DotProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[558620, 22317, 58, 1, 36, "Subsubsection",
 CellTags->"DotProduct"],
Cell[CellGroupData[{
Cell[558703, 22322, 234, 7, 26, "Input",
 CellTags->"DotProduct"],
Cell[558940, 22331, 314, 11, 29, "Output",
 CellTags->"DotProduct"]
}, Open  ]]
}, Open  ]],
Cell[559281, 22346, 77, 1, 39, "ObjectName",
 CellTags->"DotSimplifyRelations"],
Cell[CellGroupData[{
Cell[559383, 22351, 71, 1, 36, "Subsubsection",
 CellTags->"DotSimplifyRelations"],
Cell[559457, 22354, 241, 5, 41, "Text",
 CellTags->"DotSimplifyRelations"],
Cell[559701, 22361, 279, 10, 24, "Text",
 CellTags->"DotSimplifyRelations"]
}, Open  ]],
Cell[559995, 22374, 59, 1, 39, "ObjectName",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[560079, 22379, 62, 1, 36, "Subsubsection",
 CellTags->"DotSimplify"],
Cell[560144, 22382, 322, 6, 58, "Text",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[560491, 22392, 121, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[560615, 22397, 381, 11, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[561011, 22411, 664, 25, 24, "Text",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[561712, 22441, 59, 1, 36, "Subsubsection",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[561796, 22446, 342, 11, 26, "Input",
 CellTags->"DotSimplify"],
Cell[562141, 22459, 376, 11, 28, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[562554, 22475, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[562673, 22480, 1013, 39, 28, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[563701, 22522, 158, 4, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[563884, 22530, 209, 7, 26, "Input",
 CellTags->"DotSimplify"],
Cell[564096, 22539, 217, 8, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[564350, 22552, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[564469, 22557, 223, 7, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[564729, 22569, 161, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[564893, 22576, 107, 3, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[565037, 22584, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[565156, 22589, 279, 10, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[565450, 22602, 161, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[565636, 22611, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[565755, 22616, 223, 7, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[566015, 22628, 165, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[566183, 22635, 107, 3, 27, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[566327, 22643, 116, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[566446, 22648, 277, 9, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[566738, 22660, 165, 5, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[566928, 22669, 299, 9, 26, "Input",
 CellTags->"DotSimplify"],
Cell[567230, 22680, 166, 6, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[567411, 22689, 125, 3, 26, "Input",
 CellTags->"DotSimplify"],
Cell[CellGroupData[{
Cell[567561, 22696, 148, 4, 26, "Input",
 CellTags->"DotSimplify"],
Cell[567712, 22702, 140, 4, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[567889, 22711, 213, 6, 26, "Input",
 CellTags->"DotSimplify"],
Cell[568105, 22719, 140, 4, 26, "Output",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[568260, 22726, 170, 4, 26, "Input",
 CellTags->"DotSimplify"]
}, Open  ]],
Cell[568445, 22733, 57, 1, 39, "ObjectName",
 CellTags->"DummyIndex"],
Cell[CellGroupData[{
Cell[568527, 22738, 61, 1, 36, "Subsubsection",
 CellTags->"DummyIndex"],
Cell[568591, 22741, 191, 4, 24, "Text",
 CellTags->"DummyIndex"],
Cell[568785, 22747, 265, 10, 24, "Text",
 CellTags->"DummyIndex"]
}, Open  ]],
Cell[569065, 22760, 41, 1, 39, "ObjectName",
 CellTags->"D0"],
Cell[CellGroupData[{
Cell[569131, 22765, 53, 1, 36, "Subsubsection",
 CellTags->"D0"],
Cell[569187, 22768, 524, 12, 58, "Text",
 CellTags->"D0"],
Cell[569714, 22782, 771, 32, 24, "Text",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[570522, 22819, 50, 1, 36, "Subsubsection",
 CellTags->"D0"],
Cell[CellGroupData[{
Cell[570597, 22824, 315, 9, 26, "Input",
 CellTags->"D0"],
Cell[570915, 22835, 550, 20, 29, "Output",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[571502, 22860, 484, 14, 26, "Input",
 CellTags->"D0"],
Cell[571989, 22876, 550, 20, 29, "Output",
 CellTags->"D0"]
}, Open  ]],
Cell[CellGroupData[{
Cell[572576, 22901, 104, 3, 26, "Input",
 CellTags->"D0"],
Cell[572683, 22906, 550, 20, 29, "Output",
 CellTags->"D0"]
}, Open  ]]
}, Open  ]],
Cell[573260, 22930, 61, 1, 39, "ObjectName",
 CellTags->"D0Convention"],
Cell[CellGroupData[{
Cell[573346, 22935, 63, 1, 36, "Subsubsection",
 CellTags->"D0Convention"],
Cell[573412, 22938, 307, 6, 41, "Text",
 CellTags->"D0Convention"],
Cell[573722, 22946, 419, 17, 24, "Text",
 CellTags->"D0Convention"]
}, Open  ]],
Cell[574156, 22966, 43, 1, 39, "ObjectName",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[574224, 22971, 54, 1, 36, "Subsubsection",
 CellTags->"Eps"],
Cell[574281, 22974, 345, 8, 41, "Text",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[574651, 22986, 105, 3, 26, "Input",
 CellTags->"Eps"],
Cell[574759, 22991, 160, 5, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[574934, 22999, 808, 32, 24, "Text",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[575779, 23036, 51, 1, 36, "Subsubsection",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[575855, 23041, 322, 8, 26, "Input",
 CellTags->"Eps"],
Cell[576180, 23051, 477, 21, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[576694, 23077, 314, 8, 26, "Input",
 CellTags->"Eps"],
Cell[577011, 23087, 435, 19, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[577483, 23111, 181, 6, 26, "Input",
 CellTags->"Eps"],
Cell[577667, 23119, 176, 5, 38, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[577880, 23129, 142, 4, 26, "Input",
 CellTags->"Eps"],
Cell[578025, 23135, 99, 3, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[578161, 23143, 142, 4, 26, "Input",
 CellTags->"Eps"],
Cell[578306, 23149, 117, 4, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[578438, 23156, 189, 6, 26, "Input",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[578652, 23166, 460, 13, 26, "Input",
 CellTags->"Eps"],
Cell[579115, 23181, 477, 21, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[579629, 23207, 107, 3, 26, "Input",
 CellTags->"Eps"],
Cell[579739, 23212, 102, 3, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[579856, 23218, 189, 6, 26, "Input",
 CellTags->"Eps"],
Cell[CellGroupData[{
Cell[580070, 23228, 460, 13, 26, "Input",
 CellTags->"Eps"],
Cell[580533, 23243, 477, 21, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[581047, 23269, 137, 4, 26, "Input",
 CellTags->"Eps"],
Cell[581187, 23275, 102, 3, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[581326, 23283, 293, 9, 26, "Input"],
Cell[581622, 23294, 366, 9, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[582025, 23308, 77, 2, 26, "Input"],
Cell[582105, 23312, 974, 41, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[583116, 23358, 433, 14, 43, "Input",
 CellTags->"Eps"],
Cell[583552, 23374, 1119, 45, 42, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[584708, 23424, 978, 28, 43, "Input",
 CellTags->"Eps"],
Cell[585689, 23454, 1679, 61, 42, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[587405, 23520, 188, 6, 26, "Input",
 CellTags->"Eps"],
Cell[587596, 23528, 342, 12, 42, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[CellGroupData[{
Cell[587975, 23545, 124, 4, 26, "Input",
 CellTags->"Eps"],
Cell[588102, 23551, 99, 3, 26, "Output",
 CellTags->"Eps"]
}, Open  ]],
Cell[588216, 23557, 164, 5, 26, "Input",
 CellTags->"Eps"]
}, Open  ]],
Cell[588395, 23565, 59, 1, 39, "ObjectName",
 CellTags->"EpsChisholm"],
Cell[CellGroupData[{
Cell[588479, 23570, 62, 1, 36, "Subsubsection",
 CellTags->"EpsChisholm"],
Cell[588544, 23573, 166, 4, 24, "Text",
 CellTags->"EpsChisholm"],
Cell[588713, 23579, 264, 11, 24, "Text",
 CellTags->"EpsChisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[589014, 23595, 59, 1, 36, "Subsubsection",
 CellTags->"EpsChisholm"],
Cell[CellGroupData[{
Cell[589098, 23600, 208, 6, 26, "Input",
 CellTags->"EpsChisholm"],
Cell[589309, 23608, 2019, 78, 26, "Output",
 CellTags->"EpsChisholm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[591365, 23691, 115, 3, 26, "Input",
 CellTags->"EpsChisholm"],
Cell[591483, 23696, 594, 24, 26, "Output",
 CellTags->"EpsChisholm"]
}, Open  ]]
}, Open  ]],
Cell[592104, 23724, 59, 1, 39, "ObjectName",
 CellTags->"EpsContract"],
Cell[CellGroupData[{
Cell[592188, 23729, 62, 1, 36, "Subsubsection",
 CellTags->"EpsContract"],
Cell[592253, 23732, 227, 5, 41, "Text",
 CellTags->"EpsContract"],
Cell[592483, 23739, 436, 18, 24, "Text",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[592956, 23762, 59, 1, 36, "Subsubsection",
 CellTags->"EpsContract"],
Cell[CellGroupData[{
Cell[593040, 23767, 468, 13, 26, "Input",
 CellTags->"EpsContract"],
Cell[593511, 23782, 485, 21, 26, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[594033, 23808, 205, 6, 26, "Input",
 CellTags->"EpsContract"],
Cell[594241, 23816, 569, 23, 26, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[594847, 23844, 204, 6, 26, "Input",
 CellTags->"EpsContract"],
Cell[595054, 23852, 320, 11, 26, "Output",
 CellTags->"EpsContract"]
}, Open  ]],
Cell[595389, 23866, 84, 2, 26, "Input"]
}, Open  ]],
Cell[595488, 23871, 57, 1, 39, "ObjectName",
 CellTags->"EpsDiscard"],
Cell[CellGroupData[{
Cell[595570, 23876, 61, 1, 36, "Subsubsection",
 CellTags->"EpsDiscard"],
Cell[595634, 23879, 169, 4, 24, "Text",
 CellTags->"EpsDiscard"],
Cell[595806, 23885, 283, 11, 24, "Text",
 CellTags->"EpsDiscard"]
}, Open  ]],
Cell[596104, 23899, 59, 1, 39, "ObjectName",
 CellTags->"EpsEvaluate"],
Cell[CellGroupData[{
Cell[596188, 23904, 62, 1, 36, "Subsubsection",
 CellTags->"EpsEvaluate"],
Cell[596253, 23907, 173, 4, 24, "Text",
 CellTags->"EpsEvaluate"],
Cell[596429, 23913, 816, 32, 24, "Text",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[597282, 23950, 59, 1, 36, "Subsubsection",
 CellTags->"EpsEvaluate"],
Cell[CellGroupData[{
Cell[597366, 23955, 335, 10, 26, "Input",
 CellTags->"EpsEvaluate"],
Cell[597704, 23967, 466, 20, 26, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[598207, 23992, 115, 3, 26, "Input",
 CellTags->"EpsEvaluate"],
Cell[598325, 23997, 466, 20, 26, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[598828, 24022, 116, 3, 26, "Input",
 CellTags->"EpsEvaluate"],
Cell[598947, 24027, 434, 12, 38, "Output",
 CellTags->"EpsEvaluate"]
}, Open  ]]
}, Open  ]],
Cell[599408, 24043, 63, 1, 39, "ObjectName",
 CellTags->"EpsUncontract"],
Cell[CellGroupData[{
Cell[599496, 24048, 64, 1, 36, "Subsubsection",
 CellTags->"EpsUncontract"],
Cell[599563, 24051, 115, 3, 24, "Text",
 CellTags->"EpsUncontract"],
Cell[599681, 24056, 446, 18, 24, "Text",
 CellTags->"EpsUncontract"]
}, Open  ]],
Cell[600142, 24077, 51, 1, 39, "ObjectName",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[600218, 24082, 58, 1, 36, "Subsubsection",
 CellTags->"Epsilon"],
Cell[600279, 24085, 237, 9, 24, "Text",
 CellTags->"Epsilon"],
Cell[600519, 24096, 256, 11, 24, "Text",
 CellTags->"Epsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[600812, 24112, 55, 1, 36, "Subsubsection",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[600892, 24117, 80, 2, 26, "Input",
 CellTags->"Epsilon"],
Cell[600975, 24121, 148, 5, 26, "Output",
 CellTags->"Epsilon"]
}, Open  ]],
Cell[601138, 24129, 114, 3, 24, "Text",
 CellTags->"Epsilon"],
Cell[CellGroupData[{
Cell[601277, 24136, 587, 19, 26, "Input",
 CellTags->"Epsilon"],
Cell[601867, 24157, 199, 6, 26, "Output",
 CellTags->"Epsilon"]
}, Open  ]]
}, Open  ]],
Cell[602093, 24167, 61, 1, 39, "ObjectName",
 CellTags->"EpsilonOrder"],
Cell[CellGroupData[{
Cell[602179, 24172, 63, 1, 36, "Subsubsection",
 CellTags->"EpsilonOrder"],
Cell[602245, 24175, 305, 12, 24, "Text",
 CellTags->"EpsilonOrder"],
Cell[602553, 24189, 449, 18, 24, "Text",
 CellTags->"EpsilonOrder"]
}, Open  ]],
Cell[603017, 24210, 55, 1, 39, "ObjectName",
 CellTags->"Expanding"],
Cell[CellGroupData[{
Cell[603097, 24215, 60, 1, 36, "Subsubsection",
 CellTags->"Expanding"],
Cell[603160, 24218, 378, 7, 58, "Text",
 CellTags->"Expanding"],
Cell[603541, 24227, 1058, 39, 24, "Text",
 CellTags->"Expanding"]
}, Open  ]],
Cell[604614, 24269, 65, 1, 39, "ObjectName",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[604704, 24274, 65, 1, 36, "Subsubsection",
 CellTags->"ExpandPartialD"],
Cell[604772, 24277, 188, 4, 24, "Text",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[604985, 24285, 98, 2, 26, "Input"],
Cell[605086, 24289, 1041, 36, 29, "Output"]
}, Open  ]],
Cell[606142, 24328, 1169, 38, 24, "Text",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[607348, 24371, 62, 1, 36, "Subsubsection",
 CellTags->"ExpandPartialD"],
Cell[CellGroupData[{
Cell[607435, 24376, 373, 10, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[607811, 24388, 345, 12, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[608193, 24405, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[608317, 24410, 773, 30, 30, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[609127, 24445, 115, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[609245, 24450, 798, 21, 58, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[610080, 24476, 259, 7, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[610342, 24485, 312, 11, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[610691, 24501, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[610815, 24506, 704, 26, 42, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[611556, 24537, 433, 13, 46, "Input",
 CellTags->"ExpandPartialD"],
Cell[611992, 24552, 332, 10, 35, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[612361, 24567, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[612485, 24572, 1940, 67, 42, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[614462, 24644, 180, 6, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[614645, 24652, 267, 8, 34, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[614949, 24665, 123, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[615075, 24670, 438, 14, 38, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[615550, 24689, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[615674, 24694, 1877, 59, 30, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[617588, 24758, 314, 9, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[617905, 24769, 1192, 47, 33, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[619134, 24821, 121, 3, 26, "Input",
 CellTags->"ExpandPartialD"],
Cell[619258, 24826, 110, 3, 26, "Output",
 CellTags->"ExpandPartialD"]
}, Open  ]]
}, Open  ]],
Cell[619395, 24833, 121, 1, 39, "ObjectName",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[619541, 24838, 95, 1, 36, "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[619639, 24841, 252, 5, 49, "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[619916, 24850, 158, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[620077, 24855, 214, 6, 26, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[620306, 24864, 717, 25, 24, "Text",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[621060, 24894, 92, 1, 36, "Subsubsection",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[CellGroupData[{
Cell[621177, 24899, 257, 7, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[621437, 24908, 338, 10, 26, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[621812, 24923, 152, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[621967, 24928, 1214, 49, 26, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[623218, 24982, 186, 5, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[623407, 24989, 277, 9, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[623721, 25003, 156, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[623880, 25008, 331, 12, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[624248, 25025, 190, 5, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[624441, 25032, 227, 7, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[624705, 25044, 156, 3, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[624864, 25049, 441, 18, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[625342, 25072, 239, 7, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[625584, 25081, 312, 10, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[625933, 25096, 184, 4, 26, "Input",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}],
Cell[626120, 25102, 565, 22, 28, "Output",
 CellTags->{"ScalarProductCancel", "ExpandScalarProduct"}]
}, Open  ]]
}, Open  ]],
Cell[626712, 25128, 51, 1, 39, "ObjectName",
 CellTags->"Expand2"],
Cell[CellGroupData[{
Cell[626788, 25133, 58, 1, 36, "Subsubsection",
 CellTags->"Expand2"],
Cell[626849, 25136, 164, 4, 24, "Text",
 CellTags->"Expand2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[627050, 25145, 55, 1, 36, "Subsubsection",
 CellTags->"Expand2"],
Cell[CellGroupData[{
Cell[627130, 25150, 579, 21, 26, "Input",
 CellTags->"Expand2"],
Cell[627712, 25173, 574, 19, 26, "Output",
 CellTags->"Expand2"]
}, Open  ]]
}, Open  ]],
Cell[628313, 25196, 53, 1, 39, "ObjectName",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[628391, 25201, 59, 1, 36, "Subsubsection",
 CellTags->"Explicit"],
Cell[628453, 25204, 401, 9, 41, "Text",
 CellTags->"Explicit"],
Cell[628857, 25215, 509, 18, 24, "Text",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[629391, 25237, 88, 2, 26, "Input"],
Cell[629482, 25241, 345, 10, 26, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[629876, 25257, 56, 1, 36, "Subsubsection",
 CellTags->"Explicit"],
Cell[CellGroupData[{
Cell[629957, 25262, 134, 3, 26, "Input"],
Cell[630094, 25267, 183, 6, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[630314, 25278, 86, 2, 26, "Input"],
Cell[630403, 25282, 241, 8, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[630681, 25295, 233, 6, 26, "Input"],
Cell[630917, 25303, 776, 28, 58, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[631730, 25336, 230, 6, 26, "Input",
 CellTags->"Explicit"],
Cell[631963, 25344, 940, 42, 28, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[632940, 25391, 109, 3, 26, "Input",
 CellTags->"Explicit"],
Cell[633052, 25396, 1971, 79, 28, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[635060, 25480, 184, 5, 26, "Input",
 CellTags->"Explicit"],
Cell[635247, 25487, 892, 36, 42, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[636176, 25528, 109, 3, 26, "Input",
 CellTags->"Explicit"],
Cell[636288, 25533, 2644, 102, 42, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[638969, 25640, 155, 4, 26, "Input",
 CellTags->"Explicit"],
Cell[639127, 25646, 163, 5, 30, "Output",
 CellTags->"Explicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[639327, 25656, 109, 3, 26, "Input",
 CellTags->"Explicit"],
Cell[639439, 25661, 1744, 77, 30, "Output",
 CellTags->"Explicit"]
}, Open  ]]
}, Open  ]],
Cell[641210, 25742, 77, 1, 39, "ObjectName",
 CellTags->"ExplicitLorentzIndex"],
Cell[CellGroupData[{
Cell[641312, 25747, 71, 1, 36, "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],
Cell[641386, 25750, 141, 4, 24, "Text",
 CellTags->"ExplicitLorentzIndex"],
Cell[641530, 25756, 459, 17, 24, "Text",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[642026, 25778, 68, 1, 36, "Subsubsection",
 CellTags->"ExplicitLorentzIndex"],
Cell[CellGroupData[{
Cell[642119, 25783, 224, 6, 26, "Input",
 CellTags->"ExplicitLorentzIndex"],
Cell[642346, 25791, 199, 7, 26, "Output",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[642582, 25803, 121, 3, 26, "Input",
 CellTags->"ExplicitLorentzIndex"],
Cell[642706, 25808, 247, 6, 38, "Output",
 CellTags->"ExplicitLorentzIndex"]
}, Open  ]]
}, Open  ]],
Cell[642980, 25818, 69, 1, 39, "ObjectName",
 CellTags->"ExplicitPartialD"],
Cell[CellGroupData[{
Cell[643074, 25823, 67, 1, 36, "Subsubsection",
 CellTags->"ExplicitPartialD"],
Cell[643144, 25826, 162, 4, 24, "Text",
 CellTags->"ExplicitPartialD"],
Cell[643309, 25832, 747, 24, 24, "Text",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644093, 25861, 64, 1, 36, "Subsubsection",
 CellTags->"ExplicitPartialD"],
Cell[CellGroupData[{
Cell[644182, 25866, 183, 4, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[644368, 25872, 525, 19, 42, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[644930, 25896, 184, 4, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[645117, 25902, 430, 16, 33, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[645584, 25923, 186, 4, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[645773, 25929, 461, 15, 42, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[646271, 25949, 183, 6, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[646457, 25957, 270, 8, 34, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[646764, 25970, 125, 3, 26, "Input",
 CellTags->"ExplicitPartialD"],
Cell[646892, 25975, 440, 14, 38, "Output",
 CellTags->"ExplicitPartialD"]
}, Open  ]]
}, Open  ]],
Cell[647359, 25993, 69, 1, 39, "ObjectName",
 CellTags->"ExplicitSUNIndex"],
Cell[CellGroupData[{
Cell[647453, 25998, 67, 1, 36, "Subsubsection",
 CellTags->"ExplicitSUNIndex"],
Cell[647523, 26001, 178, 6, 24, "Text",
 CellTags->"ExplicitSUNIndex"],
Cell[647704, 26009, 803, 31, 24, "Text",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[648544, 26045, 64, 1, 36, "Subsubsection",
 CellTags->"ExplicitSUNIndex"],
Cell[CellGroupData[{
Cell[648633, 26050, 125, 3, 26, "Input",
 CellTags->"ExplicitSUNIndex"],
Cell[648761, 26055, 112, 3, 26, "Output",
 CellTags->"ExplicitSUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[648910, 26063, 176, 6, 26, "Input"],
Cell[649089, 26071, 204, 5, 38, "Output"]
}, Open  ]]
}, Open  ]],
Cell[649320, 26080, 59, 1, 39, "ObjectName",
 CellTags->"ExtraFactor"],
Cell[CellGroupData[{
Cell[649404, 26085, 62, 1, 36, "Subsubsection",
 CellTags->"ExtraFactor"],
Cell[649469, 26088, 199, 4, 24, "Text",
 CellTags->"ExtraFactor"],
Cell[649671, 26094, 282, 10, 24, "Text",
 CellTags->"ExtraFactor"]
}, Open  ]],
Cell[649968, 26107, 65, 1, 39, "ObjectName",
 CellTags->"ExtraVariables"],
Cell[CellGroupData[{
Cell[650058, 26112, 65, 1, 36, "Subsubsection",
 CellTags->"ExtraVariables"],
Cell[650126, 26115, 213, 5, 41, "Text",
 CellTags->"ExtraVariables"],
Cell[650342, 26122, 463, 18, 24, "Text",
 CellTags->"ExtraVariables"]
}, Open  ]],
Cell[650820, 26143, 57, 1, 39, "ObjectName",
 CellTags->"FactorFull"],
Cell[CellGroupData[{
Cell[650902, 26148, 61, 1, 36, "Subsubsection",
 CellTags->"FactorFull"],
Cell[650966, 26151, 509, 24, 24, "Text",
 CellTags->"FactorFull"],
Cell[651478, 26177, 259, 11, 24, "Text",
 CellTags->"FactorFull"]
}, Open  ]],
Cell[651752, 26191, 55, 1, 39, "ObjectName",
 CellTags->"Factoring"],
Cell[CellGroupData[{
Cell[651832, 26196, 60, 1, 36, "Subsubsection",
 CellTags->"Factoring"],
Cell[651895, 26199, 235, 5, 41, "Text",
 CellTags->"Factoring"],
Cell[652133, 26206, 622, 25, 24, "Text",
 CellTags->"Factoring"]
}, Open  ]],
Cell[652770, 26234, 55, 1, 39, "ObjectName",
 CellTags->"Factorout"],
Cell[CellGroupData[{
Cell[652850, 26239, 60, 1, 36, "Subsubsection",
 CellTags->"Factorout"],
Cell[652913, 26242, 74, 1, 24, "Text",
 CellTags->"Factorout"],
Cell[652990, 26245, 254, 11, 24, "Text",
 CellTags->"Factorout"]
}, Open  ]],
Cell[653259, 26259, 57, 1, 39, "ObjectName",
 CellTags->"FactorTime"],
Cell[CellGroupData[{
Cell[653341, 26264, 61, 1, 36, "Subsubsection",
 CellTags->"FactorTime"],
Cell[653405, 26267, 166, 4, 24, "Text",
 CellTags->"FactorTime"],
Cell[653574, 26273, 259, 11, 24, "Text",
 CellTags->"FactorTime"]
}, Open  ]],
Cell[653848, 26287, 51, 1, 39, "ObjectName",
 CellTags->"Factor1"],
Cell[CellGroupData[{
Cell[653924, 26292, 58, 1, 36, "Subsubsection",
 CellTags->"Factor1"],
Cell[653985, 26295, 144, 4, 24, "Text",
 CellTags->"Factor1"],
Cell[654132, 26301, 256, 11, 24, "Text",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[654425, 26317, 55, 1, 36, "Subsubsection",
 CellTags->"Factor1"],
Cell[CellGroupData[{
Cell[654505, 26322, 213, 8, 26, "Input",
 CellTags->"Factor1"],
Cell[654721, 26332, 224, 8, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[654982, 26345, 222, 7, 26, "Input",
 CellTags->"Factor1"],
Cell[655207, 26354, 435, 16, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[655679, 26375, 251, 9, 26, "Input",
 CellTags->"Factor1"],
Cell[655933, 26386, 176, 6, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[656146, 26397, 107, 3, 26, "Input",
 CellTags->"Factor1"],
Cell[656256, 26402, 224, 8, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[656517, 26415, 108, 3, 26, "Input",
 CellTags->"Factor1"],
Cell[656628, 26420, 176, 6, 26, "Output",
 CellTags->"Factor1"]
}, Open  ]],
Cell[656819, 26429, 141, 4, 26, "Input",
 CellTags->"Factor1"]
}, Open  ]],
Cell[656975, 26436, 51, 1, 39, "ObjectName",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[657051, 26441, 58, 1, 36, "Subsubsection",
 CellTags->"Factor2"],
Cell[657112, 26444, 398, 7, 58, "Text",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[657535, 26455, 113, 3, 26, "Input",
 CellTags->"Factor2"],
Cell[657651, 26460, 230, 7, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[657896, 26470, 260, 11, 24, "Text",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[658193, 26486, 55, 1, 36, "Subsubsection",
 CellTags->"Factor2"],
Cell[CellGroupData[{
Cell[658273, 26491, 213, 8, 26, "Input",
 CellTags->"Factor2"],
Cell[658489, 26501, 224, 8, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[658750, 26514, 222, 7, 26, "Input",
 CellTags->"Factor2"],
Cell[658975, 26523, 435, 16, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[659447, 26544, 251, 9, 26, "Input",
 CellTags->"Factor2"],
Cell[659701, 26555, 176, 6, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[659914, 26566, 107, 3, 26, "Input",
 CellTags->"Factor2"],
Cell[660024, 26571, 224, 8, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[660285, 26584, 108, 3, 26, "Input",
 CellTags->"Factor2"],
Cell[660396, 26589, 176, 6, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[660609, 26600, 173, 5, 26, "Input",
 CellTags->"Factor2"],
Cell[660785, 26607, 224, 8, 26, "Output",
 CellTags->"Factor2"]
}, Open  ]],
Cell[661024, 26618, 141, 4, 26, "Input",
 CellTags->"Factor2"]
}, Open  ]],
Cell[661180, 26625, 43, 1, 39, "ObjectName",
 CellTags->"FAD"],
Cell[CellGroupData[{
Cell[661248, 26630, 54, 1, 36, "Subsubsection",
 CellTags->"FAD"],
Cell[661305, 26633, 335, 6, 58, "Text",
 CellTags->"FAD"],
Cell[661643, 26641, 1320, 46, 24, "Text",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[663000, 26692, 51, 1, 36, "Subsubsection",
 CellTags->"FAD"],
Cell[CellGroupData[{
Cell[663076, 26697, 146, 5, 26, "Input",
 CellTags->"FAD"],
Cell[663225, 26704, 700, 20, 48, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[663962, 26729, 197, 7, 26, "Input",
 CellTags->"FAD"],
Cell[664162, 26738, 819, 22, 48, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[665018, 26765, 245, 9, 26, "Input",
 CellTags->"FAD"],
Cell[665266, 26776, 161, 5, 38, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[665464, 26786, 215, 8, 26, "Input",
 CellTags->"FAD"],
Cell[665682, 26796, 527, 16, 58, "Output",
 CellTags->"FAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[666246, 26817, 280, 9, 26, "Input",
 CellTags->"FAD"],
Cell[666529, 26828, 527, 16, 58, "Output",
 CellTags->"FAD"]
}, Open  ]]
}, Open  ]],
Cell[667083, 26848, 41, 1, 39, "ObjectName",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[667149, 26853, 53, 1, 36, "Subsubsection",
 CellTags->"FC"],
Cell[667205, 26856, 117, 3, 24, "Text",
 CellTags->"FC"],
Cell[667325, 26861, 887, 32, 24, "Text",
 CellTags->"FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[668249, 26898, 50, 1, 36, "Subsubsection",
 CellTags->"FC"],
Cell[668302, 26901, 70, 2, 26, "Input",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[668397, 26907, 227, 7, 26, "Input",
 CellTags->"FC"],
Cell[668627, 26916, 100, 2, 25, "Output",
 CellTags->"FC"]
}, Open  ]],
Cell[668742, 26921, 70, 2, 26, "Input",
 CellTags->"FC"],
Cell[CellGroupData[{
Cell[668837, 26927, 227, 7, 26, "Input",
 CellTags->"FC"],
Cell[669067, 26936, 305, 11, 29, "Output",
 CellTags->"FC"]
}, Open  ]]
}, Open  ]],
Cell[669399, 26951, 43, 1, 39, "ObjectName",
 CellTags->"FCE"],
Cell[CellGroupData[{
Cell[669467, 26956, 54, 1, 36, "Subsubsection",
 CellTags->"FCE"],
Cell[669524, 26959, 126, 4, 24, "Text",
 CellTags->"FCE"],
Cell[669653, 26965, 72, 1, 24, "Text",
 CellTags->"FCE"],
Cell[669728, 26968, 684, 25, 24, "Text",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[670449, 26998, 51, 1, 36, "Subsubsection",
 CellTags->"FCE"],
Cell[CellGroupData[{
Cell[670525, 27003, 262, 8, 26, "Input",
 CellTags->"FCE"],
Cell[670790, 27013, 225, 8, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[671052, 27026, 104, 3, 26, "Input",
 CellTags->"FCE"],
Cell[671159, 27031, 190, 6, 38, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[671386, 27042, 437, 13, 26, "Input",
 CellTags->"FCE"],
Cell[671826, 27057, 547, 18, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[672410, 27080, 104, 3, 26, "Input",
 CellTags->"FCE"],
Cell[672517, 27085, 452, 13, 38, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[673006, 27103, 95, 3, 26, "Input",
 CellTags->"FCE"],
Cell[673104, 27108, 862, 34, 29, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[674003, 27147, 104, 3, 26, "Input",
 CellTags->"FCE"],
Cell[674110, 27152, 893, 24, 77, "Output",
 CellTags->"FCE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[675040, 27181, 134, 4, 26, "Input",
 CellTags->"FCE"],
Cell[675177, 27187, 452, 13, 38, "Output",
 CellTags->"FCE"]
}, Open  ]]
}, Open  ]],
Cell[675656, 27204, 43, 1, 39, "ObjectName",
 CellTags->"FCI"],
Cell[CellGroupData[{
Cell[675724, 27209, 54, 1, 36, "Subsubsection",
 CellTags->"FCI"],
Cell[675781, 27212, 159, 4, 24, "Text",
 CellTags->"FCI"],
Cell[675943, 27218, 684, 25, 24, "Text",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[676664, 27248, 51, 1, 36, "Subsubsection",
 CellTags->"FCI"],
Cell[CellGroupData[{
Cell[676740, 27253, 437, 13, 26, "Input",
 CellTags->"FCI"],
Cell[677180, 27268, 547, 18, 29, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[677764, 27291, 104, 3, 26, "Input",
 CellTags->"FCI"],
Cell[677871, 27296, 452, 13, 38, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[678360, 27314, 95, 3, 26, "Input",
 CellTags->"FCI"],
Cell[678458, 27319, 862, 34, 29, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[679357, 27358, 104, 3, 26, "Input",
 CellTags->"FCI"],
Cell[679464, 27363, 893, 24, 77, "Output",
 CellTags->"FCI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[680394, 27392, 134, 4, 26, "Input",
 CellTags->"FCI"],
Cell[680531, 27398, 452, 13, 38, "Output",
 CellTags->"FCI"]
}, Open  ]]
}, Open  ]],
Cell[681010, 27415, 57, 1, 39, "ObjectName",
 CellTags->"FCIntegral"],
Cell[CellGroupData[{
Cell[681092, 27420, 61, 1, 36, "Subsubsection",
 CellTags->"FCIntegral"],
Cell[681156, 27423, 209, 5, 41, "Text",
 CellTags->"FCIntegral"],
Cell[681368, 27430, 547, 18, 24, "Text",
 CellTags->"FCIntegral"]
}, Open  ]],
Cell[681930, 27451, 59, 1, 39, "ObjectName",
 CellTags->"FCIntegrate"],
Cell[CellGroupData[{
Cell[682014, 27456, 62, 1, 36, "Subsubsection",
 CellTags->"FCIntegrate"],
Cell[682079, 27459, 285, 5, 41, "Text",
 CellTags->"FCIntegrate"],
Cell[682367, 27466, 280, 11, 24, "Text",
 CellTags->"FCIntegrate"]
}, Open  ]],
Cell[682662, 27480, 61, 1, 39, "ObjectName",
 CellTags->"FCNIntegrate"],
Cell[CellGroupData[{
Cell[682748, 27485, 63, 1, 36, "Subsubsection",
 CellTags->"FCNIntegrate"],
Cell[682814, 27488, 418, 7, 58, "Text",
 CellTags->"FCNIntegrate"],
Cell[683235, 27497, 277, 11, 24, "Text",
 CellTags->"FCNIntegrate"]
}, Open  ]],
Cell[683527, 27511, 50, 1, 39, "ObjectName",
 CellTags->"FC2RHI"],
Cell[CellGroupData[{
Cell[683602, 27516, 57, 1, 36, "Subsubsection",
 CellTags->"FC2RHI"],
Cell[683662, 27519, 306, 6, 41, "Text",
 CellTags->"FC2RHI"],
Cell[CellGroupData[{
Cell[683993, 27529, 111, 3, 26, "Input",
 CellTags->"FC2RHI"],
Cell[684107, 27534, 277, 8, 26, "Output",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[684421, 27547, 349, 12, 26, "Input"],
Cell[684773, 27561, 506, 18, 49, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[685316, 27584, 119, 3, 26, "Input"],
Cell[685438, 27589, 473, 20, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[685948, 27614, 83, 2, 26, "Input"],
Cell[686034, 27618, 94, 1, 38, "Output"]
}, Open  ]],
Cell[686143, 27622, 239, 11, 24, "Text",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[686419, 27638, 54, 1, 36, "Subsubsection",
 CellTags->"FC2RHI"],
Cell[686476, 27641, 72, 2, 26, "Input",
 CellTags->"FC2RHI"]
}, Open  ]],
Cell[686563, 27646, 50, 1, 39, "ObjectName",
 CellTags->"FC2TLI"],
Cell[CellGroupData[{
Cell[686638, 27651, 57, 1, 36, "Subsubsection",
 CellTags->"FC2TLI"],
Cell[686698, 27654, 256, 5, 41, "Text",
 CellTags->"FC2TLI"],
Cell[CellGroupData[{
Cell[686979, 27663, 111, 3, 26, "Input",
 CellTags->"FC2TLI"],
Cell[687093, 27668, 277, 8, 26, "Output",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[687407, 27681, 71, 2, 26, "Input"],
Cell[687481, 27685, 321, 5, 112, "Print",
 CellTags->"Info3535453193-5096475"]
}, Open  ]],
Cell[CellGroupData[{
Cell[687839, 27695, 365, 12, 26, "Input"],
Cell[688207, 27709, 1483, 42, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[689727, 27756, 119, 3, 26, "Input"],
Cell[689849, 27761, 699, 29, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[690585, 27795, 83, 2, 26, "Input"],
Cell[690671, 27799, 100, 1, 38, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[690808, 27805, 84, 2, 26, "Input"],
Cell[690895, 27809, 588, 21, 49, "Output"]
}, Open  ]],
Cell[691498, 27833, 239, 11, 24, "Text",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[691774, 27849, 54, 1, 36, "Subsubsection",
 CellTags->"FC2TLI"],
Cell[691831, 27852, 72, 2, 26, "Input",
 CellTags->"FC2TLI"]
}, Open  ]],
Cell[691918, 27857, 43, 1, 39, "ObjectName",
 CellTags->"FDS"],
Cell[CellGroupData[{
Cell[691986, 27862, 54, 1, 36, "Subsubsection",
 CellTags->"FDS"],
Cell[692043, 27865, 82, 1, 24, "Text",
 CellTags->"FDS"],
Cell[692128, 27868, 328, 11, 24, "Text",
 CellTags->"FDS"]
}, Open  ]],
Cell[692471, 27882, 65, 1, 39, "ObjectName",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[692561, 27887, 65, 1, 36, "Subsubsection",
 CellTags->"FermionSpinSum"],
Cell[692629, 27890, 118, 3, 24, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[692772, 27897, 127, 3, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[692902, 27902, 314, 9, 26, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[693231, 27914, 851, 32, 24, "Text",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[694119, 27951, 62, 1, 36, "Subsubsection",
 CellTags->"FermionSpinSum"],
Cell[694184, 27954, 261, 12, 24, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[694470, 27970, 296, 9, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[694769, 27981, 353, 13, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[695159, 27999, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[695305, 28005, 330, 12, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[695672, 28022, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[695818, 28028, 133, 4, 26, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[695988, 28037, 296, 9, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[696287, 28048, 353, 13, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[696677, 28066, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[696823, 28072, 330, 12, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[697190, 28089, 143, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[697336, 28095, 152, 5, 26, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[697525, 28105, 334, 10, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[697862, 28117, 508, 17, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[698385, 28137, 164, 4, 24, "Text",
 CellTags->"FermionSpinSum"],
Cell[CellGroupData[{
Cell[698574, 28145, 147, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[698724, 28151, 540, 18, 28, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[699301, 28174, 145, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[699449, 28180, 1074, 36, 29, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[700560, 28221, 148, 4, 26, "Input",
 CellTags->"FermionSpinSum"],
Cell[700711, 28227, 837, 32, 29, "Output",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[701563, 28262, 141, 4, 26, "Input",
 CellTags->"FermionSpinSum"]
}, Open  ]],
Cell[701719, 28269, 51, 1, 39, "ObjectName",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[701795, 28274, 58, 1, 36, "Subsubsection",
 CellTags->"FeynAmp"],
Cell[701856, 28277, 243, 5, 41, "Text",
 CellTags->"FeynAmp"],
Cell[702102, 28284, 165, 4, 24, "Text",
 CellTags->"FeynAmp"],
Cell[702270, 28290, 264, 11, 24, "Text",
 CellTags->"FeynAmp"]
}, Open  ]],
Cell[CellGroupData[{
Cell[702571, 28306, 55, 1, 36, "Subsubsection",
 CellTags->"FeynAmp"],
Cell[702629, 28309, 125, 4, 24, "Text",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[702779, 28317, 846, 22, 46, "Input",
 CellTags->"FeynAmp"],
Cell[703628, 28341, 2586, 102, 42, "Output",
 CellTags->"FeynAmp"]
}, Open  ]],
Cell[706229, 28446, 73, 1, 24, "Text",
 CellTags->"FeynAmp"],
Cell[CellGroupData[{
Cell[706327, 28451, 198, 6, 26, "Input",
 CellTags->"FeynAmp"],
Cell[706528, 28459, 414, 14, 42, "Output",
 CellTags->"FeynAmp"]
}, Open  ]]
}, Open  ]],
Cell[706969, 28477, 73, 1, 39, "ObjectName",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[707067, 28482, 69, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominator"],
Cell[707139, 28485, 256, 5, 41, "Text",
 CellTags->"FeynAmpDenominator"],
Cell[707398, 28492, 515, 18, 24, "Text",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[707950, 28515, 66, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[708041, 28520, 201, 5, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[708245, 28527, 211, 7, 48, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[708493, 28539, 326, 9, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[708822, 28550, 430, 15, 48, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[709267, 28568, 247, 7, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[CellGroupData[{
Cell[709539, 28579, 149, 4, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[709691, 28585, 279, 7, 38, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[710007, 28597, 149, 4, 26, "Input",
 CellTags->"FeynAmpDenominator"],
Cell[710159, 28603, 176, 5, 38, "Output",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[710350, 28611, 134, 4, 26, "Input",
 CellTags->"FeynAmpDenominator"]
}, Open  ]],
Cell[710499, 28618, 87, 1, 39, "ObjectName",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[CellGroupData[{
Cell[710611, 28623, 76, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[710690, 28626, 228, 5, 41, "Text",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[710921, 28633, 338, 11, 24, "Text",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[711296, 28649, 73, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[CellGroupData[{
Cell[711394, 28654, 234, 7, 26, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[711631, 28663, 765, 23, 48, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[712433, 28691, 208, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[712644, 28699, 203, 6, 38, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[712884, 28710, 206, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorCombine"],
Cell[713093, 28718, 233, 7, 38, "Output",
 CellTags->"FeynAmpDenominatorCombine"]
}, Open  ]]
}, Open  ]],
Cell[713353, 28729, 89, 1, 39, "ObjectName",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[713467, 28734, 77, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[713547, 28737, 398, 7, 58, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[713948, 28746, 85, 1, 24, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[714058, 28751, 95, 2, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[714156, 28755, 147, 3, 26, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[714318, 28761, 307, 11, 24, "Text",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[714662, 28777, 74, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[714739, 28780, 432, 14, 27, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[715196, 28798, 280, 8, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[715479, 28808, 122, 3, 26, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[715616, 28814, 468, 18, 27, "Text",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[CellGroupData[{
Cell[716109, 28836, 328, 10, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[716440, 28848, 336, 11, 49, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[716813, 28864, 260, 7, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[717076, 28873, 308, 10, 49, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[717421, 28888, 339, 10, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[717763, 28900, 362, 12, 49, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[718162, 28917, 449, 14, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[718614, 28933, 1091, 40, 50, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[719742, 28978, 215, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[719960, 28986, 795, 29, 50, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[720792, 29020, 157, 4, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[720952, 29026, 493, 18, 46, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[721482, 29049, 373, 13, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"],
Cell[721858, 29064, 1361, 50, 51, "Output",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[723234, 29117, 147, 4, 26, "Input",
 CellTags->"FeynAmpDenominatorSimplify"]
}, Open  ]],
Cell[723396, 29124, 83, 1, 39, "ObjectName",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[CellGroupData[{
Cell[723504, 29129, 74, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[723581, 29132, 357, 7, 58, "Text",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[723941, 29141, 344, 11, 24, "Text",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[724322, 29157, 71, 1, 36, "Subsubsection",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[CellGroupData[{
Cell[724418, 29162, 308, 9, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[724729, 29173, 489, 16, 48, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[725255, 29194, 116, 3, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[725374, 29199, 136, 3, 26, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[725547, 29207, 139, 3, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[725689, 29212, 489, 16, 48, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[726215, 29233, 150, 4, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[726368, 29239, 425, 13, 38, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[726830, 29257, 230, 7, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[727063, 29266, 368, 12, 38, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[727468, 29283, 206, 6, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"],
Cell[727677, 29291, 305, 9, 38, "Output",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[727997, 29303, 121, 3, 26, "Input",
 CellTags->"FeynAmpDenominatorSplit"]
}, Open  ]],
Cell[728133, 29309, 59, 1, 39, "ObjectName",
 CellTags->"FeynAmpList"],
Cell[CellGroupData[{
Cell[728217, 29314, 62, 1, 36, "Subsubsection",
 CellTags->"FeynAmpList"],
Cell[728282, 29317, 145, 4, 24, "Text",
 CellTags->"FeynAmpList"],
Cell[728430, 29323, 117, 3, 24, "Text",
 CellTags->"FeynAmpList"],
Cell[728550, 29328, 260, 11, 24, "Text",
 CellTags->"FeynAmpList"]
}, Open  ]],
Cell[728825, 29342, 53, 1, 39, "ObjectName",
 CellTags->"FeynCalc"],
Cell[CellGroupData[{
Cell[728903, 29347, 59, 1, 36, "Subsubsection",
 CellTags->"FeynCalc"],
Cell[728965, 29350, 91, 1, 24, "Text",
 CellTags->"FeynCalc"],
Cell[CellGroupData[{
Cell[729081, 29355, 99, 3, 26, "Input",
 CellTags->"FeynCalc"],
Cell[729183, 29360, 703, 11, 319, "Print",
 CellTags->{"FeynCalc", "Info3535453199-5096475"}]
}, Open  ]]
}, Open  ]],
Cell[729913, 29375, 69, 1, 39, "ObjectName",
 CellTags->"FeynCalcExternal"],
Cell[CellGroupData[{
Cell[730007, 29380, 67, 1, 36, "Subsubsection",
 CellTags->"FeynCalcExternal"],
Cell[730077, 29383, 156, 4, 24, "Text",
 CellTags->"FeynCalcExternal"],
Cell[730236, 29389, 301, 11, 24, "Text",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[730574, 29405, 64, 1, 36, "Subsubsection",
 CellTags->"FeynCalcExternal"],
Cell[CellGroupData[{
Cell[730663, 29410, 162, 4, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[730828, 29416, 143, 4, 26, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[731008, 29425, 117, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[731128, 29430, 126, 3, 38, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[731291, 29438, 450, 13, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[731744, 29453, 560, 18, 29, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[732341, 29476, 117, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[732461, 29481, 465, 13, 38, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[732963, 29499, 121, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[733087, 29504, 875, 34, 29, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[733999, 29543, 117, 3, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[734119, 29548, 906, 24, 77, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[735062, 29577, 163, 5, 26, "Input",
 CellTags->"FeynCalcExternal"],
Cell[735228, 29584, 465, 13, 38, "Output",
 CellTags->"FeynCalcExternal"]
}, Open  ]]
}, Open  ]],
Cell[735720, 29601, 61, 1, 39, "ObjectName",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[735806, 29606, 63, 1, 36, "Subsubsection",
 CellTags->"FeynCalcForm"],
Cell[735872, 29609, 534, 10, 75, "Text",
 CellTags->"FeynCalcForm"],
Cell[736409, 29621, 689, 25, 24, "Text",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[CellGroupData[{
Cell[737135, 29651, 60, 1, 36, "Subsubsection",
 CellTags->"FeynCalcForm"],
Cell[737198, 29654, 79, 1, 24, "Text",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[737302, 29659, 242, 7, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[737547, 29668, 527, 22, 26, "Output",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[738089, 29693, 112, 3, 24, "Text",
 CellTags->"FeynCalcForm"],
Cell[738204, 29698, 148, 4, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[738355, 29704, 647, 19, 73, "Input",
 CellTags->"FeynCalcForm"],
Cell[CellGroupData[{
Cell[739027, 29727, 242, 7, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[739272, 29736, 564, 17, 26, "Output",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[739851, 29756, 76, 1, 24, "Text",
 CellTags->"FeynCalcForm"],
Cell[739930, 29759, 123, 4, 26, "Input",
 CellTags->"FeynCalcForm"],
Cell[740056, 29765, 652, 19, 51, "Input",
 CellTags->"FeynCalcForm"]
}, Open  ]],
Cell[740723, 29787, 69, 1, 39, "ObjectName",
 CellTags->"FeynCalcInternal"],
Cell[CellGroupData[{
Cell[740817, 29792, 67, 1, 36, "Subsubsection",
 CellTags->"FeynCalcInternal"],
Cell[740887, 29795, 157, 4, 24, "Text",
 CellTags->"FeynCalcInternal"],
Cell[741047, 29801, 645, 25, 24, "Text",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[741729, 29831, 64, 1, 36, "Subsubsection",
 CellTags->"FeynCalcInternal"],
Cell[CellGroupData[{
Cell[741818, 29836, 450, 13, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[742271, 29851, 560, 18, 29, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[742868, 29874, 117, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[742988, 29879, 465, 13, 38, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[743490, 29897, 121, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[743614, 29902, 875, 34, 29, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[744526, 29941, 117, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[744646, 29946, 906, 24, 77, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[745589, 29975, 163, 5, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[745755, 29982, 465, 13, 38, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[746257, 30000, 517, 15, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[746777, 30017, 1461, 63, 48, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]],
Cell[CellGroupData[{
Cell[748275, 30085, 117, 3, 26, "Input",
 CellTags->"FeynCalcInternal"],
Cell[748395, 30090, 1128, 30, 96, "Output",
 CellTags->"FeynCalcInternal"]
}, Open  ]]
}, Open  ]],
Cell[749550, 30124, 63, 1, 39, "ObjectName",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[749638, 30129, 64, 1, 36, "Subsubsection",
 CellTags->"FeynCalc2FORM"],
Cell[749705, 30132, 226, 5, 41, "Text",
 CellTags->"FeynCalc2FORM"],
Cell[749934, 30139, 286, 11, 24, "Text",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[750245, 30154, 125, 3, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[750373, 30159, 1022, 22, 67, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[751444, 30187, 61, 1, 36, "Subsubsection",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[751530, 30192, 273, 9, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[751806, 30203, 276, 9, 45, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[752119, 30217, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[752259, 30223, 68, 1, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[752364, 30229, 199, 6, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[752566, 30237, 199, 6, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[752802, 30248, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[752942, 30254, 66, 1, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[753045, 30260, 212, 6, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[753260, 30268, 373, 11, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[753670, 30284, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[753810, 30290, 88, 3, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[753935, 30298, 298, 9, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[754236, 30309, 473, 16, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[754746, 30330, 137, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[754886, 30336, 88, 3, 21, "Print",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[754989, 30342, 328, 10, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[755342, 30356, 332, 10, 29, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[755677, 30368, 154, 4, 26, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[755868, 30377, 314, 10, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[756185, 30389, 6179, 240, 50, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[762379, 30632, 209, 6, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[CellGroupData[{
Cell[762613, 30642, 376, 11, 29, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[762992, 30655, 1338, 36, 225, "Output",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[764345, 30694, 313, 10, 26, "Input",
 CellTags->"FeynCalc2FORM"],
Cell[764661, 30706, 129, 4, 26, "Input",
 CellTags->"FeynCalc2FORM"]
}, Open  ]],
Cell[764805, 30713, 67, 1, 39, "ObjectName",
 CellTags->"FeynCalcToLaTeX"],
Cell[CellGroupData[{
Cell[764897, 30718, 66, 1, 36, "Subsubsection",
 CellTags->"FeynCalcToLaTeX"],
Cell[764966, 30721, 300, 6, 66, "Text",
 CellTags->"FeynCalcToLaTeX"],
Cell[CellGroupData[{
Cell[765291, 30731, 158, 4, 26, "Input"],
Cell[765452, 30737, 477, 18, 51, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[765966, 30760, 93, 2, 26, "Input"],
Cell[766062, 30764, 531, 9, 127, "Output"]
}, Open  ]]
}, Open  ]],
Cell[766620, 30777, 79, 1, 39, "ObjectName",
 CellTags->"FeynmanParameterNames"],
Cell[CellGroupData[{
Cell[766724, 30782, 72, 1, 36, "Subsubsection",
 CellTags->"FeynmanParameterNames"],
Cell[766799, 30785, 143, 4, 24, "Text",
 CellTags->"FeynmanParameterNames"],
Cell[766945, 30791, 546, 18, 24, "Text",
 CellTags->"FeynmanParameterNames"]
}, Open  ]],
Cell[767506, 30812, 90, 1, 39, "ObjectName",
 CellTags->"FeynmanParametrize"],
Cell[CellGroupData[{
Cell[767621, 30817, 69, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize"],
Cell[767693, 30820, 176, 4, 24, "Text",
 CellTags->"FeynmanParametrize"],
Cell[CellGroupData[{
Cell[767894, 30828, 135, 3, 26, "Input",
 CellTags->"FeynmanParametrize"],
Cell[768032, 30833, 251, 8, 26, "Output",
 CellTags->"FeynmanParametrize"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[768332, 30847, 66, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize"],
Cell[768401, 30850, 84, 2, 26, "Input",
 CellTags->"FeynmanParametrize"]
}, Open  ]],
Cell[768500, 30855, 53, 1, 39, "ObjectName",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[768578, 30860, 59, 1, 36, "Subsubsection",
 CellTags->"FeynRule"],
Cell[768640, 30863, 168, 4, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[768833, 30871, 115, 3, 26, "Input",
 CellTags->"FeynRule"],
Cell[768951, 30876, 630, 16, 47, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[769596, 30895, 92, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[769691, 30898, 133, 4, 24, "Text",
 CellTags->"FeynRule"],
Cell[769827, 30904, 269, 11, 24, "Text",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[770133, 30920, 56, 1, 36, "Subsubsection",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[770214, 30925, 146, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[770363, 30931, 680, 23, 42, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[771080, 30959, 146, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[771229, 30965, 1003, 36, 42, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[772269, 31006, 111, 3, 26, "Input",
 CellTags->"FeynRule"],
Cell[772383, 31011, 4554, 176, 42, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[776974, 31192, 141, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[777118, 31198, 2656, 113, 30, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[779789, 31314, 74, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[779888, 31319, 503, 16, 26, "Input",
 CellTags->"FeynRule"],
Cell[780394, 31337, 700, 30, 30, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[781131, 31372, 307, 9, 26, "Input",
 CellTags->"FeynRule"],
Cell[781441, 31383, 3795, 138, 49, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[785251, 31524, 72, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[785348, 31529, 503, 16, 26, "Input",
 CellTags->"FeynRule"],
Cell[785854, 31547, 700, 30, 30, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[786591, 31582, 307, 9, 26, "Input",
 CellTags->"FeynRule"],
Cell[786901, 31593, 4465, 151, 49, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[791403, 31749, 218, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[791624, 31758, 1044, 42, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[792683, 31803, 110, 3, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[792818, 31810, 360, 10, 26, "Input",
 CellTags->"FeynRule"],
Cell[793181, 31822, 1108, 45, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[794304, 31870, 77, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[794406, 31875, 145, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[794554, 31881, 618, 22, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[795187, 31906, 90, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[795302, 31911, 145, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[795450, 31917, 618, 22, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[796105, 31944, 696, 21, 46, "Input",
 CellTags->"FeynRule"],
Cell[796804, 31967, 893, 37, 30, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[797734, 32009, 227, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[797964, 32018, 4957, 177, 114, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[802958, 32200, 420, 12, 26, "Input",
 CellTags->"FeynRule"],
Cell[803381, 32214, 4764, 159, 78, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[808182, 32378, 362, 12, 26, "Input",
 CellTags->"FeynRule"],
Cell[808547, 32392, 104, 3, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[808666, 32398, 210, 5, 41, "Text",
 CellTags->"FeynRule"],
Cell[808879, 32405, 144, 4, 26, "Input",
 CellTags->"FeynRule"],
Cell[809026, 32411, 51, 1, 24, "Text",
 CellTags->"FeynRule"],
Cell[CellGroupData[{
Cell[809102, 32416, 693, 21, 46, "Input",
 CellTags->"FeynRule"],
Cell[809798, 32439, 984, 42, 30, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[810819, 32486, 212, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[811034, 32495, 1971, 79, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[813042, 32579, 385, 11, 26, "Input",
 CellTags->"FeynRule"],
Cell[813430, 32592, 1419, 57, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[814886, 32654, 197, 6, 26, "Input",
 CellTags->"FeynRule"],
Cell[815086, 32662, 104, 3, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[815227, 32670, 884, 26, 46, "Input",
 CellTags->"FeynRule"],
Cell[816114, 32698, 1270, 54, 32, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[817421, 32757, 212, 7, 26, "Input",
 CellTags->"FeynRule"],
Cell[817636, 32766, 3305, 131, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[820978, 32902, 461, 13, 26, "Input",
 CellTags->"FeynRule"],
Cell[821442, 32917, 3476, 132, 28, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[CellGroupData[{
Cell[824955, 33054, 197, 6, 26, "Input",
 CellTags->"FeynRule"],
Cell[825155, 33062, 104, 3, 26, "Output",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[825274, 33068, 418, 9, 46, "Input",
 CellTags->"FeynRule"]
}, Open  ]],
Cell[825707, 33080, 41, 1, 39, "ObjectName",
 CellTags->"FI"],
Cell[CellGroupData[{
Cell[825773, 33085, 53, 1, 36, "Subsubsection",
 CellTags->"FI"],
Cell[825829, 33088, 195, 4, 41, "Text",
 CellTags->"FI"],
Cell[826027, 33094, 887, 32, 24, "Text",
 CellTags->"FI"]
}, Open  ]],
Cell[826929, 33129, 67, 1, 39, "ObjectName",
 CellTags->"FieldDerivative"],
Cell[CellGroupData[{
Cell[827021, 33134, 66, 1, 36, "Subsubsection",
 CellTags->"FieldDerivative"],
Cell[827090, 33137, 549, 9, 75, "Text",
 CellTags->"FieldDerivative"],
Cell[827642, 33148, 484, 18, 24, "Text",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[828163, 33171, 63, 1, 36, "Subsubsection",
 CellTags->"FieldDerivative"],
Cell[CellGroupData[{
Cell[828251, 33176, 620, 19, 26, "Input",
 CellTags->"FieldDerivative"],
Cell[828874, 33197, 465, 17, 29, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[829376, 33219, 218, 6, 26, "Input",
 CellTags->"FieldDerivative"],
Cell[829597, 33227, 1529, 55, 30, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[831163, 33287, 218, 6, 26, "Input",
 CellTags->"FieldDerivative"],
Cell[831384, 33295, 4269, 153, 54, "Output",
 CellTags->"FieldDerivative"]
}, Open  ]]
}, Open  ]],
Cell[835680, 33452, 63, 1, 39, "ObjectName",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[835768, 33457, 64, 1, 36, "Subsubsection",
 CellTags->"FieldStrength"],
Cell[835835, 33460, 1285, 37, 62, "Text",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[837145, 33501, 125, 3, 26, "Input",
 CellTags->"FieldStrength"],
Cell[837273, 33506, 529, 15, 47, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[837817, 33524, 282, 11, 24, "Text",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[838136, 33540, 61, 1, 36, "Subsubsection",
 CellTags->"FieldStrength"],
Cell[CellGroupData[{
Cell[838222, 33545, 150, 4, 26, "Input",
 CellTags->"FieldStrength"],
Cell[838375, 33551, 174, 5, 30, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[838586, 33561, 160, 4, 26, "Input",
 CellTags->"FieldStrength"],
Cell[838749, 33567, 168, 5, 30, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[838954, 33577, 200, 5, 26, "Input",
 CellTags->"FieldStrength"],
Cell[839157, 33584, 722, 30, 30, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[839916, 33619, 210, 5, 26, "Input",
 CellTags->"FieldStrength"],
Cell[840129, 33626, 1507, 65, 30, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[841673, 33696, 241, 6, 26, "Input",
 CellTags->"FieldStrength"],
Cell[841917, 33704, 547, 15, 58, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[842501, 33724, 244, 6, 26, "Input",
 CellTags->"FieldStrength"],
Cell[842748, 33732, 545, 15, 58, "Output",
 CellTags->"FieldStrength"]
}, Open  ]],
Cell[CellGroupData[{
Cell[843330, 33752, 291, 7, 26, "Input",
 CellTags->"FieldStrength"],
Cell[843624, 33761, 1553, 66, 30, "Output",
 CellTags->"FieldStrength"]
}, Open  ]]
}, Open  ]],
Cell[845204, 33831, 63, 1, 39, "ObjectName",
 CellTags->"FinalFunction"],
Cell[CellGroupData[{
Cell[845292, 33836, 64, 1, 36, "Subsubsection",
 CellTags->"FinalFunction"],
Cell[845359, 33839, 86, 1, 24, "Text",
 CellTags->"FinalFunction"],
Cell[845448, 33842, 274, 11, 24, "Text",
 CellTags->"FinalFunction"]
}, Open  ]],
Cell[845737, 33856, 73, 1, 39, "ObjectName",
 CellTags->"FinalSubstitutions"],
Cell[CellGroupData[{
Cell[845835, 33861, 69, 1, 36, "Subsubsection",
 CellTags->"FinalSubstitutions"],
Cell[845907, 33864, 204, 4, 41, "Text",
 CellTags->"FinalSubstitutions"],
Cell[846114, 33870, 651, 25, 24, "Text",
 CellTags->"FinalSubstitutions"]
}, Open  ]],
Cell[846780, 33898, 45, 1, 39, "ObjectName",
 CellTags->"FORM"],
Cell[CellGroupData[{
Cell[846850, 33903, 55, 1, 36, "Subsubsection",
 CellTags->"FORM"],
Cell[846908, 33906, 204, 5, 41, "Text",
 CellTags->"FORM"],
Cell[847115, 33913, 449, 18, 24, "Text",
 CellTags->"FORM"]
}, Open  ]],
Cell[847579, 33934, 57, 1, 39, "ObjectName",
 CellTags->"FORMEpilog"],
Cell[CellGroupData[{
Cell[847661, 33939, 61, 1, 36, "Subsubsection",
 CellTags->"FORMEpilog"],
Cell[847725, 33942, 161, 4, 24, "Text",
 CellTags->"FORMEpilog"],
Cell[847889, 33948, 483, 18, 24, "Text",
 CellTags->"FORMEpilog"]
}, Open  ]],
Cell[848387, 33969, 57, 1, 39, "ObjectName",
 CellTags->"FORMProlog"],
Cell[CellGroupData[{
Cell[848469, 33974, 61, 1, 36, "Subsubsection",
 CellTags->"FORMProlog"],
Cell[848533, 33977, 178, 4, 24, "Text",
 CellTags->"FORMProlog"],
Cell[848714, 33983, 483, 18, 24, "Text",
 CellTags->"FORMProlog"]
}, Open  ]],
Cell[849212, 34004, 63, 1, 39, "ObjectName",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[849300, 34009, 64, 1, 36, "Subsubsection",
 CellTags->"FORM2FeynCalc"],
Cell[849367, 34012, 470, 8, 58, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[849840, 34022, 286, 11, 24, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[850151, 34037, 125, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[850279, 34042, 662, 18, 47, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[850990, 34066, 61, 1, 36, "Subsubsection",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[851076, 34071, 139, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[851218, 34076, 354, 14, 28, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[851609, 34095, 114, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[851726, 34100, 243, 8, 38, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[851984, 34111, 159, 4, 24, "Text",
 CellTags->"FORM2FeynCalc"],
Cell[CellGroupData[{
Cell[852168, 34119, 314, 8, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[852485, 34129, 298, 10, 26, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[852820, 34144, 114, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[852937, 34149, 303, 9, 38, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[853277, 34163, 150, 4, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[853430, 34169, 331, 12, 26, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[853798, 34186, 117, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[853918, 34191, 259, 9, 26, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[854214, 34205, 150, 4, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[854367, 34211, 257, 9, 28, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[854661, 34225, 114, 3, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[854778, 34230, 307, 10, 38, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[855122, 34245, 333, 9, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[855458, 34256, 269, 9, 28, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]],
Cell[CellGroupData[{
Cell[855764, 34270, 195, 5, 26, "Input",
 CellTags->"FORM2FeynCalc"],
Cell[855962, 34277, 3922, 121, 71, "Output",
 CellTags->"FORM2FeynCalc"]
}, Open  ]]
}, Open  ]],
Cell[859911, 34402, 65, 1, 39, "ObjectName",
 CellTags->"FourDivergence"],
Cell[CellGroupData[{
Cell[860001, 34407, 65, 1, 36, "Subsubsection",
 CellTags->"FourDivergence"],
Cell[860069, 34410, 242, 5, 41, "Text",
 CellTags->"FourDivergence"],
Cell[860314, 34417, 283, 11, 24, "Text",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[860634, 34433, 62, 1, 36, "Subsubsection",
 CellTags->"FourDivergence"],
Cell[CellGroupData[{
Cell[860721, 34438, 167, 5, 26, "Input",
 CellTags->"FourDivergence"],
Cell[860891, 34445, 212, 8, 28, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[861140, 34458, 211, 6, 26, "Input",
 CellTags->"FourDivergence"],
Cell[861354, 34466, 242, 10, 28, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[861633, 34481, 192, 6, 26, "Input",
 CellTags->"FourDivergence"],
Cell[861828, 34489, 247, 9, 28, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[CellGroupData[{
Cell[862112, 34503, 237, 7, 26, "Input",
 CellTags->"FourDivergence"],
Cell[862352, 34512, 110, 3, 26, "Output",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[862477, 34518, 112, 3, 26, "Input",
 CellTags->"FourDivergence"]
}, Open  ]],
Cell[862604, 34524, 63, 1, 39, "ObjectName",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[862692, 34529, 64, 1, 36, "Subsubsection",
 CellTags->"FourLaplacian"],
Cell[862759, 34532, 426, 16, 27, "Text",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[863210, 34552, 125, 3, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[863338, 34557, 170, 5, 26, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[863523, 34565, 498, 18, 24, "Text",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[864058, 34588, 61, 1, 36, "Subsubsection",
 CellTags->"FourLaplacian"],
Cell[CellGroupData[{
Cell[864144, 34593, 131, 4, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[864278, 34599, 167, 6, 28, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[864482, 34610, 152, 4, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[864637, 34616, 132, 4, 26, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[864806, 34625, 264, 9, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[865073, 34636, 440, 16, 48, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[865550, 34657, 157, 4, 26, "Input",
 CellTags->"FourLaplacian"],
Cell[865710, 34663, 3857, 136, 96, "Output",
 CellTags->"FourLaplacian"]
}, Open  ]]
}, Open  ]],
Cell[869594, 34803, 57, 1, 39, "ObjectName",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[869676, 34808, 61, 1, 36, "Subsubsection",
 CellTags->"FourVector"],
Cell[869740, 34811, 339, 10, 41, "Text",
 CellTags->"FourVector"],
Cell[870082, 34823, 587, 25, 24, "Text",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[870706, 34853, 58, 1, 36, "Subsubsection",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[870789, 34858, 140, 4, 26, "Input",
 CellTags->"FourVector"],
Cell[870932, 34864, 132, 4, 29, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[871101, 34873, 164, 5, 26, "Input",
 CellTags->"FourVector"],
Cell[871268, 34880, 249, 10, 30, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[871554, 34895, 180, 5, 26, "Input",
 CellTags->"FourVector"],
Cell[871737, 34902, 155, 4, 38, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[871929, 34911, 229, 6, 26, "Input",
 CellTags->"FourVector"],
Cell[872161, 34919, 203, 5, 38, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[872401, 34929, 212, 6, 26, "Input",
 CellTags->"FourVector"],
Cell[872616, 34937, 155, 4, 38, "Output",
 CellTags->"FourVector"]
}, Open  ]],
Cell[872786, 34944, 193, 4, 24, "Text",
 CellTags->"FourVector"],
Cell[CellGroupData[{
Cell[873004, 34952, 212, 6, 26, "Input",
 CellTags->"FourVector"],
Cell[873219, 34960, 407, 18, 28, "Output",
 CellTags->"FourVector"]
}, Open  ]]
}, Open  ]],
Cell[873653, 34982, 55, 1, 39, "ObjectName",
 CellTags->"FreeIndex"],
Cell[CellGroupData[{
Cell[873733, 34987, 60, 1, 36, "Subsubsection",
 CellTags->"FreeIndex"],
Cell[873796, 34990, 152, 4, 24, "Text",
 CellTags->"FreeIndex"],
Cell[873951, 34996, 454, 18, 24, "Text",
 CellTags->"FreeIndex"]
}, Open  ]],
Cell[874420, 35017, 49, 1, 39, "ObjectName",
 CellTags->"FreeQ2"],
Cell[CellGroupData[{
Cell[874494, 35022, 57, 1, 36, "Subsubsection",
 CellTags->"FreeQ2"],
Cell[874554, 35025, 228, 5, 41, "Text",
 CellTags->"FreeQ2"],
Cell[874785, 35032, 479, 18, 24, "Text",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[875301, 35055, 54, 1, 36, "Subsubsection",
 CellTags->"FreeQ2"],
Cell[CellGroupData[{
Cell[875380, 35060, 246, 8, 26, "Input",
 CellTags->"FreeQ2"],
Cell[875629, 35070, 106, 3, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[875772, 35078, 241, 8, 26, "Input",
 CellTags->"FreeQ2"],
Cell[876016, 35088, 105, 3, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[876158, 35096, 133, 4, 26, "Input",
 CellTags->"FreeQ2"],
Cell[876294, 35102, 105, 3, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[876436, 35110, 162, 5, 26, "Input",
 CellTags->"FreeQ2"],
Cell[876601, 35117, 106, 3, 26, "Output",
 CellTags->"FreeQ2"]
}, Open  ]]
}, Open  ]],
Cell[876734, 35124, 43, 1, 39, "ObjectName",
 CellTags->"FRH"],
Cell[CellGroupData[{
Cell[876802, 35129, 54, 1, 36, "Subsubsection",
 CellTags->"FRH"],
Cell[876859, 35132, 134, 4, 24, "Text",
 CellTags->"FRH"],
Cell[876996, 35138, 252, 11, 24, "Text",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[877285, 35154, 51, 1, 36, "Subsubsection",
 CellTags->"FRH"],
Cell[CellGroupData[{
Cell[877361, 35159, 200, 6, 26, "Input",
 CellTags->"FRH"],
Cell[877564, 35167, 248, 9, 26, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[877849, 35181, 99, 3, 26, "Input",
 CellTags->"FRH"],
Cell[877951, 35186, 99, 3, 26, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[878087, 35194, 304, 10, 26, "Input",
 CellTags->"FRH"],
Cell[878394, 35206, 531, 20, 26, "Output",
 CellTags->"FRH"]
}, Open  ]],
Cell[CellGroupData[{
Cell[878962, 35231, 128, 4, 26, "Input",
 CellTags->"FRH"],
Cell[879093, 35237, 128, 4, 26, "Output",
 CellTags->"FRH"]
}, Open  ]]
}, Open  ]],
Cell[879248, 35245, 51, 1, 39, "ObjectName",
 CellTags->"FromTFi"],
Cell[CellGroupData[{
Cell[879324, 35250, 58, 1, 36, "Subsubsection",
 CellTags->"FromTFi"],
Cell[879385, 35253, 200, 5, 41, "Text",
 CellTags->"FromTFi"],
Cell[879588, 35260, 414, 17, 24, "Text",
 CellTags->"FromTFI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[880039, 35282, 55, 1, 36, "Subsubsection",
 CellTags->"FromTFi"],
Cell[CellGroupData[{
Cell[880119, 35287, 431, 14, 26, "Input",
 CellTags->"FromTFi"],
Cell[880553, 35303, 776, 24, 92, "Output",
 CellTags->"FromTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[881366, 35332, 693, 22, 26, "Input",
 CellTags->"FromTFi"],
Cell[882062, 35356, 966, 31, 92, "Output",
 CellTags->"FromTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[883065, 35392, 756, 24, 26, "Input",
 CellTags->"FromTFi"],
Cell[883824, 35418, 1027, 33, 92, "Output",
 CellTags->"FromTFi"]
}, Open  ]]
}, Open  ]],
Cell[884878, 35455, 53, 1, 39, "ObjectName",
 CellTags->"FUNCTION"],
Cell[CellGroupData[{
Cell[884956, 35460, 59, 1, 36, "Subsubsection",
 CellTags->"FUNCTION"],
Cell[885018, 35463, 160, 4, 24, "Text",
 CellTags->"FUNCTION"],
Cell[885181, 35469, 253, 11, 24, "Text",
 CellTags->"FUNCTION"]
}, Open  ]],
Cell[885449, 35483, 59, 1, 39, "ObjectName",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[885533, 35488, 62, 1, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[885598, 35491, 416, 7, 58, "Text",
 CellTags->"FunctionalD"],
Cell[886017, 35500, 95, 1, 24, "Text",
 CellTags->"FunctionalD"],
Cell[886115, 35503, 472, 18, 24, "Text",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[886624, 35526, 59, 1, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[886686, 35529, 610, 20, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[887321, 35553, 228, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[887552, 35561, 107, 3, 26, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[887696, 35569, 253, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[887952, 35578, 135, 4, 26, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[888102, 35585, 1269, 44, 51, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[889396, 35633, 295, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[889694, 35643, 273, 10, 33, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[889982, 35656, 778, 24, 27, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[890785, 35684, 668, 22, 46, "Input",
 CellTags->"FunctionalD"],
Cell[891456, 35708, 735, 26, 42, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[892228, 35739, 217, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[892448, 35747, 652, 26, 29, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[893115, 35776, 489, 15, 28, "Text",
 CellTags->"FunctionalD"],
Cell[893607, 35793, 97, 2, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[893729, 35799, 315, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[894047, 35809, 1303, 56, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[895387, 35870, 315, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[895705, 35880, 1303, 56, 32, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[897045, 35941, 216, 8, 26, "Input",
 CellTags->"FunctionalD"],
Cell[897264, 35951, 2956, 115, 42, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[900235, 36069, 560, 13, 61, "Text",
 CellTags->"FunctionalD"],
Cell[900798, 36084, 1398, 50, 27, "Text",
 CellTags->"FunctionalD"],
Cell[902199, 36136, 445, 13, 26, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[902669, 36153, 231, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[902903, 36162, 286, 12, 28, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[903226, 36179, 165, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[903394, 36186, 2288, 96, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[905697, 36285, 120, 3, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[905842, 36292, 176, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[906021, 36299, 2288, 96, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[908324, 36398, 290, 8, 26, "Text",
 CellTags->"FunctionalD"],
Cell[908617, 36408, 523, 16, 26, "Input",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[909165, 36428, 230, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[909398, 36437, 384, 17, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[909819, 36459, 240, 7, 26, "Input",
 CellTags->"FunctionalD"],
Cell[910062, 36468, 416, 18, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[910515, 36491, 579, 16, 46, "Input",
 CellTags->"FunctionalD"],
Cell[911097, 36509, 2958, 127, 42, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[914092, 36641, 184, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[914279, 36648, 1405, 62, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[915721, 36715, 352, 11, 26, "Input",
 CellTags->"FunctionalD"],
Cell[916076, 36728, 1159, 50, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[917250, 36781, 269, 9, 27, "Text",
 CellTags->"FunctionalD"],
Cell[917522, 36792, 98, 2, 24, "Text",
 CellTags->"FunctionalD"],
Cell[917623, 36796, 167, 4, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[917815, 36804, 105, 3, 26, "Input",
 CellTags->"FunctionalD"],
Cell[917923, 36809, 2956, 115, 42, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[920916, 36929, 194, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[921113, 36937, 23483, 1005, 156, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[944611, 37945, 227, 5, 41, "Text",
 CellTags->"FunctionalD"],
Cell[944841, 37952, 345, 6, 58, "Text",
 CellTags->"FunctionalD"],
Cell[945189, 37960, 132, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[945324, 37966, 2583, 67, 122, "Input",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[947932, 38037, 169, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[948104, 38044, 12210, 526, 80, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[960351, 38575, 164, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[960518, 38582, 6535, 286, 42, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[967068, 38871, 212, 8, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[967305, 38883, 315, 11, 26, "Input",
 CellTags->"FunctionalD"],
Cell[967623, 38896, 2734, 119, 42, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[970394, 39020, 647, 20, 26, "Input",
 CellTags->"FunctionalD"],
Cell[971044, 39042, 1310, 60, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[972391, 39107, 154, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[972548, 39113, 5227, 228, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[977790, 39344, 131, 4, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[977946, 39352, 79, 2, 26, "Input",
 CellTags->"FunctionalD"],
Cell[978028, 39356, 1159, 50, 30, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[979224, 39411, 798, 21, 46, "Input",
 CellTags->"FunctionalD"],
Cell[980025, 39434, 1064, 41, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[981126, 39480, 137, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[981266, 39486, 4174, 167, 34, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[985477, 39658, 204, 6, 26, "Input",
 CellTags->"FunctionalD"],
Cell[985684, 39666, 6098, 267, 32, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[991819, 39938, 138, 4, 26, "Input",
 CellTags->"FunctionalD"],
Cell[991960, 39944, 11189, 491, 56, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1003186, 40440, 163, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[1003352, 40447, 10243, 451, 56, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[1013610, 40901, 66, 1, 24, "Text",
 CellTags->"FunctionalD"],
Cell[1013679, 40904, 539, 13, 46, "Input",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1014255, 40922, 110, 2, 36, "Subsubsection",
 CellTags->"FunctionalD"],
Cell[1014368, 40926, 79, 1, 24, "Text",
 CellTags->"FunctionalD"],
Cell[CellGroupData[{
Cell[1014472, 40931, 1073, 26, 84, "Input"],
Cell[1015548, 40959, 1337, 59, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1016922, 41023, 765, 22, 46, "Input",
 CellTags->"FunctionalD"],
Cell[1017690, 41047, 8897, 329, 91, "Output",
 CellTags->"FunctionalD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1026624, 41381, 165, 5, 26, "Input",
 CellTags->"FunctionalD"],
Cell[1026792, 41388, 2097, 84, 42, "Output",
 CellTags->"FunctionalD"]
}, Open  ]]
}, Open  ]],
Cell[1028916, 41476, 41, 1, 39, "ObjectName",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[1028982, 41481, 53, 1, 36, "Subsubsection",
 CellTags->"FV"],
Cell[1029038, 41484, 275, 13, 26, "Text",
 CellTags->"FV"],
Cell[1029316, 41499, 955, 39, 24, "Text",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1030308, 41543, 50, 1, 36, "Subsubsection",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[1030383, 41548, 124, 4, 26, "Input",
 CellTags->"FV"],
Cell[1030510, 41554, 126, 4, 28, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1030673, 41563, 148, 5, 26, "Input",
 CellTags->"FV"],
Cell[1030824, 41570, 185, 7, 28, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1031046, 41582, 160, 5, 26, "Input",
 CellTags->"FV"],
Cell[1031209, 41589, 139, 4, 38, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1031385, 41598, 195, 7, 26, "Input",
 CellTags->"FV"],
Cell[1031583, 41607, 217, 6, 38, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[1031815, 41616, 130, 4, 24, "Text",
 CellTags->"FV"],
Cell[CellGroupData[{
Cell[1031970, 41624, 196, 6, 26, "Input",
 CellTags->"FV"],
Cell[1032169, 41632, 399, 18, 28, "Output",
 CellTags->"FV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1032605, 41655, 107, 3, 26, "Input",
 CellTags->"FV"],
Cell[1032715, 41660, 380, 11, 38, "Output",
 CellTags->"FV"]
}, Open  ]]
}, Open  ]],
Cell[1033122, 41675, 43, 1, 39, "ObjectName",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[1033190, 41680, 54, 1, 36, "Subsubsection",
 CellTags->"FVD"],
Cell[1033247, 41683, 312, 15, 22, "Text",
 CellTags->"FVD"],
Cell[1033562, 41700, 952, 39, 22, "Text",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1034551, 41744, 51, 1, 36, "Subsubsection",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[1034627, 41749, 126, 4, 26, "Input",
 CellTags->"FVD"],
Cell[1034756, 41755, 127, 4, 28, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1034920, 41764, 150, 5, 26, "Input",
 CellTags->"FVD"],
Cell[1035073, 41771, 186, 7, 28, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1035296, 41783, 162, 5, 26, "Input",
 CellTags->"FVD"],
Cell[1035461, 41790, 141, 4, 38, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1035639, 41799, 197, 7, 26, "Input",
 CellTags->"FVD"],
Cell[1035839, 41808, 268, 8, 38, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[1036122, 41819, 88, 1, 24, "Text",
 CellTags->"FVD"],
Cell[CellGroupData[{
Cell[1036235, 41824, 198, 6, 26, "Input",
 CellTags->"FVD"],
Cell[1036436, 41832, 400, 18, 28, "Output",
 CellTags->"FVD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1036873, 41855, 108, 3, 26, "Input",
 CellTags->"FVD"],
Cell[1036984, 41860, 485, 15, 38, "Output",
 CellTags->"FVD"]
}, Open  ]]
}, Open  ]],
Cell[1037496, 41879, 41, 1, 39, "ObjectName",
 CellTags->"GA"],
Cell[CellGroupData[{
Cell[1037562, 41884, 53, 1, 28, "Subsubsection",
 CellTags->"GA"],
Cell[1037618, 41887, 654, 24, 40, "Text",
 CellTags->"GA"],
Cell[1038275, 41913, 607, 25, 22, "Text",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1038919, 41943, 50, 1, 36, "Subsubsection",
 CellTags->"GA"],
Cell[CellGroupData[{
Cell[1038994, 41948, 101, 3, 26, "Input",
 CellTags->"GA"],
Cell[1039098, 41953, 133, 4, 26, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1039268, 41962, 213, 7, 26, "Input",
 CellTags->"GA"],
Cell[1039484, 41971, 317, 10, 26, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1039838, 41986, 171, 5, 26, "Input",
 CellTags->"GA"],
Cell[1040012, 41993, 163, 4, 38, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1040212, 42002, 163, 5, 26, "Input",
 CellTags->"GA"],
Cell[1040378, 42009, 289, 8, 26, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1040704, 42022, 203, 6, 26, "Input",
 CellTags->"GA"],
Cell[1040910, 42030, 262, 7, 38, "Output",
 CellTags->"GA"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1041209, 42042, 250, 8, 26, "Input",
 CellTags->"GA"],
Cell[1041462, 42052, 301, 9, 28, "Output",
 CellTags->"GA"]
}, Open  ]]
}, Open  ]],
Cell[1041790, 42065, 43, 1, 39, "ObjectName",
 CellTags->"GAD"],
Cell[CellGroupData[{
Cell[1041858, 42070, 54, 1, 28, "Subsubsection",
 CellTags->"GAD"],
Cell[1041915, 42073, 671, 24, 40, "Text",
 CellTags->"GAD"],
Cell[1042589, 42099, 604, 25, 22, "Text",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1043230, 42129, 51, 1, 36, "Subsubsection",
 CellTags->"GAD"],
Cell[CellGroupData[{
Cell[1043306, 42134, 103, 3, 26, "Input",
 CellTags->"GAD"],
Cell[1043412, 42139, 134, 4, 26, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1043583, 42148, 216, 7, 26, "Input",
 CellTags->"GAD"],
Cell[1043802, 42157, 318, 10, 26, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1044157, 42172, 173, 5, 26, "Input",
 CellTags->"GAD"],
Cell[1044333, 42179, 213, 6, 38, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1044583, 42190, 165, 5, 26, "Input",
 CellTags->"GAD"],
Cell[1044751, 42197, 290, 8, 26, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1045078, 42210, 205, 6, 26, "Input",
 CellTags->"GAD"],
Cell[1045286, 42218, 267, 7, 38, "Output",
 CellTags->"GAD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1045590, 42230, 254, 8, 26, "Input",
 CellTags->"GAD"],
Cell[1045847, 42240, 302, 9, 28, "Output",
 CellTags->"GAD"]
}, Open  ]]
}, Open  ]],
Cell[1046176, 42253, 61, 1, 39, "ObjectName",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[1046262, 42258, 63, 1, 36, "Subsubsection",
 CellTags->"GammaEpsilon"],
Cell[1046328, 42261, 167, 4, 24, "Text",
 CellTags->"GammaEpsilon"],
Cell[1046498, 42267, 465, 18, 24, "Text",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1047000, 42290, 60, 1, 36, "Subsubsection",
 CellTags->"GammaEpsilon"],
Cell[1047063, 42293, 141, 4, 24, "Text",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[1047229, 42301, 170, 5, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1047402, 42308, 1361, 48, 48, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1048800, 42361, 170, 5, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1048973, 42368, 1109, 41, 48, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[1050097, 42412, 91, 1, 24, "Text",
 CellTags->"GammaEpsilon"],
Cell[CellGroupData[{
Cell[1050213, 42417, 123, 3, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1050339, 42422, 2148, 77, 93, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1052524, 42504, 117, 3, 26, "Input",
 CellTags->"GammaEpsilon"],
Cell[1052644, 42509, 138, 5, 26, "Output",
 CellTags->"GammaEpsilon"]
}, Open  ]]
}, Open  ]],
Cell[1052809, 42518, 59, 1, 39, "ObjectName",
 CellTags->"GammaExpand"],
Cell[CellGroupData[{
Cell[1052893, 42523, 62, 1, 36, "Subsubsection",
 CellTags->"GammaExpand"],
Cell[1052958, 42526, 123, 3, 24, "Text",
 CellTags->"GammaExpand"],
Cell[1053084, 42531, 280, 11, 24, "Text",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1053401, 42547, 59, 1, 36, "Subsubsection",
 CellTags->"GammaExpand"],
Cell[CellGroupData[{
Cell[1053485, 42552, 187, 5, 26, "Input",
 CellTags->"GammaExpand"],
Cell[1053675, 42559, 329, 11, 26, "Output",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1054041, 42575, 197, 6, 26, "Input",
 CellTags->"GammaExpand"],
Cell[1054241, 42583, 632, 22, 45, "Output",
 CellTags->"GammaExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1054910, 42610, 187, 5, 26, "Input",
 CellTags->"GammaExpand"],
Cell[1055100, 42617, 203, 6, 26, "Output",
 CellTags->"GammaExpand"]
}, Open  ]]
}, Open  ]],
Cell[1055330, 42627, 49, 1, 39, "ObjectName",
 CellTags->"Gamma1"],
Cell[CellGroupData[{
Cell[1055404, 42632, 57, 1, 36, "Subsubsection",
 CellTags->"Gamma1"],
Cell[1055464, 42635, 139, 4, 24, "Text",
 CellTags->"Gamma1"],
Cell[1055606, 42641, 435, 18, 24, "Text",
 CellTags->"Gamma1"]
}, Open  ]],
Cell[1056056, 42662, 49, 1, 39, "ObjectName",
 CellTags->"Gamma2"],
Cell[CellGroupData[{
Cell[1056130, 42667, 57, 1, 36, "Subsubsection",
 CellTags->"Gamma2"],
Cell[1056190, 42670, 172, 4, 24, "Text",
 CellTags->"Gamma2"],
Cell[1056365, 42676, 435, 18, 24, "Text",
 CellTags->"Gamma2"]
}, Open  ]],
Cell[1056815, 42697, 49, 1, 39, "ObjectName",
 CellTags->"Gamma3"],
Cell[CellGroupData[{
Cell[1056889, 42702, 57, 1, 36, "Subsubsection",
 CellTags->"Gamma3"],
Cell[1056949, 42705, 237, 5, 41, "Text",
 CellTags->"Gamma3"],
Cell[1057189, 42712, 435, 18, 24, "Text",
 CellTags->"Gamma3"]
}, Open  ]],
Cell[1057639, 42733, 47, 1, 39, "ObjectName",
 CellTags->"Gauge"],
Cell[CellGroupData[{
Cell[1057711, 42738, 56, 1, 36, "Subsubsection",
 CellTags->"Gauge"],
Cell[1057770, 42741, 133, 4, 24, "Text",
 CellTags->"Gauge"],
Cell[1057906, 42747, 470, 18, 24, "Text",
 CellTags->"Gauge"]
}, Open  ]],
Cell[1058391, 42768, 57, 1, 39, "ObjectName",
 CellTags->"GaugeField"],
Cell[CellGroupData[{
Cell[1058473, 42773, 61, 1, 36, "Subsubsection",
 CellTags->"GaugeField"],
Cell[1058537, 42776, 212, 5, 41, "Text",
 CellTags->"GaugeField"],
Cell[1058752, 42783, 491, 18, 24, "Text",
 CellTags->"GaugeField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1059280, 42806, 58, 1, 36, "Subsubsection",
 CellTags->"GaugeField"],
Cell[CellGroupData[{
Cell[1059363, 42811, 86, 2, 26, "Input",
 CellTags->"GaugeField"],
Cell[1059452, 42815, 106, 3, 27, "Output",
 CellTags->"GaugeField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1059595, 42823, 237, 6, 26, "Input",
 CellTags->"GaugeField"],
Cell[1059835, 42831, 316, 14, 30, "Output",
 CellTags->"GaugeField"]
}, Open  ]]
}, Open  ]],
Cell[1060178, 42849, 51, 1, 39, "ObjectName",
 CellTags->"GaugeXi"],
Cell[CellGroupData[{
Cell[1060254, 42854, 58, 1, 36, "Subsubsection",
 CellTags->"GaugeXi"],
Cell[1060315, 42857, 77, 1, 24, "Text",
 CellTags->"GaugeXi"],
Cell[1060395, 42860, 448, 18, 24, "Text",
 CellTags->"GaugeXi"]
}, Open  ]],
Cell[1060858, 42881, 43, 1, 39, "ObjectName",
 CellTags->"GA5"],
Cell[CellGroupData[{
Cell[1060926, 42886, 54, 1, 36, "Subsubsection",
 CellTags->"GA5"],
Cell[1060983, 42889, 88, 1, 24, "Text",
 CellTags->"GA5"],
Cell[1061074, 42892, 600, 25, 24, "Text",
 CellTags->"GA5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1061711, 42922, 51, 1, 36, "Subsubsection",
 CellTags->"GA5"],
Cell[CellGroupData[{
Cell[1061787, 42927, 72, 2, 26, "Input",
 CellTags->"GA5"],
Cell[1061862, 42931, 130, 4, 26, "Output",
 CellTags->"GA5"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1062029, 42940, 94, 3, 26, "Input",
 CellTags->"GA5"],
Cell[1062126, 42945, 99, 3, 26, "Output",
 CellTags->"GA5"]
}, Open  ]]
}, Open  ]],
Cell[1062252, 42952, 43, 1, 39, "ObjectName",
 CellTags->"GGV"],
Cell[CellGroupData[{
Cell[1062320, 42957, 54, 1, 36, "Subsubsection",
 CellTags->"GGV"],
Cell[1062377, 42960, 72, 1, 24, "Text",
 CellTags->"GGV"],
Cell[1062452, 42963, 288, 11, 24, "Text",
 CellTags->"GGV"]
}, Open  ]],
Cell[1062755, 42977, 67, 1, 39, "ObjectName",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1062847, 42982, 66, 1, 36, "Subsubsection",
 CellTags->"GhostPropagator"],
Cell[1062916, 42985, 324, 9, 24, "Text",
 CellTags->"GhostPropagator"],
Cell[1063243, 42996, 692, 25, 24, "Text",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1063960, 43025, 99, 2, 26, "Input"],
Cell[1064062, 43029, 148, 5, 26, "Output"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1064259, 43040, 63, 1, 36, "Subsubsection",
 CellTags->"GhostPropagator"],
Cell[CellGroupData[{
Cell[1064347, 43045, 156, 4, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1064506, 43051, 193, 6, 28, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1064736, 43062, 206, 5, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1064945, 43069, 409, 16, 48, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1065391, 43090, 123, 3, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1065517, 43095, 176, 6, 28, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1065730, 43106, 188, 5, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1065921, 43113, 413, 16, 48, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1066371, 43134, 285, 9, 26, "Input",
 CellTags->"GhostPropagator"],
Cell[1066659, 43145, 228, 6, 38, "Output",
 CellTags->"GhostPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1066914, 43155, 43, 1, 39, "ObjectName",
 CellTags->"GHP"],
Cell[CellGroupData[{
Cell[1066982, 43160, 54, 1, 36, "Subsubsection",
 CellTags->"GHP"],
Cell[1067039, 43163, 288, 9, 24, "Text",
 CellTags->"GHP"],
Cell[1067330, 43174, 728, 25, 24, "Text",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1068095, 43204, 51, 1, 36, "Subsubsection",
 CellTags->"GHP"],
Cell[CellGroupData[{
Cell[1068171, 43209, 132, 4, 26, "Input",
 CellTags->"GHP"],
Cell[1068306, 43215, 181, 6, 28, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1068524, 43226, 130, 4, 26, "Input",
 CellTags->"GHP"],
Cell[1068657, 43232, 157, 5, 48, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1068851, 43242, 132, 4, 26, "Input",
 CellTags->"GHP"],
Cell[1068986, 43248, 185, 6, 28, "Output",
 CellTags->"GHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1069208, 43259, 264, 9, 26, "Input",
 CellTags->"GHP"],
Cell[1069475, 43270, 216, 6, 38, "Output",
 CellTags->"GHP"]
}, Open  ]]
}, Open  ]],
Cell[1069718, 43280, 57, 1, 39, "ObjectName",
 CellTags->"GluonField"],
Cell[CellGroupData[{
Cell[1069800, 43285, 61, 1, 36, "Subsubsection",
 CellTags->"GluonField"],
Cell[1069864, 43288, 79, 1, 24, "Text",
 CellTags->"GluonField"],
Cell[1069946, 43291, 271, 11, 24, "Text",
 CellTags->"GluonField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1070254, 43307, 58, 1, 36, "Subsubsection",
 CellTags->"GluonField"],
Cell[CellGroupData[{
Cell[1070337, 43312, 86, 2, 26, "Input",
 CellTags->"GluonField"],
Cell[1070426, 43316, 106, 3, 27, "Output",
 CellTags->"GluonField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1070569, 43324, 237, 6, 26, "Input",
 CellTags->"GluonField"],
Cell[1070809, 43332, 316, 14, 30, "Output",
 CellTags->"GluonField"]
}, Open  ]]
}, Open  ]],
Cell[1071152, 43350, 69, 1, 39, "ObjectName",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1071246, 43355, 67, 1, 28, "Subsubsection",
 CellTags->"GluonGhostVertex"],
Cell[1071316, 43358, 422, 7, 52, "Text",
 CellTags->"GluonGhostVertex"],
Cell[1071741, 43367, 935, 31, 24, "Text",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1072701, 43402, 131, 3, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1072835, 43407, 324, 10, 26, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1073208, 43423, 64, 1, 36, "Subsubsection",
 CellTags->"GluonGhostVertex"],
Cell[CellGroupData[{
Cell[1073297, 43428, 348, 10, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1073648, 43440, 517, 21, 26, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1074202, 43466, 86, 2, 26, "Input"],
Cell[1074291, 43470, 578, 26, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1074906, 43501, 121, 3, 26, "Input"],
Cell[1075030, 43506, 181, 6, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1075248, 43517, 86, 2, 26, "Input"],
Cell[1075337, 43521, 286, 12, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1075660, 43538, 148, 4, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1075811, 43544, 333, 10, 38, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1076181, 43559, 191, 5, 26, "Input",
 CellTags->"GluonGhostVertex"],
Cell[1076375, 43566, 521, 21, 30, "Output",
 CellTags->"GluonGhostVertex"]
}, Open  ]]
}, Open  ]],
Cell[1076923, 43591, 67, 1, 39, "ObjectName",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1077015, 43596, 66, 1, 36, "Subsubsection",
 CellTags->"GluonPropagator"],
Cell[1077084, 43599, 535, 10, 58, "Text",
 CellTags->"GluonPropagator"],
Cell[1077622, 43611, 99, 1, 24, "Text",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1077746, 43616, 83, 2, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1077832, 43620, 125, 3, 26, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1077994, 43628, 129, 3, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1078126, 43633, 482, 13, 26, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[1078623, 43649, 944, 32, 24, "Text",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1079604, 43686, 72, 1, 33, "Subsubsection",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1079701, 43691, 187, 5, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1079891, 43698, 233, 8, 28, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1080161, 43711, 189, 5, 26, "Input"],
Cell[1080353, 43718, 477, 18, 48, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1080867, 43741, 271, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1081141, 43750, 1035, 39, 58, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1082213, 43794, 357, 10, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1082573, 43806, 2748, 104, 59, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1085358, 43915, 199, 6, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1085560, 43923, 271, 9, 48, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[1085846, 43935, 149, 4, 24, "Text",
 CellTags->"GluonPropagator"],
Cell[CellGroupData[{
Cell[1086020, 43943, 242, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1086265, 43952, 1028, 39, 60, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1087330, 43996, 267, 9, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1087600, 44007, 314, 9, 38, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1087951, 44021, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1088229, 44030, 1274, 46, 53, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1089540, 44081, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1089818, 44090, 1288, 47, 49, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1091143, 44142, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1091421, 44151, 1282, 46, 53, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1092740, 44202, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1093018, 44211, 1274, 46, 53, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1094329, 44262, 275, 7, 26, "Input",
 CellTags->"GluonPropagator"],
Cell[1094607, 44271, 2422, 88, 53, "Output",
 CellTags->"GluonPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1097056, 44363, 67, 1, 39, "ObjectName",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1097148, 44368, 66, 1, 36, "Subsubsection",
 CellTags->"GluonSelfEnergy"],
Cell[1097217, 44371, 122, 3, 24, "Text",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1097364, 44378, 129, 3, 26, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1097496, 44383, 794, 23, 29, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[1098305, 44409, 944, 32, 24, "Text",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1099286, 44446, 72, 1, 33, "Subsubsection",
 CellTags->"GluonSelfEnergy"],
Cell[CellGroupData[{
Cell[1099383, 44451, 252, 8, 26, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1099638, 44461, 2391, 92, 45, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1102066, 44558, 300, 9, 26, "Input",
 CellTags->"GluonSelfEnergy"],
Cell[1102369, 44569, 2734, 103, 55, "Output",
 CellTags->"GluonSelfEnergy"]
}, Open  ]]
}, Open  ]],
Cell[1105130, 44676, 59, 1, 39, "ObjectName",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1105214, 44681, 62, 1, 28, "Subsubsection",
 CellTags->"GluonVertex"],
Cell[1105279, 44684, 88, 1, 22, "Text",
 CellTags->"GluonVertex"],
Cell[1105370, 44687, 621, 10, 67, "Text",
 CellTags->"GluonVertex"],
Cell[1105994, 44699, 91, 1, 24, "Text",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1106110, 44704, 79, 2, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1106192, 44708, 117, 3, 26, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1106346, 44716, 121, 3, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1106470, 44721, 378, 11, 26, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[1106863, 44735, 516, 18, 24, "Text",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1107416, 44758, 59, 1, 36, "Subsubsection",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1107500, 44763, 338, 10, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1107841, 44775, 943, 42, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1108821, 44822, 379, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1109203, 44836, 1974, 79, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1111214, 44920, 308, 10, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1111525, 44932, 272, 9, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1111834, 44946, 349, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1112186, 44960, 1682, 65, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1113905, 45030, 414, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1114322, 45044, 352, 11, 29, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1114711, 45060, 431, 13, 26, "Input"],
Cell[1115145, 45075, 3453, 131, 28, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1118635, 45211, 374, 12, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1119012, 45225, 160, 5, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1119209, 45235, 259, 7, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1119471, 45244, 3479, 132, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[1122965, 45379, 200, 4, 41, "Text",
 CellTags->"GluonVertex"],
Cell[CellGroupData[{
Cell[1123190, 45387, 172, 5, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1123365, 45394, 942, 42, 28, "Output",
 CellTags->"GluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1124344, 45441, 230, 7, 26, "Input",
 CellTags->"GluonVertex"],
Cell[1124577, 45450, 3431, 132, 52, "Output",
 CellTags->"GluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1128035, 45586, 41, 1, 39, "ObjectName",
 CellTags->"GO"],
Cell[CellGroupData[{
Cell[1128101, 45591, 53, 1, 36, "Subsubsection",
 CellTags->"GO"],
Cell[1128157, 45594, 73, 1, 22, "Text",
 CellTags->"GO"],
Cell[1128233, 45597, 299, 11, 22, "Text",
 CellTags->"GO"]
}, Open  ]],
Cell[1128547, 45611, 41, 1, 39, "ObjectName",
 CellTags->"GP"],
Cell[CellGroupData[{
Cell[1128613, 45616, 53, 1, 28, "Subsubsection",
 CellTags->"GP"],
Cell[1128669, 45619, 69, 1, 22, "Text",
 CellTags->"GP"],
Cell[1128741, 45622, 283, 11, 22, "Text",
 CellTags->"GP"]
}, Open  ]],
Cell[1129039, 45636, 67, 1, 39, "ObjectName",
 CellTags->"GrassmannParity"],
Cell[CellGroupData[{
Cell[1129131, 45641, 66, 1, 28, "Subsubsection",
 CellTags->"GrassmannParity"],
Cell[1129200, 45644, 216, 5, 22, "Text",
 CellTags->"GrassmannParity"],
Cell[1129419, 45651, 268, 11, 22, "Text",
 CellTags->"GrassmannParity"]
}, Open  ]],
Cell[1129702, 45665, 41, 1, 39, "ObjectName",
 CellTags->"GS"],
Cell[CellGroupData[{
Cell[1129768, 45670, 53, 1, 28, "Subsubsection",
 CellTags->"GS"],
Cell[1129824, 45673, 635, 18, 40, "Text",
 CellTags->"GS"],
Cell[1130462, 45693, 803, 32, 22, "Text",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1131302, 45730, 50, 1, 36, "Subsubsection",
 CellTags->"GS"],
Cell[CellGroupData[{
Cell[1131377, 45735, 97, 3, 26, "Input",
 CellTags->"GS"],
Cell[1131477, 45740, 139, 4, 28, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1131653, 45749, 162, 6, 26, "Input",
 CellTags->"GS"],
Cell[1131818, 45757, 155, 4, 38, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1132010, 45766, 140, 4, 26, "Input",
 CellTags->"GS"],
Cell[1132153, 45772, 412, 13, 28, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1132602, 45790, 179, 6, 26, "Input",
 CellTags->"GS"],
Cell[1132784, 45798, 242, 7, 38, "Output",
 CellTags->"GS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1133063, 45810, 237, 8, 26, "Input",
 CellTags->"GS"],
Cell[1133303, 45820, 361, 12, 28, "Output",
 CellTags->"GS"]
}, Open  ]]
}, Open  ]],
Cell[1133691, 45836, 43, 1, 39, "ObjectName",
 CellTags->"GSD"],
Cell[CellGroupData[{
Cell[1133759, 45841, 54, 1, 28, "Subsubsection",
 CellTags->"GSD"],
Cell[1133816, 45844, 644, 18, 40, "Text",
 CellTags->"GSD"],
Cell[1134463, 45864, 804, 32, 22, "Text",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1135304, 45901, 51, 1, 36, "Subsubsection",
 CellTags->"GSD"],
Cell[CellGroupData[{
Cell[1135380, 45906, 99, 3, 26, "Input",
 CellTags->"GSD"],
Cell[1135482, 45911, 140, 4, 28, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1135659, 45920, 164, 6, 26, "Input",
 CellTags->"GSD"],
Cell[1135826, 45928, 205, 6, 38, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1136068, 45939, 142, 4, 26, "Input",
 CellTags->"GSD"],
Cell[1136213, 45945, 413, 13, 28, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1136663, 45963, 181, 6, 26, "Input",
 CellTags->"GSD"],
Cell[1136847, 45971, 247, 7, 38, "Output",
 CellTags->"GSD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1137131, 45983, 241, 8, 26, "Input",
 CellTags->"GSD"],
Cell[1137375, 45993, 362, 12, 28, "Output",
 CellTags->"GSD"]
}, Open  ]]
}, Open  ]],
Cell[1137764, 46009, 51, 1, 39, "ObjectName",
 CellTags->"Gstrong"],
Cell[CellGroupData[{
Cell[1137840, 46014, 58, 1, 28, "Subsubsection",
 CellTags->"Gstrong"],
Cell[1137901, 46017, 83, 1, 22, "Text",
 CellTags->"Gstrong"],
Cell[1137987, 46020, 334, 15, 22, "Text",
 CellTags->"Gstrong"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1138358, 46040, 55, 1, 34, "Subsubsection",
 CellTags->"Gstrong"],
Cell[1138416, 46043, 119, 3, 24, "Text",
 CellTags->"Gstrong"],
Cell[CellGroupData[{
Cell[1138560, 46050, 80, 2, 26, "Input",
 CellTags->"Gstrong"],
Cell[1138643, 46054, 125, 4, 26, "Output",
 CellTags->"Gstrong"]
}, Open  ]]
}, Open  ]],
Cell[1138795, 46062, 43, 1, 39, "ObjectName",
 CellTags->"GTI"],
Cell[CellGroupData[{
Cell[1138863, 46067, 54, 1, 28, "Subsubsection",
 CellTags->"GTI"],
Cell[1138920, 46070, 85, 1, 22, "Text",
 CellTags->"GTI"],
Cell[1139008, 46073, 236, 11, 22, "Text",
 CellTags->"GTI"]
}, Open  ]],
Cell[1139259, 46087, 41, 1, 39, "ObjectName",
 CellTags->"GV"],
Cell[CellGroupData[{
Cell[1139325, 46092, 53, 1, 28, "Subsubsection",
 CellTags->"GV"],
Cell[1139381, 46095, 65, 1, 22, "Text",
 CellTags->"GV"],
Cell[1139449, 46098, 267, 11, 22, "Text",
 CellTags->"GV"]
}, Open  ]],
Cell[1139731, 46112, 45, 1, 39, "ObjectName",
 CellTags->"Hill"],
Cell[CellGroupData[{
Cell[1139801, 46117, 55, 1, 36, "Subsubsection",
 CellTags->"Hill"],
Cell[1139859, 46120, 129, 4, 24, "Text",
 CellTags->"Hill"],
Cell[1139991, 46126, 285, 11, 24, "Text",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1140313, 46142, 52, 1, 36, "Subsubsection",
 CellTags->"Hill"],
Cell[CellGroupData[{
Cell[1140390, 46147, 124, 4, 26, "Input",
 CellTags->"Hill"],
Cell[1140517, 46153, 2145, 73, 132, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1142699, 46231, 270, 8, 26, "Input",
 CellTags->"Hill"],
Cell[1142972, 46241, 100, 3, 26, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1143109, 46249, 262, 9, 26, "Input",
 CellTags->"Hill"],
Cell[1143374, 46260, 985, 32, 45, "Output",
 CellTags->"Hill"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1144396, 46297, 273, 9, 26, "Input",
 CellTags->"Hill"],
Cell[1144672, 46308, 100, 3, 26, "Output",
 CellTags->"Hill"]
}, Open  ]]
}, Open  ]],
Cell[1144799, 46315, 69, 1, 39, "ObjectName",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1144893, 46320, 67, 1, 36, "Subsubsection",
 CellTags->"HypergeometricAC"],
Cell[1144963, 46323, 553, 9, 75, "Text",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1145541, 46336, 131, 3, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1145675, 46341, 177, 6, 26, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[1145867, 46350, 953, 32, 24, "Text",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1146857, 46387, 64, 1, 36, "Subsubsection",
 CellTags->"HypergeometricAC"],
Cell[1146924, 46390, 101, 1, 24, "Text",
 CellTags->"HypergeometricAC"],
Cell[CellGroupData[{
Cell[1147050, 46395, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1147317, 46404, 1411, 43, 47, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1148765, 46452, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1149032, 46461, 1375, 43, 52, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1150444, 46509, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1150711, 46518, 1416, 44, 52, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1152164, 46567, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1152431, 46576, 1695, 51, 100, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1154163, 46632, 264, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1154430, 46641, 415, 13, 40, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1154882, 46659, 265, 7, 26, "Input",
 CellTags->"HypergeometricAC"],
Cell[1155150, 46668, 416, 13, 40, "Output",
 CellTags->"HypergeometricAC"]
}, Open  ]]
}, Open  ]],
Cell[1155593, 46685, 69, 1, 39, "ObjectName",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1155687, 46690, 67, 1, 36, "Subsubsection",
 CellTags->"HypergeometricIR"],
Cell[1155757, 46693, 269, 5, 41, "Text",
 CellTags->"HypergeometricIR"],
Cell[1156029, 46700, 749, 25, 24, "Text",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1156803, 46729, 131, 3, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1156937, 46734, 182, 6, 26, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1157168, 46746, 64, 1, 36, "Subsubsection",
 CellTags->"HypergeometricIR"],
Cell[CellGroupData[{
Cell[1157257, 46751, 241, 7, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1157501, 46760, 640, 24, 48, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1158178, 46789, 394, 14, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1158575, 46805, 488, 15, 46, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1159100, 46825, 148, 4, 26, "Input",
 CellTags->"HypergeometricIR"],
Cell[1159251, 46831, 352, 13, 26, "Output",
 CellTags->"HypergeometricIR"]
}, Open  ]]
}, Open  ]],
Cell[1159630, 46848, 69, 1, 39, "ObjectName",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1159724, 46853, 67, 1, 28, "Subsubsection",
 CellTags->"HypergeometricSE"],
Cell[1159794, 46856, 420, 13, 41, "Text",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1160239, 46873, 131, 3, 26, "Input",
 CellTags->"HypergeometricSE"],
Cell[1160373, 46878, 186, 6, 26, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[1160574, 46887, 301, 11, 22, "Text",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1160912, 46903, 64, 1, 36, "Subsubsection",
 CellTags->"HypergeometricSE"],
Cell[CellGroupData[{
Cell[1161001, 46908, 245, 7, 26, "Input",
 CellTags->"HypergeometricSE"],
Cell[1161249, 46917, 585, 21, 46, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1161871, 46943, 351, 11, 26, "Input",
 CellTags->"HypergeometricSE"],
Cell[1162225, 46956, 809, 29, 49, "Output",
 CellTags->"HypergeometricSE"]
}, Open  ]]
}, Open  ]],
Cell[1163061, 46989, 59, 1, 39, "ObjectName",
 CellTags->"HypExplicit"],
Cell[CellGroupData[{
Cell[1163145, 46994, 62, 1, 28, "Subsubsection",
 CellTags->"HypExplicit"],
Cell[1163210, 46997, 325, 10, 22, "Text",
 CellTags->"HypExplicit"],
Cell[1163538, 47009, 296, 11, 22, "Text",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1163871, 47025, 59, 1, 36, "Subsubsection",
 CellTags->"HypExplicit"],
Cell[CellGroupData[{
Cell[1163955, 47030, 164, 4, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1164122, 47036, 161, 5, 26, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1164320, 47046, 142, 4, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1164465, 47052, 580, 21, 46, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1165082, 47078, 262, 8, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1165347, 47088, 1495, 55, 26, "Output",
 CellTags->"HypExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1166879, 47148, 142, 4, 26, "Input",
 CellTags->"HypExplicit"],
Cell[1167024, 47154, 804, 29, 49, "Output",
 CellTags->"HypExplicit"]
}, Open  ]]
}, Open  ]],
Cell[1167855, 47187, 49, 1, 39, "ObjectName",
 CellTags->"HypInt"],
Cell[CellGroupData[{
Cell[1167929, 47192, 57, 1, 28, "Subsubsection",
 CellTags->"HypInt"],
Cell[1167989, 47195, 206, 5, 22, "Text",
 CellTags->"HypInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1168232, 47205, 54, 1, 36, "Subsubsection",
 CellTags->"HypInt"],
Cell[CellGroupData[{
Cell[1168311, 47210, 159, 4, 26, "Input",
 CellTags->"HypInt"],
Cell[1168473, 47216, 156, 5, 26, "Output",
 CellTags->"HypInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1168666, 47226, 128, 4, 26, "Input",
 CellTags->"HypInt"],
Cell[1168797, 47232, 945, 36, 53, "Output",
 CellTags->"HypInt"]
}, Open  ]]
}, Open  ]],
Cell[1169769, 47272, 45, 1, 39, "ObjectName",
 CellTags->"IFPD"],
Cell[CellGroupData[{
Cell[1169839, 47277, 55, 1, 28, "Subsubsection",
 CellTags->"IFPD"],
Cell[1169897, 47280, 66, 1, 22, "Text",
 CellTags->"IFPD"],
Cell[1169966, 47283, 309, 11, 22, "Text",
 CellTags->"IFPD"]
}, Open  ]],
Cell[1170290, 47297, 51, 1, 39, "ObjectName",
 CellTags->"IFPDOff"],
Cell[CellGroupData[{
Cell[1170366, 47302, 58, 1, 28, "Subsubsection",
 CellTags->"IFPDOff"],
Cell[1170427, 47305, 182, 4, 22, "Text",
 CellTags->"IFPDOff"],
Cell[1170612, 47311, 428, 18, 22, "Text",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171077, 47334, 55, 1, 36, "Subsubsection",
 CellTags->"IFPDOff"],
Cell[CellGroupData[{
Cell[1171157, 47339, 163, 5, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1171323, 47346, 384, 16, 28, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1171744, 47367, 108, 3, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1171855, 47372, 178, 5, 38, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1172070, 47382, 130, 4, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1172203, 47388, 212, 8, 28, "Output",
 CellTags->"IFPDOff"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1172452, 47401, 108, 3, 26, "Input",
 CellTags->"IFPDOff"],
Cell[1172563, 47406, 283, 9, 38, "Output",
 CellTags->"IFPDOff"]
}, Open  ]]
}, Open  ]],
Cell[1172873, 47419, 49, 1, 39, "ObjectName",
 CellTags->"IFPDOn"],
Cell[CellGroupData[{
Cell[1172947, 47424, 57, 1, 28, "Subsubsection",
 CellTags->"IFPDOn"],
Cell[1173007, 47427, 317, 6, 37, "Text",
 CellTags->"IFPDOn"],
Cell[1173327, 47435, 431, 18, 22, "Text",
 CellTags->"IFPDOn"]
}, Open  ]],
Cell[1173773, 47456, 59, 1, 39, "ObjectName",
 CellTags->"IncludePair"],
Cell[CellGroupData[{
Cell[1173857, 47461, 62, 1, 28, "Subsubsection",
 CellTags->"IncludePair"],
Cell[1173922, 47464, 124, 3, 22, "Text",
 CellTags->"IncludePair"],
Cell[1174049, 47469, 256, 11, 22, "Text",
 CellTags->"IncludePair"]
}, Open  ]],
Cell[1174320, 47483, 63, 1, 39, "ObjectName",
 CellTags->"IndexPosition"],
Cell[CellGroupData[{
Cell[1174408, 47488, 64, 1, 28, "Subsubsection",
 CellTags->"IndexPosition"],
Cell[1174475, 47491, 89, 1, 22, "Text",
 CellTags->"IndexPosition"],
Cell[1174567, 47494, 286, 11, 22, "Text",
 CellTags->"IndexPosition"]
}, Open  ]],
Cell[1174868, 47508, 51, 1, 39, "ObjectName",
 CellTags->"Indices"],
Cell[CellGroupData[{
Cell[1174944, 47513, 58, 1, 28, "Subsubsection",
 CellTags->"Indices"],
Cell[1175005, 47516, 203, 4, 22, "Text",
 CellTags->"Indices"],
Cell[1175211, 47522, 280, 11, 24, "Text",
 CellTags->"Indices"]
}, Open  ]],
Cell[1175506, 47536, 67, 1, 39, "ObjectName",
 CellTags->"InitialFunction"],
Cell[CellGroupData[{
Cell[1175598, 47541, 66, 1, 36, "Subsubsection",
 CellTags->"InitialFunction"],
Cell[1175667, 47544, 183, 4, 24, "Text",
 CellTags->"InitialFunction"],
Cell[1175853, 47550, 268, 11, 24, "Text",
 CellTags->"InitialFunction"]
}, Open  ]],
Cell[1176136, 47564, 77, 1, 39, "ObjectName",
 CellTags->"InitialSubstitutions"],
Cell[CellGroupData[{
Cell[1176238, 47569, 71, 1, 36, "Subsubsection",
 CellTags->"InitialSubstitutions"],
Cell[1176312, 47572, 208, 4, 41, "Text",
 CellTags->"InitialSubstitutions"],
Cell[1176523, 47578, 653, 25, 24, "Text",
 CellTags->"InitialSubstitutions"]
}, Open  ]],
Cell[1177191, 47606, 69, 1, 39, "ObjectName",
 CellTags->"InsideDiracTrace"],
Cell[CellGroupData[{
Cell[1177285, 47611, 67, 1, 36, "Subsubsection",
 CellTags->"InsideDiracTrace"],
Cell[1177355, 47614, 498, 9, 58, "Text",
 CellTags->"InsideDiracTrace"],
Cell[1177856, 47625, 489, 18, 24, "Text",
 CellTags->"InsideDiracTrace"]
}, Open  ]],
Cell[1178360, 47646, 63, 1, 39, "ObjectName",
 CellTags->"IntegralTable"],
Cell[CellGroupData[{
Cell[1178448, 47651, 64, 1, 36, "Subsubsection",
 CellTags->"IntegralTable"],
Cell[1178515, 47654, 233, 5, 41, "Text",
 CellTags->"IntegralTable"],
Cell[1178751, 47661, 778, 25, 24, "Text",
 CellTags->"IntegralTable"]
}, Open  ]],
Cell[1179544, 47689, 69, 1, 39, "ObjectName",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1179638, 47694, 67, 1, 36, "Subsubsection",
 CellTags->"IntegrateByParts"],
Cell[1179708, 47697, 184, 4, 24, "Text",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1179917, 47705, 131, 3, 26, "Input",
 CellTags->"IntegrateByParts"],
Cell[1180051, 47710, 172, 5, 26, "Output",
 CellTags->"IntegrateByParts"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1180272, 47721, 64, 1, 36, "Subsubsection",
 CellTags->"IntegrateByParts"],
Cell[CellGroupData[{
Cell[1180361, 47726, 539, 19, 26, "Input",
 CellTags->"IntegrateByParts"],
Cell[1180903, 47747, 648, 23, 43, "Output",
 CellTags->"IntegrateByParts"]
}, Open  ]]
}, Open  ]],
Cell[1181578, 47774, 59, 1, 39, "ObjectName",
 CellTags->"Integratedx"],
Cell[CellGroupData[{
Cell[1181662, 47779, 62, 1, 36, "Subsubsection",
 CellTags->"Integratedx"],
Cell[1181727, 47782, 154, 4, 24, "Text",
 CellTags->"Integratedx"],
Cell[1181884, 47788, 1384, 46, 24, "Text",
 CellTags->"Integratedx"]
}, Open  ]],
Cell[1183283, 47837, 57, 1, 39, "ObjectName",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1183365, 47842, 61, 1, 36, "Subsubsection",
 CellTags->"Integrate2"],
Cell[1183429, 47845, 866, 17, 92, "Text",
 CellTags->"Integrate2"],
Cell[1184298, 47864, 835, 32, 22, "Text",
 CellTags->"Integrate2"],
Cell[1185136, 47898, 405, 7, 52, "Text",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1185578, 47910, 58, 1, 36, "Subsubsection",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1185661, 47915, 425, 15, 26, "Input",
 CellTags->"Integrate2"],
Cell[1186089, 47932, 368, 13, 44, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1186472, 47948, 190, 6, 24, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1186687, 47958, 281, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1186971, 47969, 230, 7, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1187238, 47981, 276, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1187517, 47992, 345, 12, 42, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1187899, 48009, 336, 11, 26, "Input",
 CellTags->"Integrate2"],
Cell[1188238, 48022, 575, 20, 45, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1188850, 48047, 317, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1189170, 48059, 134, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1189319, 48066, 1589, 53, 53, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1190933, 48123, 270, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1191206, 48134, 106, 3, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1191327, 48140, 649, 24, 44, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1192001, 48168, 317, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1192321, 48180, 106, 3, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1192464, 48188, 386, 13, 26, "Input",
 CellTags->"Integrate2"],
Cell[1192853, 48203, 164, 5, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1193054, 48213, 417, 14, 26, "Input",
 CellTags->"Integrate2"],
Cell[1193474, 48229, 179, 7, 42, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1193690, 48241, 328, 11, 26, "Input",
 CellTags->"Integrate2"],
Cell[1194021, 48254, 160, 6, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1194218, 48265, 354, 12, 26, "Input",
 CellTags->"Integrate2"],
Cell[1194575, 48279, 309, 11, 42, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1194921, 48295, 313, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1195237, 48307, 189, 6, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1195463, 48318, 281, 9, 26, "Input",
 CellTags->"Integrate2"],
Cell[1195747, 48329, 296, 11, 42, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1196080, 48345, 445, 15, 26, "Input",
 CellTags->"Integrate2"],
Cell[1196528, 48362, 487, 17, 46, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1197052, 48384, 721, 24, 26, "Input",
 CellTags->"Integrate2"],
Cell[1197776, 48410, 4351, 147, 139, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1202164, 48562, 471, 16, 26, "Input",
 CellTags->"Integrate2"],
Cell[1202638, 48580, 819, 29, 44, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1203494, 48614, 567, 19, 42, "Input",
 CellTags->"Integrate2"],
Cell[1204064, 48635, 5252, 177, 146, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1209353, 48817, 142, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[1209498, 48823, 1696, 53, 80, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1211231, 48881, 104, 3, 26, "Input",
 CellTags->"Integrate2"],
Cell[1211338, 48886, 125, 3, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1211500, 48894, 418, 13, 42, "Input"],
Cell[1211921, 48909, 101, 2, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1212059, 48916, 684, 23, 42, "Input",
 CellTags->"Integrate2"],
Cell[1212746, 48941, 3678, 139, 125, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1216461, 49085, 107, 3, 26, "Input"],
Cell[1216571, 49090, 447, 16, 42, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1217055, 49111, 79, 2, 26, "Input"],
Cell[1217137, 49115, 100, 2, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1217274, 49122, 656, 22, 42, "Input"],
Cell[1217933, 49146, 100, 2, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1218070, 49153, 151, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[1218224, 49159, 109, 3, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1218370, 49167, 368, 12, 26, "Input",
 CellTags->"Integrate2"],
Cell[1218741, 49181, 106, 3, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1218862, 49187, 137, 4, 24, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1219024, 49195, 147, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[1219174, 49201, 5377, 174, 230, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1224588, 49380, 230, 8, 26, "Input",
 CellTags->"Integrate2"],
Cell[1224821, 49390, 175, 6, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1225011, 49399, 165, 4, 24, "Text",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1225201, 49407, 326, 11, 26, "Input",
 CellTags->"Integrate2"],
Cell[1225530, 49420, 176, 6, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[1225721, 49429, 126, 4, 26, "Input",
 CellTags->"Integrate2"],
Cell[CellGroupData[{
Cell[1225872, 49437, 317, 10, 26, "Input",
 CellTags->"Integrate2"],
Cell[1226192, 49449, 134, 4, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1226363, 49458, 325, 11, 29, "Input",
 CellTags->"Integrate2"],
Cell[1226691, 49471, 240, 8, 43, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1226968, 49484, 99, 3, 26, "Input",
 CellTags->"Integrate2"],
Cell[1227070, 49489, 125, 3, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1227232, 49497, 325, 11, 29, "Input",
 CellTags->"Integrate2"],
Cell[1227560, 49510, 125, 3, 26, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1227722, 49518, 381, 13, 29, "Input",
 CellTags->"Integrate2"],
Cell[1228106, 49533, 4862, 184, 139, "Output",
 CellTags->"Integrate2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1233005, 49722, 107, 3, 26, "Input"],
Cell[1233115, 49727, 210, 7, 43, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1233362, 49739, 79, 2, 26, "Input"],
Cell[1233444, 49743, 99, 2, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1233580, 49750, 353, 12, 29, "Input"],
Cell[1233936, 49764, 99, 2, 26, "Output"]
}, Open  ]]
}, Open  ]],
Cell[1234062, 49770, 57, 1, 39, "ObjectName",
 CellTags->"Integrate3"],
Cell[CellGroupData[{
Cell[1234144, 49775, 61, 1, 28, "Subsubsection",
 CellTags->"Integrate3"],
Cell[1234208, 49778, 331, 6, 37, "Text",
 CellTags->"Integrate3"],
Cell[1234542, 49786, 271, 11, 24, "Text",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1234850, 49802, 58, 1, 36, "Subsubsection",
 CellTags->"Integrate3"],
Cell[CellGroupData[{
Cell[1234933, 49807, 280, 9, 29, "Input",
 CellTags->"Integrate3"],
Cell[1235216, 49818, 225, 8, 47, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1235478, 49831, 400, 13, 44, "Input",
 CellTags->"Integrate3"],
Cell[1235881, 49846, 513, 20, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1236431, 49871, 676, 23, 44, "Input",
 CellTags->"Integrate3"],
Cell[1237110, 49896, 2035, 73, 68, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1239182, 49974, 266, 8, 26, "Input",
 CellTags->"Integrate3"],
Cell[1239451, 49984, 106, 3, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1239594, 49992, 322, 10, 26, "Input",
 CellTags->"Integrate3"],
Cell[1239919, 50004, 195, 6, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1240151, 50015, 270, 9, 26, "Input",
 CellTags->"Integrate3"],
Cell[1240424, 50026, 106, 3, 26, "Output",
 CellTags->"Integrate3"]
}, Open  ]]
}, Open  ]],
Cell[1240557, 50033, 87, 1, 39, "ObjectName",
 CellTags->"IntermediateSubstitutions"],
Cell[CellGroupData[{
Cell[1240669, 50038, 76, 1, 36, "Subsubsection",
 CellTags->"IntermediateSubstitutions"],
Cell[1240748, 50041, 206, 4, 24, "Text",
 CellTags->"IntermediateSubstitutions"],
Cell[1240957, 50047, 274, 11, 24, "Text",
 CellTags->"IntermediateSubstitutions"]
}, Open  ]],
Cell[1241246, 50061, 63, 1, 39, "ObjectName",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1241334, 50066, 64, 1, 36, "Subsubsection",
 CellTags->"InverseMellin"],
Cell[1241401, 50069, 531, 14, 58, "Text",
 CellTags->"InverseMellin"],
Cell[1241935, 50085, 936, 37, 24, "Text",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1242908, 50127, 61, 1, 36, "Subsubsection",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1242994, 50132, 169, 5, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1243166, 50139, 157, 5, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1243360, 50149, 220, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1243583, 50158, 157, 5, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1243777, 50168, 142, 4, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1243922, 50174, 280, 10, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1244239, 50189, 194, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1244436, 50197, 293, 11, 42, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1244766, 50213, 194, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1244963, 50221, 360, 13, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1245360, 50239, 204, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1245567, 50248, 361, 13, 26, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1245943, 50264, 113, 3, 24, "Text",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1246081, 50271, 294, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1246378, 50282, 131, 4, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1246524, 50289, 117, 3, 24, "Text",
 CellTags->"InverseMellin"],
Cell[1246644, 50294, 3132, 100, 166, "Input",
 CellTags->"InverseMellin"],
Cell[1249779, 50396, 244, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1250048, 50407, 179, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1250230, 50415, 398, 14, 43, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1250665, 50434, 179, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1250847, 50442, 356, 12, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1251240, 50459, 172, 5, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1251415, 50466, 726, 25, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1252156, 50494, 81, 2, 26, "Input",
 CellTags->"InverseMellin"],
Cell[CellGroupData[{
Cell[1252262, 50500, 192, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1252457, 50508, 529, 20, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1253023, 50533, 270, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1253296, 50544, 502, 18, 42, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1253835, 50567, 270, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1254108, 50578, 695, 25, 42, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1254840, 50608, 267, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1255110, 50619, 693, 25, 43, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1255840, 50649, 246, 9, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1256089, 50660, 549, 20, 44, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1256675, 50685, 274, 10, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1256952, 50697, 914, 31, 49, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1257903, 50733, 192, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1258098, 50741, 700, 25, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1258835, 50771, 221, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1259059, 50780, 687, 24, 43, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1259783, 50809, 192, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1259978, 50817, 793, 29, 48, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1260808, 50851, 202, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1261013, 50859, 544, 20, 47, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1261594, 50884, 223, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1261820, 50893, 931, 33, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1262788, 50931, 223, 7, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1263014, 50940, 921, 33, 46, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1263972, 50978, 212, 6, 26, "Input",
 CellTags->"InverseMellin"],
Cell[1264187, 50986, 632, 23, 49, "Output",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1264834, 51012, 158, 5, 26, "Input",
 CellTags->"InverseMellin"]
}, Open  ]],
Cell[1265007, 51020, 51, 1, 39, "ObjectName",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1265083, 51025, 58, 1, 36, "Subsubsection",
 CellTags->"Isolate"],
Cell[1265144, 51028, 422, 7, 58, "Text",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1265591, 51039, 114, 3, 26, "Input",
 CellTags->"Isolate"],
Cell[1265708, 51044, 404, 12, 47, "Output",
 CellTags->"Isolate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1266161, 51062, 55, 1, 36, "Subsubsection",
 CellTags->"Isolate"],
Cell[CellGroupData[{
Cell[1266241, 51067, 156, 5, 26, "Input",
 CellTags->"Isolate"],
Cell[1266400, 51074, 160, 6, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1266597, 51085, 378, 13, 26, "Input",
 CellTags->"Isolate"],
Cell[1266978, 51100, 309, 12, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1267324, 51117, 114, 3, 26, "Input",
 CellTags->"Isolate"],
Cell[1267441, 51122, 286, 11, 38, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1267764, 51138, 182, 5, 26, "Input",
 CellTags->"Isolate"],
Cell[1267949, 51145, 646, 25, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1268632, 51175, 561, 19, 26, "Input",
 CellTags->"Isolate"],
Cell[1269196, 51196, 359, 14, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1269592, 51215, 94, 3, 26, "Input",
 CellTags->"Isolate"],
Cell[1269689, 51220, 90, 1, 40, "Print",
 CellTags->{"Info3535453244-5096475", "Isolate"}],
Cell[1269782, 51223, 1031, 30, 52, "Print",
 CellTags->{"Info3535453244-5096475", "Isolate"}]
}, Open  ]],
Cell[CellGroupData[{
Cell[1270850, 51258, 278, 7, 26, "Input",
 CellTags->"Isolate"],
Cell[1271131, 51267, 159, 6, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1271327, 51278, 178, 6, 26, "Input",
 CellTags->"Isolate"],
Cell[1271508, 51286, 211, 7, 26, "Output",
 CellTags->"Isolate"]
}, Open  ]],
Cell[1271734, 51296, 152, 4, 26, "Input",
 CellTags->"Isolate"]
}, Open  ]],
Cell[1271901, 51303, 59, 1, 39, "ObjectName",
 CellTags->"IsolateHead"],
Cell[CellGroupData[{
Cell[1271985, 51308, 62, 1, 36, "Subsubsection",
 CellTags->"IsolateHead"],
Cell[1272050, 51311, 84, 1, 24, "Text",
 CellTags->"IsolateHead"],
Cell[1272137, 51314, 280, 11, 24, "Text",
 CellTags->"IsolateHead"]
}, Open  ]],
Cell[1272432, 51328, 61, 1, 39, "ObjectName",
 CellTags->"IsolateNames"],
Cell[CellGroupData[{
Cell[1272518, 51333, 63, 1, 36, "Subsubsection",
 CellTags->"IsolateNames"],
Cell[1272584, 51336, 220, 5, 41, "Text",
 CellTags->"IsolateNames"],
Cell[1272807, 51343, 453, 18, 24, "Text",
 CellTags->"IsolateNames"]
}, Open  ]],
Cell[1273275, 51364, 61, 1, 39, "ObjectName",
 CellTags->"IsolatePrint"],
Cell[CellGroupData[{
Cell[1273361, 51369, 63, 1, 36, "Subsubsection",
 CellTags->"IsolatePrint"],
Cell[1273427, 51372, 284, 6, 41, "Text",
 CellTags->"IsolatePrint"],
Cell[1273714, 51380, 261, 11, 24, "Text",
 CellTags->"IsolatePrint"]
}, Open  ]],
Cell[1273990, 51394, 61, 1, 39, "ObjectName",
 CellTags->"IsolateSplit"],
Cell[CellGroupData[{
Cell[1274076, 51399, 63, 1, 36, "Subsubsection",
 CellTags->"IsolateSplit"],
Cell[1274142, 51402, 413, 7, 58, "Text",
 CellTags->"IsolateSplit"],
Cell[1274558, 51411, 261, 11, 24, "Text",
 CellTags->"IsolateSplit"]
}, Open  ]],
Cell[1274834, 51425, 53, 1, 39, "ObjectName",
 CellTags->"KeepOnly"],
Cell[CellGroupData[{
Cell[1274912, 51430, 59, 1, 36, "Subsubsection",
 CellTags->"KeepOnly"],
Cell[1274974, 51433, 293, 6, 41, "Text",
 CellTags->"KeepOnly"],
Cell[1275270, 51441, 761, 32, 24, "Text",
 CellTags->"KeepOnly"]
}, Open  ]],
Cell[1276046, 51476, 41, 1, 39, "ObjectName",
 CellTags->"KK"],
Cell[CellGroupData[{
Cell[1276112, 51481, 53, 1, 36, "Subsubsection",
 CellTags->"KK"],
Cell[1276168, 51484, 235, 5, 41, "Text",
 CellTags->"KK"],
Cell[1276406, 51491, 459, 18, 24, "Text",
 CellTags->"KK"]
}, Open  ]],
Cell[1276880, 51512, 49, 1, 39, "ObjectName",
 CellTags->"Kummer"],
Cell[CellGroupData[{
Cell[1276954, 51517, 57, 1, 36, "Subsubsection",
 CellTags->"Kummer"],
Cell[1277014, 51520, 260, 5, 41, "Text",
 CellTags->"Kummer"],
Cell[1277277, 51527, 291, 11, 24, "Text",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1277605, 51543, 54, 1, 36, "Subsubsection",
 CellTags->"Kummer"],
Cell[CellGroupData[{
Cell[1277684, 51548, 337, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1278024, 51559, 452, 14, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1278513, 51578, 337, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1278853, 51589, 467, 15, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1279357, 51609, 337, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1279697, 51620, 467, 15, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1280201, 51640, 388, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1280592, 51653, 452, 14, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1281081, 51672, 479, 13, 26, "Input",
 CellTags->"Kummer"],
Cell[1281563, 51687, 531, 15, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1282131, 51707, 388, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1282522, 51720, 434, 13, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1282993, 51738, 479, 13, 26, "Input",
 CellTags->"Kummer"],
Cell[1283475, 51753, 548, 17, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1284060, 51775, 388, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1284451, 51788, 434, 13, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1284922, 51806, 479, 13, 26, "Input",
 CellTags->"Kummer"],
Cell[1285404, 51821, 548, 17, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1285989, 51843, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1286479, 51860, 621, 20, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1287137, 51885, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1287769, 51906, 722, 22, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1288528, 51933, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1289018, 51950, 593, 19, 44, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1289648, 51974, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1290280, 51995, 707, 23, 44, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1291024, 52023, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1291514, 52040, 569, 18, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1292120, 52063, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1292752, 52084, 750, 23, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1293539, 52112, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1294029, 52129, 621, 20, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1294687, 52154, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1295319, 52175, 745, 20, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1296101, 52200, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1296591, 52217, 543, 17, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1297171, 52239, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1297803, 52260, 789, 22, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1298629, 52287, 487, 15, 26, "Input",
 CellTags->"Kummer"],
Cell[1299119, 52304, 543, 17, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1299699, 52326, 629, 19, 46, "Input",
 CellTags->"Kummer"],
Cell[1300331, 52347, 680, 19, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1301048, 52371, 531, 15, 46, "Input",
 CellTags->"Kummer"],
Cell[1301582, 52388, 576, 18, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1302195, 52411, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1302536, 52422, 452, 14, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1303025, 52441, 531, 15, 46, "Input",
 CellTags->"Kummer"],
Cell[1303559, 52458, 632, 19, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1304228, 52482, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1304569, 52493, 441, 14, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1305047, 52512, 531, 15, 46, "Input",
 CellTags->"Kummer"],
Cell[1305581, 52529, 632, 19, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1306250, 52553, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1306591, 52564, 441, 14, 40, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1307069, 52583, 582, 17, 46, "Input",
 CellTags->"Kummer"],
Cell[1307654, 52602, 587, 15, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1308278, 52622, 389, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1308670, 52635, 452, 14, 26, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1309159, 52654, 582, 17, 46, "Input",
 CellTags->"Kummer"],
Cell[1309744, 52673, 690, 18, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1310471, 52696, 389, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1310863, 52709, 492, 15, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1311392, 52729, 582, 17, 46, "Input",
 CellTags->"Kummer"],
Cell[1311977, 52748, 632, 16, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1312646, 52769, 389, 11, 26, "Input",
 CellTags->"Kummer"],
Cell[1313038, 52782, 434, 13, 43, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1313509, 52800, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1313850, 52811, 1310, 43, 71, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1315197, 52859, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1315538, 52870, 1370, 46, 52, "Output",
 CellTags->"Kummer"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1316945, 52921, 338, 9, 26, "Input",
 CellTags->"Kummer"],
Cell[1317286, 52932, 1396, 46, 52, "Output",
 CellTags->"Kummer"]
}, Open  ]]
}, Open  ]],
Cell[1318709, 52982, 57, 1, 39, "ObjectName",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1318791, 52987, 61, 1, 36, "Subsubsection",
 CellTags->"Lagrangian"],
Cell[1318855, 52990, 454, 9, 58, "Text",
 CellTags->"Lagrangian"],
Cell[1319312, 53001, 263, 11, 24, "Text",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1319612, 53017, 58, 1, 36, "Subsubsection",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1319695, 53022, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1319822, 53027, 332, 11, 42, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[1320169, 53041, 85, 1, 24, "Text",
 CellTags->"Lagrangian"],
Cell[CellGroupData[{
Cell[1320279, 53046, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1320406, 53051, 683, 23, 42, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1321126, 53079, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1321253, 53084, 1006, 36, 42, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1322296, 53125, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1322423, 53130, 621, 22, 26, "Output",
 CellTags->"Lagrangian"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1323081, 53157, 124, 3, 26, "Input",
 CellTags->"Lagrangian"],
Cell[1323208, 53162, 664, 23, 26, "Output",
 CellTags->"Lagrangian"]
}, Open  ]]
}, Open  ]],
Cell[1323899, 53189, 41, 1, 39, "ObjectName",
 CellTags->"LC"],
Cell[CellGroupData[{
Cell[1323965, 53194, 53, 1, 36, "Subsubsection",
 CellTags->"LC"],
Cell[1324021, 53197, 240, 5, 41, "Text",
 CellTags->"LC"],
Cell[1324264, 53204, 435, 18, 24, "Text",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1324736, 53227, 50, 1, 36, "Subsubsection",
 CellTags->"LC"],
Cell[CellGroupData[{
Cell[1324811, 53232, 164, 5, 26, "Input",
 CellTags->"LC"],
Cell[1324978, 53239, 184, 6, 26, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1325199, 53250, 95, 3, 26, "Input",
 CellTags->"LC"],
Cell[1325297, 53255, 477, 21, 26, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1325811, 53281, 104, 3, 26, "Input",
 CellTags->"LC"],
Cell[1325918, 53286, 337, 8, 38, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1326292, 53299, 181, 6, 26, "Input",
 CellTags->"LC"],
Cell[1326476, 53307, 169, 5, 26, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1326682, 53317, 130, 4, 26, "Input",
 CellTags->"LC"],
Cell[1326815, 53323, 367, 10, 38, "Output",
 CellTags->"LC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1327219, 53338, 381, 13, 26, "Input",
 CellTags->"LC"],
Cell[1327603, 53353, 396, 16, 28, "Output",
 CellTags->"LC"]
}, Open  ]]
}, Open  ]],
Cell[1328026, 53373, 43, 1, 39, "ObjectName",
 CellTags->"LCD"],
Cell[CellGroupData[{
Cell[1328094, 53378, 54, 1, 36, "Subsubsection",
 CellTags->"LCD"],
Cell[1328151, 53381, 234, 5, 41, "Text",
 CellTags->"LCD"],
Cell[1328388, 53388, 161, 4, 24, "Text",
 CellTags->"LCD"],
Cell[1328552, 53394, 432, 18, 24, "Text",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1329021, 53417, 51, 1, 36, "Subsubsection",
 CellTags->"LCD"],
Cell[1329075, 53420, 190, 6, 26, "Input",
 CellTags->"LCD"],
Cell[CellGroupData[{
Cell[1329290, 53430, 166, 5, 26, "Input",
 CellTags->"LCD"],
Cell[1329459, 53437, 185, 6, 26, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1329681, 53448, 96, 3, 26, "Input",
 CellTags->"LCD"],
Cell[1329780, 53453, 478, 21, 26, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1330295, 53479, 105, 3, 26, "Input",
 CellTags->"LCD"],
Cell[1330403, 53484, 438, 12, 38, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1330878, 53501, 183, 6, 26, "Input",
 CellTags->"LCD"],
Cell[1331064, 53509, 170, 5, 26, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1331271, 53519, 131, 4, 26, "Input",
 CellTags->"LCD"],
Cell[1331405, 53525, 418, 12, 38, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1331860, 53542, 400, 13, 26, "Input",
 CellTags->"LCD"],
Cell[1332263, 53557, 397, 15, 28, "Output",
 CellTags->"LCD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1332697, 53577, 170, 5, 26, "Input",
 CellTags->"LCD"],
Cell[1332870, 53584, 161, 5, 26, "Output",
 CellTags->"LCD"]
}, Open  ]]
}, Open  ]],
Cell[1333058, 53593, 61, 1, 39, "ObjectName",
 CellTags->"LeftPartialD"],
Cell[CellGroupData[{
Cell[1333144, 53598, 63, 1, 28, "Subsubsection",
 CellTags->"LeftPartialD"],
Cell[1333210, 53601, 307, 12, 30, "Text",
 CellTags->"LeftPartialD"],
Cell[1333520, 53615, 921, 32, 22, "Text",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1334478, 53652, 69, 1, 33, "Subsubsection",
 CellTags->"LeftPartialD"],
Cell[CellGroupData[{
Cell[1334572, 53657, 253, 7, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1334828, 53666, 306, 11, 33, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1335171, 53682, 120, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1335294, 53687, 355, 14, 30, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1335686, 53706, 118, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1335807, 53711, 291, 7, 38, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1336135, 53723, 161, 4, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1336299, 53729, 176, 4, 38, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1336512, 53738, 372, 10, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1336887, 53750, 344, 12, 33, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1337268, 53767, 120, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1337391, 53772, 774, 30, 30, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1338202, 53807, 118, 3, 26, "Input",
 CellTags->"LeftPartialD"],
Cell[1338323, 53812, 799, 21, 58, "Output",
 CellTags->"LeftPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1339149, 53837, 71, 1, 39, "ObjectName",
 CellTags->"LeftRightPartialD"],
Cell[CellGroupData[{
Cell[1339245, 53842, 68, 1, 28, "Subsubsection",
 CellTags->"LeftRightPartialD"],
Cell[1339316, 53845, 592, 21, 30, "Text",
 CellTags->"LeftRightPartialD"],
Cell[1339911, 53868, 1356, 45, 22, "Text",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1341304, 53918, 65, 1, 36, "Subsubsection",
 CellTags->"LeftRightPartialD"],
Cell[CellGroupData[{
Cell[1341394, 53923, 132, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1341529, 53928, 260, 9, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1341826, 53942, 127, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1341956, 53947, 527, 19, 42, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1342520, 53971, 263, 7, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1342786, 53980, 316, 11, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1343139, 53996, 125, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1343267, 54001, 708, 26, 42, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1344012, 54032, 382, 10, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1344397, 54044, 354, 12, 33, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1344788, 54061, 125, 3, 26, "Input",
 CellTags->"LeftRightPartialD"],
Cell[1344916, 54066, 902, 33, 42, "Output",
 CellTags->"LeftRightPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1345845, 54103, 73, 1, 39, "ObjectName",
 CellTags->"LeftRightPartialD2"],
Cell[CellGroupData[{
Cell[1345943, 54108, 69, 1, 28, "Subsubsection",
 CellTags->"LeftRightPartialD2"],
Cell[1346015, 54111, 591, 21, 30, "Text",
 CellTags->"LeftRightPartialD2"],
Cell[1346609, 54134, 1131, 39, 22, "Text",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1347777, 54178, 66, 1, 36, "Subsubsection",
 CellTags->"LeftRightPartialD2"],
Cell[CellGroupData[{
Cell[1347868, 54183, 134, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1348005, 54188, 261, 9, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1348303, 54202, 128, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1348434, 54207, 433, 16, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1348904, 54228, 265, 7, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1349172, 54237, 317, 11, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1349526, 54253, 126, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1349655, 54258, 600, 23, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1350292, 54286, 384, 10, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1350679, 54298, 355, 12, 33, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1351071, 54315, 126, 3, 26, "Input",
 CellTags->"LeftRightPartialD2"],
Cell[1351200, 54320, 780, 30, 30, "Output",
 CellTags->"LeftRightPartialD2"]
}, Open  ]]
}, Open  ]],
Cell[1352007, 54354, 58, 1, 39, "ObjectName",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1352090, 54359, 61, 1, 36, "Subsubsection",
 CellTags->"LeviCivita"],
Cell[1352154, 54362, 1210, 40, 80, "Text",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1353389, 54406, 120, 3, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1353512, 54411, 168, 5, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1353695, 54419, 775, 32, 24, "Text",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1354507, 54456, 58, 1, 36, "Subsubsection",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1354590, 54461, 187, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1354780, 54468, 492, 21, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1355309, 54494, 204, 6, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1355516, 54502, 411, 17, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1355964, 54524, 180, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1356147, 54531, 313, 13, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1356497, 54549, 186, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1356686, 54556, 107, 3, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1356830, 54564, 242, 7, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1357075, 54573, 332, 8, 38, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1357444, 54586, 365, 10, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1357812, 54598, 932, 40, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1358781, 54643, 113, 3, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1358897, 54648, 126, 4, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1359038, 54655, 316, 9, 26, "Input",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1359379, 54668, 403, 11, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1359785, 54681, 297, 10, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1360097, 54694, 316, 9, 26, "Input",
 CellTags->"LeviCivita"],
Cell[CellGroupData[{
Cell[1360438, 54707, 179, 5, 26, "Input",
 CellTags->"LeviCivita"],
Cell[1360620, 54714, 199, 6, 26, "Output",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1360834, 54723, 110, 3, 26, "Input",
 CellTags->"LeviCivita"]
}, Open  ]],
Cell[1360959, 54729, 65, 1, 39, "ObjectName",
 CellTags->"LeviCivitaSign"],
Cell[CellGroupData[{
Cell[1361049, 54734, 65, 1, 28, "Subsubsection",
 CellTags->"LeviCivitaSign"],
Cell[1361117, 54737, 203, 4, 22, "Text",
 CellTags->"LeviCivitaSign"],
Cell[1361323, 54743, 479, 18, 22, "Text",
 CellTags->"LeviCivitaSign"]
}, Open  ]],
Cell[1361817, 54764, 81, 1, 39, "ObjectName",
 CellTags->"ListIntegrals"],
Cell[CellGroupData[{
Cell[1361923, 54769, 64, 1, 28, "Subsubsection",
 CellTags->"ListIntegrals"],
Cell[1361990, 54772, 470, 8, 52, "Text",
 CellTags->"ListIntegrals"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1362497, 54785, 61, 1, 36, "Subsubsection",
 CellTags->"ListIntegrals"],
Cell[1362561, 54788, 80, 2, 26, "Input",
 CellTags->"ListIntegrals"]
}, Open  ]],
Cell[1362656, 54793, 44, 1, 39, "ObjectName",
 CellTags->"Li2"],
Cell[CellGroupData[{
Cell[1362725, 54798, 54, 1, 28, "Subsubsection",
 CellTags->"Li2"],
Cell[1362782, 54801, 117, 3, 22, "Text",
 CellTags->"Li2"],
Cell[1362902, 54806, 230, 10, 24, "Text",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1363169, 54821, 51, 1, 36, "Subsubsection",
 CellTags->"Li2"],
Cell[CellGroupData[{
Cell[1363245, 54826, 100, 3, 26, "Input",
 CellTags->"Li2"],
Cell[1363348, 54831, 136, 5, 26, "Output",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1363521, 54841, 107, 3, 26, "Input",
 CellTags->"Li2"],
Cell[1363631, 54846, 162, 5, 38, "Output",
 CellTags->"Li2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1363830, 54856, 253, 9, 42, "Input",
 CellTags->"Li2"],
Cell[1364086, 54867, 136, 5, 26, "Output",
 CellTags->"Li2"]
}, Open  ]]
}, Open  ]],
Cell[1364249, 54876, 44, 1, 39, "ObjectName",
 CellTags->"Li3"],
Cell[CellGroupData[{
Cell[1364318, 54881, 54, 1, 36, "Subsubsection",
 CellTags->"Li3"],
Cell[1364375, 54884, 118, 3, 24, "Text",
 CellTags->"Li3"],
Cell[1364496, 54889, 230, 10, 24, "Text",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1364763, 54904, 51, 1, 36, "Subsubsection",
 CellTags->"Li3"],
Cell[CellGroupData[{
Cell[1364839, 54909, 100, 3, 26, "Input",
 CellTags->"Li3"],
Cell[1364942, 54914, 136, 5, 26, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1365115, 54924, 107, 3, 26, "Input",
 CellTags->"Li3"],
Cell[1365225, 54929, 162, 5, 38, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1365424, 54939, 152, 5, 26, "Input",
 CellTags->"Li3"],
Cell[1365579, 54946, 159, 6, 44, "Output",
 CellTags->"Li3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1365775, 54957, 204, 7, 44, "Input",
 CellTags->"Li3"],
Cell[1365982, 54966, 136, 5, 26, "Output",
 CellTags->"Li3"]
}, Open  ]]
}, Open  ]],
Cell[1366145, 54975, 45, 1, 39, "ObjectName",
 CellTags->"Loop"],
Cell[CellGroupData[{
Cell[1366215, 54980, 55, 1, 36, "Subsubsection",
 CellTags->"Loop"],
Cell[1366273, 54983, 94, 1, 24, "Text",
 CellTags->"Loop"]
}, Open  ]],
Cell[1366382, 54987, 61, 1, 39, "ObjectName",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1366468, 54992, 63, 1, 36, "Subsubsection",
 CellTags->"LorentzIndex"],
Cell[1366534, 54995, 243, 5, 41, "Text",
 CellTags->"LorentzIndex"],
Cell[1366780, 55002, 485, 18, 24, "Text",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1367302, 55025, 60, 1, 34, "Subsubsection",
 CellTags->"LorentzIndex"],
Cell[1367365, 55028, 89, 1, 24, "Text",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1367479, 55033, 125, 3, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1367607, 55038, 116, 3, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1367738, 55044, 125, 3, 24, "Text",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1367888, 55051, 148, 4, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1368039, 55057, 116, 3, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1368192, 55065, 106, 2, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1368301, 55069, 140, 4, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1368478, 55078, 165, 4, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1368646, 55084, 113, 3, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1368796, 55092, 207, 5, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1369006, 55099, 139, 4, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1369160, 55106, 141, 4, 24, "Text",
 CellTags->"LorentzIndex"],
Cell[1369304, 55112, 137, 4, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[CellGroupData[{
Cell[1369466, 55120, 347, 10, 26, "Input",
 CellTags->"LorentzIndex"],
Cell[1369816, 55132, 385, 16, 26, "Output",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1370216, 55151, 148, 4, 26, "Input",
 CellTags->"LorentzIndex"]
}, Open  ]],
Cell[1370379, 55158, 47, 1, 39, "ObjectName",
 CellTags->"Lower"],
Cell[CellGroupData[{
Cell[1370451, 55163, 56, 1, 28, "Subsubsection",
 CellTags->"Lower"],
Cell[1370510, 55166, 120, 3, 22, "Text",
 CellTags->"Lower"],
Cell[1370633, 55171, 650, 25, 22, "Text",
 CellTags->"Lower"]
}, Open  ]],
Cell[1371298, 55199, 59, 1, 39, "ObjectName",
 CellTags->"MakeContext"],
Cell[CellGroupData[{
Cell[1371382, 55204, 62, 1, 28, "Subsubsection",
 CellTags->"MakeContext"],
Cell[1371447, 55207, 238, 5, 37, "Text",
 CellTags->"MakeContext"],
Cell[1371688, 55214, 864, 32, 22, "Text",
 CellTags->"MakeContext"]
}, Open  ]],
Cell[1372567, 55249, 57, 1, 39, "ObjectName",
 CellTags->"Mandelstam"],
Cell[CellGroupData[{
Cell[1372649, 55254, 61, 1, 28, "Subsubsection",
 CellTags->"Mandelstam"],
Cell[1372713, 55257, 1575, 73, 37, "Text",
 CellTags->"Mandelstam"],
Cell[1374291, 55332, 1259, 46, 22, "Text",
 CellTags->"Mandelstam"]
}, Open  ]],
Cell[1375565, 55381, 45, 1, 39, "ObjectName",
 CellTags->"Map2"],
Cell[CellGroupData[{
Cell[1375635, 55386, 55, 1, 28, "Subsubsection",
 CellTags->"Map2"],
Cell[1375693, 55389, 135, 4, 22, "Text",
 CellTags->"Map2"],
Cell[1375831, 55395, 249, 11, 22, "Text",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1376117, 55411, 52, 1, 36, "Subsubsection",
 CellTags->"Map2"],
Cell[CellGroupData[{
Cell[1376194, 55416, 149, 5, 26, "Input",
 CellTags->"Map2"],
Cell[1376346, 55423, 202, 7, 26, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1376585, 55435, 125, 4, 26, "Input",
 CellTags->"Map2"],
Cell[1376713, 55441, 129, 4, 26, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1376879, 55450, 184, 6, 26, "Input",
 CellTags->"Map2"],
Cell[1377066, 55458, 191, 7, 26, "Output",
 CellTags->"Map2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1377294, 55470, 125, 4, 26, "Input",
 CellTags->"Map2"],
Cell[1377422, 55476, 129, 4, 26, "Output",
 CellTags->"Map2"]
}, Open  ]]
}, Open  ]],
Cell[1377578, 55484, 67, 1, 39, "ObjectName",
 CellTags->"MemoryAvailable"],
Cell[CellGroupData[{
Cell[1377670, 55489, 66, 1, 28, "Subsubsection",
 CellTags->"MemoryAvailable"],
Cell[1377739, 55492, 230, 5, 37, "Text",
 CellTags->"MemoryAvailable"],
Cell[1377972, 55499, 484, 18, 24, "Text",
 CellTags->"MemoryAvailable"]
}, Open  ]],
Cell[1378471, 55520, 49, 1, 39, "ObjectName",
 CellTags->"MemSet"],
Cell[CellGroupData[{
Cell[1378545, 55525, 57, 1, 36, "Subsubsection",
 CellTags->"MemSet"],
Cell[1378605, 55528, 345, 9, 41, "Text",
 CellTags->"MemSet"],
Cell[1378953, 55539, 287, 11, 24, "Text",
 CellTags->"MemSet"]
}, Open  ]],
Cell[1379255, 55553, 61, 1, 39, "ObjectName",
 CellTags->"MetricTensor"],
Cell[CellGroupData[{
Cell[1379341, 55558, 63, 1, 36, "Subsubsection",
 CellTags->"MetricTensor"],
Cell[1379407, 55561, 214, 7, 24, "Text",
 CellTags->"MetricTensor"],
Cell[1379624, 55570, 981, 39, 24, "Text",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1380642, 55614, 60, 1, 36, "Subsubsection",
 CellTags->"MetricTensor"],
Cell[CellGroupData[{
Cell[1380727, 55619, 154, 4, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1380884, 55625, 165, 5, 26, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1381086, 55635, 147, 4, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1381236, 55641, 109, 3, 26, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1381382, 55649, 202, 5, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1381587, 55656, 165, 5, 26, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1381789, 55666, 147, 4, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1381939, 55672, 109, 3, 27, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1382085, 55680, 181, 5, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1382269, 55687, 156, 4, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1382462, 55696, 230, 6, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1382695, 55704, 204, 5, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1382936, 55714, 213, 6, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1383152, 55722, 156, 4, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1383345, 55731, 263, 7, 26, "Input",
 CellTags->"MetricTensor"],
Cell[1383611, 55740, 204, 5, 38, "Output",
 CellTags->"MetricTensor"]
}, Open  ]]
}, Open  ]],
Cell[1383842, 55749, 49, 1, 39, "ObjectName",
 CellTags->"MLimit"],
Cell[CellGroupData[{
Cell[1383916, 55754, 57, 1, 36, "Subsubsection",
 CellTags->"MLimit"],
Cell[1383976, 55757, 116, 3, 24, "Text",
 CellTags->"MLimit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1384129, 55765, 54, 1, 36, "Subsubsection",
 CellTags->"MLimit"],
Cell[CellGroupData[{
Cell[1384208, 55770, 462, 16, 26, "Input",
 CellTags->"MLimit"],
Cell[1384673, 55788, 103, 3, 26, "Output",
 CellTags->"MLimit"]
}, Open  ]]
}, Open  ]],
Cell[1384803, 55795, 53, 1, 39, "ObjectName",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1384881, 55800, 59, 1, 36, "Subsubsection",
 CellTags->"Momentum"],
Cell[1384943, 55803, 258, 5, 41, "Text",
 CellTags->"Momentum"],
Cell[1385204, 55810, 865, 32, 24, "Text",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1386106, 55847, 56, 1, 36, "Subsubsection",
 CellTags->"Momentum"],
Cell[1386165, 55850, 75, 1, 24, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1386265, 55855, 110, 3, 26, "Input",
 CellTags->"Momentum"],
Cell[1386378, 55860, 105, 3, 29, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1386498, 55866, 138, 4, 24, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1386661, 55874, 133, 4, 26, "Input",
 CellTags->"Momentum"],
Cell[1386797, 55880, 105, 3, 29, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1386917, 55886, 86, 1, 24, "Text",
 CellTags->"Momentum"],
Cell[CellGroupData[{
Cell[1387028, 55891, 169, 5, 26, "Input",
 CellTags->"Momentum"],
Cell[1387200, 55898, 148, 4, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1387385, 55907, 153, 5, 26, "Input",
 CellTags->"Momentum"],
Cell[1387541, 55914, 123, 4, 28, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1387701, 55923, 111, 3, 26, "Input",
 CellTags->"Momentum"],
Cell[1387815, 55928, 143, 4, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1387995, 55937, 259, 8, 26, "Input",
 CellTags->"Momentum"],
Cell[1388257, 55947, 201, 7, 28, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1388495, 55959, 111, 3, 26, "Input",
 CellTags->"Momentum"],
Cell[1388609, 55964, 231, 7, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1388877, 55976, 148, 4, 26, "Input",
 CellTags->"Momentum"],
Cell[1389028, 55982, 183, 5, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1389248, 55992, 149, 4, 26, "Input",
 CellTags->"Momentum"],
Cell[1389400, 55998, 231, 7, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1389668, 56010, 216, 7, 26, "Input",
 CellTags->"Momentum"],
Cell[1389887, 56019, 148, 4, 38, "Output",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1390050, 56026, 132, 4, 26, "Input",
 CellTags->"Momentum"]
}, Open  ]],
Cell[1390197, 56033, 67, 1, 39, "ObjectName",
 CellTags->"MomentumCombine"],
Cell[CellGroupData[{
Cell[1390289, 56038, 66, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine"],
Cell[1390358, 56041, 183, 4, 24, "Text",
 CellTags->"MomentumCombine"],
Cell[1390544, 56047, 938, 31, 24, "Text",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1391519, 56083, 63, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine"],
Cell[CellGroupData[{
Cell[1391607, 56088, 303, 9, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1391913, 56099, 214, 6, 38, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1392164, 56110, 288, 9, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1392455, 56121, 218, 7, 28, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1392710, 56133, 125, 3, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1392838, 56138, 218, 7, 28, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1393093, 56150, 163, 5, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1393259, 56157, 259, 8, 38, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1393555, 56170, 170, 5, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1393728, 56177, 444, 19, 28, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1394209, 56201, 121, 3, 26, "Input",
 CellTags->"MomentumCombine"],
Cell[1394333, 56206, 421, 12, 38, "Output",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[1394769, 56221, 115, 3, 26, "Input",
 CellTags->"MomentumCombine"]
}, Open  ]],
Cell[1394899, 56227, 69, 1, 39, "ObjectName",
 CellTags->"MomentumCombine2"],
Cell[CellGroupData[{
Cell[1394993, 56232, 67, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine2"],
Cell[1395063, 56235, 192, 4, 41, "Text",
 CellTags->"MomentumCombine2"],
Cell[1395258, 56241, 943, 31, 24, "Text",
 CellTags->"MomentumCombine2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1396238, 56277, 64, 1, 36, "Subsubsection",
 CellTags->"MomentumCombine2"],
Cell[CellGroupData[{
Cell[1396327, 56282, 434, 13, 26, "Input",
 CellTags->"MomentumCombine2"],
Cell[1396764, 56297, 476, 20, 28, "Output",
 CellTags->"MomentumCombine2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1397277, 56322, 164, 5, 26, "Input",
 CellTags->"MomentumCombine2"],
Cell[1397444, 56329, 309, 9, 38, "Output",
 CellTags->"MomentumCombine2"]
}, Open  ]]
}, Open  ]],
Cell[1397780, 56342, 65, 1, 39, "ObjectName",
 CellTags->"MomentumExpand"],
Cell[CellGroupData[{
Cell[1397870, 56347, 65, 1, 36, "Subsubsection",
 CellTags->"MomentumExpand"],
Cell[1397938, 56350, 149, 4, 24, "Text",
 CellTags->"MomentumExpand"],
Cell[1398090, 56356, 531, 18, 24, "Text",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1398658, 56379, 62, 1, 36, "Subsubsection",
 CellTags->"MomentumExpand"],
Cell[CellGroupData[{
Cell[1398745, 56384, 218, 6, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1398966, 56392, 189, 5, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1399192, 56402, 168, 5, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1399363, 56409, 248, 9, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1399648, 56423, 116, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1399767, 56428, 247, 7, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1400051, 56440, 211, 6, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1400265, 56448, 358, 15, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1400660, 56468, 116, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1400779, 56473, 284, 8, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1401100, 56486, 236, 7, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1401339, 56495, 384, 16, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1401760, 56516, 116, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1401879, 56521, 367, 11, 38, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1402283, 56537, 112, 3, 26, "Input",
 CellTags->"MomentumExpand"],
Cell[1402398, 56542, 560, 23, 28, "Output",
 CellTags->"MomentumExpand"]
}, Open  ]]
}, Open  ]],
Cell[1402985, 56569, 41, 1, 39, "ObjectName",
 CellTags->"MT"],
Cell[CellGroupData[{
Cell[1403051, 56574, 53, 1, 36, "Subsubsection",
 CellTags->"MT"],
Cell[1403107, 56577, 182, 7, 24, "Text",
 CellTags->"MT"],
Cell[1403292, 56586, 1011, 39, 24, "Text",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1404340, 56630, 50, 1, 36, "Subsubsection",
 CellTags->"MT"],
Cell[CellGroupData[{
Cell[1404415, 56635, 134, 4, 26, "Input",
 CellTags->"MT"],
Cell[1404552, 56641, 155, 5, 26, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1404744, 56651, 263, 8, 26, "Input",
 CellTags->"MT"],
Cell[1405010, 56661, 99, 3, 26, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1405146, 56669, 157, 5, 26, "Input",
 CellTags->"MT"],
Cell[1405306, 56676, 136, 4, 38, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1405479, 56685, 189, 6, 26, "Input",
 CellTags->"MT"],
Cell[1405671, 56693, 218, 6, 38, "Output",
 CellTags->"MT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1405926, 56704, 225, 8, 26, "Input",
 CellTags->"MT"],
Cell[1406154, 56714, 136, 4, 38, "Output",
 CellTags->"MT"]
}, Open  ]]
}, Open  ]],
Cell[1406317, 56722, 43, 1, 39, "ObjectName",
 CellTags->"MTD"],
Cell[CellGroupData[{
Cell[1406385, 56727, 54, 1, 36, "Subsubsection",
 CellTags->"MTD"],
Cell[1406442, 56730, 184, 7, 24, "Text",
 CellTags->"MTD"],
Cell[1406629, 56739, 1008, 39, 24, "Text",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1407674, 56783, 51, 1, 36, "Subsubsection",
 CellTags->"MTD"],
Cell[CellGroupData[{
Cell[1407750, 56788, 136, 4, 26, "Input",
 CellTags->"MTD"],
Cell[1407889, 56794, 156, 5, 26, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1408082, 56804, 266, 8, 26, "Input",
 CellTags->"MTD"],
Cell[1408351, 56814, 100, 3, 27, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1408488, 56822, 175, 6, 26, "Input",
 CellTags->"MTD"],
Cell[1408666, 56830, 151, 4, 38, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1408854, 56839, 207, 7, 26, "Input",
 CellTags->"MTD"],
Cell[1409064, 56848, 282, 8, 38, "Output",
 CellTags->"MTD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1409383, 56861, 235, 8, 26, "Input",
 CellTags->"MTD"],
Cell[1409621, 56871, 146, 4, 38, "Output",
 CellTags->"MTD"]
}, Open  ]]
}, Open  ]],
Cell[1409794, 56879, 67, 1, 39, "ObjectName",
 CellTags->"NegativeInteger"],
Cell[CellGroupData[{
Cell[1409886, 56884, 66, 1, 36, "Subsubsection",
 CellTags->"NegativeInteger"],
Cell[1409955, 56887, 141, 4, 24, "Text",
 CellTags->"NegativeInteger"],
Cell[1410099, 56893, 268, 11, 24, "Text",
 CellTags->"NegativeInteger"]
}, Open  ]],
Cell[1410382, 56907, 41, 1, 39, "ObjectName",
 CellTags->"Nf"],
Cell[CellGroupData[{
Cell[1410448, 56912, 53, 1, 36, "Subsubsection",
 CellTags->"Nf"],
Cell[1410504, 56915, 66, 1, 24, "Text",
 CellTags->"Nf"],
Cell[1410573, 56918, 1119, 39, 24, "Text",
 CellTags->"Nf"]
}, Open  ]],
Cell[1411707, 56960, 51, 1, 39, "ObjectName",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1411783, 56965, 58, 1, 36, "Subsubsection",
 CellTags->"Nielsen"],
Cell[1411844, 56968, 86, 1, 24, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1411955, 56973, 114, 3, 26, "Input",
 CellTags->"Nielsen"],
Cell[1412072, 56978, 169, 6, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1412256, 56987, 288, 11, 24, "Text",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1412581, 57003, 55, 1, 36, "Subsubsection",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1412661, 57008, 141, 4, 26, "Input",
 CellTags->"Nielsen"],
Cell[1412805, 57014, 251, 10, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1413071, 57027, 176, 4, 24, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1413272, 57035, 172, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1413447, 57042, 123, 3, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[1413585, 57048, 71, 1, 24, "Text",
 CellTags->"Nielsen"],
Cell[CellGroupData[{
Cell[1413681, 57053, 467, 14, 26, "Input",
 CellTags->"Nielsen"],
Cell[1414151, 57069, 320, 13, 44, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1414508, 57087, 190, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1414701, 57094, 573, 20, 42, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1415311, 57119, 190, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1415504, 57126, 807, 27, 45, "Output",
 CellTags->"Nielsen"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1416348, 57158, 190, 5, 26, "Input",
 CellTags->"Nielsen"],
Cell[1416541, 57165, 140, 5, 26, "Output",
 CellTags->"Nielsen"]
}, Open  ]]
}, Open  ]],
Cell[1416708, 57174, 61, 1, 39, "ObjectName",
 CellTags->"NonCommFreeQ"],
Cell[CellGroupData[{
Cell[1416794, 57179, 63, 1, 36, "Subsubsection",
 CellTags->"NonCommFreeQ"],
Cell[1416860, 57182, 244, 5, 41, "Text",
 CellTags->"NonCommFreeQ"],
Cell[1417107, 57189, 849, 32, 24, "Text",
 CellTags->"NonCommFreeQ"]
}, Open  ]],
Cell[1417971, 57224, 53, 1, 39, "ObjectName",
 CellTags->"NonCommQ"],
Cell[CellGroupData[{
Cell[1418049, 57229, 59, 1, 36, "Subsubsection",
 CellTags->"NonCommQ"],
Cell[1418111, 57232, 203, 4, 41, "Text",
 CellTags->"NonCommQ"],
Cell[1418317, 57238, 861, 32, 24, "Text",
 CellTags->"NonCommQ"]
}, Open  ]],
Cell[1419193, 57273, 65, 1, 39, "ObjectName",
 CellTags->"NonCommutative"],
Cell[CellGroupData[{
Cell[1419283, 57278, 65, 1, 36, "Subsubsection",
 CellTags->"NonCommutative"],
Cell[1419351, 57281, 150, 4, 24, "Text",
 CellTags->"NonCommutative"],
Cell[1419504, 57287, 510, 18, 24, "Text",
 CellTags->"NonCommutative"]
}, Open  ]],
Cell[1420029, 57308, 49, 1, 39, "ObjectName",
 CellTags->"NTerms"],
Cell[CellGroupData[{
Cell[1420103, 57313, 57, 1, 36, "Subsubsection",
 CellTags->"NTerms"],
Cell[1420163, 57316, 154, 4, 24, "Text",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1420354, 57325, 54, 1, 36, "Subsubsection",
 CellTags->"NTerms"],
Cell[CellGroupData[{
Cell[1420433, 57330, 129, 4, 26, "Input",
 CellTags->"NTerms"],
Cell[1420565, 57336, 103, 3, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1420705, 57344, 139, 4, 26, "Input",
 CellTags->"NTerms"],
Cell[1420847, 57350, 103, 3, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1420987, 57358, 106, 3, 26, "Input",
 CellTags->"NTerms"],
Cell[1421096, 57363, 103, 3, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1421236, 57371, 106, 3, 26, "Input",
 CellTags->"NTerms"],
Cell[1421345, 57376, 103, 3, 26, "Output",
 CellTags->"NTerms"]
}, Open  ]]
}, Open  ]],
Cell[1421475, 57383, 79, 1, 39, "ObjectName",
 CellTags->"NumberOfMetricTensors"],
Cell[CellGroupData[{
Cell[1421579, 57388, 72, 1, 36, "Subsubsection",
 CellTags->"NumberOfMetricTensors"],
Cell[1421654, 57391, 95, 1, 24, "Text",
 CellTags->"NumberOfMetricTensors"],
Cell[1421752, 57394, 245, 11, 24, "Text",
 CellTags->"NonCommQ"]
}, Open  ]],
Cell[1422012, 57408, 67, 1, 39, "ObjectName",
 CellTags->"NumericalFactor"],
Cell[CellGroupData[{
Cell[1422104, 57413, 66, 1, 36, "Subsubsection",
 CellTags->"NumericalFactor"],
Cell[1422173, 57416, 119, 3, 24, "Text",
 CellTags->"NumericalFactor"]
}, Open  ]],
Cell[1422307, 57422, 55, 1, 39, "ObjectName",
 CellTags->"NumericQ1"],
Cell[CellGroupData[{
Cell[1422387, 57427, 60, 1, 36, "Subsubsection",
 CellTags->"NumericQ1"],
Cell[1422450, 57430, 139, 4, 24, "Text",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1422626, 57439, 57, 1, 36, "Subsubsection",
 CellTags->"NumericQ1"],
Cell[CellGroupData[{
Cell[1422708, 57444, 224, 7, 29, "Input",
 CellTags->"NumericQ1"],
Cell[1422935, 57453, 110, 3, 26, "Output",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1423082, 57461, 271, 9, 29, "Input",
 CellTags->"NumericQ1"],
Cell[1423356, 57472, 110, 3, 26, "Output",
 CellTags->"NumericQ1"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1423503, 57480, 311, 10, 29, "Input",
 CellTags->"NumericQ1"],
Cell[1423817, 57492, 109, 3, 26, "Output",
 CellTags->"NumericQ1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1423975, 57501, 69, 1, 39, "ObjectName",
 CellTags->"OneLoop"],
Cell[1424047, 57504, 708, 18, 58, "Text",
 CellTags->"OneLoop"],
Cell[1424758, 57524, 968, 26, 41, "Text",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1425763, 57555, 58, 1, 36, "Subsubsection",
 CellTags->"OneLoop"],
Cell[1425824, 57558, 391, 12, 41, "Text",
 CellTags->"OneLoop"],
Cell[1426218, 57572, 352, 6, 58, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1426595, 57582, 114, 3, 26, "Input",
 CellTags->"OneLoop"],
Cell[1426712, 57587, 1275, 29, 87, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1428002, 57619, 1336, 53, 24, "Text",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1429375, 57677, 55, 1, 36, "Subsubsection",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1429455, 57682, 275, 10, 26, "Input",
 CellTags->"OneLoop"],
Cell[1429733, 57694, 323, 11, 48, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1430093, 57710, 131, 4, 26, "Input",
 CellTags->"OneLoop"],
Cell[1430227, 57716, 266, 10, 29, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1430508, 57729, 363, 11, 26, "Input",
 CellTags->"OneLoop"],
Cell[1430874, 57742, 588, 20, 27, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1431487, 57766, 908, 30, 49, "Input",
 CellTags->"OneLoop"],
Cell[1432398, 57798, 1871, 65, 55, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1434284, 57866, 219, 5, 41, "Text",
 CellTags->"OneLoop"],
Cell[CellGroupData[{
Cell[1434528, 57875, 131, 4, 26, "Input",
 CellTags->"OneLoop"],
Cell[1434662, 57881, 2406, 79, 59, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1437105, 57965, 113, 3, 26, "Input",
 CellTags->"OneLoop"],
Cell[1437221, 57970, 1707, 58, 49, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1438965, 58033, 333, 11, 26, "Input",
 CellTags->"OneLoop"],
Cell[1439301, 58046, 442, 17, 50, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1439780, 58068, 131, 4, 26, "Input",
 CellTags->"OneLoop"],
Cell[1439914, 58074, 1917, 67, 49, "Output",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1441846, 58144, 106, 3, 26, "Input",
 CellTags->"OneLoop"]
}, Open  ]],
Cell[1441967, 58150, 67, 1, 39, "ObjectName",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1442059, 58155, 66, 1, 36, "Subsubsection",
 CellTags->"OneLoopSimplify"],
Cell[1442128, 58158, 172, 4, 24, "Text",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1442325, 58166, 130, 3, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1442458, 58171, 737, 18, 47, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[1443210, 58192, 612, 25, 24, "Text",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1443859, 58222, 63, 1, 36, "Subsubsection",
 CellTags->"OneLoopSimplify"],
Cell[CellGroupData[{
Cell[1443947, 58227, 341, 11, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1444291, 58240, 450, 17, 50, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1444778, 58262, 147, 4, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1444928, 58268, 1129, 43, 50, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1446094, 58316, 203, 6, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1446300, 58324, 400, 16, 46, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1446737, 58345, 401, 12, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1447141, 58359, 580, 22, 50, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1447758, 58386, 152, 4, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1447913, 58392, 7168, 261, 98, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1455118, 58658, 253, 9, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1455374, 58669, 4274, 162, 50, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1459685, 58836, 424, 15, 26, "Input",
 CellTags->"OneLoopSimplify"],
Cell[1460112, 58853, 2174, 83, 52, "Output",
 CellTags->"OneLoopSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1462313, 58940, 57, 1, 39, "ObjectName",
 CellTags->"OneLoopSum"],
Cell[CellGroupData[{
Cell[1462395, 58945, 61, 1, 36, "Subsubsection",
 CellTags->"OneLoopSum"],
Cell[1462459, 58948, 190, 4, 24, "Text",
 CellTags->"OneLoopSum"],
Cell[1462652, 58954, 259, 11, 24, "Text",
 CellTags->"OneLoopSum"]
}, Open  ]],
Cell[1462926, 58968, 43, 1, 39, "ObjectName",
 CellTags->"OPE"],
Cell[CellGroupData[{
Cell[1462994, 58973, 54, 1, 36, "Subsubsection",
 CellTags->"OPE"],
Cell[1463051, 58976, 170, 4, 49, "Text",
 CellTags->"OPE"],
Cell[1463224, 58982, 86, 2, 24, "Text"],
Cell[CellGroupData[{
Cell[1463335, 58988, 60, 1, 26, "Input"],
Cell[1463398, 58991, 156, 5, 26, "Output"]
}, Open  ]],
Cell[1463569, 58999, 256, 11, 24, "Text",
 CellTags->"OPE"]
}, Open  ]],
Cell[1463840, 59013, 53, 1, 39, "ObjectName",
 CellTags->"OPEDelta"],
Cell[CellGroupData[{
Cell[1463918, 59018, 59, 1, 36, "Subsubsection",
 CellTags->"OPEDelta"],
Cell[1463980, 59021, 140, 4, 24, "Text",
 CellTags->"OPEDelta"],
Cell[1464123, 59027, 305, 11, 24, "Text",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1464465, 59043, 56, 1, 36, "Subsubsection",
 CellTags->"OPEDelta"],
Cell[CellGroupData[{
Cell[1464546, 59048, 146, 4, 26, "Input",
 CellTags->"OPEDelta"],
Cell[1464695, 59054, 179, 6, 29, "Output",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1464911, 59065, 133, 4, 26, "Input",
 CellTags->"OPEDelta"],
Cell[1465047, 59071, 105, 3, 26, "Output",
 CellTags->"OPEDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1465189, 59079, 141, 4, 26, "Input",
 CellTags->"OPEDelta"],
Cell[1465333, 59085, 105, 3, 26, "Output",
 CellTags->"OPEDelta"]
}, Open  ]]
}, Open  ]],
Cell[1465465, 59092, 81, 1, 39, "ObjectName",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1465571, 59097, 55, 1, 36, "Subsubsection",
 CellTags->"OPEi"],
Cell[1465629, 59100, 171, 4, 24, "Text",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1465837, 59109, 52, 1, 36, "Subsubsection",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1465914, 59114, 75, 2, 26, "Input",
 CellTags->"OPEi"],
Cell[1465992, 59118, 101, 3, 27, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1466130, 59126, 259, 7, 26, "Input",
 CellTags->"OPEi"],
Cell[1466392, 59135, 228, 7, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1466657, 59147, 1076, 40, 46, "Input",
 CellTags->"OPEi"],
Cell[1467736, 59189, 202, 6, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[1467953, 59198, 55, 1, 24, "Text",
 CellTags->"OPEi"],
Cell[CellGroupData[{
Cell[1468033, 59203, 519, 17, 26, "Input",
 CellTags->"OPEi"],
Cell[1468555, 59222, 529, 19, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1469121, 59246, 534, 19, 26, "Input",
 CellTags->"OPEi"],
Cell[1469658, 59267, 413, 15, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1470108, 59287, 519, 17, 26, "Input",
 CellTags->"OPEi"],
Cell[1470630, 59306, 529, 19, 26, "Output",
 CellTags->"OPEi"]
}, Open  ]]
}, Open  ]],
Cell[1471186, 59329, 73, 1, 39, "ObjectName",
 CellTags->"OPEInsert"],
Cell[CellGroupData[{
Cell[1471284, 59334, 60, 1, 36, "Subsubsection",
 CellTags->"OPEInsert"],
Cell[1471347, 59337, 264, 5, 41, "Text",
 CellTags->"OPEInsert"],
Cell[1471614, 59344, 242, 11, 24, "Text",
 CellTags->"OPEInsert"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1471893, 59360, 57, 1, 36, "Subsubsection",
 CellTags->"OPEInsert"],
Cell[1471953, 59363, 76, 2, 26, "Input",
 CellTags->"OPEInsert"]
}, Open  ]],
Cell[1472044, 59368, 67, 1, 39, "ObjectName",
 CellTags->"OPEInt"],
Cell[CellGroupData[{
Cell[1472136, 59373, 57, 1, 36, "Subsubsection",
 CellTags->"OPEInt"],
Cell[1472196, 59376, 257, 5, 41, "Text",
 CellTags->"OPEInt"],
Cell[1472456, 59383, 239, 11, 24, "Text",
 CellTags->"OPEInt"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1472732, 59399, 54, 1, 36, "Subsubsection",
 CellTags->"OPEInt"],
Cell[1472789, 59402, 73, 2, 26, "Input",
 CellTags->"OPEInt"]
}, Open  ]],
Cell[1472877, 59407, 77, 1, 39, "ObjectName",
 CellTags->"OPEIntDelta"],
Cell[CellGroupData[{
Cell[1472979, 59412, 62, 1, 36, "Subsubsection",
 CellTags->"OPEIntDelta"],
Cell[1473044, 59415, 478, 10, 66, "Text",
 CellTags->"OPEIntDelta"],
Cell[1473525, 59427, 244, 11, 24, "Text",
 CellTags->"OPEIntDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1473806, 59443, 59, 1, 36, "Subsubsection",
 CellTags->"OPEIntDelta"],
Cell[1473868, 59446, 78, 2, 26, "Input",
 CellTags->"OPEIntDelta"]
}, Open  ]],
Cell[1473961, 59451, 79, 1, 39, "ObjectName",
 CellTags->"OPEIntegrate"],
Cell[CellGroupData[{
Cell[1474065, 59456, 63, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrate"],
Cell[1474131, 59459, 266, 5, 41, "Text",
 CellTags->"OPEIntegrate"],
Cell[1474400, 59466, 245, 11, 24, "Text",
 CellTags->"OPEIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1474682, 59482, 60, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrate"],
Cell[1474745, 59485, 79, 2, 26, "Input",
 CellTags->"OPEIntegrate"]
}, Open  ]],
Cell[1474839, 59490, 89, 1, 39, "ObjectName",
 CellTags->"OPEIntegrateDelta"],
Cell[CellGroupData[{
Cell[1474953, 59495, 68, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrateDelta"],
Cell[1475024, 59498, 499, 10, 66, "Text",
 CellTags->"OPEIntegrateDelta"],
Cell[1475526, 59510, 250, 11, 24, "Text",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1475813, 59526, 65, 1, 36, "Subsubsection",
 CellTags->"OPEIntegrateDelta"],
Cell[1475881, 59529, 84, 2, 26, "Input",
 CellTags->"OPEIntegrateDelta"]
}, Open  ]],
Cell[1475980, 59534, 65, 1, 39, "ObjectName",
 CellTags->"OPESumExplicit"],
Cell[CellGroupData[{
Cell[1476070, 59539, 65, 1, 36, "Subsubsection",
 CellTags->"OPESumExplicit"],
Cell[1476138, 59542, 85, 1, 24, "Text",
 CellTags->"OPESumExplicit"],
Cell[1476226, 59545, 475, 18, 24, "Text",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1476738, 59568, 62, 1, 36, "Subsubsection",
 CellTags->"OPESumExplicit"],
Cell[CellGroupData[{
Cell[1476825, 59573, 387, 13, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1477215, 59588, 407, 15, 54, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1477659, 59608, 123, 3, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1477785, 59613, 408, 16, 44, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1478230, 59634, 554, 18, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1478787, 59654, 702, 27, 56, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1479526, 59686, 123, 3, 26, "Input",
 CellTags->"OPESumExplicit"],
Cell[1479652, 59691, 1747, 66, 47, "Output",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[1481414, 59760, 157, 5, 26, "Input",
 CellTags->"OPESumExplicit"]
}, Open  ]],
Cell[1481586, 59768, 49, 1, 39, "ObjectName",
 CellTags->"OPESum"],
Cell[CellGroupData[{
Cell[1481660, 59773, 57, 1, 36, "Subsubsection",
 CellTags->"OPESum"],
Cell[1481720, 59776, 127, 4, 24, "Text",
 CellTags->"OPESum"],
Cell[1481850, 59782, 499, 18, 24, "Text",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1482386, 59805, 54, 1, 36, "Subsubsection",
 CellTags->"OPESum"],
Cell[CellGroupData[{
Cell[1482465, 59810, 460, 15, 26, "Input",
 CellTags->"OPESum"],
Cell[1482928, 59827, 855, 31, 54, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1483820, 59863, 115, 3, 26, "Input",
 CellTags->"OPESum"],
Cell[1483938, 59868, 1604, 60, 49, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1485579, 59933, 546, 18, 26, "Input",
 CellTags->"OPESum"],
Cell[1486128, 59953, 694, 27, 56, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1486859, 59985, 115, 3, 26, "Input",
 CellTags->"OPESum"],
Cell[1486977, 59990, 1739, 66, 47, "Output",
 CellTags->"OPESum"]
}, Open  ]],
Cell[1488731, 60059, 149, 5, 26, "Input",
 CellTags->"OPESum"]
}, Open  ]],
Cell[1488895, 60067, 65, 1, 39, "ObjectName",
 CellTags->"OPESumSimplify"],
Cell[CellGroupData[{
Cell[1488985, 60072, 65, 1, 36, "Subsubsection",
 CellTags->"OPESumSimplify"],
Cell[1489053, 60075, 92, 1, 24, "Text",
 CellTags->"OPESumSimplify"],
Cell[1489148, 60078, 475, 18, 24, "Text",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1489660, 60101, 62, 1, 36, "Subsubsection",
 CellTags->"OPESumSimplify"],
Cell[CellGroupData[{
Cell[1489747, 60106, 143, 4, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1489893, 60112, 1052, 37, 53, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1490982, 60154, 89, 2, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1491074, 60158, 1152, 39, 54, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1492263, 60202, 273, 8, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1492539, 60212, 283, 11, 53, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1492859, 60228, 439, 14, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1493301, 60244, 553, 22, 53, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1493891, 60271, 116, 3, 26, "Input",
 CellTags->"OPESumSimplify"],
Cell[1494010, 60276, 404, 13, 43, "Output",
 CellTags->"OPESumSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1494441, 60293, 54, 1, 39, "ObjectName",
 CellTags->"OPE1Loop"],
Cell[CellGroupData[{
Cell[1494520, 60298, 59, 1, 36, "Subsubsection",
 CellTags->"OPE1Loop"],
Cell[1494582, 60301, 119, 3, 24, "Text",
 CellTags->"OPE1Loop"],
Cell[1494704, 60306, 253, 11, 24, "Text",
 CellTags->"OPE1Loop"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1494994, 60322, 56, 1, 36, "Subsubsection",
 CellTags->"OPE1Loop"],
Cell[1495053, 60325, 75, 2, 26, "Input",
 CellTags->"OPE1Loop"]
}, Open  ]],
Cell[1495143, 60330, 69, 1, 39, "ObjectName",
 CellTags->"OPE2TID"],
Cell[CellGroupData[{
Cell[1495237, 60335, 58, 1, 28, "Subsubsection",
 CellTags->"OPE2TID"],
Cell[1495298, 60338, 184, 4, 24, "Text",
 CellTags->"OPE2TID"],
Cell[CellGroupData[{
Cell[1495507, 60346, 114, 3, 26, "Input",
 CellTags->"OPE2TID"],
Cell[1495624, 60351, 343, 10, 26, "Output",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[1495982, 60364, 252, 11, 22, "Text",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1496271, 60380, 55, 1, 36, "Subsubsection",
 CellTags->"OPE2TID"],
Cell[1496329, 60383, 74, 2, 26, "Input",
 CellTags->"OPE2TID"]
}, Open  ]],
Cell[1496418, 60388, 63, 1, 39, "ObjectName",
 CellTags->"OptionsSelect"],
Cell[CellGroupData[{
Cell[1496506, 60393, 64, 1, 28, "Subsubsection",
 CellTags->"OptionsSelect"],
Cell[1496573, 60396, 219, 5, 41, "Text",
 CellTags->"OptionsSelect"]
}, Open  ]],
Cell[1496807, 60404, 45, 1, 39, "ObjectName",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1496877, 60409, 55, 1, 36, "Subsubsection",
 CellTags->"Pair"],
Cell[1496935, 60412, 499, 11, 60, "Text",
 CellTags->"Pair"],
Cell[1497437, 60425, 1705, 67, 24, "Text",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1499179, 60497, 52, 1, 36, "Subsubsection",
 CellTags->"Pair"],
Cell[1499234, 60500, 84, 1, 24, "Text",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1499343, 60505, 218, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1499564, 60513, 157, 5, 26, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[1499736, 60521, 73, 1, 24, "Text",
 CellTags->"Pair"],
Cell[CellGroupData[{
Cell[1499834, 60526, 268, 8, 26, "Input",
 CellTags->"Pair"],
Cell[1500105, 60536, 157, 5, 26, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1500299, 60546, 208, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1500510, 60554, 236, 10, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1500783, 60569, 197, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1500983, 60577, 203, 8, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1501223, 60590, 197, 6, 26, "Input",
 CellTags->"Pair"],
Cell[1501423, 60598, 159, 6, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1501619, 60609, 222, 7, 26, "Input",
 CellTags->"Pair"],
Cell[1501844, 60618, 238, 9, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1502119, 60632, 223, 7, 26, "Input",
 CellTags->"Pair"],
Cell[1502345, 60641, 223, 9, 30, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1502605, 60655, 223, 7, 26, "Input",
 CellTags->"Pair"],
Cell[1502831, 60664, 223, 9, 30, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1503091, 60678, 308, 10, 26, "Input",
 CellTags->"Pair"],
Cell[1503402, 60690, 101, 3, 26, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1503540, 60698, 272, 8, 26, "Input",
 CellTags->"Pair"],
Cell[1503815, 60708, 292, 12, 28, "Output",
 CellTags->"Pair"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1504144, 60725, 383, 12, 26, "Input",
 CellTags->"Pair"],
Cell[1504530, 60739, 101, 3, 26, "Output",
 CellTags->"Pair"]
}, Open  ]]
}, Open  ]],
Cell[1504658, 60746, 59, 1, 39, "ObjectName",
 CellTags->"PairCollect"],
Cell[CellGroupData[{
Cell[1504742, 60751, 62, 1, 36, "Subsubsection",
 CellTags->"PairCollect"],
Cell[1504807, 60754, 153, 4, 24, "Text",
 CellTags->"PairCollect"],
Cell[1504963, 60760, 448, 18, 24, "Text",
 CellTags->"PairCollect"]
}, Open  ]],
Cell[1505426, 60781, 61, 1, 39, "ObjectName",
 CellTags->"PairContract"],
Cell[CellGroupData[{
Cell[1505512, 60786, 63, 1, 36, "Subsubsection",
 CellTags->"PairContract"],
Cell[1505578, 60789, 118, 3, 24, "Text",
 CellTags->"PairContract"],
Cell[1505699, 60794, 435, 17, 24, "Text",
 CellTags->"PairContract"]
}, Open  ]],
Cell[1506149, 60814, 63, 1, 39, "ObjectName",
 CellTags->"PairContract2"],
Cell[CellGroupData[{
Cell[1506237, 60819, 64, 1, 36, "Subsubsection",
 CellTags->"PairContract2"],
Cell[1506304, 60822, 142, 4, 24, "Text",
 CellTags->"PairContract2"],
Cell[1506449, 60828, 644, 24, 24, "Text",
 CellTags->"PairContract2"]
}, Open  ]],
Cell[1507108, 60855, 63, 1, 39, "ObjectName",
 CellTags->"PairContract3"],
Cell[CellGroupData[{
Cell[1507196, 60860, 64, 1, 36, "Subsubsection",
 CellTags->"PairContract3"],
Cell[1507263, 60863, 142, 4, 24, "Text",
 CellTags->"PairContract3"],
Cell[1507408, 60869, 644, 24, 24, "Text",
 CellTags->"PairContract3"]
}, Open  ]],
Cell[1508067, 60896, 53, 1, 39, "ObjectName",
 CellTags->"PartialD"],
Cell[CellGroupData[{
Cell[1508145, 60901, 59, 1, 36, "Subsubsection",
 CellTags->"PartialD"],
Cell[1508207, 60904, 864, 35, 47, "Text",
 CellTags->"PartialD"],
Cell[1509074, 60941, 933, 32, 24, "Text",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1510044, 60978, 65, 1, 33, "Subsubsection",
 CellTags->"PartialD"],
Cell[CellGroupData[{
Cell[1510134, 60983, 233, 7, 26, "Input",
 CellTags->"PartialD"],
Cell[1510370, 60992, 302, 11, 33, "Output",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1510709, 61008, 116, 3, 26, "Input",
 CellTags->"PartialD"],
Cell[1510828, 61013, 351, 14, 30, "Output",
 CellTags->"PartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1511216, 61032, 114, 3, 26, "Input",
 CellTags->"PartialD"],
Cell[1511333, 61037, 287, 7, 38, "Output",
 CellTags->"PartialD"]
}, Open  ]]
}, Open  ]],
Cell[1511647, 61048, 71, 1, 39, "ObjectName",
 CellTags->"PartialDRelations"],
Cell[CellGroupData[{
Cell[1511743, 61053, 68, 1, 36, "Subsubsection",
 CellTags->"PartialDRelations"],
Cell[1511814, 61056, 191, 5, 24, "Text",
 CellTags->"PartialDRelations"],
Cell[1512008, 61063, 1134, 39, 24, "Text",
 CellTags->"PartialDRelations"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1513179, 61107, 74, 1, 33, "Subsubsection",
 CellTags->"PartialDRelations"],
Cell[CellGroupData[{
Cell[1513278, 61112, 344, 10, 26, "Input",
 CellTags->"PartialDRelations"],
Cell[1513625, 61124, 348, 12, 33, "Output",
 CellTags->"PartialDRelations"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1514010, 61141, 256, 7, 26, "Input",
 CellTags->"PartialDRelations"],
Cell[1514269, 61150, 855, 34, 30, "Output",
 CellTags->"PartialDRelations"]
}, Open  ]]
}, Open  ]],
Cell[1515151, 61188, 71, 1, 39, "ObjectName",
 CellTags->"PartialFourVector"],
Cell[CellGroupData[{
Cell[1515247, 61193, 68, 1, 36, "Subsubsection",
 CellTags->"PartialFourVector"],
Cell[1515318, 61196, 277, 6, 41, "Text",
 CellTags->"PartialFourVector"],
Cell[1515598, 61204, 646, 25, 24, "Text",
 CellTags->"PartialFourVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1516281, 61234, 74, 1, 33, "Subsubsection",
 CellTags->"PartialFourVector"],
Cell[CellGroupData[{
Cell[1516380, 61239, 382, 11, 35, "Input",
 CellTags->"PartialFourVector"],
Cell[1516765, 61252, 399, 15, 30, "Output",
 CellTags->"PartialFourVector"]
}, Open  ]]
}, Open  ]],
Cell[1517191, 61271, 69, 1, 39, "ObjectName",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1517285, 61276, 67, 1, 36, "Subsubsection",
 CellTags->"PartialIntegrate"],
Cell[1517355, 61279, 283, 6, 41, "Text",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1517663, 61289, 132, 3, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1517798, 61294, 185, 6, 26, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1518032, 61306, 73, 1, 33, "Subsubsection",
 CellTags->"PartialIntegrate"],
Cell[CellGroupData[{
Cell[1518130, 61311, 326, 10, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1518459, 61323, 1229, 40, 45, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1519725, 61368, 84, 2, 26, "Input"],
Cell[1519812, 61372, 220, 5, 57, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1520069, 61382, 275, 9, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1520347, 61393, 383, 14, 42, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1520767, 61412, 293, 10, 44, "Input",
 CellTags->"PartialIntegrate"],
Cell[1521063, 61424, 378, 14, 48, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1521478, 61443, 443, 14, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1521924, 61459, 246, 8, 46, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1522207, 61472, 320, 11, 26, "Input",
 CellTags->"PartialIntegrate"],
Cell[1522530, 61485, 246, 8, 46, "Output",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[1522791, 61496, 138, 4, 26, "Input",
 CellTags->"PartialIntegrate"]
}, Open  ]],
Cell[1522944, 61503, 57, 1, 39, "ObjectName",
 CellTags->"PartitHead"],
Cell[CellGroupData[{
Cell[1523026, 61508, 61, 1, 28, "Subsubsection",
 CellTags->"PartitHead"],
Cell[1523090, 61511, 167, 4, 22, "Text",
 CellTags->"PartitHead"]
}, Open  ]],
Cell[1523272, 61518, 57, 1, 39, "ObjectName",
 CellTags->"PauliSigma"],
Cell[CellGroupData[{
Cell[1523354, 61523, 61, 1, 28, "Subsubsection",
 CellTags->"PauliSigma"],
Cell[1523418, 61526, 252, 5, 41, "Text",
 CellTags->"PauliSigma"],
Cell[CellGroupData[{
Cell[1523695, 61535, 142, 4, 26, "Input",
 CellTags->"PauliSigma"],
Cell[1523840, 61541, 4559, 125, 122, "Output",
 CellTags->"PauliSigma"]
}, Open  ]]
}, Open  ]],
Cell[1528426, 61670, 45, 1, 39, "ObjectName",
 CellTags->"PaVe"],
Cell[CellGroupData[{
Cell[1528496, 61675, 55, 1, 28, "Subsubsection",
 CellTags->"PaVe"],
Cell[1528554, 61678, 428, 7, 58, "Text",
 CellTags->"PaVe"],
Cell[1528985, 61687, 266, 8, 24, "Text",
 CellTags->"PaVe"],
Cell[1529254, 61697, 265, 11, 24, "Text",
 CellTags->"PaVe"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1529556, 61713, 52, 1, 36, "Subsubsection",
 CellTags->"PaVe"],
Cell[1529611, 61716, 78, 1, 24, "Text",
 CellTags->"PaVe"],
Cell[CellGroupData[{
Cell[1529714, 61721, 303, 10, 26, "Input",
 CellTags->"PaVe"],
Cell[1530020, 61733, 304, 9, 26, "Output",
 CellTags->"PaVe"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1530361, 61747, 271, 9, 26, "Input",
 CellTags->"PaVe"],
Cell[1530635, 61758, 311, 11, 29, "Output",
 CellTags->"PaVe"]
}, Open  ]]
}, Open  ]],
Cell[1530973, 61773, 63, 1, 39, "ObjectName",
 CellTags->"PaVeOrderList"],
Cell[CellGroupData[{
Cell[1531061, 61778, 64, 1, 36, "Subsubsection",
 CellTags->"PaVeOrderList"],
Cell[1531128, 61781, 176, 4, 24, "Text",
 CellTags->"PaVeOrderList"],
Cell[1531307, 61787, 808, 31, 24, "Text",
 CellTags->"PaVeOrderList"]
}, Open  ]],
Cell[1532130, 61821, 55, 1, 39, "ObjectName",
 CellTags->"PaVeOrder"],
Cell[CellGroupData[{
Cell[1532210, 61826, 60, 1, 36, "Subsubsection",
 CellTags->"PaVeOrder"],
Cell[1532273, 61829, 564, 16, 41, "Text",
 CellTags->"PaVeOrder"],
Cell[1532840, 61847, 270, 11, 24, "Text",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1533147, 61863, 57, 1, 36, "Subsubsection",
 CellTags->"PaVeOrder"],
Cell[CellGroupData[{
Cell[1533229, 61868, 413, 12, 26, "Input",
 CellTags->"PaVeOrder"],
Cell[1533645, 61882, 382, 12, 26, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1534064, 61899, 413, 12, 26, "Input",
 CellTags->"PaVeOrder"],
Cell[1534480, 61913, 382, 12, 26, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1534899, 61930, 706, 20, 46, "Input",
 CellTags->"PaVeOrder"],
Cell[1535608, 61952, 623, 18, 26, "Output",
 CellTags->"PaVeOrder"]
}, Open  ]]
}, Open  ]],
Cell[1536258, 61974, 57, 1, 39, "ObjectName",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1536340, 61979, 61, 1, 36, "Subsubsection",
 CellTags->"PaVeReduce"],
Cell[1536404, 61982, 167, 4, 24, "Text",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1536596, 61990, 120, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1536719, 61995, 446, 13, 26, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1537202, 62013, 106, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1537311, 62018, 332, 5, 58, "Print",
 CellTags->{"Info3535453273-5096475", "PaVeReduce"}]
}, Open  ]],
Cell[1537658, 62026, 439, 18, 24, "Text",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1538134, 62049, 58, 1, 36, "Subsubsection",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1538217, 62054, 520, 16, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1538740, 62072, 3016, 104, 96, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1541793, 62181, 106, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1541902, 62186, 136, 4, 26, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1542075, 62195, 107, 3, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1542185, 62200, 136, 4, 26, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1542358, 62209, 491, 15, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1542852, 62226, 468, 15, 42, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1543357, 62246, 398, 12, 51, "Input",
 CellTags->"PaVeReduce"],
Cell[1543758, 62260, 792, 23, 72, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1544565, 62286, 209, 7, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[1544777, 62295, 162, 5, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1544964, 62304, 633, 17, 46, "Input",
 CellTags->"PaVeReduce"],
Cell[1545600, 62323, 8399, 260, 305, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1554014, 62586, 267, 8, 26, "Input",
 CellTags->"PaVeReduce"],
Cell[CellGroupData[{
Cell[1554306, 62598, 375, 11, 29, "Input",
 CellTags->"PaVeReduce"],
Cell[1554684, 62611, 2976, 57, 653, "Output",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1557675, 62671, 253, 7, 26, "Input",
 CellTags->"PaVeReduce"]
}, Open  ]],
Cell[1557943, 62681, 41, 1, 39, "ObjectName",
 CellTags->"PD"],
Cell[CellGroupData[{
Cell[1558009, 62686, 53, 1, 36, "Subsubsection",
 CellTags->"PD"],
Cell[1558065, 62689, 81, 1, 24, "Text",
 CellTags->"PD"],
Cell[1558149, 62692, 483, 18, 24, "Text",
 CellTags->"PD"]
}, Open  ]],
Cell[1558647, 62713, 69, 1, 39, "ObjectName",
 CellTags->"PlusDistribution"],
Cell[CellGroupData[{
Cell[1558741, 62718, 67, 1, 36, "Subsubsection",
 CellTags->"PlusDistribution"],
Cell[1558811, 62721, 147, 4, 24, "Text",
 CellTags->"PlusDistribution"],
Cell[1558961, 62727, 277, 11, 24, "Text",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1559275, 62743, 64, 1, 36, "Subsubsection",
 CellTags->"PlusDistribution"],
Cell[CellGroupData[{
Cell[1559364, 62748, 198, 6, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1559565, 62756, 208, 7, 44, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1559810, 62768, 254, 8, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1560067, 62778, 268, 9, 46, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1560372, 62792, 329, 10, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1560704, 62804, 113, 3, 26, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1560854, 62812, 389, 12, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1561246, 62826, 113, 3, 26, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1561396, 62834, 417, 13, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1561816, 62849, 113, 3, 26, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1561966, 62857, 307, 10, 26, "Input",
 CellTags->"PlusDistribution"],
Cell[1562276, 62869, 376, 13, 46, "Output",
 CellTags->"PlusDistribution"]
}, Open  ]]
}, Open  ]],
Cell[1562679, 62886, 61, 1, 39, "ObjectName",
 CellTags->"Polarization"],
Cell[CellGroupData[{
Cell[1562765, 62891, 63, 1, 36, "Subsubsection",
 CellTags->"Polarization"],
Cell[1562831, 62894, 769, 12, 92, "Text",
 CellTags->"Polarization"],
Cell[1563603, 62908, 757, 25, 24, "Text",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1564397, 62938, 60, 1, 36, "Subsubsection",
 CellTags->"Polarization"],
Cell[CellGroupData[{
Cell[1564482, 62943, 118, 3, 26, "Input",
 CellTags->"Polarization"],
Cell[1564603, 62948, 151, 4, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1564791, 62957, 153, 4, 26, "Input",
 CellTags->"Polarization"],
Cell[1564947, 62963, 168, 4, 38, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1565152, 62972, 160, 5, 26, "Input",
 CellTags->"Polarization"],
Cell[1565315, 62979, 237, 8, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1565589, 62992, 196, 6, 26, "Input",
 CellTags->"Polarization"],
Cell[1565788, 63000, 187, 5, 38, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1566012, 63010, 155, 4, 26, "Input",
 CellTags->"Polarization"],
Cell[1566170, 63016, 270, 9, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1566477, 63030, 194, 6, 26, "Input",
 CellTags->"Polarization"],
Cell[1566674, 63038, 206, 5, 38, "Output",
 CellTags->"Polarization"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1566917, 63048, 302, 8, 26, "Input",
 CellTags->"Polarization"],
Cell[1567222, 63058, 109, 3, 26, "Output",
 CellTags->"Polarization"]
}, Open  ]]
}, Open  ]],
Cell[1567358, 63065, 67, 1, 39, "ObjectName",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1567450, 63070, 66, 1, 36, "Subsubsection",
 CellTags->"PolarizationSum"],
Cell[1567519, 63073, 277, 5, 41, "Text",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1567821, 63082, 130, 3, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1567954, 63087, 173, 5, 26, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[1568142, 63095, 284, 11, 24, "Text",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1568463, 63111, 63, 1, 36, "Subsubsection",
 CellTags->"PolarizationSum"],
Cell[CellGroupData[{
Cell[1568551, 63116, 155, 4, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1568709, 63122, 183, 6, 26, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1568929, 63133, 165, 4, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1569097, 63139, 605, 25, 45, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1569739, 63169, 175, 4, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1569917, 63175, 1686, 71, 47, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1571640, 63251, 247, 7, 26, "Input",
 CellTags->"PolarizationSum"],
Cell[1571890, 63260, 4044, 162, 50, "Output",
 CellTags->"PolarizationSum"]
}, Open  ]]
}, Open  ]],
Cell[1575961, 63426, 81, 1, 39, "ObjectName",
 CellTags->"PolarizationUncontract"],
Cell[CellGroupData[{
Cell[1576067, 63431, 73, 1, 36, "Subsubsection",
 CellTags->"PolarizationUncontract"],
Cell[1576143, 63434, 152, 4, 24, "Text",
 CellTags->"PolarizationUncontract"],
Cell[1576298, 63440, 491, 18, 24, "Text",
 CellTags->"PolarizationUncontract"]
}, Open  ]],
Cell[1576804, 63461, 73, 1, 39, "ObjectName",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1576902, 63466, 69, 1, 36, "Subsubsection",
 CellTags->"PolarizationVector"],
Cell[1576974, 63469, 103, 1, 24, "Text",
 CellTags->"PolarizationVector"],
Cell[1577080, 63472, 657, 24, 24, "Text",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1577774, 63501, 66, 1, 36, "Subsubsection",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1577865, 63506, 157, 4, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1578025, 63512, 258, 9, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1578320, 63526, 194, 5, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1578517, 63533, 268, 9, 30, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1578800, 63545, 241, 9, 27, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1579066, 63558, 190, 5, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1579259, 63565, 310, 8, 38, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1579584, 63576, 440, 18, 27, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1580049, 63598, 284, 8, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1580336, 63608, 314, 11, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1580687, 63624, 121, 3, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1580811, 63629, 115, 3, 26, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1580963, 63637, 310, 9, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1581276, 63648, 376, 14, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1581689, 63667, 121, 3, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1581813, 63672, 336, 12, 28, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1582164, 63687, 1511, 51, 43, "Text",
 CellTags->"PolarizationVector"],
Cell[CellGroupData[{
Cell[1583700, 63742, 310, 9, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1584013, 63753, 376, 14, 29, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1584426, 63772, 121, 3, 26, "Input",
 CellTags->"PolarizationVector"],
Cell[1584550, 63777, 336, 12, 28, "Output",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1584901, 63792, 153, 4, 26, "Input",
 CellTags->"PolarizationVector"]
}, Open  ]],
Cell[1585069, 63799, 67, 1, 39, "ObjectName",
 CellTags->"PositiveInteger"],
Cell[CellGroupData[{
Cell[1585161, 63804, 66, 1, 36, "Subsubsection",
 CellTags->"PositiveInteger"],
Cell[1585230, 63807, 136, 4, 24, "Text",
 CellTags->"PositiveInteger"],
Cell[1585369, 63813, 268, 11, 24, "Text",
 CellTags->"PositiveInteger"]
}, Open  ]],
Cell[1585652, 63827, 65, 1, 39, "ObjectName",
 CellTags->"PositiveNumber"],
Cell[CellGroupData[{
Cell[1585742, 63832, 65, 1, 36, "Subsubsection",
 CellTags->"PositiveNumber"],
Cell[1585810, 63835, 146, 4, 24, "Text",
 CellTags->"PositiveNumber"],
Cell[1585959, 63841, 267, 11, 24, "Text",
 CellTags->"PositiveNumber"]
}, Open  ]],
Cell[1586241, 63855, 59, 1, 39, "ObjectName",
 CellTags->"PowerFactor"],
Cell[CellGroupData[{
Cell[1586325, 63860, 62, 1, 36, "Subsubsection",
 CellTags->"PowerFactor"],
Cell[1586390, 63863, 375, 16, 24, "Text",
 CellTags->"PowerFactor"],
Cell[1586768, 63881, 284, 11, 24, "Text",
 CellTags->"PowerFactor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1587089, 63897, 59, 1, 36, "Subsubsection",
 CellTags->"PowerFactor"],
Cell[CellGroupData[{
Cell[1587173, 63902, 144, 5, 26, "Input",
 CellTags->"PowerFactor"],
Cell[1587320, 63909, 181, 6, 26, "Output",
 CellTags->"PowerFactor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1587538, 63920, 116, 3, 26, "Input",
 CellTags->"PowerFactor"],
Cell[1587657, 63925, 181, 6, 26, "Output",
 CellTags->"PowerFactor"]
}, Open  ]]
}, Open  ]],
Cell[1587865, 63935, 63, 1, 39, "ObjectName",
 CellTags->"PowerSimplify"],
Cell[CellGroupData[{
Cell[1587953, 63940, 64, 1, 36, "Subsubsection",
 CellTags->"PowerSimplify"],
Cell[1588020, 63943, 2559, 112, 58, "Text",
 CellTags->"PowerSimplify"],
Cell[1590582, 64057, 364, 16, 24, "Text",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1590983, 64078, 61, 1, 36, "Subsubsection",
 CellTags->"PowerSimplify"],
Cell[CellGroupData[{
Cell[1591069, 64083, 234, 8, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1591306, 64093, 110, 3, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1591453, 64101, 239, 8, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1591695, 64111, 178, 6, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1591910, 64122, 239, 8, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1592152, 64132, 178, 6, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1592367, 64143, 190, 6, 26, "Input",
 CellTags->"PowerSimplify"],
Cell[1592560, 64151, 178, 6, 26, "Output",
 CellTags->"PowerSimplify"]
}, Open  ]]
}, Open  ]],
Cell[1592765, 64161, 49, 1, 39, "ObjectName",
 CellTags->"Power2"],
Cell[CellGroupData[{
Cell[1592839, 64166, 57, 1, 36, "Subsubsection",
 CellTags->"Power2"],
Cell[1592899, 64169, 213, 5, 41, "Text",
 CellTags->"Power2"],
Cell[1593115, 64176, 271, 11, 24, "Text",
 CellTags->"Power2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1593423, 64192, 54, 1, 36, "Subsubsection",
 CellTags->"Power2"],
Cell[CellGroupData[{
Cell[1593502, 64197, 147, 5, 26, "Input",
 CellTags->"Power2"],
Cell[1593652, 64204, 171, 6, 26, "Output",
 CellTags->"Power2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1593860, 64215, 148, 5, 26, "Input",
 CellTags->"Power2"],
Cell[1594011, 64222, 257, 10, 26, "Output",
 CellTags->"Power2"]
}, Open  ]]
}, Open  ]],
Cell[1594295, 64236, 67, 1, 39, "ObjectName",
 CellTags->"PostFortranFile"],
Cell[CellGroupData[{
Cell[1594387, 64241, 66, 1, 36, "Subsubsection",
 CellTags->"PostFortranFile"],
Cell[1594456, 64244, 229, 5, 41, "Text",
 CellTags->"PostFortranFile"],
Cell[1594688, 64251, 476, 18, 24, "Text",
 CellTags->"PostFortranFile"]
}, Open  ]],
Cell[1595179, 64272, 55, 1, 39, "ObjectName",
 CellTags->"Prefactor"],
Cell[CellGroupData[{
Cell[1595259, 64277, 60, 1, 36, "Subsubsection",
 CellTags->"Prefactor"],
Cell[1595322, 64280, 290, 6, 41, "Text",
 CellTags->"Prefactor"],
Cell[1595615, 64288, 458, 18, 24, "Text",
 CellTags->"Prefactor"]
}, Open  ]],
Cell[1596088, 64309, 65, 1, 39, "ObjectName",
 CellTags->"PreFortranFile"],
Cell[CellGroupData[{
Cell[1596178, 64314, 65, 1, 36, "Subsubsection",
 CellTags->"PreFortranFile"],
Cell[1596246, 64317, 232, 5, 41, "Text",
 CellTags->"PreFortranFile"],
Cell[1596481, 64324, 479, 18, 24, "Text",
 CellTags->"PreFortranFile"]
}, Open  ]],
Cell[1596975, 64345, 79, 1, 39, "ObjectName",
 CellTags->"PropagatorDenominator"],
Cell[CellGroupData[{
Cell[1597079, 64350, 72, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominator"],
Cell[1597154, 64353, 549, 16, 60, "Text",
 CellTags->"PropagatorDenominator"],
Cell[1597706, 64371, 766, 25, 24, "Text",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1598509, 64401, 69, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominator"],
Cell[CellGroupData[{
Cell[1598603, 64406, 178, 5, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1598784, 64413, 215, 7, 48, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1599036, 64425, 136, 3, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1599175, 64430, 164, 5, 48, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1599376, 64440, 189, 5, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1599568, 64447, 215, 7, 48, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1599820, 64459, 158, 4, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1599981, 64465, 235, 6, 38, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1600253, 64476, 194, 5, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1600450, 64483, 235, 6, 38, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1600722, 64494, 145, 3, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1600870, 64499, 253, 9, 48, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1601160, 64513, 127, 3, 26, "Input",
 CellTags->"PropagatorDenominator"],
Cell[1601290, 64518, 325, 10, 57, "Output",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[1601630, 64531, 121, 3, 26, "Input",
 CellTags->"PropagatorDenominator"]
}, Open  ]],
Cell[1601766, 64537, 95, 1, 39, "ObjectName",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[CellGroupData[{
Cell[1601886, 64542, 80, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1601969, 64545, 243, 5, 41, "Text",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1602215, 64552, 566, 18, 24, "Text",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1602818, 64575, 77, 1, 36, "Subsubsection",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[CellGroupData[{
Cell[1602920, 64580, 308, 11, 26, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1603231, 64593, 357, 12, 48, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1603625, 64610, 181, 5, 26, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1603809, 64617, 766, 29, 48, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1604612, 64651, 131, 3, 26, "Input",
 CellTags->"PropagatorDenominatorExplicit"],
Cell[1604746, 64656, 547, 18, 62, "Output",
 CellTags->"PropagatorDenominatorExplicit"]
}, Open  ]]
}, Open  ]],
Cell[1605320, 64678, 43, 1, 39, "ObjectName",
 CellTags->"QGV"],
Cell[CellGroupData[{
Cell[1605388, 64683, 54, 1, 36, "Subsubsection",
 CellTags->"QGV"],
Cell[1605445, 64686, 72, 1, 24, "Text",
 CellTags->"QGV"],
Cell[1605520, 64689, 288, 11, 24, "Text",
 CellTags->"QGV"]
}, Open  ]],
Cell[1605823, 64703, 41, 1, 39, "ObjectName",
 CellTags->"QO"],
Cell[CellGroupData[{
Cell[1605889, 64708, 53, 1, 36, "Subsubsection",
 CellTags->"QO"],
Cell[1605945, 64711, 73, 1, 24, "Text",
 CellTags->"QO"],
Cell[1606021, 64714, 299, 11, 24, "Text",
 CellTags->"QO"]
}, Open  ]],
Cell[1606335, 64728, 41, 1, 39, "ObjectName",
 CellTags->"QP"],
Cell[CellGroupData[{
Cell[1606401, 64733, 53, 1, 36, "Subsubsection",
 CellTags->"QP"],
Cell[1606457, 64736, 169, 4, 24, "Text",
 CellTags->"QP"],
Cell[1606629, 64742, 283, 11, 24, "Text",
 CellTags->"QP"]
}, Open  ]],
Cell[1606927, 64756, 61, 1, 39, "ObjectName",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1607013, 64761, 63, 1, 36, "Subsubsection",
 CellTags->"QuantumField"],
Cell[1607079, 64764, 404, 9, 58, "Text",
 CellTags->"QuantumField"],
Cell[1607486, 64775, 673, 25, 24, "Text",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1608196, 64805, 60, 1, 36, "Subsubsection",
 CellTags->"QuantumField"],
Cell[1608259, 64808, 71, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1608355, 64813, 118, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1608476, 64818, 109, 3, 27, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1608622, 64826, 131, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1608756, 64831, 137, 4, 26, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1608930, 64840, 127, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1609060, 64845, 114, 3, 26, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1609189, 64851, 80, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1609294, 64856, 169, 5, 26, "Input",
 CellTags->"QuantumField"],
Cell[1609466, 64863, 135, 4, 29, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1609616, 64870, 94, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1609735, 64875, 212, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1609950, 64883, 319, 14, 30, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1610306, 64902, 114, 3, 26, "Input",
 CellTags->"QuantumField"],
Cell[1610423, 64907, 255, 6, 38, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1610693, 64916, 358, 13, 28, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1611076, 64933, 206, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1611285, 64941, 259, 10, 28, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1611559, 64954, 185, 4, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1611769, 64962, 244, 7, 26, "Input",
 CellTags->"QuantumField"],
Cell[1612016, 64971, 245, 6, 38, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[1612276, 64980, 88, 1, 24, "Text",
 CellTags->"QuantumField"],
Cell[CellGroupData[{
Cell[1612389, 64985, 219, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1612611, 64993, 355, 14, 30, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1613003, 65012, 184, 5, 26, "Input",
 CellTags->"QuantumField"],
Cell[1613190, 65019, 257, 9, 28, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1613484, 65033, 259, 7, 26, "Input",
 CellTags->"QuantumField"],
Cell[1613746, 65042, 404, 16, 28, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1614187, 65063, 291, 9, 26, "Input",
 CellTags->"QuantumField"],
Cell[1614481, 65074, 413, 16, 28, "Output",
 CellTags->"QuantumField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1614931, 65095, 232, 6, 26, "Input",
 CellTags->"QuantumField"],
Cell[1615166, 65103, 112, 3, 26, "Output",
 CellTags->"QuantumField"]
}, Open  ]]
}, Open  ]],
Cell[1615305, 65110, 57, 1, 39, "ObjectName",
 CellTags->"QuarkField"],
Cell[CellGroupData[{
Cell[1615387, 65115, 61, 1, 36, "Subsubsection",
 CellTags->"QuarkField"],
Cell[1615451, 65118, 189, 4, 24, "Text",
 CellTags->"QuarkField"],
Cell[1615643, 65124, 495, 18, 24, "Text",
 CellTags->"QuarkField"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1616175, 65147, 58, 1, 36, "Subsubsection",
 CellTags->"QuarkField"],
Cell[CellGroupData[{
Cell[1616258, 65152, 87, 2, 26, "Input",
 CellTags->"QuarkField"],
Cell[1616348, 65156, 112, 3, 26, "Output",
 CellTags->"QuarkField"]
}, Open  ]]
}, Open  ]],
Cell[1616487, 65163, 69, 1, 39, "ObjectName",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1616581, 65168, 67, 1, 36, "Subsubsection",
 CellTags->"QuarkGluonVertex"],
Cell[1616651, 65171, 260, 9, 24, "Text",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1616936, 65184, 132, 3, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1617071, 65189, 491, 13, 26, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[1617577, 65205, 281, 11, 24, "Text",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1617895, 65221, 64, 1, 36, "Subsubsection",
 CellTags->"QuarkGluonVertex"],
Cell[CellGroupData[{
Cell[1617984, 65226, 153, 4, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1618140, 65232, 149, 4, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1618326, 65241, 208, 5, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1618537, 65248, 149, 4, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1618723, 65257, 208, 5, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1618934, 65264, 149, 4, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1619120, 65273, 208, 5, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1619331, 65280, 149, 4, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1619517, 65289, 315, 9, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1619835, 65300, 366, 11, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1620238, 65316, 316, 9, 26, "Input",
 CellTags->"QuarkGluonVertex"],
Cell[1620557, 65327, 367, 11, 28, "Output",
 CellTags->"QuarkGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1620951, 65342, 55, 1, 39, "ObjectName",
 CellTags->"QuarkMass"],
Cell[CellGroupData[{
Cell[1621031, 65347, 60, 1, 36, "Subsubsection",
 CellTags->"QuarkMass"],
Cell[1621094, 65350, 76, 1, 24, "Text",
 CellTags->"QuarkMass"],
Cell[1621173, 65353, 266, 11, 24, "Text",
 CellTags->"QuarkMass"]
}, Open  ]],
Cell[1621454, 65367, 67, 1, 39, "ObjectName",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1621546, 65372, 66, 1, 36, "Subsubsection",
 CellTags->"QuarkPropagator"],
Cell[1621615, 65375, 174, 4, 24, "Text",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1621814, 65383, 130, 3, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1621947, 65388, 534, 14, 26, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[1622496, 65405, 520, 18, 24, "Text",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1623053, 65428, 63, 1, 36, "Subsubsection",
 CellTags->"QuarkPropagator"],
Cell[CellGroupData[{
Cell[1623141, 65433, 124, 3, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1623268, 65438, 177, 6, 30, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1623482, 65449, 171, 5, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1623656, 65456, 177, 6, 30, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1623870, 65467, 157, 4, 26, "Input",
 CellTags->"QuarkPropagator"],
Cell[1624030, 65473, 177, 6, 30, "Output",
 CellTags->"QuarkPropagator"]
}, Open  ]]
}, Open  ]],
Cell[1624234, 65483, 59, 1, 39, "ObjectName",
 CellTags->"ReduceGamma"],
Cell[CellGroupData[{
Cell[1624318, 65488, 62, 1, 36, "Subsubsection",
 CellTags->"ReduceGamma"],
Cell[1624383, 65491, 194, 4, 24, "Text",
 CellTags->"ReduceGamma"],
Cell[1624580, 65497, 260, 11, 24, "Text",
 CellTags->"ReduceGamma"]
}, Open  ]],
Cell[1624855, 65511, 67, 1, 39, "ObjectName",
 CellTags->"ReduceToScalars"],
Cell[CellGroupData[{
Cell[1624947, 65516, 66, 1, 36, "Subsubsection",
 CellTags->"ReduceToScalars"],
Cell[1625016, 65519, 210, 5, 41, "Text",
 CellTags->"ReduceToScalars"],
Cell[1625229, 65526, 464, 18, 24, "Text",
 CellTags->"ReduceToScalars"]
}, Open  ]],
Cell[1625708, 65547, 49, 1, 39, "ObjectName",
 CellTags->"Rename"],
Cell[CellGroupData[{
Cell[1625782, 65552, 57, 1, 36, "Subsubsection",
 CellTags->"Rename"],
Cell[1625842, 65555, 144, 4, 24, "Text",
 CellTags->"Rename"],
Cell[1625989, 65561, 259, 11, 24, "Text",
 CellTags->"Rename"]
}, Open  ]],
Cell[1626263, 65575, 61, 1, 39, "ObjectName",
 CellTags->"RHI"],
Cell[CellGroupData[{
Cell[1626349, 65580, 54, 1, 36, "Subsubsection",
 CellTags->"RHI"],
Cell[1626406, 65583, 611, 12, 141, "Text",
 CellTags->"RHI"],
Cell[1627020, 65597, 248, 11, 24, "Text",
 CellTags->"RHI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1627305, 65613, 51, 1, 36, "Subsubsection",
 CellTags->"RHI"],
Cell[1627359, 65616, 70, 2, 26, "Input",
 CellTags->"RHI"]
}, Open  ]],
Cell[1627444, 65621, 67, 1, 39, "ObjectName",
 CellTags->"RHI2FC"],
Cell[CellGroupData[{
Cell[1627536, 65626, 57, 1, 36, "Subsubsection",
 CellTags->"RHI2FC"],
Cell[1627596, 65629, 614, 12, 141, "Text",
 CellTags->"RHI2FC"],
Cell[1628213, 65643, 251, 11, 24, "Text",
 CellTags->"RHI2FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1628501, 65659, 54, 1, 36, "Subsubsection",
 CellTags->"RHI2FC"],
Cell[1628558, 65662, 73, 2, 26, "Input",
 CellTags->"RHI2FC"]
}, Open  ]],
Cell[1628646, 65667, 61, 1, 39, "ObjectName",
 CellTags->"RHM"],
Cell[CellGroupData[{
Cell[1628732, 65672, 54, 1, 36, "Subsubsection",
 CellTags->"RHM"],
Cell[1628789, 65675, 77, 1, 24, "Text",
 CellTags->"RHM"],
Cell[1628869, 65678, 236, 11, 24, "Text",
 CellTags->"RHM"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1629142, 65694, 51, 1, 36, "Subsubsection",
 CellTags->"RHM"],
Cell[1629196, 65697, 70, 2, 26, "Input",
 CellTags->"RHM"]
}, Open  ]],
Cell[1629281, 65702, 61, 1, 39, "ObjectName",
 CellTags->"RHO"],
Cell[CellGroupData[{
Cell[1629367, 65707, 54, 1, 36, "Subsubsection",
 CellTags->"RHO"],
Cell[1629424, 65710, 267, 5, 41, "Text",
 CellTags->"RHO"],
Cell[1629694, 65717, 236, 11, 24, "Text",
 CellTags->"RHO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1629967, 65733, 51, 1, 36, "Subsubsection",
 CellTags->"RHO"],
Cell[1630021, 65736, 70, 2, 26, "Input",
 CellTags->"RHO"]
}, Open  ]],
Cell[1630106, 65741, 45, 1, 39, "ObjectName",
 CellTags->"RHP"],
Cell[CellGroupData[{
Cell[1630176, 65746, 54, 1, 36, "Subsubsection",
 CellTags->"RHP"],
Cell[1630233, 65749, 272, 5, 41, "Text",
 CellTags->"RHP"],
Cell[1630508, 65756, 236, 11, 24, "Text",
 CellTags->"RHP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1630781, 65772, 51, 1, 36, "Subsubsection",
 CellTags->"RHP"],
Cell[CellGroupData[{
Cell[1630857, 65777, 133, 3, 26, "Input"],
Cell[1630993, 65782, 1937, 77, 49, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1632967, 65864, 133, 3, 26, "Input"],
Cell[1633103, 65869, 713, 29, 49, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1633853, 65903, 133, 3, 26, "Input"],
Cell[1633989, 65908, 1415, 59, 47, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1635441, 65972, 133, 3, 26, "Input"],
Cell[1635577, 65977, 507, 22, 47, "Output"]
}, Open  ]],
Cell[1636099, 66002, 70, 2, 26, "Input",
 CellTags->"RHP"]
}, Open  ]],
Cell[1636184, 66007, 61, 1, 39, "ObjectName",
 CellTags->"RTL"],
Cell[CellGroupData[{
Cell[1636270, 66012, 54, 1, 36, "Subsubsection",
 CellTags->"RTL"],
Cell[1636327, 66015, 188, 4, 24, "Text",
 CellTags->"RTL"],
Cell[1636518, 66021, 424, 18, 24, "Text",
 CellTags->"RTL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1636979, 66044, 51, 1, 36, "Subsubsection",
 CellTags->"RTL"],
Cell[1637033, 66047, 70, 2, 26, "Input",
 CellTags->"RTL"]
}, Open  ]],
Cell[1637118, 66052, 63, 1, 39, "ObjectName",
 CellTags->"RightPartialD"],
Cell[CellGroupData[{
Cell[1637206, 66057, 64, 1, 36, "Subsubsection",
 CellTags->"RightPartialD"],
Cell[1637273, 66060, 256, 10, 27, "Text",
 CellTags->"RightPartialD"],
Cell[1637532, 66072, 690, 25, 24, "Text",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1638259, 66102, 61, 1, 36, "Subsubsection",
 CellTags->"RightPartialD"],
Cell[CellGroupData[{
Cell[1638345, 66107, 124, 3, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1638472, 66112, 252, 9, 33, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1638761, 66126, 255, 7, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1639019, 66135, 308, 11, 33, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1639364, 66151, 121, 3, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1639488, 66156, 356, 14, 30, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1639881, 66175, 115, 3, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1639999, 66180, 292, 7, 38, "Output",
 CellTags->"RightPartialD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1640328, 66192, 162, 5, 26, "Input",
 CellTags->"RightPartialD"],
Cell[1640493, 66199, 178, 4, 38, "Output",
 CellTags->"RightPartialD"]
}, Open  ]]
}, Open  ]],
Cell[1640698, 66207, 61, 1, 39, "ObjectName",
 CellTags->"RussianTrick"],
Cell[CellGroupData[{
Cell[1640784, 66212, 63, 1, 36, "Subsubsection",
 CellTags->"RussianTrick"],
Cell[1640850, 66215, 318, 6, 41, "Text",
 CellTags->"RussianTrick"],
Cell[1641171, 66223, 90, 1, 24, "Text",
 CellTags->"RussianTrick"],
Cell[1641264, 66226, 501, 18, 24, "Text",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1641802, 66249, 60, 1, 36, "Subsubsection",
 CellTags->"RussianTrick"],
Cell[CellGroupData[{
Cell[1641887, 66254, 374, 12, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1642264, 66268, 501, 21, 28, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1642802, 66294, 108, 3, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1642913, 66299, 661, 23, 49, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1643611, 66327, 172, 5, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1643786, 66334, 3469, 121, 141, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1647292, 66460, 112, 3, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1647407, 66465, 2675, 111, 28, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1650119, 66581, 135, 4, 26, "Input",
 CellTags->"RussianTrick"],
Cell[1650257, 66587, 2947, 112, 45, "Output",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[1653219, 66702, 111, 3, 26, "Input",
 CellTags->"RussianTrick"]
}, Open  ]],
Cell[1653345, 66708, 71, 1, 39, "ObjectName",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1653441, 66713, 68, 1, 36, "Subsubsection",
 CellTags->"ScalarGluonVertex"],
Cell[1653512, 66716, 430, 8, 74, "Text",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1653967, 66728, 134, 3, 26, "Input",
 CellTags->"ScalarGluonVertex"],
Cell[1654104, 66733, 271, 8, 26, "Output",
 CellTags->"ScalarGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1654424, 66747, 65, 1, 36, "Subsubsection",
 CellTags->"ScalarGluonVertex"],
Cell[CellGroupData[{
Cell[1654514, 66752, 274, 8, 26, "Input",
 CellTags->"ScalarGluonVertex"],
Cell[1654791, 66762, 479, 19, 28, "Output",
 CellTags->"ScalarGluonVertex"]
}, Open  ]]
}, Open  ]],
Cell[1655297, 66785, 63, 1, 39, "ObjectName",
 CellTags->"ScalarProduct"],
Cell[CellGroupData[{
Cell[1655385, 66790, 64, 1, 36, "Subsubsection",
 CellTags->"ScalarProduct"],
Cell[1655452, 66793, 836, 21, 75, "Text",
 CellTags->"ScalarProduct"],
Cell[1656291, 66816, 1468, 52, 24, "Text",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1657796, 66873, 61, 1, 36, "Subsubsection",
 CellTags->"ScalarProduct"],
Cell[CellGroupData[{
Cell[1657882, 66878, 143, 4, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1658028, 66884, 212, 8, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1658277, 66897, 186, 6, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1658466, 66905, 301, 11, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1658804, 66921, 143, 4, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1658950, 66927, 168, 6, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1659155, 66938, 120, 3, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1659278, 66943, 168, 6, 28, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1659483, 66954, 179, 5, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1659665, 66961, 221, 6, 38, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1659923, 66972, 231, 7, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1660157, 66981, 271, 8, 38, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1660465, 66994, 248, 8, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1660716, 67004, 131, 4, 40, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1660884, 67013, 295, 9, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1661182, 67024, 571, 23, 40, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1661790, 67052, 280, 9, 26, "Input",
 CellTags->"ScalarProduct"],
Cell[1662073, 67063, 571, 23, 40, "Output",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[1662659, 67089, 99, 2, 26, "Input",
 CellTags->"ScalarProduct"]
}, Open  ]],
Cell[1662773, 67094, 75, 1, 39, "ObjectName",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1662873, 67099, 70, 1, 36, "Subsubsection",
 CellTags->"ScalarProductCancel"],
Cell[1662946, 67102, 186, 4, 24, "Text",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1663157, 67110, 89, 2, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1663249, 67114, 134, 3, 26, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1663420, 67122, 134, 3, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1663557, 67127, 392, 10, 26, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1663986, 67142, 106, 3, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1664095, 67147, 179, 3, 38, "Print",
 CellTags->{"Info3535453289-5096475", "ScalarProductCancel"}]
}, Open  ]],
Cell[1664289, 67153, 1238, 45, 24, "Text",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1665564, 67203, 67, 1, 36, "Subsubsection",
 CellTags->"ScalarProductCancel"],
Cell[1665634, 67206, 344, 11, 24, "Text",
 CellTags->"ScalarProductCancel"],
Cell[CellGroupData[{
Cell[1666003, 67221, 429, 15, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1666435, 67238, 454, 17, 45, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1666926, 67260, 156, 4, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1667085, 67266, 849, 32, 56, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1667971, 67303, 162, 4, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1668136, 67309, 708, 26, 56, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1668881, 67340, 185, 5, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1669069, 67347, 708, 26, 56, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1669814, 67378, 578, 18, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1670395, 67398, 898, 33, 47, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1671330, 67436, 197, 5, 26, "Input",
 CellTags->"ScalarProductCancel"],
Cell[1671530, 67443, 3108, 111, 112, "Output",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[1674653, 67557, 162, 5, 26, "Input",
 CellTags->"ScalarProductCancel"]
}, Open  ]],
Cell[1674830, 67565, 51, 1, 39, "ObjectName",
 CellTags->"ScaleMu"],
Cell[CellGroupData[{
Cell[1674906, 67570, 58, 1, 36, "Subsubsection",
 CellTags->"ScaleMu"],
Cell[1674967, 67573, 128, 4, 24, "Text",
 CellTags->"ScaleMu"],
Cell[CellGroupData[{
Cell[1675120, 67581, 81, 2, 26, "Input",
 CellTags->"ScaleMu"],
Cell[1675204, 67585, 108, 3, 26, "Output",
 CellTags->"ScaleMu"]
}, Open  ]]
}, Open  ]],
Cell[1675339, 67592, 53, 1, 39, "ObjectName",
 CellTags->"Schouten"],
Cell[CellGroupData[{
Cell[1675417, 67597, 59, 1, 36, "Subsubsection",
 CellTags->"Schouten"],
Cell[1675479, 67600, 460, 8, 83, "Text",
 CellTags->"Schouten"],
Cell[1675942, 67610, 455, 17, 24, "Text",
 CellTags->"Schouten"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1676434, 67632, 56, 1, 36, "Subsubsection",
 CellTags->"Schouten"],
Cell[CellGroupData[{
Cell[1676515, 67637, 1938, 54, 138, "Input",
 CellTags->"Schouten"],
Cell[1678456, 67693, 15696, 628, 194, "Output",
 CellTags->"Schouten"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1694189, 68326, 106, 3, 26, "Input",
 CellTags->"Schouten"],
Cell[1694298, 68331, 105, 3, 26, "Output",
 CellTags->"Schouten"]
}, Open  ]],
Cell[1694418, 68337, 84, 2, 26, "Input"]
}, Open  ]],
Cell[1694517, 68342, 41, 1, 39, "ObjectName",
 CellTags->"SD"],
Cell[CellGroupData[{
Cell[1694583, 68347, 53, 1, 36, "Subsubsection",
 CellTags->"SD"],
Cell[1694639, 68350, 265, 7, 41, "Text",
 CellTags->"SD"],
Cell[1694907, 68359, 255, 11, 24, "Text",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1695199, 68375, 50, 1, 36, "Subsubsection",
 CellTags->"SD"],
Cell[CellGroupData[{
Cell[1695274, 68380, 121, 4, 26, "Input",
 CellTags->"SD"],
Cell[1695398, 68386, 147, 5, 26, "Output",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1695582, 68396, 130, 4, 26, "Input",
 CellTags->"SD"],
Cell[1695715, 68402, 214, 6, 38, "Output",
 CellTags->"SD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1695966, 68413, 130, 4, 26, "Input",
 CellTags->"SD"],
Cell[1696099, 68419, 136, 4, 38, "Output",
 CellTags->"SD"]
}, Open  ]]
}, Open  ]],
Cell[1696262, 68427, 57, 1, 39, "ObjectName",
 CellTags->"SelectFree"],
Cell[CellGroupData[{
Cell[1696344, 68432, 61, 1, 36, "Subsubsection",
 CellTags->"SelectFree"],
Cell[1696408, 68435, 255, 5, 41, "Text",
 CellTags->"SelectFree"],
Cell[1696666, 68442, 77, 1, 24, "Text",
 CellTags->"SelectFree"],
Cell[1696746, 68445, 467, 18, 24, "Text",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1697250, 68468, 58, 1, 36, "Subsubsection",
 CellTags->"SelectFree"],
Cell[CellGroupData[{
Cell[1697333, 68473, 211, 6, 26, "Input",
 CellTags->"SelectFree"],
Cell[1697547, 68481, 130, 4, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1697714, 68490, 166, 5, 26, "Input",
 CellTags->"SelectFree"],
Cell[1697883, 68497, 107, 3, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1698027, 68505, 275, 8, 26, "Input",
 CellTags->"SelectFree"],
Cell[1698305, 68515, 130, 4, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1698472, 68524, 137, 4, 26, "Input",
 CellTags->"SelectFree"],
Cell[1698612, 68530, 107, 3, 27, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1698756, 68538, 137, 4, 26, "Input",
 CellTags->"SelectFree"],
Cell[1698896, 68544, 107, 3, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1699040, 68552, 137, 4, 26, "Input",
 CellTags->"SelectFree"],
Cell[1699180, 68558, 107, 3, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1699324, 68566, 166, 5, 26, "Input",
 CellTags->"SelectFree"],
Cell[1699493, 68573, 107, 3, 26, "Output",
 CellTags->"SelectFree"]
}, Open  ]]
}, Open  ]],
Cell[1699627, 68580, 61, 1, 39, "ObjectName",
 CellTags->"SelectGraphs"],
Cell[CellGroupData[{
Cell[1699713, 68585, 63, 1, 36, "Subsubsection",
 CellTags->"SelectGraphs"],
Cell[1699779, 68588, 486, 8, 60, "Text",
 CellTags->"SelectGraphs"],
Cell[1700268, 68598, 273, 11, 24, "Text",
 CellTags->"SelectGraphs"]
}, Open  ]],
Cell[1700556, 68612, 63, 1, 39, "ObjectName",
 CellTags->"SelectNotFree"],
Cell[CellGroupData[{
Cell[1700644, 68617, 64, 1, 36, "Subsubsection",
 CellTags->"SelectNotFree"],
Cell[1700711, 68620, 143, 4, 24, "Text",
 CellTags->"SelectNotFree"],
Cell[1700857, 68626, 270, 5, 41, "Text",
 CellTags->"SelectNotFree"],
Cell[1701130, 68633, 83, 1, 24, "Text",
 CellTags->"SelectNotFree"],
Cell[1701216, 68636, 458, 18, 24, "Text",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1701711, 68659, 61, 1, 36, "Subsubsection",
 CellTags->"SelectNotFree"],
Cell[CellGroupData[{
Cell[1701797, 68664, 207, 6, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1702007, 68672, 162, 5, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702206, 68682, 276, 8, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1702485, 68692, 172, 5, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702694, 68702, 143, 4, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1702840, 68708, 110, 3, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1702987, 68716, 167, 5, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1703157, 68723, 110, 3, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703304, 68731, 143, 4, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1703450, 68737, 110, 3, 27, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703597, 68745, 143, 4, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1703743, 68751, 110, 3, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1703890, 68759, 172, 5, 26, "Input",
 CellTags->"SelectNotFree"],
Cell[1704065, 68766, 138, 4, 26, "Output",
 CellTags->"SelectNotFree"]
}, Open  ]]
}, Open  ]],
Cell[1704230, 68774, 59, 1, 39, "ObjectName",
 CellTags->"SelectSplit"],
Cell[CellGroupData[{
Cell[1704314, 68779, 62, 1, 36, "Subsubsection",
 CellTags->"SelectSplit"],
Cell[1704379, 68782, 249, 5, 41, "Text",
 CellTags->"SelectSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1704665, 68792, 59, 1, 36, "Subsubsection",
 CellTags->"SelectSplit"],
Cell[CellGroupData[{
Cell[1704749, 68797, 804, 26, 29, "Input",
 CellTags->"SelectSplit"],
Cell[1705556, 68825, 532, 19, 29, "Output",
 CellTags->"SelectSplit"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1706125, 68849, 652, 21, 29, "Input",
 CellTags->"SelectSplit"],
Cell[1706780, 68872, 495, 16, 29, "Output",
 CellTags->"SelectSplit"]
}, Open  ]]
}, Open  ]],
Cell[1707302, 68892, 51, 1, 39, "ObjectName",
 CellTags->"Select1"],
Cell[CellGroupData[{
Cell[1707378, 68897, 58, 1, 36, "Subsubsection",
 CellTags->"Select1"],
Cell[1707439, 68900, 243, 5, 41, "Text",
 CellTags->"Select1"],
Cell[1707685, 68907, 256, 11, 24, "Text",
 CellTags->"Select1"]
}, Open  ]],
Cell[1707956, 68921, 51, 1, 39, "ObjectName",
 CellTags->"Select2"],
Cell[CellGroupData[{
Cell[1708032, 68926, 58, 1, 36, "Subsubsection",
 CellTags->"Select2"],
Cell[1708093, 68929, 245, 5, 41, "Text",
 CellTags->"Select2"],
Cell[1708341, 68936, 256, 11, 24, "Text",
 CellTags->"Select2"]
}, Open  ]],
Cell[1708612, 68950, 51, 1, 39, "ObjectName",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1708688, 68955, 58, 1, 36, "Subsubsection",
 CellTags->"Series2"],
Cell[1708749, 68958, 383, 9, 41, "Text",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1709157, 68971, 114, 3, 26, "Input",
 CellTags->"Series2"],
Cell[1709274, 68976, 528, 15, 26, "Output",
 CellTags->"Series2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1709851, 68997, 55, 1, 36, "Subsubsection",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1709931, 69002, 339, 12, 26, "Input",
 CellTags->"Series2"],
Cell[1710273, 69016, 357, 12, 42, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1710667, 69033, 174, 5, 26, "Input",
 CellTags->"Series2"],
Cell[1710844, 69040, 242, 8, 44, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1711123, 69053, 222, 7, 26, "Input",
 CellTags->"Series2"],
Cell[1711348, 69062, 806, 25, 44, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1712191, 69092, 174, 5, 26, "Input",
 CellTags->"Series2"],
Cell[1712368, 69099, 385, 14, 46, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1712790, 69118, 290, 8, 26, "Input",
 CellTags->"Series2"],
Cell[1713083, 69128, 994, 32, 45, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1714092, 69163, 187, 6, 24, "Text"],
Cell[CellGroupData[{
Cell[1714304, 69173, 460, 15, 29, "Input",
 CellTags->"Series2"],
Cell[1714767, 69190, 1000, 33, 43, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1715782, 69226, 113, 3, 24, "Text",
 CellTags->"Series2"],
Cell[CellGroupData[{
Cell[1715920, 69233, 484, 15, 26, "Input",
 CellTags->"Series2"],
Cell[1716407, 69250, 648, 24, 45, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1717092, 69279, 578, 17, 46, "Input",
 CellTags->"Series2"],
Cell[1717673, 69298, 988, 35, 42, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1718698, 69338, 242, 6, 26, "Input",
 CellTags->"Series2"],
Cell[1718943, 69346, 292, 7, 26, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1719272, 69358, 147, 4, 26, "Input",
 CellTags->"Series2"],
Cell[1719422, 69364, 3114, 106, 102, "Output",
 CellTags->"Series2"]
}, Open  ]],
Cell[1722551, 69473, 351, 11, 41, "Text",
 CellTags->"Series2"]
}, Open  ]],
Cell[1722917, 69487, 51, 1, 39, "ObjectName",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1722993, 69492, 58, 1, 36, "Subsubsection",
 CellTags->"Series3"],
Cell[1723054, 69495, 267, 5, 41, "Text",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1723346, 69504, 114, 3, 26, "Input",
 CellTags->"Series3"],
Cell[1723463, 69509, 262, 8, 26, "Output",
 CellTags->"Series3"]
}, Open  ]],
Cell[1723740, 69520, 256, 11, 24, "Text",
 CellTags->"Series3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1724033, 69536, 55, 1, 36, "Subsubsection",
 CellTags->"Series3"],
Cell[CellGroupData[{
Cell[1724113, 69541, 339, 12, 26, "Input",
 CellTags->"Series3"],
Cell[1724455, 69555, 307, 11, 42, "Output",
 CellTags->"Series3"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1724799, 69571, 214, 7, 26, "Input",
 CellTags->"Series3"],
Cell[1725016, 69580, 215, 7, 44, "Output",
 CellTags->"Series3"]
}, Open  ]]
}, Open  ]],
Cell[1725258, 69591, 63, 1, 39, "ObjectName",
 CellTags->"SetMandelstam"],
Cell[CellGroupData[{
Cell[1725346, 69596, 64, 1, 36, "Subsubsection",
 CellTags->"SetMandelstam"],
Cell[1725413, 69599, 462, 9, 83, "Text",
 CellTags->"SetMandelstam"],
Cell[CellGroupData[{
Cell[1725900, 69612, 126, 3, 26, "Input",
 CellTags->"SetMandelstam"],
Cell[1726029, 69617, 237, 8, 26, "Output",
 CellTags->"SetMandelstam"]
}, Open  ]],
Cell[1726281, 69628, 274, 11, 24, "Text",
 CellTags->"SetMandelstam"]
}, Open  ]],
Cell[1726570, 69642, 79, 1, 39, "ObjectName",
 CellTags->"SimplifyDeltaFunction"],
Cell[CellGroupData[{
Cell[1726674, 69647, 72, 1, 36, "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],
Cell[1726749, 69650, 299, 6, 41, "Text",
 CellTags->"SimplifyDeltaFunction"],
Cell[1727051, 69658, 526, 18, 24, "Text",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1727614, 69681, 69, 1, 36, "Subsubsection",
 CellTags->"SimplifyDeltaFunction"],
Cell[CellGroupData[{
Cell[1727708, 69686, 203, 6, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1727914, 69694, 268, 9, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1728219, 69708, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1728386, 69714, 392, 13, 37, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1728815, 69732, 203, 6, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1729021, 69740, 301, 10, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1729359, 69755, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1729526, 69761, 812, 27, 37, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1730375, 69793, 220, 6, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1730598, 69801, 313, 10, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1730948, 69816, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1731115, 69822, 212, 7, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1731364, 69834, 263, 8, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1731630, 69844, 330, 11, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1731997, 69860, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1732164, 69866, 234, 8, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1732435, 69879, 298, 9, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1732736, 69890, 371, 12, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1733144, 69907, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1733311, 69913, 118, 3, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1733466, 69921, 263, 8, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1733732, 69931, 330, 11, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1734099, 69947, 164, 4, 26, "Input",
 CellTags->"SimplifyDeltaFunction"],
Cell[1734266, 69953, 234, 8, 26, "Output",
 CellTags->"SimplifyDeltaFunction"]
}, Open  ]]
}, Open  ]],
Cell[1734527, 69965, 77, 1, 39, "ObjectName",
 CellTags->"SimplifyGTI"],
Cell[CellGroupData[{
Cell[1734629, 69970, 62, 1, 36, "Subsubsection",
 CellTags->"SimplifyGTI"],
Cell[1734694, 69973, 71, 1, 24, "Text",
 CellTags->"SimplifyGTI"],
Cell[1734768, 69976, 244, 11, 24, "Text",
 CellTags->"SimplifyGTI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1735049, 69992, 59, 1, 36, "Subsubsection",
 CellTags->"SimplifyGTI"],
Cell[1735111, 69995, 78, 2, 26, "Input",
 CellTags->"SimplifyGTI"]
}, Open  ]],
Cell[1735204, 70000, 67, 1, 39, "ObjectName",
 CellTags->"SimplifyPolyLog"],
Cell[CellGroupData[{
Cell[1735296, 70005, 66, 1, 36, "Subsubsection",
 CellTags->"SimplifyPolyLog"],
Cell[1735365, 70008, 199, 4, 24, "Text",
 CellTags->"SimplifyPolyLog"],
Cell[1735567, 70014, 264, 11, 24, "Text",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1735868, 70030, 63, 1, 36, "Subsubsection",
 CellTags->"SimplifyPolyLog"],
Cell[1735934, 70033, 224, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[CellGroupData[{
Cell[1736183, 70043, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1736381, 70051, 664, 20, 43, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1737082, 70076, 170, 5, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1737255, 70083, 432, 14, 26, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1737724, 70102, 221, 7, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1737948, 70111, 631, 20, 29, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1738616, 70136, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1738814, 70144, 608, 19, 29, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1739459, 70168, 271, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1739733, 70179, 670, 22, 42, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1740440, 70206, 248, 8, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1740691, 70216, 645, 21, 42, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1741373, 70242, 281, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1741657, 70253, 732, 29, 42, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1742426, 70287, 216, 7, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1742645, 70296, 496, 17, 46, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1743178, 70318, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1743376, 70326, 115, 3, 26, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1743528, 70334, 195, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1743726, 70342, 1063, 36, 29, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1744826, 70383, 271, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1745100, 70394, 1480, 50, 77, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1746617, 70449, 221, 7, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1746841, 70458, 1255, 45, 46, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1748133, 70508, 271, 9, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1748407, 70519, 2740, 96, 115, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1751184, 70620, 191, 6, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1751378, 70628, 336, 12, 45, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1751751, 70645, 166, 5, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1751920, 70652, 251, 8, 43, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1752208, 70665, 146, 4, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1752357, 70671, 406, 14, 43, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1752800, 70690, 146, 4, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1752949, 70696, 355, 13, 42, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1753341, 70714, 146, 4, 26, "Input",
 CellTags->"SimplifyPolyLog"],
Cell[1753490, 70720, 355, 13, 42, "Output",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[1753860, 70736, 116, 3, 26, "Input",
 CellTags->"SimplifyPolyLog"]
}, Open  ]],
Cell[1753991, 70742, 73, 1, 39, "ObjectName",
 CellTags->"Simplify2"],
Cell[CellGroupData[{
Cell[1754089, 70747, 60, 1, 36, "Subsubsection",
 CellTags->"Simplify2"],
Cell[1754152, 70750, 81, 1, 24, "Text",
 CellTags->"Simplify2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1754270, 70756, 57, 1, 36, "Subsubsection",
 CellTags->"Simplify2"],
Cell[1754330, 70759, 76, 2, 26, "Input",
 CellTags->"Simplify2"]
}, Open  ]],
Cell[1754421, 70764, 57, 1, 39, "ObjectName",
 CellTags->"SmallDelta"],
Cell[CellGroupData[{
Cell[1754503, 70769, 61, 1, 36, "Subsubsection",
 CellTags->"SmallDelta"],
Cell[1754567, 70772, 87, 1, 24, "Text",
 CellTags->"SmallDelta"]
}, Open  ]],
Cell[1754669, 70776, 61, 1, 39, "ObjectName",
 CellTags->"SmallEpsilon"],
Cell[CellGroupData[{
Cell[1754755, 70781, 63, 1, 36, "Subsubsection",
 CellTags->"SmallEpsilon"],
Cell[1754821, 70784, 91, 1, 24, "Text",
 CellTags->"SmallEpsilon"]
}, Open  ]],
Cell[1754927, 70788, 63, 1, 39, "ObjectName",
 CellTags->"SmallVariable"],
Cell[CellGroupData[{
Cell[1755015, 70793, 64, 1, 36, "Subsubsection",
 CellTags->"SmallVariable"],
Cell[1755082, 70796, 281, 5, 41, "Text",
 CellTags->"SmallVariable"],
Cell[1755366, 70803, 694, 25, 24, "Text",
 CellTags->"SmallVariable"]
}, Open  ]],
Cell[1756075, 70831, 65, 1, 39, "ObjectName",
 CellTags->"SmallVariables"],
Cell[CellGroupData[{
Cell[1756165, 70836, 65, 1, 36, "Subsubsection",
 CellTags->"SmallVariables"],
Cell[1756233, 70839, 225, 5, 41, "Text",
 CellTags->"SmallVariables"],
Cell[1756461, 70846, 263, 11, 24, "Text",
 CellTags->"SmallVariables"]
}, Open  ]],
Cell[1756739, 70860, 43, 1, 39, "ObjectName",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1756807, 70865, 54, 1, 36, "Subsubsection",
 CellTags->"SMP"],
Cell[1756864, 70868, 232, 5, 41, "Text",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1757121, 70877, 127, 4, 26, "Input",
 CellTags->"SMP"],
Cell[1757251, 70883, 166, 5, 38, "Output",
 CellTags->"SMP"]
}, Open  ]],
Cell[1757432, 70891, 250, 10, 24, "Text",
 CellTags->"SMP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1757719, 70906, 51, 1, 36, "Subsubsection",
 CellTags->"SMP"],
Cell[CellGroupData[{
Cell[1757795, 70911, 109, 3, 26, "Input",
 CellTags->"SMP"],
Cell[1757907, 70916, 101, 3, 26, "Output",
 CellTags->"SMP"]
}, Open  ]]
}, Open  ]],
Cell[1758035, 70923, 53, 1, 39, "ObjectName",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1758113, 70928, 59, 1, 36, "Subsubsection",
 CellTags->"SMVertex"],
Cell[1758175, 70931, 304, 6, 41, "Text",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1758504, 70941, 112, 3, 26, "Input",
 CellTags->"SMVertex"],
Cell[1758619, 70946, 235, 8, 26, "Output",
 CellTags->"SMVertex"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1758903, 70960, 56, 1, 36, "Subsubsection",
 CellTags->"SMVertex"],
Cell[CellGroupData[{
Cell[1758984, 70965, 223, 6, 26, "Input",
 CellTags->"SMVertex"],
Cell[1759210, 70973, 1179, 44, 28, "Output",
 CellTags->"SMVertex"]
}, Open  ]]
}, Open  ]],
Cell[1760416, 71021, 61, 1, 39, "ObjectName",
 CellTags->"Smu"],
Cell[CellGroupData[{
Cell[1760502, 71026, 54, 1, 36, "Subsubsection",
 CellTags->"Smu"],
Cell[1760559, 71029, 45, 1, 24, "Text",
 CellTags->"Smu"]
}, Open  ]],
Cell[1760619, 71033, 41, 1, 39, "ObjectName",
 CellTags->"Sn"],
Cell[CellGroupData[{
Cell[1760685, 71038, 53, 1, 36, "Subsubsection",
 CellTags->"Sn"],
Cell[1760741, 71041, 300, 14, 24, "Text",
 CellTags->"Sn"]
}, Open  ]],
Cell[1761056, 71058, 41, 1, 39, "ObjectName",
 CellTags->"SO"],
Cell[CellGroupData[{
Cell[1761122, 71063, 53, 1, 36, "Subsubsection",
 CellTags->"SO"],
Cell[1761178, 71066, 173, 4, 24, "Text",
 CellTags->"SO"],
Cell[1761354, 71072, 981, 38, 24, "Text",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1762372, 71115, 50, 1, 36, "Subsubsection",
 CellTags->"SO"],
Cell[CellGroupData[{
Cell[1762447, 71120, 98, 3, 26, "Input",
 CellTags->"SO"],
Cell[1762548, 71125, 265, 10, 28, "Output",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1762850, 71140, 121, 4, 26, "Input",
 CellTags->"SO"],
Cell[1762974, 71146, 384, 15, 28, "Output",
 CellTags->"SO"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1763395, 71166, 163, 6, 26, "Input",
 CellTags->"SO"],
Cell[1763561, 71174, 217, 6, 38, "Output",
 CellTags->"SO"]
}, Open  ]]
}, Open  ]],
Cell[1763805, 71184, 43, 1, 39, "ObjectName",
 CellTags->"SOD"],
Cell[CellGroupData[{
Cell[1763873, 71189, 54, 1, 36, "Subsubsection",
 CellTags->"SOD"],
Cell[1763930, 71192, 189, 4, 41, "Text",
 CellTags->"SOD"],
Cell[1764122, 71198, 816, 32, 24, "Text",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1764975, 71235, 51, 1, 36, "Subsubsection",
 CellTags->"SOD"],
Cell[CellGroupData[{
Cell[1765051, 71240, 100, 3, 26, "Input",
 CellTags->"SOD"],
Cell[1765154, 71245, 266, 10, 28, "Output",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1765457, 71260, 123, 4, 26, "Input",
 CellTags->"SOD"],
Cell[1765583, 71266, 385, 15, 28, "Output",
 CellTags->"SOD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1766005, 71286, 165, 6, 26, "Input",
 CellTags->"SOD"],
Cell[1766173, 71294, 268, 8, 38, "Output",
 CellTags->"SOD"]
}, Open  ]]
}, Open  ]],
Cell[1766468, 71306, 49, 1, 39, "ObjectName",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1766542, 71311, 57, 1, 36, "Subsubsection",
 CellTags->"Solve2"],
Cell[1766602, 71314, 346, 7, 41, "Text",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1766973, 71325, 112, 3, 26, "Input",
 CellTags->"Solve2"],
Cell[1767088, 71330, 264, 8, 26, "Output",
 CellTags->"Solve2"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1767401, 71344, 54, 1, 36, "Subsubsection",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1767480, 71349, 390, 14, 26, "Input",
 CellTags->"Solve2"],
Cell[1767873, 71365, 373, 13, 44, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[1768261, 71381, 111, 3, 24, "Text",
 CellTags->"Solve2"],
Cell[CellGroupData[{
Cell[1768397, 71388, 153, 5, 26, "Input",
 CellTags->"Solve2"],
Cell[1768553, 71395, 176, 6, 26, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1768766, 71406, 267, 8, 26, "Input",
 CellTags->"Solve2"],
Cell[1769036, 71416, 176, 6, 26, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1769249, 71427, 443, 15, 26, "Input",
 CellTags->"Solve2"],
Cell[1769695, 71444, 314, 11, 45, "Output",
 CellTags->"Solve2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1770046, 71460, 389, 14, 26, "Input",
 CellTags->"Solve2"],
Cell[1770438, 71476, 406, 14, 44, "Output",
 CellTags->"Solve2"]
}, Open  ]]
}, Open  ]],
Cell[1770871, 71494, 49, 1, 39, "ObjectName",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1770945, 71499, 57, 1, 36, "Subsubsection",
 CellTags->"Solve3"],
Cell[1771005, 71502, 271, 5, 41, "Text",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1771301, 71511, 112, 3, 26, "Input",
 CellTags->"Solve3"],
Cell[1771416, 71516, 320, 10, 26, "Output",
 CellTags->"Solve3"]
}, Open  ]],
Cell[1771751, 71529, 242, 10, 24, "Text",
 CellTags->"SMP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1772030, 71544, 54, 1, 36, "Subsubsection",
 CellTags->"Solve3"],
Cell[CellGroupData[{
Cell[1772109, 71549, 390, 14, 26, "Input",
 CellTags->"Solve3"],
Cell[1772502, 71565, 373, 13, 44, "Output",
 CellTags->"Solve3"]
}, Open  ]]
}, Open  ]],
Cell[1772902, 71582, 41, 1, 39, "ObjectName",
 CellTags->"SP"],
Cell[CellGroupData[{
Cell[1772968, 71587, 53, 1, 36, "Subsubsection",
 CellTags->"SP"],
Cell[1773024, 71590, 327, 11, 43, "Text",
 CellTags->"SP"],
Cell[1773354, 71603, 687, 25, 24, "Text",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1774078, 71633, 50, 1, 36, "Subsubsection",
 CellTags->"SP"],
Cell[CellGroupData[{
Cell[1774153, 71638, 184, 6, 26, "Input",
 CellTags->"SP"],
Cell[1774340, 71646, 366, 16, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1774743, 71667, 189, 7, 26, "Input",
 CellTags->"SP"],
Cell[1774935, 71676, 297, 11, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1775269, 71692, 234, 8, 26, "Input",
 CellTags->"SP"],
Cell[1775506, 71702, 412, 17, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1775955, 71724, 168, 5, 26, "Input",
 CellTags->"SP"],
Cell[1776126, 71731, 415, 17, 28, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1776578, 71753, 157, 5, 26, "Input",
 CellTags->"SP"],
Cell[1776738, 71760, 136, 4, 38, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1776911, 71769, 188, 7, 26, "Input",
 CellTags->"SP"],
Cell[1777102, 71778, 210, 6, 38, "Output",
 CellTags->"SP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1777349, 71789, 217, 8, 26, "Input",
 CellTags->"SP"],
Cell[1777569, 71799, 136, 4, 38, "Output",
 CellTags->"SP"]
}, Open  ]]
}, Open  ]],
Cell[1777732, 71807, 43, 1, 39, "ObjectName",
 CellTags->"SPC"],
Cell[CellGroupData[{
Cell[1777800, 71812, 54, 1, 36, "Subsubsection",
 CellTags->"SPC"],
Cell[1777857, 71815, 81, 1, 24, "Text",
 CellTags->"SPC"],
Cell[1777941, 71818, 300, 11, 24, "Text",
 CellTags->"SPC"]
}, Open  ]],
Cell[1778256, 71832, 43, 1, 39, "ObjectName",
 CellTags->"SPD"],
Cell[CellGroupData[{
Cell[1778324, 71837, 54, 1, 36, "Subsubsection",
 CellTags->"SPD"],
Cell[1778381, 71840, 346, 11, 43, "Text",
 CellTags->"SPD"],
Cell[1778730, 71853, 856, 32, 24, "Text",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1779623, 71890, 51, 1, 36, "Subsubsection",
 CellTags->"SPD"],
Cell[CellGroupData[{
Cell[1779699, 71895, 187, 6, 26, "Input",
 CellTags->"SPD"],
Cell[1779889, 71903, 367, 16, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1780293, 71924, 191, 7, 26, "Input",
 CellTags->"SPD"],
Cell[1780487, 71933, 298, 11, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1780822, 71949, 236, 8, 26, "Input",
 CellTags->"SPD"],
Cell[1781061, 71959, 413, 17, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1781511, 71981, 170, 5, 26, "Input",
 CellTags->"SPD"],
Cell[1781684, 71988, 416, 17, 28, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1782137, 72010, 159, 5, 26, "Input",
 CellTags->"SPD"],
Cell[1782299, 72017, 138, 4, 38, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1782474, 72026, 190, 7, 26, "Input",
 CellTags->"SPD"],
Cell[1782667, 72035, 261, 8, 38, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1782965, 72048, 219, 8, 26, "Input",
 CellTags->"SPD"],
Cell[1783187, 72058, 138, 4, 38, "Output",
 CellTags->"SPD"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1783362, 72067, 270, 9, 26, "Input",
 CellTags->"SPD"],
Cell[1783635, 72078, 138, 4, 38, "Output",
 CellTags->"SPD"]
}, Open  ]]
}, Open  ]],
Cell[1783800, 72086, 49, 1, 39, "ObjectName",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1783874, 72091, 57, 1, 36, "Subsubsection",
 CellTags->"Spinor"],
Cell[1783934, 72094, 1326, 49, 108, "Text",
 CellTags->"Spinor"],
Cell[1785263, 72145, 1271, 46, 24, "Text",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1786571, 72196, 54, 1, 36, "Subsubsection",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1786650, 72201, 106, 3, 26, "Input",
 CellTags->"Spinor"],
Cell[1786759, 72206, 175, 6, 28, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1786971, 72217, 129, 4, 26, "Input",
 CellTags->"Spinor"],
Cell[1787103, 72223, 185, 6, 28, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1787303, 72232, 146, 4, 24, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1787474, 72240, 250, 8, 26, "Input",
 CellTags->"Spinor"],
Cell[1787727, 72250, 126, 4, 26, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1787890, 72259, 254, 8, 26, "Input",
 CellTags->"Spinor"],
Cell[1788147, 72269, 251, 9, 28, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1788435, 72283, 141, 4, 26, "Input",
 CellTags->"Spinor"],
Cell[1788579, 72289, 192, 6, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1788786, 72298, 158, 4, 24, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1788969, 72306, 212, 7, 26, "Input",
 CellTags->"Spinor"],
Cell[1789184, 72315, 192, 6, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1789413, 72326, 165, 5, 26, "Input",
 CellTags->"Spinor"],
Cell[1789581, 72333, 192, 6, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]],
Cell[1789788, 72342, 77, 1, 24, "Text",
 CellTags->"Spinor"],
Cell[CellGroupData[{
Cell[1789890, 72347, 210, 7, 26, "Input",
 CellTags->"Spinor"],
Cell[1790103, 72356, 192, 6, 38, "Output",
 CellTags->"Spinor"]
}, Open  ]]
}, Open  ]],
Cell[1790322, 72366, 63, 1, 39, "ObjectName",
 CellTags->"SpinorCollect"],
Cell[CellGroupData[{
Cell[1790410, 72371, 64, 1, 36, "Subsubsection",
 CellTags->"SpinorCollect"],
Cell[1790477, 72374, 191, 4, 24, "Text",
 CellTags->"SpinorCollect"],
Cell[1790671, 72380, 469, 17, 24, "Text",
 CellTags->"SpinorCollect"]
}, Open  ]],
Cell[1791155, 72400, 57, 1, 39, "ObjectName",
 CellTags->"SpinorUBar"],
Cell[CellGroupData[{
Cell[1791237, 72405, 61, 1, 36, "Subsubsection",
 CellTags->"SpinorUBar"],
Cell[1791301, 72408, 170, 7, 24, "Text",
 CellTags->"SpinorUBar"],
Cell[1791474, 72417, 831, 32, 24, "Text",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1792342, 72454, 58, 1, 36, "Subsubsection",
 CellTags->"SpinorUBar"],
Cell[1792403, 72457, 85, 1, 24, "Text",
 CellTags->"SpinorUBar"],
Cell[CellGroupData[{
Cell[1792513, 72462, 114, 3, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1792630, 72467, 202, 7, 28, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1792869, 72479, 137, 4, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1793009, 72485, 212, 7, 28, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1793258, 72497, 173, 5, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1793434, 72504, 152, 4, 38, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1793623, 72513, 204, 7, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1793830, 72522, 196, 6, 38, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1794063, 72533, 233, 8, 26, "Input",
 CellTags->"SpinorUBar"],
Cell[1794299, 72543, 196, 6, 38, "Output",
 CellTags->"SpinorUBar"]
}, Open  ]]
}, Open  ]],
Cell[1794522, 72553, 51, 1, 39, "ObjectName",
 CellTags->"SpinorU"],
Cell[CellGroupData[{
Cell[1794598, 72558, 58, 1, 36, "Subsubsection",
 CellTags->"SpinorU"],
Cell[1794659, 72561, 79, 1, 24, "Text",
 CellTags->"SpinorU"],
Cell[1794741, 72564, 840, 32, 24, "Text",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1795618, 72601, 55, 1, 36, "Subsubsection",
 CellTags->"SpinorU"],
Cell[CellGroupData[{
Cell[1795698, 72606, 108, 3, 26, "Input",
 CellTags->"SpinorU"],
Cell[1795809, 72611, 132, 4, 28, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1795978, 72620, 131, 4, 26, "Input",
 CellTags->"SpinorU"],
Cell[1796112, 72626, 142, 4, 28, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1796291, 72635, 167, 5, 26, "Input",
 CellTags->"SpinorU"],
Cell[1796461, 72642, 146, 4, 38, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1796644, 72651, 198, 7, 26, "Input",
 CellTags->"SpinorU"],
Cell[1796845, 72660, 193, 6, 38, "Output",
 CellTags->"SpinorU"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1797075, 72671, 227, 8, 26, "Input",
 CellTags->"SpinorU"],
Cell[1797305, 72681, 193, 6, 38, "Output",
 CellTags->"SpinorU"]
}, Open  ]]
}, Open  ]],
Cell[1797525, 72691, 57, 1, 39, "ObjectName",
 CellTags->"SpinorVBar"],
Cell[CellGroupData[{
Cell[1797607, 72696, 61, 1, 36, "Subsubsection",
 CellTags->"SpinorVBar"],
Cell[1797671, 72699, 175, 7, 24, "Text",
 CellTags->"SpinorVBar"],
Cell[1797849, 72708, 831, 32, 24, "Text",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1798717, 72745, 58, 1, 36, "Subsubsection",
 CellTags->"SpinorVBar"],
Cell[CellGroupData[{
Cell[1798800, 72750, 114, 3, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1798917, 72755, 202, 7, 28, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1799156, 72767, 137, 4, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1799296, 72773, 212, 7, 28, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1799545, 72785, 173, 5, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1799721, 72792, 152, 4, 38, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1799910, 72801, 204, 7, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1800117, 72810, 217, 7, 38, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1800371, 72822, 233, 8, 26, "Input",
 CellTags->"SpinorVBar"],
Cell[1800607, 72832, 217, 7, 38, "Output",
 CellTags->"SpinorVBar"]
}, Open  ]]
}, Open  ]],
Cell[1800851, 72843, 51, 1, 39, "ObjectName",
 CellTags->"SpinorV"],
Cell[CellGroupData[{
Cell[1800927, 72848, 58, 1, 36, "Subsubsection",
 CellTags->"SpinorV"],
Cell[1800988, 72851, 79, 1, 24, "Text",
 CellTags->"SpinorV"],
Cell[1801070, 72854, 840, 32, 24, "Text",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1801947, 72891, 55, 1, 36, "Subsubsection",
 CellTags->"SpinorV"],
Cell[CellGroupData[{
Cell[1802027, 72896, 108, 3, 26, "Input",
 CellTags->"SpinorV"],
Cell[1802138, 72901, 132, 4, 28, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1802307, 72910, 131, 4, 26, "Input",
 CellTags->"SpinorV"],
Cell[1802441, 72916, 142, 4, 28, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1802620, 72925, 167, 5, 26, "Input",
 CellTags->"SpinorV"],
Cell[1802790, 72932, 146, 4, 38, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1802973, 72941, 198, 7, 26, "Input",
 CellTags->"SpinorV"],
Cell[1803174, 72950, 214, 7, 38, "Output",
 CellTags->"SpinorV"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1803425, 72962, 227, 8, 26, "Input",
 CellTags->"SpinorV"],
Cell[1803655, 72972, 214, 7, 38, "Output",
 CellTags->"SpinorV"]
}, Open  ]]
}, Open  ]],
Cell[1803896, 72983, 75, 1, 39, "ObjectName",
 CellTags->"SpinPolarizationSum"],
Cell[CellGroupData[{
Cell[1803996, 72988, 70, 1, 36, "Subsubsection",
 CellTags->"SpinPolarizationSum"],
Cell[1804069, 72991, 164, 4, 24, "Text",
 CellTags->"SpinPolarizationSum"],
Cell[1804236, 72997, 291, 10, 24, "Text",
 CellTags->"SpinPolarizationSum"]
}, Open  ]],
Cell[1804542, 73010, 43, 1, 39, "ObjectName",
 CellTags->"SPL"],
Cell[CellGroupData[{
Cell[1804610, 73015, 54, 1, 36, "Subsubsection",
 CellTags->"SPL"],
Cell[1804667, 73018, 77, 1, 24, "Text",
 CellTags->"SPL"],
Cell[1804747, 73021, 284, 11, 24, "Text",
 CellTags->"SPL"]
}, Open  ]],
Cell[1805046, 73035, 71, 1, 39, "ObjectName",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1805142, 73040, 68, 1, 36, "Subsubsection",
 CellTags->"SplittingFunction"],
Cell[1805213, 73043, 218, 7, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1805456, 73054, 134, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1805593, 73059, 181, 6, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[1805789, 73068, 310, 11, 24, "Text",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1806136, 73084, 65, 1, 36, "Subsubsection",
 CellTags->"SplittingFunction"],
Cell[1806204, 73087, 65, 1, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[1806272, 73090, 221, 6, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1806496, 73098, 170, 4, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1806691, 73106, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1806824, 73111, 531, 19, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1807392, 73135, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1807525, 73140, 301, 10, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1807863, 73155, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1807996, 73160, 269, 9, 43, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1808302, 73174, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1808435, 73179, 859, 29, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1809331, 73213, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1809464, 73218, 1128, 38, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1810629, 73261, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1810762, 73266, 599, 21, 43, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1811398, 73292, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1811531, 73297, 541, 19, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[1812087, 73319, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1812220, 73324, 63, 1, 24, "Text",
 CellTags->"SplittingFunction"],
Cell[1812286, 73327, 221, 6, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[CellGroupData[{
Cell[1812532, 73337, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1812665, 73342, 531, 19, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1813233, 73366, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1813366, 73371, 236, 8, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1813639, 73384, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1813772, 73389, 235, 8, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1814044, 73402, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1814177, 73407, 811, 28, 44, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1815025, 73440, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1815158, 73445, 1247, 42, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1816442, 73492, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1816575, 73497, 503, 18, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1817115, 73520, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1817249, 73525, 445, 16, 26, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1817731, 73546, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1817864, 73551, 445, 16, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1818346, 73572, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1818480, 73577, 445, 16, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1818962, 73598, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1819095, 73603, 1362, 46, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1820494, 73654, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1820628, 73659, 1362, 46, 46, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1822027, 73710, 131, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1822161, 73715, 637, 22, 29, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1822835, 73742, 132, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1822970, 73747, 5385, 174, 230, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1828392, 73926, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1828525, 73931, 2906, 92, 98, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1831468, 74028, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1831601, 74033, 3584, 117, 149, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1835222, 74155, 130, 3, 26, "Input",
 CellTags->"SplittingFunction"],
Cell[1835355, 74160, 4258, 139, 193, "Output",
 CellTags->"SplittingFunction"]
}, Open  ]]
}, Open  ]],
Cell[1839640, 74303, 79, 1, 39, "ObjectName",
 CellTags->"StandardMatrixElement"],
Cell[CellGroupData[{
Cell[1839744, 74308, 72, 1, 36, "Subsubsection",
 CellTags->"StandardMatrixElement"],
Cell[1839819, 74311, 134, 3, 24, "Text",
 CellTags->"StandardMatrixElement"],
Cell[1839956, 74316, 270, 11, 24, "Text",
 CellTags->"StandardMatrixElement"]
}, Open  ]],
Cell[1840241, 74330, 57, 1, 39, "ObjectName",
 CellTags->"SubContext"],
Cell[CellGroupData[{
Cell[1840323, 74335, 61, 1, 36, "Subsubsection",
 CellTags->"SubContext"],
Cell[1840387, 74338, 120, 3, 24, "Text",
 CellTags->"SubContext"],
Cell[1840510, 74343, 439, 18, 24, "Text",
 CellTags->"SubContext"]
}, Open  ]],
Cell[1840964, 74364, 51, 1, 39, "ObjectName",
 CellTags->"SubLoop"],
Cell[CellGroupData[{
Cell[1841040, 74369, 58, 1, 36, "Subsubsection",
 CellTags->"SubLoop"],
Cell[1841101, 74372, 152, 4, 24, "Text",
 CellTags->"SubLoop"],
Cell[1841256, 74378, 260, 11, 24, "Text",
 CellTags->"SubLoop"]
}, Open  ]],
Cell[1841531, 74392, 45, 1, 39, "ObjectName",
 CellTags->"SumP"],
Cell[CellGroupData[{
Cell[1841601, 74397, 55, 1, 36, "Subsubsection",
 CellTags->"SumP"],
Cell[1841659, 74400, 767, 31, 27, "Text",
 CellTags->"SumP"],
Cell[1842429, 74433, 417, 18, 24, "Text",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1842883, 74456, 52, 1, 36, "Subsubsection",
 CellTags->"SumP"],
Cell[CellGroupData[{
Cell[1842960, 74461, 149, 5, 26, "Input",
 CellTags->"SumP"],
Cell[1843112, 74468, 263, 8, 28, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1843412, 74481, 149, 5, 26, "Input",
 CellTags->"SumP"],
Cell[1843564, 74488, 263, 8, 28, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1843864, 74501, 125, 4, 26, "Input",
 CellTags->"SumP"],
Cell[1843992, 74507, 238, 7, 28, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1844267, 74519, 125, 4, 26, "Input",
 CellTags->"SumP"],
Cell[1844395, 74525, 124, 4, 43, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1844556, 74534, 301, 12, 53, "Input",
 CellTags->"SumP"],
Cell[1844860, 74548, 124, 4, 43, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1845021, 74557, 186, 6, 26, "Input",
 CellTags->"SumP"],
Cell[1845210, 74565, 764, 29, 43, "Output",
 CellTags->"SumP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1846011, 74599, 124, 4, 26, "Input",
 CellTags->"SumP"],
Cell[1846138, 74605, 124, 4, 43, "Output",
 CellTags->"SumP"]
}, Open  ]]
}, Open  ]],
Cell[1846289, 74613, 45, 1, 39, "ObjectName",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1846359, 74618, 55, 1, 36, "Subsubsection",
 CellTags->"SumS"],
Cell[1846417, 74621, 1438, 57, 44, "Text",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1847880, 74682, 108, 3, 26, "Input",
 CellTags->"SumS"],
Cell[1847991, 74687, 165, 6, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1848171, 74696, 417, 18, 24, "Text",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1848625, 74719, 52, 1, 36, "Subsubsection",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1848702, 74724, 149, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1848854, 74731, 211, 8, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1849102, 74744, 149, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1849254, 74751, 211, 8, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1849502, 74764, 144, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1849649, 74771, 172, 6, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1849858, 74782, 173, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1850034, 74789, 261, 10, 44, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1850332, 74804, 197, 6, 26, "Input",
 CellTags->"SumS"],
Cell[1850532, 74812, 340, 13, 47, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1850887, 74828, 192, 6, 26, "Input",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1851104, 74838, 149, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1851256, 74845, 509, 19, 47, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[1851780, 74867, 193, 6, 26, "Input",
 CellTags->"SumS"],
Cell[CellGroupData[{
Cell[1851998, 74877, 125, 4, 26, "Input",
 CellTags->"SumS"],
Cell[1852126, 74883, 124, 4, 43, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1852287, 74892, 174, 6, 53, "Input",
 CellTags->"SumS"],
Cell[1852464, 74900, 124, 4, 43, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1852625, 74909, 159, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1852787, 74916, 212, 8, 26, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1853036, 74929, 146, 4, 26, "Input",
 CellTags->"SumS"],
Cell[1853185, 74935, 149, 5, 44, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1853371, 74945, 144, 5, 26, "Input",
 CellTags->"SumS"],
Cell[1853518, 74952, 142, 5, 42, "Output",
 CellTags->"SumS"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1853697, 74962, 125, 4, 26, "Input",
 CellTags->"SumS"],
Cell[1853825, 74968, 142, 5, 42, "Output",
 CellTags->"SumS"]
}, Open  ]]
}, Open  ]],
Cell[1853994, 74977, 45, 1, 39, "ObjectName",
 CellTags->"SumT"],
Cell[CellGroupData[{
Cell[1854064, 74982, 55, 1, 36, "Subsubsection",
 CellTags->"SumT"],
Cell[1854122, 74985, 461, 15, 43, "Text",
 CellTags->"SumT"],
Cell[1854586, 75002, 417, 18, 24, "Text",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1855040, 75025, 52, 1, 36, "Subsubsection",
 CellTags->"SumT"],
Cell[CellGroupData[{
Cell[1855117, 75030, 149, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1855269, 75037, 236, 9, 26, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1855542, 75051, 149, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1855694, 75058, 236, 9, 26, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1855967, 75072, 125, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1856095, 75078, 180, 7, 26, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1856312, 75090, 173, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1856488, 75097, 361, 14, 45, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1856886, 75116, 125, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1857014, 75122, 142, 5, 42, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1857193, 75132, 244, 9, 53, "Input",
 CellTags->"SumT"],
Cell[1857440, 75143, 142, 5, 42, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1857619, 75153, 159, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1857781, 75160, 237, 9, 26, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1858055, 75174, 136, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1858194, 75180, 253, 7, 44, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1858484, 75192, 125, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1858612, 75198, 142, 5, 42, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1858791, 75208, 144, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1858938, 75215, 142, 5, 42, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1859117, 75225, 268, 10, 53, "Input",
 CellTags->"SumT"],
Cell[1859388, 75237, 336, 12, 26, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1859761, 75254, 136, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1859900, 75260, 170, 6, 43, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1860107, 75271, 155, 5, 26, "Input",
 CellTags->"SumT"],
Cell[1860265, 75278, 253, 7, 44, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1860555, 75290, 129, 4, 26, "Input",
 CellTags->"SumT"],
Cell[1860687, 75296, 458, 16, 45, "Output",
 CellTags->"SumT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1861182, 75317, 240, 9, 26, "Input",
 CellTags->"SumT"],
Cell[1861425, 75328, 458, 16, 45, "Output",
 CellTags->"SumT"]
}, Open  ]]
}, Open  ]],
Cell[1861910, 75348, 45, 1, 39, "ObjectName",
 CellTags->"SUND"],
Cell[CellGroupData[{
Cell[1861980, 75353, 55, 1, 36, "Subsubsection",
 CellTags->"SUND"],
Cell[1862038, 75356, 224, 10, 24, "Text",
 CellTags->"SUND"],
Cell[1862265, 75368, 637, 25, 24, "Text",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1862939, 75398, 52, 1, 36, "Subsubsection",
 CellTags->"SUND"],
Cell[CellGroupData[{
Cell[1863016, 75403, 135, 4, 26, "Input",
 CellTags->"SUND"],
Cell[1863154, 75409, 147, 5, 26, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1863338, 75419, 211, 6, 26, "Input",
 CellTags->"SUND"],
Cell[1863552, 75427, 1086, 44, 26, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1864675, 75476, 135, 4, 26, "Input",
 CellTags->"SUND"],
Cell[1864813, 75482, 147, 5, 26, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1864997, 75492, 135, 4, 26, "Input",
 CellTags->"SUND"],
Cell[1865135, 75498, 147, 5, 26, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1865319, 75508, 266, 8, 26, "Input",
 CellTags->"SUND"],
Cell[1865588, 75518, 260, 9, 29, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1865885, 75532, 377, 12, 26, "Input",
 CellTags->"SUND"],
Cell[1866265, 75546, 304, 11, 46, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1866606, 75562, 377, 12, 26, "Input",
 CellTags->"SUND"],
Cell[1866986, 75576, 474, 19, 46, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1867497, 75600, 173, 6, 26, "Input",
 CellTags->"SUND"],
Cell[1867673, 75608, 150, 4, 38, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1867860, 75617, 202, 7, 26, "Input",
 CellTags->"SUND"],
Cell[1868065, 75626, 258, 7, 38, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1868360, 75638, 231, 8, 26, "Input",
 CellTags->"SUND"],
Cell[1868594, 75648, 150, 4, 38, "Output",
 CellTags->"SUND"]
}, Open  ]],
Cell[1868759, 75655, 104, 3, 26, "Input",
 CellTags->"SUND"]
}, Open  ]],
Cell[1868878, 75661, 53, 1, 39, "ObjectName",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1868956, 75666, 59, 1, 36, "Subsubsection",
 CellTags->"SUNDelta"],
Cell[1869018, 75669, 172, 6, 24, "Text",
 CellTags->"SUNDelta"],
Cell[1869193, 75677, 1207, 45, 24, "Text",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1870437, 75727, 56, 1, 36, "Subsubsection",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1870518, 75732, 133, 4, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1870654, 75738, 153, 5, 26, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1870844, 75748, 262, 8, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1871109, 75758, 301, 13, 26, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1871447, 75776, 205, 6, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1871655, 75784, 301, 13, 26, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1871993, 75802, 169, 5, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1872165, 75809, 148, 4, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1872350, 75818, 200, 7, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1872553, 75827, 220, 6, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1872810, 75838, 229, 8, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1873042, 75848, 142, 4, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1873221, 75857, 194, 7, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1873418, 75866, 220, 6, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[1873653, 75875, 256, 5, 41, "Text",
 CellTags->"SUNDelta"],
Cell[CellGroupData[{
Cell[1873934, 75884, 315, 10, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1874252, 75896, 460, 20, 26, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1874749, 75921, 110, 3, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1874862, 75926, 395, 11, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1875294, 75942, 200, 7, 26, "Input",
 CellTags->"SUNDelta"],
Cell[1875497, 75951, 236, 6, 38, "Output",
 CellTags->"SUNDelta"]
}, Open  ]]
}, Open  ]],
Cell[1875760, 75961, 69, 1, 39, "ObjectName",
 CellTags->"SUNDeltaContract"],
Cell[CellGroupData[{
Cell[1875854, 75966, 67, 1, 36, "Subsubsection",
 CellTags->"SUNDeltaContract"],
Cell[1875924, 75969, 397, 9, 58, "Text",
 CellTags->"SUNDeltaContract"],
Cell[1876324, 75980, 461, 18, 24, "Text",
 CellTags->"SUNDeltaContract"]
}, Open  ]],
Cell[1876800, 76001, 45, 1, 39, "ObjectName",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1876870, 76006, 55, 1, 36, "Subsubsection",
 CellTags->"SUNF"],
Cell[1876928, 76009, 190, 6, 24, "Text",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1877143, 76019, 108, 3, 26, "Input",
 CellTags->"SUNF"],
Cell[1877254, 76024, 168, 6, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[1877437, 76033, 1185, 46, 24, "Text",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1878659, 76084, 52, 1, 36, "Subsubsection",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1878736, 76089, 294, 10, 26, "Input",
 CellTags->"SUNF"],
Cell[1879033, 76101, 272, 10, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1879342, 76116, 103, 3, 26, "Input",
 CellTags->"SUNF"],
Cell[1879448, 76121, 764, 34, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1880249, 76160, 110, 3, 26, "Input",
 CellTags->"SUNF"],
Cell[1880362, 76165, 455, 20, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1880854, 76190, 135, 4, 26, "Input",
 CellTags->"SUNF"],
Cell[1880992, 76196, 147, 5, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1881176, 76206, 163, 5, 26, "Input",
 CellTags->"SUNF"],
Cell[1881342, 76213, 101, 3, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[1881458, 76219, 319, 10, 24, "Text",
 CellTags->"SUNF"],
Cell[CellGroupData[{
Cell[1881802, 76233, 185, 5, 26, "Input",
 CellTags->"SUNF"],
Cell[1881990, 76240, 1147, 44, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1883174, 76289, 266, 8, 26, "Input",
 CellTags->"SUNF"],
Cell[1883443, 76299, 353, 15, 28, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1883833, 76319, 239, 7, 26, "Input",
 CellTags->"SUNF"],
Cell[1884075, 76328, 1166, 45, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1885278, 76378, 173, 6, 26, "Input",
 CellTags->"SUNF"],
Cell[1885454, 76386, 150, 4, 38, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1885641, 76395, 202, 7, 26, "Input",
 CellTags->"SUNF"],
Cell[1885846, 76404, 258, 7, 38, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1886141, 76416, 231, 8, 26, "Input",
 CellTags->"SUNF"],
Cell[1886375, 76426, 150, 4, 38, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1886562, 76435, 135, 4, 26, "Input",
 CellTags->"SUNF"],
Cell[1886700, 76441, 147, 5, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1886884, 76451, 162, 5, 26, "Input",
 CellTags->"SUNF"],
Cell[1887049, 76458, 401, 18, 26, "Output",
 CellTags->"SUNF"]
}, Open  ]]
}, Open  ]],
Cell[1887477, 76480, 57, 1, 39, "ObjectName",
 CellTags->"SUNFJacobi"],
Cell[CellGroupData[{
Cell[1887559, 76485, 61, 1, 36, "Subsubsection",
 CellTags->"SUNFJacobi"],
Cell[1887623, 76488, 146, 4, 24, "Text",
 CellTags->"SUNFJacobi"],
Cell[1887772, 76494, 446, 17, 24, "Text",
 CellTags->"SUNFJacobi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1888255, 76516, 58, 1, 36, "Subsubsection",
 CellTags->"SUNFJacobi"],
Cell[CellGroupData[{
Cell[1888338, 76521, 380, 12, 26, "Input",
 CellTags->"SUNFJacobi"],
Cell[1888721, 76535, 694, 32, 29, "Output",
 CellTags->"SUNFJacobi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1889452, 76572, 379, 12, 26, "Input",
 CellTags->"SUNFJacobi"],
Cell[1889834, 76586, 1362, 62, 29, "Output",
 CellTags->"SUNFJacobi"]
}, Open  ]]
}, Open  ]],
Cell[1891223, 76652, 53, 1, 39, "ObjectName",
 CellTags->"SUNIndex"],
Cell[CellGroupData[{
Cell[1891301, 76657, 59, 1, 36, "Subsubsection",
 CellTags->"SUNIndex"],
Cell[1891363, 76660, 135, 6, 24, "Text",
 CellTags->"SUNIndex"],
Cell[1891501, 76668, 655, 24, 24, "Text",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1892193, 76697, 56, 1, 36, "Subsubsection",
 CellTags->"SUNIndex"],
Cell[CellGroupData[{
Cell[1892274, 76702, 110, 3, 26, "Input",
 CellTags->"SUNIndex"],
Cell[1892387, 76707, 105, 3, 27, "Output",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1892529, 76715, 110, 3, 26, "Input",
 CellTags->"SUNIndex"],
Cell[1892642, 76720, 125, 3, 38, "Output",
 CellTags->"SUNIndex"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1892804, 76728, 200, 7, 26, "Input",
 CellTags->"SUNIndex"],
Cell[1893007, 76737, 220, 6, 38, "Output",
 CellTags->"SUNIndex"]
}, Open  ]]
}, Open  ]],
Cell[1893254, 76747, 65, 1, 39, "ObjectName",
 CellTags->"SUNIndexRename"],
Cell[CellGroupData[{
Cell[1893344, 76752, 65, 1, 36, "Subsubsection",
 CellTags->"SUNIndexRename"],
Cell[1893412, 76755, 163, 4, 24, "Text",
 CellTags->"SUNIndexRename"],
Cell[1893578, 76761, 471, 18, 24, "Text",
 CellTags->"SUNIndexRename"]
}, Open  ]],
Cell[1894064, 76782, 45, 1, 39, "ObjectName",
 CellTags->"SUNN"],
Cell[CellGroupData[{
Cell[1894134, 76787, 55, 1, 36, "Subsubsection",
 CellTags->"SUNN"],
Cell[1894192, 76790, 203, 7, 24, "Text",
 CellTags->"SUNN"],
Cell[1894398, 76799, 977, 39, 24, "Text",
 CellTags->"SUNN"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1895412, 76843, 52, 1, 36, "Subsubsection",
 CellTags->"SUNN"],
Cell[CellGroupData[{
Cell[1895489, 76848, 317, 9, 26, "Input",
 CellTags->"SUNN"],
Cell[1895809, 76859, 149, 5, 26, "Output",
 CellTags->"SUNN"]
}, Open  ]]
}, Open  ]],
Cell[1895985, 76868, 57, 1, 39, "ObjectName",
 CellTags->"SUNNToCACF"],
Cell[CellGroupData[{
Cell[1896067, 76873, 61, 1, 36, "Subsubsection",
 CellTags->"SUNNToCACF"],
Cell[1896131, 76876, 187, 4, 24, "Text",
 CellTags->"SUNNToCACF"],
Cell[1896321, 76882, 967, 39, 24, "Text",
 CellTags->"SUNNToCACF"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1897325, 76926, 58, 1, 36, "Subsubsection",
 CellTags->"SUNNToCACF"],
Cell[CellGroupData[{
Cell[1897408, 76931, 322, 9, 26, "Input",
 CellTags->"SUNNToCACF"],
Cell[1897733, 76942, 186, 6, 26, "Output",
 CellTags->"SUNNToCACF"]
}, Open  ]]
}, Open  ]],
Cell[1897946, 76952, 59, 1, 39, "ObjectName",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1898030, 76957, 62, 1, 36, "Subsubsection",
 CellTags->"SUNSimplify"],
Cell[1898095, 76960, 376, 7, 58, "Text",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1898496, 76971, 122, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1898621, 76976, 512, 13, 47, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1899148, 76992, 467, 17, 24, "Text",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1899652, 77014, 59, 1, 36, "Subsubsection",
 CellTags->"SUNSimplify"],
Cell[CellGroupData[{
Cell[1899736, 77019, 247, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1899986, 77029, 231, 8, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1900254, 77042, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1900374, 77047, 304, 13, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1900715, 77065, 189, 6, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1900907, 77073, 249, 10, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1901193, 77088, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1901313, 77093, 130, 4, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1901480, 77102, 183, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1901666, 77109, 203, 7, 44, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1901906, 77121, 254, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1902163, 77131, 227, 8, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1902427, 77144, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1902547, 77149, 360, 15, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1902944, 77169, 269, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1903216, 77179, 227, 8, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1903480, 77192, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1903600, 77197, 197, 6, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1903834, 77208, 259, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1904096, 77218, 227, 8, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1904360, 77231, 117, 3, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1904480, 77236, 360, 15, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1904877, 77256, 259, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1905139, 77266, 227, 8, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1905403, 77279, 180, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1905586, 77286, 108, 3, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1905731, 77294, 384, 12, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1906118, 77308, 311, 11, 46, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1906466, 77324, 389, 12, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1906858, 77338, 448, 18, 46, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1907343, 77361, 246, 7, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1907592, 77370, 1173, 45, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1908802, 77420, 246, 7, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1909051, 77429, 1132, 44, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1910220, 77478, 273, 8, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1910496, 77488, 251, 8, 42, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1910784, 77501, 439, 14, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1911226, 77517, 379, 14, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1911642, 77536, 181, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1911826, 77543, 705, 32, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1912568, 77580, 238, 6, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1912809, 77588, 1363, 62, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1914209, 77655, 200, 6, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1914412, 77663, 164, 5, 28, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1914613, 77673, 341, 10, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1914957, 77685, 543, 21, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1915537, 77711, 181, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1915721, 77718, 543, 21, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1916301, 77744, 180, 5, 26, "Input",
 CellTags->"SUNSimplify"],
Cell[1916484, 77751, 571, 22, 26, "Output",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1917092, 77778, 267, 7, 26, "Input"],
Cell[1917362, 77787, 306, 11, 42, "Output"]
}, Open  ]],
Cell[1917683, 77801, 208, 6, 26, "Input",
 CellTags->"SUNSimplify"]
}, Open  ]],
Cell[1917906, 77810, 45, 1, 39, "ObjectName",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1917976, 77815, 55, 1, 36, "Subsubsection",
 CellTags->"SUNT"],
Cell[1918034, 77818, 237, 10, 24, "Text",
 CellTags->"SUNT"],
Cell[1918274, 77830, 1149, 46, 24, "Text",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1919460, 77881, 52, 1, 36, "Subsubsection",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1919537, 77886, 102, 3, 26, "Input",
 CellTags->"SUNT"],
Cell[1919642, 77891, 157, 6, 26, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1919814, 77900, 203, 7, 24, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1920042, 77911, 207, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1920252, 77919, 311, 13, 26, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1920600, 77937, 145, 4, 26, "Input",
 CellTags->"SUNT"],
Cell[1920748, 77943, 365, 16, 28, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1921150, 77964, 242, 7, 26, "Input",
 CellTags->"SUNT"],
Cell[1921395, 77973, 268, 11, 42, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1921700, 77989, 184, 5, 26, "Input",
 CellTags->"SUNT"],
Cell[1921887, 77996, 133, 4, 28, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1922057, 78005, 174, 5, 26, "Input",
 CellTags->"SUNT"],
Cell[1922234, 78012, 399, 16, 42, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1922670, 78033, 242, 7, 26, "Input",
 CellTags->"SUNT"],
Cell[1922915, 78042, 268, 11, 42, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1923198, 78056, 422, 15, 24, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1923645, 78075, 161, 5, 26, "Input",
 CellTags->"SUNT"],
Cell[1923809, 78082, 328, 14, 42, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[1924152, 78099, 197, 7, 27, "Text",
 CellTags->"SUNT"],
Cell[CellGroupData[{
Cell[1924374, 78110, 197, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1924574, 78118, 353, 15, 29, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1924964, 78138, 198, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1925165, 78146, 263, 8, 54, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1925465, 78159, 167, 6, 26, "Input",
 CellTags->"SUNT"],
Cell[1925635, 78167, 152, 4, 38, "Output",
 CellTags->"SUNT"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1925824, 78176, 195, 7, 26, "Input",
 CellTags->"SUNT"],
Cell[1926022, 78185, 117, 3, 38, "Output",
 CellTags->"SUNT"]
}, Open  ]]
}, Open  ]],
Cell[1926166, 78192, 53, 1, 39, "ObjectName",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1926244, 78197, 59, 1, 36, "Subsubsection",
 CellTags->"SUNTrace"],
Cell[1926306, 78200, 81, 1, 24, "Text",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1926412, 78205, 116, 3, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1926531, 78210, 172, 6, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[1926718, 78219, 441, 18, 24, "Text",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1927196, 78242, 56, 1, 36, "Subsubsection",
 CellTags->"SUNTrace"],
Cell[CellGroupData[{
Cell[1927277, 78247, 165, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1927445, 78254, 332, 14, 42, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1927814, 78273, 175, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1927992, 78280, 524, 22, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1928553, 78307, 240, 7, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1928796, 78316, 811, 35, 44, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1929644, 78356, 185, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1929832, 78363, 641, 27, 28, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1930510, 78395, 278, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1930791, 78405, 4270, 184, 43, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1935098, 78594, 177, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1935278, 78601, 641, 27, 28, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1935956, 78633, 288, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1936247, 78643, 11352, 436, 174, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1947636, 79084, 177, 5, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1947816, 79091, 758, 32, 28, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1948611, 79128, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1948884, 79138, 105, 3, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1949026, 79146, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1949299, 79156, 105, 3, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1949441, 79164, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1949714, 79174, 105, 3, 26, "Output",
 CellTags->"SUNTrace"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1949856, 79182, 270, 8, 26, "Input",
 CellTags->"SUNTrace"],
Cell[1950129, 79192, 576, 23, 29, "Output",
 CellTags->"SUNTrace"]
}, Open  ]]
}, Open  ]],
Cell[1950732, 79219, 61, 1, 39, "ObjectName",
 CellTags->"SymbolicSum2"],
Cell[CellGroupData[{
Cell[1950818, 79224, 63, 1, 36, "Subsubsection",
 CellTags->"SymbolicSum2"],
Cell[1950884, 79227, 284, 6, 41, "Text",
 CellTags->"SymbolicSum2"]
}, Open  ]],
Cell[1951183, 79236, 61, 1, 39, "ObjectName",
 CellTags->"SymbolicSum3"],
Cell[CellGroupData[{
Cell[1951269, 79241, 63, 1, 36, "Subsubsection",
 CellTags->"SymbolicSum3"],
Cell[1951335, 79244, 292, 6, 41, "Text",
 CellTags->"SymbolicSum3"]
}, Open  ]],
Cell[1951642, 79253, 57, 1, 39, "ObjectName",
 CellTags->"Symmetrize"],
Cell[CellGroupData[{
Cell[1951724, 79258, 61, 1, 36, "Subsubsection",
 CellTags->"Symmetrize"],
Cell[1951788, 79261, 145, 4, 24, "Text",
 CellTags->"Symmetrize"],
Cell[1951936, 79267, 281, 10, 24, "Text",
 CellTags->"Symmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1952254, 79282, 58, 1, 36, "Subsubsection",
 CellTags->"Symmetrize"],
Cell[CellGroupData[{
Cell[1952337, 79287, 240, 8, 26, "Input",
 CellTags->"Symmetrize"],
Cell[1952580, 79297, 317, 11, 42, "Output",
 CellTags->"Symmetrize"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1952934, 79313, 260, 8, 26, "Input",
 CellTags->"Symmetrize"],
Cell[1953197, 79323, 652, 20, 42, "Output",
 CellTags->"Symmetrize"]
}, Open  ]]
}, Open  ]],
Cell[1953876, 79347, 61, 1, 39, "ObjectName",
 CellTags->{"TBox", "Tbox"}],
Cell[CellGroupData[{
Cell[1953962, 79352, 65, 1, 36, "Subsubsection",
 CellTags->{"TBox", "Tbox"}],
Cell[1954030, 79355, 262, 5, 41, "Text",
 CellTags->{"TBox", "Tbox"}]
}, Open  ]],
Cell[1954307, 79363, 45, 1, 39, "ObjectName",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1954377, 79368, 55, 1, 36, "Subsubsection",
 CellTags->"Tdec"],
Cell[1954435, 79371, 256, 5, 41, "Text",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1954716, 79380, 108, 3, 26, "Input",
 CellTags->"Tdec"],
Cell[1954827, 79385, 412, 11, 26, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1955254, 79399, 633, 25, 24, "Text",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1955924, 79429, 52, 1, 36, "Subsubsection",
 CellTags->"Tdec"],
Cell[1955979, 79432, 608, 22, 34, "Text",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1956612, 79458, 202, 7, 26, "Input",
 CellTags->"Tdec"],
Cell[1956817, 79467, 556, 21, 43, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1957410, 79493, 199, 7, 26, "Input",
 CellTags->"Tdec"],
Cell[1957612, 79502, 354, 14, 43, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1957981, 79519, 776, 28, 27, "Text",
 CellTags->"Tdec"],
Cell[1958760, 79549, 529, 18, 26, "Input",
 CellTags->"Tdec"],
Cell[CellGroupData[{
Cell[1959314, 79571, 199, 7, 26, "Input",
 CellTags->"Tdec"],
Cell[1959516, 79580, 7304, 266, 93, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1966857, 79851, 380, 13, 26, "Input",
 CellTags->"Tdec"],
Cell[1967240, 79866, 4966, 181, 72, "Output",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1972221, 80050, 121, 4, 26, "Input",
 CellTags->"Tdec"]
}, Open  ]],
Cell[1972357, 80057, 65, 1, 39, "ObjectName",
 CellTags->"TensorFunction"],
Cell[CellGroupData[{
Cell[1972447, 80062, 65, 1, 36, "Subsubsection",
 CellTags->"TensorFunction"],
Cell[1972515, 80065, 360, 7, 41, "Text",
 CellTags->"TensorFunction"],
Cell[1972878, 80074, 275, 11, 24, "Text",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1973190, 80090, 62, 1, 36, "Subsubsection",
 CellTags->"TensorFunction"],
Cell[CellGroupData[{
Cell[1973277, 80095, 178, 4, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1973458, 80101, 212, 6, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1973707, 80112, 116, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1973826, 80117, 193, 4, 38, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1974056, 80126, 198, 6, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1974257, 80134, 376, 14, 28, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1974670, 80153, 116, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1974789, 80158, 358, 10, 38, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1975184, 80173, 228, 7, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1975415, 80182, 263, 8, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1975715, 80195, 228, 7, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1975946, 80204, 263, 8, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1976246, 80217, 116, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1976365, 80222, 243, 7, 38, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1976645, 80234, 118, 3, 26, "Input",
 CellTags->"TensorFunction"],
Cell[1976766, 80239, 129, 4, 26, "Output",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[1976910, 80246, 154, 4, 26, "Input",
 CellTags->"TensorFunction"]
}, Open  ]],
Cell[1977079, 80253, 41, 1, 39, "ObjectName",
 CellTags->"Tf"],
Cell[CellGroupData[{
Cell[1977145, 80258, 53, 1, 36, "Subsubsection",
 CellTags->"Tf"],
Cell[1977201, 80261, 214, 10, 27, "Text",
 CellTags->"Tf"],
Cell[1977418, 80273, 669, 24, 24, "Text",
 CellTags->"Tf"]
}, Open  ]],
Cell[1978102, 80300, 43, 1, 39, "ObjectName",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[1978170, 80305, 54, 1, 36, "Subsubsection",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[1978249, 80310, 72, 2, 26, "Input"],
Cell[1978324, 80314, 469, 10, 21, "Message"]
}, Open  ]],
Cell[1978808, 80327, 576, 10, 75, "Text",
 CellTags->"TFi"],
Cell[1979387, 80339, 237, 8, 24, "Text",
 CellTags->"TFi"],
Cell[1979627, 80349, 185, 4, 24, "Text",
 CellTags->"TFi"],
Cell[1979815, 80355, 426, 17, 24, "Text",
 CellTags->"TFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1980278, 80377, 51, 1, 36, "Subsubsection",
 CellTags->"TFi"],
Cell[CellGroupData[{
Cell[1980354, 80382, 506, 17, 26, "Input",
 CellTags->"TFi"],
Cell[1980863, 80401, 778, 24, 93, "Output",
 CellTags->"TFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1981678, 80430, 295, 10, 26, "Input",
 CellTags->"TFi"],
Cell[1981976, 80442, 333, 11, 29, "Output",
 CellTags->"TFi"]
}, Open  ]]
}, Open  ]],
Cell[1982336, 80457, 59, 1, 39, "ObjectName",
 CellTags->"ThreeVector"],
Cell[CellGroupData[{
Cell[1982420, 80462, 62, 1, 36, "Subsubsection",
 CellTags->"ThreeVector"],
Cell[1982485, 80465, 91, 1, 24, "Text",
 CellTags->"ThreeVector"],
Cell[1982579, 80468, 474, 17, 24, "Text",
 CellTags->"ThreeVector"]
}, Open  ]],
Cell[1983068, 80488, 43, 1, 39, "ObjectName",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[1983136, 80493, 54, 1, 36, "Subsubsection",
 CellTags->"TID"],
Cell[1983193, 80496, 80, 1, 24, "Text",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[1983298, 80501, 106, 3, 26, "Input",
 CellTags->"TID"],
Cell[1983407, 80506, 655, 15, 47, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[1984077, 80524, 460, 18, 24, "Text",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1984574, 80547, 51, 1, 36, "Subsubsection",
 CellTags->"TID"],
Cell[CellGroupData[{
Cell[1984650, 80552, 374, 12, 26, "Input",
 CellTags->"TID"],
Cell[1985027, 80566, 532, 21, 50, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1985596, 80592, 181, 6, 26, "Input",
 CellTags->"TID"],
Cell[1985780, 80600, 4307, 136, 64, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1990124, 80741, 271, 9, 26, "Input",
 CellTags->"TID"],
Cell[1990398, 80752, 389, 16, 50, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1990824, 80773, 123, 4, 26, "Input",
 CellTags->"TID"],
Cell[1990950, 80779, 627, 27, 49, "Output",
 CellTags->"TID"]
}, Open  ]],
Cell[CellGroupData[{
Cell[1991614, 80811, 195, 6, 26, "Input",
 CellTags->"TID"],
Cell[1991812, 80819, 627, 27, 49, "Output",
 CellTags->"TID"]
}, Open  ]]
}, Open  ]],
Cell[1992466, 80850, 45, 1, 39, "ObjectName",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[1992536, 80855, 55, 1, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[1992594, 80858, 87, 1, 24, "Text",
 CellTags->"TIDL"],
Cell[1992684, 80861, 231, 10, 24, "Text",
 CellTags->"TIDL"]
}, Open  ]],
Cell[1992930, 80874, 52, 1, 36, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[1993007, 80879, 154, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[1993164, 80886, 414, 13, 27, "Text",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[1993603, 80903, 202, 7, 26, "Input",
 CellTags->"TIDL"],
Cell[1993808, 80912, 465, 20, 47, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1994322, 80938, 159, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[1994506, 80947, 312, 11, 26, "Input",
 CellTags->"TIDL"],
Cell[1994821, 80960, 1876, 74, 51, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[1996746, 81040, 154, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[1996925, 81049, 277, 10, 26, "Input",
 CellTags->"TIDL"],
Cell[1997205, 81061, 2812, 108, 49, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2000066, 81175, 159, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2000250, 81184, 387, 14, 26, "Input",
 CellTags->"TIDL"],
Cell[2000640, 81200, 24393, 841, 325, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2025082, 82047, 165, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[2025250, 82054, 434, 15, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2025721, 82074, 173, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[2025897, 82081, 506, 17, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2026440, 82103, 154, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2026619, 82112, 312, 11, 26, "Input",
 CellTags->"TIDL"],
Cell[2026934, 82125, 14684, 522, 143, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2041667, 82653, 159, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[2041829, 82660, 374, 13, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2042240, 82678, 165, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[2042408, 82685, 444, 15, 26, "Input",
 CellTags->"TIDL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2042889, 82705, 173, 5, 38, "Subsubsection",
 CellTags->"TIDL"],
Cell[2043065, 82712, 529, 18, 26, "Input",
 CellTags->"TIDL"],
Cell[CellGroupData[{
Cell[2043619, 82734, 240, 7, 26, "Input",
 CellTags->"TIDL"],
Cell[2043862, 82743, 161, 5, 26, "Output",
 CellTags->"TIDL"]
}, Open  ]]
}, Open  ]],
Cell[2044050, 82752, 65, 1, 39, "ObjectName",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2044140, 82757, 65, 1, 36, "Subsubsection",
 CellTags->"TimedIntegrate"],
Cell[2044208, 82760, 225, 5, 41, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2044458, 82769, 128, 3, 26, "Input",
 CellTags->"TimedIntegrate"],
Cell[2044589, 82774, 424, 13, 26, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2045062, 82793, 62, 1, 36, "Subsubsection",
 CellTags->"TimedIntegrate"],
Cell[2045127, 82796, 73, 1, 24, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2045225, 82801, 305, 9, 26, "Input",
 CellTags->"TimedIntegrate"],
Cell[2045533, 82812, 129, 4, 26, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]],
Cell[2045677, 82819, 59, 1, 24, "Text",
 CellTags->"TimedIntegrate"],
Cell[CellGroupData[{
Cell[2045761, 82824, 389, 11, 26, "Input",
 CellTags->"TimedIntegrate"],
Cell[2046153, 82837, 449, 14, 29, "Output",
 CellTags->"TimedIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[2046629, 82855, 61, 1, 39, "ObjectName",
 CellTags->"TLI"],
Cell[CellGroupData[{
Cell[2046715, 82860, 54, 1, 36, "Subsubsection",
 CellTags->"TLI"],
Cell[2046772, 82863, 616, 12, 116, "Text",
 CellTags->"TLI"],
Cell[CellGroupData[{
Cell[2047413, 82879, 106, 3, 26, "Input",
 CellTags->"TLI"],
Cell[2047522, 82884, 229, 8, 28, "Output",
 CellTags->"TLI"]
}, Open  ]],
Cell[2047766, 82895, 248, 11, 24, "Text",
 CellTags->"TLI"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2048051, 82911, 51, 1, 36, "Subsubsection",
 CellTags->"TLI"],
Cell[2048105, 82914, 70, 2, 26, "Input",
 CellTags->"TLI"]
}, Open  ]],
Cell[2048190, 82919, 65, 1, 39, "ObjectName",
 CellTags->"TLIFP"],
Cell[CellGroupData[{
Cell[2048280, 82924, 56, 1, 36, "Subsubsection",
 CellTags->"TLIFP"],
Cell[2048339, 82927, 93, 1, 24, "Text",
 CellTags->"TLIFP"],
Cell[CellGroupData[{
Cell[2048457, 82932, 110, 3, 26, "Input",
 CellTags->"TLIFP"],
Cell[2048570, 82937, 425, 12, 28, "Output",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[2049010, 82952, 238, 11, 24, "Text",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2049285, 82968, 53, 1, 36, "Subsubsection",
 CellTags->"TLIFP"],
Cell[2049341, 82971, 72, 2, 26, "Input",
 CellTags->"TLIFP"]
}, Open  ]],
Cell[2049428, 82976, 67, 1, 39, "ObjectName",
 CellTags->"TLIHYP"],
Cell[CellGroupData[{
Cell[2049520, 82981, 57, 1, 36, "Subsubsection",
 CellTags->"TLIHYP"],
Cell[2049580, 82984, 136, 4, 24, "Text",
 CellTags->"TLIHYP"],
Cell[CellGroupData[{
Cell[2049741, 82992, 112, 3, 26, "Input",
 CellTags->"TLIHYP"],
Cell[2049856, 82997, 241, 8, 28, "Output",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[2050112, 83008, 239, 11, 24, "Text",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2050388, 83024, 54, 1, 36, "Subsubsection",
 CellTags->"TLIHYP"],
Cell[2050445, 83027, 73, 2, 26, "Input",
 CellTags->"TLIHYP"]
}, Open  ]],
Cell[2050533, 83032, 45, 1, 39, "ObjectName",
 CellTags->"TLI2"],
Cell[CellGroupData[{
Cell[2050603, 83037, 55, 1, 36, "Subsubsection",
 CellTags->"TLI2"],
Cell[2050661, 83040, 83, 1, 24, "Text",
 CellTags->"TLI2"],
Cell[2050747, 83043, 237, 11, 24, "Text",
 CellTags->"TLI2"]
}, Open  ]],
Cell[2050999, 83057, 67, 1, 39, "ObjectName",
 CellTags->"TLI2FC"],
Cell[CellGroupData[{
Cell[2051091, 83062, 57, 1, 36, "Subsubsection",
 CellTags->"TLI2FC"],
Cell[2051151, 83065, 109, 3, 24, "Text",
 CellTags->"TLI2FC"],
Cell[CellGroupData[{
Cell[2051285, 83072, 112, 3, 26, "Input",
 CellTags->"TLI2FC"],
Cell[2051400, 83077, 355, 11, 28, "Output",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[2051770, 83091, 419, 17, 24, "Text",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2052226, 83113, 54, 1, 36, "Subsubsection",
 CellTags->"TLI2FC"],
Cell[2052283, 83116, 73, 2, 26, "Input",
 CellTags->"TLI2FC"]
}, Open  ]],
Cell[2052371, 83121, 65, 1, 39, "ObjectName",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2052461, 83126, 65, 1, 36, "Subsubsection",
 CellTags->"ToDistribution"],
Cell[2052529, 83129, 424, 7, 58, "Text",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2052978, 83140, 128, 3, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2053109, 83145, 189, 6, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[2053313, 83154, 299, 11, 24, "Text",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2053649, 83170, 62, 1, 36, "Subsubsection",
 CellTags->"ToDistribution"],
Cell[CellGroupData[{
Cell[2053736, 83175, 217, 6, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2053956, 83183, 197, 6, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2054190, 83194, 277, 9, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2054470, 83205, 1338, 48, 49, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2055845, 83258, 206, 5, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2056054, 83265, 189, 6, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2056280, 83276, 277, 9, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2056560, 83287, 1380, 50, 50, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2057977, 83342, 461, 14, 29, "Input",
 CellTags->"ToDistribution"],
Cell[2058441, 83358, 371, 13, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2058849, 83376, 118, 3, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2058970, 83381, 334, 9, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2059341, 83395, 409, 13, 26, "Input",
 CellTags->"ToDistribution"],
Cell[2059753, 83410, 371, 13, 26, "Output",
 CellTags->"ToDistribution"]
}, Open  ]]
}, Open  ]],
Cell[2060151, 83427, 69, 1, 39, "ObjectName",
 CellTags->"ToHypergeometric"],
Cell[CellGroupData[{
Cell[2060245, 83432, 67, 1, 36, "Subsubsection",
 CellTags->"ToHypergeometric"],
Cell[2060315, 83435, 1686, 67, 42, "Text",
 CellTags->"ToHypergeometric"],
Cell[2062004, 83504, 749, 25, 24, "Text",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2062790, 83534, 64, 1, 36, "Subsubsection",
 CellTags->"ToHypergeometric"],
Cell[CellGroupData[{
Cell[2062879, 83539, 395, 13, 29, "Input",
 CellTags->"ToHypergeometric"],
Cell[2063277, 83554, 489, 15, 46, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2063803, 83574, 488, 16, 29, "Input",
 CellTags->"ToHypergeometric"],
Cell[2064294, 83592, 356, 13, 46, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2064687, 83610, 395, 13, 29, "Input",
 CellTags->"ToHypergeometric"],
Cell[2065085, 83625, 552, 17, 50, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2065674, 83647, 488, 16, 29, "Input",
 CellTags->"ToHypergeometric"],
Cell[2066165, 83665, 431, 15, 50, "Output",
 CellTags->"ToHypergeometric"]
}, Open  ]]
}, Open  ]],
Cell[2066623, 83684, 51, 1, 39, "ObjectName",
 CellTags->"ToLarin"],
Cell[CellGroupData[{
Cell[2066699, 83689, 58, 1, 36, "Subsubsection",
 CellTags->"ToLarin"],
Cell[2066760, 83692, 605, 19, 24, "Text",
 CellTags->"ToLarin"],
Cell[2067368, 83713, 240, 11, 24, "Text",
 CellTags->"ToLarin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2067645, 83729, 55, 1, 36, "Subsubsection",
 CellTags->"ToLarin"],
Cell[CellGroupData[{
Cell[2067725, 83734, 229, 6, 26, "Input",
 CellTags->"ToLarin"],
Cell[2067957, 83742, 389, 15, 26, "Output",
 CellTags->"ToLarin"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2068383, 83762, 108, 3, 26, "Input",
 CellTags->"ToLarin"],
Cell[2068494, 83767, 1097, 45, 42, "Output",
 CellTags->"ToLarin"]
}, Open  ]]
}, Open  ]],
Cell[2069618, 83816, 47, 1, 39, "ObjectName",
 CellTags->"ToTFi"],
Cell[CellGroupData[{
Cell[2069690, 83821, 56, 1, 36, "Subsubsection",
 CellTags->"ToTFi"],
Cell[2069749, 83824, 272, 5, 41, "Text",
 CellTags->"ToTFi"],
Cell[2070024, 83831, 420, 17, 24, "Text",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2070481, 83853, 53, 1, 36, "Subsubsection",
 CellTags->"ToTFi"],
Cell[CellGroupData[{
Cell[2070559, 83858, 333, 11, 26, "Input",
 CellTags->"ToTFi"],
Cell[2070895, 83871, 1870, 49, 48, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2072802, 83925, 149, 4, 26, "Input",
 CellTags->"ToTFi"],
Cell[2072954, 83931, 2126, 54, 48, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2075117, 83990, 107, 3, 26, "Input",
 CellTags->"ToTFi"],
Cell[2075227, 83995, 491, 16, 38, "Output",
 CellTags->"ToTFi"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2075755, 84016, 468, 15, 26, "Input",
 CellTags->"ToTFi"],
Cell[2076226, 84033, 916, 33, 48, "Output",
 CellTags->"ToTFi"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2077191, 84072, 41, 1, 39, "ObjectName",
 CellTags->"TR"],
Cell[2077235, 84075, 100, 3, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2077372, 84083, 53, 1, 36, "Subsubsection",
 CellTags->"TR"],
Cell[2077428, 84086, 420, 12, 41, "Text",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2077873, 84102, 104, 3, 26, "Input",
 CellTags->"TR"],
Cell[2077980, 84107, 761, 18, 47, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[2078756, 84128, 877, 31, 24, "Text",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2079670, 84164, 50, 1, 36, "Subsubsection",
 CellTags->"TR"],
Cell[CellGroupData[{
Cell[2079745, 84169, 129, 4, 26, "Input",
 CellTags->"TR"],
Cell[2079877, 84175, 194, 6, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2080108, 84186, 98, 3, 26, "Input",
 CellTags->"TR"],
Cell[2080209, 84191, 175, 6, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2080421, 84202, 375, 12, 26, "Input",
 CellTags->"TR"],
Cell[2080799, 84216, 1257, 50, 29, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2082093, 84271, 205, 6, 26, "Input",
 CellTags->"TR"],
Cell[2082301, 84279, 559, 23, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2082897, 84307, 240, 7, 26, "Input",
 CellTags->"TR"],
Cell[2083140, 84316, 8363, 321, 52, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2091540, 84642, 421, 13, 26, "Input",
 CellTags->"TR"],
Cell[2091964, 84657, 10947, 412, 67, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2102948, 85074, 171, 5, 26, "Input",
 CellTags->"TR"],
Cell[2103122, 85081, 991, 39, 28, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2104150, 85125, 444, 14, 26, "Input",
 CellTags->"TR"],
Cell[2104597, 85141, 342, 13, 29, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2104976, 85159, 224, 7, 26, "Input",
 CellTags->"TR"],
Cell[2105203, 85168, 180, 6, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2105420, 85179, 104, 3, 26, "Input",
 CellTags->"TR"],
Cell[2105527, 85184, 173, 5, 38, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2105737, 85194, 347, 10, 26, "Input",
 CellTags->"TR"],
Cell[2106087, 85206, 190, 6, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2106314, 85217, 104, 3, 26, "Input",
 CellTags->"TR"],
Cell[2106421, 85222, 183, 5, 38, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2106641, 85232, 339, 11, 26, "Input",
 CellTags->"TR"],
Cell[2106983, 85245, 428, 16, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2107448, 85266, 214, 6, 26, "Input",
 CellTags->"TR"],
Cell[2107665, 85274, 208, 7, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2107910, 85286, 213, 6, 26, "Input",
 CellTags->"TR"],
Cell[2108126, 85294, 208, 7, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2108371, 85306, 214, 6, 26, "Input",
 CellTags->"TR"],
Cell[2108588, 85314, 99, 3, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2108724, 85322, 213, 6, 26, "Input",
 CellTags->"TR"],
Cell[2108940, 85330, 99, 3, 26, "Output",
 CellTags->"TR"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2109076, 85338, 139, 4, 26, "Input"],
Cell[2109218, 85344, 150, 5, 26, "Output"]
}, Open  ]],
Cell[2109383, 85352, 49, 0, 24, "Text"],
Cell[CellGroupData[{
Cell[2109457, 85356, 71, 2, 26, "Input"],
Cell[2109531, 85360, 644, 13, 156, "Print",
 CellTags->"Info3535453335-5096475"]
}, Open  ]],
Cell[2110190, 85376, 1240, 36, 84, "Input"],
Cell[CellGroupData[{
Cell[2111455, 85416, 80, 2, 26, "Input"],
Cell[2111538, 85420, 215, 5, 22, "Print"],
Cell[2111756, 85427, 300, 11, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2112093, 85443, 80, 2, 26, "Input"],
Cell[2112176, 85447, 215, 5, 22, "Print"],
Cell[2112394, 85454, 618, 20, 29, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2113049, 85479, 81, 2, 26, "Input"],
Cell[2113133, 85483, 213, 5, 22, "Print"],
Cell[2113349, 85490, 787, 25, 50, "Output"]
}, Open  ]],
Cell[2114151, 85518, 143, 5, 26, "Input",
 CellTags->"TR"]
}, Open  ]],
Cell[2114309, 85526, 65, 1, 39, "ObjectName",
 CellTags->"TraceDimension"],
Cell[CellGroupData[{
Cell[2114399, 85531, 65, 1, 36, "Subsubsection",
 CellTags->"TraceDimension"],
Cell[2114467, 85534, 142, 4, 24, "Text",
 CellTags->"TraceDimension"],
Cell[2114612, 85540, 287, 11, 24, "Text",
 CellTags->"TraceDimension"]
}, Open  ]],
Cell[2114914, 85554, 57, 1, 39, "ObjectName",
 CellTags->"TraceOfOne"],
Cell[CellGroupData[{
Cell[2114996, 85559, 61, 1, 36, "Subsubsection",
 CellTags->"TraceOfOne"],
Cell[2115060, 85562, 149, 4, 24, "Text",
 CellTags->"TraceOfOne"],
Cell[2115212, 85568, 357, 15, 24, "Text",
 CellTags->"TraceOfOne"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2115606, 85588, 58, 1, 36, "Subsubsection",
 CellTags->"TraceOfOne"],
Cell[CellGroupData[{
Cell[2115689, 85593, 273, 7, 26, "Input",
 CellTags->"TraceOfOne"],
Cell[2115965, 85602, 109, 3, 26, "Output",
 CellTags->"TraceOfOne"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2116111, 85610, 272, 7, 26, "Input",
 CellTags->"TraceOfOne"],
Cell[2116386, 85619, 109, 3, 26, "Output",
 CellTags->"TraceOfOne"]
}, Open  ]]
}, Open  ]],
Cell[2116522, 85626, 47, 1, 39, "ObjectName",
 CellTags->"Trick"],
Cell[CellGroupData[{
Cell[2116594, 85631, 56, 1, 36, "Subsubsection",
 CellTags->"Trick"],
Cell[2116653, 85634, 174, 4, 24, "Text",
 CellTags->"Trick"],
Cell[2116830, 85640, 1038, 39, 24, "Text",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2117905, 85684, 62, 1, 33, "Subsubsection",
 CellTags->"Trick"],
Cell[2117970, 85687, 376, 15, 27, "Text",
 CellTags->"Trick"],
Cell[CellGroupData[{
Cell[2118371, 85706, 350, 11, 26, "Input",
 CellTags->"Trick"],
Cell[2118724, 85719, 264, 10, 26, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2119025, 85734, 330, 11, 26, "Input",
 CellTags->"Trick"],
Cell[2119358, 85747, 426, 16, 28, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2119821, 85768, 104, 3, 26, "Input",
 CellTags->"Trick"],
Cell[2119928, 85773, 527, 22, 28, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2120492, 85800, 227, 6, 26, "Input",
 CellTags->"Trick"],
Cell[2120722, 85808, 391, 15, 28, "Output",
 CellTags->"Trick"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2121150, 85828, 133, 4, 26, "Input",
 CellTags->"Trick"],
Cell[2121286, 85834, 213, 6, 38, "Output",
 CellTags->"Trick"]
}, Open  ]]
}, Open  ]],
Cell[2121526, 85844, 65, 1, 39, "ObjectName",
 CellTags->"TrickIntegrate"],
Cell[CellGroupData[{
Cell[2121616, 85849, 65, 1, 36, "Subsubsection",
 CellTags->"TrickIntegrate"],
Cell[2121684, 85852, 483, 8, 58, "Text",
 CellTags->"TrickIntegrate"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2122204, 85865, 62, 1, 36, "Subsubsection",
 CellTags->"TrickIntegrate"],
Cell[CellGroupData[{
Cell[2122291, 85870, 364, 12, 26, "Input",
 CellTags->"TrickIntegrate"],
Cell[2122658, 85884, 517, 18, 29, "Output",
 CellTags->"TrickIntegrate"]
}, Open  ]]
}, Open  ]],
Cell[2123202, 85906, 67, 1, 39, "ObjectName",
 CellTags->"TrickMandelstam"],
Cell[CellGroupData[{
Cell[2123294, 85911, 66, 1, 36, "Subsubsection",
 CellTags->"TrickMandelstam"],
Cell[2123363, 85914, 1340, 55, 45, "Text",
 CellTags->"TrickMandelstam"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2124740, 85974, 63, 1, 36, "Subsubsection",
 CellTags->"TrickMandelstam"],
Cell[CellGroupData[{
Cell[2124828, 85979, 528, 18, 29, "Input",
 CellTags->"TrickMandelstam"],
Cell[2125359, 85999, 250, 9, 29, "Output",
 CellTags->"TrickMandelstam"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2125646, 86013, 568, 17, 26, "Input",
 CellTags->"TrickMandelstam"],
Cell[2126217, 86032, 235, 8, 26, "Output",
 CellTags->"TrickMandelstam"]
}, Open  ]]
}, Open  ]],
Cell[2126479, 86044, 43, 1, 39, "ObjectName",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2126547, 86049, 54, 1, 36, "Subsubsection",
 CellTags->"Tr2"],
Cell[2126604, 86052, 347, 6, 58, "Text",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2126976, 86062, 106, 3, 26, "Input",
 CellTags->"Tr2"],
Cell[2127085, 86067, 168, 6, 26, "Output",
 CellTags->"Tr2"]
}, Open  ]],
Cell[2127268, 86076, 998, 38, 24, "Text",
 CellTags->"Tr2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2128303, 86119, 51, 1, 36, "Subsubsection",
 CellTags->"Tr2"],
Cell[CellGroupData[{
Cell[2128379, 86124, 164, 4, 26, "Input"],
Cell[2128546, 86130, 548, 20, 31, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2129131, 86155, 187, 6, 26, "Input",
 CellTags->"Tr2"],
Cell[2129321, 86163, 156, 5, 29, "Output",
 CellTags->"Tr2"]
}, Open  ]]
}, Open  ]],
Cell[2129504, 86172, 116, 2, 39, "ObjectName",
 CellTags->"Twist2AlienOperator"],
Cell[CellGroupData[{
Cell[2129645, 86178, 70, 1, 36, "Subsubsection",
 CellTags->"Twist2AlienOperator"],
Cell[2129718, 86181, 194, 4, 24, "Text",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2129949, 86190, 67, 1, 36, "Subsubsection",
 CellTags->"Twist2AlienOperator"],
Cell[CellGroupData[{
Cell[2130041, 86195, 160, 4, 26, "Input",
 CellTags->"Twist2AlienOperator"],
Cell[2130204, 86201, 887, 30, 49, "Output",
 CellTags->"Twist2AlienOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2131128, 86236, 246, 7, 26, "Input",
 CellTags->"Twist2AlienOperator"],
Cell[2131377, 86245, 3113, 103, 53, "Output",
 CellTags->"Twist2AlienOperator"]
}, Open  ]]
}, Open  ]],
Cell[2134517, 86352, 120, 2, 39, "ObjectName",
 CellTags->"Twist2CounterOperator"],
Cell[CellGroupData[{
Cell[2134662, 86358, 72, 1, 28, "Subsubsection",
 CellTags->"Twist2CounterOperator"],
Cell[2134737, 86361, 239, 5, 37, "Text",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2135013, 86371, 69, 1, 36, "Subsubsection",
 CellTags->"Twist2CounterOperator"],
Cell[CellGroupData[{
Cell[2135107, 86376, 204, 5, 26, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2135314, 86383, 4002, 136, 85, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2139353, 86524, 159, 4, 26, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2139515, 86530, 1020, 35, 49, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2140572, 86570, 242, 6, 26, "Input",
 CellTags->"Twist2CounterOperator"],
Cell[2140817, 86578, 2610, 89, 53, "Output",
 CellTags->"Twist2CounterOperator"]
}, Open  ]]
}, Open  ]],
Cell[2143454, 86671, 75, 1, 39, "ObjectName",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2143554, 86676, 70, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[2143627, 86679, 964, 15, 126, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2144594, 86696, 316, 11, 24, "Text",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2144947, 86712, 99, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2145071, 86717, 352, 8, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2145426, 86727, 444, 12, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2145885, 86742, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2145972, 86747, 271, 8, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2146246, 86757, 894, 35, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2147177, 86797, 321, 9, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2147501, 86808, 1120, 45, 28, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2148658, 86858, 221, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2148882, 86866, 336, 12, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2149255, 86883, 271, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2149529, 86892, 705, 28, 28, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2150249, 86923, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2150336, 86928, 355, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2150694, 86940, 1193, 48, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2151924, 86993, 325, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2152252, 87005, 492, 18, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2152781, 87028, 375, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2153159, 87041, 11308, 411, 133, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2164482, 87455, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2164569, 87460, 431, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2165003, 87474, 4976, 185, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2170016, 87664, 391, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2170410, 87678, 606, 22, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2171053, 87705, 441, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2171497, 87720, 24305, 869, 387, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2195851, 88595, 100, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2195976, 88600, 268, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2196247, 88608, 444, 12, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2196706, 88623, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2196793, 88628, 271, 8, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2197067, 88638, 904, 36, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2198008, 88679, 321, 9, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2198332, 88690, 2656, 102, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2201025, 88797, 221, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2201249, 88805, 336, 12, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2201622, 88822, 271, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2201896, 88831, 2231, 84, 29, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2204142, 88918, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2204229, 88923, 355, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2204587, 88935, 1267, 50, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2205891, 88990, 325, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2206219, 89002, 492, 18, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2206748, 89025, 375, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2207126, 89038, 16342, 587, 207, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2223483, 89628, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2223570, 89633, 431, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2224004, 89647, 4986, 186, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2229027, 89838, 391, 12, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2229421, 89852, 606, 22, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2230064, 89879, 441, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2230508, 89894, 31861, 1141, 463, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2262418, 91041, 76, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[2262497, 91044, 97, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2262597, 91047, 295, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2262895, 91056, 380, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2263312, 91072, 102, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2263439, 91077, 284, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2263726, 91085, 443, 12, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2264184, 91100, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2264271, 91105, 320, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2264594, 91117, 3609, 129, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2268218, 91249, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2268305, 91254, 355, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2268663, 91266, 54187, 1690, 848, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2322865, 92959, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2322952, 92964, 482, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2323437, 92979, 4976, 185, 30, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2328462, 93170, 104, 1, 36, "Subsubsection",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2328591, 93175, 284, 6, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2328878, 93183, 443, 12, 26, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2329336, 93198, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2329423, 93203, 320, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2329746, 93215, 3074, 116, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2332835, 93334, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2332900, 93337, 380, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2333305, 93352, 325, 10, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2333633, 93364, 3075, 116, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2336745, 93485, 375, 11, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2337123, 93498, 3075, 116, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2340213, 93617, 62, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2340300, 93622, 482, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2340785, 93637, 4986, 186, 42, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2345786, 93826, 78, 1, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2345889, 93831, 520, 16, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2346412, 93849, 10907, 376, 150, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]],
Cell[2357334, 94228, 109, 3, 24, "Text",
 CellTags->"Twist2GluonOperator"],
Cell[2357446, 94233, 295, 7, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[CellGroupData[{
Cell[2357766, 94244, 428, 13, 26, "Input",
 CellTags->"Twist2GluonOperator"],
Cell[2358197, 94259, 12432, 394, 173, "Output",
 CellTags->"Twist2GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[2370656, 94657, 75, 1, 39, "ObjectName",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2370756, 94662, 70, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[2370829, 94665, 980, 15, 126, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[2371812, 94682, 316, 11, 24, "Text",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2372143, 94696, 67, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2372235, 94701, 98, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2372358, 94706, 268, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2372629, 94714, 443, 12, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2373087, 94729, 75, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2373187, 94734, 156, 4, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2373346, 94740, 600, 21, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2373961, 94764, 81, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2374067, 94769, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2374375, 94780, 1581, 57, 54, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2375971, 94840, 87, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2376083, 94845, 380, 11, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2376466, 94858, 4917, 163, 114, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2381432, 95027, 100, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2381557, 95032, 273, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2381833, 95040, 443, 12, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2382291, 95055, 75, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2382391, 95060, 156, 4, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2382550, 95066, 481, 17, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2383046, 95086, 81, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2383152, 95091, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2383460, 95102, 1492, 54, 54, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2384967, 95159, 87, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2385079, 95164, 380, 11, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2385462, 95177, 4776, 159, 78, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2390253, 95339, 93, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2390371, 95344, 197, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2390571, 95352, 263, 8, 38, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2390871, 95365, 376, 10, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2391250, 95377, 1971, 72, 49, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2393258, 95454, 449, 12, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2393710, 95468, 9405, 298, 96, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2403164, 95772, 102, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2403291, 95777, 269, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2403563, 95785, 444, 12, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2404022, 95800, 178, 4, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[2404203, 95806, 275, 5, 41, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2404503, 95815, 205, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2404711, 95823, 800, 28, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2405526, 95854, 100, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2405651, 95859, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2405959, 95870, 2305, 78, 54, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2408279, 95951, 87, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2408391, 95956, 156, 4, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2408550, 95962, 1102, 33, 62, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2409701, 96001, 104, 1, 36, "Subsubsection",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2409830, 96006, 269, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2410102, 96014, 444, 12, 26, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2410561, 96029, 75, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2410661, 96034, 205, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2410869, 96042, 800, 28, 28, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2411684, 96073, 81, 1, 24, "Text",
 CellTags->"Twist2QuarkOperator"],
Cell[CellGroupData[{
Cell[2411790, 96078, 305, 9, 26, "Input",
 CellTags->"Twist2QuarkOperator"],
Cell[2412098, 96089, 2305, 78, 54, "Output",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2414418, 96170, 210, 6, 26, "Input",
 CellTags->"Twist2QuarkOperator"]
}, Open  ]],
Cell[2414643, 96179, 75, 1, 39, "ObjectName",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2414743, 96184, 70, 1, 36, "Subsubsection",
 CellTags->"Twist3QuarkOperator"],
Cell[2414816, 96187, 768, 12, 92, "Text",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2415609, 96203, 134, 3, 26, "Input",
 CellTags->"Twist3QuarkOperator"],
Cell[2415746, 96208, 328, 10, 26, "Output",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],
Cell[2416089, 96221, 552, 18, 24, "Text",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2416678, 96244, 67, 1, 36, "Subsubsection",
 CellTags->"Twist3QuarkOperator"],
Cell[CellGroupData[{
Cell[2416770, 96249, 132, 3, 26, "Input",
 CellTags->"Twist3QuarkOperator"],
Cell[2416905, 96254, 654, 23, 28, "Output",
 CellTags->"Twist3QuarkOperator"]
}, Open  ]]
}, Open  ]],
Cell[2417586, 96281, 75, 1, 39, "ObjectName",
 CellTags->"Twist4GluonOperator"],
Cell[CellGroupData[{
Cell[2417686, 96286, 70, 1, 36, "Subsubsection",
 CellTags->"Twist4GluonOperator"],
Cell[2417759, 96289, 150, 4, 24, "Text",
 CellTags->"Twist4GluonOperator"],
Cell[2417912, 96295, 788, 25, 24, "Text",
 CellTags->"Twist4GluonOperator"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2418737, 96325, 67, 1, 36, "Subsubsection",
 CellTags->"Twist4GluonOperator"],
Cell[CellGroupData[{
Cell[2418829, 96330, 529, 15, 26, "Input",
 CellTags->"Twist4GluonOperator"],
Cell[2419361, 96347, 119831, 4588, 1110, "Output",
 CellTags->"Twist4GluonOperator"]
}, Open  ]]
}, Open  ]],
Cell[2539219, 100939, 104, 1, 39, "ObjectName",
 CellTags->"TwoLoopSimplify"],
Cell[CellGroupData[{
Cell[2539348, 100944, 66, 1, 36, "Subsubsection",
 CellTags->"TwoLoopSimplify"],
Cell[2539417, 100947, 335, 6, 58, "Text",
 CellTags->"TwoLoopSimplify"],
Cell[2539755, 100955, 296, 11, 24, "Text",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2540088, 100971, 63, 1, 36, "Subsubsection",
 CellTags->"TwoLoopSimplify"],
Cell[2540154, 100974, 82, 2, 26, "Input",
 CellTags->"TwoLoopSimplify"]
}, Open  ]],
Cell[2540251, 100979, 57, 1, 39, "ObjectName",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2540333, 100984, 61, 1, 36, "Subsubsection",
 CellTags->"Uncontract"],
Cell[2540397, 100987, 256, 5, 41, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2540678, 100996, 120, 3, 26, "Input",
 CellTags->"Uncontract"],
Cell[2540801, 101001, 372, 11, 26, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2541188, 101015, 263, 11, 24, "Text",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2541488, 101031, 58, 1, 36, "Subsubsection",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2541571, 101036, 234, 7, 26, "Input",
 CellTags->"Uncontract"],
Cell[2541808, 101045, 397, 17, 26, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2542242, 101067, 143, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2542388, 101073, 781, 31, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2543206, 101109, 138, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2543347, 101115, 224, 8, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2543608, 101128, 143, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2543754, 101134, 224, 8, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2544015, 101147, 153, 4, 26, "Input",
 CellTags->"Uncontract"],
Cell[2544171, 101153, 1138, 44, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2545324, 101200, 89, 1, 24, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2545438, 101205, 208, 6, 26, "Input",
 CellTags->"Uncontract"],
Cell[2545649, 101213, 209, 8, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2545873, 101224, 152, 4, 24, "Text",
 CellTags->"Uncontract"],
Cell[CellGroupData[{
Cell[2546050, 101232, 248, 7, 26, "Input",
 CellTags->"Uncontract"],
Cell[2546301, 101241, 570, 22, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2546908, 101268, 274, 8, 26, "Input",
 CellTags->"Uncontract"],
Cell[2547185, 101278, 1030, 40, 28, "Output",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2548230, 101321, 134, 4, 26, "Input",
 CellTags->"Uncontract"]
}, Open  ]],
Cell[2548379, 101328, 83, 1, 39, "ObjectName",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2548487, 101333, 74, 1, 36, "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],
Cell[2548564, 101336, 198, 4, 24, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[2548765, 101342, 520, 18, 24, "Text",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2549322, 101365, 71, 1, 36, "Subsubsection",
 CellTags->"UnDeclareNonCommutative"],
Cell[2549396, 101368, 138, 3, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2549537, 101373, 153, 4, 24, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2549715, 101381, 107, 2, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2549825, 101385, 123, 3, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2549963, 101391, 103, 1, 24, "Text",
 CellTags->"UnDeclareNonCommutative"],
Cell[2550069, 101394, 140, 3, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2550234, 101401, 161, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2550398, 101407, 123, 3, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2550536, 101413, 161, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2550722, 101421, 171, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2550896, 101427, 173, 5, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]],
Cell[2551084, 101435, 163, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[CellGroupData[{
Cell[2551272, 101443, 171, 4, 26, "Input",
 CellTags->"UnDeclareNonCommutative"],
Cell[2551446, 101449, 173, 5, 26, "Output",
 CellTags->"UnDeclareNonCommutative"]
}, Open  ]]
}, Open  ]],
Cell[2551646, 101458, 47, 1, 39, "ObjectName",
 CellTags->"Upper"],
Cell[CellGroupData[{
Cell[2551718, 101463, 56, 1, 28, "Subsubsection",
 CellTags->"Upper"],
Cell[2551777, 101466, 126, 4, 22, "Text",
 CellTags->"Upper"],
Cell[2551906, 101472, 645, 24, 22, "Text",
 CellTags->"Upper"]
}, Open  ]],
Cell[2552566, 101499, 86, 1, 39, "ObjectName",
 CellTags->"UVPart"],
Cell[CellGroupData[{
Cell[2552677, 101504, 57, 1, 28, "Subsubsection",
 CellTags->"UVPart"],
Cell[2552737, 101507, 128, 4, 22, "Text",
 CellTags->"UVPart"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2552902, 101516, 54, 1, 36, "Subsubsection",
 CellTags->"UVPart"],
Cell[2552959, 101519, 73, 2, 26, "Input",
 CellTags->"UVPart"]
}, Open  ]],
Cell[2553047, 101524, 51, 1, 39, "ObjectName",
 CellTags->"Vectors"],
Cell[CellGroupData[{
Cell[2553123, 101529, 58, 1, 28, "Subsubsection",
 CellTags->"Vectors"],
Cell[2553184, 101532, 203, 4, 22, "Text",
 CellTags->"Vectors"],
Cell[2553390, 101538, 275, 10, 22, "Text",
 CellTags->"Vectors"]
}, Open  ]],
Cell[2553680, 101551, 53, 1, 39, "ObjectName",
 CellTags->"WriteOut"],
Cell[CellGroupData[{
Cell[2553758, 101556, 59, 1, 28, "Subsubsection",
 CellTags->"WriteOut"],
Cell[2553820, 101559, 216, 5, 22, "Text",
 CellTags->"WriteOut"],
Cell[2554039, 101566, 252, 10, 22, "Text",
 CellTags->"WriteOut"]
}, Open  ]],
Cell[2554306, 101579, 61, 1, 39, "ObjectName",
 CellTags->"WriteOutPaVe"],
Cell[CellGroupData[{
Cell[2554392, 101584, 63, 1, 28, "Subsubsection",
 CellTags->"WriteOutPaVe"],
Cell[2554458, 101587, 260, 5, 37, "Text",
 CellTags->"WriteOutPaVe"],
Cell[2554721, 101594, 644, 24, 22, "Text",
 CellTags->"WriteOutPaVe"]
}, Open  ]],
Cell[2555380, 101621, 49, 1, 39, "ObjectName",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2555454, 101626, 57, 1, 28, "Subsubsection",
 CellTags->"Write2"],
Cell[2555514, 101629, 297, 6, 37, "Text",
 CellTags->"Write2"],
Cell[2555814, 101637, 153, 4, 24, "Text",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2555992, 101645, 136, 3, 29, "Input",
 CellTags->"Write2"],
Cell[2556131, 101650, 248, 9, 39, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2556416, 101664, 112, 3, 26, "Input",
 CellTags->"Write2"],
Cell[2556531, 101669, 637, 15, 47, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2557205, 101689, 115, 3, 26, "Input",
 CellTags->"Write2"],
Cell[2557323, 101694, 133, 4, 26, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2557471, 101701, 455, 18, 22, "Text",
 CellTags->"Write2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2557963, 101724, 54, 1, 36, "Subsubsection",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2558042, 101729, 429, 15, 35, "Input",
 CellTags->"Write2"],
Cell[2558474, 101746, 855, 29, 31, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2559344, 101778, 208, 7, 29, "Input",
 CellTags->"Write2"],
Cell[2559555, 101787, 78, 1, 24, "Text",
 CellTags->"Write2"],
Cell[2559636, 101790, 184, 6, 26, "Input",
 CellTags->"Write2"],
Cell[2559823, 101798, 73, 1, 24, "Text",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2559921, 101803, 363, 11, 29, "Input",
 CellTags->"Write2"],
Cell[2560287, 101816, 801, 22, 71, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2561103, 101841, 286, 8, 29, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2561414, 101853, 233, 7, 26, "Input",
 CellTags->"Write2"],
Cell[2561650, 101862, 710, 27, 26, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2562375, 101892, 185, 6, 26, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2562585, 101902, 363, 11, 29, "Input",
 CellTags->"Write2"],
Cell[2562951, 101915, 1061, 30, 195, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2564027, 101948, 286, 8, 29, "Input",
 CellTags->"Write2"],
Cell[2564316, 101958, 106, 3, 24, "Text",
 CellTags->"Write2"],
Cell[2564425, 101963, 247, 8, 26, "Input",
 CellTags->"Write2"],
Cell[CellGroupData[{
Cell[2564697, 101975, 363, 11, 29, "Input",
 CellTags->"Write2"],
Cell[2565063, 101988, 954, 25, 126, "Output",
 CellTags->"Write2"]
}, Open  ]],
Cell[2566032, 102016, 397, 11, 29, "Input",
 CellTags->"Write2"]
}, Open  ]],
Cell[2566444, 102030, 43, 1, 39, "ObjectName",
 CellTags->"XYT"],
Cell[CellGroupData[{
Cell[2566512, 102035, 54, 1, 36, "Subsubsection",
 CellTags->"XYT"],
Cell[2566569, 102038, 215, 9, 24, "Text",
 CellTags->"XYT"]
}, Open  ]],
Cell[2566799, 102050, 79, 1, 39, "ObjectName",
 CellTags->"ZeroMomentumInsertion"],
Cell[CellGroupData[{
Cell[2566903, 102055, 72, 1, 36, "Subsubsection",
 CellTags->"ZeroMomentumInsertion"],
Cell[2566978, 102058, 154, 4, 24, "Text",
 CellTags->"ZeroMomentumInsertion"],
Cell[2567135, 102064, 741, 24, 24, "Text",
 CellTags->"ZeroMomentumInsertion"]
}, Open  ]],
Cell[2567891, 102091, 47, 1, 39, "ObjectName",
 CellTags->"Zeta2"],
Cell[CellGroupData[{
Cell[2567963, 102096, 56, 1, 36, "Subsubsection",
 CellTags->"Zeta2"],
Cell[2568022, 102099, 94, 1, 24, "Text",
 CellTags->"Zeta2"],
Cell[2568119, 102102, 280, 10, 24, "Text",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2568436, 102117, 53, 1, 36, "Subsubsection",
 CellTags->"Zeta2"],
Cell[CellGroupData[{
Cell[2568514, 102122, 77, 2, 26, "Input",
 CellTags->"Zeta2"],
Cell[2568594, 102126, 136, 4, 26, "Output",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2568767, 102135, 104, 3, 26, "Input",
 CellTags->"Zeta2"],
Cell[2568874, 102140, 120, 3, 26, "Output",
 CellTags->"Zeta2"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2569031, 102148, 138, 4, 26, "Input",
 CellTags->"Zeta2"],
Cell[2569172, 102154, 160, 5, 26, "Output",
 CellTags->"Zeta2"]
}, Open  ]]
}, Open  ]],
Cell[2569359, 102163, 65, 1, 39, "ObjectName",
 CellTags->"$Abbreviations"],
Cell[CellGroupData[{
Cell[2569449, 102168, 65, 1, 28, "Subsubsection",
 CellTags->"$Abbreviations"],
Cell[2569517, 102171, 293, 6, 41, "Text",
 CellTags->"$Abbreviations"],
Cell[CellGroupData[{
Cell[2569835, 102181, 129, 3, 26, "Input",
 CellTags->"$Abbreviations"],
Cell[2569967, 102186, 867, 17, 77, "Output",
 CellTags->"$Abbreviations"]
}, Open  ]],
Cell[2570849, 102206, 1071, 39, 22, "Text",
 CellTags->"$Abbreviations"]
}, Open  ]],
Cell[2571935, 102248, 43, 1, 39, "ObjectName",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2572003, 102253, 54, 1, 28, "Subsubsection",
 CellTags->"$AL"],
Cell[2572060, 102256, 114, 3, 24, "Text",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2572199, 102263, 73, 2, 26, "Input",
 CellTags->"$AL"],
Cell[2572275, 102267, 102, 3, 26, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2572414, 102275, 241, 7, 26, "Input",
 CellTags->"$AL"],
Cell[2572658, 102284, 563, 22, 28, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[2573236, 102309, 117, 4, 26, "Input",
 CellTags->"$AL"],
Cell[CellGroupData[{
Cell[2573378, 102317, 241, 7, 26, "Input",
 CellTags->"$AL"],
Cell[2573622, 102326, 477, 20, 28, "Output",
 CellTags->"$AL"]
}, Open  ]],
Cell[2574114, 102349, 109, 4, 26, "Input",
 CellTags->"$AL"]
}, Open  ]],
Cell[2574238, 102356, 61, 1, 39, "ObjectName",
 CellTags->"$BreitMaison"],
Cell[CellGroupData[{
Cell[2574324, 102361, 63, 1, 28, "Subsubsection",
 CellTags->"$BreitMaison"],
Cell[2574390, 102364, 319, 9, 37, "Text",
 CellTags->"$BreitMaison"],
Cell[2574712, 102375, 605, 25, 24, "Text",
 CellTags->"$BreitMaison"]
}, Open  ]],
Cell[2575332, 102403, 49, 1, 39, "ObjectName",
 CellTags->"$Color"],
Cell[CellGroupData[{
Cell[2575406, 102408, 57, 1, 36, "Subsubsection",
 CellTags->"$Color"],
Cell[2575466, 102411, 130, 4, 24, "Text",
 CellTags->"$Color"]
}, Open  ]],
Cell[2575611, 102418, 57, 1, 39, "ObjectName",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2575693, 102423, 61, 1, 36, "Subsubsection",
 CellTags->"$Covariant"],
Cell[2575757, 102426, 183, 4, 24, "Text",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2575965, 102434, 87, 2, 26, "Input",
 CellTags->"$Covariant"],
Cell[2576055, 102438, 111, 3, 26, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2576203, 102446, 110, 3, 26, "Input",
 CellTags->"$Covariant"],
Cell[2576316, 102451, 142, 4, 26, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2576473, 102458, 130, 4, 26, "Input",
 CellTags->"$Covariant"],
Cell[CellGroupData[{
Cell[2576628, 102466, 110, 3, 26, "Input",
 CellTags->"$Covariant"],
Cell[2576741, 102471, 140, 4, 29, "Output",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2576896, 102478, 131, 4, 26, "Input",
 CellTags->"$Covariant"]
}, Open  ]],
Cell[2577042, 102485, 45, 1, 39, "ObjectName",
 CellTags->"$FCS"],
Cell[CellGroupData[{
Cell[2577112, 102490, 55, 1, 36, "Subsubsection",
 CellTags->"$FCS"],
Cell[2577170, 102493, 142, 4, 24, "Text",
 CellTags->"$FCS"],
Cell[CellGroupData[{
Cell[2577337, 102501, 75, 2, 26, "Input",
 CellTags->"$FCS"],
Cell[2577415, 102505, 700, 14, 47, "Output",
 CellTags->"$FCS"]
}, Open  ]]
}, Open  ]],
Cell[2578142, 102523, 45, 1, 39, "ObjectName",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2578212, 102528, 55, 1, 36, "Subsubsection",
 CellTags->"$FCT"],
Cell[2578270, 102531, 177, 4, 24, "Text",
 CellTags->"$FCT"],
Cell[2578450, 102537, 128, 4, 26, "Input",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2578603, 102545, 170, 5, 26, "Input",
 CellTags->"$FCT"],
Cell[2578776, 102552, 196, 6, 26, "Output",
 CellTags->"$FCT"]
}, Open  ]],
Cell[2578987, 102561, 129, 4, 26, "Input",
 CellTags->"$FCT"],
Cell[CellGroupData[{
Cell[2579141, 102569, 170, 5, 26, "Input",
 CellTags->"$FCT"],
Cell[2579314, 102576, 196, 6, 26, "Output",
 CellTags->"$FCT"]
}, Open  ]]
}, Open  ]],
Cell[2579537, 102586, 65, 1, 39, "ObjectName",
 CellTags->"$FeynCalcStuff"],
Cell[CellGroupData[{
Cell[2579627, 102591, 65, 1, 36, "Subsubsection",
 CellTags->"$FeynCalcStuff"],
Cell[2579695, 102594, 102, 1, 24, "Text",
 CellTags->"$FeynCalcStuff"]
}, Open  ]],
Cell[2579812, 102598, 69, 1, 39, "ObjectName",
 CellTags->"$FeynCalcVersion"],
Cell[CellGroupData[{
Cell[2579906, 102603, 67, 1, 36, "Subsubsection",
 CellTags->"$FeynCalcVersion"],
Cell[2579976, 102606, 124, 3, 24, "Text",
 CellTags->"$FeynCalcVersion"],
Cell[CellGroupData[{
Cell[2580125, 102613, 99, 2, 26, "Input",
 CellTags->"$FeynCalcVersion"],
Cell[2580227, 102617, 125, 3, 26, "Output",
 CellTags->"$FeynCalcVersion"]
}, Open  ]]
}, Open  ]],
Cell[2580379, 102624, 95, 1, 39, "ObjectName",
 CellTags->"$FortranContinuationCharacter"],
Cell[CellGroupData[{
Cell[2580499, 102629, 80, 1, 36, "Subsubsection",
 CellTags->"$FortranContinuationCharacter"],
Cell[2580582, 102632, 143, 3, 24, "Text",
 CellTags->"$FortranContinuationCharacter"],
Cell[CellGroupData[{
Cell[2580750, 102639, 125, 2, 26, "Input",
 CellTags->"$FortranContinuationCharacter"],
Cell[2580878, 102643, 134, 3, 26, "Output",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],
Cell[2581027, 102649, 274, 11, 24, "Text",
 CellTags->"$FortranContinuationCharacter"]
}, Open  ]],
Cell[2581316, 102663, 49, 1, 39, "ObjectName",
 CellTags->"$Gauge"],
Cell[CellGroupData[{
Cell[2581390, 102668, 57, 1, 36, "Subsubsection",
 CellTags->"$Gauge"],
Cell[2581450, 102671, 260, 5, 41, "Text",
 CellTags->"$Gauge"],
Cell[2581713, 102678, 247, 11, 24, "Text",
 CellTags->"$Gauge"]
}, Open  ]],
Cell[2581975, 102692, 49, 1, 39, "ObjectName",
 CellTags->"$Larin"],
Cell[CellGroupData[{
Cell[2582049, 102697, 57, 1, 36, "Subsubsection",
 CellTags->"$Larin"],
Cell[2582109, 102700, 1133, 30, 92, "Text",
 CellTags->"$Larin"],
Cell[CellGroupData[{
Cell[2583267, 102734, 79, 2, 26, "Input",
 CellTags->"$Larin"],
Cell[2583349, 102738, 107, 3, 26, "Output",
 CellTags->"$Larin"]
}, Open  ]],
Cell[2583471, 102744, 618, 24, 24, "Text",
 CellTags->"$Larin"]
}, Open  ]],
Cell[2584104, 102771, 55, 1, 39, "ObjectName",
 CellTags->"$LimitTo4"],
Cell[CellGroupData[{
Cell[2584184, 102776, 60, 1, 36, "Subsubsection",
 CellTags->"$LimitTo4"],
Cell[2584247, 102779, 203, 4, 24, "Text",
 CellTags->"$LimitTo4"],
Cell[CellGroupData[{
Cell[2584475, 102787, 85, 2, 26, "Input",
 CellTags->"$LimitTo4"],
Cell[2584563, 102791, 109, 3, 26, "Output",
 CellTags->"$LimitTo4"]
}, Open  ]],
Cell[2584687, 102797, 634, 25, 24, "Text",
 CellTags->"$LimitTo4"]
}, Open  ]],
Cell[2585336, 102825, 67, 1, 39, "ObjectName",
 CellTags->"$LorentzIndices"],
Cell[CellGroupData[{
Cell[2585428, 102830, 66, 1, 36, "Subsubsection",
 CellTags->"$LorentzIndices"],
Cell[2585497, 102833, 164, 4, 24, "Text",
 CellTags->"$LorentzIndices"],
Cell[2585664, 102839, 140, 4, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[CellGroupData[{
Cell[2585829, 102847, 333, 9, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2586165, 102858, 669, 28, 26, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2586871, 102891, 117, 3, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2586991, 102896, 380, 10, 38, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2587408, 102911, 302, 9, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2587713, 102922, 884, 38, 28, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2588634, 102965, 117, 3, 26, "Input",
 CellTags->"$LorentzIndices"],
Cell[2588754, 102970, 655, 19, 58, "Output",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[2589424, 102992, 141, 4, 26, "Input",
 CellTags->"$LorentzIndices"]
}, Open  ]],
Cell[2589580, 102999, 59, 1, 39, "ObjectName",
 CellTags->"$MIntegrate"],
Cell[CellGroupData[{
Cell[2589664, 103004, 62, 1, 36, "Subsubsection",
 CellTags->"$MIntegrate"],
Cell[2589729, 103007, 136, 4, 24, "Text",
 CellTags->"$MIntegrate"],
Cell[CellGroupData[{
Cell[2589890, 103015, 89, 2, 26, "Input",
 CellTags->"$MIntegrate"],
Cell[2589982, 103019, 126, 4, 26, "Output",
 CellTags->"$MIntegrate"]
}, Open  ]],
Cell[2590123, 103026, 276, 11, 24, "Text",
 CellTags->"$MIntegrate"]
}, Open  ]],
Cell[2590414, 103040, 43, 1, 39, "ObjectName",
 CellTags->"$MU"],
Cell[CellGroupData[{
Cell[2590482, 103045, 54, 1, 36, "Subsubsection",
 CellTags->"$MU"],
Cell[2590539, 103048, 151, 4, 24, "Text",
 CellTags->"$MU"],
Cell[CellGroupData[{
Cell[2590715, 103056, 73, 2, 26, "Input",
 CellTags->"$MU"],
Cell[2590791, 103060, 102, 3, 26, "Output",
 CellTags->"$MU"]
}, Open  ]],
Cell[2590908, 103066, 256, 11, 24, "Text",
 CellTags->"$MU"]
}, Open  ]],
Cell[2591179, 103080, 69, 1, 39, "ObjectName",
 CellTags->"$MemoryAvailable"],
Cell[CellGroupData[{
Cell[2591273, 103085, 67, 1, 36, "Subsubsection",
 CellTags->"$MemoryAvailable"],
Cell[2591343, 103088, 434, 11, 41, "Text",
 CellTags->"$MemoryAvailable"],
Cell[CellGroupData[{
Cell[2591802, 103103, 99, 2, 26, "Input",
 CellTags->"$MemoryAvailable"],
Cell[2591904, 103107, 116, 3, 26, "Output",
 CellTags->"$MemoryAvailable"]
}, Open  ]]
}, Open  ]],
Cell[2592047, 103114, 53, 1, 39, "ObjectName",
 CellTags->"$NonComm"],
Cell[CellGroupData[{
Cell[2592125, 103119, 59, 1, 36, "Subsubsection",
 CellTags->"$NonComm"],
Cell[2592187, 103122, 109, 3, 24, "Text",
 CellTags->"$NonComm"],
Cell[CellGroupData[{
Cell[2592321, 103129, 83, 2, 26, "Input",
 CellTags->"$NonComm"],
Cell[2592407, 103133, 803, 15, 89, "Output",
 CellTags->"$NonComm"]
}, Open  ]],
Cell[2593225, 103151, 477, 18, 24, "Text",
 CellTags->"$NonComm"]
}, Open  ]],
Cell[2593717, 103172, 63, 1, 39, "ObjectName",
 CellTags->"$PairBrackets"],
Cell[CellGroupData[{
Cell[2593805, 103177, 64, 1, 36, "Subsubsection",
 CellTags->"$PairBrackets"],
Cell[2593872, 103180, 155, 4, 24, "Text",
 CellTags->"$PairBrackets"],
Cell[CellGroupData[{
Cell[2594052, 103188, 93, 2, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2594148, 103192, 114, 3, 26, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2594299, 103200, 204, 7, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2594506, 103209, 425, 18, 28, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2594968, 103232, 118, 3, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2595089, 103237, 113, 3, 26, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2595239, 103245, 204, 7, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2595446, 103254, 445, 18, 28, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2595928, 103277, 119, 3, 26, "Input",
 CellTags->"$PairBrackets"],
Cell[2596050, 103282, 114, 3, 26, "Output",
 CellTags->"$PairBrackets"]
}, Open  ]]
}, Open  ]],
Cell[2596191, 103289, 65, 1, 39, "ObjectName",
 CellTags->"$SpinorMinimal"],
Cell[CellGroupData[{
Cell[2596281, 103294, 65, 1, 36, "Subsubsection",
 CellTags->"$SpinorMinimal"],
Cell[2596349, 103297, 241, 5, 41, "Text",
 CellTags->"$SpinorMinimal"],
Cell[CellGroupData[{
Cell[2596615, 103306, 95, 2, 26, "Input",
 CellTags->"$SpinorMinimal"],
Cell[2596713, 103310, 115, 3, 26, "Output",
 CellTags->"$SpinorMinimal"]
}, Open  ]],
Cell[2596843, 103316, 287, 11, 24, "Text",
 CellTags->"$SpinorMinimal"]
}, Open  ]],
Cell[2597145, 103330, 61, 1, 39, "ObjectName",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2597231, 103335, 63, 1, 36, "Subsubsection",
 CellTags->"$VeryVerbose"],
Cell[2597297, 103338, 221, 5, 41, "Text",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2597543, 103347, 91, 2, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[2597637, 103351, 109, 3, 26, "Output",
 CellTags->"$VeryVerbose"]
}, Open  ]],
Cell[2597761, 103357, 141, 4, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[CellGroupData[{
Cell[2597927, 103365, 287, 10, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[2598217, 103377, 121, 3, 21, "Print",
 CellTags->"$VeryVerbose"],
Cell[2598341, 103382, 255, 7, 22, "Print",
 CellTags->"$VeryVerbose"],
Cell[2598599, 103391, 273, 7, 22, "Print",
 CellTags->"$VeryVerbose"]
}, Open  ]],
Cell[2598887, 103401, 131, 4, 26, "Input",
 CellTags->"$VeryVerbose"],
Cell[2599021, 103407, 236, 11, 24, "Text",
 CellTags->"$V0"]
}, Open  ]],
Cell[2599272, 103421, 47, 1, 39, "ObjectName",
 CellTags->"$West"],
Cell[CellGroupData[{
Cell[2599344, 103426, 56, 1, 36, "Subsubsection",
 CellTags->"$West"],
Cell[2599403, 103429, 577, 14, 66, "Text",
 CellTags->"$West"],
Cell[2599983, 103445, 234, 10, 24, "Text",
 CellTags->"$West"],
Cell[2600220, 103457, 54, 0, 29, "Commentary"],
Cell[CellGroupData[{
Cell[2600299, 103461, 125, 3, 26, "Input"],
Cell[2600427, 103466, 109, 2, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2600573, 103473, 85, 2, 26, "Input"],
Cell[2600661, 103477, 90, 2, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2600788, 103484, 87, 2, 26, "Input"],
Cell[2600878, 103488, 90, 2, 26, "Output"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2601005, 103495, 82, 2, 26, "Input"],
Cell[2601090, 103499, 91, 2, 26, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2601242, 103508, 70, 1, 66, "Section",
 CellTags->"Experimental"],
Cell[2601315, 103511, 73, 1, 39, "ObjectName",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2601413, 103516, 69, 1, 36, "Subsubsection",
 CellTags->"DoPolarizationSums"],
Cell[2601485, 103519, 291, 7, 99, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2601779, 103528, 487, 18, 24, "Text",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2602303, 103551, 66, 1, 36, "Subsubsection",
 CellTags->"DoPolarizationSums"],
Cell[2602372, 103554, 310, 6, 41, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2602685, 103562, 123, 3, 24, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2602833, 103569, 571, 17, 46, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2603407, 103588, 448, 16, 30, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2603892, 103609, 116, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2604011, 103614, 133, 4, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2604159, 103621, 1270, 48, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2605432, 103671, 562, 21, 24, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2605997, 103694, 142, 4, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2606142, 103700, 311, 10, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2606478, 103714, 271, 8, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2606752, 103724, 735, 20, 75, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2607502, 103747, 213, 9, 24, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2607718, 103758, 276, 9, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2607997, 103769, 328, 10, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2608328, 103781, 323, 13, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[2608654, 103796, 697, 30, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2609376, 103830, 434, 13, 53, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2609813, 103845, 775, 24, 75, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2610603, 103872, 624, 27, 27, "Text",
 CellTags->"DoPolarizationSums"],
Cell[CellGroupData[{
Cell[2611252, 103903, 433, 13, 53, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2611688, 103918, 775, 24, 75, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2612500, 103947, 859, 24, 65, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2613362, 103973, 606, 22, 30, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2614005, 104000, 130, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2614138, 104005, 323, 12, 29, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2614498, 104022, 116, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2614617, 104027, 133, 4, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2614787, 104036, 120, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2614910, 104041, 133, 4, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2615080, 104050, 126, 3, 26, "Input",
 CellTags->"DoPolarizationSums"],
Cell[2615209, 104055, 133, 4, 26, "Output",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2615357, 104062, 167, 4, 26, "Input",
 CellTags->"DoPolarizationSums"]
}, Open  ]],
Cell[2615539, 104069, 90, 1, 39, "ObjectName",
 CellTags->"FeynmanDoIntegrals"],
Cell[CellGroupData[{
Cell[2615654, 104074, 69, 1, 36, "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],
Cell[2615726, 104077, 1518, 23, 210, "Text",
 CellTags->"FeynmanDoIntegrals"],
Cell[2617247, 104102, 527, 18, 24, "Text",
 CellTags->"FeynmanDoIntegrals"],
Cell[CellGroupData[{
Cell[2617799, 104124, 136, 3, 26, "Input",
 CellTags->"FeynmanDoIntegrals"],
Cell[2617938, 104129, 1037, 29, 67, "Output",
 CellTags->"FeynmanDoIntegrals"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2619024, 104164, 66, 1, 36, "Subsubsection",
 CellTags->"FeynmanDoIntegrals"],
Cell[2619093, 104167, 85, 2, 26, "Input",
 CellTags->"FeynmanDoIntegrals"]
}, Open  ]],
Cell[2619193, 104172, 92, 1, 39, "ObjectName",
 CellTags->"FeynmanParametrize1"],
Cell[CellGroupData[{
Cell[2619310, 104177, 70, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize1"],
Cell[2619383, 104180, 717, 12, 117, "Text",
 CellTags->"FeynmanParametrize1"],
Cell[CellGroupData[{
Cell[2620125, 104196, 138, 3, 26, "Input",
 CellTags->"FeynmanParametrize1"],
Cell[2620266, 104201, 505, 13, 26, "Output",
 CellTags->"FeynmanParametrize1"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2620820, 104220, 67, 1, 36, "Subsubsection",
 CellTags->"FeynmanParametrize1"],
Cell[2620890, 104223, 86, 2, 26, "Input",
 CellTags->"FeynmanParametrize1"]
}, Open  ]],
Cell[2620991, 104228, 80, 1, 39, "ObjectName",
 CellTags->"FeynmanReduce"],
Cell[CellGroupData[{
Cell[2621096, 104233, 64, 1, 36, "Subsubsection",
 CellTags->"FeynmanReduce"],
Cell[2621163, 104236, 723, 12, 117, "Text",
 CellTags->"FeynmanReduce"],
Cell[CellGroupData[{
Cell[2621911, 104252, 126, 3, 26, "Input",
 CellTags->"FeynmanReduce"],
Cell[2622040, 104257, 288, 9, 26, "Output",
 CellTags->"FeynmanReduce"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[2622377, 104272, 61, 1, 36, "Subsubsection",
 CellTags->"FeynmanReduce"],
Cell[2622441, 104275, 80, 2, 26, "Input",
 CellTags->"FeynmanReduce"],
Cell[2622524, 104279, 122, 3, 24, "Text",
 CellTags->"Experimental"],
Cell[CellGroupData[{
Cell[2622671, 104286, 198, 6, 26, "Input",
 CellTags->"Experimental"],
Cell[2622872, 104294, 193, 6, 26, "Output",
 CellTags->"Experimental"]
}, Open  ]],
Cell[2623080, 104303, 100, 1, 24, "Text",
 CellTags->"Experimental"],
Cell[CellGroupData[{
Cell[2623205, 104308, 222, 6, 29, "Input",
 CellTags->"Experimental"],
Cell[2623430, 104316, 246, 7, 26, "Output",
 CellTags->"Experimental"]
}, Open  ]],
Cell[CellGroupData[{
Cell[2623713, 104328, 83, 3, 26, "Input"],
Cell[2623799, 104333, 109, 2, 26, "Output"]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}, Open  ]]
}
]
*)

(* End of internal cache information *)
